/********************************************************
 * @author  Airead Fan <fgh1987168@gmail.com>
 * @date    2012  2月 20 12:05:05 CST
 ********************************************************/

/*
 * This program demonstration actions of a client which send a number to
 * server and receive the return value from server
 *
 * This just a simple implement for practicing socket communication.
 *
 * Note: It should run with server_sum.c
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

int get_connected_socket(char *ip, unsigned short port)
{
    int fd;
    struct sockaddr_in remote_ipv4_address;

    /* Create a socket */
    if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
        fprintf(stderr, "Create new TCP socket failed: %s\n", strerror(errno));
        exit(1);
    }

    /* Connet */
    memset(&remote_ipv4_address, 0, sizeof(remote_ipv4_address));

    remote_ipv4_address.sin_family = AF_INET;
    remote_ipv4_address.sin_port = htons(port);
    inet_pton(AF_INET, ip, &remote_ipv4_address.sin_addr);

    if (connect(fd, (struct sockaddr *)&remote_ipv4_address, sizeof(remote_ipv4_address)) < 0) {
        fprintf(stderr, "Connect to remote server %s:%d failed: %s\n", ip, port, strerror(errno));
        close(fd);
        exit(1);
    }

    fprintf(stdout, "Connect to remote server %s:%d successful\n", ip, port);
    
    return fd;
}

int main(int argc, char *argv[])
{
    int fd;
    char *ip;
    unsigned short port;
    int num;

    if (argc < 3) {
        fprintf(stderr, "usage: %s <ip_addr> <port>\n", argv[0]);
        exit(1);
    }

    ip = argv[1];
    port = (unsigned short)strtol(argv[2], NULL, 10);

    if ((fd = get_connected_socket(ip, port)) < 0) {
        fprintf(stderr, "get connected socket failed: %s\n", strerror(errno));
        exit(1);
    }
    
    srand(time(NULL));
    num = rand() % 10;
    write(fd, &num, sizeof(num));
    
    read(fd, &num, sizeof(num));

    fprintf(stdout, "receive data: %d\n", num);

    close(fd);
    return 0;
}
