/********************************************************
 * @author  Airead Fan <fgh1987168@gmail.com>
 * @date    2012  2月 20 11:44:51 CST
 ********************************************************/

/*
 * This program demonstration an action of a service which receive three numbers
 * from client and return the sum of them to each client. After the server
 * start, it need to receive three numbers from client, then it will send the
 * sum of them to each client. That is to say, after server start, you should
 * run three times clinet_sum.c in different shell for the sum.
 *
 * This just a simple implement for practicing socket communication.
 *
 * Note: It should run with client_sum.c
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

int get_listen_socket(char *ip, unsigned short port, int listen_backlog)
{
    int listening_socket;
    int optval;
    struct sockaddr_in local_ipv4_address;

    /* Create a socket */
    //int socket(int domain, int type, int protocol);
    if ((listening_socket = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
        fprintf(stderr, "Failed to create new TCP socket: %s\n", strerror(errno));
        exit(1);
    }

    /* set socket option */
    optval = 1;
    //int setsockopt(int sockfd, int level, int optname, const void *optval, socklen_t optlen);
    setsockopt(listening_socket, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));

#ifdef SO_REUSEPORT
    optval = 1;
    setsockopt(listening_socket, SOL_SOCKET, SO_REUSEPORT, &optval, sizeof(optval));
#endif

    /* bind */
    memset(&local_ipv4_address, 0, sizeof(local_ipv4_address));

    local_ipv4_address.sin_family = AF_INET;
    local_ipv4_address.sin_port = htons(port);
    inet_pton(AF_INET, ip, &local_ipv4_address.sin_addr);

    if (bind(listening_socket, (struct sockaddr *)&local_ipv4_address, sizeof(local_ipv4_address)) < 0) {
        fprintf(stderr, "Bind to %s:%d failed: %s\n", ip, port, strerror(errno));
        close(listening_socket);
        exit(1);
    }

    fprintf(stdout, "Bound to %s:%d successfully.\n", ip, port);

    /* Listen */
    if (listen(listening_socket, listen_backlog) < 0) {
        fprintf(stderr, "Listen on %s:%d failed: %s\n", ip, port, strerror(errno));
        close(listening_socket);
        exit(1);
    }

    fprintf(stdout, "Listen on %s:%d successfully, waiting for incoming connections ...\n", ip, port);

    return listening_socket;
}

int main(int argc, char *argv[])
{
    int listening_socket;
    char *ip;
    unsigned short port;
    int listen_backlog;

    int new_connected_socket[3];
    struct sockaddr_in peer_ipv4_address;
    socklen_t peer_ipv4_address_length;
    char peer_ipv4_address_string[INET_ADDRSTRLEN];

    int i;
    int num[3];
    int sum;

    if (argc < 4) {
        fprintf(stderr, "usage: %s <ip_addr> <port> <backlog>\n", argv[0]);
        exit(1);
    }

    ip = argv[1];
    port = (unsigned short)strtol(argv[2], NULL, 10);
    listen_backlog = (int) strtol(argv[3], NULL, 10);

    if ((listening_socket = get_listen_socket(ip, port, listen_backlog)) < 0) {
        fprintf(stderr, "get listen socket failed\n");
        exit(1);
    }

    i = 0;
    /* accept */
    for(;;) {
        peer_ipv4_address_length = sizeof(peer_ipv4_address);
        new_connected_socket[i] = accept(listening_socket, (struct sockaddr *)&peer_ipv4_address, &peer_ipv4_address_length);
        if (new_connected_socket[i] < 0) {
            fprintf(stderr, "accept() failed: %s\n", strerror(errno));
            break;
        } else {
            memset(peer_ipv4_address_string, 0, sizeof(peer_ipv4_address_string));
            inet_ntop(AF_INET, &peer_ipv4_address.sin_addr, peer_ipv4_address_string, sizeof(peer_ipv4_address_string));
            fprintf(stdout, "Accepted a new connection %d, from %s:%d\n", new_connected_socket[i], peer_ipv4_address_string, ntohs(peer_ipv4_address.sin_port));

            //ssize_t read(int fd, void *buf, size_t count);
            if (read(new_connected_socket[i], &num[i], sizeof(num[i])) < 0) {
                fprintf(stderr, "read data error: %s\n", strerror(errno));
                break;
            }
                
            fprintf(stdout, "receive num %d: %d\n", i, num[i]);
            i++;
            
            if (i >= 3) {
                sum = 0;
                for (i = 0; i < 3; i++) {
                    sum += num[i];
                }
                for (i = 0; i < 3; i++) {
                    write(new_connected_socket[i], &sum, sizeof(sum));
                    close(new_connected_socket[i]);
                }
                
                i = 0;
            }
        }
    }//for end

    close(listening_socket);

    return 0;
}

