/********************************************************
 * @author  Airead Fan <fgh1987168@gmail.com>
 * @date    2012  2月 20 09:55:14 CST
 ********************************************************/

/*
 * This program demonstration an action of a service which receive a string from
 * client and return the capitalization of it to client.
 *
 * This just a simple implement for practicing socket communication.
 *
 * Note: It should run with client_simple.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

int main(int argc, char *argv[])
{
    int listening_socket;
    int optval;
    struct sockaddr_in local_ipv4_address;
    char *ip;
    unsigned short port;
    int listen_backlog;

    int new_connected_socket;
    struct sockaddr_in peer_ipv4_address;
    socklen_t peer_ipv4_address_length;
    char peer_ipv4_address_string[INET_ADDRSTRLEN];

    char buf[512];              /* not good idea */
    int i;

    if (argc < 3) {
        fprintf(stderr, "usage: %s <ip_addr> <port> <backlog>\n", argv[0]);
        exit(1);
    }

    ip = argv[1];
    port = (unsigned short)strtol(argv[2], NULL, 10);
    listen_backlog = (int) strtol(argv[3], NULL, 10);

    /* Create a socket */
    //int socket(int domain, int type, int protocol);
    if ((listening_socket = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
        fprintf(stderr, "Failed to create new TCP socket: %s\n", strerror(errno));
        exit(1);
    }

    /* set socket option */
    optval = 1;
    //int setsockopt(int sockfd, int level, int optname, const void *optval, socklen_t optlen);
    setsockopt(listening_socket, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));

#ifdef SO_REUSEPORT
    optval = 1;
    setsockopt(listening_socket, SOL_SOCKET, SO_REUSEPORT, &optval, sizeof(optval));
#endif

    /* bind */
    memset(&local_ipv4_address, 0, sizeof(local_ipv4_address));

    local_ipv4_address.sin_family = AF_INET;
    local_ipv4_address.sin_port = htons(port);
    inet_pton(AF_INET, argv[1], &local_ipv4_address.sin_addr);

    if (bind(listening_socket, (struct sockaddr *)&local_ipv4_address, sizeof(local_ipv4_address)) < 0) {
        fprintf(stderr, "Bind to %s:%d failed: %s\n", ip, port, strerror(errno));
        close(listening_socket);
        exit(1);
    }

    fprintf(stdout, "Bound to %s:%d successfully.\n", argv[1], port);

    /* Listen */
    if (listen(listening_socket, listen_backlog) < 0) {
        fprintf(stderr, "Listen on %s:%d failed: %s\n", argv[1], port, strerror(errno));
        close(listening_socket);
        exit(1);
    }

    fprintf(stdout, "Listen on %s:%d successfully, waiting for incoming connections ...\n", ip, port);

    /* accept */
    for(;;) {
        peer_ipv4_address_length = sizeof(peer_ipv4_address);
        new_connected_socket = accept(listening_socket, (struct sockaddr *)&peer_ipv4_address, &peer_ipv4_address_length);
        if (new_connected_socket < 0) {
            fprintf(stderr, "accept() failed: %s\n", strerror(errno));
            break;
        } else {
            memset(peer_ipv4_address_string, 0, sizeof(peer_ipv4_address_string));
            inet_ntop(AF_INET, &peer_ipv4_address.sin_addr, peer_ipv4_address_string, sizeof(peer_ipv4_address_string));
            fprintf(stdout, "Accepted a new connection %d, from %s:%d\n", new_connected_socket, peer_ipv4_address_string, ntohs(peer_ipv4_address.sin_port));

            memset(buf, 0, sizeof(buf));
            //ssize_t read(int fd, void *buf, size_t count);
            if (read(new_connected_socket, buf, sizeof(buf) - 1) < 0) {
                fprintf(stderr, "read data error: %s\n", strerror(errno));
                break;
            }

            fprintf(stdout, "receive buf: %s\n", buf);
            
            for (i = 0; buf[i] != '\0'; i++) {
                buf[i] = toupper(buf[i]);
            }

            write(new_connected_socket, buf, sizeof(buf));

            close(new_connected_socket);
        }
    }//for end

    close(listening_socket);

    return 0;
}
