/**
 * @file client.c
 * @brief 
 * @author Airead Fan <fgh1987168@gmail.com>
 * @date 2012  2月 22 12:00:28 CST
 */

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <assert.h>

#include "server.h"
#include "select.h"
#include "client.h"
#include "common_debug.h"

/**
 * @brief conect to server
 * @param ip : server ip address
 * @param port : server port
 * @return 
 */
int connect_server(char *ip, unsigned short port)
{
    int fd;
    struct sockaddr_in remote_ipv4_address;

    /* Create a socket */
    if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
        fprintf(stderr, "Create new TCP socket failed: %s\n", strerror(errno));
        return -1;
    }

    /* Connet */
    memset(&remote_ipv4_address, 0, sizeof(remote_ipv4_address));

    remote_ipv4_address.sin_family = AF_INET;
    remote_ipv4_address.sin_port = htons(port);
    inet_pton(AF_INET, ip, &remote_ipv4_address.sin_addr);

    if (connect(fd, (struct sockaddr *)&remote_ipv4_address, sizeof(remote_ipv4_address)) < 0) {
        fprintf(stderr, "Connect to remote server %s:%d failed: %s\n", ip, port, strerror(errno));
        close(fd);
        return -1;
    }

    fprintf(stdout, "Connect to remote server %s:%d successful\n", ip, port);
    
    return fd;
}

/**
 * @brief a swicth function
 * @param connfd
 * @return 
 */
int client_deal(int connfd) 
{
    struct client client;
    struct command cmd;
    ssize_t read_n;

    read_n = read(connfd, &cmd, sizeof(cmd));
    if (read_n < 0) {
        PRINTERR("read data failed: %s\n", strerror(errno));
        return -1;
    } else if (read_n == 0) {
        PRINTERR("remote close\n");
        close(connfd);
        return -1;
    } else {
        switch (cmd.type) {
        case CMD_SEND_SERVER_DATA:
            printf("receive server_data.th: %d\n", cmd.server_data.th);
            break;
        case CMD_GET_CLIENT_DATA:
            client.data.three = rand() % 100;
            cmd.type = CMD_SEND_CLIENT_DATA;
            cmd.client_data = client.data;
            write(connfd, &cmd, sizeof(cmd));
            DBG("write over\n");
            break;
        case CMD_COUNT_TIME:
            printf("=================time: %d\n", cmd.server_data.count_time);
            break;
        default:
            PRINTERR("error, can't reach here\n");
        }
    }

    return 0;
}

/**
 * @brief client main function
 * @param ip_addr: server ip address needed to connect
 * @param port: server port needed to connect
 * @return on error returns -1
 */
int client_main(char *ip_addr, unsigned short port)
{
    fd_set readfds;
    int ret;
    int connfd;

    STEP_CHECK();

    srand(47);

    assert(ip_addr);

    if ((connfd = connect_server(ip_addr, port)) < 0) {
        PRINTERR("connect to server failed: %s\n", strerror(errno));
        return -1;
    }
    
    FD_ZERO(&readfds);
    FD_SET(connfd, &readfds);

    while (1) {
        ret = select(connfd + 1, &readfds, NULL, NULL, NULL);
        if (ret < 0) {
            PRINTERR("select failed:%s\n", strerror(errno));
            return -1;
        } else if (ret == 0) {

        } else {
            if (FD_ISSET(connfd, &readfds)) {
                client_deal(connfd);
            }
        }
        FD_SET(connfd, &readfds);
    }

    return 0;
}

/**
 * @brief print client all information for debug
 * @param self
 * @return void
 */
void client_print(struct client *self)
{
    DBG("client id: %d, client_data: %d\n", self->id, self->data.three);
}
