/**
 * @file select.c
 * @brief select functions
 * @author Airead Fan <fgh1987168@gmail.com>
 * @date 2012  2月 22 14:19:45 CST
 */

#include <stdlib.h>

#include "common_debug.h"
#include "select.h"

/**
 * @brief create select data
 * @return a pointer to struct select_data
 */
struct select_data *select_new(void)
{
    struct select_data *sdata;

    sdata = (struct select_data *)malloc(sizeof(struct select_data));
    if (sdata == NULL) {
        PRINTERR("malloc failed!\n");
        return NULL;
    }

    return sdata;
}

/**
 * @brief initialize a select_data
 * @param self : struct select_data *
 * @return on error returns -1
 *
 * initialize a select_data. if error occurs, It doesn't destory sdata.
 */
int select_init(struct select_data *self)
{
    self->nfds = 0;
    FD_ZERO(&self->readfds);
    FD_ZERO(&self->writefds);
    FD_ZERO(&self->exceptfds);
    self->timeout.tv_sec = 0;
    self->timeout.tv_usec = 0;

    return 0;
}

/**
 * @brief register connected_fd to readfds for select
 * @param readfds : fd_set of readable
 * @param connected_fd : come from accept function
 * @return on error returns -1, otherwise return 0
 */
int select_register_readfds(struct select_data *self, int connected_fd)
{
    FD_SET(connected_fd, &self->readfds);
    if (self->nfds <= connected_fd) {
        self->nfds = connected_fd + 1;
    }
    
    return 0;
}
