//
//  alloc_template.c
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 12/17/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

//--------------------OBJECT--------------------

typedef struct
{
    int var;
}
OBJECT_t;

typedef OBJECT_t* OBJECT_ptr_t;

OBJECT_ptr_t OBJECT_alloc(void);
void OBJECT_free(OBJECT_ptr_t* OBJECT_ptr_ptr);

//--------------------OBJECT--------------------

OBJECT_ptr_t OBJECT_alloc(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] OBJECT_alloc()\n");
    
    OBJECT_ptr_t OBJECT_ptr = malloc(sizeof(OBJECT_t));
    OBJECT_ptr->var = 0;
    return OBJECT_ptr;
}

void OBJECT_free(OBJECT_ptr_t* OBJECT_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] OBJECT_free()\n");

    OBJECT_ptr_t OBJECT_ptr = *(OBJECT_ptr_ptr);
    free(OBJECT_ptr);
    *(OBJECT_ptr_ptr) = NULL;
}

//--------------------CONTAINER--------------------

typedef struct
{
    OBJECT_ptr_t OBJECT_ptr;
    int var;
}
CONTAINER_t;

typedef CONTAINER_t* CONTAINER_ptr_t;

CONTAINER_ptr_t CONTAINER_alloc(OBJECT_ptr_t OBJECT_ptr);
void CONTAINER_free(CONTAINER_ptr_t* CONTAINER_ptr_ptr);

void CONTAINER_set_OBJECT(CONTAINER_ptr_t CONTAINER_ptr, OBJECT_ptr_t OBJECT_ptr);

//--------------------CONTAINER--------------------

CONTAINER_ptr_t CONTAINER_alloc(OBJECT_ptr_t OBJECT_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] CONTAINER_alloc()\n");
    
    CONTAINER_ptr_t CONTAINER_ptr = malloc(sizeof(CONTAINER_t));
    CONTAINER_ptr->OBJECT_ptr = OBJECT_ptr;
    CONTAINER_ptr->var = 0;
    return CONTAINER_ptr;
}

void CONTAINER_free(CONTAINER_ptr_t* CONTAINER_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] CONTAINER_free()\n");

    CONTAINER_ptr_t CONTAINER_ptr = *(CONTAINER_ptr_ptr);
    // free OBJECT
    if (CONTAINER_ptr->OBJECT_ptr != NULL)
    {
        OBJECT_free(&(CONTAINER_ptr->OBJECT_ptr));
    }
    // free CONTAINER
    free(CONTAINER_ptr);
    *(CONTAINER_ptr_ptr) = NULL;
}

void CONTAINER_set_OBJECT(CONTAINER_ptr_t CONTAINER_ptr, OBJECT_ptr_t OBJECT_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] CONTAINER_set_OBJECT()\n");

    if (CONTAINER_ptr->OBJECT_ptr != NULL)
    {
        OBJECT_free(&(CONTAINER_ptr->OBJECT_ptr));
    }
    CONTAINER_ptr->OBJECT_ptr = OBJECT_ptr;
}

//--------------------COLLECTION--------------------

typedef struct
{
    OBJECT_ptr_t* OBJECT_ptr_array;     // array of pointers
    int OBJECT_ptr_array_count;         // size of array
    int var;
}
COLLECTION_t;

typedef COLLECTION_t* COLLECTION_ptr_t;

COLLECTION_ptr_t COLLECTION_alloc(void);
void COLLECTION_free(COLLECTION_ptr_t* COLLECTION_ptr_ptr);

void COLLECTION_add_OBJECT(COLLECTION_ptr_t COLLECTION_ptr, OBJECT_ptr_t OBJECT_ptr);
void COLLECTION_remove_OBJECT(COLLECTION_ptr_t COLLECTION_ptr, OBJECT_ptr_t OBJECT_ptr);

//--------------------COLLECTION--------------------

COLLECTION_ptr_t COLLECTION_alloc(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] COLLECTION_alloc()\n");
    
    COLLECTION_ptr_t COLLECTION_ptr = malloc(sizeof(COLLECTION_t));
    COLLECTION_ptr->OBJECT_ptr_array = NULL;
    COLLECTION_ptr->OBJECT_ptr_array_count = 0;
    COLLECTION_ptr->var = 0;
    return COLLECTION_ptr;
}

void COLLECTION_free(COLLECTION_ptr_t* COLLECTION_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] COLLECTION_free()\n");

    COLLECTION_ptr_t COLLECTION_ptr = *(COLLECTION_ptr_ptr);
    // free OBJECT array
    if (COLLECTION_ptr->OBJECT_ptr_array != NULL)
    {
        for (int index = 0; index < COLLECTION_ptr->OBJECT_ptr_array_count; index++)
        {
            OBJECT_free(&(COLLECTION_ptr->OBJECT_ptr_array[index]));
        }
        free(COLLECTION_ptr->OBJECT_ptr_array);
    }
    // free COLLECTION
    free(COLLECTION_ptr);
    *(COLLECTION_ptr_ptr) = NULL;
}

void COLLECTION_add_OBJECT(COLLECTION_ptr_t COLLECTION_ptr, OBJECT_ptr_t OBJECT_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] COLLECTION_add_OBJECT()\n");
    
    if (COLLECTION_ptr->OBJECT_ptr_array == NULL)
    {
        COLLECTION_ptr->OBJECT_ptr_array = malloc(sizeof(OBJECT_ptr_t));
        COLLECTION_ptr->OBJECT_ptr_array[0] = OBJECT_ptr;
        COLLECTION_ptr->OBJECT_ptr_array_count = 1;
    }
    else
    {
        OBJECT_ptr_t* old_OBJECT_ptr_array = COLLECTION_ptr->OBJECT_ptr_array;
        int old_OBJECT_ptr_array_count = COLLECTION_ptr->OBJECT_ptr_array_count;
        COLLECTION_ptr->OBJECT_ptr_array = malloc(sizeof(OBJECT_ptr_t) * (old_OBJECT_ptr_array_count + 1));
        memcpy(COLLECTION_ptr->OBJECT_ptr_array, old_OBJECT_ptr_array, sizeof(OBJECT_ptr_t) * old_OBJECT_ptr_array_count);
        COLLECTION_ptr->OBJECT_ptr_array[old_OBJECT_ptr_array_count] = OBJECT_ptr;
        COLLECTION_ptr->OBJECT_ptr_array_count = old_OBJECT_ptr_array_count + 1;
        free(old_OBJECT_ptr_array);
    }
}

void COLLECTION_remove_OBJECT(COLLECTION_ptr_t COLLECTION_ptr, OBJECT_ptr_t OBJECT_ptr)
{
    // COLLECTION_remove_OBJECT is not responsible for freeing the OBJECT_t pointed to by OBJECT_ptr

    /*** debug ***/ fprintf(stdout, "[Trace] COLLECTION_remove_OBJECT()\n");

    int OBJECT_index = -1;
    for (int index = 0; index < COLLECTION_ptr->OBJECT_ptr_array_count; index++)
    {
        if (COLLECTION_ptr->OBJECT_ptr_array[index] == OBJECT_ptr)
        {
            OBJECT_index = index;
            break;
        }
    }
    if (OBJECT_index != -1)
    {
        if (COLLECTION_ptr->OBJECT_ptr_array_count == 1)
        {
            free(COLLECTION_ptr->OBJECT_ptr_array);
            COLLECTION_ptr->OBJECT_ptr_array = NULL;
            COLLECTION_ptr->OBJECT_ptr_array_count = 0;
        }
        else
        {
            OBJECT_ptr_t* old_OBJECT_ptr_array = COLLECTION_ptr->OBJECT_ptr_array;
            int old_OBJECT_ptr_array_count = COLLECTION_ptr->OBJECT_ptr_array_count;
            COLLECTION_ptr->OBJECT_ptr_array = malloc(sizeof(OBJECT_ptr_t) * (old_OBJECT_ptr_array_count - 1));
            if (OBJECT_index > 0)
            {
                memcpy(COLLECTION_ptr->OBJECT_ptr_array, old_OBJECT_ptr_array, sizeof(OBJECT_ptr_t) * OBJECT_index);
            }
            if (OBJECT_index < old_OBJECT_ptr_array_count - 1)
            {
                memcpy(COLLECTION_ptr->OBJECT_ptr_array + OBJECT_index, old_OBJECT_ptr_array + OBJECT_index + 1, sizeof(OBJECT_ptr_t) * (old_OBJECT_ptr_array_count - 1 - OBJECT_index));
            }
            COLLECTION_ptr->OBJECT_ptr_array_count = old_OBJECT_ptr_array_count - 1;
            free(old_OBJECT_ptr_array);
        }
    }
}

