//
//  draw_gl.c
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 2/9/12.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <math.h>

#include "SDL.h"
#include "SDL_gl.h"

#include "draw_gl.h"

GLuint draw_surface_to_texture(SDL_Surface* sdl_surface_ptr)
{
    Gluint gl_texture_id = -1;
    glGenTextures(1, &gl_texture_id);
    glBindTexture(GL_TEXTURE_2D, gl_texture_id);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    GLenum texture_format;
    if (sdl_surface_ptr->format->BytesPerPixel == 4)
    {
        if (sdl_surface_ptr->format->Rmask == 0x000000ff)
        {
            texture_format = GL_RGBA;
        }
        else
        {
            texture_format = GL_BGRA;
        }
    }
    else if (sdl_surface_ptr->format->BytesPerPixel == 3)
    {
        if (sdl_surface_ptr->format->Rmask == 0x000000ff)
        {
            texture_format = GL_RGB;
        }
        else
        {
            texture_format = GL_BGR;
        }
    }
    else
    {
        return -1;
    }
    glTexImage2D(GL_TEXTURE_2D, 0, sdl_surface_ptr->format->BytesPerPixel, sdl_surface_ptr->w, sdl_surface_ptr->h, 0, texture_format, GL_UNSIGNED_BYTE, sdl_surface_ptr->pixels);
    return gl_texture_id;
}

#ifdef FOO
/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 */
Uint32 draw_get_pixel(SDL_Surface* surface_ptr, int x, int y)
{
    int bpp = surface_ptr->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8* ptr = (Uint8*) surface_ptr->pixels + y * surface_ptr->pitch + x * bpp;
    
    switch (bpp)
    {
        case 1:
            return *(ptr);
            
        case 2:
            return *((Uint16*) ptr);
            
        case 3:
            if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
                return ptr[0] << 16 | ptr[1] << 8 | ptr[2];
            else
                return ptr[0] | ptr[1] << 8 | ptr[2] << 16;

        case 4:
            return *((Uint32*) ptr);
            
        default:
            return 0;       /* shouldn't happen, but avoids warnings */
    }
}

/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void draw_put_pixel(SDL_Surface* surface_ptr, int x, int y, Uint32 color)
{
    if (x < 0 || x >= surface_ptr->w || y < 0 || y >= surface_ptr->h)
    {
        return;
    }

    int bpp = surface_ptr->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8* ptr = (Uint8*) surface_ptr->pixels + y * surface_ptr->pitch + x * bpp;
    
    switch (bpp)
    {
        case 1:
            *(ptr) = color;
            break;
            
        case 2:
            *((Uint16*) ptr) = color;
            break;
            
        case 3:
            if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
            {
                ptr[0] = (color >> 16) & 0xff;
                ptr[1] = (color >> 8) & 0xff;
                ptr[2] = color & 0xff;
            }
            else
            {
                ptr[0] = color & 0xff;
                ptr[1] = (color >> 8) & 0xff;
                ptr[2] = (color >> 16) & 0xff;
            }
            break;
            
        case 4:
            *((Uint32*) ptr) = color;
            break;
    }
}

void draw_line(SDL_Surface* surface_ptr, int x1, int y1, int x2, int y2, Uint32 color)
{
    int dx, dy, inx, iny, e;

    dx = x2 - x1;
    dy = y2 - y1;
    inx = dx > 0 ? 1 : -1;
    iny = dy > 0 ? 1 : -1;

    dx = abs(dx);
    dy = abs(dy);

    if (dx >= dy)
    {
        dy <<= 1;
        e = dy - dx;
        dx <<= 1;
        while (x1 != x2)
        {
            draw_put_pixel(surface_ptr, x1, y1, color);
            if (e >= 0)
            {
                y1 += iny;
                e-= dx;
            }
            e += dy;
            x1 += inx;
        }
    }
    else
    {
        dx <<= 1;
        e = dx - dy;
        dy <<= 1;
        while (y1 != y2)
        {
            draw_put_pixel(surface_ptr, x1, y1, color);
            if (e >= 0)
            {
                x1 += inx;
                e -= dy;
            }
            e += dx;
            y1 += iny;
        }
    }
    draw_put_pixel(surface_ptr, x1, y1, color);
}

void draw_circle_8(SDL_Surface* surface_ptr, int x, int y, int dx, int dy, Uint32 color);

void draw_circle(SDL_Surface* surface_ptr, int x, int y, int r, Uint32 color)
{
    int dx, dy, inx, iny, e;

    dx = r;
    dy = 0;
    inx = 1 - 2 * r;
    iny = 1;
    e = 0;
    while (dx >= dy)
    {
        draw_circle_8(surface_ptr, x, y, dx, dy, color);
        dy++;
        e += iny;
        iny += 2;
        if (2 * e + inx > 0)
        {
            dx--;
            e += inx;
            inx += 2;
        }
    }
}

void draw_circle_8(SDL_Surface* surface_ptr, int x, int y, int dx, int dy, Uint32 color)
{
    draw_put_pixel(surface_ptr, x + dx, y + dy, color);
    draw_put_pixel(surface_ptr, x - dx, y + dy, color);
    draw_put_pixel(surface_ptr, x - dx, y - dy, color);
    draw_put_pixel(surface_ptr, x + dx, y - dy, color);
    draw_put_pixel(surface_ptr, x + dy, y + dx, color);
    draw_put_pixel(surface_ptr, x - dy, y + dx, color);
    draw_put_pixel(surface_ptr, x - dy, y - dx, color);
    draw_put_pixel(surface_ptr, x + dy, y - dx, color);
}

void draw_rotate_scale_surface(SDL_Surface* dest_surface_ptr, SDL_Surface* src_surface_ptr, float angle, float x_scale, float y_scale)
{
    int sx;
    int sy;
    float sx_dx = cos(-angle) * (1.0 / x_scale);
    float sx_dy = sin(-angle) * (1.0 / y_scale);
    float sy_dx = -sin(-angle) * (1.0 / x_scale);
    float sy_dy = cos(-angle) * (1.0 / y_scale);
    for (int dy = 0; dy < dest_surface_ptr->h; dy++)
    {
        for (int dx = 0; dx < dest_surface_ptr->w; dx++)
        {
            sx = dx * sx_dx + dy * sx_dy;
            sy = dx * sy_dx + dy * sy_dy;
            draw_put_pixel(dest_surface_ptr, dx, dy, draw_get_pixel(src_surface_ptr, sx, sy));
        }
    }
}
#endif
