#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>

#include "framework.h"
#include "runtime.h"

//--------------------SURFACE--------------------

//--------------------FONT--------------------

//--------------------COLOR--------------------

color_t color_black =           { 0x00, 0x00, 0x00 };
color_t color_dark_blue =       { 0x00, 0x00, 0x80 };
color_t color_blue =            { 0x00, 0x00, 0xff };
color_t color_dark_green =      { 0x00, 0x80, 0x00 };
color_t color_dark_cyan =       { 0x00, 0x80, 0x80 };
color_t color_blue_08f =        { 0x00, 0x80, 0xff };
color_t color_green =           { 0x00, 0xff, 0x00 };
color_t color_green_0f8 =       { 0x00, 0xff, 0x80 };
color_t color_cyan =            { 0x00, 0xff, 0xff };
color_t color_dark_red =        { 0x80, 0x00, 0x00 };
color_t color_purple =          { 0x80, 0x00, 0x80 };
color_t color_blue_80f =        { 0x80, 0x00, 0xff };
color_t color_dark_yellow =     { 0x80, 0x80, 0x00 };
color_t color_grey =            { 0x80, 0x80, 0x80 };
color_t color_light_blue =      { 0x80, 0x80, 0xff };
color_t color_green_8f0 =       { 0x80, 0xff, 0x00 };
color_t color_light_green =     { 0x80, 0xff, 0x80 };
color_t color_light_cyan =      { 0x80, 0xff, 0xff };
color_t color_red =             { 0xff, 0x00, 0x00 };
color_t color_red_f08 =         { 0xff, 0x00, 0x80 };
color_t color_magenta =         { 0xff, 0x00, 0xff };
color_t color_orange =          { 0xff, 0x80, 0x00 };
color_t color_light_red =       { 0xff, 0x80, 0x80 };
color_t color_light_magenta =   { 0xff, 0x80, 0xff };
color_t color_yellow =          { 0xff, 0xff, 0x00 };
color_t color_light_yellow =    { 0xff, 0xff, 0x80 };
color_t color_white =           { 0xff, 0xff, 0xff };

//--------------------INPUTSTATE--------------------

input_state_ptr_t input_state_alloc(void)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] input_state_alloc()\n");
    
    input_state_ptr_t input_state_ptr = malloc(sizeof(input_state_t));
    input_state_ptr->event = event_none;
    input_state_ptr->x = 0;
    input_state_ptr->y = 0;
    input_state_ptr->button = 0;
    input_state_ptr->key = 0;
    input_state_ptr->mod_key = mod_key_none;
    return input_state_ptr;
}

void input_state_free(input_state_ptr_t* input_state_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] input_state_free()\n");

    input_state_ptr_t input_state_ptr = *(input_state_ptr_ptr);
    // free input_state
    free(input_state_ptr);
    *(input_state_ptr_ptr) = NULL;
}

//--------------------IMAGE--------------------

image_ptr_t image_alloc_file(char* file_name_ptr, color_ptr_t color_key_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] image_alloc_file()\n");
    
    image_ptr_t image_ptr = malloc(sizeof(image_t));
    image_ptr->surface_ptr = runtime_alloc_file_surface(file_name_ptr, color_key_ptr);
    return image_ptr;
}

image_ptr_t image_alloc_color(color_ptr_t color_ptr, int width, int height, color_ptr_t color_key_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] image_alloc_file()\n");
    
    image_ptr_t image_ptr = malloc(sizeof(image_t));
    image_ptr->surface_ptr = runtime_alloc_color_surface(color_ptr, width, height, color_key_ptr);
    return image_ptr;
}

void image_free(image_ptr_t* image_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] image_free()\n");

    image_ptr_t image_ptr = *(image_ptr_ptr);
    // free surface
    if (image_ptr->surface_ptr != NULL)
    {
        runtime_free_surface(&(image_ptr->surface_ptr));
    }
    // free image
    free(image_ptr);
    *(image_ptr_ptr) = NULL;
}

void image_scale(image_ptr_t image_ptr, int width, int height)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] image_scale()\n");
    
    surface_ptr_t unscaled_surface_ptr = image_ptr->surface_ptr;
    surface_ptr_t scaled_surface_ptr = runtime_scale_surface(image_ptr->surface_ptr, width, height);
    image_ptr->surface_ptr = scaled_surface_ptr;
    runtime_free_surface(&unscaled_surface_ptr);
}

//--------------------BACKGROUND--------------------

background_ptr_t background_alloc_file(char* file_name_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] background_alloc_file()\n");
    
    background_ptr_t background_ptr = malloc(sizeof(background_t));
    background_ptr->image_ptr = image_alloc_file(file_name_ptr, NULL);
    return background_ptr;
}

background_ptr_t background_alloc_color(color_ptr_t color_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] background_alloc_color()\n");
    
    background_ptr_t background_ptr = malloc(sizeof(background_t));
    background_ptr->image_ptr = image_alloc_color(color_ptr, runtime.screen_width, runtime.screen_height, &color_grey);
    return background_ptr;
}

void background_free(background_ptr_t* background_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] background_free()\n");

    background_ptr_t background_ptr = *(background_ptr_ptr);
    // free image
    if (background_ptr->image_ptr != NULL)
    {
        image_free(&(background_ptr->image_ptr));
    }
    // free background
    free(background_ptr);
    *(background_ptr_ptr) = NULL;
}

//--------------------SPRITE--------------------

void sprite_quicksort_swap(sprite_ptr_t* sprite_ptr_a_ptr, sprite_ptr_t* sprite_ptr_b_ptr);
void sprite_quicksort(sprite_ptr_t sprite_ptr_list[], int start, int end);

sprite_ptr_t sprite_alloc(char* file_name_ptr, color_ptr_t color_key_ptr, int x, int y, int z)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] sprite_alloc()\n");
    
    sprite_ptr_t sprite_ptr = malloc(sizeof(sprite_t));
    sprite_ptr->image_ptr = image_alloc_file(file_name_ptr, color_key_ptr);
    sprite_ptr->x = x;
    sprite_ptr->y = y;
    sprite_ptr->z = z;
    return sprite_ptr;
}

void sprite_free(sprite_ptr_t* sprite_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] sprite_free()\n");

    sprite_ptr_t sprite_ptr = *(sprite_ptr_ptr);
    // free image
    if (sprite_ptr->image_ptr != NULL)
    {
        image_free(&(sprite_ptr->image_ptr));
    }
    // free sprite
    free(sprite_ptr);
    *(sprite_ptr_ptr) = NULL;
}

void sprite_quicksort_swap(sprite_ptr_t* sprite_ptr_a_ptr, sprite_ptr_t* sprite_ptr_b_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] sprite_quicksort_swap()\n");
    
    sprite_ptr_t temp_sprite_ptr = *(sprite_ptr_a_ptr);
    *(sprite_ptr_a_ptr) = *(sprite_ptr_b_ptr);
    *(sprite_ptr_b_ptr) = temp_sprite_ptr;
}

void sprite_quicksort(sprite_ptr_t sprite_ptr_list[], int start, int end)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] sprite_quicksort()\n");
    
    int key, i, j, pivot;
    if (start < end)
    {
        pivot = (start + end) / 2;
        sprite_quicksort_swap(&sprite_ptr_list[start], &sprite_ptr_list[pivot]);
        key = sprite_ptr_list[start]->z;
        i = start + 1;
        j = end;
        while (i < j)
        {
            while ((i <= end) && (sprite_ptr_list[i]->z <= key))
            {
                i++;
            }
            while ((j >= start) && (sprite_ptr_list[j]->z > key))
            {
                j--;
            }
            if (i < j)
            {
                sprite_quicksort_swap(&sprite_ptr_list[i], &sprite_ptr_list[j]);
            }
        }
        sprite_quicksort_swap(&sprite_ptr_list[start], &sprite_ptr_list[j]);
        sprite_quicksort(sprite_ptr_list, start, j - 1);
        sprite_quicksort(sprite_ptr_list, j + 1, end);
    }
}

//--------------------TEXT--------------------

void text_quicksort_swap(text_ptr_t* text_ptr_a_ptr, text_ptr_t* text_ptr_b_ptr);
void text_quicksort(text_ptr_t text_ptr_list[], int start, int end);

text_ptr_t text_alloc(char* font_name_ptr, int font_size, color_ptr_t color_key_ptr, char* string_ptr, int x, int y, int z)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] text_alloc()\n");
    
    text_ptr_t text_ptr = malloc(sizeof(text_t));
    text_ptr->surface_ptr = runtime_alloc_font_surface(font_name_ptr, font_size, color_key_ptr, string_ptr);
    text_ptr->x = x;
    text_ptr->y = y;
    text_ptr->z = z;
    return text_ptr;
}

void text_free(text_ptr_t* text_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] text_free()\n");

    text_ptr_t text_ptr = *(text_ptr_ptr);
    // free surface
    if (text_ptr->surface_ptr != NULL)
    {
        runtime_free_surface(&(text_ptr->surface_ptr));
    }
    // free text
    free(text_ptr);
    *(text_ptr_ptr) = NULL;
}

void text_set_string(text_ptr_t text_ptr, char* font_name_ptr, int font_size, color_ptr_t color_ptr, char* string_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] text_set()\n");

    if (text_ptr->surface_ptr != NULL)
    {
        runtime_free_surface(&(text_ptr->surface_ptr));
    }
    text_ptr->surface_ptr = runtime_alloc_font_surface(font_name_ptr, font_size, color_ptr, string_ptr);
}

void text_quicksort_swap(text_ptr_t* text_ptr_a_ptr, text_ptr_t* text_ptr_b_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] text_quicksort_swap()\n");
    
    text_ptr_t temp_text_ptr = *(text_ptr_a_ptr);
    *(text_ptr_a_ptr) = *(text_ptr_b_ptr);
    *(text_ptr_b_ptr) = temp_text_ptr;
}

void text_quicksort(text_ptr_t text_ptr_list[], int start, int end)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] text_quicksort()\n");
    
    int key, i, j, pivot;
    if (start < end)
    {
        pivot = (start + end) / 2;
        text_quicksort_swap(&text_ptr_list[start], &text_ptr_list[pivot]);
        key = text_ptr_list[start]->z;
        i = start + 1;
        j = end;
        while (i < j)
        {
            while ((i <= end) && (text_ptr_list[i]->z <= key))
            {
                i++;
            }
            while ((j >= start) && (text_ptr_list[j]->z > key))
            {
                j--;
            }
            if (i < j)
            {
                text_quicksort_swap(&text_ptr_list[i], &text_ptr_list[j]);
            }
        }
        text_quicksort_swap(&text_ptr_list[start], &text_ptr_list[j]);
        text_quicksort(text_ptr_list, start, j - 1);
        text_quicksort(text_ptr_list, j + 1, end);
    }
}

//--------------------MENUTASK--------------------

//--------------------MENUITEM--------------------

surface_ptr_t menu_item_surface(menu_item_ptr_t menu_item_ptr);

menu_item_ptr_t menu_item_text_alloc(char* string_ptr, text_ptr_t text_ptr, int key, mod_key_t mod_key, menu_item_task_fptr_t menu_item_task_fptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_item_text_alloc()\n");
    
    menu_item_ptr_t menu_item_ptr = malloc(sizeof(menu_item_t));
    menu_item_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(menu_item_ptr->string_ptr, string_ptr);
    menu_item_ptr->text_ptr = text_ptr;
    menu_item_ptr->image_ptr = NULL;
    menu_item_ptr->key = key;
    menu_item_ptr->mod_key = mod_key;
    menu_item_ptr->menu_item_task_fptr = menu_item_task_fptr;
    return menu_item_ptr;
}

menu_item_ptr_t menu_item_image_alloc(char* string_ptr, image_ptr_t image_ptr, int key, mod_key_t mod_key, menu_item_task_fptr_t menu_item_task_fptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_item_image_alloc()\n");
    
    menu_item_ptr_t menu_item_ptr = malloc(sizeof(menu_item_t));
    menu_item_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(menu_item_ptr->string_ptr, string_ptr);
    menu_item_ptr->text_ptr = NULL;
    menu_item_ptr->image_ptr = image_ptr;
    menu_item_ptr->key = key;
    menu_item_ptr->mod_key = mod_key;
    menu_item_ptr->menu_item_task_fptr = menu_item_task_fptr;
    return menu_item_ptr;
}

void menu_item_free(menu_item_ptr_t* menu_item_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_item_free()\n");

    menu_item_ptr_t menu_item_ptr = *(menu_item_ptr_ptr);
    // free string
    if (menu_item_ptr->string_ptr != NULL)
    {
        free(menu_item_ptr->string_ptr);
    }
    // free text and image
    if (menu_item_ptr->text_ptr != NULL)
    {
        text_free(&(menu_item_ptr->text_ptr));
    }
    if (menu_item_ptr->image_ptr != NULL)
    {
        image_free(&(menu_item_ptr->image_ptr));
    }
    // free menu item
    free(menu_item_ptr);
    *(menu_item_ptr_ptr) = NULL;
}

void menu_item_task(menu_item_ptr_t menu_item_ptr, menu_ptr_t menu_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_item_task()\n");
    
    // call menu item's task through function pointer
    if (menu_item_ptr->menu_item_task_fptr != NULL)
    {
        menu_item_ptr->menu_item_task_fptr(menu_item_ptr, menu_ptr);
    }
    // at some point either a screen's input task or a menu item's menu task must
    // call game_set_screen(NULL) to exit game
}

surface_ptr_t menu_item_surface(menu_item_ptr_t menu_item_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_item_surface()\n");
    
    surface_ptr_t surface_ptr = NULL;
    if (menu_item_ptr->text_ptr != NULL)
    {
        surface_ptr = menu_item_ptr->text_ptr->surface_ptr;
    }
    if (menu_item_ptr->image_ptr != NULL)
    {
        surface_ptr = menu_item_ptr->image_ptr->surface_ptr;
    }
    return surface_ptr;
}

//--------------------MENU--------------------

surface_ptr_t menu_get_surface(menu_ptr_t menu_ptr);
void menu_size(menu_ptr_t menu_ptr, int* max_width, int* total_height, int* average_height, int* padding);

menu_ptr_t menu_text_alloc(char* string_ptr, text_ptr_t text_ptr, int key, mod_key_t mod_key, anchor_type_t anchor, int x, int y, bool open)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_text_alloc()\n");
    
    menu_ptr_t menu_ptr = malloc(sizeof(menu_t));
    menu_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(menu_ptr->string_ptr, string_ptr);
    menu_ptr->text_ptr = text_ptr;
    menu_ptr->image_ptr = NULL;
    menu_ptr->key = key;
    menu_ptr->mod_key = mod_key;
    menu_ptr->menu_item_ptr_array = NULL;
    menu_ptr->menu_item_ptr_array_count = 0;
    menu_ptr->anchor = anchor;
    menu_ptr->x = x;
    menu_ptr->y = y;
    menu_ptr->open = open;
    return menu_ptr;
}

menu_ptr_t menu_image_alloc(char* string_ptr, image_ptr_t image_ptr, int key, mod_key_t mod_key, anchor_type_t anchor, int x, int y, bool open)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_image_alloc()\n");
    
    menu_ptr_t menu_ptr = malloc(sizeof(menu_t));
    menu_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(menu_ptr->string_ptr, string_ptr);
    menu_ptr->text_ptr = NULL;
    menu_ptr->image_ptr = image_ptr;
    menu_ptr->key = key;
    menu_ptr->mod_key = mod_key;
    menu_ptr->menu_item_ptr_array = NULL;
    menu_ptr->menu_item_ptr_array_count = 0;
    menu_ptr->anchor = anchor;
    menu_ptr->x = x;
    menu_ptr->y = y;
    menu_ptr->open = open;
    return menu_ptr;
}

void menu_free(menu_ptr_t* menu_ptr_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_free()\n");

    menu_ptr_t menu_ptr = *(menu_ptr_ptr);
    // free string
    if (menu_ptr->string_ptr != NULL)
    {
        free(menu_ptr->string_ptr);
    }
    // free text and image
    if (menu_ptr->text_ptr != NULL)
    {
        text_free(&(menu_ptr->text_ptr));
    }
    if (menu_ptr->image_ptr != NULL)
    {
        image_free(&(menu_ptr->image_ptr));
    }
    // free menu item array
    if (menu_ptr->menu_item_ptr_array != NULL)
    {
        for (int index = 0; index < menu_ptr->menu_item_ptr_array_count; index++)
        {
            menu_item_free(&(menu_ptr->menu_item_ptr_array[index]));
        }
        free(menu_ptr->menu_item_ptr_array);
    }
    // free menu
    free(menu_ptr);
    *(menu_ptr_ptr) = NULL;
}

void menu_add_menu_item(menu_ptr_t menu_ptr, menu_item_ptr_t menu_item_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_add_menu_item()\n");
    
    if (menu_ptr->menu_item_ptr_array == NULL)
    {
        menu_ptr->menu_item_ptr_array = malloc(sizeof(menu_item_ptr_t));
        menu_ptr->menu_item_ptr_array[0] = menu_item_ptr;
        menu_ptr->menu_item_ptr_array_count = 1;
    }
    else
    {
        menu_item_ptr_t* old_menu_item_ptr_array = menu_ptr->menu_item_ptr_array;
        int old_menu_item_ptr_array_count = menu_ptr->menu_item_ptr_array_count;
        menu_ptr->menu_item_ptr_array = malloc(sizeof(menu_item_ptr_t) * (old_menu_item_ptr_array_count + 1));
        memcpy(menu_ptr->menu_item_ptr_array, old_menu_item_ptr_array, sizeof(menu_item_ptr_t) * old_menu_item_ptr_array_count);
        menu_ptr->menu_item_ptr_array[old_menu_item_ptr_array_count] = menu_item_ptr;
        menu_ptr->menu_item_ptr_array_count = old_menu_item_ptr_array_count + 1;
        free(old_menu_item_ptr_array);
    }
}

surface_ptr_t menu_get_surface(menu_ptr_t menu_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_surface()\n");
    
    surface_ptr_t surface_ptr = NULL;
    if (menu_ptr->text_ptr != NULL)
    {
        surface_ptr = menu_ptr->text_ptr->surface_ptr;
    }
    if (menu_ptr->image_ptr != NULL)
    {
        surface_ptr = menu_ptr->image_ptr->surface_ptr;
    }
    return surface_ptr;
}

void menu_size(menu_ptr_t menu_ptr, int* max_width, int* total_height, int* average_height, int* padding)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] menu_size()\n");
    
    *(max_width) = 0;
    *(total_height) = 0;
    *(average_height) = 0;
    *(padding) = 0;
    int menu_item_index;
    for (menu_item_index = 0; menu_item_index < menu_ptr->menu_item_ptr_array_count; menu_item_index++)
    {
        menu_item_ptr_t menu_item_ptr = menu_ptr->menu_item_ptr_array[menu_item_index];
        surface_ptr_t menu_item_surface_ptr = menu_item_surface(menu_item_ptr);
        int width = runtime_surface_width(menu_item_surface_ptr);
        int height = runtime_surface_height(menu_item_surface_ptr);
        if (width > *(max_width))
        {
            *(max_width) = width;
        }
        *(total_height) += height;
    }
    if (menu_ptr->menu_item_ptr_array_count > 0)
    {
        *(average_height) = *(total_height) / menu_ptr->menu_item_ptr_array_count;
    }
    *(padding) = *(average_height) / 4;
    *(total_height) += *(padding) * (menu_ptr->menu_item_ptr_array_count - 1);
}

//--------------------ALLOCTASK--------------------

//--------------------INITTASK--------------------

//--------------------INPUTTASK--------------------

//--------------------EXITTASK--------------------

//--------------------SCREEN--------------------

int screen_get_anchor_x(int width, int height, anchor_type_t anchor, int custom_x, int custom_y, int padding);
int screen_get_anchor_y(int width, int height, anchor_type_t anchor, int custom_x, int custom_y, int padding);

screen_ptr_t screen_alloc(char* string_ptr, alloc_task_fptr_t alloc_task_fptr, init_task_fptr_t init_task_fptr, input_task_fptr_t input_task_fptr, exit_task_fptr_t exit_task_fptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] screen_alloc()\n");

    screen_ptr_t screen_ptr = malloc(sizeof(screen_t));
    screen_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(screen_ptr->string_ptr, string_ptr);
    screen_ptr->background_ptr = NULL;
    screen_ptr->sprite_ptr_array = NULL;
    screen_ptr->sprite_ptr_array_count = 0;
    screen_ptr->z_sprite_ptr_array = NULL;
    screen_ptr->text_ptr_array = NULL;
    screen_ptr->text_ptr_array_count = 0;
    screen_ptr->z_text_ptr_array = NULL;
    screen_ptr->menu_ptr_array = NULL;
    screen_ptr->menu_ptr_array_count = 0;
    screen_ptr->alloc_task_fptr = alloc_task_fptr;
    screen_ptr->init_task_fptr = init_task_fptr;
    screen_ptr->input_task_fptr = input_task_fptr;
    screen_ptr->exit_task_fptr = exit_task_fptr;
    screen_alloc_task(screen_ptr);
    return screen_ptr;
}

void screen_free(screen_ptr_t* screen_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] screen_free()\n");

    screen_ptr_t screen_ptr = *(screen_ptr_ptr);
    // free string
    if (screen_ptr->string_ptr != NULL)
    {
        free(screen_ptr->string_ptr);
    }
    // free background
    if (screen_ptr->background_ptr != NULL)
    {
        background_free(&(screen_ptr->background_ptr));
    }
    // free sprite array
    if (screen_ptr->sprite_ptr_array != NULL)
    {
        for (int index = 0; index < screen_ptr->sprite_ptr_array_count; index++)
        {
            sprite_free(&(screen_ptr->sprite_ptr_array[index]));
        }
        free(screen_ptr->sprite_ptr_array);
    }
    if (screen_ptr->z_sprite_ptr_array != NULL)
    {
        free(screen_ptr->z_sprite_ptr_array);
    }
    // free text array
    if (screen_ptr->text_ptr_array != NULL)
    {
        for (int index = 0; index < screen_ptr->text_ptr_array_count; index++)
        {
            text_free(&(screen_ptr->text_ptr_array[index]));
        }
        free(screen_ptr->text_ptr_array);
    }
    if (screen_ptr->z_text_ptr_array != NULL)
    {
        free(screen_ptr->z_text_ptr_array);
    }
    // free menu array
    if (screen_ptr->menu_ptr_array != NULL)
    {
        for (int index = 0; index < screen_ptr->menu_ptr_array_count; index++)
        {
            menu_free(&(screen_ptr->menu_ptr_array[index]));
        }
        free(screen_ptr->menu_ptr_array);
    }
    // free screen
    free(screen_ptr);
    *(screen_ptr_ptr) = NULL;
}

void screen_add_sprite(screen_ptr_t screen_ptr, sprite_ptr_t sprite_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_add_sprite()\n");
    
    if (screen_ptr->sprite_ptr_array == NULL)
    {
        screen_ptr->sprite_ptr_array = malloc(sizeof(sprite_ptr_t));
        screen_ptr->sprite_ptr_array[0] = sprite_ptr;
        screen_ptr->sprite_ptr_array_count = 1;
        // create z sorted array
        screen_ptr->z_sprite_ptr_array = malloc(sizeof(sprite_ptr_t));
        screen_ptr->z_sprite_ptr_array[0] = sprite_ptr;
    }
    else
    {
        sprite_ptr_t* old_sprite_ptr_array = screen_ptr->sprite_ptr_array;
        int old_sprite_ptr_array_count = screen_ptr->sprite_ptr_array_count;
        screen_ptr->sprite_ptr_array = malloc(sizeof(sprite_ptr_t) * (old_sprite_ptr_array_count + 1));
        memcpy(screen_ptr->sprite_ptr_array, old_sprite_ptr_array, sizeof(sprite_ptr_t) * old_sprite_ptr_array_count);
        screen_ptr->sprite_ptr_array[old_sprite_ptr_array_count] = sprite_ptr;
        screen_ptr->sprite_ptr_array_count = old_sprite_ptr_array_count + 1;
        free(old_sprite_ptr_array);
        // create z sorted array
        free(screen_ptr->z_sprite_ptr_array);
        screen_ptr->z_sprite_ptr_array = malloc(sizeof(sprite_ptr_t) * screen_ptr->sprite_ptr_array_count);
        memcpy(screen_ptr->z_sprite_ptr_array, screen_ptr->sprite_ptr_array, sizeof(sprite_ptr_t) * screen_ptr->sprite_ptr_array_count);
        sprite_quicksort(screen_ptr->z_sprite_ptr_array, 0, screen_ptr->sprite_ptr_array_count - 1);
    }
}

void screen_add_text(screen_ptr_t screen_ptr, text_ptr_t text_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_add_text()\n");
    
    if (screen_ptr->text_ptr_array == NULL)
    {
        screen_ptr->text_ptr_array = malloc(sizeof(text_ptr_t));
        screen_ptr->text_ptr_array[0] = text_ptr;
        screen_ptr->text_ptr_array_count = 1;
        // create z sorted array
        screen_ptr->z_text_ptr_array = malloc(sizeof(text_ptr_t));
        screen_ptr->z_text_ptr_array[0] = text_ptr;
    }
    else
    {
        text_ptr_t* old_text_ptr_array = screen_ptr->text_ptr_array;
        int old_text_ptr_array_count = screen_ptr->text_ptr_array_count;
        screen_ptr->text_ptr_array = malloc(sizeof(text_ptr_t) * (old_text_ptr_array_count + 1));
        memcpy(screen_ptr->text_ptr_array, old_text_ptr_array, sizeof(text_ptr_t) * old_text_ptr_array_count);
        screen_ptr->text_ptr_array[old_text_ptr_array_count] = text_ptr;
        screen_ptr->text_ptr_array_count = old_text_ptr_array_count + 1;
        free(old_text_ptr_array);
        // create z sorted array
        free(screen_ptr->z_text_ptr_array);
        screen_ptr->z_text_ptr_array = malloc(sizeof(text_ptr_t) * screen_ptr->text_ptr_array_count);
        memcpy(screen_ptr->z_text_ptr_array, screen_ptr->text_ptr_array, sizeof(text_ptr_t) * screen_ptr->text_ptr_array_count);
        text_quicksort(screen_ptr->z_text_ptr_array, 0, screen_ptr->text_ptr_array_count - 1);
    }
}

void screen_add_menu(screen_ptr_t screen_ptr, menu_ptr_t menu_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_add_menu()\n");
    
    if (screen_ptr->menu_ptr_array == NULL)
    {
        screen_ptr->menu_ptr_array = malloc(sizeof(menu_ptr_t));
        screen_ptr->menu_ptr_array[0] = menu_ptr;
        screen_ptr->menu_ptr_array_count = 1;
    }
    else
    {
        menu_ptr_t* old_menu_ptr_array = screen_ptr->menu_ptr_array;
        int old_menu_ptr_array_count = screen_ptr->menu_ptr_array_count;
        screen_ptr->menu_ptr_array = malloc(sizeof(menu_ptr_t) * (old_menu_ptr_array_count + 1));
        memcpy(screen_ptr->menu_ptr_array, old_menu_ptr_array, sizeof(menu_ptr_t) * old_menu_ptr_array_count);
        screen_ptr->menu_ptr_array[old_menu_ptr_array_count] = menu_ptr;
        screen_ptr->menu_ptr_array_count = old_menu_ptr_array_count + 1;
        free(old_menu_ptr_array);
    }
}

// called when screen is added
void screen_alloc_task(screen_ptr_t screen_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_alloc_task()\n");
    
    // call screen's alloc through function pointer
    if (screen_ptr->alloc_task_fptr != NULL)
    {
        screen_ptr->alloc_task_fptr(screen_ptr);
    }
}

// called when active screen is set
void screen_init_task(screen_ptr_t screen_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_init_task()\n");
    
    // call screen's init through function pointer
    if (screen_ptr->init_task_fptr != NULL)
    {
        screen_ptr->init_task_fptr(screen_ptr);
    }
}

void screen_render_frame(screen_ptr_t screen_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_render_frame()\n");
    // background
    if (screen_ptr->background_ptr != NULL)
    {
        runtime_clear_background();
        runtime_blit_surface(screen_ptr->background_ptr->image_ptr->surface_ptr, 0, 0);
    }
    else
    {
        runtime_clear_background();
    }

    // sprites
    int sprite_index;
    for (sprite_index = 0; sprite_index < screen_ptr->sprite_ptr_array_count; sprite_index++)
    {
        sprite_ptr_t sprite_ptr = screen_ptr->z_sprite_ptr_array[sprite_index];
        if (sprite_ptr->z >= 0)
        {
            runtime_blit_surface(sprite_ptr->image_ptr->surface_ptr, sprite_ptr->x, sprite_ptr->y);
        }
    }

    // text
    int text_index;
    for (text_index = 0; text_index < screen_ptr->text_ptr_array_count; text_index++)
    {
        text_ptr_t text_ptr = screen_ptr->z_text_ptr_array[text_index];
        if (text_ptr->z >= 0)
        {
            runtime_blit_surface(text_ptr->surface_ptr, text_ptr->x, text_ptr->y);
        }
    }
    
    // menus
    int menu_index;
    for (menu_index = 0; menu_index < screen_ptr->menu_ptr_array_count; menu_index++)
    {
        menu_ptr_t menu_ptr = screen_ptr->menu_ptr_array[menu_index];
        if (menu_ptr->open == false)
        {
            // show menu title
            surface_ptr_t surface_ptr = menu_get_surface(menu_ptr);
            int width = runtime_surface_width(surface_ptr);
            int height = runtime_surface_height(surface_ptr);
            int padding = height / 4;
            int x = screen_get_anchor_x(width, height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            int y = screen_get_anchor_y(width, height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            runtime_blit_surface(surface_ptr, x, y);
        }
        else if (menu_ptr->menu_item_ptr_array_count > 0)   // menu is open
        {
            // show menu items
            // find max width, total height and average height
            int max_width = 0;
            int total_height = 0;
            int average_height = 0;
            int padding = 0;
            menu_size(menu_ptr, &max_width, &total_height, &average_height, &padding);
            // add menu items
            int x = screen_get_anchor_x(max_width, total_height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            int y = screen_get_anchor_y(max_width, total_height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            int menu_item_index;
            for (menu_item_index = 0; menu_item_index < menu_ptr->menu_item_ptr_array_count; menu_item_index++)
            {
                menu_item_ptr_t menu_item_ptr = menu_ptr->menu_item_ptr_array[menu_item_index];
                surface_ptr_t menu_item_surface_ptr = menu_item_surface(menu_item_ptr);
                int width = runtime_surface_width(menu_item_surface_ptr);
                int height = runtime_surface_height(menu_item_surface_ptr);
                int x_offset = 0;
                if (menu_ptr->anchor == anchor_top_right || menu_ptr->anchor == anchor_bottom_right)
                {
                    x_offset = max_width - width;
                }
                else if (menu_ptr->anchor == anchor_center)
                {
                    x_offset = (max_width - width) / 2;
                }
                runtime_blit_surface(menu_item_surface_ptr, x + x_offset, y);
                y += height + padding;
            }
        }
    }
}

void screen_input_task(screen_ptr_t screen_ptr, input_state_ptr_t input_state_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_input_task()\n");
    
    // handle menu interaction first
    bool menu_selected = false;
    for (int menu_index = 0; menu_index < screen_ptr->menu_ptr_array_count; menu_index++)
    {
        menu_ptr_t menu_ptr = screen_ptr->menu_ptr_array[menu_index];
        // has menu been clicked or shortcut key pressed?
        if (menu_ptr->open == false)
        {
            if (input_state_ptr->event == event_button_down && input_state_ptr->button == 1)
            {
                surface_ptr_t menu_surface_ptr = menu_get_surface(menu_ptr);
                int width = runtime_surface_width(menu_surface_ptr);
                int height = runtime_surface_height(menu_surface_ptr);
                int min_x = screen_get_anchor_x(width, height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, height / 4);
                int min_y = screen_get_anchor_y(width, height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, height / 4);
                int max_x = min_x + width;
                int max_y = min_y + height;
                if (input_state_ptr->x >= min_x && input_state_ptr->x <= max_x && input_state_ptr->y >= min_y && input_state_ptr->y <= max_y)
                {
                    /*** debug ***/ fprintf(stdout, "[Debug] menu string = %s\n", menu_ptr->string_ptr);
                    menu_ptr->open = true;
                    menu_selected = true;
                }
            }
            else if (input_state_ptr->event == event_key_down && input_state_ptr->key == menu_ptr->key && input_state_ptr->mod_key == menu_ptr->mod_key)
            {
                /*** debug ***/ fprintf(stdout, "[Debug] menu string = %s\n", menu_ptr->string_ptr);
                menu_ptr->open = true;
                menu_selected = true;
            }
        }
        else if (menu_ptr->menu_item_ptr_array_count > 0)   // menu is open
        {
            // find max width, total height and average height
            int max_width = 0;
            int total_height = 0;
            int average_height = 0;
            int padding = 0;
            menu_size(menu_ptr, &max_width, &total_height, &average_height, &padding);
            int min_x = screen_get_anchor_x(max_width, total_height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            int min_y = screen_get_anchor_y(max_width, total_height, menu_ptr->anchor, menu_ptr->x, menu_ptr->y, padding);
            int max_x = min_x + max_width;
            int max_y = 0;
            int menu_item_index;
            for (menu_item_index = 0; menu_item_index < menu_ptr->menu_item_ptr_array_count; menu_item_index++)
            {
                menu_item_ptr_t menu_item_ptr = menu_ptr->menu_item_ptr_array[menu_item_index];
                if (input_state_ptr->event == event_button_down && input_state_ptr->button == 1)
                {
                    surface_ptr_t menu_item_surface_ptr = menu_item_surface(menu_item_ptr);
                    int height = runtime_surface_height(menu_item_surface_ptr);
                    max_y = min_y + height;
                    if (input_state_ptr->x >= min_x && input_state_ptr->x <= max_x && input_state_ptr->y >= min_y && input_state_ptr->y <= max_y)
                    {
                        // call selected menu item's menu item task
                        menu_item_task(menu_item_ptr, menu_ptr);
                        menu_selected = true;
                    }
                    min_y += height + padding;
                }
                else if (input_state_ptr->event == event_key_down && input_state_ptr->key == menu_item_ptr->key && input_state_ptr->mod_key == menu_item_ptr->mod_key)
                {
                    // call selected menu item's menu item task
                    menu_item_task(menu_item_ptr, menu_ptr);
                    menu_selected = true;
                }
            }
        }
    }
    // handle other interaction
    if (menu_selected == false)
    {
        // call screen's input task through function pointer, passing in input state
        if (screen_ptr->input_task_fptr != NULL)
        {
            screen_ptr->input_task_fptr(screen_ptr, input_state_ptr);
        }
    }
    // at some point either a screen's input task or a menu item's menu item task must
    // call game_set_screen(NULL) to exit game
}

// called when active screen is set
void screen_exit_task(screen_ptr_t screen_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_exit_task()\n");
    
    // call screen's exit through function pointer
    if (screen_ptr->exit_task_fptr != NULL)
    {
        screen_ptr->exit_task_fptr(screen_ptr);
    }
}

int screen_get_anchor_x(int width, int height, anchor_type_t anchor, int custom_x, int custom_y, int padding)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_get_anchor_x()\n");
    
    int x;
    switch (anchor)
    {
        case anchor_custom:
            x = custom_x;
            break;
        case anchor_top_left:
            x = padding;
            break;
        case anchor_top_right:
            x = runtime.screen_width - width - padding;
            break;
        case anchor_bottom_left:
            x = padding;
            break;
        case anchor_bottom_right:
            x = runtime.screen_width - width - padding;
            break;
        case anchor_center:
            x = runtime.screen_width / 2 - width / 2;
            break;
    }
    return x;
}

int screen_get_anchor_y(int width, int height, anchor_type_t anchor, int custom_x, int custom_y, int padding)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] screen_get_anchor_y()\n");
    
    int y;
    switch (anchor)
    {
        case anchor_custom:
            y = custom_y;
            break;
        case anchor_top_left:
            y = padding;
            break;
        case anchor_top_right:
            y = padding;
            break;
        case anchor_bottom_left:
            y = runtime.screen_height - height - padding;
            break;
        case anchor_bottom_right:
            y = runtime.screen_height - height - padding;
            break;
        case anchor_center:
            y = runtime.screen_height / 2 - height / 2;
            break;
    }
    return y;
}

//--------------------GAME--------------------

game_t game;

void game_alloc(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_alloc()\n");
    
    game.screen_ptr = NULL;
    game.previous_screen_ptr = NULL;
    game.screen_ptr_array = NULL;
    game.screen_ptr_array_count = 0;
}

void game_free(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_free()\n");
    
    // free screens array
    if (game.screen_ptr_array != NULL)
    {
        for (int index = 0; index < game.screen_ptr_array_count; index++)
        {
            screen_free(&(game.screen_ptr_array[index]));
        }
        free(game.screen_ptr_array);
    }
}

void game_add_screen(screen_ptr_t screen_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] game_add_screen()\n");
    
    if (game.screen_ptr_array == NULL)
    {
        game.screen_ptr_array = malloc(sizeof(screen_ptr_t));
        game.screen_ptr_array[0] = screen_ptr;
        game.screen_ptr_array_count = 1;
    }
    else
    {
        screen_ptr_t* old_screen_ptr_array = game.screen_ptr_array;
        int old_screen_ptr_array_count = game.screen_ptr_array_count;
        game.screen_ptr_array = malloc(sizeof(screen_ptr_t) * (old_screen_ptr_array_count + 1));
        memcpy(game.screen_ptr_array, old_screen_ptr_array, sizeof(screen_ptr_t) * old_screen_ptr_array_count);
        game.screen_ptr_array[old_screen_ptr_array_count] = screen_ptr;
        game.screen_ptr_array_count = old_screen_ptr_array_count + 1;
        free(old_screen_ptr_array);
    }
}

void game_set_screen(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_set_screen()\n");
    
	// exit previous active screen
	if (game.screen_ptr != NULL)
	{
		screen_exit_task(game.screen_ptr);
	}
    game.previous_screen_ptr = game.screen_ptr;

	// set new active screen and initialize
    game.screen_ptr = screen_ptr;
    if (screen_ptr != NULL)
    {
    	screen_init_task(screen_ptr);
    }
}

/* game_pre_init to be implemented by game instance
void game_pre_init(void)
{
    // call framework_set_resolution
}
*/

/* game_init to be implemented by game instance
void game_init(void)
{
    // create at least one screen_t and set its alloc, init, input and exit tasks
    // add it to the screens array
    // call game_set_screen for the first screen

    // allocate model resources
    // ... model_alloc(&game.model_ptr);
    // ... -or- model_alloc(); game.model_ptr = &model;
}
*/

/* game_exit to be implemented by game instance
void game_exit(void)
{
    // free model resources
    // ... model_free(&game.model_ptr);
    // ... -or- model_free(); game.model_ptr = NULL;

    // game_free called by framework
    // screens are freed by game_free
}
*/

//--------------------FRAMEWORK--------------------

framework_t framework;

// called by runtime init after graphics init
void framework_alloc(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_alloc()\n");
    
    game_alloc();
}

// called by runtime
void framework_free(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_free()\n");
    
    game_free();
}

// called by runtime after graphics init but before runtime_screen_open
// screen width, screen height and fps must be set before this function exits
void framework_pre_init(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_pre_init()\n");
    
    game_pre_init();
}

// called by runtime after runtime_screen_open
void framework_init(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_init()\n");

    game_init();
}

// called by runtime before graphics exit
void framework_exit(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_exit()\n");
    
    game_exit();
}

// called by runtime to get active screen
// return NULL to cause runtime to exit
screen_ptr_t framework_get_screen(void)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] framework_get_screen()\n");
    
    return game.screen_ptr;
}

// called by runtime each frame
// called before input task is called
void framework_render_frame(void)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] framework_render_frame()\n");
    
    screen_render_frame(game.screen_ptr);
}

// called by runtime each frame
// called after render frame is called
void framework_input_task(input_state_ptr_t input_state_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] framework_input_task()\n");
    
    // call active screen's input task
    screen_input_task(game.screen_ptr, input_state_ptr);
}

// expected to be called by game_init
void framework_set_resolution(resolution_t resolution)
{
    /*** debug ***/ fprintf(stdout, "[Trace] framework_set_resolution()\n");
    
    switch (resolution)
    {
        case resolution_480_320:
            runtime.screen_width = 480;
            runtime.screen_height = 320;
            runtime.fps = 60;
            break;
        case resolution_960_640:
            runtime.screen_width = 960;
            runtime.screen_height = 640;
            runtime.fps = 60;
            break;
        case resolution_1024_768:
            runtime.screen_width = 1024;
            runtime.screen_height = 768;
            runtime.fps = 60;
            break;
        case resolution_1280_720:
            runtime.screen_width = 1280;
            runtime.screen_height = 720;
            runtime.fps = 60;
            break;
        case resolution_1920_1080:
            runtime.screen_width = 1920;
            runtime.screen_height = 1080;
            runtime.fps = 60;
            break;
    }
}
