//
//  framework.h
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 11/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#ifndef FrameworkPrototype_framework_h
#define FrameworkPrototype_framework_h

//--------------------SURFACE--------------------

typedef struct surface_t surface_t;
typedef surface_t* surface_ptr_t;

//--------------------COLOR--------------------

typedef struct
{
    int r;
    int g;
    int b;
}
color_t;

typedef color_t* color_ptr_t;

extern color_t color_black;
extern color_t color_dark_blue;
extern color_t color_blue;
extern color_t color_dark_green;
extern color_t color_dark_cyan;
extern color_t color_blue_08f;
extern color_t color_green;
extern color_t color_green_0f8;
extern color_t color_cyan;
extern color_t color_dark_red;
extern color_t color_purple;
extern color_t color_blue_80f;
extern color_t color_dark_yellow;
extern color_t color_grey;
extern color_t color_light_blue;
extern color_t color_green_8f0;
extern color_t color_light_green;
extern color_t color_light_cyan;
extern color_t color_red;
extern color_t color_red_f08;
extern color_t color_magenta;
extern color_t color_orange;
extern color_t color_light_red;
extern color_t color_light_magenta;
extern color_t color_yellow;
extern color_t color_light_yellow;
extern color_t color_white;

//--------------------INPUTSTATE--------------------

typedef enum
{
    event_none = 0,
    event_key_down,
    event_key_up,
    event_button_down,
    event_button_up,
    event_move
}
input_event_t;

typedef enum
{
    mod_key_none = 0,
    mod_key_shift = 1,
    mod_key_control = 2,
    mod_key_alt = 3,
    mod_key_option = 3,
    mod_key_command = 4
}
mod_key_t;

typedef struct
{
    input_event_t event;
    int x;
    int y;
    int button;
    int key;
    mod_key_t mod_key;
}
input_state_t;

typedef input_state_t* input_state_ptr_t;

input_state_ptr_t input_state_alloc(void);
void input_state_free(input_state_ptr_t* input_state_ptr_ptr);

//--------------------IMAGE--------------------

typedef struct
{
    surface_ptr_t surface_ptr;
}
image_t;

typedef image_t* image_ptr_t;

image_ptr_t image_alloc_file(char* file_name_ptr, color_ptr_t color_key_ptr);
image_ptr_t image_alloc_color(color_ptr_t color_ptr, int width, int height, color_ptr_t color_key_ptr);
void image_free(image_ptr_t* image_ptr_ptr);

void image_scale(image_ptr_t image_ptr, int width, int height);

//--------------------BACKGROUND--------------------

typedef struct
{
    image_ptr_t image_ptr;
}
background_t;

typedef background_t* background_ptr_t;

background_ptr_t background_alloc_file(char* file_name_ptr);
background_ptr_t background_alloc_color(color_ptr_t color_ptr);
void background_free(background_ptr_t* background_ptr_ptr);

//--------------------SPRITE--------------------

typedef struct
{
    image_ptr_t image_ptr;
    int x;
    int y;
    int z;
}
sprite_t;

typedef sprite_t* sprite_ptr_t;

sprite_ptr_t sprite_alloc(char* file_name_ptr, color_ptr_t color_key_ptr, int x, int y, int z);
void sprite_free(sprite_ptr_t* sprite_ptr_ptr);

//--------------------TEXT--------------------

typedef struct
{
    surface_ptr_t surface_ptr;
    int x;
    int y;
    int z;
}
text_t;

typedef text_t* text_ptr_t;

text_ptr_t text_alloc(char* font_name_ptr, int font_size, color_ptr_t color_key_ptr, char* string_ptr, int x, int y, int z);
void text_free(text_ptr_t* text_ptr_ptr);

void text_set_string(text_ptr_t text_ptr, char* font_name_ptr, int font_size, color_ptr_t color_ptr, char* string_ptr);

//--------------------MENUITEMTASK--------------------

typedef struct menu_item_t menu_item_t;
typedef menu_item_t* menu_item_ptr_t;
typedef struct menu_t menu_t;
typedef menu_t* menu_ptr_t;

typedef void (*menu_item_task_fptr_t) (menu_item_ptr_t menu_item_ptr, menu_ptr_t menu_ptr);   // function pointer

// at some point either a screen's input task or a menu item's menu task must
// call game_set_screen(NULL) to exit game

//--------------------MENUITEM--------------------

struct menu_item_t
{
    char* string_ptr;
    text_ptr_t text_ptr;
    image_ptr_t image_ptr;
    int key;
    mod_key_t mod_key;
    menu_item_task_fptr_t menu_item_task_fptr;  // function pointer
};
// typedef menu_item_t forward declared

menu_item_ptr_t menu_item_text_alloc(char* string_ptr, text_ptr_t text_ptr, int key, mod_key_t mod_key, menu_item_task_fptr_t menu_item_task_fptr);
menu_item_ptr_t menu_item_image_alloc(char* string_ptr, image_ptr_t image_ptr, int key, mod_key_t mod_key, menu_item_task_fptr_t menu_item_task_fptr);
void menu_item_free(menu_item_ptr_t* menu_item_ptr_ptr);

void menu_item_task(menu_item_ptr_t menu_item_ptr, menu_ptr_t menu_ptr);

//--------------------MENU--------------------

typedef enum
{
    anchor_custom = 0,
    anchor_top_left,
    anchor_top_right,
    anchor_bottom_left,
    anchor_bottom_right,
    anchor_center
}
anchor_type_t;

struct menu_t
{
    char* string_ptr;
    text_ptr_t text_ptr;
    image_ptr_t image_ptr;
    int key;
    mod_key_t mod_key;
    menu_item_ptr_t* menu_item_ptr_array;   // array of pointers
    int menu_item_ptr_array_count;          // size of array
    anchor_type_t anchor;
    int x;
    int y;
    bool open;
};
// typedef menu_t forward declared

menu_ptr_t menu_text_alloc(char* string_ptr, text_ptr_t text_ptr, int key, mod_key_t mod_key, anchor_type_t anchor, int x, int y, bool open);
menu_ptr_t menu_image_alloc(char* string_ptr, image_ptr_t image_ptr, int key, mod_key_t mod_key, anchor_type_t anchor, int x, int y, bool open);
void menu_free(menu_ptr_t* menu_ptr_ptr);

void menu_add_menu_item(menu_ptr_t menu_ptr, menu_item_ptr_t menu_item_ptr);

//--------------------ALLOCTASK--------------------

typedef struct screen_t screen_t;

typedef screen_t* screen_ptr_t;

typedef void (*alloc_task_fptr_t) (screen_ptr_t screen_ptr);    // function pointer

//--------------------INITTASK--------------------

typedef void (*init_task_fptr_t) (screen_ptr_t screen_ptr);     // function pointer

//--------------------INPUTTASK--------------------

typedef void (*input_task_fptr_t) (screen_ptr_t screen_ptr, input_state_ptr_t input_state_ptr);    // function pointer

// at some point either a screen's input task or a menu item's menu task must
// call game_set_screen(NULL) to exit game

//--------------------EXITTASK--------------------

typedef void (*exit_task_fptr_t) (screen_ptr_t screen_ptr);     // function pointer

//--------------------SCREEN--------------------

struct screen_t
{
    char* string_ptr;
    background_ptr_t background_ptr;
    sprite_ptr_t* sprite_ptr_array;     // array of pointers
    int sprite_ptr_array_count;         // size of array
    sprite_ptr_t* z_sprite_ptr_array;   // array of pointers, sorted by z
    text_ptr_t* text_ptr_array;         // array of pointers
    int text_ptr_array_count;           // size of array
    text_ptr_t* z_text_ptr_array;       // array of pointers, sorted by z
    menu_ptr_t* menu_ptr_array;         // array of pointers
    int menu_ptr_array_count;           // size of array
    alloc_task_fptr_t alloc_task_fptr;  // function pointer
    init_task_fptr_t init_task_fptr;    // function pointer
    input_task_fptr_t input_task_fptr;  // function pointer
    exit_task_fptr_t exit_task_fptr;    // function pointer
};
// typedef screen_t forward declared

screen_ptr_t screen_alloc(char* string_ptr, alloc_task_fptr_t alloc_task_fptr, init_task_fptr_t init_task_fptr, input_task_fptr_t input_task_fptr, exit_task_fptr_t exit_task_fptr);
void screen_free(screen_ptr_t* screen_ptr_ptr);

void screen_add_sprite(screen_ptr_t screen_ptr, sprite_ptr_t sprite_ptr);
void screen_add_text(screen_ptr_t screen_ptr, text_ptr_t text_ptr);
void screen_add_menu(screen_ptr_t screen_ptr, menu_ptr_t menu_ptr);

void screen_alloc_task(screen_ptr_t screen_ptr);
void screen_init_task(screen_ptr_t screen_ptr);
void screen_render_frame(screen_ptr_t screen_ptr);
void screen_input_task(screen_ptr_t screen_ptr, input_state_ptr_t input_state_ptr);
void screen_exit_task(screen_ptr_t screen_ptr);

//--------------------GAME--------------------

typedef enum
{
    resolution_480_320 = 1, // 3:2
    resolution_960_640,     // 3:2
    resolution_1024_768,    // 4:3
    resolution_1280_720,    // 16:9
    resolution_1920_1080    // 16:9
}
resolution_t;

typedef struct
{
    screen_ptr_t screen_ptr;            // active screen - do not free
    screen_ptr_t previous_screen_ptr;   // previous screen - do not free
    screen_ptr_t* screen_ptr_array;     // array of pointers - owns
    int screen_ptr_array_count;         // size of array
}
game_t;

typedef game_t* game_ptr_t;

extern game_t game;

void game_alloc(void);
void game_free(void);
void game_add_screen(screen_ptr_t screen_ptr);

void game_set_screen(screen_ptr_t screen_ptr);

// to be implemented by game instance

void game_pre_init(void);
void game_init(void);
void game_exit(void);

//--------------------FRAMEWORK--------------------

typedef struct
{
}
framework_t;

extern framework_t framework;

typedef framework_t* framework_ptr_t;

void framework_alloc(void);
void framework_free(void);

// directly called by runtime
void framework_pre_init(void);
void framework_init(void);
void framework_exit(void);
screen_ptr_t framework_get_screen(void);
void framework_render_frame(void);
void framework_input_task(input_state_ptr_t input_state_ptr);

// called by other parts of framework (game_pre_init)
void framework_set_resolution(resolution_t resolution);

#endif
