//
//  game.c
//  GameSwampSquid
//
//  Created by Robert Hoferkamp on 12/17/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include <stdbool.h>

#include "framework.h"
#include "runtime.h"
#include "game.h"
#include "title_screen.h"
#include "game_screen.h"
#include "load_screen.h"
#include "save_screen.h"
#include "options_screen.h"
#include "model.h"

//-- game

void game_pre_init(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_pre_init()\n");
    
    // call framework_set_resolution
    framework_set_resolution(resolution_1280_720);

    // set resource directory
    runtime_set_resource_dir(RESOURCE_DIR);
}

void game_init(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_init()\n");
    
    // create title screen_t and set its alloc, init, input and exit tasks
    title_screen_ptr = screen_alloc("title", title_screen_alloc, title_screen_init, title_screen_input_task, title_screen_exit);
    // add it to the screens array
    game_add_screen(title_screen_ptr);
    
    // create game screen_t and set its alloc, init, input and exit tasks
    game_screen_ptr = screen_alloc("game", game_screen_alloc, game_screen_init, game_screen_input_task, game_screen_exit);
    // add it to the screens array
    game_add_screen(game_screen_ptr);
    
    // create load screen_t and set its alloc, init, input and exit tasks
    load_screen_ptr = screen_alloc("load", load_screen_alloc, load_screen_init, load_screen_input_task, load_screen_exit);
    // add it to the screens array
    game_add_screen(load_screen_ptr);
    
    // create save screen_t and set its alloc, init, input and exit tasks
    save_screen_ptr = screen_alloc("save", save_screen_alloc, save_screen_init, save_screen_input_task, save_screen_exit);
    // add it to the screens array
    game_add_screen(save_screen_ptr);
    
    // create options screen_t and set its alloc, init, input and exit tasks
    options_screen_ptr = screen_alloc("options", options_screen_alloc, options_screen_init, options_screen_input_task, options_screen_exit);
    // add it to the screens array
    game_add_screen(options_screen_ptr);
    
    // call game_set_screen for the first screen
    game_set_screen(game.screen_ptr_array[TITLE_SCREEN_INDEX]);
    
    // allocate model resources
    // model_alloc is called by title screen
    model_ptr = NULL;
}

void game_exit(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_exit()\n");
    
    // free model resources
    if (model_ptr != NULL)
    {
        model_free(&model_ptr);
    }
    
    // game_free called by framework
    // screens are freed by game_free
}
