//
//  game_screen.c
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 12/17/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include <stdbool.h>
#include <string.h>

#include "framework.h"
#include "runtime.h"
#include "game.h"
#include "game_screen.h"
#include "model.h"

#define SYSTEM_MENU_INDEX 0

screen_ptr_t game_screen_ptr;

void game_screen_get_area_paths(actor_ptr_t actor_ptr, area_ptr_t area_ptr, area_ptr_t* left_area_ptr_ptr, area_ptr_t* right_area_ptr_ptr, area_ptr_t* back_area_ptr_ptr);

void game_screen_alloc(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_screen_alloc()\n");
    
    int x_res = runtime.screen_width;
    int y_res = runtime.screen_height;
    
    // create trace text
    text_ptr_t trace_text_ptr = text_alloc("DejaVuSans", 24, &color_white, "Game", 0, 0, 0);
    trace_text_ptr->x = x_res / 2 - runtime_surface_width(trace_text_ptr->surface_ptr) / 2;
    trace_text_ptr->y = runtime_surface_height(trace_text_ptr->surface_ptr) / 4;
    screen_add_text(screen_ptr, trace_text_ptr);
    
    // create a menu [SYSTEM_MENU_INDEX]
    menu_ptr_t menu_ptr = menu_image_alloc("system", image_alloc_file("gear_32.png", NULL), 27, mod_key_none, anchor_bottom_right, 0, 0, false);
    screen_add_menu(screen_ptr, menu_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_1_ptr = menu_item_text_alloc("save", text_alloc("DejaVuSans", 32, &color_white, "Save", 0, 0, 0), '1', mod_key_none, game_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_1_ptr);

    // create a menu item
    menu_item_ptr_t menu_item_2_ptr = menu_item_text_alloc("load", text_alloc("DejaVuSans", 32, &color_white, "Load", 0, 0, 0), '2', mod_key_none, game_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_2_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_3_ptr = menu_item_text_alloc("options", text_alloc("DejaVuSans", 32, &color_white, "Options", 0, 0, 0), '3', mod_key_none, game_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_3_ptr);

    // create a menu item
    menu_item_ptr_t menu_item_4_ptr = menu_item_text_alloc("title", text_alloc("DejaVuSans", 32, &color_white, "Title", 0, 0, 0), '4', mod_key_none, game_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_4_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_5_ptr = menu_item_text_alloc("cancel", text_alloc("DejaVuSans", 32, &color_white, "Cancel", 0, 0, 0), '5', mod_key_none, game_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_5_ptr);

    // debug
    text_ptr_t area_text_ptr = text_alloc("DejaVuSans", 128, &color_white, " ", 0, 0, 0);                               // AREA_TEXT_INDEX
    screen_add_text(screen_ptr, area_text_ptr);
    text_ptr_t left_area_text_ptr = text_alloc("DejaVuSans", 128, &color_white, " ", 0, 0, 0);     // LEFT_AREA_TEXT_INDEX
    screen_add_text(screen_ptr, left_area_text_ptr);
    text_ptr_t right_area_text_ptr = text_alloc("DejaVuSans", 128, &color_white, " ", 0, 0, 0);   // RIGHT_AREA_TEXT_INDEX
    screen_add_text(screen_ptr, right_area_text_ptr);
    text_ptr_t back_area_text_ptr = text_alloc("DejaVuSans", 128, &color_white, " ", 0, 0, 0);     // BACK_AREA_TEXT_INDEX
    screen_add_text(screen_ptr, back_area_text_ptr);
    
    text_ptr_t squid_text_ptr = text_alloc("DejaVuSans", 48, &color_cyan, "swamp squid", 0, 0, -1);
    squid_text_ptr->x = x_res / 2 - runtime_surface_width(squid_text_ptr->surface_ptr) / 2;
    squid_text_ptr->y = y_res / 3 - runtime_surface_height(squid_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, squid_text_ptr);    // SQUID_TEXT_INDEX

    text_ptr_t wisp_text_ptr = text_alloc("DejaVuSans", 48, &color_yellow, "will-o-wisp", 0, 0, -1);
    wisp_text_ptr->x = x_res / 5 * 4 - runtime_surface_width(wisp_text_ptr->surface_ptr) / 2;
    wisp_text_ptr->y = y_res / 3 - runtime_surface_height(wisp_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, wisp_text_ptr);     // WISP_TEXT_INDEX

    text_ptr_t vines_text_ptr = text_alloc("DejaVuSans", 48, &color_green, "poison vines", 0, 0, -1);
    vines_text_ptr->x = x_res / 5 - runtime_surface_width(vines_text_ptr->surface_ptr) / 2;
    vines_text_ptr->y = y_res / 3 - runtime_surface_height(vines_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, vines_text_ptr);    // VINES_TEXT_INDEX

    text_ptr_t squid_near_text_ptr = text_alloc("DejaVuSans", 48, &color_light_cyan, "splash", 0, 0, -1);
    squid_near_text_ptr->x = x_res / 2 - runtime_surface_width(squid_near_text_ptr->surface_ptr) / 2;
    squid_near_text_ptr->y = y_res / 3 * 2 - runtime_surface_height(squid_near_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, squid_near_text_ptr);   // SQUID_NEAR_TEXT_INDEX

    text_ptr_t wisp_near_text_ptr = text_alloc("DejaVuSans", 48, &color_light_yellow, "glow", 0, 0, -1);
    wisp_near_text_ptr->x = x_res / 5 * 4 - runtime_surface_width(wisp_near_text_ptr->surface_ptr) / 2;
    wisp_near_text_ptr->y = y_res / 3 * 2- runtime_surface_height(wisp_near_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, wisp_near_text_ptr);    // WISP_NEAR_TEXT_INDEX

    text_ptr_t vines_near_text_ptr = text_alloc("DejaVuSans", 48, &color_light_green, "mist", 0, 0, -1);
    vines_near_text_ptr->x = x_res / 5 - runtime_surface_width(vines_near_text_ptr->surface_ptr) / 2;
    vines_near_text_ptr->y = y_res / 3 * 2- runtime_surface_height(vines_near_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, vines_near_text_ptr);   // VINES_NEAR_TEXT_INDEX
}

void game_screen_init(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_screen_init()\n");

    screen_ptr->menu_ptr_array[SYSTEM_MENU_INDEX]->open = false;
}

void game_screen_input_task(screen_ptr_t screen_ptr, input_state_ptr_t input_state_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] game_screen_input_task()\n");
    
    // handle input
    actor_ptr_t player_ptr = model_ptr->actor_ptr;
    area_ptr_t area_ptr = player_ptr->area_ptr;
    area_ptr_t left_area_ptr;
    area_ptr_t right_area_ptr;
    area_ptr_t back_area_ptr;
    game_screen_get_area_paths(player_ptr, area_ptr, &left_area_ptr, &right_area_ptr, &back_area_ptr);

    area_ptr_t next_area_ptr = NULL;
    if (input_state_ptr->event == event_button_down && input_state_ptr->button == 1 && input_state_ptr->x < runtime.screen_width / 4)
    {
        next_area_ptr = left_area_ptr;
    }
    else if (input_state_ptr->event == event_button_down && input_state_ptr->button == 1 && input_state_ptr->x > runtime.screen_width / 4 * 3)
    {
        next_area_ptr = right_area_ptr;
    }
    else if (input_state_ptr->event == event_button_down && input_state_ptr->button == 1 && input_state_ptr->y > runtime.screen_height / 4 * 3)
    {
        next_area_ptr = back_area_ptr;
    }
    if (next_area_ptr != NULL)
    {
        player_ptr->prev_area_ptr = player_ptr->area_ptr;
        player_ptr->area_ptr = next_area_ptr;
        area_remove_actor(player_ptr->prev_area_ptr, player_ptr);
        area_add_actor(player_ptr->area_ptr, player_ptr);
        area_ptr = player_ptr->area_ptr;
        game_screen_get_area_paths(player_ptr, area_ptr, &left_area_ptr, &right_area_ptr, &back_area_ptr);
    }

    // update screen objects based on model
    char* area_string = area_ptr->string_ptr;
    text_ptr_t area_text_ptr = screen_ptr->text_ptr_array[AREA_TEXT_INDEX];
    text_set_string(area_text_ptr, "DejaVuSans", 128, &color_white, area_string);
    area_text_ptr->x = runtime.screen_width / 2 - runtime_surface_width(area_text_ptr->surface_ptr) / 2;
    area_text_ptr->y = runtime_surface_height(area_text_ptr->surface_ptr) / 4;

    char* left_area_string = left_area_ptr->string_ptr;
    text_ptr_t left_area_text_ptr = screen_ptr->text_ptr_array[LEFT_AREA_TEXT_INDEX];
    text_set_string(left_area_text_ptr, "DejaVuSans", 128, &color_white, left_area_string);
    left_area_text_ptr->x = runtime_surface_height(left_area_text_ptr->surface_ptr) / 4;
    left_area_text_ptr->y = runtime.screen_height / 2 - runtime_surface_height(left_area_text_ptr->surface_ptr) / 2;
    
    char* right_area_string = right_area_ptr->string_ptr;
    text_ptr_t right_area_text_ptr = screen_ptr->text_ptr_array[RIGHT_AREA_TEXT_INDEX];
    text_set_string(right_area_text_ptr, "DejaVuSans", 128, &color_white, right_area_string);
    right_area_text_ptr->x = runtime.screen_width - runtime_surface_width(right_area_text_ptr->surface_ptr) - runtime_surface_height(right_area_text_ptr->surface_ptr) / 4;
    right_area_text_ptr->y = runtime.screen_height / 2 - runtime_surface_height(right_area_text_ptr->surface_ptr) / 2;
    
    char* back_area_string = back_area_ptr->string_ptr;
    text_ptr_t back_area_text_ptr = screen_ptr->text_ptr_array[BACK_AREA_TEXT_INDEX];
    text_set_string(back_area_text_ptr, "DejaVuSans", 128, &color_white, back_area_string);
    back_area_text_ptr->x = runtime.screen_width / 2 - runtime_surface_width(back_area_text_ptr->surface_ptr) / 2;
    back_area_text_ptr->y = runtime.screen_height - runtime_surface_height(back_area_text_ptr->surface_ptr) - runtime_surface_height(back_area_text_ptr->surface_ptr) / 4;

    text_ptr_t squid_text_ptr = screen_ptr->text_ptr_array[SQUID_TEXT_INDEX];
    if (area_has_actor_type(area_ptr, ACTOR_TYPE_SQUID))
    {
        squid_text_ptr->z = 0;
    }
    else
    {
        squid_text_ptr->z = -1;
    }
    text_ptr_t wisp_text_ptr = screen_ptr->text_ptr_array[WISP_TEXT_INDEX];
    if (area_has_actor_type(area_ptr, ACTOR_TYPE_WISP))
    {
        wisp_text_ptr->z = 0;
    }
    else
    {
        wisp_text_ptr->z = -1;
    }
    text_ptr_t vines_text_ptr = screen_ptr->text_ptr_array[VINES_TEXT_INDEX];
    if (area_has_actor_type(area_ptr, ACTOR_TYPE_VINES))
    {
        vines_text_ptr->z = 0;
    }
    else
    {
        vines_text_ptr->z = -1;
    }
    text_ptr_t squid_near_text_ptr = screen_ptr->text_ptr_array[SQUID_NEAR_TEXT_INDEX];
    if (area_has_actor_type(left_area_ptr, ACTOR_TYPE_SQUID) || area_has_actor_type(right_area_ptr, ACTOR_TYPE_SQUID) || area_has_actor_type(back_area_ptr, ACTOR_TYPE_SQUID))
    {
        squid_near_text_ptr->z = 0;
    }
    else
    {
        squid_near_text_ptr->z = -1;
    }
    text_ptr_t wisp_near_text_ptr = screen_ptr->text_ptr_array[WISP_NEAR_TEXT_INDEX];
    if (area_has_actor_type(left_area_ptr, ACTOR_TYPE_WISP) || area_has_actor_type(right_area_ptr, ACTOR_TYPE_WISP) || area_has_actor_type(back_area_ptr, ACTOR_TYPE_WISP))
    {
        wisp_near_text_ptr->z = 0;
    }
    else
    {
        wisp_near_text_ptr->z = -1;
    }
    text_ptr_t vines_near_text_ptr = screen_ptr->text_ptr_array[VINES_NEAR_TEXT_INDEX];
    if (area_has_actor_type(left_area_ptr, ACTOR_TYPE_VINES) || area_has_actor_type(right_area_ptr, ACTOR_TYPE_VINES) || area_has_actor_type(back_area_ptr, ACTOR_TYPE_VINES))
    {
        vines_near_text_ptr->z = 0;
    }
    else
    {
        vines_near_text_ptr->z = -1;
    }
}

void game_screen_get_area_paths(actor_ptr_t actor_ptr, area_ptr_t area_ptr, area_ptr_t* left_area_ptr_ptr, area_ptr_t* right_area_ptr_ptr, area_ptr_t* back_area_ptr_ptr)
{
    if (actor_ptr->prev_area_ptr == area_ptr->area_ptr_array[0])
    {
        *(left_area_ptr_ptr) = area_ptr->area_ptr_array[1];
        *(right_area_ptr_ptr) = area_ptr->area_ptr_array[2];
        *(back_area_ptr_ptr) = area_ptr->area_ptr_array[0];
    }
    else if (actor_ptr->prev_area_ptr == area_ptr->area_ptr_array[1])
    {
        *(left_area_ptr_ptr) = area_ptr->area_ptr_array[2];
        *(right_area_ptr_ptr) = area_ptr->area_ptr_array[0];
        *(back_area_ptr_ptr) = area_ptr->area_ptr_array[1];
    }
    else // if (actor_ptr->prev_area_ptr == area_ptr->area_ptr_array[2])
    {
        *(left_area_ptr_ptr) = area_ptr->area_ptr_array[0];
        *(right_area_ptr_ptr) = area_ptr->area_ptr_array[1];
        *(back_area_ptr_ptr) = area_ptr->area_ptr_array[2];
    }
}

void game_screen_menu_item_task(menu_item_ptr_t menu_item_ptr, menu_ptr_t menu_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_screen_menu_item_task()\n");
    
    /*** debug ***/ fprintf(stdout, "[Debug] menu item string = %s\n", menu_item_ptr->string_ptr);

    if (strcmp(menu_item_ptr->string_ptr, "save") == 0)
    {
        game_set_screen(game.screen_ptr_array[SAVE_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "load") == 0)
    {
        game_set_screen(game.screen_ptr_array[LOAD_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "options") == 0)
    {
        game_set_screen(game.screen_ptr_array[OPTIONS_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "title") == 0)
    {
        game_set_screen(game.screen_ptr_array[TITLE_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "cancel") == 0)
    {
        menu_ptr->open = false;
    }
}

void game_screen_exit(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] game_screen_exit()\n");
    
    // background, sprites, text and menus are freed by screen_free
}
