//
//  model.c
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 11/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>

#include "model.h"

//--------------------ACTOR--------------------

actor_ptr_t actor_alloc(string_ptr_t string_ptr, int type, area_ptr_t area_ptr, area_ptr_t prev_area_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] actor_alloc()\n");
    
    actor_ptr_t actor_ptr = malloc(sizeof(actor_t));
    actor_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(actor_ptr->string_ptr, string_ptr);
    actor_ptr->type = type;
    actor_ptr->area_ptr = area_ptr;
    actor_ptr->prev_area_ptr = prev_area_ptr;
    return actor_ptr;
}

void actor_free(actor_ptr_t* actor_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] actor_free()\n");
    
    actor_ptr_t actor_ptr = *(actor_ptr_ptr);
    // free string
    if (actor_ptr->string_ptr != NULL)
    {
        free(actor_ptr->string_ptr);
    }
    // area is not owned, therefore not freed
    // prev area is not owned, therefore not freed
    // free actor
    free(actor_ptr);
    *(actor_ptr_ptr) = NULL;
}

//--------------------AREA--------------------

area_ptr_t area_alloc(string_ptr_t string_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] area_alloc()\n");
    
    area_ptr_t area_ptr = malloc(sizeof(area_t));
    area_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(area_ptr->string_ptr, string_ptr);
    area_ptr->actor_ptr_array = NULL;
    area_ptr->actor_ptr_array_count = 0;
    area_ptr->area_ptr_array = NULL;
    area_ptr->area_ptr_array_count = 0;
    return area_ptr;
}

void area_free(area_ptr_t* area_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] area_free()\n");
    
    area_ptr_t area_ptr = *(area_ptr_ptr);
    // free string
    if (area_ptr->string_ptr != NULL)
    {
        free(area_ptr->string_ptr);
    }
    // free actor array
    if (area_ptr->actor_ptr_array != NULL)
    {
        // actors are not owned, therefore only free array
        free(area_ptr->actor_ptr_array);
    }
    // free area array
    if (area_ptr->area_ptr_array != NULL)
    {
        // areas are not owned, therefore only free array
        free(area_ptr->area_ptr_array);
    }
    // free area
    free(area_ptr);
    *(area_ptr_ptr) = NULL;
}

void area_add_area(area_ptr_t area_ptr, area_ptr_t area_2_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] area_add_area()\n");
    
    if (area_ptr->area_ptr_array == NULL)
    {
        area_ptr->area_ptr_array = malloc(sizeof(area_ptr_t));
        area_ptr->area_ptr_array[0] = area_2_ptr;
        area_ptr->area_ptr_array_count = 1;
    }
    else
    {
        area_ptr_t* old_area_ptr_array = area_ptr->area_ptr_array;
        int old_area_ptr_array_count = area_ptr->area_ptr_array_count;
        area_ptr->area_ptr_array = malloc(sizeof(area_ptr_t) * (old_area_ptr_array_count + 1));
        memcpy(area_ptr->area_ptr_array, old_area_ptr_array, sizeof(area_ptr_t) * old_area_ptr_array_count);
        area_ptr->area_ptr_array[old_area_ptr_array_count] = area_2_ptr;
        area_ptr->area_ptr_array_count = old_area_ptr_array_count + 1;
        free(old_area_ptr_array);
    }
}

void area_add_actor(area_ptr_t area_ptr, actor_ptr_t actor_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] area_add_actor()\n");
    
    if (area_ptr->actor_ptr_array == NULL)
    {
        area_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t));
        area_ptr->actor_ptr_array[0] = actor_ptr;
        area_ptr->actor_ptr_array_count = 1;
    }
    else
    {
        actor_ptr_t* old_actor_ptr_array = area_ptr->actor_ptr_array;
        int old_actor_ptr_array_count = area_ptr->actor_ptr_array_count;
        area_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t) * (old_actor_ptr_array_count + 1));
        memcpy(area_ptr->actor_ptr_array, old_actor_ptr_array, sizeof(actor_ptr_t) * old_actor_ptr_array_count);
        area_ptr->actor_ptr_array[old_actor_ptr_array_count] = actor_ptr;
        area_ptr->actor_ptr_array_count = old_actor_ptr_array_count + 1;
        free(old_actor_ptr_array);
    }
}

void area_remove_actor(area_ptr_t area_ptr, actor_ptr_t actor_ptr)
{
    // area_remove_actor is not responsible for freeing the actor_t pointed to by actor_ptr
    
    /*** debug ***/ // fprintf(stdout, "[Trace] area_remove_actor()\n");
    
    int actor_index = -1;
    for (int index = 0; index < area_ptr->actor_ptr_array_count; index++)
    {
        if (area_ptr->actor_ptr_array[index] == actor_ptr)
        {
            actor_index = index;
            break;
        }
    }
    if (actor_index != -1)
    {
        if (area_ptr->actor_ptr_array_count == 1)
        {
            free(area_ptr->actor_ptr_array);
            area_ptr->actor_ptr_array = NULL;
            area_ptr->actor_ptr_array_count = 0;
        }
        else
        {
            actor_ptr_t* old_actor_ptr_array = area_ptr->actor_ptr_array;
            int old_actor_ptr_array_count = area_ptr->actor_ptr_array_count;
            area_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t) * (old_actor_ptr_array_count - 1));
            if (actor_index > 0)
            {
                memcpy(area_ptr->actor_ptr_array, old_actor_ptr_array, sizeof(actor_ptr_t) * actor_index);
            }
            if (actor_index < old_actor_ptr_array_count - 1)
            {
                memcpy(area_ptr->actor_ptr_array + actor_index, old_actor_ptr_array + actor_index + 1, sizeof(actor_ptr_t) * (old_actor_ptr_array_count - 1 - actor_index));
            }
            area_ptr->actor_ptr_array_count = old_actor_ptr_array_count - 1;
            free(old_actor_ptr_array);
        }
    }
}

bool area_has_actor_type(area_ptr_t area_ptr, int type)
{
    for (int a_index = 0; a_index < area_ptr->actor_ptr_array_count; a_index++)
    {
        actor_ptr_t actor_ptr = area_ptr->actor_ptr_array[a_index];
        if (actor_ptr->type == type)
        {
            return true;
        }
    }
    return false;
}

//--------------------LEVEL--------------------

level_ptr_t level_alloc(string_ptr_t string_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] level_alloc()\n");

    level_ptr_t level_ptr = malloc(sizeof(level_t));
    level_ptr->string_ptr = malloc(strlen(string_ptr) + 1);
    strcpy(level_ptr->string_ptr, string_ptr);
    level_ptr->actor_ptr_array = NULL;
    level_ptr->actor_ptr_array_count = 0;
    level_ptr->area_ptr_array = NULL;
    level_ptr->area_ptr_array_count = 0;
    return level_ptr;
}

void level_free(level_ptr_t* level_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] level_free()\n");

    level_ptr_t level_ptr = *(level_ptr_ptr);
    // free string
    if (level_ptr->string_ptr != NULL)
    {
        free(level_ptr->string_ptr);
    }
    // free actor array
    if (level_ptr->actor_ptr_array != NULL)
    {
        for (int index = 0; index < level_ptr->actor_ptr_array_count; index++)
        {
            actor_free(&(level_ptr->actor_ptr_array[index]));
        }
        free(level_ptr->actor_ptr_array);
    }
    // free area array
    if (level_ptr->area_ptr_array != NULL)
    {
        for (int index = 0; index < level_ptr->area_ptr_array_count; index++)
        {
            area_free(&(level_ptr->area_ptr_array[index]));
        }
        free(level_ptr->area_ptr_array);
    }
    // free level
    free(level_ptr);
    *(level_ptr_ptr) = NULL;
}

void level_add_actor(level_ptr_t level_ptr, actor_ptr_t actor_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] level_add_actor()\n");

    if (level_ptr->actor_ptr_array == NULL)
    {
        level_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t));
        level_ptr->actor_ptr_array[0] = actor_ptr;
        level_ptr->actor_ptr_array_count = 1;
    }
    else
    {
        actor_ptr_t* old_actor_ptr_array = level_ptr->actor_ptr_array;
        int old_actor_ptr_array_count = level_ptr->actor_ptr_array_count;
        level_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t) * (old_actor_ptr_array_count + 1));
        memcpy(level_ptr->actor_ptr_array, old_actor_ptr_array, sizeof(actor_ptr_t) * old_actor_ptr_array_count);
        level_ptr->actor_ptr_array[old_actor_ptr_array_count] = actor_ptr;
        level_ptr->actor_ptr_array_count = old_actor_ptr_array_count + 1;
        free(old_actor_ptr_array);
    }
}

void level_remove_actor(level_ptr_t level_ptr, actor_ptr_t actor_ptr)
{
    // level_remove_actor is not responsible for freeing the actor_t pointed to by actor_ptr

    /*** debug ***/ // fprintf(stdout, "[Trace] level_remove_actor()\n");

    int actor_index = -1;
    for (int index = 0; index < level_ptr->actor_ptr_array_count; index++)
    {
        if (level_ptr->actor_ptr_array[index] == actor_ptr)
        {
            actor_index = index;
            break;
        }
    }
    if (actor_index != -1)
    {
        if (level_ptr->actor_ptr_array_count == 1)
        {
            free(level_ptr->actor_ptr_array);
            level_ptr->actor_ptr_array = NULL;
            level_ptr->actor_ptr_array_count = 0;
        }
        else
        {
            actor_ptr_t* old_actor_ptr_array = level_ptr->actor_ptr_array;
            int old_actor_ptr_array_count = level_ptr->actor_ptr_array_count;
            level_ptr->actor_ptr_array = malloc(sizeof(actor_ptr_t) * (old_actor_ptr_array_count - 1));
            if (actor_index > 0)
            {
                memcpy(level_ptr->actor_ptr_array, old_actor_ptr_array, sizeof(actor_ptr_t) * actor_index);
            }
            if (actor_index < old_actor_ptr_array_count - 1)
            {
                memcpy(level_ptr->actor_ptr_array + actor_index, old_actor_ptr_array + actor_index + 1, sizeof(actor_ptr_t) * (old_actor_ptr_array_count - 1 - actor_index));
            }
            level_ptr->actor_ptr_array_count = old_actor_ptr_array_count - 1;
            free(old_actor_ptr_array);
        }
    }
}

void level_add_area(level_ptr_t level_ptr, area_ptr_t area_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] level_add_area()\n");
    
    if (level_ptr->area_ptr_array == NULL)
    {
        level_ptr->area_ptr_array = malloc(sizeof(area_ptr_t));
        level_ptr->area_ptr_array[0] = area_ptr;
        level_ptr->area_ptr_array_count = 1;
    }
    else
    {
        area_ptr_t* old_area_ptr_array = level_ptr->area_ptr_array;
        int old_area_ptr_array_count = level_ptr->area_ptr_array_count;
        level_ptr->area_ptr_array = malloc(sizeof(area_ptr_t) * (old_area_ptr_array_count + 1));
        memcpy(level_ptr->area_ptr_array, old_area_ptr_array, sizeof(area_ptr_t) * old_area_ptr_array_count);
        level_ptr->area_ptr_array[old_area_ptr_array_count] = area_ptr;
        level_ptr->area_ptr_array_count = old_area_ptr_array_count + 1;
        free(old_area_ptr_array);
    }
}

actor_ptr_t level_get_actor(level_ptr_t level_ptr, string_ptr_t actor_string_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] level_get_actor()\n");

    for (int a_index = 0; a_index < level_ptr->actor_ptr_array_count; a_index++)
    {
        actor_ptr_t actor_ptr = level_ptr->actor_ptr_array[a_index];
        if (strcmp(actor_ptr->string_ptr, actor_string_ptr) == 0)
        {
            return actor_ptr;
        }
    }
    return NULL;
}

area_ptr_t level_get_area(level_ptr_t level_ptr, string_ptr_t area_string_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] level_get_area()\n");
    
    for (int a_index = 0; a_index < level_ptr->area_ptr_array_count; a_index++)
    {
        area_ptr_t area_ptr = level_ptr->area_ptr_array[a_index];
        if (strcmp(area_ptr->string_ptr, area_string_ptr) == 0)
        {
            return area_ptr;
        }
    }
    return NULL;
}

//--------------------MODEL--------------------

model_ptr_t model_ptr;

model_ptr_t model_alloc(void)
{
    /*** debug ***/ fprintf(stdout, "[Trace] model_alloc()\n");

    model_ptr_t model_ptr = malloc(sizeof(model_t));
    model_ptr->level_ptr_array = NULL;
    model_ptr->level_ptr_array_count = 0;
    model_ptr->actor_ptr = NULL;
    model_ptr->level_ptr = NULL;
    model_ptr->ticks = 0;
    return model_ptr;
}

void model_free(model_ptr_t* model_ptr_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] model_free()\n");

    // free level array
    if ((*(model_ptr_ptr))->level_ptr_array != NULL)
    {
        for (int index = 0; index < (*(model_ptr_ptr))->level_ptr_array_count; index++)
        {
            level_free(&((*(model_ptr_ptr))->level_ptr_array[index]));
        }
        free((*(model_ptr_ptr))->level_ptr_array);
        (*(model_ptr_ptr))->level_ptr_array = NULL;
    }
    // level is not owned, therefore not freed
    // actor is not owned, therefore not freed
    // free model
    free(*(model_ptr_ptr));
    *(model_ptr_ptr) = NULL;
}

void model_add_level(model_ptr_t model_ptr, level_ptr_t level_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] model_add_level()\n");

    if (model_ptr->level_ptr_array == NULL)
    {
        model_ptr->level_ptr_array = malloc(sizeof(level_ptr_t));
        model_ptr->level_ptr_array[0] = level_ptr;
        model_ptr->level_ptr_array_count = 1;
    }
    else
    {
        level_ptr_t* old_level_ptr_array = model_ptr->level_ptr_array;
        int old_level_ptr_array_count = model_ptr->level_ptr_array_count;
        model_ptr->level_ptr_array = malloc(sizeof(level_ptr_t) * (old_level_ptr_array_count + 1));
        memcpy(model_ptr->level_ptr_array, old_level_ptr_array, sizeof(level_ptr_t) * old_level_ptr_array_count);
        model_ptr->level_ptr_array[old_level_ptr_array_count] = level_ptr;
        model_ptr->level_ptr_array_count = old_level_ptr_array_count + 1;
        free(old_level_ptr_array);
    }
}

void model_remove_level(model_ptr_t model_ptr, level_ptr_t level_ptr)
{
    // model_remove_level is not responsible for freeing the level_t pointed to by level_ptr

    /*** debug ***/ fprintf(stdout, "[Trace] model_remove_level()\n");

    int level_index = -1;
    for (int index = 0; index < model_ptr->level_ptr_array_count; index++)
    {
        if (model_ptr->level_ptr_array[index] == level_ptr)
        {
            level_index = index;
            break;
        }
    }
    if (level_index != -1)
    {
        if (model_ptr->level_ptr_array_count == 1)
        {
            free(model_ptr->level_ptr_array);
            model_ptr->level_ptr_array = NULL;
            model_ptr->level_ptr_array_count = 0;
        }
        else
        {
            level_ptr_t* old_level_ptr_array = model_ptr->level_ptr_array;
            int old_level_ptr_array_count = model_ptr->level_ptr_array_count;
            model_ptr->level_ptr_array = malloc(sizeof(level_ptr_t) * (old_level_ptr_array_count - 1));
            if (level_index > 0)
            {
                memcpy(model_ptr->level_ptr_array, old_level_ptr_array, sizeof(level_ptr_t) * level_index);
            }
            if (level_index < old_level_ptr_array_count - 1)
            {
                memcpy(model_ptr->level_ptr_array + level_index, old_level_ptr_array + level_index + 1, sizeof(level_ptr_t) * (old_level_ptr_array_count - 1 - level_index));
            }
            model_ptr->level_ptr_array_count = old_level_ptr_array_count - 1;
            free(old_level_ptr_array);
        }
    }
}

level_ptr_t model_get_load_level(model_ptr_t model_ptr, string_ptr_t level_string_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] model_get_load_level()\n");

    // is level already loaded?
    /*** debug ***/ fprintf(stdout, "[Debug] level_string_ptr = %s\n", level_string_ptr);
    for (int l_index = 0; l_index < model_ptr->level_ptr_array_count; l_index++)
    {
        level_ptr_t level_ptr = model_ptr->level_ptr_array[l_index];
        if (strcmp(level_ptr->string_ptr, level_string_ptr) == 0)
        {
            return level_ptr;
        }
    }

    // if not, load level
    level_ptr_t level_ptr = NULL;
    char file_path[strlen(RESOURCE_DIR) + 1 + strlen(level_string_ptr) + 6 + 1];
    sprintf(file_path, "%s/%s.level", RESOURCE_DIR, level_string_ptr);
    FILE* file = fopen(file_path, "r");
    if (file == NULL)
    {
        /*** debug ***/ fprintf(stderr, "[Error] Unable to open level file \"%s\".\n", file_path);
        exit(-1);
    }
    char line[256];
    while (fgets(line, sizeof(line), file) != NULL)
    {
        char line_id[32];
        char string_ptr[32];
        if (strncmp(line, "//", 2) == 0)
        {
            /*** debug ***/ // fprintf(stdout, "[Debug] Skipping comment \"%s\".\n", line);
            continue;
        }
        if (strncmp(line, "level", 5) == 0)
        {
            /*** debug ***/ // fprintf(stdout, "[Debug] Parsing level \"%s\".\n", line);
            if (sscanf(line, "%s %s", line_id, string_ptr) != 2)
            {
                fprintf(stderr, "[Error] \"level\" incorrect format or incomplete.\n");
                exit(-1);
            }
            level_ptr = level_alloc(string_ptr);
            continue;
        }
        if (level_ptr == NULL)
        {
            fprintf(stderr, "[Error] \"level\" not found or not first.\n");
            exit(-1);
        }
        if (strncmp(line, "area", 4) == 0 && strncmp(line, "area_", 5) != 0)
        {
            /*** debug ***/ // fprintf(stdout, "[Debug] Parsing area \"%s\".\n", line);
            if (sscanf(line, "%s %s", line_id, string_ptr) != 2)
            {
                fprintf(stderr, "[Error] \"area\" incorrect format or incomplete.\n");
                exit(-1);
            }
            /*** debug ***/ fprintf(stdout, "alloc area \"%s\"\n", string_ptr);
            area_ptr_t area_ptr = area_alloc(string_ptr);
            level_add_area(level_ptr, area_ptr);
        }
        else if (strncmp(line, "area_add_areas", 14) == 0)
        {
            char area_1_string_ptr[32];
            char area_2_string_ptr[32];
            char area_3_string_ptr[32];
            /*** debug ***/ // fprintf(stdout, "[Debug] Parsing area \"%s\".\n", line);
            if (sscanf(line, "%s %s %s %s %s", line_id, string_ptr, area_1_string_ptr, area_2_string_ptr, area_3_string_ptr) != 5)
            {
                fprintf(stderr, "[Error] \"area_add_areas\" incorrect format or incomplete.\n");
                exit(-1);
            }
            area_ptr_t area_ptr = NULL;
            if (strcmp(string_ptr, "null") != 0)
            {
                area_ptr = level_get_area(level_ptr, string_ptr);
                if (area_ptr == NULL)
                {
                    fprintf(stderr, "[Error] Area references non-existant area.\n");
                    exit(-1);
                }
            }
            area_ptr_t area_1_ptr = NULL;
            if (strcmp(area_1_string_ptr, "null") != 0)
            {
                area_1_ptr = level_get_area(level_ptr, area_1_string_ptr);
                if (area_1_ptr == NULL)
                {
                    fprintf(stderr, "[Error] Area references non-existant area.\n");
                    exit(-1);
                }
            }
            area_ptr_t area_2_ptr = NULL;
            if (strcmp(area_2_string_ptr, "null") != 0)
            {
                area_2_ptr = level_get_area(level_ptr, area_2_string_ptr);
                if (area_2_ptr == NULL)
                {
                    fprintf(stderr, "[Error] Area references non-existant area.\n");
                    exit(-1);
                }
            }
            area_ptr_t area_3_ptr = NULL;
            if (strcmp(area_3_string_ptr, "null") != 0)
            {
                area_3_ptr = level_get_area(level_ptr, area_3_string_ptr);
                if (area_3_ptr == NULL)
                {
                    fprintf(stderr, "[Error] Area references non-existant area.\n");
                    exit(-1);
                }
            }
            area_add_area(area_ptr, area_1_ptr);
            area_add_area(area_ptr, area_2_ptr);
            area_add_area(area_ptr, area_3_ptr);
        }
        else if (strncmp(line, "actor", 5) == 0)
        {
            int type;
            char area_string_ptr[32];
            char prev_area_string_ptr[32];
            /*** debug ***/ // fprintf(stdout, "[Debug] Parsing actor \"%s\".\n", line);
            if (sscanf(line, "%s %s %d %s %s", line_id, string_ptr, &type, area_string_ptr, prev_area_string_ptr) != 5)
            {
                fprintf(stderr, "[Error] \"actor\" incorrect format or incomplete.\n");
                exit(-1);
            }
            /*** debug ***/ fprintf(stdout, "actor area \"%s\"\n", area_string_ptr);
            /*** debug ***/ fprintf(stdout, "actor prev area \"%s\"\n", prev_area_string_ptr);
            area_ptr_t area_ptr = NULL;
            if (strcmp(area_string_ptr, "null") != 0)
            {
                if (strcmp(area_string_ptr, "random") == 0)
                {
                    int index;
                    do
                    {
                        index = model_rand(level_ptr->area_ptr_array_count);
                    }
                    while (level_ptr->area_ptr_array[index]->actor_ptr_array_count != 0);
                    area_ptr = level_ptr->area_ptr_array[index];
                }
                else
                {
                    area_ptr = level_get_area(level_ptr, area_string_ptr);
                    if (area_ptr == NULL)
                    {
                        fprintf(stderr, "[Error] Actor references non-existant area.\n");
                        exit(-1);
                    }
                }
            }
            area_ptr_t prev_area_ptr = NULL;
            if (strcmp(prev_area_string_ptr, "null") != 0)
            {
                prev_area_ptr = level_get_area(level_ptr, prev_area_string_ptr);
                if (prev_area_ptr == NULL)
                {
                    fprintf(stderr, "[Error] Actor references non-existant area.\n");
                    exit(-1);
                }
            }
            actor_ptr_t actor_ptr = actor_alloc(string_ptr, type, area_ptr, prev_area_ptr);
            level_add_actor(level_ptr, actor_ptr);
            area_add_actor(area_ptr, actor_ptr);
        }
        else
        {
            fprintf(stderr, "[Error] Invalid line.\n");
            exit(-1);
        }
    }
    fclose(file);

    model_add_level(model_ptr, level_ptr);

    return level_ptr;
}

void model_new(model_ptr_t model_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] model_new()\n");

    model_ptr->level_ptr = model_get_load_level(model_ptr, "start");
    if (model_ptr->level_ptr == NULL)
    {
        fprintf(stderr, "[Error] Level \"start\" does not exist.\n");
        exit(-1);
    }
    model_ptr->actor_ptr = level_get_actor(model_ptr->level_ptr, "player");
    if (model_ptr->level_ptr == NULL)
    {
        fprintf(stderr, "[Error] Level \"start\" does not have \"player\" actor.\n");
        exit(-1);
    }
}

void model_load(model_ptr_t model_ptr, string_ptr_t string_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] model_load()\n");

    /*** debug ***/ fprintf(stdout, "[Debug] string_ptr = %s\n", string_ptr);

    // clear existing model (? or assume called after model_alloc?)
    // ...

    // load specified model
    // ...
}

int model_rand(int range)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] model_rand()\n");
    
    return rand() / (RAND_MAX + 1.0) * range;
}
