//
//  model.h
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 11/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#ifndef FrameworkPrototype_model_h
#define FrameworkPrototype_model_h

#ifndef STRING_PTR_T
#define STRING_PTR_T
typedef char* string_ptr_t;
#endif

#define RESOURCE_DIR "GameSwampSquid.app/Contents/Resources"

//--------------------FORWARD DECLARATIONS--------------------

typedef struct actor_t actor_t;
typedef actor_t* actor_ptr_t;

typedef struct area_t area_t;
typedef area_t* area_ptr_t;

typedef struct level_t level_t;
typedef level_t* level_ptr_t;

typedef struct model_t model_t;
typedef model_t* model_ptr_t;

//--------------------ACTOR--------------------

#define ACTOR_TYPE_PLAYER 0
#define ACTOR_TYPE_SQUID 1
#define ACTOR_TYPE_WISP 2
#define ACTOR_TYPE_VINES 3

struct actor_t
{
    string_ptr_t string_ptr;
    int type;
    area_ptr_t area_ptr;        // reference - does not own
    area_ptr_t prev_area_ptr;   // reference - does not own
};
// typedef actor_t forward declared

// typedef actor_t* actor_ptr_t;

actor_ptr_t actor_alloc(string_ptr_t string_ptr, int type, area_ptr_t area_ptr, area_ptr_t prev_area_ptr);
void actor_free(actor_ptr_t* actor_ptr_ptr);

//--------------------AREA--------------------

struct area_t
{
    string_ptr_t string_ptr;
    area_ptr_t* area_ptr_array;             // array of pointers - does not own
    int area_ptr_array_count;
    actor_ptr_t* actor_ptr_array;           // array of pointers - does not own
    int actor_ptr_array_count;
};
// typedef actor_t forward declared

// typedef actor_t* actor_ptr_t;

area_ptr_t area_alloc(string_ptr_t string_ptr);
void area_free(area_ptr_t* area_ptr_ptr);

void area_add_area(area_ptr_t area_ptr, area_ptr_t area_2_ptr);
void area_add_actor(area_ptr_t area_ptr, actor_ptr_t actor_ptr);
void area_remove_actor(area_ptr_t area_ptr, actor_ptr_t actor_ptr);

bool area_has_actor_type(area_ptr_t area_ptr, int type);

//--------------------LEVEL--------------------

struct level_t
{
    string_ptr_t string_ptr;
    actor_ptr_t* actor_ptr_array;       // array of pointers - owns
    int actor_ptr_array_count;
    area_ptr_t* area_ptr_array;         // array of pointers - owns
    int area_ptr_array_count;
};
// typedef level_t forward declared

// typedef level_t* level_ptr_t;

level_ptr_t level_alloc(string_ptr_t string_ptr);
void level_free(level_ptr_t* level_ptr_ptr);

void level_add_actor(level_ptr_t level_ptr, actor_ptr_t actor_ptr);
void level_remove_actor(level_ptr_t level_ptr, actor_ptr_t actor_ptr);
void level_add_area(level_ptr_t level_ptr, area_ptr_t area_ptr);

actor_ptr_t level_get_actor(level_ptr_t level_ptr, string_ptr_t actor_string_ptr);
area_ptr_t level_get_area(level_ptr_t level_ptr, string_ptr_t area_string_ptr);

//--------------------MODEL--------------------

struct model_t
{
    level_ptr_t* level_ptr_array;   // array of pointers - owns
    int level_ptr_array_count;
    actor_ptr_t actor_ptr;          // ref only, does not own
    level_ptr_t level_ptr;          // ref only, does not own
    int ticks;
};
// typedef model_t forward declared

// typedef model_t* model_ptr_t;

extern model_ptr_t model_ptr;

model_ptr_t model_alloc(void);
void model_free(model_ptr_t* model_ptr_ptr);

void model_add_level(model_ptr_t model_ptr, level_ptr_t level_ptr);
void model_remove_level(model_ptr_t model_ptr, level_ptr_t level_ptr);

level_ptr_t model_get_load_level(model_ptr_t model_ptr, char* level_id);

void model_new(model_ptr_t model_ptr);
void model_load(model_ptr_t model_ptr, char* string_ptr);

int model_rand(int range);

#endif
