//
//  title_screen.c
//  FrameworkPrototype
//
//  Created by Robert Hoferkamp on 12/17/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include <stdbool.h>
#include <string.h>

#include "framework.h"
#include "runtime.h"
#include "game.h"
#include "title_screen.h"
#include "model.h"

#define TITLE_MENU_INDEX 0

screen_ptr_t title_screen_ptr;

void title_screen_alloc(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] title_screen_alloc()\n");
    
    int x_res = runtime.screen_width;
    int y_res = runtime.screen_height;

    // create trace text
    text_ptr_t trace_text_ptr = text_alloc("DejaVuSans", 24, &color_white, "Title", 0, 0, 0);
    trace_text_ptr->x = x_res / 2 - runtime_surface_width(trace_text_ptr->surface_ptr) / 2;
    trace_text_ptr->y = runtime_surface_height(trace_text_ptr->surface_ptr) / 4;
    screen_add_text(screen_ptr, trace_text_ptr);
    
    // create title text
    text_ptr_t title_text_ptr = text_alloc("DejaVuSans", 128, &color_white, "Game Zero", 0, 0, 0);
    title_text_ptr->x = x_res / 2 - runtime_surface_width(title_text_ptr->surface_ptr) / 2;
    title_text_ptr->y = y_res / 2 - runtime_surface_height(title_text_ptr->surface_ptr) / 2;
    screen_add_text(screen_ptr, title_text_ptr);
    
    // create a menu [TITLE_MENU_INDEX]
    menu_ptr_t menu_ptr = menu_text_alloc("title", text_alloc("DejaVuSans", 32, &color_white, "Title", 0, 0, 0), 27, mod_key_none, anchor_bottom_right, 0, 0, true);
    screen_add_menu(screen_ptr, menu_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_1_ptr = menu_item_text_alloc("new", text_alloc("DejaVuSans", 32, &color_white, "New", 0, 0, 0), '1', mod_key_none, title_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_1_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_2_ptr = menu_item_text_alloc("continue", text_alloc("DejaVuSans", 32, &color_white, "Continue", 0, 0, 0), '2', mod_key_none, title_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_2_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_3_ptr = menu_item_text_alloc("load", text_alloc("DejaVuSans", 32, &color_white, "Load", 0, 0, 0), '3', mod_key_none, title_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_3_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_4_ptr = menu_item_text_alloc("options", text_alloc("DejaVuSans", 32, &color_white, "Options", 0, 0, 0), '4', mod_key_none, title_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_4_ptr);
    
    // create a menu item
    menu_item_ptr_t menu_item_5_ptr = menu_item_text_alloc("exit", text_alloc("DejaVuSans", 32, &color_white, "Exit", 0, 0, 0), '5', mod_key_none, title_screen_menu_item_task);
    menu_add_menu_item(menu_ptr, menu_item_5_ptr);
}

void title_screen_init(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] title_screen_init()\n");

    if (model_ptr != NULL)
    {
        model_free(&model_ptr);
    }
}

void title_screen_input_task(screen_ptr_t screen_ptr, input_state_ptr_t input_state_ptr)
{
    /*** debug ***/ // fprintf(stdout, "[Trace] title_screen_input_task()\n");
    
    // ...
}

void title_screen_menu_item_task(menu_item_ptr_t menu_item_ptr, menu_ptr_t menu_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] title_screen_menu_item_task()\n");
    
    /*** debug ***/ fprintf(stdout, "[Debug] menu item string = %s\n", menu_item_ptr->string_ptr);
    
    if (strcmp(menu_item_ptr->string_ptr, "new") == 0)
    {
        model_ptr = model_alloc();
        model_new(model_ptr);
        game_set_screen(game.screen_ptr_array[GAME_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "continue") == 0)
    {
        model_ptr = model_alloc();
        model_load(model_ptr, "Auto Save");
        game_set_screen(game.screen_ptr_array[GAME_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "load") == 0)
    {
        game_set_screen(game.screen_ptr_array[LOAD_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "options") == 0)
    {
        game_set_screen(game.screen_ptr_array[OPTIONS_SCREEN_INDEX]);
    }
    else if (strcmp(menu_item_ptr->string_ptr, "exit") == 0)
    {
        game_set_screen(NULL);
    }
}

void title_screen_exit(screen_ptr_t screen_ptr)
{
    /*** debug ***/ fprintf(stdout, "[Trace] title_screen_exit()\n");
    
    // background, sprites, text and menus are freed by screen_free
}
