#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <glib.h>

#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "debug.h"
#include "common.h"

Collection* collection_new(Repository* repository, char* name)
{
	Collection* collection = NULL;
	
	collection = (Collection*)calloc(1, sizeof(Collection));
	if(!collection) {
		debug_log(DEBUG, "collection_new(): Insuffisant Memory");
		return NULL;
	}
	
	collection->repository = repository;
	strcpy(collection->name, name);
	
	return collection;
}

void get_packages(Collection* collection, unsigned char forced)
{
	DIR* dirp;
	DIR* dirp2;
	struct dirent* dp;
	struct dirent* pkgdp;
	struct stat node_stat;
	char collection_path[BUFSIZ];
	char package_path[BUFSIZ];
	char pkgfile_path[BUFSIZ];
	char pkg_name[256];
	int base_dir_len;
	Ports* _ports = collection->repository->ports;
	
	if (collection->packages)
		g_list_free(collection->packages);
	collection->packages = NULL;
	
	base_dir_len = strlen(collection->repository->base_directory);
	
	memset(collection_path, 0, sizeof(collection_path));
	strcpy(collection_path, collection->repository->base_directory);
	strcat(collection_path, "/");
	strcat(collection_path, collection->name);
	
	dirp = opendir (collection_path);
	while ( (dp = readdir (dirp)) != NULL ) {
		memset(package_path, 0, sizeof(package_path));
		
		strcpy(package_path, collection_path);
		strcat(package_path, "/");
		strcat(package_path, dp->d_name);
		
		stat(package_path, &node_stat);
		if (S_ISDIR(node_stat.st_mode) && strcmp(dp->d_name, ".") != 0
		    && strcmp(dp->d_name, "..") != 0) {
			dirp2 = opendir (package_path);
			while ( (pkgdp = readdir (dirp2)) != NULL ) {
				if (strcmp(pkgdp->d_name, "Pkgfile") == 0) {
					collection->packages = g_list_append(
						collection->packages,
						package_new(collection,
							dp->d_name));
				}
			}
			closedir(dirp2);
		}
		ports_on_refresh(_ports);
	}
	closedir(dirp);
	
	if (collection->packages)
		collection->packages = g_list_sort (
				collection->packages,
				compare_package_list_data);
}

gint compare_package_list_data(gconstpointer object1, gconstpointer object2)
{
	return strcmp(((Package*)object1)->name, ((Package*)object2)->name);
}

void collection_list_differences(Collection* collection, GHashTable* pkg_db)
{
	GList* packages;
	char* version_installed = NULL;
	Ports* _ports = collection->repository->ports;

	if (!pkg_db) return;

	if (!collection->packages)
		get_packages(collection, 0);

	packages = collection->packages;
	while (packages) {
		version_installed = (char*)g_hash_table_lookup(pkg_db, ((Package*)packages->data)->name);
		if (version_installed) {
			strcpy(((Package*)packages->data)->installed_version, version_installed);
			if (!is_package_uptodate((Package*)packages->data)) {
				ports_on_print(_ports, "%s", package_to_string((Package*)packages->data));
			}
		}
		ports_on_refresh(_ports);
		packages = packages->next;
	}
}

void collection_list_installed(Collection* collection, GHashTable* pkg_db)
{
	GList* packages;
	char* version_installed = NULL;
	Ports* _ports = collection->repository->ports;

	if (!pkg_db) return;

	if (!collection->packages)
		get_packages(collection, 0);

	packages = collection->packages;
	while (packages) {
		version_installed = (char*)g_hash_table_lookup(pkg_db, ((Package*)packages->data)->name);
		if (version_installed) {
			strcpy(((Package*)packages->data)->installed_version, version_installed);
			if (strcmp(((Package*)packages->data)->installed_version, ((Package*)packages->data)->release_version) <= 0) {
				ports_on_print(_ports, "%s", package_to_string((Package*)packages->data));
			}
			free(version_installed);
			g_hash_table_remove(pkg_db, ((Package*)packages->data)->name);
		}
		ports_on_refresh(_ports);
		packages = packages->next;
	}
}

