#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#define _GNU_SOURCE
#include <string.h>

#include "debug.h"
#include "init.h"

const char* CONFIG_FILE = "/etc/gports.conf\0";
const char* DEFAULT_LOGDIR = "/tmp\0";
const LogLevel DEFAULT_LOGLEVEL = NONE;

const char* LOGLEVEL_STR = "loglevel=\0";
const char* LOGDIR_STR = "logdirectory=\0";

char* LOGDIRECTORY;
LogLevel LOGLEVEL;

int load_config(void)
{
	int ret = 0;
	FILE* fconfig;
	char* config_buf;
	char* pos;
	char* endpos;
	char* endptr;
	int config_buflen;
	int loglevelstr_len = strlen (LOGLEVEL_STR);
	int logdirstr_len = strlen (LOGDIR_STR);
	int loglevel_len = 0;
	int logdir_len = 0;
	char* loglevel;

	fconfig = fopen (CONFIG_FILE, "r");
	if (!fconfig) {
		LOGLEVEL = DEFAULT_LOGLEVEL;
		LOGDIRECTORY = strdup (DEFAULT_LOGDIR);
	} else {
		fseek (fconfig, 0, SEEK_END);
		config_buflen = ftell (fconfig);
		fseek (fconfig, 0, SEEK_SET);
		config_buf = (char*)calloc (config_buflen+1, 1);
		fread (config_buf, 1, config_buflen, fconfig);
		fclose (fconfig);

		pos = strstr (config_buf, LOGLEVEL_STR);
		if (!pos) {
			LOGLEVEL = DEFAULT_LOGLEVEL;
		} else {
			endpos = strchr (pos, '\n');
			if (!endpos) {
				endpos = strchr (pos, '\0');
			}
			loglevel_len = (int)(endpos - (pos+loglevelstr_len));
			if (loglevel_len == 0) {
				LOGLEVEL = DEFAULT_LOGLEVEL;
			} else {
				loglevel = (char*)calloc (loglevel_len+1, 1);
				strncpy (loglevel, pos+loglevelstr_len, loglevel_len);
				LOGLEVEL = (LogLevel)strtol(loglevel, (char**)NULL, 10);
			}
		}
		pos = strstr (config_buf, LOGDIR_STR);
		if (!pos) {
			LOGDIRECTORY = strdup (DEFAULT_LOGDIR);
		} else {
			endpos = strchr (pos, '\n');
			if (!endpos) {
				endpos = strchr (pos, '\0');
			}
			logdir_len = (int)(endpos - (pos+logdirstr_len));
			if (logdir_len == 0) {
				LOGDIRECTORY = strdup (DEFAULT_LOGDIR);
			} else {
				LOGDIRECTORY = (char*)strndup (pos+logdirstr_len, (endpos - (pos+logdirstr_len)));
			}
		}
		free (config_buf);
	}

	return ret;
}

