#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#define _(x) (x)

#include "packages_db.h"
#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "debug.h"
#include "common.h"

#include "interface.h"

void
do_events(void)
{
	while (gtk_events_pending ()) {
		gtk_main_iteration_do(FALSE);
	}
}

void populate_list_ports_tree(MainWindow* main_win)
{
	GtkTreeIter iter;
	GtkTreeIter iter1;
	char temp_collection[256] = {0};
	GList* repo = NULL;
	GList* coll = NULL;
	GList* pack = NULL;
	guint context_id;
	
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (main_win->infos_sb), "ports list");
	gtk_statusbar_push (GTK_STATUSBAR (main_win->infos_sb), context_id, "Listing all ports...");
	do_events ();
	
	ports_list_packages (main_win->ports);
	repo = main_win->ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			pack = ((Collection*)coll->data)->packages;
			while (pack) {
				if (strcmp(temp_collection, ((Package*)coll->data)->name) != 0) {
					gtk_tree_store_append (main_win->ports_tree_store,
						&iter, NULL);
					gtk_tree_store_set (main_win->ports_tree_store, &iter,
							NAME_COLUMN, ((Package*)coll->data)->name,
							-1);
					strcpy(temp_collection, ((Package*)coll->data)->name);
				}
		
				gtk_tree_store_append (main_win->ports_tree_store, &iter1, &iter);
				gtk_tree_store_set (main_win->ports_tree_store, &iter1,
						NAME_COLUMN, ((Package*)pack->data)->name,
						CURRENT_VERSION_COLUMN, ((Package*)pack->data)->installed_version,
						NEW_VERSION_COLUMN, ((Package*)pack->data)->release_version,
						CHECKED_COLUMN, FALSE,
						-1);
					pack = pack->next;
			}
			coll = coll->next;
		}
		do_events();
		repo = repo->next;
	}
	
	gtk_statusbar_pop (GTK_STATUSBAR (main_win->infos_sb), context_id);
}

void populate_diff_ports_tree(MainWindow* main_win)
{
	static GtkTreeIter iter;
	GtkTreeIter iter1;
	char temp_collection[256] = {0};
	GList* repo = NULL;
	GList* coll = NULL;
	GList* pack = NULL;
	guint context_id;
	
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (main_win->infos_sb), "diff list");
	gtk_statusbar_push (GTK_STATUSBAR (main_win->infos_sb), context_id, "Listing differences...");
	do_events ();

	ports_list_differences (main_win->ports);
	repo = main_win->ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			pack = ((Collection*)coll->data)->packages;
			while (pack) {
				if (!is_package_uptodate( (Package*)pack->data )) {
					if (strcmp(temp_collection, ((Collection*)coll->data)->name) != 0) {
						gtk_tree_store_append (main_win->ports_tree_store, &iter, NULL);
						gtk_tree_store_set (main_win->ports_tree_store, &iter,
							NAME_COLUMN, ((Collection*)coll->data)->name,
							-1);
						strcpy(temp_collection, ((Collection*)coll->data)->name);
					}
		
					gtk_tree_store_append (main_win->ports_tree_store, &iter1, &iter);
					gtk_tree_store_set (main_win->ports_tree_store, &iter1,
						NAME_COLUMN, ((Package*)pack->data)->name,
						CURRENT_VERSION_COLUMN, ((Package*)pack->data)->installed_version,
						NEW_VERSION_COLUMN, ((Package*)pack->data)->release_version,
						CHECKED_COLUMN, FALSE,
						-1);
				}	
				pack = pack->next;
			}
			coll = coll->next;
		}
		repo = repo->next;
	}
	gtk_statusbar_pop (GTK_STATUSBAR (main_win->infos_sb), context_id);
}


void populate_installed_ports_tree(MainWindow* main_win)
{
	static GtkTreeIter iter;
	GtkTreeIter iter1;
	char temp_collection[256] = {0};
	GList* repo = NULL;
	GList* coll = NULL;
	GList* pack = NULL;
	guint context_id;
	
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (main_win->infos_sb), "diff list");
	gtk_statusbar_push (GTK_STATUSBAR (main_win->infos_sb), context_id, "Listing installed...");
	do_events ();

	ports_list_installed (main_win->ports);
	repo = main_win->ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			pack = ((Collection*)coll->data)->packages;
			while (pack) {
				if (is_package_installed( (Package*)pack->data )) {
					if (strcmp(temp_collection, ((Collection*)coll->data)->name) != 0) {
						gtk_tree_store_append (main_win->ports_tree_store, &iter, NULL);
						gtk_tree_store_set (main_win->ports_tree_store, &iter,
								NAME_COLUMN, ((Collection*)coll->data)->name,
								-1);
						strcpy(temp_collection, ((Collection*)coll->data)->name);
					}
		
					gtk_tree_store_append (main_win->ports_tree_store, &iter1, &iter);
					gtk_tree_store_set (main_win->ports_tree_store, &iter1,
							NAME_COLUMN, ((Package*)pack->data)->name,
							CURRENT_VERSION_COLUMN, ((Package*)pack->data)->installed_version,
							NEW_VERSION_COLUMN, ((Package*)pack->data)->release_version,
							CHECKED_COLUMN, FALSE,
							-1);
				}	
				pack = pack->next;
			}
			coll = coll->next;
		}
		repo = repo->next;
	}
	gtk_statusbar_pop (GTK_STATUSBAR (main_win->infos_sb), context_id);
}

static void
ports_tree_checked_toggled_cb (GtkCellRendererToggle* cell, gchar* path, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	GtkTreeModel* model = GTK_TREE_MODEL (main_win->ports_tree_store);
	GtkTreeIter iter;
	gboolean uninstall;
	
	if (gtk_tree_path_get_depth (gtk_tree_path_new_from_string (path)) != 1) {
		gtk_tree_model_get_iter_from_string (model, &iter, path);
		gtk_tree_model_get (model, &iter,
				CHECKED_COLUMN, &uninstall,
				-1);

		gtk_tree_store_set (main_win->ports_tree_store, &iter,
				CHECKED_COLUMN, !uninstall,
				-1);
	}
}

static void
ports_tree_row_activated_cb(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *col, gpointer data)
{
	if (gtk_tree_path_get_depth (path) == 1) {
		if (gtk_tree_view_row_expanded (tv, path))
			gtk_tree_view_collapse_row (tv, path);
		else
			gtk_tree_view_expand_row (tv, path, FALSE);
	}
}

static void
main_quit()
{
	gtk_main_quit ();
}

static gboolean
on_win_delete_event(GtkWidget* widget, GdkEvent* event, gpointer data)
{
	debug_log (DEBUG, "on_win_delete_event called");
	return FALSE;
}

static void
on_win_destroy(GtkWidget* widget, gpointer data)
{
	debug_log (DEBUG, "on_win_destroy called");
	main_quit ();
}

static void
on_quit1_activate(GtkMenuItem* menuitem, gpointer user_data)
{
	gtk_main_quit ();
}

static void
load_ports_list(MainWindow* main_win)
{
	gtk_tree_store_clear (main_win->ports_tree_store);
	
	gtk_widget_set_sensitive (main_win->list1, FALSE);
	gtk_widget_set_sensitive (main_win->list_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->update1, FALSE);
	gtk_widget_set_sensitive (main_win->update_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->diff1, FALSE);
	gtk_widget_set_sensitive (main_win->diff_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->installed_tbt, FALSE);
	
	do_events ();
	
	populate_list_ports_tree (main_win);
	
	gtk_widget_set_sensitive (main_win->list1, TRUE);
	gtk_widget_set_sensitive (main_win->list_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->update1, TRUE);
	gtk_widget_set_sensitive (main_win->update_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->diff1, TRUE);
	gtk_widget_set_sensitive (main_win->diff_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->installed_tbt, TRUE);
}

static void
on_list1_activate(GtkMenuItem* menuitem, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_list1_activate called");

	load_ports_list (main_win);
}

static void
on_list_tbt_clicked(GtkToolButton* toolbutton, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_list_tbt_clicked called");
	
	load_ports_list (main_win);
}

static void
load_diff_list(MainWindow* main_win)
{
	gtk_tree_store_clear (main_win->ports_tree_store);
	
	gtk_widget_set_sensitive (main_win->list1, FALSE);
	gtk_widget_set_sensitive (main_win->list_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->update1, FALSE);
	gtk_widget_set_sensitive (main_win->update_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->diff1, FALSE);
	gtk_widget_set_sensitive (main_win->diff_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->installed_tbt, FALSE);
	
	do_events ();
	
	populate_diff_ports_tree (main_win);
	
	gtk_widget_set_sensitive (main_win->list1, TRUE);
	gtk_widget_set_sensitive (main_win->list_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->update1, TRUE);
	gtk_widget_set_sensitive (main_win->update_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->diff1, TRUE);
	gtk_widget_set_sensitive (main_win->diff_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->installed_tbt, TRUE);
}

static void
on_diff1_activate(GtkMenuItem* menuitem, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_diff1_activate called");

	load_diff_list (main_win);
}

static void
on_diff_tbt_clicked(GtkToolButton* toolbutton, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_diff_tbt_clicked called");
	
	load_diff_list (main_win);
}

void add_results(const char* format, va_list va)
{
	char message[16358];
	char new_message[16358];
	char* buf;
	GtkTextIter start_iter, end_iter;
	GtkTextBuffer* buffer;
	
	memset(message, 0, sizeof(message));
	memset(new_message, 0, sizeof(new_message));
	(void)vsprintf (message, format, va);
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (results_dlg->txt_result));
	gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
	buf = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
	snprintf (new_message, sizeof (new_message), "%s%s", buf, message);
	gtk_text_buffer_set_text (buffer, g_strdup(new_message), (-1));
}

static void
on_dlg_results_show (GtkWidget *widget, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	ports_update (main_win->ports);
	gtk_widget_set_sensitive (results_dlg->btn_close, TRUE);
}

static void
do_update_ports (MainWindow* main_win)
{
	main_win->ports->print = (print_func)add_results;
	results_dlg = create_results_dlg(main_win);
	gtk_dialog_run (GTK_DIALOG (results_dlg->dialog));

	gtk_widget_destroy (results_dlg->dialog);
	free (results_dlg);
	results_dlg = NULL;
	main_win->ports->print = NULL;
}

static void
on_update1_activate(GtkMenuItem* menuitem, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_update1_activate called");

	do_update_ports(main_win);
}

static void
on_update_tbt_clicked(GtkToolButton* toolbutton, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_update_tbt_clicked called");
	
	do_update_ports(main_win);
}


static void
load_installed_list(MainWindow* main_win)
{
	gtk_tree_store_clear (main_win->ports_tree_store);
	
	gtk_widget_set_sensitive (main_win->list1, FALSE);
	gtk_widget_set_sensitive (main_win->list_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->update1, FALSE);
	gtk_widget_set_sensitive (main_win->update_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->diff1, FALSE);
	gtk_widget_set_sensitive (main_win->diff_tbt, FALSE);
	gtk_widget_set_sensitive (main_win->installed_tbt, FALSE);
	
	do_events ();
	
	populate_installed_ports_tree (main_win);
	
	gtk_widget_set_sensitive (main_win->list1, TRUE);
	gtk_widget_set_sensitive (main_win->list_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->update1, TRUE);
	gtk_widget_set_sensitive (main_win->update_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->diff1, TRUE);
	gtk_widget_set_sensitive (main_win->diff_tbt, TRUE);
	gtk_widget_set_sensitive (main_win->installed_tbt, TRUE);
}


static void
on_installed_tbt_clicked(GtkToolButton* toolbutton, gpointer user_data)
{
	MainWindow* main_win = (MainWindow*)user_data;
	debug_log(DEBUG, "on_update_tbt_clicked called");
	
	load_installed_list (main_win);
}

void
show_main_window (MainWindow* main_win)
{
	gtk_widget_show_all(main_win->win);
}

MainWindow*
create_main_window ()
{
	GtkWidget *main_vbox;
	GtkWidget *menubar1;
	GtkWidget *ports1;
	GtkWidget *ports1_menu;
	GtkWidget *separator4;
	GtkWidget *preferences1;
	GtkWidget *separator2;
	GtkWidget *quit1;
	GtkWidget *tools1;
	GtkWidget *tools1_menu;
	GtkWidget *separator3;
	GtkWidget *help1;
	GtkWidget *help1_menu;
	GtkWidget *about1;
	GtkWidget *handlebox1;
	GtkWidget *ports_tbar;
	GtkIconSize tmp_toolbar_icon_size;
	GtkWidget *tmp_image;
	GtkWidget *ports_sw;
	GtkWidget *bottom_hbox;
	
	GtkCellRenderer* cell;
	GtkTreeViewColumn* column;
	GtkTreeIter iter1;
	GtkTreeIter iter2;
	GtkTreeSelection* select;
	
	MainWindow* main_win;
	
	main_win = (MainWindow*)calloc (1, sizeof (MainWindow)); 

	main_win->tooltips = gtk_tooltips_new ();

	main_win->accel_group = gtk_accel_group_new ();

	main_win->win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (main_win->win, 700, 500);
	gtk_window_set_title (GTK_WINDOW (main_win->win), _("gports"));
	gtk_window_set_position (GTK_WINDOW (main_win->win), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size (GTK_WINDOW (main_win->win), 700, 500);
	gtk_window_set_resizable (GTK_WINDOW (main_win->win), FALSE);

	main_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (main_vbox);
	gtk_container_add (GTK_CONTAINER (main_win->win), main_vbox);

	menubar1 = gtk_menu_bar_new ();
	gtk_widget_show (menubar1);
	gtk_box_pack_start (GTK_BOX (main_vbox), menubar1, FALSE, FALSE, 0);

	ports1 = gtk_menu_item_new_with_mnemonic (_("_Ports"));
	gtk_widget_show (ports1);
	gtk_container_add (GTK_CONTAINER (menubar1), ports1);

	ports1_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (ports1), ports1_menu);

	main_win->update1 = gtk_menu_item_new_with_mnemonic (_("_Update"));
	gtk_widget_show (main_win->update1);
	gtk_container_add (GTK_CONTAINER (ports1_menu), main_win->update1);
	gtk_tooltips_set_tip (main_win->tooltips, main_win->update1, _("Update ports collections"), NULL);

	main_win->diff1 = gtk_menu_item_new_with_mnemonic (_("_Diff"));
	gtk_widget_show (main_win->diff1);
	gtk_container_add (GTK_CONTAINER (ports1_menu), main_win->diff1);
	gtk_tooltips_set_tip (main_win->tooltips, main_win->diff1, _("Show ports differences"), NULL);

	main_win->list1 = gtk_menu_item_new_with_mnemonic (_("_List"));
	gtk_widget_show (main_win->list1);
	gtk_container_add (GTK_CONTAINER (ports1_menu), main_win->list1);
	gtk_tooltips_set_tip (main_win->tooltips, main_win->list1, _("List all ports"), NULL);

	separator4 = gtk_separator_menu_item_new ();
	gtk_widget_show (separator4);
	gtk_container_add (GTK_CONTAINER (ports1_menu), separator4);
	gtk_widget_set_sensitive (separator4, FALSE);

	preferences1 = gtk_image_menu_item_new_from_stock ("gtk-preferences", main_win->accel_group);
	gtk_widget_show (preferences1);
	gtk_container_add (GTK_CONTAINER (ports1_menu), preferences1);

	separator2 = gtk_separator_menu_item_new ();
	gtk_widget_show (separator2);
	gtk_container_add (GTK_CONTAINER (ports1_menu), separator2);
	gtk_widget_set_sensitive (separator2, FALSE);

	quit1 = gtk_image_menu_item_new_from_stock ("gtk-quit", main_win->accel_group);
	gtk_widget_show (quit1);
	gtk_container_add (GTK_CONTAINER (ports1_menu), quit1);

	tools1 = gtk_menu_item_new_with_mnemonic (_("_Tools"));
	gtk_widget_show (tools1);
	gtk_container_add (GTK_CONTAINER (menubar1), tools1);

	tools1_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (tools1), tools1_menu);

	main_win->sysup1 = gtk_menu_item_new_with_mnemonic (_("_Sysup"));
	gtk_widget_set_sensitive (main_win->sysup1, FALSE);
	gtk_widget_show (main_win->sysup1);
	gtk_container_add (GTK_CONTAINER (tools1_menu), main_win->sysup1);

	separator3 = gtk_separator_menu_item_new ();
	gtk_widget_show (separator3);
	gtk_container_add (GTK_CONTAINER (tools1_menu), separator3);
	gtk_widget_set_sensitive (separator3, FALSE);

	main_win->clean_all1 = gtk_menu_item_new_with_mnemonic (_("_Clean All"));
	gtk_widget_set_sensitive (main_win->clean_all1, FALSE);
	gtk_widget_show (main_win->clean_all1);
	gtk_container_add (GTK_CONTAINER (tools1_menu), main_win->clean_all1);

	help1 = gtk_menu_item_new_with_mnemonic (_("_Help"));
	gtk_widget_show (help1);
	gtk_container_add (GTK_CONTAINER (menubar1), help1);

	help1_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (help1), help1_menu);

	about1 = gtk_menu_item_new_with_mnemonic (_("About"));
	gtk_widget_show (about1);
	gtk_container_add (GTK_CONTAINER (help1_menu), about1);

	handlebox1 = gtk_handle_box_new ();
	gtk_widget_show (handlebox1);
	gtk_box_pack_start (GTK_BOX (main_vbox), handlebox1, FALSE, TRUE, 0);

	ports_tbar = gtk_toolbar_new ();
	gtk_widget_show (ports_tbar);
	gtk_container_add (GTK_CONTAINER (handlebox1), ports_tbar);
	gtk_toolbar_set_style (GTK_TOOLBAR (ports_tbar), GTK_TOOLBAR_ICONS);
	tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (ports_tbar));

	tmp_image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	main_win->update_tbt = (GtkWidget*) gtk_tool_button_new (tmp_image, _("Update"));
	gtk_widget_show (main_win->update_tbt);
	gtk_container_add (GTK_CONTAINER (ports_tbar), main_win->update_tbt);

	tmp_image = gtk_image_new_from_stock (GTK_STOCK_COPY, tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	main_win->diff_tbt = (GtkWidget*) gtk_tool_button_new (tmp_image, _("Diff"));
	gtk_widget_show (main_win->diff_tbt);
	gtk_container_add (GTK_CONTAINER (ports_tbar), main_win->diff_tbt);

	tmp_image = gtk_image_new_from_stock (GTK_STOCK_JUSTIFY_FILL, tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	main_win->list_tbt = (GtkWidget*) gtk_tool_button_new (tmp_image, _("List"));
	gtk_widget_show (main_win->list_tbt);
	gtk_container_add (GTK_CONTAINER (ports_tbar), main_win->list_tbt);

	tmp_image = gtk_image_new_from_stock (GTK_STOCK_GOTO_BOTTOM, tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	main_win->installed_tbt = (GtkWidget*) gtk_tool_button_new (tmp_image, _("Installed"));
	gtk_widget_show (main_win->installed_tbt);
	gtk_container_add (GTK_CONTAINER (ports_tbar), main_win->installed_tbt);

	ports_sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (ports_sw);
	gtk_box_pack_start (GTK_BOX (main_vbox), ports_sw, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ports_sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ports_sw), GTK_SHADOW_IN);

	/* --------------------------------------------------------------------- */
	
	main_win->ports_tree_store = gtk_tree_store_new (N_COLUMNS,
			GDK_TYPE_PIXBUF,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING,
			GDK_TYPE_PIXBUF,
			G_TYPE_BOOLEAN);
	main_win->ports_tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (main_win->ports_tree_store));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (main_win->ports_tree_view), TRUE);

	column = gtk_tree_view_column_new ();
	cell = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_set_attributes(column, cell,
			"pixbuf", INSTALLED_COLUMN,
			NULL);

	cell = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start(column, cell, TRUE);
	gtk_tree_view_column_set_attributes(column, cell,
			"markup", NAME_COLUMN,
			NULL);
	gtk_tree_view_column_set_min_width (column, 324);
	gtk_tree_view_column_set_max_width (column, 324);
	gtk_tree_view_column_set_title (column, _("Name"));
	gtk_tree_view_column_set_alignment (column, .5f);
	gtk_tree_view_append_column (GTK_TREE_VIEW (main_win->ports_tree_view), column);

	cell = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (_("Installed"),
			cell,
			"text",
			CURRENT_VERSION_COLUMN,
			NULL);
	gtk_tree_view_column_set_alignment (column, .5f);
	gtk_tree_view_column_set_min_width (column, 128);
	gtk_tree_view_column_set_max_width (column, 128);
	g_object_set (cell, "xalign", 0.5f, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (main_win->ports_tree_view), column);

	cell = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (_("Port"),
			cell,
			"text",
			NEW_VERSION_COLUMN,
			NULL);
	gtk_tree_view_column_set_alignment (column, .5f);
	gtk_tree_view_column_set_min_width (column, 128);
	gtk_tree_view_column_set_max_width (column, 128);
	g_object_set (cell, "xalign", 0.5f, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (main_win->ports_tree_view), column);

	cell = gtk_cell_renderer_pixbuf_new ();
	column = gtk_tree_view_column_new_with_attributes (" ",
			cell,
			"pixbuf",
			INFOS_COLUMN,
			NULL);
	gtk_tree_view_column_set_alignment (column, .5f);
	gtk_tree_view_column_set_min_width (column, 24);
	gtk_tree_view_column_set_max_width (column, 24);
	gtk_tree_view_append_column (GTK_TREE_VIEW (main_win->ports_tree_view), column);

	cell = gtk_cell_renderer_toggle_new ();
	column = gtk_tree_view_column_new_with_attributes (_("group ?"),
			cell,
			"active",
			CHECKED_COLUMN,
			NULL);
	gtk_tree_view_column_set_alignment (column, .5f);
	g_signal_connect (G_OBJECT (cell), "toggled",
					  G_CALLBACK (ports_tree_checked_toggled_cb),
					  NULL);
	gtk_object_set (GTK_OBJECT (cell), "activatable", FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (main_win->ports_tree_view), column);
	
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (main_win->ports_tree_view));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
		
	gtk_widget_show (main_win->ports_tree_view);
	gtk_container_add (GTK_CONTAINER (ports_sw), main_win->ports_tree_view);

	/* --------------------------------------------------------------------- */

	bottom_hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (bottom_hbox);
	gtk_box_pack_start (GTK_BOX (main_vbox), bottom_hbox, FALSE, FALSE, 0);

	main_win->infos_sb = gtk_statusbar_new ();
	gtk_widget_show (main_win->infos_sb);
	gtk_box_pack_start (GTK_BOX (bottom_hbox), main_win->infos_sb, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (main_win->infos_sb), 1);
	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (main_win->infos_sb), FALSE);

	main_win->load_pb = gtk_progress_bar_new ();
	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (main_win->load_pb), 0.01);
	//gtk_progress_bar_pulse (GTK_PROGRESS_BAR (main_win->load_pb));
	gtk_widget_show (main_win->load_pb);
	gtk_box_pack_start (GTK_BOX (bottom_hbox), main_win->load_pb, FALSE, FALSE, 0);

	/* --------------------------------------------------------------------- */
	
	g_signal_connect (G_OBJECT (main_win->win), "delete_event",
			G_CALLBACK (on_win_delete_event),
			NULL);
	g_signal_connect (G_OBJECT (main_win->win), "destroy",
			G_CALLBACK (on_win_destroy),
			NULL);
	g_signal_connect (G_OBJECT (quit1), "activate",
			G_CALLBACK (on_win_destroy),
			NULL);

	g_signal_connect (G_OBJECT (main_win->list1), "activate",
					  G_CALLBACK (on_list1_activate),
					  main_win);
	g_signal_connect (G_OBJECT (main_win->list_tbt), "clicked",
					  G_CALLBACK (on_list_tbt_clicked),
					  main_win);

	g_signal_connect (G_OBJECT (main_win->diff1), "activate",
					  G_CALLBACK (on_diff1_activate),
					  main_win);
	g_signal_connect (G_OBJECT (main_win->diff_tbt), "clicked",
					  G_CALLBACK (on_diff_tbt_clicked),
					  main_win);
	
	g_signal_connect (G_OBJECT (main_win->update1), "activate",
					  G_CALLBACK (on_update1_activate),
					  main_win);
	g_signal_connect (G_OBJECT (main_win->update_tbt), "clicked",
					  G_CALLBACK (on_update_tbt_clicked),
					  main_win);
	
	g_signal_connect (G_OBJECT (main_win->installed_tbt), "clicked",
			  G_CALLBACK (on_installed_tbt_clicked),
			  main_win);

	g_signal_connect (G_OBJECT(main_win->ports_tree_view), "row-activated",
					  G_CALLBACK(ports_tree_row_activated_cb),
					  main_win);
	
	/* --------------------------------------------------------------------- */

	gtk_window_add_accel_group (GTK_WINDOW (main_win->win), main_win->accel_group);

	return main_win;
}

ResultDialog*
create_results_dlg (MainWindow* main_win)
{
	GtkWidget *dialog_vbox1;
	GtkWidget *scrolledwindow1;
	GtkWidget *dialog_action_area1;
	ResultDialog* results;
	
	results = (ResultDialog*)calloc (1, sizeof (ResultDialog)); 

	results->dialog = gtk_dialog_new ();
	gtk_widget_set_size_request (results->dialog, 500, 400);
	gtk_window_set_title (GTK_WINDOW (results->dialog), _("Updates Results"));
	gtk_window_set_position (GTK_WINDOW (results->dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (results->dialog), TRUE);
	gtk_window_set_default_size (GTK_WINDOW (results->dialog), 500, 500);
	gtk_window_set_resizable (GTK_WINDOW (results->dialog), FALSE);
	gtk_window_set_type_hint (GTK_WINDOW (results->dialog), GDK_WINDOW_TYPE_HINT_DIALOG);

	dialog_vbox1 = GTK_DIALOG (results->dialog)->vbox;
	gtk_widget_show (dialog_vbox1);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

	results->txt_result = gtk_text_view_new ();
	gtk_widget_show (results->txt_result);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), results->txt_result);
	gtk_container_set_border_width (GTK_CONTAINER (results->txt_result), 1);
	gtk_text_view_set_editable (GTK_TEXT_VIEW (results->txt_result), FALSE);
	gtk_text_view_set_left_margin (GTK_TEXT_VIEW (results->txt_result), 5);

	dialog_action_area1 = GTK_DIALOG (results->dialog)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

	results->btn_close = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (results->btn_close);
	gtk_dialog_add_action_widget (GTK_DIALOG (results->dialog), results->btn_close, GTK_RESPONSE_CLOSE);
	gtk_widget_set_sensitive (results->btn_close, FALSE);
	GTK_WIDGET_SET_FLAGS (results->btn_close, GTK_CAN_DEFAULT);
	
	g_signal_connect_after (G_OBJECT (results->dialog), "show",
			G_CALLBACK (on_dlg_results_show),
			main_win);

	return results;
}
