#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#define _(x) (x)

#include "debug.h"
#include "init.h"

#include "packages_db.h"
#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "common.h"

#include "interface.h"

unsigned char list_ports = 0;
unsigned char diff_ports = 0;
unsigned char updt_ports = 0;
unsigned char inst_ports = 0;
//unsigned char clean_ports = 0;

char collection[256] = {0};
//char package[256] = {0};

int parse_args(int argc, char* argv[])
{
	int c;
	int ret = 0;
	int digit_optind = 0;

	while (1) {
		static struct option long_options[] = {
			{"installed", 0, 0, 'i'},
			{"list", 0, 0, 'l'},
			{"diff", 0, 0, 'd'},
			{"update", 2, 0, 'u'},
			//{"clean", 2, 0, 'c'},
		};
		int opt_index = 0;
		c = getopt_long (argc, argv, "dilu", long_options, &opt_index);
		if (c == (-1)) {
			ret = c;
			break;
		}
		switch (c) {
			case 'i':
				inst_ports = 1;
				break;
			case 'l':
				list_ports = 1;
				break;
			case 'd':
				diff_ports = 1;
				break;
			case 'u':
				updt_ports = 1;
				if (optarg)
					strcpy (collection, optarg);
				break;
//			case 'c':
//				clean_ports = 1;
//				if (optarg)
//					strcpy (package, optarg);
//				break;
			case '?':
			default:
				break;
		}
		if (list_ports | diff_ports | updt_ports | inst_ports)
			break;
	}
	return ret;
}

int
main (int argc, char *argv[])
{
	load_config();
	MainWindow* main_win = NULL;
	Ports* ports = ports_new();
	
	if (argc == 1) {
		gtk_set_locale ();
		gtk_init (&argc, &argv);
	
		ports->refresh = (refresh_func)do_events;
		
		main_win = create_main_window ();
		main_win->ports = ports;
		show_main_window (main_win);

		gtk_main ();
	} else {
		if (parse_args(argc, argv) != 0)
			return (-1);
		else {
			ports->print = (print_func)vprintf;
			if (list_ports) {
				/* 
				 * List all ports
				 */
				ports_list_packages(ports);
			} else if (diff_ports) {
				/*
				 * List all ports difference
				 */
				ports_list_differences(ports);
			} else if (updt_ports) {
				/*
				 * Update ports collections
				 */
				if (strcmp(collection, "") != 0)
					ports_update_repository(ports, collection);
				else
					ports_update(ports);
			} else if (inst_ports) {
				/*
				 * List all installed ports
				 */
				ports_list_installed(ports);
			}
		}
	}
	return 0;
}
