#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <glib.h>

#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "debug.h"
#include "common.h"

static const char* NAME_TAG 	= "name=\0";
static const char* VERSION_TAG 	= "version=\0";
static const char* RELEASE_TAG 	= "release=\0";
static const char* SOURCE_TAG 	= "source=\0";

static const char* package_files[] = {
	".footprint",
	".md5sum",
	"Pkgfile",
};

const char* PACKAGE_FORMAT_STRING = "%-15s %-36s %-15s %-15s\n";

static int package_load_pkgfile(Package* package)
{
	FILE* fh;
	char* buffer;
	int buflen = 0;
	char* newline;
	char pkgfile_name[BUFSIZ];
	char release_version[32];
	
	memset(pkgfile_name, 0, sizeof(pkgfile_name));
	strcpy(pkgfile_name, package->collection->repository->base_directory);
	strcat(pkgfile_name, "/");
	strcat(pkgfile_name, package->collection->name);
	strcat(pkgfile_name, "/");
	strcat(pkgfile_name, package->name);
	strcat(pkgfile_name, "/");
	strcat(pkgfile_name, "Pkgfile");

	fh = fopen(pkgfile_name, "r");
	if (!fh) {
		debug_log(ERROR, "package_load_pkgfile(): could not open %s file", pkgfile_name);
		return (-1);
	}
	
	memset(release_version, 0, sizeof(release_version));
	while (getline(&buffer, &buflen, fh) != -1) {
		newline = strchr(buffer, '\n');
		if (newline)
			*newline = 0;
		if (strncmp(buffer, VERSION_TAG, strlen(VERSION_TAG)) == 0) {
			strcpy(release_version, buffer+strlen(VERSION_TAG));
		} else if (strncmp(buffer, RELEASE_TAG, strlen(RELEASE_TAG)) == 0) {
			strcat(release_version, "-");
			strcat(release_version, buffer+strlen(RELEASE_TAG));
			break;
		}
	}
	fclose (fh);
	if (buffer)
		free(buffer);
	
	if (*release_version == 0) {
		debug_log(ERROR, "package_load_pkgfile(): could not get version and release informations from %s file", pkgfile_name);
		return (-1);
	}
	
	strcpy(package->release_version, release_version);
	return 0;
}

Package* package_new(Collection* collection, char* name)
{
	Package* package = NULL;
	
	package = (Package*)calloc(1, sizeof(Package));
	if(!package) {
		debug_log(DEBUG, "package_new(): memoire insuffisante");
		return NULL;
	}
	
	package->collection = collection;
	strcpy(package->name, name);
	
	package_load_pkgfile(package);
	return package;
}

unsigned char is_package_installed(Package* package)
{
	return (strcmp(package->installed_version, "") != 0);
}

unsigned char is_package_uptodate(Package* package)
{
	return (is_package_installed(package) ? (strcmp(package->release_version, package->installed_version) == 0) : 1);
}

char* package_to_string(Package* package)
{
	char* result = NULL;
	
	result = (char*)calloc(100, 1);
	sprintf(result, PACKAGE_FORMAT_STRING,
		package->collection->name,
		package->name,
		package->release_version,
		package->installed_version);
	
	return result;
}

void print_packages_header(Ports* _ports)
{
	ports_on_print(_ports, PACKAGE_FORMAT_STRING,
		"Collection",
		"Name",
		"Ports",
		"Installed");
}

