#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <glib.h>

#include "packages_db.h"
#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "debug.h"
#include "common.h"

Ports* ports;

static gint compare_repository_list_data(gconstpointer object1, gconstpointer object2)
{
	return strcmp(((Repository*)object1)->name, ((Repository*)object2)->name);
}

Ports* ports_new()
{
	Ports* _ports = NULL;
	
	_ports = (Ports*)calloc(1, sizeof(Ports));
	if(!_ports) {
		debug_log(DEBUG, "ports_new(): Insuffisant Memory");
		return NULL;
	}

	_ports->repositories = get_repositories(_ports);
	if (!_ports->repositories) {
		debug_log(DEBUG, "ports_new(): Couldn't create repositories");
		return NULL;
	}
	_ports->repositories = g_list_sort(_ports->repositories, compare_repository_list_data);
	
	return _ports;
}

void ports_list_packages(Ports* _ports)
{
	GList* repo;
	GList* coll;
	GList* packages;
	Collection* collection;

	print_packages_header(_ports);
	repo = _ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			collection = (Collection*)coll->data;
			get_packages(collection, 0);
			packages = collection->packages;
			while (packages) {
				ports_on_print(_ports, package_to_string((Package*)packages->data));
				ports_on_refresh(_ports);
				packages = packages->next;
			}
			coll = coll->next;
		}
		repo = 	repo->next;
	}
}

void ports_list_differences(Ports* _ports)
{
	GList* repo;
	GList* coll;

	GHashTable* pkg_db = load_packages_db_file();
	if (!pkg_db) return;

	print_packages_header(_ports);
	repo = _ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			collection_list_differences((Collection*)coll->data, pkg_db);
			ports_on_refresh(_ports);
			coll = coll->next;
		}
		repo = repo->next;
	}
}

void ports_list_installed(Ports* _ports)
{
	GList* repo;
	GList* coll;

	GHashTable* pkg_db = load_packages_db_file();
	if (!pkg_db) return;

	print_packages_header(_ports);
	repo = _ports->repositories;
	while (repo) {
		coll = ((Repository*)repo->data)->collections;
		while (coll) {
			collection_list_installed((Collection*)coll->data, pkg_db);
			ports_on_refresh(_ports);
			coll = coll->next;
		}
		repo = repo->next;
	}
}

void ports_update(Ports* _ports)
{
	GList* repo;
	DriverBase* driver = NULL;
	char* result;
	char cmd[1024];
	
	repo = _ports->repositories;
	while (repo) {
		driver = driver_base_new(((Repository*)repo->data)->driver_type);
		memset (cmd, 0, sizeof(cmd));
		snprintf(cmd, sizeof(cmd), "%s %s", driver->command, ((Repository*)repo->data)->config_file);
		
		ports_on_print(_ports, "Updating %s collections...\n", ((Repository*)repo->data)->name);
		ports_on_refresh(_ports);
		
		driver->exec_shell(_ports, cmd, &result);
		
		ports_on_print(_ports, "%s\n", result);
		ports_on_refresh(_ports);
		
		free(driver);
		repo = repo->next;
	}
}

void ports_update_repository(Ports* _ports, const char* repo_name)
{
	GList* repo;
	DriverBase* driver = NULL;
	char* result;
	char cmd[1024];
	
	repo = _ports->repositories;
	while (repo) {
		if (strcmp(((Repository*)repo->data)->name, repo_name) == 0) {
			driver = driver_base_new(((Repository*)repo->data)->driver_type);
			memset (cmd, 0, sizeof(cmd));
			snprintf(cmd, sizeof(cmd), "%s %s", driver->command, ((Repository*)repo->data)->config_file);
			
			ports_on_print(_ports, "Updating %s collections...\n", ((Repository*)repo->data)->name);
			ports_on_refresh(_ports);
			
			driver->exec_shell(_ports, cmd, &result);
			
			ports_on_print(_ports, "%s\n", result);
			ports_on_refresh(_ports);
			
			free(driver);
		}
		repo = repo->next;
	}
}

void ports_on_refresh(Ports* _ports)
{
	if (_ports->refresh)
		_ports->refresh();
}

void ports_on_print(const Ports* _ports, const char* format, ...)
{
	va_list ap;
	if (_ports->print) {
		va_start (ap, format);
		(_ports->print)(format, ap);
		va_end (ap);
	}
}

