#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <glib.h>

#include "ports.h"
#include "repository.h"
#include "driver_base.h"
#include "collection.h"
#include "package.h"
#include "debug.h"
#include "common.h"

Repository* repository_new(Ports* _ports, char* name, char* config_file)
{
	Repository* repository = NULL;
	
	repository = (Repository*)calloc(1, sizeof(Repository));
	if(!repository) {
		debug_log(DEBUG, "repository_new(): Insuffisant Memory");
		return NULL;
	}

	repository->ports = _ports;
	
	strcpy(repository->name, name);
	strcpy(repository->config_file, config_file);
	
	return repository;
}

GList* get_repositories(Ports* _ports)
{
	DIR* dirp;
	struct dirent* dp;
	struct stat node_stat;
	char path[BUFSIZ];
	char repo_name[256];
	char* extpos = NULL;
	int config_dir_len = strlen(config_dir);

	GList* repositories = NULL;
	Repository* repository = NULL;
	DriverBase* driver = NULL;

	dirp = opendir (config_dir);
	while ( (dp = readdir (dirp)) != NULL ) {
		driver = NULL;
		memset(path, 0, sizeof(path));
		memset(repo_name, 0, sizeof(repo_name));

		strcpy(path, config_dir);
		strcat(path, "/");
		strcat(path, dp->d_name);

		stat (path, &node_stat);
		if (S_ISREG(node_stat.st_mode)) {
			strcpy(repo_name, path+config_dir_len+1);
			extpos = strstr(repo_name, ".");
			if (extpos) {
				*extpos = '\0';
				if (strcmp(extpos+1, "cvsup") == 0
				 || strcmp(extpos+1, "httpup") == 0
				 || strcmp(extpos+1, "rsync") == 0) {
				    repository = repository_new(_ports, repo_name, path);
				    if (strcmp(extpos+1, "cvsup") == 0)
					    repository->driver_type = CVSUP;
				    else if (strcmp(extpos+1, "httpup") == 0)
					    repository->driver_type = HTTPUP;
				    else if (strcmp(extpos+1, "rsync") == 0)
					    repository->driver_type = RSYNC;
				    driver = driver_base_new(repository->driver_type);
				    if ((driver->load_config_file)(repository) == (-1))
					    return NULL;
				    repositories = g_list_append(repositories, repository);
				}
			}
		}
	}
	closedir(dirp);

	return repositories;
}
