/*
 * Copyright 2004-2009 Alper Akcan <alper.akcan@gmail.com>
 * Copyright 2012 Vasily Khoruzhick <anarsoul@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 */

#include "xynth_server.h"

#if defined(CONFIG_IMAGE_PNG)

#include <png.h>

int s_image_png_is (char *file)
{
	FILE *fp;
	int ret = -1;
	char magic[8];

	if ((fp = fopen(file, "rb")) == NULL) {
		debugf(DCLI | DFAT, "Could not open file (%s) for reading", file);
	}
	fread(magic, 1, sizeof(magic), fp);
	ret = png_sig_cmp((png_bytep) magic, (png_size_t) 0, 8);
	fclose(fp);

	return ret;
}

int s_image_png (char *file, s_image_t *img)
{
	int x;
	int y;
	FILE *fp;
	png_uint_32 width, height;
	char header[8];
	unsigned int *tmp;
	int bit_depth, color_type;
	png_infop info_ptr;
	png_structp png_ptr;
	int __attribute__((unused)) number_of_passes;
	png_bytep *row_pointers;
	
	fp = fopen(file, "rb");
	if (!fp) {
		debugf(DFAT, "Could not open file %s", file);
	}
	fread(header, 1, 8, fp);
	if (png_sig_cmp((png_bytep) header, 0, 8)) {
		debugf(DFAT, "File is not a PNG file %s", file);
	}
	
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if (!png_ptr) {
		debugf(DFAT, "png_create_read_struct failed");
	}

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr) {
		debugf(DFAT, "png_create_info_struct failed");
	}

	if (setjmp(png_jmpbuf(png_ptr))) {
		debugf(DFAT, "setjmp failed");
	}
	
	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);
	png_read_info(png_ptr, info_ptr);
	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type,
		NULL, NULL, NULL);
	number_of_passes = png_set_interlace_handling(png_ptr);
	png_read_update_info(png_ptr, info_ptr);
	
	if (setjmp(png_jmpbuf(png_ptr))) {
		debugf(DFAT, "Error during read_image");
	}

	row_pointers = (png_bytep *) s_malloc(sizeof(png_bytep) * height);
	for (y = 0; y < height; y++) {
		row_pointers[y] = (png_byte* ) s_malloc(png_get_rowbytes(png_ptr, info_ptr));
	}

        png_read_image(png_ptr, row_pointers);
	
	img->w = width;
	img->h = height;
	img->rgba = (unsigned int *) s_calloc(1, img->w * img->h * sizeof(unsigned int));
	tmp = img->rgba;

	if (color_type == PNG_COLOR_TYPE_RGBA) {
		for (y = 0; y < height; y++) {
			png_byte *row = row_pointers[y];
			for (x = 0; x < width; x++) {
				png_byte *ptr = &(row[x * 4]);

				*tmp |= (ptr[0] << 0x18);
				*tmp |= (ptr[1] << 0x10);
				*tmp |= (ptr[2] << 0x08);
				*tmp |= (~(ptr[3] << 0x00)) & 0xFF;

				ptr[0] = 0;
				ptr[1] = ptr[2];
				tmp++;
			}
		}
	} else if (color_type == PNG_COLOR_TYPE_RGB) {
		for (y = 0; y < height; y++) {
			png_byte *row = row_pointers[y];
			for (x = 0; x < width; x++) {
				png_byte *ptr = &(row[x * 3]);

				*tmp |= (ptr[0] << 0x18);
				*tmp |= (ptr[1] << 0x10);
				*tmp |= (ptr[2] << 0x08);
				*tmp |= 0x0;

				ptr[0] = 0;
				ptr[1] = ptr[2];
				tmp++;
			}
		}
	} else if (color_type == PNG_COLOR_TYPE_PALETTE) {
		int i;
		png_colorp palette;
		png_bytep trans_alpha;
		png_color_16p trans_color;
		int num_palette, num_trans;
		struct Palette {
			int r;
			int g;
			int b;
			int a;
		};
		struct Palette *pal;
		pal = (struct Palette *) s_malloc(256 * sizeof(struct Palette));
		memset(pal, 0, 256 * sizeof(struct Palette));
		
		for (i = 0; i < 256; i++) {
			pal[i].a = 255;
		}
		
		if (png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette)) {
			if (png_get_tRNS(png_ptr, info_ptr,
			    &trans_alpha, &num_trans, &trans_color)) {
				for (i = 0; i < num_palette; i++) {
					pal[i].r = palette[i].red;
					pal[i].g = palette[i].green;
					pal[i].b = palette[i].blue;
					if (i < num_trans) {
						pal[i].a = trans_alpha[i];
					}
				}
				for (y = 0; y < height; y++) {
					png_byte *row = row_pointers[y];
					for (x = 0; x < width; x++) {
						png_byte *ptr = &(row[x]);
						*tmp |= (pal[ptr[0]].r << 0x18);
						*tmp |= (pal[ptr[0]].g << 0x10);
						*tmp |= (pal[ptr[0]].b << 0x08);
						*tmp |= (~(pal[ptr[0]].a << 0x00)) & 0xff;
						tmp++;
					}
				}
			}
		}

		s_free(pal);
	} else {
		debugf(DFAT, "Unknown color_type : %d (%s)", color_type, file);
	}

	for (y = 0; y < height; y++) {
		s_free(row_pointers[y]);
	}
	s_free(row_pointers);
	
	png_destroy_info_struct(png_ptr, &info_ptr);
	png_destroy_read_struct(&png_ptr, NULL, NULL);
	
	fclose(fp);
	
	return 0;
}

#else

int s_image_png_is (char *file)
{
	return -1;
}

int s_image_png (char *file, s_image_t *img)
{
	return -1;
}

#endif
