/*
 * Copyright © 2011
 * Andrea Costa <nukemup@hotmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 * or write to the Free Software Foundation, Inc., 51 Franklin St
 * Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "temp_notifier.h"

extern void temp_notifier_about (GtkMenuItem *menuitem, gpointer user_data);
extern void temp_notifier_setting (GtkButton *button, temp_notifier_s *core);

/*
  Show menu when you click in GtkStatusIcon
*/
int
icon_popup_menu (GtkStatusIcon *icon, gint button, gint time, GtkMenu *menu)
{
  gtk_menu_popup (menu, NULL, NULL, gtk_status_icon_position_menu, icon, button, time);
  return TRUE;
}

/*
  Check temperature
*/
int
temp_notifier_check_temp (int temp, int critical_temp)
{
  if (temp >= critical_temp)
    return -1;
  return 0;
}


/*
  Read data and update tooltip
*/
gboolean
temp_notifier_timeout_func (temp_notifier_s *core)
{
  FILE *file;
  char string[50];
  char temperature[6];
  int i = 0, j = 0;

  /* Read CPU0's temperature */
  file = fopen (core->thermal_file_cpu0, "r");
  if (file != NULL)
  {
    fread (&string, sizeof (string), 1, file);
    snprintf (temperature, 3, "%c%c\0", string[25], string[26]);
    fclose (file);

    core->cpu0_temp = atoi (temperature);

    /* Check temperature */
    i = temp_notifier_check_temp (core->cpu0_temp, core->critical_temp);
	}
  else
    core->cpu0_temp = -1;
	
  /* Read CPU0's temperature */
  file = fopen (core->thermal_file_cpu1, "r");
  if (file != NULL)
  {
    fread (&string, sizeof (string), 1, file);
    snprintf (temperature, 3, "%c%c\0", string[25], string[26]);
    fclose (file);

    core->cpu1_temp = atoi (temperature);

    /* Check temperature */
    j = temp_notifier_check_temp (core->cpu1_temp, core->critical_temp);
  }
  else
    core->cpu1_temp = -1;
	
  /* Check GtkStatusIcon */
  if ((i == -1) || (j == -1))
    gtk_status_icon_set_blinking (GTK_STATUS_ICON(core->icon), TRUE);
  else
    gtk_status_icon_set_blinking (GTK_STATUS_ICON(core->icon), FALSE);

  /* Update GtkTooltip */
  if ((core->cpu0_temp == -1) && (core->cpu1_temp == -1))
    gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON(core->icon), g_strdup_printf ("- CPU1: ERROR\n- CPU2: ERROR"));
  else if (core->cpu0_temp == -1)
    gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON(core->icon), g_strdup_printf ("- CPU1: ERROR\n- CPU2: %d C", core->cpu1_temp));
  else if (core->cpu1_temp == -1)
    gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON(core->icon), g_strdup_printf ("- CPU1: %d C\n- CPU2: ERROR", core->cpu0_temp));
  else
    gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON(core->icon), g_strdup_printf ("- CPU1: %d C\n- CPU2: %d C", core->cpu0_temp, core->cpu1_temp));

  return TRUE;
}


/*
  Create GtkStatusIcon menu
*/
void
temp_notifier_create_icon_menu (temp_notifier_s *core)
{
  core->icon_menu = gtk_menu_new ();
  core->setting_item = gtk_menu_item_new_with_label ("Setting");
  gtk_menu_shell_append (GTK_MENU_SHELL(core->icon_menu), core->setting_item);
  core->about_item = gtk_menu_item_new_with_label ("About");
  gtk_menu_shell_append (GTK_MENU_SHELL(core->icon_menu), core->about_item);
  core->quit_item = gtk_menu_item_new_with_label ("Quit");
  gtk_menu_shell_append (GTK_MENU_SHELL(core->icon_menu), core->quit_item);
  gtk_widget_show_all (core->icon_menu);
}	


/*
  Quit all - Quit item
*/
void
temp_notifier_quit_menu (GtkMenuItem *menuitem, temp_notifier_s *core)
{
  g_free (core);
  gtk_main_quit ();
}


int
main (int argc, char *argv[])
{
  /* Init */
  gtk_init(&argc, &argv);

  /* Create app's data structure */
  temp_notifier_s *core;
  core = g_new0 (temp_notifier_s, 1);

  /* Parse the XML conf. file */
  if (temp_notifier_xml_parse (core, XML_FILE) == -1)
    libui_gtk_dialog_error ("Impossibilie avviare l'applicazione\n. XML file missing!");

	
	core->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  core->icon = (GtkWidget *) libui_gtk_gadget_window_icon_new (core->window,
                                                               IMAGE_LOGO,
                                                               "Temperature Notifier");

  /* Menu status icon */
  temp_notifier_create_icon_menu (core);

  g_signal_connect (core->icon, "popup-menu",G_CALLBACK(icon_popup_menu), core->icon_menu);
  gtk_signal_connect (GTK_OBJECT(core->setting_item), "activate", G_CALLBACK (temp_notifier_setting), core);
  gtk_signal_connect (GTK_OBJECT(core->about_item), "activate", G_CALLBACK (temp_notifier_about), NULL);
  gtk_signal_connect (GTK_OBJECT(core->quit_item), "activate", G_CALLBACK (temp_notifier_quit_menu), core);
  g_timeout_add (2000, (GSourceFunc) temp_notifier_timeout_func, core);
  g_signal_connect (G_OBJECT(core->window), "destroy", G_CALLBACK(gtk_main_quit), NULL);

 	gtk_main ();
	
	return 0;
}	

