/*
 * Copyright © 2011
 * Andrea Costa <nukemup@hotmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 * or write to the Free Software Foundation, Inc., 51 Franklin St
 * Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "temp_notifier.h"


/*
  Checks temp data 
*/
int
temp_notifier_check_data (char a, char b)
{
  if ((a < '0') || (a > '9'))
    return -1;
  if ((b < '0') || (b > '9'))
    return -1;
  return 0;
}


/*
  Creates & runs dialog setting
*/
void
temp_notifier_setting (GtkButton *button, temp_notifier_s *core)
{
  GtkWidget *dialog;
  GtkWidget *label, *hbox, *entry_cpu1, *entry_cpu2, *entry_logo, *entry_temp;
  char *data;
  int check;

  dialog = gtk_dialog_new_with_buttons ("Information", NULL, GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
  gtk_window_set_position (GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW(dialog), 150, 120);
  gtk_window_set_title (GTK_WINDOW(dialog), "Temp Notifier - Setting");
  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);


  label = libui_gtk_label_new_with_descriptor ("- Temp. Source Files -", "sans bold 9", -1, -1, NULL);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 10);

  hbox = gtk_hbox_new (FALSE, 0);
    label = libui_gtk_label_new_with_descriptor ("CPU1:", "sans 8", 60, 20, NULL);
    gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);

    entry_cpu1 = libui_gtk_entry_new_with_descriptor (TRUE, 100, core->thermal_file_cpu0, "sans 8", 350, 20, NULL);
    gtk_box_pack_start (GTK_BOX(hbox), entry_cpu1, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show_all (hbox);

  hbox = gtk_hbox_new(FALSE, 0);
    label = libui_gtk_label_new_with_descriptor ("CPU2:", "sans 8", 60, 20, NULL);
    gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);

    entry_cpu2 = libui_gtk_entry_new_with_descriptor (TRUE, 100, core->thermal_file_cpu1, "sans 8", 350, 20, NULL);
    gtk_box_pack_start (GTK_BOX(hbox), entry_cpu2, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show_all (hbox);
	
  label = libui_gtk_label_new_with_descriptor ("- Logo Path -", "sans bold 9", -1, -1, NULL);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 10);

  hbox = gtk_hbox_new (FALSE, 0);
    label = libui_gtk_label_new_with_descriptor ("Image:", "sans 8", 60, 20, NULL);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    entry_logo = libui_gtk_entry_new_with_descriptor (TRUE, 100, core->logo, "sans 8", 350, 20, NULL);
    gtk_box_pack_start(GTK_BOX(hbox), entry_logo, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);
	
  label = libui_gtk_label_new_with_descriptor ("- Critical Temp. [°C] -", "sans bold 9", 200, 20, NULL);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 5);

  hbox = gtk_hbox_new(FALSE, 0);
    entry_temp = libui_gtk_entry_new_with_descriptor (TRUE, 2, g_strdup_printf ("%d", core->critical_temp), "sans 8", 25, 20, NULL);
    gtk_box_pack_start (GTK_BOX(hbox), entry_temp, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show_all (hbox);	
	
  gtk_widget_show_all ((GTK_DIALOG(dialog)->vbox));

  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
    case GTK_RESPONSE_DELETE_EVENT:
      gtk_widget_destroy (dialog);
      break;
    case GTK_RESPONSE_CANCEL:
      gtk_widget_destroy (dialog);
      break;
    case GTK_RESPONSE_OK:
      /* Critical temp */
			data = (gchar *) gtk_entry_get_text (GTK_ENTRY(entry_temp));
			check = temp_notifier_check_data (data[0], data[1]);

			if (check == 0)
        core->critical_temp = atoi (data);

      /* CPU0 temp file */
      core->thermal_file_cpu0 = malloc ((strlen (gtk_entry_get_text (GTK_ENTRY(entry_cpu1))) + 1) * sizeof (char));
      snprintf (core->thermal_file_cpu0, strlen((char *) gtk_entry_get_text (GTK_ENTRY(entry_cpu1))) + 1, "%s", (char *) gtk_entry_get_text (GTK_ENTRY(entry_cpu1)));

      /* CPU1 temp file */
      core->thermal_file_cpu1 = malloc ((strlen (gtk_entry_get_text (GTK_ENTRY(entry_cpu2))) + 1) * sizeof (char));
      snprintf (core->thermal_file_cpu1, strlen((char *) gtk_entry_get_text (GTK_ENTRY(entry_cpu2))) + 1, "%s", (char *) gtk_entry_get_text (GTK_ENTRY(entry_cpu2)));

      /* Logo */
      core->logo = malloc ((strlen (gtk_entry_get_text (GTK_ENTRY(entry_logo))) + 1) * sizeof (char));
      snprintf (core->logo, strlen (gtk_entry_get_text (GTK_ENTRY(entry_logo))) + 1, "%s", gtk_entry_get_text (GTK_ENTRY(entry_logo)) );

      /* update setting */
      temp_notifier_xml_update_setting (core, XML_FILE);

      gtk_widget_destroy (dialog);
      break;
  } 
}
