/*
 * Copyright © 2011
 * Andrea Costa <nukemup@hotmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 * or write to the Free Software Foundation, Inc., 51 Franklin St
 * Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "temp_notifier.h"

#include <libxml/parser.h>
#include <libxml/xinclude.h>
#include <libxml/tree.h>
#include <libxml/xmlIO.h>
#include <string.h>


/*
 Get all XML configuration data
*/
int
temp_notifier_xml_parse (temp_notifier_s *core, char *xml_file)
{
  xmlDocPtr doc;
  xmlNodePtr cur;
  char *data;

  doc = xmlParseFile (xml_file);
	
  if (doc == NULL)
    return -1;

  cur = xmlDocGetRootElement (doc);

  if (cur == NULL)
  {
    xmlFreeDoc (doc);
    return -1;
  }
	
  if (xmlStrcmp (cur->name, (const xmlChar *) "temp-notifier"))
  {
    xmlFreeDoc (doc);
    return -1;
  }

  cur = cur->xmlChildrenNode;

  while (cur != NULL)
  {
    if ((!xmlStrcmp (cur->name, (const xmlChar *) "setting")))
    {
      /* CPU1 thermal file */
      core->thermal_file_cpu0 = malloc ((strlen((char *) xmlGetProp (cur, "thermal_file_1")) + 1) * sizeof (char));
      snprintf (core->thermal_file_cpu0, strlen((char *) xmlGetProp (cur, "thermal_file_1")) + 1, "%s", (char *) xmlGetProp (cur, "thermal_file_1"));
      /* CPU1 thermal file */
      core->thermal_file_cpu1 = malloc ((strlen((char *) xmlGetProp (cur, "thermal_file_2")) + 1) * sizeof (char));
      snprintf (core->thermal_file_cpu1, strlen((char *) xmlGetProp (cur, "thermal_file_2")) + 1, "%s", (char *) xmlGetProp (cur, "thermal_file_2"));
      /* Critical temperature */
      core->critical_temp = atoi ((char *) xmlGetProp (cur, "critical_temperature"));

      data = (char *) xmlGetProp (cur, "logo");

      if ((int) data[0] == 0)
      {
        core->logo = malloc ((strlen (IMAGE_LOGO) + 1) * sizeof (char));
        snprintf (core->logo, strlen (IMAGE_LOGO) + 1, "%s", IMAGE_LOGO);
      }
		}

		cur = cur->next;
	}

  /* Print app's info */
  g_print ("*   CPU0 temp file: %s\n", core->thermal_file_cpu0);
  g_print ("*   CPU1 temp file: %s\n", core->thermal_file_cpu1);
  g_print ("*   Critical temp:  %d\n", core->critical_temp);

	return 0;
}


/*
 Update XML setting
*/
int
temp_notifier_xml_update_setting (temp_notifier_s *core, char *xml_file)
{
  xmlDocPtr doc;
  xmlNodePtr cur;

  doc = xmlParseFile (xml_file);
	
  if (doc == NULL )
    return -1;

	cur = xmlDocGetRootElement (doc);

  if (cur == NULL)
  {
    xmlFreeDoc (doc);
    return -1;
  }
	
  if (xmlStrcmp (cur->name, (const xmlChar *) "temp-notifier"))
  {
    xmlFreeDoc (doc);
    return -1;
  }

  cur = cur->xmlChildrenNode;

  while (cur != NULL)
  {
    if ((!xmlStrcmp(cur->name, (const xmlChar *) "setting")))
    {
      xmlSetProp (cur, "thermal_file_1", g_strdup_printf ("%s", core->thermal_file_cpu0));
      xmlSetProp (cur, "thermal_file_2", g_strdup_printf ("%s", core->thermal_file_cpu1));
      xmlSetProp (cur, "critical_temperature", g_strdup_printf ("%d", core->critical_temp));
    }
    cur = cur->next;
  }

  if (doc != NULL)
  {
    xmlSaveFormatFile (xml_file, doc, 0);
    xmlFreeDoc (doc);
  }
	return 0;
}
