/*

    filp_interp.c

    Filp Interpreter.

    filp - Embeddable, Reverse Polish Notation Programming Language

    Copyright (C) 2000/2004 Angel Ortega <angel@triptico.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    http://www.triptico.com

*/

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "filp.h"

#ifndef FILP_ADDON_FUNC
#define FILP_ADDON_FUNC (void)0
#endif


int main(int argc, char *argv[])
{
	int n;
	char *prg = NULL;
	char *iprg = NULL;

	if (argc > 1 && strcmp(argv[1], "--version") == 0) {
		printf("filp " VERSION
		       " - Embeddable, Reverse Polish Notation Programming Language\n");
		printf("Copyright (C) 2000/2004 Angel Ortega <angel@triptico.com>\n");
		exit(0);
	}

	filp_startup();

	/* awesome help function */
	filp_exec("/help { "
		  "tpop \"filp_lib.c\" open dup "
		  "{ /f #= } { \"Cannot open 'filp_lib.c'\" ? break } ifelse "
		  "{ $f read } "
		  "{ chop dup dup \"/**\" instr # tpush instr && "
		  "{ ' (filp_lib)' . ? break } { pop } ifelse } while "
		  "$f close "
		  "\"filp_slib.c\" open dup "
		  "{ /f #= } { \"Cannot open 'filp_slib.c'\" ? break } ifelse "
		  "{ $f read } "
		  "{ chop dup dup \"/**\" instr # tpush instr && "
		  "{ ' (filp_slib)' . ? break } { pop } ifelse } while " "$f close } set ");

	/* awesome benchmark function */
	filp_exec("/_filp_benchmark { "
		  "/_ 0 = "
		  "time { dup time != { break } if } loop "
		  "time { dup time != { break } if /_ ++ } loop " "pop pop $_ } set");

	filp_exec("{ './config.filp' load } eval pop");

	FILP_ADDON_FUNC;

	n = 1;

	if (argc > n && strcmp(argv[n], "-r") == 0) {
		/* -r activates real numbers */
		_filp_real = 1;
		n++;
	}

	if (argc > n) {
		if (strcmp(argv[n], "-e") == 0) {
			n++;
			if (argc > n)
				iprg = argv[n];
		}
		else
			prg = argv[n];

		n++;
	}

	/* store the rest as ARGV parameters */
	filp_exec("/ARGV (");

	for (; n < argc; n++)
		filp_scalar_push(argv[n]);
	filp_exec(") =");

	if (iprg)
		filp_exec(iprg);
	else if (prg)
		filp_load_exec(prg);
	else
		filp_console();

	filp_shutdown();

	return 0;
}
