/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_ARRAY_H_
#define SHTTP_ARRAY_H_

#include <sys/types.h>
#include <stdbool.h>

#include <shttp_error.h>

#define SHTTP_ARRAY_RECOMMENDED_ALLOCATE 8

typedef struct shttp_array_t {
  void *elems;
  int length;
  int allocated;
  size_t elem_size;
} shttp_array_t;

shttp_error_t       shttp_array_init        (shttp_array_t *array,
                                             size_t elem_size, 
                                             int allocate);

void                shttp_array_cleanup     (shttp_array_t *array);

shttp_error_t       shttp_array_append      (shttp_array_t *array, 
                                             void *elem);

#define shttp_array_at(_array, _idx, _type) \
    ((_type *) (((char *)(_array)->elems) + ((_array)->elem_size * _idx)))

#endif // SHTTP_ARRAY_H_
