/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_BUFFER_H_
#define SHTTP_BUFFER_H_

#include <stddef.h>
#include <stdbool.h>

#include <shttp_error.h>

#define SHTTP_BUFFER_RECOMMENDED_ALLOCATE 4096

typedef struct shttp_buffer_t {
	char *data;
	size_t length;
	size_t allocated;
} shttp_buffer_t;

shttp_error_t       shttp_buffer_init       (shttp_buffer_t *buffer,
                                             size_t allocate);

void                shttp_buffer_cleanup    (shttp_buffer_t *buffer);

bool                shttp_buffer_append     (shttp_buffer_t *buffer, 
                                             const char *data, 
                                             size_t length);

void                shttp_buffer_clear      (shttp_buffer_t *buffer);

void                shttp_buffer_rtrim      (shttp_buffer_t *buffer, 
                                             const char *trim_chars);

size_t              shttp_buffer_find_char  (const shttp_buffer_t *buffer, 
                                             size_t start_pos, 
                                             char ch);

#endif // SHTTP_BUFFER_H_
