/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_ERROR_H_
#define SHTTP_ERROR_H_

#include <stdarg.h>

typedef enum shttp_error_t {
    SHTTP_OK = 0,
    SHTTP_NOTHING = 1,
    SHTTP_MEM_ERROR = 2,
    SHTTP_RUNTIME_ERROR = 3,
    SHTTP_MALFORMED_URI = 4,
    SHTTP_INVALID_REQUEST = 5,
    SHTTP_MALFORMED_REQUEST = 6,
    SHTTP_UNKNOWN_TRANSFER_ENCODING = 7,
    SHTTP_NETWORK_ERROR = 8,
} shttp_error_t;

typedef enum shttp_log_level_t {
    SHTTP_DEBUG,
    SHTTP_INFO,
    SHTTP_WARN,
    SHTTP_ERR
} shttp_log_level_t;

typedef void    (*shttp_log_callback_t) (shttp_log_level_t level, 
                                         int line,
                                         char *file,
                                         const char *msg);

void            shttp_log_              (shttp_log_level_t level, 
                                         int line, 
                                         char *file, 
                                         const char *fmt, 
                                         ...);

void            shttp_log_errno_        (shttp_log_level_t level, 
                                         int code, 
                                         int line,
                                         char *file,
                                         const char *fmt, 
                                         ...);

void            shttp_set_log_callback  (shttp_log_callback_t callback);

const char *    shttp_log_level_str     (shttp_log_level_t level);

#define shttp_log(_level, ...) \
    shttp_log_((_level), __LINE__, __FILE__, __VA_ARGS__)

#define shttp_log_errno(_level, _code, ...) \
    shttp_log_errno_((_level), (_code), __LINE__, __FILE__, __VA_ARGS__)

#endif // SHTTP_ERROR_H_
