/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_REQUEST_PARSER_H_
#define SHTTP_REQUEST_PARSER_H_

#include <shttp_error.h>
#include <shttp_buffer.h>
#include <shttp_array.h>
#include <shttp_uri.h>
#include <shttp_http.h>
#include <shttp_table.h>

typedef enum shttp_request_parser_state_t {
    SHTTP_PARSE_DONE = 0,
    SHTTP_PARSE_REQUEST_LINE = 1,
    SHTTP_PARSE_HEADERS = 2,
    SHTTP_PARSE_BODY = 3,
} shttp_request_parser_state_t;

typedef shttp_error_t (*shttp_request_parser_body_callback_t)(const char *data,
                                                              size_t length);

typedef struct shttp_request_parser_t {
    shttp_request_parser_state_t state;
    shttp_error_t error;
    shttp_buffer_t staging_buffer;
    shttp_buffer_t header_buffer;
    shttp_array_t body_callbacks;
    size_t content_length;
    size_t content_remaining;

    shttp_method_t method;
    shttp_uri_t uri;
    shttp_version_t version;
    shttp_table_t headers;
} shttp_request_parser_t;

shttp_error_t       shttp_request_parser_init               (shttp_request_parser_t *parser);

void                shttp_request_parser_cleanup            (shttp_request_parser_t *parser);

shttp_error_t       shttp_request_parser_add_body_callback  (shttp_request_parser_t *parser,
                                                             shttp_request_parser_body_callback_t callback);

size_t              shttp_request_parser_feed               (shttp_request_parser_t *parser,
                                                             const char *data,
                                                             size_t length);

#endif // SHTTP_REQUEST_PARSER_H_
