/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_TABLE_H_
#define SHTTP_TABLE_H_

#include <shttp_error.h>
#include <shttp_array.h>

#define SHTTP_TABLE_RECOMMENDED_ALLOCATE SHTTP_ARRAY_RECOMMENDED_ALLOCATE

typedef struct shttp_table_entry_t {
    char *name;
    char *value;
} shttp_table_entry_t;

typedef struct shttp_table_t {
    shttp_array_t entries;
} shttp_table_t;

shttp_error_t           shttp_table_init            (shttp_table_t *table,
                                                     int allocate);

void                    shttp_table_cleanup         (shttp_table_t *table);

shttp_table_entry_t *   shttp_table_find            (shttp_table_t *table,
                                                     char *name);

shttp_error_t           shttp_table_add             (shttp_table_t *table,
                                                     char *name,
                                                     char *value);

shttp_error_t           shttp_table_append          (shttp_table_t *table,
                                                     char *name,
                                                     char *value,
                                                     char sep);

#define shttp_table_at(_table, _idx) \
    shttp_array_at((_table)->entries, _idx, shttp_table_entry_t)

static inline int
shttp_table_num_elems(shttp_table_t *table)
{
    return table->entries.length;
}

#endif // SHTTP_TABLE_H_
