/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_URI_H_
#define SHTTP_URI_H_

#include <stdbool.h>

#include <shttp_error.h>

typedef enum shttp_uri_type_t {
    SHTTP_ABS_PATH = 1,
    SHTTP_ABS_URI = 2,
} shttp_uri_type_t;

typedef enum shttp_uri_scheme_t {
    SHTTP_HTTP = 1,
} shttp_uri_scheme_t;

typedef struct shttp_uri_t {
    shttp_uri_type_t type;
    shttp_uri_scheme_t scheme;
    char *host;
    int port;
    char *path;
    char *query;
} shttp_uri_t;

shttp_error_t       shttp_uri_init      (shttp_uri_t *uri);

void                shttp_uri_cleanup   (shttp_uri_t *uri);

shttp_error_t       shttp_uri_parse     (shttp_uri_t *uri,
                                         const char *uri_str);

#endif // SHTTP_URI_H_
