/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#ifndef SHTTP_UTIL_H_
#define SHTTP_UTIL_H_

#include <stdbool.h>
#include <assert.h>

#define SHTTP_STR_AND_LEN(_s) \
    (_s), (sizeof(_s)-1)

#define SHTTP_STR_LEN(_s) \
    (sizeof(_s)-1)

#define SHTTP_MIN(_a, _b) \
    ((_a) <= (_b) ? (_a) : (_b))

#define SHTTP_MAX(_a, _b) \
    ((_a) >= (_b) ? (_a) : (_b))

static inline bool
shttp_is_digit(char ch)
{
    return ch >= '0' && ch <= '9';
}

static inline int
shttp_digit(char ch)
{
    return ch - '0';
}

static inline bool
shttp_is_cr_or_lf(char ch)
{
    return ch == '\r' || ch == '\n';
}

static inline bool
shttp_is_continuation(char ch)
{
    return ch == ' ' || ch == '\t';
}

static inline bool
shttp_is_control(char ch)
{
    return (ch >=0 && ch <= 31) || ch == 127;
}

static inline bool
shttp_is_lws(char ch)
{
    return shttp_is_cr_or_lf(ch) || shttp_is_continuation(ch);
}

static inline bool
shttp_is_separator(char ch)
{
    return ch == '(' || ch == ')' || ch == '<' || ch == '>' || ch == '@' ||
           ch == ',' || ch == ';' || ch == ':' || ch == '\\' || ch == '"' ||
           ch == '/' || ch == '[' || ch == ']' || ch == '?' || ch == '=' ||
           ch == '{' || ch == '}' || ch == ' ' || ch == '\t';
}

static inline bool
shttp_is_token(char ch)
{
    return !shttp_is_control(ch) && !shttp_is_separator(ch);
}

static inline bool
shttp_is_upper(char ch)
{
    return ch >= 'A' && ch <= 'Z';
}

static inline char
shttp_lower(char ch)
{
    return ch + 0x20;
}

#endif // SHTTP_UTIL_H_
