/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#include <shttp_array.h>

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <shttp_mem.h>

static inline void *
shttp_array_get_mem_loc(shttp_array_t *array, int idx) 
{
    return ((char *) array->elems) + (array->elem_size * array->length);
}

static bool
shttp_array_expand(shttp_array_t *array, int allocate)
{
    assert(allocate > array->allocated);

    void *new_elems = realloc(array->elems, array->elem_size * allocate);

    if (!new_elems)
        return false;

    array->elems = new_elems;
    array->allocated = allocate;

    return true;
}

shttp_error_t
shttp_array_init(shttp_array_t *array, size_t elem_size, int allocate)
{
    assert(array != NULL);
    assert(allocate >= 0);
    assert(elem_size >= 0);

    shttp_error_t ret;

    array->length = 0;
    array->allocated = allocate;
    array->elem_size = elem_size;

    array->elems = shttp_malloc(elem_size * allocate);

    if (!array->elems) {
        ret = SHTTP_MEM_ERROR;
        goto malloc_failed;
    }

    return SHTTP_OK;

malloc_failed:
    return ret;
}

void
shttp_array_cleanup(shttp_array_t *array)
{
    if (array) {
        shttp_free(array->elems);
        array->elems = NULL;
    }
}

shttp_error_t
shttp_array_append(shttp_array_t *array, void *elem)
{
    assert(array != NULL);
    assert(elem != NULL);
    assert(array->length < array->allocated);

    if (array->length == array->allocated)
        if (!shttp_array_expand(array, array->length * 2))
            return SHTTP_MEM_ERROR;

    void *mem_loc = shttp_array_get_mem_loc(array, array->length);
    memcpy(mem_loc, elem, array->elem_size);

    array->length++;

    return SHTTP_OK;
}
