/*
Copyright (c) 2011, Christopher Utz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of SimpleHttp nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER UTZ BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Author: Christopher Utz <utz.m.christopher@gmail.com>
*/

#include <shttp_string.h>

#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include <shttp_mem.h>

char *
shttp_string_vprintf(const char *fmt, int *length, va_list ap)
{
    assert(fmt != NULL);

    int actual_length;
    int curr_size;

    if (length && *length > 0)
        curr_size = *length;
    else
        curr_size = 16;

    char *str = shttp_malloc(curr_size);

    if (!str)
        goto malloc_failed;

    for (;;) {
        // if the return of vsnprintf is >= size or -1, the output was 
        // truncated.
        actual_length = vsnprintf(str, curr_size, fmt, ap);

        if (actual_length >= curr_size) {
            char *tmp;
            curr_size = actual_length + 1;
            tmp = shttp_realloc(str, curr_size);
            if (!tmp)
                goto realloc_failed;
            str = tmp;
        } else if (actual_length < 0) {
            char *tmp;
            curr_size *= 2;
            tmp = shttp_realloc(str, curr_size);
            if (!tmp)
                goto realloc_failed;
            str = tmp;
        } else {
            break;
        }
    }
    
    if (length)
        *length = actual_length;

    return str;

realloc_failed:
    shttp_free(str);
malloc_failed:
    return NULL;
}

char *
shttp_string_printf(const char *fmt, int *length, ...)
{
    assert(fmt != NULL);
    
    va_list ap;
    
    va_start(ap, length);
    char *str = shttp_string_vprintf(fmt, length, ap);
    va_end(ap);
    
    return str;
}
