/* blp-window.c
 *
 * Copyright (C) 2014 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "blp-canvas.h"
#include "blp-window.h"

struct _BlpWindowPrivate
{
   GtkNotebook  *notebook;
   GtkHeaderBar *header;
};

enum
{
   PROP_0,
   LAST_PROP
};

G_DEFINE_TYPE_WITH_PRIVATE (BlpWindow, blp_window, GTK_TYPE_WINDOW)

static GParamSpec *gParamSpecs [LAST_PROP];

GtkWidget *
blp_window_new (void)
{
   return g_object_new (BLP_TYPE_WINDOW, NULL);
}

void
blp_window_set_document (BlpWindow   *window,
                         BlpDocument *document)
{
   GtkWidget *canvas;

   g_return_if_fail (BLP_IS_WINDOW (window));
   g_return_if_fail (BLP_IS_DOCUMENT (document));

   canvas = gtk_bin_get_child (GTK_BIN (window));
   g_assert (BLP_IS_CANVAS (canvas));

   blp_canvas_set_document (BLP_CANVAS (canvas), document);
}

static void
blp_window_finalize (GObject *object)
{
   BlpWindowPrivate *priv;

   priv = BLP_WINDOW (object)->priv;

   G_OBJECT_CLASS (blp_window_parent_class)->finalize (object);
}

static void
blp_window_get_property (GObject    *object,
                         guint       prop_id,
                         GValue     *value,
                         GParamSpec *pspec)
{
   BlpWindow *window = BLP_WINDOW (object);

   switch (prop_id) {
   default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
}

static void
blp_window_set_property (GObject      *object,
                         guint         prop_id,
                         const GValue *value,
                         GParamSpec   *pspec)
{
   BlpWindow *window = BLP_WINDOW (object);

   switch (prop_id) {
   default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
}

static void
blp_window_class_init (BlpWindowClass *klass)
{
   GObjectClass *object_class;

   object_class = G_OBJECT_CLASS (klass);
   object_class->finalize = blp_window_finalize;
   object_class->get_property = blp_window_get_property;
   object_class->set_property = blp_window_set_property;
}

static void
blp_window_init (BlpWindow *window)
{
   window->priv = blp_window_get_instance_private (window);

   gtk_window_set_title (GTK_WINDOW (window), _("Blueprint"));
   gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);

   window->priv->header = g_object_new (GTK_TYPE_HEADER_BAR,
                                        "show-close-button", TRUE,
                                        "title", _("Blueprint"),
                                        "visible", TRUE,
                                        NULL);
   gtk_window_set_titlebar (GTK_WINDOW (window),
                            GTK_WIDGET (window->priv->header));

   /* XXX */ {
      GtkWidget *view;

      view = blp_canvas_new ();
      gtk_container_add (GTK_CONTAINER (window), view);
      gtk_widget_show (view);
   }
}
