/* main.c
 *
 * Copyright (C) 2014 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <clutter-gtk/clutter-gtk.h>
#include <gtk/gtk.h>
#include <stdlib.h>

#include "blp-document.h"
#include "blp-item.h"
#include "blp-pixbuf-item.h"
#include "blp-window.h"

int
main (int argc,
      char *argv[])
{
   BlpDocument *document;
   GtkWidget *window;
   BlpItem *item;
   BlpItem *root;
   GFile *file;

   if (CLUTTER_INIT_SUCCESS != gtk_clutter_init (&argc, &argv)) {
      g_printerr ("Failed to initialize clutter.\n");
      exit (1);
   }

   window = blp_window_new ();
   g_signal_connect (window, "delete-event", gtk_main_quit, NULL);
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);

   document = blp_document_new ();
   g_assert (BLP_IS_DOCUMENT (document));

   root = blp_document_get_root (document);
   g_assert (BLP_IS_ITEM (root));

   file = g_file_new_for_path ("lespaul.jpg");
   item = blp_pixbuf_item_new (file);
   blp_item_append_child (root, item);
   g_clear_object (&file);
   g_clear_object (&item);

   blp_window_set_document (BLP_WINDOW (window), document);

   g_clear_object (&document);

   gtk_window_present (GTK_WINDOW (window));
   gtk_main ();

   return 0;
}
