/* blp-util.c
 *
 * Copyright (C) 2014 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <cogl/cogl.h>

#include "blp-util.h"

ClutterContent *
blp_clutter_image_new_from_pixbuf (GdkPixbuf *pixbuf)
{
   ClutterContent *content;
   gboolean r;
   GError *error = NULL;

   g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);

   content = clutter_image_new ();

   r = clutter_image_set_data (CLUTTER_IMAGE (content),
                               gdk_pixbuf_get_pixels (pixbuf),
                               gdk_pixbuf_get_has_alpha (pixbuf) ?
                                  COGL_PIXEL_FORMAT_RGBA_8888 :
                                  COGL_PIXEL_FORMAT_RGB_888,
                               gdk_pixbuf_get_width (pixbuf),
                               gdk_pixbuf_get_height (pixbuf),
                               gdk_pixbuf_get_rowstride (pixbuf),
                               &error);

   if (!r) {
      g_warning ("%s", error->message);
      g_clear_error (&error);
      g_clear_object (&content);
   }

   return content;
}

ClutterContent *
blp_clutter_image_new_from_file (GFile *file)
{
   GFileInputStream *stream = NULL;
   ClutterContent *ret = NULL;
   GdkPixbuf *pixbuf = NULL;

   g_return_val_if_fail (G_IS_FILE (file), NULL);

   if ((stream = g_file_read (file, NULL, NULL))) {
      if ((pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (stream),
                                                NULL, NULL))) {
         ret = blp_clutter_image_new_from_pixbuf (pixbuf);
      }
   }

   g_clear_object (&stream);
   g_clear_object (&pixbuf);

   return ret;
}
