/* main.c
 *
 * Copyright (C) 2014 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <clutter-gtk/clutter-gtk.h>
#include <girepository.h>

#include "blp-canvas.h"
#include "blp-document.h"
#include "blp-item.h"
#include "blp-item-pixbuf.h"
#include "blp-item-plaque.h"

static ClutterActor *
build_item (GIBaseInfo *info)
{
   ClutterActor *plaque;
   ClutterActor *group;
   ClutterActor *text;
   const gchar *str;

   str = g_base_info_get_name (info);

   plaque = g_object_new (BLP_TYPE_ITEM_PLAQUE,
                          "x", 100.0f,
                          "y", 100.0f,
                          "width", 200.0f,
                          "height", 100.0f,
                          NULL);

   group = clutter_actor_new ();
   clutter_actor_add_child (plaque, group);

   text = g_object_new (CLUTTER_TYPE_TEXT,
                        "font-name", "Sans 12",
                        "text", str,
                        "x", 30.0f,
                        "y", 40.0f,
                        NULL);
   clutter_actor_add_child (group, text);

   return plaque;
}

gint
main (gint argc,
      gchar *argv[])
{
   GIRepository *repository;
   ClutterColor black;
   BlpDocument *document;
   GtkWidget *header;
   GtkWindow *window;
   GtkWidget *canvas;
   BlpItem *root;
   GError *error = NULL;
   GFile *background_file;

   if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS) {
      return 1;
   }

   repository = g_irepository_get_default ();
   g_irepository_require (repository, "Clutter", "1.0", 0, &error);
   g_assert_no_error (error);

   clutter_color_from_string (&black, "#000000");

   header = g_object_new (GTK_TYPE_HEADER_BAR,
                          "show-close-button", TRUE,
                          "title", "Blueprint",
                          "visible", TRUE,
                          NULL);

   window = g_object_new (GTK_TYPE_WINDOW,
                          "default-width", 800,
                          "default-height", 600,
                          "window-position", GTK_WIN_POS_CENTER,
                          NULL);
   gtk_window_set_titlebar (GTK_WINDOW (window), header);
   g_signal_connect (window, "delete-event", gtk_main_quit, NULL);

   document = blp_document_new ();
   root = blp_document_get_root (document);

   {
      ClutterActor *actor;
      GIBaseInfo *info;
      int len;
      int i;
      int j = 0;

      len = g_irepository_get_n_infos (repository, "Clutter");

      g_print ("len %d\n", len);

      for (i = 0; i < len; i++) {
         info = g_irepository_get_info (repository, "Clutter", i);
         if (g_base_info_get_type (info) == GI_INFO_TYPE_OBJECT) {
            actor = build_item (info);
            g_object_set (actor,
                          "x", 25.0 + 225.0 * (j % 6),
                          "y", (j / 6) * 125.0 + 25.0,
                          NULL);
            j++;
            printf ("%f %f\n", clutter_actor_get_x (actor), clutter_actor_get_y (actor));
            clutter_actor_add_child (CLUTTER_ACTOR (root), actor);
         }
      }
   }

   background_file = g_file_new_for_path ("../grid.png");

   canvas = g_object_new (BLP_TYPE_CANVAS,
                          "background-file", background_file,
                          "document", document,
                          "visible", TRUE,
                          NULL);
   gtk_container_add (GTK_CONTAINER (window), canvas);

   gtk_window_present (window);

   gtk_main ();

   return 0;
}
