/* mail-prefs.c
 *
 * Copyright (C) 2010 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "mail-log.h"
#include "mail-prefs.h"

static GOptionContext *context   = NULL;
static gchar          *log_file  = NULL;
static GOptionEntry    entries[] = {
	{ "log", 'l', 0, G_OPTION_ARG_FILENAME, &log_file, N_("A filename to log to") },
	{ NULL }
};

/**
 * mail_prefs_init:
 *
 * Initializes the preferences subsystem and parses command line
 * arguments.
 *
 * Returns: TRUE if successful; otherwise FALSE.
 * Side effects: None.
 */
gboolean
mail_prefs_init (gint *argc,
                 gchar ***argv)
{
	GError *error = NULL;

	context = g_option_context_new(_("- Simply Mail"));
	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
	g_option_context_add_group(context, gtk_get_option_group(TRUE));
	if (!g_option_context_parse(context, argc, argv, &error)) {
		g_printerr("%s\n", error->message);
		g_error_free(error);
		return FALSE;
	}

	if (log_file) {
		mail_log_add_filename(log_file);
	}

	return TRUE;
}

/**
 * mail_prefs_shutdown:
 *
 * Shuts down the preferences subsystem and frees resources.
 *
 * Returns: None.
 * Side effects: Resources are freed.
 */
void
mail_prefs_shutdown (void)
{
	g_option_context_free(context);
	context = NULL;
	g_free(log_file);
}
