/* mail-search-tool-item.c
 *
 * Copyright (C) 2010 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mail-search-tool-item.h"

G_DEFINE_TYPE(MailSearchToolItem, mail_search_tool_item, GTK_TYPE_TOOL_ITEM)

struct _MailSearchToolItemPrivate
{
	GtkWidget *entry;
};

GtkWidget*
mail_search_tool_item_get_entry (MailSearchToolItem *item)
{
	g_return_val_if_fail(MAIL_IS_SEARCH_TOOL_ITEM(item), NULL);
	return item->priv->entry;
}

static void
mail_search_tool_item_finalize (GObject *object)
{
	G_OBJECT_CLASS(mail_search_tool_item_parent_class)->finalize(object);
}

static void
mail_search_tool_item_class_init (MailSearchToolItemClass *klass)
{
	GObjectClass *object_class;

	object_class = G_OBJECT_CLASS(klass);
	object_class->finalize = mail_search_tool_item_finalize;
	g_type_class_add_private(object_class, sizeof(MailSearchToolItemPrivate));
}

static void
mail_search_tool_item_init (MailSearchToolItem *item)
{
	MailSearchToolItemPrivate *priv;
	GtkWidget *align;

	priv = G_TYPE_INSTANCE_GET_PRIVATE(item, MAIL_TYPE_SEARCH_TOOL_ITEM,
	                                   MailSearchToolItemPrivate);
	item->priv = priv;

	align = g_object_new(GTK_TYPE_ALIGNMENT,
	                     "visible", TRUE,
	                     "xalign", 1.0f,
	                     "xscale", 1.0f,
	                     "yalign", 0.5f,
	                     "yscale", 0.0f,
	                     NULL);
	gtk_container_add(GTK_CONTAINER(item), align);

	priv->entry = g_object_new(GTK_TYPE_ENTRY,
	                           "primary-icon-name", GTK_STOCK_FIND,
	                           "visible", TRUE,
	                           NULL);
	gtk_container_add(GTK_CONTAINER(align), priv->entry);
}
