/* mail-task.h
 *
 * Copyright (C) 2010 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAIL_TASK_H
#define MAIL_TASK_H

#include <glib-object.h>

G_BEGIN_DECLS

#define MAIL_TYPE_TASK            (mail_task_get_type())
#define MAIL_TYPE_TASK_CATEGORY   (mail_task_category_get_type())
#define MAIL_TASK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_TASK, MailTask))
#define MAIL_TASK_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_TASK, MailTask const))
#define MAIL_TASK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  MAIL_TYPE_TASK, MailTaskClass))
#define MAIL_IS_TASK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_TASK))
#define MAIL_IS_TASK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  MAIL_TYPE_TASK))
#define MAIL_TASK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  MAIL_TYPE_TASK, MailTaskClass))

typedef struct _MailTask         MailTask;
typedef struct _MailTaskClass    MailTaskClass;
typedef struct _MailTaskPrivate  MailTaskPrivate;
typedef enum   _MailTaskCategory MailTaskCategory;

struct _MailTask
{
	GInitiallyUnowned parent;

	/*< private >*/
	MailTaskPrivate *priv;
};

struct _MailTaskClass
{
	GInitiallyUnownedClass parent_class;

	void (*run) (MailTask *task);
};

enum _MailTaskCategory
{
	MAIL_TASK_SYNCING,
	MAIL_TASK_INDEXING,
	MAIL_TASK_LAST
};

void     mail_task_finish            (MailTask         *task);
void     mail_task_finish_take_error (MailTask         *task,
                                      GError           *error);
void     mail_task_finish_with_error (MailTask         *task,
                                      const GError     *error);
GError*  mail_task_get_error         (MailTask         *task);
gboolean mail_task_get_finished      (MailTask         *task);
GType    mail_task_get_type          (void) G_GNUC_CONST;
GType    mail_task_category_get_type (void) G_GNUC_CONST;
void     mail_task_set_category      (MailTask         *task,
                                      MailTaskCategory  category);

G_END_DECLS

#endif /* MAIL_TASK_H */
