/* main.c
 *
 * Copyright (C) 2010 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "mail-private.h"
#include "mail-runtime.h"
#include "mail-window.h"

gint
main (gint argc,
      gchar *argv[])
{
	GtkWindow *window;
	gint code;

	/*
	 * Initialize i18n first thing so it can be used early.
	 */
	textdomain(GETTEXT_PACKAGE);
	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");

	/*
	 * Initialize application and libraries.
	 */
	g_set_application_name(_("Simply Mail"));
	g_thread_init(NULL);

	/*
	 * Initialize the runtime services.
	 */
	if (!mail_runtime_init(&argc, &argv)) {
		return EXIT_FAILURE;
	}

	/*
	 * Show the mail window.
	 */
	window = g_object_new(MAIL_TYPE_WINDOW, NULL);
	gtk_window_present(window);

	/*
	 * Run the main loop. Exit if/when there is an error code.
	 */
	if ((code = mail_runtime_run())) {
		return code;
	}

	/*
	 * Gracefully shutdown the runtime services.
	 */
	mail_runtime_shutdown();

	return EXIT_SUCCESS;
}
