/* news-animation.h
 *
 * Copyright (C) 2010 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_ANIMATION_H
#define NEWS_ANIMATION_H

#include <glib-object.h>

G_BEGIN_DECLS

#define NEWS_TYPE_ANIMATION            (news_animation_get_type())
#define NEWS_TYPE_ANIMATION_MODE       (news_animation_mode_get_type())
#define NEWS_ANIMATION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_ANIMATION, NewsAnimation))
#define NEWS_ANIMATION_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_ANIMATION, NewsAnimation const))
#define NEWS_ANIMATION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_ANIMATION, NewsAnimationClass))
#define NEWS_IS_ANIMATION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_ANIMATION))
#define NEWS_IS_ANIMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_ANIMATION))
#define NEWS_ANIMATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_ANIMATION, NewsAnimationClass))

typedef struct _NewsAnimation        NewsAnimation;
typedef struct _NewsAnimationClass   NewsAnimationClass;
typedef struct _NewsAnimationPrivate NewsAnimationPrivate;
typedef enum   _NewsAnimationMode    NewsAnimationMode;

enum _NewsAnimationMode
{
   NEWS_ANIMATION_LINEAR,
   NEWS_ANIMATION_EASE_IN_QUAD,
   NEWS_ANIMATION_EASE_OUT_QUAD,
   NEWS_ANIMATION_EASE_IN_OUT_QUAD,
   NEWS_ANIMATION_EASE_IN_CUBIC,

   NEWS_ANIMATION_LAST
};

struct _NewsAnimation
{
   GInitiallyUnowned parent;

   /*< private >*/
   NewsAnimationPrivate *priv;
};

struct _NewsAnimationClass
{
   GInitiallyUnownedClass parent_class;
};

GType news_animation_get_type         (void) G_GNUC_CONST;
GType news_animation_mode_get_type    (void) G_GNUC_CONST;
void  news_animation_start            (NewsAnimation      *animation);
void  news_animation_stop             (NewsAnimation      *animation);
void  news_animation_add_property     (NewsAnimation      *animation,
                                     GParamSpec       *pspec,
                                     const GValue     *value);
NewsAnimation* news_object_animate      (gpointer          object,
                                     NewsAnimationMode   mode,
                                     guint             duration_msec,
                                     const gchar      *first_property,
                                     ...) G_GNUC_NULL_TERMINATED;
NewsAnimation* news_object_animate_full (gpointer          object,
                                     NewsAnimationMode   mode,
                                     guint             duration_msec,
                                     guint             frame_rate,
                                     GDestroyNotify    notify,
                                     gpointer          notify_data,
                                     const gchar      *first_property,
                                     ...) G_GNUC_NULL_TERMINATED;

G_END_DECLS

#endif /* NEWS_ANIMATION_H */
