/* news-category.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "news-category.h"
#include "news-debug.h"

G_DEFINE_TYPE(NewsCategory, news_category, GOM_TYPE_RESOURCE)

/**
 * news_category_class_init:
 * @klass: (in): A #NewsCategoryClass.
 *
 * Initializes the #NewsCategoryClass and prepares the vtable.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_category_class_init (NewsCategoryClass *klass)
{
   GObjectClass *object_class;
   GomResourceClass *resource_class;

   ENTRY;

   object_class = G_OBJECT_CLASS(klass);
   object_class->get_property = gom_resource_get_property;
   object_class->set_property = gom_resource_set_property;

   /*
    * TODO: We need unique-groups in gom. Something like:
    *       "unique-group", "blah",
    */

   resource_class = GOM_RESOURCE_CLASS(klass);
   gom_resource_class_table(resource_class, "categories");
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_int64("id",
                                                          _("Id"),
                                                          _("The identifier"),
                                                          0,
                                                          G_MAXINT64,
                                                          0,
                                                          G_PARAM_READWRITE),
                                       "key", TRUE,
                                       "serial", TRUE,
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("name",
                                                           _("Name"),
                                                           _("The category name"),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("label",
                                                           _("Label"),
                                                           _("The category label"),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("schema",
                                                           _("Schema"),
                                                           _("The category schema"),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);

   EXIT;
}

/**
 * news_category_init:
 * @: (in): A #NewsCategory.
 *
 * Initializes the newly created #NewsCategory instance.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_category_init (NewsCategory *category)
{
   ENTRY;
   EXIT;
}
