/* news-client.h
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_CLIENT_H
#define NEWS_CLIENT_H

#include <glib-object.h>

#include "news-source.h"

G_BEGIN_DECLS

#define NEWS_CLIENT_DEFAULT         (news_client_get_default())
#define NEWS_TYPE_CLIENT            (news_client_get_type())
#define NEWS_CLIENT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_CLIENT, NewsClient))
#define NEWS_CLIENT_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_CLIENT, NewsClient const))
#define NEWS_CLIENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_CLIENT, NewsClientClass))
#define NEWS_IS_CLIENT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_CLIENT))
#define NEWS_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_CLIENT))
#define NEWS_CLIENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_CLIENT, NewsClientClass))
#define NEWS_CLIENT_ERROR           (news_client_error_quark())

typedef struct _NewsClient        NewsClient;
typedef struct _NewsClientClass   NewsClientClass;
typedef struct _NewsClientPrivate NewsClientPrivate;
typedef enum   _NewsClientError   NewsClientError;

enum _NewsClientError
{
   NEWS_CLIENT_ERROR_FILE_NOT_FOUND = 1,
   NEWS_CLIENT_ERROR_BAD_FORMAT,
};

struct _NewsClient
{
   GObject parent;

   /*< private >*/
   NewsClientPrivate *priv;
};

struct _NewsClientClass
{
   GObjectClass parent_class;
};

void        news_client_add_source     (NewsClient   *client,
                                        NewsSource   *source);
void        news_client_cancel_tasks   (NewsClient   *client);
GQuark      news_client_error_quark    (void) G_GNUC_CONST;
NewsClient *news_client_get_default    (void);
GList      *news_client_get_sources    (NewsClient   *client);
GType       news_client_get_type       (void) G_GNUC_CONST;
gboolean    news_client_load_from_file (NewsClient   *client,
                                        const gchar  *filename,
                                        GError      **error);
gboolean    news_client_save_to_file   (NewsClient   *client,
                                        const gchar  *filename,
                                        GError      **error);
void        news_client_refresh_all    (NewsClient   *client);

G_END_DECLS

#endif /* NEWS_CLIENT_H */
