/* news-content-kind.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news-content-kind.h"

GType
news_content_kind_get_type (void)
{
   static GType type_id = 0;
   static gsize initialized = FALSE;
   static const GEnumValue values[] = {
      { NEWS_CONTENT_TEXT, "NEWS_CONTENT_TEXT", "TEXT" },
      { NEWS_CONTENT_HTML, "NEWS_CONTENT_HTML", "HTML" },
      { NEWS_CONTENT_XHTML, "NEWS_CONTENT_XHTML", "XHTML" },
      { 0 }
   };

   if (g_once_init_enter(&initialized)) {
      type_id = g_enum_register_static("NewsContentKind", values);
      g_once_init_leave(&initialized, TRUE);
   }

   return type_id;
}

gboolean
news_content_kind_parse (NewsContentKind *kind,
                         const gchar *text)
{
   g_return_val_if_fail(kind != NULL, FALSE);

   if (!text || !text[0]) {
      return FALSE;
   } else if (!g_strcmp0(text, "text")) {
      *kind = NEWS_CONTENT_TEXT;
   } else if (!g_strcmp0(text, "html")) {
      *kind = NEWS_CONTENT_HTML;
   } else if (!g_strcmp0(text, "xhtml")) {
      *kind = NEWS_CONTENT_XHTML;
   } else {
      return FALSE;
   }

   return TRUE;
}
