/* news-format.h
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_FORMAT_H
#define NEWS_FORMAT_H

#include <gio/gio.h>
#include <glib-object.h>
#include <gom/gom.h>

#include "news-feed.h"
#include "news-source.h"

G_BEGIN_DECLS

#define NEWS_TYPE_FORMAT            (news_format_get_type())
#define NEWS_FORMAT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_FORMAT, NewsFormat))
#define NEWS_FORMAT_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_FORMAT, NewsFormat const))
#define NEWS_FORMAT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_FORMAT, NewsFormatClass))
#define NEWS_IS_FORMAT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_FORMAT))
#define NEWS_IS_FORMAT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_FORMAT))
#define NEWS_FORMAT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_FORMAT, NewsFormatClass))
#define NEWS_FORMAT_ERROR           (news_format_error_quark())

typedef struct _NewsFormat        NewsFormat;
typedef struct _NewsFormatClass   NewsFormatClass;
typedef struct _NewsFormatPrivate NewsFormatPrivate;
typedef enum   _NewsFormatError   NewsFormatError;

enum _NewsFormatError
{
   NEWS_FORMAT_ERROR_UNKNOWN = 1,
};

struct _NewsFormat
{
   GObject parent;

   /*< private >*/
   NewsFormatPrivate *priv;
};

struct _NewsFormatClass
{
   GObjectClass parent_class;

   NewsFeed *(*read)        (NewsFormat           *format,
                             GInputStream         *stream,
                             GError              **error);
   void      (*read_async)  (NewsFormat           *format,
                             GInputStream         *stream,
                             GCancellable         *cancellable,
                             GAsyncReadyCallback   callback,
                             gpointer              user_data);
   NewsFeed *(*read_finish) (NewsFormat           *format,
                             GAsyncResult         *result,
                             GError              **error);
};

GQuark      news_format_error_quark (void) G_GNUC_CONST;
GomAdapter *news_format_get_adapter (NewsFormat           *format);
NewsSource *news_format_get_source  (NewsFormat           *format);
GType       news_format_get_type    (void) G_GNUC_CONST;
void        news_format_read_async  (NewsFormat           *format,
                                     GInputStream         *stream,
                                     GCancellable         *cancellable,
                                     GAsyncReadyCallback   callback,
                                     gpointer              user_data);
NewsFeed   *news_format_read_finish (NewsFormat           *format,
                                     GAsyncResult         *result,
                                     GError              **error);

G_END_DECLS

#endif /* NEWS_FORMAT_H */
