/* news-link.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "news-debug.h"
#include "news-link.h"

G_DEFINE_TYPE(NewsLink, news_link, GOM_TYPE_RESOURCE)

/**
 * news_link_class_init:
 * @klass: (in): A #NewsLinkClass.
 *
 * Initializes the #NewsLinkClass and prepares the vtable.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_link_class_init (NewsLinkClass *klass)
{
   GObjectClass *object_class;
   GomResourceClass *resource_class;

   ENTRY;

   object_class = G_OBJECT_CLASS(klass);
   object_class->get_property = gom_resource_get_property;
   object_class->set_property = gom_resource_set_property;

   resource_class = GOM_RESOURCE_CLASS(klass);
   gom_resource_class_table(resource_class, "links");
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_int64("id",
                            _("Id"),
                        _("The link identifier"),
                        0,
                        G_MAXINT64,
                        0,
                        G_PARAM_READWRITE),
         "key", TRUE,
         "serial", TRUE,
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_string("base-uri",
                             _("Base URI"),
                             _("The base uri of the link"),
                             NULL,
                             G_PARAM_READWRITE),
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_string("href",
                             _("Href"),
                        _("The link href"),
                        NULL,
                        G_PARAM_READWRITE),
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_int64("length",
                        _("Length"),
                        _("The length of the content"),
                        G_MININT64,
                        G_MAXINT64,
                        0UL,
                        G_PARAM_READWRITE),
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_string("media-type",
                        _("Media Type"),
                        _("The link media type"),
                        NULL,
                        G_PARAM_READWRITE),
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_string("relation",
                        _("Relation"),
                        _("The link relation"),
                        NULL,
                        G_PARAM_READWRITE),
         NULL);
   gom_resource_class_install_property(
         resource_class,
         g_param_spec_string("title",
                        _("Title"),
                        _("The link title"),
                        NULL,
                        G_PARAM_READWRITE),
         NULL);

   EXIT;
}

/**
 * news_link_init:
 * @: (in): A #NewsLink.
 *
 * Initializes the newly created #NewsLink instance.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_link_init (NewsLink *link_)
{
   ENTRY;
   EXIT;
}
