/* news-parser.h
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_PARSER_H
#define NEWS_PARSER_H

#include <gio/gio.h>
#include <glib-object.h>
#include <gom/gom.h>

#include "news-feed.h"
#include "news-source.h"

G_BEGIN_DECLS

#define NEWS_TYPE_PARSER            (news_parser_get_type())
#define NEWS_PARSER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_PARSER, NewsParser))
#define NEWS_PARSER_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_PARSER, NewsParser const))
#define NEWS_PARSER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_PARSER, NewsParserClass))
#define NEWS_IS_PARSER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_PARSER))
#define NEWS_IS_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_PARSER))
#define NEWS_PARSER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_PARSER, NewsParserClass))
#define NEWS_PARSER_ERROR           (news_parser_error_quark())

typedef struct _NewsParser        NewsParser;
typedef struct _NewsParserClass   NewsParserClass;
typedef struct _NewsParserPrivate NewsParserPrivate;
typedef enum   _NewsParserError   NewsParserError;

typedef GType (*NewsFormatDiscoverer) (const guint8 *buffer,
                                       gsize         length);

enum _NewsParserError
{
   NEWS_PARSER_ERROR_NO_ADAPTER = 1,
   NEWS_PARSER_ERROR_NO_DATA,
   NEWS_PARSER_ERROR_NO_FORMAT,
};

struct _NewsParser
{
   GObject parent;

   /*< private >*/
   NewsParserPrivate *priv;
};

struct _NewsParserClass
{
   GObjectClass parent_class;
};

GQuark      news_parser_error_quark     (void) G_GNUC_CONST;
GomAdapter *news_parser_get_adapter     (NewsParser            *parser);
NewsSource *news_parser_get_source      (NewsParser            *parser);
GType       news_parser_get_type        (void) G_GNUC_CONST;
void        news_parser_parse_async     (NewsParser            *parser,
                                         GInputStream          *stream,
                                         GCancellable          *cancellable,
                                         GAsyncReadyCallback    callback,
                                         gpointer               user_data);
NewsFeed   *news_parser_parse_finish    (NewsParser            *parser,
                                         GAsyncResult          *result,
                                         GError               **error);
void        news_parser_register_format (NewsFormatDiscoverer   discoverer,
                                         gint                   priority);

G_END_DECLS

#endif /* NEWS_PARSER_H */
