/* news-person.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "news-debug.h"
#include "news-person.h"

G_DEFINE_TYPE(NewsPerson, news_person, GOM_TYPE_RESOURCE)

/**
 * news_person_class_init:
 * @klass: (in): A #NewsPersonClass.
 *
 * Initializes the #NewsPersonClass and prepares the vtable.
 *
 * Returns: None.
 */
static void
news_person_class_init (NewsPersonClass *klass)
{
   GObjectClass *object_class;
   GomResourceClass *resource_class;

   object_class = G_OBJECT_CLASS(klass);
   object_class->get_property = gom_resource_get_property;
   object_class->set_property = gom_resource_set_property;

   resource_class = GOM_RESOURCE_CLASS(klass);
   gom_resource_class_table(resource_class, "persons");
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_int64("id",
                                                          _("Id"),
                                                          _("The persons identifier."),
                                                          0,
                                                          G_MAXINT64,
                                                          0,
                                                          G_PARAM_READWRITE),
                                       "key", TRUE,
                                       "serial", TRUE,
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("email",
                                                           _("Email"),
                                                           _("Persons email address."),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("name",
                                                           _("Name"),
                                                           _("Persons name."),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);
   gom_resource_class_install_property(resource_class,
                                       g_param_spec_string("uri",
                                                           _("Uri"),
                                                           _("Persons website or URI."),
                                                           NULL,
                                                           G_PARAM_READWRITE),
                                       NULL);
}

/**
 * news_person_init:
 * @: (in): A #NewsPerson.
 *
 * Initializes the newly created #NewsPerson instance.
 *
 * Returns: None.
 */
static void
news_person_init (NewsPerson *person)
{
   ENTRY;
   EXIT;
}
