/* news-section.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "news-section.h"

G_DEFINE_TYPE(NewsSection, news_section, GOM_TYPE_RESOURCE)

/**
 * news_section_class_init:
 * @klass: (in): A #NewsSectionClass.
 *
 * Initializes the #NewsSectionClass and prepares the vtable.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_section_class_init (NewsSectionClass *klass)
{
   GomResourceClass *resource_class;

   resource_class = GOM_RESOURCE_CLASS(klass);
   gom_resource_class_table(resource_class, "sections");
   gom_resource_class_install_property(resource_class,
                              g_param_spec_int64("id",
                                             _("Id"),
                                             _("Section identifier"),
                                             0,
                                             G_MAXINT64,
                                             0,
                                             G_PARAM_READWRITE),
                              "key", TRUE,
                              "serial", TRUE,
                              NULL);
   gom_resource_class_install_property(resource_class,
                              g_param_spec_string("label",
                                             _("Label"),
                                             _("The sections label"),
                                             NULL,
                                             G_PARAM_READWRITE),
                              NULL);
}

/**
 * news_section_init:
 * @: (in): A #NewsSection.
 *
 * Initializes the newly created #NewsSection instance.
 *
 * Returns: None.
 * Side effects: None.
 */
static void
news_section_init (NewsSection *section)
{
   section->priv =
      G_TYPE_INSTANCE_GET_PRIVATE(section,
                                  NEWS_TYPE_SECTION,
                                  NewsSectionPrivate);
}
