/* news-source.h
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_SOURCE_H
#define NEWS_SOURCE_H

#include <glib-object.h>
#include <gio/gio.h>

G_BEGIN_DECLS

#define NEWS_TYPE_SOURCE            (news_source_get_type())
#define NEWS_SOURCE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_SOURCE, NewsSource))
#define NEWS_SOURCE_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_SOURCE, NewsSource const))
#define NEWS_SOURCE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_SOURCE, NewsSourceClass))
#define NEWS_IS_SOURCE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_SOURCE))
#define NEWS_IS_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_SOURCE))
#define NEWS_SOURCE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_SOURCE, NewsSourceClass))

typedef struct _NewsSource        NewsSource;
typedef struct _NewsSourceClass   NewsSourceClass;
typedef struct _NewsSourcePrivate NewsSourcePrivate;

struct _NewsSource
{
   GInitiallyUnowned parent;

   /*< private >*/
   NewsSourcePrivate *priv;
};

struct _NewsSourceClass
{
   GInitiallyUnownedClass parent_class;

   void          (*fetch_async)  (NewsSource           *source,
                                  GCancellable         *cancellable,
                                  GAsyncReadyCallback   callback,
                                  gpointer              user_data);
   GInputStream *(*fetch_finish) (NewsSource           *source,
                                  GAsyncResult         *result,
                                  GError              **error);
   void          (*refreshed)    (NewsSource           *source);
};

void          news_source_add_error      (NewsSource           *source,
                                          const GError         *error);
void          news_source_take_error     (NewsSource           *source,
                                          GError               *error);
void          news_source_clear_errors   (NewsSource           *source);
GList        *news_source_get_errors     (NewsSource           *source);
const gchar  *news_source_get_id         (NewsSource           *source);
GType         news_source_get_type       (void) G_GNUC_CONST;
void          news_source_emit_refreshed (NewsSource           *source);
void          news_source_fetch_async    (NewsSource           *source,
                                          GCancellable         *cancellable,
                                          GAsyncReadyCallback   callback,
                                          gpointer              user_data);
GInputStream *news_source_fetch_finish   (NewsSource           *source,
                                          GAsyncResult         *result,
                                          GError              **error);
void          news_source_refresh        (NewsSource           *source);

G_END_DECLS

#endif /* NEWS_SOURCE_H */
