/* news-status.c
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include "news-status.h"

GType
news_status_get_type (void)
{
   static gsize initialized = FALSE;
   static GType type_id = 0;

   if (g_once_init_enter(&initialized)) {
      type_id = g_boxed_type_register_static(
            "NewsStatus",
            (GBoxedCopyFunc)news_status_copy,
            (GBoxedFreeFunc)news_status_free);
      g_once_init_leave(&initialized, TRUE);
   }

   return type_id;
}

NewsStatus *
news_status_copy (const NewsStatus *status)
{
   NewsStatus *copy;

   copy = g_new0(NewsStatus, 1);
   copy->total = status->total;
   copy->completed = status->completed;
   copy->fraction = status->fraction;
   return copy;
}

void
news_status_free (NewsStatus *status)
{
   g_free(status);
}

void
news_status_add (NewsStatus *status,
                 gulong      count)
{
   g_return_if_fail(status != NULL);

   if (status->total == status->completed) {
      status->total = 0;
      status->completed = 0;
   }

   status->total += count;

   if (status->total) {
      status->fraction = CLAMP(status->completed / (gdouble)status->total,
                               0.0, 1.0);
   } else {
      status->fraction = 1.0;
   }
}

void
news_status_complete (NewsStatus *status,
                      gulong      count)
{
   g_return_if_fail(status != NULL);

   status->completed += count;
   if (status->completed > status->total) {
      status->completed = status->total;
   }

   if (status->total) {
      status->fraction = CLAMP(status->completed / (gdouble)status->total,
                               0.0, 1.0);
   } else {
      status->fraction = 1.0;
   }
}
