/* news-sync-task.h
 *
 * Copyright (C) 2011 Christian Hergert <chris@dronelabs.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NEWS_SYNC_TASK_H
#define NEWS_SYNC_TASK_H

#include <glib-object.h>
#include <gio/gio.h>

G_BEGIN_DECLS

#define NEWS_TYPE_SYNC_TASK            (news_sync_task_get_type())
#define NEWS_SYNC_TASK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_SYNC_TASK, NewsSyncTask))
#define NEWS_SYNC_TASK_CONST(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEWS_TYPE_SYNC_TASK, NewsSyncTask const))
#define NEWS_SYNC_TASK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NEWS_TYPE_SYNC_TASK, NewsSyncTaskClass))
#define NEWS_IS_SYNC_TASK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEWS_TYPE_SYNC_TASK))
#define NEWS_IS_SYNC_TASK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  NEWS_TYPE_SYNC_TASK))
#define NEWS_SYNC_TASK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  NEWS_TYPE_SYNC_TASK, NewsSyncTaskClass))
#define NEWS_SYNC_TASK_ERROR           (news_sync_task_error_quark())

typedef struct _NewsSyncTask        NewsSyncTask;
typedef struct _NewsSyncTaskClass   NewsSyncTaskClass;
typedef struct _NewsSyncTaskPrivate NewsSyncTaskPrivate;
typedef enum   _NewsSyncTaskError   NewsSyncTaskError;

enum _NewsSyncTaskError
{
   NEWS_SYNC_TASK_ERROR_INVALID = 1,
   NEWS_SYNC_TASK_ERROR_CANCELLED,
};

struct _NewsSyncTask
{
   GObject parent;

   /*< private >*/
   NewsSyncTaskPrivate *priv;
};

struct _NewsSyncTaskClass
{
   GObjectClass parent_class;
};

GQuark    news_sync_task_error_quark (void) G_GNUC_CONST;
GType     news_sync_task_get_type    (void) G_GNUC_CONST;
void      news_sync_task_cancel      (NewsSyncTask         *task);
void      news_sync_task_run_async   (NewsSyncTask         *task,
                                      GCancellable         *cancellable,
                                      GAsyncReadyCallback   callback,
                                      gpointer              user_data);
gboolean  news_sync_task_run_finish  (NewsSyncTask         *task,
                                      GAsyncResult         *result,
                                      GError              **error);

G_END_DECLS

#endif /* NEWS_SYNC_TASK_H */
