#include "btree.h"
#include <stdint.h>
#include <stdio.h>

/* Dictionary-like lookup table, for comparing ASCII characters. This was
   generated by tools/str_table.py. Refer to str_table.py to make changes
   to this. */
static const char CE_BTREE_STRING_TABLE[] = {95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 2, 105, 106, 1, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 3, 32, 7, 18, 19, 20, 22, 6, 11, 12, 23, 24, 30, 5, 0, 9, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 125, 126, 28, 25, 29, 31, 17, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 13, 8, 14, 21, 4, 26, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 15, 10, 16, 27, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, };

/**
* Included string compare function. This uses CE_BTREE_STRING_TABLE as
* a lookup table, to do fast comparisons. The ascii characters being
* compared are used as keys, and the values are the ordering weight. Eg:
*   if (lookup['a'] > lookup['b'])
*       return CE_BTREE_COMPARE_GT;
*
* @param    void *A, pointer to string
* @param    void *B, pointer to string
* @return   ceBtree_Compare
**/
ceBtree_Compare ceBtreeStrcmp(void *A, void *B)
{
    int i = 0,
        which_is_bigger = 0;
    
    char *lookup = (char *)&CE_BTREE_STRING_TABLE;
    char *a = (char *)A;
    char *b = (char *)B;
    
    while (1) {
        // Check for NULL byte
        if (a[i] == '\0') {
            // Check if b is longer than a
            if (b[i] != '\0') {
                which_is_bigger = 1;
            }

            break;
        }
        
        // Check for NULL byte
        if (b[i] == '\0') {
            // Check if a is longer than b
            if (a[i] != '\0') {
                which_is_bigger = 2;
            }
            
            break;
        }
        
        // Check if a[i] > b[i], using lookup table
        if (lookup[(int)a[i]] > lookup[(int)b[i]]) {
            return CE_BTREE_COMPARE_GT;
        }
        
        // Check if a[i] < b[i], using lookup table
        if (lookup[(int)a[i]] < lookup[(int)b[i]]) {
            return CE_BTREE_COMPARE_LT;
        }
        
        ++i;
    }
    
    /* Check length of strings. If they're different, then they
       obviously aren't equal */
    if (which_is_bigger == 1) {
        return CE_BTREE_COMPARE_LT;
    }
    
    if (which_is_bigger == 2) {
        return CE_BTREE_COMPARE_GT;
    }
    
    return CE_BTREE_COMPARE_EQ;
}

/**
* Included int compare function. Uses whatever the compiler's default
* for int is for. Only useful for general purpose cases.
*
* You'll likely want to replace this with your own, especially if you
* need to compare unsigned 32-bit or 64-bit integers.
*
* @param    void *A, pointer to int
* @param    void *B, pointer to int
* @return   ceBtree_Compare
**/
ceBtree_Compare ceBtreeIntcmp(void *A, void *B)
{
    int a = *((int *)A),
        b = *((int *)B);
    
    if (a > b) {
        return CE_BTREE_COMPARE_GT;
    }
    
    if (b > a) {
        return CE_BTREE_COMPARE_LT;
    }
    
    if (a == b) {
        return CE_BTREE_COMPARE_EQ;
    }
    
    return CE_BTREE_COMPARE_ERR;
}
