#define _CRT_SECURE_NO_WARNINGS
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <io.h>

extern "C" {
#include "lua/src/lua.h"
#include "lua/src/lualib.h"
#include "lua/src/lauxlib.h"
}
#include "res.h"
#include "gui.h"
#include "reminder.h"
#include "crontask.h"

#pragma comment( lib, "lua5.1.lib" )
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' " "version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")

#define WM_SHELLNOTIFY          (WM_USER+1)

lua_State *L;
gui_struct *gui = 0;

char * SHELL;

/*extern int gui_printf( const char *format, va_list arglist )
{
	char buff[4096];	
	int result = vsprintf(buff, format, arglist);
	guiAddText(buff);
	return result;
}*/

void error (lua_State *L, const char *fmt, ...) {
	va_list argp;
	va_start(argp, fmt);
	
	//if (gui && gui->hwndConsole) gui_printf(fmt, argp);
	//else {
		FILE *f = fopen("error.log","a");
		vfprintf(f, fmt, argp);
		fclose(f);
	//}
	va_end(argp);
	//lua_close(L);
	//exit(EXIT_FAILURE);
}



int LoadScript(lua_State *L, const char* filename){
	if (luaL_loadfile(L, filename) || lua_pcall(L, 0, 0, 0))
        error(L, "cannot run configuration file: %s\r\n", lua_tostring(L, -1) );
	return 1;
}


void ShellCmd(char *cmd) {
	char cmdl[300];
	sprintf(cmdl,"%s /c %s",SHELL,cmd);
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
	char *env = 0;
	char *cwd = 0;

	int a = CreateProcess(0, cmdl, 0,0, true, CREATE_NEW_CONSOLE, env, cwd, &si, &pi);
}

static int l_Reminder(lua_State *L) {
	char * time = (char *)luaL_checkstring(L, 1);
	char * text = (char *)luaL_checkstring(L, 2);
	reminder *r = NewReminder();
	CreateReminder(0,r,time,text);
	if (IsMissedReminder(r))
		PopDialog(r);
	return 0;
}

static int l_Task(lua_State *L) {
	char * cronstring = (char *)luaL_checkstring(L, 1);

	char *shellcmd = 0;
	int func_ref = 0;
	if (lua_isstring(L, 2))
		shellcmd = (char *)lua_tostring(L, 2);
	else if (lua_isfunction(L, 2)) {
		luaL_checktype(L, 2, LUA_TFUNCTION);
		func_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}
	else error(L, "2nd arg: string or function expected \r\n" );

	CreateCrontask(cronstring, shellcmd, func_ref);	
	
	return 0;
}
int InitLua() {
	L = luaL_newstate(); 
	luaL_openlibs(L);   //import std lua libs
	lua_register(L, "Task", l_Task);
	lua_register(L, "Reminder", l_Reminder);
	return 1;
}


void FireTask(crontask *task) {
	if (task->func_ref) {
		lua_rawgeti(L, LUA_REGISTRYINDEX, task->func_ref);
		if (lua_pcall(L, 0, 0, 0) != 0)
			error(L, "error running function: %s", lua_tostring(L, -1));
	} else ShellCmd(task->shellcmd);
}

void __stdcall TimerFunc( HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime){
	time_t rawtime;
	struct tm * t; //timeinfo

	time ( &rawtime );
	t = localtime ( &rawtime );

	crontask *task = head;
	while (task) {
		if (task->min[t->tm_min] && task->hour[t->tm_hour] && task->mday[t->tm_mday] && task->mon[t->tm_mon] && task->wday[t->tm_wday] )
			FireTask(task);

		task = task->next;
	}

	reminder *r = rhead;
	while(r) {
		if (r->min == t->tm_min && r->hour == t->tm_hour && r->mday == t->tm_mday && r->mon == t->tm_mon ) {
			PopDialog(r);
		}
		r = r->next;
	}
}










int OnCreate( HWND hwnd)
{
	gui->hwnd = hwnd;

	if (gui->createtray) {
		NOTIFYICONDATA dta;

		gui->hmenuTray = CreatePopupMenu();
		AppendMenu(gui->hmenuTray, MF_STRING, ID_NEWREMINDER, "&New Reminder");
		AppendMenu(gui->hmenuTray, MF_STRING, ID_DISPLAYREMINDERS, "&Display Reminders");
		AppendMenu(gui->hmenuTray, MF_STRING, ID_EXIT, "E&xit");
		//AppendMenu(gui->hmenuTray, MF_SEPARATOR, 0,0);

		dta.cbSize = sizeof(NOTIFYICONDATA);
		dta.hWnd = hwnd;
		dta.uID = ID_TRAY;
		dta.uFlags = NIF_ICON|NIF_MESSAGE|NIF_TIP;
		dta.uCallbackMessage = WM_SHELLNOTIFY;
		dta.hIcon = LoadIcon(gui->hInstance, MAKEINTRESOURCE(ID_ICON1));
		strcpy(dta.szTip, "Dirty Deeds");
		Shell_NotifyIcon(NIM_ADD, &dta);
	}

	return 1;
}

void KillTrayIcon(HWND hwnd)
{
    NOTIFYICONDATA dta;
     
    dta.cbSize = sizeof(NOTIFYICONDATA);
    dta.hWnd = hwnd;
    dta.uID = ID_TRAY;
    dta.uFlags = 0;
    Shell_NotifyIcon(NIM_DELETE, &dta);
}

static void OnCommand(HWND hwnd, int id)
					  //, HWND hwndCtl, UINT codeNotify)
{
	if( id == ID_EXIT ) {
		DestroyWindow(hwnd);
	}
	else if (id == ID_SHOWCONSOLE )
		ShowWindow(gui->hwnd, SW_SHOW);
	else if (id == ID_HIDECONSOLE )
		ShowWindow(gui->hwnd, SW_HIDE);
	else if (id == ID_DISPLAYREMINDERS )
		RemindersMessageBox();
	else if (id == ID_NEWREMINDER ) {
		reminder *r = NewReminder();
		PopDialog(r);
	}
}

static int OnShellNotify(HWND hwnd, int uID, int uMessage)
{
    if(uID == ID_TRAY) {
        if(uMessage == WM_RBUTTONDOWN) {
			POINT cp;
			GetCursorPos(&cp);
			SetForegroundWindow(gui->hwnd);
			TrackPopupMenu(gui->hmenuTray, TPM_LEFTALIGN|TPM_LEFTBUTTON, cp.x, cp.y, 0, gui->hwnd, NULL);
		}
    }
    return 0;
}

LRESULT CALLBACK WndProc (HWND hwnd , UINT msg,WPARAM wParam , LPARAM lParam)
{
	switch(msg) {
		case WM_SHELLNOTIFY:
			return OnShellNotify(hwnd, wParam, lParam);
		case WM_CREATE:
			OnCreate(hwnd);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_COMMAND:
			OnCommand(hwnd, (int)wParam);
			break;
		default:
			return DefWindowProc(hwnd,msg,wParam,lParam);
	}
}

BOOL CALLBACK 	ToolDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
		case WM_INITDIALOG:
			reminder *r;
			r = (reminder *)lParam;
			r->hwnd = hwnd;
			if (r->text) 
				SetDlgItemText(hwnd, ID_MESSAGE, r->text);
			SetDlgItemText(hwnd, ID_TIME, "10");
			break;
		case WM_CLOSE:
			reminder *del;
			del = FindReminder(hwnd);
			DeleteReminder(del);
			GenerateFile();
			EndDialog(hwnd, 0); 
			break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case ID_CLOSE:
					PostMessage(hwnd,WM_CLOSE,0,0);
					break;
				case ID_SNOOZE:
					int len;

					len = GetWindowTextLength(GetDlgItem(hwnd, ID_TIME));
					if (!len) {
						MessageBox(hwnd, "Time not specified.","Err",MB_ICONERROR | MB_OK);
						break;
					}
					char *time = (char *)malloc(sizeof(char)*(len+1) );
					GetDlgItemText(hwnd, ID_TIME, time, len+1);

					len = GetWindowTextLength(GetDlgItem(hwnd, ID_MESSAGE));
					if (!len) {
						MessageBox(hwnd, "Text not specified.","Err",MB_ICONERROR | MB_OK);
						break;
					}
					char *text = (char *)malloc(sizeof(char)*(len+1) );
					GetDlgItemText(hwnd, ID_MESSAGE, text, len+1);

					reminder *r = FindReminder(hwnd);
					if (!CreateReminder(hwnd,r,time,text))
						break;
					GenerateFile();

					EndDialog(hwnd,1);

					break;
			}
		break;
		default:
			return FALSE;
	}
	return TRUE;
}


void SpawnDialogThread( void *param  ){\
	reminder *r = (reminder *)param;
	DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(ID_REMINDER),gui->hwnd, (DLGPROC)ToolDlgProc, (LPARAM)param  );
	r->hwnd = 0;
}

void PopDialog(reminder *r) {
	if (!r->hwnd)
		_beginthread(SpawnDialogThread, 0, (void *)r);
}


int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{	
//#ifdef _DEBUG
//	AllocConsole();
//	HWND h;
//	h=GetConsoleWindow();
//	SetActiveWindow(h);
//	int fd = _open_osfhandle( (long)GetStdHandle( STD_OUTPUT_HANDLE ), 0);
//	FILE *fp = _fdopen( fd, "w" );
//	*stdout = *fp;
//	setvbuf( stdout, NULL, _IONBF, 0 );
//#endif

	gui = (gui_struct *)malloc(sizeof(gui_struct));

	gui->createtray = 1;
	gui->height = 500;
	gui->width = 500;
	gui->hInstance = hInstance;

	WNDCLASSEX wc;
	LoadLibrary("RichEd20.dll");
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style  = 0;
	wc.lpfnWndProc = WndProc ;
	wc.cbClsExtra  = 0;
	wc.cbWndExtra  = 0;
	wc.hInstance   = hInstance;
	wc.hIcon       = LoadIcon  (NULL,IDI_APPLICATION);
	wc.hCursor     = LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground = (HBRUSH)3;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "DirtyDeedsWnd";
	wc.hIconSm       = LoadIcon(NULL,IDI_APPLICATION);

	if(!RegisterClassEx(&wc)) {
        	MessageBox(NULL , "Failed to register window class" , "Error" , MB_ICONEXCLAMATION | MB_OK );
 	        return 0;
	}

	HWND hwnd = CreateWindowEx(
		//WS_EX_CLIENTEDGE,
		WS_EX_TOOLWINDOW|WS_EX_TOPMOST,
		wc.lpszClassName,
		"DirtyDeeds" ,
		WS_OVERLAPPED|WS_SYSMENU,
		CW_USEDEFAULT , CW_USEDEFAULT , gui->width, gui->height,
		NULL , NULL , 
		hInstance , NULL
		);


	if(hwnd == NULL) {
		MessageBox(NULL , "Failed to create window", "Error", MB_ICONEXCLAMATION | MB_OK );
		return 0;
    }

	ShowWindow  (hwnd,SW_HIDE);
	UpdateWindow(hwnd);

	InitLua();
	LoadScript(L, REMINDER_FILE);
	LoadScript(L, "dirtydeeds.lua");

	lua_getglobal(L, "SHELL");
	SHELL = (char *)luaL_optstring(L, 1, "cmd.exe");

	// Checking time every 60sec
	int tID = SetTimer(hwnd,1,60*1000, (TIMERPROC)TimerFunc);

	MSG Msg;
	while(GetMessage(&Msg,NULL,0,0)) {	
		TranslateMessage(&Msg);
		DispatchMessage (&Msg);
	}

	return 1;
}


/*static int luaB_print (lua_State *L) {
  int n = lua_gettop(L);
  int i;
  lua_getglobal(L, "tostring");
  guiAddText(">");
  for (i=1; i<=n; i++) {
    const char *s;
    lua_pushvalue(L, -1);
    lua_pushvalue(L, i);
    lua_call(L, 1, 1);
    s = lua_tostring(L, -1);
    if (s == NULL)
      return luaL_error(L, LUA_QL("tostring") " must return a string to "
                           LUA_QL("print"));
    if (i>1) guiAddText("\t");
    guiAddText(s);
    lua_pop(L, 1);
  }
  guiAddText("\n");
  return 0;
}*/