#include "reminder.h"
#include <stdio.h>

reminder *rhead = 0;

int ParseReminderTimeField(char *ptr, char **endptr) {
	while (*ptr == ' ') ptr++;
	if (*ptr == 0) return -1;
	bool nonZero = ( *ptr != '0');
	int ret = strtol(ptr,&ptr,10);
	*endptr = ptr;
	if (!ret && nonZero) return -1; // when strtol returns 0 it either means it parsed 0, or there was no match
	return ret;
}
int ParseReminderTime(reminder *r, char *ptr) {
	while (*ptr == ' ') ptr ++;
	if (*ptr == '@') {
		ptr++;
		
		time_t rawtime;
		struct tm * t;
		time ( &rawtime );
		t = localtime ( &rawtime );

		int min = ParseReminderTimeField(ptr,&ptr);
		int hour = ParseReminderTimeField(ptr,&ptr);
		if (hour == -1) hour = t->tm_hour;
		int mday = ParseReminderTimeField(ptr,&ptr);
		if (mday == -1) mday = t->tm_mday;
		int mon = ParseReminderTimeField(ptr,&ptr);
		if (mon == -1) mon = t->tm_mon;

		if (min > -1 && min < 60) r->min = min;
		else return 0;
		if (hour > -1 && hour < 24) r->hour = hour;
		else return 0;
		if (mday > -1 && mday < 32 ) r->mday = mday;
		else return 0;
		if (mon > -1 && mon < 13) r->mon = mon;
		else return 0;
		
		return 1;
	}


	int min = ParseReminderTimeField(ptr,&ptr);
	int hour = ParseReminderTimeField(ptr,&ptr);
	__int64 day = ParseReminderTimeField(ptr,&ptr);
	min = (min > -1) ? min * 60 : 0 ;
	hour = (hour > -1) ? hour * 3600 : 0;
	day = (day > -1) ? day * 3600 * 24 : 0;
	unsigned __int64 total_seconds = min + hour + day;
	if (total_seconds == 0) return 0;

	time_t rawtime;
	struct tm * t;
	time ( &rawtime );
	rawtime += total_seconds;
	t = localtime ( &rawtime );

	r->min = t->tm_min;
	r->hour = t->tm_hour;
	r->mday = t->tm_mday;
	r->mon = t->tm_mon;

	return 1;
}
int CreateReminder(HWND hwnd, reminder *r, char *time, char *text) {
	if (!ParseReminderTime(r,time)) {
		if (hwnd) MessageBox(hwnd,"Time validation error.","Err",MB_ICONERROR | MB_OK);
		return 0;
	}

	r->text = text;
	return 1;
}


int GenerateFile(){
	FILE *f = fopen(REMINDER_FILE,"w");
	reminder *r = rhead;
	while (r) {
		fprintf(f, "Reminder('@%d %d %d %d',[[%s]])\n",r->min, r->hour, r->mday, r->mon, r->text);
		r = r->next;
	}
	fclose(f);
	return 1;
}

void RemindersMessageBox(){
	char *ptr = (char*)malloc(sizeof(char)*(80*20+1));
	memset(ptr,0,(sizeof(char)*(80*20+1)));
	char *str = ptr;
	reminder *r = rhead;
	int i = 0;
	while (r && i < 20) {
		ptr += _snprintf(ptr, 80, "%02d:%02d %02d.%02d | %s \n",r->hour,r->min, r->mday, r->mon, r->text);;
		r = r->next;
		i++;
	}
	MessageBox(0,str,"Reminders List",MB_ICONINFORMATION|MB_OK);
	free(str);
}

reminder* NewReminder() {
	reminder *r = (reminder *)malloc(sizeof(reminder));
	memset(r,0,sizeof(reminder));

	time_t rawtime;
	struct tm * t;
	time ( &rawtime );
	t = localtime ( &rawtime );

	r->min = t->tm_min;
	r->hour = t->tm_hour;
	r->mday = t->tm_mday;
	r->mon = t->tm_mon;

	if (!rhead)
		rhead = r;
	else
		r->next = rhead;
		rhead = r;
	
	return r;
}

reminder* FindReminder(HWND hwnd) {
	reminder *r = rhead;
	while (r) {
		if (r->hwnd == hwnd)
			return r;
		r = r->next;
	}
	return 0;
}

void DeleteReminder(reminder *todel) {
	if (!rhead) return;
	if (rhead == todel)
		rhead = rhead->next;
	reminder *r = rhead;
	reminder *prev = rhead;
	while (r) {
		if (todel == r) {
			prev->next = r->next;
			break;
		}
		prev = r;
		r = r->next;
	}
	free(todel);
	return;
}

int IsMissedReminder(reminder *r){
	time_t rawtime;
	struct tm * t;
	time ( &rawtime );
	t = localtime ( &rawtime );

	t->tm_min = r->min;
	t->tm_hour = r->hour;
	t->tm_mday = r->mday;
	t->tm_mon = r->mon;

	time_t rtime = mktime(t);
	if (rawtime >= rtime)
		return 1;
	return 0;
}