// File: seek.c
// Author: Doug VK
// Professor: Vladimir Rokhlin
// Description: O(n^2) and O(nlogn) for nearest k points

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

//I know -- too much work to deal with
int num_leaves = 0;

void find_range(double* a, int n, double* max_x, double* max_y, double* min_x, double* min_y);
void create_children(double qt[][13], int ix[], double* a, int index, int max_nodes, int n);
void expand_tree(double qt[][13], int ix[], double* a, int max_nodes, int k, int n);
void calc_distance(double x1, double y1, double x2, double y2, double* distance);
int bounded(double x, double y, double x1, double y1, double x2, double y2);
void find_nearest_boxes(int boxes[][num_leaves+3], double qt[][13]);
void print_matrix(double A[][13], char* string, int m, int n);
void seek_naive(double* a, int n, int k, int* iz);
void seek(double* a, int n, int k, int* iz);

enum tree
{
    id = 0,
    parent = 5,
    begin_ix = 6,
    end_ix = 7,
    num_pts = 8,
    x_1 = 9,
    y_1 = 10,
    x_2 = 11,
    y_2 = 12,
};

int main(int argc, const char *argv[])
{
    cprini("stdout","output");

    int i;
    int k = 100;
    int n = 10000;
    int* iz = (int*) malloc(k*n*sizeof(int));
    double* a = (double*) malloc(2*n*sizeof(double));
    unsigned int iseed = (unsigned int)time(NULL);
    srand(iseed);

    for (i = 0; i < n; i++) {
        a[2*i] = 1 + (int)( 100.0 * rand() / ( RAND_MAX + 1.0 ) );
        a[2*i+1] = 1 + (int)( 100.0 * rand() / ( RAND_MAX + 1.0 ) );
    }

    double start, end;
    double cpu_time_used;

    start = clock();
    seek(a,n,k,iz);
    /*cprinf("iz = ", iz, k*n);*/
    end = clock();
    cpu_time_used = ((double)(end - start)) / ((double)CLOCKS_PER_SEC);

    printf("start = %.20f\nend   = %.20f\n", start, end);
    printf("delta = %.20f\n", ((double) (end - start)));
    printf("cpu_time_used  = %.15f\n", cpu_time_used);

    /*start = clock();*/
    /*seek_naive(a,n,k,iz);*/
    /*[>cprinf("iz naive = ", iz, k*n);<]*/
    /*end = clock();*/
    /*cpu_time_used = ((double)(end - start)) / ((double)CLOCKS_PER_SEC);*/

    /*printf("start = %.20f\nend   = %.20f\n", start, end);*/
    /*printf("delta = %.20f\n", ((double) (end - start)));*/
    /*printf("cpu_time_used  = %.15f\n", cpu_time_used);*/


    return 0;
}

void seek(double* a, int n, int k, int* iz)
{
    int i, j, max_nodes = 0;

    //log base4 of n gives number of levels
    //this is just a rough estimate
    double max_leaves = ceil(((double)n)/((double)k));
    double num_levels = ceil((log10(max_leaves))/(log10(4)));
    for (i = 0; i <= num_levels+1; i++) {
        max_nodes += pow(4,i);
    }

    //initialize all parents to -2, 0 o.w.
    double qt[max_nodes][13];
    for (i = 0; i < max_nodes; i++) {
        for (j = 0; j < 13; j++) {
            qt[i][j] = 0;
        }
        qt[i][parent] = -2;
    }

    double max_x = -1e9, max_y = -1e9, min_x = 1e9, min_y = 1e9;

    int ix[n];
    for (i = 0; i < n; i++)
        ix[i] = i;

    //get bounding box of the n points
    find_range(a,n,&max_x,&max_y,&min_x,&min_y);

    //instantiate first row of tree table
    qt[0][id] = qt[0][1] = qt[0][2] = qt[0][3] = qt[0][4] = 0;
    qt[0][parent] = -1;
    qt[0][begin_ix] = 0;
    qt[0][end_ix] = n-1;
    qt[0][num_pts] = n;
    qt[0][x_1] = min_x;
    qt[0][y_1] = min_y;
    qt[0][x_2] = max_x;
    qt[0][y_2] = max_y;

    //create children until there are <= k points in each leaf
    expand_tree(qt, ix, a, max_nodes, k, n);

    //count leaves to investigate
    num_leaves = 0;
    for (i = 0; i < max_nodes; i++) {
        if(qt[i][num_pts] <= k && qt[i][num_pts] != 0) {
            num_leaves++;
        }
    }

    //initialize boxes, set boxes[i][0] == leaf index in qt
    int boxes[num_leaves][num_leaves+3];
    int temp;
    for (i = 0; i < num_leaves; i++) {
        for (j = 0; j < num_leaves+3; j++) {
            boxes[i][j] = -1;
        }
    }
    for (i = 0, temp = 0; i < max_nodes; i++) {
        if(qt[i][num_pts] <= k && qt[i][num_pts] != 0) {
            boxes[temp][0] = i;
            temp++;
        }
    }

    //for each index in boxes, find nearby boxes to check for points
    find_nearest_boxes(boxes,qt);

    int m,l,z;
    //go through all leaves
    for (i = 0; i < num_leaves; i++) {
        //get the leaf index
        int index = boxes[i][0];
        double distance;
        //find where in ix the leaf matches
        for (j = qt[index][begin_ix]; j <= qt[index][end_ix]; j++) {
            //declare a helper array to find the closest points
            double helper[2*k];
            for (z = 0; z < 2*k; z++)
                helper[z] = 1e9;

            //find out the point we're interested in, get coordinates
            int point_index = ix[j];
            /*printf("%i ", j);*/
            int x = a[2*point_index];
            int y = a[2*point_index+1];

            //go through nearby boxes
            for (m = 1; boxes[i][m] != -1; m++) {
                int leaf_ix = boxes[i][m];
                //for each box, find the point(s) it contains
                for (l = qt[leaf_ix][begin_ix]; l <= qt[leaf_ix][end_ix]; l++) {
                    int comp_ix = ix[l];
                    if(comp_ix == point_index)
                        continue;
                    int x_comp = a[2*comp_ix];
                    int y_comp = a[2*comp_ix+1];
                    calc_distance(x,y,x_comp,y_comp,&distance);

                    //add if it is a minimum point
                    double temp_dist = 1e10, temp_ix = 1e10, temp_dist2 = 1e10, temp_ix2 = 1e10;
                    int placed = 0;
                    for (z = 0; z < k; z++) {
                        if(temp_dist < helper[2*z]) {
                            temp_dist2 = helper[2*z];
                            temp_ix2 = helper[2*z+1];
                            helper[2*z] = temp_dist;
                            helper[2*z+1] = temp_ix;
                            temp_dist = temp_dist2;
                            temp_ix = temp_ix2;
                        }

                        if(!placed && distance < helper[2*z]) {
                            temp_dist = helper[2*z];
                            temp_ix = helper[2*z+1];
                            helper[2*z] = distance;
                            helper[2*z+1] = comp_ix;
                            placed = 1;
                        }
                    }
                }
            }

            //go through minimum points, add all k of them to iz
            for (m = 0; m < k; m++)
                iz[point_index*k+m] = helper[2*m+1];
        }
    }

    /*cprinf("iz = ", iz, k*n);*/
}

void find_nearest_boxes(int boxes[][num_leaves+3], double qt[][13])
{
    int i,j,k;
    /*printf("NUM LEAVES: %i", num_leaves);*/
    
    //for every leaf
    for (i = 0; i < num_leaves; i++) {
        //find max radius to look within
        int leaf_ix = boxes[i][0];
        int parent_ix = qt[leaf_ix][parent];
        double x1 = qt[parent_ix][x_1], y1 = qt[parent_ix][y_1],
               x2 = qt[parent_ix][x_2], y2 = qt[parent_ix][y_2];

        //calc center of leaf and radius
        double center_x = (qt[leaf_ix][x_2] - qt[leaf_ix][x_1]) / 2 + qt[leaf_ix][x_1];
        double center_y = (qt[leaf_ix][y_2] - qt[leaf_ix][y_1]) / 2 + qt[leaf_ix][y_1];
        double radius = sqrt(pow(x2-x1,2)+pow(y2-y1,2));

        //go through all leaves, add the leaves that touch the radius
        for (j = 0, k = 1; j < num_leaves; j++) {
            if(nearby(boxes[j][0],qt,radius,center_x,center_y)) {
                boxes[i][k] = boxes[j][0];
                k++;
            }
        }
        /*cprinf("boxes = ", &boxes[i][0], num_leaves+3);*/
    }
}

int nearby(int comp_ix, double qt[][13], double radius, double center_x, double center_y)
{
    double x1 = qt[comp_ix][x_1], y1 = qt[comp_ix][y_1], x2 = qt[comp_ix][x_2], y2 = qt[comp_ix][y_2];
    double distance_1, distance_2, distance_3, distance_4;

    calc_distance(center_x,center_y,x1,y1,&distance_1);
    calc_distance(center_x,center_y,x1,y2,&distance_2);
    calc_distance(center_x,center_y,x2,y1,&distance_3);
    calc_distance(center_x,center_y,x2,y2,&distance_4);

    if(distance_1 < radius || distance_2 < radius || distance_3 < radius || distance_4 < radius)
        return 1;
    return 0;
}

void expand_tree(double qt[][13], int ix[], double* a, int max_nodes, int k, int n)
{
    int i;
    for (i = 0; i < max_nodes; i++) {
        //for every node with > k points, create 4 children
        if(qt[i][num_pts] > k) {
            create_children(qt,ix,a,i,max_nodes,n);
            /*printf("-------------------------------------------");*/
            /*print_matrix(qt, "qt = ", max_nodes, 13);*/
        }
    }
}

void create_children(double qt[][13], int ix[], double* a, int index, int max_nodes, int n)
{
    int i,j;
    int child_indices[4];

    //searches for open spots to put boxes in the table
    for (i = 0, j = 0; i < max_nodes && j < 4; i++) {
        if(qt[i][parent] == -2) {
            child_indices[j] = i;
            j++;
        }
    }
    /*cprinf("indices = ", child_indices, 4);*/

    //sets the child indices in the parent node
    double increment_x = (qt[index][x_2] - qt[index][x_1]) / 2;
    double increment_y = (qt[index][y_2] - qt[index][y_1]) / 2;
    for (i = 0; i < 4; i++) {
        qt[child_indices[i]][parent] = index;
        qt[child_indices[i]][id] = child_indices[i];
        qt[index][i+1] = child_indices[i];
    }

    //assigns the new bounding box to each of the children
    qt[child_indices[0]][x_1] = qt[child_indices[2]][x_1] = qt[index][x_1];
    qt[child_indices[0]][y_1] = qt[child_indices[1]][y_1] = qt[index][y_1];
    qt[child_indices[3]][y_2] = qt[child_indices[2]][y_2] = qt[index][y_2];
    qt[child_indices[1]][x_2] = qt[child_indices[3]][x_2] = qt[index][x_2];
    qt[child_indices[1]][x_1] = qt[child_indices[2]][x_2] = increment_x + qt[index][x_1];
    qt[child_indices[3]][x_1] = qt[child_indices[0]][x_2] = increment_x + qt[index][x_1];
    qt[child_indices[2]][y_1] = qt[child_indices[0]][y_2] = increment_y + qt[index][y_1];
    qt[child_indices[1]][y_2] = qt[child_indices[3]][y_1] = increment_y + qt[index][y_1];

    //create temp arrays and values to store new point assignments in ix
    int child_1[n], child_2[n], child_3[n], child_4[n];
    int ix_1 = qt[index][1], ix_2 = qt[index][2], ix_3 = qt[index][3], ix_4 = qt[index][4];

    //find all points that belong to child 1
    for (i = qt[index][begin_ix], j = 0; i <= qt[index][end_ix]; i++) {
        double x1 = qt[ix_1][x_1], x2 = qt[ix_1][x_2], y1 = qt[ix_1][y_1], y2 = qt[ix_1][y_2];
        int real_ix = ix[i];
        if(bounded(a[2*real_ix],a[2*real_ix+1],x1,y1,x2,y2)) {
            child_1[j] = real_ix;
            j++;
        }
    }
    //store number of points assigned to 1
    qt[ix_1][num_pts] = j;
    child_1[j] = 1e9;

    //find all points that belong to child 2
    for (i = qt[index][begin_ix], j = 0; i <= qt[index][end_ix]; i++) {
        double x1 = qt[ix_2][x_1], x2 = qt[ix_2][x_2], y1 = qt[ix_2][y_1], y2 = qt[ix_2][y_2];
        int real_ix = ix[i];
        if(bounded(a[2*real_ix],a[2*real_ix+1],x1,y1,x2,y2)) {
            child_2[j] = real_ix;
            j++;
        }
    }
    //store number of points assigned to 2
    qt[ix_2][num_pts] = j;
    child_2[j] = 1e9;

    //and so on...
    for (i = qt[index][begin_ix], j = 0; i <= qt[index][end_ix]; i++) {
        double x1 = qt[ix_3][x_1], x2 = qt[ix_3][x_2], y1 = qt[ix_3][y_1], y2 = qt[ix_3][y_2];
        int real_ix = ix[i];
        if(bounded(a[2*real_ix],a[2*real_ix+1],x1,y1,x2,y2)) {
            child_3[j] = real_ix;
            j++;
        }
    }
    qt[ix_3][num_pts] = j;
    child_3[j] = 1e9;

    //etc...
    for (i = qt[index][begin_ix], j = 0; i <= qt[index][end_ix]; i++) {
        double x1 = qt[ix_4][x_1], x2 = qt[ix_4][x_2], y1 = qt[ix_4][y_1], y2 = qt[ix_4][y_2];
        int real_ix = ix[i];
        if(bounded(a[2*real_ix],a[2*real_ix+1],x1,y1,x2,y2)) {
            child_4[j] = real_ix;
            j++;
        }
    }
    qt[ix_4][num_pts] = j;
    child_4[j] = 1e9;

    /*cprinf("ix = ", ix, 12);*/

    //rearrange ix with new children indices
    i = qt[index][begin_ix];
    qt[ix_1][begin_ix] = i;
    for (j = 0; j < qt[ix_1][num_pts]; j++)
        ix[i+j] = child_1[j];
    i += j;
    qt[ix_1][end_ix] = i-1;

    //do for each child
    qt[ix_2][begin_ix] = i;
    for (j = 0; j < qt[ix_2][num_pts]; j++)
        ix[i+j] = child_2[j];
    i += j;
    qt[ix_2][end_ix] = i-1;

    //yup..
    qt[ix_3][begin_ix] = i;
    for (j = 0; j < qt[ix_3][num_pts]; j++)
        ix[i+j] = child_3[j];
    i += j;
    qt[ix_3][end_ix] = i-1;

    //and done.
    qt[ix_4][begin_ix] = i;
    for (j = 0; j < qt[ix_4][num_pts]; j++)
        ix[i+j] = child_4[j];
    i += j;
    qt[ix_4][end_ix] = i-1;

    /*cprinf("child_1 = ", child_1, 12);*/
    /*cprinf("child_2 = ", child_2, 12);*/
    /*cprinf("child_3 = ", child_3, 12);*/
    /*cprinf("child_4 = ", child_4, 12);*/
    /*cprinf("ix = ", ix, 12);*/
}

int bounded(double x, double y, double x1, double y1, double x2, double y2)
{
    if(x < x1 || x > x2) return 0;
    if(y < y1 || y > y2) return 0;
    return 1;
}

//print matrix A(m,n)
void print_matrix(double A[][13], char* string, int m, int n)
{
    int i;
    cprin2(string, A[0], n);
    for(i=1;i<m;i++)
        cprin2("", A[i], n);
}

void seek_naive(double* a, int n, int k, int* iz)
{
    double x1,y1,x2,y2,distance;
    int i,j,z;
    double helper[2*k];
    for (i = 0; i < 2*k; i++)
        helper[i] = 1e9;

    //for each point...
    for (i = 0; i < n; i++) {
        x1 = a[2*i];
        y1 = a[2*i+1];
        //go through every other point
        for (j = 0; j < n; j++) {
            if(i==j) continue;
            x2 = a[2*j];
            y2 = a[2*j+1];
            calc_distance(x1,y1,x2,y2,&distance);

            //add if it is a minimum point
            double temp_dist = 1e10, temp_ix = 1e10, temp_dist2 = 1e10, temp_ix2 = 1e10;
            int placed = 0;
            for (z = 0; z < k; z++) {
                if(temp_dist < helper[2*z]) {
                    temp_dist2 = helper[2*z];
                    temp_ix2 = helper[2*z+1];
                    helper[2*z] = temp_dist;
                    helper[2*z+1] = temp_ix;
                    temp_dist = temp_dist2;
                    temp_ix = temp_ix2;
                }

                if(!placed && distance < helper[2*z]) {
                    temp_dist = helper[2*z];
                    temp_ix = helper[2*z+1];
                    helper[2*z] = distance;
                    helper[2*z+1] = j;
                    placed = 1;
                }
            }
        }

        //add the minimum points to iz for each point
        for (j = 0; j < k; j++)
            iz[i*k+j] = helper[2*j+1];
        //reset
        for (z = 0; z < 2*k; z++)
            helper[z] = 1e9;
    }
}

void calc_distance(double x1, double y1, double x2, double y2, double* distance)
{
    *distance = sqrt(pow(x1-x2,2)+pow(y1-y2,2));
}

void find_range(double* a, int n, double* max_x, double* max_y, double* min_x, double* min_y)
{
    int i;
    for (i = 0; i < n; i++) {
        if(a[2*i] > *max_x) *max_x = a[2*i];
        if(a[2*i+1] > *max_y) *max_y = a[2*i+1];
    }

    for (i = 0; i < n; i++) {
        if(a[2*i] < *min_x) *min_x = a[2*i];
        if(a[2*i+1] < *min_y) *min_y = a[2*i+1];
    }
}
