/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/cdb-private.h"
#include "sv/cdb.h"

/**
 * Initialize a cdb struct to use the specified file descriptor.
 *
 * @param cdb cdb
 * @param fd  cdb file descriptor
 *
 * @return  0 success
 * @return -1 failure
 */

int_fast8_t cdb_init(cdb_t *cdb, int fd) {
  struct stat st;
  
  /* If the file exist and doesn't exceed the maximum file size then setup the
   * cdb struct. */
  if (fstat(fd, &st) == 0) {
    if (st.st_size <= CDB_MAX_SIZE) {
      cdb->loop = 0;
      cdb->fd = fd;
      cdb->size = st.st_size;
      
      cdb->map = mmap(0, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
      if (cdb->map == MAP_FAILED)
        cdb->map = 0;
        
      return 0;
    }
  }
  
  return -1;
}
