/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/cdb-private.h"
#include "sv/cdb.h"

/**
 * Load part of the cdb file into the data buffer.
 *
 * @param data data buffer
 * @param dlen data buffer size
 * @param cdb  cdb
 * @param pos  cdb file position
 */

int_fast8_t cdb_read(void *data, uint32_t dlen, cdb_t *cdb, uint32_t pos) {
  ssize_t r;
  
  /* If the cdb file was successfully mmap'd then copy the file data;
   * otherwise use seek/read to get it. */
  if (cdb->map) {
    if ((pos > cdb->size) || (cdb->size - pos < dlen))
      return -1;
    memcpy(data, cdb->map + pos, dlen);
  } else {
    if (lseek(cdb->fd, pos, SEEK_SET) == -1)
      return -1;
    while (dlen > 0) {
      do {
        r = read(cdb->fd, data, dlen);
      } while ((r == -1) && (errno == EINTR));
      if (r <= 0)
        return -1;
      data += r;
      dlen -= r;
    }
  }
  
  return 0;
}
