/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/cdb-private.h"
#include "sv/cdbmake-private.h"
#include "sv/cdbmake.h"

/**
 * Add a new record to the cdb file.
 *
 * @param cm   cdbmake
 * @param key  key
 * @param klen key length
 * @param data data
 * @param dlen data length
 *
 * @return  0 success
 * @return -1 failure
 */

int_fast8_t cdbmake_add(cdbmake_t *cm, void *key, uint32_t klen, void *data, uint32_t dlen) {
  uint32_t buf[2];
  uint32_t hash;
  struct cdbmake_hplist *head;
  
  hash = _cdb_hash(key, klen);
  
  /* Create the record by packing the key length and data length into the
   * buffer and writing it to the cdb file followed by the actual key and
   * data. */
  buf[0] = htobe32(klen);
  buf[1] = htobe32(dlen);
  if (write(cm->fd, buf, sizeof(buf)) == -1)
    return -1;
  if (write(cm->fd, key, klen) == -1)
    return -1;
  if (write(cm->fd, data, dlen) == -1)
    return -1;
    
  /* Allocate a new hplist if one doesn't exist or the current one is full. */
  head = cm->head;
  if (!head || (head->num >= CDBMAKE_HPLIST_SIZE)) {
    head = (struct cdbmake_hplist *)malloc(sizeof(struct cdbmake_hplist));
    if (!head)
      return -1;
    head->num = 0;
    head->next = cm->head;
    cm->head = head;
  }
  
  /* Add the hash/pointer to the list. */
  head->hp[head->num].hash = hash;
  head->hp[head->num].pos = cm->pos;
  head->num++;
  
  if (_cdbmake_update_pos(cm, sizeof(buf) + klen + dlen) == -1)
    return -1;
    
  return 0;
}
