/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_CDB_H
#define SV_CDB_H

#include <stdint.h>
#include <sys/types.h>

/**
 * @file cdb.h
 * @brief Constant Database Routines
 * @defgroup sv_cdb Constant Database Routines
 * @{
 */

#define CDB_MAX_SIZE UINT32_MAX /**< cdb file size limit */

/** cdb */
typedef struct cdb {
  int       fd;    /**< file descriptor */
  void     *map;   /**< mmap() address */
  uint32_t  size;  /**< cdb file size */
  uint32_t  loop;  /**< lookup loop */
  uint32_t  khash; /**< key hash */
  uint32_t  kpos;  /**< key position in hash table */
  uint32_t  hpos;  /**< index's hash position */
  uint32_t  hlen;  /**< index's hash length */
  uint32_t  dlen;  /**< data length */
  uint32_t  dpos;  /**< data position */
} cdb_t;

int_fast8_t cdb_read(void *, uint32_t, cdb_t *cdb, uint32_t);
int_fast8_t cdb_init(cdb_t *, int);
int_fast8_t cdb_find(cdb_t *, void *, uint32_t);
int_fast8_t cdb_find_next(cdb_t *, void *, uint32_t);
void cdb_free(cdb_t *);

/** @} */

#endif
