/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_CDBMAKE_H
#define SV_CDBMAKE_H

#include <stdint.h>

/**
 * @file cdbmake.h
 * @brief Constant Database Creation Routines
 * @defgroup sv_cdbmake Constant Database Creation Routines
 * @{
 */

#define CDBMAKE_HPLIST_SIZE 1000 /**< elements in each hp list */

/** hash/position entry */
struct cdbmake_hp {
  uint32_t hash; /**< hash */
  uint32_t pos;  /**< position */
};

/** hash/position entry list */
struct cdbmake_hplist {
  struct cdbmake_hp      hp[CDBMAKE_HPLIST_SIZE]; /**< hp array */
  struct cdbmake_hplist *next;                    /**< next hplist */
  int                    num;                     /**< hp array number */
};

/** cdb make */
typedef struct cdbmake {
  int                    fd;          /**< file descriptor */
  uint32_t               pos;         /**< file position */
  struct cdbmake_hplist *head;        /**< hash/position list */
} cdbmake_t;

int_fast8_t cdbmake_add(cdbmake_t *, void *, uint32_t, void *, uint32_t);
int_fast8_t cdbmake_finish(cdbmake_t *);
int_fast8_t cdbmake_start(cdbmake_t *, int fd);

/** @} */

#endif
