/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include "sv/cdb.h"
#include "sv/cdbmake.h"

static char *alphanum = \
                        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

static uint32_t random_value(uint8_t *value, uint16_t vlen) {
  uint16_t i;
  uint16_t len = (rand() % vlen) + 1;
  for (i = 0; i < len; i++)
    value[i] = alphanum[rand() % 62];
  return len;
}

int main() {
  cdbmake_t cm;
  int fd;
  int i;
  uint8_t key[32];
  uint32_t klen;
  uint8_t value[1024];
  uint32_t vlen;
  
  /* Create an empty cdb file. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 1;
  if (cdbmake_start(&cm, fd) == -1)
    return 2;
  if (cdbmake_finish(&cm) == -1)
    return 3;
    
  /* Create a single record cdb file. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 4;
  if (cdbmake_start(&cm, fd) == -1)
    return 5;
  if (cdbmake_add(&cm, "key", 3, "value", 5) == -1)
    return 6;
  if (cdbmake_finish(&cm) == -1)
    return 7;
  close(fd);
  
  /* Create a multi-record cdb file. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 8;
  if (cdbmake_start(&cm, fd) == -1)
    return 9;
  if (cdbmake_add(&cm, "key1", 4, "value1", 6) == -1)
    return 10;
  if (cdbmake_add(&cm, "key2", 4, "value2", 6) == -1)
    return 11;
  if (cdbmake_add(&cm, "key3", 4, "value3", 6) == -1)
    return 12;
  if (cdbmake_finish(&cm) == -1)
    return 13;
  close(fd);
  
  /* Create a random multi-record cdb file. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 14;
  if (cdbmake_start(&cm, fd) == -1)
    return 15;
  for (i = 0; i < 10000; i++) {
    klen = random_value(key, sizeof(key));
    vlen = random_value(value, sizeof(value));
    if (cdbmake_add(&cm, key, klen, value, vlen) == -1)
      return 16;
  }
  if (cdbmake_finish(&cm) == -1)
    return 17;
  close(fd);
  
  return 0;
}
