/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <fcntl.h>
#include <unistd.h>
#include "sv/cdb.h"
#include "sv/cdbmake.h"

int main() {
  cdb_t c;
  cdbmake_t cm;
  int fd;
  
  /* Create an empty cdb file and try to find a value in it. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 1;
  if (cdbmake_start(&cm, fd) == -1)
    return 2;
  if (cdbmake_finish(&cm) == -1)
    return 3;
  close(fd);
  fd = open("test.cdb", O_RDONLY);
  if (fd == -1)
    return 4;
  if (cdb_init(&c, fd) == -1)
    return 5;
  if (cdb_find(&c, "key", 3) != 0)
    return 6;
  cdb_free(&c);
  close(fd);
  
  /* Create a single record cdb file and try to find both a value that's in it
   * and a value that's not in it. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 7;
  if (cdbmake_start(&cm, fd) == -1)
    return 8;
  if (cdbmake_add(&cm, "key", 3, "value", 5) == -1)
    return 9;
  if (cdbmake_finish(&cm) == -1)
    return 10;
  close(fd);
  fd = open("test.cdb", O_RDONLY);
  if (fd == -1)
    return 11;
  if (cdb_init(&c, fd) == -1)
    return 12;
  if (cdb_find(&c, "0", 1) != 0)
    return 13;
  if (cdb_find(&c, "key", 3) != 1)
    return 14;
  cdb_free(&c);
  close(fd);
  
  /* Create a duplicate record cdb file and make sure they're both found. */
  fd = open("test.cdb", O_CREAT | O_TRUNC | O_WRONLY, 0644);
  if (fd == -1)
    return 15;
  if (cdbmake_start(&cm, fd) == -1)
    return 16;
  if (cdbmake_add(&cm, "key", 3, "value1", 6) == -1)
    return 17;
  if (cdbmake_add(&cm, "key", 3, "value2", 6) == -1)
    return 18;
  if (cdbmake_finish(&cm) == -1)
    return 19;
  close(fd);
  fd = open("test.cdb", O_RDONLY);
  if (fd == -1)
    return 20;
  if (cdb_init(&c, fd) == -1)
    return 21;
  if (cdb_find(&c, "key", 3) != 1)
    return 22;
  if (cdb_find_next(&c, "key", 3) != 1)
    return 23;
  if (cdb_find_next(&c, "key", 3) != 0)
    return 24;
  cdb_free(&c);
  close(fd);
  
  return 0;
}
