/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/base.h"

const int8_t base64_charset[64] =
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
const int8_t base64url_charset[64] =
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
const int8_t base64pad = '=';

/**
 * Encode input buffer to base64 and store in output buffer.  It is the callers
 * responsibility to ensure the target buffer is large enough to hold the
 * entire encoded value which can be found using the BASE64_ENCODE_SIZE() macro.
 *
 * @param dst     output buffer
 * @param src     input buffer
 * @param len     input buffer length
 * @param charset character set
 *
 * @return Number of bytes written to output buffer.
 */

ssize_t _base64_encode(void *dst, void *src, size_t len, const int8_t *charset) {
  uint8_t a, b, c;
  unsigned int i;
  uint8_t *d = dst;
  uint8_t *p = src;
  
  if (len == 0)
    return 0;
    
  /* Read three bytes at a time from the input buffer and convert them to
   * base64 and put them in the output buffer. */
  for (i = 0; i < len; i += 3) {
    a = p[i];
    b = (i + 1) < len ? p[i + 1] : 0;
    c = (i + 2) < len ? p[i + 2] : 0;
    
    *d++ = charset[a >> 2];
    *d++ = charset[((a & 3) << 4) | (b >> 4)];
    *d++ = (len <= i + 1) ? base64pad : charset[((b & 15) << 2) | (c >> 6)];
    *d++ = (len <= i + 2) ? base64pad : charset[c & 63];
  }
  
  return d - (uint8_t *)dst;
}
