/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/array.h"
#include "sv/array-private.h"
#include "sv/bio.h"

/**
 * Read line of data from bio into array.
 *
 * @param array array
 * @param bio   bio
 *
 * @return -1 : failed
 * @return x : bytes read
 */

ssize_t bio_get_line(array_t *array, bio_t *bio) {
  char *data;
  int len;
  int pos;
  int total = 0;
  void *p;
  
  if (array_failed(array))
    return -1;
    
  for (;;) {
    len = bio_feed(bio);
    if (len < 0)
      return -1;
    if (len == 0)
      return total;
      
    p = memchr(bio->data + bio->size, '\n', len);
    pos = (p) ? (p - (bio->data + bio->size)) : len;
    
    _array_pad(array, pos);
    data = array_start(array) + array_size(array);
    len = bio_get(data, pos, bio);
    
    array->size += len;
    total += len;
    
    if (((char *)bio->data)[bio->size] == '\n') {
      BIO_SEEK(bio, 1);
      return total;
    }
  }
}
