/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/cache.h"

/**
 * Initialize cache.
 *
 * @param cache cache
 * @param size cache size
 * @return 0 error
 * @return 1 success
 */

int cache_init(cache_t *cache, size_t size) {
  if (cache->data) {
    free(cache->data);
    cache->data = 0;
  }
  
  if (size > CACHE_MAX_SIZE)
    size = CACHE_MAX_SIZE;
  if (size < CACHE_MIN_SIZE)
    size = CACHE_MIN_SIZE;
    
  cache->size = size;
  
  cache->hash_size = 4;
  while (cache->hash_size <= (cache->size >> 5))
    cache->hash_size <<= 1;
    
  cache->data = malloc(cache->size);
  if (!cache->data)
    return 0;
  memset(cache->data, 0, cache->size);
  
  cache->oldest = cache->size;
  cache->unused = cache->size;
  cache->writer = cache->hash_size;
  
  cache->stats.get = 0;
  cache->stats.miss = 0;
  cache->stats.motion = 0;
  cache->stats.set = 0;
  
  return 1;
}
