/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_FILE_H
#define SV_FILE_H

#include <fcntl.h>
#include <unistd.h>

/**
 * @file file.h
 * @brief File Routines
 * @defgroup sv_file File Routines
 * @{
 */

/** close file descriptor on exec */
#define close_on_exec(x)     fcntl(x, F_SETFD, FD_CLOEXEC)

/** disable blocking on file descriptor */
#define blocking_disable(x)  fcntl(x, F_SETFL, fcntl(x, F_GETFL, 0) | O_NONBLOCK)

/** enable blocking on file descriptor */
#define blocking_enable(x)   fcntl(x, F_SETFL, fcntl(x, F_GETFL, 0) & ~O_NONBLOCK)

/** test if socket is blocking */
#define is_blocking(x)       ((fcntl(x, F_GETFL, 0) & O_NONBLOCK) ? 0 : 1)

/** open file for appending */
#define open_append(x)       open(x, O_APPEND | O_CREAT | O_NONBLOCK | O_WRONLY , 0600)

/** open file for reading */
#define open_read(x)         open(x, O_NONBLOCK |  O_RDONLY)

/** open file for writing and truncate */
#define open_trunc(x)        open(x, O_CREAT | O_NONBLOCK | O_TRUNC | O_WRONLY, 0644)

/** open file for writing */
#define open_write(x)        open(x, O_NONBLOCK | O_WRONLY)

int fd_copy(int, int);
int fd_move(int, int);
int flock_ex(int);
int flock_exnb(int);
int flock_un(int);

/** @} */

#endif
