/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_FMT_H
#define SV_FMT_H

#include <stddef.h>
#include <unistd.h>

/**
 * @file fmt.h
 * @brief Formatting Routines
 * @defgroup sv_fmt Formatting Routines
 * @{
 */

#define FMT_IP4_MAX   15 /**< max size of an IP4 address */
#define FMT_IP6_MAX   39 /**< max size of an IP6 address */
#define FMT_ULONG_MAX 20 /**< max size of an unsigned long value */

size_t fmt_hex(char *, void *, size_t);
size_t fmt_hex_pad(char *, void *, size_t);
size_t fmt_ip4(char *, char *);
size_t fmt_ip6(char *, char *);
size_t fmt_str(char *, char *);
size_t fmt_str_len(char *, char *, size_t);
size_t fmt_ulong(char *, unsigned long);
size_t fmt_ulong_pad(char *, unsigned long, unsigned int);

/** @} */

#endif
