/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_HASH_H
#define SV_HASH_H

#include <stddef.h>
#include <stdint.h>

/**
 * @file hash.h
 * @brief Hash Routines
 * @defgroup sv_hash Hash Routines
 * @{
 */

/** djb hash context */
typedef struct hash_djb_ctx {
  uint32_t hash; /**< 32-bit djb hash */
} hash_djb_ctx_t;

/** djb64 hash context */
typedef struct hash_djb64_ctx {
  uint64_t hash; /**< 64-bit djb hash */
} hash_djb64_ctx_t;

#define hash_djb_init(x)     ((x)->hash = 5381) /**< Initialize a djb hash. */
#define hash_djb_finish(x)                      /**< Finalize a djb hash. */
#define hash_djb64_init(x)   ((x)->hash = 5381) /**< Initialize a djb64 hash. */
#define hash_djb64_finish(x)                    /**< Finalize a djb64 hash. */

void hash_djb(hash_djb_ctx_t *, void *, size_t);
void hash_djb_update(hash_djb_ctx_t *, void *, size_t);
void hash_djb64(hash_djb64_ctx_t *, void *, size_t);
void hash_djb64_update(hash_djb64_ctx_t *, void *, size_t);

/** @} */

#endif
