/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_STRERR_H
#define SV_STRERR_H

/**
 * @file strerr.h
 * @brief Strerr Routines
 * @defgroup sv_strerr Strerr Routines
 * @{
 */

/** strerr */
typedef struct strerr {
  struct strerr *who; /**< next in chain */
  const char    *x;   /**< first message */
  const char    *y;   /**< second message */
  const char    *z;   /**< third message */
} strerr_t;

extern strerr_t strerr_sys;

#define strerr(r,se,a) { se.who = 0; se.x = a; se.y = 0; se.z = 0; return r; } /**< strerr */
#define strerr_sys(r,se,a) { se.who = &strerr_sys; se.x = a; se.y = 0; se.z = 0; return r; } /**< strerr_sys */
#define strerr_sys3(r,se,a,b,c) { se.who = &strerr_sys; se.x = a; se.y = b; se.z = c; return r; } /**< strerr_sys3 */

#define strerr_die1(e,x1,se) strerr_die((e),(x1),0,0,0,0,0,0,0,(se)) /**< strerr_die1 */
#define strerr_die2(e,x1,x2,se) strerr_die((e),(x1),(x2),0,0,0,0,0,0,(se)) /**< strerr_die2 */
#define strerr_die3(e,x1,x2,x3,se) strerr_die((e),(x1),(x2),(x3),0,0,0,0,0,(se)) /**< strerr_die3 */
#define strerr_die4(e,x1,x2,x3,x4,se) strerr_die((e),(x1),(x2),(x3),(x4),0,0,0,0,(se)) /**< strerr_die4 */
#define strerr_die5(e,x1,x2,x3,x4,x5,se) strerr_die((e),(x1),(x2),(x3),(x4),(x5),0,0,0,(se)) /**< strerr_die5 */
#define strerr_die6(e,x1,x2,x3,x4,x5,x6,se) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),0,0,(se)) /**< strerr_die6 */
#define strerr_die7(e,x1,x2,x3,x4,x5,x6,x7,se) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),0,(se)) /**< strerr_die7 */
#define strerr_die8(e,x1,x2,x3,x4,x5,x6,x7,x8,se) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),(se)) /**< strerr_die8 */

#define strerr_die1sys(e,x1) strerr_die((e),(x1),0,0,0,0,0,0,0,&strerr_sys) /**< strerr_die1sys */
#define strerr_die2sys(e,x1,x2) strerr_die((e),(x1),(x2),0,0,0,0,0,0,&strerr_sys) /**< strerr_die2sys */
#define strerr_die3sys(e,x1,x2,x3) strerr_die((e),(x1),(x2),(x3),0,0,0,0,0,&strerr_sys) /**< strerr_die3sys */
#define strerr_die4sys(e,x1,x2,x3,x4) strerr_die((e),(x1),(x2),(x3),(x4),0,0,0,0,&strerr_sys) /**< strerr_die4sys */
#define strerr_die5sys(e,x1,x2,x3,x4,x5) strerr_die((e),(x1),(x2),(x3),(x4),(x5),0,0,0,&strerr_sys) /**< strerr_die5sys */
#define strerr_die6sys(e,x1,x2,x3,x4,x5,x6) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),0,0,&strerr_sys) /**< strerr_die6sys */
#define strerr_die7sys(e,x1,x2,x3,x4,x5,x6,x7) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),0,&strerr_sys) /**< strerr_die7sys */
#define strerr_die8sys(e,x1,x2,x3,x4,x5,x6,x7,x8) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),&strerr_sys) /**< strerr_die8sys */

#define strerr_die1x(e,x1) strerr_die((e),(x1),0,0,0,0,0,0,0,0) /**< strerr_die1x */
#define strerr_die2x(e,x1,x2) strerr_die((e),(x1),(x2),0,0,0,0,0,0,0) /**< strerr_die2x */
#define strerr_die3x(e,x1,x2,x3) strerr_die((e),(x1),(x2),(x3),0,0,0,0,0,0) /**< strerr_die3x */
#define strerr_die4x(e,x1,x2,x3,x4) strerr_die((e),(x1),(x2),(x3),(x4),0,0,0,0,0) /**< strerr_die4x */
#define strerr_die5x(e,x1,x2,x3,x4,x5) strerr_die((e),(x1),(x2),(x3),(x4),(x5),0,0,0,0) /**< strerr_die5x */
#define strerr_die6x(e,x1,x2,x3,x4,x5,x6) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),0,0,0) /**< strerr_die6x */
#define strerr_die7x(e,x1,x2,x3,x4,x5,x6,x7) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),0,0) /**< strerr_die7x */
#define strerr_die8x(e,x1,x2,x3,x4,x5,x6,x7,x8) strerr_die((e),(x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),0) /**< strerr_die8x */

#define strerr_warn1(x1,se) strerr_warn((x1),0,0,0,0,0,0,0,(se)) /**< strerr_warn1 */
#define strerr_warn2(x1,x2,se) strerr_warn((x1),(x2),0,0,0,0,0,0,(se)) /**< strerr_warn2 */
#define strerr_warn3(x1,x2,x3,se) strerr_warn((x1),(x2),(x3),0,0,0,0,0,(se)) /**< strerr_warn3 */
#define strerr_warn4(x1,x2,x3,x4,se) strerr_warn((x1),(x2),(x3),(x4),0,0,0,0,(se)) /**< strerr_warn4 */
#define strerr_warn5(x1,x2,x3,x4,x5,se) strerr_warn((x1),(x2),(x3),(x4),(x5),0,0,0,(se)) /**< strerr_warn5 */
#define strerr_warn6(x1,x2,x3,x4,x5,x6,se) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),0,0,(se)) /**< strerr_warn6 */
#define strerr_warn7(x1,x2,x3,x4,x5,x6,x7,se) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),0,(se)) /**< strerr_warn7 */
#define strerr_warn8(x1,x2,x3,x4,x5,x6,x7,x8,se) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),(se)) /**< strerr_warn8 */

#define strerr_warn1sys(x1) strerr_warn((x1),0,0,0,0,0,0,0,&strerr_sys) /**< strerr_warn1sys */
#define strerr_warn2sys(x1,x2) strerr_warn((x1),(x2),0,0,0,0,0,0,&strerr_sys) /**< strerr_warn2sys */
#define strerr_warn3sys(x1,x2,x3) strerr_warn((x1),(x2),(x3),0,0,0,0,0,&strerr_sys) /**< strerr_warn3sys */
#define strerr_warn4sys(x1,x2,x3,x4) strerr_warn((x1),(x2),(x3),(x4),0,0,0,0,&strerr_sys) /**< strerr_warn4sys */
#define strerr_warn5sys(x1,x2,x3,x4,x5) strerr_warn((x1),(x2),(x3),(x4),(x5),0,0,0,&strerr_sys) /**< strerr_warn5sys */
#define strerr_warn6sys(x1,x2,x3,x4,x5,x6) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),0,0,&strerr_sys) /**< strerr_warn6sys */
#define strerr_warn7sys(x1,x2,x3,x4,x5,x6,x7) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),0,&strerr_sys) /**< strerr_warn7sys */
#define strerr_warn8sys(x1,x2,x3,x4,x5,x6,x7,x8) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),&strerr_sys) /**< strerr_warn8sys */

#define strerr_warn1x(x1) strerr_warn((x1),0,0,0,0,0,0,0,0) /**< strerr_warn1x */
#define strerr_warn2x(x1,x2) strerr_warn((x1),(x2),0,0,0,0,0,0,0) /**< strerr_warn2x */
#define strerr_warn3x(x1,x2,x3) strerr_warn((x1),(x2),(x3),0,0,0,0,0,0) /**< strerr_warn3x */
#define strerr_warn4x(x1,x2,x3,x4) strerr_warn((x1),(x2),(x3),(x4),0,0,0,0,0) /**< strerr_warn4x */
#define strerr_warn5x(x1,x2,x3,x4,x5) strerr_warn((x1),(x2),(x3),(x4),(x5),0,0,0,0) /**< strerr_warn5x */
#define strerr_warn6x(x1,x2,x3,x4,x5,x6) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),0,0,0) /**< strerr_warn6x */
#define strerr_warn7x(x1,x2,x3,x4,x5,x6,x7) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),0,0) /**< strerr_warn7x */
#define strerr_warn8x(x1,x2,x3,x4,x5,x6,x7,x8) strerr_warn((x1),(x2),(x3),(x4),(x5),(x6),(x7),(x8),0) /**< strerr_warn8x */

void strerr_die(int, const char *, const char *, const char *, const char *,
                const char *, const char *, const char *, const char *,
                strerr_t *);
void strerr_sysinit();
void strerr_warn(const char *, const char *, const char *, const char *,
                 const char *, const char *, const char *, const char *,
                 strerr_t *);

/** @} */

#endif
