/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/array.h"
#include "sv/pathexec.h"

array_t _pathexec_plus = ARRAY_INIT(1); /**< internal variable */
array_t _pathexec_tmp = ARRAY_INIT(1);  /**< internal variable */

/**
 * Run command with environment variables set with pathexec_env.
 *
 * @param argv command
 */

void pathexec(char **argv) {
  char **envp;
  unsigned int elen;
  unsigned int i;
  unsigned int j;
  char *pdata;
  char *s;
  unsigned int split;
  unsigned int t;
  
  if (array_failed(&_pathexec_plus))
    return;
    
  pdata = array_start(&_pathexec_plus);
  
  elen = 0;
  for (i = 0; environ[i]; ++i)
    ++elen;
  for (i = 0; i < array_size(&_pathexec_plus); ++i)
    if (!pdata[i])
      ++elen;
      
  envp = malloc((elen + 1) * sizeof(char *));
  if (!envp)
    return;
    
  elen = 0;
  for (i = 0; environ[i]; ++i)
    envp[elen++] = environ[i];
    
  j = 0;
  for (i = 0; i < array_size(&_pathexec_plus); ++i) {
    if (!pdata[i]) {
      s = strchr(pdata + j, '=');
      split = s ? s - (pdata + j) : strlen(pdata + j);
      for (t = 0; t < elen; ++t) {
        if (!memcmp(pdata + j, envp[t], split)) {
          if (envp[t][split] == '=') {
            envp[t] = envp[--elen];
            break;
          }
        }
      }
      if (pdata[j + split])
        envp[elen++] = pdata + j;
      j = i + 1;
    }
  }
  envp[elen] = 0;
  
  pathexec_run(*argv, argv, envp);
  free(envp);
}
