/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/array.h"
#include "sv/pathexec.h"

/**
 * Run command.
 *
 * @param file command
 * @param argv parameters
 * @param envp environment
 */

void pathexec_run(char *file, char **argv, char **envp) {
  char *path;
  int savederrno;
  char *s;
  unsigned int split;
  
  s = strchr(file, '/');
  split = s ? s - file : strlen(file);
  if (file[split]) {
    execve(file, argv, envp);
    return;
  }
  
  path = getenv("PATH");
  if (!path || (strlen(path) == 0))
    path = "/bin:/usr/bin";
    
  savederrno = 0;
  for (;;) {
    s = strchr(path, ':');
    split = s ? s - path : 0;
    
    array_trunc(&_pathexec_tmp);
    array_append(&_pathexec_tmp, path, split);
    if (!split)
      array_append(&_pathexec_tmp, ".", 1);
    array_append(&_pathexec_tmp, "/", 1);
    array_append(&_pathexec_tmp, file, strlen(file));
    array_append_null(&_pathexec_tmp);
    
    if (!array_allocated(&_pathexec_tmp))
      return;
      
    execve(_pathexec_tmp.data, argv, envp);
    if (errno != ENOENT) {
      savederrno = errno;
      if ((errno != EACCES) && (errno != EPERM) && (errno != EISDIR))
        return;
    }
    
    if (!path[split]) {
      if (savederrno)
        errno = savederrno;
      return;
    }
    
    path += split;
    path += 1;
  }
}
