/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/random.h"
#include "sv/taia.h"

extern uint32_t _random_base[12];
extern uint32_t _random_seed[32];

/**
 * Initialize random number generator.
 *
 * @param data seed data
 */

void random_init(char data[128]) {
  int i;
  taia_t taia = TAIA_INIT;
  char pack[TAIA_PACK_SIZE];
  
  /* Initialize the seed using the passed in data. */
  for (i = 0; i < 32; i++)
    _random_seed[i] = be32toh((uint32_t *)(data + 4 * i));
    
  /* Add the current system time, pid, and parent pid to the base data. */
  taia_now(&taia);
  taia_pack(pack, &taia);
  for (i = 0; i < 4; i++)
    _random_base[i] = be32toh((uint32_t *)(pack + 4 * i));
  _random_base[8] = getpid();
  _random_base[9] = getppid();
}
