/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/random.h"

#define ROTATE(x,n) (((x) >> (n)) | ((x) << (32 - (n))))
#define MUSH(i,b) x = t[i] += (((x ^ _random_seed[i]) + sum) ^ ROTATE(x,b))

uint32_t _random_base[12];
uint32_t _random_seed[32];

/**
 * Random unsigned 32-bit value.
 */

uint32_t random_uint32() {
  static uint32_t data[8];
  static int pos = 0;
  
  int i;
  int loop;
  int r;
  uint32_t sum = 0;
  uint32_t t[12];
  uint32_t x;
  
  /* data is an array of unsigned 32-bit values which get's updated
   * once every 8 calls.  One value is returned each call and once all have
   * been used then the array is regenerated. */
  if (!pos) {
    if ((!++_random_base[0]) && (!++_random_base[1]) && (!++_random_base[2]))
      ++_random_base[3];
    for (i = 0; i < 12; ++i)
      t[i] = _random_base[i] ^ _random_seed[12 + i];
    for (i = 0; i < 8; ++i)
      data[i] = _random_seed[24 + i];
    x = t[11];
    
    for (loop = 0; loop < 2; loop++) {
      for (r = 0; r < 16; r++) {
        sum += 0x9e3779b9;
        MUSH(0, 5);
        MUSH(1, 7);
        MUSH(2, 9);
        MUSH(3, 13);
        MUSH(4, 5);
        MUSH(5, 7);
        MUSH(6, 9);
        MUSH(7, 13);
        MUSH(8, 5);
        MUSH(9, 7);
        MUSH(10, 9);
        MUSH(11, 13);
      }
      for (i = 0; i < 8; i++)
        data[i] ^= t[i + 4];
    }
    pos = 8;
  }
  
  return data[--pos];
}
