/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/bio.h"
#include "sv/sgetopt.h"

char *soptarg = 0;      /**< argument */
char *soptprogname = 0; /**< program name */
int sopterr = 1;        /**< error */
int soptind = 1;        /**< index */
int soptopt = 0;        /**< option */
int soptpos = 0;        /**< position */
int soptproblem = 0;    /**< problem */

static int subgetopt(int argc, char *const *argv, const char *opts) {
  int c;
  const char *s;
  
  soptarg = 0;
  if (!argv || (soptind >= argc) || !argv[soptind])
    return -1;
  if (soptpos && !argv[soptind][soptpos]) {
    ++soptind;
    soptpos = 0;
    if ((soptind >= argc) || !argv[soptind])
      return -1;
  }
  if (!soptpos) {
    if (argv[soptind][0] != '-')
      return -1;
    ++soptpos;
    c = argv[soptind][1];
    if ((c == '-') || (c == 0)) {
      if (c)
        ++soptind;
      soptpos = 0;
      return -1;
    }
  }
  c = argv[soptind][soptpos];
  ++soptpos;
  s = opts;
  while (*s) {
    if (c == *s) {
      if (s[1] == ':') {
        soptarg = argv[soptind] + soptpos;
        ++soptind;
        soptpos = 0;
        if (!*soptarg) {
          soptarg = argv[soptind];
          if ((soptind >= argc) || !soptarg) {
            soptproblem = c;
            return '?';
          }
          ++soptind;
        }
      }
      return c;
    }
    ++s;
    if (*s == ':')
      ++s;
  }
  soptproblem = c;
  return '?';
}

/**
 * getopt replacement.
 *
 * @param argc argc
 * @param argv argv
 * @param opts opts
 */

int sgetopt(int argc, char *const *argv, const char *opts) {
  int c;
  char *s;
  
  if (!soptprogname) {
    soptprogname = *argv;
    if (!soptprogname)
      soptprogname = "";
    for (s = soptprogname; *s; ++s)
      if (*s == '/')
        soptprogname = s + 1;
  }
  
  c = subgetopt(argc, argv, opts);
  
  if (sopterr)
    if (c == '?') {
      char chp[2];
      chp[0] = soptproblem;
      chp[1] = '\n';
      bio_put_str(bio_2, soptprogname);
      if (argv[soptind] && (soptind < argc))
        bio_put_str(bio_2, ": illegal option -- ");
      else
        bio_put_str(bio_2, ": option requires an argument -- ");
      bio_put(bio_2, chp, 2);
      bio_flush(bio_2);
    }
    
  return c;
}
