/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common-private.h"
#include "sv/bio.h"
#include "sv/strerr.h"

/**
 * Print a warning to stderr.
 *
 * @param x1 string
 * @param x2 string
 * @param x3 string
 * @param x4 string
 * @param x5 string
 * @param x6 string
 * @param x7 string
 * @param x8 string
 * @param se strerr_t
 */

void strerr_warn(const char *x1, const char *x2, const char *x3,
                 const char *x4, const char *x5, const char *x6,
                 const char *x7, const char *x8, strerr_t *se) {
  strerr_sysinit();
  
  if (x1)
    bio_put(bio_2, x1, strlen(x1));
  if (x2)
    bio_put(bio_2, x2, strlen(x2));
  if (x3)
    bio_put(bio_2, x3, strlen(x3));
  if (x4)
    bio_put(bio_2, x4, strlen(x4));
  if (x5)
    bio_put(bio_2, x5, strlen(x5));
  if (x6)
    bio_put(bio_2, x6, strlen(x6));
  if (x7)
    bio_put(bio_2, x7, strlen(x7));
  if (x8)
    bio_put(bio_2, x8, strlen(x8));
    
  while (se) {
    if (se->x)
      bio_put(bio_2, se->x, strlen(se->x));
    if (se->y)
      bio_put(bio_2, se->y, strlen(se->y));
    if (se->z)
      bio_put(bio_2, se->z, strlen(se->z));
      
    se = se->who;
  }
  
  bio_put(bio_2, "\n", strlen("\n"));
  bio_flush(bio_2);
}
