/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common.h"

/* Set 3 different cache entries and then retreive them and verify the data
 * is correct. */
int main() {
  cache_t cache = CACHE_INIT;
  char *data;
  char *data1 = "data1";
  char *data2 = "data2";
  char *data3 = "data3";
  size_t dlen;
  char *key1 = "key1";
  char *key2 = "key2";
  char *key3 = "key3";
  uint32_t ttl;
  
  if (!cache_init(&cache, 512))
    return 1;
    
  cache_set(&cache, key1, strlen(key1), data1, strlen(data1), 2);
  cache_set(&cache, key2, strlen(key2), data2, strlen(data2), CACHE_MAX_TTL);
  cache_set(&cache, key3, strlen(key3), data3, strlen(data3), CACHE_MAX_TTL);
  
  data = cache_get(&cache, key1, strlen(key1), &dlen, &ttl);
  if (!data)
    return 2;
  if (memcmp(data1, data, dlen))
    return 3;
    
  data = cache_get(&cache, key2, strlen(key2), &dlen, &ttl);
  if (!data)
    return 4;
  if (memcmp(data2, data, dlen))
    return 5;
    
  data = cache_get(&cache, key3, strlen(key3), &dlen, &ttl);
  if (!data)
    return 6;
  if (memcmp(data3, data, dlen))
    return 7;
    
  cache_free(&cache);
  
  return 0;
}
