/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common.h"

/* Do a number of sets and gets on different sized caches to make sure no
 * errors occur. Also print the cache size and the number of hits in the
 * cache. */
int main() {
  cache_t cache = CACHE_INIT;
  char data[FMT_ULONG_MAX + 5];
  size_t dlen;
  unsigned int i;
  char ul_str[FMT_ULONG_MAX + 1];
  unsigned int j;
  char key[FMT_ULONG_MAX + 4];
  size_t klen;
  size_t size = 16;
  uint32_t ttl;
  
  fmt_str(data, "data");
  fmt_str(key, "key");
  
  for (i = 0; i < 16; i++) {
    size <<= 1;
    
    if (!cache_init(&cache, size))
      return 1;
      
    for (j = 1; j < size; j++) {
      dlen = fmt_ulong(data + 4, j) + 4;
      klen = fmt_ulong(key + 3, j) + 3;
      cache_set(&cache, key, klen, data, dlen, CACHE_MAX_TTL);
    }
    
    for (j = 1; j < size; j++) {
      klen = fmt_ulong(key + 3, j) + 3;
      cache_get(&cache, key, klen, &dlen, &ttl);
    }
    
    cache_free(&cache);
  }
  
  return 0;
}
