/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/common.h"

void sigalrm_handler() {
}

/* Set an entry with a short ttl, check that it exists in the cache, sleep for
 * a few seconds and attempt to get the same entry which should fail. */
int main() {
  cache_t cache = CACHE_INIT;
  char *data;
  char *data1 = "data1";
  size_t dlen;
  char *key1 = "key1";
  uint32_t ttl;
  
  sig_catch(SIGALRM, sigalrm_handler);
  
  if (!cache_init(&cache, 512))
    return 1;
    
  /* Set an entry with a TTL of 2 seconds. */
  cache_set(&cache, key1, strlen(key1), data1, strlen(data1), 2);
  
  /* Verify the entry is in the cache. */
  data = cache_get(&cache, key1, strlen(key1), &dlen, &ttl);
  if (!data)
    return 2;
  if (ttl > 2)
    return 3;
    
  /* Instruct a SIGALRM to be sent in 3 seconds and wait for a signal. */
  alarm(3);
  sig_pause();
  
  /* Attempt to get the cache entry which should fail due to timing out. */
  data = cache_get(&cache, key1, strlen(key1), &dlen, &ttl);
  if (data)
    return 4;
    
  cache_free(&cache);
  
  return 0;
}
