/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <pthread.h>
#include "sv/common.h"

#define THREADS 16
#define LOOPS   1000000

spinlock_t lock;
int a = 0;
int b = 0;
int c = 0;

void *thread_main(void *arg) {
  int i;
  
  for (i = 0; i < LOOPS; i++) {
    spinlock_acquire(&lock);
    a++;
    b++;
    
    b++;
    c++;
    
    a++;
    c++;
    spinlock_unlock(&lock);
  }
  
  return 0;
}

int main() {
  int i;
  int *j;
  int x;
  pthread_t threads[THREADS];
  int thread_args[THREADS];
  
  spinlock_init(&lock);
  
  for (i = THREADS - 1; i >= 0; i--) {
    thread_args[i] = i;
    pthread_create(&threads[i], NULL, thread_main, &thread_args[i]);
  }
  
  for (i = 0; i < THREADS; i++) {
    pthread_join(threads[i], NULL);
  }
  
  if (a != THREADS * LOOPS * 2)
    return 1;
  if ((a != b) || (b != c))
    return 2;
    
  return 0;
}
