/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-cpp.h"

namespace sv {
  namespace time {
    /* Constructors */
    
    Tai::Tai() {
      tai_init(&data, 0);
    }
    
    Tai::Tai(uint64_t sec = 0) {
      tai_init(&data, sec);
    }
    
    Tai::Tai(const Tai &s) {
      tai_init(&data, s.data.sec);
    }
    
    /* Operators */
    
    Tai Tai::operator + (Tai &a) {
      tai_t t;
      tai_add(&t, &data, &a.data);
      return Tai(t.sec);
    }
    
    Tai Tai::operator - (Tai &a) {
      tai_t t;
      tai_sub(&t, &data, &a.data);
      return Tai(t.sec);
    }
    
    bool Tai::operator == (Tai &a) {
      return (tai_cmp(&data, &a.data) == 0) ? true : false;
    }
    
    bool Tai::operator != (Tai &a) {
      return (tai_cmp(&data, &a.data) != 0) ? true : false;
    }
    
    /* Methods */
    
    uint64_t Tai::getSeconds() {
      return data.sec;
    }
  }
}
