/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <string.h>
#include "sv/time-jni.h"

JNIEXPORT void JNICALL Java_sv_time_Taia_decode(JNIEnv *env, jobject taia, jstring s) {
  jchar *data;
  jcharArray dataArray;
  jfieldID dataFid;
  jclass exception;
  const jbyte *str;
  jclass taiaClass;
  
  /* Obtain the taia objects data field which is a packed taia value. */
  taiaClass = (*env)->GetObjectClass(env, taia);
  dataFid = (*env)->GetFieldID(env, taiaClass, "data", "[C");
  dataArray = (*env)->GetObjectField(env, taia, dataFid);
  data = (*env)->GetCharArrayElements(env, dataArray, JNI_FALSE);
  
  str = (*env)->GetStringUTFChars(env, s, NULL);
  switch (strlen(str)) {
    case TAI64N_ENCODE_SIZE:
      tai64n_decode((char *)data, (char *)str);
      break;
    case TAI64NA_ENCODE_SIZE:
      tai64na_decode((char *)data, (char *)str);
      break;
    default:
      exception = (*env)->FindClass(env, TAIA_EXCEPTION_CLASS_NAME);
      (*env)->ThrowNew(env, exception, "invalid encoded taia value");
      return;
  }
  (*env)->ReleaseStringUTFChars(env, s, str);
  
  (*env)->ReleaseCharArrayElements(env, dataArray, data, 0);
}
