/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <Python.h>
#include "sv/time-python.h"

static PyMethodDef SvtimeModuleMethods[] = {
  { "TaiNow",  TaiNow,  METH_VARARGS, "Get the current system TAI time." },
  { "TaiaNow", TaiaNow, METH_VARARGS, "Get the current system TAIA time." },
  { NULL }
};

PyMODINIT_FUNC initsvtime() {
  PyObject *module;
  
  /* Initialize the module. */
  module = Py_InitModule("svtime", SvtimeModuleMethods);
  if (!module) {
    PyErr_SetString(PyExc_ImportError, "Py_InitModule(svtime) failed.");
    return;
  }
  
  /* Initialize the Tai type. */
  PyType_Ready(&TaiType);
  Py_INCREF(&TaiType);
  PyModule_AddObject(module, "Tai", (PyObject *)&TaiType);
  
  /* Initialize the Taia type. */
  PyType_Ready(&TaiaType);
  Py_INCREF(&TaiaType);
  PyModule_AddObject(module, "Taia", (PyObject *)&TaiaType);
}
