/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-private.h"
#include "sv/caldate.h"

/**
 * Format a caldate struct into a string.
 *
 * @param s  string byte array
 * @param cd caldate struct
 *
 * @remarks If s is null then the return value is the number of characters
 *          needed to convert the caldate struct into a string.
 */

unsigned int caldate_fmt(char *s, caldate_t *cd) {
  long x;
  int i = 0;
  
  /* Store the year into the temporary x variable and abs it. */
  x = cd->year;
  if (x < 0)
    x = -x;
    
  /* Determine the number of characters needed to convert the year into a
   * string. */
  do {
    i++;
    x /= 10;
  } while (x);
  
  if (s) {
    /* Set x to the year again to overwrite the previous do loop determining
     * the number of characters needed. */
    x = cd->year;
    
    /* If the year is negative then abs(x) and write out a '-'. */
    if (x < 0) {
      x = -x;
      *s++ = '-';
    }
    
    /* Increment the string pointer by the number of characters needed to
     * write out the year since it's going to be done in reverse, write out
     * the year, and then update the pointer again to skip to the next unused
     * position. */
    s += i;
    do {
      *--s = '0' + (x % 10);
      x /= 10;
    } while (x);
    s += i;
    
    /* Write out the two digit month. */
    x = cd->month;
    s[0] = '-';
    s[2] = '0' + (x % 10);
    x /= 10;
    s[1] = '0' + (x % 10);
    
    /* Write out the two digit day. */
    x = cd->day;
    s[3] = '-';
    s[5] = '0' + (x % 10);
    x /= 10;
    s[4] = '0' + (x % 10);
  }
  
  return (cd->year < 0) + i + 6;
}
