/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-private.h"
#include "sv/caldate.h"

void caldate_frommjd(caldate_t *cd, long day, int *pwday, int *pyday) {
  long year;
  long month;
  int yday;
  
  year = day / 146097L;
  day %= 146097L;
  day += 678881L;
  while (day >= 146097L) {
    day -= 146097L;
    ++year;
  }
  
  /* year * 146097 + day - 678881 is MJD; 0 <= day < 146097 */
  /* 2000-03-01, MJD 51604, is year 5, day 0 */
  
  if (pwday)
    *pwday = (day + 3) % 7;
    
  year *= 4;
  if (day == 146096L) {
    year += 3;
    day = 36524L;
  } else {
    year += day / 36524L;
    day %= 36524L;
  }
  
  year *= 25;
  year += day / 1461;
  day %= 1461;
  year *= 4;
  
  yday = (day < 306);
  if (day == 1460) {
    year += 3;
    day = 365;
  } else {
    year += day / 365;
    day %= 365;
  }
  yday += day;
  
  day *= 10;
  month = (day + 5) / 306;
  day = (day + 5) % 306;
  day /= 10;
  
  if (month >= 10) {
    yday -= 306;
    ++year;
    month -= 10;
  } else {
    yday += 59;
    month += 2;
  }
  
  cd->year = year;
  cd->month = month + 1;
  cd->day = day + 1;
  
  if (pyday)
    *pyday = yday;
}
