/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-private.h"
#include "sv/caldate.h"

static unsigned long times365[4] =   { 0, 365, 730, 1095 };
static unsigned long times36524[4] = { 0, 36524UL, 73048UL, 109572UL };
static unsigned long montab[12] =    { 0, 31, 61, 92, 122, 153, 184, 214, 245,
                                       275, 306, 337
                                     };

/**
 * Return the modified julian day number corresponding to a caldate.
 *
 * @param cd caldate struct
 * @return mjd day number
 */

long caldate_mjd(caldate_t *cd) {
  long y;
  long m;
  long d;
  
  d = cd->day - 678882L;
  m = cd->month - 1;
  y = cd->year;
  
  d += 146097L * (y / 400);
  y %= 400;
  
  if (m >= 2)
    m -= 2;
  else {
    m += 10;
    --y;
  }
  
  y += (m / 12);
  m %= 12;
  if (m < 0) {
    m += 12;
    --y;
  }
  
  d += montab[m];
  
  d += 146097L * (y / 400);
  y %= 400;
  if (y < 0) {
    y += 400;
    d -= 146097L;
  }
  
  d += times365[y & 3];
  y >>= 2;
  
  d += 1461L * (y % 25);
  y /= 25;
  
  d += times36524[y & 3];
  
  return d;
}
