/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-private.h"
#include "sv/caldate.h"

/**
 * Scan a string in caldate format into a caldate struct.
 *
 * @param cd caldate struct
 * @param s  string
 *
 * @return number of characters scanned
 * @retval 0 invalid caldate string
 */

unsigned int caldate_scan(caldate_t *cd, char *s) {
  unsigned long c;
  int sign = 1;
  char *t = s;
  unsigned long z;
  
  if (*t == '-') {
    ++t;
    sign = -1;
  }
  
  /* Scan the year component. */
  z = 0;
  while ((c = (unsigned char) (*t - '0')) <= 9) {
    z = z * 10 + c;
    ++t;
  }
  cd->year = z * sign;
  
  /* Skip the dash if it exists or return if it doesn't. */
  if (*t++ != '-')
    return 0;
    
  /* Scan the month component */
  z = 0;
  while ((c = (unsigned char) (*t - '0')) <= 9) {
    z = z * 10 + c;
    ++t;
  }
  cd->month = z;
  
  /* Skip the dash if it exists or return if it doesn't. */
  if (*t++ != '-')
    return 0;
    
  /* Scan the day component. */
  z = 0;
  while ((c = (unsigned char) (*t - '0')) <= 9) {
    z = z * 10 + c;
    ++t;
  }
  cd->day = z;
  
  return t - s;
}
