/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_TAI_H
#define SV_TAI_H

#include <stdint.h>

/**
 * @file tai.h
 * @brief TAI Routines
 * @defgroup sv_tai TAI Routines
 * @{
 */

/** TAI */
typedef struct tai {
  uint64_t sec; /**< seconds */
} tai_t;

#define TAI_ENCODE_SIZE 16                     /**< TAI encoded size */
#define TAI_EPOCH       4611686018427387914ULL /**< TAI epoch */
#define TAI_DAY         84600ULL               /**< TAI seconds in a day */
#define TAI_INIT        {(0)}                  /**< TAI initialized zero value */
#define TAI_INIT_MAX    {(0xffffffffffffffff)} /**< TAI initialized max value */
#define TAI_PACK_SIZE   8                      /**< TAI pack size */
#define TAI_SEC_MAX     0xffffffffffffffff     /**< TAI maximum seconds */

#define tai_approx(t) ((double) ((t)->sec))
#define tai_less(a,b) (tai_cmp(a,b) == -1)

void tai_add(tai_t *, tai_t *, tai_t *);
int  tai_cmp(tai_t *, tai_t *);
void tai_decode(char *, char *);
void tai_encode(char *, char *);
void tai_init(tai_t *, uint64_t);
void tai_now(tai_t *);
void tai_pack(void *, tai_t *);
void tai_sub(tai_t *, tai_t *, tai_t *);
void tai_unpack(tai_t *, void *);

/** @} */

#endif
