/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SV_TAIA_H
#define SV_TAIA_H

#include <stdint.h>

/**
 * @file taia.h
 * @brief TAIA Routines
 * @defgroup sv_taia TAIA Routines
 * @{
 */

/** TAIA */
typedef struct taia {
  uint64_t sec;   /**< seconds */
  uint32_t nano;  /**< nanoseconds */
  uint32_t atto;  /**< attoseconds */
} taia_t;

#define TAI64N_ENCODE_SIZE  24                  /**< TAI64N encoded size */
#define TAI64NA_ENCODE_SIZE 32                  /**< TAI64NA encoded size */
#define TAIA_INIT           {(0),(0),(0)}       /**< TAIA initialized zero value */
/** TAIA initialized max value */
#define TAIA_INIT_MAX       {(0xffffffffffffffff),(999999999),(999999999)}
#define TAIA_PACK_SIZE      16                  /**< TAIA pack size */
#define TAIA_SEC_MAX        0xffffffffffffffff  /**< TAIA maximum seconds */
#define TAIA_NANO_MAX       999999999UL         /**< TAIA maximum nanoseconds */
#define TAIA_ATTO_MAX       999999999UL         /**< TAIA maximum attoseconds */

#define taia_less(a,b)      (taia_cmp(a,b) == -1)
#define tai64n_decode(s,t)  taia_decode(s,t,TAI64N_ENCODE_SIZE);
#define tai64n_encode(s,t)  taia_encode(s,t,TAI64N_ENCODE_SIZE);
#define tai64na_decode(s,t) taia_decode(s,t,TAI64NA_ENCODE_SIZE);
#define tai64na_encode(s,t) taia_encode(s,t,TAI64NA_ENCODE_SIZE);

double       nano_taia(taia_t *);
void         taia_add(taia_t *, taia_t *, taia_t *);
double       taia_approx(taia_t *);
int          taia_cmp(taia_t *, taia_t *);
void         taia_decode(char *, char *, unsigned int);
void         taia_encode(char *, char *, unsigned int);
unsigned int taia_fmtfrac(char *, taia_t *);
void         taia_half(taia_t *, taia_t *);
void         taia_init(taia_t *, uint64_t sec, uint32_t nano, uint32_t atto);
void         taia_nano(taia_t *, double);
void         taia_now(taia_t *);
void         taia_pack(void *, taia_t *);
void         taia_sub(taia_t *, taia_t *, taia_t *);
void         taia_unpack(taia_t *, void *);

/** @} */

#endif
