/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv/time-private.h"
#include "sv/leapsecs.h"
#include "sv/tai.h"

tai_t *leapsecs = 0;
int leapsecs_num = 0;

int leapsecs_read() {
  int fd;
  struct stat st;
  tai_t *t;
  int n;
  int i;
  tai_t u;
  
  fd = open(PATH_LEAPSECS_DAT, O_RDONLY | O_NONBLOCK);
  if (fd == -1) {
    if (errno != ENOENT)
      return -1;
      
    if (leapsecs)
      free(leapsecs);
      
    leapsecs = 0;
    leapsecs_num = 0;
    
    return 0;
  }
  
  if (fstat(fd, &st) == -1) {
    close(fd);
    return -1;
  }
  
  t = (tai_t *) malloc(st.st_size);
  if (!t) {
    close(fd);
    return -1;
  }
  
  n = read(fd, (char *) t, st.st_size);
  close(fd);
  if (n != st.st_size) {
    free(t);
    return -1;
  }
  
  n /= sizeof(tai_t);
  
  for (i = 0; i < n; ++i) {
    tai_unpack(&u, (char *) &t[i]);
    t[i] = u;
  }
  
  if (leapsecs)
    free(leapsecs);
    
  leapsecs = t;
  leapsecs_num = n;
}
