/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <jni.h>
#include "sv.h"
#include "sv-jni.h"

JNIEXPORT void JNICALL Java_sv_Service_control(JNIEnv *env, jobject service,
    jstring commandStr) {
  const char *command;
  jsize command_len;
  const char *path;
  jstring pathStr;
  int r;
  
  /* Get the service object's path field. */
  jclass serviceClass = (*env)->GetObjectClass(env, service);
  jfieldID pathFid = (*env)->GetFieldID(env, serviceClass, "path",
                                        "Ljava/lang/String;");
  pathStr = (*env)->GetObjectField(env, service, pathFid);
  
  /* Convert the java strings to byte arrays and get the length of command. */
  command = (*env)->GetStringUTFChars(env, commandStr, 0);
  command_len = (*env)->GetStringUTFLength(env, commandStr);
  path = (*env)->GetStringUTFChars(env, pathStr, 0);
  
  r = sv_control((char *)path, (char *)command, command_len);
  if (r) {
    jclass exception = (*env)->FindClass(env, SERVICE_EXCEPTION_CLASS_NAME);
    (*env)->ThrowNew(env, exception, sv_error_str(r));
  }
  
  /* Release the java string byte arrays. */
  (*env)->ReleaseStringUTFChars(env, pathStr, path);
  (*env)->ReleaseStringUTFChars(env, commandStr, command);
}
