/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <jni.h>
#include "sv.h"
#include "sv-jni.h"

JNIEXPORT jobject JNICALL Java_sv_Service_getStatus(JNIEnv *env, jobject service) {
  char *path;
  jfieldID pathFid;
  jstring pathStr;
  int r;
  jobject mode;
  jclass modeClass;
  jfieldID modeFid;
  jclass serviceClass;
  jobject status;
  jclass statusClass;
  jmethodID statusInitMid;
  jmethodID statusSetModeMid;
  jmethodID statusSetPausedMid;
  jmethodID statusSetPidMid;
  jmethodID statusSetTimestampMid;
  sv_stat_t svst;
  jobject taia;
  jclass taiaClass;
  jmethodID taiaInitMid;
  
  /* Get the service object's path field. */
  serviceClass = (*env)->GetObjectClass(env, service);
  pathFid = (*env)->GetFieldID(env, serviceClass, "path", "Ljava/lang/String;");
  pathStr = (*env)->GetObjectField(env, service, pathFid);
  
  /* stat the service before going through the process of creating the various
   * java objects. */
  path = (*env)->GetStringUTFChars(env, pathStr, 0);
  r = sv_stat(&svst, path);
  (*env)->ReleaseStringUTFChars(env, pathStr, path);
  
  /* Throw a ServiceException if the stat failed with a useful message. */
  if (r) {
    jclass exception = (*env)->FindClass(env, SERVICE_EXCEPTION_CLASS_NAME);
    (*env)->ThrowNew(env, exception, sv_error_str(r));
    return 0;
  }
  
  /* Create a new ServiceStatus object and fill it with the svstat info. */
  statusClass = (*env)->FindClass(env, SERVICE_STATUS_CLASS_NAME);
  statusInitMid = (*env)->GetMethodID(env, statusClass, "<init>", "()V");
  status = (*env)->NewObject(env, statusClass, statusInitMid);
  
  /* Create a new Taia object and fill it with the svstat timestamp. */
  taiaClass = (*env)->FindClass(env, "sv/time/Taia");
  taiaInitMid = (*env)->GetMethodID(env, taiaClass, "<init>", "(JII)V");
  taia = (*env)->NewObject(env, taiaClass, taiaInitMid, svst.timestamp.sec,
                           svst.timestamp.nano, svst.timestamp.atto);
                           
  modeClass = (*env)->FindClass(env, STATUS_MODE_CLASS_NAME);
  modeFid = (*env)->GetStaticFieldID(env, modeClass, MODE_ENUM_NAME(svst.mode),
                                     "L" STATUS_MODE_CLASS_NAME ";");
  mode = (*env)->GetStaticObjectField(env, modeClass, modeFid);
  
  statusSetModeMid = (*env)->GetMethodID(env, statusClass, "setMode",
                                         "(L" STATUS_MODE_CLASS_NAME ";)V");
  (*env)->CallVoidMethod(env, status, statusSetModeMid, mode);
  
  statusSetPausedMid = (*env)->GetMethodID(env, statusClass, "setPaused", "(Z)V");
  (*env)->CallVoidMethod(env, status, statusSetPausedMid, svst.paused);
  
  statusSetPidMid = (*env)->GetMethodID(env, statusClass, "setPid", "(I)V");
  (*env)->CallVoidMethod(env, status, statusSetPidMid, svst.pid);
  
  statusSetTimestampMid = (*env)->GetMethodID(env, statusClass, "setTimestamp", "(Lsv/time/Taia;)V");
  (*env)->CallVoidMethod(env, status, statusSetTimestampMid, taia);
  
  return status;
}
