/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <Python.h>
#include <sv/time-python.h>
#include "sv-python.h"

PyObject *ServiceException;

static PyMethodDef SvModuleMethods[] = {
  { NULL }
};

/**
 * Initialize the sv module.
 */

PyMODINIT_FUNC initsv() {
  PyObject *module;
  
  /* Initialize the module. */
  module = Py_InitModule("sv", SvModuleMethods);
  if (!module) {
    PyErr_SetString(PyExc_ImportError, "Py_InitModule(sv) failed.");
    return;
  }
  
  if (PyImport_ImportModule("svtime") == NULL) {
    PyErr_SetString(PyExc_ImportError, "PyImport_ImportModule(svtime) failed.");
    return;
  }
  
  /* Initialize the Service type. */
  PyType_Ready(&ServiceType);
  Py_INCREF(&ServiceType);
  PyModule_AddObject(module, "Service", (PyObject *)&ServiceType);
  
  /* Initialize the ServiceStatus type. */
  PyType_Ready(&ServiceStatusType);
  Py_INCREF(&ServiceStatusType);
  PyModule_AddObject(module, "ServiceStatus", (PyObject *)&ServiceStatusType);
  
  /* Create ServiceException. */
  ServiceException = PyErr_NewException("sv.ServiceException", NULL, NULL);
  PyModule_AddObject(module, "ServiceException", ServiceException);
}
