/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <Python.h>
#include <structmember.h>
#include "sv-python.h"

static PyGetSetDef ServiceType_GetSet[] = {
  { "status", ServiceType_GetStatus, 0, "service status" },
  { NULL }
};

static PyMemberDef ServiceType_Members[] = {
  { "path", T_OBJECT_EX, offsetof(ServiceObject, path), 0, "service path" },
  { NULL }
};

static PyMethodDef ServiceType_Methods[] = {
  { "control", ServiceType_Control, METH_VARARGS, "Send control messages to the service." },
  { "down",    ServiceType_Down,    METH_NOARGS,  "Send down message to service." },
  { "exit",    ServiceType_Exit,    METH_NOARGS,  "Send exit message to service." },
  { "sigalrm", ServiceType_Sigalrm, METH_NOARGS,  "Send sigalrm message to service." },
  { "sigcont", ServiceType_Sigcont, METH_NOARGS,  "Send sigcont message to service." },
  { "sighup",  ServiceType_Sighup,  METH_NOARGS,  "Send sighup message to service." },
  { "sigint",  ServiceType_Sigint,  METH_NOARGS,  "Send sigint message to service." },
  { "sigkill", ServiceType_Sigkill, METH_NOARGS,  "Send sigkill message to service." },
  { "sigstop", ServiceType_Sigstop, METH_NOARGS,  "Send sigstop message to service." },
  { "sigterm", ServiceType_Sigterm, METH_NOARGS,  "Send sigterm message to service." },
  { "up",      ServiceType_Up,      METH_NOARGS,  "Send up message to service." },
  { "up_once", ServiceType_UpOnce,  METH_NOARGS,  "Send up once message to service." },
  { NULL }
};

PyTypeObject ServiceType = {
  PyObject_HEAD_INIT(NULL)
  0,                       /* ob_size */
  "sv.Service",            /* tp_name */
  sizeof(ServiceObject),   /* tp_basicsize */
  0,                       /* tp_itemsize */
  ServiceType_Dealloc,     /* tp_dealloc */
  0,                       /* tp_print */
  0,                       /* tp_getattr */
  0,                       /* tp_setattr */
  0,                       /* tp_compare */
  0,                       /* tp_repr */
  0,                       /* tp_as_number */
  0,                       /* tp_as_sequence */
  0,                       /* tp_as_mapping */
  0,                       /* tp_hash */
  0,                       /* tp_call */
  0,                       /* tp_str */
  0,                       /* tp_getattro */
  0,                       /* tp_setattro */
  0,                       /* tp_as_buffer */
  Py_TPFLAGS_DEFAULT,      /* tp_flags */
  "Service object",        /* tp_doc */
  0,                       /* tp_traverse */
  0,                       /* tp_clear */
  0,                       /* tp_richcompare */
  0,                       /* tp_weaklistoffset */
  0,                       /* tp_iter */
  0,                       /* tp_iterext */
  ServiceType_Methods,     /* tp_methods */
  ServiceType_Members,     /* tp_members */
  ServiceType_GetSet,      /* tp_getset */
  0,                       /* tp_base */
  0,                       /* tp_dict */
  0,                       /* tp_descr_get */
  0,                       /* tp_descr_set */
  0,                       /* tp_dictoffset */
  ServiceType_Init,        /* tp_init */
  0,                       /* tp_alloc */
  ServiceType_New,         /* tp_new */
  0,                       /* tp_free */
  0,                       /* tp_is_gc */
  0,                       /* tp_bases */
  0,                       /* tp_mro */
  0,                       /* tp_cache */
  0,                       /* tp_subclasses */
  0                        /* tp_weaklist */
};
