/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <Python.h>
#include <errno.h>
#include <string.h>
#include <sv.h>
#include <sv/time-python.h>
#include "sv-python.h"

/**
 * Create a new Taia object and fill its data with the ServiceStatus'
 * timestamp.
 */

PyObject *ServiceStatusType_GetTimestamp(PyObject *self, void *closure) {
  PyObject *dict;
  PyObject *module;
  ServiceStatusObject *status = (ServiceStatusObject *)self;
  TaiaObject *taia;
  PyTypeObject *type;
  
  /* In order to create an object from another module it must first be
   * imported, dictionary obtained, and the object type pulled from it and
   * used in PyObject_New. */
  module = PyImport_ImportModule("svtime");
  dict = PyModule_GetDict(module);
  type = (PyTypeObject *)PyDict_GetItemString(dict, "Taia");
  
  taia = (TaiaObject *)PyObject_New(TaiaObject, type);
  memcpy(&taia->t, &status->svst.timestamp, sizeof(taia->t));
  
  Py_INCREF(taia);
  return (PyObject *)taia;
}
