/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv-private.h"
#include "sv.h"

/**
 * Sends commands to the supervise/control fifo.
 *
 * @param path        service path
 * @param command     command string
 * @param command_len command string length
 *
 * @retval 0  success
 * @retval -1 failure
 * @retval ESVOPENCONTROL error opening supervise/control fifo.
 * @retval ESVWRITECONTROL error writing to supervise/control fifo.
 */

int sv_control(char *path, char *command, int command_len) {
  int control_fd;
  array_t full_path = ARRAY_INIT(1);
  int r;
  
  /* Construct the full path into string array. */
  array_append(&full_path, path, strlen(path));
  array_append(&full_path, "/", 1);
  array_append(&full_path, SUPERVISE_CONTROL_PATH, strlen(SUPERVISE_CONTROL_PATH));
  array_append_null(&full_path);
  if (array_failed(&full_path)) {
    errno = ENOMEM;
    return -1;
  }
  
  /* Open the supervise control fifo in non-blocking mode and write the
   * commands to it. */
  control_fd = open_write(array_start(&full_path));
  array_reset(&full_path);
  
  if (control_fd == -1)
    return ESVOPENCONTROL;
    
  blocking_disable(control_fd);
  r = write(control_fd, command, command_len);
  close(control_fd);
  
  if (r == -1)
    return ESVWRITECONTROL;
    
  return 0;
}
