/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv-private.h"
#include "sv.h"

/**
 * Return a string describing the error.  If err is -1 then it will use errno
 * for the error and lookup the string associated with it.
 */

const char *sv_error_str(int err) {
  switch (err) {
    case -1:
      return strerror(errno);
    case ESVSTATDOWN:
      return ESVSTATDOWN_MSG;
    case ESVNOTRUN:
      return ESVNOTRUN_MSG;
    case ESVOPENCONTROL:
      return ESVOPENCONTROL_MSG;
    case ESVOPENOK:
      return ESVOPENOK_MSG;
    case ESVOPENSTAT:
      return ESVOPENSTAT_MSG;
    case ESVOPENSTATNEW:
      return ESVOPENSTATNEW_MSG;
    case ESVREADSTAT:
      return ESVREADSTAT_MSG;
    case ESVINVALSTAT:
      return ESVINVALSTAT_MSG;
    case ESVWRITECONTROL:
      return ESVWRITECONTROL_MSG;
    case ESVWRITESTAT:
      return ESVWRITESTAT_MSG;
    case ESVWRITESTATNEW:
      return ESVWRITESTATNEW_MSG;
    default:
      return "unknown error";
  }
}
