/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv-private.h"
#include "sv.h"

/**
 * Starts a supervised service by sending a 'up' message to the supervise
 * process and waits for supervise to update it's status with the services
 * process id.
 *
 * @param path service path
 *
 * @retval 0  success
 * @retval -1 failure
 */

int sv_start(char *path) {
  int r;
  sv_stat_t svst;
  
  /* Stat the supervise path and return with an error if it failed. */
  r = sv_stat(&svst, path);
  if (r)
    return r;
    
  /* If the status has a pid of 0 it means the service is down, otherwise
   * supervise is watching a running process. */
  if (!svst.pid) {
    r = sv_control(path, "u", 1);
    if (r)
      return r;
      
    do {
      sleep(1);
      r = sv_stat(&svst, path);
      if (r)
        return r;
    } while (!svst.pid);
  }
  
  return 0;
}
