/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv-private.h"
#include "sv.h"

/**
 * Check that path is a file, the correct size, and load it into svstat_t.
 *
 * @param svst svstat_t value
 * @param path status file path
 *
 * @remarks Errors:
 * @remarks ESVINVALSTAT: invalid status file
 * @remarks ESVOPENSTAT: failed to open status file
 * @remarks ESVREADSTAT: failed to read status file
 */

int sv_stat_load(sv_stat_t *svst, char *path) {
  int r;
  struct stat st;
  char status[SUPERVISE_STATUS_SIZE];
  int status_fd;
  uint32_t tmp;
  
  /* stat() path and check that it's actually a file and the correct size. */
  r = stat(path, &st);
  if (r)
    return r;
  if (!S_ISREG(st.st_mode) || (st.st_size != SUPERVISE_STATUS_SIZE))
    return ESVINVALSTAT;
    
  /* Open path and load its contents into the status byte array. */
  status_fd = open_read(path);
  if (status_fd == -1)
    return ESVOPENSTAT;
  r = read(status_fd, status, SUPERVISE_STATUS_SIZE);
  close(status_fd);
  if (r == -1)
    return ESVREADSTAT;
    
  /* taia_unpack will read the pid as the attoseconds, so set it to zero after
   * reading it. */
  taia_unpack(&svst->timestamp, status);
  svst->timestamp.atto = 0;
  tmp = be32toh(((uint32_t *)status)[3]);
  svst->pid = tmp;
  svst->paused = status[16];
  svst->mode = status[17];
  
  return 0;
}
