/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sv-private.h"
#include "sv.h"

/**
 * Attempts to stop a supervised log service and blocks until either the timeout
 * is reached, in which case it sends a final 'kill' message before returning,
 * or until supervise registers the service as being down.
 *
 * @param path    service path
 * @param timeout timeout in seconds
 *
 * @retval 0  success
 * @retval -1 failure
 *
 * @remarks if timeout is 0 then svstop blocks forever until the service stops.
 */

int sv_stop_log(char *path, int timeout) {
  array_t sa = ARRAY_INIT(1);
  char *path_log;
  int r;
  
  array_append(&sa, path, strlen(path));
  if (path[strlen(path) - 1] != '/')
    array_append(&sa, "/", 1);
  array_append(&sa, "log", 3);
  array_append_null(&sa);
  
  path_log = array_start(&sa);
  
  r = sv_stop(path_log, timeout);
  array_reset(&sa);
  return r;
}
