/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <unistd.h>
#include <sv/file.h>
#include <sv/strerr.h>
#include <sys/stat.h>
#include "sv.h"

int main() {
  char c;
  int fd_control;
  int fd_control_write;
  int r;
  
  /* Attempt to use svcontrol which should fail. */
  r = sv_control(".", "u", 1);
  if ((r != ESVOPENCONTROL))
    strerr_die1x(1, "svcontrol did not error");
    
  /* Emulate a running supervise process so svcontrol can work its magic. The
   * supervise directory needs to be created, control fifo created, and opened
   * for reading and writing. */
  mkdir(SUPERVISE_PATH, 0700);
  
  mkfifo(SUPERVISE_CONTROL_PATH, 0600);
  fd_control = open_read(SUPERVISE_CONTROL_PATH);
  if (fd_control == -1)
    strerr_die1sys(2, "unable to read " SUPERVISE_CONTROL_PATH ": ");
  blocking_disable(fd_control);
  fd_control_write = open_write(SUPERVISE_CONTROL_PATH);
  if (fd_control_write == -1)
    strerr_die1sys(3, "unable to write " SUPERVISE_CONTROL_PATH ": ");
    
  r = sv_control(".", "u", 1);
  if (r)
    strerr_die1sys(4, "svcontrol failed: ");
    
  r = read(fd_control, &c, 1);
  if (r == -1)
    strerr_die1sys(5, "failed to read control: ");
    
  if (c != 'u')
    strerr_die1x(6, "invalid control");
    
  _exit(0);
}
