/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <unistd.h>
#include <sv/file.h>
#include <sv/strerr.h>
#include <sv/taia.h>
#include <sys/stat.h>
#include "sv.h"

#define MODE    0
#define PAUSED  1
#define PID     5555

int main() {
  int fd_ok;
  sv_stat_t svst1;
  sv_stat_t svst2;
  
  /* Try running svstat without the proper setup which should fail. */
  if (!sv_stat(&svst1, "."))
    strerr_die1x(1, "svstat did not error");
    
  /* Emulate a running supervise process so svstat can work its magic. The
   * supervise directory needs to be created, ok fifo created and opened for
   * reading, and the status file created with dummy data. */
  mkdir(SUPERVISE_PATH, 0700);
  mkfifo(SUPERVISE_OK_PATH, 0600);
  fd_ok = open_read(SUPERVISE_OK_PATH);
  if (fd_ok == -1)
    strerr_die1sys(2, "unable to read ok fifo");
    
  taia_now(&svst1.timestamp);
  svst1.pid = PID;
  svst1.paused = PAUSED;
  svst1.mode = MODE;
  sv_stat_save(SUPERVISE_STATUS_PATH, &svst1);
  
  if (sv_stat(&svst2, "."))
    strerr_die1x(3, "svstat errored");
    
  if (svst2.mode != MODE)
    strerr_die1x(4, "incorrect mode value");
  if (svst2.paused != PAUSED)
    strerr_die1x(5, "incorrect paused value");
  if (svst2.pid != PID)
    strerr_die1x(6, "incorrect pid value");
    
  return 0;
}
