/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <unistd.h>
#include <sv/strerr.h>
#include <sv/taia.h>
#include "sv.h"

int main() {
  char *path = "status";
  sv_stat_t svst1;
  sv_stat_t svst2;
  
  taia_now(&svst1.timestamp);
  svst1.mode = 'd';
  svst1.paused = 1;
  svst1.pid = 2;
  
  if (sv_stat_save(path, &svst1) != 0)
    strerr_die1x(1, "svstat_save failed");
    
  if (sv_stat_load(&svst2, path) != 0)
    strerr_die1x(2, "svstat_load failed");
    
  if (taia_cmp(&svst1.timestamp, &svst2.timestamp))
    strerr_die1x(3, "timestamp values aren't equal");
  if (svst1.pid != svst2.pid)
    strerr_die1x(4, "pid values aren't equal");
  if (svst1.paused != svst2.paused)
    strerr_die1x(5, "paused values aren't equal");
  if (svst1.mode != svst2.mode)
    strerr_die1x(6, "mode values aren't equal");
    
  return 0;
}
