/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <sv.h>
#include <unistd.h>
#include <sv/bio.h>
#include <sv/strerr.h>

#define FATAL "check_supervise: fatal: "

enum {
  STATE_OK,
  STATE_WARNING,
  STATE_CRITICAL
};

static void die2x(int e, char *x1, char *x2) {
  bio_put_str(bio_1, x1);
  bio_put_str(bio_1, x2);
  bio_put_str(bio_1, "\n");
  bio_flush(bio_1);
  _exit(e);
}

#define OK(x)       die2x(STATE_OK, "OK: ", x)
#define UNKNOWN(x)  die2x(STATE_UNKNOWN, "UNKNOWN: ", x)
#define CRITICAL(x) die2x(STATE_CRITICAL, "CRITICAL: ", x)
#define WARNING(x)  die2x(STATE_WARNING, "WARNING: ", x)

void usage() {
  strerr_die1x(100,
               "\n"
               "check_supervise: usage: check_supervise path\n"
              );
}

int main(int argc, char **argv) {
  char *path;
  sv_stat_t svst;
  
  if ((!*argv++) || (!*argv))
    usage();
  path = *argv;
  
  if (sv_stat(&svst, path) == -1)
    CRITICAL(sv_error_str(errno));
    
  /* If the service isn't running and it's not configured to autostart then
   * it's fine, otherwise it's a problem. */
  if (!svst.pid) {
    if (!svst.autostart)
      OK("service down and disabled");
    else
      CRITICAL("service down");
  }
  
  OK("service up");
}
