/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <dirent.h>
#include <errno.h>
#include <sv/array.h>
#include <sv/file.h>
#include <sv/pathexec.h>
#include <sv/strerr.h>

#define FATAL "envdir: fatal: "

void usage() {
  strerr_die1x(100,
               "\n"
               "envdir: usage: envdir dir command\n"
              );
}

int main(int argc, char **argv) {
  array_t a_envfile = ARRAY_INIT(1);
  char *data;
  DIR *dir;
  char *dirname;
  struct dirent *entry;
  int fdenvfile;
  int fdodir;
  int i;
  
  if (!*argv || !*++argv)
    usage();
  dirname = *argv;
  
  if (!*++argv)
    usage();
    
  fdodir = open_read(".");
  if (fdodir == -1)
    strerr_die2sys(111, FATAL, "unable to switch to current directory: ");
  if (chdir(dirname) == -1)
    strerr_die4sys(111, FATAL, "unable to switch to directory ", dirname, ": ");
    
  dir = opendir(".");
  if (!dir)
    strerr_die4sys(111, FATAL, "unable to read directory ", dirname, ": ");
    
  for (;;) {
    errno = 0;
    entry = readdir(dir);
    if (!entry) {
      if (errno)
        strerr_die4sys(111, FATAL, "unable to read directory ", dirname, ": ");
      break;
    }
    if (entry->d_name[0] != '.') {
      array_trunc(&a_envfile);
      
      if (array_read_fn(&a_envfile, entry->d_name, 256) == -1)
        strerr_die6sys(111, FATAL, "unable to read ", dirname, "/", entry->d_name, ": ");
        
      if (array_size(&a_envfile)) {
        data = array_start(&a_envfile);
        for (i = 0; i < array_size(&a_envfile); i++, data++) {
          if ((*data == '\n') || (*data == ' ') || (*data == '\t')) {
            *data = 0;
            break;
          }
        }
        pathexec_env(entry->d_name, array_start(&a_envfile));
      } else
        pathexec_env(entry->d_name, 0);
    }
  }
  
  if (fchdir(fdodir) == -1)
    strerr_die2sys(111, FATAL, "unable to switch to starting directory: ");
  close(fdodir);
  
  array_reset(&a_envfile);
  
  pathexec(argv);
  strerr_die4sys(111, FATAL, "unable to run '", *argv, "': ");
}
