/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <pwd.h>
#include <sv/fmt.h>
#include <sv/pathexec.h>
#include <sv/strerr.h>

#define FATAL "envuidgid: fatal: "

void usage() {
  strerr_die1x(100,
               "\n"
               "envuidgid: usage: envuidgid username command\n"
              );
}

int main(int argc, char **argv) {
  char fmtbuf[FMT_ULONG_MAX + 1];
  struct passwd *pwd;
  char *username;
  
  if (!*argv || !*++argv)
    usage();
  username = *argv;
  
  if (!*++argv)
    usage();
    
  pwd = getpwnam(username);
  if (!pwd)
    strerr_die3x(111, FATAL, "unknown username ", username);
    
  fmtbuf[fmt_ulong(fmtbuf, pwd->pw_gid)] = 0;
  if (!pathexec_env("GID", fmtbuf))
    strerr_die2x(111, FATAL, "unable to set GID pathexec environment variable ");
  fmtbuf[fmt_ulong(fmtbuf, pwd->pw_uid)] = 0;
  if (!pathexec_env("UID", fmtbuf))
    strerr_die2x(111, FATAL, "unable to set UID pathexec environment variable ");
    
  pathexec(argv);
  strerr_die4sys(111, FATAL, "unable to run '", *argv, "': ");
}
