/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <unistd.h>
#include <sv/pathexec.h>
#include <sv/strerr.h>
#include <sys/wait.h>

#define FATAL "fghack: fatal: "

void usage() {
  strerr_die1x(100,
               "\n"
               "fghack: usage: fghack command\n"
              );
}

int main(int argc, char **argv, char **envp) {
  char ch;
  int i;
  int pid;
  int pipes[2];
  int wstat;
  
  if (!*argv || !*++argv)
    usage();
    
  if (pipe(pipes) == -1)
    strerr_die2sys(111, FATAL, "unable to create pipe: ");
    
  pid = fork();
  switch (pid) {
    case -1:
      strerr_die2sys(111, FATAL, "unable to fork: ");
    case 0:
      close(pipes[0]);
      for (i = 0; i < 30; ++i)
        dup(pipes[1]);
      pathexec_run(*argv, argv, envp);
      strerr_die4sys(111, FATAL, "unable to run '", *argv, "': ");
  }
  
  close(pipes[1]);
  
  for (;;) {
    i = read(pipes[0], &ch, 1);
    if ((i == -1) && (errno == EINTR))
      continue;
    if (i == 1)
      continue;
    break;
  }
  
  do {
    i = waitpid(pid, &wstat, 0);
  } while ((i == -1) && (errno == EINTR));
  
  if (i == -1)
    strerr_die2sys(111, FATAL, "wait failed: ");
  if (wstat & 127)
    strerr_die2x(111, FATAL, "child crashed: ");
    
  _exit(WEXITSTATUS(wstat));
}
