/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdlib.h>
#include <sv/scan.h>
#include <sv/strerr.h>
#include "multilog.h"

void c_init(char **script) {
  int i;
  struct cyclog *d;
  char *processor;
  unsigned long num;
  unsigned long size;
  
  cnum = 0;
  for (i = 0; script[i]; ++i)
    if ((script[i][0] == '.') || (script[i][0] == '/'))
      ++cnum;
      
  c = (struct cyclog *) malloc(cnum * sizeof(*c));
  if (!c)
    strerr_die2x(111, FATAL, "out of memory");
    
  d = c;
  processor = 0;
  num = 10;
  size = 99999;
  
  for (i = 0; script[i]; ++i)
    if (script[i][0] == 's') {
      scan_ulong(&size, script[i] + 1);
      if (size < 4096)
        size = 4096;
      if (size > 16777215)
        size = 16777215;
    } else if (script[i][0] == 'n') {
      scan_ulong(&num, script[i] + 1);
      if (num < 2)
        num = 2;
    } else if (script[i][0] == '!') {
      processor = script[i] + 1;
    } else if ((script[i][0] == '.') || (script[i][0] == '/')) {
      d->num = num;
      d->size = size;
      d->processor = processor;
      d->dir = script[i];
      bio_init(&d->ss, c_write, d - c, d->buf, sizeof(d->buf));
      restart(d);
      ++d;
    }
}
