/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <sv/taia.h>
#include <sys/stat.h>
#include "multilog.h"

void finish(struct cyclog *d, char *file, char *code) {
  char pack[TAIA_PACK_SIZE];
  struct stat st;
  struct taia t;
  
  for (;;) {
    if (stat(file, &st) == 0)
      break;
    if (errno == ENOENT)
      return;
    pause5("unable to stat ", d->dir, "/", file, ", pausing: ");
  }
  
  if (st.st_nlink == 1)
    for (;;) {
      taia_now(&t);
      taia_pack(pack, &t);
      tai64n_encode(fn, pack);
      fn[TAI64N_ENCODE_SIZE] = '.';
      fn[TAI64N_ENCODE_SIZE + 1] = code[0];
      fn[TAI64N_ENCODE_SIZE + 2] = 0;
      
      if (link(file, fn) == 0)
        break;
      pause5("unable to link to ", d->dir, "/", fn, ", pausing: ");
    }
    
  while (unlink(file) == -1)
    pause5("unable to remove ", d->dir, "/", file, ", pausing: ");
    
  for (;;)
    switch (filesfit(d)) {
      case 1:
        return;
      case -1:
        pause3("unable to read ", d->dir, ", pausing: ");
    }
}
