/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <stdio.h>
#include <sv/file.h>
#include <sv/strerr.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "multilog.h"

void fullcurrent(struct cyclog *d) {
  int fd;
  int i;
  int pid;
  int w;
  int wstat;
  
  while (fchdir(d->fddir) == -1)
    pause3("unable to switch to ", d->dir, ", pausing: ");
    
  while (fsync(d->fdcurrent) == -1)
    pause3("unable to write ", d->dir, "/current to disk, pausing: ");
  close(d->fdcurrent);
  
  while (rename("current", "previous") == -1)
    pause3("unable to rename current to previous in directory ", d->dir, ", pausing: ");
  while ((d->fdcurrent = open_append("current")) == -1)
    pause3("unable to create ", d->dir, "/current, pausing: ");
  close_on_exec(d->fdcurrent);
  d->bytes = 0;
  while (fchmod(d->fdcurrent, 0644) == -1)
    pause3("unable to set mode of ", d->dir, "/current, pausing: ");
    
  while (chmod("previous", 0744) == -1)
    pause3("unable to set mode of ", d->dir, "/previous, pausing: ");
    
  if (!d->processor)
    finish(d, "previous", "s");
  else {
    for (;;) {
      while ((pid = fork()) == -1)
        pause3("unable to fork for processor in ", d->dir, ", pausing: ");
      if (!pid) {
        startprocessor(d);
        strerr_die4sys(111, FATAL, "unable to run ", d->processor, ": ");
      }
      do {
        w = waitpid(pid, &wstat, 0);
      } while ((i == -1) && (errno == EINTR));
      if (w == -1)
        pause3("wait failed for processor in ", d->dir, ", pausing: ");
      else if ((wstat) & 127)
        pause3("processor crashed in ", d->dir, ", pausing: ");
      else if (!WEXITSTATUS(wstat))
        break;
      strerr_warn4x(WARNING, "processor failed in ", d->dir, ", pausing");
      sleep(5);
    }
    
    while ((fd = open_append("processed")) == -1)
      pause3("unable to create ", d->dir, "/processed, pausing: ");
    while (fsync(fd) == -1)
      pause3("unable to write ", d->dir, "/processed to disk, pausing: ");
    while (fchmod(fd, 0744) == -1)
      pause3("unable to set mode of ", d->dir, "/processed, pausing: ");
    close(fd);
    
    while ((fd = open_append("newstate")) == -1)
      pause3("unable to create ", d->dir, "/newstate, pausing: ");
    while (fsync(fd) == -1)
      pause3("unable to write ", d->dir, "/newstate to disk, pausing: ");
    close(fd);
    
    while (unlink("previous") == -1)
      pause3("unable to remove ", d->dir, "/previous, pausing: ");
    while (rename("newstate", "state") == -1)
      pause3("unable to rename newstate to state in directory ", d->dir, ", pausing: ");
    finish(d, "processed", "s");
  }
}