/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sv/bio.h>
#include <sv/file.h>
#include <sv/lseek.h>
#include <sv/sig.h>
#include <sv/strerr.h>
#include <sv/taia.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "multilog.h"

struct cyclog *c;
int cnum;
int *f;
int fdstartdir;
int flagexitasap = 0;
int flagforcerotate = 0;
int flagnewline = 1;
char fn[40];
int i;
char inbuf[1024];
char line[1001];
int linelen;
bio_t ssin = BIO_INIT(flushread, 0, inbuf, sizeof(inbuf));

void doit(char **script) {
  int flageof;
  char ch;
  int j;
  int i;
  char *action;
  int flagselected;
  int flagtimestamp;
  char pack[TAIA_PACK_SIZE];
  struct taia t;
  
  flagtimestamp = 0;
  if (script[0])
    if (script[0][0] == 't')
      flagtimestamp = 1;
      
  for (i = 0; i <= 1000; ++i)
    line[i] = '\n';
  linelen = 0;
  
  flageof = 0;
  for (;;) {
    for (i = 0; i < linelen; ++i)
      line[i] = '\n';
    linelen = 0;
    
    while (linelen < 1000) {
      if (BIO_GETC(&ch, &ssin) <= 0) {
        if (!linelen)
          return;
        flageof = 1;
        break;
      }
      if (!linelen)
        if (flagtimestamp) {
          taia_now(&t);
          taia_pack(pack, &t);
          tai64n_encode(line + 1, pack);
          line[0] = '@';
          line[TAI64N_ENCODE_SIZE + 1] = ' ';
          linelen = TAI64N_ENCODE_SIZE + 2;
        }
      if (ch == '\n')
        break;
      line[linelen++] = ch;
    }
    
    flagselected = 1;
    j = 0;
    for (i = 0; (action = script[i]); ++i)
      switch (*action) {
        case '+':
          if (!flagselected)
            if (match(action + 1, line, linelen))
              flagselected = 1;
          break;
        case '-':
          if (flagselected)
            if (match(action + 1, line, linelen))
              flagselected = 0;
          break;
        case 'e':
          if (flagselected) {
            if (linelen > 200) {
              bio_put(bio_2, line, 200);
              bio_put_str(bio_2, "...\n");
            } else {
              bio_put(bio_2, line, linelen);
              bio_put_str(bio_2, "\n");
            }
            bio_flush(bio_2);
          }
          break;
        case '=':
          if (flagselected)
            for (;;) {
              while (lseek_begin(f[i]) == -1)
                pause3("unable to move to beginning of ", action + 1, ", pausing: ");
              if (write(f[i], line, 1001) == 1001)
                break;
              pause3("unable to write ", action + 1, ", pausing: ");
            }
          break;
        case '.':
        case '/':
          c[j].flagselected = flagselected;
          ++j;
          break;
      }
      
    for (j = 0; j < cnum; ++j)
      if (c[j].flagselected)
        bio_put(&c[j].ss, line, linelen);
        
    if (linelen == 1000)
      for (;;) {
        if (BIO_GETC(&ch, &ssin) <= 0) {
          flageof = 1;
          break;
        }
        if (ch == '\n')
          break;
        for (j = 0; j < cnum; ++j)
          if (c[j].flagselected)
            BIO_PUTC(&c[j].ss, ch);
      }
      
    for (j = 0; j < cnum; ++j)
      if (c[j].flagselected) {
        ch = '\n';
        BIO_PUTC(&c[j].ss, ch);
      }
      
    if (flageof)
      return;
  }
}

void exitasap() {
  flagexitasap = 1;
}

void forcerotate() {
  flagforcerotate = 1;
}

int main(int argc, char **argv) {
  umask(022);
  
  fdstartdir = open_read(".");
  if (fdstartdir == -1)
    strerr_die2sys(111, FATAL, "unable to switch to current directory: ");
  close_on_exec(fdstartdir);
  
  sig_block(SIGTERM);
  sig_block(SIGALRM);
  sig_catch(SIGTERM, exitasap);
  sig_catch(SIGALRM, forcerotate);
  
  ++argv;
  f_init(argv);
  c_init(argv);
  doit(argv);
  c_quit();
  _exit(0);
}
