/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <sv/file.h>
#include <sv/strerr.h>
#include <sys/stat.h>
#include "multilog.h"

void restart(struct cyclog *d) {
  struct stat st;
  int fd;
  int flagprocessed;
  
  if (fchdir(fdstartdir) == -1)
    strerr_die2sys(111, FATAL, "unable to switch to starting directory: ");
    
  mkdir(d->dir, 0700);
  d->fddir = open_read(d->dir);
  if ((d->fddir == -1) || (fchdir(d->fddir) == -1))
    strerr_die4sys(111, FATAL, "unable to open directory ", d->dir, ": ");
  close_on_exec(d->fddir);
  
  d->fdlock = open_append("lock");
  if ((d->fdlock == -1) || (flock_exnb(d->fdlock) == -1))
    strerr_die4sys(111, FATAL, "unable to lock directory ", d->dir, ": ");
  close_on_exec(d->fdlock);
  
  if (stat("current", &st) == -1) {
    if (errno != ENOENT)
      strerr_die4sys(111, FATAL, "unable to stat ", d->dir, "/current: ");
  } else if (st.st_mode & 0100) {
    fd = open_append("current");
    if (fd == -1)
      strerr_die4sys(111, FATAL, "unable to append to ", d->dir, "/current: ");
    if (fchmod(fd, 0644) == -1)
      strerr_die4sys(111, FATAL, "unable to set mode of ", d->dir, "/current: ");
    close_on_exec(fd);
    d->fdcurrent = fd;
    d->bytes = st.st_size;
    return;
  }
  
  unlink("state");
  unlink("newstate");
  
  flagprocessed = 0;
  if (stat("processed", &st) == -1) {
    if (errno != ENOENT)
      strerr_die4sys(111, FATAL, "unable to stat ", d->dir, "/processed: ");
  } else if (st.st_mode & 0100)
    flagprocessed = 1;
    
  if (flagprocessed) {
    unlink("previous");
    finish(d, "processed", "s");
  } else {
    unlink("processed");
    finish(d, "previous", "u");
  }
  
  finish(d, "current", "u");
  
  fd = open_trunc("state");
  if (fd == -1)
    strerr_die4sys(111, FATAL, "unable to write to ", d->dir, "/state: ");
  close(fd);
  fd = open_append("current");
  if (fd == -1)
    strerr_die4sys(111, FATAL, "unable to write to ", d->dir, "/current: ");
  if (fchmod(fd, 0644) == -1)
    strerr_die4sys(111, FATAL, "unable to set mode of ", d->dir, "/current: ");
  close_on_exec(fd);
  d->fdcurrent = fd;
  d->bytes = 0;
}