/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <unistd.h>
#include <sv/file.h>
#include <sv/pathexec.h>
#include <sv/sgetopt.h>
#include <sv/strerr.h>

#define FATAL "setlock: fatal: "

void usage() {
  strerr_die1x(100,
               "\n"
               "setlock: usage: setlock [ options ] path command\n"
               "\n"
               "  -n  nonblocking\n"
               "  -x  return success\n"
              );
}

int main(int argc, char **argv, char **envp) {
  int fd;
  int flag_ndelay = 0;
  int flag_x = 0;
  char *file;
  int opt;
  
  while ((opt = sgetopt(argc, argv, "nx")) != -1) {
    switch (opt) {
      case 'n':
        flag_ndelay = 1;
        break;
      case 'x':
        flag_x = 1;
        break;
      default:
        usage();
    }
  }
  
  argv += soptind;
  if (!*argv)
    usage();
  file = *argv;
  
  if (!*++argv)
    usage();
    
  fd = open_append(file);
  if (fd == -1) {
    if (flag_x)
      _exit(0);
    strerr_die4sys(111, FATAL, "unable to open ", file, ": ");
  }
  
  if ((flag_ndelay ? flock_exnb(fd) : flock_ex(fd)) == -1) {
    if (flag_x)
      _exit(0);
    strerr_die4sys(111, FATAL, "unable to lock ", file, ": ");
  }
  
  pathexec_run(*argv, argv, envp);
  strerr_die4sys(111, FATAL, "unable to run ", *argv, ": ");
}
