/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <pwd.h>
#include <unistd.h>
#include <sv/pathexec.h>
#include <sv/strerr.h>
#include <sys/types.h>

#define FATAL "setuidgid: fatal: "

void usage() {
  strerr_die1x(100,
               "\n"
               "setuidgid: usage: setuidgid account command\n"
              );
}

int main(int argc, char **argv, char **envp) {
  char *account;
  struct passwd *pwd;
  
  if (!*argv || !*++argv)
    usage();
  account = *argv;
  
  if (!*++argv)
    usage();
    
  pwd = getpwnam(account);
  if (!pwd)
    strerr_die3x(111, FATAL, "unknown account ", account);
    
  if (setgid(pwd->pw_gid) == -1)
    strerr_die2sys(111, FATAL, "unable to setgid: ");
  if (setuid(pwd->pw_uid) == -1)
    strerr_die2sys(111, FATAL, "unable to setuid: ");
    
  pathexec_run(*argv, argv, envp);
  strerr_die4sys(111, FATAL, "unable to run '", *argv, "': ");
}
