/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <unistd.h>
#include <sv/pathexec.h>
#include <sv/sig.h>
#include <sv/strerr.h>
#include <sys/stat.h>
#include <sys/wait.h>

#define FATAL "startstop: fatal: "
#define WARNING "startstop: warning: "

int main(int argc, char **argv, char **envp) {
  int r;
  struct stat st;
  int wstat;
  
  /* Fork off the ./start script and wait for it to finish before waiting for
   * a signal to be received. */
  if (stat("start", &st) == 0) {
    switch (fork()) {
      case -1:
        strerr_die2sys(111, FATAL, "unable to fork for ./start: ");
      case 0:
        pathexec_run("./start", 0, envp);
        strerr_die2sys(111, FATAL, "unable to run ./start: ");
    }
    
    for (;;) {
      r = wait(&wstat);
      if (r == -1) {
        if (errno == EINTR)
          continue;
        strerr_warn2x(WARNING, "waitpid failed for child process");
      }
      break;
    }
  }
  
  sig_ignore(SIGALRM);
  sig_ignore(SIGHUP);
  sig_ignore(SIGTERM);
  
  /* Wait for a signal indefinitely.  Once one is recieved run the ./stop
   * script and exit. */
  sig_pause();
  
  if (stat("stop", &st) == 0) {
    pathexec_run("./stop", 0, envp);
    strerr_die2sys(111, FATAL, "unable to run ./stop: ");
  }
  
  _exit(0);
}
