/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <unistd.h>
#include <sv/sig.h>
#include <sv/strerr.h>
#include "supervise.h"

extern char **environ;

void start() {
  int r;
  char *run[2] = { "./run", 0 };
  
  switch (r = fork()) {
    case -1:
      /* If fork() fails, warn of the error and sleep for 60 seconds before
       * triggering the main loop to try again. */
      strerr_warn4sys(WARNING, "unable to fork for ", config.service_path,
                      ", sleeping 60 seconds: ");
      sleep(60);
      trigger();
      return;
    case 0:
      /* Clean up the child process before exec'ing the service. */
      sig_uncatch(SIGCHLD);
      sig_unblock(SIGCHLD);
      execve(*run, run, environ);
      strerr_die4sys(111, FATAL, "unable to run '", config.service_path,
                     "/run': ");
  }
  
  status.flag_paused = 0;
  status.pid = r;
  update_status();
  
  sleep(1);
}
