/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <string.h>
#include <sv.h>
#include <unistd.h>
#include <sv/bio.h>
#include <sv/sgetopt.h>
#include <sv/sig.h>
#include <sv/strerr.h>

#define FATAL "svc: fatal: "
#define WARNING "svc: warning: "

void usage() {
  strerr_die1x(100,
               "\n"
               "svc: usage: svc [ action ] dir\n"
               "\n"
               "  -u  UP    Start the service\n"
               "  -d  DOWN  Stop the service\n"
               "  -o  ONCE  Start the service once\n"
               "  -x  EXIT  Stop supervise process after service stops\n"
               "  -p  STOP  Send STOP signal\n"
               "  -c  CONT  Send CONT signal\n"
               "  -h  HUP   Send HUP signal\n"
               "  -a  ALRM  Send ALRM signal\n"
               "  -i  INT   Send INT signal\n"
               "  -t  TERM  Send TERM signal\n"
               "  -k  KILL  Send KILL signal\n"
               "  -q  QUIT  Send QUIT signal\n"
               "  -1  USR1  Send USR1 signal\n"
               "  -2  USR2  Send USR2 signal\n"
              );
}

int main(int argc, char **argv) {
  bio_t b;
  char bspace[1];
  char data[20];
  int datalen = 0;
  char *dirname;
  int fd;
  int opt;
  int r;
  
  sig_ignore(SIGPIPE);
  
  /* Fill the data byte array with the options as the commands and update
   * datalen with the total count. */
  while ((opt = sgetopt(argc, argv, "acdhikopqtux12")) != -1) {
    if (opt == '?')
      usage();
    else if (datalen < sizeof(data))
      if (memchr(data, opt, datalen) == 0)
        data[datalen++] = opt;
  }
  argv += soptind;
  
  if (!*argv)
    usage();
    
  /* Send the set of commands to each service. */
  while ((dirname = *argv++)) {
    r = sv_control(dirname, data, datalen);
    if (r)
      strerr_warn5x(WARNING, "unable to control service '", dirname, "': ", sv_error_str(r));
  }
  
  _exit(0);
}
