/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <sv.h>
#include <sv/bio.h>
#include <sv/fmt.h>
#include <sv/strerr.h>
#include <sv/tai.h>
#include <sv/taia.h>
#include <sys/stat.h>
#include "svstat.h"

void handle(char *path) {
  struct taia now;
  char pid_str[FMT_ULONG_MAX + 1];
  int r;
  struct sv_stat svst;
  char timestamp_str[FMT_ULONG_MAX + 1];
  
  bio_put_str(bio_1, path);
  bio_put_str(bio_1, ": ");
  
  r = sv_stat(&svst, path);
  if (r == -1) {
    switch (errno) {
      case ESVSTATDOWN:
        bio_put_str(bio_1, "unable to stat down: ");
        bio_put_str(bio_1, strerr_sys.x);
        break;
      case ESVNOTRUN:
        bio_put_str(bio_1, "supervise not running");
        break;
      case ESVOPENOK:
        bio_put_str(bio_1, "unable to open " SUPERVISE_OK_PATH ": ");
        bio_put_str(bio_1, strerr_sys.x);
        break;
      case ESVOPENSTAT:
        bio_put_str(bio_1, "unable to open " SUPERVISE_STATUS_PATH ": ");
        bio_put_str(bio_1, strerr_sys.x);
        break;
      case ESVREADSTAT:
        bio_put_str(bio_1, "unable to read " SUPERVISE_STATUS_PATH ": ");
        bio_put_str(bio_1, strerr_sys.x);
        break;
      case ESVINVALSTAT:
        bio_put_str(bio_1, "unable to read " SUPERVISE_STATUS_PATH ": bad format");
        break;
      default:
        bio_put_str(bio_1, "unknown error");
        break;
    }
    return;
  }
  
  /* Print out the services status information. */
  if (svst.pid) {
    pid_str[fmt_ulong(pid_str, svst.pid)] = 0;
    bio_put_str(bio_1, "up (pid ");
    bio_put_str(bio_1, pid_str);
    bio_put_str(bio_1, ") ");
  } else
    bio_put_str(bio_1, "down ");
    
  taia_now(&now);
  taia_sub(&now, &now, &svst.timestamp);
  timestamp_str[fmt_ulong(timestamp_str, tai_approx(&now))] = 0;
  bio_put_str(bio_1, timestamp_str);
  bio_put_str(bio_1, " seconds");
  
  if (svst.pid && !svst.autostart)
    bio_put_str(bio_1, ", normally down");
  if (!svst.pid && svst.autostart)
    bio_put_str(bio_1, ", normally up");
  if (svst.pid && svst.paused)
    bio_put_str(bio_1, ", paused");
  if (!svst.pid && (svst.mode == 'u'))
    bio_put_str(bio_1, ", want up");
  if (svst.pid && (svst.mode == 'd'))
    bio_put_str(bio_1, ", want down");
}
