/*
 * Copyright (c) 2011-2015 David Shane Holden. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <time.h>
#include <unistd.h>
#include <sv/bio.h>
#include <sv/fmt.h>
#include <sv/tai.h>
#include <sv/taia.h>

static void get(char *ch) {
  int r;
  r = BIO_GETC(ch, bio_0);
  if (r == 1)
    return;
  if (r == 0)
    _exit(0);
  _exit(111);
}

static void put(char *buf, int len) {
  if (bio_put(bio_1, buf, len) == -1)
    _exit(111);
}

int main() {
  char buf[TAI64N_ENCODE_SIZE];
  char ch;
  char num[FMT_ULONG_MAX + 1];
  char pack[TAIA_PACK_SIZE];
  int pos;
  time_t secs;
  struct tm *t;
  struct taia ta;
  
  for (;;) {
    get(&ch);
    if (ch == '@') {
      /* Since timestamps might fall on buffer boundaries, loop through
       * bio_gets until the entire decode buffer is full for
       * tai64n_decode. */
      pos = 0;
      do {
        pos += bio_get(&buf[pos], sizeof(buf) - pos, bio_0);
      } while (pos != TAI64N_ENCODE_SIZE);
      tai64n_decode(pack, buf);
      taia_unpack(&ta, pack);
      secs = ta.sec - TAI_EPOCH;
      t = localtime(&secs);
      
      put(num, fmt_ulong(num, 1900 + t->tm_year));
      put("-", 1);
      put(num, fmt_ulong_pad(num, 1 + t->tm_mon, 2));
      put("-", 1);
      put(num, fmt_ulong_pad(num, t->tm_mday, 2));
      put(" ", 1);
      put(num, fmt_ulong_pad(num, t->tm_hour, 2));
      put(":", 1);
      put(num, fmt_ulong_pad(num, t->tm_min, 2));
      put(":", 1);
      put(num, fmt_ulong_pad(num, t->tm_sec, 2));
      put(".", 1);
      put(num, fmt_ulong_pad(num, ta.nano, 9));
      
      get(&ch);
    }
    for (;;) {
      put(&ch, 1);
      if (ch == '\n')
        break;
      get(&ch);
    }
  }
}
