/* -*- coding: utf-8; tab-width: 2; indent-tabs-mode: t -*- */
/*
 *   Copyright 2013 Daniel Trebbien
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <verlibc/string.h>

/* Based on code by David MacKenzie <djm@gnu.ai.mit.edu> in the Public Domain
   See: gettext-0.14.6/gettext-tools/lib/memmove.c in ftp://ftp.gnu.org/gnu/gettext/gettext-0.14.6.tar.gz */
void * memmove(void *dv, const void *sv, size_t n)
{
	size_t i;
	char *d = (char *) dv;
	const char *s = (const char *) sv;

	if (s < d)
	{
		/* Moving from low mem to hi mem; start at end. */
		for (i = n; i > 0; --i)
			_(invariant n >= i && i >= 0)
			_(invariant \forall size_t j; n >= j && j > i ==> d[j - 1] == \old(s[j - 1]))
			_(invariant \forall size_t j; i >= j && j > 0 ==> \unchanged(s[j - 1]))
		{
			_(assert n >= i && i > 0)
			_(assert n > (i - 1) && (i - 1) >= 0)
			d[i - 1] = s[i - 1];
		}

		_(assert \forall size_t j; {\match_ulong(j)} n > (j - 1) && (j - 1) >= 0 ==> d[j - 1] == \old(s[j - 1]))
		_(assert \forall size_t k; {:hint \match_ulong(k + 1)} n > k && k >= 0 ==> d[k] == \old(s[k]))
	}
	else if (s != d)
	{
		_(assert d < s)

		/* Moving from hi mem to low mem; start at beginning. */
		for (i = 0; i < n; ++i)
			_(invariant \forall size_t j; j < i ==> d[j] == \old(s[j]))
			_(invariant \forall size_t j; i <= j && j < n ==> \unchanged(s[j]))
		{
			d[i] = s[i];
		}
	}

	return dv;
}

#ifdef VERIFY
static void memmove_test1()
{
	char buf[5] = { 0, 'a', 'b', 'c', 'd' };
	char *b;

	b = memmove(buf, buf + 1, 4);
	_(assert b == buf)
	_(assert buf[0] == 'a')
	_(assert buf[1] == 'b')
	_(assert buf[2] == 'c')
	_(assert buf[3] == 'd')
	_(assert buf[4] == 'd')

	b = memmove(buf + 1, buf, 4);
	_(assert b == buf + 1)
	_(assert buf[0] == 'a')
	_(assert buf[1] == 'a')
	_(assert buf[2] == 'b')
	_(assert buf[3] == 'c')
	_(assert buf[4] == 'd')
}
#endif
