/* -*- coding: utf-8; tab-width: 2; indent-tabs-mode: t -*- */
/*
 *   Copyright 2013 Daniel Trebbien
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <verlibc/string.h>

char * strcpy(char *__restrict d, const char *__restrict s,
		_(ghost size_t s_nul_pos))
{
	size_t i;

	_(ghost size_t s_length = strlen(s, s_nul_pos))

	for (i = 0; s[i] != '\0'; ++i)
		_(invariant i <= s_length)
		_(invariant \forall size_t j; j < i ==> d[j] == s[j])
		_(invariant \forall size_t j; s_length < j && j < s_nul_pos + 1 ==> \unchanged(d[j]))
		_(invariant \forall size_t j; j <= s_nul_pos ==> \unchanged(s[j]))
	{
		d[i] = s[i];
	}

	d[i] = '\0';

	return d;
}

#ifdef VERIFY
static void strcpy_test1()
{
	char str1[8];
	str1[7] = '\0';
	char c = str1[4];
	char str2[8] = { 'a', 'b', 'c', '\0', 'e', 'f', 'g', '\0' };
	_(assert strlen(str2, 7) == 3)

	char *p = strcpy(str1, str2 _(ghost 7));
	_(assert p == str1)

	_(assert strlen(str1, 7) == 3)
	_(assert str1[1] == 'b')
	_(assert str1[4] == c)
}
#endif