/* -*- coding: utf-8; tab-width: 2; indent-tabs-mode: t -*- */
/*
 *   Copyright 2013 Daniel Trebbien
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <verlibc/string.h>

char * strncat(char *__restrict b, const char *__restrict str, size_t n
		_(ghost size_t b_nul_pos))
{
	size_t i, j;

	for (i = 0; b[i] != '\0'; ++i)
		_(invariant i <= strlen(b, b_nul_pos))
	{
	}

	_(ghost size_t old_b_length = strlen(b, b_nul_pos))
	_(assert old_b_length == \old(strlen(b, b_nul_pos)))
	_(assert i == old_b_length)

	j = 0;
	for (j = 0; j < n && str[j] != '\0'; ++i, ++j)
		_(invariant j <= n)
		_(invariant \forall size_t k; k < n ==> \unchanged(str[k]))
		_(invariant i == old_b_length + j)
		_(invariant i < b_nul_pos + n + 1)
		_(invariant \forall size_t k; k < j && (\forall size_t l; l < k ==> str[l] != '\0') ==> (\forall size_t l; old_b_length <= l && l <= old_b_length + k ==> b[l] == str[l - old_b_length]))
		_(invariant \forall size_t k; k < j ==> str[k] != '\0')
		_(invariant \forall size_t k; k < old_b_length ==> \unchanged(b[k]))
		_(invariant \forall size_t k; old_b_length <= k && k < i ==> b[k] == str[k - old_b_length])
		_(invariant \forall size_t k; old_b_length + j < k && k < b_nul_pos + n + 1 ==> \unchanged(b[k]))
	{
		b[i] = str[j];
	}

	b[i] = '\0';

	_(assert \forall size_t k; {\match_ulong(k)} j < k && k < n ==> !(\forall size_t l; l < k ==> str[l] != '\0'))
	_(assert str[j] == '\0' ==> (\forall size_t k; old_b_length + j < k && k < b_nul_pos + n + 1 ==> \unchanged(b[k])))

	return b;
}

#ifdef VERIFY
static void strncat_test1()
{
	char buf[2] = { '\0', '\0' };
	char str[1] = { 'a' };

	char *b = strncat(buf, str, 1, _(ghost 0));

	_(assert b[0] == 'a')
	_(assert b[1] == '\0')
}

static void strncat_test2()
{
	char buf[3] = { '\0', '\0', '\0' };
	char str[2] = { 'a', '\0' };

	char *b = strncat(buf, str, 2, _(ghost 0));

	_(assert b[0] == 'a')
	_(assert b[1] == '\0')
}
#endif
