/* -*- coding: utf-8; tab-width: 2; indent-tabs-mode: t -*- */
/*
 *   Copyright 2013 Daniel Trebbien
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <verlibc/string.h>

char * strncpy(char *__restrict d, const char *__restrict s, size_t n
		_(ghost size_t s_nul_pos))
{
	size_t i;

	_(ghost size_t s_length = strlen(s, s_nul_pos))

	for (i = 0; s[i] != '\0' && i < n; ++i)
		_(invariant i <= s_length)
		_(invariant \forall size_t j; j < i ==> d[j] == s[j])
		_(invariant \forall size_t j; j <= s_nul_pos ==> \unchanged(s[j]))
	{
		d[i] = s[i];
	}

	if (i < n)
	{
		do
			_(invariant i < n)
			_(invariant s_length <= i)
			_(invariant \forall size_t j; j < s_length && j < n ==> d[j] == s[j])
			_(invariant \forall size_t j; s_length <= j && j < i ==> d[j] == '\0')
			_(invariant \forall size_t j; j <= s_nul_pos ==> \unchanged(s[j]))
		{
			d[i++] = '\0';
		} while(i < n);
	}

	_(assert s_length == strlen(s, s_nul_pos))

	return d;
}

#ifdef VERIFY
static void strncpy_test1()
{
	char buf[10];
	char str[3] = { 'a', 'b', '\0' };

	char *p = strncpy(buf, str, 10 _(ghost 2));
	_(assert p == buf)

	_(assert buf[0] == 'a')
	_(assert buf[1] == 'b')
	_(assert buf[2] == '\0')
	_(assert buf[9] == '\0')
}

static void strncpy_test2()
{
	char buf[2];
	char c = buf[1];
	char str[3] = { 'a', 'b', '\0' };

	char *p = strncpy(buf, str, 1 _(ghost 2));
	_(assert p == buf)

	_(assert buf[0] == 'a')
	_(assert buf[1] == c)
}
#endif
