/* -*- coding: utf-8; tab-width: 2; indent-tabs-mode: t -*- */
/*
 *   Copyright 2013 Daniel Trebbien
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <verlibc/string.h>

char * strpbrk(const char *str, const char *search_chars
		_(ghost size_t str_nul_pos) _(ghost size_t search_chars_nul_pos))
{
	size_t i, j;

	for (i = 0; str[i] != '\0'; ++i)
		_(invariant i <= strlen(str, str_nul_pos))
		_(invariant \forall size_t l, k; l < i && k < strlen(search_chars, search_chars_nul_pos) ==> str[l] != search_chars[k])
	{
		for (j = 0; search_chars[j] != '\0'; ++j)
			_(invariant j <= strlen(search_chars, search_chars_nul_pos))
			_(invariant \forall size_t k; k < j ==> str[i] != search_chars[k])
		{
			if (str[i] == search_chars[j])
			{
				_(assert str + i != 0)
				return str + i;
			}
		}
	}

	return 0;
}

#ifdef VERIFY
static void strpbrk_test1()
{
	const char search_chars[] = { '0', '1', '2', '3', '\0' };
	const char str1[] = { 'a', 'b', '\0' };
	const char str2[] = { 'a', '0', '\0' };

	_(assert strpbrk(str1, search_chars, 2, 4) == 0)
	_(assert strpbrk(str2, search_chars, 2, 4) == str2 + 1)
}
#endif
