
#include <stdio.h>
#include <verilated.h>
#include <verilated_vcd_c.h>

#include "Vmeecoate.h"


Vmeecoate *top;
#if VM_TRACE
VerilatedVcdC *trace;
#endif








char gstring[80];
char newline[1024];
unsigned char hexline[1024];

#define ROMMASK 0xFFF
unsigned int rom[(ROMMASK+1)>>2];
#define ROMBYTESMASK 0x3FFF
unsigned char rombytes[ROMBYTESMASK+1];

//-----------------------------------------------------------------------------
int readhex ( FILE *fp )
{
    unsigned int addhigh;
    unsigned int add;
    unsigned int ra;
    unsigned int line;
    unsigned char checksum;
    unsigned int len;
    unsigned int hexlen;
    unsigned int maxadd;
    unsigned char t;

    maxadd=0;

    addhigh=0;
    memset(rombytes,0xFF,sizeof(rombytes));

    line=0;
    while(fgets(newline,sizeof(newline)-1,fp))
    {
        line++;
        //printf("%s",newline);
        if(newline[0]!=':')
        {
            printf("Syntax error <%u> no colon\n",line);
            continue;
        }
        gstring[0]=newline[1];
        gstring[1]=newline[2];
        gstring[2]=0;
        len=strtoul(gstring,NULL,16);
        for(ra=0;ra<(len+5);ra++)
        {
            gstring[0]=newline[(ra<<1)+1];
            gstring[1]=newline[(ra<<1)+2];
            gstring[2]=0;
            hexline[ra]=(unsigned char)strtoul(gstring,NULL,16);
        }
        checksum=0;
        for(ra=0;ra<(len+5);ra++) checksum+=hexline[ra];
        //checksum&=0xFF;
        if(checksum)
        {
            printf("checksum error <%u>\n",line);
        }
        add=hexline[1]; add<<=8;
        add|=hexline[2];
        add|=addhigh;
        if(add>=ROMBYTESMASK)
        {
            printf("address too big 0x%08X\n",add);
            return(1);
            continue;
        }
        t=hexline[3];

        //:0011223344
        //;llaaaattdddddd
        //01234567890
        switch(t)
        {
            default:
                printf("UNKOWN type %02X <%u>\n",t,line);
                break;
            case 0x00:
                for(ra=0;ra<len;ra++)
                {
                    if(add>=ROMBYTESMASK)
                    {
                        printf("address too big 0x%08X\n",add);
                        return(1);
                    }
                    rombytes[add]=hexline[ra+4];
                    add++;
                    if(add>maxadd) maxadd=add;
                }
                break;
            case 0x01:
                printf("End of data\n");
                break;
            case 0x02:
                addhigh=hexline[5];
                addhigh<<=8;
                addhigh|=hexline[4];
                addhigh<<=16;
                break;
        }
    }

    add=0;
    for(ra=0;ra<=ROMMASK;ra++)
    {
        rom[ra] =rombytes[add++]; rom[ra]<<=8;
        rom[ra]|=rombytes[add++]; rom[ra]<<=8;
        rom[ra]|=rombytes[add++];
    }

    //printf("%u lines processed\n",line);
    printf("%08X\n",maxadd);

    add=0;
    for(ra=0;ra<maxadd;ra+=3)
    {
        printf("0x%03X: 0x%05X\n",add,rom[add]);
        add++;
    }


    return(0);

}



int main(int argc, char **argv)
{
    unsigned int tick;
    unsigned int did_reset;
    unsigned int timer0_tick;
    unsigned int timer0_latch;

    int ret;

    FILE *fp;

//    Verilated::commandArgs(argc, argv);

    if(argc<2)
    {
        fprintf(stderr,".hex file not specified\n");
        return(1);
    }
    fp=fopen(argv[1],"rt");
    if(fp==NULL)
    {
        fprintf(stderr,"Error opening file [%s]\n",argv[1]);
        return(1);
    }
    if(readhex(fp)) return(1);
    fclose(fp);

#if VM_TRACE
    Verilated::traceEverOn(true);
#endif

    top = new Vmeecoate;

#if VM_TRACE
    trace = new VerilatedVcdC;
    top->trace(trace, 99);
    trace->open("test.vcd");
#endif

    timer0_tick=0;
    timer0_latch=0;
    did_reset=0;
    top->in_reset = 1;
    while (!Verilated::gotFinish())
    {
        tick++;

        if((tick&1)==0)
        {
            timer0_tick++;
        }

        if(did_reset)
        {
            top->in_pinst = rom[ROMMASK&top->out_padd];

            if(top->out_port_we)
            {
                if((tick&1)==0)
                {
                    printf("export(0x%02X,0x%02X)\n",top->out_port_add,top->out_port_data);
                }
            }
            top->in_port_data = 0;
            if(top->out_port_oe)
            {
                switch(top->out_port_add)
                {
                    case 0x10: //lower byte of timer
                    {
                        //a read of the lower byte latches the timer.
                        timer0_latch = timer0_tick;
                        top->in_port_data = (timer0_latch>>0)&0xFF;
                        break;
                    }
                    case 0x11:
                    {
                        //use latched value
                        top->in_port_data = (timer0_latch>>8)&0xFF;
                        break;
                    }
                    case 0x12:
                    {
                        //use latched value
                        top->in_port_data = (timer0_latch>>16)&0xFF;
                        break;
                    }
                    case 0x13:
                    {
                        //use latched value
                        top->in_port_data = (timer0_latch>>24)&0xFF;
                        break;
                    }
                    default:
                    {
                        top->in_port_data = top->out_port_add;
                        break;
                    }
                }
            }
        }
        else
        {
            if (tick > 11)
            {
                top->in_reset = 0;
                did_reset = 1;
            }
        }
        top->in_clock = (tick & 1);

        top->eval();
#if VM_TRACE
        trace->dump(tick);
        if(tick>200000) break;
#endif
    }
#if VM_TRACE
    trace->close();
#endif
    top->final();
    return 0;
}

