/*
* Copyright (C) 2011 by Erik Price
* 
* Permission to use, copy, modify, and/or distribute this software for any
* purpose with or without fee is hereby granted, provided that the above
* copyright notice and this permission notice appear in all copies.
* 
* THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
* WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
* ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
* WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
* ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
* OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "isr.h"
#include "console.h"

char isr_exceptions[32][32] = { 
  "Division by 0",
  "Debug exception",
  "Non maskable interrupt",
  "Breakpoint exception",
  "Into detected overflow",
  "Out of bounds exception",
  "Invalid opcode exception",
  "No coprocessor excetion",
  "Double fault",
  "Co processor segment overrun",
  "Bad TSS",
  "Segment not present",
  "Stack fault",
  "General protection fault",
  "Page fault",
  "Unknown interrupt exception",
  "Coprocessor fault",
  "Alignent Check Exception",
  "Machine check exception",
  
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  "Reserved",
  };

static isr_handle isr_handles[256] = {NULL};

void init_isr() {
  
  idt_set_gate(0, (uint32_t)isr0 , 0x08, 0x8E);
  idt_set_gate(1, (uint32_t)isr1 , 0x08, 0x8E);
  idt_set_gate(2, (uint32_t)isr2 , 0x08, 0x8E);
  idt_set_gate(3, (uint32_t)isr3 , 0x08, 0x8E);
  idt_set_gate(4, (uint32_t)isr4 , 0x08, 0x8E);
  idt_set_gate(5, (uint32_t)isr5 , 0x08, 0x8E);
  idt_set_gate(6, (uint32_t)isr6 , 0x08, 0x8E);
  idt_set_gate(7, (uint32_t)isr7 , 0x08, 0x8E);
  idt_set_gate(8, (uint32_t)isr8 , 0x08, 0x8E);
  idt_set_gate(9, (uint32_t)isr9 , 0x08, 0x8E);
  idt_set_gate(10, (uint32_t)isr10 , 0x08, 0x8E);
  idt_set_gate(11, (uint32_t)isr11 , 0x08, 0x8E);
  idt_set_gate(12, (uint32_t)isr12 , 0x08, 0x8E);
  idt_set_gate(13, (uint32_t)isr13 , 0x08, 0x8E);
  idt_set_gate(14, (uint32_t)isr14 , 0x08, 0x8E);
  idt_set_gate(15, (uint32_t)isr15 , 0x08, 0x8E);
  idt_set_gate(16, (uint32_t)isr16 , 0x08, 0x8E);
  idt_set_gate(17, (uint32_t)isr17 , 0x08, 0x8E);
  idt_set_gate(18, (uint32_t)isr18 , 0x08, 0x8E);
  
  idt_set_gate(19, (uint32_t)isr19 , 0x08, 0x8E);
  idt_set_gate(20, (uint32_t)isr20 , 0x08, 0x8E);
  idt_set_gate(21, (uint32_t)isr21 , 0x08, 0x8E);
  idt_set_gate(22, (uint32_t)isr22 , 0x08, 0x8E);
  idt_set_gate(23, (uint32_t)isr23 , 0x08, 0x8E);
  idt_set_gate(24, (uint32_t)isr24 , 0x08, 0x8E);
  idt_set_gate(25, (uint32_t)isr25 , 0x08, 0x8E);
  idt_set_gate(26, (uint32_t)isr26 , 0x08, 0x8E);
  idt_set_gate(27, (uint32_t)isr27 , 0x08, 0x8E);
  idt_set_gate(28, (uint32_t)isr28 , 0x08, 0x8E);
  idt_set_gate(29, (uint32_t)isr29 , 0x08, 0x8E);
  idt_set_gate(30, (uint32_t)isr30 , 0x08, 0x8E);
  idt_set_gate(31, (uint32_t)isr31 , 0x08, 0x8E);
}

void handle_isr(regs_t* regs) {

  unsigned int port = regs->int_no;

  if(isr_handles[port]) {
    isr_handles[port](regs);
  } else {
    // exception
    if(port < 32) {
      kpanic(isr_exceptions[port]);
    }
  }
}

void install_isr(int port, isr_handle fcn) {
  isr_handles[port] = fcn;
}
