// Read Input part from Dr. Kundu
#include<stdio.h>

// FIX PrintTree Output + any other issues

typedef struct {
  int degree, *adjNodes;
} GraphNode;
  
GraphNode *nodes;
int numNodes = -1;

void PrintIntVector(int *vector, int size)
{ int i;
  for (i=0; i<size; i++) printf("%d ",vector[i]);
  printf("\n");
}

void SkipToEndofLine(FILE *fpInp)
{ while (getc(fpInp) != '\n');}

void ReadInput() //creates the digraph
{ int i, j, nodeNum;
  FILE *fp;
  fp = stdin;
  fscanf(fp, "%d", &numNodes); 
  SkipToEndofLine(fp);
  nodes = (GraphNode *)malloc(numNodes*sizeof(GraphNode));
  for (i=0; i<numNodes; i++) 
  { fscanf(fp, "%d", &nodeNum); 
    fscanf(fp, " ( %d ) ", &nodes[nodeNum].degree);
    nodes[nodeNum].adjNodes = (int *)malloc(nodes[nodeNum].degree*sizeof(int));
    for (j=0; j<nodes[nodeNum].degree; j++) fscanf(fp, "%d", &nodes[nodeNum].adjNodes[j]);
    SkipToEndofLine(fp);
  }  
  fclose(fp);
  printf("nodeNum (degree) adjNodes:\n");
  for (i = 0; i < numNodes; i++)
  { printf("%d (%d)", i, nodes[i].degree); 
    PrintIntVector(nodes[i].adjNodes, nodes[i].degree); }
}

// Stack Operations
void StackPush  (int *stack,int node) { stack[++stack[0]] = node; } // pre-increment  tos, add node to new tos
int  StackPop   (int *stack){ return stack[stack[0]--]; } // post-decrement tos, return node at former tos
int  StackPeek  (int *stack){ return stack[stack[0]]; }   // peek at top of stack
void StackReset (int *stack){ stack[0] = 0; };
void StackFlush (int *stack){ int i = 0; for (i = stack[0]; i > 0; i--) { printf ("%d ",StackPop(stack));} } 
void StackPrintAll (int *stack) { int i = 0; for (i = stack[0]; i > 0; i--) { printf ("%d ",stack[i]); } }

typedef struct TreeNode {
  int dflabel,node_label;
  struct TreeNode *parent,*first,*last,*left,*right;
} TreeNode;

// Strong Component Search Code
struct TreeNode *AllStrongComponents(int startNode)
{ int i; struct TreeNode *tn = NULL;   // the tree node we'd like to create
  static int parentTreeNode,lastDfLabel,CallLevel,*dflabels,*low,*strongcomps,*backtracked;

// put in call level stuff, but make sure that unvisited nodes can get revisted...

  if (0 == CallLevel)
  { if (0 == numNodes) ReadInput();
    else {free(dflabels); free(low); free(strongcomps); free(backtracked);}
    dflabels = (int *)malloc(numNodes*sizeof(int));    
    low      = (int *)malloc(numNodes*sizeof(int));
    strongcomps  = (int *)malloc(numNodes*sizeof(int));
    backtracked  = (int *)malloc(numNodes*sizeof(int));}

  // recursion unrolling condition; only if dfLabel is unset
  //printf("recursion level %d\n",CallLevel);
  if (dflabels[startNode] == 0) 
  { tn = (struct TreeNode *)malloc(sizeof(TreeNode));
    tn->node_label = startNode;
    tn->parent = tn; // initialize parent as self
    tn->first = tn->last = tn->left = tn->right = NULL;
    if (0 == CallLevel) StackPush(strongcomps,startNode);
    CallLevel++;
    for (i = 0; i < nodes[startNode].degree; i++) 
    { // only assign dflabel once, init low      
      if (0 == dflabels[startNode]) 
      { tn->dflabel = dflabels[startNode] = low[startNode] = ++lastDfLabel;
        printf("dfLabel( %d)=%d, parentNode=%d, low[%d]=%d\n",startNode,lastDfLabel,parentTreeNode,startNode,low[startNode]);
        printf("stack: ");StackPrintAll(strongcomps);printf("\n");}
      printf("processing (%d,%d): ",tn->node_label,nodes[startNode].adjNodes[i]);
      if (0 == dflabels[nodes[startNode].adjNodes[i]]) 
      { printf ("tree-edge\n");
        StackPush(strongcomps,nodes[startNode].adjNodes[i]);
	parentTreeNode = startNode;
	// tree node building code section
	struct TreeNode *child = AllStrongComponents(nodes[startNode].adjNodes[i]);    
	if (NULL != child)
	{ child->left = child->right = NULL; child->parent = tn; 
          if (NULL == tn->first) tn->first = tn->last = child;
 	  else { child->left = tn->last; tn->last->right = tn->last = child; }	
          // backtracking...
          backtracked[child->node_label]++;
	}
	printf("backtracking %d -> %d\n",nodes[startNode].adjNodes[i],startNode);
        // update parent's low with child's low
	if (low[nodes[startNode].adjNodes[i]] < low[startNode]) 
	{ low[startNode] = low[nodes[startNode].adjNodes[i]];
          printf ("low[%d] = %d on backtrack\n",startNode,low[startNode]); 
	}
	else printf("No change in low[%d] on backtrack\n",startNode);
	// check to see if child is entry node for the strong component
	if (low[nodes[startNode].adjNodes[i]] == dflabels[nodes[startNode].adjNodes[i]]) 
	{ printf ("STRONG COMPONENT ");
	  while (StackPeek(strongcomps) != nodes[startNode].adjNodes[i]) printf("%d ",StackPop(strongcomps));
  	  printf("%d ",StackPop(strongcomps));
	  printf("\n");}
      }
      else if (dflabels[nodes[startNode].adjNodes[i]] < dflabels[startNode]  && backtracked[nodes[startNode].adjNodes[i]] == 0)
      { printf ("back-edge\n");
        if ((dflabels[nodes[startNode].adjNodes[i]]) < low[startNode]) 
        { low[startNode] = dflabels[nodes[startNode].adjNodes[i]];
	  printf ("low[%d] = %d on back-edge\n",startNode,low[startNode]);}
      }
      else if (backtracked[nodes[startNode].adjNodes[i]] > 0) 
      { printf("cross-edge\n");
        if ((dflabels[nodes[startNode].adjNodes[i]]) < low[startNode]) 
        { low[startNode] = dflabels[nodes[startNode].adjNodes[i]];
	  printf ("low[%d] = %d on cross-edge\n",startNode,low[startNode]);}
      }
    }
  }
  CallLevel--; 
  // pick unvisited
  if (0 == CallLevel) 
  { for (i = 0; i < numNodes; i++) 
    { if (0 == dflabels[i]) 
      { struct TreeNode *unvisted = AllStrongComponents(i);; break; }}}
  
  return tn;
}

int test(int start) 
{ struct TreeNode *DFT;
    printf("*** starting at %d\n",start);
    DFT = AllStrongComponents(start);
//    if (0 < strongcomps[0]) {printf("STRONG COMPONENTS ");StackFlush(strongcomps);printf("\n");}
//    printf ("-----\ndf-tree: node, parent, numChildren, and the list of children\n");  
//    PrintTree(DFT);
  free(DFT);
  return 0;
}

// main - calls test function
int main() 
{test(0); return 0;}
