// Read Input part from Dr. Kundu
// cc readInput.c
// Contact kundu@csc.lsu.edu for comments and questions
// This program reads the data for a graph from graph.dat.
// Shown below is a sample input file;
// 5 //numNodes
// 0 (2) 1 2 //nodeNum, degree, adjNodes
// 1 (4) 0 2 3 4 
// 2 (3) 0 1 4
// 3 (2) 1 4
// 4 (3) 1 2 3

#include<stdio.h>
#define INPUT_FILE "graph.dat"

typedef struct {
  int degree, *adjNodes;
} GraphNode;
  
GraphNode *nodes;
int numNodes;

void PrintIntVector(int *vector, int size)
{ int i;
  for (i=0; i<size; i++) printf("%d ",vector[i]);
  printf("\n");
}

void SkipToEndofLine(FILE *fpInp)
{ while (getc(fpInp) != '\n');
}

void ReadInput() //creates the digraph
{ int i, j, nodeNum;
  FILE *fp;
  if (NULL == (fp = fopen(INPUT_FILE, "r")))
  { fprintf(stderr, "Cannot open %s\n", INPUT_FILE); exit(1);}
  fscanf(fp, "%d", &numNodes); 
  SkipToEndofLine(fp);
  nodes = (GraphNode *)malloc(numNodes*sizeof(GraphNode));
  for (i=0; i<numNodes; i++) 
  { fscanf(fp, "%d", &nodeNum); 
    fscanf(fp, " ( %d ) ", &nodes[nodeNum].degree);
    nodes[nodeNum].adjNodes = (int *)malloc(nodes[nodeNum].degree*sizeof(int));
    for (j=0; j<nodes[nodeNum].degree; j++) fscanf(fp, "%d", &nodes[nodeNum].adjNodes[j]);
    SkipToEndofLine(fp);
  }  
  fclose(fp);
  printf("nodeNum (degree) adjNodes:\n");
  for (i = 0; i < numNodes; i++)
  { printf("%d (%d)", i, nodes[i].degree); 
    PrintIntVector(nodes[i].adjNodes, nodes[i].degree); }
}

// Beginning of BDE's Code

static int *dflabels,lastDfLabel,parentTreeNode;

typedef struct TreeNode {
  int dflabel,node_label,low;
  struct TreeNode *parent,*first,*last,*left,*right;
} TreeNode;

struct TreeNode *DepthFirst(int startNode)
{ int i; struct TreeNode *tn = NULL;   // the tree node we'd like to create
  // recursion unrolling condition; only if dfLabel is unset
  if (dflabels[startNode] == 0) 
  { tn = (struct TreeNode *)malloc(sizeof(TreeNode));
    tn->node_label = startNode;
    tn->parent = tn; // initialize parent as self
    tn->first = tn->last = tn->left = tn->right = NULL;
      
    for (i = 0; i < nodes[startNode].degree; i++) 
    { if (0 == dflabels[startNode]) tn->dflabel = dflabels[startNode] = tn->low = ++lastDfLabel;  // only assign dflabel once      
    
      // edge type detection
      printf("processing (%d,%d): ",tn->node_label,nodes[startNode].adjNodes[i]);    
      if (0 == dflabels[nodes[startNode].adjNodes[i]]) printf ("tree-edge"); 
      else if (-1 == dflabels[nodes[startNode].adjNodes[i]] - dflabels[startNode]) printf ("tree-edge 2nd-visit"); 
      else if (dflabels[nodes[startNode].adjNodes[i]] < dflabels[startNode]) printf ("back-edge");
      else if (dflabels[nodes[startNode].adjNodes[i]] > dflabels[startNode]) printf ("back-edge 2nd-visit");
      printf("\n");
      parentTreeNode = startNode;

      // tree node building code section
      struct TreeNode *child = DepthFirst(nodes[startNode].adjNodes[i]); // <-- recursive call      
      if (NULL != child)
      { child->left = child->right = NULL; child->parent = tn; 
        printf("backtracking %d -> %d \n",child->node_label,tn->node_label);      
        if (NULL == tn->first) tn->first = tn->last = child;
 	else { child->left = tn->last; tn->last->right = tn->last = child; }
      }
      
    }
  } return tn;
}

int CountChildren(TreeNode *tn)
{ int count = 0;
  struct TreeNode *current = tn->first; // counts siblings of tn's first child
  while (1) { if (NULL != current) { count++;current = current->right;} else break;} 
  return count;
}

void PrintTree (TreeNode *tn) 
{ int children = 0;
  if (NULL != tn->first)
  { children = CountChildren(tn);
    printf("Node=%d (parent=%d, numChildren=%d): %d\n",tn->node_label,tn->parent->node_label,children,tn->first->node_label);
    PrintTree(tn->first);}  // <-- recursive call
  else printf("Node=%d (parent=%d, numChildren=%d): \n",tn->node_label,tn->parent->node_label,children);
}

int test(int start) 
{ struct TreeNode *DFT;                                     // final tree
  int i;
  ReadInput();                                              // read input - nice if it took stdin
  dflabels = (int *)malloc(numNodes*sizeof(int));           // allocate memory for tracking dflabels
  lastDfLabel = 0;                                          // initialize to zero (i.e., reset)
  DFT = DepthFirst(start);
  printf ("df-tree: node, parent, numChildren, and the list of children\n");  
  PrintTree(DFT);                                           // prints tree in a recursive, depth-first way
  return 0;
}

int main() 
{test(0); test(1); test(4);
 return 0;}
