// Read Input part from Dr. Kundu
// cc readInput.c
// Contact kundu@csc.lsu.edu for comments and questions
// This program reads the data for a graph from graph.dat.
// Shown below is a sample input file;
// 5 //numNodes
// 0 (2) 1 2 //nodeNum, degree, adjNodes
// 1 (4) 0 2 3 4 
// 2 (3) 0 1 4
// 3 (2) 1 4
// 4 (3) 1 2 3

#include<stdio.h>

typedef struct {
  int degree, *adjNodes;
} GraphNode;
  
GraphNode *nodes;
int numNodes;

void PrintIntVector(int *vector, int size)
{ int i;
  for (i=0; i<size; i++) printf("%d ",vector[i]);
  printf("\n");
}

void SkipToEndofLine(FILE *fpInp)
{ while (getc(fpInp) != '\n');}

void ReadInput() //creates the digraph
{ int i, j, nodeNum;
  FILE *fp;
  // modified code to take in graph def via stdin
  // example1: %cat graph.dat | ./a.out
  // example2: %./a.out < graph.out
  fp = stdin;
  fscanf(fp, "%d", &numNodes); 
  SkipToEndofLine(fp);
  nodes = (GraphNode *)malloc(numNodes*sizeof(GraphNode));
  for (i=0; i<numNodes; i++) 
  { fscanf(fp, "%d", &nodeNum); 
    fscanf(fp, " ( %d ) ", &nodes[nodeNum].degree);
    nodes[nodeNum].adjNodes = (int *)malloc(nodes[nodeNum].degree*sizeof(int));
    for (j=0; j<nodes[nodeNum].degree; j++) fscanf(fp, "%d", &nodes[nodeNum].adjNodes[j]);
    SkipToEndofLine(fp);
  }  
  fclose(fp);
  printf("nodeNum (degree) adjNodes:\n");
  for (i = 0; i < numNodes; i++)
  { printf("%d (%d)", i, nodes[i].degree); 
    PrintIntVector(nodes[i].adjNodes, nodes[i].degree); }
}

// Beginning of BDE's Code
static int *dflabels,*bicomps,*low,lastDfLabel,parentTreeNode;

typedef struct TreeNode {
  int dflabel,node_label;
  struct TreeNode *parent,*first,*last,*left,*right;
} TreeNode;

// Stack Operations
void BicompPush  (int node) { bicomps[++bicomps[0]] = node; } // pre-increment  tos, add node to new tos
int  BicompPop   (void)     { return bicomps[bicomps[0]--]; } // post-decrement tos, return node at former tos
int  BicompPeek  (void)     { return bicomps[bicomps[0]]; }   // peek at top of stack
void BicompReset (void)     { bicomps[0] = 0; };
void BicompFlush (void)     { int i = 0; for (i = bicomps[0]; i > 0; i--) { printf ("%d ",BicompPop()); } } 

// DFT Search Code
struct TreeNode *DepthFirst(int startNode)
{ int i; struct TreeNode *tn = NULL;   // the tree node we'd like to create
  // recursion unrolling condition; only if dfLabel is unset
  if (dflabels[startNode] == 0) 
  { tn = (struct TreeNode *)malloc(sizeof(TreeNode));
    tn->node_label = startNode;
    tn->parent = tn; // initialize parent as self
    tn->first = tn->last = tn->left = tn->right = NULL;
    for (i = 0; i < nodes[startNode].degree; i++) 
    { // only assign dflabel once, init low    
      // add startNode to bicomps    
      if (0 == dflabels[startNode]) tn->dflabel = dflabels[startNode] = low[startNode] = ++lastDfLabel;  
      // edge type detection
      printf("processing (%d,%d): ",tn->node_label,nodes[startNode].adjNodes[i]);
      // tree edge 
      if (0 == dflabels[nodes[startNode].adjNodes[i]])
      { printf ("tree-edge\n");
        /// push node onto Bicomp stack
        BicompPush(nodes[startNode].adjNodes[i]);}
      // tree edge second visit
      else if (nodes[startNode].adjNodes[i] == parentTreeNode) printf ("tree-edge 2nd-visit\n"); 
      // back edge
      else if (dflabels[nodes[startNode].adjNodes[i]] < dflabels[startNode])
      { printf ("back-edge\n");
        /// Update low with min(dflabels(child),low(parent)), however here the "child" is an ancestor of parent       
        if ((dflabels[nodes[startNode].adjNodes[i]]) < low[startNode]) 
        { printf ("...(LOW(%d) updated: %d -> %d)\n",startNode,low[startNode],dflabels[nodes[startNode].adjNodes[i]]);         
	  low[startNode] = dflabels[nodes[startNode].adjNodes[i]];}}
      // back edge second visit
      else if (dflabels[nodes[startNode].adjNodes[i]] > dflabels[startNode]) printf ("back-edge 2nd-visit\n");
      parentTreeNode = startNode;
      // tree node building code section
      struct TreeNode *child = DepthFirst(nodes[startNode].adjNodes[i]); // <-- recursive call      
      if (NULL != child)
      { child->left = child->right = NULL; child->parent = tn; 
        if (NULL == tn->first) tn->first = tn->last = child;
 	else { child->left = tn->last; tn->last->right = tn->last = child; }	
        // backtracking...
        /// update parent's low with child's low
	if (low[child->node_label] < low[tn->node_label]) 
	{ printf ("...(LOW(%d) updated: %d -> %d)\n",tn->node_label,low[tn->node_label],low[child->node_label]);
	  low[tn->node_label] = low[child->node_label]; 
        }
        /// bicomponent detection
	if(low[child->node_label] >= dflabels[tn->node_label]) {
	  printf("Cut vertex detected: %d\nBicomp Members: [ ",tn->node_label);	  
	  /// pop out and print bicomponent members
	  while (1) 
	  { if (BicompPeek() != child->node_label) printf("%d ",BicompPop());
            else { printf("%d ",BicompPop()); break; }
          }
	  printf("]\n");
	} 		
        printf("backtracking %d -> %d\n",child->node_label,tn->node_label);	
      }
    }
  } return tn;
}

// counts the number of children of a given node
int CountChildren(TreeNode *tn)
{ int count = 0;
  struct TreeNode *current = tn->first;
  while (1) { if (NULL != current) { count++;current = current->right;} else break;} 
  return count;
}

// prints the tree
void PrintTree (TreeNode *tn) 
{ int children = 0;
  if (NULL != tn->first)
  { children = CountChildren(tn);
    printf("Node=%d (parent=%d, numChildren=%d): %d\n",tn->node_label,tn->parent->node_label,children,tn->first->node_label);
    struct TreeNode *current = tn->first;
    while (NULL != current)
    { PrintTree(current);
      current = current->right;
    }   
  }
  else printf("Node=%d (parent=%d, numChildren=%d): \n",tn->node_label,tn->parent->node_label,children);
}

// test function
int test(int start) 
{ struct TreeNode *DFT;                                     // final tree
  int i;
  ReadInput();                                              // read input - nice if it took stdin
  dflabels = (int *)malloc(numNodes*sizeof(int));           // allocate memory for tracking dflabels
  bicomps  = (int *)malloc(numNodes*sizeof(int));           // allocate memory for tracking stack of bicomponents
  bicomps[0] = 0;                                           // initialize top of stack counter
  low      = (int *)malloc(numNodes*sizeof(int));           // allocate memory for tracking lows
  lastDfLabel = 0;                                          // initialize to zero (i.e., reset)
  DFT = DepthFirst(start);
  printf ("df-tree: node, parent, numChildren, and the list of children\n");  
  PrintTree(DFT);                                           // prints tree in a recursive, depth-first way
  return 0;
}

// main - calls test function
int main() 
{test(0);
 //test(1);
 return 0;}
