// Read Input part from Dr. Kundu
#include<stdio.h>


// FIX PrintTree Output + any other issues

typedef struct {
  int degree, *adjNodes;
} GraphNode;
  
GraphNode *nodes;
int numNodes = -1;

void PrintIntVector(int *vector, int size)
{ int i;
  for (i=0; i<size; i++) printf("%d ",vector[i]);
  printf("\n");
}

void SkipToEndofLine(FILE *fpInp)
{ while (getc(fpInp) != '\n');}

void ReadInput() //creates the digraph
{ int i, j, nodeNum;
  FILE *fp;
  fp = stdin;
  fscanf(fp, "%d", &numNodes); 
  SkipToEndofLine(fp);
  nodes = (GraphNode *)malloc(numNodes*sizeof(GraphNode));
  for (i=0; i<numNodes; i++) 
  { fscanf(fp, "%d", &nodeNum); 
    fscanf(fp, " ( %d ) ", &nodes[nodeNum].degree);
    nodes[nodeNum].adjNodes = (int *)malloc(nodes[nodeNum].degree*sizeof(int));
    for (j=0; j<nodes[nodeNum].degree; j++) fscanf(fp, "%d", &nodes[nodeNum].adjNodes[j]);
    SkipToEndofLine(fp);
  }  
  fclose(fp);
  printf("nodeNum (degree) adjNodes:\n");
  for (i = 0; i < numNodes; i++)
  { printf("%d (%d)", i, nodes[i].degree); 
    PrintIntVector(nodes[i].adjNodes, nodes[i].degree); }
}

// Beginning of BDE's Code
static int *backtracked,*dflabels,*low,*strongcomps,lastDfLabel,parentTreeNode,recurseLevel;

// Stack Operations
void StrongCompPush  (int node) { strongcomps[++strongcomps[0]] = node; } // pre-increment  tos, add node to new tos
int  StrongCompPop   (void)     { return strongcomps[strongcomps[0]--]; } // post-decrement tos, return node at former tos
int  StrongCompPeek  (void)     { return strongcomps[strongcomps[0]]; }   // peek at top of stack
void StrongCompReset (void)     { strongcomps[0] = 0; };
void StrongCompFlush (void)     { int i = 0; for (i = strongcomps[0]; i > 0; i--) { printf ("%d ",StrongCompPop()); } } 
void StrongCompPrintAll (void)  { int i = 0; for (i = strongcomps[0]; i > 0; i--) { printf ("%d ",strongcomps[i]); } }
typedef struct TreeNode {
  int dflabel,node_label;
  struct TreeNode *parent,*first,*last,*left,*right;
} TreeNode;

// Strong Component Search Code
struct TreeNode *AllStrongComponents(int startNode)
{ int i; struct TreeNode *tn = NULL;   // the tree node we'd like to create
  // recursion unrolling condition; only if dfLabel is unset
  //printf("recursion level %d\n",recurseLevel);
  if (dflabels[startNode] == 0) 
  { tn = (struct TreeNode *)malloc(sizeof(TreeNode));
    tn->node_label = startNode;
    tn->parent = tn; // initialize parent as self
    tn->first = tn->last = tn->left = tn->right = NULL;
    if (0 == recurseLevel) StrongCompPush(startNode);
    recurseLevel++;
    for (i = 0; i < nodes[startNode].degree; i++) 
    { // only assign dflabel once, init low      
      if (0 == dflabels[startNode]) 
      { tn->dflabel = dflabels[startNode] = low[startNode] = ++lastDfLabel;
        printf("dfLabel( %d)=%d, parentNode=%d, low[%d]=%d\n",startNode,lastDfLabel,parentTreeNode,startNode,low[startNode]);
        printf("stack: ");StrongCompPrintAll();printf("\n");}
      printf("processing (%d,%d): ",tn->node_label,nodes[startNode].adjNodes[i]);
      if (0 == dflabels[nodes[startNode].adjNodes[i]]) 
      { printf ("tree-edge\n");
        StrongCompPush(nodes[startNode].adjNodes[i]);
	parentTreeNode = startNode;
	// tree node building code section
	struct TreeNode *child = AllStrongComponents(nodes[startNode].adjNodes[i]); // <-- recursive call      
	if (NULL != child)
	{ child->left = child->right = NULL; child->parent = tn; 
          if (NULL == tn->first) tn->first = tn->last = child;
 	  else { child->left = tn->last; tn->last->right = tn->last = child; }	
          // backtracking...
          backtracked[child->node_label]++;
	}
	printf("backtracking %d -> %d\n",nodes[startNode].adjNodes[i],startNode);
        // update parent's low with child's low
	if (low[nodes[startNode].adjNodes[i]] < low[startNode]) 
	{ low[startNode] = low[nodes[startNode].adjNodes[i]];
          printf ("low[%d] = %d on backtrack\n",startNode,low[startNode]); 
	}
	else printf("No change in low[%d] on backtrack\n",startNode);
	// check to see if child is entry node for the strong component
	if (low[nodes[startNode].adjNodes[i]] == dflabels[nodes[startNode].adjNodes[i]]) 
	{ printf ("STRONG COMPONENT ");
	  while (StrongCompPeek() != nodes[startNode].adjNodes[i]) printf("%d ",StrongCompPop());
  	  printf("%d ",StrongCompPop());
	  printf("\n");}
      }
      else if (dflabels[nodes[startNode].adjNodes[i]] < dflabels[startNode]  && backtracked[nodes[startNode].adjNodes[i]] == 0)
      { printf ("back-edge\n");
        if ((dflabels[nodes[startNode].adjNodes[i]]) < low[startNode]) 
        { low[startNode] = dflabels[nodes[startNode].adjNodes[i]];
	  printf ("low[%d] = %d on back-edge\n",startNode,low[startNode]);}
      }
      else if (backtracked[nodes[startNode].adjNodes[i]] > 0) 
      { printf("cross-edge\n");
        if ((dflabels[nodes[startNode].adjNodes[i]]) < low[startNode]) 
        { low[startNode] = dflabels[nodes[startNode].adjNodes[i]];
	  printf ("low[%d] = %d on cross-edge\n",startNode,low[startNode]);}
      }
    }
  }
  recurseLevel--; 
  return tn;
}

// returns the first explored node if it finds dflabels[i] == 0
int GetUnused (void) 
{ int i, start = -1;
  for (i = 0; i < numNodes; i++) if (0 == dflabels[i]) { start = i; break; };
  return start;
}

void init (void)
{ if (-1 == numNodes) ReadInput();                      // read input only if it has not done so yet  
  backtracked   = (int *)malloc(numNodes*sizeof(int));  // allocate memory for tracking backtracked nodes    
  dflabels      = (int *)malloc(numNodes*sizeof(int));  // allocate memory for tracking dflabels    
  low           = (int *)malloc(numNodes*sizeof(int));  // allocate memory for tracking lows	    
  strongcomps   = (int *)malloc(numNodes*sizeof(int));  // allocate memory for tracking strongcomps members	    
  lastDfLabel   = 0;                                    // initialize to zero	    
}

void finalize ()
{ free(backtracked);
  free(dflabels);
  free(low); 
  free(strongcomps);}
  
int test(int start) 
{ struct TreeNode *DFT;
  init();
  while (-1 < start)
  { printf("*** starting at %d\n",start);
    DFT = AllStrongComponents(start);
    if (0 < strongcomps[0]) {printf("STRONG COMPONENTS ");StrongCompFlush();printf("\n");}
    start = GetUnused();
//    printf ("-----\ndf-tree: node, parent, numChildren, and the list of children\n");  
//    PrintTree(DFT);
  }
  free(DFT);
  finalize();
  return 0;
}

// main - calls test function
int main() 
{ test(0);
  printf ("-----\n");  
  test(1);
  printf ("-----\n");  
  test(8);
  return 0;}
