//cc -lm -g stud/damayanti/psfq/psfq.c (-g for dbg debugger (this requires commands "dbg", "run", "file a.out", "run", and "quit")
//22 Mar 2006, contact kundu@csc.lsu.edu or (215) 578-2246 for comments and questions
//This program simulates a reliable transport protocol for data transfer from sink to destinations i (determined by nodeTTL(i) = 0).
//(TTL = time to live) in a sensor network.  Some details:
//1.  outgoing event/message priority: TQ (transmission), RTQ (retransmission), RRTQ (request for retransmission, source-node has no RRTQ;
//    a node cannot send two RRT for the same or a different packet before a delay of at least RRT.period, which is a factor of degrees[node];
//    (but has TQ and RTQ) and destination nodes have no TQ (but has RTQ and RRTQ).
//2.  TQ is initialized when a node gets its first packet; source's TQ is initialized with all packets;
//3.  nodeTTL(i) is updated on receiving a T or RT or RRT message;
//4.  no two nodes which are at a distance <= 2 can be selected for any of T/RT/RRT at the same time and a node ust have TQ non-empty or RTQ 
//    non-empty, or RRTQ non-empty and time - node's lastRRTtoSendTime >= node's RRTperiod.
//FUTURE POSSIBILITIES
//1. Include maxTTL of all T(pj) and RT(pj) for each packet pj in dataCache;  nodeTTL then equals max of maxTTL(pj) for pj in dataCache
//2. Add both T and RT facility for destination-nodes with nodeTTL = 0.

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define INPUT_FILE_NAME   "psfq.dat"
#define PACKET_ORDER_FILE_NAME  "psfq.packetOrder.out"
#define MAXRUNS           100
#define MAX_STRNG_LENGTH  200
#define max(x,y)          ((x) > (y) ? (x) : (y))
#define stringMssgType(i)  (0 == (i) ? "T" : ((1 == (i)) ? "RT" : "RRT"))
#define selectPacketForRRTMethod   0 //0 == original round robin, 
                                     // 1 == fcairst packet contained by any neighbor in remDC
                                     // 2 == first packet contained by any neighbor in any cache or queue
                                     // 3 == #1 + desperation mode
#define blindRRTFactor    1          // Limits blind RRTs to blindRRTFactor/degrees[i]

typedef enum {T, RT, RRT, NONE, numMssgTypes} MssgType;  //NONE is used for indicating no mssg as Input
typedef enum {SOURCE, INTERMEDIATE, DESTINATION, OTHER, numNodeTypes} NodeType; 

typedef struct {
  int packetNum, packetTTL, senderNodeNum, *remDC;
  MssgType mssgType;
} MssgSentRecvd;

typedef struct {
  MssgType packetStatus;   //T=in TQ, RT=in RTQ, RRT = in RRTQ = missing, NONE = recvd but not in TQ/RTQ
  int nextInQ, prevInQ;    //for each of TQ, RTQ, or RRTQ, a packet is at most one queue; prevInQ is used only for deletion from RRTQ
} DataCacheItemInfo;

typedef struct TimeProfile {
  int numNodesSelected[numMssgTypes], time;
  struct TimeProfile *next; //linked list; first is the most recent time
} TimeProfile;

typedef struct {          
  NodeType nodeType;                         
  MssgSentRecvd mssgSentRecvd;  //= mssg sent (set by FindTransmNodes-function) for nodes selected for transmission
                                //= mssg received by successful transmission from a selected neighbor, if any
  DataCacheItemInfo *dataCache; //array of size numPackets
  int timeRecvdFirstPacket,     //= 0 for source node; used for checkting time for RRT
      numPacketsInDataCache,    
      *orderPacketsReceived,    //orderPacketsReceived[i] = ith new packet received/added (via T or RT) to the dataCache
      *successInTransm,         //arraySize = degrees(i); successInTransm[j] = 1 means success in typeOfTransm to jth adjNode
      nodeTTL,                  //max of all TTL of all mssg received, including RRT; initialization: sourceTTL for source-node, -1 for others
      nextRRTtoSend,            //initialization: 0 except for source-node (for which it does not apply)
      nextRRTsendTime,          //timeRecvdFirstPacket +  RRTperiod
      candidateCounts[numMssgTypes], //number of times node is a candiddate for selection; collected for one complete run of PSFQ
      selectionCounts[numMssgTypes], //collected for one complete run of PSFQ
      *succMssgCounts[numMssgTypes], //array size degrees(node); collected for one complete run of PSFQ
      firstTQ, lastTQ, firstRTQ, lastRTQ, //initialization: firstTQ = 0 for source; oterwise, firstTQ = firstRTQ = numPackets (meaning empty)
      firstRRTQ,                          //initialization: numPackets for source, and to 0 otherwise (with RRTQ containining all packets) 
                                          //lastRRTQ not used since no items even get added to RRTQ
      RRTperiod,                 //two consecutive RRT must be separated by at least RRTperiod = degreed[i].RRTdelayFactor
      **recentRemDCofNeighbors,   // stores most recently seen remDC(i)
      *numBlindRRTbyPacket;
} NodeState;

NodeState *nodeStates=NULL;      //array of size numNodes
TimeProfile *firstTimeProfile=NULL;

char networkName[MAX_STRNG_LENGTH];
char *strOfCandidatesForSelection; //contains each candidate node and one of T/RT/RRT that makes it a candidate; used for printing
int  numNodes,                   //number of nodes in network
     *degrees, **adjNodes, **adjNodePositions,  //degrees[i] = degree of node i; adjNodePositions is a look up table
     source, sourceTTL, maxDestNodeTTL, //maxDestNodeTTL = 0 for now (maybe generalized in future to > 0)
     numPackets,                 //number of packets to be delivered from source to destination(s)
     numNodesSelectedForTransm, *nodesSelectedForTransm,       //Contains nodes which can transmit during each time point in execution of PSFQ
     numPSFQruns, numSuccRuns, 
     totalTime, **totalCandidateCounts, **totalSelectionCounts, //for each node and each mssgType
     ***totalSuccMssgCounts,                                    //from each node i to each node j and for each mssgType
     NEED_NODETTL_POSITIVE_FOR_T_AND_RT,  //=1 normal case
	 RRT_YES_NO,                          //=1 means uses RRT, normal case; =2 means receiver of RRT-mssg ignores the
	                                      //request if packetTTL in the mssg <= receiving node's nodeTTL
     PRINT_DETAILS, STEP_EACH_TIME_POINT, PRINT_JUST_FINAL_TOTALS, PRINT_TIME_PROFILE_ONE_ITEM_PER_LINE;  
double probSuccTransm, RRTdelayFactor; //RRTdelayfactor*degrees[i] is that actual delat used for RRTs from node i
FILE *outp, *FHpacketOrderOut;

void SkipToEndOfLine(FILE *inp)
{ while (fgetc(inp) != '\n');
}

void PrintNodeDataCache(FILE *outp, int time)
{ int i, j, k, strLength, strTlength, strRTlength, strDClength, strOrdPacketsReceivedLength;
  static char buff[4], *str, *strT, *strRT, *strDC, *strOrdPacketsReceived;  //buff should accomodate nodeNum with 2 digits and a preceding blank for numPackets <= 99
  if (NULL == str) {
     str = (char *)malloc((1 + 15)*sizeof(char));                                  //,12(T) = 6 chars 
     strT = (char *)malloc((1 + 5 + 3*numPackets)*sizeof(char));                   //5 = length of "TQ() " 
     strRT = (char *)malloc((1 + 6 + 3*numPackets)*sizeof(char));                  //6 = length of "RTQ() " 
     strDC = (char *)malloc((1 + 8 + 3*numPackets)*sizeof(char));                  //8 = length of "RemDC() " 
     strOrdPacketsReceived = (char *)malloc((1 + 10 + 3*numPackets)*sizeof(char)); //10 = length of "OrdRecvd()" 
  }
  fprintf(outp, "               timeRecvdFirstPacket, nextRRTtoSend, nextRRTsendTime, and\n");
  fprintf(outp, "Time Node(TTL) TQ, RTQ, and the remainding packets in dataCache\n");
  fprintf(outp, "=================================================================================\n");
  for (i=0; i<numNodes; i++) 
      if (nodeStates[i].numPacketsInDataCache > 0) {
         sprintf(str, "%4d ", time);
         sprintf(str+5, "%2d(%2d)", i, nodeStates[i].nodeTTL); strLength = strlen(str);
         if (SOURCE == nodeStates[i].nodeType) sprintf(str+strLength, "[S] ");
         else if (DESTINATION == nodeStates[i].nodeType) sprintf(str+strLength, "[D] ");
         else sprintf(str+strLength, "    ");
         sprintf(strT, "TQ("); strTlength = 3;
         for (j=nodeStates[i].firstTQ; j<numPackets; j=nodeStates[i].dataCache[j].nextInQ) {
             sprintf(buff, "%d,", j); 
             strcat(strT + strTlength, buff); strTlength += strlen(buff);
         }
         if (',' == strT[strTlength - 1]) sprintf(strT+strTlength-1, ") ");
         else strT[0] = '\0';
         sprintf(strRT, "RTQ("); strRTlength = 4;
         for (j=nodeStates[i].firstRTQ; j<numPackets; j=nodeStates[i].dataCache[j].nextInQ) {
             sprintf(buff, "%d,", j); 
             strcat(strRT + strRTlength, buff); strRTlength += strlen(buff);
         }
         if (',' == strRT[strRTlength - 1]) sprintf(strRT+strRTlength-1, ") ");
         else strRT[0] = '\0';
         sprintf(strDC, "RemDC("); strDClength = 6;
         for (j=k=0; j<numPackets; j++) //k = numItems included in remDC
             if (NONE == nodeStates[i].dataCache[j].packetStatus) {
                k++;
                sprintf(buff, "%d,", j); 
                strcat(strDC + strDClength, buff); strDClength += strlen(buff);
             }
         if (',' == strDC[strDClength - 1]) sprintf(strDC+strDClength-1, ") ");
         else strDC[0] = '\0'; 
         if (i == source) strOrdPacketsReceived[0] = '\0';
         else { sprintf(strOrdPacketsReceived, "OrdRecvd("); strOrdPacketsReceivedLength = 9;
                for (j=0; j<nodeStates[i].numPacketsInDataCache; j++) {
                    sprintf(buff, "%d,", nodeStates[i].orderPacketsReceived[j]); 
                    strcat(strOrdPacketsReceived + strOrdPacketsReceivedLength, buff); strOrdPacketsReceivedLength += strlen(buff);
                }
                if (',' == strOrdPacketsReceived[strOrdPacketsReceivedLength - 1]) sprintf(strOrdPacketsReceived+strOrdPacketsReceivedLength-1, ")");
                else strOrdPacketsReceived[0] = '\0'; 
              } 
         fprintf(outp, "%s(%2d,%d,%2d) %s%s%s%s\n", str, nodeStates[i].timeRecvdFirstPacket, nodeStates[i].nextRRTtoSend, 
                       nodeStates[i].nextRRTsendTime, strT, strRT, strDC, strOrdPacketsReceived);
      }
  fprintf(outp, "=================================================================================\n");
}

void PrintTransmStatus(FILE *outp, int time)
{ int i, j, packetNum, packetTTL, strLength, strTransmLength; 
  MssgType mssgType;
  static char buff[8], *str, *strTransm;  //buff should accomodate "12(10) ", 2 digits for nodeNum <= 99, 2 digits for TTL, '(,, ')', and " "
  static int maxDegree, *tempSelectedForTransm; //item = 1 if node i is selected;
  if (NULL == str) {
     for (i=maxDegree=0; i<numNodes; i++) maxDegree = max(maxDegree, degrees[i]);
     str = (char *)malloc((1 + 33)*sizeof(char)); 
     strTransm = (char *)malloc((1 + 2 + 4*maxDegree)*sizeof(char)); 
     tempSelectedForTransm = (int *)calloc(numNodes, sizeof(int));
  }
  else for (i=0; i<numNodes; i++) tempSelectedForTransm[i] = 0;
  fprintf(outp, "candidatesForSelection: %s\n", strOfCandidatesForSelection);
  fprintf(outp, "               T/RT/RRT   Packet\n");
  fprintf(outp, "Time Node(TTL) (RRTdelay) (TTL)  Recving nodes (+ = success, - = failure)\n");
  fprintf(outp, "_________________________________________________________________________________\n");
  for (i=0; i<numNodesSelectedForTransm; i++) //order the nodes selected for transmission
      tempSelectedForTransm[nodesSelectedForTransm[i]] = 1;
  for (i=0; i<numNodes; i++) 
      if (1 == tempSelectedForTransm[i]) {
         packetNum = nodeStates[i].mssgSentRecvd.packetNum;
         mssgType = nodeStates[i].mssgSentRecvd.mssgType;
         packetTTL = nodeStates[i].mssgSentRecvd.packetTTL;
         sprintf(str, "%4d %2d(%2d)    ", time, i, nodeStates[i].nodeTTL); strLength = 15;
         if (T == mssgType) sprintf(str+strLength, "T          ");
         else if (RT == mssgType) sprintf(str+strLength, "RT         ");
         else sprintf(str+strLength, "RRT(%2d)    ", nodeStates[i].RRTperiod);
         strLength = 26;
         sprintf(buff, "%d(%2d) ", packetNum, packetTTL);
         if (strlen(buff) < 7) { buff[6] = ' '; buff[7] = '\0'; }
         sprintf(str+strLength, "%s", buff);
         sprintf(strTransm, "(");
         strTransmLength = 1;
         for (j=0; j<degrees[i]; j++) {
             if (1 == nodeStates[i].successInTransm[j]) sprintf(buff, "+%d,", adjNodes[i][j]); 
             else sprintf(buff, "-%d,", adjNodes[i][j]); 
             strcat(strTransm + strTransmLength, buff); strTransmLength += strlen(buff);
         }
         strTransm[strTransmLength-1] = ')';
         fprintf(outp, "%33s%s\n", str, strTransm);
      }
  fprintf(outp, "_________________________________________________________________________________\n");
}

void PrintOneRunStats(FILE *outp)
{ int i, j, strLength, strLengthT, strLengthRT, strLengthRRT;
  MssgType mssgType;
  static char *str, *strT, *strRT, *strRRT;    //allocated only once
  static int *counts;  //allocated only once
  if (NULL == str) {
     str = (char *)malloc((1 + 25 * 5*numNodes)*sizeof(char)); //15 = length of "Node T/RT/RRT  T/RT/RRT  "
     strT = (char *)malloc((1 + 25 * 5*numNodes)*sizeof(char)); //15 = length of "Node T/RT/RRT  T/RT/RRT  "
     strRT = (char *)malloc((1 + 25 * 5*numNodes)*sizeof(char)); //15 = length of "Node T/RT/RRT  T/RT/RRT  "
     strRRT = (char *)malloc((1 + 25 * 5*numNodes)*sizeof(char)); //15 = length of "Node T/RT/RRT  T/RT/RRT  "
     counts = (int *)malloc(numNodes*sizeof(int));
  }
  for (i=strLength=0; i<numNodes; i++) 
      { sprintf(str+strLength, "%4d ", i); strLength +=5; }
  fprintf(outp, "     Candidate Selected  Successful transmissions to other nodes (zero rows not shown)\n");
  fprintf(outp, "Node T/RT/RRT  T/RT/RRT  %s\n", str);
  fprintf(outp, "_________________________________________________________________________________\n");
  for (i=0; i<numNodes; i++) {
      if (nodeStates[i].selectionCounts[T]) {
         sprintf(strT, "%4d %4d(T)   %4d(T)   ", i, nodeStates[i].candidateCounts[T], nodeStates[i].selectionCounts[T]);
         for (j=0; j<numNodes; j++) counts[j] = 0;
         for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = nodeStates[i].succMssgCounts[T][j];
         for (j=0, strLengthT=25; j<numNodes; j++)
             { sprintf(strT+strLengthT, "%4d ", counts[j]); strLengthT += 5; }
         fprintf(outp, "%s\n", strT);
      }
      if (nodeStates[i].selectionCounts[RT]) {
		 if (nodeStates[i].selectionCounts[T])
            sprintf(strRT, "     %4d(RT)  %4d(RT)  ", nodeStates[i].candidateCounts[RT], nodeStates[i].selectionCounts[RT]);
         else sprintf(strRT, "%4d %4d(RT)  %4d(RT)  ", i, nodeStates[i].candidateCounts[RT], nodeStates[i].selectionCounts[RT]);
         for (j=0; j<numNodes; j++) counts[j] = 0;
         for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = nodeStates[i].succMssgCounts[RT][j];
         for (j=0, strLengthRT=25; j<numNodes; j++)
             { sprintf(strRT+strLengthRT, "%4d ", counts[j]); strLengthRT += 5; }
         fprintf(outp, "%s\n", strRT);
      }
      if (nodeStates[i].selectionCounts[RRT]) {
		 if (nodeStates[i].selectionCounts[T] || nodeStates[i].selectionCounts[RT])
            sprintf(strRRT, "     %4d(RRT) %4d(RRT) ", nodeStates[i].candidateCounts[RRT], nodeStates[i].selectionCounts[RRT]);
		 else sprintf(strRRT, "%4d %4d(RRT) %4d(RRT) ", i, nodeStates[i].candidateCounts[RRT], nodeStates[i].selectionCounts[RRT]);
         for (j=0; j<numNodes; j++) counts[j] = 0;
         for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = nodeStates[i].succMssgCounts[RRT][j];
         for (j=0, strLengthRRT=25; j<numNodes; j++)
             { sprintf(strRRT+strLengthRRT, "%4d ", counts[j]); strLengthRRT += 5; }
         fprintf(outp, "%s\n", strRRT);
      }
  }
  fprintf(outp, "_________________________________________________________________________________\n");
}

void ReadInputNetwork()
{ int i, j, node;
  FILE *inp;
  if (NULL == (inp = fopen(INPUT_FILE_NAME, "r"))) { printf("error in opening %s\n", INPUT_FILE_NAME); exit(1); }
  fscanf(inp, "%s", networkName, inp); SkipToEndOfLine(inp);
  fscanf(inp, "%d", &numNodes); SkipToEndOfLine(inp);
  degrees = (int *)malloc(numNodes * sizeof(int));
  adjNodes = (int **)calloc(numNodes, sizeof(int *));  
  adjNodePositions = (int **)malloc(numNodes * sizeof(int *));
  for (i=0; i<numNodes; i++) {
      fscanf(inp, "%d ", &node); fscanf(inp, "(%d):", &degrees[node]);
      adjNodes[node] = (int *)malloc(degrees[node] * sizeof(int));    // variably sized array of int arrays
      adjNodePositions[node] = (int *)malloc(numNodes * sizeof(int)); // numNodes x numNodes
      for (j=0; j<degrees[node]; j++) 
      { fscanf(inp, "%d", &adjNodes[node][j]);           // tracks jth neighbor of node (variably sized)
        adjNodePositions[node][adjNodes[node][j]] = j;   // given node and neighbor j, returns j's index in *adjNodes[node]
      }
      SkipToEndOfLine(inp);
  }
  fscanf(inp, "%d %d %d %d %lf %lf", &source, &sourceTTL, &maxDestNodeTTL, &numPackets, &probSuccTransm, &RRTdelayFactor); 
SkipToEndOfLine(inp);
  fscanf(inp, "%d", &numPSFQruns); SkipToEndOfLine(inp);
  fscanf(inp, "%d %d %d %d", &RRT_YES_NO, &NEED_NODETTL_POSITIVE_FOR_T_AND_RT, &PRINT_DETAILS, &STEP_EACH_TIME_POINT); SkipToEndOfLine(inp); 
  SkipToEndOfLine(inp);  //keep this; datafile has an extra line before next input PRINT_JUST_FINAL_TOTALS
  if (1 == STEP_EACH_TIME_POINT) PRINT_DETAILS = 1;
  fscanf(inp, "%d", &PRINT_JUST_FINAL_TOTALS); SkipToEndOfLine(inp); 
  if (1 == PRINT_JUST_FINAL_TOTALS) PRINT_DETAILS = STEP_EACH_TIME_POINT = 0;
  fscanf(inp, "%d", &PRINT_TIME_PROFILE_ONE_ITEM_PER_LINE); SkipToEndOfLine(inp); 
  fclose(inp);
  //print input data
  printf("networkName: %s, numNodes=%d, source=%d, sourceTTL=%d, maxDestNodeTTL=%d, numPackets=%d, probSuccTransm=%0.2f, numPSFQruns=%d,\n",
         networkName, numNodes, source, sourceTTL, maxDestNodeTTL, numPackets, probSuccTransm, numPSFQruns);
  printf("           : RRTdelayFactor=%0.2f\n", RRTdelayFactor);
  for (i=0; i<numNodes; i++) {
      printf("%2d(%2d):", i, degrees[i]);
      for (j=0; j<degrees[i]; j++)
          printf(" %d", adjNodes[i][j]);
      printf("\n");
  }
  printf("numPSFQruns(includes unsuccessful runs, if any)=%d\n", numPSFQruns); printf("-----------------------------------\n");
}

void InitNodeStates() //needed for each run of PSFQ()
{ int i, j, k;
  if (NULL == nodeStates) { //applies for first call to PSFQ
     nodeStates = (NodeState *)calloc(numNodes, sizeof(NodeState));
     for (i=0; i<numNodes; i++) { //some of the items for source is reinitialized separately
         nodeStates[i].nodeType = OTHER;
         nodeStates[i].mssgSentRecvd.mssgType = NONE;
         nodeStates[i].dataCache = (DataCacheItemInfo *)calloc(numPackets, sizeof(DataCacheItemInfo));
         nodeStates[i].orderPacketsReceived = (int *)malloc(numPackets * sizeof(int));
         nodeStates[i].firstTQ = nodeStates[i].lastTQ = numPackets;  //is changed to 0 for source-node
         nodeStates[i].firstRTQ = nodeStates[i].lastRTQ = numPackets; //nodeStates[i].firstRRTQ = 0 by calloc
         // initialize components used to track neighbor's data cache contents
         nodeStates[i].mssgSentRecvd.remDC = (int *)malloc(numPackets * sizeof(int)); //passes i's remDC flag in message
         nodeStates[i].recentRemDCofNeighbors = (int **)malloc(degrees[i] * sizeof(int *));
         nodeStates[i].numBlindRRTbyPacket = (int *)malloc(numPackets * sizeof(int));
         // init nodeStates[i].recentRemDCofNeighbors[j]
         for (j=0; j<degrees[i]; j++) {   
             nodeStates[i].recentRemDCofNeighbors[j] = (int *)malloc(numPackets * sizeof(int));
         }
         for (j=0; j<numPackets; j++) {
             nodeStates[i].dataCache[j].packetStatus = RRT; //is changed to T for source-node
             nodeStates[i].dataCache[j].nextInQ = j+1; nodeStates[i].dataCache[j].prevInQ = j-1;
             nodeStates[i].numBlindRRTbyPacket[j] = 0;
         }
         nodeStates[i].timeRecvdFirstPacket = -1;
         nodeStates[i].nodeTTL = -1; nodeStates[i].nextRRTtoSend = 0; 
         nodeStates[i].successInTransm = (int *)malloc(degrees[i] * sizeof(int));
         for (j=0; j<numMssgTypes; j++) {
             nodeStates[i].selectionCounts[j] = nodeStates[i].candidateCounts[j] = 0;
             nodeStates[i].succMssgCounts[j] = (int *)calloc(degrees[i], sizeof(int));
         }
         nodeStates[i].RRTperiod = ceil(degrees[i]*RRTdelayFactor);

     }
     nodeStates[source].nodeType = SOURCE;
     for (j=0; j<numPackets; j++) {
         nodeStates[source].dataCache[j].packetStatus = T;
         nodeStates[source].orderPacketsReceived[j] = j;
     }
     nodeStates[source].nextRRTtoSend = nodeStates[source].numPacketsInDataCache = numPackets;
     nodeStates[source].nodeTTL = sourceTTL; 
     nodeStates[source].firstTQ = 0; nodeStates[source].lastTQ = numPackets - 1;
     nodeStates[source].firstRRTQ = numPackets;
  }
  else { //free all memory before reallocating for second and successive calls to PSFQ in case of numPSFQruns > 1 (i.e., average case analysis)
         for (i=0; i<numNodes; i++) { //some of the items for source is reinitialized separately
             free(nodeStates[i].dataCache);
             free(nodeStates[i].orderPacketsReceived);
             free(nodeStates[i].successInTransm);
             for (j=0; j<numMssgTypes; j++) 
                 free(nodeStates[i].succMssgCounts[j]);
         }
         free(nodeStates); nodeStates = NULL;
         InitNodeStates();  //reallocate
      }
}

//Take the packet out of RRTQ, update one or both of firstRRT and nextRRTtoSend; packet has TTL >= 0
void ReceiveTandRT(int node, int packetNum, int senderNodeNum, int time, int *remDC)
{ int i,j,nextPacketNumInRRTQ=nodeStates[node].dataCache[packetNum].nextInQ, 
      prevPacketNumInRRTQ=nodeStates[node].dataCache[packetNum].prevInQ;
       
  //take packetNum out of RRTQ
  if (packetNum == nodeStates[node].firstRRTQ) {
     nodeStates[node].firstRRTQ = nextPacketNumInRRTQ;
     nodeStates[node].dataCache[packetNum].prevInQ = -1;
  }
  else { nodeStates[node].dataCache[prevPacketNumInRRTQ].nextInQ = nextPacketNumInRRTQ; 
         if (nextPacketNumInRRTQ < numPackets) nodeStates[node].dataCache[nextPacketNumInRRTQ].prevInQ = prevPacketNumInRRTQ; 
       }
  //update to nextRRTtoSend
  if (packetNum == nodeStates[node].nextRRTtoSend) {
     if (numPackets == (nodeStates[node].nextRRTtoSend = nextPacketNumInRRTQ))
        nodeStates[node].nextRRTtoSend = nodeStates[node].firstRRTQ; //= numPackets if RRTQ became empty
  }
  nodeStates[node].dataCache[packetNum].packetStatus = T;
  nodeStates[node].orderPacketsReceived[nodeStates[node].numPacketsInDataCache++] = packetNum;
  if (1 == nodeStates[node].numPacketsInDataCache) 
     { nodeStates[node].timeRecvdFirstPacket = time; nodeStates[node].nextRRTsendTime = time + nodeStates[node].RRTperiod; }
  if (numPackets == nodeStates[node].firstTQ) nodeStates[node].firstTQ = packetNum;
  else nodeStates[node].dataCache[nodeStates[node].lastTQ].nextInQ = packetNum;
  nodeStates[node].lastTQ = packetNum;
  nodeStates[node].dataCache[packetNum].nextInQ = numPackets;
  // update receiver's recentRemDCofNeighbor with sender's latest info
  for (i=0;i<numPackets;i++)
      nodeStates[node].recentRemDCofNeighbors[adjNodePositions[node][senderNodeNum]][i] = remDC[i];   

  //simulate passing of sender's datacache content bit flag
  //for (i=0;i<numPackets;i++) {
  //  if (nodeStates[senderNodeNum].dataCache[i].packetStatus != RRT)
  //        nodeStates[node].recentRemDCofNeighbors[adjNodePositions[node][senderNodeNum]][i] = 1;
  //}    
  //DEBUG/TEST output print what node knows about senderNodeNum's datacache
  //printf("\n------0==inTQ, 1==inRTQ, 2==missing(RRT), 3==inDC------\n$debug: packet %d, sent by node %d, was received by node %d at time %d\n",packetNum,senderNodeNum,node,time);
  //printf("$debug: What %d knows of %d's DC: ( ",node,senderNodeNum);
  //for (i=0;i<numPackets;i++)
  //    printf ("%d ",nodeStates[node].recentRemDCofNeighbors[adjNodePositions[node][senderNodeNum]][i]);
  //printf(")\n");
  //printf("$debug: %d's actual DC         : ( ",senderNodeNum);
  //for (i=0;i<numPackets;i++)
  //    printf("%d ",nodeStates[senderNodeNum].dataCache[i].packetStatus); 
  //printf(")\n$debug: %d's DC                : ( ",node);
  // for (i=0;i<numPackets;i++)
  //    printf("%d ",nodeStates[node].dataCache[i].packetStatus); 
  // printf(")\n------\n");
  //DEBUG/TEST output print what node knows about senderNodeNum's datacache
}

//packet is in dataCache of receivingNode and it has status NONE (= received, but not in TQ or RTQ)

void ReceiveRRT(int receivingNode, int packetNum) 
{ if (numPackets == nodeStates[receivingNode].firstRTQ) nodeStates[receivingNode].firstRTQ = packetNum;
  else nodeStates[receivingNode].dataCache[nodeStates[receivingNode].lastRTQ].nextInQ = packetNum;
  nodeStates[receivingNode].lastRTQ = packetNum;
  nodeStates[receivingNode].dataCache[packetNum].nextInQ = numPackets;
  nodeStates[receivingNode].dataCache[packetNum].packetStatus = RT;
}

// Will implement selection process for next RRT
// uses nested returns to break out of loop - beware
int selectPacketNumForRRT(int sendingNode)
{ int i, j, ret=numPackets, *missing, numMissing=0; 
  switch (selectPacketForRRTMethod) { // #define'd for now
    case 0: // get next in line, round robin style
      return nodeStates[sendingNode].nextRRTtoSend;
      break;
    case 1: // get the first packet found to be contained by a neighbor's remDC
      for (i=0; i<numPackets; i++) { 
        if (RRT == nodeStates[sendingNode].dataCache[i].packetStatus) {
          for (j=0; j<degrees[sendingNode]; j++) { 
            if (NONE == nodeStates[sendingNode].recentRemDCofNeighbors[j][i]) {
              return i;
            }
          } 
        }
      } 
    break;
    case 2: // get the first packet found to be contained by a neighbor
      for (i=0; i<numPackets; i++) { 
        if (RRT == nodeStates[sendingNode].dataCache[i].packetStatus) {
          for (j=0; j<degrees[sendingNode]; j++) { 
            if (RRT != nodeStates[sendingNode].recentRemDCofNeighbors[j][i]) {
              return i;
            }
          } 
        }
      } 
    break;
    case 3: // 1 + desparation mode I
      for (i=0; i<numPackets; i++) { 
        if (RRT == nodeStates[sendingNode].dataCache[i].packetStatus) {
          for (j=0; j<degrees[sendingNode]; j++) { 
            if (NONE == nodeStates[sendingNode].recentRemDCofNeighbors[j][i]) {
              return i;
            }
          } 
          if (blindRRTFactor/degrees[sendingNode] >= nodeStates[sendingNode].numBlindRRTbyPacket[i]) { 
            ret = i; // in desperation, ask for that which we think no one has
          }
        }
      }
      if ( numPackets > ret )
          nodeStates[sendingNode].numBlindRRTbyPacket[ret]++; 
    break;
    case 4: // 3 + packet is random, unlike above
      missing = (int *)malloc(numPackets*sizeof(int));
      for (i=0; i<numPackets; i++) { 
        if (RRT == nodeStates[sendingNode].dataCache[i].packetStatus) {
          for (j=0; j<degrees[sendingNode]; j++) { 
            if (NONE == nodeStates[sendingNode].recentRemDCofNeighbors[j][i]) {
              return i;
            }
          } 
          if (blindRRTFactor/degrees[sendingNode] >= nodeStates[sendingNode].numBlindRRTbyPacket[i]) { 
            missing[numMissing++] = i; // in desperation, ask for that which we think no one has
          }
        }
      }
      if ( 0 < numMissing ) {
         ret = missing[rand() % numMissing]; 
         nodeStates[sendingNode].numBlindRRTbyPacket[ret]++;
         //printf("asking for %d\n",ret);
      }
    break;
  } 
  // indicates that no packets were selected
  return ret;
}

void FindTransmNodes(int time)
{ int i, j, k, strLength, adjNode, *temp, numCandidateTransmNodes, packetToSend; 
  static int  *numAdjDist2Nodes, **adjDist2Nodes, //adjDist2Nodes[i][j] = 1 if dist(i, j) <= 2; we compute them once for all calls to FindTransmNodes()
              //for next three we avoid repeated allocation
              *positions, //positions[i] = position of i in remainderOfNodeSubset (if >= 0); used for deletion of i from remainderOfNodeSubset
              *candidateTransmNodes; //set of nodes where one of TQ/RTQ/RRTQ is not NULL
  char buff[3]; //1 + 2 for nodeNum <= 99
  if (adjDist2Nodes == NULL) {
     adjDist2Nodes = (int **)malloc(numNodes * sizeof(int *));
     numAdjDist2Nodes = (int *)malloc(numNodes * sizeof(int));
     positions = (int *)malloc(numNodes * sizeof(int));
     candidateTransmNodes = (int *)malloc(numNodes*sizeof(int));
     for (i=0; i<numNodes; i++) {
         temp = (int *)calloc(numNodes, sizeof(int)); 
         for (j=0; j<degrees[i]; j++) {
             adjNode = adjNodes[i][j]; temp[adjNode] = 1;  //distance 1 node
             for (k=0; k<degrees[adjNode]; k++)            //distance 2 nodes
                 temp[adjNodes[adjNode][k]] = 1; 
         } 
         temp[i] = 0; //keep it here; it was set to 1 if degrees(i) > 0 in the k-for-loop
         for (j=numAdjDist2Nodes[i]=0; j<numNodes; j++) //push all items to the left, replacing 1's by the index of the item.
             if (temp[j] == 1) temp[numAdjDist2Nodes[i]++] = j;
         adjDist2Nodes[i] = temp;  //i is not included in adjDist2Nodes[i] even if degree(i) = 0
     }
     nodesSelectedForTransm = (int *)malloc(numNodes*sizeof(int));  
     strOfCandidatesForSelection = (char *)malloc((1+8*numNodes) * sizeof(char)); //8 = 2 + 6; 2 for nodeNum <= 99), and 6 = length of "(RRT) "
  }
  for (i=strLength=numCandidateTransmNodes=0,strOfCandidatesForSelection[0]='\0'; i<numNodes; i++) {
      sprintf(buff, "%d", i);
      //BDE - build T & RT message - added senderNodeNum field...updating
      nodeStates[i].mssgSentRecvd.senderNodeNum = i;
      if ((nodeStates[i].firstTQ < numPackets) &&
	      ((nodeStates[i].nodeTTL > 0) || !(NEED_NODETTL_POSITIVE_FOR_T_AND_RT))) {
         sprintf(strOfCandidatesForSelection + strLength, "%s", buff); strLength += strlen(buff);
         sprintf(strOfCandidatesForSelection + strLength, "(T) "); strLength += 4;
         candidateTransmNodes[numCandidateTransmNodes++] = i;
         nodeStates[i].candidateCounts[T]++;
         nodeStates[i].mssgSentRecvd.packetNum = nodeStates[i].firstTQ;
         nodeStates[i].mssgSentRecvd.packetTTL = nodeStates[i].nodeTTL - 1;
         nodeStates[i].mssgSentRecvd.mssgType = T;
      }
      else if ((nodeStates[i].firstRTQ < numPackets) &&
			   ((nodeStates[i].nodeTTL > 0) || !(NEED_NODETTL_POSITIVE_FOR_T_AND_RT))) {
              sprintf(strOfCandidatesForSelection + strLength, "%s", buff); strLength += strlen(buff);
              sprintf(strOfCandidatesForSelection + strLength, "(RT) "); strLength += 5;
              candidateTransmNodes[numCandidateTransmNodes++] = i;
              nodeStates[i].candidateCounts[RT]++;
              nodeStates[i].mssgSentRecvd.packetNum = nodeStates[i].firstRTQ;
              nodeStates[i].mssgSentRecvd.packetTTL = nodeStates[i].nodeTTL - 1;
              nodeStates[i].mssgSentRecvd.mssgType = RT;
           }
      else if ((RRT_YES_NO) && (nodeStates[i].numPacketsInDataCache < numPackets) && 
               (time >= nodeStates[i].nextRRTsendTime) && (nodeStates[i].nodeTTL >= 0)) {
              packetToSend = selectPacketNumForRRT(i);
              //printf("$debug: %d sending RRT to neighbors for packet %d\n",i,packetToSend);
              if (numPackets > packetToSend) { // packetToSend == numPackets indicates DO NOT SEND RRT 
                sprintf(strOfCandidatesForSelection + strLength, "%s", buff); strLength += strlen(buff);
                sprintf(strOfCandidatesForSelection + strLength, "(RRT) "); strLength += 6;
                candidateTransmNodes[numCandidateTransmNodes++] = i;
                nodeStates[i].candidateCounts[RRT]++;
                nodeStates[i].mssgSentRecvd.packetNum = packetToSend;//nodeStates[i].nextRRTtoSend;
                nodeStates[i].mssgSentRecvd.packetTTL = nodeStates[i].nodeTTL - 1;
                nodeStates[i].mssgSentRecvd.mssgType = RRT;
              } 
           }
  }

  //printf("candidatesForSelection: %s\n", strOfCandidatesForSelection);
  numNodesSelectedForTransm = 0;

  for (i=0; i<numNodes; i++) positions[i] = -1;
  for (i=0; i<numCandidateTransmNodes; i++)
      positions[candidateTransmNodes[i]] = i;
  while (numCandidateTransmNodes > 0) {
        int p, selectedNode;
        i = random()%numCandidateTransmNodes;
        selectedNode = nodesSelectedForTransm[numNodesSelectedForTransm++] = candidateTransmNodes[i];
        if (i < numCandidateTransmNodes-1) {
          candidateTransmNodes[i] = candidateTransmNodes[numCandidateTransmNodes-1];
          positions[candidateTransmNodes[numCandidateTransmNodes-1]] = i;
        }
        positions[selectedNode] = -1; numCandidateTransmNodes--;
        for (k=0; k<numAdjDist2Nodes[selectedNode]; k++) {
            int nodeToEliminate = adjDist2Nodes[selectedNode][k];
            if ((p = positions[nodeToEliminate]) >= 0) { //this will fail if numCandidateTransmNodes = 0
               if (p < numCandidateTransmNodes-1) { 
                  candidateTransmNodes[p] = candidateTransmNodes[numCandidateTransmNodes-1];
                  positions[candidateTransmNodes[numCandidateTransmNodes-1]] = p;
               }
               positions[nodeToEliminate] = -1; numCandidateTransmNodes--;
            }
        }
  }
  for (i=0; i<numNodesSelectedForTransm; i++) {
	  j = nodesSelectedForTransm[i];
	  nodeStates[j].selectionCounts[nodeStates[j].mssgSentRecvd.mssgType]++;
	  //printf("nodeStates[%d].selectionCounts[%s]=%d\n", j, stringMssgType(nodeStates[j].mssgSentRecvd.mssgType), 
      //       nodeStates[j].selectionCounts[nodeStates[j].mssgSentRecvd.mssgType]);
  }
}

void ReceiveMssgs(int sendingNode, int time)  
{ int i, recvingNode, packetNum, packetTTL, senderNodeNum, *remDC;
  MssgType mssgType;
  double prob;
  packetTTL = nodeStates[sendingNode].mssgSentRecvd.packetTTL;
  packetNum = nodeStates[sendingNode].mssgSentRecvd.packetNum;
  mssgType = nodeStates[sendingNode].mssgSentRecvd.mssgType;
  remDC = nodeStates[sendingNode].mssgSentRecvd.remDC; //BDE remDC queue flag of sender 
  for (i=0; i<degrees[sendingNode]; i++) {
      recvingNode = adjNodes[sendingNode][i];
      prob = 1.0*random()/RAND_MAX;
      if (prob < probSuccTransm) {
		 nodeStates[recvingNode].mssgSentRecvd = nodeStates[sendingNode].mssgSentRecvd;
         nodeStates[sendingNode].successInTransm[i] = 1;
         nodeStates[sendingNode].succMssgCounts[mssgType][i]++;
         if (packetTTL > nodeStates[recvingNode].nodeTTL) { //only then update nodeTTL to value >= 0
            if ((nodeStates[recvingNode].nodeTTL = packetTTL) <= maxDestNodeTTL) //may go up from -1 to >= 0
               nodeStates[recvingNode].nodeType = DESTINATION; 
            else nodeStates[recvingNode].nodeType = INTERMEDIATE; //becomes (or already were) above maxDestNodeTTL
         }
         if (nodeStates[recvingNode].nodeTTL >= 0)
	    if (RRT == mssgType) { //packetTTL may be -1; it lets one goal-node to receive RRT from another goal-node
               if ((NONE == nodeStates[recvingNode].dataCache[packetNum].packetStatus ) && 
                  ((RRT_YES_NO != 2) || (packetTTL < nodeStates[recvingNode].nodeTTL)))
                  ReceiveRRT(recvingNode, packetNum);
            }
            //BDE - process newly arrived message - dispatch to ReceiveTandRT
            else //mssgType = T/RT
                 if ((packetTTL >=0) && (RRT == nodeStates[recvingNode].dataCache[packetNum].packetStatus))
                    ReceiveTandRT(recvingNode, packetNum, sendingNode, time, remDC);
      }
      else nodeStates[sendingNode].successInTransm[i] = 0;
  }
}

void PrintTimeProfile(TimeProfile *first)
{ for (; first; first=first->next) {
	  printf(" %d(%d,%d,%d)", first->time, first->numNodesSelected[T], first->numNodesSelected[RT], first->numNodesSelected[RRT]);
      if (PRINT_TIME_PROFILE_ONE_ITEM_PER_LINE) printf("\n");
  }
  if (!PRINT_TIME_PROFILE_ONE_ITEM_PER_LINE) printf("\n");
}

int PSFQ() //returns number of time units the simulation ran; in the case of abonormal terminations, -time is returned
{ int i, j, time, idleTime, minIdleTime, totalIdleTime, done;
  InitNodeStates();
  for (time=totalIdleTime=done=0; !done; time++) {
      if (STEP_EACH_TIME_POINT) { printf("hit return to continue PSFQ: "); scanf("%*c"); }
      if (PRINT_DETAILS) PrintNodeDataCache(stdout, time);
      FindTransmNodes(time);
      if (0 == numNodesSelectedForTransm) { //TQ=RTQ=empty if dataCache=non-empty and nextRRTsendTime > time
         minIdleTime = -1;
         if (RRT_YES_NO) //find minIdleTime
            for (i=0; i<numNodes; i++)
                if ((nodeStates[i].firstRRTQ < numPackets) && (nodeStates[i].nodeTTL >= 0)) { //node received a T/RT/RRT/time 
                   idleTime = nodeStates[i].nextRRTsendTime - time;
                   if  ((idleTime < minIdleTime) || (minIdleTime == -1)) minIdleTime = idleTime; 
                }
         if (minIdleTime > 0) { 
            time += minIdleTime; totalIdleTime += minIdleTime;
            if (STEP_EACH_TIME_POINT) printf("***using idleTime=%d, totalIdleTime=%d\n", minIdleTime, totalIdleTime);
            FindTransmNodes(time); 
         } 
         else { done = 1; time = -time; //printf("***abnormal termination; time=%d\n", time); 
              }
      }
	  //fill-in time-profile information
	  if (0 == done) { 
         TimeProfile *temp = (TimeProfile *)calloc(1, sizeof(TimeProfile));
	     if (temp->next = firstTimeProfile) { //copy firstTimeProfile item into temp
			temp->numNodesSelected[T] = firstTimeProfile->numNodesSelected[T];
			temp->numNodesSelected[RT] = firstTimeProfile->numNodesSelected[RT];
			temp->numNodesSelected[RRT] = firstTimeProfile->numNodesSelected[RRT];
		 } 
		 firstTimeProfile = temp;
	     firstTimeProfile->time = time;
		 //the update to account for current time takes place in the next loop
	  }
      //update packet-status in dataCache, TQ/RTQ/RRTQ for selected nodes; update for receiving nodes done later
      //if done = 1 then numNodesSelectedForTransm = 0
	  for (i=0; i<numNodesSelectedForTransm; i++) {
          int packetNum, node = nodesSelectedForTransm[i];
		  MssgType mssgType = nodeStates[node].mssgSentRecvd.mssgType;
		  if (firstTimeProfile) firstTimeProfile->numNodesSelected[mssgType]++;
          packetNum = nodeStates[node].mssgSentRecvd.packetNum;
          if (T == mssgType) {
             nodeStates[node].dataCache[packetNum].packetStatus = NONE;
             nodeStates[node].firstTQ = nodeStates[node].dataCache[packetNum].nextInQ;
          }
          else if (RT == mssgType) {
                    nodeStates[node].dataCache[packetNum].packetStatus = NONE;
                    nodeStates[node].firstRTQ = nodeStates[node].dataCache[packetNum].nextInQ;
               }
          else { if (numPackets == (nodeStates[node].nextRRTtoSend = nodeStates[node].dataCache[packetNum].nextInQ))
                    nodeStates[node].nextRRTtoSend = nodeStates[node].firstRRTQ;
                 nodeStates[node].nextRRTsendTime = time + nodeStates[node].RRTperiod;
               }
      }
      //trasmit message from each selected node, which may change dataCache, TQ, RTQ, and RRTQ for receiving nodes.
      for (i=0; i<numNodesSelectedForTransm; i++) {
          //BDE set remDC in msg (is here the right place)
          for (j=0;j<numPackets;j++) {
            nodeStates[nodesSelectedForTransm[i]].mssgSentRecvd.remDC[j] = nodeStates[nodesSelectedForTransm[i]].dataCache[j].packetStatus;
          }
          if (nodeStates[nodesSelectedForTransm[i]].mssgSentRecvd.mssgType != RRT)
              nodeStates[nodesSelectedForTransm[i]].mssgSentRecvd.remDC[nodeStates[nodesSelectedForTransm[i]].mssgSentRecvd.packetNum] = NONE;
          //BDE set remDC in msg
          ReceiveMssgs(nodesSelectedForTransm[i], time+1); // this function "sends" the message
      }
      if (PRINT_DETAILS) PrintTransmStatus(stdout, time);
      for (i=0; i<numNodes; i++) 
          if (0 == nodeStates[i].nodeTTL) {
             for (j=0; j<numNodes; j++)
                 if ((DESTINATION == nodeStates[j].nodeType) && (nodeStates[j].numPacketsInDataCache < numPackets))
                    break;  
             if (j == numNodes) done = 1;
             break;
          }
      if (done) { 
         if (PRINT_DETAILS) {
            PrintOneRunStats(stdout); 
     	    printf("timeProfile of total number of nodes selected/sending T/RT/RRT successful messages upto now\n");
            PrintTimeProfile(firstTimeProfile);
         }
         return(time); 
      }
  } //end of time loop
}

void PrintMultiRunStats(FILE *outp)
{ if (numSuccRuns > 0) {
     char *str, *strT, *strRT, *strRRT;    //allocated only once
     int mssgType, i, j, strLength, strLengthT, strLengthRT, strLengthRRT;
     double *counts, totalTrecvd, totalRTrecvd, totalRRTrecvd, 
		    totalMssgsSent, //totalMssgsSent counts one for each node selected, and does not counting degree of node
			totalTotals;
     printf("p=%6.1f, RRTdelayFactor=%0.2f, RRT_YES_NO=%d, numPSFQruns=%d, numSuccRuns=%d, averTimeforSuccRuns=%0.2f\n", 
            probSuccTransm, RRTdelayFactor, RRT_YES_NO, numPSFQruns, numSuccRuns, 1.0*totalTime/numSuccRuns);
     str = (char *)malloc((1 + 29 * 7*numNodes)*sizeof(char)); //29 = length of "Node T/RT/RRT    T/RT/RRT    "; 7 = length of "nnnn.n "  
     strT = (char *)malloc((1 + 29 * 7*numNodes)*sizeof(char)); //29 = length of "Node T/RT/RRT    T/RT/RRT    "
     strRT = (char *)malloc((1 + 29 * 7*numNodes)*sizeof(char)); //29 = length of "Node T/RT/RRT    T/RT/RRT    "
     strRRT = (char *)malloc((1 + 29 * 7*numNodes)*sizeof(char)); //29 = length of "Node T/RT/RRT    T/RT/RRT    "
     counts = (double *)malloc(numNodes*sizeof(double));
     for (i=strLength=0; i<numNodes; i++) 
         { sprintf(str+strLength, "%6d ", i); strLength +=7; }
     if (0 == PRINT_JUST_FINAL_TOTALS) {
		fprintf(outp, "     candidate   Selected    Successful transmissions to other nodes (zero rows not shown)\n");
        fprintf(outp, "Node T/RT/RRT    T/RT/RRT    %s\n", str);
	 }
     fprintf(outp, "_________________________________________________________________________________\n");
     for (i=totalTrecvd=totalRTrecvd=totalRRTrecvd=totalMssgsSent=0; i<numNodes; i++) {
		 totalMssgsSent += (totalSelectionCounts[i][T] + totalSelectionCounts[i][RT] + totalSelectionCounts[i][RRT]);
         if (totalSelectionCounts[i][T]) {
            sprintf(strT, "%4d %6.2f(T)   %6.2f(T)   ", i, (double)totalCandidateCounts[i][T]/numSuccRuns, (double)totalSelectionCounts[i][T]/numSuccRuns);
            for (j=0; j<numNodes; j++) counts[j] = 0.0;
            for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = (double)totalSuccMssgCounts[i][T][j]/numSuccRuns;
            for (j=0, strLengthT=29; j<numNodes; j++)
                { sprintf(strT+strLengthT, "%6.2f ", counts[j]); strLengthT += 7; totalTrecvd += counts[j]; }
            if (0 == PRINT_JUST_FINAL_TOTALS) fprintf(outp, "%s\n", strT);
         }
         if (totalSelectionCounts[i][RT]) {
			if (totalSelectionCounts[i][T])
               sprintf(strRT, "     %6.2f(RT)  %6.2f(RT)  ", (double)totalCandidateCounts[i][RT]/numSuccRuns, (double)totalSelectionCounts[i][RT]/numSuccRuns);
			else sprintf(strRT, "%4d %6.2f(RT)  %6.2f(RT)  ", i, (double)totalCandidateCounts[i][RT]/numSuccRuns, (double)totalSelectionCounts[i][RT]/numSuccRuns);
            for (j=0; j<numNodes; j++) counts[j] = 0.0;
            for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = (double)totalSuccMssgCounts[i][RT][j]/numSuccRuns;
            for (j=0, strLengthRT=29; j<numNodes; j++)
                { sprintf(strRT+strLengthRT, "%6.2f ", counts[j]); strLengthRT += 7; totalRTrecvd += counts[j]; }
            if (0 == PRINT_JUST_FINAL_TOTALS) fprintf(outp, "%s\n", strRT);
         }
         if (totalSelectionCounts[i][RRT]) {
			if (totalSelectionCounts[i][T] || totalSelectionCounts[i][RT])
               sprintf(strRRT, "     %6.2f(RRT) %6.2f(RRT) ", (double)totalCandidateCounts[i][RRT]/numSuccRuns, (double)totalSelectionCounts[i][RRT]/numSuccRuns);
			else sprintf(strRRT, "%4d %6.2f(RRT) %6.2f(RRT) ", i, (double)totalCandidateCounts[i][RRT]/numSuccRuns, (double)totalSelectionCounts[i][RRT]/numSuccRuns);
            for (j=0; j<numNodes; j++) counts[j] = 0.0;
            for (j=0; j<degrees[i]; j++) counts[adjNodes[i][j]] = (double)totalSuccMssgCounts[i][RRT][j]/numSuccRuns;
            for (j=0, strLengthRRT=29; j<numNodes; j++)
                { sprintf(strRRT+strLengthRRT, "%6.2f ", counts[j]); strLengthRRT += 7; totalRRTrecvd += counts[j]; }
            if (0 == PRINT_JUST_FINAL_TOTALS) fprintf(outp, "%s\n", strRRT);
         }
     }
     if (0 == PRINT_JUST_FINAL_TOTALS) 
	    fprintf(outp, "_________________________________________________________________________________\n");
     fprintf(outp, "Average of numSuccRuns=%d (out of numPSFQruns=%d)\n", numSuccRuns, numPSFQruns);
	 totalMssgsSent = 1.0*totalMssgsSent/numSuccRuns; 
	 totalTotals = totalMssgsSent + totalTrecvd + totalRTrecvd + totalRRTrecvd;
	 fprintf(outp, "totalMssgsSent=%0.2f, totalRecvd: T=%0.2f, RT=%0.2f, RRT=%0.2f, totalTotals=%0.2f\n", 
			       totalMssgsSent, totalTrecvd, totalRTrecvd, totalRRTrecvd, totalTotals);
	 fprintf(outp, "_________________________________________________________________________________\n");
  }
  else printf("*** no successful runs\n");
}

void initOutput ( void )
{ if (NULL == (FHpacketOrderOut = fopen(PACKET_ORDER_FILE_NAME, "w+"))) 
  { printf("error ininitializing %s\n", PACKET_ORDER_FILE_NAME); exit(1); }
}

// puts header info at top of PACKET_ORDER_FILE_NAME
void finalizeOutput( void )
{ FILE *fp; long len; char *buf;
  fclose(FHpacketOrderOut);
  rename(PACKET_ORDER_FILE_NAME,".psfq.tmp");
  initOutput(); fp=fopen(".psfq.tmp","rb");
  //go to end //get position at end (length) //go to beg.  //malloc buffer //read into buffer
  fseek(fp,0,SEEK_END); len=ftell(fp); fseek(fp,0,SEEK_SET); buf=(char *)malloc(len); fread(buf,len,1,fp);     
  fclose(fp); remove(".psfq.tmp"); // close fh, remove temp file
  fprintf(FHpacketOrderOut,"%d %d %d // numNodes numPackets numSuccRuns .. move to first line\n", numNodes, numPackets, numSuccRuns);
  fprintf(FHpacketOrderOut,"%s",buf); fclose(FHpacketOrderOut);
}

void TestPSFQ()
{ int mssgType, i, j, time, runNum;
  //ReadInputNetwork();
  //run PSFQ and collect totalCounts
  totalCandidateCounts = (int **)malloc(numNodes * sizeof(int *));
  totalSelectionCounts = (int **)malloc(numNodes * sizeof(int *));
  totalSuccMssgCounts = (int ***)malloc(numNodes * sizeof(int **));
  for (i=0; i<numNodes; i++) {
      totalCandidateCounts[i] = (int *)calloc(numMssgTypes, sizeof(int));
      totalSelectionCounts[i] = (int *)calloc(numMssgTypes, sizeof(int));
      totalSuccMssgCounts[i] = (int **)malloc(numMssgTypes * sizeof(int *));
      for (mssgType=0; mssgType<numMssgTypes; mssgType++) totalSuccMssgCounts[i][mssgType] = (int *)calloc(degrees[i], sizeof(int));
  }
  for (runNum=1,numSuccRuns=totalTime=0; runNum<=numPSFQruns; runNum++) {
      //printf("run number=%d out of %d\n", runNum, numPSFQruns); //printf("hit return to continue: "); scanf("%*c"); 
      if ((time = PSFQ()) >= 0) { //keep it >= 0; = 0 needed for numPackets = 1
         totalTime += time; //printf("totalTime=%d\n", totalTime);
         numSuccRuns++;
         for (i=0; i<numNodes; i++)
         {   fprintf(FHpacketOrderOut,"%d:",i); // BDE..how do I take nodes that do not receive the message? 
             for (j=0;j<numPackets; j++)        // or better yet, how do I know who has and has not gotten all packets to print?
                 fprintf(FHpacketOrderOut," %d ",nodeStates[i].orderPacketsReceived[j]);
             fprintf(FHpacketOrderOut,"\n");
             for (mssgType=0; mssgType<numMssgTypes; mssgType++) {
                 totalCandidateCounts[i][mssgType] += nodeStates[i].candidateCounts[mssgType];
                 totalSelectionCounts[i][mssgType] += nodeStates[i].selectionCounts[mssgType];
                 for (j=0; j<degrees[i]; j++)
                     totalSuccMssgCounts[i][mssgType][j] += nodeStates[i].succMssgCounts[mssgType][j];
             }
         }
      }
  }
  PrintMultiRunStats(stdout);
  finalizeOutput(); //puts header information at top of output file
}

void RunMultipleProbability()
{ double p;
  ReadInputNetwork();
  for (p=0.6; p<0.99; p+=0.6) {
      probSuccTransm = p;
      for (RRTdelayFactor=1.0; RRTdelayFactor<12.0; RRTdelayFactor+=1.0)
          TestPSFQ();
  }
}

int main()
{ ReadInputNetwork(); initOutput(); TestPSFQ();
  //RunMultipleProbability();
}
