// compile: gcc -lm measure-ordering-allperm.c
// execute: ./a.out [-n packetsize] [-v[v[v]]]
// summary:
// computes measures for all permutations for a packet of length n
// permutations are calculated systematically using a recursive algorithm

// todo:
// - clean up code, especially histogram stuff
// - assign M2_MAX with explicit formula?
// - how to implement histogram for M1?
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define _DISORDER_BASE 2.0
#define _NUM_PACKETS     5 
#define _VERBOSE         0
#define MARKED          -1

// so we can use cmdline flags
double DISORDER_BASE = _DISORDER_BASE;
int    NUM_PACKETS   = _NUM_PACKETS;
int    VERBOSE       = _VERBOSE;

// globals - no mins used bc we know that we'll see perfect as the first one
static int meth2, meth2_max, meth2_sum, send_attempts, *m2_histogram, M2_MAX,
           meth3, meth3_max, meth3_sum, *m3_histogram, M3_MAX;
static double meth1, meth1_max, meth1_sum;

// prints array
void print_array( int* array, int size )
{ int i;
  for (i = 0; i < size; i++)
      printf("%d ", array[i]);
  printf("\n");
}

// sum of base^offset for each packet
double calc_disorder_1 ( int* permutation )
{ int i;
 double disorder = 0.0;
 for (i = 0; i < NUM_PACKETS; i++)
     disorder += pow(DISORDER_BASE,abs(i - permutation[i]));
 return disorder;
}

// sum of 'larger' packets arriving before i
int calc_disorder_2 ( int* permutation )
{ int i, j, disorder = 0;
 for (i = 0; i < NUM_PACKETS; i++)
 { for (j = i - 1; j >= 0; j--)
   { if ( permutation[j] > permutation[i] )
          disorder++;
   }
 }
 return disorder;
}

// sum of abs(lastpos[i] - currpos[i])
int calc_disorder_3 ( int* permutation )
{ int i, disorder=0;
  for (i = 0; i < NUM_PACKETS; i++)
      disorder += abs(i - permutation[i]);
  return disorder;
}

// measure using method 1 and tracks max and sum for average
void makeMeasurements1 ( int *permutation ) 
{  meth1 = calc_disorder_1( permutation );
   meth1_sum += meth1;
   if (meth1 > meth1_max)
       meth1_max = meth1;
}

// measure using method 1 and tracks max and sum for average
// tracks number of the same measures for histogram of results
void makeMeasurements2 ( int *permutation ) 
{  meth2 = calc_disorder_2( permutation );
   // track disorder measure
   m2_histogram[meth2]++;
   meth2_sum += meth2;
   if (meth2 > meth2_max)
       meth2_max = meth2;
}

void makeMeasurements3 ( int *permutation )
{  meth3 = calc_disorder_3( permutation );
   // track disorder measure
   m3_histogram[meth3]++;
   meth3_sum += meth3;
   if (meth3 > meth3_max)
       meth3_max = meth3;
}

// handles output after each permutation is created
void showIntermediateStats (int *permutation)
{   // verbosity control
   if (2 == VERBOSE || 3 == VERBOSE)             // -vv and -vvv
       print_array( permutation, NUM_PACKETS );
   if (1 == VERBOSE || 3 == VERBOSE)             // -v  and -vvv
       printf("packets: %d  disorder1: %10.3f   disorder2: %5d \n",
           NUM_PACKETS,meth1,meth2);
}

// handles finale out put statistics
void showFinalStats ( void )
{  int i, sum=0, total_samples = 1;
   double meth1_avg, meth2_avg, meth3_avg;
   for (i=1;i <= NUM_PACKETS;i++)
     total_samples *= i;
     
   // averages 
   meth1_avg = meth1_sum/total_samples;
   meth2_avg = meth2_sum/total_samples;
   meth3_avg = meth3_sum/total_samples;
   
   // output every time
   printf("Samples: %d; METH1: avg = %5.3f; max = %5.3f; METH2: avg = %5.3f; max = %d; METH3: avg = %5.3f; max = %d\n", total_samples,meth1_avg,meth1_max,meth2_avg,meth2_max,meth3_avg,meth3_max);

   //output histogram for meth2
   printf(" M2  Count\n");
   for (i=0; i <= M2_MAX; i++)
   { printf("%3d %4d\n",i,m2_histogram[i]);
     sum += m2_histogram[i]; }
   printf("    total: %3d\n",sum);

   //output histogram for meth2
   sum = 0;
   printf(" M3  Count\n");
   for (i=0; i <= M3_MAX; i++)
   { printf("%3d %4d\n",i,m3_histogram[i]);
     sum += m3_histogram[i]; }
   printf("    total: %3d\n",sum);  
}

// calls functions used to evaluate permutation
void evaluatePermutation(int *permutation )
{ //print_array( permutation, NUM_PACKETS );
  makeMeasurements1(permutation);
  makeMeasurements2(permutation); 
  makeMeasurements3(permutation); 
  showIntermediateStats(permutation);
}

// recursively creates permutation ina systematic way - when the
// permutation is created, the function evaluates the it
void permuteAndMeasure ( int* permutation, int curr_size, int* choices )
{ int i;
  if ( NUM_PACKETS > curr_size )
  { for (i=0; i < NUM_PACKETS; i++) 
    { if ( MARKED != choices[i] )
      { permutation[curr_size++] = i; choices[i] = MARKED;
        permuteAndMeasure(permutation, curr_size, choices);
        choices[i] = i; curr_size--;}
    }
  } 
  else
    evaluatePermutation(permutation);
}

// takes argument processing out of main
void processArgs( int argc, char** argv )
{ int i;
  extern char *optarg;
  extern int optind, optopt;
  /* command line flag stuff */
  while ((i = getopt(argc, argv, "n:v")) != -1)
  {   switch(i)
      {case 'h':
           printf("./a.out [-n packetsize] [-p probability] [-r] [-s samples] [-v[v[v]]]\n");
	   exit (EXIT_FAILURE);
	   break;
       case 'n':
          NUM_PACKETS = atoi(optarg);
          break;
       case 'v':
          VERBOSE++;
          break;
       /* handle bad arguments */
       case ':': /* n, p, or s without operand */
          exit (EXIT_FAILURE);
          break;
       case '?':
          exit (EXIT_FAILURE);
          break;
      }
  }
}

int main( int argc, char** argv ) 
{ int i, *inputPermutation, *permutation;
  processArgs(argc,argv);

  M2_MAX = NUM_PACKETS*(NUM_PACKETS-1)/2;
  if (0 == NUM_PACKETS % 2) // even
      M3_MAX = (NUM_PACKETS*NUM_PACKETS)/2;
  else                      // odd
      M3_MAX = (NUM_PACKETS*NUM_PACKETS - 1)/2;

  inputPermutation = (int*)malloc(sizeof(int)*NUM_PACKETS); 
  permutation = (int*)malloc(sizeof(int)*NUM_PACKETS); 

  /* init */
  for (i=0;i<NUM_PACKETS;i++)
      inputPermutation[i] = i;  // init initial permutation in perfect order

  m2_histogram = (int*)malloc(sizeof(int)*M2_MAX); 
  m3_histogram = (int*)malloc(sizeof(int)*M3_MAX); 

  /* initial call to permute items */ 
  permuteAndMeasure(permutation, 0, inputPermutation); 

  showFinalStats();
  return EXIT_SUCCESS;
}
