/*
 * Copyright 2011 Fabio Baltieri <fabio.baltieri@gmail.com>
 *
 * Based on the original ben-wpan code written by
 *   Werner Almesberger, Copyright 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 */

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/boot.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "defines.h"

#include "usb.h"
#include "board.h"

void panic(void)
{
	cli();
	while (1) {
		led_a_on();
		_delay_ms(100);
		led_a_off();
		_delay_ms(100);
	}
}

void reset_cpu(void)
{

	wdt_enable(WDTO_15MS);
	for (;;)
		;
}

int gpio(uint8_t port, uint8_t data, uint8_t dir, uint8_t mask, uint8_t *res)
{
	EIMSK = 0; /* recover INT_RF to ATUSB_GPIO_CLEANUP or an MCU reset */

	switch (port) {
	case 1:
		DDRB = (DDRB & ~mask) | dir;
		PORTB = (PORTB & ~mask) | data;
		break;
	case 2:
		DDRC = (DDRC & ~mask) | dir;
		PORTC = (PORTC & ~mask) | data;
		break;
	case 3:
		DDRD = (DDRD & ~mask) | dir;
		PORTD = (PORTD & ~mask) | data;
		break;
	default:
		return 0;
	}

	switch (port) {
	case 1:
		res[0] = PINB;
		res[1] = PORTB;
		res[2] = DDRB;
		break;
	case 2:
		res[0] = PINC;
		res[1] = PORTC;
		res[2] = DDRC;
		break;
	case 3:
		res[0] = PIND;
		res[1] = PORTD;
		res[2] = DDRD;
		break;
	}

	return 1;
}

static uint8_t EEMEM ee_board_id;
static uint8_t board_id;

uint8_t get_board_id(void)
{
	return board_id;
}

void set_board_id(uint8_t id)
{
	eeprom_write_byte(&ee_board_id, id);
	board_id = id;
}

void board_init(void)
{
	/* Disable the watchdog timer */

	MCUSR = 0;		/* Remove override */
	WDTCSR |= 1 << WDCE;	/* Enable change */
	WDTCSR = 1 << WDCE;	/* Disable watchdog while still enabling change */

#if (F_CPU == 16000000UL)
	/* We start with a 1 MHz/8 clock. Disable the prescaler. */
	CLKPR = 1 << CLKPCE;
	CLKPR = 0;
#elif (F_CPU == 8000000UL)
	CLKPR = 1 << CLKPCE;
	CLKPR = 1;
#else
#error unsupported F_CPU value
#endif

	board_id = eeprom_read_byte(&ee_board_id);
}
