/* Copyright (c) 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#ifndef ABL_GPIO_H_
#define ABL_GPIO_H_

#include <avr/io.h>

#include <abl/gcc.h>

typedef struct gpio_map gpio_map_t;
struct gpio_map {
	uint8_t volatile * const port;
	uint8_t volatile * const ddr;
	uint8_t const port_value;
	uint8_t const port_mask;
	uint8_t const ddr_value;
	uint8_t const ddr_mask;
};

inline void gpio_enable_function(gpio_map_t const maps[], uint8_t const size) GCC_ALWAYS_INLINE;
inline void gpio_disable_function(gpio_map_t const maps[], uint8_t const size) GCC_ALWAYS_INLINE;

inline void gpio_enable_function(gpio_map_t const maps[], uint8_t const size) {
	for(uint8_t i = 0; i < size; ++i) {
		*maps[i].port = (*maps[i].port & ~maps[i].port_mask) | maps[i].port_value;
		*maps[i].ddr = (*maps[i].ddr & ~maps[i].ddr_mask) | maps[i].ddr_value;
	}
}

inline void gpio_disable_function(gpio_map_t const maps[], uint8_t const size) {
	for(uint8_t i = 0; i < size; ++i) {
		*maps[i].port &= ~maps[i].port_mask;
		*maps[i].ddr &= ~maps[i].ddr_mask;
	}
}

#endif /* ABL_GPIO_H_ */
