/* Copyright (c) 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#ifndef ABL_INTERRUPTS_H_
#define ABL_INTERRUPTS_H_

#include <avr/io.h>

#include <abl/gcc.h>
#include <abl/types.h>

typedef void (*interrupt_handler_void_t)(void);
typedef void (*interrupt_handler_uint8_t)(uint8_t value);
typedef void (*interrupt_handler_uint16_t)(uint16_t value);

inline void interrupts_enable_global(void) GCC_ALWAYS_INLINE;
inline void interrupts_disable_global(void) GCC_ALWAYS_INLINE;
inline uint8_t interrupts_disable_global_and_save(void) GCC_ALWAYS_INLINE;
inline void interrupts_restore_global(uint8_t sreg) GCC_ALWAYS_INLINE;

inline void interrupts_enable_global(void) {
	asm volatile("sei" : : : "memory");
}

inline void interrupts_disable_global(void) {
	asm volatile("cli" : : : "memory");
}

inline uint8_t interrupts_disable_global_and_save(void) {
	uint8_t sreg = SREG;

	interrupts_disable_global();

	return sreg;
}

inline void interrupts_restore_global(uint8_t sreg) {
	SREG = sreg;
}

#endif /* ABL_INTERRUPTS_H_ */
