/* Copyright (c) 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#ifndef ABL_SPI_H_
#define ABL_SPI_H_

#include <avr/io.h>

#include <abl/gcc.h>
#include <abl/gpio.h>
#include <abl/types.h>

#if defined(__AVR_ATmega8__)
#define SPI_GPIO_PORT					PORTB
#define SPI_GPIO_DDR					DDRB
#define SPI_GPIO_PIN_SS					PB2
#define SPI_GPIO_PIN_SCK				PB5
#define SPI_GPIO_PIN_MOSI				PB3
#define SPI_GPIO_PIN_MISO				PB4
#endif

#if defined(__AVR_ATmega88__)
#define SPI_GPIO_PORT					PORTB
#define SPI_GPIO_DDR					DDRB
#define SPI_GPIO_PIN_SS					PB2
#define SPI_GPIO_PIN_SCK				PB5
#define SPI_GPIO_PIN_MOSI				PB3
#define SPI_GPIO_PIN_MISO				PB4
#endif

#if defined(__AVR_ATmega128__)
#define SPI_GPIO_PORT					PORTB
#define SPI_GPIO_DDR					DDRB
#define SPI_GPIO_PIN_SS					PB0
#define SPI_GPIO_PIN_SCK				PB1
#define SPI_GPIO_PIN_MOSI				PB2
#define SPI_GPIO_PIN_MISO				PB3
#endif

#if defined(__AVR_ATmega2560__)
#define SPI_GPIO_PORT					PORTB
#define SPI_GPIO_DDR					DDRB
#define SPI_GPIO_PIN_SS					PB0
#define SPI_GPIO_PIN_SCK				PB1
#define SPI_GPIO_PIN_MOSI				PB2
#define SPI_GPIO_PIN_MISO				PB3
#endif

#define SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP		((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_MISO), \
			.port_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

#define SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP	((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_MISO), \
			.port_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

#define SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP		((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_MISO), \
			.port_mask = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

#define SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP		((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.port_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

#define SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP		((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.port_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_MISO), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

#define SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP		((const gpio_map_t[]) { \
		{ \
			.port = &SPI_GPIO_PORT, \
			.ddr = &SPI_GPIO_DDR, \
			.port_value = (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI), \
			.port_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_value = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_MISO), \
			.ddr_mask = (1 << SPI_GPIO_PIN_SS) | (1 << SPI_GPIO_PIN_SCK) | (1 << SPI_GPIO_PIN_MOSI) | (1 << SPI_GPIO_PIN_MISO), \
		}, \
	})

typedef enum {
	SPI_ROLE_MASTER					= (1 << MSTR),
	SPI_ROLE_SLAVE					= 0,
} GCC_PACKED spi_role_t;

typedef enum {
	SPI_CLOCK_POLARITY_0				= 0,
	SPI_CLOCK_POLARITY_1				= (1 << CPOL),
} GCC_PACKED spi_clock_polarity_t;

typedef enum {
	SPI_CLOCK_PHASE_0				= 0,
	SPI_CLOCK_PHASE_1				= (1 << CPHA),
} GCC_PACKED spi_clock_phase_t;

typedef enum {
	SPI_MODE_SAMPLE_RISING_SETUP_FALLING		= SPI_CLOCK_POLARITY_0 | SPI_CLOCK_PHASE_0,
	SPI_MODE_0					= SPI_MODE_SAMPLE_RISING_SETUP_FALLING,
	SPI_MODE_SETUP_RISING_SAMPLE_FALLING		= SPI_CLOCK_POLARITY_0 | SPI_CLOCK_PHASE_1,
	SPI_MODE_1					= SPI_MODE_SETUP_RISING_SAMPLE_FALLING,
	SPI_MODE_SAMPLE_FALLING_SETUP_RISING		= SPI_CLOCK_POLARITY_1 | SPI_CLOCK_PHASE_0,
	SPI_MODE_2					= SPI_MODE_SAMPLE_FALLING_SETUP_RISING,
	SPI_MODE_SETUP_FALLING_SAMPLE_RISING		= SPI_CLOCK_POLARITY_1 | SPI_CLOCK_PHASE_1,
	SPI_MODE_3					= SPI_MODE_SETUP_FALLING_SAMPLE_RISING,
} GCC_PACKED spi_mode_t;

typedef enum {
	SPI_DATA_ORDER_LSB_FIRST			= (1 << DORD),
	SPI_DATA_ORDER_MSB_FIRST			= 0,
} GCC_PACKED spi_data_order_t;

typedef enum {
	SPI_CLOCK_RATE_NONE				= 0,
	SPI_CLOCK_RATE_FOSC_BY_4			= 0,
	SPI_CLOCK_RATE_FOSC_BY_16			= (1 << SPR0),
	SPI_CLOCK_RATE_FOSC_BY_64			= (1 << SPR1),
	SPI_CLOCK_RATE_FOSC_BY_128			= (1 << SPR1) | (1 << SPR0),
	SPI_CLOCK_RATE_FOSC_BY_2			= (1 << (SPI2X + 8)) | SPI_CLOCK_RATE_FOSC_BY_4,
	SPI_CLOCK_RATE_FOSC_BY_8			= (1 << (SPI2X + 8)) | SPI_CLOCK_RATE_FOSC_BY_16,
	SPI_CLOCK_RATE_FOSC_BY_32			= (1 << (SPI2X + 8)) | SPI_CLOCK_RATE_FOSC_BY_64,
} GCC_PACKED spi_clock_rate_t;

typedef enum {
	SPI_INTERRUPT_ENABLED				= (1 << SPIE),
	SPI_INTERRUPT_DISABLED				= 0,
} GCC_PACKED spi_interrupt_t;

typedef enum {
	SPI_GPIO_FUNCTION_NONE,
	SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN,
	SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN,
	SPI_GPIO_FUNCTION_MASTER_NO_SS,
	SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN,
	SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN,
	SPI_GPIO_FUNCTION_SLAVE_NO_SS,
} GCC_PACKED spi_gpio_function_t;

inline spi_role_t spi_get_role(void) GCC_ALWAYS_INLINE;
inline void spi_set_role(spi_role_t role) GCC_ALWAYS_INLINE;

inline  spi_data_order_t spi_get_data_order(void);
inline void spi_set_data_order(spi_data_order_t const data_order);

inline spi_clock_polarity_t spi_get_clock_polarity(void) GCC_ALWAYS_INLINE;
inline void spi_set_clock_polarity(spi_clock_polarity_t const clock_polarity) GCC_ALWAYS_INLINE;

inline spi_clock_phase_t spi_get_clock_phase(void) GCC_ALWAYS_INLINE;
inline void spi_set_clock_phase(spi_clock_phase_t const clock_phase) GCC_ALWAYS_INLINE;

inline spi_mode_t spi_get_mode(void) GCC_ALWAYS_INLINE;
inline void spi_set_mode(spi_mode_t const mode) GCC_ALWAYS_INLINE;

inline spi_clock_rate_t spi_get_clock_rate(void) GCC_ALWAYS_INLINE;
inline void spi_set_clock_rate(spi_clock_rate_t const clock_rate) GCC_ALWAYS_INLINE;

inline uint8_t spi_is_interrupt_enabled(void) GCC_ALWAYS_INLINE;
inline void spi_enable_interrupt(void) GCC_ALWAYS_INLINE;
inline void spi_disable_interrupt(void) GCC_ALWAYS_INLINE;

inline void spi_enable_gpio_function(spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;
inline void spi_disable_gpio_function(spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;

inline void spi_configure(spi_role_t const role, spi_data_order_t const data_order, spi_clock_polarity_t const clock_polarity, spi_clock_phase_t const clock_phase, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;
inline void spi_configure2(spi_role_t const role, spi_data_order_t const data_order, spi_mode_t const mode, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;

inline uint8_t spi_is_enabled(void) GCC_ALWAYS_INLINE;
inline void spi_enable(void) GCC_ALWAYS_INLINE;
inline void spi_disable(void) GCC_ALWAYS_INLINE;

inline void spi_configure_and_enable(spi_role_t const role, spi_data_order_t const data_order, spi_clock_polarity_t const clock_polarity, spi_clock_phase_t const clock_phase, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;
inline void spi_configure_and_enable2(spi_role_t const role, spi_data_order_t const data_order, spi_mode_t const mode, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) GCC_ALWAYS_INLINE;

inline uint8_t spi_read(void) GCC_ALWAYS_INLINE;
inline void spi_write(uint8_t const data) GCC_ALWAYS_INLINE;
inline bool_t spi_write_clear_collision(uint8_t const data) GCC_ALWAYS_INLINE;
inline void spi_write_retry(uint8_t const data) GCC_ALWAYS_INLINE;

inline uint8_t spi_transmit(uint8_t const data) GCC_ALWAYS_INLINE;

inline spi_role_t spi_get_role(void) {
	return SPCR & (1 << MSTR);
}

inline void spi_set_role(spi_role_t const role) {
	SPCR = (SPCR & ~(1 << MSTR)) | role;
}

inline spi_data_order_t spi_get_data_order(void) {
	return SPCR & (1 << DORD);
}

inline void spi_set_data_order(spi_data_order_t const data_order) {
	SPCR = (SPCR & ~(1 << DORD)) | data_order;
}

inline spi_clock_polarity_t spi_get_clock_polarity(void) {
	return SPCR & (1 << CPOL);
}

inline void spi_set_clock_polarity(spi_clock_polarity_t const clock_polarity) {
	SPCR = (SPCR & ~(1 << CPOL)) | clock_polarity;
}

inline spi_clock_phase_t spi_get_clock_phase(void) {
	return SPCR & (1 << CPHA);
}

inline void spi_set_clock_phase(spi_clock_phase_t const clock_phase) {
	SPCR = (SPCR & ~(1 << CPHA)) | clock_phase;
}

inline spi_mode_t spi_get_mode(void) {
	return SPCR & ((1 << CPOL) | (1 << CPHA));
}

inline void spi_set_mode(spi_mode_t const mode) {
	SPCR = (SPCR & ~((1 << CPOL) | (1 << CPHA))) | mode;
}

inline spi_clock_rate_t spi_get_clock_rate(void) {
	uint8_t const spsr = SPSR;

	return ((spsr & (1 << SPI2X)) * 256) | (spsr & ((1 << SPR1) | (1 << SPR0)));
}

inline void spi_set_clock_rate(spi_clock_rate_t const clock_rate) {
	SPCR = (SPCR & ~((1 << SPR1) | (1 << SPR0))) | ((uint8_t const)clock_rate);
	SPSR = (uint8_t const)(clock_rate  >> 8);
}

inline uint8_t spi_is_interrupt_enabled(void) {
	return SPCR & (1 << SPIE);
}

inline void spi_enable_interrupt(void) {
	SPCR |= (1 << SPIE);
}

inline void spi_disable_interrupt(void) {
	SPCR &= ~(1 << SPIE);
}

inline void spi_enable_gpio_function(spi_gpio_function_t const gpio_function) {
	switch(gpio_function) {
		case SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN:
			gpio_enable_function(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN:
			gpio_enable_function(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_MASTER_NO_SS:
			gpio_enable_function(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN:
			gpio_enable_function(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN:
			gpio_enable_function(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_NO_SS:
			gpio_enable_function(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP[0])));
			break;
	}
}

inline void spi_disable_gpio_function(spi_gpio_function_t const gpio_function) {
	switch(gpio_function) {
		case SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN:
			gpio_disable_function(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_SS_DRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN:
			gpio_disable_function(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_SS_UNDRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_MASTER_NO_SS:
			gpio_disable_function(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP, sizeof(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_MASTER_NO_SS_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN:
			gpio_disable_function(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_SS_DRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN:
			gpio_disable_function(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_SS_UNDRIVEN_MAP[0])));
			break;
		case SPI_GPIO_FUNCTION_SLAVE_NO_SS:
			gpio_disable_function(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP, sizeof(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP) / sizeof(typeof(SPI_GPIO_FUNCTION_SLAVE_NO_SS_MAP[0])));
			break;
	}
}

inline void spi_configure(spi_role_t const role, spi_data_order_t const data_order, spi_clock_polarity_t const clock_polarity, spi_clock_phase_t const clock_phase, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) {
	SPCR = role | data_order | clock_polarity | clock_phase | ((uint8_t const)clock_rate) | interrupt;
	SPSR = (uint8_t const)(clock_rate  >> 8);

	if(gpio_function != SPI_GPIO_FUNCTION_NONE)
		spi_enable_gpio_function(gpio_function);
}

inline void spi_configure2(spi_role_t const role, spi_data_order_t const data_order, spi_mode_t const mode, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) {
	SPCR = role | data_order | mode | ((uint8_t const)clock_rate) | interrupt;
	SPSR = (uint8_t const)(clock_rate  >> 8);

	if(gpio_function != SPI_GPIO_FUNCTION_NONE)
		spi_enable_gpio_function(gpio_function);
}

inline uint8_t spi_is_enabled(void) {
	return SPCR & (1 << SPE);
}

inline void spi_enable(void) {
	SPCR |= (1 << SPE);
}

inline void spi_disable(void) {
	SPCR &= ~(1 << SPE);
}

inline void spi_configure_and_enable(spi_role_t const role, spi_data_order_t const data_order, spi_clock_polarity_t const clock_polarity, spi_clock_phase_t const clock_phase, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) {
	SPCR = (1 << SPE) | role | data_order | clock_polarity | clock_phase | ((uint8_t const)clock_rate) | interrupt;
	SPSR = (uint8_t const)(clock_rate  >> 8);

	if(gpio_function != SPI_GPIO_FUNCTION_NONE)
		spi_enable_gpio_function(gpio_function);
}

inline void spi_configure_and_enable2(spi_role_t const role, spi_data_order_t const data_order, spi_mode_t const mode, spi_clock_rate_t const clock_rate, spi_interrupt_t const interrupt, spi_gpio_function_t const gpio_function) {
	SPCR = (1 << SPE) | role | data_order | mode | ((uint8_t const)clock_rate) | interrupt;
	SPSR = (uint8_t const)(clock_rate  >> 8);

	if(gpio_function != SPI_GPIO_FUNCTION_NONE)
		spi_enable_gpio_function(gpio_function);
}

inline uint8_t spi_read(void) {
	return SPDR;
}

inline void spi_write(uint8_t const data) {
	SPDR = data;
}

inline bool_t spi_write_clear_collision(uint8_t const data) {
	bool_t collision;

	SPDR = data;

	collision = SPSR & (1 << WCOL);
	(void)SPDR;

	return collision;
}

inline void spi_write_retry(uint8_t const data) {
	SPDR = data;

	while(SPSR & (1 << WCOL)) {
		(void)SPDR;

		SPDR = data;
	}
}

inline uint8_t spi_transmit(uint8_t const data) {
	SPDR = data;

	while(!(SPSR & (1 << SPIF))) ;

	return SPDR;
}

#endif /* ABL_SPI_H_ */
