/* Copyright (c) 2011-2012 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#ifndef _NIBO_CEPRO_LCD_H
#define _NIBO_CEPRO_LCD_H

#include <abl/gcc.h>

#include <nibo/cepro/uid.h>

typedef enum {
	NIBO_LCD_CHIP_1					= NIBO_LCD_CONTROL_CS1_MASK,
	NIBO_LCD_CHIP_2					= NIBO_LCD_CONTROL_CS2_MASK,
} GCC_PACKED nibo_lcd_chip_t;

typedef enum {
	NIBO_LCD_REGISTER_DATA				= NIBO_LCD_CONTROL_RS_MASK,
	NIBO_LCD_REGISTER_INSTRUCTION			= 0,
} GCC_PACKED nibo_lcd_register_t;

typedef enum {
	NIBO_LCD_COMMAND_DISPLAY_ON			= 0b00111111,
	NIBO_LCD_COMMAND_DISPLAY_OFF			= 0b00111110,
	NIBO_LCD_COMMAND_SET_ADDRESS			= 0b01000000,
	NIBO_LCD_COMMAND_SET_PAGE			= 0b10111000,
	NIBO_LCD_COMMAND_DISPLAY_START_LINE		= 0b11000000,
} GCC_PACKED nibo_lcd_command_t;

typedef enum {
	NIBO_LCD_STATUS_NONE				= 0,
	NIBO_LCD_STATUS_RESET				= 0b00010000,
	NIBO_LCD_STATUS_DISPLAY_ON			= 0b00100000,
	NIBO_LCD_STATUS_BUSY				= 0b10000000,
} GCC_PACKED nibo_lcd_status_t;

extern void nibo_lcd_init(void);

extern uint8_t nibo_lcd_read(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register);
inline nibo_lcd_status_t nibo_lcd_read_status(nibo_lcd_chip_t const lcd_chip) GCC_ALWAYS_INLINE;
extern void nibo_lcd_write(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register,  uint8_t const data);
inline uint8_t nibo_lcd_write_when_ready(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register, uint8_t const data) GCC_ALWAYS_INLINE;

extern uint8_t nibo_lcd_wait_status(nibo_lcd_chip_t const lcd_chip, nibo_lcd_status_t const mask, nibo_lcd_status_t const value);

extern void nibo_lcd_fill(uint8_t const value);

inline void nibo_lcd_set_brightness(uint16_t const brightness) GCC_ALWAYS_INLINE;
extern void nibo_lcd_set_apparent_brightness(uint16_t const brightness);

inline nibo_lcd_status_t nibo_lcd_read_status(nibo_lcd_chip_t const lcd_chip) {
	return (nibo_lcd_status_t)nibo_lcd_read(lcd_chip, NIBO_LCD_REGISTER_INSTRUCTION);
}

inline uint8_t nibo_lcd_write_when_ready(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register, uint8_t const data) {
	if(nibo_lcd_wait_status(lcd_chip, NIBO_LCD_STATUS_BUSY, NIBO_LCD_STATUS_NONE)) {
		nibo_lcd_write(lcd_chip, lcd_register, data);

		return 1;
	}

	return 0;
}

inline void nibo_lcd_set_brightness(uint16_t const brightness) {
	NIBO_LCD_PWM_OCR = brightness;
}

#endif /* _NIBO_CEPRO_LCD_H */
