/* Copyright (c) 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#ifndef _NIBO_CEPRO_STATUSLEDS_H
#define _NIBO_CEPRO_STATUSLEDS_H

#include <abl/gcc.h>

#include <nibo/cepro/uid.h>

#define NIBO_STATUS_LED(n)				(nibo_status_leds_t)(1 << (n))

typedef enum {
	NIBO_STATUS_LED_0				= 1 << 0,
	NIBO_STATUS_LED_1				= 1 << 1,
	NIBO_STATUS_LED_2				= 1 << 2,
	NIBO_STATUS_LED_3				= 1 << 3,
	NIBO_STATUS_LED_4				= 1 << 4,
	NIBO_STATUS_LED_5				= 1 << 5,
	NIBO_STATUS_LED_6				= 1 << 6,
	NIBO_STATUS_LED_7				= 1 << 7,
	NIBO_STATUS_LED_BUMPER_LEFT			= NIBO_STATUS_LED_1,
	NIBO_STATUS_LED_BUMPER_RIGHT			= NIBO_STATUS_LED_0,
	NIBO_STATUS_LED_FRONT_LEFT			= NIBO_STATUS_LED_4,
	NIBO_STATUS_LED_FRONT_RIGHT			= NIBO_STATUS_LED_5,
	NIBO_STATUS_LED_LEFT				= NIBO_STATUS_LED_3,
	NIBO_STATUS_LED_RIGHT				= NIBO_STATUS_LED_6,
	NIBO_STATUS_LED_LEFT_MOST			= NIBO_STATUS_LED_2,
	NIBO_STATUS_LED_RIGHT_MOST			= NIBO_STATUS_LED_7,
} GCC_PACKED nibo_status_leds_t;

typedef enum {
	NIBO_STATUS_LED_COLOR_RED			= 1 << 0,
	NIBO_STATUS_LED_COLOR_GREEN			= 1 << 1,
	NIBO_STATUS_LED_COLOR_ORANGE			= NIBO_STATUS_LED_COLOR_RED | NIBO_STATUS_LED_COLOR_GREEN,
} GCC_PACKED nibo_status_led_color_t;

extern void nibo_status_leds_init(void);

inline void nibo_status_leds_on(nibo_status_leds_t const leds, nibo_status_led_color_t const color) GCC_ALWAYS_INLINE;
inline void nibo_status_leds_off(nibo_status_leds_t const leds, nibo_status_led_color_t const color) GCC_ALWAYS_INLINE;
inline void nibo_status_leds_toggle(nibo_status_leds_t const leds, nibo_status_led_color_t const color) GCC_ALWAYS_INLINE;

inline uint16_t nibo_status_leds_get_brightness(void) GCC_ALWAYS_INLINE;
inline void nibo_status_leds_set_brightness(uint16_t const brightness) GCC_ALWAYS_INLINE;
extern void nibo_status_leds_set_apparent_brightness(uint16_t const brightness);

inline void nibo_status_leds_on(nibo_status_leds_t const leds, nibo_status_led_color_t const color) {
	if(color & NIBO_STATUS_LED_COLOR_RED)
		NIBO_STATUS_LEDS_RED_PORT |= leds;
	if(color & NIBO_STATUS_LED_COLOR_GREEN)
		NIBO_STATUS_LEDS_GREEN_PORT |= leds;
}

inline void nibo_status_leds_off(nibo_status_leds_t const leds, nibo_status_led_color_t const color) {
	if(color & NIBO_STATUS_LED_COLOR_RED)
		NIBO_STATUS_LEDS_RED_PORT &= ~leds;
	if(color & NIBO_STATUS_LED_COLOR_GREEN)
		NIBO_STATUS_LEDS_GREEN_PORT &= ~leds;
}

inline void nibo_status_leds_toggle(nibo_status_leds_t const leds, nibo_status_led_color_t const color) {
	if(color & NIBO_STATUS_LED_COLOR_RED)
		NIBO_STATUS_LEDS_RED_PORT ^= leds;
	if(color & NIBO_STATUS_LED_COLOR_GREEN)
		NIBO_STATUS_LEDS_GREEN_PORT ^= leds;
}

inline uint16_t nibo_status_leds_get_brightness(void) {
	return NIBO_STATUS_LEDS_PWM_OCR;
}

inline void nibo_status_leds_set_brightness(uint16_t const brightness) {
	NIBO_STATUS_LEDS_PWM_OCR = brightness;
}

#endif /* _NIBO_CEPRO_STATUSLEDS_H */
