/* Copyright (c) 2011-2012 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#include <util/delay.h>

#include <nibo/cepro/lcd.h>
#include <nibo/cepro/pwm.h>

#include "../pwm/pwm-private.h"

static uint8_t nibo_lcd_initialized = 0;

void nibo_lcd_init(void) {
	if(!nibo_lcd_initialized) {
		NIBO_LCD_CONTROL_DDR |= NIBO_LCD_CONTROL_EN_MASK | NIBO_LCD_CONTROL_RS_MASK | NIBO_LCD_CONTROL_RW_MASK | NIBO_LCD_CONTROL_CS1_MASK | NIBO_LCD_CONTROL_CS2_MASK;
		NIBO_LCD_RESET_DDR |= NIBO_LCD_RESET_MASK;
		NIBO_LCD_DATA_DDR &= ~NIBO_LCD_DATA_MASK;
		NIBO_LCD_CONTROL_PORT &= ~(NIBO_LCD_CONTROL_EN_MASK | NIBO_LCD_CONTROL_RS_MASK | NIBO_LCD_CONTROL_RW_MASK | NIBO_LCD_CONTROL_CS1_MASK | NIBO_LCD_CONTROL_CS2_MASK);

		NIBO_LCD_RESET_PORT &= ~NIBO_LCD_RESET_MASK;

		if(NIBO_COPRO_RESET_PORT & NIBO_COPRO_RESET_MASK) {
			NIBO_COPRO_RESET_PORT &= ~NIBO_COPRO_RESET_MASK;
			_delay_ms(1);
			NIBO_COPRO_RESET_PORT |= NIBO_COPRO_RESET_MASK;
		}
		else
			_delay_ms(1);

		NIBO_LCD_RESET_PORT |= NIBO_LCD_RESET_MASK;

		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_DISPLAY_START_LINE);
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_DISPLAY_START_LINE);

		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_ADDRESS);
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_ADDRESS);

		nibo_lcd_fill(0x00);

		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_PAGE);
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_PAGE);

		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_DISPLAY_ON);
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_DISPLAY_ON);

		nibo_pwm_init();

		NIBO_LCD_PWM_DDR |= NIBO_LCD_PWM_MASK;

		nibo_lcd_initialized =  1;
	}
}

uint8_t nibo_lcd_read(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register) {
	NIBO_LCD_CONTROL_PORT = (NIBO_LCD_CONTROL_PORT & ~(NIBO_LCD_CONTROL_RS_MASK | NIBO_LCD_CONTROL_RW_MASK | NIBO_LCD_CONTROL_CS1_MASK | NIBO_LCD_CONTROL_CS2_MASK)) | lcd_register | NIBO_LCD_CONTROL_RW_MASK | lcd_chip;

	NIBO_LCD_DATA_DDR = 0x00;
	_delay_us(1);
	NIBO_LCD_CONTROL_PORT |= NIBO_LCD_CONTROL_EN_MASK;
	_delay_us(4);
	uint8_t value = NIBO_LCD_DATA_PIN;
	NIBO_LCD_CONTROL_PORT &= ~NIBO_LCD_CONTROL_EN_MASK;
	_delay_us(1);

	return value;
}

void nibo_lcd_write(nibo_lcd_chip_t const lcd_chip, nibo_lcd_register_t const lcd_register,  uint8_t const value) {
	NIBO_LCD_CONTROL_PORT = (NIBO_LCD_CONTROL_PORT & ~(NIBO_LCD_CONTROL_RS_MASK | NIBO_LCD_CONTROL_RW_MASK | NIBO_LCD_CONTROL_CS1_MASK | NIBO_LCD_CONTROL_CS2_MASK)) | lcd_register | lcd_chip;

	NIBO_LCD_DATA_DDR = 0xff;
	NIBO_LCD_DATA_PORT = value;

	_delay_us(1);
	NIBO_LCD_CONTROL_PORT |= NIBO_LCD_CONTROL_EN_MASK;
	_delay_us(1);
	NIBO_LCD_CONTROL_PORT &= ~NIBO_LCD_CONTROL_EN_MASK;
	_delay_us(1);

	NIBO_LCD_DATA_PORT = 0x00;
	NIBO_LCD_DATA_DDR = 0x00;
}

uint8_t nibo_lcd_wait_status(nibo_lcd_chip_t const lcd_chip, nibo_lcd_status_t const mask, nibo_lcd_status_t const value) {
	uint16_t timeout = 0xffff;

	while(--timeout)
		if((nibo_lcd_read_status(lcd_chip) & mask) == value)
			return 1;

	return 0;
}

// if(!(lcd_read(lcd_chip, NIBO_LCD_REGISTER_INSTRUCTION) & status))

void nibo_lcd_fill(uint8_t const value) {
	for(uint8_t x = 0; x < 8; ++x) {
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_PAGE | x);
		nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_INSTRUCTION, NIBO_LCD_COMMAND_SET_PAGE | x);
		for(uint8_t y = 0; y < 64; ++y) {
			nibo_lcd_write_when_ready(NIBO_LCD_CHIP_1, NIBO_LCD_REGISTER_DATA, value);
			nibo_lcd_write_when_ready(NIBO_LCD_CHIP_2, NIBO_LCD_REGISTER_DATA, value);
		}
	}
}

void nibo_lcd_set_apparent_brightness(uint16_t const brightness) {
	nibo_lcd_set_brightness(nibo_pwm_translate_value(brightness));
}
