/* Copyright (c) 2011-2012 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of jdp-incubator.
 *
 * jdp-incubator is licensed under the Simplified BSD License (the "License").
 * You should have received a copy of the License along with jdp-incubator.
 * If not, see <http://www.opensource.org/licenses/bsd-license.php>.
 */

#include <nibo/cepro/pwm.h>
#include <nibo/cepro/uid.h>

#include "pwm-private.h"

#define NIBO_PWM_TOP				1024
#define NIBO_PWM_TABLE_SIZE			64
#define NIBO_PWM_SEGMENT_SIZE			(NIBO_PWM_TOP / NIBO_PWM_TABLE_SIZE)

// TODO Move table into flash memory
static uint16_t const nibo_pwm_table[] = {
	0x0008, 0x0010, 0x0018, 0x0021, 0x0029, 0x0031,
	0x003a, 0x0042, 0x004b, 0x0053, 0x005c, 0x0065,
	0x006e, 0x0077, 0x0080, 0x0089, 0x0092, 0x009c,
	0x00a5, 0x00af, 0x00b9, 0x00c2, 0x00cc, 0x00d6,
	0x00e1, 0x00eb, 0x00f5, 0x0100, 0x010b, 0x0116,
	0x0121, 0x012c, 0x0137, 0x0143, 0x014f, 0x015b,
	0x0167, 0x0173, 0x0180, 0x018d, 0x019a, 0x01a8,
	0x01b6, 0x01c4, 0x01d2, 0x01e1, 0x01f0, 0x0200,
	0x0211, 0x0221, 0x0233, 0x0245, 0x0258, 0x026c,
	0x0281, 0x0297, 0x02ae, 0x02c7, 0x02e3, 0x0301,
	0x0324, 0x034d, 0x0383, 0x03ff,
};

static uint8_t nibo_pwm_initialized = 0;

void nibo_pwm_init(void) {
	if(!nibo_pwm_initialized) {
		NIBO_HEADLIGHTS_PWM_OCR = 0xffff;
		NIBO_STATUS_LEDS_PWM_OCR = 0xffff;
		NIBO_LCD_PWM_OCR = 0xffff;

		TCCR1A = (1 << COM1A1) | (1 << COM1B1) | (1 << COM1C1) | (1 << WGM11) | (1 << WGM10);
		TCCR1B = (1 << CS11) | (1 << WGM12);

		nibo_pwm_initialized = 1;
	}
}

uint16_t nibo_pwm_translate_value(uint16_t const value) {
	if(value == 0 || value == (NIBO_PWM_TOP - 1))
		return value;
	else {
		uint8_t const index = value / NIBO_PWM_SEGMENT_SIZE;
		uint8_t const y0 = (index > 0) ? nibo_pwm_table[index - 1] : 0;
		uint8_t const y1 = nibo_pwm_table[index];

		return y0 + (value & (NIBO_PWM_SEGMENT_SIZE - 1)) * ((y1 - y0) / NIBO_PWM_SEGMENT_SIZE);
	}
}
