# Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
#
# This file is part of xng-cmake-template.
#
# xng-cmake-template is licensed under the Simplified BSD License
# (the "License").
#
# You should have received a copy of the License along with xng-cmake-template.
# If not, see <http://www.opensource.org/licenses/bsd-license.php>.
#
# Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>

# Include user configuration file
if(EXISTS "${CMAKE_BINARY_DIR}/user-config.cmake")
	message(STATUS "Including user config from binary directory.")
	set(XNG_AVR32_USER_CONFIG "${CMAKE_BINARY_DIR}/user-config.cmake")
	include("${XNG_AVR32_USER_CONFIG}")
elseif(EXISTS "${CMAKE_SOURCE_DIR}/user-config.cmake")
	message(STATUS "Including user config from source directory.")
	set(XNG_AVR32_USER_CONFIG "${CMAKE_SOURCE_DIR}/user-config.cmake")
	include("${XNG_AVR32_USER_CONFIG}")
endif()

set(XNG_AVR32_CONFIG_OK TRUE)

# Part configuration
if(NOT DEFINED USER_XNG_AVR32_PART AND (NOT DEFINED XNG_AVR32_PART OR "${XNG_AVR32_PART}" STREQUAL "XNG_AVR32_PART-UNDEFINED"))
	message(WARNING "USER_XNG_AVR32_PART is undefined.")
	set(USER_XNG_AVR32_PART "XNG_AVR32_PART-UNDEFINED")
	set(XNG_AVR32_CONFIG_OK FALSE)
endif()
set(XNG_AVR32_PART "${USER_XNG_AVR32_PART}" CACHE STRING "Part number, e.g. uc3a1512, ...")

if(NOT DEFINED USER_XNG_AVR32_PART_FAMILY AND (NOT DEFINED XNG_AVR32_PART_FAMILY OR "${XNG_AVR32_PART_FAMILY}" STREQUAL "XNG_AVR32_PART_FAMILY-UNDEFINED"))
	if("${XNG_AVR32_PART}" MATCHES "^uc3[abcl][0-9]+[cs]?$")
		string(SUBSTRING "${XNG_AVR32_PART}" 0 4 USER_XNG_AVR32_PART_FAMILY)
		message(STATUS "Guessed part family: ${USER_XNG_AVR32_PART_FAMILY}")
	else()
		message(WARNING "USER_XNG_AVR32_PART_FAMILY undefined and unable to guess.")
		set(USER_XNG_AVR32_PART_FAMILY "XNG_AVR32_PART_FAMILY-UNDEFINED")
		set(XNG_AVR32_CONFIG_OK FALSE)
	endif()
endif()
set(XNG_AVR32_PART_FAMILY "${USER_XNG_AVR32_PART_FAMILY}" CACHE STRING "Part family, e.g. uc3a, uc3c, ...")

# Toolchain configuration
if(NOT DEFINED USER_XNG_AVR32_GCC AND (NOT DEFINED XNG_AVR32_GCC OR "${XNG_AVR32_GCC}" STREQUAL "XNG_AVR32_GCC-NOTFOUND"))
	find_program(PROGRAM_AVR32_GCC "avr32-gcc")
	find_program(PROGRAM_AVR32_UNKNOWN_NONE_GCC "avr32-unknown-none-gcc")
	if(NOT "${PROGRAM_AVR32_GCC}" STREQUAL "PROGRAM_AVR32_GCC-NOTFOUND")
		set(USER_XNG_AVR32_GCC "${PROGRAM_AVR32_GCC}")
		message(STATUS "Auto-detected GCC: ${USER_XNG_AVR32_GCC}")
	elseif(NOT "${PROGRAM_AVR32_UNKNOWN_NONE_GCC}" STREQUAL "PROGRAM_AVR32_UNKNOWN_NONE_GCC-NOTFOUND")
		set(USER_XNG_AVR32_GCC "${PROGRAM_AVR32_UNKNOWN_NONE_GCC}")
		message(STATUS "Auto-detected GCC: ${USER_XNG_AVR32_GCC}")
	else()
		message(WARNING "USER_XNG_AVR32_GCC undefined and unable to auto-detect.")
		set(USER_XNG_AVR32_GCC "XNG_AVR32_GCC-NOTFOUND")
		set(XNG_AVR32_CONFIG_OK FALSE)
	endif()
endif()
set(XNG_AVR32_GCC "${USER_XNG_AVR32_GCC}" CACHE FILEPATH "Path to or name of GCC compiler.")

if(NOT DEFINED USER_XNG_AVR32_OBJDUMP AND (NOT DEFINED XNG_AVR32_OBJDUMP OR "${XNG_AVR32_OBJDUMP}" STREQUAL "XNG_AVR32_OBJDUMP-NOTFOUND"))
	if(NOT XNG_AVR32_GCC STREQUAL "XNG_AVR32_GCC-NOTFOUND" AND "${XNG_AVR32_GCC}" MATCHES "-gcc$")
		string(REGEX REPLACE "-gcc$" "-objdump" USER_XNG_AVR32_OBJDUMP "${XNG_AVR32_GCC}")

		if(IS_ABSOLUTE "${USER_XNG_AVR32_OBJDUMP}" AND EXISTS "${USER_XNG_AVR32_OBJDUMP}")
			message(STATUS "Auto-detected objdump: ${USER_XNG_AVR32_OBJDUMP}")
		else()
			get_filename_component(USER_XNG_AVR32_OBJDUMP "${USER_XNG_AVR32_OBJDUMP}" NAME)
			find_program(PROGRAM_GCC_PREFIX_OBJDUMP "${USER_XNG_AVR32_OBJDUMP}")
			if(NOT "${PROGRAM_GCC_PREFIX_OBJDUMP}" STREQUAL "PROGRAM_GCC_PREFIX_OBJDUMP-NOTFOUND")
				set(USER_XNG_AVR32_OBJDUMP "${PROGRAM_GCC_PREFIX_OBJDUMP}")
				message(STATUS "Auto-detected objdump: ${USER_XNG_AVR32_OBJDUMP}")
			else()
				set(USER_XNG_AVR32_OBJDUMP "USER_XNG_AVR32_OBJDUMP-NOTFOUND")
			endif()
		endif()
	endif()

	if("${USER_XNG_AVR32_OBJDUMP}" STREQUAL "USER_XNG_AVR32_OBJDUMP-NOTFOUND")
		find_program(PROGRAM_AVR32_OBJDUMP "avr32-objdump")
		find_program(PROGRAM_AVR32_UNKNOWN_NONE_OBJDUMP "avr32-unknown-none-objdump")
		if(NOT "${PROGRAM_AVR32_OBJDUMP}" STREQUAL "PROGRAM_AVR32_OBJDUMP-NOTFOUND")
			set(USER_XNG_AVR32_OBJDUMP "${PROGRAM_AVR32_OBJDUMP}")
			message(STATUS "Auto-detected objdump: ${USER_XNG_AVR32_OBJDUMP}")
		elseif(NOT "${PROGRAM_AVR32_UNKNOWN_NONE_OBJDUMP}" STREQUAL "PROGRAM_AVR32_UNKNOWN_NONE_OBJDUMP-NOTFOUND")
			set(USER_XNG_AVR32_OBJDUMP "${PROGRAM_AVR32_UNKNOWN_NONE_OBJDUMP}")
			message(STATUS "Auto-detected objdump: ${USER_XNG_AVR32_OBJDUMP}")
		else()
			message(WARNING "USER_XNG_AVR32_OBJDUMP undefined.")
			set(USER_XNG_AVR32_OBJDUMP "XNG_AVR32_OBJDUMP-NOTFOUND")
			set(XNG_AVR32_CONFIG_OK FALSE)
		endif()
	endif()
endif()
set(XNG_AVR32_OBJDUMP "${USER_XNG_AVR32_OBJDUMP}" CACHE FILEPATH "Path to or name of objdump.")

if(NOT DEFINED USER_XNG_AVR32_AVR32PROGRAM AND (NOT DEFINED XNG_AVR32_AVR32PROGRAM OR "${XNG_AVR32_AVR32PROGRAM}" STREQUAL "XNG_AVR32_AVR32PROGRAM-NOTFOUND"))
	find_program(PROGRAM_AVR32PROGRAM "avr32program")
	if(NOT "${PROGRAM_AVR32PROGRAM}" STREQUAL "PROGRAM_AVR32PROGRAM-NOTFOUND")
		set(USER_XNG_AVR32_AVR32PROGRAM "${PROGRAM_AVR32PROGRAM}")
		message(STATUS "Auto-detected avr32program: ${USER_XNG_AVR32_AVR32PROGRAM}")
	else()
		message(WARNING "USER_XNG_AVR32_AVR32PROGRAM undefined.")
		set(USER_XNG_AVR32_AVR32PROGRAM "XNG_AVR32_AVR32PROGRAM-NOTFOUND")
		set(XNG_AVR32_CONFIG_OK FALSE)
	endif()
endif()
set(XNG_AVR32_AVR32PROGRAM "${USER_XNG_AVR32_AVR32PROGRAM}" CACHE FILEPATH "Path to or name of avr32program.")

if(NOT DEFINED USER_XNG_AVR32_AVR32PROGRAM_FLAGS AND NOT DEFINED XNG_AVR32_AVR32PROGRAM_FLAGS)
	set(USER_XNG_AVR32_AVR32PROGRAM_FLAGS "")
endif()
set(XNG_AVR32_AVR32PROGRAM_FLAGS "${USER_XNG_AVR32_AVR32PROGRAM_FLAGS}" CACHE STRING "Arguments passed to avr32program.")

if(NOT DEFINED USER_XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS AND NOT DEFINED XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS)
	set(USER_XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "-finternal@0x80000000 -e -v -R -r")
endif()
set(XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "${USER_XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS}" CACHE STRING "Arguments passed to avr32program program.")

if(NOT DEFINED USER_XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND AND (NOT DEFINED XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND OR "${XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND}" STREQUAL "XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND-UNDEFINED"))
	set(USER_XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND "XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND-UNDEFINED")
endif()
set(XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND "${USER_XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND}" CACHE BOOL "Properly resume execution following a reset and run issued by avr32program.")

if(NOT XNG_AVR32_CONFIG_OK)
	if(NOT DEFINED XNG_AVR32_USER_CONFIG)
		message(WARNING "No user configuration file (user-config.cmake) could be found.\nLooked here: ${CMAKE_BINARY_DIR}/user-config.cmake ${CMAKE_SOURCE_DIR}/user-config.cmake\nThis file is optional and you may specify required options on the commandline instead. An example user configuration is provided here: ${CMAKE_SOURCE_DIR}/user-config.cmake.example")
	endif()
endif()

set(XNG_AVR32_CONFIG_READ TRUE)
