# Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
#
# This file is part of xng-cmake-template.
#
# xng-cmake-template is licensed under the Simplified BSD License
# (the "License").
#
# You should have received a copy of the License along with xng-cmake-template.
# If not, see <http://www.opensource.org/licenses/bsd-license.php>.
#
# Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>

if(DEFINED XNG_AVR32_CONFIG_READ AND NOT XNG_AVR32_CONFIG_OK)
	message(FATAL_ERROR "The configuration is incomplete. Cannot proceed.")
endif()

set(XNG_AVR32_TOOLCHAIN_FILE "${CMAKE_SOURCE_DIR}/cmake/xng-ct/avr32/toolchain.cmake")
set(XNG_AVR32_RERUN_CMAKE_MESSAGE "Delete the contents of the build directory \"${CMAKE_BINARY_DIR}\" and re-run cmake with the toolchain file \"${XNG_AVR32_TOOLCHAIN_FILE}\".")
if(DEFINED XNG_AVR32_CONFIGURED_WITHOUT_TOOLCHAIN_FILE AND DEFINED CMAKE_TOOLCHAIN_FILE)
	message(FATAL_ERROR "The project has been previously configured either without a toolchain file or an incorrect one. ${XNG_AVR32_RERUN_CMAKE_MESSAGE}")
elseif(NOT DEFINED CMAKE_TOOLCHAIN_FILE OR NOT "${CMAKE_TOOLCHAIN_FILE}" STREQUAL "${XNG_AVR32_TOOLCHAIN_FILE}")
	set(XNG_AVR32_CONFIGURED_WITHOUT_TOOLCHAIN_FILE TRUE CACHE INTERNAL "The project has been configured either without a toolchain file or an incorrect one.")
	message(FATAL_ERROR "Either no toolchain file or an incorrect one has been specified. ${XNG_AVR32_RERUN_CMAKE_MESSAGE}")
endif()

if(NOT DEFINED XNG_AVR32_CONFIG_READ)
	include("${CMAKE_SOURCE_DIR}/cmake/xng-ct/avr32/config.cmake")
endif()

if(NOT XNG_AVR32_CONFIG_OK)
	message(FATAL_ERROR "The configuration is incomplete. Cannot proceed.")
endif()

include("CMakeParseArguments")

macro(xng_project NAME)
	xng_project_common("${NAME}" ${ARGN} AVR32)
endmacro()

function(xng_add_program_target NAME TARGET)
	cmake_parse_arguments(XNG_AVR32_ADD_PROGRAM_TARGET "" "FORCE_RESUME_WORKAROUND" "AVR32PROGRAM_FLAGS;AVR32PROGRAM_PROGRAM_FLAGS" "" ${ARGN})

	separate_arguments(XNG_AVR32_AVR32PROGRAM_FLAGS UNIX_COMMAND "${XNG_AVR32_AVR32PROGRAM_FLAGS} ${XNG_AVR32_ADD_PROGRAM_TARGET_AVR32PROGRAM_FLAGS}")
	separate_arguments(XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS UNIX_COMMAND "${XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS} ${XNG_AVR32_ADD_PROGRAM_TARGET_AVR32PROGRAM_PROGRAM_FLAGS}")
	get_property(TARGET_LOCATION TARGET "${TARGET}" PROPERTY LOCATION)

	set(
		TARGET_COMMANDS
		${TARGET_COMMANDS}
		COMMAND "${CMAKE_MAKE_PROGRAM}"
		COMMAND "${XNG_AVR32_AVR32PROGRAM}" ${XNG_AVR32_AVR32PROGRAM_FLAGS} "program" ${XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS} "${TARGET_LOCATION}"
	)

	if(DEFINED XNG_AVR32_ADD_PROGRAM_TARGET_FORCE_RESUME_WORKAROUND)
		set(USE_RESUME_WORKAROUND ${XNG_AVR32_ADD_PROGRAM_TARGET_FORCE_RESUME_WORKAROUND})
	elseif(NOT "${XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND}" STREQUAL "XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND-UNDEFINED")
		set(USE_RESUME_WORKAROUND ${XNG_AVR32_AVR32PROGRAM_FORCE_RESUME_WORKAROUND})
	else()
		list(FIND XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "-R" XNG_AVR32_ADD_PROGRAM_TARGET_RESET_INDEX1)
		list(FIND XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "--reset" XNG_AVR32_ADD_PROGRAM_TARGET_RESET_INDEX2)
		list(FIND XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "-r" XNG_AVR32_ADD_PROGRAM_TARGET_RUN_INDEX1)
		list(FIND XNG_AVR32_AVR32PROGRAM_PROGRAM_FLAGS "--run" XNG_AVR32_ADD_PROGRAM_TARGET_RUN_INDEX2)

		if((${XNG_AVR32_ADD_PROGRAM_TARGET_RUN_INDEX1} GREATER -1 OR ${XNG_AVR32_ADD_PROGRAM_TARGET_RUN_INDEX2} GREATER -1) AND
		   (${XNG_AVR32_ADD_PROGRAM_TARGET_RESET_INDEX1} GREATER -1 OR ${XNG_AVR32_ADD_PROGRAM_TARGET_RESET_INDEX2} GREATER -1))
			set(USE_RESUME_WORKAROUND TRUE)
		else()
			set(USE_RESUME_WORKAROUND FALSE)
		endif()
	endif()

	if(USE_RESUME_WORKAROUND)
		if("${XNG_AVR32_OBJDUMP}" STREQUAL "XNG_AVR32_OBJDUMP-NOTFOUND")
			set(
				TARGET_COMMANDS
				${TARGET_COMMANDS}
				COMMAND "${CMAKE_SOURCE_DIR}/cmake/xng-avr32-avr32program-resume-workaround.sh" "--avr32program" "${XNG_AVR32_AVR32PROGRAM}" "--" ${XNG_AVR32_AVR32PROGRAM_FLAGS}
			)
		else()
			set(
				TARGET_COMMANDS
				${TARGET_COMMANDS}
				COMMAND "${CMAKE_SOURCE_DIR}/cmake/xng-avr32-avr32program-resume-workaround.sh" "--avr32program" "${XNG_AVR32_AVR32PROGRAM}" "--objdump" "${XNG_AVR32_OBJDUMP}" "--binary" "${TARGET_LOCATION}" "--" ${XNG_AVR32_AVR32PROGRAM_FLAGS}
			)
		endif()
	endif()

	add_custom_target(
		"${NAME}"
		${TARGET_COMMANDS}
		WORKING_DIRECTORY "${CMAKE_BINARY_DIR}"
		COMMENT "Programming ${TARGET}"
		VERBATIM
	)
endfunction()