/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef ADC_H_
#define ADC_H_

#include <at32uc3/inttypes.h>

/// ADC Address
#define ADC_ADDRESS					0xffff3c00
/// ADC Channel Address
#define ADC_CHANNEL_ADDRESS				(ADC_ADDRESS + 0x00000030)

/// ADC Control Register (Write)
#define ADC_REG_CR					0x00000000
/// ADC Control Register Start Conversion offset
#define ADC_REG_CR_START_OFFSET				0
/// ADC Control Register Start Conversion size
#define ADC_REG_CR_START_SIZE				1
/// ADC Control Register Start Conversion mask
#define ADC_REG_CR_START_MASK				(((1 << ADC_REG_CR_START_SIZE) - 1) << ADC_REG_CR_START_OFFSET)
/// ADC Control Register Software Reset offset
#define ADC_REG_CR_SWRST_OFFSET				1
/// ADC Control Register Software Reset size
#define ADC_REG_CR_SWRST_SIZE				1
/// ADC Control Register Software Reset mask
#define ADC_REG_CR_SWRST_MASK				(((1 << ADC_REG_CR_SWRST_SIZE) - 1) << ADC_REG_CR_SWRST_OFFSET)
/// ADC Mode Register (Read/Write)
#define ADC_REG_MR					0x00000004
/// ADC Mode Register Trigger Enable offset
#define ADC_REG_MR_TRGEN_OFFSET				0
/// ADC Mode Register Trigger Enable size
#define ADC_REG_MR_TRGEN_SIZE				1
/// ADC Mode Register Trigger Enable mask
#define ADC_REG_MR_TRGEN_MASK				(((1 << ADC_REG_MR_TRGEN_SIZE) - 1) << ADC_REG_MR_TRGEN_OFFSET)
/// ADC Mode Register Trigger Selection offset
#define ADC_REG_MR_TRGSEL_OFFSET			1
/// ADC Mode Register Trigger Selection size
#define ADC_REG_MR_TRGSEL_SIZE				3
/// ADC Mode Register Trigger Selection mask
#define ADC_REG_MR_TRGSEL_MASK				(((1 << ADC_REG_MR_TRGSEL_SIZE) - 1) << ADC_REG_MR_TRGSEL_OFFSET)
/// ADC Mode Register Resolution offset
#define ADC_REG_MR_LOWRES_OFFSET			4
/// ADC Mode Register Resolution size
#define ADC_REG_MR_LOWRES_SIZE				1
/// ADC Mode Register Resolution mask
#define ADC_REG_MR_LOWRES_MASK				(((1 << ADC_REG_MR_LOWRES_SIZE) - 1) << ADC_REG_MR_LOWRES_OFFSET)
/// ADC Mode Register Sleep Mode offset
#define ADC_REG_MR_SLEEP_OFFSET				5
/// ADC Mode Register Sleep Mode size
#define ADC_REG_MR_SLEEP_SIZE				1
/// ADC Mode Register Sleep Mode mask
#define ADC_REG_MR_SLEEP_MASK				(((1 << ADC_REG_MR_SLEEP_SIZE) - 1) << ADC_REG_MR_SLEEP_OFFSET)
/// ADC Mode Register Prescaler Rate Selection offset
#define ADC_REG_MR_PRESCAL_OFFSET			8
/// ADC Mode Register Prescaler Rate Selection size
#define ADC_REG_MR_PRESCAL_SIZE				6
/// ADC Mode Register Prescaler Rate Selection mask
#define ADC_REG_MR_PRESCAL_MASK				(((1 << ADC_REG_MR_PRESCAL_SIZE) - 1) << ADC_REG_MR_PRESCAL_OFFSET)
/// ADC Mode Register Start Up Time offset
#define ADC_REG_MR_STARTUP_OFFSET			16
/// ADC Mode Register Start Up Time size
#define ADC_REG_MR_STARTUP_SIZE				5
/// ADC Mode Register Start Up Time mask
#define ADC_REG_MR_STARTUP_MASK				(((1 << ADC_REG_MR_STARTUP_SIZE) - 1) << ADC_REG_MR_STARTUP_OFFSET)
/// ADC Mode Register Sample & Hold Time offset
#define ADC_REG_MR_SHTIM_OFFSET				24
/// ADC Mode Register Sample & Hold Time size
#define ADC_REG_MR_SHTIM_SIZE				4
/// ADC Mode Register Sample & Hold Time mask
#define ADC_REG_MR_SHTIM_MASK				(((1 << ADC_REG_MR_SHTIM_SIZE) - 1) << ADC_REG_MR_SHTIM_OFFSET)
/// ADC Channel Enable Register (Write)
#define ADC_REG_CHER					0x00000010
/// ADC Channel Disable Register (Write)
#define ADC_REG_CHDR					0x00000014
/// ADC Channel Status Register (Read)
#define ADC_REG_CHSR					0x00000018
/// ADC Channel Enable/Disable/Status Register Channel 0 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH0_OFFSET				0
/// ADC Channel Enable/Disable/Status Register Channel 0 Enable/Disable/Status size
#define ADC_REG_CHXR_CH0_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 0 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH0_MASK				(((1 << ADC_REG_CHXR_CH0_SIZE) - 1) << ADC_REG_CHXR_CH0_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 1 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH1_OFFSET				1
/// ADC Channel Enable/Disable/Status Register Channel 1 Enable/Disable/Status size
#define ADC_REG_CHXR_CH1_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 1 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH1_MASK				(((1 << ADC_REG_CHXR_CH1_SIZE) - 1) << ADC_REG_CHXR_CH1_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 2 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH2_OFFSET				2
/// ADC Channel Enable/Disable/Status Register Channel 2 Enable/Disable/Status size
#define ADC_REG_CHXR_CH2_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 2 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH2_MASK				(((1 << ADC_REG_CHXR_CH2_SIZE) - 1) << ADC_REG_CHXR_CH2_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 3 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH3_OFFSET				3
/// ADC Channel Enable/Disable/Status Register Channel 3 Enable/Disable/Status size
#define ADC_REG_CHXR_CH3_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 3 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH3_MASK				(((1 << ADC_REG_CHXR_CH3_SIZE) - 1) << ADC_REG_CHXR_CH3_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 4 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH4_OFFSET				4
/// ADC Channel Enable/Disable/Status Register Channel 4 Enable/Disable/Status size
#define ADC_REG_CHXR_CH4_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 4 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH4_MASK				(((1 << ADC_REG_CHXR_CH4_SIZE) - 1) << ADC_REG_CHXR_CH4_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 5 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH5_OFFSET				5
/// ADC Channel Enable/Disable/Status Register Channel 5 Enable/Disable/Status size
#define ADC_REG_CHXR_CH5_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 5 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH5_MASK				(((1 << ADC_REG_CHXR_CH5_SIZE) - 1) << ADC_REG_CHXR_CH5_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 6 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH6_OFFSET				6
/// ADC Channel Enable/Disable/Status Register Channel 6 Enable/Disable/Status size
#define ADC_REG_CHXR_CH6_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 6 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH6_MASK				(((1 << ADC_REG_CHXR_CH6_SIZE) - 1) << ADC_REG_CHXR_CH6_OFFSET)
/// ADC Channel Enable/Disable/Status Register Channel 7 Enable/Disable/Status offset
#define ADC_REG_CHXR_CH7_OFFSET				7
/// ADC Channel Enable/Disable/Status Register Channel 7 Enable/Disable/Status size
#define ADC_REG_CHXR_CH7_SIZE				1
/// ADC Channel Enable/Disable/Status Register Channel 7 Enable/Disable/Status mask
#define ADC_REG_CHXR_CH7_MASK				(((1 << ADC_REG_CHXR_CH7_SIZE) - 1) << ADC_REG_CHXR_CH7_OFFSET)
/// ADC Status Register (Read)
#define ADC_REG_SR					0x0000001c
/// ADC Status Register End of Conversion 0 offset
#define ADC_REG_SR_EOC0_OFFSET				0
/// ADC Status Register End of Conversion 0 size
#define ADC_REG_SR_EOC0_SIZE				1
/// ADC Status Register End of Conversion 0 mask
#define ADC_REG_SR_EOC0_MASK				(((1 << ADC_REG_SR_EOC0_SIZE) - 1) << ADC_REG_SR_EOC0_OFFSET)
/// ADC Status Register End of Conversion 1 offset
#define ADC_REG_SR_EOC1_OFFSET				1
/// ADC Status Register End of Conversion 1 size
#define ADC_REG_SR_EOC1_SIZE				1
/// ADC Status Register End of Conversion 1 mask
#define ADC_REG_SR_EOC1_MASK				(((1 << ADC_REG_SR_EOC1_SIZE) - 1) << ADC_REG_SR_EOC1_OFFSET)
/// ADC Status Register End of Conversion 2 offset
#define ADC_REG_SR_EOC2_OFFSET				2
/// ADC Status Register End of Conversion 2 size
#define ADC_REG_SR_EOC2_SIZE				1
/// ADC Status Register End of Conversion 2 mask
#define ADC_REG_SR_EOC2_MASK				(((1 << ADC_REG_SR_EOC2_SIZE) - 1) << ADC_REG_SR_EOC2_OFFSET)
/// ADC Status Register End of Conversion 3 offset
#define ADC_REG_SR_EOC3_OFFSET				3
/// ADC Status Register End of Conversion 3 size
#define ADC_REG_SR_EOC3_SIZE				1
/// ADC Status Register End of Conversion 3 mask
#define ADC_REG_SR_EOC3_MASK				(((1 << ADC_REG_SR_EOC3_SIZE) - 1) << ADC_REG_SR_EOC3_OFFSET)
/// ADC Status Register End of Conversion 4 offset
#define ADC_REG_SR_EOC4_OFFSET				4
/// ADC Status Register End of Conversion 4 size
#define ADC_REG_SR_EOC4_SIZE				1
/// ADC Status Register End of Conversion 4 mask
#define ADC_REG_SR_EOC4_MASK				(((1 << ADC_REG_SR_EOC4_SIZE) - 1) << ADC_REG_SR_EOC4_OFFSET)
/// ADC Status Register End of Conversion 5 offset
#define ADC_REG_SR_EOC5_OFFSET				5
/// ADC Status Register End of Conversion 5 size
#define ADC_REG_SR_EOC5_SIZE				1
/// ADC Status Register End of Conversion 5 mask
#define ADC_REG_SR_EOC5_MASK				(((1 << ADC_REG_SR_EOC5_SIZE) - 1) << ADC_REG_SR_EOC5_OFFSET)
/// ADC Status Register End of Conversion 6 offset
#define ADC_REG_SR_EOC6_OFFSET				6
/// ADC Status Register End of Conversion 6 size
#define ADC_REG_SR_EOC6_SIZE				1
/// ADC Status Register End of Conversion 6 mask
#define ADC_REG_SR_EOC6_MASK				(((1 << ADC_REG_SR_EOC6_SIZE) - 1) << ADC_REG_SR_EOC6_OFFSET)
/// ADC Status Register End of Conversion 7 offset
#define ADC_REG_SR_EOC7_OFFSET				7
/// ADC Status Register End of Conversion 7 size
#define ADC_REG_SR_EOC7_SIZE				1
/// ADC Status Register End of Conversion 7 mask
#define ADC_REG_SR_EOC7_MASK				(((1 << ADC_REG_SR_EOC7_SIZE) - 1) << ADC_REG_SR_EOC7_OFFSET)
/// ADC Status Register Overrun Error 0 offset
#define ADC_REG_SR_OVRE0_OFFSET				8
/// ADC Status Register Overrun Error 0 size
#define ADC_REG_SR_OVRE0_SIZE				1
/// ADC Status Register Overrun Error 0 mask
#define ADC_REG_SR_OVRE0_MASK				(((1 << ADC_REG_SR_OVRE0_SIZE) - 1) << ADC_REG_SR_OVRE0_OFFSET)
/// ADC Status Register Overrun Error 1 offset
#define ADC_REG_SR_OVRE1_OFFSET				9
/// ADC Status Register Overrun Error 1 size
#define ADC_REG_SR_OVRE1_SIZE				1
/// ADC Status Register Overrun Error 1 mask
#define ADC_REG_SR_OVRE1_MASK				(((1 << ADC_REG_SR_OVRE1_SIZE) - 1) << ADC_REG_SR_OVRE1_OFFSET)
/// ADC Status Register Overrun Error 2 offset
#define ADC_REG_SR_OVRE2_OFFSET				10
/// ADC Status Register Overrun Error 2 size
#define ADC_REG_SR_OVRE2_SIZE				1
/// ADC Status Register Overrun Error 2 mask
#define ADC_REG_SR_OVRE2_MASK				(((1 << ADC_REG_SR_OVRE2_SIZE) - 1) << ADC_REG_SR_OVRE2_OFFSET)
/// ADC Status Register Overrun Error 3 offset
#define ADC_REG_SR_OVRE3_OFFSET				11
/// ADC Status Register Overrun Error 3 size
#define ADC_REG_SR_OVRE3_SIZE				1
/// ADC Status Register Overrun Error 3 mask
#define ADC_REG_SR_OVRE3_MASK				(((1 << ADC_REG_SR_OVRE3_SIZE) - 1) << ADC_REG_SR_OVRE3_OFFSET)
/// ADC Status Register Overrun Error 4 offset
#define ADC_REG_SR_OVRE4_OFFSET				12
/// ADC Status Register Overrun Error 4 size
#define ADC_REG_SR_OVRE4_SIZE				1
/// ADC Status Register Overrun Error 4 mask
#define ADC_REG_SR_OVRE4_MASK				(((1 << ADC_REG_SR_OVRE4_SIZE) - 1) << ADC_REG_SR_OVRE4_OFFSET)
/// ADC Status Register Overrun Error 5 offset
#define ADC_REG_SR_OVRE5_OFFSET				13
/// ADC Status Register Overrun Error 5 size
#define ADC_REG_SR_OVRE5_SIZE				1
/// ADC Status Register Overrun Error 5 mask
#define ADC_REG_SR_OVRE5_MASK				(((1 << ADC_REG_SR_OVRE5_SIZE) - 1) << ADC_REG_SR_OVRE5_OFFSET)
/// ADC Status Register Overrun Error 6 offset
#define ADC_REG_SR_OVRE6_OFFSET				14
/// ADC Status Register Overrun Error 6 size
#define ADC_REG_SR_OVRE6_SIZE				1
/// ADC Status Register Overrun Error 6 mask
#define ADC_REG_SR_OVRE6_MASK				(((1 << ADC_REG_SR_OVRE6_SIZE) - 1) << ADC_REG_SR_OVRE6_OFFSET)
/// ADC Status Register Overrun Error 7 offset
#define ADC_REG_SR_OVRE7_OFFSET				15
/// ADC Status Register Overrun Error 7 size
#define ADC_REG_SR_OVRE7_SIZE				1
/// ADC Status Register Overrun Error 7 mask
#define ADC_REG_SR_OVRE7_MASK				(((1 << ADC_REG_SR_OVRE7_SIZE) - 1) << ADC_REG_SR_OVRE7_OFFSET)
/// ADC Status Register Data Ready offset
#define ADC_REG_SR_DRDY_OFFSET				16
/// ADC Status Register Data Ready size
#define ADC_REG_SR_DRDY_SIZE				1
/// ADC Status Register Data Ready mask
#define ADC_REG_SR_DRDY_MASK				(((1 << ADC_REG_SR_DRDY_SIZE) - 1) << ADC_REG_SR_DRDY_OFFSET)
/// ADC Status Register General Overrun Error offset
#define ADC_REG_SR_GOVRE_OFFSET				17
/// ADC Status Register General Overrun Error size
#define ADC_REG_SR_GOVRE_SIZE				1
/// ADC Status Register General Overrun Error mask
#define ADC_REG_SR_GOVRE_MASK				(((1 << ADC_REG_SR_GOVRE_SIZE) - 1) << ADC_REG_SR_GOVRE_OFFSET)
/// ADC Status Register End of RX Buffer offset
#define ADC_REG_SR_ENDRX_OFFSET				18
/// ADC Status Register End of RX Buffer size
#define ADC_REG_SR_ENDRX_SIZE				1
/// ADC Status Register End of RX Buffer mask
#define ADC_REG_SR_ENDRX_MASK				(((1 << ADC_REG_SR_ENDRX_SIZE) - 1) << ADC_REG_SR_ENDRX_OFFSET)
/// ADC Status Register RX Buffer Full offset
#define ADC_REG_SR_RXBUFF_OFFSET			19
/// ADC Status Register RX Buffer Full size
#define ADC_REG_SR_RXBUFF_SIZE				1
/// ADC Status Register RX Buffer Full mask
#define ADC_REG_SR_RXBUFF_MASK				(((1 << ADC_REG_SR_RXBUFF_SIZE) - 1) << ADC_REG_SR_RXBUFF_OFFSET)
/// ADC Last Converted Data Register (Read)
#define ADC_REG_LCDR					0x00000020
/// ADC Last Converted Data Register Last Data Converted offset
#define ADC_REG_LCDR_LDATA_OFFSET			0
/// ADC Last Converted Data Register Last Data Converted size
#define ADC_REG_LCDR_LDATA_SIZE				10
/// ADC Last Converted Data Register Last Data Converted mask
#define ADC_REG_LCDR_LDATA_MASK				(((1 << ADC_REG_LCDR_LDATA_SIZE) - 1) << ADC_REG_LCDR_LDATA_OFFSET)
/// ADC Interrupt Enable Register (Write)
#define ADC_REG_IER					0x00000024
/// ADC Interrupt Disable Register (Write)
#define ADC_REG_IDR					0x00000028
/// ADC Interrupt Mask Register (Read)
#define ADC_REG_IMR					0x0000002c
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 0 offset
#define ADC_REG_IXR_EOC0_OFFSET				0
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 0 size
#define ADC_REG_IXR_EOC0_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 0 mask
#define ADC_REG_IXR_EOC0_MASK				(((1 << ADC_REG_IXR_EOC0_SIZE) - 1) << ADC_REG_IXR_EOC0_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 1 offset
#define ADC_REG_IXR_EOC1_OFFSET				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 1 size
#define ADC_REG_IXR_EOC1_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 1 mask
#define ADC_REG_IXR_EOC1_MASK				(((1 << ADC_REG_IXR_EOC1_SIZE) - 1) << ADC_REG_IXR_EOC1_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 2 offset
#define ADC_REG_IXR_EOC2_OFFSET				2
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 2 size
#define ADC_REG_IXR_EOC2_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 2 mask
#define ADC_REG_IXR_EOC2_MASK				(((1 << ADC_REG_IXR_EOC2_SIZE) - 1) << ADC_REG_IXR_EOC2_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 3 offset
#define ADC_REG_IXR_EOC3_OFFSET				3
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 3 size
#define ADC_REG_IXR_EOC3_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 3 mask
#define ADC_REG_IXR_EOC3_MASK				(((1 << ADC_REG_IXR_EOC3_SIZE) - 1) << ADC_REG_IXR_EOC3_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 4 offset
#define ADC_REG_IXR_EOC4_OFFSET				4
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 4 size
#define ADC_REG_IXR_EOC4_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 4 mask
#define ADC_REG_IXR_EOC4_MASK				(((1 << ADC_REG_IXR_EOC4_SIZE) - 1) << ADC_REG_IXR_EOC4_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 5 offset
#define ADC_REG_IXR_EOC5_OFFSET				5
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 5 size
#define ADC_REG_IXR_EOC5_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 5 mask
#define ADC_REG_IXR_EOC5_MASK				(((1 << ADC_REG_IXR_EOC5_SIZE) - 1) << ADC_REG_IXR_EOC5_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 6 offset
#define ADC_REG_IXR_EOC6_OFFSET				6
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 6 size
#define ADC_REG_IXR_EOC6_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 6 mask
#define ADC_REG_IXR_EOC6_MASK				(((1 << ADC_REG_IXR_EOC6_SIZE) - 1) << ADC_REG_IXR_EOC6_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 7 offset
#define ADC_REG_IXR_EOC7_OFFSET				7
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 7 size
#define ADC_REG_IXR_EOC7_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of Conversion Interrupt Enable/Disable/Mask 7 mask
#define ADC_REG_IXR_EOC7_MASK				(((1 << ADC_REG_IXR_EOC7_SIZE) - 1) << ADC_REG_IXR_EOC7_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 0 offset
#define ADC_REG_IXR_OVRE0_OFFSET			8
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 0 size
#define ADC_REG_IXR_OVRE0_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 0 mask
#define ADC_REG_IXR_OVRE0_MASK				(((1 << ADC_REG_IXR_OVRE0_SIZE) - 1) << ADC_REG_IXR_OVRE0_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 1 offset
#define ADC_REG_IXR_OVRE1_OFFSET			9
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 1 size
#define ADC_REG_IXR_OVRE1_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 1 mask
#define ADC_REG_IXR_OVRE1_MASK				(((1 << ADC_REG_IXR_OVRE1_SIZE) - 1) << ADC_REG_IXR_OVRE1_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 2 offset
#define ADC_REG_IXR_OVRE2_OFFSET			10
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 2 size
#define ADC_REG_IXR_OVRE2_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 2 mask
#define ADC_REG_IXR_OVRE2_MASK				(((1 << ADC_REG_IXR_OVRE2_SIZE) - 1) << ADC_REG_IXR_OVRE2_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 3 offset
#define ADC_REG_IXR_OVRE3_OFFSET			11
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 3 size
#define ADC_REG_IXR_OVRE3_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 3 mask
#define ADC_REG_IXR_OVRE3_MASK				(((1 << ADC_REG_IXR_OVRE3_SIZE) - 1) << ADC_REG_IXR_OVRE3_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 4 offset
#define ADC_REG_IXR_OVRE4_OFFSET			12
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 4 size
#define ADC_REG_IXR_OVRE4_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 4 mask
#define ADC_REG_IXR_OVRE4_MASK				(((1 << ADC_REG_IXR_OVRE4_SIZE) - 1) << ADC_REG_IXR_OVRE4_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 5 offset
#define ADC_REG_IXR_OVRE5_OFFSET			13
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 5 size
#define ADC_REG_IXR_OVRE5_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 5 mask
#define ADC_REG_IXR_OVRE5_MASK				(((1 << ADC_REG_IXR_OVRE5_SIZE) - 1) << ADC_REG_IXR_OVRE5_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 6 offset
#define ADC_REG_IXR_OVRE6_OFFSET			14
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 6 size
#define ADC_REG_IXR_OVRE6_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 6 mask
#define ADC_REG_IXR_OVRE6_MASK				(((1 << ADC_REG_IXR_OVRE6_SIZE) - 1) << ADC_REG_IXR_OVRE6_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 7 offset
#define ADC_REG_IXR_OVRE7_OFFSET			15
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 7 size
#define ADC_REG_IXR_OVRE7_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask 7 mask
#define ADC_REG_IXR_OVRE7_MASK				(((1 << ADC_REG_IXR_OVRE7_SIZE) - 1) << ADC_REG_IXR_OVRE7_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register Data Ready Interrupt Enable/Disable/Mask offset
#define ADC_REG_IXR_DRDY_OFFSET				16
/// ADC Interrupt Enable/Disable/Mask Register Data Ready Interrupt Enable/Disable/Mask size
#define ADC_REG_IXR_DRDY_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register Data Ready Interrupt Enable/Disable/Mask mask
#define ADC_REG_IXR_DRDY_MASK				(((1 << ADC_REG_IXR_DRDY_SIZE) - 1) << ADC_REG_IXR_DRDY_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register General Overrun Error Interrupt Enable/Disable/Mask offset
#define ADC_REG_IXR_GOVRE_OFFSET			17
/// ADC Interrupt Enable/Disable/Mask Register General Overrun Error Interrupt Enable/Disable/Mask size
#define ADC_REG_IXR_GOVRE_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register General Overrun Error Interrupt Enable/Disable/Mask mask
#define ADC_REG_IXR_GOVRE_MASK				(((1 << ADC_REG_IXR_GOVRE_SIZE) - 1) << ADC_REG_IXR_GOVRE_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register End of RX Buffer Interrupt Enable/Disable/Mask offset
#define ADC_REG_IXR_ENDRX_OFFSET			18
/// ADC Interrupt Enable/Disable/Mask Register End of RX Buffer Interrupt Enable/Disable/Mask size
#define ADC_REG_IXR_ENDRX_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register End of RX Buffer Interrupt Enable/Disable/Mask mask
#define ADC_REG_IXR_ENDRX_MASK				(((1 << ADC_REG_IXR_ENDRX_SIZE) - 1) << ADC_REG_IXR_ENDRX_OFFSET)
/// ADC Interrupt Enable/Disable/Mask Register RX Buffer Full Interrupt Enable/Disable/Mask offset
#define ADC_REG_IXR_RXBUFF_OFFSET			19
/// ADC Interrupt Enable/Disable/Mask Register RX Buffer Full Interrupt Enable/Disable/Mask size
#define ADC_REG_IXR_RXBUFF_SIZE				1
/// ADC Interrupt Enable/Disable/Mask Register RX Buffer Full Interrupt Enable/Disable/Mask mask
#define ADC_REG_IXR_RXBUFF_MASK				(((1 << ADC_REG_IXR_RXBUFF_SIZE) - 1) << ADC_REG_IXR_RXBUFF_OFFSET)
/// ADC Channel Data Register 0 (Read)
#define ADC_REG_CDR0					0x00000030
/// ADC Channel Data Register 1 (Read)
#define ADC_REG_CDR1					0x00000034
/// ADC Channel Data Register 2 (Read)
#define ADC_REG_CDR2					0x00000038
/// ADC Channel Data Register 3 (Read)
#define ADC_REG_CDR3					0x0000003c
/// ADC Channel Data Register 4 (Read)
#define ADC_REG_CDR4					0x00000040
/// ADC Channel Data Register 5 (Read)
#define ADC_REG_CDR5					0x00000044
/// ADC Channel Data Register 6 (Read)
#define ADC_REG_CDR6					0x00000048
/// ADC Channel Data Register 7 (Read)
#define ADC_REG_CDR7					0x0000004c
/// ADC Channel Data Register n Converted Data offset
#define ADC_REG_CDRN_DATA_OFFSET			0
/// ADC Channel Data Register n Converted Data size
#define ADC_REG_CDRN_DATA_SIZE				10
/// ADC Channel Data Register n Converted Data mask
#define ADC_REG_CDRN_DATA_MASK				(((1 << ADC_REG_CDRN_DATA_SIZE) - 1) << ADC_REG_CDRN_DATA_OFFSET)
/// ADC Version Register (Read)
#define ADC_REG_VERSION					0x000000fc
/// ADC Version Register Version Number offset
#define ADC_REG_VERSION_VERSION_OFFSET			0
/// ADC Version Register Version Number size
#define ADC_REG_VERSION_VERSION_SIZE			12
/// ADC Version Register Version Number mask
#define ADC_REG_VERSION_VERSION_MASK			(((1 << ADC_REG_VERSION_VERSION_SIZE) - 1) << ADC_REG_VERSION_VERSION_OFFSET)
/// ADC Version Register Variant Number offset
#define ADC_REG_VERSION_VARIANT_OFFSET			16
/// ADC Version Register Variant Number size
#define ADC_REG_VERSION_VARIANT_SIZE			4
/// ADC Version Register Variant Number mask
#define ADC_REG_VERSION_VARIANT_MASK			(((1 << ADC_REG_VERSION_VARIANT_SIZE) - 1) << ADC_REG_VERSION_VARIANT_OFFSET)

typedef enum {
	ADC_HARDWARE_TRIGGER_MODE_DISABLED		= 0,
	ADC_HARDWARE_TRIGGER_MODE_ENABLED		= ADC_REG_MR_TRGEN_MASK,
} adc_hardware_trigger_mode_t;

typedef enum {
	ADC_RESOLUTION_10BIT				= 0,
	ADC_RESOLUTION_8BIT				= ADC_REG_MR_LOWRES_MASK,
} adc_resolution_t;

typedef enum {
	ADC_SLEEP_MODE_DISABLED				= 0,
	ADC_SLEEP_MODE_ENABLED				= ADC_REG_MR_SLEEP_MASK,
} adc_sleep_mode_t;

typedef enum {
	ADC_HARDWARE_TRIGGER_NONE			= 0b000,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_0		= 0b000,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_1		= 0b001,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_2		= 0b010,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_3		= 0b011,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_4		= 0b100,
	ADC_HARDWARE_TRIGGER_INTERNAL_TRIGGER_5		= 0b101,
	ADC_HARDWARE_TRIGGER_EXERNAL_TRIGGER		= 0b110,
} adc_hardware_trigger_t;

typedef enum {
	ADC_STATUS_END_OF_CONVERSION0			= ADC_REG_SR_EOC0_MASK,
	ADC_STATUS_END_OF_CONVERSION1			= ADC_REG_SR_EOC1_MASK,
	ADC_STATUS_END_OF_CONVERSION2			= ADC_REG_SR_EOC2_MASK,
	ADC_STATUS_END_OF_CONVERSION3			= ADC_REG_SR_EOC3_MASK,
	ADC_STATUS_END_OF_CONVERSION4			= ADC_REG_SR_EOC4_MASK,
	ADC_STATUS_END_OF_CONVERSION5			= ADC_REG_SR_EOC5_MASK,
	ADC_STATUS_END_OF_CONVERSION6			= ADC_REG_SR_EOC6_MASK,
	ADC_STATUS_END_OF_CONVERSION7			= ADC_REG_SR_EOC7_MASK,
	ADC_STATUS_OVERRUN_ERROR0			= ADC_REG_SR_OVRE0_MASK,
	ADC_STATUS_OVERRUN_ERROR1			= ADC_REG_SR_OVRE1_MASK,
	ADC_STATUS_OVERRUN_ERROR2			= ADC_REG_SR_OVRE2_MASK,
	ADC_STATUS_OVERRUN_ERROR3			= ADC_REG_SR_OVRE3_MASK,
	ADC_STATUS_OVERRUN_ERROR4			= ADC_REG_SR_OVRE4_MASK,
	ADC_STATUS_OVERRUN_ERROR5			= ADC_REG_SR_OVRE5_MASK,
	ADC_STATUS_OVERRUN_ERROR6			= ADC_REG_SR_OVRE6_MASK,
	ADC_STATUS_OVERRUN_ERROR7			= ADC_REG_SR_OVRE7_MASK,
	ADC_STATUS_DATA_READY				= ADC_REG_SR_DRDY_MASK,
	ADC_STATUS_DATA_GENERAL_OVERRUN_ERROR		= ADC_REG_SR_GOVRE_MASK,
	ADC_STATUS_END_OF_RX_BUFFER			= ADC_REG_SR_ENDRX_MASK,
	ADC_STATUS_RX_BUFFER_FULL			= ADC_REG_SR_RXBUFF_MASK,
} adc_status_t;

typedef enum {
	ADC_CHANNEL_STATUS_CH0_ENABLED			= ADC_REG_CHXR_CH0_MASK,
	ADC_CHANNEL_STATUS_CH1_ENABLED			= ADC_REG_CHXR_CH1_MASK,
	ADC_CHANNEL_STATUS_CH2_ENABLED			= ADC_REG_CHXR_CH2_MASK,
	ADC_CHANNEL_STATUS_CH3_ENABLED			= ADC_REG_CHXR_CH3_MASK,
	ADC_CHANNEL_STATUS_CH4_ENABLED			= ADC_REG_CHXR_CH4_MASK,
	ADC_CHANNEL_STATUS_CH5_ENABLED			= ADC_REG_CHXR_CH5_MASK,
	ADC_CHANNEL_STATUS_CH6_ENABLED			= ADC_REG_CHXR_CH6_MASK,
	ADC_CHANNEL_STATUS_CH7_ENABLED			= ADC_REG_CHXR_CH7_MASK,
} adc_channel_status_t;

typedef enum {
	ADC_INTERRUPT_END_OF_CONVERSION0		= ADC_REG_IXR_EOC0_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION1		= ADC_REG_IXR_EOC1_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION2		= ADC_REG_IXR_EOC2_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION3		= ADC_REG_IXR_EOC3_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION4		= ADC_REG_IXR_EOC4_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION5		= ADC_REG_IXR_EOC5_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION6		= ADC_REG_IXR_EOC6_MASK,
	ADC_INTERRUPT_END_OF_CONVERSION7		= ADC_REG_IXR_EOC7_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR0			= ADC_REG_IXR_OVRE0_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR1			= ADC_REG_IXR_OVRE1_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR2			= ADC_REG_IXR_OVRE2_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR3			= ADC_REG_IXR_OVRE3_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR4			= ADC_REG_IXR_OVRE4_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR5			= ADC_REG_IXR_OVRE5_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR6			= ADC_REG_IXR_OVRE6_MASK,
	ADC_INTERRUPT_OVERRUN_ERROR7			= ADC_REG_IXR_OVRE7_MASK,
	ADC_INTERRUPT_DATA_READY			= ADC_REG_IXR_DRDY_MASK,
	ADC_INTERRUPT_DATA_GENERAL_OVERRUN_ERROR	= ADC_REG_IXR_GOVRE_MASK,
	ADC_INTERRUPT_END_OF_RX_BUFFER			= ADC_REG_IXR_ENDRX_MASK,
	ADC_INTERRUPT_RX_BUFFER_FULL			= ADC_REG_IXR_RXBUFF_MASK,
} adc_interrupts_t;

extern uint32_t adc_read_register(uint32_t const offset);
extern void adc_write_register(uint32_t const offset, uint32_t const value);

extern void adc_sw_reset(void);
extern void adc_configure(adc_hardware_trigger_mode_t const trigger_mode, adc_resolution_t const resolution, adc_sleep_mode_t const sleep_mode, adc_hardware_trigger_t const trigger, uint8_t const  prescal, uint8_t const startup, uint8_t const shtim);

extern void adc_enable_channel(uint8_t const channel_nb);
extern void adc_disable_channel(uint8_t const channel_nb);

extern void adc_start(void);
extern void adc_wait(uint8_t const channel_nb);

extern adc_status_t adc_read_status(void);
extern adc_channel_status_t adc_read_channel_status(void);

extern uint16_t adc_read_value(void);
extern uint16_t adc_read_channel_value(uint8_t const channel_nb);

extern void adc_enable_interrupts(adc_interrupts_t const interrupts);
extern void adc_disable_interrupts(adc_interrupts_t const interrupts);
extern adc_interrupts_t adc_interrupts_mask(void);

#endif /* ADC_H_ */

