/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef EXCEPTIONS_H_
#define EXCEPTIONS_H_

#include <at32uc3/gccmacros.h>
#include <at32uc3/inttypes.h>

typedef void (*exceptions_handler_t)(void);

extern void exceptions_enable(void);
extern uint32_t exceptions_enabled(void);
extern void exceptions_disable(void);
extern void exceptions_set_unrecoverable_exception_handler(exceptions_handler_t const handler);
extern void exceptions_set_tlb_multiple_hit_handler(exceptions_handler_t const handler);
extern void exceptions_set_bus_error_data_fetch_handler(exceptions_handler_t const handler);
extern void exceptions_set_bus_error_instruction_fetch_handler(exceptions_handler_t const handler);
extern void exceptions_set_nmi_handler(exceptions_handler_t const handler);
extern void exceptions_set_instruction_address_handler(exceptions_handler_t const handler);
extern void exceptions_set_itlb_protection_handler(exceptions_handler_t const handler);
extern void exceptions_set_breakpoint_handler(exceptions_handler_t const handler);
extern void exceptions_set_illegal_opcode_handler(exceptions_handler_t const handler);
extern void exceptions_set_unimplemented_instruction_handler(exceptions_handler_t const handler);
extern void exceptions_set_privilege_violation_handler(exceptions_handler_t const handler);
extern void exceptions_set_data_address_read_handler(exceptions_handler_t const handler);
extern void exceptions_set_data_address_write_handler(exceptions_handler_t const handler);
extern void exceptions_set_dtlb_protection_read_handler(exceptions_handler_t const handler);
extern void exceptions_set_dtlb_protection_write_handler(exceptions_handler_t const handler);
extern void exceptions_set_itlb_miss_handler(exceptions_handler_t const handler);
extern void exceptions_set_dtlb_miss_read_handler(exceptions_handler_t const handler);
extern void exceptions_set_dtlb_miss_write_handler(exceptions_handler_t const handler);
extern void exceptions_set_supervisor_call_handler(exceptions_handler_t const handler);

#endif /* EXCEPTIONS_H_ */

