/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <inttypes.h>

#ifndef FLASHC_H_
#define FLASHC_H_

#include <at32uc3/inttypes.h>

/// FLASHC address
#define FLASHC_ADDRESS					0xfffe1400

// FLASHC Flash Control Register (Read/Write)
#define FLASHC_REG_FCR					0x00000000
// FLASHC Flash Control Register Flash Wait State offset
#define FLASHC_REG_FCR_FWS_OFFSET			6
// FLASHC Flash Control Register Flash Wait State size
#define FLASHC_REG_FCR_FWS_SIZE				1
// FLASHC Flash Control Register Flash Wait State mask
#define FLASHC_REG_FCR_FWS_MASK				(((1 << FLASHC_REG_FCR_FWS_SIZE) - 1) << FLASHC_REG_FCR_FWS_OFFSET)
// FLASHC Flash Command Register (Read/Write)
#define FLASHC_REG_FCMD					0x00000004
// FLASHC Flash Status Register (Read/Write)
#define FLASHC_REG_FSR					0x00000008
// FLASHC Flash General Purpose Fuse Register High (Read)
#define FLASHC_REG_FGPFRHI				0x0000000c
// FLASHC Flash General Purpose Fuse Register Low (Read)
#define FLASHC_REG_FGPFRLO				0x00000010

typedef enum {
	FLASHC_WAIT_STATE_NONE				= 0,
	FLASHC_WAIT_STATE_ONE				= FLASHC_REG_FCR_FWS_MASK,
} flashc_wait_state_t;

extern void flashc_set_wait_state(flashc_wait_state_t const wait_state);

#endif /* FLASHC_H_ */

