/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef GPIO_H_
#define GPIO_H_

#include <at32uc3/inttypes.h>

/// GPIO Address
#define GPIO_ADDRESS					0xffff1000
/// GPIO Port Width
#define GPIO_PORT_WIDTH					0x00000100
/// Number of GPIO ports
#define GPIO_NUM_PORTS					4

#define GPIO_PORT(n)					((void volatile *)GPIO_ADDRESS + GPIO_PORT_WIDTH * (n))

/// GPIO Enable Register (Read/Write)
#define GPIO_REG_GPER					0x00000000
/// GPIO Enable Register (Set)
#define GPIO_REG_GPERS					0x00000004
/// GPIO Enable Register (Clear)
#define GPIO_REG_GPERC					0x00000008
/// GPIO Enable Register (Toggle)
#define GPIO_REG_GPERT					0x0000000c
/// GPIO Peripheral Mux Register 0 (Read/Write)
#define GPIO_REG_PMR0					0x00000010
/// GPIO Peripheral Mux Register 0 (Set)
#define GPIO_REG_PMR0S					0x00000014
/// GPIO Peripheral Mux Register 0 (Clear)
#define GPIO_REG_PMR0C					0x00000018
/// GPIO Peripheral Mux Register 0 (Toggle)
#define GPIO_REG_PMR0T					0x0000001c
/// GPIO Peripheral Mux Register 1 (Read/Write)
#define GPIO_REG_PMR1					0x00000020
/// GPIO Peripheral Mux Register 1 (Set)
#define GPIO_REG_PMR1S					0x00000024
/// GPIO Peripheral Mux Register 1 (Clear)
#define GPIO_REG_PMR1C					0x00000028
/// GPIO Peripheral Mux Register 1 (Toggle)
#define GPIO_REG_PMR1T					0x0000002c
/// GPIO Output Driver Enable Register (Read/Write)
#define GPIO_REG_ODER					0x00000040
/// GPIO Output Driver Enable Register (Set)
#define GPIO_REG_ODERS					0x00000044
/// GPIO Output Driver Enable Register (Clear)
#define GPIO_REG_ODERC					0x00000048
/// GPIO Output Driver Enable Register (Toggle)
#define GPIO_REG_ODERT					0x0000004c
/// GPIO Output Value Register (Read/Write)
#define GPIO_REG_OVR					0x00000050
/// GPIO Output Value Register (Set)
#define GPIO_REG_OVRS					0x00000054
/// GPIO Output Value Register (Clear)
#define GPIO_REG_OVRC					0x00000058
/// GPIO Output Value Register (Toggle)
#define GPIO_REG_OVRT					0x0000005c
/// GPIO Pin Value Register (Read)
#define GPIO_REG_PVR					0x00000060
/// GPIO Pull-up Enable Register (Read/Write)
#define GPIO_REG_PUER					0x00000070
/// GPIO Pull-up Enable Register (Set)
#define GPIO_REG_PUERS					0x00000074
/// GPIO Pull-up Enable Register (Clear)
#define GPIO_REG_PUERC					0x00000078
/// GPIO Pull-up Enable Register (Toggle)
#define GPIO_REG_PUERT					0x0000007c
/// GPIO Open Drain Mode Enable Register (Read/Write)
#define GPIO_REG_ODMER					0x00000080
/// GPIO Open Drain Mode Register (Set)
#define GPIO_REG_ODMERS					0x00000084
/// GPIO Open Drain Mode Enable Register (Clear)
#define GPIO_REG_ODMERC					0x00000088
/// GPIO Open Drain Mode Enable Register (Toggle)
#define GPIO_REG_ODMERT					0x0000008c
/// GPIO Interrupt Enable Register (Read/Write)
#define GPIO_REG_IER					0x00000090
/// GPIO Interrupt Enable Register (Set)
#define GPIO_REG_IERS					0x00000094
/// GPIO Interrupt Enable Register (Clear)
#define GPIO_REG_IERC					0x00000098
/// GPIO Interrupt Enable Register (Toggle)
#define GPIO_REG_IERT					0x0000009c
/// GPIO Interrupt Mode Register 0 (Read/Write)
#define GPIO_REG_IMR0					0x000000a0
/// GPIO Interrupt Mode Register 0 (Set)
#define GPIO_REG_IMR0S					0x000000a4
/// GPIO Interrupt Mode Register 0 (Clear)
#define GPIO_REG_IMR0C					0x000000a8
/// GPIO Interrupt Mode Register 0 (Toggle)
#define GPIO_REG_IMR0T					0x000000ac
/// GPIO Interrupt Mode Register 1 (Read/Write)
#define GPIO_REG_IMR1					0x000000b0
/// GPIO Interrupt Mode Register 1 (Set)
#define GPIO_REG_IMR1S					0x000000b4
/// GPIO Interrupt Mode Register 1 (Clear)
#define GPIO_REG_IMR1C					0x000000b8
/// GPIO Interrupt Mode Register 1 (Toggle)
#define GPIO_REG_IMR1T					0x000000bc
/// GPIO Glitch Filter Enable Register (Read/Write)
#define GPIO_REG_GFER					0x000000c0
/// GPIO Glitch Filter Enable Register (Set)
#define GPIO_REG_GFERS					0x000000c4
/// GPIO Glitch Filter Enable Register (Clear)
#define GPIO_REG_GFERC					0x000000c8
/// GPIO Glitch Filter Enable Register (Toggle)
#define GPIO_REG_GFERT					0x000000cc
/// GPIO Interrupt Flag Register (Read)
#define GPIO_REG_IFR					0x000000d0
/// GPIO Interrupt Flag Register (Clear)
#define GPIO_REG_IFRC					0x000000d8

#define GPIO_PIN_PA00					0
#define GPIO_PIN_PA01					1
#define GPIO_PIN_PA02					2
#define GPIO_PIN_PA03					3
#define GPIO_PIN_PA04					4
#define GPIO_PIN_PA05					5
#define GPIO_PIN_PA06					6
#define GPIO_PIN_PA07					7
#define GPIO_PIN_PA08					8
#define GPIO_PIN_PA09					9
#define GPIO_PIN_PA10					10
#define GPIO_PIN_PA11					11
#define GPIO_PIN_PA12					12
#define GPIO_PIN_PA13					13
#define GPIO_PIN_PA14					14
#define GPIO_PIN_PA15					15
#define GPIO_PIN_PA16					16
#define GPIO_PIN_PA17					17
#define GPIO_PIN_PA18					18
#define GPIO_PIN_PA19					19
#define GPIO_PIN_PA20					20
#define GPIO_PIN_PA21					21
#define GPIO_PIN_PA22					22
#define GPIO_PIN_PA23					23
#define GPIO_PIN_PA24					24
#define GPIO_PIN_PA25					25
#define GPIO_PIN_PA26					26
#define GPIO_PIN_PA27					27
#define GPIO_PIN_PA28					28
#define GPIO_PIN_PA29					29
#define GPIO_PIN_PA30					30
#define GPIO_PIN_PA31					31
#define GPIO_PIN_PB00					32
#define GPIO_PIN_PB01					33
#define GPIO_PIN_PB02					34
#define GPIO_PIN_PB03					35
#define GPIO_PIN_PB04					36
#define GPIO_PIN_PB05					37
#define GPIO_PIN_PB06					38
#define GPIO_PIN_PB07					39
#define GPIO_PIN_PB08					40
#define GPIO_PIN_PB09					41
#define GPIO_PIN_PB10					42
#define GPIO_PIN_PB11					43
#define GPIO_PIN_PB12					44
#define GPIO_PIN_PB13					45
#define GPIO_PIN_PB14					46
#define GPIO_PIN_PB15					47
#define GPIO_PIN_PB16					48
#define GPIO_PIN_PB17					49
#define GPIO_PIN_PB18					50
#define GPIO_PIN_PB19					51
#define GPIO_PIN_PB20					52
#define GPIO_PIN_PB21					53
#define GPIO_PIN_PB22					54
#define GPIO_PIN_PB23					55
#define GPIO_PIN_PB24					56
#define GPIO_PIN_PB25					57
#define GPIO_PIN_PB26					58
#define GPIO_PIN_PB27					59
#define GPIO_PIN_PB28					60
#define GPIO_PIN_PB29					61
#define GPIO_PIN_PB30					62
#define GPIO_PIN_PB31					63
#define GPIO_PIN_PC00					64
#define GPIO_PIN_PC01					65
#define GPIO_PIN_PC02					66
#define GPIO_PIN_PC03					67
#define GPIO_PIN_PC04					68
#define GPIO_PIN_PC05					69
#define GPIO_PIN_PX00					100
#define GPIO_PIN_PX01					99
#define GPIO_PIN_PX02					98
#define GPIO_PIN_PX03					97
#define GPIO_PIN_PX04					96
#define GPIO_PIN_PX05					95
#define GPIO_PIN_PX06					94
#define GPIO_PIN_PX07					93
#define GPIO_PIN_PX08					92
#define GPIO_PIN_PX09					91
#define GPIO_PIN_PX10					90
#define GPIO_PIN_PX11					109
#define GPIO_PIN_PX12					108
#define GPIO_PIN_PX13					107
#define GPIO_PIN_PX14					106
#define GPIO_PIN_PX15					89
#define GPIO_PIN_PX16					88
#define GPIO_PIN_PX17					87
#define GPIO_PIN_PX18					86
#define GPIO_PIN_PX19					85
#define GPIO_PIN_PX20					84
#define GPIO_PIN_PX21					83
#define GPIO_PIN_PX22					82
#define GPIO_PIN_PX23					81
#define GPIO_PIN_PX24					80
#define GPIO_PIN_PX25					79
#define GPIO_PIN_PX26					78
#define GPIO_PIN_PX27					77
#define GPIO_PIN_PX28					76
#define GPIO_PIN_PX29					75
#define GPIO_PIN_PX30					74
#define GPIO_PIN_PX31					73
#define GPIO_PIN_PX32					72
#define GPIO_PIN_PX33					71
#define GPIO_PIN_PX34					70
#define GPIO_PIN_PX35					105
#define GPIO_PIN_PX36					104
#define GPIO_PIN_PX37					103
#define GPIO_PIN_PX38					102
#define GPIO_PIN_PX39					101

#define GPIO_PIN_ADC_AD0				GPIO_PIN_PA21
#define GPIO_PIN_ADC_AD0_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD1				GPIO_PIN_PA22
#define GPIO_PIN_ADC_AD1_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD2				GPIO_PIN_PA23
#define GPIO_PIN_ADC_AD2_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD3				GPIO_PIN_PA24
#define GPIO_PIN_ADC_AD3_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD4				GPIO_PIN_PA25
#define GPIO_PIN_ADC_AD4_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD5				GPIO_PIN_PA26
#define GPIO_PIN_ADC_AD5_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD6				GPIO_PIN_PA27
#define GPIO_PIN_ADC_AD6_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_AD7				GPIO_PIN_PA28
#define GPIO_PIN_ADC_AD7_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_ADC_TRIGGER				GPIO_PIN_PB18
#define GPIO_PIN_ADC_TRIGGER_FUNCTION			GPIO_FUNCTION_B

#define GPIO_PIN_GCLK_GCLK0				GPIO_PIN_PB19
#define GPIO_PIN_GCLK_GCLK0_FUNCTION			GPIO_FUNCTION_B
#define GPIO_PIN_GCLK_GCLK1				GPIO_PIN_PB20
#define GPIO_PIN_GCLK_GCLK1_FUNCTION			GPIO_FUNCTION_B
#define GPIO_PIN_GCLK_GCLK2				GPIO_PIN_PB21
#define GPIO_PIN_GCLK_GCLK2_FUNCTION			GPIO_FUNCTION_B
#define GPIO_PIN_GCLK_GCLK3				GPIO_PIN_PB22
#define GPIO_PIN_GCLK_GCLK3_FUNCTION			GPIO_FUNCTION_B

#define GPIO_PIN_PWM_PWM0				GPIO_PIN_PB19
#define GPIO_PIN_PWM_PWM0_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_PWM_PWM1				GPIO_PIN_PB20
#define GPIO_PIN_PWM_PWM1_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_PWM_PWM2				GPIO_PIN_PB21
#define GPIO_PIN_PWM_PWM2_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_PWM_PWM3				GPIO_PIN_PB22
#define GPIO_PIN_PWM_PWM3_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_PWM_PWM4				GPIO_PIN_PB27
#define GPIO_PIN_PWM_PWM4_FUNCTION			GPIO_FUNCTION_B
#define GPIO_PIN_PWM_PWM5				GPIO_PIN_PB28
#define GPIO_PIN_PWM_PWM5_FUNCTION			GPIO_FUNCTION_B
#define GPIO_PIN_PWM_PWM6				GPIO_PIN_PB18
#define GPIO_PIN_PWM_PWM6_FUNCTION			GPIO_FUNCTION_C

#define GPIO_PIN_USART0_RXD				GPIO_PIN_PA00
#define GPIO_PIN_USART0_RXD_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_USART0_RXD_ALTERNATE			GPIO_PIN_PX00
#define GPIO_PIN_USART0_RXD_ALTERNATE_FUNCTION		GPIO_FUNCTION_B
#define GPIO_PIN_USART0_TXD				GPIO_PIN_PA01
#define GPIO_PIN_USART0_TXD_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_USART0_TXD_ALTERNATE			GPIO_PIN_PX01
#define GPIO_PIN_USART0_TXD_ALTERNATE_FUNCTION		GPIO_FUNCTION_B
#define GPIO_PIN_USART0_CLK				GPIO_PIN_PA02
#define GPIO_PIN_USART0_CLK_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_USART0_RTS				GPIO_PIN_PA03
#define GPIO_PIN_USART0_RTS_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_USART0_RTS_ALTERNATE			GPIO_PIN_PX03
#define GPIO_PIN_USART0_RTS_ALTERNATE_FUNCTION		GPIO_FUNCTION_B
#define GPIO_PIN_USART0_CTS				GPIO_PIN_PA04
#define GPIO_PIN_USART0_CTS_FUNCTION			GPIO_FUNCTION_A
#define GPIO_PIN_USART0_CTS_ALTERNATE			GPIO_PIN_PX02
#define GPIO_PIN_USART0_CTS_ALTERNATE_FUNCTION		GPIO_FUNCTION_B

/// Number of GPIO functions
#define GPIO_NUM_FUNCTIONS				3

#define GPIO_MAP_FOREACH(map, i, port, mask) \
	for (i = 0; port = (map)->ports[i], mask = (map)->masks[i], i < GPIO_NUM_PORTS; ++i) \
		if(port)

#define GPIO_FUNCTIONS_MAP_FOREACH(map, i, port, function, mask) \
	for (i = 0; port = (map)->ports[i], i < GPIO_NUM_PORTS; ++i) \
		if(port) \
			for(function = 0; mask = (map)->masks[i][function], function < GPIO_NUM_FUNCTIONS; ++function) \
				if(mask)

typedef enum {
	GPIO_FUNCTION_A					= 0,
	GPIO_FUNCTION_B					= 1,
	GPIO_FUNCTION_C					= 2,
} gpio_function_t;

typedef enum {
	GPIO_INPUT_OPTION_NONE				= 0,
	GPIO_INPUT_OPTION_PULLUP			= (1 << 0),
	GPIO_INPUT_OPTION_OPEN_DRAIN_MODE		= (1 << 1),
	GPIO_INPUT_OPTION_GLITCH_FILTER			= (1 << 2),
	GPIO_INPUT_OPTION_INTERRUPT			= (1 << 3),
} gpio_input_options_t;

typedef enum {
	GPIO_INTERRUPT_MODE_NONE			= 0,
	GPIO_INTERRUPT_MODE_ANY_EDGE			= 0,
	GPIO_INTERRUPT_MODE_RISING_EDGE			= 1,
	GPIO_INTERRUPT_MODE_FALLING_EDGE		= 2,
} gpio_interrupt_mode_t;

typedef struct gpio_map gpio_map_t;
struct gpio_map {
	void volatile *ports[GPIO_NUM_PORTS];
	uint32_t masks[GPIO_NUM_PORTS];
};

typedef struct gpio_functions_map gpio_functions_map_t;
struct gpio_functions_map {
	void volatile *ports[GPIO_NUM_PORTS];
	uint32_t masks[GPIO_NUM_PORTS][GPIO_NUM_FUNCTIONS];
};

typedef struct gpio_pin_function gpio_pin_function_t;
struct gpio_pin_function {
	uint8_t pin_nb;
	gpio_function_t function;
};

// Port API
extern uint32_t gpio_port_read_register(void volatile *port, uint32_t const offset);
extern void gpio_port_write_register(void volatile *port, uint32_t const offset, uint32_t const value);

extern void gpio_port_reset(void volatile *port, uint32_t const mask);
extern void gpio_port_enable(void volatile *port, uint32_t const mask);
extern void gpio_port_disable(void volatile *port, uint32_t const mask);
extern void gpio_port_select_input(void volatile *port, uint32_t const mask);
extern void gpio_port_select_output(void volatile *port, uint32_t const mask);
extern void gpio_port_enable_pullup(void volatile *port, uint32_t const mask);
extern void gpio_port_disable_pullup(void volatile *port, uint32_t const mask);
extern void gpio_port_enable_open_drain_mode(void volatile *port, uint32_t const mask);
extern void gpio_port_disable_open_drain_mode(void volatile *port, uint32_t const mask);
extern void gpio_port_enable_glitch_filter(void volatile *port, uint32_t const mask);
extern void gpio_port_disable_glitch_filter(void volatile *port, uint32_t const mask);
extern void gpio_port_enable_interrupts(void volatile *port, uint32_t const mask, gpio_interrupt_mode_t const mode);
extern void gpio_port_disable_interrupts(void volatile *port, uint32_t const mask);

extern void gpio_port_enable_input(void volatile *port, uint32_t const mask, gpio_input_options_t const options, gpio_interrupt_mode_t const mode);
extern void gpio_port_enable_output(void volatile *port, uint32_t const mask);

extern uint32_t gpio_port_read_pin_values(void volatile *port);
extern uint32_t gpio_port_read_output_value(void volatile *port);
extern void gpio_port_write_output_value(void volatile *port, uint32_t const value);
extern void gpio_port_set_output_value(void volatile *port, uint32_t const mask);
extern void gpio_port_clear_output_value(void volatile *port, uint32_t const mask);
extern void gpio_port_toggle_output_value(void volatile *port, uint32_t const mask);

extern uint32_t gpio_port_read_interrupt_flags(void volatile *port);
extern void gpio_port_deassert_interrupts(void volatile *port, uint32_t const mask);

extern void gpio_port_select_function(void volatile *port, uint32_t const mask, gpio_function_t const function);
extern void gpio_port_enable_function(void volatile *port, uint32_t const mask, gpio_function_t const function);

// Map API
extern gpio_map_t *gpio_map_init(gpio_map_t *self);
extern gpio_functions_map_t *gpio_functions_map_init(gpio_functions_map_t *self);

extern void gpio_map_add_pin(gpio_map_t * const map, uint8_t const pin_nb);
extern void gpio_map_add_pins(gpio_map_t * const map, uint32_t const n, uint8_t const *pin_nbs);
extern void gpio_functions_map_add_pin_function(gpio_functions_map_t * const map, gpio_pin_function_t const pin_function);
extern void gpio_functions_map_add_pin_functions(gpio_functions_map_t * const map, uint32_t const n, gpio_pin_function_t const *pin_functions);

extern void gpio_map_reset(gpio_map_t const * const map);
extern void gpio_map_enable(gpio_map_t const * const map);
extern void gpio_map_disable(gpio_map_t const * const map);
extern void gpio_map_select_input(gpio_map_t const * const map);
extern void gpio_map_select_output(gpio_map_t const * const map);
extern void gpio_map_enable_pullup(gpio_map_t const * const map);
extern void gpio_map_disable_pullup(gpio_map_t const * const map);
extern void gpio_map_enable_open_drain_mode(gpio_map_t const * const map);
extern void gpio_map_disable_open_drain_mode(gpio_map_t const * const map);
extern void gpio_map_enable_glitch_filter(gpio_map_t const * const map);
extern void gpio_map_disable_glitch_filter(gpio_map_t const * const map);
extern void gpio_map_enable_interrupts(gpio_map_t const * const map, gpio_interrupt_mode_t const mode);
extern void gpio_map_disable_interrupts(gpio_map_t const * const map);

extern void gpio_map_enable_input(gpio_map_t const * const map, gpio_input_options_t const options, gpio_interrupt_mode_t const mode);
extern void gpio_map_enable_output(gpio_map_t const * const map);

extern void gpio_map_set_output_value(gpio_map_t const * const map);
extern void gpio_map_clear_output_value(gpio_map_t const * const map);
extern void gpio_map_toggle_output_value(gpio_map_t const * const map);

extern void gpio_map_deassert_interrupts(gpio_map_t const * const map);

extern void gpio_functions_map_select_functions(gpio_functions_map_t const * const map);
extern void gpio_functions_map_enable_functions(gpio_functions_map_t const * const map);

#endif /* GPIO_H_ */

