/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef INTC_H_
#define INTC_H_

#include <at32uc3/inttypes.h>

/// INTC address
#define INTC_ADDRESS					0xffff0800

/// INTC Interrupt Priority Register (Read/Write)
#define INTC_REG_IPR					0x00000000
/// INTC Interrupt Priority Register Autovector Address offset
#define INTC_REG_IPR_AUTOVECTOR_OFFSET			0
/// INTC Interrupt Priority Register Autovector Address size
#define INTC_REG_IPR_AUTOVECTOR_SIZE			14
/// INTC Interrupt Priority Register Autovector Address mask
#define INTC_REG_IPR_AUTOVECTOR_MASK			(((1 << INTC_REG_IPR_AUTOVECTOR_SIZE) - 1) << INTC_REG_IPR_AUTOVECTOR_OFFSET)
/// INTC Interrupt Priority Register Interrupt Level offset
#define INTC_REG_IPR_INTLEVEL_OFFSET			30
/// INTC Interrupt Priority Register Interrupt Level size
#define INTC_REG_IPR_INTLEVEL_SIZE			2
/// INTC Interrupt Priority Register Interrupt Level mask
#define INTC_REG_IPR_INTLEVEL_MASK			(((1 << INTC_REG_IPR_INTLEVEL_SIZE) - 1) << INTC_REG_IPR_INTLEVEL_OFFSET)
/// INTC Interrupt Request Register (Read)
#define INTC_REG_IRR					0x00000100
/// INTC Interrupt Cause Register (Read)
#define INTC_REG_ICR					0x00000200

typedef enum {
	INTC_GROUP_SYSBLOCK_COMPARE			= 0,
	INTC_GROUP_EIC					= 1,
	INTC_GROUP_RTC					= 1,
	INTC_GROUP_PM					= 1,
	INTC_GROUP_FREQM				= 1,
	INTC_GROUP_GPIO					= 2,
	INTC_GROUP_PDCA					= 3,
	INTC_GROUP_FLASHC				= 4,
	INTC_GROUP_USART0				= 5,
	INTC_GROUP_USART1				= 6,
	INTC_GROUP_USART2				= 7,
	INTC_GROUP_USART3				= 8,
	INTC_GROUP_SPI0					= 9,
	INTC_GROUP_SPI1					= 10,
	INTC_GROUP_TWI					= 11,
	INTC_GROUP_PWM					= 12,
	INTC_GROUP_SSC					= 13,
	INTC_GROUP_TC					= 14,
	INTC_GROUP_ADC					= 15,
	INTC_GROUP_MACB					= 16,
	INTC_GROUP_USBB					= 17,
	INTC_GROUP_SDRAMC				= 18,
	INTC_GROUP_DAC					= 19,
} intc_group_t;

typedef enum {
	INTC_INTLEVEL_INT0				= 0,
	INTC_INTLEVEL_INT1				= 1,
	INTC_INTLEVEL_INT2				= 2,
	INTC_INTLEVEL_INT3				= 3,
} intc_intlevel_t;

extern uint32_t intc_read_register(uint32_t const offset, intc_group_t const group);
extern void intc_write_register(uint32_t const offset, intc_group_t const group, uint32_t const value);

extern void intc_connect(intc_group_t const group, intc_intlevel_t const intlevel, uint16_t const autovector);

#endif /* INTC_H_ */

