/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef INTERRUPTS_H_
#define INTERRUPTS_H_

#include <at32uc3/gccmacros.h>
#include <at32uc3/intc.h>
#include <at32uc3/inttypes.h>
#include <at32uc3/sysreg.h>

typedef void (*interrupts_handler_t)(void);

typedef enum {
	INTERRUPTS_MASK_GLOBAL				= SYSREG_SR_GM_MASK,
	INTERRUPTS_MASK_IRQ0				= SYSREG_SR_I0M_MASK,
	INTERRUPTS_MASK_IRQ1				= SYSREG_SR_I1M_MASK,
	INTERRUPTS_MASK_IRQ2				= SYSREG_SR_I2M_MASK,
	INTERRUPTS_MASK_IRQ3				= SYSREG_SR_I3M_MASK,
	INTERRUPTS_MASK_ALL_IRQS			= INTERRUPTS_MASK_IRQ0 | INTERRUPTS_MASK_IRQ1 | INTERRUPTS_MASK_IRQ2 | INTERRUPTS_MASK_IRQ3,
	INTERRUPTS_MASK_ALL				= INTERRUPTS_MASK_GLOBAL | INTERRUPTS_MASK_ALL_IRQS,
} interrupts_mask_t;

extern void interrupts_enable(interrupts_mask_t const mask);
extern interrupts_mask_t interrupts_enabled(interrupts_mask_t const mask);
extern void interrupts_disable(interrupts_mask_t const mask);
extern interrupts_mask_t interrupts_read_mask(void);

extern void interrupts_connect_group(intc_group_t const group, intc_intlevel_t const intlevel);

extern void interrupts_set_irq0_handler(interrupts_handler_t const handler);
extern void interrupts_set_irq1_handler(interrupts_handler_t const handler);
extern void interrupts_set_irq2_handler(interrupts_handler_t const handler);
extern void interrupts_set_irq3_handler(interrupts_handler_t const handler);

#endif /* INTERRUPTS_H_ */

