/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef MAC_H_
#define MAC_H_

#include <at32uc3/inttypes.h>

/// MAC Address
#define MAC_ADDRESS					0xfffe1800

/// Ethernet MAC Network Control Register (Read/Write)
#define MAC_REG_NCR					0x00000000
/// Ethernet MAC Network Control Register LoopBack offset
#define MAC_REG_NCR_LB_OFFSET				0
/// Ethernet MAC Network Control Register LoopBack size
#define MAC_REG_NCR_LB_SIZE				1
/// Ethernet MAC Network Control Register LoopBack mask
#define MAC_REG_NCR_LB_MASK				(((1 << MAC_REG_NCR_LB_SIZE) - 1) << MAC_REG_NCR_LB_OFFSET)
/// Ethernet MAC Network Control Register LoopBack Local offset
#define MAC_REG_NCR_LLB_OFFSET				1
/// Ethernet MAC Network Control Register LoopBack Local size
#define MAC_REG_NCR_LLB_SIZE				1
/// Ethernet MAC Network Control Register LoopBack Local mask
#define MAC_REG_NCR_LLB_MASK				(((1 << MAC_REG_NCR_LLB_SIZE) - 1) << MAC_REG_NCR_LLB_OFFSET)
/// Ethernet MAC Network Control Register Receive enable offset
#define MAC_REG_NCR_RE_OFFSET				2
/// Ethernet MAC Network Control Register Receive enable size
#define MAC_REG_NCR_RE_SIZE				1
/// Ethernet MAC Network Control Register Receive enable mask
#define MAC_REG_NCR_RE_MASK				(((1 << MAC_REG_NCR_RE_SIZE) - 1) << MAC_REG_NCR_RE_OFFSET)
/// Ethernet MAC Network Control Register Transmit enable offset
#define MAC_REG_NCR_TE_OFFSET				3
/// Ethernet MAC Network Control Register Transmit enable size
#define MAC_REG_NCR_TE_SIZE				1
/// Ethernet MAC Network Control Register Transmit enable mask
#define MAC_REG_NCR_TE_MASK				(((1 << MAC_REG_NCR_TE_SIZE) - 1) << MAC_REG_NCR_TE_OFFSET)
/// Ethernet MAC Network Control Register Management port enable offset
#define MAC_REG_NCR_MPE_OFFSET				4
/// Ethernet MAC Network Control Register Management port enable size
#define MAC_REG_NCR_MPE_SIZE				1
/// Ethernet MAC Network Control Register Management port enable mask
#define MAC_REG_NCR_MPE_MASK				(((1 << MAC_REG_NCR_MPE_SIZE) - 1) << MAC_REG_NCR_MPE_OFFSET)
/// Ethernet MAC Network Control Register Clear statistics registers offset
#define MAC_REG_NCR_CLRSTAT_OFFSET			5
/// Ethernet MAC Network Control Register Clear statistics registers size
#define MAC_REG_NCR_CLRSTAT_SIZE			1
/// Ethernet MAC Network Control Register Clear statistics registers mask
#define MAC_REG_NCR_CLRSTAT_MASK			(((1 << MAC_REG_NCR_CLRSTAT_SIZE) - 1) << MAC_REG_NCR_CLRSTAT_OFFSET)
/// Ethernet MAC Network Control Register Increment statistics registers offset
#define MAC_REG_NCR_INCSTAT_OFFSET			6
/// Ethernet MAC Network Control Register Increment statistics registers size
#define MAC_REG_NCR_INCSTAT_SIZE			1
/// Ethernet MAC Network Control Register Increment statistics registers mask
#define MAC_REG_NCR_INCSTAT_MASK			(((1 << MAC_REG_NCR_INCSTAT_SIZE) - 1) << MAC_REG_NCR_INCSTAT_OFFSET)
/// Ethernet MAC Network Control Register Write enable for statistics registers offset
#define MAC_REG_NCR_WESTAT_OFFSET			7
/// Ethernet MAC Network Control Register Write enable for statistics registers size
#define MAC_REG_NCR_WESTAT_SIZE				1
/// Ethernet MAC Network Control Register Write enable for statistics registers mask
#define MAC_REG_NCR_WESTAT_MASK				(((1 << MAC_REG_NCR_WESTAT_SIZE) - 1) << MAC_REG_NCR_WESTAT_OFFSET)
/// Ethernet MAC Network Control Register Back pressure offset
#define MAC_REG_NCR_BP_OFFSET				8
/// Ethernet MAC Network Control Register Back pressure size
#define MAC_REG_NCR_BP_SIZE				1
/// Ethernet MAC Network Control Register Back pressure mask
#define MAC_REG_NCR_BP_MASK				(((1 << MAC_REG_NCR_BP_SIZE) - 1) << MAC_REG_NCR_BP_OFFSET)
/// Ethernet MAC Network Control Register Start transmission offset
#define MAC_REG_NCR_TSTART_OFFSET			9
/// Ethernet MAC Network Control Register Start transmission size
#define MAC_REG_NCR_TSTART_SIZE				1
/// Ethernet MAC Network Control Register Start transmission mask
#define MAC_REG_NCR_TSTART_MASK				(((1 << MAC_REG_NCR_TSTART_SIZE) - 1) << MAC_REG_NCR_TSTART_OFFSET)
/// Ethernet MAC Network Control Register Transmit halt offset
#define MAC_REG_NCR_THALT_OFFSET			10
/// Ethernet MAC Network Control Register Transmit halt size
#define MAC_REG_NCR_THALT_SIZE				1
/// Ethernet MAC Network Control Register Transmit halt mask
#define MAC_REG_NCR_THALT_MASK				(((1 << MAC_REG_NCR_THALT_SIZE) - 1) << MAC_REG_NCR_THALT_OFFSET)
/// Ethernet MAC Network Control Register Transmit pause frame offset
#define MAC_REG_NCR_TPF_OFFSET				11
/// Ethernet MAC Network Control Register Transmit pause frame size
#define MAC_REG_NCR_TPF_SIZE				1
/// Ethernet MAC Network Control Register Transmit pause frame mask
#define MAC_REG_NCR_TPF_MASK				(((1 << MAC_REG_NCR_TPF_SIZE) - 1) << MAC_REG_NCR_TPF_OFFSET)
/// Ethernet MAC Network Control Register Transmit zero quantum pause frame offset
#define MAC_REG_NCR_TZQ_OFFSET				12
/// Ethernet MAC Network Control Register Transmit zero quantum pause frame size
#define MAC_REG_NCR_TZQ_SIZE				1
/// Ethernet MAC Network Control Register Transmit zero quantum pause frame mask
#define MAC_REG_NCR_TZQ_MASK				(((1 << MAC_REG_NCR_TZQ_SIZE) - 1) << MAC_REG_NCR_TZQ_OFFSET)
/// Ethernet MAC Network Configuration Register (Read/Write)
#define MAC_REG_NCFG					0x00000004
/// Ethernet MAC Network Configuration Register Speed offset
#define MAC_REG_NCFG_SPD_OFFSET				0
/// Ethernet MAC Network Configuration Register Speed size
#define MAC_REG_NCFG_SPD_SIZE				1
/// Ethernet MAC Network Configuration Register Speed mask
#define MAC_REG_NCFG_SPD_MASK				(((1 << MAC_REG_NCFG_SPD_SIZE) - 1) << MAC_REG_NCFG_SPD_OFFSET)
/// Ethernet MAC Network Configuration Register Full Duplex offset
#define MAC_REG_NCFG_FD_OFFSET				1
/// Ethernet MAC Network Configuration Register Full Duplex size
#define MAC_REG_NCFG_FD_SIZE				1
/// Ethernet MAC Network Configuration Register Full Duplex mask
#define MAC_REG_NCFG_FD_MASK				(((1 << MAC_REG_NCFG_FD_SIZE) - 1) << MAC_REG_NCFG_FD_OFFSET)
/// Ethernet MAC Network Configuration Register Bit rate offset
#define MAC_REG_NCFG_BR_OFFSET				2
/// Ethernet MAC Network Configuration Register Bit rate size
#define MAC_REG_NCFG_BR_SIZE				1
/// Ethernet MAC Network Configuration Register Bit rate mask
#define MAC_REG_NCFG_BR_MASK				(((1 << MAC_REG_NCFG_BR_SIZE) - 1) << MAC_REG_NCFG_BR_OFFSET)
/// Ethernet MAC Network Configuration Register Copy All Frames offset
#define MAC_REG_NCFG_CAF_OFFSET				3
/// Ethernet MAC Network Configuration Register Copy All Frames size
#define MAC_REG_NCFG_CAF_SIZE				1
/// Ethernet MAC Network Configuration Register Copy All Frames mask
#define MAC_REG_NCFG_CAF_MASK				(((1 << MAC_REG_NCFG_CAF_SIZE) - 1) << MAC_REG_NCFG_CAF_OFFSET)
/// Ethernet MAC Network Configuration Register Jumbo Frames offset
#define MAC_REG_NCFG_JFRAME_OFFSET			4
/// Ethernet MAC Network Configuration Register Jumbo Frames size
#define MAC_REG_NCFG_JFRAME_SIZE			1
/// Ethernet MAC Network Configuration Register Jumbo Frames mask
#define MAC_REG_NCFG_JFRAME_MASK			(((1 << MAC_REG_NCFG_JFRAME_SIZE) - 1) << MAC_REG_NCFG_JFRAME_OFFSET)
/// Ethernet MAC Network Configuration Register No Broadcast offset
#define MAC_REG_NCFG_NBC_OFFSET				5
/// Ethernet MAC Network Configuration Register No Broadcast size
#define MAC_REG_NCFG_NBC_SIZE				1
/// Ethernet MAC Network Configuration Register No Broadcast mask
#define MAC_REG_NCFG_NBC_MASK				(((1 << MAC_REG_NCFG_NBC_SIZE) - 1) << MAC_REG_NCFG_NBC_OFFSET)
/// Ethernet MAC Network Configuration Register Multicast Hash Enable offset
#define MAC_REG_NCFG_MTI_OFFSET				6
/// Ethernet MAC Network Configuration Register Multicast Hash Enable size
#define MAC_REG_NCFG_MTI_SIZE				1
/// Ethernet MAC Network Configuration Register Multicast Hash Enable mask
#define MAC_REG_NCFG_MTI_MASK				(((1 << MAC_REG_NCFG_MTI_SIZE) - 1) << MAC_REG_NCFG_MTI_OFFSET)
/// Ethernet MAC Network Configuration Register Unicast Hash Enable offset
#define MAC_REG_NCFG_UNI_OFFSET				7
/// Ethernet MAC Network Configuration Register Unicast Hash Enable size
#define MAC_REG_NCFG_UNI_SIZE				1
/// Ethernet MAC Network Configuration Register Unicast Hash Enable mask
#define MAC_REG_NCFG_UNI_MASK				(((1 << MAC_REG_NCFG_UNI_SIZE) - 1) << MAC_REG_NCFG_UNI_OFFSET)
/// Ethernet MAC Network Configuration Register Receive 1536 bytes frames offset
#define MAC_REG_NCFG_BIG_OFFSET				8
/// Ethernet MAC Network Configuration Register Receive 1536 bytes frames size
#define MAC_REG_NCFG_BIG_SIZE				1
/// Ethernet MAC Network Configuration Register Receive 1536 bytes frames mask
#define MAC_REG_NCFG_BIG_MASK				(((1 << MAC_REG_NCFG_BIG_SIZE) - 1) << MAC_REG_NCFG_BIG_OFFSET)
/// Ethernet MAC Network Configuration Register External address match enable offset
#define MAC_REG_NCFG_EAE_OFFSET				9
/// Ethernet MAC Network Configuration Register External address match enable size
#define MAC_REG_NCFG_EAE_SIZE				1
/// Ethernet MAC Network Configuration Register External address match enable mask
#define MAC_REG_NCFG_EAE_MASK				(((1 << MAC_REG_NCFG_EAE_SIZE) - 1) << MAC_REG_NCFG_EAE_OFFSET)
/// Ethernet MAC Network Configuration Register MDC clock divider offset
#define MAC_REG_NCFG_CLK_OFFSET				10
/// Ethernet MAC Network Configuration Register MDC clock divider size
#define MAC_REG_NCFG_CLK_SIZE				2
/// Ethernet MAC Network Configuration Register MDC clock divider mask
#define MAC_REG_NCFG_CLK_MASK				(((1 << MAC_REG_NCFG_CLK_SIZE) - 1) << MAC_REG_NCFG_CLK_OFFSET)
/// Ethernet MAC Network Configuration Register Retry test offset
#define MAC_REG_NCFG_RTY_OFFSET				12
/// Ethernet MAC Network Configuration Register Retry test size
#define MAC_REG_NCFG_RTY_SIZE				1
/// Ethernet MAC Network Configuration Register Retry test mask
#define MAC_REG_NCFG_RTY_MASK				(((1 << MAC_REG_NCFG_RTY_SIZE) - 1) << MAC_REG_NCFG_RTY_OFFSET)
/// Ethernet MAC Network Configuration Register Pause Enable offset
#define MAC_REG_NCFG_PAE_OFFSET				13
/// Ethernet MAC Network Configuration Register Pause Enable size
#define MAC_REG_NCFG_PAE_SIZE				1
/// Ethernet MAC Network Configuration Register Pause Enable mask
#define MAC_REG_NCFG_PAE_MASK				(((1 << MAC_REG_NCFG_PAE_SIZE) - 1) << MAC_REG_NCFG_PAE_OFFSET)
/// Ethernet MAC Network Configuration Register Receive Buffer Offset offset
#define MAC_REG_NCFG_RBOF_OFFSET			14
/// Ethernet MAC Network Configuration Register Receive Buffer Offset size
#define MAC_REG_NCFG_RBOF_SIZE				2
/// Ethernet MAC Network Configuration Register Receive Buffer Offset mask
#define MAC_REG_NCFG_RBOF_MASK				(((1 << MAC_REG_NCFG_RBOF_SIZE) - 1) << MAC_REG_NCFG_RBOF_OFFSET)
/// Ethernet MAC Network Configuration Register Receive Length field Checking Enable offset
#define MAC_REG_NCFG_RLCE_OFFSET			16
/// Ethernet MAC Network Configuration Register Receive Length field Checking Enable size
#define MAC_REG_NCFG_RLCE_SIZE				1
/// Ethernet MAC Network Configuration Register Receive Length field Checking Enable mask
#define MAC_REG_NCFG_RLCE_MASK				(((1 << MAC_REG_NCFG_RLCE_SIZE) - 1) << MAC_REG_NCFG_RLCE_OFFSET)
/// Ethernet MAC Network Configuration Register Discard Receive FCS offset
#define MAC_REG_NCFG_DRFCS_OFFSET			17
/// Ethernet MAC Network Configuration Register Discard Receive FCS size
#define MAC_REG_NCFG_DRFCS_SIZE				1
/// Ethernet MAC Network Configuration Register Discard Receive FCS mask
#define MAC_REG_NCFG_DRFCS_MASK				(((1 << MAC_REG_NCFG_DRFCS_SIZE) - 1) << MAC_REG_NCFG_DRFCS_OFFSET)
/// Ethernet MAC Network Configuration Register Enable Frame Receive in Half-Duplex Mode offset
#define MAC_REG_NCFG_EFRHD_OFFSET			18
/// Ethernet MAC Network Configuration Register Enable Frame Receive in Half-Duplex Mode size
#define MAC_REG_NCFG_EFRHD_SIZE				1
/// Ethernet MAC Network Configuration Register Enable Frame Receive in Half-Duplex Mode mask
#define MAC_REG_NCFG_EFRHD_MASK				(((1 << MAC_REG_NCFG_EFRHD_SIZE) - 1) << MAC_REG_NCFG_EFRHD_OFFSET)
/// Ethernet MAC Network Configuration Register Ignore RX FCS offset
#define MAC_REG_NCFG_IRXFCS_OFFSET			19
/// Ethernet MAC Network Configuration Register Ignore RX FCS size
#define MAC_REG_NCFG_IRXFCS_SIZE			1
/// Ethernet MAC Network Configuration Register Ignore RX FCS mask
#define MAC_REG_NCFG_IRXFCS_MASK			(((1 << MAC_REG_NCFG_IRXFCS_SIZE) - 1) << MAC_REG_NCFG_IRXFCS_OFFSET)
/// Ethernet MAC Network Status Register (Read)
#define MAC_REG_NSR					0x00000008
/// Ethernet MAC Network Status Register MDIO In pin status offset
#define MAC_REG_NSR_MDIO_OFFSET				1
/// Ethernet MAC Network Status Register MDIO In pin status size
#define MAC_REG_NSR_MDIO_SIZE				1
/// Ethernet MAC Network Status Register MDIO In pin status mask
#define MAC_REG_NSR_MDIO_MASK				(((1 << MAC_REG_NSR_MDIO_SIZE) - 1) << MAC_REG_NSR_MDIO_OFFSET)
/// Ethernet MAC Network Status Register PHY Idle offset
#define MAC_REG_NSR_IDLE_OFFSET				2
/// Ethernet MAC Network Status Register PHY Idle size
#define MAC_REG_NSR_IDLE_SIZE				1
/// Ethernet MAC Network Status Register PHY Idle mask
#define MAC_REG_NSR_IDLE_MASK				(((1 << MAC_REG_NSR_IDLE_SIZE) - 1) << MAC_REG_NSR_IDLE_OFFSET)
/// Ethernet MAC Transmit Status Register (Read/Write)
#define MAC_REG_TSR					0x00000014
/// Ethernet MAC Transmit Status Register Used Bit Read offset
#define MAC_REG_TSR_UBR_OFFSET				0
/// Ethernet MAC Transmit Status Register Used Bit Read size
#define MAC_REG_TSR_UBR_SIZE				1
/// Ethernet MAC Transmit Status Register Used Bit Read mask
#define MAC_REG_TSR_UBR_MASK				(((1 << MAC_REG_TSR_UBR_SIZE) - 1) << MAC_REG_TSR_UBR_OFFSET)
/// Ethernet MAC Transmit Status Register Collision Occurred offset
#define MAC_REG_TSR_COL_OFFSET				1
/// Ethernet MAC Transmit Status Register Collision Occurred size
#define MAC_REG_TSR_COL_SIZE				1
/// Ethernet MAC Transmit Status Register Collision Occurred mask
#define MAC_REG_TSR_COL_MASK				(((1 << MAC_REG_TSR_COL_SIZE) - 1) << MAC_REG_TSR_COL_OFFSET)
/// Ethernet MAC Transmit Status Register Retry Limit exceeded offset
#define MAC_REG_TSR_RLE_OFFSET				2
/// Ethernet MAC Transmit Status Register Retry Limit exceeded size
#define MAC_REG_TSR_RLE_SIZE				1
/// Ethernet MAC Transmit Status Register Retry Limit exceeded mask
#define MAC_REG_TSR_RLE_MASK				(((1 << MAC_REG_TSR_RLE_SIZE) - 1) << MAC_REG_TSR_RLE_OFFSET)
/// Ethernet MAC Transmit Status Register Transmit Go offset
#define MAC_REG_TSR_TGO_OFFSET				3
/// Ethernet MAC Transmit Status Register Transmit Go size
#define MAC_REG_TSR_TGO_SIZE				1
/// Ethernet MAC Transmit Status Register Transmit Go mask
#define MAC_REG_TSR_TGO_MASK				(((1 << MAC_REG_TSR_TGO_SIZE) - 1) << MAC_REG_TSR_TGO_OFFSET)
/// Ethernet MAC Transmit Status Register Buffers exhausted mid frame offset
#define MAC_REG_TSR_BEX_OFFSET				4
/// Ethernet MAC Transmit Status Register Buffers exhausted mid frame size
#define MAC_REG_TSR_BEX_SIZE				1
/// Ethernet MAC Transmit Status Register Buffers exhausted mid frame mask
#define MAC_REG_TSR_BEX_MASK				(((1 << MAC_REG_TSR_BEX_SIZE) - 1) << MAC_REG_TSR_BEX_OFFSET)
/// Ethernet MAC Transmit Status Register Transmit Complete offset
#define MAC_REG_TSR_COMP_OFFSET				5
/// Ethernet MAC Transmit Status Register Transmit Complete size
#define MAC_REG_TSR_COMP_SIZE				1
/// Ethernet MAC Transmit Status Register Transmit Complete mask
#define MAC_REG_TSR_COMP_MASK				(((1 << MAC_REG_TSR_COMP_SIZE) - 1) << MAC_REG_TSR_COMP_OFFSET)
/// Ethernet MAC Transmit Status Register Transmit Underrun offset
#define MAC_REG_TSR_UND_OFFSET				6
/// Ethernet MAC Transmit Status Register Transmit Underrun size
#define MAC_REG_TSR_UND_SIZE				1
/// Ethernet MAC Transmit Status Register Transmit Underrun mask
#define MAC_REG_TSR_UND_MASK				(((1 << MAC_REG_TSR_UND_SIZE) - 1) << MAC_REG_TSR_UND_OFFSET)
/// Ethernet MAC Receive Buffer Queue Pointer Register (Read/Write)
#define MAC_REG_RBQP					0x00000018
/// Ethernet MAC Receive Buffer Queue Pointer Register Receive buffer queue pointer address offset
#define MAC_REG_RBQP_ADDR_OFFSET			2
/// Ethernet MAC Receive Buffer Queue Pointer Register Receive buffer queue pointer address size
#define MAC_REG_RBQP_ADDR_SIZE				30
/// Ethernet MAC Receive Buffer Queue Pointer Register Receive buffer queue pointer address mask
#define MAC_REG_RBQP_ADDR_MASK				(((1 << MAC_REG_RBQP_ADDR_SIZE) - 1) << MAC_REG_RBQP_ADDR_OFFSET)
/// Ethernet MAC Transmit Buffer Queue Pointer Register (Read/Write)
#define MAC_REG_TBQP					0x0000001c
/// Ethernet MAC Transmit Buffer Queue Pointer Register Transmit buffer queue pointer address offset
#define MAC_REG_TBQP_ADDR_OFFSET			2
/// Ethernet MAC Transmit Buffer Queue Pointer Register Transmit buffer queue pointer address size
#define MAC_REG_TBQP_ADDR_SIZE				30
/// Ethernet MAC Transmit Buffer Queue Pointer Register Transmit buffer queue pointer address mask
#define MAC_REG_TBQP_ADDR_MASK				(((1 << MAC_REG_TBQP_ADDR_SIZE) - 1) << MAC_REG_TBQP_ADDR_OFFSET)
/// Ethernet MAC Receive Status Register (Read/Write)
#define MAC_REG_RSR					0x00000020
/// Ethernet MAC Receive Status Register Buffer Not Available offset
#define MAC_REG_RSR_BNA_OFFSET				0
/// Ethernet MAC Receive Status Register Buffer Not Available size
#define MAC_REG_RSR_BNA_SIZE				1
/// Ethernet MAC Receive Status Register Buffer Not Available mask
#define MAC_REG_RSR_BNA_MASK				(((1 << MAC_REG_RSR_BNA_SIZE) - 1) << MAC_REG_RSR_BNA_OFFSET)
/// Ethernet MAC Receive Status Register Frame Received offset
#define MAC_REG_RSR_REC_OFFSET				1
/// Ethernet MAC Receive Status Register Frame Received size
#define MAC_REG_RSR_REC_SIZE				1
/// Ethernet MAC Receive Status Register Frame Received mask
#define MAC_REG_RSR_REC_MASK				(((1 << MAC_REG_RSR_REC_SIZE) - 1) << MAC_REG_RSR_REC_OFFSET)
/// Ethernet MAC Receive Status Register Receive Overrun offset
#define MAC_REG_RSR_OVR_OFFSET				2
/// Ethernet MAC Receive Status Register Receive Overrun size
#define MAC_REG_RSR_OVR_SIZE				1
/// Ethernet MAC Receive Status Register Receive Overrun mask
#define MAC_REG_RSR_OVR_MASK				(((1 << MAC_REG_RSR_OVR_SIZE) - 1) << MAC_REG_RSR_OVR_OFFSET)
/// Ethernet MAC Interrupt Status Register (Read/Write)
#define MAC_REG_ISR					0x00000024
/// Ethernet MAC Interrupt Enable Register (Write)
#define MAC_REG_IER					0x00000028
/// Ethernet MAC Interrupt Disable Register (Write)
#define MAC_REG_IDR					0x0000002c
/// Ethernet MAC Interrupt Mask Register (Read)
#define MAC_REG_IMR					0x00000030
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Management Frame Done offset
#define MAC_REG_IXR_MFD_OFFSET				0
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Management Frame Done size
#define MAC_REG_IXR_MFD_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Management Frame Done mask
#define MAC_REG_IXR_MFD_MASK				(((1 << MAC_REG_IXR_MFD_SIZE) - 1) << MAC_REG_IXR_MFD_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Complete offset
#define MAC_REG_IXR_RCOMP_OFFSET			1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Complete size
#define MAC_REG_IXR_RCOMP_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Complete mask
#define MAC_REG_IXR_RCOMP_MASK				(((1 << MAC_REG_IXR_RCOMP_SIZE) - 1) << MAC_REG_IXR_RCOMP_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Used Bit Read offset
#define MAC_REG_IXR_RXUBR_OFFSET			2
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Used Bit Read size
#define MAC_REG_IXR_RXUBR_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Used Bit Read mask
#define MAC_REG_IXR_RXUBR_MASK				(((1 << MAC_REG_IXR_RXUBR_SIZE) - 1) << MAC_REG_IXR_RXUBR_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Used Bit Read offset
#define MAC_REG_IXR_TXUBR_OFFSET			3
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Used Bit Read size
#define MAC_REG_IXR_TXUBR_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Used Bit Read mask
#define MAC_REG_IXR_TXUBR_MASK				(((1 << MAC_REG_IXR_TXUBR_SIZE) - 1) << MAC_REG_IXR_TXUBR_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Ethernet Transmit Buffer Underrun offset
#define MAC_REG_IXR_TUND_OFFSET				4
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Ethernet Transmit Buffer Underrun size
#define MAC_REG_IXR_TUND_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Ethernet Transmit Buffer Underrun mask
#define MAC_REG_IXR_TUND_MASK				(((1 << MAC_REG_IXR_TUND_SIZE) - 1) << MAC_REG_IXR_TUND_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Retry Limit Exceeded offset
#define MAC_REG_IXR_RLE_OFFSET				5
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Retry Limit Exceeded size
#define MAC_REG_IXR_RLE_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Retry Limit Exceeded mask
#define MAC_REG_IXR_RLE_MASK				(((1 << MAC_REG_IXR_RLE_SIZE) - 1) << MAC_REG_IXR_RLE_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Error offset
#define MAC_REG_IXR_TXERR_OFFSET			6
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Error size
#define MAC_REG_IXR_TXERR_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Error mask
#define MAC_REG_IXR_TXERR_MASK				(((1 << MAC_REG_IXR_TXERR_SIZE) - 1) << MAC_REG_IXR_TXERR_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Complete offset
#define MAC_REG_IXR_TCOMP_OFFSET			7
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Complete size
#define MAC_REG_IXR_TCOMP_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Transmit Complete mask
#define MAC_REG_IXR_TCOMP_MASK				(((1 << MAC_REG_IXR_TCOMP_SIZE) - 1) << MAC_REG_IXR_TCOMP_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Overrun offset
#define MAC_REG_IXR_ROVR_OFFSET				10
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Overrun size
#define MAC_REG_IXR_ROVR_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Receive Overrun mask
#define MAC_REG_IXR_ROVR_MASK				(((1 << MAC_REG_IXR_ROVR_SIZE) - 1) << MAC_REG_IXR_ROVR_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Hresp not OK offset
#define MAC_REG_IXR_HRESP_OFFSET			11
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Hresp not OK size
#define MAC_REG_IXR_HRESP_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Hresp not OK mask
#define MAC_REG_IXR_HRESP_MASK				(((1 << MAC_REG_IXR_HRESP_SIZE) - 1) << MAC_REG_IXR_HRESP_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Frame Received offset
#define MAC_REG_IXR_PFR_OFFSET				12
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Frame Received size
#define MAC_REG_IXR_PFR_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Frame Received mask
#define MAC_REG_IXR_PFR_MASK				(((1 << MAC_REG_IXR_PFR_SIZE) - 1) << MAC_REG_IXR_PFR_OFFSET)
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Time Zero offset
#define MAC_REG_IXR_PTZ_OFFSET				13
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Time Zero size
#define MAC_REG_IXR_PTZ_SIZE				1
/// Ethernet MAC Interrupt Enable/Disable/Mask/Status Register Pause Time Zero mask
#define MAC_REG_IXR_PTZ_MASK				(((1 << MAC_REG_IXR_PTZ_SIZE) - 1) << MAC_REG_IXR_PTZ_OFFSET)
/// Ethernet MAC Phy Maintenance Register (Read/Write)
#define MAC_REG_MAN					0x00000034
/// Ethernet MAC Phy Maintenance Register Data offset
#define MAC_REG_MAN_DATA_OFFSET				0
/// Ethernet MAC Phy Maintenance Register Data size
#define MAC_REG_MAN_DATA_SIZE				16
/// Ethernet MAC Phy Maintenance Register Data mask
#define MAC_REG_MAN_DATA_MASK				(((1 << MAC_REG_MAN_DATA_SIZE) - 1) << MAC_REG_MAN_DATA_OFFSET)
/// Ethernet MAC Phy Maintenance Register Code offset
#define MAC_REG_MAN_CODE_OFFSET				16
/// Ethernet MAC Phy Maintenance Register Code size
#define MAC_REG_MAN_CODE_SIZE				2
/// Ethernet MAC Phy Maintenance Register Code mask
#define MAC_REG_MAN_CODE_MASK				(((1 << MAC_REG_MAN_CODE_SIZE) - 1) << MAC_REG_MAN_CODE_OFFSET)
/// Ethernet MAC Phy Maintenance Register Register Address offset
#define MAC_REG_MAN_REGA_OFFSET				18
/// Ethernet MAC Phy Maintenance Register Register Address size
#define MAC_REG_MAN_REGA_SIZE				5
/// Ethernet MAC Phy Maintenance Register Register Address mask
#define MAC_REG_MAN_REGA_MASK				(((1 << MAC_REG_MAN_REGA_SIZE) - 1) << MAC_REG_MAN_REGA_OFFSET)
/// Ethernet MAC Phy Maintenance Register PHY Address offset
#define MAC_REG_MAN_PHYA_OFFSET				23
/// Ethernet MAC Phy Maintenance Register PHY Address size
#define MAC_REG_MAN_PHYA_SIZE				5
/// Ethernet MAC Phy Maintenance Register PHY Address mask
#define MAC_REG_MAN_PHYA_MASK				(((1 << MAC_REG_MAN_PHYA_SIZE) - 1) << MAC_REG_MAN_PHYA_OFFSET)
/// Ethernet MAC Phy Maintenance Register Read/Write offset
#define MAC_REG_MAN_RW_OFFSET				28
/// Ethernet MAC Phy Maintenance Register Read/Write size
#define MAC_REG_MAN_RW_SIZE				2
/// Ethernet MAC Phy Maintenance Register Read/Write mask
#define MAC_REG_MAN_RW_MASK				(((1 << MAC_REG_MAN_RW_SIZE) - 1) << MAC_REG_MAN_RW_OFFSET)
/// Ethernet MAC Phy Maintenance Register Start of frame offset
#define MAC_REG_MAN_SOF_OFFSET				30
/// Ethernet MAC Phy Maintenance Register Start of frame size
#define MAC_REG_MAN_SOF_SIZE				2
/// Ethernet MAC Phy Maintenance Register Start of frame mask
#define MAC_REG_MAN_SOF_MASK				(((1 << MAC_REG_MAN_SOF_SIZE) - 1) << MAC_REG_MAN_SOF_OFFSET)
/// Ethernet MAC Pause Time Register (Read/Write)
#define MAC_REG_PTR					0x00000038
/// Ethernet MAC Pause Time Register Pause Time offset
#define MAC_REG_PTR_PTIME_OFFSET			0
/// Ethernet MAC Pause Time Register Pause Time size
#define MAC_REG_PTR_PTIME_SIZE				16
/// Ethernet MAC Pause Time Register Pause Time mask
#define MAC_REG_PTR_PTIME_MASK				(((1 << MAC_REG_PTR_PTIME_SIZE) - 1) << MAC_REG_PTR_PTIME_OFFSET)
/// Ethernet MAC Pause Frames Received Register (Read/Write)
#define MAC_REG_PFR					0x0000003c
/// Ethernet MAC Pause Frames Received Register Pause Frames received OK offset
#define MAC_REG_PFR_FROK_OFFSET				0
/// Ethernet MAC Pause Frames Received Register Pause Frames received OK size
#define MAC_REG_PFR_FROK_SIZE				16
/// Ethernet MAC Pause Frames Received Register Pause Frames received OK mask
#define MAC_REG_PFR_FROK_MASK				(((1 << MAC_REG_PFR_FROK_SIZE) - 1) << MAC_REG_PFR_FROK_OFFSET)
/// Ethernet MAC Frames Transmitted Ok Register (Read/Write)
#define MAC_REG_FTO					0x00000040
/// Ethernet MAC Frames Transmitted Ok Register Frames Transmitted OK offset
#define MAC_REG_FTO_FTOK_OFFSET				0
/// Ethernet MAC Frames Transmitted Ok Register Frames Transmitted OK size
#define MAC_REG_FTO_FTOK_SIZE				24
/// Ethernet MAC Frames Transmitted Ok Register Frames Transmitted OK mask
#define MAC_REG_FTO_FTOK_MASK				(((1 << MAC_REG_FTO_FTOK_SIZE) - 1) << MAC_REG_FTO_FTOK_OFFSET)
/// Ethernet MAC Single Collision Frames Register (Read/Write)
#define MAC_REG_SCF					0x00000044
/// Ethernet MAC Single Collision Frames Register Single Collision Frames offset
#define MAC_REG_SCF_SCF_OFFSET				0
/// Ethernet MAC Single Collision Frames Register Single Collision Frames size
#define MAC_REG_SCF_SCF_SIZE				16
/// Ethernet MAC Single Collision Frames Register Single Collision Frames mask
#define MAC_REG_SCF_SCF_MASK				(((1 << MAC_REG_SCF_SCF_SIZE) - 1) << MAC_REG_SCF_SCF_OFFSET)
/// Ethernet MAC Multiple Collision Frames Register (Read/Write)
#define MAC_REG_MCF					0x00000048
/// Ethernet MAC Multiple Collision Frames Register Multi Collision Frames offset
#define MAC_REG_MCF_MCF_OFFSET				0
/// Ethernet MAC Multiple Collision Frames Register Multi Collision Frames size
#define MAC_REG_MCF_MCF_SIZE				16
/// Ethernet MAC Multiple Collision Frames Register Multi Collision Frames mask
#define MAC_REG_MCF_MCF_MASK				(((1 << MAC_REG_MCF_MCF_SIZE) - 1) << MAC_REG_MCF_MCF_OFFSET)
/// Ethernet MAC Frames Received Ok Register (Read/Write)
#define MAC_REG_FRO					0x0000004c
/// Ethernet MAC Frames Received Ok Register Frames Received OK offset
#define MAC_REG_FRO_FROK_OFFSET				0
/// Ethernet MAC Frames Received Ok Register Frames Received OK size
#define MAC_REG_FRO_FROK_SIZE				24
/// Ethernet MAC Frames Received Ok Register Frames Received OK mask
#define MAC_REG_FRO_FROK_MASK				(((1 << MAC_REG_FRO_FROK_SIZE) - 1) << MAC_REG_FRO_FROK_OFFSET)
/// Ethernet MAC Frame Check Sequence Errors Register (Read/Write)
#define MAC_REG_FCSE					0x00000050
/// Ethernet MAC Frame Check Sequence Errors Register Frame Check Sequence Errors offset
#define MAC_REG_FCSE_FCSE_OFFSET			0
/// Ethernet MAC Frame Check Sequence Errors Register Frame Check Sequence Errors size
#define MAC_REG_FCSE_FCSE_SIZE				8
/// Ethernet MAC Frame Check Sequence Errors Register Frame Check Sequence Errors mask
#define MAC_REG_FCSE_FCSE_MASK				(((1 << MAC_REG_FCSE_FCSE_SIZE) - 1) << MAC_REG_FCSE_FCSE_OFFSET)
/// Ethernet MAC Alignment Errors Register (Read/Write)
#define MAC_REG_ALE					0x00000054
/// Ethernet MAC Alignment Errors Register Alignment Errors offset
#define MAC_REG_ALE_ALE_OFFSET				0
/// Ethernet MAC Alignment Errors Register Alignment Errors size
#define MAC_REG_ALE_ALE_SIZE				8
/// Ethernet MAC Alignment Errors Register Alignment Errors mask
#define MAC_REG_ALE_ALE_MASK				(((1 << MAC_REG_ALE_ALE_SIZE) - 1) << MAC_REG_ALE_ALE_OFFSET)
/// Ethernet MAC Deferred Transmission Frames Register (Read/Write)
#define MAC_REG_DTF					0x00000058
/// Ethernet MAC Deferred Transmission Frames Register Deferred Transmission Frames offset
#define MAC_REG_DTF_DTF_OFFSET				0
/// Ethernet MAC Deferred Transmission Frames Register Deferred Transmission Frames size
#define MAC_REG_DTF_DTF_SIZE				16
/// Ethernet MAC Deferred Transmission Frames Register Deferred Transmission Frames mask
#define MAC_REG_DTF_DTF_MASK				(((1 << MAC_REG_DTF_DTF_SIZE) - 1) << MAC_REG_DTF_DTF_OFFSET)
/// Ethernet MAC Late Collisions Register (Read/Write)
#define MAC_REG_LCOL					0x0000005c
/// Ethernet MAC Late Collisions Register Late Collisions offset
#define MAC_REG_LCOL_LCOL_OFFSET			0
/// Ethernet MAC Late Collisions Register Late Collisions size
#define MAC_REG_LCOL_LCOL_SIZE				8
/// Ethernet MAC Late Collisions Register Late Collisions mask
#define MAC_REG_LCOL_LCOL_MASK				(((1 << MAC_REG_LCOL_LCOL_SIZE) - 1) << MAC_REG_LCOL_LCOL_OFFSET)
/// Ethernet MAC Excessive Collisions Register (Read/Write)
#define MAC_REG_EXCOL					0x00000060
/// Ethernet MAC Excessive Collisions Register Excessive Collisions offset
#define MAC_REG_EXCOL_EXCOL_OFFSET			0
/// Ethernet MAC Excessive Collisions Register Excessive Collisions size
#define MAC_REG_EXCOL_EXCOL_SIZE			8
/// Ethernet MAC Excessive Collisions Register Excessive Collisions mask
#define MAC_REG_EXCOL_EXCOL_MASK			(((1 << MAC_REG_EXCOL_EXCOL_SIZE) - 1) << MAC_REG_EXCOL_EXCOL_OFFSET)
/// Ethernet MAC Transmit Underrun Errors Register (Read/Write)
#define MAC_REG_TUND					0x00000064
/// Ethernet MAC Transmit Underrun Errors Register Transmit Underruns offset
#define MAC_REG_TUND_TUND_OFFSET			0
/// Ethernet MAC Transmit Underrun Errors Register Transmit Underruns size
#define MAC_REG_TUND_TUND_SIZE				8
/// Ethernet MAC Transmit Underrun Errors Register Transmit Underruns mask
#define MAC_REG_TUND_TUND_MASK				(((1 << MAC_REG_TUND_TUND_SIZE) - 1) << MAC_REG_TUND_TUND_OFFSET)
/// Ethernet MAC Carrier Sense Errors Register (Read/Write)
#define MAC_REG_CSE					0x00000068
/// Ethernet MAC Carrier Sense Errors Register Carrier Sense Errors offset
#define MAC_REG_CSE_CSE_OFFSET				0
/// Ethernet MAC Carrier Sense Errors Register Carrier Sense Errors size
#define MAC_REG_CSE_CSE_SIZE				8
/// Ethernet MAC Carrier Sense Errors Register Carrier Sense Errors mask
#define MAC_REG_CSE_CSE_MASK				(((1 << MAC_REG_CSE_CSE_SIZE) - 1) << MAC_REG_CSE_CSE_OFFSET)
/// Ethernet MAC Receive Resource Errors Register (Read/Write)
#define MAC_REG_RRE					0x0000006c
/// Ethernet MAC Receive Resource Errors Register Receive Resource Errors offset
#define MAC_REG_RRE_RRE_OFFSET				0
/// Ethernet MAC Receive Resource Errors Register Receive Resource Errors size
#define MAC_REG_RRE_RRE_SIZE				16
/// Ethernet MAC Receive Resource Errors Register Receive Resource Errors mask
#define MAC_REG_RRE_RRE_MASK				(((1 << MAC_REG_RRE_RRE_SIZE) - 1) << MAC_REG_RRE_RRE_OFFSET)
/// Ethernet MAC Receive Overrun Errors Register (Read/Write)
#define MAC_REG_ROV					0x00000070
/// Ethernet MAC Receive Overrun Errors Register Receive Overrun offset
#define MAC_REG_ROV_ROVR_OFFSET				0
/// Ethernet MAC Receive Overrun Errors Register Receive Overrun size
#define MAC_REG_ROV_ROVR_SIZE				8
/// Ethernet MAC Receive Overrun Errors Register Receive Overrun mask
#define MAC_REG_ROV_ROVR_MASK				(((1 << MAC_REG_ROV_ROVR_SIZE) - 1) << MAC_REG_ROV_ROVR_OFFSET)
/// Ethernet MAC Receive Symbol Errors Register (Read/Write)
#define MAC_REG_RSE					0x00000074
/// Ethernet MAC Receive Symbol Errors Register Receive Symbol Errors offset
#define MAC_REG_RSE_RSE_OFFSET				0
/// Ethernet MAC Receive Symbol Errors Register Receive Symbol Errors size
#define MAC_REG_RSE_RSE_SIZE				8
/// Ethernet MAC Receive Symbol Errors Register Receive Symbol Errors mask
#define MAC_REG_RSE_RSE_MASK				(((1 << MAC_REG_RSE_RSE_SIZE) - 1) << MAC_REG_RSE_RSE_OFFSET)
/// Ethernet MAC Excessive Length Errors Register (Read/Write)
#define MAC_REG_ELE					0x00000078
/// Ethernet MAC Excessive Length Errors Register Excessive Length Errors offset
#define MAC_REG_ELE_EXL_OFFSET				0
/// Ethernet MAC Excessive Length Errors Register Excessive Length Errors size
#define MAC_REG_ELE_EXL_SIZE				8
/// Ethernet MAC Excessive Length Errors Register Excessive Length Errors mask
#define MAC_REG_ELE_EXL_MASK				(((1 << MAC_REG_ELE_EXL_SIZE) - 1) << MAC_REG_ELE_EXL_OFFSET)
/// Ethernet MAC Receive Jabbers Register (Read/Write)
#define MAC_REG_RJA					0x0000007c
/// Ethernet MAC Receive Jabbers Register Receive Jabbers offset
#define MAC_REG_RJA_RJB_OFFSET				0
/// Ethernet MAC Receive Jabbers Register Receive Jabbers size
#define MAC_REG_RJA_RJB_SIZE				8
/// Ethernet MAC Receive Jabbers Register Receive Jabbers mask
#define MAC_REG_RJA_RJB_MASK				(((1 << MAC_REG_RJA_RJB_SIZE) - 1) << MAC_REG_RJA_RJB_OFFSET)
/// Ethernet MAC Undersize Frames Register (Read/Write)
#define MAC_REG_USF					0x00000080
/// Ethernet MAC Undersize Frames Register Undersize frames offset
#define MAC_REG_USF_USF_OFFSET				0
/// Ethernet MAC Undersize Frames Register Undersize frames size
#define MAC_REG_USF_USF_SIZE				8
/// Ethernet MAC Undersize Frames Register Undersize frames mask
#define MAC_REG_USF_USF_MASK				(((1 << MAC_REG_USF_USF_SIZE) - 1) << MAC_REG_USF_USF_OFFSET)
/// Ethernet MAC SQE Test Errors Register (Read/Write)
#define MAC_REG_STE					0x00000084
/// Ethernet MAC SQE Test Errors Register SQE test errors offset
#define MAC_REG_STE_SQER_OFFSET				0
/// Ethernet MAC SQE Test Errors Register SQE test errors size
#define MAC_REG_STE_SQER_SIZE				8
/// Ethernet MAC SQE Test Errors Register SQE test errors mask
#define MAC_REG_STE_SQER_MASK				(((1 << MAC_REG_STE_SQER_SIZE) - 1) << MAC_REG_STE_SQER_OFFSET)
/// Ethernet MAC Received Length Field Mismatch Register (Read/Write)
#define MAC_REG_RLE					0x00000088
/// Ethernet MAC Received Length Field Mismatch Register Receive Length Field Mismatch offset
#define MAC_REG_RLE_RLFM_OFFSET				0
/// Ethernet MAC Received Length Field Mismatch Register Receive Length Field Mismatch size
#define MAC_REG_RLE_RLFM_SIZE				8
/// Ethernet MAC Received Length Field Mismatch Register Receive Length Field Mismatch mask
#define MAC_REG_RLE_RLFM_MASK				(((1 << MAC_REG_RLE_RLFM_SIZE) - 1) << MAC_REG_RLE_RLFM_OFFSET)
/// Ethernet MAC Transmitted Pause Frames Register (Read/Write)
#define MAC_REG_TPF					0x0000008c
/// Ethernet MAC Transmitted Pause Frames Register Transmitted Pause Frames offset
#define MAC_REG_TPF_TPF_OFFSET				0
/// Ethernet MAC Transmitted Pause Frames Register Transmitted Pause Frames size
#define MAC_REG_TPF_TPF_SIZE				16
/// Ethernet MAC Transmitted Pause Frames Register Transmitted Pause Frames mask
#define MAC_REG_TPF_TPF_MASK				(((1 << MAC_REG_TPF_TPF_SIZE) - 1) << MAC_REG_TPF_TPF_OFFSET)
/// Ethernet MAC Hash Register Bottom [31:0] Register (Read/Write)
#define MAC_REG_HRB					0x00000090
/// Ethernet MAC Hash Register Bottom [31:0] Register Address offset
#define MAC_REG_HRB_ADDR_OFFSET				0
/// Ethernet MAC Hash Register Bottom [31:0] Register Address size
#define MAC_REG_HRB_ADDR_SIZE				32
/// Ethernet MAC Hash Register Bottom [31:0] Register Address mask
#define MAC_REG_HRB_ADDR_MASK				(((1 << MAC_REG_HRB_ADDR_SIZE) - 1) << MAC_REG_HRB_ADDR_OFFSET)
/// Ethernet MAC Hash Register Top [63:32] Register (Read/Write)
#define MAC_REG_HRT					0x00000094
/// Ethernet MAC Hash Register Top [63:32] Register Address offset
#define MAC_REG_HRT_ADDR_OFFSET				0
/// Ethernet MAC Hash Register Top [63:32] Register Address size
#define MAC_REG_HRT_ADDR_SIZE				32
/// Ethernet MAC Hash Register Top [63:32] Register Address mask
#define MAC_REG_HRT_ADDR_MASK				(((1 << MAC_REG_HRT_ADDR_SIZE) - 1) << MAC_REG_HRT_ADDR_OFFSET)
/// Ethernet MAC Specific Address 1 Bottom Register (Read/Write)
#define MAC_REG_SA1B					0x00000098
/// Ethernet MAC Specific Address 1 Bottom Register Address offset
#define MAC_REG_SA1B_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 1 Bottom Register Address size
#define MAC_REG_SA1B_ADDR_SIZE				32
/// Ethernet MAC Specific Address 1 Bottom Register Address mask
#define MAC_REG_SA1B_ADDR_MASK				(((1 << MAC_REG_SA1B_ADDR_SIZE) - 1) << MAC_REG_SA1B_ADDR_OFFSET)
/// Ethernet MAC Specific Address 1 Top Register (Read/Write)
#define MAC_REG_SA1T					0x0000009c
/// Ethernet MAC Specific Address 1 Top Register Address offset
#define MAC_REG_SA1T_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 1 Top Register Address size
#define MAC_REG_SA1T_ADDR_SIZE				16
/// Ethernet MAC Specific Address 1 Top Register Address mask
#define MAC_REG_SA1T_ADDR_MASK				(((1 << MAC_REG_SA1T_ADDR_SIZE) - 1) << MAC_REG_SA1T_ADDR_OFFSET)
/// Ethernet MAC Specific Address 2 Bottom Register (Read/Write)
#define MAC_REG_SA2B					0x000000a0
/// Ethernet MAC Specific Address 2 Bottom Register Address offset
#define MAC_REG_SA2B_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 2 Bottom Register Address size
#define MAC_REG_SA2B_ADDR_SIZE				32
/// Ethernet MAC Specific Address 2 Bottom Register Address mask
#define MAC_REG_SA2B_ADDR_MASK				(((1 << MAC_REG_SA2B_ADDR_SIZE) - 1) << MAC_REG_SA2B_ADDR_OFFSET)
/// Ethernet MAC Specific Address 2 Top Register (Read/Write)
#define MAC_REG_SA2T					0x000000a4
/// Ethernet MAC Specific Address 2 Top Register Address offset
#define MAC_REG_SA2T_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 2 Top Register Address size
#define MAC_REG_SA2T_ADDR_SIZE				16
/// Ethernet MAC Specific Address 2 Top Register Address mask
#define MAC_REG_SA2T_ADDR_MASK				(((1 << MAC_REG_SA2T_ADDR_SIZE) - 1) << MAC_REG_SA2T_ADDR_OFFSET)
/// Ethernet MAC Specific Address 3 Bottom Register (Read/Write)
#define MAC_REG_SA3B					0x000000a8
/// Ethernet MAC Specific Address 3 Bottom Register Address offset
#define MAC_REG_SA3B_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 3 Bottom Register Address size
#define MAC_REG_SA3B_ADDR_SIZE				32
/// Ethernet MAC Specific Address 3 Bottom Register Address mask
#define MAC_REG_SA3B_ADDR_MASK				(((1 << MAC_REG_SA3B_ADDR_SIZE) - 1) << MAC_REG_SA3B_ADDR_OFFSET)
/// Ethernet MAC Specific Address 3 Top Register (Read/Write)
#define MAC_REG_SA3T					0x000000ac
/// Ethernet MAC Specific Address 3 Top Register Address offset
#define MAC_REG_SA3T_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 3 Top Register Address size
#define MAC_REG_SA3T_ADDR_SIZE				16
/// Ethernet MAC Specific Address 3 Top Register Address mask
#define MAC_REG_SA3T_ADDR_MASK				(((1 << MAC_REG_SA3T_ADDR_SIZE) - 1) << MAC_REG_SA3T_ADDR_OFFSET)
/// Ethernet MAC Specific Address 4 Bottom Register (Read/Write)
#define MAC_REG_SA4B					0x000000b0
/// Ethernet MAC Specific Address 4 Bottom Register Address offset
#define MAC_REG_SA4B_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 4 Bottom Register Address size
#define MAC_REG_SA4B_ADDR_SIZE				32
/// Ethernet MAC Specific Address 4 Bottom Register Address mask
#define MAC_REG_SA4B_ADDR_MASK				(((1 << MAC_REG_SA4B_ADDR_SIZE) - 1) << MAC_REG_SA4B_ADDR_OFFSET)
/// Ethernet MAC Specific Address 4 Top Register (Read/Write)
#define MAC_REG_SA4T					0x000000b4
/// Ethernet MAC Specific Address 4 Top Register Address offset
#define MAC_REG_SA4T_ADDR_OFFSET			0
/// Ethernet MAC Specific Address 4 Top Register Address size
#define MAC_REG_SA4T_ADDR_SIZE				16
/// Ethernet MAC Specific Address 4 Top Register Address mask
#define MAC_REG_SA4T_ADDR_MASK				(((1 << MAC_REG_SA4T_ADDR_SIZE) - 1) << MAC_REG_SA4T_ADDR_OFFSET)
/// Ethernet MAC Type ID Checking Register (Read/Write)
#define MAC_REG_TID					0x000000b8
/// Ethernet MAC Type ID Checking Register Type ID checking offset
#define MAC_REG_TID_TID_OFFSET				0
/// Ethernet MAC Type ID Checking Register Type ID checking size
#define MAC_REG_TID_TID_SIZE				16
/// Ethernet MAC Type ID Checking Register Type ID checking mask
#define MAC_REG_TID_TID_MASK				(((1 << MAC_REG_TID_TID_SIZE) - 1) << MAC_REG_TID_TID_OFFSET)
/// Ethernet MAC Transmit Pause Quantum Register (Read/Write)
#define MAC_REG_TPQ					0x000000bc
/// Ethernet MAC Transmit Pause Quantum Register Transmit Pause Quantum offset
#define MAC_REG_TPQ_TPQ_OFFSET				0
/// Ethernet MAC Transmit Pause Quantum Register Transmit Pause Quantum size
#define MAC_REG_TPQ_TPQ_SIZE				16
/// Ethernet MAC Transmit Pause Quantum Register Transmit Pause Quantum mask
#define MAC_REG_TPQ_TPQ_MASK				(((1 << MAC_REG_TPQ_TPQ_SIZE) - 1) << MAC_REG_TPQ_TPQ_OFFSET)
/// Ethernet MAC User Input/Output Register (Read/Write)
#define MAC_REG_USRIO					0x000000c0
/// Ethernet MAC User Input/Output Register RMII offset
#define MAC_REG_USRIO_RMII_OFFSET			0
/// Ethernet MAC User Input/Output Register RMII size
#define MAC_REG_USRIO_RMII_SIZE				1
/// Ethernet MAC User Input/Output Register RMII mask
#define MAC_REG_USRIO_RMII_MASK				(((1 << MAC_REG_USRIO_RMII_SIZE) - 1) << MAC_REG_USRIO_RMII_OFFSET)
/// Ethernet MAC User Input/Output Register EAM offset
#define MAC_REG_USRIO_EAM_OFFSET			1
/// Ethernet MAC User Input/Output Register EAM size
#define MAC_REG_USRIO_EAM_SIZE				1
/// Ethernet MAC User Input/Output Register EAM mask
#define MAC_REG_USRIO_EAM_MASK				(((1 << MAC_REG_USRIO_EAM_SIZE) - 1) << MAC_REG_USRIO_EAM_OFFSET)
/// Ethernet MAC User Input/Output Register Transmit Pause Frame offset
#define MAC_REG_USRIO_TX_PAUSE_OFFSET			2
/// Ethernet MAC User Input/Output Register Transmit Pause Frame size
#define MAC_REG_USRIO_TX_PAUSE_SIZE			1
/// Ethernet MAC User Input/Output Register Transmit Pause Frame mask
#define MAC_REG_USRIO_TX_PAUSE_MASK			(((1 << MAC_REG_USRIO_TX_PAUSE_SIZE) - 1) << MAC_REG_USRIO_TX_PAUSE_OFFSET)
/// Ethernet MAC User Input/Output Register Transmitted Pause Frame Quantum Select offset
#define MAC_REG_USRIO_TX_PAUSE_ZERO_OFFSET		3
/// Ethernet MAC User Input/Output Register Transmitted Pause Frame Quantum Select size
#define MAC_REG_USRIO_TX_PAUSE_ZERO_SIZE		1
/// Ethernet MAC User Input/Output Register Transmitted Pause Frame Quantum Select mask
#define MAC_REG_USRIO_TX_PAUSE_ZERO_MASK		(((1 << MAC_REG_USRIO_TX_PAUSE_ZERO_SIZE) - 1) << MAC_REG_USRIO_TX_PAUSE_ZERO_OFFSET)
/// Ethernet MAC Wake on LAN Register (Read/Write)
#define MAC_REG_WOL					0x000000c4
/// Ethernet MAC Wake on LAN Register ARP request IP address offset
#define MAC_REG_WOL_IP_OFFSET				0
/// Ethernet MAC Wake on LAN Register ARP request IP address size
#define MAC_REG_WOL_IP_SIZE				16
/// Ethernet MAC Wake on LAN Register ARP request IP address mask
#define MAC_REG_WOL_IP_MASK				(((1 << MAC_REG_WOL_IP_SIZE) - 1) << MAC_REG_WOL_IP_OFFSET)
/// Ethernet MAC Wake on LAN Register Magic packet event enable offset
#define MAC_REG_WOL_MAG_OFFSET				16
/// Ethernet MAC Wake on LAN Register Magic packet event enable size
#define MAC_REG_WOL_MAG_SIZE				1
/// Ethernet MAC Wake on LAN Register Magic packet event enable mask
#define MAC_REG_WOL_MAG_MASK				(((1 << MAC_REG_WOL_MAG_SIZE) - 1) << MAC_REG_WOL_MAG_OFFSET)
/// Ethernet MAC Wake on LAN Register ARP request event enable offset
#define MAC_REG_WOL_ARP_OFFSET				17
/// Ethernet MAC Wake on LAN Register ARP request event enable size
#define MAC_REG_WOL_ARP_SIZE				1
/// Ethernet MAC Wake on LAN Register ARP request event enable mask
#define MAC_REG_WOL_ARP_MASK				(((1 << MAC_REG_WOL_ARP_SIZE) - 1) << MAC_REG_WOL_ARP_OFFSET)
/// Ethernet MAC Wake on LAN Register Specific address register 1 event enable offset
#define MAC_REG_WOL_SA1_OFFSET				18
/// Ethernet MAC Wake on LAN Register Specific address register 1 event enable size
#define MAC_REG_WOL_SA1_SIZE				1
/// Ethernet MAC Wake on LAN Register Specific address register 1 event enable mask
#define MAC_REG_WOL_SA1_MASK				(((1 << MAC_REG_WOL_SA1_SIZE) - 1) << MAC_REG_WOL_SA1_OFFSET)
/// Ethernet MAC Wake on LAN Register Multicast hash event enable offset
#define MAC_REG_WOL_MTI_OFFSET				19
/// Ethernet MAC Wake on LAN Register Multicast hash event enable size
#define MAC_REG_WOL_MTI_SIZE				1
/// Ethernet MAC Wake on LAN Register Multicast hash event enable mask
#define MAC_REG_WOL_MTI_MASK				(((1 << MAC_REG_WOL_MTI_SIZE) - 1) << MAC_REG_WOL_MTI_OFFSET)

extern uint32_t mac_read_register(uint32_t const offset);
extern void mac_write_register(uint32_t const offset, uint32_t const value);

#endif /* MAC_H_ */

