/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef PDC_H_
#define PDC_H_

#include <at32uc3/inttypes.h>

/// PDC Address
#define PDC_ADDRESS					0xffff0000
/// PDC Channel Width
#define PDC_CHANNEL_WIDTH				0x00000040
/// Number of PDC channels
#define PDC_NUM_CHANNELS				15 // Really?

#define PDC_CHANNEL(n)					((void volatile *)PDC_ADDRESS + PDC_CHANNEL_WIDTH * (n))

/// PDC Channel Memory Address Register (Read/Write)
#define PDC_CHANNEL_REG_MAR				0x00000000
/// PDC Channel Peripheral Select Register (Read/Write)
#define PDC_CHANNEL_REG_PSR				0x00000004
/// PDC Channel Transfer Counter Register (Read/Write)
#define PDC_CHANNEL_REG_TCR				0x00000008
/// PDC Channel Memory Address Reload Register (Read/Write)
#define PDC_CHANNEL_REG_MARR				0x0000000c
/// PDC Channel Transfer Counter Reload Register (Read/Write)
#define PDC_CHANNEL_REG_TCRR				0x00000010
/// PDC Channel Control Register (Write)
#define PDC_CHANNEL_REG_CR				0x00000014
/// PDC Channel Control Register Transfer Enable offset
#define PDC_CHANNEL_REG_CR_TEN_OFFSET			0
/// PDC Channel Control Register Transfer Enable size
#define PDC_CHANNEL_REG_CR_TEN_SIZE			1
/// PDC Channel Control Register Transfer Enable mask
#define PDC_CHANNEL_REG_CR_TEN_MASK			(((1 << PDC_CHANNEL_REG_CR_TEN_SIZE) - 1) << PDC_CHANNEL_REG_CR_TEN_OFFSET)
/// PDC Channel Control Register Transfer Disable offset
#define PDC_CHANNEL_REG_CR_TDIS_OFFSET			1
/// PDC Channel Control Register Transfer Disable size
#define PDC_CHANNEL_REG_CR_TDIS_SIZE			1
/// PDC Channel Control Register Transfer Disable mask
#define PDC_CHANNEL_REG_CR_TDIS_MASK			(((1 << PDC_CHANNEL_REG_CR_TDIS_SIZE) - 1) << PDC_CHANNEL_REG_CR_TDIS_OFFSET)
/// PDC Channel Control Register Error Clear offset
#define PDC_CHANNEL_REG_CR_ECLR_OFFSET			8
/// PDC Channel Control Register Error Clear size
#define PDC_CHANNEL_REG_CR_ECLR_SIZE			1
/// PDC Channel Control Register Error Clear mask
#define PDC_CHANNEL_REG_CR_ECLR_MASK			(((1 << PDC_CHANNEL_REG_CR_ECLR_SIZE) - 1) << PDC_CHANNEL_REG_CR_ECLR_OFFSET)
/// PDC Channel Mode Register (Read/Write)
#define PDC_CHANNEL_REG_MR				0x00000018
/// PDC Channel Mode Register Size of Transfer offset
#define PDC_CHANNEL_REG_MR_SIZE_OFFSET			0
/// PDC Channel Mode Register Size of Transfer size
#define PDC_CHANNEL_REG_MR_SIZE_SIZE			2
/// PDC Channel Mode Register Size of Transfer mask
#define PDC_CHANNEL_REG_MR_SIZE_MASK			(((1 << PDC_CHANNEL_REG_MR_SIZE_SIZE) - 1) << PDC_CHANNEL_REG_MR_SIZE_OFFSET)
/// PDC Channel Status Register (Read)
#define PDC_CHANNEL_REG_SR				0x0000001c
/// PDC Channel Status Register Transfer Enabled offset
#define PDC_CHANNEL_REG_SR_TEN_OFFSET			0
/// PDC Channel Status Register Transfer Enabled size
#define PDC_CHANNEL_REG_SR_TEN_SIZE			1
/// PDC Channel Status Register Transfer Enabled mask
#define PDC_CHANNEL_REG_SR_TEN_MASK			(((1 << PDC_CHANNEL_REG_SR_TEN_SIZE) - 1) << PDC_CHANNEL_REG_SR_TEN_OFFSET)
/// PDC Channel Interrupt Enable Register (Write)
#define PDC_CHANNEL_REG_IER				0x00000020
/// PDC Channel Interrupt Disable Register (Write)
#define PDC_CHANNEL_REG_IDR				0x00000024
/// PDC Channel Interrupt Mask Register (Read)
#define PDC_CHANNEL_REG_IMR				0x00000028
/// PDC Channel Interrupt Status Register (Read)
#define PDC_CHANNEL_REG_ISR				0x0000002c
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Reload Counter Zero Interrupt Enable/Disable/Mask/Status offset
#define PDC_CHANNEL_REG_IXR_RCZ_OFFSET			0
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Reload Counter Zero Interrupt Enable/Disable/Mask/Status size
#define PDC_CHANNEL_REG_IXR_RCZ_SIZE			1
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Reload Counter Zero Interrupt Enable/Disable/Mask/Status mask
#define PDC_CHANNEL_REG_IXR_RCZ_MASK			(((1 << PDC_CHANNEL_REG_IXR_RCZ_SIZE) - 1) << PDC_CHANNEL_REG_IXR_RCZ_OFFSET)
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Complete Interrupt Enable/Disable/Mask/Status offset
#define PDC_CHANNEL_REG_IXR_TRC_OFFSET			1
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Complete Interrupt Enable/Disable/Mask/Status size
#define PDC_CHANNEL_REG_IXR_TRC_SIZE			1
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Complete Interrupt Enable/Disable/Mask/Status mask
#define PDC_CHANNEL_REG_IXR_TRC_MASK			(((1 << PDC_CHANNEL_REG_IXR_TRC_SIZE) - 1) << PDC_CHANNEL_REG_IXR_TRC_OFFSET)
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Error Interrupt Enable/Disable/Mask/Status offset
#define PDC_CHANNEL_REG_IXR_TERR_OFFSET			2
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Error Interrupt Enable/Disable/Mask/Status size
#define PDC_CHANNEL_REG_IXR_TERR_SIZE			1
/// PDC Channel Interrupt Enable/Disable/Mask/Status Register Transfer Error Interrupt Enable/Disable/Mask/Status mask
#define PDC_CHANNEL_REG_IXR_TERR_MASK			(((1 << PDC_CHANNEL_REG_IXR_TERR_SIZE) - 1) << PDC_CHANNEL_REG_IXR_TERR_OFFSET)

typedef enum {
	PDC_PID_ADC					= 0,
	PDC_PID_SSC_RX					= 1,
	PDC_PID_USART0_RX				= 2,
	PDC_PID_USART1_RX				= 3,
	PDC_PID_USART2_RX				= 4,
	PDC_PID_USART3_RX				= 5,
	PDC_PID_TWI_RX					= 6,
	PDC_PID_SPI0_RX					= 7,
	PDC_PID_SPI1_RX					= 8,
	PDC_PID_SSC_TX					= 9,
	PDC_PID_USART0_TX				= 10,
	PDC_PID_USART1_TX				= 11,
	PDC_PID_USART2_TX				= 12,
	PDC_PID_USART3_TX				= 13,
	PDC_PID_TWI_TX					= 14,
	PDC_PID_SPI0_TX					= 15,
	PDC_PID_SPI1_TX					= 16,
	PDC_PID_ABDAC					= 17,
} pdc_pid_t;

typedef enum {
	PDC_TRANSFER_SIZE_BYTE				= 0b00 << PDC_CHANNEL_REG_MR_SIZE_OFFSET,
	PDC_TRANSFER_SIZE_HALF_WORD			= 0b01 << PDC_CHANNEL_REG_MR_SIZE_OFFSET,
	PDC_TRANSFER_SIZE_WORD				= 0b10 << PDC_CHANNEL_REG_MR_SIZE_OFFSET,
} pdc_transfer_size_t;

typedef enum {
	PDC_INTERRUPT_RELOAD_COUNTER_ZERO		= PDC_CHANNEL_REG_IXR_RCZ_MASK,
	PDC_INTERRUPT_TRANSFER_COMPLETE			= PDC_CHANNEL_REG_IXR_TRC_MASK,
	PDC_INTERRUPT_TRANSFER_ERROR			= PDC_CHANNEL_REG_IXR_TERR_MASK,
} pdc_interrupts_t;

extern uint32_t pdc_channel_read_register(void volatile *channel, uint32_t const offset);
extern void pdc_channel_write_register(void volatile *channel, uint32_t const offset, uint32_t const value);

extern pdc_pid_t pdc_channel_get_pid(void volatile *channel);
extern void pdc_channel_set_pid(void volatile *channel, pdc_pid_t const pid);
extern pdc_transfer_size_t pdc_channel_get_transfer_size(void volatile *channel);
extern void pdc_channel_set_transfer_size(void volatile *channel, pdc_transfer_size_t const size);
extern uint32_t pdc_channel_get_memory_address(void volatile *channel);
extern void pdc_channel_set_memory_address(void volatile *channel, uint32_t const address);
extern uint32_t pdc_channel_get_memory_address_reload(void volatile *channel);
extern void pdc_channel_set_memory_address_reload(void volatile *channel, uint32_t const address);
extern uint16_t pdc_channel_get_transfer_count(void volatile *channel);
extern void pdc_channel_set_transfer_count(void volatile *channel, uint16_t const count);
extern uint16_t pdc_channel_get_transfer_count_reload(void volatile *channel);
extern void pdc_channel_set_transfer_count_reload(void volatile *channel, uint16_t const count);

extern void pdc_channel_configure(void volatile *channel, pdc_pid_t const pid, pdc_transfer_size_t const size);
extern void pdc_channel_load(void volatile *channel, uint32_t const address, uint16_t const count);
extern void pdc_channel_load2(void volatile *channel, uint32_t const address, uint16_t const count, uint32_t const reload_address, uint16_t const reload_count);
extern void pdc_channel_reload(void volatile *channel, uint32_t const address, uint16_t const count);

extern void pdc_channel_enable_transfer(void volatile *channel);
extern void pdc_channel_disable_transfer(void volatile *channel);
extern uint8_t pdc_channel_is_transfer_enabled(void volatile *channel);
extern void pdc_channel_clear_error(void volatile *channel);

extern void pdc_channel_enable_interrupts(void volatile *channel, pdc_interrupts_t const interrupts);
extern void pdc_channel_disable_interrupts(void volatile *channel, pdc_interrupts_t const interrupts);
extern pdc_interrupts_t pdc_channel_get_interrupt_mask(void volatile *channel);
extern pdc_interrupts_t pdc_channel_get_interrupt_status(void volatile *channel);

#endif /* PDC_H_ */
