/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef PM_H_
#define PM_H_

#include <at32uc3/gccmacros.h>
#include <at32uc3/inttypes.h>

/// PM address
#define PM_ADDRESS					0xffff0c00

/// PM Main Clock Control Register (Read/Write)
#define PM_REG_MCCTRL					0x00000000
/// PM Main Clock Control Register Main Clock Select offset
#define PM_REG_MCCTRL_MCSEL_OFFSET			0
/// PM Main Clock Control Register Main Clock Select size
#define PM_REG_MCCTRL_MCSEL_SIZE			2
/// PM Main Clock Control Register Main Clock Select mask
#define PM_REG_MCCTRL_MCSEL_MASK			(((1 << PM_REG_MCCTRL_MCSEL_SIZE) - 1) << PM_REG_MCCTRL_MCSEL_OFFSET)
/// PM Main Clock Control Register Oscillator 0 Enable offset
#define PM_REG_MCCTRL_OSC0EN_OFFSET			2
/// PM Main Clock Control Register Oscillator 0 Enable size
#define PM_REG_MCCTRL_OSC0EN_SIZE			1
/// PM Main Clock Control Register Oscillator 0 Enable mask
#define PM_REG_MCCTRL_OSC0EN_MASK			(((1 << PM_REG_MCCTRL_OSC0EN_SIZE) - 1) << PM_REG_MCCTRL_OSC0EN_OFFSET)
/// PM Main Clock Control Register Oscillator 1 Enable offset
#define PM_REG_MCCTRL_OSC1EN_OFFSET			3
/// PM Main Clock Control Register Oscillator 1 Enable size
#define PM_REG_MCCTRL_OSC1EN_SIZE			1
/// PM Main Clock Control Register Oscillator 1 Enable mask
#define PM_REG_MCCTRL_OSC1EN_MASK			(((1 << PM_REG_MCCTRL_OSC1EN_SIZE) - 1) << PM_REG_MCCTRL_OSC1EN_OFFSET)
/// PM Clock Select Register (Read/Write)
#define PM_REG_CKSEL					0x00000004
/// PM Clock Select Register CPU Clock Select offset
#define PM_REG_CKSEL_CPUSEL_OFFSET			0
/// PM Clock Select Register CPU Clock Select size
#define PM_REG_CKSEL_CPUSEL_SIZE			3
/// PM Clock Select Register CPU Clock Select mask
#define PM_REG_CKSEL_CPUSEL_MASK			(((1 << PM_REG_CKSEL_CPUSEL_SIZE) - 1) << PM_REG_CKSEL_CPUSEL_OFFSET)
/// PM Clock Select Register CPU Division offset
#define PM_REG_CKSEL_CPUDIV_OFFSET			7
/// PM Clock Select Register CPU Division size
#define PM_REG_CKSEL_CPUDIV_SIZE			1
/// PM Clock Select Register CPU Division mask
#define PM_REG_CKSEL_CPUDIV_MASK			(((1 << PM_REG_CKSEL_CPUDIV_SIZE) - 1) << PM_REG_CKSEL_CPUDIV_OFFSET)
/// PM Clock Select Register HSB Clock Select offset
#define PM_REG_CKSEL_HSBSEL_OFFSET			8
/// PM Clock Select Register HSB Clock Select size
#define PM_REG_CKSEL_HSBSEL_SIZE			3
/// PM Clock Select Register HSB Clock Select mask
#define PM_REG_CKSEL_HSBSEL_MASK			(((1 << PM_REG_CKSEL_HSBSEL_SIZE) - 1) << PM_REG_CKSEL_HSBSEL_OFFSET)
/// PM Clock Select Register HSB Division offset
#define PM_REG_CKSEL_HSBDIV_OFFSET			15
/// PM Clock Select Register HSB Division size
#define PM_REG_CKSEL_HSBDIV_SIZE			1
/// PM Clock Select Register HSB Division mask
#define PM_REG_CKSEL_HSBDIV_MASK			(((1 << PM_REG_CKSEL_HSBDIV_SIZE) - 1) << PM_REG_CKSEL_HSBDIV_OFFSET)
/// PM Clock Select Register PBA Clock Select offset
#define PM_REG_CKSEL_PBASEL_OFFSET			16
/// PM Clock Select Register PBA Clock Select size
#define PM_REG_CKSEL_PBASEL_SIZE			3
/// PM Clock Select Register PBA Clock Select mask
#define PM_REG_CKSEL_PBASEL_MASK			(((1 << PM_REG_CKSEL_PBASEL_SIZE) - 1) << PM_REG_CKSEL_PBASEL_OFFSET)
/// PM Clock Select Register PBA Division offset
#define PM_REG_CKSEL_PBADIV_OFFSET			23
/// PM Clock Select Register PBA Division size
#define PM_REG_CKSEL_PBADIV_SIZE			1
/// PM Clock Select Register PBA Division mask
#define PM_REG_CKSEL_PBADIV_MASK			(((1 << PM_REG_CKSEL_PBADIV_SIZE) - 1) << PM_REG_CKSEL_PBADIV_OFFSET)
/// PM Clock Select Register PBB Clock Select offset
#define PM_REG_CKSEL_PBBSEL_OFFSET			24
/// PM Clock Select Register PBB Clock Select size
#define PM_REG_CKSEL_PBBSEL_SIZE			3
/// PM Clock Select Register PBB Clock Select mask
#define PM_REG_CKSEL_PBBSEL_MASK			(((1 << PM_REG_CKSEL_PBBSEL_SIZE) - 1) << PM_REG_CKSEL_PBBSEL_OFFSET)
/// PM Clock Select Register PBB Division offset
#define PM_REG_CKSEL_PBBDIV_OFFSET			31
/// PM Clock Select Register PBB Division size
#define PM_REG_CKSEL_PBBDIV_SIZE			1
/// PM Clock Select Register PBB Division mask
#define PM_REG_CKSEL_PBBDIV_MASK			(((1 << PM_REG_CKSEL_PBBDIV_SIZE) - 1) << PM_REG_CKSEL_PBBDIV_OFFSET)
/// PM CPU Mask Register (Read/Write)
#define PM_REG_CPUMASK					0x00000008
/// PM HSB Mask Register (Read/Write)
#define PM_REG_HSBMASK					0x0000000c
/// PM PBA Mask Register (Read/Write)
#define PM_REG_PBAMASK					0x00000010
/// PM PBB Mask Register (Read/Write)
#define PM_REG_PBBMASK					0x00000014
/// PM RW Clock Mask offset
#define PM_REG_XMASK_MASK_OFFSET			0
/// PM RW Clock Mask size
#define PM_REG_XMASK_MASK_SIZE				32
/// PM RW Clock Mask mask
#define PM_REG_XMASK_MASK_MASK				(((1 << PM_REG_XMASK_MASK_SIZE) - 1) << PM_REG_XMASK_MASK_OFFSET)
/// PM PLL0 Control Register (Read/Write)
#define PM_REG_PLL0					0x00000020
/// PM PLL1 Control Register (Read/Write)
#define PM_REG_PLL1					0x00000024
/// PM RW PLL Enable offset
#define PM_REG_PLLX_PLLEN_OFFSET			0
/// PM RW PLL Enable size
#define PM_REG_PLLX_PLLEN_SIZE				1
/// PM RW PLL Enable mask
#define PM_REG_PLLX_PLLEN_MASK				(((1 << PM_REG_PLLX_PLLEN_SIZE) - 1) << PM_REG_PLLX_PLLEN_OFFSET)
/// PM RW PLL Oscillator Select offset
#define PM_REG_PLLX_PLLOSC_OFFSET			1
/// PM RW PLL Oscillator Select size
#define PM_REG_PLLX_PLLOSC_SIZE				1
/// PM RW PLL Oscillator Select mask
#define PM_REG_PLLX_PLLOSC_MASK				(((1 << PM_REG_PLLX_PLLOSC_SIZE) - 1) << PM_REG_PLLX_PLLOSC_OFFSET)
/// PM RW PLL Option offset
#define PM_REG_PLLX_PLLOPT_OFFSET			2
/// PM RW PLL Option size
#define PM_REG_PLLX_PLLOPT_SIZE				3
/// PM RW PLL Option mask
#define PM_REG_PLLX_PLLOPT_MASK				(((1 << PM_REG_PLLX_PLLOPT_SIZE) - 1) << PM_REG_PLLX_PLLOPT_OFFSET)
/// PM RW PLL Option Bit 0 offset
#define PM_REG_PLLX_PLLOPT0_OFFSET			2
/// PM RW PLL Option Bit 0 size
#define PM_REG_PLLX_PLLOPT0_SIZE			1
/// PM RW PLL Option Bit 0 mask
#define PM_REG_PLLX_PLLOPT0_MASK			(((1 << PM_REG_PLLX_PLLOPT0_SIZE) - 1) << PM_REG_PLLX_PLLOPT0_OFFSET)
/// PM RW PLL Option Bit 1 offset
#define PM_REG_PLLX_PLLOPT1_OFFSET			3
/// PM RW PLL Option Bit 1 size
#define PM_REG_PLLX_PLLOPT1_SIZE			1
/// PM RW PLL Option Bit 1 mask
#define PM_REG_PLLX_PLLOPT1_MASK			(((1 << PM_REG_PLLX_PLLOPT1_SIZE) - 1) << PM_REG_PLLX_PLLOPT1_OFFSET)
/// PM RW PLL Option Bit 2 offset
#define PM_REG_PLLX_PLLOPT2_OFFSET			4
/// PM RW PLL Option Bit 2 size
#define PM_REG_PLLX_PLLOPT2_SIZE			1
/// PM RW PLL Option Bit 2 mask
#define PM_REG_PLLX_PLLOPT2_MASK			(((1 << PM_REG_PLLX_PLLOPT2_SIZE) - 1) << PM_REG_PLLX_PLLOPT2_OFFSET)
/// PM RW PLL Division Factor offset
#define PM_REG_PLLX_PLLDIV_OFFSET			8
/// PM RW PLL Division Factor size
#define PM_REG_PLLX_PLLDIV_SIZE				4
/// PM RW PLL Division Factor mask
#define PM_REG_PLLX_PLLDIV_MASK				(((1 << PM_REG_PLLX_PLLDIV_SIZE) - 1) << PM_REG_PLLX_PLLDIV_OFFSET)
/// PM RW PLL Multiply Factor offset
#define PM_REG_PLLX_PLLMUL_OFFSET			16
/// PM RW PLL Multiply Factor size
#define PM_REG_PLLX_PLLMUL_SIZE				4
/// PM RW PLL Multiply Factor mask
#define PM_REG_PLLX_PLLMUL_MASK				(((1 << PM_REG_PLLX_PLLMUL_SIZE) - 1) << PM_REG_PLLX_PLLMUL_OFFSET)
/// PM RW PLL Count offset
#define PM_REG_PLLX_PLLCOUNT_OFFSET			24
/// PM RW PLL Count size
#define PM_REG_PLLX_PLLCOUNT_SIZE			6
/// PM RW PLL Count mask
#define PM_REG_PLLX_PLLCOUNT_MASK			(((1 << PM_REG_PLLX_PLLCOUNT_SIZE) - 1) << PM_REG_PLLX_PLLCOUNT_OFFSET)
/// PM Oscillator 0 Control Register (Read/Write)
#define PM_REG_OSCCTRL0					0x00000028
/// PM Oscillator 1 Control Register (Read/Write)
#define PM_REG_OSCCTRL1					0x0000002c
/// PM Oscillator 0/1 Control Register Oscillator Mode offset
#define PM_REG_OSCCTRLX_MODE_OFFSET			0
/// PM Oscillator 0/1 Control Register Oscillator Mode size
#define PM_REG_OSCCTRLX_MODE_SIZE			3
/// PM Oscillator 0/1 Control Register Oscillator Mode mask
#define PM_REG_OSCCTRLX_MODE_MASK			(((1 << PM_REG_OSCCTRLX_MODE_SIZE) - 1) << PM_REG_OSCCTRLX_MODE_OFFSET)
/// PM Oscillator 0/1 Control Register Oscillator Startup Time offset
#define PM_REG_OSCCTRLX_STARTUP_OFFSET			8
/// PM Oscillator 0/1 Control Register Oscillator Startup Time size
#define PM_REG_OSCCTRLX_STARTUP_SIZE			3
/// PM Oscillator 0/1 Control Register Oscillator Startup Time mask
#define PM_REG_OSCCTRLX_STARTUP_MASK			(((1 << PM_REG_OSCCTRLX_STARTUP_SIZE) - 1) << PM_REG_OSCCTRLX_STARTUP_OFFSET)
/// PM 32 kHz Oscillator Control Register (Read/Write)
#define PM_REG_OSCCTRL32				0x00000030
/// PM 32 kHz Oscillator Control Register 32 kHz Oscillator Enable offset
#define PM_REG_OSCCTRL32_OSC32EN_OFFSET			0
/// PM 32 kHz Oscillator Control Register 32 kHz Oscillator Enable size
#define PM_REG_OSCCTRL32_OSC32EN_SIZE			1
/// PM 32 kHz Oscillator Control Register 32 kHz Oscillator Enable mask
#define PM_REG_OSCCTRL32_OSC32EN_MASK			(((1 << PM_REG_OSCCTRL32_OSC32EN_SIZE) - 1) << PM_REG_OSCCTRL32_OSC32EN_OFFSET)
/// PM 32 kHz Oscillator Control Register Oscillator Mode offset
#define PM_REG_OSCCTRL32_MODE_OFFSET			8
/// PM 32 kHz Oscillator Control Register Oscillator Mode size
#define PM_REG_OSCCTRL32_MODE_SIZE			3
/// PM 32 kHz Oscillator Control Register Oscillator Mode mask
#define PM_REG_OSCCTRL32_MODE_MASK			(((1 << PM_REG_OSCCTRL32_MODE_SIZE) - 1) << PM_REG_OSCCTRL32_MODE_OFFSET)
/// PM 32 kHz Oscillator Control Register Oscillator Startup Time offset
#define PM_REG_OSCCTRL32_STARTUP_OFFSET			16
/// PM 32 kHz Oscillator Control Register Oscillator Startup Time size
#define PM_REG_OSCCTRL32_STARTUP_SIZE			3
/// PM 32 kHz Oscillator Control Register Oscillator Startup Time mask
#define PM_REG_OSCCTRL32_STARTUP_MASK			(((1 << PM_REG_OSCCTRL32_STARTUP_SIZE) - 1) << PM_REG_OSCCTRL32_STARTUP_OFFSET)
/// PM Interrupt Enable Register (Write)
#define PM_REG_IER					0x00000040
/// PM Interrupt Disable Register (Write)
#define PM_REG_IDR					0x00000044
/// PM Interrupt Mask Register (Read)
#define PM_REG_IMR					0x00000048
/// PM Interrupt Status Register (Read)
#define PM_REG_ISR					0x0000004c
/// PM Interrupt Clear Register (Write)
#define PM_REG_ICR					0x00000050
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL0 Locked offset
#define PM_REG_IXR_LOCK0_OFFSET				0
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL0 Locked size
#define PM_REG_IXR_LOCK0_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL0 Locked mask
#define PM_REG_IXR_LOCK0_MASK				(((1 << PM_REG_IXR_LOCK0_SIZE) - 1) << PM_REG_IXR_LOCK0_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL1 Locked offset
#define PM_REG_IXR_LOCK1_OFFSET				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL1 Locked size
#define PM_REG_IXR_LOCK1_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register PLL1 Locked mask
#define PM_REG_IXR_LOCK1_MASK				(((1 << PM_REG_IXR_LOCK1_SIZE) - 1) << PM_REG_IXR_LOCK1_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Clock Ready offset
#define PM_REG_IXR_CKRDY_OFFSET				5
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Clock Ready size
#define PM_REG_IXR_CKRDY_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Clock Ready mask
#define PM_REG_IXR_CKRDY_MASK				(((1 << PM_REG_IXR_CKRDY_SIZE) - 1) << PM_REG_IXR_CKRDY_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Mask Ready offset
#define PM_REG_IXR_MSKRDY_OFFSET			6
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Mask Ready size
#define PM_REG_IXR_MSKRDY_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Mask Ready mask
#define PM_REG_IXR_MSKRDY_MASK				(((1 << PM_REG_IXR_MSKRDY_SIZE) - 1) << PM_REG_IXR_MSKRDY_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 0 Ready offset
#define PM_REG_IXR_OSC0RDY_OFFSET			7
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 0 Ready size
#define PM_REG_IXR_OSC0RDY_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 0 Ready mask
#define PM_REG_IXR_OSC0RDY_MASK				(((1 << PM_REG_IXR_OSC0RDY_SIZE) - 1) << PM_REG_IXR_OSC0RDY_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 1 Ready offset
#define PM_REG_IXR_OSC1RDY_OFFSET			8
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 1 Ready size
#define PM_REG_IXR_OSC1RDY_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Oscillator 1 Ready mask
#define PM_REG_IXR_OSC1RDY_MASK				(((1 << PM_REG_IXR_OSC1RDY_SIZE) - 1) << PM_REG_IXR_OSC1RDY_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register 32 kHz Oscillator Ready offset
#define PM_REG_IXR_OSC32RDY_OFFSET			9
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register 32 kHz Oscillator Ready size
#define PM_REG_IXR_OSC32RDY_SIZE			1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register 32 kHz Oscillator Ready mask
#define PM_REG_IXR_OSC32RDY_MASK			(((1 << PM_REG_IXR_OSC32RDY_SIZE) - 1) << PM_REG_IXR_OSC32RDY_OFFSET)
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Brown-out Detected offset
#define PM_REG_IXR_BODDET_OFFSET			16
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Brown-out Detected size
#define PM_REG_IXR_BODDET_SIZE				1
/// PM Interrupt Enable/Disable/Mask/Status/Clear Register Brown-out Detected mask
#define PM_REG_IXR_BODDET_MASK				(((1 << PM_REG_IXR_BODDET_SIZE) - 1) << PM_REG_IXR_BODDET_OFFSET)
/// PM Power and Oscillators Status Register (Read/Write)
#define PM_REG_POSCSR					0x00000054
/// PM Power and Oscillators Status Register PLL0 Locked offset
#define PM_REG_POSCSR_LOCK0_OFFSET			0
/// PM Power and Oscillators Status Register PLL0 Locked size
#define PM_REG_POSCSR_LOCK0_SIZE			1
/// PM Power and Oscillators Status Register PLL0 Locked mask
#define PM_REG_POSCSR_LOCK0_MASK			(((1 << PM_REG_POSCSR_LOCK0_SIZE) - 1) << PM_REG_POSCSR_LOCK0_OFFSET)
/// PM Power and Oscillators Status Register PLL1 Locked offset
#define PM_REG_POSCSR_LOCK1_OFFSET			1
/// PM Power and Oscillators Status Register PLL1 Locked size
#define PM_REG_POSCSR_LOCK1_SIZE			1
/// PM Power and Oscillators Status Register PLL1 Locked mask
#define PM_REG_POSCSR_LOCK1_MASK			(((1 << PM_REG_POSCSR_LOCK1_SIZE) - 1) << PM_REG_POSCSR_LOCK1_OFFSET)
/// PM Power and Oscillators Status Register Clock Ready offset
#define PM_REG_POSCSR_CKRDY_OFFSET			5
/// PM Power and Oscillators Status Register Clock Ready size
#define PM_REG_POSCSR_CKRDY_SIZE			1
/// PM Power and Oscillators Status Register Clock Ready mask
#define PM_REG_POSCSR_CKRDY_MASK			(((1 << PM_REG_POSCSR_CKRDY_SIZE) - 1) << PM_REG_POSCSR_CKRDY_OFFSET)
/// PM Power and Oscillators Status Register Mask Ready offset
#define PM_REG_POSCSR_MSKRDY_OFFSET			6
/// PM Power and Oscillators Status Register Mask Ready size
#define PM_REG_POSCSR_MSKRDY_SIZE			1
/// PM Power and Oscillators Status Register Mask Ready mask
#define PM_REG_POSCSR_MSKRDY_MASK			(((1 << PM_REG_POSCSR_MSKRDY_SIZE) - 1) << PM_REG_POSCSR_MSKRDY_OFFSET)
/// PM Power and Oscillators Status Register Oscillator 0 Ready offset
#define PM_REG_POSCSR_OSC0RDY_OFFSET			7
/// PM Power and Oscillators Status Register Oscillator 0 Ready size
#define PM_REG_POSCSR_OSC0RDY_SIZE			1
/// PM Power and Oscillators Status Register Oscillator 0 Ready mask
#define PM_REG_POSCSR_OSC0RDY_MASK			(((1 << PM_REG_POSCSR_OSC0RDY_SIZE) - 1) << PM_REG_POSCSR_OSC0RDY_OFFSET)
/// PM Power and Oscillators Status Register Oscillator 1 Ready offset
#define PM_REG_POSCSR_OSC1RDY_OFFSET			8
/// PM Power and Oscillators Status Register Oscillator 1 Ready size
#define PM_REG_POSCSR_OSC1RDY_SIZE			1
/// PM Power and Oscillators Status Register Oscillator 1 Ready mask
#define PM_REG_POSCSR_OSC1RDY_MASK			(((1 << PM_REG_POSCSR_OSC1RDY_SIZE) - 1) << PM_REG_POSCSR_OSC1RDY_OFFSET)
/// PM Power and Oscillators Status Register 32 kHz Oscillator Ready offset
#define PM_REG_POSCSR_OSC32RDY_OFFSET			9
/// PM Power and Oscillators Status Register 32 kHz Oscillator Ready size
#define PM_REG_POSCSR_OSC32RDY_SIZE			1
/// PM Power and Oscillators Status Register 32 kHz Oscillator Ready mask
#define PM_REG_POSCSR_OSC32RDY_MASK			(((1 << PM_REG_POSCSR_OSC32RDY_SIZE) - 1) << PM_REG_POSCSR_OSC32RDY_OFFSET)
/// PM Power and Oscillators Status Register Brownout Detected offset
#define PM_REG_POSCSR_BODDET_OFFSET			16
/// PM Power and Oscillators Status Register Brownout Detected size
#define PM_REG_POSCSR_BODDET_SIZE			1
/// PM Power and Oscillators Status Register Brownout Detected mask
#define PM_REG_POSCSR_BODDET_MASK			(((1 << PM_REG_POSCSR_BODDET_SIZE) - 1) << PM_REG_POSCSR_BODDET_OFFSET)
/// PM Generic Clock Control GCLK0 (Read/Write)
#define PM_REG_GCCTRL_GCLK0				0x00000060
/// PM Generic Clock Control GCLK0 (Read/Write)
#define PM_REG_GCCTRL_GCLK1				0x00000064
/// PM Generic Clock Control GCLK0 (Read/Write)
#define PM_REG_GCCTRL_GCLK2				0x00000068
/// PM Generic Clock Control GCLK0 (Read/Write)
#define PM_REG_GCCTRL_GCLK3				0x0000006c
/// PM Generic Clock Control USBB (Read/Write)
#define PM_REG_GCCTRL_USBB				0x00000070
/// PM Generic Clock Control ABDAC (Read/Write)
#define PM_REG_GCCTRL_ABDAC				0x00000074
/// PM Generic Clock Control (Read/Write)
#define PM_REG_GCCTRL					0x00000060
/// PM Generic Clock Control Oscillator Select offset
#define PM_REG_GCCTRL_OSCSEL_OFFSET			0
/// PM Generic Clock Control Oscillator Select size
#define PM_REG_GCCTRL_OSCSEL_SIZE			1
/// PM Generic Clock Control Oscillator Select mask
#define PM_REG_GCCTRL_OSCSEL_MASK			(((1 << PM_REG_GCCTRL_OSCSEL_SIZE) - 1) << PM_REG_GCCTRL_OSCSEL_OFFSET)
/// PM Generic Clock Control PLL Select offset
#define PM_REG_GCCTRL_PLLSEL_OFFSET			1
/// PM Generic Clock Control PLL Select size
#define PM_REG_GCCTRL_PLLSEL_SIZE			1
/// PM Generic Clock Control PLL Select mask
#define PM_REG_GCCTRL_PLLSEL_MASK			(((1 << PM_REG_GCCTRL_PLLSEL_SIZE) - 1) << PM_REG_GCCTRL_PLLSEL_OFFSET)
/// PM Generic Clock Control Clock Enable offset
#define PM_REG_GCCTRL_CEN_OFFSET			2
/// PM Generic Clock Control Clock Enable size
#define PM_REG_GCCTRL_CEN_SIZE				1
/// PM Generic Clock Control Clock Enable mask
#define PM_REG_GCCTRL_CEN_MASK				(((1 << PM_REG_GCCTRL_CEN_SIZE) - 1) << PM_REG_GCCTRL_CEN_OFFSET)
/// PM Generic Clock Control Divide Enable offset
#define PM_REG_GCCTRL_DIVEN_OFFSET			4
/// PM Generic Clock Control Divide Enable size
#define PM_REG_GCCTRL_DIVEN_SIZE			1
/// PM Generic Clock Control Divide Enable mask
#define PM_REG_GCCTRL_DIVEN_MASK			(((1 << PM_REG_GCCTRL_DIVEN_SIZE) - 1) << PM_REG_GCCTRL_DIVEN_OFFSET)
/// PM Generic Clock Control Division Factor offset
#define PM_REG_GCCTRL_DIV_OFFSET			8
/// PM Generic Clock Control Division Factor size
#define PM_REG_GCCTRL_DIV_SIZE				8
/// PM Generic Clock Control Division Factor mask
#define PM_REG_GCCTRL_DIV_MASK				(((1 << PM_REG_GCCTRL_DIV_SIZE) - 1) << PM_REG_GCCTRL_DIV_OFFSET)
/// PM RC Oscillator Calibration Register (Read/Write)
#define PM_REG_RCCR					0x000000c0
/// PM RC Oscillator Calibration Register Calibration Value offset
#define PM_REG_RCCR_CALIB_OFFSET			0
/// PM RC Oscillator Calibration Register Calibration Value size
#define PM_REG_RCCR_CALIB_SIZE				10
/// PM RC Oscillator Calibration Register Calibration Value mask
#define PM_REG_RCCR_CALIB_MASK				(((1 << PM_REG_RCCR_CALIB_SIZE) - 1) << PM_REG_RCCR_CALIB_OFFSET)
/// PM RC Oscillator Calibration Register Flash Calibration Done offset
#define PM_REG_RCCR_FCD_OFFSET				16
/// PM RC Oscillator Calibration Register Flash Calibration Done size
#define PM_REG_RCCR_FCD_SIZE				1
/// PM RC Oscillator Calibration Register Flash Calibration Done mask
#define PM_REG_RCCR_FCD_MASK				(((1 << PM_REG_RCCR_FCD_SIZE) - 1) << PM_REG_RCCR_FCD_OFFSET)
/// PM RC Oscillator Calibration Register Register Write Protection offset
#define PM_REG_RCCR_KEY_OFFSET				24
/// PM RC Oscillator Calibration Register Register Write Protection size
#define PM_REG_RCCR_KEY_SIZE				8
/// PM RC Oscillator Calibration Register Register Write Protection mask
#define PM_REG_RCCR_KEY_MASK				(((1 << PM_REG_RCCR_KEY_SIZE) - 1) << PM_REG_RCCR_KEY_OFFSET)
/// PM Bandgap Calibration Register (Read/Write)
#define PM_REG_BGCR					0x000000c4
/// PM Bandgap Calibration Register Calibration Value offset
#define PM_REG_BGCR_CALIB_OFFSET			0
/// PM Bandgap Calibration Register Calibration Value size
#define PM_REG_BGCR_CALIB_SIZE				3
/// PM Bandgap Calibration Register Calibration Value mask
#define PM_REG_BGCR_CALIB_MASK				(((1 << PM_REG_BGCR_CALIB_SIZE) - 1) << PM_REG_BGCR_CALIB_OFFSET)
/// PM Bandgap Calibration Register Flash Calibration Done offset
#define PM_REG_BGCR_FCD_OFFSET				16
/// PM Bandgap Calibration Register Flash Calibration Done size
#define PM_REG_BGCR_FCD_SIZE				1
/// PM Bandgap Calibration Register Flash Calibration Done mask
#define PM_REG_BGCR_FCD_MASK				(((1 << PM_REG_BGCR_FCD_SIZE) - 1) << PM_REG_BGCR_FCD_OFFSET)
/// PM Bandgap Calibration Register Register Write Protection offset
#define PM_REG_BGCR_KEY_OFFSET				24
/// PM Bandgap Calibration Register Register Write Protection size
#define PM_REG_BGCR_KEY_SIZE				8
/// PM Bandgap Calibration Register Register Write Protection mask
#define PM_REG_BGCR_KEY_MASK				(((1 << PM_REG_BGCR_KEY_SIZE) - 1) << PM_REG_BGCR_KEY_OFFSET)
/// PM Linear Regulator Calibration Register (Read/Write)
#define PM_REG_VREGCR					0x000000c8
/// PM Linear Regulator Calibration Register Calibration Value offset
#define PM_REG_VREGCR_CALIB_OFFSET			0
/// PM Linear Regulator Calibration Register Calibration Value size
#define PM_REG_VREGCR_CALIB_SIZE			10
/// PM Linear Regulator Calibration Register Calibration Value mask
#define PM_REG_VREGCR_CALIB_MASK			(((1 << PM_REG_VREGCR_CALIB_SIZE) - 1) << PM_REG_VREGCR_CALIB_OFFSET)
/// PM Linear Regulator Calibration Register Flash Calibration Done offset
#define PM_REG_VREGCR_FCD_OFFSET			16
/// PM Linear Regulator Calibration Register Flash Calibration Done size
#define PM_REG_VREGCR_FCD_SIZE				1
/// PM Linear Regulator Calibration Register Flash Calibration Done mask
#define PM_REG_VREGCR_FCD_MASK				(((1 << PM_REG_VREGCR_FCD_SIZE) - 1) << PM_REG_VREGCR_FCD_OFFSET)
/// PM Linear Regulator Calibration Register Register Write Protection offset
#define PM_REG_VREGCR_KEY_OFFSET			24
/// PM Linear Regulator Calibration Register Register Write Protection size
#define PM_REG_VREGCR_KEY_SIZE				8
/// PM Linear Regulator Calibration Register Register Write Protection mask
#define PM_REG_VREGCR_KEY_MASK				(((1 << PM_REG_VREGCR_KEY_SIZE) - 1) << PM_REG_VREGCR_KEY_OFFSET)
/// PM BOD Level Register (Read/Write)
#define PM_REG_BOD					0x000000d0
/// PM BOD Level Register BOD Level offset
#define PM_REG_BOD_LEVEL_OFFSET				0
/// PM BOD Level Register BOD Level size
#define PM_REG_BOD_LEVEL_SIZE				6
/// PM BOD Level Register BOD Level mask
#define PM_REG_BOD_LEVEL_MASK				(((1 << PM_REG_BOD_LEVEL_SIZE) - 1) << PM_REG_BOD_LEVEL_OFFSET)
/// PM BOD Level Register BOD Hysteresis offset
#define PM_REG_BOD_HYST_OFFSET				6
/// PM BOD Level Register BOD Hysteresis size
#define PM_REG_BOD_HYST_SIZE				1
/// PM BOD Level Register BOD Hysteresis mask
#define PM_REG_BOD_HYST_MASK				(((1 << PM_REG_BOD_HYST_SIZE) - 1) << PM_REG_BOD_HYST_OFFSET)
/// PM BOD Level Register BOD Control offset
#define PM_REG_BOD_CTRL_OFFSET				8
/// PM BOD Level Register BOD Control size
#define PM_REG_BOD_CTRL_SIZE				2
/// PM BOD Level Register BOD Control mask
#define PM_REG_BOD_CTRL_MASK				(((1 << PM_REG_BOD_CTRL_SIZE) - 1) << PM_REG_BOD_CTRL_OFFSET)
/// PM BOD Level Register Flash Calibration Done offset
#define PM_REG_BOD_FCD_OFFSET				16
/// PM BOD Level Register Flash Calibration Done size
#define PM_REG_BOD_FCD_SIZE				1
/// PM BOD Level Register Flash Calibration Done mask
#define PM_REG_BOD_FCD_MASK				(((1 << PM_REG_BOD_FCD_SIZE) - 1) << PM_REG_BOD_FCD_OFFSET)
/// PM BOD Level Register Register Write Protection offset
#define PM_REG_BOD_KEY_OFFSET				24
/// PM BOD Level Register Register Write Protection size
#define PM_REG_BOD_KEY_SIZE				8
/// PM BOD Level Register Register Write Protection mask
#define PM_REG_BOD_KEY_MASK				(((1 << PM_REG_BOD_KEY_SIZE) - 1) << PM_REG_BOD_KEY_OFFSET)
/// PM Reset Cause Register (Read)
#define PM_REG_RCAUSE					0x00000140
/// PM Reset Cause Register Power-on Reset offset
#define PM_REG_RCAUSE_POR_OFFSET			0
/// PM Reset Cause Register Power-on Reset size
#define PM_REG_RCAUSE_POR_SIZE				1
/// PM Reset Cause Register Power-on Reset mask
#define PM_REG_RCAUSE_POR_MASK				(((1 << PM_REG_RCAUSE_POR_SIZE) - 1) << PM_REG_RCAUSE_POR_OFFSET)
/// PM Reset Cause Register Brown-out Reset offset
#define PM_REG_RCAUSE_BOD_OFFSET			1
/// PM Reset Cause Register Brown-out Reset size
#define PM_REG_RCAUSE_BOD_SIZE				1
/// PM Reset Cause Register Brown-out Reset mask
#define PM_REG_RCAUSE_BOD_MASK				(((1 << PM_REG_RCAUSE_BOD_SIZE) - 1) << PM_REG_RCAUSE_BOD_OFFSET)
/// PM Reset Cause Register External Reset Pin offset
#define PM_REG_RCAUSE_EXT_OFFSET			2
/// PM Reset Cause Register External Reset Pin size
#define PM_REG_RCAUSE_EXT_SIZE				1
/// PM Reset Cause Register External Reset Pin mask
#define PM_REG_RCAUSE_EXT_MASK				(((1 << PM_REG_RCAUSE_EXT_SIZE) - 1) << PM_REG_RCAUSE_EXT_OFFSET)
/// PM Reset Cause Register Watchdog Reset offset
#define PM_REG_RCAUSE_WDT_OFFSET			3
/// PM Reset Cause Register Watchdog Reset size
#define PM_REG_RCAUSE_WDT_SIZE				1
/// PM Reset Cause Register Watchdog Reset mask
#define PM_REG_RCAUSE_WDT_MASK				(((1 << PM_REG_RCAUSE_WDT_SIZE) - 1) << PM_REG_RCAUSE_WDT_OFFSET)
/// PM Reset Cause Register JTAG Reset offset
#define PM_REG_RCAUSE_JTAG_OFFSET			4
/// PM Reset Cause Register JTAG Reset size
#define PM_REG_RCAUSE_JTAG_SIZE				1
/// PM Reset Cause Register JTAG Reset mask
#define PM_REG_RCAUSE_JTAG_MASK				(((1 << PM_REG_RCAUSE_JTAG_SIZE) - 1) << PM_REG_RCAUSE_JTAG_OFFSET)
/// PM Reset Cause Register CPU Error offset
#define PM_REG_RCAUSE_CPUERR_OFFSET			7
/// PM Reset Cause Register CPU Error size
#define PM_REG_RCAUSE_CPUERR_SIZE			1
/// PM Reset Cause Register CPU Error mask
#define PM_REG_RCAUSE_CPUERR_MASK			(((1 << PM_REG_RCAUSE_CPUERR_SIZE) - 1) << PM_REG_RCAUSE_CPUERR_OFFSET)
/// PM Reset Cause Register OCD Reset offset
#define PM_REG_RCAUSE_OCDRST_OFFSET			8
/// PM Reset Cause Register OCD Reset size
#define PM_REG_RCAUSE_OCDRST_SIZE			1
/// PM Reset Cause Register OCD Reset mask
#define PM_REG_RCAUSE_OCDRST_MASK			(((1 << PM_REG_RCAUSE_OCDRST_SIZE) - 1) << PM_REG_RCAUSE_OCDRST_OFFSET)
/// PM Reset Cause Register JTAG Hard Reset offset
#define PM_REG_RCAUSE_JTAGHARD_OFFSET			9
/// PM Reset Cause Register JTAG Hard Reset size
#define PM_REG_RCAUSE_JTAGHARD_SIZE			1
/// PM Reset Cause Register JTAG Hard Reset mask
#define PM_REG_RCAUSE_JTAGHARD_MASK			(((1 << PM_REG_RCAUSE_JTAGHARD_SIZE) - 1) << PM_REG_RCAUSE_JTAGHARD_OFFSET)
/// PM General Purpose Low-Power Register 0 (Read/Write)
#define PM_REG_GPLP0					0x00000200
/// PM General Purpose Low-Power Register 1 (Read/Write)
#define PM_REG_GPLP1					0x00000204

typedef enum {
	PM_OSCILLATOR_OSC0				= PM_REG_MCCTRL_OSC0EN_MASK,
	PM_OSCILLATOR_OSC1				= PM_REG_MCCTRL_OSC1EN_MASK,
} pm_oscillator_t;

typedef enum {
	PM_MAIN_CLOCK_SLOW_CLOCK			= 0b00,
	PM_MAIN_CLOCK_OSC0				= 0b01,
	PM_MAIN_CLOCK_PLL0				= 0b10,
} pm_main_clock_t;

typedef enum {
	PM_PLL_PLL0					= PM_REG_PLL0,
	PM_PLL_PLL1					= PM_REG_PLL1,
} pm_pll_t;

typedef enum {
	PM_PLL_VCO_FREQUENCY_RANGE_160MHZ_TO_240MHZ		= 0,
	PM_PLL_VCO_FREQUENCY_RANGE_80MHZ_TO_180MHZ		= PM_REG_PLLX_PLLOPT0_MASK,
} pm_pll_vco_freqeuncy_range_t;

typedef enum {
	PM_PLL_OUTPUT_DIVIDER_DISABLED			= 0,
	PM_PLL_OUTPUT_DIVIDER_ENABLED			= PM_REG_PLLX_PLLOPT1_MASK,
} pm_pll_output_divider_t;

typedef enum {
	PM_PLL_WIDE_BANDWIDTH_MODE_ENABLED		= 0,
	PM_PLL_WIDE_BANDWIDTH_MODE_DISABLED		= PM_REG_PLLX_PLLOPT2_MASK,
	PM_PLL_WIDE_BANDWIDTH_MODE_DEFAULT		= PM_PLL_WIDE_BANDWIDTH_MODE_ENABLED,
} pm_pll_wide_bandwidth_mode_t;

typedef enum {
	PM_POWER_OSCILLATOR_STATUS_PLL0_LOCKED		= PM_REG_POSCSR_LOCK0_MASK,
	PM_POWER_OSCILLATOR_STATUS_PLL1_LOCKED		= PM_REG_POSCSR_LOCK1_MASK,
	PM_POWER_OSCILLATOR_STATUS_CLOCKS_READY		= PM_REG_POSCSR_CKRDY_MASK,
	PM_POWER_OSCILLATOR_STATUS_MASK_READY		= PM_REG_POSCSR_MSKRDY_MASK,
	PM_POWER_OSCILLATOR_STATUS_OSC0_READY		= PM_REG_POSCSR_OSC0RDY_MASK,
	PM_POWER_OSCILLATOR_STATUS_OSC1_READY		= PM_REG_POSCSR_OSC1RDY_MASK,
	PM_POWER_OSCILLATOR_STATUS_OSC32_READY		= PM_REG_POSCSR_OSC32RDY_MASK,
	PM_POWER_OSCILLATOR_STATUS_BROWN_OUT		= PM_REG_POSCSR_BODDET_MASK,
} pm_power_oscillator_status_t;

typedef enum {
	PM_GENERIC_CLOCK_GCLK0				= PM_REG_GCCTRL_GCLK0,
	PM_GENERIC_CLOCK_GCLK1				= PM_REG_GCCTRL_GCLK1,
	PM_GENERIC_CLOCK_GCLK2				= PM_REG_GCCTRL_GCLK2,
	PM_GENERIC_CLOCK_GCLK3				= PM_REG_GCCTRL_GCLK3,
	PM_GENERIC_CLOCK_USBB				= PM_REG_GCCTRL_USBB,
	PM_GENERIC_CLOCK_ABDAC				= PM_REG_GCCTRL_ABDAC,
} pm_generic_clock_t;

typedef enum {
	PM_GENERIC_CLOCK_SOURCE_OSC0			= 0,
	PM_GENERIC_CLOCK_SOURCE_OSC1			= PM_REG_GCCTRL_OSCSEL_MASK,
	PM_GENERIC_CLOCK_SOURCE_PLL0			= PM_REG_GCCTRL_PLLSEL_MASK,
	PM_GENERIC_CLOCK_SOURCE_PLL1			= PM_REG_GCCTRL_OSCSEL_MASK | PM_REG_GCCTRL_PLLSEL_MASK,
} pm_generic_clock_source_t;

typedef enum {
	PM_GENERIC_CLOCK_DIVIDER_ENABLED		= PM_REG_GCCTRL_DIVEN_MASK,
	PM_GENERIC_CLOCK_DIVIDER_DISABLED		= 0,
} pm_generic_clock_divider_t;

extern uint32_t pm_read_register(uint32_t const offset);
extern void pm_write_register(uint32_t const offset, uint32_t const value);
extern void pm_write_register_field(uint32_t const offset, uint32_t const field_mask, uint32_t const value);

extern void pm_enable_oscillator(pm_oscillator_t const oscillator);
extern void pm_disable_oscillator(pm_oscillator_t const oscillator);

extern void pm_enable_pll(pm_pll_t const pll);
extern void pm_disable_pll(pm_pll_t const pll);
extern void pm_configure_pll(pm_pll_t const pll, pm_pll_vco_freqeuncy_range_t const frequency_range, pm_pll_output_divider_t const output_divider, pm_pll_wide_bandwidth_mode_t const wide_bandwidth_mode, uint8_t const multiplier, uint8_t const divider, uint8_t const lock_delay);
extern void pm_configure_and_enable_pll(pm_pll_t const pll, pm_pll_vco_freqeuncy_range_t const frequency_range, pm_pll_output_divider_t const output_divider, pm_pll_wide_bandwidth_mode_t const wide_bandwidth_mode, uint8_t const multiplier, uint8_t const divider, uint8_t const lock_delay);

extern void pm_enable_osc0(void) GCC_DEPRECATED;
extern void pm_disable_osc0(void) GCC_DEPRECATED;
extern void pm_enable_osc1(void) GCC_DEPRECATED;
extern void pm_disable_osc1(void) GCC_DEPRECATED;
extern void pm_switch_main_clock(pm_main_clock_t const clock);

extern void pm_wait(pm_power_oscillator_status_t status);

extern void pm_generic_clock_enable(pm_generic_clock_t const generic_clock);
extern void pm_generic_clock_disable(pm_generic_clock_t const generic_clock);
extern void pm_generic_clock_configure(pm_generic_clock_t const generic_clock, pm_generic_clock_source_t const generic_clock_source, pm_generic_clock_divider_t const divider, uint32_t const divider_value);

#endif /* PM_H_ */
