/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef PWM_H_
#define PWM_H_

#include <at32uc3/inttypes.h>

/// PWM address
#define PWM_ADDRESS					0xffff3000
/// PWM channel address
#define PWM_CHANNEL_ADDRESS				(PWM_ADDRESS + 0x00000200)
/// PWM channel width
#define PWM_CHANNEL_WIDTH				0x00000020

#define PWM_CHANNEL(n)					((void volatile *)PWM_CHANNEL_ADDRESS + PWM_CHANNEL_WIDTH * (n))

/// PWM Mode Register (Read/Write)
#define PWM_REG_MR					0x00000000
/// PWM Mode Register CLKA Divide Factor offset
#define PWM_REG_MR_DIVA_OFFSET				0
/// PWM Mode Register CLKA Divide Factor size
#define PWM_REG_MR_DIVA_SIZE				8
/// PWM Mode Register CLKA Divide Factor mask
#define PWM_REG_MR_DIVA_MASK				(((1 << PWM_REG_MR_DIVA_SIZE) - 1) << PWM_REG_MR_DIVA_OFFSET)
/// PWM Mode Register CLKA Prescaler Ratio offset
#define PWM_REG_MR_PREA_OFFSET				8
/// PWM Mode Register CLKA Prescaler Ratio size
#define PWM_REG_MR_PREA_SIZE				4
/// PWM Mode Register CLKA Prescaler Ratio mask
#define PWM_REG_MR_PREA_MASK				(((1 << PWM_REG_MR_PREA_SIZE) - 1) << PWM_REG_MR_PREA_OFFSET)
/// PWM Mode Register CLKB Divide Factor offset
#define PWM_REG_MR_DIVB_OFFSET				16
/// PWM Mode Register CLKB Divide Factor size
#define PWM_REG_MR_DIVB_SIZE				8
/// PWM Mode Register CLKB Divide Factor mask
#define PWM_REG_MR_DIVB_MASK				(((1 << PWM_REG_MR_DIVB_SIZE) - 1) << PWM_REG_MR_DIVB_OFFSET)
/// PWM Mode Register CLKB Prescaler Ratio offset
#define PWM_REG_MR_PREB_OFFSET				24
/// PWM Mode Register CLKB Prescaler Ratio size
#define PWM_REG_MR_PREB_SIZE				4
/// PWM Mode Register CLKB Prescaler Ratio mask
#define PWM_REG_MR_PREB_MASK				(((1 << PWM_REG_MR_PREB_SIZE) - 1) << PWM_REG_MR_PREB_OFFSET)
/// PWM Enable Register (Write)
#define PWM_REG_ENA					0x00000004
/// PWM Disable Register (Write)
#define PWM_REG_DIS					0x00000008
/// PWM Status Register (Read)
#define PWM_REG_SR					0x0000000c
/// PWM Interrupt Enable Register (Write)
#define PWM_REG_IER					0x00000010
/// PWM Interrupt Disable Register (Write)
#define PWM_REG_IDR					0x00000014
/// PWM Interrupt Mask Register (Read)
#define PWM_REG_IMR					0x00000018
/// PWM Interrupt Status Register (Read)
#define PWM_REG_ISR					0x0000001c
/// PWM Channel Mode Register (Read/Write)
#define PWM_CHANNEL_REG_CMR				0x00000000
/// PWM Channel Mode Register Channel Prescaler offset
#define PWM_CHANNEL_REG_CMR_CPRE_OFFSET			0
/// PWM Channel Mode Register Channel Prescaler size
#define PWM_CHANNEL_REG_CMR_CPRE_SIZE			4
/// PWM Channel Mode Register Channel Prescaler mask
#define PWM_CHANNEL_REG_CMR_CPRE_MASK			(((1 << PWM_CHANNEL_REG_CMR_CPRE_SIZE) - 1) << PWM_CHANNEL_REG_CMR_CPRE_OFFSET)
/// PWM Channel Mode Register Channel Alignment offset
#define PWM_CHANNEL_REG_CMR_CALG_OFFSET			8
/// PWM Channel Mode Register Channel Alignment size
#define PWM_CHANNEL_REG_CMR_CALG_SIZE			1
/// PWM Channel Mode Register Channel Alignment mask
#define PWM_CHANNEL_REG_CMR_CALG_MASK			(((1 << PWM_CHANNEL_REG_CMR_CALG_SIZE) - 1) << PWM_CHANNEL_REG_CMR_CALG_OFFSET)
/// PWM Channel Mode Register Channel Polarity offset
#define PWM_CHANNEL_REG_CMR_CPOL_OFFSET			9
/// PWM Channel Mode Register Channel Polarity size
#define PWM_CHANNEL_REG_CMR_CPOL_SIZE			1
/// PWM Channel Mode Register Channel Polarity mask
#define PWM_CHANNEL_REG_CMR_CPOL_MASK			(((1 << PWM_CHANNEL_REG_CMR_CPOL_SIZE) - 1) << PWM_CHANNEL_REG_CMR_CPOL_OFFSET)
/// PWM Channel Mode Register Channel Update Period offset
#define PWM_CHANNEL_REG_CMR_CPD_OFFSET			10
/// PWM Channel Mode Register Channel Update Period size
#define PWM_CHANNEL_REG_CMR_CPD_SIZE			1
/// PWM Channel Mode Register Channel Update Period mask
#define PWM_CHANNEL_REG_CMR_CPD_MASK			(((1 << PWM_CHANNEL_REG_CMR_CPD_SIZE) - 1) << PWM_CHANNEL_REG_CMR_CPD_OFFSET)
/// PWM Channel Duty Cycle Register (Read/Write)
#define PWM_CHANNEL_REG_CDTY				0x00000004
/// PWM Channel Duty Cycle Register Channel Duty Cycle offset
#define PWM_CHANNEL_REG_CDTY_CDTY_OFFSET		0
/// PWM Channel Duty Cycle Register Channel Duty Cycle size
#define PWM_CHANNEL_REG_CDTY_CDTY_SIZE			32
/// PWM Channel Duty Cycle Register Channel Duty Cycle mask
#define PWM_CHANNEL_REG_CDTY_CDTY_MASK			(((1 << PWM_CHANNEL_REG_CDTY_CDTY_SIZE) - 1) << PWM_CHANNEL_REG_CDTY_CDTY_OFFSET)
/// PWM Channel Period Register (Read/Write)
#define PWM_CHANNEL_REG_CPRD				0x00000008
/// PWM Channel Period Register Channel Period offset
#define PWM_CHANNEL_REG_CPRD_CPRD_OFFSET		0
/// PWM Channel Period Register Channel Period size
#define PWM_CHANNEL_REG_CPRD_CPRD_SIZE			32
/// PWM Channel Period Register Channel Period mask
#define PWM_CHANNEL_REG_CPRD_CPRD_MASK			(((1 << PWM_CHANNEL_REG_CPRD_CPRD_SIZE) - 1) << PWM_CHANNEL_REG_CPRD_CPRD_OFFSET)
/// PWM Channel Counter Register (Read)
#define PWM_CHANNEL_REG_CCNT				0x0000000c
/// PWM Channel Counter Register Channel Counter Value offset
#define PWM_CHANNEL_REG_CCNT_CNT_OFFSET			0
/// PWM Channel Counter Register Channel Counter Value size
#define PWM_CHANNEL_REG_CCNT_CNT_SIZE			32
/// PWM Channel Counter Register Channel Counter Value mask
#define PWM_CHANNEL_REG_CCNT_CNT_MASK			(((1 << PWM_CHANNEL_REG_CCNT_CNT_SIZE) - 1) << PWM_CHANNEL_REG_CCNT_CNT_OFFSET)
/// PWM Channel Update Register (Write)
#define PWM_CHANNEL_REG_CUPD				0x00000010
/// PWM Channel Update Register Channel Update Value offset
#define PWM_CHANNEL_REG_CUPD_CUPD_OFFSET		0
/// PWM Channel Update Register Channel Update Value size
#define PWM_CHANNEL_REG_CUPD_CUPD_SIZE			32
/// PWM Channel Update Register Channel Update Value mask
#define PWM_CHANNEL_REG_CUPD_CUPD_MASK			(((1 << PWM_CHANNEL_REG_CUPD_CUPD_SIZE) - 1) << PWM_CHANNEL_REG_CUPD_CUPD_OFFSET)

/// PWM clock
typedef enum {
	PWM_CLOCK_MCK					= 0b0000,
	PWM_CLOCK_MCK_BY_2				= 0b0001,
	PWM_CLOCK_MCK_BY_4				= 0b0010,
	PWM_CLOCK_MCK_BY_8				= 0b0011,
	PWM_CLOCK_MCK_BY_16				= 0b0100,
	PWM_CLOCK_MCK_BY_32				= 0b0101,
	PWM_CLOCK_MCK_BY_64				= 0b0110,
	PWM_CLOCK_MCK_BY_128				= 0b0111,
	PWM_CLOCK_MCK_BY_256				= 0b1000,
	PWM_CLOCK_MCK_BY_512				= 0b1001,
	PWM_CLOCK_MCK_BY_1024				= 0b1010,
} pwm_clock_t;

/// PWM channel clock
typedef enum {
	PWM_CHANNEL_CLOCK_MCK				= (0b0000 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_2			= (0b0001 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_4			= (0b0010 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_8			= (0b0011 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_16			= (0b0100 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_32			= (0b0101 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_64			= (0b0110 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_128			= (0b0111 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_256			= (0b1000 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_512			= (0b1001 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_MCK_BY_1024			= (0b1010 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_CLKA				= (0b1011 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
	PWM_CHANNEL_CLOCK_CLKB				= (0b1100 << PWM_CHANNEL_REG_CMR_CPRE_OFFSET),
} pwm_channel_clock_t;

typedef enum {
	PWM_CHANNEL_ALIGNMENT_LEFT			= 0,
	PWM_CHANNEL_ALIGNMENT_CENTER			= PWM_CHANNEL_REG_CMR_CALG_MASK,
} pwm_channel_alignment_t;

typedef enum {
	PWM_CHANNEL_POLARITY_LOW			= 0,
	PWM_CHANNEL_POLARITY_HIGH			= PWM_CHANNEL_REG_CMR_CPOL_MASK,
} pwm_channel_polarity_t;

typedef enum {
	PWM_CHANNEL_UPDATE_MODE_DUTY_CYCLE		= 0,
	PWM_CHANNEL_UPDATE_MODE_PERIOD			= PWM_CHANNEL_REG_CMR_CPD_MASK,
} pwm_channel_update_mode_t;

extern uint32_t pwm_read_register(uint32_t const offset);
extern void pwm_write_register(uint32_t const offset, uint32_t const value);

extern void pwm_configure_clock_a(pwm_clock_t const prea, uint8_t const diva);
extern void pwm_configure_clock_b(pwm_clock_t const preb, uint8_t const divb);
extern void pwm_configure_clocks(pwm_clock_t const prea, uint8_t const diva, pwm_clock_t const preb, uint8_t const divb);

extern void pwm_enable_channel(uint8_t const channel_no);
extern void pwm_disable_channel(uint8_t const channel_no);

extern uint32_t pwm_channel_read_register(void volatile *channel, const uint32_t offset);
extern void pwm_channel_write_register(void volatile *channel, uint32_t const offset, uint32_t const value);
extern void pwm_channel_write_register_field(void volatile *channel, uint32_t const offset, uint32_t const field_mask, uint32_t const value);

extern void pwm_channel_set_clock(void volatile *channel, pwm_channel_clock_t const clock);
extern void pwm_channel_set_alignment(void volatile *channel, pwm_channel_alignment_t const alignment);
extern void pwm_channel_set_polarity(void volatile *channel, pwm_channel_polarity_t const polarity);
extern void pwm_channel_set_update_mode(void volatile *channel, pwm_channel_update_mode_t const update_mode);
extern void pwm_channel_set_period(void volatile *channel, uint32_t const period);
extern void pwm_channel_set_duty_cycle(void volatile *channel, uint32_t const duty_cycle);
extern void pwm_channel_configure(void volatile *channel, pwm_channel_clock_t const clock, pwm_channel_alignment_t const alignment, pwm_channel_polarity_t const polarity, pwm_channel_update_mode_t const update_mode);
extern void pwm_channel_configure2(void volatile *channel, pwm_channel_clock_t const clock, pwm_channel_alignment_t const alignment, pwm_channel_polarity_t const polarity, pwm_channel_update_mode_t const update_mode, uint32_t const period, uint32_t const duty_cycle);
extern void pwm_channel_update(void volatile *channel, uint32_t const value);

#endif /* PWM_H_ */

