/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef SYSREG_H_
#define SYSREG_H_

#include <at32uc3/gccmacros.h>
#include <at32uc3/inttypes.h>

/// Status Register number
#define SYSREG_SR_NO					0
/// Status Register address
#define SYSREG_SR_ADDR					(SYSREG_SR_NO << 2)
/// Status Register
#define SYSREG_SR_GM_OFFSET				16
#define SYSREG_SR_GM_SIZE				1
#define SYSREG_SR_GM_MASK				(((1 << SYSREG_SR_GM_SIZE) - 1) << SYSREG_SR_GM_OFFSET)
#define SYSREG_SR_I0M_OFFSET				17
#define SYSREG_SR_I0M_SIZE				1
#define SYSREG_SR_I0M_MASK				(((1 << SYSREG_SR_I0M_SIZE) - 1) << SYSREG_SR_I0M_OFFSET)
#define SYSREG_SR_I1M_OFFSET				18
#define SYSREG_SR_I1M_SIZE				1
#define SYSREG_SR_I1M_MASK				(((1 << SYSREG_SR_I1M_SIZE) - 1) << SYSREG_SR_I1M_OFFSET)
#define SYSREG_SR_I2M_OFFSET				19
#define SYSREG_SR_I2M_SIZE				1
#define SYSREG_SR_I2M_MASK				(((1 << SYSREG_SR_I2M_SIZE) - 1) << SYSREG_SR_I2M_OFFSET)
#define SYSREG_SR_I3M_OFFSET				20
#define SYSREG_SR_I3M_SIZE				1
#define SYSREG_SR_I3M_MASK				(((1 << SYSREG_SR_I3M_SIZE) - 1) << SYSREG_SR_I3M_OFFSET)
#define SYSREG_SR_EM_OFFSET				21
#define SYSREG_SR_EM_SIZE				1
#define SYSREG_SR_EM_MASK				(((1 << SYSREG_SR_EM_SIZE) - 1) << SYSREG_SR_EM_OFFSET)
/// Exception Vector Base Address Register number
#define SYSREG_EVBA_NO					1
/// Exception Vector Base Address Register address
#define SYSREG_EVBA_ADDR				(SYSREG_EVBA_NO << 2)
/// Application Call Base Address Register number
#define SYSREG_ACBA_NO					2
/// Application Call Base Address Register address
#define SYSREG_ACBA_ADDR				(SYSREG_ACBA_NO << 2)
/// CPU Control Register number
#define SYSREG_CPUCR_NO					3
/// CPU Control Register address
#define SYSREG_CPUCR_ADDR				(SYSREG_CPUCR_NO << 2)
/// Exception Cause Register number
#define SYSREG_ECR_NO					4
/// Exception Cause Register address
#define SYSREG_ECR_ADDR					(SYSREG_ECR_NO << 2)
/// Return Status Register for Debug Mode number
#define SYSREG_RSR_DBG_NO				12
/// Return Status Register for Debug Mode address
#define SYSREG_RSR_DBG_ADDR				(SYSREG_RSR_DBG_NO << 2)
/// Return Address Register for Debug Mode number
#define SYSREG_RAR_DBG_NO				20
/// Return Address Register for Debug Mode address
#define SYSREG_RAR_DBG_ADDR				(SYSREG_RAR_DBG_NO << 2)
/// Configuration Register 0 number
#define SYSREG_CONFIG0_NO				64
/// Configuration Register 0 address
#define SYSREG_CONFIG0_ADDR				(SYSREG_CONFIG0_NO << 2)
/// Configuration Register 1 number
#define SYSREG_CONFIG1_NO				65
/// Configuration Register 1 address
#define SYSREG_CONFIG1_ADDR				(SYSREG_CONFIG1_NO << 2)
/// Cycle Counter Register number
#define SYSREG_COUNT_NO					66
/// Cycle Counter Register address
#define SYSREG_COUNT_ADDR				(SYSREG_COUNT_NO << 2)
/// Compare Register number
#define SYSREG_COMPARE_NO				67
/// Compare Register address
#define SYSREG_COMPARE_ADDR				(SYSREG_COMPARE_NO << 2)
/// Bus Error Address Register number
#define SYSREG_BEAR_NO					79
/// Bus Error Address Register address
#define SYSREG_BEAR_ADDR				(SYSREG_BEAR_NO << 2)
/// MPU Address Register Region 0 number
#define SYSREG_MPUAR0_NO				80
/// MPU Address Register Region 0 address
#define SYSREG_MPUAR0_ADDR				(SYSREG_MPUAR0_NO << 2)
/// MPU Address Register Region 1 number
#define SYSREG_MPUAR1_NO				81
/// MPU Address Register Region 1 address
#define SYSREG_MPUAR1_ADDR				(SYSREG_MPUAR1_NO << 2)
/// MPU Address Register Region 2 number
#define SYSREG_MPUAR2_NO				82
/// MPU Address Register Region 2 address
#define SYSREG_MPUAR2_ADDR				(SYSREG_MPUAR2_NO << 2)
/// MPU Address Register Region 3 number
#define SYSREG_MPUAR3_NO				83
/// MPU Address Register Region 3 address
#define SYSREG_MPUAR3_ADDR				(SYSREG_MPUAR3_NO << 2)
/// MPU Address Register Region 4 number
#define SYSREG_MPUAR4_NO				84
/// MPU Address Register Region 4 address
#define SYSREG_MPUAR4_ADDR				(SYSREG_MPUAR4_NO << 2)
/// MPU Address Register Region 5 number
#define SYSREG_MPUAR5_NO				85
/// MPU Address Register Region 5 address
#define SYSREG_MPUAR5_ADDR				(SYSREG_MPUAR5_NO << 2)
/// MPU Address Register Region 6 number
#define SYSREG_MPUAR6_NO				86
/// MPU Address Register Region 6 address
#define SYSREG_MPUAR6_ADDR				(SYSREG_MPUAR6_NO << 2)
/// MPU Address Register Region 7 number
#define SYSREG_MPUAR7_NO				87
/// MPU Address Register Region 7 address
#define SYSREG_MPUAR7_ADDR				(SYSREG_MPUAR7_NO << 2)
/// MPU Privilege Select Register Region 0 number
#define SYSREG_MPUPSR0_NO				88
/// MPU Privilege Select Register Region 0 address
#define SYSREG_MPUPSR0_ADDR				(SYSREG_MPUPSR0_NO << 2)
/// MPU Privilege Select Register Region 1 number
#define SYSREG_MPUPSR1_NO				89
/// MPU Privilege Select Register Region 1 address
#define SYSREG_MPUPSR1_ADDR				(SYSREG_MPUPSR1_NO << 2)
/// MPU Privilege Select Register Region 2 number
#define SYSREG_MPUPSR2_NO				90
/// MPU Privilege Select Register Region 2 address
#define SYSREG_MPUPSR2_ADDR				(SYSREG_MPUPSR2_NO << 2)
/// MPU Privilege Select Register Region 3 number
#define SYSREG_MPUPSR3_NO				91
/// MPU Privilege Select Register Region 3 address
#define SYSREG_MPUPSR3_ADDR				(SYSREG_MPUPSR3_NO << 2)
/// MPU Privilege Select Register Region 4 number
#define SYSREG_MPUPSR4_NO				92
/// MPU Privilege Select Register Region 4 address
#define SYSREG_MPUPSR4_ADDR				(SYSREG_MPUPSR4_NO << 2)
/// MPU Privilege Select Register Region 5 number
#define SYSREG_MPUPSR5_NO				93
/// MPU Privilege Select Register Region 5 address
#define SYSREG_MPUPSR5_ADDR				(SYSREG_MPUPSR5_NO << 2)
/// MPU Privilege Select Register Region 6 number
#define SYSREG_MPUPSR6_NO				94
/// MPU Privilege Select Register Region 6 address
#define SYSREG_MPUPSR6_ADDR				(SYSREG_MPUPSR6_NO << 2)
/// MPU Privilege Select Register Region 7 number
#define SYSREG_MPUPSR7_NO				95
/// MPU Privilege Select Register Region 7 address
#define SYSREG_MPUPSR7_ADDR				(SYSREG_MPUPSR7_NO << 2)
/// MPU Access Permission Register A number
#define SYSREG_MPUAPRA_NO				100
/// MPU Access Permission Register A address
#define SYSREG_MPUAPRA_ADDR				(SYSREG_MPUAPRA_NO << 2)
/// MPU Access Permission Register B number
#define SYSREG_MPUAPRB_NO				101
/// MPU Access Permission Register B address
#define SYSREG_MPUAPRB_ADDR				(SYSREG_MPUAPRB_NO << 2)
/// MPU Control Register number
#define SYSREG_MPUCR_NO					102
/// MPU Control Register address
#define SYSREG_MPUCR_ADDR				(SYSREG_MPUCR_NO << 2)

// If you're easily offended by hacky code, you may want to turn away.
extern uint32_t sysreg_read(uint8_t const sysreg_no) GCC_SECTION(".data,\"aw\",@progbits #");
extern void sysreg_write(uint8_t const sysreg_no, uint32_t const value) GCC_SECTION(".data,\"aw\",@progbits #");
extern void sysreg_set(uint8_t const sysreg_no, uint32_t const mask);
extern void sysreg_clear(uint8_t const sysreg_no, uint32_t const mask);

#endif /* SYSREG_H_ */

