/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <inttypes.h>

#ifndef TC_H_
#define TC_H_

#include <at32uc3/inttypes.h>

/// TC address
#define TC_ADDRESS					0xffff3800
/// TC channel address
#define TC_CHANNEL_ADDRESS				(TC_ADDRESS + 0x00000000)
/// TC channel width
#define TC_CHANNEL_WIDTH				0x00000040

#define TC_CHANNEL(n)					((uint32_t volatile *)(TC_CHANNEL_ADDRESS + TC_CHANNEL_WIDTH * (n)))

/// TC Block Control Register (Write)
#define TC_REG_BCR					0x000000c0
/// TC Block Control Register Synchro Command offset
#define TC_REG_BCR_SYNC_OFFSET				0
/// TC Block Control Register Synchro Command size
#define TC_REG_BCR_SYNC_SIZE				1
/// TC Block Control Register Synchro Command mask
#define TC_REG_BCR_SYNC_MASK				(((1 << TC_REG_BCR_SYNC_SIZE) - 1) << TC_REG_BCR_SYNC_OFFSET)
/// TC Block Mode Register (Read/Write)
#define TC_REG_BMR					0x000000c4
/// TC Block Mode Register External Clock Signal 0 Selection offset
#define TC_REG_BMR_TC0XC0S_OFFSET			0
/// TC Block Mode Register External Clock Signal 0 Selection size
#define TC_REG_BMR_TC0XC0S_SIZE				2
/// TC Block Mode Register External Clock Signal 0 Selection mask
#define TC_REG_BMR_TC0XC0S_MASK				(((1 << TC_REG_BMR_TC0XC0S_SIZE) - 1) << TC_REG_BMR_TC0XC0S_OFFSET)
/// TC Block Mode Register External Clock Signal 1 Selection offset
#define TC_REG_BMR_TC1XC1S_OFFSET			2
/// TC Block Mode Register External Clock Signal 1 Selection size
#define TC_REG_BMR_TC1XC1S_SIZE				2
/// TC Block Mode Register External Clock Signal 1 Selection mask
#define TC_REG_BMR_TC1XC1S_MASK				(((1 << TC_REG_BMR_TC1XC1S_SIZE) - 1) << TC_REG_BMR_TC1XC1S_OFFSET)
/// TC Block Mode Register External Clock Signal 2 Selection offset
#define TC_REG_BMR_TC2XC2S_OFFSET			4
/// TC Block Mode Register External Clock Signal 2 Selection size
#define TC_REG_BMR_TC2XC2S_SIZE				2
/// TC Block Mode Register External Clock Signal 2 Selection mask
#define TC_REG_BMR_TC2XC2S_MASK				(((1 << TC_REG_BMR_TC2XC2S_SIZE) - 1) << TC_REG_BMR_TC2XC2S_OFFSET)
/// TC Channel Control Register (Write)
#define TC_CHANNEL_REG_CCR				0x00000000
/// TC Channel Control Register Counter Clock Enable Command offset
#define TC_CHANNEL_REG_CCR_CLKEN_OFFSET			0
/// TC Channel Control Register Counter Clock Enable Command size
#define TC_CHANNEL_REG_CCR_CLKEN_SIZE			1
/// TC Channel Control Register Counter Clock Enable Command mask
#define TC_CHANNEL_REG_CCR_CLKEN_MASK			(((1 << TC_CHANNEL_REG_CCR_CLKEN_SIZE) - 1) << TC_CHANNEL_REG_CCR_CLKEN_OFFSET)
/// TC Channel Control Register Counter Clock Disable Command offset
#define TC_CHANNEL_REG_CCR_CLKDIS_OFFSET		1
/// TC Channel Control Register Counter Clock Disable Command size
#define TC_CHANNEL_REG_CCR_CLKDIS_SIZE			1
/// TC Channel Control Register Counter Clock Disable Command mask
#define TC_CHANNEL_REG_CCR_CLKDIS_MASK			(((1 << TC_CHANNEL_REG_CCR_CLKDIS_SIZE) - 1) << TC_CHANNEL_REG_CCR_CLKDIS_OFFSET)
/// TC Channel Control Register Software Trigger Command offset
#define TC_CHANNEL_REG_CCR_SWTRG_OFFSET			2
/// TC Channel Control Register Software Trigger Command size
#define TC_CHANNEL_REG_CCR_SWTRG_SIZE			1
/// TC Channel Control Register Software Trigger Command mask
#define TC_CHANNEL_REG_CCR_SWTRG_MASK			(((1 << TC_CHANNEL_REG_CCR_SWTRG_SIZE) - 1) << TC_CHANNEL_REG_CCR_SWTRG_OFFSET)
/// TC Channel Mode Register (Read/Write)
#define TC_CHANNEL_REG_CMR				0x00000004
/// TC Channel Mode Register Wave Mode offset
#define TC_CHANNEL_REG_CMR_WAVE_OFFSET			15
/// TC Channel Mode Register Wave Mode size
#define TC_CHANNEL_REG_CMR_WAVE_SIZE			1
/// TC Channel Mode Register Wave Mode mask
#define TC_CHANNEL_REG_CMR_WAVE_MASK			(((1 << TC_CHANNEL_REG_CMR_WAVE_SIZE) - 1) << TC_CHANNEL_REG_CMR_WAVE_OFFSET)
/// TC Channel Mode Register (Capture Mode) Clock Selection offset
#define TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET		0
/// TC Channel Mode Register (Capture Mode) Clock Selection size
#define TC_CHANNEL_REG_CMR_CM_TCCLKS_SIZE		3
/// TC Channel Mode Register (Capture Mode) Clock Selection mask
#define TC_CHANNEL_REG_CMR_CM_TCCLKS_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_TCCLKS_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET)
/// TC Channel Mode Register (Capture Mode) Clock Invert offset
#define TC_CHANNEL_REG_CMR_CM_CLKI_OFFSET		3
/// TC Channel Mode Register (Capture Mode) Clock Invert size
#define TC_CHANNEL_REG_CMR_CM_CLKI_SIZE			1
/// TC Channel Mode Register (Capture Mode) Clock Invert mask
#define TC_CHANNEL_REG_CMR_CM_CLKI_MASK			(((1 << TC_CHANNEL_REG_CMR_CM_CLKI_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_CLKI_OFFSET)
/// TC Channel Mode Register (Capture Mode) Burst Signal Selection offset
#define TC_CHANNEL_REG_CMR_CM_BURST_OFFSET		4
/// TC Channel Mode Register (Capture Mode) Burst Signal Selection size
#define TC_CHANNEL_REG_CMR_CM_BURST_SIZE		2
/// TC Channel Mode Register (Capture Mode) Burst Signal Selection mask
#define TC_CHANNEL_REG_CMR_CM_BURST_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_BURST_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_BURST_OFFSET)
/// TC Channel Mode Register (Capture Mode) Counter Clock Stopped with RB Loading offset
#define TC_CHANNEL_REG_CMR_CM_LDBSTOP_OFFSET		6
/// TC Channel Mode Register (Capture Mode) Counter Clock Stopped with RB Loading size
#define TC_CHANNEL_REG_CMR_CM_LDBSTOP_SIZE		1
/// TC Channel Mode Register (Capture Mode) Counter Clock Stopped with RB Loading mask
#define TC_CHANNEL_REG_CMR_CM_LDBSTOP_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_LDBSTOP_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_LDBSTOP_OFFSET)
/// TC Channel Mode Register (Capture Mode) Counter Clock Disable with RB Loading offset
#define TC_CHANNEL_REG_CMR_CM_LDBDIS_OFFSET		7
/// TC Channel Mode Register (Capture Mode) Counter Clock Disable with RB Loading size
#define TC_CHANNEL_REG_CMR_CM_LDBDIS_SIZE		1
/// TC Channel Mode Register (Capture Mode) Counter Clock Disable with RB Loading mask
#define TC_CHANNEL_REG_CMR_CM_LDBDIS_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_LDBDIS_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_LDBDIS_OFFSET)
/// TC Channel Mode Register (Capture Mode) External Trigger Edge Selection offset
#define TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET		8
/// TC Channel Mode Register (Capture Mode) External Trigger Edge Selection size
#define TC_CHANNEL_REG_CMR_CM_ETRGEDG_SIZE		2
/// TC Channel Mode Register (Capture Mode) External Trigger Edge Selection mask
#define TC_CHANNEL_REG_CMR_CM_ETRGEDG_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_ETRGEDG_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET)
/// TC Channel Mode Register (Capture Mode) TIOA or TIOB External Trigger offset
#define TC_CHANNEL_REG_CMR_CM_ABETRG_OFFSET		10
/// TC Channel Mode Register (Capture Mode) TIOA or TIOB External Trigger size
#define TC_CHANNEL_REG_CMR_CM_ABETRG_SIZE		1
/// TC Channel Mode Register (Capture Mode) TIOA or TIOB External Trigger mask
#define TC_CHANNEL_REG_CMR_CM_ABETRG_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_ABETRG_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_ABETRG_OFFSET)
/// TC Channel Mode Register (Capture Mode) RC Compare Trigger Enable offset
#define TC_CHANNEL_REG_CMR_CM_CPCTRG_OFFSET		14
/// TC Channel Mode Register (Capture Mode) RC Compare Trigger Enable size
#define TC_CHANNEL_REG_CMR_CM_CPCTRG_SIZE		1
/// TC Channel Mode Register (Capture Mode) RC Compare Trigger Enable mask
#define TC_CHANNEL_REG_CMR_CM_CPCTRG_MASK		(((1 << TC_CHANNEL_REG_CMR_CM_CPCTRG_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_CPCTRG_OFFSET)
/// TC Channel Mode Register (Capture Mode) RA Loading Selection offset
#define TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET		16
/// TC Channel Mode Register (Capture Mode) RA Loading Selection size
#define TC_CHANNEL_REG_CMR_CM_LDRA_SIZE			2
/// TC Channel Mode Register (Capture Mode) RA Loading Selection mask
#define TC_CHANNEL_REG_CMR_CM_LDRA_MASK			(((1 << TC_CHANNEL_REG_CMR_CM_LDRA_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET)
/// TC Channel Mode Register (Capture Mode) RB Loading Selection offset
#define TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET		18
/// TC Channel Mode Register (Capture Mode) RB Loading Selection size
#define TC_CHANNEL_REG_CMR_CM_LDRB_SIZE			2
/// TC Channel Mode Register (Capture Mode) RB Loading Selection mask
#define TC_CHANNEL_REG_CMR_CM_LDRB_MASK			(((1 << TC_CHANNEL_REG_CMR_CM_LDRB_SIZE) - 1) << TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Clock Selection offset
#define TC_CHANNEL_REG_CMR_WM_TCCLKS_OFFSET		0
/// TC Channel Mode Register (Waveform Mode) Clock Selection size
#define TC_CHANNEL_REG_CMR_WM_TCCLKS_SIZE		3
/// TC Channel Mode Register (Waveform Mode) Clock Selection mask
#define TC_CHANNEL_REG_CMR_WM_TCCLKS_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_TCCLKS_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_TCCLKS_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Clock Invert offset
#define TC_CHANNEL_REG_CMR_WM_CLKI_OFFSET		3
/// TC Channel Mode Register (Waveform Mode) Clock Invert size
#define TC_CHANNEL_REG_CMR_WM_CLKI_SIZE			1
/// TC Channel Mode Register (Waveform Mode) Clock Invert mask
#define TC_CHANNEL_REG_CMR_WM_CLKI_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_CLKI_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_CLKI_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Burst Signal Selection offset
#define TC_CHANNEL_REG_CMR_WM_BURST_OFFSET		4
/// TC Channel Mode Register (Waveform Mode) Burst Signal Selection size
#define TC_CHANNEL_REG_CMR_WM_BURST_SIZE		2
/// TC Channel Mode Register (Waveform Mode) Burst Signal Selection mask
#define TC_CHANNEL_REG_CMR_WM_BURST_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_BURST_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_BURST_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Counter Clock Stopped with RC Compare offset
#define TC_CHANNEL_REG_CMR_WM_CPCSTOP_OFFSET		6
/// TC Channel Mode Register (Waveform Mode) Counter Clock Stopped with RC Compare size
#define TC_CHANNEL_REG_CMR_WM_CPCSTOP_SIZE		1
/// TC Channel Mode Register (Waveform Mode) Counter Clock Stopped with RC Compare mask
#define TC_CHANNEL_REG_CMR_WM_CPCSTOP_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_CPCSTOP_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_CPCSTOP_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Counter Clock Disable with RC Compare offset
#define TC_CHANNEL_REG_CMR_WM_CPCDIS_OFFSET		7
/// TC Channel Mode Register (Waveform Mode) Counter Clock Disable with RC Compare size
#define TC_CHANNEL_REG_CMR_WM_CPCDIS_SIZE		1
/// TC Channel Mode Register (Waveform Mode) Counter Clock Disable with RC Compare mask
#define TC_CHANNEL_REG_CMR_WM_CPCDIS_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_CPCDIS_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_CPCDIS_OFFSET)
/// TC Channel Mode Register (Waveform Mode) External Event Edge Selection offset
#define TC_CHANNEL_REG_CMR_WM_EEVTEDG_OFFSET		8
/// TC Channel Mode Register (Waveform Mode) External Event Edge Selection size
#define TC_CHANNEL_REG_CMR_WM_EEVTEDG_SIZE		2
/// TC Channel Mode Register (Waveform Mode) External Event Edge Selection mask
#define TC_CHANNEL_REG_CMR_WM_EEVTEDG_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_EEVTEDG_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_EEVTEDG_OFFSET)
/// TC Channel Mode Register (Waveform Mode) External Event Selection offset
#define TC_CHANNEL_REG_CMR_WM_EEVT_OFFSET		10
/// TC Channel Mode Register (Waveform Mode) External Event Selection size
#define TC_CHANNEL_REG_CMR_WM_EEVT_SIZE			2
/// TC Channel Mode Register (Waveform Mode) External Event Selection mask
#define TC_CHANNEL_REG_CMR_WM_EEVT_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_EEVT_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_EEVT_OFFSET)
/// TC Channel Mode Register (Waveform Mode) External Event Trigger Enable offset
#define TC_CHANNEL_REG_CMR_WM_ENETRG_OFFSET		12
/// TC Channel Mode Register (Waveform Mode) External Event Trigger Enable size
#define TC_CHANNEL_REG_CMR_WM_ENETRG_SIZE		1
/// TC Channel Mode Register (Waveform Mode) External Event Trigger Enable mask
#define TC_CHANNEL_REG_CMR_WM_ENETRG_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_ENETRG_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_ENETRG_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Waveform Selection offset
#define TC_CHANNEL_REG_CMR_WM_WAVSEL_OFFSET		13
/// TC Channel Mode Register (Waveform Mode) Waveform Selection size
#define TC_CHANNEL_REG_CMR_WM_WAVSEL_SIZE		2
/// TC Channel Mode Register (Waveform Mode) Waveform Selection mask
#define TC_CHANNEL_REG_CMR_WM_WAVSEL_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_WAVSEL_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_WAVSEL_OFFSET)
/// TC Channel Mode Register (Waveform Mode) RA Compare Effect on TIOA offset
#define TC_CHANNEL_REG_CMR_WM_ACPA_OFFSET		16
/// TC Channel Mode Register (Waveform Mode) RA Compare Effect on TIOA size
#define TC_CHANNEL_REG_CMR_WM_ACPA_SIZE			2
/// TC Channel Mode Register (Waveform Mode) RA Compare Effect on TIOA mask
#define TC_CHANNEL_REG_CMR_WM_ACPA_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_ACPA_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_ACPA_OFFSET)
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOA offset
#define TC_CHANNEL_REG_CMR_WM_ACPC_OFFSET		18
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOA size
#define TC_CHANNEL_REG_CMR_WM_ACPC_SIZE			2
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOA mask
#define TC_CHANNEL_REG_CMR_WM_ACPC_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_ACPC_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_ACPC_OFFSET)
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOA offset
#define TC_CHANNEL_REG_CMR_WM_AEEVT_OFFSET		20
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOA size
#define TC_CHANNEL_REG_CMR_WM_AEEVT_SIZE		2
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOA mask
#define TC_CHANNEL_REG_CMR_WM_AEEVT_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_AEEVT_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_AEEVT_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOA offset
#define TC_CHANNEL_REG_CMR_WM_ASWTRG_OFFSET		22
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOA size
#define TC_CHANNEL_REG_CMR_WM_ASWTRG_SIZE		2
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOA mask
#define TC_CHANNEL_REG_CMR_WM_ASWTRG_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_ASWTRG_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_ASWTRG_OFFSET)
/// TC Channel Mode Register (Waveform Mode) RB Compare Effect on TIOB offset
#define TC_CHANNEL_REG_CMR_WM_BCPB_OFFSET		24
/// TC Channel Mode Register (Waveform Mode) RB Compare Effect on TIOB size
#define TC_CHANNEL_REG_CMR_WM_BCPB_SIZE			2
/// TC Channel Mode Register (Waveform Mode) RB Compare Effect on TIOB mask
#define TC_CHANNEL_REG_CMR_WM_BCPB_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_BCPB_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_BCPB_OFFSET)
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOB offset
#define TC_CHANNEL_REG_CMR_WM_BCPC_OFFSET		26
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOB size
#define TC_CHANNEL_REG_CMR_WM_BCPC_SIZE			2
/// TC Channel Mode Register (Waveform Mode) RC Compare Effect on TIOB mask
#define TC_CHANNEL_REG_CMR_WM_BCPC_MASK			(((1 << TC_CHANNEL_REG_CMR_WM_BCPC_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_BCPC_OFFSET)
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOB offset
#define TC_CHANNEL_REG_CMR_WM_BEEVT_OFFSET		28
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOB size
#define TC_CHANNEL_REG_CMR_WM_BEEVT_SIZE		2
/// TC Channel Mode Register (Waveform Mode) External Event Effect on TIOB mask
#define TC_CHANNEL_REG_CMR_WM_BEEVT_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_BEEVT_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_BEEVT_OFFSET)
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOB offset
#define TC_CHANNEL_REG_CMR_WM_BSWTRG_OFFSET		30
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOB size
#define TC_CHANNEL_REG_CMR_WM_BSWTRG_SIZE		2
/// TC Channel Mode Register (Waveform Mode) Software Trigger Effect on TIOB mask
#define TC_CHANNEL_REG_CMR_WM_BSWTRG_MASK		(((1 << TC_CHANNEL_REG_CMR_WM_BSWTRG_SIZE) - 1) << TC_CHANNEL_REG_CMR_WM_BSWTRG_OFFSET)
/// TC Channel Counter Value Register (Read)
#define TC_CHANNEL_REG_CV				0x00000010
/// TC Channel Counter Value Register Counter Value offset
#define TC_CHANNEL_REG_CV_CV_OFFSET			0
/// TC Channel Counter Value Register Counter Value size
#define TC_CHANNEL_REG_CV_CV_SIZE			16
/// TC Channel Counter Value Register Counter Value mask
#define TC_CHANNEL_REG_CV_CV_MASK			(((1 << TC_CHANNEL_REG_CV_CV_SIZE) - 1) << TC_CHANNEL_REG_CV_CV_OFFSET)
/// TC Channel Register A (Read [Wave=0]; Read/Write [Wave=1])
#define TC_CHANNEL_REG_RA				0x00000014
/// TC Channel Register A Register A Value offset
#define TC_CHANNEL_REG_RA_RA_OFFSET			0
/// TC Channel Register A Register A Value size
#define TC_CHANNEL_REG_RA_RA_SIZE			16
/// TC Channel Register A Register A Value mask
#define TC_CHANNEL_REG_RA_RA_MASK			(((1 << TC_CHANNEL_REG_RA_RA_SIZE) - 1) << TC_CHANNEL_REG_RA_RA_OFFSET)
/// TC Channel Register B (Read [Wave=0]; Read/Write [Wave=1])
#define TC_CHANNEL_REG_RB				0x00000018
/// TC Channel Register B Register B Value offset
#define TC_CHANNEL_REG_RB_RB_OFFSET			0
/// TC Channel Register B Register B Value size
#define TC_CHANNEL_REG_RB_RB_SIZE			16
/// TC Channel Register B Register B Value mask
#define TC_CHANNEL_REG_RB_RB_MASK			(((1 << TC_CHANNEL_REG_RB_RB_SIZE) - 1) << TC_CHANNEL_REG_RB_RB_OFFSET)
/// TC Channel Register C (Read [Wave=0]; Read/Write [Wave=1])
#define TC_CHANNEL_REG_RC				0x0000001c
/// TC Channel Register C Register C Value offset
#define TC_CHANNEL_REG_RC_RC_OFFSET			0
/// TC Channel Register C Register C Value size
#define TC_CHANNEL_REG_RC_RC_SIZE			16
/// TC Channel Register C Register C Value mask
#define TC_CHANNEL_REG_RC_RC_MASK			(((1 << TC_CHANNEL_REG_RC_RC_SIZE) - 1) << TC_CHANNEL_REG_RC_RC_OFFSET)
/// TC Channel Status Register (Read)
#define TC_CHANNEL_REG_SR				0x00000020
/// TC Channel Status Register Counter Overflow Status offset
#define TC_CHANNEL_REG_SR_COVFS_OFFSET			0
/// TC Channel Status Register Counter Overflow Status size
#define TC_CHANNEL_REG_SR_COVFS_SIZE			1
/// TC Channel Status Register Counter Overflow Status mask
#define TC_CHANNEL_REG_SR_COVFS_MASK			(((1 << TC_CHANNEL_REG_SR_COVFS_SIZE) - 1) << TC_CHANNEL_REG_SR_COVFS_OFFSET)
/// TC Channel Status Register Load Overrun Status offset
#define TC_CHANNEL_REG_SR_LOVRS_OFFSET			1
/// TC Channel Status Register Load Overrun Status size
#define TC_CHANNEL_REG_SR_LOVRS_SIZE			1
/// TC Channel Status Register Load Overrun Status mask
#define TC_CHANNEL_REG_SR_LOVRS_MASK			(((1 << TC_CHANNEL_REG_SR_LOVRS_SIZE) - 1) << TC_CHANNEL_REG_SR_LOVRS_OFFSET)
/// TC Channel Status Register RA Compare Status offset
#define TC_CHANNEL_REG_SR_CPAS_OFFSET			2
/// TC Channel Status Register RA Compare Status size
#define TC_CHANNEL_REG_SR_CPAS_SIZE			1
/// TC Channel Status Register RA Compare Status mask
#define TC_CHANNEL_REG_SR_CPAS_MASK			(((1 << TC_CHANNEL_REG_SR_CPAS_SIZE) - 1) << TC_CHANNEL_REG_SR_CPAS_OFFSET)
/// TC Channel Status Register RB Compare Status offset
#define TC_CHANNEL_REG_SR_CPBS_OFFSET			3
/// TC Channel Status Register RB Compare Status size
#define TC_CHANNEL_REG_SR_CPBS_SIZE			1
/// TC Channel Status Register RB Compare Status mask
#define TC_CHANNEL_REG_SR_CPBS_MASK			(((1 << TC_CHANNEL_REG_SR_CPBS_SIZE) - 1) << TC_CHANNEL_REG_SR_CPBS_OFFSET)
/// TC Channel Status Register RC Compare Status offset
#define TC_CHANNEL_REG_SR_CPCS_OFFSET			4
/// TC Channel Status Register RC Compare Status size
#define TC_CHANNEL_REG_SR_CPCS_SIZE			1
/// TC Channel Status Register RC Compare Status mask
#define TC_CHANNEL_REG_SR_CPCS_MASK			(((1 << TC_CHANNEL_REG_SR_CPCS_SIZE) - 1) << TC_CHANNEL_REG_SR_CPCS_OFFSET)
/// TC Channel Status Register RA Loading Status offset
#define TC_CHANNEL_REG_SR_LDRAS_OFFSET			5
/// TC Channel Status Register RA Loading Status size
#define TC_CHANNEL_REG_SR_LDRAS_SIZE			1
/// TC Channel Status Register RA Loading Status mask
#define TC_CHANNEL_REG_SR_LDRAS_MASK			(((1 << TC_CHANNEL_REG_SR_LDRAS_SIZE) - 1) << TC_CHANNEL_REG_SR_LDRAS_OFFSET)
/// TC Channel Status Register RB Loading Status offset
#define TC_CHANNEL_REG_SR_LDRBS_OFFSET			6
/// TC Channel Status Register RB Loading Status size
#define TC_CHANNEL_REG_SR_LDRBS_SIZE			1
/// TC Channel Status Register RB Loading Status mask
#define TC_CHANNEL_REG_SR_LDRBS_MASK			(((1 << TC_CHANNEL_REG_SR_LDRBS_SIZE) - 1) << TC_CHANNEL_REG_SR_LDRBS_OFFSET)
/// TC Channel Status Register External Trigger Status offset
#define TC_CHANNEL_REG_SR_ETRGS_OFFSET			7
/// TC Channel Status Register External Trigger Status size
#define TC_CHANNEL_REG_SR_ETRGS_SIZE			1
/// TC Channel Status Register External Trigger Status mask
#define TC_CHANNEL_REG_SR_ETRGS_MASK			(((1 << TC_CHANNEL_REG_SR_ETRGS_SIZE) - 1) << TC_CHANNEL_REG_SR_ETRGS_OFFSET)
/// TC Channel Status Register Clock Enabling Status offset
#define TC_CHANNEL_REG_SR_CLKSTA_OFFSET			16
/// TC Channel Status Register Clock Enabling Status size
#define TC_CHANNEL_REG_SR_CLKSTA_SIZE			1
/// TC Channel Status Register Clock Enabling Status mask
#define TC_CHANNEL_REG_SR_CLKSTA_MASK			(((1 << TC_CHANNEL_REG_SR_CLKSTA_SIZE) - 1) << TC_CHANNEL_REG_SR_CLKSTA_OFFSET)
/// TC Channel Status Register TIOA Mirror offset
#define TC_CHANNEL_REG_SR_MTIOA_OFFSET			17
/// TC Channel Status Register TIOA Mirror size
#define TC_CHANNEL_REG_SR_MTIOA_SIZE			1
/// TC Channel Status Register TIOA Mirror mask
#define TC_CHANNEL_REG_SR_MTIOA_MASK			(((1 << TC_CHANNEL_REG_SR_MTIOA_SIZE) - 1) << TC_CHANNEL_REG_SR_MTIOA_OFFSET)
/// TC Channel Status Register TIOB Mirror offset
#define TC_CHANNEL_REG_SR_MTIOB_OFFSET			18
/// TC Channel Status Register TIOB Mirror size
#define TC_CHANNEL_REG_SR_MTIOB_SIZE			1
/// TC Channel Status Register TIOB Mirror mask
#define TC_CHANNEL_REG_SR_MTIOB_MASK			(((1 << TC_CHANNEL_REG_SR_MTIOB_SIZE) - 1) << TC_CHANNEL_REG_SR_MTIOB_OFFSET)
/// TC Channel Interrupt Enable Register (Write)
#define TC_CHANNEL_REG_IER				0x00000024
/// TC Channel Interrupt Disable Register (Write)
#define TC_CHANNEL_REG_IDR				0x00000028
/// TC Channel Interrupt Mask Register (Read)
#define TC_CHANNEL_REG_IMR				0x0000002c
/// TC Channel Interrupt Enable/Disable/Mask Register Counter Overflow Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_COVFS_OFFSET			0
/// TC Channel Interrupt Enable/Disable/Mask Register Counter Overflow Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_COVFS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register Counter Overflow Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_COVFS_MASK			(((1 << TC_CHANNEL_REG_IXR_COVFS_SIZE) - 1) << TC_CHANNEL_REG_IXR_COVFS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register Load Overrun Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_LOVRS_OFFSET			1
/// TC Channel Interrupt Enable/Disable/Mask Register Load Overrun Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_LOVRS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register Load Overrun Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_LOVRS_MASK			(((1 << TC_CHANNEL_REG_IXR_LOVRS_SIZE) - 1) << TC_CHANNEL_REG_IXR_LOVRS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register RA Compare Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_CPAS_OFFSET			2
/// TC Channel Interrupt Enable/Disable/Mask Register RA Compare Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_CPAS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register RA Compare Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_CPAS_MASK			(((1 << TC_CHANNEL_REG_IXR_CPAS_SIZE) - 1) << TC_CHANNEL_REG_IXR_CPAS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register RB Compare Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_CPBS_OFFSET			3
/// TC Channel Interrupt Enable/Disable/Mask Register RB Compare Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_CPBS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register RB Compare Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_CPBS_MASK			(((1 << TC_CHANNEL_REG_IXR_CPBS_SIZE) - 1) << TC_CHANNEL_REG_IXR_CPBS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register RC Compare Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_CPCS_OFFSET			4
/// TC Channel Interrupt Enable/Disable/Mask Register RC Compare Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_CPCS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register RC Compare Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_CPCS_MASK			(((1 << TC_CHANNEL_REG_IXR_CPCS_SIZE) - 1) << TC_CHANNEL_REG_IXR_CPCS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register RA Loading Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_LDRAS_OFFSET			5
/// TC Channel Interrupt Enable/Disable/Mask Register RA Loading Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_LDRAS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register RA Loading Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_LDRAS_MASK			(((1 << TC_CHANNEL_REG_IXR_LDRAS_SIZE) - 1) << TC_CHANNEL_REG_IXR_LDRAS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register RB Loading Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_LDRBS_OFFSET			6
/// TC Channel Interrupt Enable/Disable/Mask Register RB Loading Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_LDRBS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register RB Loading Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_LDRBS_MASK			(((1 << TC_CHANNEL_REG_IXR_LDRBS_SIZE) - 1) << TC_CHANNEL_REG_IXR_LDRBS_OFFSET)
/// TC Channel Interrupt Enable/Disable/Mask Register External Trigger Interrupt Enable/Disable/Mask offset
#define TC_CHANNEL_REG_IXR_ETRGS_OFFSET			7
/// TC Channel Interrupt Enable/Disable/Mask Register External Trigger Interrupt Enable/Disable/Mask size
#define TC_CHANNEL_REG_IXR_ETRGS_SIZE			1
/// TC Channel Interrupt Enable/Disable/Mask Register External Trigger Interrupt Enable/Disable/Mask mask
#define TC_CHANNEL_REG_IXR_ETRGS_MASK			(((1 << TC_CHANNEL_REG_IXR_ETRGS_SIZE) - 1) << TC_CHANNEL_REG_IXR_ETRGS_OFFSET)

typedef enum {
	TC_EXTERNAL_CLOCK0_TCLK0			= 0b00 << TC_REG_BMR_TC0XC0S_OFFSET,
	TC_EXTERNAL_CLOCK0_NONE				= 0b01 << TC_REG_BMR_TC0XC0S_OFFSET,
	TC_EXTERNAL_CLOCK0_TIOA1			= 0b10 << TC_REG_BMR_TC0XC0S_OFFSET,
	TC_EXTERNAL_CLOCK0_TIOA2			= 0b11 << TC_REG_BMR_TC0XC0S_OFFSET,
} tc_external_clock0_t;

typedef enum {
	TC_EXTERNAL_CLOCK1_TCLK1			= 0b00 << TC_REG_BMR_TC1XC1S_OFFSET,
	TC_EXTERNAL_CLOCK1_NONE				= 0b01 << TC_REG_BMR_TC1XC1S_OFFSET,
	TC_EXTERNAL_CLOCK1_TIOA0			= 0b10 << TC_REG_BMR_TC1XC1S_OFFSET,
	TC_EXTERNAL_CLOCK1_TIOA2			= 0b11 << TC_REG_BMR_TC1XC1S_OFFSET,
} tc_external_clock1_t;

typedef enum {
	TC_EXTERNAL_CLOCK2_TCLK2			= 0b00 << TC_REG_BMR_TC2XC2S_OFFSET,
	TC_EXTERNAL_CLOCK2_NONE				= 0b01 << TC_REG_BMR_TC2XC2S_OFFSET,
	TC_EXTERNAL_CLOCK2_TIOA0			= 0b10 << TC_REG_BMR_TC2XC2S_OFFSET,
	TC_EXTERNAL_CLOCK2_TIOA1			= 0b11 << TC_REG_BMR_TC2XC2S_OFFSET,
} tc_external_clock2_t;

typedef enum {
	TC_CHANNEL_CLOCK_TIMER_CLOCK1			= 0b000 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_TIMER_CLOCK2			= 0b001 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_TIMER_CLOCK3			= 0b010 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_TIMER_CLOCK4			= 0b011 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_TIMER_CLOCK5			= 0b100 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_XC0				= 0b101 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_XC1				= 0b110 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_XC2				= 0b111 << TC_CHANNEL_REG_CMR_CM_TCCLKS_OFFSET,
	TC_CHANNEL_CLOCK_32KHZ_OSCILLATOR		= TC_CHANNEL_CLOCK_TIMER_CLOCK1,
	TC_CHANNEL_CLOCK_PBA_BY_2			= TC_CHANNEL_CLOCK_TIMER_CLOCK2,
	TC_CHANNEL_CLOCK_PBA_BY_8			= TC_CHANNEL_CLOCK_TIMER_CLOCK3,
	TC_CHANNEL_CLOCK_PBA_BY_32			= TC_CHANNEL_CLOCK_TIMER_CLOCK4,
	TC_CHANNEL_CLOCK_PBA_BY_128			= TC_CHANNEL_CLOCK_TIMER_CLOCK5,
	TC_CHANNEL_CLOCK_INVERTED			= TC_CHANNEL_REG_CMR_CM_CLKI_MASK,
} tc_channel_clock_t;

typedef enum {
	TC_CHANNEL_BURST_SIGNAL_NONE			= 0b00 << TC_CHANNEL_REG_CMR_CM_BURST_OFFSET,
	TC_CHANNEL_BURST_SIGNAL_XC0			= 0b01 << TC_CHANNEL_REG_CMR_CM_BURST_OFFSET,
	TC_CHANNEL_BURST_SIGNAL_XC1			= 0b10 << TC_CHANNEL_REG_CMR_CM_BURST_OFFSET,
	TC_CHANNEL_BURST_SIGNAL_XC2			= 0b11 << TC_CHANNEL_REG_CMR_CM_BURST_OFFSET,
} tc_channel_burst_signal_t;

typedef enum {
	TC_CHANNEL_RB_LOAD_ACTION_NONE			= 0,
	TC_CHANNEL_RB_LOAD_ACTION_STOP_CLOCK		= TC_CHANNEL_REG_CMR_CM_LDBSTOP_MASK,
	TC_CHANNEL_RB_LOAD_ACTION_DISABLE_CLOCK		= TC_CHANNEL_REG_CMR_CM_LDBDIS_MASK,
} tc_channel_rb_load_action_t;

typedef enum {
	TC_CHANNEL_RA_LOAD_EDGE_NONE			= 0b00 << TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET,
	TC_CHANNEL_RA_LOAD_EDGE_RISING			= 0b01 << TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET,
	TC_CHANNEL_RA_LOAD_EDGE_FALLING			= 0b10 << TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET,
	TC_CHANNEL_RA_LOAD_EDGE_ANY			= 0b11 << TC_CHANNEL_REG_CMR_CM_LDRA_OFFSET,
} tc_channel_ra_load_edge_t;

typedef enum {
	TC_CHANNEL_RB_LOAD_EDGE_NONE			= 0b00 << TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET,
	TC_CHANNEL_RB_LOAD_EDGE_RISING			= 0b01 << TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET,
	TC_CHANNEL_RB_LOAD_EDGE_FALLING			= 0b10 << TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET,
	TC_CHANNEL_RB_LOAD_EDGE_ANY			= 0b11 << TC_CHANNEL_REG_CMR_CM_LDRB_OFFSET,
} tc_channel_rb_load_edge_t;

typedef enum {
	TC_CHANNEL_RC_COMPARE_TRIGGER_DISABLED		= 0,
	TC_CHANNEL_RC_COMPARE_TRIGGER_ENABLED		= TC_CHANNEL_REG_CMR_CM_CPCTRG_MASK,
} tc_channel_rc_compare_trigger_t;

typedef enum {
	TC_CHANNEL_EXTERNAL_TRIGGER_NONE		= 0,
	TC_CHANNEL_EXTERNAL_TRIGGER_TIOB		= 0,
	TC_CHANNEL_EXTERNAL_TRIGGER_TIOA		= TC_CHANNEL_REG_CMR_CM_ABETRG_MASK,
} tc_channel_external_trigger_t;

typedef enum {
	TC_CHANNEL_EXTERNAL_TRIGGER_EDGE_NONE		= 0b00 << TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET,
	TC_CHANNEL_EXTERNAL_TRIGGER_EDGE_RISING		= 0b01 << TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET,
	TC_CHANNEL_EXTERNAL_TRIGGER_EDGE_FALLING	= 0b10 << TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET,
	TC_CHANNEL_EXTERNAL_TRIGGER_EDGE_ANY		= 0b11 << TC_CHANNEL_REG_CMR_CM_ETRGEDG_OFFSET,
} tc_channel_external_trigger_edge_t;

typedef enum {
	TC_CHANNEL_INTERRUPT_COUNTER_OVERFLOW		= TC_CHANNEL_REG_IXR_COVFS_MASK,
	TC_CHANNEL_INTERRUPT_LOAD_OVERRUN		= TC_CHANNEL_REG_IXR_LOVRS_MASK,
	TC_CHANNEL_INTERRUPT_RA_COMPARE			= TC_CHANNEL_REG_IXR_CPAS_MASK,
	TC_CHANNEL_INTERRUPT_RB_COMPARE			= TC_CHANNEL_REG_IXR_CPBS_MASK,
	TC_CHANNEL_INTERRUPT_RC_COMPARE			= TC_CHANNEL_REG_IXR_CPCS_MASK,
	TC_CHANNEL_INTERRUPT_RA_LOAD			= TC_CHANNEL_REG_IXR_LDRAS_MASK,
	TC_CHANNEL_INTERRUPT_RB_LOAD			= TC_CHANNEL_REG_IXR_LDRBS_MASK,
	TC_CHANNEL_INTERRUPT_EXTERNAL_TRIGGER		= TC_CHANNEL_REG_IXR_ETRGS_MASK,
} tc_channel_interrupts_t;

typedef enum {
	TC_CHANNEL_STATUS_COUNTER_OVERFLOW		= TC_CHANNEL_REG_SR_COVFS_MASK,
	TC_CHANNEL_STATUS_LOAD_OVERRUN			= TC_CHANNEL_REG_SR_LOVRS_MASK,
	TC_CHANNEL_STATUS_RA_COMPARE			= TC_CHANNEL_REG_SR_CPAS_MASK,
	TC_CHANNEL_STATUS_RB_COMPARE			= TC_CHANNEL_REG_SR_CPBS_MASK,
	TC_CHANNEL_STATUS_RC_COMPARE			= TC_CHANNEL_REG_SR_CPCS_MASK,
	TC_CHANNEL_STATUS_RA_LOAD			= TC_CHANNEL_REG_SR_LDRAS_MASK,
	TC_CHANNEL_STATUS_RB_LOAD			= TC_CHANNEL_REG_SR_LDRBS_MASK,
	TC_CHANNEL_STATUS_EXTERNAL_TRIGGER		= TC_CHANNEL_REG_SR_ETRGS_MASK,
	TC_CHANNEL_STATUS_CLOCK_ENABLED			= TC_CHANNEL_REG_SR_CLKSTA_MASK,
	TC_CHANNEL_STATUS_TIOA_MIRROR			= TC_CHANNEL_REG_SR_MTIOA_MASK,
	TC_CHANNEL_STATUS_TIOB_MIRROR			= TC_CHANNEL_REG_SR_MTIOB_MASK,
} tc_channel_status_t;

extern uint32_t tc_read_register(uint32_t const offset);
extern void tc_write_register(uint32_t const offset, uint32_t const value);
extern void tc_write_register_field(uint32_t const offset, uint32_t const field_mask, uint32_t const value);

extern void tc_select_external_clock0(tc_external_clock0_t const external_clock0);
extern void tc_select_external_clock1(tc_external_clock1_t const external_clock1);
extern void tc_select_external_clock2(tc_external_clock2_t const external_clock2);
extern void tc_configure_external_clocks(tc_external_clock0_t const external_clock0, tc_external_clock1_t const external_clock1, tc_external_clock2_t const external_clock2);

extern void tc_sync(void);

extern uint32_t tc_channel_read_register(void volatile *channel, uint32_t const offset);
extern void tc_channel_write_register(void volatile *channel, uint32_t const offset, uint32_t const value);
extern void tc_channel_write_register_field(void volatile *channel, uint32_t const offset, uint32_t const field_mask, uint32_t const value);

extern void tc_channel_reset(void volatile *channel);
extern void tc_channel_enable_clock(void volatile *channel);
extern void tc_channel_disable_clock(void volatile *channel);
extern void tc_channel_reset_and_enable_clock(void volatile *channel);

extern void tc_channel_select_capture_mode(void volatile *channel);
extern void tc_channel_select_clock(void volatile *channel, tc_channel_clock_t const clock);
extern void tc_channel_select_burst_signal(void volatile *channel, tc_channel_burst_signal_t const burst);
extern void tc_channel_select_ra_load_edge(void volatile *channel, tc_channel_ra_load_edge_t const ra_load_edge);
extern void tc_channel_select_rb_load_edge(void volatile *channel, tc_channel_rb_load_edge_t const rb_load_edge);
extern void tc_channel_select_rb_load_action(void volatile *channel, tc_channel_rb_load_action_t const rb_load_action);
extern void tc_channel_select_rc_compare_trigger(void volatile *channel, tc_channel_rc_compare_trigger_t const rc_compare_trigger);
extern void tc_channel_select_external_trigger(void volatile *channel, tc_channel_external_trigger_t const external_trigger);
extern void tc_channel_select_external_trigger_edge(void volatile *channel, tc_channel_external_trigger_edge_t const external_trigger_edge);

extern uint16_t tc_channel_read_register_a(void volatile *channel);
extern void tc_channel_write_register_a(void volatile *channel, uint16_t const value);
extern uint16_t tc_channel_read_register_b(void volatile *channel);
extern void tc_channel_write_register_b(void volatile *channel, uint16_t const value);
extern uint16_t tc_channel_read_register_c(void volatile *channel);
extern void tc_channel_write_register_c(void volatile *channel, uint16_t const value);
extern uint16_t tc_channel_read_counter_value(void volatile *channel);
extern tc_channel_status_t tc_channel_read_status(void volatile *channel);

extern void tc_channel_configure_capture_mode(void volatile *channel, tc_channel_clock_t const clock, tc_channel_burst_signal_t const burst, tc_channel_ra_load_edge_t const ra_load_edge, tc_channel_rb_load_edge_t const rb_load_edge, tc_channel_rb_load_action_t const rb_load_action, tc_channel_rc_compare_trigger_t const rc_compare_trigger, tc_channel_external_trigger_t const external_trigger, tc_channel_external_trigger_edge_t const external_trigger_edge, uint16_t const rc_value);
extern void tc_channel_enable_capture_mode(void volatile *channel, tc_channel_clock_t const clock, tc_channel_burst_signal_t const burst, tc_channel_ra_load_edge_t const ra_load_edge, tc_channel_rb_load_edge_t const rb_load_edge, tc_channel_rb_load_action_t const rb_load_action, tc_channel_rc_compare_trigger_t const rc_compare_trigger, tc_channel_external_trigger_t const external_trigger, tc_channel_external_trigger_edge_t const external_trigger_edge, uint16_t const rc_value);

extern void tc_channel_enable_interrupts(void volatile *channel, tc_channel_interrupts_t const interrupts);
extern void tc_channel_disable_interrupts(void volatile *channel, tc_channel_interrupts_t const interrupts);
extern tc_channel_interrupts_t tc_channel_read_interrupt_mask(void volatile *channel);

#endif /* TC_H_ */

