/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef USART_H_
#define USART_H_

#include <at32uc3/inttypes.h>

#define USART0_ADDRESS					0xffff1400
#define USART1_ADDRESS					0xffff1800
#define USART2_ADDRESS					0xffff1c00
#define USART3_ADDRESS					0xffff2000

#define USART_WIDTH					0x00000400
#define USART_NUM_USARTS				4

#define USART(n)					((void volatile *)USART0_ADDRESS + USART_WIDTH * (n))

/// USART Control Register (Write)
#define USART_REG_CR					0x00000000
/// USART Control Register Reset Receiver offset
#define USART_REG_CR_RSTRX_OFFSET			2
/// USART Control Register Reset Receiver size
#define USART_REG_CR_RSTRX_SIZE				1
/// USART Control Register Reset Receiver mask
#define USART_REG_CR_RSTRX_MASK				(((1 << USART_REG_CR_RSTRX_SIZE) - 1) << USART_REG_CR_RSTRX_OFFSET)
/// USART Control Register Reset Transmitter offset
#define USART_REG_CR_RSTTX_OFFSET			3
/// USART Control Register Reset Transmitter size
#define USART_REG_CR_RSTTX_SIZE				1
/// USART Control Register Reset Transmitter mask
#define USART_REG_CR_RSTTX_MASK				(((1 << USART_REG_CR_RSTTX_SIZE) - 1) << USART_REG_CR_RSTTX_OFFSET)
/// USART Control Register Receiver Enable offset
#define USART_REG_CR_RXEN_OFFSET			4
/// USART Control Register Receiver Enable size
#define USART_REG_CR_RXEN_SIZE				1
/// USART Control Register Receiver Enable mask
#define USART_REG_CR_RXEN_MASK				(((1 << USART_REG_CR_RXEN_SIZE) - 1) << USART_REG_CR_RXEN_OFFSET)
/// USART Control Register Receiver Disable offset
#define USART_REG_CR_RXDIS_OFFSET			5
/// USART Control Register Receiver Disable size
#define USART_REG_CR_RXDIS_SIZE				1
/// USART Control Register Receiver Disable mask
#define USART_REG_CR_RXDIS_MASK				(((1 << USART_REG_CR_RXDIS_SIZE) - 1) << USART_REG_CR_RXDIS_OFFSET)
/// USART Control Register Transmitter Enable offset
#define USART_REG_CR_TXEN_OFFSET			6
/// USART Control Register Transmitter Enable size
#define USART_REG_CR_TXEN_SIZE				1
/// USART Control Register Transmitter Enable mask
#define USART_REG_CR_TXEN_MASK				(((1 << USART_REG_CR_TXEN_SIZE) - 1) << USART_REG_CR_TXEN_OFFSET)
/// USART Control Register Transmitter Disable offset
#define USART_REG_CR_TXDIS_OFFSET			7
/// USART Control Register Transmitter Disable size
#define USART_REG_CR_TXDIS_SIZE				1
/// USART Control Register Transmitter Disable mask
#define USART_REG_CR_TXDIS_MASK				(((1 << USART_REG_CR_TXDIS_SIZE) - 1) << USART_REG_CR_TXDIS_OFFSET)
/// USART Control Register Reset Status Bits offset
#define USART_REG_CR_RSTSTA_OFFSET			8
/// USART Control Register Reset Status Bits size
#define USART_REG_CR_RSTSTA_SIZE			1
/// USART Control Register Reset Status Bits mask
#define USART_REG_CR_RSTSTA_MASK			(((1 << USART_REG_CR_RSTSTA_SIZE) - 1) << USART_REG_CR_RSTSTA_OFFSET)
/// USART Control Register Start Break offset
#define USART_REG_CR_STTBRK_OFFSET			9
/// USART Control Register Start Break size
#define USART_REG_CR_STTBRK_SIZE			1
/// USART Control Register Start Break mask
#define USART_REG_CR_STTBRK_MASK			(((1 << USART_REG_CR_STTBRK_SIZE) - 1) << USART_REG_CR_STTBRK_OFFSET)
/// USART Control Register Stop Break offset
#define USART_REG_CR_STPBRK_OFFSET			10
/// USART Control Register Stop Break size
#define USART_REG_CR_STPBRK_SIZE			1
/// USART Control Register Stop Break mask
#define USART_REG_CR_STPBRK_MASK			(((1 << USART_REG_CR_STPBRK_SIZE) - 1) << USART_REG_CR_STPBRK_OFFSET)
/// USART Control Register Start Time-out offset
#define USART_REG_CR_STTTO_OFFSET			11
/// USART Control Register Start Time-out size
#define USART_REG_CR_STTTO_SIZE				1
/// USART Control Register Start Time-out mask
#define USART_REG_CR_STTTO_MASK				(((1 << USART_REG_CR_STTTO_SIZE) - 1) << USART_REG_CR_STTTO_OFFSET)
/// USART Control Register Send Address offset
#define USART_REG_CR_SENDA_OFFSET			12
/// USART Control Register Send Address size
#define USART_REG_CR_SENDA_SIZE				1
/// USART Control Register Send Address mask
#define USART_REG_CR_SENDA_MASK				(((1 << USART_REG_CR_SENDA_SIZE) - 1) << USART_REG_CR_SENDA_OFFSET)
/// USART Control Register Reset Iterations offset
#define USART_REG_CR_RSTIT_OFFSET			13
/// USART Control Register Reset Iterations size
#define USART_REG_CR_RSTIT_SIZE				1
/// USART Control Register Reset Iterations mask
#define USART_REG_CR_RSTIT_MASK				(((1 << USART_REG_CR_RSTIT_SIZE) - 1) << USART_REG_CR_RSTIT_OFFSET)
/// USART Control Register Reset Non Acknowledge offset
#define USART_REG_CR_RSTNACK_OFFSET			14
/// USART Control Register Reset Non Acknowledge size
#define USART_REG_CR_RSTNACK_SIZE			1
/// USART Control Register Reset Non Acknowledge mask
#define USART_REG_CR_RSTNACK_MASK			(((1 << USART_REG_CR_RSTNACK_SIZE) - 1) << USART_REG_CR_RSTNACK_OFFSET)
/// USART Control Register Rearm Time-out offset
#define USART_REG_CR_RETTO_OFFSET			15
/// USART Control Register Rearm Time-out size
#define USART_REG_CR_RETTO_SIZE				1
/// USART Control Register Rearm Time-out mask
#define USART_REG_CR_RETTO_MASK				(((1 << USART_REG_CR_RETTO_SIZE) - 1) << USART_REG_CR_RETTO_OFFSET)
/// USART Control Register Request to Send Enable offset
#define USART_REG_CR_RTSEN_OFFSET			18
/// USART Control Register Request to Send Enable size
#define USART_REG_CR_RTSEN_SIZE				1
/// USART Control Register Request to Send Enable mask
#define USART_REG_CR_RTSEN_MASK				(((1 << USART_REG_CR_RTSEN_SIZE) - 1) << USART_REG_CR_RTSEN_OFFSET)
/// USART Control Register Force SPI Chip Select offset
#define USART_REG_CR_FCS_OFFSET				18
/// USART Control Register Force SPI Chip Select size
#define USART_REG_CR_FCS_SIZE				1
/// USART Control Register Force SPI Chip Select mask
#define USART_REG_CR_FCS_MASK				(((1 << USART_REG_CR_FCS_SIZE) - 1) << USART_REG_CR_FCS_OFFSET)
/// USART Control Register Request to Send Disable offset
#define USART_REG_CR_RTSDIS_OFFSET			19
/// USART Control Register Request to Send Disable size
#define USART_REG_CR_RTSDIS_SIZE			1
/// USART Control Register Request to Send Disable mask
#define USART_REG_CR_RTSDIS_MASK			(((1 << USART_REG_CR_RTSDIS_SIZE) - 1) << USART_REG_CR_RTSDIS_OFFSET)
/// USART Control Register Release SPI Chip Select offset
#define USART_REG_CR_RCS_OFFSET				19
/// USART Control Register Release SPI Chip Select size
#define USART_REG_CR_RCS_SIZE				1
/// USART Control Register Release SPI Chip Select mask
#define USART_REG_CR_RCS_MASK				(((1 << USART_REG_CR_RCS_SIZE) - 1) << USART_REG_CR_RCS_OFFSET)
/// USART Mode Register (Read/Write)
#define USART_REG_MR					0x00000004
/// USART Mode Register Mode of the USART offset
#define USART_REG_MR_MODE_OFFSET			0
/// USART Mode Register Mode of the USART size
#define USART_REG_MR_MODE_SIZE				4
/// USART Mode Register Mode of the USART mask
#define USART_REG_MR_MODE_MASK				(((1 << USART_REG_MR_MODE_SIZE) - 1) << USART_REG_MR_MODE_OFFSET)
/// USART Mode Register Clock Selection offset
#define USART_REG_MR_USCLKS_OFFSET			4
/// USART Mode Register Clock Selection size
#define USART_REG_MR_USCLKS_SIZE			2
/// USART Mode Register Clock Selection mask
#define USART_REG_MR_USCLKS_MASK			(((1 << USART_REG_MR_USCLKS_SIZE) - 1) << USART_REG_MR_USCLKS_OFFSET)
/// USART Mode Register Character Length offset
#define USART_REG_MR_CHRL_OFFSET			6
/// USART Mode Register Character Length size
#define USART_REG_MR_CHRL_SIZE				2
/// USART Mode Register Character Length mask
#define USART_REG_MR_CHRL_MASK				(((1 << USART_REG_MR_CHRL_SIZE) - 1) << USART_REG_MR_CHRL_OFFSET)
/// USART Mode Register Synchronous Mode Select offset
#define USART_REG_MR_SYNC_OFFSET			8
/// USART Mode Register Synchronous Mode Select size
#define USART_REG_MR_SYNC_SIZE				1
/// USART Mode Register Synchronous Mode Select mask
#define USART_REG_MR_SYNC_MASK				(((1 << USART_REG_MR_SYNC_SIZE) - 1) << USART_REG_MR_SYNC_OFFSET)
/// USART Mode Register SPI Clock Phase offset
#define USART_REG_MR_CPHA_OFFSET			8
/// USART Mode Register SPI Clock Phase size
#define USART_REG_MR_CPHA_SIZE				1
/// USART Mode Register SPI Clock Phase mask
#define USART_REG_MR_CPHA_MASK				(((1 << USART_REG_MR_CPHA_SIZE) - 1) << USART_REG_MR_CPHA_OFFSET)
/// USART Mode Register Parity Type offset
#define USART_REG_MR_PAR_OFFSET				9
/// USART Mode Register Parity Type size
#define USART_REG_MR_PAR_SIZE				3
/// USART Mode Register Parity Type mask
#define USART_REG_MR_PAR_MASK				(((1 << USART_REG_MR_PAR_SIZE) - 1) << USART_REG_MR_PAR_OFFSET)
/// USART Mode Register Number of Stop Bits offset
#define USART_REG_MR_NBSTOP_OFFSET			12
/// USART Mode Register Number of Stop Bits size
#define USART_REG_MR_NBSTOP_SIZE			2
/// USART Mode Register Number of Stop Bits mask
#define USART_REG_MR_NBSTOP_MASK			(((1 << USART_REG_MR_NBSTOP_SIZE) - 1) << USART_REG_MR_NBSTOP_OFFSET)
/// USART Mode Register Channel Mode offset
#define USART_REG_MR_CHMODE_OFFSET			14
/// USART Mode Register Channel Mode size
#define USART_REG_MR_CHMODE_SIZE			2
/// USART Mode Register Channel Mode mask
#define USART_REG_MR_CHMODE_MASK			(((1 << USART_REG_MR_CHMODE_SIZE) - 1) << USART_REG_MR_CHMODE_OFFSET)
/// USART Mode Register Bit Order offset
#define USART_REG_MR_MSBF_OFFSET			16
/// USART Mode Register Bit Order size
#define USART_REG_MR_MSBF_SIZE				1
/// USART Mode Register Bit Order mask
#define USART_REG_MR_MSBF_MASK				(((1 << USART_REG_MR_MSBF_SIZE) - 1) << USART_REG_MR_MSBF_OFFSET)
/// USART Mode Register SPI Clock Polarity offset
#define USART_REG_MR_CPOL_OFFSET			16
/// USART Mode Register SPI Clock Polarity size
#define USART_REG_MR_CPOL_SIZE				1
/// USART Mode Register SPI Clock Polarity mask
#define USART_REG_MR_CPOL_MASK				(((1 << USART_REG_MR_CPOL_SIZE) - 1) << USART_REG_MR_CPOL_OFFSET)
/// USART Mode Register 9-bit Character Length offset
#define USART_REG_MR_MODE9_OFFSET			17
/// USART Mode Register 9-bit Character Length size
#define USART_REG_MR_MODE9_SIZE				1
/// USART Mode Register 9-bit Character Length mask
#define USART_REG_MR_MODE9_MASK				(((1 << USART_REG_MR_MODE9_SIZE) - 1) << USART_REG_MR_MODE9_OFFSET)
/// USART Mode Register Clock Output Select offset
#define USART_REG_MR_CLKO_OFFSET			18
/// USART Mode Register Clock Output Select size
#define USART_REG_MR_CLKO_SIZE				1
/// USART Mode Register Clock Output Select mask
#define USART_REG_MR_CLKO_MASK				(((1 << USART_REG_MR_CLKO_SIZE) - 1) << USART_REG_MR_CLKO_OFFSET)
/// USART Mode Register Oversampling Mode offset
#define USART_REG_MR_OVER_OFFSET			19
/// USART Mode Register Oversampling Mode size
#define USART_REG_MR_OVER_SIZE				1
/// USART Mode Register Oversampling Mode mask
#define USART_REG_MR_OVER_MASK				(((1 << USART_REG_MR_OVER_SIZE) - 1) << USART_REG_MR_OVER_OFFSET)
/// USART Mode Register Inhibit Non Acknowledge offset
#define USART_REG_MR_INACK_OFFSET			20
/// USART Mode Register Inhibit Non Acknowledge size
#define USART_REG_MR_INACK_SIZE				1
/// USART Mode Register Inhibit Non Acknowledge mask
#define USART_REG_MR_INACK_MASK				(((1 << USART_REG_MR_INACK_SIZE) - 1) << USART_REG_MR_INACK_OFFSET)
/// USART Mode Register Disable Successive NACK offset
#define USART_REG_MR_DSNACK_OFFSET			21
/// USART Mode Register Disable Successive NACK size
#define USART_REG_MR_DSNACK_SIZE			1
/// USART Mode Register Disable Successive NACK mask
#define USART_REG_MR_DSNACK_MASK			(((1 << USART_REG_MR_DSNACK_SIZE) - 1) << USART_REG_MR_DSNACK_OFFSET)
/// USART Mode Register Variable Synchronization of Command/Data Sync Start Frame Delimiter offset
#define USART_REG_MR_VAR_SYNC_OFFSET			22
/// USART Mode Register Variable Synchronization of Command/Data Sync Start Frame Delimiter size
#define USART_REG_MR_VAR_SYNC_SIZE			1
/// USART Mode Register Variable Synchronization of Command/Data Sync Start Frame Delimiter mask
#define USART_REG_MR_VAR_SYNC_MASK			(((1 << USART_REG_MR_VAR_SYNC_SIZE) - 1) << USART_REG_MR_VAR_SYNC_OFFSET)
/// USART Mode Register Maximum Iterations offset
#define USART_REG_MR_MAX_ITERATION_OFFSET		24
/// USART Mode Register Maximum Iterations size
#define USART_REG_MR_MAX_ITERATION_SIZE			3
/// USART Mode Register Maximum Iterations mask
#define USART_REG_MR_MAX_ITERATION_MASK			(((1 << USART_REG_MR_MAX_ITERATION_SIZE) - 1) << USART_REG_MR_MAX_ITERATION_OFFSET)
/// USART Mode Register Infrared Receive Line Filter offset
#define USART_REG_MR_FILTER_OFFSET			28
/// USART Mode Register Infrared Receive Line Filter size
#define USART_REG_MR_FILTER_SIZE			1
/// USART Mode Register Infrared Receive Line Filter mask
#define USART_REG_MR_FILTER_MASK			(((1 << USART_REG_MR_FILTER_SIZE) - 1) << USART_REG_MR_FILTER_OFFSET)
/// USART Mode Register Manchester Encoder/Decoder Enable offset
#define USART_REG_MR_MAN_OFFSET				29
/// USART Mode Register Manchester Encoder/Decoder Enable size
#define USART_REG_MR_MAN_SIZE				1
/// USART Mode Register Manchester Encoder/Decoder Enable mask
#define USART_REG_MR_MAN_MASK				(((1 << USART_REG_MR_MAN_SIZE) - 1) << USART_REG_MR_MAN_OFFSET)
/// USART Mode Register Manchester Synchronization Mode offset
#define USART_REG_MR_MODSYNC_OFFSET			30
/// USART Mode Register Manchester Synchronization Mode size
#define USART_REG_MR_MODSYNC_SIZE			1
/// USART Mode Register Manchester Synchronization Mode mask
#define USART_REG_MR_MODSYNC_MASK			(((1 << USART_REG_MR_MODSYNC_SIZE) - 1) << USART_REG_MR_MODSYNC_OFFSET)
/// USART Mode Register Start Frame Delimiter Selector offset
#define USART_REG_MR_ONEBIT_OFFSET			31
/// USART Mode Register Start Frame Delimiter Selector size
#define USART_REG_MR_ONEBIT_SIZE			1
/// USART Mode Register Start Frame Delimiter Selector mask
#define USART_REG_MR_ONEBIT_MASK			(((1 << USART_REG_MR_ONEBIT_SIZE) - 1) << USART_REG_MR_ONEBIT_OFFSET)
/// USART Interrupt Enable Register (Write)
#define USART_REG_IER					0x00000008
/// USART Interrupt Disable Register (Write)
#define USART_REG_IDR					0x0000000c
/// USART Interrupt Mask Register (Read)
#define USART_REG_IMR					0x00000010
/// USART Interrupt Enable/Disable/Mask Register RXRDY Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_RXRDY_OFFSET			0
/// USART Interrupt Enable/Disable/Mask Register RXRDY Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_RXRDY_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register RXRDY Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_RXRDY_MASK			(((1 << USART_REG_IXR_RXRDY_SIZE) - 1) << USART_REG_IXR_RXRDY_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register TXRDY Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_TXRDY_OFFSET			1
/// USART Interrupt Enable/Disable/Mask Register TXRDY Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_TXRDY_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register TXRDY Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_TXRDY_MASK			(((1 << USART_REG_IXR_TXRDY_SIZE) - 1) << USART_REG_IXR_TXRDY_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Receiver Break Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_RXBRK_OFFSET			2
/// USART Interrupt Enable/Disable/Mask Register Receiver Break Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_RXBRK_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Receiver Break Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_RXBRK_MASK			(((1 << USART_REG_IXR_RXBRK_SIZE) - 1) << USART_REG_IXR_RXBRK_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register End of Receive Transfer Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_ENDRX_OFFSET			3
/// USART Interrupt Enable/Disable/Mask Register End of Receive Transfer Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_ENDRX_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register End of Receive Transfer Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_ENDRX_MASK			(((1 << USART_REG_IXR_ENDRX_SIZE) - 1) << USART_REG_IXR_ENDRX_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register End of Transmit Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_ENDTX_OFFSET			4
/// USART Interrupt Enable/Disable/Mask Register End of Transmit Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_ENDTX_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register End of Transmit Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_ENDTX_MASK			(((1 << USART_REG_IXR_ENDTX_SIZE) - 1) << USART_REG_IXR_ENDTX_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_OVRE_OFFSET			5
/// USART Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_OVRE_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register Overrun Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_OVRE_MASK				(((1 << USART_REG_IXR_OVRE_SIZE) - 1) << USART_REG_IXR_OVRE_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Framing Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_FRAME_OFFSET			6
/// USART Interrupt Enable/Disable/Mask Register Framing Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_FRAME_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Framing Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_FRAME_MASK			(((1 << USART_REG_IXR_FRAME_SIZE) - 1) << USART_REG_IXR_FRAME_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Parity Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_PARE_OFFSET			7
/// USART Interrupt Enable/Disable/Mask Register Parity Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_PARE_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register Parity Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_PARE_MASK				(((1 << USART_REG_IXR_PARE_SIZE) - 1) << USART_REG_IXR_PARE_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Time-out Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_TIMEOUT_OFFSET			9
/// USART Interrupt Enable/Disable/Mask Register Time-out Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_TIMEOUT_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Time-out Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_TIMEOUT_MASK			(((1 << USART_REG_IXR_TIMEOUT_SIZE) - 1) << USART_REG_IXR_TIMEOUT_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register TXEMPTY Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_TXEMPTY_OFFSET			8
/// USART Interrupt Enable/Disable/Mask Register TXEMPTY Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_TXEMPTY_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register TXEMPTY Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_TXEMPTY_MASK			(((1 << USART_REG_IXR_TXEMPTY_SIZE) - 1) << USART_REG_IXR_TXEMPTY_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Maximum Number of Repetitions Reached Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_ITER_OFFSET			10
/// USART Interrupt Enable/Disable/Mask Register Maximum Number of Repetitions Reached Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_ITER_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register Maximum Number of Repetitions Reached Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_ITER_MASK				(((1 << USART_REG_IXR_ITER_SIZE) - 1) << USART_REG_IXR_ITER_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register SPI Underrun Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_UNRE_OFFSET			10
/// USART Interrupt Enable/Disable/Mask Register SPI Underrun Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_UNRE_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register SPI Underrun Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_UNRE_MASK				(((1 << USART_REG_IXR_UNRE_SIZE) - 1) << USART_REG_IXR_UNRE_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Buffer Empty Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_TXBUFE_OFFSET			11
/// USART Interrupt Enable/Disable/Mask Register Buffer Empty Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_TXBUFE_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Buffer Empty Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_TXBUFE_MASK			(((1 << USART_REG_IXR_TXBUFE_SIZE) - 1) << USART_REG_IXR_TXBUFE_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Buffer Full Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_RXBUFF_OFFSET			12
/// USART Interrupt Enable/Disable/Mask Register Buffer Full Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_RXBUFF_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Buffer Full Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_RXBUFF_MASK			(((1 << USART_REG_IXR_RXBUFF_SIZE) - 1) << USART_REG_IXR_RXBUFF_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Non Acknowledge Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_NACK_OFFSET			13
/// USART Interrupt Enable/Disable/Mask Register Non Acknowledge Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_NACK_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register Non Acknowledge Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_NACK_MASK				(((1 << USART_REG_IXR_NACK_SIZE) - 1) << USART_REG_IXR_NACK_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Clear to Send Input Change Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_CTSIC_OFFSET			19
/// USART Interrupt Enable/Disable/Mask Register Clear to Send Input Change Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_CTSIC_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Clear to Send Input Change Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_CTSIC_MASK			(((1 << USART_REG_IXR_CTSIC_SIZE) - 1) << USART_REG_IXR_CTSIC_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_MANE_OFFSET			20
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_MANE_SIZE				1
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_MANE_MASK				(((1 << USART_REG_IXR_MANE_SIZE) - 1) << USART_REG_IXR_MANE_OFFSET)
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask offset
#define USART_REG_IXR_MANEA_OFFSET			24
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask size
#define USART_REG_IXR_MANEA_SIZE			1
/// USART Interrupt Enable/Disable/Mask Register Manchester Error Interrupt Enable/Disable/Mask mask
#define USART_REG_IXR_MANEA_MASK			(((1 << USART_REG_IXR_MANEA_SIZE) - 1) << USART_REG_IXR_MANEA_OFFSET)
/// USART Channel Status Register (Read)
#define USART_REG_CSR					0x00000014
/// USART Channel Status Register Receiver Ready offset
#define USART_REG_CSR_RXRDY_OFFSET			0
/// USART Channel Status Register Receiver Ready size
#define USART_REG_CSR_RXRDY_SIZE			1
/// USART Channel Status Register Receiver Ready mask
#define USART_REG_CSR_RXRDY_MASK			(((1 << USART_REG_CSR_RXRDY_SIZE) - 1) << USART_REG_CSR_RXRDY_OFFSET)
/// USART Channel Status Register Transmitter Ready offset
#define USART_REG_CSR_TXRDY_OFFSET			1
/// USART Channel Status Register Transmitter Ready size
#define USART_REG_CSR_TXRDY_SIZE			1
/// USART Channel Status Register Transmitter Ready mask
#define USART_REG_CSR_TXRDY_MASK			(((1 << USART_REG_CSR_TXRDY_SIZE) - 1) << USART_REG_CSR_TXRDY_OFFSET)
/// USART Channel Status Register Break Received/End of Break offset
#define USART_REG_CSR_RXBRK_OFFSET			2
/// USART Channel Status Register Break Received/End of Break size
#define USART_REG_CSR_RXBRK_SIZE			1
/// USART Channel Status Register Break Received/End of Break mask
#define USART_REG_CSR_RXBRK_MASK			(((1 << USART_REG_CSR_RXBRK_SIZE) - 1) << USART_REG_CSR_RXBRK_OFFSET)
/// USART Channel Status Register End of Receiver Transfer offset
#define USART_REG_CSR_ENDRX_OFFSET			3
/// USART Channel Status Register End of Receiver Transfer size
#define USART_REG_CSR_ENDRX_SIZE			1
/// USART Channel Status Register End of Receiver Transfer mask
#define USART_REG_CSR_ENDRX_MASK			(((1 << USART_REG_CSR_ENDRX_SIZE) - 1) << USART_REG_CSR_ENDRX_OFFSET)
/// USART Channel Status Register End of Transmitter Transfer offset
#define USART_REG_CSR_ENDTX_OFFSET			4
/// USART Channel Status Register End of Transmitter Transfer size
#define USART_REG_CSR_ENDTX_SIZE			1
/// USART Channel Status Register End of Transmitter Transfer mask
#define USART_REG_CSR_ENDTX_MASK			(((1 << USART_REG_CSR_ENDTX_SIZE) - 1) << USART_REG_CSR_ENDTX_OFFSET)
/// USART Channel Status Register Overrun Error offset
#define USART_REG_CSR_OVRE_OFFSET			5
/// USART Channel Status Register Overrun Error size
#define USART_REG_CSR_OVRE_SIZE				1
/// USART Channel Status Register Overrun Error mask
#define USART_REG_CSR_OVRE_MASK				(((1 << USART_REG_CSR_OVRE_SIZE) - 1) << USART_REG_CSR_OVRE_OFFSET)
/// USART Channel Status Register Framing Error offset
#define USART_REG_CSR_FRAME_OFFSET			6
/// USART Channel Status Register Framing Error size
#define USART_REG_CSR_FRAME_SIZE			1
/// USART Channel Status Register Framing Error mask
#define USART_REG_CSR_FRAME_MASK			(((1 << USART_REG_CSR_FRAME_SIZE) - 1) << USART_REG_CSR_FRAME_OFFSET)
/// USART Channel Status Register Parity Error offset
#define USART_REG_CSR_PARE_OFFSET			7
/// USART Channel Status Register Parity Error size
#define USART_REG_CSR_PARE_SIZE				1
/// USART Channel Status Register Parity Error mask
#define USART_REG_CSR_PARE_MASK				(((1 << USART_REG_CSR_PARE_SIZE) - 1) << USART_REG_CSR_PARE_OFFSET)
/// USART Channel Status Register Receiver Time-out offset
#define USART_REG_CSR_TIMEOUT_OFFSET			9
/// USART Channel Status Register Receiver Time-out size
#define USART_REG_CSR_TIMEOUT_SIZE			1
/// USART Channel Status Register Receiver Time-out mask
#define USART_REG_CSR_TIMEOUT_MASK			(((1 << USART_REG_CSR_TIMEOUT_SIZE) - 1) << USART_REG_CSR_TIMEOUT_OFFSET)
/// USART Channel Status Register Transmitter Empty offset
#define USART_REG_CSR_TXEMPTY_OFFSET			8
/// USART Channel Status Register Transmitter Empty size
#define USART_REG_CSR_TXEMPTY_SIZE			1
/// USART Channel Status Register Transmitter Empty mask
#define USART_REG_CSR_TXEMPTY_MASK			(((1 << USART_REG_CSR_TXEMPTY_SIZE) - 1) << USART_REG_CSR_TXEMPTY_OFFSET)
/// USART Channel Status Register Maximum Number of Repetitions Reached offset
#define USART_REG_CSR_ITER_OFFSET			10
/// USART Channel Status Register Maximum Number of Repetitions Reached size
#define USART_REG_CSR_ITER_SIZE				1
/// USART Channel Status Register Maximum Number of Repetitions Reached mask
#define USART_REG_CSR_ITER_MASK				(((1 << USART_REG_CSR_ITER_SIZE) - 1) << USART_REG_CSR_ITER_OFFSET)
/// USART Channel Status Register SPI Underrun Error offset
#define USART_REG_CSR_UNRE_OFFSET			10
/// USART Channel Status Register SPI Underrun Error size
#define USART_REG_CSR_UNRE_SIZE				1
/// USART Channel Status Register SPI Underrun Error mask
#define USART_REG_CSR_UNRE_MASK				(((1 << USART_REG_CSR_UNRE_SIZE) - 1) << USART_REG_CSR_UNRE_OFFSET)
/// USART Channel Status Register Transmission Buffer Empty offset
#define USART_REG_CSR_TXBUFE_OFFSET			11
/// USART Channel Status Register Transmission Buffer Empty size
#define USART_REG_CSR_TXBUFE_SIZE			1
/// USART Channel Status Register Transmission Buffer Empty mask
#define USART_REG_CSR_TXBUFE_MASK			(((1 << USART_REG_CSR_TXBUFE_SIZE) - 1) << USART_REG_CSR_TXBUFE_OFFSET)
/// USART Channel Status Register Reception Buffer Full offset
#define USART_REG_CSR_RXBUFF_OFFSET			12
/// USART Channel Status Register Reception Buffer Full size
#define USART_REG_CSR_RXBUFF_SIZE			1
/// USART Channel Status Register Reception Buffer Full mask
#define USART_REG_CSR_RXBUFF_MASK			(((1 << USART_REG_CSR_RXBUFF_SIZE) - 1) << USART_REG_CSR_RXBUFF_OFFSET)
/// USART Channel Status Register Non Acknowledge offset
#define USART_REG_CSR_NACK_OFFSET			13
/// USART Channel Status Register Non Acknowledge size
#define USART_REG_CSR_NACK_SIZE				1
/// USART Channel Status Register Non Acknowledge mask
#define USART_REG_CSR_NACK_MASK				(((1 << USART_REG_CSR_NACK_SIZE) - 1) << USART_REG_CSR_NACK_OFFSET)
/// USART Channel Status Register Clear to Send Input Change Flag offset
#define USART_REG_CSR_CTSIC_OFFSET			19
/// USART Channel Status Register Clear to Send Input Change Flag size
#define USART_REG_CSR_CTSIC_SIZE			1
/// USART Channel Status Register Clear to Send Input Change Flag mask
#define USART_REG_CSR_CTSIC_MASK			(((1 << USART_REG_CSR_CTSIC_SIZE) - 1) << USART_REG_CSR_CTSIC_OFFSET)
/// USART Channel Status Register Image of CTS Input offset
#define USART_REG_CSR_CTS_OFFSET			23
/// USART Channel Status Register Image of CTS Input size
#define USART_REG_CSR_CTS_SIZE				1
/// USART Channel Status Register Image of CTS Input mask
#define USART_REG_CSR_CTS_MASK				(((1 << USART_REG_CSR_CTS_SIZE) - 1) << USART_REG_CSR_CTS_OFFSET)
/// USART Channel Status Register Manchester Error offset
#define USART_REG_CSR_MANERR_OFFSET			24
/// USART Channel Status Register Manchester Error size
#define USART_REG_CSR_MANERR_SIZE			1
/// USART Channel Status Register Manchester Error mask
#define USART_REG_CSR_MANERR_MASK			(((1 << USART_REG_CSR_MANERR_SIZE) - 1) << USART_REG_CSR_MANERR_OFFSET)
/// USART Receiver Holding Register (Read)
#define USART_REG_RHR					0x00000018
/// USART Receiver Holding Register Received Character offset
#define USART_REG_RHR_RXCHR_OFFSET			0
/// USART Receiver Holding Register Received Character size
#define USART_REG_RHR_RXCHR_SIZE			9
/// USART Receiver Holding Register Received Character mask
#define USART_REG_RHR_RXCHR_MASK			(((1 << USART_REG_RHR_RXCHR_SIZE) - 1) << USART_REG_RHR_RXCHR_OFFSET)
/// USART Receiver Holding Register Received Sync offset
#define USART_REG_RHR_RXSYNH_OFFSET			15
/// USART Receiver Holding Register Received Sync size
#define USART_REG_RHR_RXSYNH_SIZE			1
/// USART Receiver Holding Register Received Sync mask
#define USART_REG_RHR_RXSYNH_MASK			(((1 << USART_REG_RHR_RXSYNH_SIZE) - 1) << USART_REG_RHR_RXSYNH_OFFSET)
/// USART Transmitter Holding Register (Write)
#define USART_REG_THR					0x0000001c
/// USART Transmitter Holding Register Transmit Character offset
#define USART_REG_THR_TXCHR_OFFSET			0
/// USART Transmitter Holding Register Transmit Character size
#define USART_REG_THR_TXCHR_SIZE			9
/// USART Transmitter Holding Register Transmit Character mask
#define USART_REG_THR_TXCHR_MASK			(((1 << USART_REG_THR_TXCHR_SIZE) - 1) << USART_REG_THR_TXCHR_OFFSET)
/// USART Transmitter Holding Register Transmit Sync offset
#define USART_REG_THR_TXSYNH_OFFSET			15
/// USART Transmitter Holding Register Transmit Sync size
#define USART_REG_THR_TXSYNH_SIZE			1
/// USART Transmitter Holding Register Transmit Sync mask
#define USART_REG_THR_TXSYNH_MASK			(((1 << USART_REG_THR_TXSYNH_SIZE) - 1) << USART_REG_THR_TXSYNH_OFFSET)
/// USART Baud Rate Generator Register (Read/Write)
#define USART_REG_BRGR					0x00000020
/// USART Baud Rate Generator Register Clock Divider offset
#define USART_REG_BRGR_CD_OFFSET			0
/// USART Baud Rate Generator Register Clock Divider size
#define USART_REG_BRGR_CD_SIZE				16
/// USART Baud Rate Generator Register Clock Divider mask
#define USART_REG_BRGR_CD_MASK				(((1 << USART_REG_BRGR_CD_SIZE) - 1) << USART_REG_BRGR_CD_OFFSET)
/// USART Baud Rate Generator Register Fractional Part offset
#define USART_REG_BRGR_FP_OFFSET			16
/// USART Baud Rate Generator Register Fractional Part size
#define USART_REG_BRGR_FP_SIZE				3
/// USART Baud Rate Generator Register Fractional Part mask
#define USART_REG_BRGR_FP_MASK				(((1 << USART_REG_BRGR_FP_SIZE) - 1) << USART_REG_BRGR_FP_OFFSET)
/// USART Receiver Time-out Register (Read/Write)
#define USART_REG_RTOR					0x00000024
/// USART Receiver Time-out Register Time-out Value offset
#define USART_REG_RTOR_TO_OFFSET			0
/// USART Receiver Time-out Register Time-out Value size
#define USART_REG_RTOR_TO_SIZE				16
/// USART Receiver Time-out Register Time-out Value mask
#define USART_REG_RTOR_TO_MASK				(((1 << USART_REG_RTOR_TO_SIZE) - 1) << USART_REG_RTOR_TO_OFFSET)
/// USART Transmitter Timeguard Register (Read/Write)
#define USART_REG_TTGR					0x00000028
/// USART Transmitter Timeguard Register Timeguard Value offset
#define USART_REG_TTGR_TG_OFFSET			0
/// USART Transmitter Timeguard Register Timeguard Value size
#define USART_REG_TTGR_TG_SIZE				8
/// USART Transmitter Timeguard Register Timeguard Value mask
#define USART_REG_TTGR_TG_MASK				(((1 << USART_REG_TTGR_TG_SIZE) - 1) << USART_REG_TTGR_TG_OFFSET)
/// USART FI DI Ratio Register (Read/Write)
#define USART_REG_FIDI					0x00000040
/// USART FI DI Ratio Register FI Over DI Ratio Value offset
#define USART_REG_FIDI_FI_DI_RATIO_OFFSET		0
/// USART FI DI Ratio Register FI Over DI Ratio Value size
#define USART_REG_FIDI_FI_DI_RATIO_SIZE			10
/// USART FI DI Ratio Register FI Over DI Ratio Value mask
#define USART_REG_FIDI_FI_DI_RATIO_MASK			(((1 << USART_REG_FIDI_FI_DI_RATIO_SIZE) - 1) << USART_REG_FIDI_FI_DI_RATIO_OFFSET)
/// USART Number of Errors Register (Read)
#define USART_REG_NER					0x00000044
/// USART Number of Errors Register Number of Errors offset
#define USART_REG_NER_NB_ERRORS_OFFSET			0
/// USART Number of Errors Register Number of Errors size
#define USART_REG_NER_NB_ERRORS_SIZE			8
/// USART Number of Errors Register Number of Errors mask
#define USART_REG_NER_NB_ERRORS_MASK			(((1 << USART_REG_NER_NB_ERRORS_SIZE) - 1) << USART_REG_NER_NB_ERRORS_OFFSET)
/// USART IrDA Filter Register (Read/Write)
#define USART_REG_IFR					0x0000004c
/// USART IrDA Filter Register IrDA Filter offset
#define USART_REG_IFR_IRDA_FILTER_OFFSET		0
/// USART IrDA Filter Register IrDA Filter size
#define USART_REG_IFR_IRDA_FILTER_SIZE			8
/// USART IrDA Filter Register IrDA Filter mask
#define USART_REG_IFR_IRDA_FILTER_MASK			(((1 << USART_REG_IFR_IRDA_FILTER_SIZE) - 1) << USART_REG_IFR_IRDA_FILTER_OFFSET)
/// USART Manchester Encoder Decoder Register (Read/Write)
#define USART_REG_MAN					0x00000050
/// USART Manchester Encoder Decoder Register Transmitter Preamble Length offset
#define USART_REG_MAN_TX_PL_OFFSET			0
/// USART Manchester Encoder Decoder Register Transmitter Preamble Length size
#define USART_REG_MAN_TX_PL_SIZE			4
/// USART Manchester Encoder Decoder Register Transmitter Preamble Length mask
#define USART_REG_MAN_TX_PL_MASK			(((1 << USART_REG_MAN_TX_PL_SIZE) - 1) << USART_REG_MAN_TX_PL_OFFSET)
/// USART Manchester Encoder Decoder Register Transmitter Preamble Pattern offset
#define USART_REG_MAN_TX_PP_OFFSET			8
/// USART Manchester Encoder Decoder Register Transmitter Preamble Pattern size
#define USART_REG_MAN_TX_PP_SIZE			2
/// USART Manchester Encoder Decoder Register Transmitter Preamble Pattern mask
#define USART_REG_MAN_TX_PP_MASK			(((1 << USART_REG_MAN_TX_PP_SIZE) - 1) << USART_REG_MAN_TX_PP_OFFSET)
/// USART Manchester Encoder Decoder Register Transmitter Manchester Polarity offset
#define USART_REG_MAN_TX_MPOL_OFFSET			12
/// USART Manchester Encoder Decoder Register Transmitter Manchester Polarity size
#define USART_REG_MAN_TX_MPOL_SIZE			1
/// USART Manchester Encoder Decoder Register Transmitter Manchester Polarity mask
#define USART_REG_MAN_TX_MPOL_MASK			(((1 << USART_REG_MAN_TX_MPOL_SIZE) - 1) << USART_REG_MAN_TX_MPOL_OFFSET)
/// USART Manchester Encoder Decoder Register Receiver Preamble Length offset
#define USART_REG_MAN_RX_PL_OFFSET			16
/// USART Manchester Encoder Decoder Register Receiver Preamble Length size
#define USART_REG_MAN_RX_PL_SIZE			4
/// USART Manchester Encoder Decoder Register Receiver Preamble Length mask
#define USART_REG_MAN_RX_PL_MASK			(((1 << USART_REG_MAN_RX_PL_SIZE) - 1) << USART_REG_MAN_RX_PL_OFFSET)
/// USART Manchester Encoder Decoder Register Receiver Preamble Pattern offset
#define USART_REG_MAN_RX_PP_OFFSET			24
/// USART Manchester Encoder Decoder Register Receiver Preamble Pattern size
#define USART_REG_MAN_RX_PP_SIZE			2
/// USART Manchester Encoder Decoder Register Receiver Preamble Pattern mask
#define USART_REG_MAN_RX_PP_MASK			(((1 << USART_REG_MAN_RX_PP_SIZE) - 1) << USART_REG_MAN_RX_PP_OFFSET)
/// USART Manchester Encoder Decoder Register Receiver Manchester Polarity offset
#define USART_REG_MAN_RX_MPOL_OFFSET			28
/// USART Manchester Encoder Decoder Register Receiver Manchester Polarity size
#define USART_REG_MAN_RX_MPOL_SIZE			1
/// USART Manchester Encoder Decoder Register Receiver Manchester Polarity mask
#define USART_REG_MAN_RX_MPOL_MASK			(((1 << USART_REG_MAN_RX_MPOL_SIZE) - 1) << USART_REG_MAN_RX_MPOL_OFFSET)
/// USART Manchester Encoder Decoder Register Drift compensation offset
#define USART_REG_MAN_DRIFT_OFFSET			30
/// USART Manchester Encoder Decoder Register Drift compensation size
#define USART_REG_MAN_DRIFT_SIZE			1
/// USART Manchester Encoder Decoder Register Drift compensation mask
#define USART_REG_MAN_DRIFT_MASK			(((1 << USART_REG_MAN_DRIFT_SIZE) - 1) << USART_REG_MAN_DRIFT_OFFSET)
/// USART Version Register (Read)
#define USART_REG_VERSION				0x000000fc
/// USART Version Register Version offset
#define USART_REG_VERSION_VERSION_OFFSET		0
/// USART Version Register Version size
#define USART_REG_VERSION_VERSION_SIZE			12
/// USART Version Register Version mask
#define USART_REG_VERSION_VERSION_MASK			(((1 << USART_REG_VERSION_VERSION_SIZE) - 1) << USART_REG_VERSION_VERSION_OFFSET)
/// USART Version Register Variant offset
#define USART_REG_VERSION_VARIANT_OFFSET		16
/// USART Version Register Variant size
#define USART_REG_VERSION_VARIANT_SIZE			3
/// USART Version Register Variant mask
#define USART_REG_VERSION_VARIANT_MASK			(((1 << USART_REG_VERSION_VARIANT_SIZE) - 1) << USART_REG_VERSION_VARIANT_OFFSET)

typedef enum {
	USART_RC_SUCCESS				= 0,
	USART_RC_FAILURE				= (1 << 0),
	USART_RC_RX_EMPTY				= (1 << 1) | USART_RC_FAILURE,
	USART_RC_RX_ERROR				= (1 << 2) | USART_RC_FAILURE,
	USART_RC_TX_BUSY				= (1 << 3) | USART_RC_FAILURE,
} usart_rc_t;

typedef enum {
	USART_MODE_NORMAL				= 0b0000 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_RS485				= 0b0001 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_HARDWARE_HANDSHAKING			= 0b0010 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_ISO07816_PROTOCOL_T0			= 0b0100 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_ISO07816_PROTOCOL_T1			= 0b0110 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_IRDA					= 0b1000 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_SPI_MASTER				= 0b1110 << USART_REG_MR_MODE_OFFSET,
	USART_MODE_SPI_SLAVE				= 0b1111 << USART_REG_MR_MODE_OFFSET,
} usart_mode_t;

typedef enum {
	USART_RS232_MODE_NORMAL_SYNC			= 0b0000 << USART_REG_MR_MODE_OFFSET | USART_REG_MR_SYNC_MASK,
	USART_RS232_MODE_NORMAL_ASYNC			= 0b0000 << USART_REG_MR_MODE_OFFSET,
	USART_RS232_MODE_HARDWARE_HANDSHAKING_SYNC	= 0b0010 << USART_REG_MR_MODE_OFFSET | USART_REG_MR_SYNC_MASK,
	USART_RS232_MODE_HARDWARE_HANDSHAKING_ASYNC	= 0b0010 << USART_REG_MR_MODE_OFFSET,
} usart_rs232_mode_t;

typedef enum {
	USART_CLOCK_MCK					= 0b00 << USART_REG_MR_USCLKS_OFFSET,
	USART_CLOCK_MCK_DIV				= 0b01 << USART_REG_MR_USCLKS_OFFSET,
	USART_CLOCK_CLK					= 0b11 << USART_REG_MR_USCLKS_OFFSET,
} usart_clock_t;

typedef enum {
	USART_SPI_CLOCK_PHASE_CAPTURE_FOLLOWING_EDGE	= 0,
	USART_SPI_CLOCK_PHASE_CAPTURE_LEADING_EDGE	= USART_REG_MR_CPHA_MASK,
} usart_spi_clock_phase_t;

typedef enum {
	USART_PARITY_EVEN				= 0b000 << USART_REG_MR_PAR_OFFSET,
	USART_PARITY_ODD				= 0b001 << USART_REG_MR_PAR_OFFSET,
	USART_PARITY_SPACE				= 0b010 << USART_REG_MR_PAR_OFFSET,
	USART_PARITY_MARK				= 0b011 << USART_REG_MR_PAR_OFFSET,
	USART_PARITY_NONE				= 0b100 << USART_REG_MR_PAR_OFFSET,
	USART_PARITY_MULTIDROP_MODE			= 0b110 << USART_REG_MR_PAR_OFFSET,
} usart_parity_t;

typedef enum {
	USART_STOPBITS_1				= 0,
	USART_STOPBITS_1_5				= 1,
	USART_STOPBITS_2				= 2,
	USART_STOPBITS_MAX				= 0xff + 2,
} usart_stopbits_t;

typedef enum {
	USART_CHANNEL_MODE_NORMAL			= 0b00,
	USART_CHANNEL_MODE_AUTOMATIC_ECHO		= 0b01,
	USART_CHANNEL_MODE_LOCAL_LOOPBACK		= 0b10,
	USART_CHANNEL_MODE_REMOTE_LOOPBACK		= 0b11,
} usart_channel_mode_t;

typedef enum {
	USART_BIT_ORDER_LSB_FIRST			= 0,
	USART_BIT_ORDER_MSB_FIRST			= USART_REG_MR_MSBF_MASK,
} usart_bit_order_t;

typedef enum {
	USART_OVERSAMPLING_X16				= 0,
	USART_OVERSAMPLING_X8				= USART_REG_MR_OVER_MASK,
} usart_oversampling_t;

extern uint32_t usart_read_register(void volatile *usart, uint32_t const offset);
extern void usart_write_register(void volatile *usart, uint32_t const offset, uint32_t const value);
extern void usart_write_register_field(void volatile *usart, uint32_t const offset, uint32_t const field_mask, uint32_t const value);

extern void usart_configure_rs232(void volatile *usart, usart_rs232_mode_t const mode, usart_channel_mode_t const channel_mode, usart_bit_order_t const bit_order, uint8_t const char_length, usart_stopbits_t const stopbits, usart_parity_t const parity, uint32_t const baud, uint32_t const pba_freq);

extern void usart_enable_rx(void volatile *usart);
extern void usart_disable_rx(void volatile *usart);
extern void usart_enable_tx(void volatile *usart);
extern void usart_disable_tx(void volatile *usart);
extern void usart_enable(void volatile *usart);
extern void usart_disable(void volatile *usart);

extern void usart_reset_status(void volatile *usart);

extern usart_rc_t usart_try_read_char(void volatile *usart, char *c);
extern usart_rc_t usart_read_char(void volatile *usart, char *c);

extern void usart_write_char(void volatile *usart, char const c);
extern void usart_write_char_auto_cr(void volatile *usart, char const c);
extern void usart_write_string(void volatile *usart, char const *str);

#endif /* USART_H_ */

