/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/adc.h>

uint32_t adc_read_register(uint32_t const offset) {
	uint32_t volatile const * const p = (uint32_t volatile *)(ADC_ADDRESS + offset);

	return *p;
}

void adc_write_register(uint32_t const offset, uint32_t const value) {
	uint32_t volatile * const p = (uint32_t volatile *)(ADC_ADDRESS + offset);

	*p = value;
}

void adc_sw_reset(void) {
	adc_write_register(ADC_REG_CR, ADC_REG_CR_SWRST_MASK);
}

void adc_configure(adc_hardware_trigger_mode_t const trigger_mode, adc_resolution_t const resolution, adc_sleep_mode_t const sleep_mode, adc_hardware_trigger_t const trigger, uint8_t const  prescal, uint8_t const startup, uint8_t const shtim) {
	adc_write_register(ADC_REG_MR, trigger_mode | resolution | sleep_mode |
			(trigger << ADC_REG_MR_TRGSEL_OFFSET) |
			(prescal << ADC_REG_MR_PRESCAL_OFFSET) |
			(startup << ADC_REG_MR_STARTUP_OFFSET) |
			(shtim << ADC_REG_MR_SHTIM_OFFSET));
}

void adc_enable_channel(uint8_t const channel_nb) {
	adc_write_register(ADC_REG_CHER, 1 << channel_nb);
}

void adc_disable_channel(uint8_t const channel_nb) {
	adc_write_register(ADC_REG_CHDR, 1 << channel_nb);
}

void adc_start(void) {
	adc_write_register(ADC_REG_CR, ADC_REG_CR_START_MASK);
}

void adc_wait(uint8_t const channel_nb) {
	while(!(adc_read_register(ADC_REG_SR) & (1 << channel_nb))) ;
}

adc_status_t adc_read_status(void) {
	return adc_read_register(ADC_REG_SR);
}

adc_channel_status_t adc_read_channel_status(void) {
	return adc_read_register(ADC_REG_CHSR);
}

uint16_t adc_read_value(void) {
	return (uint16_t)adc_read_register(ADC_REG_LCDR);
}

uint16_t adc_read_channel_value(uint8_t const channel_nb) {
	return (uint16_t)adc_read_register(ADC_REG_CDR0 + (4 * channel_nb));
}

void adc_enable_interrupts(adc_interrupts_t const interrupts) {
	adc_write_register(ADC_REG_IER, interrupts);
}

void adc_disable_interrupts(adc_interrupts_t const interrupts) {
	adc_write_register(ADC_REG_IDR, interrupts);
}

adc_interrupts_t adc_read_interrupts_mask(void) {
	return adc_read_register(ADC_REG_IMR);
}

