/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/exceptions.h>
#include <at32uc3/sysreg.h>

#include "evba.h"

#include "exceptionhandlers_common.c"

extern void _exceptions_handle_unrecoverable_exception(void) GCC_NAKED;
extern void _exceptions_handle_tlb_multiple_hit(void) GCC_NAKED;
extern void _exceptions_handle_bus_error_data_fetch(void) GCC_NAKED;
extern void _exceptions_handle_bus_error_instruction_fetch(void) GCC_NAKED;
extern void _exceptions_handle_nmi(void) GCC_NAKED;
extern void _exceptions_handle_instruction_address(void) GCC_NAKED;
extern void _exceptions_handle_itlb_protection(void) GCC_NAKED;
extern void _exceptions_handle_breakpoint(void) GCC_NAKED;
extern void _exceptions_handle_illegal_opcode(void) GCC_NAKED;
extern void _exceptions_handle_unimplemented_instruction(void) GCC_NAKED;
extern void _exceptions_handle_privilege_violation(void) GCC_NAKED;
extern void _exceptions_handle_data_address_read(void) GCC_NAKED;
extern void _exceptions_handle_data_address_write(void) GCC_NAKED;
extern void _exceptions_handle_dtlb_protection_read(void) GCC_NAKED;
extern void _exceptions_handle_dtlb_protection_write(void) GCC_NAKED;
extern void _exceptions_handle_itlb_miss(void) GCC_NAKED;
extern void _exceptions_handle_dtlb_miss_read(void) GCC_NAKED;
extern void _exceptions_handle_dtlb_miss_write(void) GCC_NAKED;
extern void _exceptions_handle_supervisor_call(void) GCC_NAKED;

void _exceptions_handle_unrecoverable_exception(void) {
	if(unrecoverable_exception_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (unrecoverable_exception_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_tlb_multiple_hit(void) {
	if(tlb_multiple_hit_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (tlb_multiple_hit_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_bus_error_data_fetch(void) {
	if(bus_error_data_fetch_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (bus_error_data_fetch_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_bus_error_instruction_fetch(void) {
	if(bus_error_instruction_fetch_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (bus_error_instruction_fetch_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_nmi(void) {
	if(nmi_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (nmi_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_instruction_address(void) {
	if(instruction_address_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (instruction_address_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_itlb_protection(void) {
	if(itlb_protection_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (itlb_protection_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_breakpoint(void) {
	if(breakpoint_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (breakpoint_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_illegal_opcode(void) {
	if(illegal_opcode_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (illegal_opcode_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_unimplemented_instruction(void) {
	if(unimplemented_instruction_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (unimplemented_instruction_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_privilege_violation(void) {
	if(privilege_violation_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (privilege_violation_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_data_address_read(void) {
	if(data_address_read_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (data_address_read_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_data_address_write(void) {
	if(data_address_write_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (data_address_write_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_dtlb_protection_read(void) {
	if(dtlb_protection_read_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (dtlb_protection_read_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_dtlb_protection_write(void) {
	if(dtlb_protection_write_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (dtlb_protection_write_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_itlb_miss(void) {
	if(itlb_miss_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (itlb_miss_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_dtlb_miss_read(void) {
	if(dtlb_miss_read_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (dtlb_miss_read_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_dtlb_miss_write(void) {
	if(dtlb_miss_write_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (dtlb_miss_write_handler)
		);
	else
		asm volatile("rjmp    $\n");
}

void _exceptions_handle_supervisor_call(void) {
	if(supervisor_call_handler)
		asm volatile(
			"rjmp    %[handler]\n"
			:
			: [handler] "m" (supervisor_call_handler)
		);
	else
		asm volatile("rjmp    $\n");
}
