/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/gccmacros.h>
#include <at32uc3/interrupts.h>
#include <at32uc3/sysreg.h>

#include "../exceptions/evba.h"

static uint16_t const autovectors[] = {
	[INTC_INTLEVEL_INT0] = EVBA_OFFSET_INTERRUPT0_REQUEST,
	[INTC_INTLEVEL_INT1] = EVBA_OFFSET_INTERRUPT1_REQUEST,
	[INTC_INTLEVEL_INT2] = EVBA_OFFSET_INTERRUPT2_REQUEST,
	[INTC_INTLEVEL_INT3] = EVBA_OFFSET_INTERRUPT3_REQUEST,
};

void interrupts_enable(interrupts_mask_t const mask) {
	sysreg_clear(SYSREG_SR_NO, mask);
}

interrupts_mask_t interrupts_enabled(interrupts_mask_t const mask) {
	return sysreg_read(SYSREG_SR_NO) & mask;
}

void interrupts_disable(interrupts_mask_t const mask) {
	sysreg_set(SYSREG_SR_NO, mask);
}

interrupts_mask_t interrupts_read_mask(void) {
	return sysreg_read(SYSREG_SR_NO) & INTERRUPTS_MASK_ALL;
}

void interrupts_connect_group(intc_group_t const group, intc_intlevel_t const intlevel) {
	intc_connect(group, intlevel, autovectors[intlevel]);
}

