/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/pdc.h>

uint32_t pdc_channel_read_register(void volatile *channel, uint32_t const offset) {
	uint32_t const volatile * const p = (uint32_t *)(channel + offset);

	return *p;
}

void pdc_channel_write_register(void volatile *channel, uint32_t const offset, uint32_t const value) {
	uint32_t volatile * const p = (uint32_t *)(channel + offset);

	*p = value;
}

pdc_pid_t pdc_channel_get_pid(void volatile *channel) {
	return (pdc_pid_t)pdc_channel_read_register(channel, PDC_CHANNEL_REG_PSR);
}

void pdc_channel_set_pid(void volatile *channel, pdc_pid_t const pid) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_PSR, pid << 0/*PDC_CHANNEL_REG_PSR_PID_OFFSET*/);
}

pdc_transfer_size_t pdc_channel_get_transfer_size(void volatile *channel) {
	return (pdc_transfer_size_t)pdc_channel_read_register(channel, PDC_CHANNEL_REG_MR);
}

void pdc_channel_set_transfer_size(void volatile *channel, pdc_transfer_size_t const size) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_MR, size << 0/*PDC_CHANNEL_REG_MR_SIZE_OFFSET*/);
}

uint32_t pdc_channel_get_memory_address(void volatile *channel) {
	return pdc_channel_read_register(channel, PDC_CHANNEL_REG_MAR);
}

void pdc_channel_set_memory_address(void volatile *channel, uint32_t const address) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_MAR, address);
}

uint32_t pdc_channel_get_reload_memory_address(void volatile *channel) {
	return pdc_channel_read_register(channel, PDC_CHANNEL_REG_MARR);
}

void pdc_channel_set_reload_memory_address(void volatile *channel, uint32_t const address) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_MARR, address);
}

uint16_t pdc_channel_get_transfer_count(void volatile *channel) {
	return (uint16_t)pdc_channel_read_register(channel, PDC_CHANNEL_REG_TCR);
}

void pdc_channel_set_transfer_count(void volatile *channel, uint16_t const count) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_TCR, count << 0/*PDC_CHANNEL_REG_TCR_TCV_OFFSET*/);
}

uint16_t pdc_channel_get_reload_transfer_count(void volatile *channel) {
	return (uint16_t)pdc_channel_read_register(channel, PDC_CHANNEL_REG_TCRR);
}

void pdc_channel_set_reload_transfer_count(void volatile *channel, uint16_t const count) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_TCRR, count << 0/*PDC_CHANNEL_REG_TCRR_TCRV_OFFSET*/);
}

void pdc_channel_configure(void volatile *channel, pdc_pid_t const pid, pdc_transfer_size_t const size) {
	pdc_channel_set_pid(channel, pid);
	pdc_channel_set_transfer_size(channel, size);
}

void pdc_channel_load(void volatile *channel, uint32_t const address, uint16_t const count) {
	pdc_channel_set_memory_address(channel, address);
	pdc_channel_set_transfer_count(channel, count);

}

void pdc_channel_load2(void volatile *channel, uint32_t const address, uint16_t const count, uint32_t const reload_address, uint16_t const reload_count) {
	pdc_channel_set_memory_address(channel, address);
	pdc_channel_set_transfer_count(channel, count);
	pdc_channel_set_reload_memory_address(channel, reload_address);
	pdc_channel_set_reload_transfer_count(channel, reload_count);
}

void pdc_channel_reload(void volatile *channel, uint32_t const address, uint16_t const count) {
	pdc_channel_set_reload_memory_address(channel, address);
	pdc_channel_set_reload_transfer_count(channel, count);
}

void pdc_channel_enable_transfer(void volatile *channel) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_CR, PDC_CHANNEL_REG_CR_TEN_MASK);
}

void pdc_channel_disable_transfer(void volatile *channel) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_CR, PDC_CHANNEL_REG_CR_TDIS_MASK);
}

uint8_t pdc_channel_is_transfer_enabled(void volatile *channel) {
	return pdc_channel_read_register(channel, PDC_CHANNEL_REG_SR);
}

void pdc_channel_clear_error(void volatile *channel) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_CR, PDC_CHANNEL_REG_CR_ECLR_MASK);
}

void pdc_channel_enable_interrupts(void volatile *channel, pdc_interrupts_t const interrupts) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_IER, interrupts);
}

void pdc_channel_disable_interrupts(void volatile *channel, pdc_interrupts_t const interrupts) {
	pdc_channel_write_register(channel, PDC_CHANNEL_REG_IDR, interrupts);
}

pdc_interrupts_t pdc_channel_get_interrupt_mask(void volatile *channel) {
	return pdc_channel_read_register(channel, PDC_CHANNEL_REG_IMR);
}

pdc_interrupts_t pdc_channel_get_interrupt_status(void volatile *channel) {
	return pdc_channel_read_register(channel, PDC_CHANNEL_REG_ISR);
}
