/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/sysreg.h>

uint32_t sysreg_read(uint8_t const sysreg_no) {
	uint32_t value;

	asm volatile(
		"mov     r8, lo(0f)\n"
		"orh     r8, hi(0f)\n"
		"st.b    r8[0x03], %[sysreg_no]\n"
		"rjmp    0f\n"
		"0:\n"
		"mfsr    %[value], 0\n"
		: [value] "=r" (value)
		: [sysreg_no] "r" (sysreg_no)
		: "r8"
	);

	return value;
}

void sysreg_write(uint8_t const sysreg_no, uint32_t const value) {
	asm volatile(
		"mov     r8, lo(0f)\n"
		"orh     r8, hi(0f)\n"
		"st.b    r8[0x03], %[sysreg_no]\n"
		"rjmp    0f\n"
		"0:\n"
		"mtsr    0, %[value]\n"
		:
		: [sysreg_no] "r" (sysreg_no), [value] "r" (value)
		: "r8"
	);
}

void sysreg_set(uint8_t const sysreg_no, uint32_t const mask) {
	uint32_t const value = sysreg_read(sysreg_no);

	sysreg_write(sysreg_no, value | mask);
}

void sysreg_clear(uint8_t const sysreg_no, uint32_t const mask) {
	uint32_t const value = sysreg_read(sysreg_no);

	sysreg_write(sysreg_no, value & ~mask);
}
