/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#include <at32uc3/tc.h>

uint32_t tc_read_register(uint32_t const offset) {
	uint32_t const volatile * const p = (uint32_t *)(TC_ADDRESS + offset);

	return *p;
}

void tc_write_register(uint32_t const offset, uint32_t const value) {
	uint32_t volatile * const p = (uint32_t *)(TC_ADDRESS + offset);

	*p = value;
}

void tc_write_register_field(uint32_t const offset, uint32_t const field_mask, uint32_t const value) {
        uint32_t const tmp = tc_read_register(offset);

        tc_write_register(offset, (tmp & ~field_mask) | value);
}

void tc_select_external_clock0(tc_external_clock0_t const external_clock0) {
	tc_write_register_field(TC_REG_BMR, TC_REG_BMR_TC0XC0S_MASK, external_clock0);
}

void tc_select_external_clock1(tc_external_clock1_t const external_clock1) {
	tc_write_register_field(TC_REG_BMR, TC_REG_BMR_TC1XC1S_MASK, external_clock1);
}

void tc_select_external_clock2(tc_external_clock2_t const external_clock2) {
	tc_write_register_field(TC_REG_BMR, TC_REG_BMR_TC2XC2S_MASK, external_clock2);
}

void tc_configure_external_clocks(tc_external_clock0_t const external_clock0, tc_external_clock1_t const external_clock1, tc_external_clock2_t const external_clock2) {
	tc_write_register(TC_REG_BMR,
		external_clock0 | external_clock1 | external_clock2
	);
}

void tc_sync(void) {
	tc_write_register(TC_REG_BCR, TC_REG_BCR_SYNC_MASK);
}

uint32_t tc_channel_read_register(void volatile *channel, uint32_t const offset) {
	uint32_t const volatile * const p = (uint32_t *)(channel + offset);

	return *p;
}

void tc_channel_write_register(void volatile *channel, uint32_t const offset, uint32_t const value) {
	uint32_t volatile * const p = (uint32_t *)(channel + offset);

	*p = value;
}

void tc_channel_write_register_field(void volatile *channel, uint32_t const offset, uint32_t const field_mask, uint32_t const value) {
        uint32_t const tmp = tc_channel_read_register(channel, offset);

        tc_channel_write_register(channel, offset, (tmp & ~field_mask) | value);
}

void tc_channel_reset(void volatile *channel) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_CCR, TC_CHANNEL_REG_CCR_SWTRG_MASK);
}

void tc_channel_enable_clock(void volatile *channel) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_CCR, TC_CHANNEL_REG_CCR_CLKEN_MASK);
}

void tc_channel_disable_clock(void volatile *channel) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_CCR, TC_CHANNEL_REG_CCR_CLKDIS_MASK);
}

void tc_channel_reset_and_enable_clock(void volatile *channel) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_CCR, TC_CHANNEL_REG_CCR_SWTRG_MASK | TC_CHANNEL_REG_CCR_CLKEN_MASK);
}

void tc_channel_select_capture_mode(void volatile *channel) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_WAVE_MASK, 0);
}

void tc_channel_select_clock(void volatile *channel, tc_channel_clock_t const clock) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_TCCLKS_MASK, clock);
}
void tc_channel_select_burst_signal(void volatile *channel, tc_channel_burst_signal_t const burst) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_BURST_MASK, burst);
}

void tc_channel_select_ra_load_edge(void volatile *channel, tc_channel_ra_load_edge_t const ra_load_edge) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_LDRA_MASK, ra_load_edge);
}

void tc_channel_select_rb_load_edge(void volatile *channel, tc_channel_rb_load_edge_t const rb_load_edge) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_LDRB_MASK, rb_load_edge);
}

void tc_channel_select_rb_load_action(void volatile *channel, tc_channel_rb_load_action_t const rb_load_action) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_LDBSTOP_MASK | TC_CHANNEL_REG_CMR_CM_LDBDIS_MASK, rb_load_action);
}

void tc_channel_select_rc_compare_trigger(void volatile *channel, tc_channel_rc_compare_trigger_t const rc_compare_trigger) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_CPCTRG_MASK, rc_compare_trigger);
}

void tc_channel_select_external_trigger(void volatile *channel, tc_channel_external_trigger_t const external_trigger) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_ABETRG_MASK, external_trigger);
}

void tc_channel_select_external_trigger_edge(void volatile *channel, tc_channel_external_trigger_edge_t const external_trigger_edge) {
	tc_channel_write_register_field(channel, TC_CHANNEL_REG_CMR, TC_CHANNEL_REG_CMR_CM_ETRGEDG_MASK, external_trigger_edge);
}

uint16_t tc_channel_read_register_a(void volatile *channel) {
	tc_channel_read_register(channel, TC_CHANNEL_REG_RA);
}

void tc_channel_write_register_a(void volatile *channel, uint16_t const value) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_RA, value);
}

uint16_t tc_channel_read_register_b(void volatile *channel) {
	tc_channel_read_register(channel, TC_CHANNEL_REG_RB);
}

void tc_channel_write_register_b(void volatile *channel, uint16_t const value) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_RB, value);
}

uint16_t tc_channel_read_register_c(void volatile *channel) {
	tc_channel_read_register(channel, TC_CHANNEL_REG_RC);
}

void tc_channel_write_register_c(void volatile *channel, uint16_t const value) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_RC, value);
}

uint16_t tc_channel_read_counter_value(void volatile *channel) {
	tc_channel_read_register(channel, TC_CHANNEL_REG_CV);
}

tc_channel_status_t tc_channel_read_status(void volatile *channel) {
	tc_channel_read_register(channel, TC_CHANNEL_REG_SR);
}

void tc_channel_configure_capture_mode(void volatile *channel, tc_channel_clock_t const clock, tc_channel_burst_signal_t const burst, tc_channel_ra_load_edge_t const ra_load_edge, tc_channel_rb_load_edge_t const rb_load_edge, tc_channel_rb_load_action_t const rb_load_action, tc_channel_rc_compare_trigger_t const rc_compare_trigger, tc_channel_external_trigger_t const external_trigger, tc_channel_external_trigger_edge_t const external_trigger_edge, uint16_t const rc_value) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_CMR,
		clock | burst |
		ra_load_edge | rb_load_edge |
		rb_load_action |
		rc_compare_trigger |
		external_trigger | external_trigger_edge
	);
	tc_channel_write_register_c(channel, rc_value);
}

void tc_channel_enable_capture_mode(void volatile *channel, tc_channel_clock_t const clock, tc_channel_burst_signal_t const burst, tc_channel_ra_load_edge_t const ra_load_edge, tc_channel_rb_load_edge_t const rb_load_edge, tc_channel_rb_load_action_t const rb_load_action, tc_channel_rc_compare_trigger_t const rc_compare_trigger, tc_channel_external_trigger_t const external_trigger, tc_channel_external_trigger_edge_t const external_trigger_edge, uint16_t const rc_value) {
	tc_channel_configure_capture_mode(channel, clock, burst, ra_load_edge, rb_load_edge, rb_load_action, rc_compare_trigger, external_trigger, external_trigger_edge, rc_value);
	tc_channel_reset_and_enable_clock(channel);
}

void tc_channel_enable_interrupts(void volatile *channel, tc_channel_interrupts_t const interrupts) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_IER, interrupts);
}

void tc_channel_disable_interrupts(void volatile *channel, tc_channel_interrupts_t const interrupts) {
	tc_channel_write_register(channel, TC_CHANNEL_REG_IDR, interrupts);
}

tc_channel_interrupts_t tc_channel_read_interrupt_mask(void volatile *channel) {
	return tc_channel_read_register(channel, TC_CHANNEL_REG_IER);
}
