/* graphs.c  - produces graphs used by EDAcS */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <gd.h>
#include <gdfontt.h>
#include <gdfonts.h>
#include <gdfontmb.h>
#include "../edacs.h"
#include "graphs.h"

#define COLOR1 green                       /* graph color - hits  */
#define COLOR2 blue                        /* files               */
#define COLOR3 orange                      /* sites               */
#define COLOR4 red                         /* KBytes              */
#define COLOR5 cyan                        /* Files               */
#define COLOR6 yellow                      /* Visits              */
#define CY 145                             /* center y  (chart)   */
#define INDEX_LINES 4

/* forward reference internal routines */
void    init_graph(char *);

/* common public declarations */
char *numchar[] = { " 0"," 1"," 2"," 3"," 4"," 5"," 6"," 7"," 8"," 9","10",
                    "11","12","13","14","15","16","17","18","19","20",
                    "21","22","23","24","25","26","27","28","29","30","31"};

gdImagePtr	imgbuf;                    /* image buffer        */
FILE		*pngout;                   /* output file for PNG */
char		maxvaltxt[32];             /* graph values        */
float		percent;                   /* percent storage     */
u_long		julday;                    /* julday value        */

/* colors */
int		black, white, grey, dkgrey, red, dkblue, dkgreen,
                blue, orange, green, cyan, yellow;

/*****************************************************************/
/*                                                               */
/* con_perday_y  - Year graph connections/day                    */
/*                                                               */
/*****************************************************************/

int con_perday_y(  char *fname,            /* file name use      */
                   char *title,            /* title for graph    */
                    int fmonth,            /* begin month number */
                 u_long data1[18])         /* data1 (connections)*/
{

   /* local variables */
   int i,x1,y1,x2;
   int s_mth;

   u_long maxval=1;
   char * msg_xlegend = "Months";
   char * msg_ylegend = "Connections";
   /* short month names MUST BE 3 CHARS in size... pad if needed*/
   static char *s_month[12]={"Jan", "Feb", "Mar", "Apr",
                             "May", "Jun", "Jul", "Aug",
                             "Sep", "Oct", "Nov", "Dec"};


   /* initalize the graph */
   init_graph(title);              /* init as 512 x 256  */

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=0;i<INDEX_LINES;i++)
     gdImageLine(imgbuf,20,((i+1)*y1)+25,380,((i+1)*y1)+25,dkgrey);

   /* draw the x-axis legend */
   s_mth = fmonth;
   /* 18 is how many x-axis values we will display */
   for (i=0;i<18;i++) {
      /* 23 is the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf, gdFontSmall, 23+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) s_month[s_mth-1], black);
      if(strcmp(s_month[s_mth-1], "Dec") == 0) {
        gdImageLine(imgbuf, 23+(i*20)+18, 20, 23+(i*20)+18, 132, dkgrey);
      }
      s_mth++;
      if (s_mth > 12) s_mth = 1;
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   if (maxval <= 0) maxval = 1;
   sprintf(maxvaltxt, "%lu", maxval);
   gdImageStringUp(imgbuf,gdFontSmall,6,20+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf,gdFontMediumBold,388-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   s_mth = fmonth;
   for (i=0; i<18; i++)
   {
      if (s_mth > 12) s_mth = 1;
      percent = ((float)data1[17-i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = 25 + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf, x1, y1, x2, 132, black);
   }

   maxval=0;
   for (i=0; i<12; i++)
   {
       if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }
   if (maxval <= 0) maxval = 1;
   sprintf(maxvaltxt, "%lu", maxval);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL)
   {
      gdImagePng(imgbuf, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf);

   return (0);
}

/*****************************************************************/
/*                                                               */
/* INIT_GRAPH  - initalize graph and draw borders                */
/*                                                               */
/*****************************************************************/

void init_graph(char *title)
{
   int i;
   int xsize = 400;
   int ysize = 160;

   imgbuf = gdImageCreate(xsize,ysize);

   /* allocate color maps, background color first (grey) */
   grey    = gdImageColorAllocate(imgbuf, 204, 204, 204);
   dkgreen = gdImageColorAllocate(imgbuf, 0, 102, 102);
   dkblue  = gdImageColorAllocate(imgbuf, 0, 0, 153);
   white   = gdImageColorAllocate(imgbuf, 255, 255, 255);

   dkgrey  = gdImageColorAllocate(imgbuf, 128, 128, 128);
   black   = gdImageColorAllocate(imgbuf, 0, 0, 0);

   orange  = gdImageColorAllocate(imgbuf, 255, 128, 0);
   blue    = gdImageColorAllocate(imgbuf, 0, 0, 255);
   red     = gdImageColorAllocate(imgbuf, 255, 0, 0);
   cyan    = gdImageColorAllocate(imgbuf, 0, 192, 255);
   yellow  = gdImageColorAllocate(imgbuf, 255, 255, 0);

   /* make borders */

   /* makes shadow effect around the image, 2 pixels wide */
   for (i=0; i<2 ;i++) { /* do shadow effect around the image, 2 pixels wide */
      gdImageLine(imgbuf, i, i, xsize-i, i, white);
      gdImageLine(imgbuf, i, i, i, ysize-i, white);
      gdImageLine(imgbuf, i, ysize-i-1, xsize-i-1, ysize-i-1, dkgrey);
      gdImageLine(imgbuf, xsize-i-1, i, xsize-i-1, ysize-i-1, dkgrey);
   }

   /* draw the inner frame around the data, 2 pixels wide */
   gdImageRectangle(imgbuf, 19, 20, xsize-19, ysize-27, black);
   //gdImageRectangle(imgbuf, 20, 21, xsize-20, ysize-28, white);
   gdImageFilledRectangle(imgbuf, 20, 21, xsize-20, ysize-28, white);

   /* draw the outermost black frame line around the image, 1 pixel wide */
   gdImageRectangle(imgbuf, 0, 0, xsize-1, ysize-1, black);

   /* display the graph title */
   gdImageString(imgbuf, gdFontMediumBold, 24, 5, (unsigned char*) title, dkblue);

   return;
}

