/*
 * -----------------------------------------------------------------------------
 * file:        graphtest.c
 * purpose:     test making graph from SQL query, writes a png file
 * author:      04/23/2006 Frank4DD
 *
 * Requirements: EDAcS DB with data in edacs_monthstats
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sqlora.h>
#include <edacs.h>
#include "graphs.h"

/**********************************************************/
/* adjust these parameters to get connectivity to Oracle  */
/* The Defines are equivalent to their counterpart in     */
/* edacs.h (removing the MY_), so when this program works */
/* the variables in edacs.h can be updated and the main   */
/* application will equally work.                         */
/**********************************************************/
#define MY_ORACLEHOME_ENV "ORACLE_HOME=/var/edacsdb/oracle"
#define MY_TNS_STRING "edacsadmin/mypasswdhere@work1"
/**********************************************************/
/*      No need to adjust anything below this line        */
/**********************************************************/

#define error_exit(handle, msg)  do_error_exit(handle, __FILE__, __LINE__, msg)
void do_error_exit(sqlo_db_handle_t dbh, const char * file,
                                 int line, const char * msg);

/* already defined in edacs.h: */
/* static int _abort_flag = 0; */

/* functions */
void sigint_handler(void) {
  printf("Catched SIGINT\n");
  _abort_flag++;
}

int col_count(sqlo_db_handle_t dbh, char * table_name) {
  int stat;
  if ( 0 > (stat = sqlo_count(dbh, "USER_TAB_COLUMNS", "TABLE_NAME",
                              table_name, NULL)))
    error_exit(dbh, "sqlo_count");
  return stat;
}

/* print error and exit with failure */
void do_error_exit(sqlo_db_handle_t dbh, const char * file,
                                 int line, const char * msg) {
  printf("%s (line: %d):\n%s: %s\n", file, line, msg, sqlo_geterror(dbh));
  sqlo_rollback(dbh);
  sqlo_finish(dbh);
  exit(EXIT_FAILURE);
}

int cgiMain() {

  sqlo_db_handle_t dbh;		/* database handle */
  int stat;                     /* status of sqlo calls */
  sqlo_stmt_handle_t sth1;      /* statement handle 1 */
  char server_version[1024];	/* string for returned server version */
  char sqlquery_str[1024];	/* SQL query string */
  int handle;                   /* handle of the interrupt handler */
  const char ** values;         /* values */
  const char ** n;              /* column names */
  CONST int *nl;                /* column name lengths */
  CONST unsigned short *vl;     /* value lengths */
  int colnum;                   /* number of columns */
  unsigned int rowcount = 0;	/* rowcounter */
  u_long year_connects[] = {'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};

  /***********************************************************************/
  /* ORACLE_HOME is needed for OCI8 to find tnsnames.ora and other stuff */
  /***********************************************************************/
  putenv(MY_ORACLEHOME_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100)) {
    printf("Failed to init libsqlora8\n");
    return EXIT_FAILURE;
  }

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, MY_TNS_STRING)) {
    printf("Cannot login with %s\n", MY_TNS_STRING);
    return EXIT_FAILURE;
  }
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  if (SQLO_SUCCESS != sqlo_server_version(dbh, server_version,
                                        sizeof(server_version))) {
    printf("Failed to get the server version: %s\n", sqlo_geterror(dbh));
    return EXIT_FAILURE;
  }
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  printf("---------------------------------------------------------------------\n");
  printf("Connected to: %s\n", server_version);

  /* enable autocommit, each statement is committet as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT MONTH, CONNECTS FROM edacsadmin.EDACS_MONSTATS WHERE ROUTER='192.168.247.253' ORDER BY MONTH DESC");

  sth1 = SQLO_STH_INIT;
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
    error_exit(dbh, "sqlo_open");

  printf("---------------------------------------------------------------------\n");
  printf("Select from table: %s\n", sqlquery_str);
  printf("---------------------------------------------------------------------\n");
  /* get the output column names */
  n = sqlo_ocol_names(sth1, &colnum);

  /* get the output column name lengths */
  nl = sqlo_ocol_name_lens(sth1, NULL);

  printf("number of output columns: %d \n\n", colnum);

  /* print the table column header(s) */
  printf("Month\tConns\n");
  printf("---------------------------------------------------------------------\n");

  /* fetch the data */
  while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {

    /* get one record */
    values = sqlo_values(sth1, NULL, 1);

    /* get the length of the data items */
    vl = sqlo_value_lens(sth1, NULL);


    if(rowcount <18) { 
      /* print the rows */
      printf("%d\t%s\t%s\n", rowcount, values[0], values[1]);
      year_connects[rowcount] = atoi(values[1]);
    }

    rowcount++;
  }

  if (0 > stat)
    error_exit(dbh, "sqlo_fetch");

  if ( SQLO_SUCCESS != sqlo_close(sth1))
    error_exit(dbh, "sqlo_close");
  printf("---------------------------------------------------------------------\n\n");

   con_perday_y(   "connections.png",   /* filename          */
                   "routername",        /* graph title       */
                   1+7,                 /* calculate begin month, current + 7 */
                   year_connects);    

return 1;
}
