/* -------------------------------------------------------------------------- *
 * file:        mysqltest2.c                                                  *
 * purpose:     tests SQL query of a MySQL database with libmysqlclient       *
 * author:      05/15/2010 Frank4DD                                           *
 *                                                                            *
 * gcc -O3 -Wall -g -Wall   -c -o mysqltest.o mysqltest.c                     *
 *                                                                            *
 *  use -I/path-to-include for a nonstd location of libmysqlclient headers    * 
 *                                                                            *
 * gcc -lclntsh mysqltest.o  -o mysqltest                                     *
 *                                                                            *
 *  use -L/path-to-lib for a nonstd location of libmysqlclient libraries      * 
 * -------------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mysql/mysql.h>

/**********************************************************/
/* adjust below parameters to get a connection to MYSQL . */
/* If this program works the application will also work.  */
/**********************************************************/
#define MYSQLIP "127.0.0.1"
#define DB_PORT 3306
#define DB_NAME "edacs"
#define DB_USER "edacsadmin"
#define DB_PASS "mypasswdhere"
/**********************************************************/
/*      No need to adjust anything below this line        */
/**********************************************************/

#define error_exit(handle, msg)  do_error_exit(handle, __FILE__, __LINE__, msg)
void do_error_exit(MYSQL *dbh, const char * file, int line, const char * msg);

/* print error and exit with failure */
void do_error_exit(MYSQL *dbh, const char * file, int line, const char * msg) {
  printf("%s (line: %d):\n%s", file, line, msg);
  printf("MySQL Error %u: %s\n", mysql_errno(dbh), mysql_error(dbh));
  mysql_close(dbh);
  exit(-1);
}

int main() {

  static MYSQL *dbh;		/* database connect handle */
  static MYSQL_RES *result1, *result2;     /* database query results  */
  static MYSQL_FIELD *fields;   /* column names returned   */
  static MYSQL_ROW row;		/* query data returned     */
  char   error_str[4096];	/* error string to return  */
  int    server_version;        /* server version received */
  unsigned int col_count;       /* number of columns       */
  unsigned int fieldlength=0;   /* column length           */
  unsigned int i, j;		/* loop variable, why is it always i? */
  //char insert_str[4096] = "INSERT INTO edacs_templog (action, router, username, action_date, bytes_in, bytes_out, ip_addr, tty, protocol, packets_in, packets_out, task_id, service, ip_or_phone) VALUES ('Start', '192.168.89.11', 'kschlieg', date_format('2008-08-15 01:57:35', '%Y-%m-%d %H:%i:%s'), 740120, 17082256, '192.168.91.1', '1048','PPP', 10665, 25044, 'D100005D', 'Framed-User', '192.168.42.175')";
  // static char insert_str[] = "INSERT IGNORE INTO edacs_users (username, first_connect, last_connect) VALUES ('kschlwerwgrwer', DATE_FORMAT('2008-08-15 01:57:35', '%Y-%m-%d %H:%i:%s'), DATE_FORMAT('2008-08-16 14:57:35', '%Y-%m-%d %H:%i:%s'))";
  // static char query_str[] = "SELECT * FROM edacs_users";
  // static char delete_str[] = "DELETE FROM edacs_users";
  static char insert_str1[] = "INSERT INTO edacs_templog (action, router, username, action_date, bytes_in, bytes_out, ip_addr, tty, protocol, packets_in, packets_out, task_id, service, ip_or_phone) VALUES ('Start', '192.168.247.253', 'bmuelri2', DATE_FORMAT(sysdate(), '%Y-%m-%d %H:%i:%s'), 0, 0, '192.168.237.36', '20002', 'PPP', 0, 0, '00000F0E', 'Framed-User', '006587049666')";
  static char insert_str2[] = "INSERT INTO edacs_templog (action, router, username, action_date, bytes_in, bytes_out, ip_addr, tty, protocol, packets_in, packets_out, task_id, service, ip_or_phone) VALUES ('Update', '192.168.247.253', 'bmuelri2', DATE_FORMAT(sysdate(), '%Y-%m-%d %H:%i:%s'), 0, 0, '192.168.237.14', '20002', 'PPP', 0, 0, '00000F0E', 'Framed-User', '006587049666')";
  static char insert_str3[] = "INSERT INTO edacs_templog (action, router, username, action_date, bytes_in, bytes_out, ip_addr, tty, protocol, packets_in, packets_out, task_id, service, ip_or_phone) VALUES ('Stop', '192.168.247.253', 'bmuelri2', DATE_FORMAT(DATE_ADD(sysdate(), INTERVAL 1 MINUTE), '%Y-%m-%d %H:%i:%s'), 122921, 337046, '192.168.237.14', '20002', 'PPP', 5509, 3414, '00000F0E', 'Framed-User', '006587049666')";
  static char query_str1[] = "SELECT * FROM edacs_templog";
  static char query_str2[] = "SELECT * FROM edacs_mainlog";

  /* initialize the connection handler */
  dbh = mysql_init(NULL);
  if(dbh == NULL) {
    printf("Error - mysql_init failed\n");
  }

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, DB_USER, DB_PASS, DB_NAME, DB_PORT, NULL, 0) == 0) {
    snprintf(error_str, sizeof(error_str), "Cannot login to %s:%d db %s with user %s pass %s\n", 
           MYSQLIP, DB_PORT, DB_NAME, DB_USER, DB_PASS);
    error_exit(dbh, error_str);
  }

  server_version = mysql_get_server_version(dbh);
  printf("Connected to MySQL v%d.\n\n", server_version);

  /* Prepare and execute the SQL statement */
  printf("Insert into MySQL: %s.\n\n", insert_str1);
  if(mysql_query(dbh, insert_str1) != 0) {
    snprintf(error_str, sizeof(error_str), "Cannot execute insert: %s\n", insert_str1);
    error_exit(dbh, error_str);
  }
  /* Prepare and execute the SQL statement */
  printf("Insert into MySQL: %s.\n\n", insert_str2);
  if(mysql_query(dbh, insert_str2) != 0) {
    snprintf(error_str, sizeof(error_str), "Cannot execute insert: %s\n", insert_str2);
    error_exit(dbh, error_str);
  }
  /* Prepare and execute the SQL statement */
  printf("Insert into MySQL: %s.\n\n", insert_str3);
  if(mysql_query(dbh, insert_str3) != 0) {
    snprintf(error_str, sizeof(error_str), "Cannot execute insert: %s\n", insert_str3);
    error_exit(dbh, error_str);
  }

  /* Prepare and execute the SQL statement */
  printf("Query from MySQL: %s.\n\n", query_str1);
  if(mysql_query(dbh, query_str1) != 0) {
    snprintf(error_str, sizeof(error_str), "Cannot execute query: %s\n", query_str1);
    error_exit(dbh, error_str);
  }
  /* get query results set */
  result1 = mysql_store_result(dbh);
  if (result1 == NULL) {
    snprintf(error_str, sizeof(error_str), "No results for query: %s\n", query_str1);
    error_exit(dbh, error_str);
  }
  col_count = mysql_num_fields(result1);
  printf("number of output columns: %d \n\n", col_count);
  /* get the output column names */
  fields = mysql_fetch_fields(result1);
  putchar('+');
  /* draw the upper column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; j++) putchar('-');
    putchar('+');
  }
  putchar('\n');
  putchar('|');
  /* show the output column names */
  for (i=0; i<col_count; i++) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    printf("%-*s|", fieldlength, fields[i].name);
  }
  printf("\n");
  putchar('+');
  /* draw the lower column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; ++j) putchar('-');
    putchar('+');
  }
  putchar('\n');
  /* fetch the data */
  while((row = mysql_fetch_row(result1)) != NULL) {
    putchar('|');
    for(i=0; i<col_count; i++) {
      if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
      else fieldlength = fields[i].max_length;
      if (row[i] != NULL) printf("%-*s|", fieldlength, row[i]);
      else printf("%-*s|", fieldlength, "");
    }
    printf("\n");
  }
  putchar('+');
  /* draw the lower column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; ++j) putchar('-');
    putchar('+');
  }
  putchar('\n');

  /* Prepare and execute the SQL statement */
  printf("Query from MySQL: %s.\n\n", query_str2);
  if(mysql_query(dbh, query_str2) != 0) {
    snprintf(error_str, sizeof(error_str), "Cannot execute query: %s\n", query_str2);
    error_exit(dbh, error_str);
  }
  /* get query results set */
  result2 = mysql_store_result(dbh);
  if (result2 == NULL) {
    snprintf(error_str, sizeof(error_str), "No results for query: %s\n", query_str2);
    error_exit(dbh, error_str);
  }
  col_count = mysql_num_fields(result2);
  printf("number of output columns: %d \n\n", col_count);
  /* get the output column names */
  fields = mysql_fetch_fields(result2);
  putchar('+');
  /* draw the upper column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; j++) putchar('-');
    putchar('+');
  }
  putchar('\n');
  putchar('|');
  /* show the output column names */
  for (i=0; i<col_count; i++) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    printf("%-*s|", fieldlength, fields[i].name);
  }
  printf("\n");
  putchar('+');
  /* draw the lower column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; ++j) putchar('-');
    putchar('+');
  }
  putchar('\n');
  /* fetch the data */
  while((row = mysql_fetch_row(result2)) != NULL) {
    putchar('|');    for(i=0; i<col_count; i++) {
      if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
      else fieldlength = fields[i].max_length;
      if (row[i] != NULL) printf("%-*s|", fieldlength, row[i]);
      else printf("%-*s|", fieldlength, "");
    }
    printf("\n");
  }
  putchar('+');
  /* draw the lower column header frame */
  for (i = 0; i < col_count; ++i) {
    if(fields[i].max_length<strlen(fields[i].name)) fieldlength = strlen(fields[i].name);
    else fieldlength = fields[i].max_length;
    for (j = 0; j < fieldlength; ++j) putchar('-');
    putchar('+');
  }
  putchar('\n');

  mysql_close(dbh);
  exit(0);
}
// # ./mysqltest
// Connected to MySQL v50136.
// 
// Insert into MySQL: INSERT IGNORE INTO edacs_users (username, first_connect, last_connect) VALUES ('kschlwerwgrwer', DATE_FORMAT('2008-08-15 01:57:35', '%Y-%m-%d %H:%i:%s'), DATE_FORMAT('2008-08-16 14:57:35', '%Y-%m-%d %H:%i:%s')).
// 
// Query from MySQL: SELECT * FROM edacs_users.
// 
// number of output columns: 3 
// 
// +--------------+-------------+------------+
// |username      |first_connect|last_connect|
// +--------------+-------------+------------+
// |kschlwerwgrwer|2008-08-15   |2008-08-16  |
// +--------------+-------------+------------+
// Delete from MySQL: DELETE FROM edacs_users.
// 
