/* ---------------------------------------------------------------------------*
 * file:        edacs.h                                                       *
 * purpose:     provide standard definitions accross cgi's                    *
 * author:      27/04/2006 Frank4DD                                           *
 * ---------------------------------------------------------------------------*/

#include <stdlib.h>
#include <time.h>
#include <dirent.h>
#include <gd.h>
#include <gdfontt.h>
#include <gdfonts.h>
#include <gdfontmb.h>

/**********************************************************************/
/********** Choose database: MySQL >= 5.1 or Oracle >= 10g ************/
/**********************************************************************/
#define MYSQL_DB

#ifdef MYSQL_DB
  #include "mysql.h"
#endif

#ifdef ORACLE_DB
  #include "oracle.h"
#endif

/**********************************************************************/
/********** edacsdb loader settings, directories and files ************/
/**********************************************************************/
/* this is a hack to avoid coding of zlib compression, re-inventing ***/
/* the wheel. Although it would make the code more independend      ***/
#define GZIP_BIN     "/usr/bin/gzip"

/**********************************************************************/
/*********** radius and tacacs+ accounting log file directory *********/
/**********************************************************************/
#define RADACCT_DIR  "/srv/app/radius/radacct"
#define TACACCT_DIR  ""

/**********************************************************************/
/****************** the edacsdb directory structure *******************/
/**********************************************************************/
#define EDACSDB_DIR  "/srv/app/edacsdb";
#define BIN_DIR      "/srv/app/edacsdb/bin"
#define RAW_DIR      "/srv/app/edacsdb/raw"
#define GOOD_DIR     "/srv/app/edacsdb/good"
#define ERROR_DIR    "/srv/app/edacsdb/error"
#define ARCHIVE_DIR  "/srv/app/edacsdb/archive"
#define LOG_DIR      "/srv/app/edacsdb/logs"

/**********************************************************************/
/********** edacsdb housekeeping settings to expire data **************/
/**********************************************************************/
/******** How long do we want to keep old loader logs in days *********/
#define KEEP_LOGS     60
/******** How long do we want to keep sql error files in days *********/
#define KEEP_ERRS     120 
/*** After how many days is a TMP table entry orphaned and deleted ****/
#define EXPIRE_ORPHAN 30

/**********************************************************************/
/*********** edacsweb reporter settings, home URL and user ************/
/**********************************************************************/
#define CGI_DIR        "/srv/www/std-root/frank4dd.com/sw/edacs/cgi-bin"
#define IMG_DIR        "/srv/www/std-root/frank4dd.com/sw/edacs/images"
#define HOMELINK       "/sw/edacs/"
#define REQLINK	       "/edacs/cgi-bin/index.cgi"
#define CSSLINK	       "/sw/edacs/style/edacs.css"

/*********** html code template for populating the sidebar  *******************/
#define SIDEBAR_TEMPL   "../sidebar-template.htm" /* optional */
/*********** html code template for populating the help data  *****************/
#define HELP_TEMPL      "../edacshelp-template.htm" /* mandatory */
/*********** html code template for populating DB help data  ******************/
#define EDACSDB_TEMPL   "../edacsdb-template.htm" /* mandatory */
/*********** html code template for populating radius help data  **************/
#define RADIUS_TEMPL    "../radius-template.htm" /* mandatory */
/*********** html code template for populating web help data  *****************/
#define EDACSWEB_TEMPL  "../edacsweb-template.htm" /* mandatory */
/*********** html code template for populating the about text  ****************/
#define ABOUT_TEMPL     "../about-template.htm" /* mandatory */
/****** html code template for the menu navigation bar header on top **********/
#define NAVI_TEMPL      "../navigation-template.htm" /* mandatory */
/****** html code template for adding code or scripts into the footer *********/
#define FOOTER_TEMPL    "../footer-template.htm" /* optional */

/******** How many months we want to display the daily stats **********/
#define DAYSTATS_HIST  6
/******** Here we set the max routers edacs can handle, <1000 *********/
#define MAX_ROUTERS    200
/***** Here we set the max number of records to display, <1000 ********/
#define MAX_RECORDS    500

/******** enable extra DEBUG Output, increase for verbosity **********/
#define DEBUG	2

/**********************************************************************/
/*************** no changes required below this line ******************/
/**********************************************************************/

/**************** general settings ****************/
#define CONTACT_EMAIL	 "support@frank4dd.com"
#define EDACSDB_VERSION  "EDAcS Loader v1.2 (05/18/2010)"
#define EDACSWEB_VERSION "EDAcS Web v1.2.3 (07/18/2011)"

/* function definitions */
#define cgi_error(msg)  handle_error_cgi(__FILE__, __LINE__, msg)
#define ldr_error(msg)  handle_error_ldr(__FILE__, __LINE__, msg)

/* function prototypes */
void handle_error_ldr(const char *file, int lineno, const char *msg);
void handle_error_cgi(const char *file, int lineno, const char *msg);

int sql_select(const struct dirent *);
int err_select(const struct dirent *);
int rad_select(const struct dirent *);
int log_select(const struct dirent *);
int split_radius();
int parse_radius();
int verify_dbase();
int insert_dbase();
int erase_orphan();
int erase_sqltmp();
int erase_sqlerr();
int erase_ldrlog();
int archive_file();
void sigint_handler(void);
void pagehead(char *title);
void pageside();
void pagefoot();
char * calc_units(char *, char *);
void get_dns_name(char *);
void sigint_handler(void);
void init_graph(char *);
int con_permonth(char *, char *, int, unsigned long long *);
int con_perday(char *, char *, unsigned long long *);
int bytes_permonth(char *, char *, int, unsigned long long *, unsigned long long *, unsigned long long *);
int bytes_perday(char *, char *, unsigned long long *, unsigned long long *, unsigned long long *);
int user_permonth(char *, char *, int, unsigned long long *);
int user_perday(char *, char *, unsigned long long *);
int avgb_permonth(char *, char *, int, unsigned long long *);
int avgb_perday(char *, char *, unsigned long long *);

/* global defined variables */
FILE *logfile;			/* logfile for errors and success messages   */
char errormsg[8192];		/* error messages passed on to error handler */
char ldr_tstring[16];           /* timstamp string identifying loader files  */
int rad_detailfiles;		/* counter of how many detail files we found */
int rad_parsedfiles;		/* counter of how many radius files parsed   */
int sql_erasedfiles;		/* counter of how many .sql files we deleted */
int err_erasedfiles;		/* counter of how many .err files we deleted */
int log_erasedfiles;		/* counter of how many .log files we deleted */
int radius_rawrecords;		/* counter of orig. radius records processed */
int sql_loadedrecords;		/* counter of generated sql statements       */
int sql_insertedfiles;		/* counter of how many sql files processed   */
int rad_archivedfiles;		/* counter of archived radius files          */
int _abort_flag;
char dns_name[255];             /* Router DNS name if available  */
gdImagePtr imgbuf_year, imgbuf_mon; /* GD charts image buffer    */
FILE *pngout;                   /* GD charts output file for PNG */
char maxvaltxt[32];             /* GD charts graph values        */
float percent;                  /* GD charts percent storage     */
int black, white, grey, dkgrey, dkblue, cyan, orange; /* charts color values */

/* ------------------------------------------------------------------------- *
 * Here we define the key=value pair we want from the radius accounting file *
 * The structure should match the SQL table definition, plus  one space for  *
 * the extra '\0' character in strings.                                      *
 * ------------------------------------------------------------------------- */

struct radius_pair {
  char action[11];         /* radacctkey Acct-Status-Type    */
  char router[21];         /* radacctkey NAS-IP-Address      */
  char username[49];       /* radacctkey User-Name           */
  char action_date[60];    /* radacctkey DB Timestamp String */
  int bytes_in;            /* radacctkey Acct-Input-Octets   */
  int bytes_out;           /* radacctkey Acct-Output-Octets  */
  char ip_addr[16];        /* radacctkey Framed-IP-Address   */
  char tty[8];             /* radacctkey NAS-Port            */
  char protocol[21];       /* radacctkey Framed-Protocol     */
  int packets_in;          /* radacctkey Acct-Input-Packets  */
  int packets_out;         /* radacctkey Acct-Output-Packets */
  char task_id[21];        /* radacctkey Acct-Session-Id     */
  char service[21];        /* radacctkey Service-Type        */
  char ip_or_phone[21];    /* radacctkey Calling-Station-Id  */
};

/****************************** end edacs.h *********************************/
