/* ---------------------------------------------------------------------- *
 * file:        edacs-modernizer.c                                        *
 * purpose:     take X-year old archived raw data, change the             *
 *              timestamp and feed it to edacs-loader raw dir             *
 * author:      06/03/2010 Frank4DD                                       *
 * usage:       run via crontab hourly                                    *
 * ---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
//#include "getline.h"
#include "../edacs.h"

char datestring[14];

/* ---------------------------------------------------------------------- *
 * rad_select(): runs through the Archive directory and selects all files *
 * identified by the .rad extension. -> parse_radius.c and edacs_modernizer.c *
 * ---------------------------------------------------------------------- */
int rad_select(const struct dirent *entry) {

  /* --------------------------------------------------------------------- *
   * check for "." and ".." directory entries to sort them out             *
   * --------------------------------------------------------------------- */
  if(entry->d_name[0]=='.') return 0;

  /* --------------------------------------------------------------------- *
   * single out the datestr matching archive files in the archive          *
   * --------------------------------------------------------------------- */
  if(strstr(entry->d_name, datestring) != NULL) {
    if(DEBUG >0) printf("[edacs_modernizer] %s archive file matching %s.\n", entry->d_name, datestring);
    return 1;
  }

  return 0;
}

/* ----------------------------------------------------------------------- *
 * **************    main routine for the edacs modernizer      ********** *
 * ------------------------------------------------------------------------*/
int main() {
  int radcount = 0;
  int i        = 0;
  time_t now, past;
  struct tm *nowtmptr, *pasttmptr;
  char nowstring[14];
  char datepath[14];
  struct dirent **radfile_list;
  char archfilepath[256]  = "";
  char pastyear[5];
  char thisyear[5];
  // timediff to determine which hsitoric data to pick up
  int timediff = ((86400 * 365) * 10);

  /* --------------------------------------------------------- *
   * Get past -x years date/time, round up to the nearest hour *
   * --------------------------------------------------------- */
  past = (time(NULL) - (timediff));
  pasttmptr = localtime(&past);
  strftime(datepath, sizeof(datepath), "%Y/%m/%d", pasttmptr);
  strftime(datestring, sizeof(datestring), "%Y.%m.%d.%H", pasttmptr);
  strftime(pastyear, sizeof(pastyear), "%Y", pasttmptr);

  /* --------------------------------------------------------- *
   * Generate sub-dir path for now - 3 years, i.e. /2007/06/09 *
   * --------------------------------------------------------- */
  snprintf(archfilepath, sizeof(archfilepath),
        "%s/%s", ARCHIVE_DIR, datepath);

  /* --------------------------------------------------------- *
   * debug: print the name of the .rad raw file                *
   * --------------------------------------------------------- */
  if(DEBUG >0) printf("[edacs_modernizer] RAD data dir: %s\n", archfilepath);
  if(DEBUG >0) printf("[edacs_modernizer] Timediff set: %d\n", timediff);

  /* --------------------------------------------------------- *
   * select files with name *-2007.06.09.<hour>*.rad.gz        *
   * i.e. archive/2007/06/09/dial1-2007.06.09.0005.rad.gz      *
   * --------------------------------------------------------- */
  if (opendir(archfilepath) != NULL) {
    printf("[edacs_modernizer] archive dir %s exists.\n", archfilepath);
    radcount = scandir(archfilepath, &radfile_list, rad_select, alphasort);
  }
  else {
    printf("[edacs_modernizer] !Error!: No archive dir %s.\n", archfilepath);
    exit(-1);
  }

  if(radcount == 0) {
    if(DEBUG >0) printf("[edacs_modernizer] !Error!: No .gz file found in %s.\n", archfilepath);
    return 0;
  } else {
    if(DEBUG >0) printf("[edacs_modernizer] RAD %d archive file(s) found.\n", radcount);
  }
  /* --------------------------------------------------------- *
   * Cycle through the ordered .rad files found in RAW_DIR     *
   * --------------------------------------------------------- */
  i = 0;
  while(i<radcount) {
    struct stat radentrybuf;
    char radentrypath[256]  = "";
    char radtemppath[256]  = "";
    char radtempnew[256]  = "";
    char loadrawpath[256]  = "";

    /* --------------------------------------------------------- *
     * generate the full path and file name of the .rad file     *
     * --------------------------------------------------------- */
    snprintf(radentrypath, sizeof(radentrypath),
        "%s/%s", archfilepath, radfile_list[i]->d_name);

    if(DEBUG >0) printf("[edacs_modernizer] RAD %s archive file(s) selected.\n", radentrypath);
    /* --------------------------------------------------------- *
     * try to stat the .rad file                                 *
     * --------------------------------------------------------- */
    if (stat(radentrypath, &radentrybuf) == -1) {
      snprintf(errormsg, sizeof(errormsg),
             "[parse_radius] !Error!: Cannot stat %s.\n", radentrypath);
      printf("%s", errormsg);
      continue;
    }

    /* ------------------------------------------------------- *
     * copy the archive file to temp directory                 *
     * ------------------------------------------------------- */
    char systemcall[256] = "";
    snprintf(systemcall, sizeof(systemcall),
        "/bin/cp %s %s", radentrypath, "/tmp");

    if(system(systemcall) == -1) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot copy %s.\n", radentrypath);
      printf("%s", errormsg);
      i++;
      continue;
    } else {
      if(DEBUG >0) printf("[edacs_modernizer] executed %s\n", systemcall);
    }

    /* ------------------------------------------------- *
     * uncompress the archive data file in  temp dir     *
     * ------------------------------------------------- */
    snprintf(radtemppath, sizeof(radtemppath),
       "/tmp/%s", radfile_list[i]->d_name);

    snprintf(systemcall, sizeof(systemcall),
        "%s -d %s", GZIP_BIN, radtemppath);

    if(system(systemcall) == -1) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot execute %s.\n", systemcall);
      printf("%s", errormsg);
      i++;
      continue;
    } else {
      if(DEBUG >0) printf("[edacs_modernizer] executed %s\n", systemcall);
    }

    /* ------------------------------------------------------- *
     * update the 2x archive file records timestamps:          *
     *Wed Jun  9 16:59:10 2007->2010                           *
     * 	Timestamp = 1086793150 -> +xxxxxxx                     *
     * ------------------------------------------------------- */
    radtemppath[strlen(radtemppath)-3] = '\0';

    now = time(NULL);
    nowtmptr = localtime(&now);
    strftime(nowstring, sizeof(nowstring), "%Y.%m.%d.%H", nowtmptr);
    if(DEBUG >0) printf("[edacs_modernizer] todays date: %s\n", nowstring);
    strftime(thisyear, sizeof(thisyear), "%Y", nowtmptr);

    /* first, change the records header datetime string */
    snprintf(systemcall, sizeof(systemcall),
        "/bin/sed -i 's/ %s$/ %s/' %s", pastyear, thisyear, radtemppath);

    if(system(systemcall) == -1) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot execute %s.\n", systemcall);
      printf("%s", errormsg);
      i++;
      continue;
    } else {
      if(DEBUG >0) printf("[edacs_modernizer] executed %s\n", systemcall);
    }

    /* check file and get file handle */
    snprintf(radtempnew, sizeof(radtempnew),
       "%s.new", radtemppath);

    FILE *fhr, *fhw;
    fhr = fopen(radtemppath, "r");
    fhw = fopen(radtempnew, "w");

    /* check if file exists */
    if (fhr == NULL){
      printf("file does not exist: %s", radtemppath);
      return -1;
    }

    /* read line by line */
    const size_t line_size = 300;
    char* line = malloc(line_size);
    while (fgets(line, line_size, fhr) != NULL)  {

      /* if line  "Timestamp = 1395128929" */
      if(strstr(line, "Timestamp = ") != NULL) {
        /* delete the newline char */
        if(line[strlen(line)-1] == '\n') line[strlen(line)-1] = '\0';
        /* extract the timestamp */
        char *tstamp_str = (strchr(line, '=')) +1;

        time_t rec_tstamp = (time_t) atoi(tstamp_str);
        struct tm *rectmptr = localtime(&rec_tstamp);

        // convert timestamp to time struct
        char rec_year[5];
        strftime(rec_year, sizeof(rec_year), "%Y", rectmptr);

        // calc difference to current year
        int ydiff = atoi(thisyear) - atoi(rec_year);
        if(DEBUG >0)
          printf("[edacs_modernizer] timestamp %d from %s is %d years off. adjusting...\n",
            (unsigned int) rec_tstamp, rec_year, ydiff);

        // Construct a new timestamp with todays date
        now = time(NULL);
        nowtmptr = localtime(&now);
        rectmptr->tm_year = nowtmptr->tm_year;
        rectmptr->tm_mon = nowtmptr->tm_mon;
        rectmptr->tm_mday = nowtmptr->tm_mday;
        rectmptr->tm_hour = nowtmptr->tm_hour+8;

        rec_tstamp = mktime(rectmptr);
        snprintf(line, 255, "\tTimestamp = %d\n", (unsigned int) rec_tstamp);
        if(DEBUG >0)
          printf("[edacs_modernizer] new timestamp: %d year: %d month: %d day: %d\n",
            (unsigned int) rec_tstamp, 1900+rectmptr->tm_year, 1+rectmptr->tm_mon, rectmptr->tm_mday);

      }
      fputs(line, fhw);
    }
    fclose(fhr);
    fclose(fhw);

    /* ------------------------------------------------------- *
     * generate the new filename with the updated timestamp    *
     * i.e. archive/2007/06/09/dial1-2007.06.09.0005.rad.gz    *
     * ->   archive/2007/06/09/dial1-2010.06.09.0005.rad.gz    *
     * ------------------------------------------------------- */
    radfile_list[i]->d_name[strlen(radfile_list[i]->d_name)-22] = '\0';

    snprintf(loadrawpath, sizeof(loadrawpath),
       "%s/%s%s05.rad", RAW_DIR, radfile_list[i]->d_name, nowstring);

    /* ------------------------------------------------------- *
     * copy the updated archive file to the loaders raw dir    *
     * ------------------------------------------------------- */
    snprintf(systemcall, sizeof(systemcall),
        "/bin/cp %s.new %s", radtemppath, loadrawpath);

    if(system(systemcall) == -1) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot copy %s.\n", radtemppath);
      printf("%s", errormsg);
      i++;
      continue;
    } else {
      if(DEBUG >0) printf("[edacs_modernizer] executed %s\n", systemcall);
    }

    /* ------------------------------------------------- *
     * delete the sed updated file in the temp dir           *
     * ------------------------------------------------- */
    if(unlink(radtemppath) != 0) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot unlink %s.\n", radtemppath);
      printf("%s", errormsg);
      i++;
      continue;
    }  else {
      if(DEBUG >0) printf("[edacs_modernizer] deleted %s\n", radtemppath);
    }

    /* ------------------------------------------------- *
     * delete the awk updated file in the temp dir           *
     * ------------------------------------------------- */
    strncat(radtemppath, ".new", sizeof(radtemppath)-strlen(radtemppath)-1);

    if(unlink(radtemppath) != 0) {
      snprintf(errormsg, sizeof(errormsg),
               "[edacs_modernizer] !Error!: Cannot unlink %s.\n", radtemppath);
      printf("%s", errormsg);
      i++;
      continue;
    }  else {
      if(DEBUG >0) printf("[edacs_modernizer] deleted %s.\n", radtemppath);
    }
    i++;
  }
  return 0;
}
