/* ------------------------------------------------------------------- *
 * file:      erase_fullrec_mysql.c                                    *
 * purpose:   erase completed records after STOP from temp table       *
 * author:    06/03/2010 Frank4DD                                      *
 * remarks:   Complete records are records with STOP action received   *
 *            They have been updated in the main tables                *
 * select STOP from edacs_templog                                      *
 * foreach line DELETE from edacs_templog where ID = stopid            *
 * ------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
//#include "getline.h"
#include "../edacs.h"

/* ------------------------------------------------------------------- *
 * insert_dbase picks up the SQL files in GOOD_DIR and loads them into *
 * the database.                                                       *
 * ------------------------------------------------------------------- */
int erase_orphan() {
  static MYSQL *dbh;                 /* database connect handle   */
  char sql_delete_str[16384] = "";   /* SQL string for DELETE     */
  long orphan_temprecs;              /* erased orphan record count*/

  /* ----------------------------------------------------------------- * 
   * Reset the global values                                           * 
   * ----------------------------------------------------------------- */
  orphan_temprecs           = 0;    /* zero the orphan records counter */

  /* ----------------------------------------------------------------- *
   * initialize the connection                                         *
   * ----------------------------------------------------------------- */
  dbh = mysql_init(NULL);
  if(dbh == NULL) {
    dbs_error(dbh, "[verify_dbase] !Error!:  mysql_init failed.");
    return EXIT_FAILURE;
  }

  /* ----------------------------------------------------------------- *
   * login to the database                                             *
   * ----------------------------------------------------------------- */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !ERROR! Cannot login to %s:%d db %s with user %s pass %s\n", MYSQLIP, DB_PORT, DB_NAME, EDACSADMIN, ADMIN_PASS);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ----------------------------------------------------------------- *
   * Database connecting complete                                      *
   * ----------------------------------------------------------------- */
  if(DEBUG > 0) printf("[erase_orphan] Successful DB login.\n");

  /* ----------------------------------------------------------------- *
   * Prepare the SQL statement                                         *
   * ----------------------------------------------------------------- */
  snprintf(sql_delete_str, sizeof(sql_delete_str),
   "DELETE FROM edacs_templog WHERE DATE(action_date) = DATE(DATE_SUB(NOW(), INTERVAL %d DAY))", EXPIRE_ORPHAN);
  if(DEBUG >1) printf("[erase_orphan] SQL string: %s.\n", sql_delete_str);

  /* ----------------------------------------------------------------- *
   * Execute the SQL command here:                                     *
   * ----------------------------------------------------------------- */
  if(mysql_query(dbh, sql_delete_str) != 0) {
    if(DEBUG >1)
      printf("[erase_orphan] Cannot execute delete, problem in %s.",
           sql_delete_str);
    snprintf(errormsg, sizeof(errormsg),
          "Cannot execute delete, problem in %s", sql_delete_str);
    dbs_error(dbh, errormsg);
   }
   orphan_temprecs = (long) mysql_affected_rows(dbh);

  /* ------------------------------------------------------------------ *
   * close database connection                                          *
   * ------------------------------------------------------------------ */
  mysql_close(dbh);

  if(orphan_temprecs > 0) {
    fprintf(logfile, "[erase_orphan] Erased orphaned records: %ld.\n", orphan_temprecs);
  }

  if(orphan_temprecs==0) {
    fprintf(logfile, "[erase_orphan] No orphaned records expired.\n");
  }

  return orphan_temprecs;
}
