/* ------------------------------------------------------------------- *
 * file:      erase_orphan_oracle.c                                    *
 * purpose:   erase orphaned temp records in database                  *
 * author:    08/23/2006 Frank4DD                                      *
 * remarks:   orphans are database records with not end record update. *
 *            Either a log file was missing, or a packet dropped, etc. *
 * ------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sqlora.h>
#include <time.h>
//#include "getline.h"
#include "../edacs.h"

/* ------------------------------------------------------------------- *
 * insert_dbase picks up the SQL files in GOOD_DIR and loads them into *
 * the database.                                                       *
 * ------------------------------------------------------------------- */
int erase_orphan() {
  char sql_delete_str[16384] = "";   /* SQL string for DELETE     */
  int sqlretcode             = 0;    /* SQL command return code   */
  sqlo_stmt_handle_t sth;	     /* statement handle          */
  int orphan_temprecs;               /* erased orphan record count*/


  /* ----------------------------------------------------------------- * 
   * Reset the global values                                           * 
   * ----------------------------------------------------------------- */
  orphan_temprecs           = 0;    /* zero the orphan records counter */

  /* ----------------------------------------------------------------- *
   * ORACLE_HOME is needed for OCI8 to find things like tnsnames.ora   *
   * ----------------------------------------------------------------- */
  putenv(DB_ORACLE_ENV);

  /* ----------------------------------------------------------------- *
   * initialize the connection                                         *
   * ----------------------------------------------------------------- */
  sqlretcode = sqlo_init(SQLO_OFF, 1, 100);
  if (sqlretcode != SQLO_SUCCESS) {
    snprintf(errormsg, sizeof(errormsg),
    "[erase_orphan] !Error!: %d Failed to init libsqlora8.", sqlretcode);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ----------------------------------------------------------------- *
   * login to the database                                             *
   * ----------------------------------------------------------------- */
  sqlretcode = sqlo_connect(&dbh, DB_TNS_STRING);
  if (sqlretcode != SQLO_SUCCESS) {
    snprintf(errormsg, sizeof(errormsg),
             "[erase_orphan] !ERROR! %d Cannot login with %s.",
    sqlretcode, DB_TNS_STRING);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ----------------------------------------------------------------- *
   * set autocommit, each statement is committed as single transaction *
   * ----------------------------------------------------------------- */
  sqlretcode = sqlo_set_autocommit(dbh, 1);
  if(sqlretcode != SQLO_SUCCESS) {
    snprintf(errormsg, sizeof(errormsg),
             "[erase_orphan] !ERROR! %d Cannot set autocommit.", sqlretcode);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ----------------------------------------------------------------- *
   * Database connecting complete                                      *
   * ----------------------------------------------------------------- */
  if(DEBUG > 0) printf("[erase_orphan] Successful login.\n");

  /* ----------------------------------------------------------------- *
   * Prepare and the SQL statement                                     *
   * ----------------------------------------------------------------- */
  snprintf(sql_delete_str, sizeof(sql_delete_str),
   "DELETE FROM %s.%s WHERE action_date < (SYSDATE-%d)", EDACSADMIN, TEMP_TABLE, EXPIRE_ORPHAN);
  if(DEBUG >1) printf("[erase_orphan] SQL string: %s.\n", sql_delete_str);

  if(0>(sth = sqlo_prepare(dbh, sql_delete_str)))
          dbs_error(dbh, "[erase_orphan] !Error!: sqlo_prepare.");

  /* ----------------------------------------------------------------- *
   * Execute the SQL command here:                                     *
   * ----------------------------------------------------------------- */
   sqlretcode = sqlo_execute(sth, 1);
   if(sqlretcode != SQLO_SUCCESS) {

     if(DEBUG >1)
       printf("[erase_orphan] %d sqlo_execute problem in %s: %s.\n",
               sqlretcode, sql_delete_str, errormsg);

       snprintf(errormsg, sizeof(errormsg),
              "[erase_orphan] !Error!: %d sqlo_execute problem in %s.",
               sqlretcode, sql_delete_str);
        dbs_error(dbh, errormsg);
   }
   orphan_temprecs = sqlretcode;

  /* ------------------------------------------------------------------ *
   * close database connection                                          *
   * ------------------------------------------------------------------ */
  if(SQLO_SUCCESS != sqlo_finish(dbh))
    dbs_error(dbh, "[erase_orphan] !Error!: sqlo_finish.");

  if(orphan_temprecs > 0) {
    fprintf(logfile, "[erase_orphan] Erased orphaned records: %d.\n", orphan_temprecs);
  }

  if(orphan_temprecs==0) {
    fprintf(logfile, "[erase_orphan] No orphaned records expired.\n");
  }

  return orphan_temprecs;
}
