/* ------------------------------------------------------------------------ *
 * file:        erase_sqltmp.c                                              *
 * purpose:     deletes the temporary .sql files in edacsdb/good directory  *
 * author:      08/23/2006 Frank4DD                                         *
 * ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
//#include "getline.h"
#include "../edacs.h"

/* ----------------------------------------------------------- *
 * sql_select runs through the sql directory and selects all   *
 * files identified by .sql extension. -> insert_dbase()       *
 * ----------------------------------------------------------- */
int sql_select(const struct dirent *entry) {
  char *lastdot = NULL;         /* file extension separator    */

  /* --------------------------------------------------------- *
   * check for "." and ".." directory entries to sort them out *
   * --------------------------------------------------------- */
  if(entry->d_name[0]=='.') return 0;

  /* --------------------------------------------------------- *
   * single out the .sql files in GOOD_DIR to process          *
   * --------------------------------------------------------- */
  lastdot = strrchr(entry->d_name, '.');
  if(DEBUG >0) printf("[erase_sqltmp] checking %s for .sql extension.\n", entry->d_name);
  if(strcmp(lastdot, ".sql") != 0) return 0;

  return 1;
}

/* ------------------------------------------------------------ *
 * erase_sqltmp():  delete the sql loader files to clean up.    *
 * ------------------------------------------------------------ */
int erase_sqltmp() {
  int sqlcount = 0;
  int i        = 0;
  struct dirent **sqlfile_list;

  /* ------------------------------------------------------------ *
   * Reset the global values                                      *
   * ------------------------------------------------------------ */
  sql_erasedfiles = 0;

  /* ------------------------------------------------------------ *
   * Select and order the .sql files found in GOOD_DIR            *
   * ------------------------------------------------------------ */
  if(DEBUG >0) printf("[erase_sqltmp] searching .sql files in %s. Found: %d files.\n", GOOD_DIR, sqlcount);
  sqlcount = scandir(GOOD_DIR, &sqlfile_list, sql_select, alphasort);

  if(sqlcount == 0) {
    fprintf(logfile, "[erase_sqltmp] !Error!: No .sql file found in %s.\n", GOOD_DIR);
    return sqlcount;
  }

  /* ----------------------------------------------------------- *
   * Cycle through the ordered .sql files found in GOOD_DIR      *
   * ----------------------------------------------------------- */
  i = 0;
  while(i<sqlcount) {
   char sqlentrypath[256] = "";

    /* --------------------------------------------------------- *
     * generate the full path and file name of the .sql file     *
     * --------------------------------------------------------- */
    snprintf(sqlentrypath, sizeof(sqlentrypath),
        "%s/%s", GOOD_DIR, sqlfile_list[i]->d_name);

    /* --------------------------------------------------------- *
     * debug: print the name of the sql data file                *
     * --------------------------------------------------------- */
    if(DEBUG >1) printf("[erase_sqltmp] SQL data file %d: %s.\n", i, sqlentrypath);

    /* --------------------------------------------------------- *
     * delete the .sql file                                      *
     * --------------------------------------------------------- */
    if(unlink(sqlentrypath) != 0) {
      snprintf(errormsg, sizeof(errormsg),
           "[erase_sqltmp] !Error!: Cannot unlink %s.\n", sqlentrypath);
      ldr_error(errormsg);
    }
    else
      fprintf(logfile, "[erase_sqltmp] %s: erased SQL file\n", sqlfile_list[i]->d_name);

    sql_erasedfiles++;
    i++;
  }

  if(sql_erasedfiles > 0) {
    if(DEBUG >0) printf("[erase_sqltmp] Deleted SQL data files: %d\n", sql_erasedfiles);
  }
  else {
    fprintf(logfile, "[erase_sqltmp] !Error!: No SQL files deleted.\n");
  }

  return sql_erasedfiles;
}
