/* ------------------------------------------------------------------------ *
 * file:        insert_dbase_mysql.c                                        *
 * purpose:     load accounting data into the database with libsqlora       *
 * author:      05/19/2010 Frank4DD                                         *
 *                                                                          *
 * It is awfully important that files are loaded in correct timely order,   *
 * otherwise the trigger does not work and the mainlog table is not filled. *
 * ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
//#include "getline.h"
#include "../edacs.h"

/* ------------------------------------------------------------------- *
 * insert_dbase picks up the SQL files in GOOD_DIR and loads them into *
 * the database.                                                       *
 * ------------------------------------------------------------------- */
int insert_dbase() {

  static MYSQL *dbh;                 /* database connect handle   */
  struct dirent **sqlfile_list;      /* list of SQL files in dir  */
  char sqlentrypath[256]    = "";    /* SQL data full path & name */
  char sqlerrorfile[80]     = "";    /* SQL error file name       */
  char sqlerrorpath[256]    = "";    /* SQL error full path &name */
  struct stat sqlentrybuf;           /* file information struct   */
  FILE *sqlfiledesc;                 /* file descriptor for .sql  */
  FILE *sqlerrordesc;                /* file descriptor for .err  */
  char *linebuf              = NULL; /* buffer for every sql line */
  char lastbuf[16384]        = "";   /* buffer previous sql line  */
  ssize_t read               = 0;    /* read data                 */
  size_t len                 = 0;    /* length of data            */
  char sql_insert_str[16384] = "";   /* SQL string for INSERT     */
  int linecount              = 1;    /* linecounter in sql file   */
  int i                      = 0;    /* helper variable in loop   */
  int scancount              = 0;    /* count of .sql files       */
  int sqlcounter             = 0;    /* counts records in file    */
  int errcounter             = 0;    /* counts SQL load errors    */


  /* ------------------------------------------------------------ * 
   * Reset the global values                                      * 
   * ------------------------------------------------------------ */
  sql_insertedfiles          = 0;    /* counts processed files    */
  sql_loadedrecords          = 0;    /* zero the SQL loadnumber   */

  /* ------------------------------------------------------------ * 
   * Select and order the .sql files found in GOOD_DIR            * 
   * ------------------------------------------------------------ */
  scancount = scandir(GOOD_DIR, &sqlfile_list, sql_select, alphasort);

  if(scancount == 0) {
    fprintf(logfile, "[insert_dbase] !Error!: No .sql file found in %s.\n", GOOD_DIR);
    return scancount;
  }

  /* ------------------------------------------------------------ *
   * initialize the connection                                    *
   * ------------------------------------------------------------ */
  dbh = mysql_init(NULL);
  if(dbh == NULL) {
    dbs_error(dbh, "[verify_dbase] !Error!:  mysql_init failed.");
    return EXIT_FAILURE;
  }

  /* ------------------------------------------------------------ *
   * login to the database                                        *
   * ------------------------------------------------------------ */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !ERROR! Cannot login to %s:%d db %s with user %s pass %s\n", MYSQLIP, DB_PORT, DB_NAME, EDACSADMIN, ADMIN_PASS);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ------------------------------------------------------------------- *
   * Set autocommit. By default, MySQL runs with autocommit mode enabled.*
   * ------------------------------------------------------------------- */

  /* ----------------------------------------------------------- *
   * Database connecting complete                                *
   * ----------------------------------------------------------- */
  if(DEBUG > 0) printf("[insert_dbase] Successful login.\n");

  /* ----------------------------------------------------------- * 
   * Cycle through the ordered .sql files found in GOOD_DIR      * 
   * ----------------------------------------------------------- */
  while(i<scancount) {
    
    /* --------------------------------------------------------- * 
     * generate the full path and file name of the .sql file     * 
     * --------------------------------------------------------- */
    snprintf(sqlentrypath, sizeof(sqlentrypath),
        "%s/%s", GOOD_DIR, sqlfile_list[i]->d_name);
  
    /* --------------------------------------------------------- * 
     * debug: print the name of the sql data file                *
     * --------------------------------------------------------- */
    if(DEBUG >0) printf("[insert_dbase] SQL data file %d: %s.\n", i, sqlentrypath);
  
    /* --------------------------------------------------------- * 
     * prepare the opening of the sql data file                  *
     * --------------------------------------------------------- */
  
    /* --------------------------------------------------------- * 
     * try to stat the .sql file, if not able skip processing    *
     * --------------------------------------------------------- */
    if (stat(sqlentrypath, &sqlentrybuf) == -1) {
      snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot stat %s.\n", sqlentrypath);
      ldr_error(errormsg);
      i++;
      continue;
    }
  
    /* --------------------------------------------------------- * 
     * if the .sql file is empty, delete it and skip processing  * 
     * --------------------------------------------------------- */
    if( (sqlentrybuf.st_size) == 0) {
      fprintf(logfile, "[insert_dbase] Warning: Empty file %s.\n", sqlentrypath);
      if(unlink(sqlentrypath) != 0) {
        snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot unlink %s.\n", sqlentrypath);
        ldr_error(errormsg);
      }
      else
        fprintf(logfile, "[insert_dbase] Warning: Erased file %s.\n", sqlentrypath);

      i++;
      continue;
    }
  
    /* -------------------------------------------------------- * 
     * open the sql data file, if not able skip processing      *
     * -------------------------------------------------------- */
    sqlfiledesc = fopen(sqlentrypath, "r");
    if(sqlfiledesc == NULL) {
      snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot open %s.\n", sqlentrypath);
      ldr_error(errormsg);
      i++;
      continue;
    }

    /* -------------------------------------------------------- * 
     * cycle through each line of the .sql data file            *
     * -------------------------------------------------------- */
    sqlcounter = 0; /* reset the sql record counter */
    while ((read = getline(&linebuf, &len, sqlfiledesc)) != -1) {

      /* ----------------------------------------------------------- *
       * Sometimes the original radius accounting file has duplicate *
       * entries, i.e. 2 fully identical Interim-Update records.     *
       * Usually duplicates are neighbor entries in the raw file and *
       * resulting .sql file, so we keep copy of the last processed  *
       * record and compare it with the current one for dropping it. *
       * ----------------------------------------------------------- */
       if(strcmp(lastbuf, linebuf) == 0) {
         /* Record is a duplicate, do not process it further */
         fprintf(logfile,
              "[insert_dbase] Warning: Duplicate SQL record:\n%s", linebuf);
         continue;
       }
       /* Record is not a duplicate, save as new lastrecord and process it */
       strncpy(lastbuf, linebuf, sizeof(lastbuf));

      /* ------------------------------------------------------ *
       * Remove the last character newline from linebuf here    *
       * ------------------------------------------------------ */
       *rindex(linebuf, '\n') = '\0';
  
      /* ------------------------------------------------------ *
       * Prepare and parse the SQL statement                    *
       * ------------------------------------------------------ */
      snprintf(sql_insert_str, sizeof(sql_insert_str),
             "INSERT INTO %s (action, router, username, action_date, bytes_in, bytes_out, ip_addr, tty, protocol, packets_in, packets_out, task_id, service, ip_or_phone) VALUES (%s)\n", TEMP_TABLE, linebuf);
  
      if(DEBUG >1) printf("[insert_dbase] SQL string: %s.\n", sql_insert_str);
  
      /* ------------------------------------------------------ *
       * Execute the SQL command here:                          *
       * ------------------------------------------------------ */
      if(mysql_query(dbh, sql_insert_str) != 0) {
        if(DEBUG >1)
          printf("[insert_dbase] Cannot execute insert, problem in %s line %d.",
               sql_insert_str, linecount);
        snprintf(errormsg, sizeof(errormsg),
              "Cannot execute insert, problem in %s line %d", sql_insert_str, linecount);
        dbs_error(dbh, errormsg);
  
        /* ---------------------------------------------------------------- *
         * There is a problem with this particular offending sql statement, *
         * we copy it to a new file in the error directory for review.      *
         * -----------------------------------------------------------------*/
        errcounter++;

        /* ------------------------------------------------- *
         * generate the file name of the sql error file      *
         * by using the sql file name and changing the       *
         * extension from .sql to .err                       *
         * ------------------------------------------------- */
        strncpy(sqlerrorfile, sqlfile_list[i]->d_name, sizeof(sqlerrorfile));
        strncpy(strstr(sqlerrorfile, ".sql"), ".err", 4);
  
        /* ------------------------------------------------- *
         * generate the full path of the sql error file      *
         * ------------------------------------------------- */
        snprintf(sqlerrorpath, sizeof(sqlerrorpath),
            "%s/%s", ERROR_DIR, sqlerrorfile);
        if(DEBUG >2) printf("[insert_dbase] SQL error file: %s.\n",
                                                             sqlerrorpath);
  
        /* --------------------------------------- *
         * open the sql error file in ERROR_DIR    *
         * --------------------------------------- */
        sqlerrordesc = fopen(sqlerrorpath, "a");
        if(sqlerrordesc == NULL) {
           snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot open %s.\n", sqlerrorpath);
          ldr_error(errormsg);
        }
        fprintf(sqlerrordesc, "%s\n", sql_insert_str);
        /* --------------------------------------- *
         * close the sql error file in ERROR_DIR   *
         * --------------------------------------- */
        if(fclose(sqlerrordesc) != 0) {
          snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot close %s.\n", sqlerrorpath);
          ldr_error(errormsg);
        } /* close if(fclose() failed) */

      /*  exit(-1);  -- emergency exit for troubleshooting sql errors */
      } /* close if(SQLO_SUCCESS failed) */
      else {
        /* update the total number of processed sql statements */
        sql_loadedrecords++;
        /* update the number of sql statements per file */
        sqlcounter++;
      }
  
      /* ---------------------------------------------- *
       * Close the statement handle here:               *
       * ---------------------------------------------- */
  
      /* increase the linecounter */
      linecount++;
    }

    /* ---------------------------------------------- *
     * Close the .sql loader file handle here:        *
     * ---------------------------------------------- */
    if(fclose(sqlfiledesc) != 0) {
      snprintf(errormsg, sizeof(errormsg),
               "[insert_dbase] !Error!: Cannot close %s.\n", sqlentrypath);
      ldr_error(errormsg);
      i++;
      continue;
    }

    sql_insertedfiles++;

    fprintf(logfile, "[insert_dbase] %s: loaded SQL records: %d\n",
                                   sqlfile_list[i]->d_name, sqlcounter);
    i++;
  } /* close while loop processing the .sql files in GOOD_DIR */
    
  /* ------------------------- *
   * close database connection *
   * ------------------------- */
  mysql_close(dbh);

  if(errcounter > 0) {
    fprintf(logfile, "[insert_dbase] !Error!: %d SQL loader errors occured.\n", errcounter);
  }

  if(sql_insertedfiles==0) {
    fprintf(logfile, "[insert_dbase] !Error!: No SQL file parsed in %s.\n", GOOD_DIR);
  }

  return sql_insertedfiles;
}
