/* ------------------------------------------------------------------------ *
 * file:        split_radius.c                                              *
 * purpose:     splits radius accounting files                              *
 * author:      08/18/2006 Frank4DD                                         *
 * ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "../edacs.h"

/* ----------------------------------------------------------- *
 * detail_select: helper function used in scandir() to select  *
 * detail files in a directory. It also does the processing,   *
 * i.e. move files and create empty new ones.                  *
 * ----------------------------------------------------------- */
int detail_select(const struct dirent *entry) {
  struct stat dirstatbuf;
  struct stat detailbuf;
  char direntries[256] = "";
  char detailpath[256] = "";
  char rawdirpath[256] = "";

  /* --------------------------------------------------------- * 
   * check for "." and ".." directory entries to sort them out * 
   * --------------------------------------------------------- */
  if(entry->d_name[0]=='.') return 0;

  /* --------------------------------------------------------- *
   * Check if entry is a directory and ignore all other files  *
   * entry->d_type is to unreliable to use, even with          *
   * #ifdef _DIRENT_HAVED_TYPE                                 *
   * --------------------------------------------------------- */

  /* --------------------------------------------------------- *
   * generate the full path and file name                      * 
   * --------------------------------------------------------- */
  snprintf(direntries, sizeof(direntries),
                     "%s/%s", RADACCT_DIR, entry->d_name);

  /* --------------------------------------------------------- *
   * try to stat the directories                               * 
   * --------------------------------------------------------- */
  if (stat(direntries, &dirstatbuf) == -1) {
    snprintf(errormsg, sizeof(errormsg), 
             "[split_radius] !Error!: Cannot stat %s.\n", direntries);
    ldr_error(errormsg);
    return 0;
  }

  /* --------------------------------------------------------- *
   * wipe out any file that is not a directory                 *
   * --------------------------------------------------------- */
  if( (dirstatbuf.st_mode&S_IFMT) != S_IFDIR) return 0;

  /* --------------------------------------------------------- *
   * generate the full path and file name of the detail file   *
   * --------------------------------------------------------- */
  snprintf(detailpath, sizeof(detailpath),
                     "%s/detail", direntries);

  /* --------------------------------------------------------- *
   * try to stat the detail file, for success increase counter *
   * --------------------------------------------------------- */
  if (stat(detailpath, &detailbuf) == -1) {
    snprintf(errormsg, sizeof(errormsg), 
             "[split_radius] !Error!: Cannot stat %s.\n", detailpath);
    ldr_error(errormsg);
    return 0;
  }
  rad_detailfiles++;

  /* --------------------------------------------------------- *
   * forget about detail files that are empty                  *
   * --------------------------------------------------------- */
  if( (detailbuf.st_size) == 0) {
    if(DEBUG >0) printf("[split_radius] Warning: Empty file %s.\n", detailpath);
    return 0;
  }

  /* generate the full path/file name to move detail file */
  snprintf(rawdirpath, sizeof(rawdirpath),
                       "%s/%s_%s.rad",
                        RAW_DIR,
                        entry->d_name,
                        ldr_tstring);

  /* --------------------------------------------------------- *
   * move detail file to RAW_DIR and create a new detail file  *
   * --------------------------------------------------------- */
  if(link(detailpath, rawdirpath) != 0) {
    snprintf(errormsg, sizeof(errormsg), 
             "[split_radius] !Error!: Cannot link %s to %s.\n", detailpath, rawdirpath);
    ldr_error(errormsg);
    return 0;
  }
  if(unlink(detailpath) != 0) {
    snprintf(errormsg, sizeof(errormsg), 
             "[split_radius] !Error!: Cannot unlink %s.\n", detailpath);
    ldr_error(errormsg);
    return 0;
  }

  /* -------------------------------------------------------------------- *
   * create new file, S_IRWXU = file owner has read/write/exec permission *
   * -------------------------------------------------------------------- */
  if(creat(detailpath, S_IRWXU) == -1) {
    snprintf(errormsg, sizeof(errormsg), 
             "[split_radius] !Error!: Cannot touch a new %s.\n", detailpath);
    ldr_error(errormsg);
  }

  return 1;
}

int split_radius() {
  int i = 0;
  int rescounter = 0;
  int detailcount = 0;
  struct dirent **radacct_dirlist;

  if(DEBUG>0) printf("[split_radius] Started in %s.\n", RAW_DIR);

  /* ------------------------------------------------------------------ *
   * The detail_select function in scandir does all the work to process *
   * detail files found in the radius accounting directory. It only     *
   * returns zero if not a single file could be processed correctly.    *
   * ------------------------------------------------------------------ */
  rescounter = scandir(RADACCT_DIR, &radacct_dirlist, detail_select, alphasort);

  if(rad_detailfiles>0)
    fprintf(logfile, "[split_radius] number of detail files found: %d\n", rad_detailfiles);
  else
    fprintf(logfile, "[split_radius] No detail files found in:%s\n", RADACCT_DIR);

  if(rescounter>0) {
    for(i=0; i < rescounter; i++) {
      fprintf(logfile, "[split_radius] successful split in: %s\n", radacct_dirlist[i]->d_name);
      detailcount++;
    }
  }
  else {
    fprintf(logfile, "[split_radius] !Error!: No successful radius_split in %s.\n", RADACCT_DIR);
  }

  return detailcount;
}
