/* ------------------------------------------------------------------------ *
 * file:        verify_dbase_mysql.c                                        *
 * purpose:     verifies the database structures are in place               *
 * author:      05/18/2010 Frank4DD                                         *
 * ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include "../edacs.h"

int verify_dbase() {
  static MYSQL *dbh;            /* database connect handle */
  static MYSQL_RES *result;     /* database query results  */
  int server_version;   	/* returned server version */
  unsigned int col_count;       /* number of columns       */
  char query_str[1024];         /* query number of columns */

  /* ------------------------- *
   * initialize the connection *
   * ------------------------- */

  dbh = mysql_init(NULL);
  if(dbh == NULL) {
    dbs_error(dbh, "[verify_dbase] !Error!:  mysql_init failed.");
    return EXIT_FAILURE;
  }

  /* --------------------- *
   * login to the database *
   * --------------------- */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !ERROR! Cannot login to %s:%d db %s with user %s pass %s\n", MYSQLIP, DB_PORT, DB_NAME, EDACSADMIN, ADMIN_PASS);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ------------------------------- *
   * Get the database version string *
   * ------------------------------- */
  server_version = mysql_get_server_version(dbh);
  printf("[verify_dbase] Connected to MySQL v%d.\n", server_version);

  /* ------------------------------------ *
   * Test if the table TEMP_TABLE exists  *
   * ------------------------------------ */
  result = mysql_list_tables(dbh, TEMP_TABLE);

  if ( result == NULL ) { 
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !Error! table %s doesn't exist.", TEMP_TABLE);
    dbs_error(dbh, errormsg);
  }
  if(DEBUG >1) printf("[verify_dbase] Table %s exists.\n", TEMP_TABLE);

  /* ----------------------------------------------- *
   * Test if the table TEMP_TABLE has enough columns *
   * ----------------------------------------------- */
  snprintf(query_str, sizeof(query_str),
    "DESCRIBE %s", TEMP_TABLE);
  if(mysql_query(dbh, query_str) != 0) {
    snprintf(errormsg, sizeof(errormsg),
      "[verify_dbase] Cannot execute DB operation, problem in %s.", query_str);
    dbs_error(dbh, errormsg);
  }

  /* get query results set */
  result = mysql_store_result(dbh);
  col_count = mysql_num_rows(result);
  printf("[verify_dbase] Number of output columns: %d [%d]\n", col_count, TEMP_TABLE_COLCOUNT);

  if(TEMP_TABLE_COLCOUNT != col_count) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !Error! table column count %d is wrong.", col_count);
    dbs_error(dbh, errormsg);
  }

  /* ------------------------- *
   * close database connection *
   * ------------------------- */
  mysql_close(dbh);
  return 0;
}
