/* ------------------------------------------------------------------------ *
 * file:        verify_dbase_oracle.c                                       *
 * purpose:     verifies the database structures are in place               *
 * author:      04/23/2006 Frank4DD                                         *
 * ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include "../edacs.h"

int verify_dbase() {
  char server_version[1024];	/* string for returned server version */

  /* ------------------------------------------------------------------- *
   * ORACLE_HOME is needed for OCI8 to find tnsnames.ora and other stuff *
   * ------------------------------------------------------------------- */
  putenv(DB_ORACLE_ENV);

  /* ------------------------- *
   * initialize the connection *
   * ------------------------- */

  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100)) {
    dbs_error(dbh, "[verify_dbase] !Error!: Failed to init libsqlora8.");
    return EXIT_FAILURE;
  }

  /* --------------------- *
   * login to the database *
   * --------------------- */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, DB_TNS_STRING)) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !ERROR! Cannot login with %s.", DB_TNS_STRING);
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }

  /* ------------------------------- *
   * Get the database version string *
   * ------------------------------- */
  if (SQLO_SUCCESS != sqlo_server_version(dbh, server_version,
                                        sizeof(server_version))) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !Error! Cannot get DB server version.");
    dbs_error(dbh, errormsg);
    return EXIT_FAILURE;
  }
  if(DEBUG >0) printf("[verify_dbase] %s.\n", server_version);

  /* ------------------------------------ *
   * Test if the table TEMP_TABLE exists  *
   * ------------------------------------ */
  if ( SQLO_SUCCESS != sqlo_exists(dbh, "USER_TABLES",
                                        "TABLE_NAME", TEMP_TABLE, NULL)) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !Error! table doesn't exist.");
    dbs_error(dbh, errormsg);
  }
  if(DEBUG >1) printf("[verify_dbase] Table %s exists.\n", TEMP_TABLE);

  /* ----------------------------------------------- *
   * Test if the table TEMP_TABLE has enough columns *
   * ----------------------------------------------- */
  if(TEMP_TABLE_COLCOUNT != sqlo_count(dbh, "USER_TAB_COLUMNS",
                                     "TABLE_NAME", TEMP_TABLE, NULL)) {
    snprintf(errormsg, sizeof(errormsg),
             "[verify_dbase] !Error! table column count is wrong.");
    dbs_error(dbh, errormsg);
  }
  if(DEBUG >1) printf("[verify_dbase] Number of columns is %d.\n", TEMP_TABLE_COLCOUNT);

  /* ------------------------- *
   * close database connection *
   * ------------------------- */
  if ( SQLO_SUCCESS != sqlo_finish(dbh)) dbs_error(dbh, "sqlo_finish");
  return 0;
}
