/* -------------------------------------------------------------------------- */
/* file:         charts.c                                                     */
/* purpose:      collection of functions used in statistics CGI's             */
/* ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <cgic.h>
#include <gd.h>
#include <gdfontt.h>
#include <gdfonts.h>
#include <gdfontmb.h>
#include <edacs.h>

#define INDEX_LINES 9
   int grey; int dkblue; int white; int dkgrey; int black;

/* -------------------------------------------------------------------- */
/* init_yeargraph - initalize year graph, draw borders and set title    */
/* -------------------------------------------------------------------- */
void init_yeargraph(char *title) {
   int i  = 0;
   int xsize = 400;
   int ysize = 160;

   imgbuf_year = gdImageCreate(xsize,ysize);

   /* allocate color maps, background color first (grey) */
   grey    = gdImageColorAllocate(imgbuf_year, 204, 204, 204);
   dkblue  = gdImageColorAllocate(imgbuf_year, 0, 0, 153);
   white   = gdImageColorAllocate(imgbuf_year, 255, 255, 255);
   dkgrey  = gdImageColorAllocate(imgbuf_year, 128, 128, 128);
   black   = gdImageColorAllocate(imgbuf_year, 0, 0, 0);

   /* makes shadow effect around the image, 2 pixels wide */
   for (i=0; i<2 ;i++) { /* do shadow effect around the image, 2 pixels wide */
      gdImageLine(imgbuf_year, i, i, xsize-i, i, white);
      gdImageLine(imgbuf_year, i, i, i, ysize-i, white);
      gdImageLine(imgbuf_year, i, ysize-i-1, xsize-i-1, ysize-i-1, dkgrey);
      gdImageLine(imgbuf_year, xsize-i-1, i, xsize-i-1, ysize-i-1, dkgrey);
   }

   /* draw the inner frame around the data, 2 pixels wide */
   gdImageRectangle(imgbuf_year, 19, 20, xsize-19, ysize-27, black);
   gdImageFilledRectangle(imgbuf_year, 20, 21, xsize-20, ysize-28, white);

   /* draw the outermost black frame line around the image, 1 pixel wide */
   gdImageRectangle(imgbuf_year, 0, 0, xsize-1, ysize-1, black);

   /* display the graph title */
   gdImageString(imgbuf_year, gdFontMediumBold, 26, 5, (unsigned char*) title, dkblue);

   return;
}

/* -------------------------------------------------------------------- */
/* init_monthgraph - initalize month graph, draw borders and set title  */
/* -------------------------------------------------------------------- */
void init_monthgraph(char *title) {
   int i  = 0;
   int xsize = 670;
   int ysize = 160;

   imgbuf_mon = gdImageCreate(xsize,ysize);

   /* allocate color maps, background color first (grey) */
   grey    = gdImageColorAllocate(imgbuf_mon, 204, 204, 204);
   dkblue  = gdImageColorAllocate(imgbuf_mon, 0, 0, 153);
   white   = gdImageColorAllocate(imgbuf_mon, 255, 255, 255);
   dkgrey  = gdImageColorAllocate(imgbuf_mon, 128, 128, 128);
   black   = gdImageColorAllocate(imgbuf_mon, 0, 0, 0);

   /* makes shadow effect around the image, 2 pixels wide */
   for (i=0; i<2 ;i++) { /* do shadow effect around the image, 2 pixels wide */
      gdImageLine(imgbuf_mon, i, i, xsize-i, i, white);
      gdImageLine(imgbuf_mon, i, i, i, ysize-i, white);
      gdImageLine(imgbuf_mon, i, ysize-i-1, xsize-i-1, ysize-i-1, dkgrey);
      gdImageLine(imgbuf_mon, xsize-i-1, i, xsize-i-1, ysize-i-1, dkgrey);
   }

   /* draw the inner frame around the data, 2 pixels wide */
   gdImageRectangle(imgbuf_mon, 19, 20, xsize-19, ysize-27, black);
   gdImageFilledRectangle(imgbuf_mon, 20, 21, xsize-20, ysize-28, white);

   /* draw the outermost black frame line around the image, 1 pixel wide */
   gdImageRectangle(imgbuf_mon, 0, 0, xsize-1, ysize-1, black);

   /* display the graph title */
   gdImageString(imgbuf_mon, gdFontMediumBold, 26, 5, (unsigned char*) title, dkblue);
   if(DEBUG>=2) printf("Finished init_monthgraph().\n");
   return;
}

/* -------------------------------------------------------------------- */
/* con_permonth  - 18 month graph router connections/month              */
/* -------------------------------------------------------------------- */
int con_permonth(  char *fname,           /* PNG file name to use */
                   char *title,           /* title for graph      */
                    int fmonth,           /* begin month number   */
                 unsigned long long data1[18]) {      /* data1 (connections)  */
   int i,x1,y1,x2,s_mth;
   unsigned long long maxval=0;
   char * msg_xlegend = "Months";
   char * msg_ylegend = "Connections";

   /* short month names MUST BE 3 CHARS in size... pad if needed*/
   static char *s_month[12] ={ "Jan", "Feb", "Mar", "Apr",
                               "May", "Jun", "Jul", "Aug",
                               "Sep", "Oct", "Nov", "Dec" };


   /* initalize the graph image */
   init_yeargraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_year,20,(i*y1)+22,380,(i*y1)+22,dkgrey);

   /* draw the x-axis legend */
   s_mth = fmonth;
   /* 18 is how many x-axis values we will display */
   for (i=0;i<18;i++) {
      /* 23 is the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_year, gdFontSmall, 23+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) s_month[s_mth-1], black);
      if(strcmp(s_month[s_mth-1], "Dec") == 0) {
        gdImageLine(imgbuf_year, 23+(i*20)+18, 21, 23+(i*20)+18, 132, dkgrey);
      }
      s_mth++;
      if (s_mth > 12) s_mth = 1;
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaltxt, "%llu", maxval);
   gdImageStringUp(imgbuf_year,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_year,gdFontMediumBold,388-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_year,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   s_mth = fmonth;
   for (i=0; i<18; i++) {
     if(data1[17-i] > 0) {
      if (s_mth > 12) s_mth = 1;
      percent = ((float)data1[17-i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = 25 + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_year, x1, y1, x2, 132, black);
    }
   }

   sprintf(maxvaltxt, "%llu", maxval);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_year, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_year);
   return (0);
}

/* -------------------------------------------------------------------- */
/* con_perday  - 1 month graph router connections/day                   */
/* -------------------------------------------------------------------- */
int con_perday(  char *fname,           /* PNG file name to use */
                 char *title,           /* title for graph      */
                 unsigned long long data1[31]) {    /* data1 (connections)  */
   int i,x1,y1,x2;
   unsigned long long maxval=0;
   char * msg_xlegend = "Day";
   char * msg_ylegend = "Connections";
   char daydate[3]    = "";
   static int sleft   = 29;

   /* initalize the graph image */
   init_monthgraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_mon,20,(i*y1)+22,650,(i*y1)+22,dkgrey);

   /* draw the x-axis legend, is how many x-axis values we will display */
   for (i=0;i<31;i++) {
      snprintf(daydate, sizeof(daydate), "%02d", i+1);
      /* sleft is start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_mon, gdFontSmall, (sleft+1)+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char *) daydate, black);
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaltxt, "%llu", maxval);
   gdImageStringUp(imgbuf_mon,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_mon,gdFontMediumBold,658-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_mon,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   for (i=0; i<31; i++) {
     if(data1[i] > 0) {
      percent = ((float)data1[i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = sleft + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_mon, x1, y1, x2, 132, black);
    }
   }

   sprintf(maxvaltxt, "%llu", maxval);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_mon, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_mon);
   return (0);
}

/* -------------------------------------------------------------------- */
/* bytes_permonth  - 18 month graph for router bytes/month              */
/* -------------------------------------------------------------------- */
int bytes_permonth(char *fname,           /* PNG file name to use */
                   char *title,           /* title for graph      */
                    int fmonth,           /* begin month number   */
                 unsigned long long data1[18],        /* data1 (bytes in)     */
                 unsigned long long data2[18],        /* data2 (bytes out)    */
                 unsigned long long data3[18]) {      /* data3 (total bytes)  */
   int i = 0;
   int x1 =0;
   int y1 = 0;
   int x2 = 0;
   int s_mth = 0;
   unsigned long long maxval1,maxval2,maxval3=0;
   char * msg_xlegend = "Months";
   char * msg_ylegend = "Data";
   char maxvaldata[255] = "";
   /* short month names MUST BE 3 CHARS in size... pad if needed */
   static char *s_month[12] ={ "Jan", "Feb", "Mar", "Apr",
                               "May", "Jun", "Jul", "Aug",
                               "Sep", "Oct", "Nov", "Dec" };


   /* initalize the graph image */
   init_yeargraph(title);

   /* allocate additional colors, ater init_yeargraph()  */
   int cyan    = gdImageColorAllocate(imgbuf_year, 0, 128, 128);
   int purple  = gdImageColorAllocate(imgbuf_year, 128, 0, 128);


   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_year,20,(i*y1)+22,380,(i*y1)+22,dkgrey);

   /* draw the x-axis legend */
   s_mth = fmonth;
   /* 18 is how many x-axis values we will display */
   for (i=0;i<18;i++) {
      /* 23 is the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_year, gdFontSmall, 23+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) s_month[s_mth-1], black);
      /* print a line between years (Dec and Jan) */
      if(strcmp(s_month[s_mth-1], "Dec") == 0) {
        gdImageLine(imgbuf_year, 23+(i*20)+18, 21, 23+(i*20)+18, 132, dkgrey);
      }
      s_mth++;
      if (s_mth > 12) s_mth = 1;
       if (data1[i] > maxval1) maxval1 = data1[i];           /* get max val 1  */
       if (data2[i] > maxval2) maxval2 = data2[i];           /* get max val 2  */
       if (data3[i] > maxval3) maxval3 = data3[i];           /* get max val 3  */
   }

   sprintf(maxvaldata, "%llu", maxval3);
   calc_units(maxvaldata, maxvaltxt);
   gdImageStringUp(imgbuf_year,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   gdImageString(imgbuf_year,gdFontMediumBold,310,4,(unsigned char*) "In",cyan);
   gdImageString(imgbuf_year,gdFontMediumBold,328,4,(unsigned char*) "Out",purple);
   gdImageString(imgbuf_year,gdFontMediumBold,350,4,(unsigned char*) "Total",dkblue);

   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_year,gdFontMediumBold,388-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_year,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   s_mth = fmonth;
   for (i=0; i<18; i++) {
     if(data1[17-i] > 0) {
      if (s_mth > 12) s_mth = 1;
      percent = ((float)data1[17-i] / (float)maxval3);
      if (percent <= 0.0) continue;
      x1 = 24 + (i*20);
      x2 = x1 + 12;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1, y1, x1+5, 132, cyan);
    }

     if(data2[17-i] > 0) {
      percent = ((float)data2[17-i] / (float)maxval3);
      if (percent <= 0.0) continue;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1+4, y1, x1+10, 132, purple);
    }

     if(data3[17-i] > 0) {
      percent = ((float)data3[17-i] / (float)maxval3);
      if (percent <= 0.0) continue;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1+8, y1, x1+8+4, 132, dkblue);
    }
   }

   sprintf(maxvaldata, "%llu", maxval3);
   calc_units(maxvaldata, maxvaltxt);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_year, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_year);
   return (0);
} /* end bytes_permonth() */

/* -------------------------------------------------------------------- */
/* bytes_perday  - 1 month graph router bytes/day                       */
/* -------------------------------------------------------------------- */
int bytes_perday(char *fname,           /* PNG file name to use */
                   char *title,           /* title for graph      */
                 unsigned long long data1[31],        /* data1 (bytes in)     */
                 unsigned long long data2[31],        /* data2 (bytes out)    */
                 unsigned long long data3[31]) {      /* data3 (total bytes)  */
   int i = 0;
   int x1 = 0;
   int y1 = 0;
   int x2 = 0;
   unsigned long long maxval1,maxval2,maxval3=0;
   char * msg_xlegend = "Day";
   char * msg_ylegend = "Data";
   char daydate[3]    = "";
   static int sleft   = 29;
   char maxvaldata[255] ="";

   /* initalize the graph image */
   init_monthgraph(title);

   /* allocate additional colors, ater init_monthgraph()  */
   int cyan    = gdImageColorAllocate(imgbuf_mon, 0, 128, 128);
   int purple  = gdImageColorAllocate(imgbuf_mon, 128, 0, 128);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_mon,20,(i*y1)+22,650,(i*y1)+22,dkgrey);

   /* draw the x-axis legend, 31 is how many x-axis values we will display */
   for (i=0;i<31;i++) {
      snprintf(daydate, sizeof(daydate), "%02d", i+1);
      /* sleft is start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_mon, gdFontSmall, (sleft+1)+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char *) daydate, black);
       if (data1[i] > maxval1) maxval1 = data1[i];           /* get max val 1  */
       if (data2[i] > maxval2) maxval2 = data2[i];           /* get max val 2  */
       if (data3[i] > maxval3) maxval3 = data3[i];           /* get max val 3  */
   }

   /* create the legend max value */
   sprintf(maxvaldata, "%llu", maxval3);
   calc_units(maxvaldata, maxvaltxt);
   gdImageStringUp(imgbuf_mon,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* write the legends data description */
   gdImageString(imgbuf_mon,gdFontMediumBold,560,4,(unsigned char*) "In",cyan);
   gdImageString(imgbuf_mon,gdFontMediumBold,590,4,(unsigned char*) "Out",purple);
   gdImageString(imgbuf_mon,gdFontMediumBold,620,4,(unsigned char*) "Total",dkblue);

   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_mon,gdFontMediumBold,658-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_mon,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here, but only if there is data */
   for (i=0; i<31; i++) {
     if(data1[i] > 0) {
      percent = ((float)data1[i] / (float)maxval3);
      if (percent <= 0.0) continue;
      x1 = (sleft-1) + (i*20);
      x2 = x1 + 12;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1, y1, x1+5, 132, cyan);
    }

     if(data2[i] > 0) {
      percent = ((float)data2[i] / (float)maxval3);
      if (percent <= 0.0) continue;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1+4, y1, x1+10, 132, purple);
    }

     if(data3[i] > 0) {
      percent = ((float)data3[i] / (float)maxval3);
      if (percent <= 0.0) continue;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1+8, y1, x1+8+4, 132, dkblue);
    }
   }

   sprintf(maxvaldata, "%llu", maxval3);
   calc_units(maxvaldata, maxvaltxt);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_mon, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_mon);
   return (0);
} /* end bytes_perday() */

/* -------------------------------------------------------------------- */
/* user_permonth  - 18 month graph users/month                          */
/* -------------------------------------------------------------------- */
int user_permonth(  char *fname,          /* PNG file name to use */
                   char *title,           /* title for graph      */
                    int fmonth,           /* begin month number   */
                 unsigned long long data1[18]) {      /* data1 (users)        */
   int i,x1,y1,x2,s_mth;
   unsigned long long maxval=0;
   char * msg_xlegend = "Months";
   char * msg_ylegend = "Users";

   /* short month names MUST BE 3 CHARS in size... pad if needed*/
   static char *s_month[12] ={ "Jan", "Feb", "Mar", "Apr",
                               "May", "Jun", "Jul", "Aug",
                               "Sep", "Oct", "Nov", "Dec" };


   /* initalize the graph image */
   init_yeargraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_year,20,(i*y1)+22,380,(i*y1)+22,dkgrey);

   /* draw the x-axis legend */
   s_mth = fmonth;
   /* 18 is how many x-axis values we will display */
   for (i=0;i<18;i++) {
      /* 23 is the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_year, gdFontSmall, 23+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) s_month[s_mth-1], black);
      if(strcmp(s_month[s_mth-1], "Dec") == 0) {
        gdImageLine(imgbuf_year, 23+(i*20)+18, 21, 23+(i*20)+18, 132, dkgrey);
      }
      s_mth++;
      if (s_mth > 12) s_mth = 1;
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaltxt, "%llu", maxval);
   gdImageStringUp(imgbuf_year,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_year,gdFontMediumBold,388-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_year,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   s_mth = fmonth;
   for (i=0; i<18; i++) {
     if(data1[17-i] > 0) {
      if (s_mth > 12) s_mth = 1;
      percent = ((float)data1[17-i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = 25 + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_year, x1, y1, x2, 132, black);
    }
   }

   sprintf(maxvaltxt, "%llu", maxval);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_year, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_year);
   return (0);
} /* end user_permonth() */

/* -------------------------------------------------------------------- */
/* user_perday - 1 month graph unique users/day                         */
/* -------------------------------------------------------------------- */
int user_perday( char *fname,          /* PNG file name to use */
                 char *title,          /* title for graph      */
                 unsigned long long data1[31]) {   /* data1 (users)        */
   int i,x1,y1,x2;
   unsigned long long maxval=0;
   char * msg_xlegend = "Day";
   char * msg_ylegend = "Users";
   char daydate[3]    = "";
   static int sleft   = 29;

   /* initalize the graph image */
   init_monthgraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_mon,20,(i*y1)+22,650,(i*y1)+22,dkgrey);

   /* draw the x-axis legend, 31 is how many x-axis values we will display */
   for (i=0;i<31;i++) {
      snprintf(daydate, sizeof(daydate), "%02d", i+1);
      /* sleft the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_mon, gdFontSmall, (sleft+1)+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) daydate, black);
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaltxt, "%llu", maxval);
   gdImageStringUp(imgbuf_mon,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_mon,gdFontMediumBold,658-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_mon,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   for (i=0; i<31; i++) {
     if(data1[i] > 0) {
      percent = ((float)data1[i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = sleft + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_mon, x1, y1, x2, 132, black);
    }
   }
   sprintf(maxvaltxt, "%llu", maxval);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_mon, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_mon);
   return (0);
} /* end user_perday() */

/* -------------------------------------------------------------------- */
/* avgb_permonth  - 18 month graph avg bytes per user/month             */
/* -------------------------------------------------------------------- */
int avgb_permonth( char *fname,          /* PNG file name to use */
                   char *title,           /* title for graph      */
                    int fmonth,           /* begin month number   */
                 unsigned long long data1[18]) {      /* data1 (users)        */
   int i,x1,y1,x2,s_mth;
   unsigned long long maxval=0;
   char * msg_xlegend = "Months";
   char * msg_ylegend = "Data";
   char maxvaldata[255] ="";

   /* short month names MUST BE 3 CHARS in size... pad if needed*/
   static char *s_month[12] ={ "Jan", "Feb", "Mar", "Apr",
                               "May", "Jun", "Jul", "Aug",
                               "Sep", "Oct", "Nov", "Dec" };


   /* initalize the graph image */
   init_yeargraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_year,20,(i*y1)+22,380,(i*y1)+22,dkgrey);

   /* draw the x-axis legend */
   s_mth = fmonth;
   /* 18 is how many x-axis values we will display */
   for (i=0;i<18;i++) {
      /* 23 is the start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_year, gdFontSmall, 23+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) s_month[s_mth-1], black);
      if(strcmp(s_month[s_mth-1], "Dec") == 0) {
        gdImageLine(imgbuf_year, 23+(i*20)+18, 21, 23+(i*20)+18, 132, dkgrey);
      }
      s_mth++;
      if (s_mth > 12) s_mth = 1;
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaldata, "%llu", maxval);
   calc_units(maxvaldata, maxvaltxt);
   gdImageStringUp(imgbuf_year,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_year,gdFontMediumBold,388-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_year,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   s_mth = fmonth;
   for (i=0; i<18; i++) {
     if(data1[17-i] > 0) {
      if (s_mth > 12) s_mth = 1;
      percent = ((float)data1[17-i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = 25 + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_year, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_year, x1, y1, x2, 132, black);
    }
   }

   sprintf(maxvaldata, "%llu", maxval);
   calc_units(maxvaldata, maxvaltxt);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_year, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_year);
   return (0);
}

/* -------------------------------------------------------------------- */
/* avgb_perday  -  1 month graph average MB/user per day                */
/* -------------------------------------------------------------------- */
int avgb_perday( char *fname,          /* PNG file name to use */
                 char *title,          /* title for graph      */
                 unsigned long long data1[31]) {   /* data1 (users)        */
   int i,x1,y1,x2;
   unsigned long long maxval=0;
   char * msg_xlegend = "Day";
   char * msg_ylegend = "Data";
   char daydate[3]    = "";
   static int sleft   = 29;
   char maxvaldata[255] = "";

   /* initalize the graph image */
   init_monthgraph(title);

   /* draw index lines */
   y1=110/(INDEX_LINES+1);
   for (i=1;i<=INDEX_LINES;i++)
     gdImageLine(imgbuf_mon,20,(i*y1)+22,650,(i*y1)+22,dkgrey);

   /* draw the x-axis legend, 31 is how many x-axis values we will display */
   for (i=0;i<31;i++) {
      snprintf(daydate, sizeof(daydate), "%02d", i+1);
      /* sleft is start pixel from left, 20 is the distance between each value */
      gdImageString(imgbuf_mon, gdFontSmall, (sleft+1)+(i*20),
                    /* 134 is the start pixel from top, black the font color */
                    134, (unsigned char*) daydate, black);
      if (data1[i] > maxval) maxval = data1[i];           /* get max val    */
   }

   sprintf(maxvaldata, "%llu", maxval);
   calc_units(maxvaldata, maxvaltxt);
   gdImageStringUp(imgbuf_mon,gdFontSmall,6,19+(strlen(maxvaltxt)*6),(unsigned char*) maxvaltxt,black);

   /* print color coded legends */
   /* X Legend */
   i = (strlen(msg_xlegend)*6);
   gdImageString(imgbuf_mon,gdFontMediumBold,658-i,144,(unsigned char*) msg_xlegend,black);

   /* Y Legend */
   i = (strlen(msg_ylegend)*6);
   gdImageStringUp(imgbuf_mon,gdFontMediumBold,6,132,(unsigned char*) msg_ylegend,black);

   /* draw the data bars here */
   for (i=0; i<31; i++) {
     if(data1[i] > 0) {
      percent = ((float)data1[i] / (float)maxval);
      if (percent <= 0.0) continue;
      x1 = sleft + (i*20);
      x2 = x1 + 11;
      y1 = 132 - (percent * 110);
      gdImageFilledRectangle(imgbuf_mon, x1, y1, x2, 132, dkblue);
      gdImageRectangle(imgbuf_mon, x1, y1, x2, 132, black);
    }
   }

   sprintf(maxvaldata, "%llu", maxval);
   calc_units(maxvaldata, maxvaltxt);

   /* save png image */
   if ((pngout = fopen(fname, "wb")) != NULL) {
      gdImagePng(imgbuf_mon, pngout);
      fclose(pngout);
   }
   /* deallocate memory */
   gdImageDestroy(imgbuf_mon);
   return (0);
}
