/* -------------------------------------------------------------------------- *
 * file:         edacs-chartsgen.c                                            *
 * purpose:      generate the edacs charts image files in IMG_DIR             *
 * Requirements: EDAcS DB with data in edacs_monthstats                       *
 * -------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../edacs.h"

unsigned int i,j        = 0;	/* for-loop counters */
int month, year         = 0;  /* the date values for calculation */
char time_range[18][8];
char time_range_mon[DAYSTATS_HIST][8];
char time_range_day[DAYSTATS_HIST][31][11];

/* ------------------------------------------------------------------- */
/* function: get_hist_month()                                          */
/* Calculate the start time string for the graphs, based on this month */
/* for thew past 18 months, saving the strings in the array time_range */
/* ------------------------------------------------------------------- */
void get_hist_month(struct tm *tm) {

  for(i=0; i<18; i++) {
    month = (tm->tm_mon+1) - i;
    if(month > 0) {
       year = tm->tm_year + 1900;
       month = (tm->tm_mon+1) - i;
    }
    else if (month > -12 ) {
       year = tm->tm_year + 1899;
       month = (tm->tm_mon+13) - i;
    }
    else {
       year = tm->tm_year + 1898;
       month = (tm->tm_mon+25) - i;
    }

    snprintf(time_range[i], sizeof(time_range[i]), "%d-%02d", year, month);
    /* debug only */
    if(DEBUG >= 2)
      printf("get_hist_month() i: %d Month: %s\n", i, time_range[i]);
  }
}

/* ------------------------------------------------------------------- */
/* function: get_hist_day()                                            */
/* Calculate the start time string for the graphs, based on this month */
/* for thew past DAYSTATS_HIST months, saving the strings in the array */
/* time_range_day                                                      */
/* ------------------------------------------------------------------- */
void get_hist_day(struct tm *tm) {

  for(i=0; i<DAYSTATS_HIST; i++) {
    month = (tm->tm_mon+1) - i;
    if(month > 0) {
       year = tm->tm_year + 1900;
       month = (tm->tm_mon+1) - i;
    }
    else if (month > -12 ) {
       year = tm->tm_year + 1899;
       month = (tm->tm_mon+13) - i;
    }
    else {
       year = tm->tm_year + 1898;
       month = (tm->tm_mon+25) - i;
    }

    snprintf(time_range_mon[i], sizeof(time_range_mon[i]), "%d-%02d", year, month);
    /* debug only */
    if(DEBUG >= 2)
      printf("get_hist_day() i: %d Month: %s\n", i, time_range_mon[i]);

    for(j=0; j<31; j++) {
      snprintf(time_range_day[i][j], sizeof(time_range_day[i][j]), "%d-%02d-%02d", year, month, j+1);
      /* debug only */
      if(DEBUG >= 2)
        printf("get_hist_day() i: %d Day: %s\n", i, time_range_day[i][j]);
    }
  }
}

int cgiMain() {
#ifdef MYSQL_DB
  static MYSQL *dbh;              /* database connect handle */
  static MYSQL_RES *result;       /* database query results  */
  static MYSQL_ROW values;        /* query data returned     */
  unsigned int colcount    =0;    /* number of returned columns */
  int server_version;             /* returned server version */
#endif
#ifdef ORACLE_DB
  sqlo_db_handle_t dbh;           /* database handle */
  sqlo_stmt_handle_t sth1;        /* statement handle 1 */
  char server_version[1024]="";   /* string for returned server version */
  int stat                 =0;    /* status of sqlo calls */
  int handle               =0;    /* handle of the interrupt handler */
  const char ** values;           /* values */
#endif
  int i,j = 0;                           /* loop counter i,j */
  char sqlquery_str[1024]  = "";         /* SQL query string 1              */
  char charts_file[1024]   = "";         /* charts image filename and path  */
  char charts_title[1024]  = "";         /* charts image title              */
  unsigned int routercount = 0;	         /* router counter                  */
  unsigned int rowcount   = 0;	         /* rowcounter                      */
  struct tm *tm_ptr;                     /* containing time structure       */
  time_t now;                            /* containing timestamp            */
  char router_list[MAX_ROUTERS][1024];   /* list of all routers reporting   */
  int latestmonth         = -1;
  int beginmonth          = -1;
  char err_str[2048]       ="";           /* use for combined error string  */
  now = time(NULL);
  tm_ptr = localtime(&now);
  get_hist_month(tm_ptr);
  get_hist_day(tm_ptr);
  _abort_flag     = 0;

    if(DEBUG >= 0) printf("[edacs-chartsgen] Start charts generation.\n");

#ifdef ORACLE_DB
  /* ------------------------------------------------------------------- */
  /* ORACLE_HOME is needed for OCI8 to find tnsnames.ora                 */
  /* ------------------------------------------------------------------- */
  putenv(WEB_ORACLE_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100))
    printf("Error: Failed to init libsqlora8.");

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, WEB_TNS_STRING))
    printf("Error: Cannot connect to database.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* enable autocommit, each statement is commited as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  /* ------------------------------------------------------------------- */
  /* We query the list of available routers from edacs_router.           */
  /* ------------------------------------------------------------------- */

  /* define the SQL query */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT ROUTER FROM %s.EDACS_ROUTER", EDACSADMIN);
  if(DEBUG >= 2) printf("[edacs-chartsgen] DB query: %s.\n", sqlquery_str);

  /* initialize the statement handle */
  sth1 = SQLO_STH_INIT;

  /* opens a cursor for the query statement */
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
    printf(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  i=0;
  while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
    /* get one record */
    values = sqlo_values(sth1, NULL, 1);
    strncpy(router_list[i], values[0], sizeof(router_list[i])-1);
    i++;
  }

  if (SQLO_SUCCESS != sqlo_close(sth1))
    printf("Error Closing the SQL statment handle.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
  /* initialize the connection */
  dbh = mysql_init(NULL);
  if(dbh == NULL) printf("[edacs-chartsgen] Error:  Failed to init MySQL DB.");

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0)
    printf("[edacs-chartsgen] Error: Cannot connect to database.");

  /* Get the database version */
  server_version = mysql_get_server_version(dbh);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT router FROM edacs_router");
  if(DEBUG >= 2) printf("[edacs-chartsgen] DB query: %s;\n", sqlquery_str);

  /* Prepare and execute the SQL statement */
  if(mysql_query(dbh, sqlquery_str) != 0) {
    snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] DB error %s\n\nQuery string %s",
    mysql_error(dbh), sqlquery_str);
    printf(err_str);
  }
  /* get query results set */
  result = mysql_store_result(dbh);
  if (result == NULL) {
    snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] Error: no results for query: %s\n", sqlquery_str);
    printf(err_str);
  }

  i=0;
  while((values = mysql_fetch_row(result)) != NULL) {
    strncpy(router_list[i],values[0], sizeof(router_list[i])-1);
    i++;
  }
  colcount = mysql_num_fields(result);
#endif

  if(DEBUG >= 1) printf("[edacs-chartsgen] executed query: %s;\n", sqlquery_str);

  /* debug only, print the router list */
  if(DEBUG >= 1) {
    i=0;
    while (strlen(router_list[i]) != 0) {
      printf("[edacs-chartsgen] Router %d: %s\n", i, router_list[i]);
      i++;
    }
  }

  /* ------------------------------------------------------------------- */
  /* For every router, we process the values and call the chart function */
  /* ------------------------------------------------------------------- */

  routercount=0;
  while (strlen(router_list[routercount]) != 0) {

    /* create and zero all values for the 18-months charts */
    unsigned long long year_connects[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned long long year_bytesin[]   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned long long year_bytesout[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned long long year_bytesall[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned long long year_usercount[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    unsigned long long year_avgbytes[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    /* select data for this router from edacs_monstats */
#ifdef ORACLE_DB
    snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT ROUTER, MONTH, CONNECTS, BYTES_IN, BYTES_OUT, USERCOUNT FROM edacsadmin.EDACS_MONSTATS WHERE ROUTER='%s'", router_list[routercount]);
    if(DEBUG >= 1) printf("Query: %s\n", sqlquery_str); /* debug only */
    
    sth1 = SQLO_STH_INIT;
    if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
      printf(sqlo_geterror(dbh));

    /* fetch the selected edacs_monstats data */
    rowcount = 0;
    while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
      /* get one record and keep it in values */
      values = sqlo_values(sth1, NULL, 1);
#endif
#ifdef MYSQL_DB
    /* create the SQL query string */
    snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT router, month, connects, bytes_in, bytes_out, usercount FROM edacs_monstats WHERE router='%s'", router_list[routercount]);

    /* Prepare and execute the SQL statement */
    if(mysql_query(dbh, sqlquery_str) != 0) {
      snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] DB error %s\n\nQuery string %s",
      mysql_error(dbh), sqlquery_str);
      if(DEBUG >= 1) printf(err_str);
    }
    /* get query results set */
    result = mysql_store_result(dbh);
    if (result == NULL) {
      snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] No results for query: %s\n", sqlquery_str);
      printf( err_str);
    }

    colcount = mysql_num_fields(result);
    rowcount = 0;
    while((values = mysql_fetch_row(result)) != NULL) {
#endif

      printf("[edacs-chartsgen] router: %s date: %s\n", values[0], values[1]);

      for(i=0; i<18; i++) {
        if(strcmp(time_range[i], values[1]) == 0) {
          if(values[2]) year_connects[i] = atoll(values[2]);
          if(values[3]) year_bytesin[i]  = atoll(values[3]);
          if(values[4]) year_bytesout[i] = atoll(values[4]);
          if(values[3] || values[4]) year_bytesall[i] = atoll(values[3]) + atoll(values[4]);
          if(values[5]) year_usercount[i] = atoll(values[5]);
          if((values[3] || values[4]) && values[5]) year_avgbytes[i] = (atoll(values[3]) + atoll(values[4]))/atoll(values[5]);
        }

        /* debug: this line prints the data processed for the graph */
        if(DEBUG >= 2) printf("row %d\trtr %s\tmon %s\tcon  %4s\tbin (%s)%4d\tbout (%s)%4d\tball %4d\n", rowcount, values[0], values[1], values[2], values[3], (int) year_bytesin[i], values[4], (int) year_bytesout[i], (int) year_bytesall[i]);
      }
      rowcount++;
    }
    /* finished query for this router, close the statement handle */
#ifdef ORACLE_DB
    if (SQLO_SUCCESS != sqlo_close(sth1))
      printf("Error Closing the SQL statment handle.");
    RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif

    /* calculate begin month (now-17) */
    latestmonth = atoi(strchr(time_range[0], '-')+1);
    if(latestmonth >5) beginmonth = latestmonth -5;
    else beginmonth = latestmonth +7;

    /* create the 18-month charts here */
    snprintf(charts_file, sizeof(charts_file), "%s/constats-%s.png", IMG_DIR, router_list[routercount]);
    if(DEBUG >= 2) printf("Create file: %s\n", charts_file);
    snprintf(charts_title, sizeof(charts_title), "Router %s - Connections per Month", router_list[routercount]);
    if(DEBUG >= 2) printf("Charts title: %s\n", charts_title);
    con_permonth( charts_file,          /* filename          */
                  charts_title,         /* graph title       */
                  beginmonth,           /* begin month       */
                  year_connects);    
    snprintf(charts_file, sizeof(charts_file), "%s/bytestats-%s.png", IMG_DIR, router_list[routercount]);
    if(DEBUG >= 2) printf("Create file: %s\n", charts_file);
    snprintf(charts_title, sizeof(charts_title), "Router %s - Data/Month", router_list[routercount]);
    if(DEBUG >= 2) printf("Charts title: %s\n", charts_title);
    bytes_permonth( charts_file,          /* filename          */
                    charts_title,         /* graph title       */
                    beginmonth,           /* begin month       */
                    year_bytesin,
                    year_bytesout,
                    year_bytesall);
    snprintf(charts_file, sizeof(charts_file), "%s/usercount-%s.png", IMG_DIR, router_list[routercount]);
    if(DEBUG >= 2) printf("Create file: %s\n", charts_file);
    snprintf(charts_title, sizeof(charts_title), "Router %s - Users per Month", router_list[routercount]);
    if(DEBUG >= 2) printf("Charts title: %s\n", charts_title);
    user_permonth(  charts_file,          /* filename          */
                    charts_title,         /* graph title       */
                    beginmonth,           /* begin month       */
                    year_usercount);
    snprintf(charts_file, sizeof(charts_file), "%s/avgbytes-%s.png", IMG_DIR, router_list[routercount]);
    if(DEBUG >= 2) printf("Create file: %s\n", charts_file);
    snprintf(charts_title, sizeof(charts_title), "Router %s - Avg Data per User/Month", router_list[routercount]);
    if(DEBUG >= 2) printf("Charts title: %s\n", charts_title);
    avgb_permonth(  charts_file,          /* filename          */
                    charts_title,         /* graph title       */
                    beginmonth,           /* begin month       */
                    year_avgbytes);

    /* ----------------------------------------------------------- */
    /* End creation of 18-month charts  for each router            */
    /* ----------------------------------------------------------- */
    /* Start data collection of monthly day charts for each router */
    /* ----------------------------------------------------------- */

    for(i=0; i<DAYSTATS_HIST; i++) {

      /* create values for the monthly day charts */
      unsigned long long month_connects[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      unsigned long long month_bytesin[]   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      unsigned long long month_bytesout[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      unsigned long long month_bytesall[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      unsigned long long month_usercount[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      unsigned long long month_avgbytes[]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    
      /* select data for this router from edacs_daystats */
#ifdef ORACLE_DB
      snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT ROUTER, DAY, CONNECTS, BYTES_IN, BYTES_OUT, USERCOUNT FROM edacsadmin.EDACS_DAYSTATS WHERE ROUTER='%s' AND DAY LIKE '%s%%' ORDER BY DAY ASC", router_list[routercount], time_range_mon[i]);
      if(DEBUG >= 1) printf("[edacs-chartsgen] Query: %s\n", sqlquery_str); /* debug only */

      sth1 = SQLO_STH_INIT;
      if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
        printf(sqlo_geterror(dbh));
    
      /* fetch the selected edacs_daystats data */
      rowcount = 0;
      while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
        /* get one record and keep it in values */
        values = sqlo_values(sth1, NULL, 1);
#endif
#ifdef MYSQL_DB
      /* create the SQL query string */
      snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT router, day, connects, bytes_in, bytes_out, usercount FROM edacs_daystats WHERE router='%s' AND day LIKE '%s%%' ORDER BY day ASC", router_list[routercount], time_range_mon[i]);
      if(DEBUG >= 1) printf("[edacs-chartsgen] Query: %s\n", sqlquery_str); /* debug only */
    
      /* Prepare and execute the SQL statement */
      if(mysql_query(dbh, sqlquery_str) != 0) {
        snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] DB error %s\n\nQuery string %s",
        mysql_error(dbh), sqlquery_str);
        if(DEBUG >= 0) printf(err_str);
      }
      /* get query results set */
      result = mysql_store_result(dbh);
      if (result == NULL) {
        snprintf(err_str, sizeof(err_str), "[edacs-chartsgen] No results for query: %s\n", sqlquery_str);
        printf( err_str);
      }
    
      colcount = mysql_num_fields(result);

      while((values = mysql_fetch_row(result)) != NULL) {
#endif
        if(DEBUG >= 2) printf("[edacs-chartsgen] router: %s date: %s\n", values[0], values[1]);

          for(j=0; j<31; j++) {
            if(DEBUG>=2) printf("i: %d j: %d time_range_day: %s values[1]: %s\n", i, j, time_range_day[i][j], values[1]);
            if(strcmp(time_range_day[i][j], values[1]) == 0) {
              month_connects[j]  = atoll(values[2]);
              month_bytesin[j]   = atoll(values[3]);
              month_bytesout[j]  = atoll(values[4]);
              month_bytesall[j]  = atoll(values[3]) + atoll(values[4]);
              month_usercount[j] = atoll(values[5]);
              month_avgbytes[j]  = (atol(values[3]) + atoll(values[4]))/atoll(values[5]);
            }

            /* debug: this line prints the data processed for the graph */
            if(DEBUG >= 2) printf("row %d\trtr %s\tmon %s\tcon  %4s\tbin (%s)%4d\tbout (%s)%4d\tball %4d\n", rowcount, values[0], values[1], values[2], values[3], (int) month_bytesin[j], values[4], (int) month_bytesout[j], (int) month_bytesall[j]);
          } /* close j - daycount */
        } /* close while db rowcount */
        rowcount++;
        /* create the day charts here, one per month in DAYSTATS_HIST */
        snprintf(charts_file, sizeof(charts_file), "%s/constats-mon%d-%s.png", IMG_DIR, i, router_list[routercount]);
        if(DEBUG >= 2) printf("Create file: %s\n", charts_file);
        snprintf(charts_title, sizeof(charts_title), "Router %s - Daily Connections %s", router_list[routercount], time_range_mon[i]);
        if(DEBUG >= 2) printf("Charts title: %s\n", charts_title);
        con_perday( charts_file,          /* filename          */
                   charts_title,         /* graph title       */
                   month_connects);
        /* bytes per day and router */
        snprintf(charts_file, sizeof(charts_file), "%s/bytestats-mon%d-%s.png", IMG_DIR, i, router_list[routercount]);
        snprintf(charts_title, sizeof(charts_title), "Router %s - Data/Day %s", router_list[routercount], time_range_mon[i]);
        bytes_perday( charts_file,          /* filename          */
                     charts_title,         /* graph title       */
                     month_bytesin,
                     month_bytesout,
                     month_bytesall);
        /* users per day and router */
        snprintf(charts_file, sizeof(charts_file), "%s/usercount-mon%d-%s.png", IMG_DIR, i, router_list[routercount]);
        snprintf(charts_title, sizeof(charts_title), "Router %s - Users/Day %s", router_list[routercount], time_range_mon[i]);
        user_perday(  charts_file,          /* filename          */
                      charts_title,         /* graph title       */
                      month_usercount);
        /* average mb/user per day and router */
        snprintf(charts_file, sizeof(charts_file), "%s/avgbytes-mon%d-%s.png", IMG_DIR, i, router_list[routercount]);
        snprintf(charts_title, sizeof(charts_title), "Router %s - Avg Data per User/Day %s", router_list[routercount], time_range_mon[i]);
        avgb_perday(  charts_file,          /* filename          */
                      charts_title,         /* graph title       */
                      month_avgbytes);
    } /* close i - monthcount */
    /* ---------------------------------------------------------- */
    /* End data collection of monthly day charts for each router  */
    /* ---------------------------------------------------------- */
    /* finished query for this router, close the statement handle */
    /* ---------------------------------------------------------- */
#ifdef ORACLE_DB
    if (SQLO_SUCCESS != sqlo_close(sth1))
      printf("Error Closing the SQL statment handle.");
    RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
    routercount++;
  }  
  /* ------------------------------------------------------- */
  /* End creation of monthly day charts for each router      */
  /* ------------------------------------------------------- */
  return 1;
}
