/* -------------------------------------------------------------------------- *
 * file:         index.c                                                      *
 * purpose:      display EDAcS main index page                                *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {
#ifdef MYSQL_DB
  static MYSQL *dbh;              /* database connect handle */
  static MYSQL_RES *result;       /* database query results  */
  static MYSQL_ROW values;        /* query data returned     */
  int server_version;             /* returned server version */
#endif
#ifdef ORACLE_DB
  sqlo_db_handle_t dbh;           /* database handle */
  sqlo_stmt_handle_t sth1;        /* statement handle 1 */
  char server_version[1024]="";   /* string for returned server version */
  int stat                 =0;    /* status of sqlo calls */
  int handle               =0;    /* handle of the interrupt handler */
  const char **values;            /* values */
  //const char ** colnames;       /* column names */
#endif
  char sqlquery_str[1024]  ="";   /* SQL query string */
  char err_str[2048]       ="";   /* use for combined error string */

  static char title[] = "Main Page";

#ifdef ORACLE_DB
  /* ------------------------------------------------------------------- *
   * ORACLE_HOME is needed for OCI8 to find tnsnames.ora                 *
   * ------------------------------------------------------------------- */
  putenv(WEB_ORACLE_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100))
    cgi_error("Error: Failed to init libsqlora8.");

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, WEB_TNS_STRING))
    cgi_error("Error: Cannot connect to database.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* Get the database version */
  if (SQLO_SUCCESS != s/lo_server_version(dbh, server_version,
                                        sizeof(server_version)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* enable autocommit, each statement is commited as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT %s, %s, %s, %s, %s FROM %s.EDACS_VERSION",
           "ID_STRING", "TO_CHAR(CREATION, 'dd.mm.yyyy hh24:mi:ss')",
           "USER_COUNT", "ROUTER_COUNT",
           "SESSION_COUNT", EDACSADMIN);

  /* initialize the statement handle */
  sth1 = SQLO_STH_INIT;

  /* opens a cursor for the query statement */
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL))) {
    if(DEBUG == 0) cgi_error(sqlo_geterror(dbh));
    else snprintf(err_str, sizeof(err_str), "DB error %s\n\nQuery string %s",
             sqlo_geterror(dbh), sqlquery_str);
    cgi_error(err_str);
  }
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* get the output column names */
  //if (SQLO_SUCCESS != sqlo_ocol_names2(sth1, &colcount, &colnames))
  //  cgi_error("Error getting the DB columns with sqlo_ocol_names2()");
  //RETURN_ON_ABORT; /* finish if SIGINT was catched */
 
  if (SQLO_SUCCESS != (stat = (sqlo_fetch(sth1, 1))))
    cgi_error("Error retrieving the rows from table.");

  values = sqlo_values(sth1, NULL, 1);
#endif

#ifdef MYSQL_DB
  /* initialize the connection */
  dbh = mysql_init(NULL);
  if(dbh == NULL) cgi_error("Error:  Failed to init MySQL DB.");

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0)
    cgi_error("Error: Cannot connect to database.");

  /* Get the database version */
  server_version = mysql_get_server_version(dbh);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT %s, %s, %s, %s, %s FROM edacs_version",
           "id_string", "DATE_FORMAT(creation, '%d.%m.%Y %H:%i:%s')",
           "user_count", "router_count",
           "session_count");

  /* Prepare and execute the SQL statement */
  if(mysql_query(dbh, sqlquery_str) != 0) {
    if(DEBUG == 0) cgi_error(mysql_error(dbh));
    else snprintf(err_str, sizeof(err_str), "DB error %s\n\nQuery string %s",
             mysql_error(dbh), sqlquery_str);
    cgi_error(err_str);
  }
 /* get query results set */
  result = mysql_store_result(dbh);
  if (result == NULL) {
    snprintf(err_str, sizeof(err_str), "No results for query: %s\n", sqlquery_str);
    cgi_error( err_str);
  }

  mysql_num_fields(result);

  if((values = mysql_fetch_row(result)) == NULL) {
    snprintf(err_str, sizeof(err_str), "Cannot access results for query: %s\n", sqlquery_str);
    cgi_error( err_str);
  }
#endif

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

  pagehead(title);
  fprintf(cgiOut, "<div id=\"content\">\n");

  fprintf(cgiOut, "<table class=\"inner\" width=\"100%%\">");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" width=200>");
  fprintf(cgiOut, "Routers");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "<th class=\"inner\" width=200>");
  fprintf(cgiOut, "Users");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "<th class=\"inner\" width=200>");
  fprintf(cgiOut, "Service and IP Address");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>");
   /* 1st row, 1st column */
   fprintf(cgiOut, "<form action=\"router-list.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"List of Routers\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 1st row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-list.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"List of Users\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 1st row, 3rd column */
   fprintf(cgiOut, "<form action=\"service-list.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"List of Services\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "<tr>");
   /* 2nd row, 1st column */
   fprintf(cgiOut, "<form action=\"router-acttime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Activity by Time\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 2nd row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-acttime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Activity by User\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 2nd row, 3rd column */
   fprintf(cgiOut, "<form action=\"service-acttime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Activity by Time\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "<tr>");
   /* 3rd row, 1st column */
   fprintf(cgiOut, "<form action=\"router-actlast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last Activities\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 3rd row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-actlast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last Activities\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 3rd row, 2nd column */
   fprintf(cgiOut, "<form action=\"service-actlast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last Activities\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "<tr>");
   /* 4th row, 1st column */
   fprintf(cgiOut, "<form action=\"ip-toptime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Top IP by Time\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 4th row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-toptime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Top Users by Time\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 4th row, 3rd column */
   fprintf(cgiOut, "<form action=\"ip-acttime.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"IP->User by Time\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "<tr>");
   /* 5th row, 1st colum */
   fprintf(cgiOut, "<form action=\"ip-toplast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last Top IP\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 5th row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-toplast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last Top Users\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 5th row, 3rd column */
   fprintf(cgiOut, "<form action=\"ip-actlast.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Last IP->User\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
 fprintf(cgiOut, "</tr>");
 fprintf(cgiOut, "<tr>");
   /* 6th row, 1st column */
   fprintf(cgiOut, "<form action=\"router-stats.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Router Statistics\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 6th row, 2nd column */
   fprintf(cgiOut, "<form action=\"user-stats.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"User Statistics\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
   /* 5th row of buttons */
   fprintf(cgiOut, "<form action=\"edacs-help.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td class=\"inner-ctr\">");
   fprintf(cgiOut, "<input type=\"submit\" style=\"width:180px\" value=\"Documentation\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" colspan=3>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</table>\n");

  fprintf(cgiOut, "<p>");
  fprintf(cgiOut, "<table class=\"inner\" width=\"100%%\">");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" colspan=2>");
  fprintf(cgiOut, "%s System Information", EDACSWEB_VERSION);
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\" width=230>");
  fprintf(cgiOut, "Database Version:");
  fprintf(cgiOut, "</td>");
#ifdef MYSQL_DB
  fprintf(cgiOut, "<td>MySQL version %d</td>", server_version);      
#endif
#ifdef ORACLE_DB
  fprintf(cgiOut, "<td>%s</td>", server_version);      
#endif
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\">");
  fprintf(cgiOut, "Schema Version:");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "<td>%s</td>", values[0]);      
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\">");
  fprintf(cgiOut, "Schema Created:");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "<td>%s</td>", values[1]);      
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\">");
  fprintf(cgiOut, "Number of Users:");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "<td>%s</td>", values[2]);      
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\">");
  fprintf(cgiOut, "Reporting Routers:");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "<td>%s</td>", values[3]);      
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<td class=\"inner\">");
  fprintf(cgiOut, "Sessions Recorded:");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "<td>%s</td>", values[4]);      
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" colspan=2>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</table>\n");

  pageside();
  pagefoot();
  return(0);
}
