/* -------------------------------------------------------------------------- *
 * file:         router-list.c                                                *
 * purpose:      display the content of the edacs_router table                *
 *               select router, country,                                      *
 *               to_char(first_connect, 'dd-mm-yyyy hh24:mi:ss'),             *
 *               to_char(last_connect, 'dd-mm-yyyy hh24:mi:ss')               *
 *               from edacs_router                                            *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {
#ifdef MYSQL_DB
  static MYSQL *dbh;              /* database connect handle */
  static MYSQL_RES *result;       /* database query results  */
  static MYSQL_ROW values;        /* query data returned     */
  unsigned int colcount    =0;    /* number of returned columns */
  int server_version;             /* returned server version */
#endif
#ifdef ORACLE_DB
  sqlo_db_handle_t dbh;           /* database handle */
  sqlo_stmt_handle_t sth1;        /* statement handle 1 */
  char server_version[1024]="";   /* string for returned server version */
  int stat                 =0;    /* status of sqlo calls */
  int handle               =0;    /* handle of the interrupt handler */
  const char ** values;           /* values */
#endif
  char sqlquery_str[1024]  ="";   /* SQL query string */
  char first_start_date[11]="";   /* first connect start date */
  char first_start_time[6] ="";   /* first connect start time */
  char first_end_date[11]  ="";   /* first connect end date */
  char first_end_time[6]   ="";   /* first connect end time */
  char last_start_date[11] ="";   /* last connect start date */
  char last_start_time[6]  ="";   /* last connect start time */
  char last_end_date[11]   ="";   /* last connect end date */
  char last_end_time[6]    ="";   /* last connect end time */
  int allrows              =0;    /* number of returned rows */
  int rowcount             =0;    /* row iteration counter */
  div_t oddline_calc;             /* calculates even/odd row color */
  char err_str[2048]       ="";   /* use for combined error string */
  static char title[]      = "List of Routers";
  _abort_flag = 0;

#ifdef ORACLE_DB
  /* ------------------------------------------------------------------- * 
   * ORACLE_HOME is needed for OCI8 to find tnsnames.ora                 *
   * ------------------------------------------------------------------- */
  putenv(WEB_ORACLE_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100))
    cgi_error("Error: Failed to init libsqlora8.");

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, WEB_TNS_STRING))
    cgi_error("Error: Cannot connect to database.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  if (SQLO_SUCCESS != sqlo_server_version(dbh, server_version,
                                        sizeof(server_version)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* enable autocommit, each statement is commited as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  /* define the SQL query */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT ROUTER, COUNTRY, %s, %s, %s, %s, %s, %s, %s, %s FROM %s.EDACS_ROUTER ORDER BY ROUTER",
           "TO_CHAR(FIRST_CONNECT, 'dd-mm-yyyy')",   /* select_by=start_date, first_start_date */
           "TO_CHAR(FIRST_CONNECT, 'hh24:mi:ss')",   /* select_by=start_date, first_start_time */
           "TO_CHAR(FIRST_CONNECT+INTERVAL '1' MINUTE, 'dd-mm-yyyy')",     /* first_end_date */
           "TO_CHAR(FIRST_CONNECT+INTERVAL '1' MINUTE, 'hh24:mi')",        /* first_end_time */
           "TO_CHAR(LAST_CONNECT, 'dd-mm-yyyy')",    /* select_by=stop_date, last_start_date */
           "TO_CHAR(LAST_CONNECT, 'hh24:mi:ss')",    /* select_by=stop_date, last_start_time */
           "TO_CHAR(LAST_CONNECT+INTERVAL '1' MINUTE, 'dd-mm-yyyy')",       /* last_end_date */
           "TO_CHAR(LAST_CONNECT+INTERVAL '1' MINUTE, 'hh24:mi')",          /* last_end_time */
           EDACSADMIN);

  /* initialize the statement handle */
  sth1 = SQLO_STH_INIT;

  /* opens a cursor for the query statement */
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
  /* initialize the connection */
  dbh = mysql_init(NULL);
  if(dbh == NULL) cgi_error("Error:  Failed to init MySQL DB.");

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0)
    cgi_error("Error: Cannot connect to database.");

  /* Get the database version */
  server_version = mysql_get_server_version(dbh);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT router, country, %s, %s, %s, %s, %s, %s, %s, %s FROM edacs_router ORDER BY router",
           "DATE_FORMAT(edacs_router.first_connect, '%d-%m-%Y')",                     /* 03 */
           "DATE_FORMAT(edacs_router.first_connect, '%H:%i:%s')",                     /* 04 */
           "DATE_FORMAT(DATE_ADD(edacs_router.first_connect, INTERVAL 1 MINUTE), '%d-%m-%Y')", /* 05 */
           "DATE_FORMAT(DATE_ADD(edacs_router.first_connect, INTERVAL 1 MINUTE), '%H:%i')",    /* 06 */
           "DATE_FORMAT(edacs_router.last_connect, '%d-%m-%Y')",                      /* 07 */
           "DATE_FORMAT(edacs_router.last_connect, '%H:%i:%s')",                      /* 08 */
           "DATE_FORMAT(DATE_ADD(edacs_router.last_connect, INTERVAL 1 MINUTE), '%d-%m-%Y')",  /* 09 */
           "DATE_FORMAT(DATE_ADD(edacs_router.last_connect, INTERVAL 1 MINUTE), '%H:%i')" );   /* 10 */


  /* Prepare and execute the SQL statement */
  if(mysql_query(dbh, sqlquery_str) != 0) {
    if(DEBUG == 0) cgi_error(mysql_error(dbh));
    else snprintf(err_str, sizeof(err_str), "DB error %s\n\nQuery string %s",
             mysql_error(dbh), sqlquery_str);
    cgi_error(err_str);
  }
 /* get query results set */
  result = mysql_store_result(dbh);
  if (result == NULL) {
    snprintf(err_str, sizeof(err_str), "No results for query: %s\n", sqlquery_str);
    cgi_error( err_str);
  }

  allrows = mysql_num_rows(result);
  colcount = mysql_num_fields(result);
#endif

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title);
   fprintf(cgiOut, "<div id=\"content\">\n");

   fprintf(cgiOut, "<table class=\"inner\" width=\"100%%\">\n");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\" width=40>#</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Router IP</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Router Name</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Country</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Active Since</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Last Update</th>\n");
   fprintf(cgiOut, "</tr>\n");

   /* fetch the data */
#ifdef ORACLE_DB
   while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
     /* get one record */
     values = sqlo_values(sth1, NULL, 1);
#endif
#ifdef MYSQL_DB
   while((values = mysql_fetch_row(result)) != NULL) {
#endif
     rowcount++;

     /* convert IP of values[0] to DNS name, if possible */
     get_dns_name((char *) values[0]);

     /* calculate start and end times for link to session query */
     strncpy(first_start_date, values[2], sizeof(first_start_date)-1);
     first_start_date[2] = '.';
     first_start_date[5] = '.';
     first_start_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_start_time, values[3], sizeof(first_start_time)-1);
     first_start_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_end_date, values[4], sizeof(first_end_date)-1);
     first_end_date[2] = '.';
     first_end_date[5] = '.';
     first_end_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_end_time, values[5], sizeof(first_end_time)-1);
     first_end_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     if(values[6] != NULL) {
       strncpy(last_start_date, values[6], sizeof(last_start_date)-1);
       last_start_date[2] = '.';
       last_start_date[5] = '.';
       last_start_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[7] != NULL) {
       strncpy(last_start_time, values[7], sizeof(last_start_time)-1);
       last_start_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[8] != NULL) {
       strncpy(last_end_date, values[8], sizeof(last_end_date)-1);
       last_end_date[2] = '.';
       last_end_date[5] = '.';
       last_end_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[9] != NULL) {
       strncpy(last_end_time, values[9], sizeof(last_end_time)-1);
       last_end_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     /* check for even/odd rows */
     oddline_calc = div(rowcount, 2);
     if(oddline_calc.rem) fprintf(cgiOut, "<tr class=\"odd\">\n");
     else fprintf(cgiOut, "<tr class=\"even\">\n");

     /* start the html output  column 0 = row counter */
     fprintf(cgiOut, "<td>%d</td>\n", rowcount);

     /* column 1 = router IP */
     fprintf(cgiOut, "<td>%s</td>", values[0]);
     /* column 2 = routername */
     fprintf(cgiOut, "<td>%s</td>", dns_name);

     /* column 3 if we got a value for country, we try to match it for a jpeg file */
     fprintf(cgiOut, "<td class=\"country\">");
     if(values[1] != NULL) {
       if(strlen(values[1]) != 0)
         fprintf(cgiOut, "<img src=\"../images/%s.jpg\" border=0></td>", values[1]);
       }
       else fprintf(cgiOut, "n/a</td>");

     /* column 4 = router start record */
     fprintf(cgiOut, "<td>");
     fprintf(cgiOut, "<a href=router-acttime.cgi?start_date=%s&start_time=%s&end_date=%s&end_time=%s&sort_order=asc&router=%s&order_by=start_date>", first_start_date, first_start_time, first_end_date, first_end_time, values[0]);
     fprintf(cgiOut, "%s %s</a></td>", values[2], values[3]);

     /* column 5 = router end record */
     fprintf(cgiOut, "<td>");
     if(values[6] != NULL && values[7] != NULL) {
       fprintf(cgiOut, "<a href=router-acttime.cgi?start_date=%s&start_time=%s&end_date=%s&end_time=%s&sort_order=asc&router=%s&order_by=start_date&select_by=stop_date>", last_start_date, last_start_time, last_end_date, last_end_time, values[0]);
       fprintf(cgiOut, "%s %s</a></td>\n", values[6], values[7]);
     }
     fprintf(cgiOut, "</tr>\n");
   }
#ifdef ORACLE_DB
  if (SQLO_SUCCESS != sqlo_close(sth1))
    cgi_error("Error Closing the SQL statment handle.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
   mysql_close(dbh);
#endif
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\" colspan=6>");
   fprintf(cgiOut, "&nbsp;");
   fprintf(cgiOut, "</th>\n");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "</table>\n");

   fprintf(cgiOut, "<h3>Additional Information</h3>\n");
   fprintf(cgiOut, "<hr>");
   fprintf(cgiOut, "<p>");
   fprintf(cgiOut, "This list represents all routers that sent accounting data to EDAcS.");
   fprintf(cgiOut, "</p>");
   fprintf(cgiOut, "<ul>");
   fprintf(cgiOut, "<li>The router is marked inactive if there is no data received in more then a week.");
   fprintf(cgiOut, "<li>If DNS resolution is unavailable for routers, add the name to edacsweb server's hosts file.");
   fprintf(cgiOut, "<li>The routers country flag can be set by adding the 2-char country code to the DB:");
   fprintf(cgiOut, "<br>i.e. <i>update edacs_router set country='uk' where router='192.168.249.253';</i>");
   fprintf(cgiOut, "<li>The \"Active Since\" is the first time session information was received. The time links to the first recorded session for this router.");
   fprintf(cgiOut, "<li>The \"Last Update\" time shows when the latest session record was received. The link tries to find the latest session, however often the session just started or is still in progress, therefore incomplete or no data is returned.");
   fprintf(cgiOut, "</ul>");

   pageside();
   pagefoot();
   return(0);
}
