/* -------------------------------------------------------------------------- *
 * file:         router-stats-day.c                                           *
 * purpose:      display the stats from the edacs_daystats table              *
 *               the images should be pre-generated with edacs-statsgen       *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {

  char router[16]          ="";   /* selected router IP address */
  char title[256]          ="";   /* cgi title string */
  char type[4]             ="";   /* selected image type [cpd|mpd] */
  int i;                          /* loop counter */

  if ( cgiFormString("router", router, sizeof(router)) != cgiFormSuccess )
      cgi_error("Error retrieving the IP address of the router.");
  if ( cgiFormString("type", type, sizeof(type)) != cgiFormSuccess )
      cgi_error("Error retrieving the image type information.");

  if(strcmp(type,"cpd") == 0)
     snprintf(title, sizeof(title), "Daily Connection Statistics for Router %s", router);
  else if(strcmp(type,"mpd") == 0)
     snprintf(title, sizeof(title), "Daily Data Statistics for Router %s", router);
  else cgi_error("Error image type is invalid.");

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

  pagehead(title);
  fprintf(cgiOut, "<div id=\"content\">\n");

  fprintf(cgiOut, "<table class=\"inner\" width=100%%>\n");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\"width=20>#</th>\n");
  if(strcmp(type,"cpd") == 0)
    fprintf(cgiOut, "<th class=\"inner\">Connections/Day</th>\n");
  else if(strcmp(type,"mpd") == 0)
    fprintf(cgiOut, "<th class=\"inner\">Data/Day</th>\n");
  fprintf(cgiOut, "</tr>\n");

  for(i=0;i<DAYSTATS_HIST; i++) {
    fprintf(cgiOut, "<tr>\n");
    fprintf(cgiOut, "<th class=\"inner\">%d</th>\n", i+1);
    fprintf(cgiOut, "<td class=\"inner\">");
    if(strcmp(type,"cpd") == 0)
      fprintf(cgiOut, "<img src=\"../images/constats-mon%d-%s.png\" usemap=\"daystats\">", i, router);
    else if(strcmp(type,"mpd") == 0)
      fprintf(cgiOut, "<img src=\"../images/bytestats-mon%d-%s.png\" usemap=\"daystats\">", i, router);
    fprintf(cgiOut, "</td>");
    fprintf(cgiOut, "</tr>\n");
  }

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" colspan=3>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</table>\n");

  fprintf(cgiOut, "<h3>Additional Information</h3>\n");
  fprintf(cgiOut, "<hr>\n");
  fprintf(cgiOut, "<p>");
  if(strcmp(type,"cpd") == 0) {
    fprintf(cgiOut, "This page shows the daily summary for router connections.");
    fprintf(cgiOut, "<ul>");
    fprintf(cgiOut, "<li>The number of connections is a summary of all connections on this router, per day.");
  }
  else if(strcmp(type,"mpd") == 0) {
    fprintf(cgiOut, "This page shows the daily summary for transfered data.");
    fprintf(cgiOut, "<ul>");
    fprintf(cgiOut, "<li>The data transfer is the summary of all data in, data out and total during the day.");
  }
  fprintf(cgiOut, "<li>The graphs are pre-generated on the web server by edacs-chartsgen, which runs through cron.");
  fprintf(cgiOut, "</ul>");
  fprintf(cgiOut, "</p>\n");

  fprintf(cgiOut, "<map name=\"daystats\">\n");
  fprintf(cgiOut, "<area shape=\"rect\" coords=\"0,0,44,44\" href=\"sun.htm\" alt=\"1\" />\n");
  fprintf(cgiOut, "</map>\n");

  pageside();
  pagefoot();
  return(0);
}
