/* -------------------------------------------------------------------------- *
 * file:         router-stats.c                                               *
 * purpose:      display the stats from the edacs_monstats table              *
 *               the images should be pre-generated with edacs-statsgen       *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {
#ifdef MYSQL_DB
  static MYSQL *dbh;              /* database connect handle */
  static MYSQL_RES *result;       /* database query results  */
  static MYSQL_ROW row;           /* query data returned     */
  unsigned int colcount    =0;    /* number of returned columns */
  int server_version;             /* returned server version */
  int i = 0;                      /* loop counter i */
  char *values[10];               /* values */
#endif
#ifdef ORACLE_DB
  sqlo_db_handle_t dbh;           /* database handle */
  sqlo_stmt_handle_t sth1;        /* statement handle 1 */
  char server_version[1024]="";   /* string for returned server version */
  int stat                 =0;    /* status of sqlo calls */
  int handle               =0;    /* handle of the interrupt handler */
  //const char ** colnames;         /* column names */
  const char ** values;           /* values */
#endif
  char sqlquery_str[1024]  ="";   /* SQL query string */
  int rowcount             =0;    /* number of returned rows */
  char err_str[2048]       ="";   /* use for combined error string */
  static char title[] = "Router Statistics";
  _abort_flag     = 0;
#ifdef ORACLE_DB
  /* ------------------------------------------------------------------- * 
   * ORACLE_HOME is needed for OCI8 to find tnsnames.ora                 *
   * ------------------------------------------------------------------- */
  putenv(WEB_ORACLE_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100))
    cgi_error("Error: Failed to init libsqlora8.");

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, WEB_TNS_STRING))
    cgi_error("Error: Cannot connect to database.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  if (SQLO_SUCCESS != sqlo_server_version(dbh, server_version,
                                        sizeof(server_version)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* enable autocommit, each statement is commited as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  /* define the SQL query */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT ROUTER FROM %s.EDACS_ROUTER ORDER BY ROUTER",
           EDACSADMIN);

  /* initialize the statement handle */
  sth1 = SQLO_STH_INIT;

  /* opens a cursor for the query statement */
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
  /* initialize the connection */
  dbh = mysql_init(NULL);
  if(dbh == NULL) cgi_error("Error:  Failed to init MySQL DB.");

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0)
    cgi_error("Error: Cannot connect to database.");

  /* Get the database version */
  server_version = mysql_get_server_version(dbh);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT router FROM edacs_router ORDER BY router");

  /* Prepare and execute the SQL statement */
  if(mysql_query(dbh, sqlquery_str) != 0) {
    if(DEBUG == 0) cgi_error(mysql_error(dbh));
    else snprintf(err_str, sizeof(err_str), "DB error %s\n\nQuery string %s",
           mysql_error(dbh), sqlquery_str);
    cgi_error(err_str);
  }
  /* get query results set */
  result = mysql_store_result(dbh);
  if (result == NULL) {
    snprintf(err_str, sizeof(err_str), "No results for query: %s\n", sqlquery_str);
    cgi_error( err_str);
  }

  colcount = mysql_num_fields(result);
#endif
/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title);
   fprintf(cgiOut, "<div id=\"content-wide\">\n");

   fprintf(cgiOut, "<table class=\"inner\" widthg=100%%>\n");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\">#</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Connections/Month</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Data/Month</th>\n");
   fprintf(cgiOut, "</tr>\n");

   /* fetch the data */
#ifdef ORACLE_DB
   while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
     /* get one record */
     values = sqlo_values(sth1, NULL, 1);
#endif
#ifdef MYSQL_DB
    while((row = mysql_fetch_row(result)) != NULL) {
      for (i = 0; i < colcount; ++i) {
        values[i] = row[i];
      }
#endif
     rowcount++;

     fprintf(cgiOut, "<tr>\n");
     fprintf(cgiOut, "<th class=\"inner\" width=20>%d</th>\n", rowcount);
     fprintf(cgiOut, "<td class=\"inner-ctr\">");
     fprintf(cgiOut, "<a href=router-stats-day.cgi?router=%s&type=cpd>", values[0]);
     fprintf(cgiOut, "<img src=\"../images/constats-%s.png\" border=0>", values[0]);
     fprintf(cgiOut, "</a></td>");
     fprintf(cgiOut, "<td class=\"inner-ctr\">");
     fprintf(cgiOut, "<a href=router-stats-day.cgi?router=%s&type=mpd>", values[0]);
     fprintf(cgiOut, "<img src=\"../images/bytestats-%s.png\" border=0>", values[0]);
     fprintf(cgiOut, "</a></td>");
     fprintf(cgiOut, "</tr>\n");
   }
#ifdef ORACLE_DB
  if (SQLO_SUCCESS != sqlo_close(sth1))
    cgi_error("Error Closing the SQL statment handle.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
   mysql_close(dbh);
#endif
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\" colspan=3>");
   fprintf(cgiOut, "&nbsp;</th>\n");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "</table>\n");

   fprintf(cgiOut, "<h3>Additional Information</h3>\n");
   fprintf(cgiOut, "<hr>\n");
   fprintf(cgiOut, "<p>");
   fprintf(cgiOut, "This page shows the monthly summary for router connections and transfered data.");
   fprintf(cgiOut, "<ul>");
   fprintf(cgiOut, "<li>Click on a graph image to display the daily graph data, with a %d months history if available.", DAYSTATS_HIST);
   fprintf(cgiOut, "<li>The number of connections is a summary of all connections per month on this router, for the last 18 months.");
   fprintf(cgiOut, "<li>The data transfer is the summary of all transfered data in, data out and total during the last 18 months.");
   fprintf(cgiOut, "<li>The graphs are pre-generated on the web server by edacs-chartsgen, which runs through cron.");
   fprintf(cgiOut, "</ul>");
   fprintf(cgiOut, "</p>\n");

   pagefoot();
   return(0);
}
