/* -------------------------------------------------------------------------- *
 * file:         user-list.c                                                  *
 * purpose:      display the content of the edacs_user table                  *
 *                                                                            *
 *               select edacs_users.username, count(edacs_remote.username)    *
 *               from edacsadmin.edacs_users, edacsadmin.edacs_remote where   *
 *               edacsadmin.edacs_users.username =                            *
 *               edacsadmin.edacs_remote.username                             *
 *               group by edacsadmin.edacs_users.username;                    *
 *                                                                            *
 *               This statement is a nice example of a equality joint select  *
 *                                                                            *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {
#ifdef MYSQL_DB
  static MYSQL *dbh;              /* database connect handle */
  static MYSQL_RES *result;       /* database query results  */
  static MYSQL_ROW values;        /* query data returned     */
  unsigned int colcount    =0;    /* number of returned columns */
  int server_version;             /* returned server version */
#endif
#ifdef ORACLE_DB
  sqlo_db_handle_t dbh;           /* database handle */
  sqlo_stmt_handle_t sth1;        /* statement handle 1 */
  char server_version[1024]="";   /* string for returned server version */
  int stat                 =0;    /* status of sqlo calls */
  int handle               =0;    /* handle of the interrupt handler */
  const char ** values;           /* values */
#endif

  char sqlquery_str[1024]  ="";   /* SQL query string */
  char first_start_date[11]="";   /* first connect start date */
  char first_start_time[6] ="";   /* first connect start time */
  char first_end_date[11]  ="";   /* first connect end date */
  char first_end_time[6]   ="";   /* first connect end time */
  char last_start_date[11] ="";   /* last connect start date */
  char last_start_time[6]  ="";   /* last connect start time */
  char last_end_date[11]   ="";   /* last connect end date */
  char last_end_time[6]    ="";   /* last connect end time */
  int rowcount             =0;    /* number of returned rows */
  div_t oddline_calc;             /* calculates even/odd row color */
  char err_str[2048]       ="";   /* use for combined error string */
  static char title[] = "List of all Users";
  _abort_flag     = 0;

#ifdef ORACLE_DB
  /* ------------------------------------------------------------------- * 
   * ORACLE_HOME is needed for OCI8 to find tnsnames.ora                 *
   * ------------------------------------------------------------------- */
  putenv(WEB_ORACLE_ENV);

  /* initialize the connection */
  if (SQLO_SUCCESS != sqlo_init(SQLO_OFF, 1, 100))
    cgi_error("Error: Failed to init libsqlora8.");

  /* register the interrupt handler */
  sqlo_register_int_handler(&handle, sigint_handler);

  /* login to the database */
  if (SQLO_SUCCESS != sqlo_connect(&dbh, WEB_TNS_STRING))
    cgi_error("Error: Cannot connect to database.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  if (SQLO_SUCCESS != sqlo_server_version(dbh, server_version,
                                        sizeof(server_version)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */

  /* enable autocommit, each statement is commited as a single transaction */
  stat = sqlo_set_autocommit(dbh, 1);

  /* define the SQL query */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s, %s FROM %s.EDACS_USERS, %s.EDACS_REMOTE WHERE %s.EDACS_USERS.USERNAME=%s.EDACS_REMOTE.USERNAME GROUP BY %s.EDACS_USERS.USERNAME, %s.EDACS_USERS.FIRST_CONNECT,%s.EDACS_USERS.LAST_CONNECT ORDER BY %s.EDACS_USERS.USERNAME",
           "EDACS_USERS.USERNAME",                                                 /* 01 */
           "TO_CHAR(EDACS_USERS.FIRST_CONNECT, 'dd-mm-yyyy')",                     /* 02 */
           "TO_CHAR(EDACS_USERS.FIRST_CONNECT, 'hh24:mi:ss')",                     /* 03 */
           "TO_CHAR(EDACS_USERS.FIRST_CONNECT+INTERVAL '1' MINUTE, 'dd-mm-yyyy')", /* 04 */
           "TO_CHAR(EDACS_USERS.FIRST_CONNECT+INTERVAL '1' MINUTE, 'hh24:mi')",    /* 05 */
           "TO_CHAR(EDACS_USERS.LAST_CONNECT, 'dd-mm-yyyy')",                      /* 06 */
           "TO_CHAR(EDACS_USERS.LAST_CONNECT, 'hh24:mi:ss')",                      /* 07 */
           "TO_CHAR(EDACS_USERS.LAST_CONNECT+INTERVAL '1' MINUTE, 'dd-mm-yyyy')",  /* 08 */
           "TO_CHAR(EDACS_USERS.LAST_CONNECT+INTERVAL '1' MINUTE, 'hh24:mi')",     /* 09 */
           "COUNT (EDACS_REMOTE.USERNAME)",                                        /* 10 */
           EDACSADMIN, EDACSADMIN, EDACSADMIN, EDACSADMIN,
	   EDACSADMIN, EDACSADMIN, EDACSADMIN, EDACSADMIN);
	   //cgi_error(sqlquery_str);

  /* initialize the statement handle */
  sth1 = SQLO_STH_INIT;

  /* opens a cursor for the query statement */
  if ( 0 > (sqlo_open2(&sth1, dbh, sqlquery_str, 0, NULL)))
    cgi_error(sqlo_geterror(dbh));
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
  /* initialize the connection */
  dbh = mysql_init(NULL);
  if(dbh == NULL) cgi_error("Error:  Failed to init MySQL DB.");

  /* login to the database */
  if (mysql_real_connect(dbh, MYSQLIP, EDACSADMIN, ADMIN_PASS, DB_NAME, DB_PORT, NULL, 0) == 0)
    cgi_error("Error: Cannot connect to database.");

  /* Get the database version */
  server_version = mysql_get_server_version(dbh);

  /* create the SQL query string */
  snprintf(sqlquery_str, sizeof(sqlquery_str), "SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s, %s FROM edacs_users, edacs_remote WHERE edacs_users.username=edacs_remote.username GROUP BY edacs_users.username, edacs_users.first_connect, edacs_users.last_connect ORDER BY edacs_users.username",
           "edacs_users.username",                                                   /* 01 */
           "DATE_FORMAT(edacs_users.first_connect, '%d-%m-%Y')",                     /* 02 */
           "DATE_FORMAT(edacs_users.first_connect, '%H:%i:%s')",                     /* 03 */
           "DATE_FORMAT(DATE_ADD(edacs_users.first_connect, INTERVAL 1 MINUTE), '%d-%m-%Y')", /* 04 */
           "DATE_FORMAT(DATE_ADD(edacs_users.first_connect, INTERVAL 1 MINUTE), '%H:%i')",    /* 05 */
           "DATE_FORMAT(edacs_users.last_connect, '%d-%m-%Y')",                      /* 06 */
           "DATE_FORMAT(edacs_users.last_connect, '%H:%i:%s')",                      /* 07 */
           "DATE_FORMAT(DATE_ADD(edacs_users.last_connect, INTERVAL 1 MINUTE), '%d-%m-%Y')",  /* 08 */
           "DATE_FORMAT(DATE_ADD(edacs_users.last_connect, INTERVAL 1 MINUTE), '%H:%i')",     /* 09 */
           "COUNT(edacs_remote.username)");                                          /* 10 */
           //cgi_error(sqlquery_str);

  /* Prepare and execute the SQL statement */
  if(mysql_query(dbh, sqlquery_str) != 0) {
    if(DEBUG == 0) cgi_error(mysql_error(dbh));
    else snprintf(err_str, sizeof(err_str), "DB error %s\n\nQuery string %s",
             mysql_error(dbh), sqlquery_str);
    cgi_error(err_str);
  }
 /* get query results set */
  result = mysql_store_result(dbh);
  if (result == NULL) {
    snprintf(err_str, sizeof(err_str), "No results for query: %s\n", sqlquery_str);
    cgi_error( err_str);
  }

  colcount = mysql_num_fields(result);
#endif
/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title);
   fprintf(cgiOut, "<div id=\"content\">\n");

   fprintf(cgiOut, "<table class=\"inner\" width=100%%>\n");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\" width=40>#</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">User Name</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Active Since</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Last Update</th>\n");
   fprintf(cgiOut, "<th class=\"inner\">Remotes</th>\n");
   fprintf(cgiOut, "</tr>\n");

   /* fetch the data */
#ifdef ORACLE_DB
   while ( SQLO_SUCCESS == (stat = (sqlo_fetch(sth1, 1)))) {
     /* get one record */
     values = sqlo_values(sth1, NULL, 1);
#endif
#ifdef MYSQL_DB
   while((values = mysql_fetch_row(result)) != NULL) {
#endif
     rowcount++;

     /* calculate start and end times for link to session query */
     strncpy(first_start_date, values[1], sizeof(first_start_date)-1);
     first_start_date[2] = '.';
     first_start_date[5] = '.';
     first_start_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_start_time, values[2], sizeof(first_start_time)-1);
     first_start_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_end_date, values[3], sizeof(first_end_date)-1);
     first_end_date[2] = '.';
     first_end_date[5] = '.';
     first_end_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     strncpy(first_end_time, values[4], sizeof(first_end_time)-1);
     first_end_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */

     if(values[5] != NULL) {
       strncpy(last_start_date, values[5], sizeof(last_start_date)-1);
       last_start_date[2] = '.';
       last_start_date[5] = '.';
       last_start_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[6] != NULL) {
       strncpy(last_start_time, values[6], sizeof(last_start_time)-1);
       last_start_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[7] != NULL) {
       strncpy(last_end_date, values[7], sizeof(last_end_date)-1);
       last_end_date[2] = '.';
       last_end_date[5] = '.';
       last_end_date[10] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     if(values[8] != NULL) {
       strncpy(last_end_time, values[8], sizeof(last_end_time)-1);
       last_end_time[5] = '\0'; /* strncpy does not terminate the string, therefore we have to */
     }

     /* check for even/odd rows */
     oddline_calc = div(rowcount, 2);
     if(oddline_calc.rem) fprintf(cgiOut, "<tr class=\"odd\">\n");
     else fprintf(cgiOut, "<tr class=\"even\">\n");

     /* column 0 = row counter */
     fprintf(cgiOut, "<td  class=\"center\">%d</td>\n", rowcount);

     /* column 1 = username */
     fprintf(cgiOut, "<td><a href=user-info.cgi?username=%s>", values[0]);
     fprintf(cgiOut, "%s</a></td>", values[0]);

     /* column 2 = active since */
     fprintf(cgiOut, "<td><a href=user-acttime.cgi?start_date=%s&start_time=%s&end_date=%s&end_time=%s&sort_order=asc&username=%s&order_by=start_date&select_by=start_date>", first_start_date, first_start_time, first_end_date, first_end_time, values[0]);
     fprintf(cgiOut, "%s %s</a></td>", values[1], values[2]);

     /* column 3 = last update */
     fprintf(cgiOut, "<td>");
     if(values[5] != NULL && values[6] != NULL) {
       fprintf(cgiOut, "<a href=user-acttime.cgi?start_date=%s&start_time=%s&end_date=%s&end_time=%s&sort_order=asc&username=%s&order_by=start_date&select_by=stop_date>", last_start_date, last_start_time, last_end_date, last_end_time, values[0]);
       fprintf(cgiOut, "%s %s</a></td>", values[5], values[6]);
     }
     else fprintf(cgiOut, "no data</td>");

     /* column 4 = remotes */
     fprintf(cgiOut, "<td class=\"center\">");
     if(values[9] == NULL) { fprintf(cgiOut,"&nbsp;</td>"); }
     else { fprintf(cgiOut,"%s</td>", values[9]); }
     fprintf(cgiOut, "</tr>\n");
   }
#ifdef ORACLE_DB
  if (SQLO_SUCCESS != sqlo_close(sth1))
    cgi_error("Error Closing the SQL statment handle.");
  RETURN_ON_ABORT; /* finish if SIGINT was catched */
#endif
#ifdef MYSQL_DB
   mysql_close(dbh);
#endif
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th class=\"inner\" colspan=5>");
   fprintf(cgiOut, "&nbsp;");
   fprintf(cgiOut, "</th>\n");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "</table>\n");

   fprintf(cgiOut, "<h3>Additional Information</h3>\n");
   fprintf(cgiOut, "<hr>\n");
   fprintf(cgiOut, "<p>");
   fprintf(cgiOut, "This list represents all users who connected to routers sending data to EDAcS.");
   fprintf(cgiOut, "<ul>");
      fprintf(cgiOut, "<li>The \"Active Since\" is the first time session information was received. The time links to the first recorded session for this user.");
         fprintf(cgiOut, "<li>The \"Last Update\" time shows when the latest session record was received. The link tries to find the latest session. Sometimes the session is still in progress and incomplete or no data is returned.");
         fprintf(cgiOut, "<li>The \"Remotes\" number is the count of the different remote IP or phone numbers were this users connections originate from.");
   fprintf(cgiOut, "</ul>");
   fprintf(cgiOut, "</p>\n");

   pageside();
   pagefoot();
   return(0);
}
