/* -------------------------------------------------------------------------- *
 * file:         user-stats-day.c                                             *
 * purpose:      display the stats from the edacs_daystats table              *
 *               the images should be pre-generated with edacs-statsgen       *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <cgic.h>
#include "../edacs.h"

int cgiMain() {

  char router[16]          ="";   /* selected router IP address */
  char title[256]          ="";   /* cgi title string */
  char type[4]             ="";   /* selected image type [upd|apd] */
  int i;                          /* loop counter */

  if ( cgiFormString("router", router, sizeof(router)) != cgiFormSuccess )
      cgi_error("Error retrieving the IP address of the router.");
  if ( cgiFormString("type", type, sizeof(type)) != cgiFormSuccess )
      cgi_error("Error retrieving the image type information.");

  if(strcmp(type,"upd") == 0)
     snprintf(title, sizeof(title), "Daily User Statistics for Router %s", router);
  else if(strcmp(type,"apd") == 0)
     snprintf(title, sizeof(title), "Daily Data per User for Router %s", router);
  else cgi_error("Error image type is invalid.");

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

  pagehead(title);
  fprintf(cgiOut, "<div id=\"content\">\n");

  fprintf(cgiOut, "<table class=\"inner\" width=100%%>\n");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\">#</th>\n");
  if(strcmp(type,"upd") == 0)
    fprintf(cgiOut, "<th class=\"inner\">Unique Users/Day</th>\n");
  else if(strcmp(type,"apd") == 0)
    fprintf(cgiOut, "<th class=\"inner\">Average Data per User/Day</th>\n");
  fprintf(cgiOut, "</tr>\n");

  for(i=0;i<DAYSTATS_HIST; i++) {
    fprintf(cgiOut, "<tr>\n");
    fprintf(cgiOut, "<th class=\"inner\" width=20>%d</th>\n", i+1);
    fprintf(cgiOut, "<td class=\"inner\">");
    if(strcmp(type,"upd") == 0)
      fprintf(cgiOut, "<img src=\"../images/usercount-mon%d-%s.png\" border=0>", i, router);
    else if(strcmp(type,"apd") == 0)
      fprintf(cgiOut, "<img src=\"../images/avgbytes-mon%d-%s.png\" border=0>", i, router);
    fprintf(cgiOut, "</td>");
    fprintf(cgiOut, "</tr>\n");
  }

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th class=\"inner\" colspan=3>&nbsp;</th>");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</table>\n");

  fprintf(cgiOut, "<h3>Additional information</h3>\n");
  fprintf(cgiOut, "<hr>\n");
  fprintf(cgiOut, "<p>");
  if(strcmp(type,"upd") == 0) {
    fprintf(cgiOut, "This page shows the daily summary for unique users per router");
    fprintf(cgiOut, "<ul>");
    fprintf(cgiOut, "<li>The number of connections is a summary of all unique users on this router, per day.");
  }
  else if(strcmp(type,"apd") == 0) {
    fprintf(cgiOut, "This page shows the daily summary of the average transfered data per user.");
    fprintf(cgiOut, "<ul>");
    fprintf(cgiOut, "<li>The data transfer is the summary of all data in and data out, calculated as the average per user.");
  }
  fprintf(cgiOut, "<li>The graphs are pre-generated on the web server by edacs-chartsgen, which runs through cron.");
  fprintf(cgiOut, "</ul>");
  fprintf(cgiOut, "</p>\n");

  pageside();
  pagefoot();
  return(0);
}
