## EDAcS 

### Data Mining System for Remote Connection Accounting Data

EDAcS is a TACACS/RADIUS reporting and accounting system.  It has been
developed to collect, archive and report on remote user connections
for security monitoring and network investigations. 
 
Most network gateways for remote connectivity support session accounting using
Radius protocol (RFC 2866). While Radius is often already used for user
authentication, it is also a very good choice for user and session tracking.
Such information provides essential log trails for remote user login across
different devices and regions in large organisations.
 
Initially designed for dial-in users, it expanded for also tracking VPN user 
connections, both hardware VPN and software VPN solutions. A database backend
provides the necessary views to sort and view the connection data under
different aspects and to cross-reference IP addresses or user names to other
reporting systems, i.e. firewall, IDS, system or access control logs.
 
The name "EDAcS" stands for its initial purpose:
[E]nterprise [D]ial-in [Ac]counting [S]ystem.

Before we had network jacks in our homes, we used modems to connect to a ISP
using the telephone system, a process called 'dial-in". Modems are long gone,
but authentication and accounting protocols are still the same for todays VPN.

### Design:

EDAcS consists of two parts, the "edacsdb" database loader and the "edacsweb"
reporting CGI's. The two parts can be installed independend from each other,
even reside on different systems. Both need connectivity to a backend database.
Originally developed for Oracle using Oracle OCI with libsqlora, it now supports
also MySQL in addition.

The "edacsdb" loader is the data collector for remote user login and connection
information in EDAcS. The loader is a cron-scheduled batch uploader for Radius
plaintext logfiles. It extracts Radius network session accounting records
by splitting the log files of a Lucent Radius daemon, parses them and loads
the data into the database. Database triggers distribute the data into their
respective tables.

The separate reporting system "edacsweb" connects to this database and displays
information such as session length, transmitted bytes or calculates top-ten
lists (like users per time, traffic, ...).

But most importantly:
EDAcS links a remote user account name to the IP address that has been assigned
by DHCP. Any network missbehaviour can now be almost instantly tracked down to
the user who received the IP in that given timeframe, allowing to cross-
reference data from diverse sources such as firewall and network device logs
and OS messages.

### Dependencies:

*	Thomas Boutell's CGIC library, see http://www.boutell.com/cgic/
*	The EDAcS statistics graphs depend on the GD library,
        see http://www.libgd.org/
*	Oracle Database and a Oracle Client or Oracle Instantclient
*	Kai Poitschke's libsqlora, see http://www.poitschke.de/libsqlora8/
or
*	MySQL Database and mysqlclient libs and headers (mysqlclientr-devel)

### Configuration:

Apart from the Makefiles in the root and src/ dirextories, check the file
"edacs.h" in the edacs root directory. the upper section can be configured to
set the URL location and the default parameters for both parts of edacs,
edacsdb and edacsweb.

Making and installing edacs:

*	vi the Makefiles "edacsdb-src/Makefile" and "edacsweb-src/Makefile" to
	adjust various path's for cgi and html destinations, database include and
	library directories of the required libraries.

*	vi "edacs.h" to adjust the path's for your webserver and program
	locations

*	make && make install
	"make install" expects the directory structure to exist. Please read
        the INSTALL instructions in the source subdirectories. For "edacsweb"
        the directories should be somewhere below your webserver document root
        i.e. apache/htdocs/edacsweb containing the following subdirectories:
        images cgi-bin style. The application is expected
	to be accessed via URL http://<www.yourdomain.com>/edacsweb.

*	don't forget to enable the cgi directory in your webserver, i.e.
	in apache's httpd.conf add the line:
 	ScriptAlias /edacs/cgi-bin/ "/var/apache/htdocs/edacs/cgi-bin/"

A more complete installation procedure is provided in INSTALL.

### Security:

It is highly adviseable to provide access control and SSL encryption to the
"edacsweb" interface for use other then just experimental.

### Copyrights and License, Legal Disclaimer:

EDAcS was written by Frank4DD. It is distributed under the GPL.
Anybody may reproduce it, use it, send it, print it, transfer on a T-shirt,
etc. without modifying its content or removing the copyright note.

Of course this software comes WITHOUT ANY WARRANTY.

Oracle Database is a product of Oracle Corporation, protected under their
trademark and licensing policies.

CGIC, copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002 by Thomas Boutell
and Boutell.Com, Inc.. Permission is granted to use CGIC in any
application, commercial or noncommercial, at no cost. HOWEVER,
this copyright paragraph must appear on a "credits" page accessible
in the public online and offline documentation of the program.
Modified versions of the CGIC library should not be distributed without
the attachment of a clear statement regarding the author of the
modifications, and this notice may in no case be removed.
Modifications may also be submitted to the author for inclusion
in the main CGIC distribution.

### Thanks and Credits:

*	to Thomas Boutell for providing the CGIC and GD library.
*	to Kai Poetschke for libsqlora.
*	to Brad Barrett, the author of webalizer, whose code I analyzed
        for learning how to do the GD graphs
*	to my colleagues for their encouragement an teaching, in no
        particular order: Bruce, Matthias, Greg, Steffen and others.

