#include <unistd.h>
#include <errno.h>
#include <linux/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <pthread.h>
#include <sys/time.h>
#include <termios.h>
#include "at_channel.h"
#include "typedef.h"
#define TTY_PORT_NAME "/dev/ttyUSB3"
#define MODEMCONTROL_IOC_MAGIC  'h'
#define MODEM_PWR_ON                _IOW(MODEMCONTROL_IOC_MAGIC, 10, int)
#define MODEM_PWR_OFF               _IOW(MODEMCONTROL_IOC_MAGIC, 11, int)
#define MODEM_RESET                 _IOW(MODEMCONTROL_IOC_MAGIC, 12, int)
#define MAX_BUFFER_SIZE 500

struct _ATChannel
{
	int fd;
	char device[100];
	pthread_mutex_t mutex;
	pthread_cond_t cond;
	int resp;
	int close_reader;
	char buffer[MAX_BUFFER_SIZE];
	char sms_pdu[MAX_BUFFER_SIZE];
	pthread_t reader_tid;
};

ATChannel* g_AT_channel = NULL;

static int RIL_ModemReset()
{
	int fd = 0;

	fd = open("/dev/modem", O_RDWR);

	ioctl(fd, MODEM_RESET);
	close (fd);
	sleep(2);

	return 1;
}

static int RIL_ModemPowerOn()
{
	int fd = 0;

	fd = open("/dev/modem", O_RDWR);

	ioctl(fd, MODEM_PWR_ON);
	close (fd);
	sleep(15);

	return 1;
}

static int RIL_ModemPowerOff()
{
	int fd = 0;

	fd = open("/dev/modem", O_RDWR);

	ioctl(fd, MODEM_PWR_OFF);
	close (fd);
	sleep(2);

	return 1;
}
static int open_serial_port (const char* serial_tio_name)
{
	int fd = 0;
	struct termios newtio_p;
	unsigned int tio_baudrate = B115200;

	fd = open (serial_tio_name, O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd <0)
	{           
		perror(serial_tio_name);
		return 0;
	}

	bzero(&newtio_p, sizeof(newtio_p));

	fcntl(fd, F_SETFL, 0);
#ifdef USE_CRTSCTS
	newtio_p.c_cflag = CLOCAL | CREAD | CS8 | HUPCL | tio_baudrate | CRTSCTS;
#else   
	newtio_p.c_cflag = CLOCAL | CREAD | CS8 | HUPCL | tio_baudrate;
#endif

	newtio_p.c_cflag &= ~tio_baudrate;
	newtio_p.c_cflag |= tio_baudrate;

	newtio_p.c_iflag = IGNBRK;
	newtio_p.c_iflag &= ~(INLCR | ICRNL | IGNCR);
	/* set input mode */
	newtio_p.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	// set raw output
	newtio_p.c_oflag &= ~OPOST;
	newtio_p.c_oflag &= ~OLCUC;
	newtio_p.c_oflag &= ~ONLRET;
	newtio_p.c_oflag &= ~ONOCR;
	newtio_p.c_oflag &= ~OCRNL;

	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio_p);

	return fd;
}

static int at_channel_write_ctrlz (ATChannel* thiz, const char* sms_pdu)
{
	size_t cur = 0;
	size_t len = strlen(sms_pdu);
	ssize_t written;
	char data[512];

	snprintf(data, 512, "%s%c%c", sms_pdu, 0x1A, '\r');
	len++;

	/* the main string */
	while (cur < len) 
	{
		do 
		{
			written = write (thiz->fd, data + cur, len - cur);
		} while (written < 0  && errno == EAGAIN);

		if (written < 0)
		{
			printf("write ctrlz error");
			return -1;
		}
		
		printf("write pdu part success\n");
		cur += written;

		if(cur < len) 
		{
			usleep(1000 * 500);
		}
	}

	return 0;
}

static void* at_channel_reader_mainloop(void* arg)
{
	ATChannel* thiz = (ATChannel* )arg;
	int cur = 0;

	while(thiz->close_reader == 0)
	{
		fd_set read_fd;
		int max_fd = 0;

		FD_ZERO(&read_fd);
		FD_SET (thiz->fd, &read_fd);
		max_fd = thiz->fd;

		select (max_fd + 1, &read_fd, NULL, NULL, NULL);
		if(!FD_ISSET(thiz->fd, &read_fd))
		{
			continue;
		}

		pthread_mutex_lock(&thiz->mutex);
		int count = read(thiz->fd, thiz->buffer+cur, MAX_BUFFER_SIZE-cur);
		if(count>0)
		{
			cur += count;
		}
		else if(count==0)
		{
			printf("EOF reached.\n");
			thiz->close_reader = 1;
			//TODO 
			exit(1);
			continue;
		}
		else
		{
			printf("read error.\n");
			continue;
		}

		printf("%s\n", thiz->buffer);
		if(strstr(thiz->buffer, "> ") && thiz->sms_pdu[0]!=0)
		{
			at_channel_write_ctrlz(thiz, thiz->sms_pdu);
			printf("AT < > %s\n", thiz->sms_pdu);
			memset(thiz->sms_pdu, 0, MAX_BUFFER_SIZE);
		}
		else if(strstr(thiz->buffer, "OK") || strstr(thiz->buffer, "ERROR") || strstr(thiz->buffer, "CONNECT"))
		{
			thiz->resp = 1;
			cur = 0;
			pthread_cond_signal(&thiz->cond);
			memset(thiz->buffer, 0, MAX_BUFFER_SIZE);
		}

		pthread_mutex_unlock(&thiz->mutex);
	}

	return (void* )NULL;
}
static void set_timespec_relative(struct timespec *p_ts, long long msec)
{
	struct timeval tv;

	gettimeofday(&tv, (struct timezone *) NULL);

	/* what's really funny about this is that I know
	 *        pthread_cond_timedwait just turns around and makes this
	 *               a relative time again */
	p_ts->tv_sec = tv.tv_sec + (msec / 1000);
	p_ts->tv_nsec = (tv.tv_usec + (msec % 1000) * 1000) * 1000;
	if (p_ts->tv_nsec >= 1000000000)
	{  
		p_ts->tv_sec++;
		p_ts->tv_nsec -= 1000000000;   
	} 
}

void at_channel_reset()
{
	RIL_ModemPowerOff();
	RIL_ModemReset();
	RIL_ModemPowerOn();
}

int at_channel_execute_cmd(ATChannel* thiz, const char* cmd_name, const char* sms_pdu, long long timeout)
{
	int written = 0;
	int err = 0;
	struct timespec ts;
	char cmd[MAX_BUFFER_SIZE] = {0};
	int cur = 0;

	return_val_if_fail(thiz!=NULL && cmd_name!=NULL, -1);
	if(strcmp(cmd_name, "quit") == 0)	
	{
		exit(0);
	}
	snprintf(cmd, MAX_BUFFER_SIZE, "%s\r\n", cmd_name);
	int len = strlen(cmd);
	set_timespec_relative(&ts, timeout);
	if(sms_pdu != NULL)
	{
		strncpy(thiz->sms_pdu, sms_pdu, MAX_BUFFER_SIZE);
	}

	pthread_mutex_lock(&thiz->mutex);
	while(cur < len)
	{
		do
		{
			written = write(thiz->fd, cmd+cur, len-cur);
		}while(written<0 && errno == EINTR);
		if(written < 0) return -2;
		cur += written;
	}

	while(thiz->resp != 1 && err != ETIMEDOUT)
	{
		err = pthread_cond_timedwait(&thiz->cond, &thiz->mutex, &ts);
	}
	pthread_mutex_unlock(&thiz->mutex);

	if(err == ETIMEDOUT)
	{	
		printf("exec cmd %s timeout\n", cmd_name);
		return -1;
	}
	else thiz->resp = 0;
	
	return 0;
}

Ret at_channel_instance_create(const char* device)
{
	return_val_if_fail(g_AT_channel==NULL && device!=NULL, RET_FAIL);
	g_AT_channel  = calloc(1, sizeof(ATChannel));

	if(g_AT_channel != NULL)
	{
		strncpy(g_AT_channel->device, device, strlen(device));
		g_AT_channel->fd = open_serial_port(device);
		if(g_AT_channel->fd <= 0)
		{
			free(g_AT_channel);
			g_AT_channel = NULL;
		
			return RET_FAIL;
		}

		pthread_mutex_init(&g_AT_channel->mutex, NULL);
		pthread_cond_init(&g_AT_channel->cond, NULL);
		pthread_attr_t attr;
		pthread_attr_init(&attr);
		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
		pthread_create(&g_AT_channel->reader_tid, &attr, at_channel_reader_mainloop, g_AT_channel);
	}
	else
	{
		return RET_OOM;
	}

	return RET_OK;
}

ATChannel* at_channel_get_instance(void)
{
	return g_AT_channel;
}

