#include <stdio.h>
#include <string.h>
#include <regex.h>
#include "at_cmd.h"
#include "typedef.h"
#include "at_channel.h"
#define RSP_ITEM_COUNT 20

enum
{
	MATCH_FAIL = 0,
	MATCH_SUCCESS
};

struct _ATCmd
{
	long timeout;
	char cmd_name[BUFFER_LEN];
	char expect_rsp[RSP_ITEM_COUNT][BUFFER_LEN];
	int rsp_count;
};

static int at_cmd_reg_match(ATCmd* thiz, const char* rsp_buf)
{
	regex_t reg;
	regmatch_t pmatch;
	int i = 0;
	int offset = 0;

	for(i=0; i<(thiz->rsp_count); ++i)
	{
		regcomp(&reg, thiz->expect_rsp[i], 0);
		int rt = regexec(&reg, rsp_buf+offset, 1, &pmatch, 0);
		if(rt != 0)
		{
			regfree(&reg);
			return MATCH_FAIL;
		}
		offset += pmatch.rm_eo;
	}
	
	regfree(&reg);
	return MATCH_SUCCESS;
}

ATCmd* at_cmd_create(long timeout)
{
	ATCmd* thiz = calloc(1, sizeof(ATCmd));

	if(thiz != NULL)
	{
		thiz->timeout = timeout;
		thiz->rsp_count = 0;
	}

	return thiz;
}

void at_cmd_destroy(ATCmd* thiz)
{
	free(thiz);
	return;
}

void at_cmd_set_name(ATCmd* thiz, const char* text, size_t length)
{
	memcpy(thiz->cmd_name, text, length);
	thiz->cmd_name[length] = '\r';
	thiz->cmd_name[length+1] = '\n';
	thiz->cmd_name[length+2] = '\0';
}

void at_cmd_add_rsp(ATCmd* thiz, const char* text, size_t length)
{
	return_if_fail(thiz!=NULL && text!=NULL);
	int rsp_count = thiz->rsp_count;
	
	memcpy(thiz->expect_rsp[rsp_count], text, length);
	thiz->rsp_count += 1;
	
	return;
}

void at_cmd_test(ATCmd* thiz)
{
	return_if_fail(thiz != NULL);
	char rsp_buf[BUFFER_LEN] = {0};
	ATChannel* instance = at_channel_get_instance();

	at_channel_execute_cmd(instance, thiz->cmd_name, thiz->timeout, rsp_buf);
	
	int rt = at_cmd_reg_match(thiz, rsp_buf);
	if(rt == MATCH_FAIL)
	{
		printf("\nmatch fail: %s", thiz->cmd_name);
		printf("the rsp is: %s\n", rsp_buf);
	}
}

