//author: wp.4163196@gmail.com 
//all rights reserved.

#include <string.h>
#include <ctype.h>
#include "at_channel.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#define DEFAULT_TIMEOUT 1000
#define isnewline(p) ((p)=='\r'||(p)=='\n')

typedef enum _Mode
{
	MODE_INTERACTIVE,
	MODE_CYCLE_COMMAND,
	MODE_SCRIPT
}Mode;

char* read_file(const char* file_name)
{
	char* buffer = NULL;
	FILE* fp = fopen(file_name, "r");

	if(fp != NULL)
	{
		struct stat st = {0};
		if(stat(file_name, &st) == 0)
		{
			buffer = malloc(st.st_size + 1);
			fread(buffer, st.st_size, 1, fp);
			buffer[st.st_size] = '\0';
		}
	}

	return buffer;
}

Ret parse_script(char* file, ATChannel* channel)
{
	enum _state
	{
		STAT_BEFORE_COMMAND,
		STAT_COMMAND,
		STAT_BEFORE_TIMEOUT,
		STAT_TIMEOUT
	}state = STAT_BEFORE_COMMAND;

	char* buffer = read_file(file);
	
	if(buffer == NULL)
	{
		printf("can not find the script file.\n");
		return RET_FAIL;
	}
	
	char command[BUFFER_LEN] = {0};
	long long timeout = 0;
	char* p_cur = buffer;
	char* p_start = NULL;
	char c_tmp = '\0';	

	for(p_cur=buffer; *p_cur!=0; p_cur++)	
	{
		switch(state)
		{
			case STAT_BEFORE_COMMAND:
			{
				if(!isspace(*p_cur))
				{
					p_start = p_cur;
					state = STAT_COMMAND;
				}
				break;
			}
			case STAT_COMMAND:
			{
				if(isspace(*p_cur))
				{
					c_tmp = *p_cur;
					*p_cur = '\0';
					state = STAT_BEFORE_TIMEOUT;
					strcpy(command, p_start);
					*p_cur = c_tmp;
				}
				break;
			}
			case STAT_BEFORE_TIMEOUT:
			{
				if(!isspace(*p_cur))
				{
					p_start = p_cur;
					state = STAT_TIMEOUT;
				}
				break;
			}
			case STAT_TIMEOUT:
			{
				if(isspace(*p_cur))
				{
					c_tmp = *p_cur;
					*p_cur = '\0';
					state = STAT_BEFORE_COMMAND;
					timeout = atoll(p_start);
					*p_cur = c_tmp;

				}
				break;
			}
		}
		if(isnewline(*p_cur))
		{
			state = STAT_BEFORE_COMMAND;
			if(command[0]=='\0')continue;
			if(timeout == 0) timeout = DEFAULT_TIMEOUT;
			int ret = at_channel_execute_cmd(channel, command, NULL, timeout);
			if(ret != 0)exit(1);
			memset(command, 0, BUFFER_LEN);	
			timeout = 0;
		}
	}

	free(buffer);
	return RET_OK;
}

static void usage(char* s)
{
	printf("<usage>: ./at_test -d <device> -c <AT CMD> -f <script> -i\n");
}
int main(int argc, char* argv[])
{
	int opt = 0;
	char* device = NULL;
	Mode mode = MODE_INTERACTIVE;
	char* command = NULL;
	char* file = NULL;
	ATChannel* thiz = NULL;
	int times = -1;

	while((opt = getopt(argc, argv, "d:c:f:n:i")) != -1) 
	{
		switch(opt)
		{
			case 'd': 
			{
				device = optarg;
				break;
			}
			case 'c': 
			{
				command = optarg;
				break;
			}
			case 'f': 
			{
				file = optarg;
				break;
			}
			case 'n':
			{
				times = atoi(optarg);
				break;
			}
			case 'i': 
			{
				mode = MODE_INTERACTIVE;
				break;
			}
			default:
			{
				usage(argv[0]);
				return -1;
			}
		}
	}

	if(device != NULL)
	{
		int ret = at_channel_instance_create(device);
		thiz = at_channel_get_instance();
		if(ret !=RET_OK)
		{
			return -1;
		}
	}
	else
	{
		usage(argv[0]);
		return -1;
	}

	if(file != NULL)
	{
		mode = MODE_SCRIPT;
	}
	else if(command != NULL)
	{
		mode = MODE_CYCLE_COMMAND;
	}
	else
	{
		mode = MODE_INTERACTIVE;
	}
	
	if(mode == MODE_INTERACTIVE)
	{
		command = (char* )calloc(1, BUFFER_LEN);
		
		for(;;)
		{
			scanf("%s", command);
			at_channel_execute_cmd(thiz, command, NULL, 5000);
		}
	}
	else if(mode == MODE_CYCLE_COMMAND)
	{
		if(times == -1)
		{
			for(;;)
			{
				at_channel_execute_cmd(thiz, command, NULL, 5000);
			}
		}
		else
		{
			int i;
			for(i=0; i<times; i++)
			{
				at_channel_execute_cmd(thiz, command, NULL, 5000);
				sleep(1);
			}
		}
	}
	else if(mode == MODE_SCRIPT)
	{
		parse_script(file, thiz);
	}
#if 0	
	int ret = at_channel_instance_create(argv[1]);
	if(ret !=RET_OK)
	{
		return -1;
	}

	ATChannel* thiz = at_channel_get_instance();

	char cmd[100] = "AT^HCMGS=\"13423865441\"";
	char sms_pdu[100] = "0000021002040702C4D08E195104081300032C2EB0010C105F7DFCF469EFD7865DD9C0";
	char cmd1[100] = {0};

	if(argc > 2)
	{
		if(strncmp(argv[2], "reset", strlen(argv[2])) == 0)
		{
			printf("reset the module\n");
			at_channel_reset();
			printf("finished reseting the module\n");
			return 0;
		}
		else if(strncmp(argv[2], "default", strlen(argv[2])) != 0)
		{
			strncpy(cmd1, argv[2], strlen(argv[2]));
		}
	}
	else
	{
		printf("<usage>: ./at_test reset|default|<AT COMMAND>\n");
		return 0;
	}

	while(1)
	{
		if(strlen(cmd1) > 0)
		{
			at_channel_execute_cmd(thiz, cmd1, NULL, 50000);
			//sleep(1);
			continue;
		}
		else
		{
			printf("start testing commands......\n");
			at_channel_execute_cmd(thiz, "AT", NULL, 10000);
			at_channel_execute_cmd(thiz, "AT", NULL, 10000);
			at_channel_execute_cmd(thiz, "AT", NULL, 10000);
			printf("test commands send ok, continue\n");

			at_channel_execute_cmd(thiz, cmd, sms_pdu, 60000);
			sleep(6);
			continue;
		}
	}
#endif
	return 0;
}

