#include <string.h>
#include "xml_builder_at.h"
#include "at_cmd.h"

enum
{
	TEXT_NONE = 0,
	TEXT_NAME,
	TEXT_RSP
};

typedef struct _PrivInfo
{
	//FILE* fp;
	int text_state;
	ATCmd* cmd;
}PrivInfo;

static void xml_builder_at_on_start_element(XmlBuilder* thiz, const char* tag, const char** attrs)
{
	return_if_fail(thiz != NULL && tag != NULL && attrs != NULL);
	PrivInfo* info = (PrivInfo* )thiz->priv;
	
	if(strcmp(tag, "cmd") == 0)
	{
		info->cmd = at_cmd_create(atol(attrs[1]));
	}
	else if(strcmp(tag, "name") == 0)
	{
		info->text_state = TEXT_NAME;
	}
	else if(strcmp(tag, "rsp") == 0)
	{
		info->text_state = TEXT_RSP;
	}

	return;
}

static void xml_builder_at_on_end_element(XmlBuilder* thiz, const char* tag)
{
	return_if_fail(thiz != NULL && tag != NULL);
	PrivInfo* info = (PrivInfo* )thiz->priv;
	
	if(strcmp(tag, "cmd") == 0)
	{
		at_cmd_test(info->cmd);
		at_cmd_destroy(info->cmd);
	}

	return;
}

static void xml_builder_at_on_text(XmlBuilder* thiz, const char* text, size_t length)
{
	return_if_fail(thiz != NULL && text != NULL);
	PrivInfo* info = (PrivInfo* )thiz->priv;
	
	switch(info->text_state)
	{
		case TEXT_NAME:
		{
			at_cmd_set_name(info->cmd, text, length);
			break;
		}
		case TEXT_RSP:
		{
			at_cmd_add_rsp(info->cmd, text, length);
			break;
		}
		default:
		{
			break;
		}
	}

	return;
}

static void xml_builder_at_on_comment(XmlBuilder* thiz, const char* text, size_t length)
{
	return;
}

static void xml_builder_at_on_pi_element(XmlBuilder* thiz, const char* tag, const char** attrs)
{
	return;	
}

static void xml_builder_at_on_error(XmlBuilder* thiz, int line, int row, const char* message)
{
	return;
}

static void xml_builder_at_destroy(XmlBuilder* thiz)
{
	return_if_fail(thiz!=NULL);
	free(thiz);

	return;
}

XmlBuilder* xml_builder_at_create()
{
	XmlBuilder* thiz = (XmlBuilder* )calloc(1, sizeof(XmlBuilder)+sizeof(PrivInfo));
	if(thiz != NULL)
	{
		PrivInfo* info = (PrivInfo*)thiz->priv;

		thiz->on_start_element = xml_builder_at_on_start_element;
		thiz->on_end_element = xml_builder_at_on_end_element;
		thiz->on_text = xml_builder_at_on_text;
		thiz->on_comment = xml_builder_at_on_comment;
		thiz->on_pi_element = xml_builder_at_on_pi_element;
		thiz->on_error = xml_builder_at_on_error;
		thiz->destroy = xml_builder_at_destroy;

		//info->fp = fp!=NULL ? fp : stdout;
		info->text_state = TEXT_NONE;
	}

	return thiz;
}
