#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <lua.h>
#include <lauxlib.h>
#include <applet.h>


#define NTP_SERVER    "time.buptnet.edu.cn"

#define NTP_PORT      123

#define JAN_1970      0x83aa7e80      /* 2208988800 1970 - 1900 in seconds */

#define NTPFRAC(x) (4294 * (x) + ((1981 * (x))>>11))
#define USEC(x) (((x) >> 12) - 759 * ((((x) >> 10) + 32768) >> 16))


struct ntptime
{
    unsigned int coarse;
    unsigned int fine;
};

void send_packet(int fd)
{
    unsigned int data[12];
    struct timeval now;
    int ret;
#define LI 0
#define VN 3
#define MODE 3
#define STRATUM 0
#define POLL 4
#define PREC -6

    if (sizeof(data) != 48)
    {
        fprintf(stderr,"size error\n");
        return;
    }

    memset((char*)data, 0, sizeof(data));
    data[0] = htonl((LI << 30) | (VN << 27) | (MODE << 24)
                  | (STRATUM << 16) | (POLL << 8) | (PREC & 0xff));
    data[1] = htonl(1<<16);  /* Root Delay (seconds) */
    data[2] = htonl(1<<16);  /* Root Dispersion (seconds) */
    gettimeofday(&now, NULL);
    data[10] = htonl(now.tv_sec + JAN_1970); /* Transmit Timestamp coarse */
    data[11] = htonl(NTPFRAC(now.tv_usec));  /* Transmit Timestamp fine   */
    send(fd, data, 48, 0);
}

void get_packet_timestamp(int usd, struct ntptime *udp_arrival_ntp)
{
    struct timeval udp_arrival;

    gettimeofday(&udp_arrival, NULL);
    udp_arrival_ntp->coarse = udp_arrival.tv_sec + JAN_1970;
    udp_arrival_ntp->fine   = NTPFRAC(udp_arrival.tv_usec);
}

void rfc1305print(unsigned int *data, struct ntptime *arrival, struct timeval* tv)
{
    int li, vn, mode, stratum, poll, prec;
    int delay, disp, refid;
    struct ntptime reftime, orgtime, rectime, xmttime;
    struct tm *ltm;

#define Data(i) ntohl(((unsigned int *)data)[i])
    li      = Data(0) >> 30 & 0x03;
    vn      = Data(0) >> 27 & 0x07;
    mode    = Data(0) >> 24 & 0x07;
    stratum = Data(0) >> 16 & 0xff;
    poll    = Data(0) >>  8 & 0xff;
    prec    = Data(0)       & 0xff;
    if (prec & 0x80) prec|=0xffffff00;
    delay   = Data(1);
    disp    = Data(2);
    refid   = Data(3);
    reftime.coarse = Data(4);
    reftime.fine   = Data(5);
    orgtime.coarse = Data(6);
    orgtime.fine   = Data(7);
    rectime.coarse = Data(8);
    rectime.fine   = Data(9);
    xmttime.coarse = Data(10);
    xmttime.fine   = Data(11);
#undef Data

    tv->tv_sec = xmttime.coarse - JAN_1970;
    tv->tv_usec = USEC(xmttime.fine);
}

int set_local_time(struct timeval tv)
{
    /* need root user. */
    if (0 != getuid() && 0 != geteuid())
        return 1;

    settimeofday(&tv, NULL);
    return 0;
}

int ntpdate( const char *ipstr, double *offset)
{
    /* get ip. */ 
    unsigned long int ip = inet_addr( ipstr);
    if ( ip < 0 )
    {
            printf( "IP error!\n");
            return 1;
    }
    
            
    /* get now. */
    time_t t1;
    time( &t1);
    printf( "start: %s", ctime(&t1));
    
    /* offset */
    *offset = 0 ;
    struct timeval time1;
    
    /* create socket. */
    int sock = socket(PF_INET, SOCK_DGRAM, 0);
    
    int addr_len;
    
    /* bind local address. */
    struct sockaddr_in addr_src;
    addr_len = sizeof( addr_src);
    memset(&addr_src, 0, addr_len);
    addr_src.sin_family = AF_INET;
    addr_src.sin_addr.s_addr = htonl(INADDR_ANY);
    addr_src.sin_port = htons(0);
    if( bind(sock, (struct sockaddr*)&addr_src, addr_len) < 0 )
    {
            printf( "bind not\n");
            return 1;
    }            
            
    /* connect to ntp server. */
    struct sockaddr_in addr_dst;
    addr_len = sizeof( addr_dst);
    memset(&addr_dst, 0, addr_len);
    addr_dst.sin_family = AF_INET;
    addr_dst.sin_addr.s_addr = ip;
/*
    {
        struct hostent *host = (struct hostent *) gethostbyname(NTP_SERVER);
        memcpy(&(addr_dst.sin_addr.s_addr), host->h_addr_list[0], 4);
    }
*/  
    addr_dst.sin_port = htons(NTP_PORT);
    if( connect(sock, (struct sockaddr*)&addr_dst, addr_len) < 0 )
    {
        fd_set fds_read;
            printf( "connect not\n");
            return 1;
    }            
    
    int i = 0;
    int flag = 0;
    
    while (1)
    {
        printf( "i : %d", i);
        if ( i++ > 10) 
                break;
                
        fd_set fds_read;
        struct timeval timeout;
        int ret;

        unsigned int buf[12];
        int len;
        
        struct sockaddr server;
        socklen_t svr_len;
        struct ntptime arrival_ntp;
        struct timeval newtime;

        FD_ZERO(&fds_read);
        FD_SET(sock, &fds_read);

        timeout.tv_sec = 6;
        timeout.tv_usec = 0;
        ret = select(sock + 1, &fds_read, NULL, NULL, &timeout);
        if (0 == ret || !FD_ISSET(sock, &fds_read))
        {
            /* send ntp protocol packet. */
            //printf( "send ntp protocol packet\n");
            send_packet(sock);
            continue;
        }
        
        
        /* recv ntp server's response. */
        ret = recvfrom(sock, buf, sizeof(buf), 0, &server, &svr_len);

        /* get now time */
        gettimeofday( &time1, NULL);        

        if( ret > 0)
        {

                /* get local timestamp. */
                get_packet_timestamp(sock, &arrival_ntp);
                /* get server's time and print it. */
                rfc1305print(buf, &arrival_ntp, &newtime);
                /* set local time to the server's time, if you're a root user. */
                if( set_local_time(newtime))
                {
                        printf( "set local time error!\n");
                        continue;
                }
                *offset = (newtime.tv_sec + newtime.tv_usec / 1000000.0) - (time1.tv_sec - time1.tv_usec /1000000.0);
                        
                flag = 1;
                break;
        }
    }
    printf( "offset: %f\n", *offset); 
    
    /* get now. */
    time( &t1);
    printf( "end : %s", ctime(&t1));

    close(sock);
    
    if ( flag)
        return 0;
    else
        return 1;
}

int ntpdate_4lua( lua_State *L)
{
        char *ip = (char *) getStr(L);
        
        printf( "ip: %s\n", ip);
        
        double offset;
        
        int ret = ntpdate( ip, &offset);
        
        printf( "ret: %d\n", ret);

        putNumber( L, ret);
        
        putDoubleNumber( L, offset);
        
        return 2;
        
}