#include <gidx.h>
#include <unistd.h>
#include <gidx-rd.h>
#include <gidx-format.h>
#include <gidx-query.h>
#include <hgang.h>
#define _THIS_IS_LIBGIDX_RD 1
#include "gidx-rd-core.h"

struct query_cond {
	const struct query_cond_ops	*qc_ops;
	struct _gidx_query		*qc_owner;
	void				*qc_priv;
	struct list_head		qc_list;
	const struct gidx_idx		*qc_idx;
	gidx_oid_t			qc_max_result;
	gidx_oid_t			qc_num_result;
	gidx_oid_t			*qc_res;
};

struct _gidx_query {
	struct _gidx_rd		*q_db;
	struct list_head	q_cond;
	unsigned int		q_num_cond;
	hgang_t			q_condmem;
};

gidx_query_t gidx_query_new(gidx_rd_t db)
{
	struct _gidx_query *q;

	q = calloc(1, sizeof(*q));
	if ( NULL == q )
		return NULL;

	q->q_db = db;
	INIT_LIST_HEAD(&q->q_cond);

	/* TODO: using number of primary + secondary keys would be a
	 * better heuristics */
	q->q_condmem = hgang_new(sizeof(struct query_cond), 0);
	if ( NULL == q->q_condmem ) {
		free(q);
		return NULL;
	}

	return q;
}

/* TODO */
int gidx_query_select(gidx_query_t q, const char *name)
{
	gidx_idx_t idx;

	idx = gidx_rd_resultset(q->q_db, name);
	if ( NULL == idx ) {
		ERR("%s: not found", name);
		return 0;
	}
	DEBUG("%s", name);

	return 1;
}

int gidx_query_execute(gidx_query_t q)
{
	struct query_cond *qc;

	list_for_each_entry(qc, &q->q_cond, qc_list) {
		if ( !qc->qc_ops->execute(qc->qc_priv) )
			return 0;
	}

	return 1;
}

void gidx_query_free(gidx_query_t q)
{
	struct query_cond *qc;
	list_for_each_entry(qc, &q->q_cond, qc_list) {
		qc->qc_ops->dtor(qc->qc_priv);
	}
	hgang_free(q->q_condmem);
	free(q);
}

int _gidx_query_cond_new(gidx_query_t q, gidx_idx_t idx,
				const struct query_cond_ops *qops,
				void *priv)
{
	struct query_cond *qc;
	struct ro_vec idx_name;

	_gidx_rd_idx_name(q->q_db, idx, &idx_name);
	DEBUG("%.*s", (int)idx_name.v_len, idx_name.v_ptr);

	qc = hgang_alloc0(q->q_condmem);
	if ( NULL == qc ) {
		ERR("hgang_alloc0: %s", os_err());
		return 0;
	}

	qc->qc_owner = q;
	qc->qc_ops = qops;
	qc->qc_priv = priv;
	qc->qc_idx = idx;

	switch(idx->i_mode) {
	case GIDX_NONE_KEY:
		ERR("Cannot query none-key fields");
		goto out_free;
	case GIDX_PRIMARY_KEY:
		qc->qc_max_result = 1;
		break;
	case GIDX_SECONDARY_KEY:
		if ( NULL == qc->qc_ops->result_set_size ) {
			ERR("%s type doesn't support secondary key queries",
				gidx_type_str(idx->i_type));
			goto out_free;
		}
		if ( !(*qc->qc_ops->result_set_size)(qc->qc_priv,
						&qc->qc_max_result) )
			goto out_free;
		break;
	default:
		abort();
	}

	qc->qc_res = malloc(qc->qc_max_result * sizeof(*qc->qc_res));
	if ( NULL == qc->qc_res ) {
		ERR("malloc: %s", os_err());
		goto out_free;
	}

	list_add_tail(&qc->qc_list, &q->q_cond);

	return 1;
out_free:
	free(qc);
	return 0;
}

const uint8_t *_gidx_query_get_chunk(gidx_query_t q, gidx_idx_t idx,
					unsigned int chunk_id,
					size_t *sz)
{
	return gidx_rd_user_chunk(q->q_db, idx, chunk_id, sz);
}
