#include <gidx.h>
#include <fobuf.h>
#include <gidx-format.h>
#include <gidx-wr.h>
#define _THIS_IS_LIBGIDX_WR 1
#include "gidx-wr-core.h"
#define _THIS_IS_GIDX_WR_C 1
#include "gidx-wr-skey.h"

static gidx_oid_t pbuf_max;
static gidx_oid_t pbuf_sz;
static gidx_oid_t *pbuf;

int skey_init(skey_ctx_t ctx, const gidx_wr_t wr, fobuf_t buf)
{
	ctx->ctx_buf = buf;
	ctx->ctx_tot_rec = gidx_wr_num_oid(wr);
	ctx->ctx_num_rec = 0;
	ctx->ctx_num_val = 0;
	ctx->ctx_cur_val = 0;
	return 1;
}

int skey_tbl_entry(skey_ctx_t ctx, gidx_oid_t num_oid)
{
	struct gidx_skey_tbl tbl;

	tbl.num_oid = num_oid;
	tbl.oid_idx = ctx->ctx_num_rec;
	if ( !fobuf_write(ctx->ctx_buf, &tbl, sizeof(tbl)) )
		return 0;

	ctx->ctx_num_rec += num_oid;
	ctx->ctx_num_val++;

	if ( num_oid > pbuf_max )
		pbuf_max = num_oid;
	return 1;
}

int skey_init2(skey_ctx_t ctx)
{
	if ( ctx->ctx_tot_rec < ctx->ctx_num_rec ) {
		ERR("tot_rec/num_rec mismatch %u/%u",
			ctx->ctx_tot_rec, ctx->ctx_num_rec);
		return 0;
	}

	if ( pbuf_max > pbuf_sz ) {
		gidx_oid_t *new;

		new = realloc(pbuf, sizeof(*pbuf) * pbuf_max);
		if ( NULL == new ) {
			ERR("realloc: %s", os_err());
			return 0;
		}

		pbuf = new;
		pbuf_sz = pbuf_max;
	}

	return 1;
}

static int skey_cb(void *priv, void *obj)
{
	gidx_oid_t **pb = priv;
	gidx_oid_t *key = obj;

	if ( (*pb) > (pbuf + pbuf_sz) )
		return 0;

	(**pb) = (*key);
	(*pb)++;
	return 1;
}

int skey_keyblock(skey_ctx_t ctx, hgang_t oids)
{
	gidx_oid_t *pb_tmp;
	gidx_oid_t num_oid;

	pb_tmp = pbuf;

	if ( !hgang_foreach(oids, skey_cb, &pb_tmp) ) {
		ERR("skey_block larger than expected");
		return 0;
	}

	num_oid = pb_tmp - pbuf;

	sort(pbuf, num_oid, sizeof(*pbuf), gidx_oid_cmp);
	ctx->ctx_cur_val++;

	if ( !fobuf_write(ctx->ctx_buf, pbuf, sizeof(*pbuf) * num_oid) )
		return 0;

	return 1;
}

int skey_fini(skey_ctx_t ctx)
{
	if ( ctx->ctx_cur_val != ctx->ctx_num_val ) {
		ERR("cur_val/num_val mismatch %u/%u",
			ctx->ctx_cur_val, ctx->ctx_num_val);
		return 0;
	}
	if ( ctx->ctx_num_rec > ctx->ctx_tot_rec ) {
		ERR("num_rec/tot_rec mismatch %u/%u",
			ctx->ctx_num_rec, ctx->ctx_tot_rec);
		return 0;
	}
	return 1;
}

struct oid_priv {
	gidx_oid_t val;
	gidx_oid_t *blk;
	gidx_oid_t num_rec;
};

static int oid_cb(void *priv, void *obj)
{
	struct oid_priv *oid_priv = priv;
	gidx_oid_t oid = *(gidx_oid_t *)obj;

	DEBUG("val[%u] maps to oid %u", oid_priv->val, oid);
	assert(oid < oid_priv->num_rec);
	oid_priv->blk[oid] = oid_priv->val;

	return 1;
}

int oid_from_hgang(gidx_oid_t val_idx, hgang_t oids,
			gidx_oid_t *blk, gidx_oid_t num_rec)
{
	struct oid_priv oid_priv;
	oid_priv.val = val_idx;
	oid_priv.blk = blk;
	oid_priv.num_rec = num_rec;
	return hgang_foreach(oids, oid_cb, &oid_priv);
}
