#include <gidx.h>
#include <gidx-wr.h>
#include <gang.h>
#include <gidx-format.h>
#define _THIS_IS_LIBGIDX_WR 1
#include "gidx-wr-core.h"
#define _THIS_IS_GIDX_STR_WR 1
#include "gidx-str-wr.h"

int _str_ordered_prep(struct str_wr_priv *strp)
{
	return 1;
}

uint16_t _str_ordered_num_chunks(struct str_wr_priv *strp)
{
	return STRTAB_NUM_CHUNKS;
}

static size_t strtab_sz(struct str_wr_priv *strp)
{
	struct str_val *v;
	size_t ret;

	for(ret = 0, v = (struct str_val *)node_first(&strp->p_vals);
		v; v = (struct str_val *)node_next(&v->n_rbt)) {
		ret += v->n_str.v_len;
	}

	return ret;
}

off_t _str_ordered_chunk_size(struct str_wr_priv *strp, unsigned int chunk_id)
{
	switch(chunk_id) {
	case STRTAB_CHUNK_VALS:
		return sizeof(struct ordered_val) * strp->p_num_vals;
	case STRTAB_CHUNK_STRTAB:
		return strtab_sz(strp);
	default:
		abort();
	}
	return 0;
}

static int write_vals(fobuf_t buf, struct rb_node *n, uint32_t *ofs)
{
	struct str_val *node;
	struct ordered_val val;

	if ( NULL == n )
		return 1;

	if ( n->rb_child[CHILD_LEFT] )
		if ( !write_vals(buf, n->rb_child[CHILD_LEFT], ofs) )
			return 0;

	node = (struct str_val *)n;
	val.o_str = (*ofs);
	val.o_len = node->n_str.v_len;
	DEBUG("val: val = %.*s @ ofs %u",
		(int)node->n_str.v_len, node->n_str.v_ptr, (*ofs));
	if ( !fobuf_write(buf, &val, sizeof(val)) )
		return 0;
	(*ofs) += node->n_str.v_len;

	if ( n->rb_child[CHILD_RIGHT] )
		if ( !write_vals(buf, n->rb_child[CHILD_RIGHT], ofs) )
			return 0;

	return 1;
}

static int write_strtab(fobuf_t buf, struct rb_node *n)
{
	struct str_val *node;

	if ( NULL == n )
		return 1;

	if ( n->rb_child[CHILD_LEFT] )
		if ( !write_strtab(buf, n->rb_child[CHILD_LEFT]) )
			return 0;

	node = (struct str_val *)n;
	if ( !fobuf_write(buf, node->n_str.v_ptr, node->n_str.v_len) )
		return 0;

	if ( n->rb_child[CHILD_RIGHT] )
		if ( !write_strtab(buf, n->rb_child[CHILD_RIGHT]) )
			return 0;

	return 1;
}
int _str_ordered_chunk(struct str_wr_priv *strp, unsigned int chunk_id,
				fobuf_t buf)
{
	uint32_t ofs = 0;

	switch(chunk_id) {
	case STRTAB_CHUNK_VALS:
		return write_vals(buf, strp->p_vals.rbt_root, &ofs);
	case STRTAB_CHUNK_STRTAB:
		return write_strtab(buf, strp->p_vals.rbt_root);
	default:
		abort();
	}
	return 1;
}

void _str_ordered_free(struct str_wr_priv *strp)
{
}
