/*
 * CAN_NODE_1.c
 * 
 * Status: NODE 1
 * 
 * Created: 01-06-2012 AM 11:11:52
 *  Author: Ayush Sinha
 */ 
#define  F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/atomic.h>
#include <avr/wdt.h>
#include "init.h"
#include "spi.h"
#include "adc.h"
#include "uart.h"
#include "mcp2515reg.h"
#include "mcp2515.h"
#include "can.h"
#include "can_buffer.h"
#include "messagedef.h"
#include "message.h"
#include "pneumatic.h"

#if ( TERMINAL == 1 )
#include "uart.h"
#include "uart_buffer.h"
#include "terminal.h"							
extern volatile MSGSTRM_STATE	strm;
volatile UartBuffer UART_RxBuffer;
#endif

/************************************************************************
 *	GLOBALS
 */
/* CAN Buffer */
volatile CanBuffer  RxBuffer;
volatile CanBuffer  TxBuffer;

/* Messages */
volatile CanMessage rec;
CanMessage PneumaticShift, Oil, Brake;

/* Timer */
volatile uint16_t	tick_msg1 = 0, tick_msg2 = 0,
					tick_msg3 = 0, tick_msg4 = 0, tick_msg5 = 0, 
					tick_msg6 = 0, tick_msg7 = 0;
					
/* Pneumatic */
volatile uint16_t debounce_ticker1 = 0;
volatile uint16_t debounce_ticker2 = 0;
volatile ButtonState button_int1 = BUTTON_RELEASED;
volatile ButtonState button_int2 = BUTTON_RELEASED;

/************************************************************************
 *	MAIN
 */

int main(void)
{	 
	CanMessage tmp;											/* Local Variables */

#if ( TERMINAL == 1 )
	UART_Init();											/* UART */
	ADC_Init();												/* ADC */
#endif	
												
	GPIO_Init();											/* GPIO */
	ExtINT_Init();											/* External Interrupt */
	Timer_Init();											/* Timers */											
	SPI_Init();												/* SPI */
											
	CanStatus res = CAN_Init(CAN_SPEED);					/* Start CAN */	
	
#if ( TERMINAL == 1 )		
	term_Start(res);										/* Start Terminal */
	UART_BufInit( &UART_RxBuffer, UART_RX_BUFFER_SIZE );	/* UART receive buffer */
#endif
	
	Msg_Init();												/* Construct Messages to be sent */
	
	CAN_BufInit( &RxBuffer, CAN_RX_BUFFER_SIZE );			/* Initialize Receive Buffer */
	CAN_BufInit( &TxBuffer, CAN_TX_BUFFER_SIZE );			/* Initialize Transmit Buffer */

	sei();													/* Enable Global Interrupts */
	
	wdt_enable(WDTO_1S);									/* Enable Watchdog Timer */

/* --------------- LOOP FOREVER ----------------------*/
	while(1){
				
		//Get Data
		
		/* Send Messages ---------------------------- */
		ATOMIC_BLOCK( ATOMIC_FORCEON ){
			
			if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_EMPTY ){
				
				CAN_BufDeq( &TxBuffer, &tmp );				/* Dequeue Transmit Buffer */
				CAN_SendMsg( &tmp );						/* Send */
			}
		}										
				
		/* Receive Messages ------------------------- */
		
		ATOMIC_BLOCK( ATOMIC_FORCEON ){						
			
			if( CAN_BufState( &RxBuffer ) != CAN_BUFFER_EMPTY ){
		
				CAN_BufDeq( &RxBuffer, &tmp );				/* Dequeue Receive Buffer */
				Msg_Chk( &tmp );							/* Check Received Message */		

#if ( TERMINAL == 1 )		
				if( strm == MS_STREAM )						/* Enable Terminal Message Stream */
					term_RxMsg( &tmp );
#endif
			}										
		}				

		/* ------------------------------------------ */
		
#if ( TERMINAL == 1 )											
		term_Main();										/* TERMINAL FOR DEBUGGING */
#endif
		wdt_reset();										/* Reset Watchdog */
    }
	return 0;
}

/************************************************************************
 *	INT0 - CAN RECEIVED MESSAGE 
 */
ISR(INT0_vect)
{	
	CAN_ReadMsg( &rec );									/* Read Message Received */
	
	if( CAN_BufState( &RxBuffer ) != CAN_BUFFER_FULL ){	
		CAN_BufEnq( &RxBuffer, &rec );						/* Enqueue to Receive Buffer */
	}	
}
 
/************************************************************************
 *	TIMER0 INTERRUPT (1 MS)
 */ 
 ISR(TIMER0_COMP_vect)
{
	/* Enqueue into Transmit Buffer ------------------------------------ */	
	if( tick_msg1++ >= RATE_MSG1 ){							/* Message 1 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			CAN_BufEnq( &TxBuffer, &Oil );
			
		tick_msg1 = 0;	
	}	
	
	if( tick_msg2++ >= RATE_MSG2 ){							/* Message 2 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			CAN_BufEnq( &TxBuffer, &Brake );	
		
		tick_msg2 = 0;		
	}
	
	if( tick_msg3++ >= RATE_MSG3 ){							/* Message 3 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			//CAN_BufEnq( &TxBuffer, &throttle );	
		
		tick_msg3 = 0;		
	}
	
	if( tick_msg4++ >= RATE_MSG4 ){							/* Message 4 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			//CAN_BufEnq( &TxBuffer, &throttle );	
		
		tick_msg4 = 0;		
	}
	
	if( tick_msg5++ >= RATE_MSG5 ){							/* Message 5 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			//CAN_BufEnq( &TxBuffer, &throttle );	
		
		tick_msg5 = 0;		
	}
	
	if( tick_msg6++ >= RATE_MSG6 ){							/* Message 6 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			//CAN_BufEnq( &TxBuffer, &throttle );	
		
		tick_msg6 = 0;		
	}
	
	if( tick_msg7++ >= RATE_MSG7 ){							/* Message 7 */
		
		if( CAN_BufState( &TxBuffer ) != CAN_BUFFER_FULL )
			//CAN_BufEnq( &TxBuffer, &throttle );	
		
		tick_msg7 = 0;		
	}
}

/************************************************************************
 *	INT1 INTERRUPT - UPSHIFT
 */
ISR( INT1_vect )
{	
	Debounce_UpShift();
}

/************************************************************************
 *	INT2 INTERRUPT - DOWNSHIFT
 */
ISR( INT2_vect )
{	
	Debounce_Downshift();
}

/************************************************************************
 *	TIMER2 INTERRUPT (1 MS)
 */ 
 ISR(TIMER2_COMP_vect)
 {	
	DebTimer_Upshift();
	DebTimer_Downshift();
 }
 