#include "LPC17xx.h"
#include "uart.h"

// Counter holding number of times external interrupt 0 triggered
volatile unsigned int eint3_count;

// ***** EINT3 Initialisation function *****
// Setup External Interrupt 0 for RDB1768 ISP button
void EINT3_init (void)
{
	eint3_count = 0;
	LPC_GPIOINT->IO2IntEnF |= 1 << 0;
	LPC_GPIOINT->IO2IntEnR |= 1 << 0;
	
	// Setup counter for number of ExtInt0's received

	// Enabled External Interrupt 0 (the ISP button on RDB1768).
	NVIC_EnableIRQ(EINT3_IRQn);

}

void EINT3_enable() {
	eint3_count = 0;
	LPC_GPIOINT->IO2IntClr |= (1 << 0);
	LPC_GPIOINT->IO2IntEnF |= 1 << 0;
	LPC_GPIOINT->IO2IntEnR |= 1 << 0;
	//LPC_GPIOINT->IO2IntEnR |= 1 << 0;
}

// ***** EINT3 Interrupt Handler*****
// Each time an external interrupt is generated by pressing the ISP
// button, the state of the user LEDs (LEDs 2-5) will toggle.
void EINT3_IRQHandler (void)
{
	//disable interrupt
	LPC_GPIOINT->IO2IntEnF &= ~(1 << 0);


	if (LPC_GPIOINT->IO2IntStatF & (1 << 0)) {
		//UARTSendStringln(2, "1");
		// Interrupt has been generated due to a falling edge on P2.0
		eint3_count = 1;
		//led2_invert();
		//LPC_GPIOINT->IO2IntEnF &= ~(1 << 0);
	}
	if (LPC_GPIOINT->IO2IntStatR & (1 << 0)) {
		//UARTSendStringln(2, "1");
		// Interrupt has been generated due to a falling edge on P2.0
		eint3_count = 1;
		//led2_invert();
		//LPC_GPIOINT->IO2IntEnF &= ~(1 << 0);
	}
	/*
	if (LPC_GPIOINT->IO2IntStatR & (1 << 0)) {
		//UARTSendStringln(2, "2");
		// Interrupt has been generated due to a falling edge on P2.0
		eint3_count = 1;
	}
	*/

	// Clear interrupt
	LPC_GPIOINT->IO2IntClr |= (1 << 0);

}

