/*
 ============================================================================
 Name        : hev-impathy-server-tls-handler.c
 Author      : Heiher <admin@heiher.info>
 Version     : 0.0.1
 Copyright   : Copyright (C) 2011 everyone.
 Description : 
 ============================================================================
 */

#include <telepathy-glib/util.h>
#include <telepathy-glib/interfaces.h>

#include "hev-impathy-server-tls-handler.h"
#include "hev-impathy-tls-certificate.h"

enum
{
	PROP_CHANNEL = 1,
	PROP_TLS_CERTIFICATE,
	PROP_HOSTNAME,
	PROP_REFERENCE_IDENTITIES,
	N_PROPERTIES
};

#define HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), HEV_TYPE_IMPATHY_SERVER_TLS_HANDLER, HevImpathyServerTLSHandlerPrivate))

typedef struct _HevImpathyServerTLSHandlerPrivate HevImpathyServerTLSHandlerPrivate;

struct _HevImpathyServerTLSHandlerPrivate
{
	TpChannel *channel;

	GObject *certificate;
	gchar *hostname;
	gchar **reference_identities;

	GSimpleAsyncResult *async_init_res;
};

static void hev_async_initable_iface_init(GAsyncInitableIface *iface);

G_DEFINE_TYPE_WITH_CODE(HevImpathyServerTLSHandler, hev_impathy_server_tls_handler, G_TYPE_OBJECT,
			G_IMPLEMENT_INTERFACE(G_TYPE_ASYNC_INITABLE, hev_async_initable_iface_init));

static void hev_impathy_server_tls_handler_dispose(GObject * obj)
{
	HevImpathyServerTLSHandler * self = HEV_IMPATHY_SERVER_TLS_HANDLER(obj);
	HevImpathyServerTLSHandlerPrivate * priv = HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	G_OBJECT_CLASS(hev_impathy_server_tls_handler_parent_class)->dispose(obj);
}

static void hev_impathy_server_tls_handler_finalize(GObject * obj)
{
	HevImpathyServerTLSHandler * self = HEV_IMPATHY_SERVER_TLS_HANDLER(obj);
	HevImpathyServerTLSHandlerPrivate * priv = HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(priv->channel)
	{
		tp_clear_object(&priv->channel);
		priv->channel = NULL;
	}

	if(priv->certificate)
	{
		tp_clear_object(&priv->certificate);
		priv->certificate = NULL;
	}

	if(priv->reference_identities)
	{
		g_strfreev(priv->reference_identities);
		priv->reference_identities = NULL;
	}

	if(priv->hostname)
	{
		g_free(priv->hostname);
		priv->hostname = NULL;
	}

	G_OBJECT_CLASS(hev_impathy_server_tls_handler_parent_class)->finalize(obj);
}

static GObject * hev_impathy_server_tls_handler_constructor(GType type, guint n, GObjectConstructParam * param)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);
	return G_OBJECT_CLASS(hev_impathy_server_tls_handler_parent_class)->constructor(type, n, param);
}

static void hev_impathy_server_tls_handler_constructed(GObject * obj)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	G_OBJECT_CLASS(hev_impathy_server_tls_handler_parent_class)->constructed(obj);
}

static void hev_impathy_server_tls_handler_get_property(GObject *object,
			guint property_id, GValue *value, GParamSpec *pspec)
{
	HevImpathyServerTLSHandler *self = HEV_IMPATHY_SERVER_TLS_HANDLER(object);
	HevImpathyServerTLSHandlerPrivate *priv =
		HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	switch(property_id)
	{
	case PROP_CHANNEL:
		g_value_set_object(value, priv->channel);
		break;
	case PROP_TLS_CERTIFICATE:
		g_value_set_object(value, priv->certificate);
		break;
	case PROP_HOSTNAME:
		g_value_set_string(value, priv->hostname);
		break;
	case PROP_REFERENCE_IDENTITIES:
		g_value_set_boxed(value,
					priv->reference_identities);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
					property_id, pspec);
		break;
	}
}

static void hev_impathy_server_tls_handler_set_property(GObject *object,
			guint property_id, const GValue *value, GParamSpec *pspec)
{
	HevImpathyServerTLSHandler *self = HEV_IMPATHY_SERVER_TLS_HANDLER(object);
	HevImpathyServerTLSHandlerPrivate *priv =
		HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	switch(property_id)
	{
	case PROP_CHANNEL:
		priv->channel = g_value_dup_object(value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
					property_id, pspec);
		break;
	}
}

static void hev_impathy_server_tls_handler_class_init(HevImpathyServerTLSHandlerClass * klass)
{
	GObjectClass * obj_class = G_OBJECT_CLASS(klass);
	GParamSpec *pspec = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	obj_class->constructor = hev_impathy_server_tls_handler_constructor;
	obj_class->constructed = hev_impathy_server_tls_handler_constructed;
	obj_class->dispose = hev_impathy_server_tls_handler_dispose;
	obj_class->finalize = hev_impathy_server_tls_handler_finalize;
	obj_class->get_property = hev_impathy_server_tls_handler_get_property;
	obj_class->set_property = hev_impathy_server_tls_handler_set_property;

	pspec = g_param_spec_object("channel", "The TpChannel",
				"The tpChannel this handler is supposed to handle.",
				TP_TYPE_CHANNEL, G_PARAM_READWRITE |
				G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_CHANNEL, pspec);

	pspec = g_param_spec_object("certificate", "The HevImpathyTLSCertificate",
				"The HevImpathyTLSCertificate carried by the channel.",
				HEV_TYPE_IMPATHY_TLS_CERTIFICATE,
				G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_TLS_CERTIFICATE, pspec);

	pspec = g_param_spec_string("hostname", "The hostname",
				"The hostname the user is expecting to connect to.",
				NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_HOSTNAME, pspec);


	pspec = g_param_spec_boxed("reference-identities", "Reference Identities",
				"The server certificate should certify one of these identities",
				G_TYPE_STRV, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_REFERENCE_IDENTITIES, pspec);

	g_type_class_add_private(klass, sizeof(HevImpathyServerTLSHandlerPrivate));
}

static void tls_certificate_prepared_handler(GObject *source,
			GAsyncResult *res, gpointer user_data)
{
	HevImpathyServerTLSHandler *self = HEV_IMPATHY_SERVER_TLS_HANDLER(user_data);
	HevImpathyServerTLSHandlerPrivate *priv =
		HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);
	HevImpathyTLSCertificate *certificate = HEV_IMPATHY_TLS_CERTIFICATE(source);
	GError *error = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	hev_impathy_tls_certificate_prepare_finish(certificate, res, &error);
	if(NULL != error)
	{
		g_simple_async_result_set_from_error(priv->async_init_res, error);
		g_error_free(error);
	}

	g_simple_async_result_complete_in_idle(priv->async_init_res);
	tp_clear_object(&priv->async_init_res);
}

static void hev_impathy_server_tls_handler_init_async(GAsyncInitable *initable,
			gint io_priority, GCancellable *cancellable,
			GAsyncReadyCallback callback, gpointer user_data)
{
	HevImpathyServerTLSHandler *self = HEV_IMPATHY_SERVER_TLS_HANDLER(initable);
	HevImpathyServerTLSHandlerPrivate *priv =
		HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);
	GHashTable *properties = NULL;
	const gchar *cert_object_path = NULL;
	const gchar *hostname = NULL;
	const gchar * const *identities = NULL;
	const gchar *bus_name = NULL;
	TpDBusDaemon *dbus_daemon = NULL;
	GError *error = NULL;
	/*
	 * Used when channel doesn't implement ReferenceIdentities. A GStrv
	 * with [0] the hostname, and [1] a NULL terminator.
	 */
	gchar *default_identities[2];

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_if_fail(NULL != priv->channel);

	priv->async_init_res = g_simple_async_result_new(G_OBJECT(self),
				callback, user_data, hev_impathy_server_tls_handler_new_async);
	properties = tp_channel_borrow_immutable_properties(priv->channel);
	
	hostname = (const gchar *)tp_asv_get_string(properties,
				TP_PROP_CHANNEL_TYPE_SERVER_TLS_CONNECTION_HOSTNAME);
	priv->hostname = g_strdup(hostname);

	identities = (const gchar * const *)tp_asv_get_strv(properties,
				TP_PROP_CHANNEL_TYPE_SERVER_TLS_CONNECTION_REFERENCE_IDENTITIES);

	/*
	 * If the channel doesn't implement the ReferenceIdentities parameter
	 * then fallback to the hostname.
	 */
	if(NULL == identities)
	{
		default_identities[0] = (gchar *)hostname;
		default_identities[1] = NULL;

		identities = (const gchar **)default_identities;
	}

	priv->reference_identities = g_strdupv((gchar **)identities);

	cert_object_path = (const gchar *)tp_asv_get_object_path(properties,
				TP_IFACE_CHANNEL_TYPE_SERVER_TLS_CONNECTION ".ServerCertificate");
	bus_name = tp_proxy_get_bus_name(TP_PROXY(priv->channel));
	dbus_daemon = tp_proxy_get_dbus_daemon(TP_PROXY(priv->channel));

	priv->certificate = hev_impathy_tls_certificate_new(dbus_daemon, bus_name,
				cert_object_path, &error);

	if(NULL != error)
	{
		g_simple_async_result_set_from_error(priv->async_init_res, error);
		g_simple_async_result_complete_in_idle(priv->async_init_res);

		g_error_free(error);
		tp_clear_object(&priv->async_init_res);

		return;
	}

	hev_impathy_tls_certificate_prepare_async(
				HEV_IMPATHY_TLS_CERTIFICATE(priv->certificate),
				tls_certificate_prepared_handler, self);
}

static gboolean hev_impathy_server_tls_handler_init_finish(GAsyncInitable *intiable,
			GAsyncResult *res, GError **error)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(res),
					error))
	  return FALSE;

	return TRUE;
}

static void hev_async_initable_iface_init(GAsyncInitableIface *iface)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	iface->init_async = hev_impathy_server_tls_handler_init_async;
	iface->init_finish = hev_impathy_server_tls_handler_init_finish;
}

static void hev_impathy_server_tls_handler_init(HevImpathyServerTLSHandler * self)
{
	HevImpathyServerTLSHandlerPrivate * priv = HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);
}

void hev_impathy_server_tls_handler_new_async(TpChannel *channel,
			GAsyncReadyCallback callback, gpointer user_data)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_if_fail(TP_IS_CHANNEL(channel));

	g_async_initable_new_async(HEV_TYPE_IMPATHY_SERVER_TLS_HANDLER,
				G_PRIORITY_DEFAULT, NULL, callback, user_data,
				"channel", channel, NULL);
}

GObject * hev_impathy_server_tls_handler_new_finish(GAsyncResult *res,
			GError **error)
{
	GObject *object = NULL, *source_object = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	source_object = g_async_result_get_source_object(res);

	object = g_async_initable_new_finish(G_ASYNC_INITABLE(source_object),
				res, error);
	g_object_unref(source_object);

	return object;
}

GObject * hev_impathy_server_tls_handler_get_certificate(
			HevImpathyServerTLSHandler *self)
{
	HevImpathyServerTLSHandlerPrivate *priv = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_val_if_fail(HEV_IS_IMPATHY_SERVER_TLS_HANDLER(self),
				NULL);
	priv = HEV_IMPATHY_SERVER_TLS_HANDLER_GET_PRIVATE(self);

	return priv->certificate;
}

