/*
 ============================================================================
 Name        : hev-impathy-tls-certificate.c
 Author      : Heiher <admin@heiher.info>
 Version     : 0.0.1
 Copyright   : Copyright (C) 2011 everyone.
 Description : 
 ============================================================================
 */

#include <telepathy-glib/dbus.h>
#include <telepathy-glib/util.h>
#include <telepathy-glib/gtypes.h>
#include <telepathy-glib/interfaces.h>
#include <telepathy-glib/telepathy-glib.h>

#include "hev-impathy-tls-certificate.h"

enum
{
	PROP_CERT_TYPE = 1,
	PROP_CERT_DATA,
	PROP_STATE,
	N_PROPERTIES
};

#define HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), HEV_TYPE_IMPATHY_TLS_CERTIFICATE, HevImpathyTLSCertificatePrivate))

typedef struct _HevImpathyTLSCertificatePrivate HevImpathyTLSCertificatePrivate;

struct _HevImpathyTLSCertificatePrivate
{
	GSimpleAsyncResult *async_prepare_res;

	/* TLSCertificate properties */
	gchar *cert_type;
	GPtrArray *cert_data;
	guint state;

	gboolean is_prepared;
};

G_DEFINE_TYPE(HevImpathyTLSCertificate, hev_impathy_tls_certificate, TP_TYPE_PROXY);

static void hev_impathy_tls_certificate_dispose(GObject * obj)
{
	HevImpathyTLSCertificate * self = HEV_IMPATHY_TLS_CERTIFICATE(obj);
	HevImpathyTLSCertificatePrivate * priv = HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	G_OBJECT_CLASS(hev_impathy_tls_certificate_parent_class)->dispose(obj);
}

static void hev_impathy_tls_certificate_finalize(GObject * obj)
{
	HevImpathyTLSCertificate * self = HEV_IMPATHY_TLS_CERTIFICATE(obj);
	HevImpathyTLSCertificatePrivate * priv = HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(priv->cert_type)
	{
		g_free(priv->cert_type);
		priv->cert_type = NULL;
	}

	if(priv->cert_data)
	{
		tp_clear_boxed(TP_ARRAY_TYPE_UCHAR_ARRAY_LIST, &priv->cert_data);
		priv->cert_data = NULL;
	}

	G_OBJECT_CLASS(hev_impathy_tls_certificate_parent_class)->finalize(obj);
}

static void hev_impathy_tls_certificate_get_property(GObject *object,
			guint property_id, GValue *value, GParamSpec *pspec)
{
	HevImpathyTLSCertificate *self = HEV_IMPATHY_TLS_CERTIFICATE(object);
	HevImpathyTLSCertificatePrivate * priv = HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);

	switch(property_id)
	{
	case PROP_CERT_TYPE:
		g_value_set_string(value, priv->cert_type);
		break;
	case PROP_CERT_DATA:
		g_value_set_boxed(value, priv->cert_data);
		break;
	case PROP_STATE:
		g_value_set_uint(value, priv->state);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,
					property_id, pspec);
		break;
	}
}

static GObject * hev_impathy_tls_certificate_constructor(GType type, guint n, GObjectConstructParam * param)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);
	return G_OBJECT_CLASS(hev_impathy_tls_certificate_parent_class)->constructor(type, n, param);
}

static void hev_impathy_tls_certificate_constructed(GObject * obj)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	G_OBJECT_CLASS(hev_impathy_tls_certificate_parent_class)->constructed(obj);
}

static void hev_impathy_tls_certificate_class_init(HevImpathyTLSCertificateClass * klass)
{
	GObjectClass * obj_class = G_OBJECT_CLASS(klass);
	TpProxyClass *proxy_class = TP_PROXY_CLASS(klass);
	GParamSpec *pspec = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	obj_class->constructor = hev_impathy_tls_certificate_constructor;
	obj_class->constructed = hev_impathy_tls_certificate_constructed;
	obj_class->dispose = hev_impathy_tls_certificate_dispose;
	obj_class->finalize = hev_impathy_tls_certificate_finalize;
	obj_class->get_property = hev_impathy_tls_certificate_get_property;

	proxy_class->interface =
		g_quark_from_static_string(
					"org.freedesktop.Telepathy.Authentication.TLSCertificate");
	proxy_class->must_have_unique_name = TRUE;

	pspec = g_param_spec_string("cert-type", "Certificate type",
				"The type of this certificate.",
				NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_CERT_TYPE, pspec);

	pspec = g_param_spec_boxed("cert-data", "Certificate chain data",
				"The raw DER-encoded certificate chain data.",
				TP_ARRAY_TYPE_UCHAR_ARRAY_LIST,
				G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_CERT_DATA, pspec);

	pspec = g_param_spec_uint("state", "State",
				"The state of this certificate.",
				0, 2, 0,
				G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
	g_object_class_install_property(obj_class, PROP_STATE, pspec);

	g_type_class_add_private(klass, sizeof(HevImpathyTLSCertificatePrivate));
}

static void hev_impathy_tls_certificate_init(HevImpathyTLSCertificate * self)
{
	HevImpathyTLSCertificatePrivate * priv = HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);
}

GObject * hev_impathy_tls_certificate_new(TpDBusDaemon *dbus_daemon,
			const gchar *bus_name, const gchar *object_path,
			GError **error)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(!tp_dbus_check_valid_bus_name(bus_name,
					TP_DBUS_NAME_TYPE_UNIQUE, error))
	  return NULL;
	if(!tp_dbus_check_valid_object_path(object_path,
					error))
	  return NULL;

	return g_object_new(HEV_TYPE_IMPATHY_TLS_CERTIFICATE,
				"dbus-daemon", dbus_daemon,
				"bus-name", bus_name,
				"object-path", object_path,
				NULL);
}

static void tls_certificate_got_all_handler(TpProxy *proxy,
			GHashTable *properties, const GError *error,
			gpointer user_data, GObject *weak_object)
{
	HevImpathyTLSCertificate *self = HEV_IMPATHY_TLS_CERTIFICATE(weak_object);
	HevImpathyTLSCertificatePrivate *priv =
		HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);
	GPtrArray *cert_data = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(NULL != error)
	{
		g_simple_async_result_set_from_error(priv->async_prepare_res, error);
		g_simple_async_result_complete(priv->async_prepare_res);
		tp_clear_object(&priv->async_prepare_res);

		return;
	}

	priv->cert_type = g_strdup(tp_asv_get_string(properties,
					"CertificateType"));
	priv->state = tp_asv_get_uint32(properties, "State", NULL);

	cert_data = tp_asv_get_boxed(properties, "CertificateChainData",
				TP_ARRAY_TYPE_UCHAR_ARRAY_LIST);
	g_assert(NULL != cert_data);
	priv->cert_data = g_boxed_copy(TP_ARRAY_TYPE_UCHAR_ARRAY_LIST,
				cert_data);

	priv->is_prepared = TRUE;

	g_simple_async_result_complete(priv->async_prepare_res);
	tp_clear_object(&priv->async_prepare_res);
}

void hev_impathy_tls_certificate_prepare_async(HevImpathyTLSCertificate *self,
			GAsyncReadyCallback callback, gpointer user_data)
{
	HevImpathyTLSCertificatePrivate *priv = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_if_fail(HEV_IS_IMPATHY_TLS_CERTIFICATE(self));
	priv = HEV_IMPATHY_TLS_CERTIFICATE_GET_PRIVATE(self);

	/* emit an error if we're already preparing the object */
	if(NULL != priv->async_prepare_res)
	{
		g_simple_async_report_error_in_idle(G_OBJECT(self),
					callback, user_data,
					G_IO_ERROR, G_IO_ERROR_PENDING,
					"%s",
					"Prepare operation already in progress"
					" on the TLS certificate.");
		return;
	}

	/* if the object is already prepared, just complete in idle */
	if(priv->is_prepared)
	{
		tp_simple_async_report_success_in_idle(G_OBJECT(self),
					callback, user_data,
					hev_impathy_tls_certificate_prepare_async);
		return;
	}

	priv->async_prepare_res = g_simple_async_result_new(G_OBJECT(self),
				callback, user_data,
				hev_impathy_tls_certificate_prepare_async);

	/* call GetAll() on the certificate */
	tp_cli_dbus_properties_call_get_all(self,
				-1, TP_IFACE_AUTHENTICATION_TLS_CERTIFICATE,
				tls_certificate_got_all_handler, NULL, NULL,
				G_OBJECT(self));
}

gboolean hev_impathy_tls_certificate_prepare_finish(HevImpathyTLSCertificate *self,
			GAsyncResult *res, GError **error)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(res),
					error))
	  return FALSE;

	return TRUE;
}

typedef void (*hev_cli_authentication_tls_certificate_callback_for_accept)(TpProxy *proxy,
			const GError *error, gpointer user_data, GObject *weak_object);

static void hev_cli_authentication_tls_certificate_invoke_callback_accept(TpProxy *proxy,
			GError *error, GValueArray *args, GCallback generic_callback,
			gpointer user_data, GObject *weak_object)
{
	hev_cli_authentication_tls_certificate_callback_for_accept callback =
		(hev_cli_authentication_tls_certificate_callback_for_accept)generic_callback;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(NULL != error)
	{
		callback(proxy, error, user_data, weak_object);
		g_error_free(error);
		return;
	}

	callback(proxy, error, user_data, weak_object);
	
	if(NULL != args)
	  g_value_array_free(args);
}

static void hev_cli_authentication_tls_certificate_collect_callback_accept(DBusGProxy *proxy,
			DBusGProxyCall *call, gpointer user_data)
{
	GError *error = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	dbus_g_proxy_end_call(proxy, call, &error, G_TYPE_INVALID);
	tp_proxy_pending_call_v0_take_results(user_data, error, NULL);
}

static TpProxyPendingCall * hev_cli_authentication_tls_certificate_call_accept(
			TpProxy *proxy, gint timeout_ms,
			hev_cli_authentication_tls_certificate_callback_for_accept callback,
			gpointer user_data, GDestroyNotify destroy, GObject *weak_object)
{
	GError *error = NULL;
	GQuark interface = TP_IFACE_QUARK_AUTHENTICATION_TLS_CERTIFICATE;
	DBusGProxy *iface = NULL;
	TpProxyPendingCall *data = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	iface = tp_proxy_borrow_interface_by_id(proxy, interface,
				&error);
	if(NULL == iface)
	{
		if(NULL != callback)
		  callback(proxy, error, user_data, weak_object);

		if(NULL != destroy)
		  destroy(user_data);

		g_error_free(error);
		return NULL;
	}

	if(NULL == callback)
	{
		dbus_g_proxy_call_no_reply(iface, "Accept",
					G_TYPE_INVALID);
		return NULL;
	}

	data = tp_proxy_pending_call_v0_new(proxy,
				interface, "Accept", iface,
				hev_cli_authentication_tls_certificate_invoke_callback_accept,
				G_CALLBACK (callback), user_data, destroy,
				weak_object, FALSE);
	tp_proxy_pending_call_v0_take_pending_call(data,
				dbus_g_proxy_begin_call_with_timeout (iface,
					"Accept",
					hev_cli_authentication_tls_certificate_collect_callback_accept,
					data, tp_proxy_pending_call_v0_completed,
					timeout_ms, G_TYPE_INVALID));
	return data;
}

static void cert_proxy_accept_handler(TpProxy *proxy, const GError *error,
			gpointer user_data, GObject *weak_object)
{
	GSimpleAsyncResult *accept_result = user_data;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(NULL != error)
	{
		g_simple_async_result_set_from_error(accept_result,
					error);
	}

	g_simple_async_result_complete(accept_result);
}

void hev_impathy_tls_certificate_accept_async(HevImpathyTLSCertificate *self,
			GAsyncReadyCallback callback, gpointer user_data)
{
	GSimpleAsyncResult *accept_result = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_if_fail(HEV_IS_IMPATHY_TLS_CERTIFICATE(self));

	accept_result = g_simple_async_result_new(G_OBJECT(self),
				callback, user_data,
				hev_impathy_tls_certificate_accept_async);

	hev_cli_authentication_tls_certificate_call_accept(TP_PROXY(self),
				-1, cert_proxy_accept_handler, accept_result,
				g_object_unref, G_OBJECT(self));
}

gboolean hev_impathy_tls_certificate_accept_finish(HevImpathyTLSCertificate *self,
			GAsyncResult *res, GError **error)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(res),
					error))
	  return FALSE;

	return TRUE;
}

typedef void (*hev_cli_authentication_tls_certificate_callback_for_reject) (TpProxy *proxy,
			const GError *error, gpointer user_data, GObject *weak_object);

static void hev_cli_authentication_tls_certificate_invoke_callback_reject(TpProxy *proxy,
			GError *error, GValueArray *args, GCallback generic_callback,
			gpointer user_data, GObject *weak_object)
{
	hev_cli_authentication_tls_certificate_callback_for_reject callback =
		(hev_cli_authentication_tls_certificate_callback_for_reject)generic_callback;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(NULL != error)
	{
		callback(proxy, error, user_data, weak_object);
		g_error_free(error);
		return;
	}
	
	callback(proxy, error, user_data, weak_object);

	if(NULL != args)
	  g_value_array_free(args);
}

static void hev_cli_authentication_tls_certificate_collect_callback_reject(DBusGProxy *proxy,
			DBusGProxyCall *call, gpointer user_data)
{
	GError *error = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	dbus_g_proxy_end_call(proxy, call, &error,
				G_TYPE_INVALID);
	tp_proxy_pending_call_v0_take_results(user_data, error,
				NULL);
}

static TpProxyPendingCall * hev_cli_authentication_tls_certificate_call_reject(TpProxy *proxy,
			gint timeout_ms, const GPtrArray *in_rejections,
			hev_cli_authentication_tls_certificate_callback_for_reject callback,
			gpointer user_data, GDestroyNotify destroy, GObject *weak_object)
{
	GError *error = NULL;
	GQuark interface = TP_IFACE_QUARK_AUTHENTICATION_TLS_CERTIFICATE;
	DBusGProxy *iface = NULL;
    TpProxyPendingCall *data = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	iface = tp_proxy_borrow_interface_by_id(proxy, interface,
				&error);
	if(NULL == iface)
	{
		if(NULL != callback)
		  callback(proxy, error, user_data, weak_object);

		if(NULL != destroy)
		  destroy(user_data);

		g_error_free(error);
		return NULL;
	}

	if(NULL == callback)
	{
		dbus_g_proxy_call_no_reply(iface, "Reject",
			(dbus_g_type_get_collection("GPtrArray",
				(dbus_g_type_get_struct("GValueArray",
					G_TYPE_UINT, G_TYPE_STRING,
					(dbus_g_type_get_map("GHashTable",
						G_TYPE_STRING, G_TYPE_VALUE)),
					G_TYPE_INVALID)))),
			in_rejections, G_TYPE_INVALID);
		return NULL;
	}

	data = tp_proxy_pending_call_v0_new(proxy, interface,
				"Reject", iface,
				hev_cli_authentication_tls_certificate_invoke_callback_reject,
				G_CALLBACK(callback), user_data, destroy, weak_object, FALSE);
	tp_proxy_pending_call_v0_take_pending_call(data, dbus_g_proxy_begin_call_with_timeout(iface,
					"Reject", hev_cli_authentication_tls_certificate_collect_callback_reject,
					data, tp_proxy_pending_call_v0_completed, timeout_ms,
					(dbus_g_type_get_collection("GPtrArray",
						(dbus_g_type_get_struct("GValueArray", G_TYPE_UINT, G_TYPE_STRING,
							(dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)),
							G_TYPE_INVALID)))), in_rejections, G_TYPE_INVALID));

	return data;
}

static const gchar * reject_reason_get_dbus_error(TpTLSCertificateRejectReason reason)
{
	const gchar *retval = NULL;

	switch (reason)
	{
	case TP_TLS_CERTIFICATE_REJECT_REASON_UNTRUSTED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_UNTRUSTED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_EXPIRED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_EXPIRED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_NOT_ACTIVATED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_NOT_ACTIVATED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_FINGERPRINT_MISMATCH:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_FINGERPRINT_MISMATCH);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_HOSTNAME_MISMATCH:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_HOSTNAME_MISMATCH);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_SELF_SIGNED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_SELF_SIGNED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_REVOKED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_REVOKED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_INSECURE:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_INSECURE);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_LIMIT_EXCEEDED:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_LIMIT_EXCEEDED);
		break;
	case TP_TLS_CERTIFICATE_REJECT_REASON_UNKNOWN:
	default:
		retval = tp_error_get_dbus_name (TP_ERROR_CERT_INVALID);
		break;
	}

	return retval;
}

static GPtrArray * build_rejections_array(TpTLSCertificateRejectReason reason,
			GHashTable *details)
{
	GPtrArray *retval = NULL;
	GValueArray *rejection = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	retval = g_ptr_array_new();
	rejection = tp_value_array_build(3,
				G_TYPE_UINT, reason,
				G_TYPE_STRING, reject_reason_get_dbus_error(reason),
				TP_HASH_TYPE_STRING_VARIANT_MAP, details,
				NULL);

	g_ptr_array_add(retval, rejection);

	return retval;
}

static void cert_proxy_reject_handler(TpProxy *proxy,
			const GError *error, gpointer user_data,
			GObject *weak_object)
{
	GSimpleAsyncResult *reject_result = user_data;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(NULL != error)
	  g_simple_async_result_set_from_error(reject_result, error);

	g_simple_async_result_complete(reject_result);
}

void hev_impathy_tls_certificate_reject_async(HevImpathyTLSCertificate *self,
			TpTLSCertificateRejectReason reason, GHashTable *details,
			GAsyncReadyCallback callback, gpointer user_data)
{
	GPtrArray *rejections = NULL;
	GSimpleAsyncResult *reject_result = NULL;

	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	g_return_if_fail(HEV_IS_IMPATHY_TLS_CERTIFICATE(self));

	rejections = build_rejections_array(reason, details);
	reject_result = g_simple_async_result_new(G_OBJECT(self),
				callback, user_data,
				hev_impathy_tls_certificate_reject_async);

	hev_cli_authentication_tls_certificate_call_reject(TP_PROXY(self),
				-1, rejections, cert_proxy_reject_handler,
				reject_result, g_object_unref, G_OBJECT(self));
	
	tp_clear_boxed(TP_ARRAY_TYPE_TLS_CERTIFICATE_REJECTION_LIST,
				&rejections);
}

gboolean hev_impathy_tls_certificate_reject_finish(HevImpathyTLSCertificate *self,
			GAsyncResult *res, GError **error)
{
	g_debug("%s:%d[%s]", __FILE__, __LINE__, __FUNCTION__);

	if(g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(res),
					error))
	  return FALSE;

	return TRUE;
}

