/*
Copyright (c) 2012, Ilya Arefiev <arefiev.id@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.
 * Neither the name of the author nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DATA_H
#define DATA_H

#include "common.h"

/** represents node of the list object.
 *
 * list is signle-linked
 * @see LIST_ROOT
 */
typedef struct LIST_NODE_STRUCT {
	void* data;						/**< pointer to data location */
	size_t sz;						/**< size of data. used by #d_list_get */
	struct LIST_NODE_STRUCT* next;	/**< pointer to the next node */
} LIST_NODE;

/** represents root node of the list.
 *
 * like std::list object
 * @see LIST_NODE
 */
typedef struct LIST_ROOT_STRUCT {
	LIST_NODE* head;				/**< head of the list */
	LIST_NODE* tail;				/**< tail of the list */
	int count;						/**< count of nodes in this list */
} LIST_ROOT;

bool d_list_init(LIST_ROOT*);
bool d_list_deinit(LIST_ROOT*);
bool d_list_insert(LIST_ROOT*, const int, const void*, const size_t);
bool d_list_insert_head(LIST_ROOT*, const void*, const size_t);
bool d_list_insert_tail(LIST_ROOT*, const void*, const size_t);
bool d_list_remove(LIST_ROOT*, const int);
bool d_list_remove_head(LIST_ROOT*);
bool d_list_remove_tail(LIST_ROOT*);
bool d_list_get(LIST_ROOT*, const int, void*, size_t);
bool d_list_get_head(LIST_ROOT*, void*, size_t);
bool d_list_get_tail(LIST_ROOT*, void*, size_t);

/** represents node of the queue.
 *
 * alias for the LIST_NODE due to queue
 * is the list with access only to the first
 * or last node
 *
 * @see LIST_NODE
 */
typedef LIST_NODE QUEUE_NODE;

/** represents root node of the queue.
 *
 * like std::queue object.
 * alias for the LIST_ROOT due to queue
 * is the list with access  only to the first
 * or last node
 *
 * @see LIST_ROOT
 */
typedef LIST_ROOT QUEUE_ROOT;

bool d_queue_init(QUEUE_ROOT*);
bool d_queue_deinit(QUEUE_ROOT*);
bool d_queue_push(QUEUE_ROOT*, const void*, const size_t);
bool d_queue_pop(QUEUE_ROOT*, void*, size_t);

#endif
