/*
Copyright (c) 2012, Ilya Arefiev <arefiev.id@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.
 * Neither the name of the author nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "common.h"
#include "utils.h"
#include "log.h"
#include "datetime.h"
#include "net.h"
#include "data.h"

int main(int argc, char** argv)
{
	LOGLEVEL llevel;
#ifdef _DEBUG
	llevel = LOGLEVEL_DEBUG;
#else
	llevel = LOGLEVEL_LOG;
#endif
	if ( !l_init( "debug.log", llevel ) )
	{
		fprintf( stderr, "-- unable to initialize log\n" );
	}

/* #ifdef _DEBUG */
	l_printf( LOGLEVEL_LOG, "%s:%d %s: startup rev %s\n",
		__FILE__, __LINE__, __func__,
		_VERSION
		);
/* #endif */

	printf("tsync rev %s built on %s %s\n", _VERSION, __DATE__, __TIME__);
	printf("Copyright (c) 2012, Ilya Arefiev <arefiev.id@gmail.com>\n");
	printf("licensed under the terms of 'BSD New' license\n");
	printf("\n");


	struct timeval tv;
	struct timezone tz;
	memset( &tv, 0, sizeof(struct timeval) );
	memset( &tz, 0, sizeof(struct timezone) );

	if ( gettimeofday( &tv, &tz ) != 0 )
	{
		l_printf( LOGLEVEL_ERROR, "gettimeofday error %d: %s\n",
			errno, strerror(errno) );

		return 1;
	}

	char tbuf[BUFSZ];
	memset( tbuf, 0, sizeof(tbuf) );

#ifdef _DEBUG
	printf("tv:\n");
	u_hexout( (unsigned char*)&tv, sizeof(struct timeval) );
#endif

	if ( dt_datetime_sprintf( tbuf, sizeof(tbuf), &tv, true, DT_FORMAT_ALL ) )
	{
		printf( "localtime:\t%s\n", tbuf );
	}


	if ( dt_datetime_sprintf( tbuf, sizeof(tbuf), &tv, false, DT_FORMAT_ALL ) )
	{
		printf( "gmtime:\t%s\n", tbuf );
	}

	/* LIST_ROOT list; */
	/* if ( d_list_init(&list) ) */
	/* { */
	/* 	d_list_insert_head( &list, tbuf, sizeof(tbuf) ); */
	/* 	d_list_insert_head( &list, tbuf, sizeof(tbuf) ); */
	/* 	d_list_insert_tail( &list, tbuf, sizeof(tbuf) ); */
	/* 	d_list_insert( &list, 1, tbuf, sizeof(tbuf) ); */
	/* 	d_list_insert( &list, -1, tbuf, sizeof(tbuf) ); */
	/* 	d_list_insert( &list, 10, tbuf, sizeof(tbuf) ); */

	/* 	char dbuf[BUFSZ]; */
	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get_head( &list, dbuf, sizeof(dbuf) ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get_tail( &list, dbuf, sizeof(dbuf) ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get( &list, 1, dbuf, sizeof(dbuf) ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get( &list, 1, dbuf, sizeof(dbuf)-100 ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get( &list, 1, dbuf, sizeof(dbuf)+100 ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get( &list, -1, dbuf, sizeof(dbuf) ); */

	/* 	memset( dbuf, 0, sizeof(dbuf) ); */
	/* 	d_list_get( &list, 10, dbuf, sizeof(dbuf) ); */

	/* 	d_list_remove_head( &list ); */
	/* 	d_list_remove_tail( &list ); */
	/* 	d_list_remove( &list, 1 ); */
	/* 	d_list_remove( &list, -1 ); */
	/* 	d_list_remove( &list, 10 ); */
	/* } */

	/* d_list_deinit(&list); */

	QUEUE_ROOT queue;
	if ( d_queue_init( &queue ) )
	{
		d_queue_push( &queue, tbuf, sizeof(tbuf) );
		d_queue_push( &queue, tbuf, sizeof(tbuf) );
		d_queue_push( &queue, tbuf, sizeof(tbuf) );

		char dbuf[BUFSZ];
		memset( dbuf, 0, sizeof(dbuf) );
		d_queue_pop( &queue, dbuf, sizeof(dbuf) );

		memset( dbuf, 0, sizeof(dbuf) );
		d_queue_pop( &queue, dbuf, sizeof(dbuf)-100 );

		memset( dbuf, 0, sizeof(dbuf) );
		d_queue_pop( &queue, dbuf, sizeof(dbuf)+100 );

		memset( dbuf, 0, sizeof(dbuf) );
		d_queue_pop( &queue, dbuf, sizeof(dbuf) );

		memset( dbuf, 0, sizeof(dbuf) );
		d_queue_pop( &queue, dbuf, sizeof(dbuf) );

		d_queue_push( &queue, tbuf, sizeof(tbuf) );
		d_queue_push( &queue, tbuf, sizeof(tbuf) );
		d_queue_push( &queue, tbuf, sizeof(tbuf) );
	}

	d_queue_deinit( &queue );
	
/* #ifdef _DEBUG */
	l_printf( LOGLEVEL_LOG, "%s:%d %s: shutdown\n",
		__FILE__, __LINE__, __func__
		);
/* #endif */
	
	l_deinit();
	return 0;
}
