/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_ARCH_DEBUG_H
#define HiME_ARCH_DEBUG_H

#include "hime-arch-types.h"
#include <assert.h>

#ifdef __cplusplus
extern "C"
{
#endif

extern void HiME_Trace(const char *fmt, ...);
extern void HiME_TimeTrace(const char *fmt, ...);
extern void HiME_InitializeDebug();
extern void HiME_ShutdownDebug();

extern hime_bool_t HiME_AssertionBreak(const char* msg, const char* file,
    int line, const char *func);

#define HiME_FILE __FILE__
#define HiME_LINE __LINE__

#if (defined(_MSC_VER) && _MSC_VER >= 1400)
    #define HiME_BREAK() __debugbreak()
    #define HiME_FUNC __FUNCTION__
#else
    #define HiME_BREAK() assert(0)
    #define HiME_FUNC __func__
#endif

#define HiME_ASSUME(A) \
    do \
    { \
        if (!(A) && HiME_AssertionBreak(#A, HiME_FILE, HiME_LINE, HiME_FUNC)) \
        { \
            HiME_BREAK(); \
        } \
    } while (0)


#ifdef HiME_ENABLE_ASSERT
    #define HiME_ASSERT(A) HiME_ASSUME(A)
#else
    #define HiME_ASSERT(A)
#endif

#ifdef __cplusplus
}
#endif

#endif /* HiME_ARCH_DEBUG_H */
