/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_ARCH_THREAD_H
#define HiME_ARCH_THREAD_H

#include "hime-arch-types.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum
{
    HiME_ThreadPriorityIdle     = 0,
    HiME_ThreadPriorityLow      = 1,
    HiME_ThreadPriorityNormal   = 2,
    HiME_ThreadPriorityHigh     = 3
} HiME_ThreadPriority;

typedef struct
{
    void* data;
} HiME_Thread;

typedef uint64_t HiME_ThreadID;

extern void HiME_CreateThread(HiME_Thread** dst, uint32_t (*proc)(void*),
    void* arg);
extern void HiME_DeleteThread(HiME_Thread** t);
extern void HiME_ExitThread(uint32_t val);
extern uint32_t HiME_JoinThread(HiME_Thread* t);
extern HiME_ThreadID HiME_GetThreadId(HiME_Thread* t);
extern HiME_ThreadID HiME_GetCurrentThreadId();
extern void HiME_SetThreadPriority(HiME_Thread* t, HiME_ThreadPriority pr);
extern void HiME_Sleep(uint32_t ms);

typedef struct
{
    void* data;
} HiME_Mutex;

HiME_Mutex* HiME_CreateMutex();
void HiME_DeleteMutex(HiME_Mutex** mutex);
void HiME_LockMutex(HiME_Mutex* mutex);
void HiME_UnlockMutex(HiME_Mutex* mutex);

typedef struct
{
    void* data;
} HiME_Semaphore;

HiME_Semaphore* HiME_CreateSemaphore();
void HiME_DeleteSemaphore(HiME_Semaphore** sem);
void HiME_SemaphoreDown(HiME_Semaphore* sem);
void HiME_SemaphoreUp(HiME_Semaphore* sem);

#ifdef __cplusplus
}
#endif

#endif /* HiME_ARCH_THREAD_H */
