/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-win32.h"
#include "hime-arch-debug.h"
#include "hime-arch-time.h"
#include <stdio.h>

void
HiME_Trace(const char *fmt, ...)
{
	char buff[1024];
	wchar_t wbuff[1024];
	int ss = 0;
	va_list lst;

	va_start(lst, fmt);
	_vsnprintf(buff + ss, 2047 - ss, fmt, lst);
	printf(buff);
	MultiByteToWideChar(CP_UTF8, 0, buff, -1, (LPWSTR)wbuff, 1024);
	OutputDebugStringW(wbuff);
	va_end(lst);
}


static uint64_t startTicks;
void HiME_TimeTrace(const char *fmt, ...)
{
	char buff[1024];
	wchar_t wbuff[1024];
	int ss = 0;
	va_list lst;

	va_start(lst, fmt);
	ss = sprintf(buff, "(% 8d) 0x%06x >> ",
		(uint32_t)(HiME_SystemTicks() - startTicks),
		(uint32_t)GetCurrentThreadId());
	_vsnprintf(buff + ss, 2047 - ss, fmt, lst);
	printf(buff);
	MultiByteToWideChar(CP_UTF8, 0, buff, -1, (LPWSTR)wbuff, 1024);
	OutputDebugStringW(wbuff);
	va_end(lst);
}


void HiME_InitializeDebug()
{
    startTicks = HiME_SystemTicks();
}


void HiME_ShutdownDebug()
{
}


hime_bool_t
HiME_AssertionBreak(const char* msg, const char* file, int line,
	const char *func)
{
	char buff[1024];
	int res;
    static hime_bool_t active = HiME_False;

    if (active)
        Sleep(INFINITE);

    active = HiME_True;

	_snprintf(buff, sizeof(buff) - 2,
		"Assertion\r\n%s\r\n"
		"failed at\r\n%s:%i, \'%s\'\r\n"
		"Break in to debbuger?", msg, file, file, func);
	HiME_Trace("%s", buff);

	res = MessageBoxA (0, buff, "Assertion failed",
				MB_ICONERROR | MB_YESNO );
    active = HiME_False;
	return res == IDYES ? HiME_True : HiME_False;
}
