/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-win32.h"
#include "hime-win32-errors.h"
#include "hime-io-exceptions.h"
#include <stdio.h>

#define HiME_MAX_MSG 1024
namespace HiME
{

void Win32ThrowLastError(const char* fmt, ...)
{
    DWORD err;
    char buff[HiME_MAX_MSG];
    char* s;
    wchar_t winmsg[512];
    char* ret;
    va_list lst;

    err = GetLastError();
    if (err == ERROR_SUCCESS)
        return;

    s = buff;
    strcpy(s, "win32 error: ");
    s += strlen(s);
            
    if (fmt)
    {
        va_start(lst, fmt);
        s += _vsnprintf(s, HiME_MAX_MSG - (s - buff) - 2, fmt, lst);
        va_end(lst);
        strcat(s, ": ");
        s += 2;
    }

    FormatMessageW(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                    NULL,
                    err,
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                    winmsg,
                    sizeof(winmsg) / sizeof(winmsg[0]) - 1,
                    NULL);
    OutputDebugStringW(winmsg);

    WideCharToMultiByte(CP_ACP, 0, winmsg, -1, (LPSTR)s, (int)(Exception::MAX_MSG - (s - buff) - 2), NULL, NULL);

    ret = strstr(buff, "\r\n");
    if (ret)
        *ret = 0;

    switch (err)
    {
    case ERROR_FILE_NOT_FOUND:
    case ERROR_PATH_NOT_FOUND:
        throw FileNotFound(buff, err);

    case ERROR_ACCESS_DENIED:
    case ERROR_SHARING_VIOLATION:
        throw AccessDenied(buff, err);
        break;

    default:
        throw Exception(buff, err);
        break;
    }
}

} // namespace HiME
