/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-win32.h"
#include "hime-arch-memory.h"

void HiME_InitializeSystemMemory()
{
}


void HiME_ShutdownSystemMemory()
{
}


void* HiME_SystemAlloc(size_t sz)
{
    return malloc(sz);
}


void* HiME_SystemRealloc(void* p, size_t sz)
{
	return realloc(p, sz);
}


void HiME_SystemFree(void* p)
{
	free(p);
}


// dlmalloc implementation:
#define MAX_SIZE_T           (~(size_t)0)
#define MFAIL                ((void*)(MAX_SIZE_T))

void* HiME_SystemVirtualAlloc(size_t size)
{
    void* ptr = VirtualAlloc(0, size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
    return (ptr != 0)? ptr: MFAIL;
}


void* HiME_SystemDirectVirtualAlloc(size_t size)
{
    void* ptr = VirtualAlloc(0, size, MEM_RESERVE | MEM_COMMIT | MEM_TOP_DOWN,
                                PAGE_READWRITE);
    return (ptr != 0)? ptr: MFAIL;
}


int HiME_SystemVirtualFree(void* ptr, size_t size)
{
    MEMORY_BASIC_INFORMATION minfo;
    char* cptr = (char*)ptr;
    while (size)
    {
        if (VirtualQuery(cptr, &minfo, sizeof(minfo)) == 0)
            return -1;

        if (minfo.BaseAddress != cptr || minfo.AllocationBase != cptr ||
            minfo.State != MEM_COMMIT || minfo.RegionSize > size)
        {
            return -1;
        }

        if (VirtualFree(cptr, 0, MEM_RELEASE) == 0)
            return -1;
        cptr += minfo.RegionSize;
        size -= minfo.RegionSize;
    }
    return 0;
}
