/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_ALGORITHM_H
#define HiME_ALGORITHM_H

#include "hime-memory.h"

namespace HiME
{

static const size_t NPOS = (size_t)-1;

// Finds the first occurrence of squence str2 in str1.
template<class T>
size_t findSequence(const T* str1, size_t n1, const T* str2, size_t n2)
{
    HiME_ASSERT(str1 && str2);

    size_t i, k;
    const T* p = str1;
    for (i = 0; i < n1; ++i)
    {
        const T* s1 = p;
        const T* s2 = str2;
        for (k = 0; i < n1 && k < n2 && *s1 == *s2; ++i, ++k)
        {
            ++s1;
            ++s2;
        }
        if (k == n2)
            return p - str1;
        ++p;
    }
    return NPOS;
}


// Finds the first occurrence of zero-terminated str2 in str1.
template<class T>
size_t findString(const T* str1, const T* str2)
{
    HiME_ASSERT(str1 && str2);

    const T* p = str1;
    while (*p)
    {
        const T* s1 = p;
        const T* s2 = str2;
        while (*s1 && *s2 && *s1 == *s2)
        {
            ++s1;
            ++s2;
        }
        if (!*s2)
            return p - str1;
        ++p;
    }
    return NPOS;
}


} // namespace HiME

#endif // HiME_ALGORITHM_H
