/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_FILE_MANAGER_H
#define HiME_FILE_MANAGER_H

#include "hime-no-copy.h"
#include "hime-file-path.h"

namespace HiME
{

namespace VFS
{
    class Driver;
    class Iterator;
    class Node;
}


class FileManager : public NoCopy
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    FileManager();
    ~FileManager();

    void registerDriver(const String& fs, VFS::Driver* drv);

    void mount(const FilePath& path, const String& fs, const String& location);
    void unmount(const FilePath& path);

    VFS::Node* open(const FilePath& path);
    void create(const FilePath& path, uint32_t flags);
    void remove(const FilePath& path);
    void rename(const FilePath& old_name, const FileName& new_name);
    VFS::Iterator* children(const FilePath& path);

    static void initialize();
    static void shutdown();
    static FileManager* instance();
};

} // namspace HiME

#endif // HiME_FILE_MANAGER_H
