/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_FILE_NAME_H
#define HiME_FILE_NAME_H

#include "hime-string.h"
#include "hime-io-exceptions.h"

namespace HiME
{

class FileName : private String
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    typedef String BaseType;
    using BaseType::clear;
    using BaseType::empty;
    using BaseType::size;
    using BaseType::byteSize;
    using BaseType::capacity;
    using BaseType::reserve;
    using BaseType::trim;
    using BaseType::resize;
    using BaseType::grow;
    using BaseType::front;
    using BaseType::back;
    using BaseType::erase;

    using BaseType::find;
    using BaseType::findFrom;

    using BaseType::operator==;
    using BaseType::operator!=;
    using BaseType::compare;

    FileName()
    {
    }

    FileName(const FileName& s)
    {
        BaseType::assign(s);
    }

    FileName(const String& s)
    {
        validate(s.data(), s.size());
        BaseType::assign(s);
    }

    FileName(const char* s)
    {
        validate(s, String::length(s));
        BaseType::assign(s);
    }

    ValueType operator[](size_t i) const
    {
        return BaseType::operator[](i);
    }

    const char* c_str() const
    {
        return BaseType::c_str();
    }

    const char* data() const
    {
        return BaseType::data();
    }

    const BaseType& str() const
    {
        return *this;
    }

    void replace(size_t pos, size_t old_num, const char* str, size_t new_num)
    {
        validate(str, new_num);
        BaseType::replace(pos, old_num, str, new_num);
    }

    void replace(size_t pos, size_t old_num, const char* str)
    {
        replace(pos, old_num, str, BaseType::length(str));
    }

    void fill(size_t pos, size_t old_num, char src, size_t new_num)
    {
        validate(&src, 1);
        BaseType::fill(pos, old_num, src, new_num);
    }

    void insert(size_t pos, char c)
    {
        validate(&c, 1);
        BaseType::insert(pos, c);
    }

    void append(const FileName& s, size_t src_pos = 0, size_t src_n = NPOS)
    {
        BaseType::append(s, src_pos, src_n);
    }

    void append(const char* s)
    {
        append(s, BaseType::length(s));
    }

    void append(const char* s, size_t n)
    {
        validate(s, n);
        BaseType::append(s, n);
    }

    void append(char c)
    {
        validate(&c, 1);
        BaseType::append(c);
    }

    FileName& operator+=(const FileName& s)
    {
        BaseType::append(s);
        return *this;
    }

    FileName& operator+=(const char* s)
    {
        append(s);
        return *this;
    }

    FileName& operator+=(char c)
    {
        append(c);
        return *this;
    }

protected:
    void validate();
    static void validate(const char* s, size_t n);
};

} // namspace HiME

#endif // HiME_FILE_NAME_H
