/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-hash.h"

extern "C"
{
    #include <md5.h>
    #include <sha1.h>
    #include <sha256.h>
}

namespace HiME
{

uint32_t adler32(const uint8_t* buff, size_t sz)
{
    const int MOD_ADLER = 65521;
    unsigned long a = 1, b = 0;
    size_t index;

    for (index = 0; index < sz; ++index)
    {
        a = (a + buff[index]) % MOD_ADLER;
        b = (b + a) % MOD_ADLER;
    }

    return (b << 16) | a;
}


void md5(uint8_t digest[16], const uint8_t* buff, size_t sz)
{
    context_md5_t cc;
    MD5Init(&cc);
    MD5Update(&cc, (unsigned char*)buff, (unsigned int)sz);
    MD5Final(digest, &cc);
}

void sha1(uint8_t digest[20], const uint8_t* buff, size_t sz)
{
    SHA1_Context cc;
    SHA1Init(&cc);
    SHA1Update(&cc, (unsigned char*)buff, (unsigned int)sz);
    SHA1Final(digest, &cc);
}

void sha256(uint8_t digest[32], const uint8_t* buff, size_t sz)
{
    context_sha256_t cc;
    sha256_starts(&cc);
    sha256_update(&cc, (uint8_t*)buff, (uint32_t)sz);
    sha256_finish(&cc, digest);
}

} // namespace HiME
