/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_JOB_H
#define HiME_JOB_H

#include "hime-semaphore.h"
#include "hime-sync-object.h"

namespace HiME
{

///
class Job : public SyncObject
{
    HiME_DEFINE_NEW_OPERATOR();

    friend class ThreadPool;
public:
    Job();
    virtual ~Job();

    // Returns some user-readable information about job.
    virtual const char* jobInfo() const;

    /// Starts executing a job. Normally it is called by thread pool.
    void startJob();

    /// Tells a job to interrupt and exit.
    inline void interrupt();

    /// Returns true if a job was interrupted during its run.
    inline bool interrupted() const;

    /// Returns true if a job is currently running.
    inline bool running() const;

    /// Wait until current run is finished.
    /// Returns immediately if a job was not running.
    virtual void wait();
    virtual void release();

protected:
    virtual void run() = 0;

private:
    Semaphore m_sync;
    bool m_interrupted;
    bool m_running;
};


void Job::interrupt()
{
    m_interrupted = true;
}

inline bool Job::interrupted() const
{
    return m_interrupted;
}

inline bool Job::running() const
{
    return m_running;
}

} // namespace HiME

#endif // HiME_JOB_H
