/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_MEMORY_H
#define HiME_MEMORY_H

#include "hime-core.h"

#ifdef __cplusplus
extern "C"
{
#endif

void* HiME_DebugAlloc(size_t sz, const char* file, int line);

void* HiME_DebugRealloc(void* p, size_t old_sz, size_t new_sz,
    const char* file, int line);

void HiME_DebugFree(void* p, size_t sz, const char* file, int line);

void* HiME_DebugVirtualAlloc(size_t sz, const char* file, int line);

void HiME_DebugVirtualFree(void* p, size_t sz);

char* HiME_DebugStrDup(const char* s, const char* file, int line);

void HiME_ReportLeaks();

void HiME_DebugSetMaxAllocSize(size_t sz);

void HiME_InitializeMemory();
void HiME_ShutdownMemory();

#define HiME_Alloc(SZ)                      HiME_DebugAlloc((SZ), HiME_FILE, HiME_LINE)
#define HiME_Realloc(PTR, OLD_SZ, NEW_SZ)   HiME_DebugRealloc((PTR), (OLD_SZ), (NEW_SZ), HiME_FILE, HiME_LINE)
#define HiME_Free(PTR, SZ)                  HiME_DebugFree((PTR), (SZ), HiME_FILE, HiME_LINE)
#define HiME_VirtualAlloc(SZ)               HiME_DebugVirtualAlloc(SZ, HiME_FILE, HiME_LINE)
#define HiME_VirtualFree(P, SZ)             HiME_DebugVirtualFree(P, SZ);
#define HiME_StrDup(S)                      HiME_DebugStrDup((S), HiME_FILE, HiME_LINE)

#ifdef __cplusplus
}
#endif

#endif /* HiME_MEMORY_H */
