/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_STATS_H
#define HiME_STATS_H

#include "hime-core.h"

namespace HiME
{
    namespace Stats
    {
        struct Counters
        {
            const char*         name;
            int64_t             count;
            int64_t             countPerFrame;
            int64_t             total;
            int64_t             amount;
            int64_t             amountPerFrame;
            int64_t             totalAmount;
        };

        enum StatID
        {
            Allocs,
            VirtualAllocs,
            Threads,
            Mutexes,
            MutexLocks,
            MutexUnlocks,
            Semaphores,
            SemaphoreUps,
            SemaphoreDowns,
            JobsRun,
            Strings,
            Vectors,
            FileHandles,
            BytesRead,
            BytesWritten,
            Stats_Max
        };

        void InitializeStats();
        void ShutdownStats();
        const Counters* HiME_GetStatistics(StatID id);
        void IncrementCount(StatID id);
        void DecrementCount(StatID id);
        void IncrementAmount(StatID id, int64_t n);
        void DecrementAmount(StatID id, int64_t n);
        void Report();
    } // namespace Stats
} // namespace HiME

#endif // HiME_STATS_H
