/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-thread-pool.h"

namespace HiME
{

ThreadPool::ThreadPool()
{
}


ThreadPool::~ThreadPool()
{
    for (size_t i = 0; i < m_pool.size(); ++i)
    {
        Runner* r = m_pool[i];

        // Ask thread to interrupts
        m_lock.lock();
        if (r->job)
            r->job->interrupt();
        m_lock.unlock();

        r->end.down();      // wait to finish current job
        HiME_ASSERT(r->job == 0);
        r->start.up();      // starting new job with NULL runnable signals to exit
        r->thread.join();

        m_lock.lock();
        delete r;
        m_pool[i] = 0;
        m_lock.unlock();
    }

    m_pool.clear();
}


void ThreadPool::start(Job* job, HiME_ThreadPriority pr)
{
    job->m_sync.down();     // wait previous run

    m_lock.lock();

    Runner* p = 0;
    for (size_t i = 0; i < m_pool.size(); ++i)
    {
        if (!m_pool[i]->job)
        {
            p = m_pool[i];
            p->end.down();
            break;
        }
    }

    if (!p)
    {
        p = new Runner;
        m_pool.append(p);
        p->pool = this;
        p->thread.create(startProc, p);
    }

    //p->lock.lock();
    job->m_interrupted = false;
    job->m_running = true;
    p->job = job;
    //p->lock.unlock();
    p->thread.setPriority(pr);
    p->start.up();

    m_lock.unlock();
}


uint32_t ThreadPool::startProc(void* p)
{
    Runner* t = (Runner*)p;
    t->pool->startProc(t);
    return 0;
}


void ThreadPool::startProc(Runner* p)
{
#ifdef HiME_THREAD_POOL_VERBOSE
    HiME_TimeTrace("thread pool: new thread '0x%x' created\n",
        p->thread.identifier());
#endif
    for (;;)
    {
        p->start.down();
        if (!p->job)
        {
#ifdef HiME_THREAD_POOL_VERBOSE
            HiME_TimeTrace("thread pool: thread '0x%x' ended\n",
                p->thread.identifier());
#endif
            HiME_ExitThread(0);
            break;
        }

        //const char* inf = p->job->jobInfo();
        //dbg::trace("thread pool: job '0x%x' %s started in thread '0x%x'\n", p->job, inf, p->thread.identifier());
        p->job->startJob();
        //dbg::trace("thread pool: job '0x%x' %s ended\n", p->job, inf);

        p->pool->m_lock.lock();
        p->job = NULL;
        p->pool->m_lock.unlock();
        p->end.up();
    }
}

} // namespace HiME
