/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_THREAD_H
#define HiME_THREAD_H

#include "hime-sync-object.h"

namespace HiME
{

///
class Thread : public SyncObject
{
public:
    Thread() :
        m_thread(NULL)
    {
    }

    ~Thread()
    {
        Stats::DecrementCount(Stats::Threads);
        HiME_DeleteThread(&m_thread);
    }

    void create(uint32_t (*proc)(void*), void* arg)
    {
        Stats::IncrementCount(Stats::Threads);
        HiME_CreateThread(&m_thread, proc, arg);
    }

    uint32_t join()
    {
        return HiME_JoinThread(m_thread);
    }

    HiME_ThreadID identifier()
    {
        return HiME_GetThreadId(m_thread);
    }

    void setPriority(HiME_ThreadPriority pr)
    {
        HiME_SetThreadPriority(m_thread, pr);
    }

    virtual void wait()
    {
        join();
    }

    virtual void release()
    {
    }

private:
    HiME_Thread* m_thread;
};

} // namespace HiME

#endif // HiME_THREAD_H
