/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-utf8.h"

namespace HiME
{

void UTF8ToUCS2(WideString& dst, const String& src)
{
    const unsigned char* s = (const unsigned char*)src.data();
    size_t n = src.size();
    unsigned char a, b, c;

    dst.reserve(dst.size() + n);

    while (true)
    {
        if ((n--) == 0 || (a = (unsigned char)*s++) == '\0')
            break;
        
        // 0x80
        if (a < 0x80)
        {
            dst += a;
            continue;
        }

        if ((n--) == 0 || (b = (unsigned char)*s++) == '\0')
            break;

        // b11100000
        if ((a & 0xe0) == 0xe0)
        {
            if ((n--) == 0 || (c = (unsigned char)*s++) == '\0')
                break;

            dst += (a & 0x0f) << 12 | (b & 0x3f) << 6  | (c & 0x3f);
            continue;
        }

        // b11000000
        if ((a & 0xc0) == 0xc0)
        {
            dst += (a & 0x1f) << 6 | (b & 0x3f);
            continue;
        }
    }
}


void UCS2ToUTF8(String& dst, const WideString& src)
{
    const wchar_t* s = src.data();
    size_t n = src.size();
    wchar_t a;

    dst.reserve(dst.size() + n * 3);

    while (true)
    {
        if ((n--) == 0 || (a = *s++) == '\0')
            break;

        if (a < 0x80)
        {
            dst += (char)a;
            continue;
        }

        if (a >= 0x80 && a < 0x800)
        {
            dst += (char)((a >> 6) | 0xc0);
            dst += (char)((a & 0x3f) | 0x80);
        }
    
        if (a >= 0x800 && a < 0xffff)
        {
            dst += (char)((a >> 12) | 0xe0);
            dst += (char)(((a >> 6) & 0x3f) | 0x80);
            dst += (char)((a & 0x3f) | 0x80);
        }
    }
}

} // namespace HiME
