/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_VFS_NODE_H
#define HiME_VFS_NODE_H

#include "hime-io.h"
#include "hime-file-name.h"
#include "hime-vfs-stream.h"

namespace HiME
{
namespace VFS
{

class Iterator;

class Node
{
    HiME_DEFINE_NEW_OPERATOR();

public:

    virtual ~Node() {}

    //
    // Directory operation.
    //
    virtual Node* open(const FileName& child) = 0;
    virtual void create(const FileName& child, uint32_t flags) = 0;
    virtual void remove(const FileName& child) = 0;
    virtual void rename(const FileName& old_name, const FileName& new_name) = 0;
    virtual Iterator* children() = 0;

    //
    //
    //
    virtual void stat(IO::Stat& dst) = 0;
    virtual const FileName& name() = 0;

    //
    // Data streams.
    //
    virtual size_t streams() = 0;
    virtual FileStream* stream(int i, IO::Access access) = 0;

    //
    // Attributes.
    //

protected:
    Node() {}
};

} // namespace VFS
} // namspace HiME

#endif // HiME_VFS_NODE_H
