/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_VFS_POSIX_H
#define HiME_VFS_POSIX_H

#include "hime-vfs-driver.h"
#include "hime-vfs-iterator.h"
#include "hime-vfs-node.h"
#include "hime-vfs-stream.h"

namespace HiME
{
namespace VFS
{


class PosixDriver : public Driver
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    PosixDriver();
    virtual ~PosixDriver();

    virtual Node* mount(const FilePath& mpoint, const String& nativePath);
};


class PosixNode : public Node
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    PosixNode(const FileName& name, const String& path);

    virtual Node* open(const FileName& child);
    virtual void create(const FileName& child, uint32_t flags);
    virtual void remove(const FileName& child);
    virtual void rename(const FileName& old_name, const FileName& new_name);
    virtual Iterator* children();
    virtual void stat(IO::Stat& dst);
    virtual const FileName& name();
    virtual size_t streams();
    virtual FileStream* stream(int i, IO::Access access);

private:
    FileName m_name;
    String m_path;
};


class PosixIterator : public Iterator
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    explicit PosixIterator(const String& path);
    virtual ~PosixIterator();

    virtual void current(FileName& dst);
    virtual bool more();
    virtual bool next();
    virtual void reset();

private:
    String m_path;
    void* m_dir;
    FileName m_current;
    bool m_more;
};


class PosixStream : public FileStream
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    PosixStream(const String& path,  IO::Access access);
    virtual ~PosixStream();

    virtual hime_lsize_t read(void* dst, hime_lsize_t bytes);
    virtual hime_lsize_t write(const void* src, hime_lsize_t bytes);
    virtual hime_lsize_t seek(hime_lpos_t offset, IO::SeekOrigin org);
    virtual hime_lsize_t tell();
    virtual hime_lsize_t size();

private:
    String m_path;
    void* m_file;
};

} // namspace VFS
} // namspace HiME

#endif // HiME_VFS_POSIX_H
