/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_VFS_WIN32_H
#define HiME_VFS_WIN32_H

#include "hime-vfs-driver.h"
#include "hime-vfs-iterator.h"
#include "hime-vfs-node.h"
#include "hime-vfs-stream.h"

namespace HiME
{
namespace VFS
{

class Win32Driver : public Driver
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    Win32Driver();
    virtual ~Win32Driver();

    virtual Node* mount(const FilePath& mpoint, const String& nativePath);
};


class Win32Node : public Node
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    Win32Node(const FileName& name, const WideString& path);

    virtual Node* open(const FileName& child);
    virtual void create(const FileName& child, uint32_t flags);
    virtual void remove(const FileName& child);
    virtual void rename(const FileName& old_name, const FileName& new_name);
    virtual Iterator* children();
    virtual void stat(IO::Stat& dst);
    virtual const FileName& name();
    virtual size_t streams();
    virtual FileStream* stream(int i, IO::Access access);

private:
    FileName m_name;
    WideString m_path;
};


class Win32Iterator : public Iterator
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    explicit Win32Iterator(const WideString& path);
    virtual ~Win32Iterator();

    virtual void current(FileName& dst);
    virtual bool more();
    virtual bool next();
    virtual void reset();

private:
    WideString m_path;
    void* m_hFind;
    bool m_more;
    FileName m_current;
};


class Win32Stream : public FileStream
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    Win32Stream(const WideString& path,  IO::Access access);
    virtual ~Win32Stream();

    virtual hime_lsize_t read(void* dst, hime_lsize_t bytes);
    virtual hime_lsize_t write(const void* src, hime_lsize_t bytes);
    virtual hime_lsize_t seek(hime_lpos_t offset, IO::SeekOrigin org);
    virtual hime_lsize_t tell();
    virtual hime_lsize_t size();

private:
    WideString m_path;
    void* m_hFile;
};

} // namspace VFS
} // namspace HiME

#endif // HiME_VFS_WIN32_H
