/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_POINT_H
#define HiME_POINT_H

#include "hime-core.h"
#include "hime-new.h"

namespace HiME
{

/**
 * 2D point coordinate class template.
 */
template<class T>
class Point
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    /**
     * Constructs a point at (0, 0) position.
     */
    Point();

    /**
     * Constructs a copy of a point object.
     */
    Point(const Point& point);

    /**
     * Constructs a point at (x, y) position.
     */
    Point(T x, T y);

    Point& operator=(const Point& p) const;
    Point& operator+=(const Point& p) const;
    Point& operator-=(const Point& p) const;

    Point operator+(const Point& p) const;
    Point operator-(const Point& p) const;

    T x, y;
};


/**
 * Defines a single precision floating point 2D point.
 */
typedef Point<float> Pointf;

/**
 * Defines a integer 2D point.
 */
typedef Point<int> Pointi;

} // namespace HiME

#include "bits/hime-point.inl"

#endif // HiME_POINT_H
