/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_RECT_H
#define HiME_RECT_H

#include "hime-point.h"
#include "hime-size.h"

namespace HiME
{

/**
 * A rectangle class template.
 */
template<class T>
class Rect
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    /**
     * Constructs an empty rectangle at (0, 0) position.
     */
    Rect();

    /**
     * Constructs a copy of a rectangle.
     *
     * @param r     Source rectangle.
     */
    Rect(const Rect& r);

    /**
     * Constructs a rectangle with a given position and a given size.
     *
     * @param x     X coordinate of top-left angle of the new rectangle.
     * @param y     Y coordinate of top-left angle of the new rectangle.
     * @param w     Width of the new rectangle.
     * @param h     Height of the new rectangle.
     */
    Rect(T x, T y, T w, T h);

    /**
     * Constructs a rectangle with a given top-left andf bottom-right
     * angle positions.
     *
     * @param x1y1  Top-left angle coordinates.
     * @param x2y2  Bottom-right angle coordinates.
     */
    Rect(const Point<T>& x1y1, const Point<T>& x2y2);

    /**
     * Constructs a rectangle with a given position and a given size.
     *
     * @param pos   Top-left angle coordinates.
     * @param sz    Size of the rectangle.
     */
    Rect(const Point<T>& pos, const Size<T>& sz);

    Point<T> pos;           ///< Position of the top-left angle.
    Size<T> size;           ///< Size of the rectangle.
};


/**
 * Defines a single precision floating point rectangle.
 */
typedef Rect<float> Rectf;

/**
 * Defines a integer rectangle.
 */
typedef Rect<int> Recti;

} // namespace HiME

#include "bits/hime-rect.inl"

#endif // HiME_RECT_H
