/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#ifndef HiME_SIZE_H
#define HiME_SIZE_H

#include "hime-point.h"

namespace HiME
{

/**
 * A 2D size class template.
 */
template<class T>
class Size
{
    HiME_DEFINE_NEW_OPERATOR();

public:
    /**
     * Constructs an empty size object.
     */
    Size();

    /**
     * Constructs a copy of a 2D size object.
     *
     * @param size  Source size object.
     */
    Size(const Size& size);

    /**
     * Constructs a 2D size object of the given width and height.
     *
     * @param w     Width.
     * @param h     Height
     */
    Size(T w, T h);

    T w;            ///< Width.
    T h;            ///< Height.
};


/**
 * Defines a single precision floating point 2D size.
 */
typedef Size<float> Sizef;

/**
 * Defines a integer 2D size.
 */
typedef Size<int> Sizei;

} // namespace HiME

#include "bits/hime-size.inl"

#endif // HiME_SIZE_H
