/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-test.h"
#include "hime-array.h"

namespace HiME
{

template<class T>
static void generateSample(T* dst, size_t n)
{
    T t = 0;
    for (size_t i = 0; i < n; ++i)
    {
        t += 2;
        t *= 2;
        dst[i] = t;
    }
}


template<class T, class TT>
class testArrayAlloc
{
public:
    static void run()
    {
        Array<T, TT> is;

        is.reserve(555);
        if (is.capacity() != 555)
            throw TestError("array capacity");

        is.trim();
        if (is.capacity() != 0)
            throw TestError("array trim");

        is.resize(44);
        if (is.size() != 44)
            throw TestError("array resize");

        is.reserve(777);
        if (is.capacity() != 777 || is.size() != 44)
            throw TestError("array reserve");

        is.trim();
        if (is.capacity() != 44 || is.size() != 44)
            throw TestError("array trim");

        is.resize(44);
        is.clear();
        if (is.size() != 0 || is.capacity() != 0 || is.data() != NULL)
            throw TestError("array clear");
    }
};


template<class T, class TT>
class testArrayReplace
{
public:
    static void run()
    {
        const size_t N = 50;

        T arr[N];
        generateSample(arr, N);

        Array<T, TT> is;
        is.replace(0, 0, arr, N);

        for (size_t i = 0; i < N; ++i)
        {
            if (is[i] != arr[i])
                throw TestError("array replace");
        }
    }
};


template<class T, class TT>
class testArrayAPI
{
public:
    static void run()
    {
        Array<T, TT> is;
        Array<T, TT> is2(is);
        is2 = is;
        is2.assign(is);

        is.clear();
        is.reserve(55);
        is.trim();
        is.resize(10);
        is.grow(20);
        is.size();
        is.capacity();
        is.data();

        const Array<T, TT>& cis = is;
        cis.data();

        T t = T(), t1;

        is[0] = t;
        t1 = is[0];
        t1 = cis[0];
        is[0] = t1;
        is[0] = t;

        is.front();
        cis.front();
        is.back();
        cis.back();

        is2 = is;
        is.replace(0, 0, &t, 1);
        is.replace(0, 0, is2, 1);
        is.fill(0, 0, t, 2);
        is.insert(1, t);
        is.append(t);
        is.append(is2);
        is.erase(0, 1);
        is.pop();
    }
};


template< template<class T, class TT> class F >
static void testTypes()
{
    F<int, PlainTypeTraits<int> >::run();
    F<int, PlainTypeTraits<int> >::run();
    F<float, PlainTypeTraits<float> >::run();
    F<double, PlainTypeTraits<double> >::run();
    F<char, PlainTypeTraits<char> >::run();

    //F<int>::run();
//    F<float, PlainTypeTraits<float>, DefaultGrowPolicy>::run();
//    F<double, PlainTypeTraits<double>, DefaultGrowPolicy>::run();
//    F<char, PlainTypeTraits<char>, DefaultGrowPolicy>::run();
}

void testArray()
{
    testTypes<testArrayAlloc>();
    testTypes<testArrayReplace>();
    testTypes<testArrayAPI>();
}

} // namespace HiME
