/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-test.h"
#include "hime-string.h"

namespace HiME
{

template<class T> struct Samples;

template<> struct Samples<char>
{
    static const char* s1()   { return "12ab55abcggabc"; }
    static const char* s2()   { return "12Ab55AbcggAbc"; }
    static const char* s3()   { return "12ab55Abcggabc"; }
    static const char* s4()   { return "12ab55AbcggAbc"; }
    static const char* s5()   { return "111111"; }
    static const char* s6()   { return "1234"; }
    static const char* s7()   { return "abc"; }
    static const char* zero() { return ""; }
};

template<> struct Samples<unsigned char>
{
    static unsigned char* s1()   { return (unsigned char*)"12ab55abcggabc"; }
    static unsigned char* s2()   { return (unsigned char*)"12Ab55AbcggAbc"; }
    static unsigned char* s3()   { return (unsigned char*)"12ab55Abcggabc"; }
    static unsigned char* s4()   { return (unsigned char*)"12ab55AbcggAbc"; }
    static unsigned char* s5()   { return (unsigned char*)"111111"; }
    static unsigned char* s6()   { return (unsigned char*)"1234"; }
    static unsigned char* s7()   { return (unsigned char*)"abc"; }
    static unsigned char* zero() { return (unsigned char*)""; }
};

template<> struct Samples<signed char>
{
    static signed char* s1()   { return (signed char*)"12ab55abcggabc"; }
    static signed char* s2()   { return (signed char*)"12Ab55AbcggAbc"; }
    static signed char* s3()   { return (signed char*)"12ab55Abcggabc"; }
    static signed char* s4()   { return (signed char*)"12ab55AbcggAbc"; }
    static signed char* s5()   { return (signed char*)"111111"; }
    static signed char* s6()   { return (signed char*)"1234"; }
    static signed char* s7()   { return (signed char*)"abc"; }
    static signed char* zero() { return (signed char*)""; }
};

template<> struct Samples<wchar_t>
{
    static const wchar_t* s1()   { return L"12ab55abcggabc"; }
    static const wchar_t* s2()   { return L"12Ab55AbcggAbc"; }
    static const wchar_t* s3()   { return L"12ab55Abcggabc"; }
    static const wchar_t* s4()   { return L"12ab55AbcggAbc"; }
    static const wchar_t* s5()   { return L"111111"; }
    static const wchar_t* s6()   { return L"1234"; }
    static const wchar_t* s7()   { return L"abc"; }
    static const wchar_t* zero() { return L""; }
};

template<> struct Samples<int>
{
    static int* s1()   { static int s[] = { '1', '2', 'a', 'b', '5', '5', 'a', 'b', 'c', 'g', 'g', 'a', 'b', 'c' }; return s; }
    static int* s2()   { static int s[] = { '1', '2', 'A', 'b', '5', '5', 'A', 'b', 'c', 'g', 'g', 'A', 'b', 'c' }; return s; }
    static int* s3()   { static int s[] = { '1', '2', 'a', 'b', '5', '5', 'A', 'b', 'c', 'g', 'g', 'a', 'b', 'c' }; return s; }
    static int* s4()   { static int s[] = { '1', '2', 'a', 'b', '5', '5', 'A', 'b', 'c', 'g', 'g', 'A', 'b', 'c' }; return s; }
    static int* s5()   { static int s[] = { '1', '1', '1', '1', '1', '1' }; return s; }
    static int* s6()   { static int s[] = { '1', '2', '3', '4' }; return s; }
    static int* s7()   { static int s[] = { 'a', 'b', 'c' }; return s; }
    static int* zero() { static int s[] = { '\0' }; return s; }
};

template<class T> static void testStringAPI()
{
    BaseString<T> s;
    BaseString<T> s1(s);
    BaseString<T> s2(Samples<T>::s1());
    BaseString<T> s3(50, 'y');
    BaseString<T> s4;
    BaseString<T> s5;

    s.clear();
    s.size();
    s.capacity();
    s.data();
    s.c_str();
    s.c_str();
    s.resize(1);
    s[0];
    s.reserve(100);
    s.trim();
    s.resize(10);
    s.grow(1);
    s.replace(0, 0, Samples<T>::s5(), 5);
    s.replace(0, 0, Samples<T>::s6());
    s.fill(0, 1, 5, 'a');
    s.insert(0, 'b');
    s.append('a');
    s.append(Samples<T>::s5());
    s.append(s1);
    s += 'a';
    s += Samples<T>::s5();
    s += s1;
    s.erase(0, 1);
    s4 += 'a';

    s4.find('a');
    
    BaseString<T> t;
    t.replace(0, 0, Samples<T>::s7(), 3);
    s.clear();
    s.replace(0, 0, Samples<T>::s1(), 14);
    size_t i = s.find(t);
    i = s.findFrom(i + 1, NPOS, t);
    i = s.findFrom(0, NPOS, t, 0, 2);

    i = s.find(Samples<T>::s7());
    i = s.findFrom(i + 1, NPOS, Samples<T>::s7());
    i = s.findFrom(0, NPOS, Samples<T>::s7(), 0, 2);
    //HiME_Trace("%d\n", i);
}


template<class T> static void testAssign()
{
    BaseString<T> s1;

    s1 = NULL;
    if (s1.data() || s1.size() || s1.capacity())
        throw TestError("string assign");

    BaseString<T> s2(s1);
    if (s2.data() || s2.size() || s2.capacity())
        throw TestError("string assign");

    s1 = Samples<T>::zero();
    if (!s1.data() || s1.size() || !s1.capacity())
        throw TestError("string assign");

    s2 = BaseString<T>(s1);
    if (!s2.data() || s2.size() || !s2.capacity())
        throw TestError("string assign");

    s2 = Samples<T>::s5();
    s2 = s1;
    if (!s2.data() || s2.size() || !s2.capacity())
        throw TestError("string assign");
}


template<class T> static void testFind()
{
    BaseString<T> s1;

    s1.find('a');
}


template<class T> static void testSubstChar()
{
    BaseString<T> s1(Samples<T>::s1());
    s1.subst('a', 'A');
    if (s1 != Samples<T>::s2())
        throw TestError("string subst char");

    s1.subst('A', 'a');
    s1.substFrom(3, 5, 'a', 'A');
    if (s1 != Samples<T>::s3())
        throw TestError("string subst char");

    s1.subst('A', 'a');
    s1.substFrom(3, NPOS, 'a', 'A');
    if (s1 != Samples<T>::s4())
        throw TestError("string subst char");

    s1.subst('A', 'a');
    s1.subst('z', 'A');
    if (s1 != Samples<T>::s1())
        throw TestError("string subst char");

    s1.clear();
    s1.subst('z', 'A');
    if (!s1.empty())
        throw TestError("string subst char");
}


template<class T> static void testSubstString()
{
    const T* s = Samples<T>::s1();
    const T* from = Samples<T>::s7();
    const T* to = Samples<T>::s6();

    BaseString<T> s1 = s;
    s1.subst(from, to);
}


void testString()
{
    testStringAPI<char>();
    testAssign<char>();
    testFind<char>();
    testSubstChar<char>();

    testStringAPI<unsigned char>();
    testAssign<unsigned char>();
    testFind<unsigned char>();
    testSubstChar<unsigned char>();

    testStringAPI<signed char>();
    testAssign<signed char>();
    testFind<signed char>();
    testSubstChar<signed char>();

    testStringAPI<wchar_t>();
    testAssign<wchar_t>();
    testFind<wchar_t>();
    testSubstChar<wchar_t>();

    testStringAPI<int>();
    testAssign<int>();
    testFind<int>();
    testSubstChar<int>();

    //testSubstString<char>();
    String s = "123abc567abcIIabc";
    s.substFrom(5, 8, "abc", "qwe");

    String ss = "567";
    String ww = "uuuuuuu";
    s.subst(ss, ww);

    String s2;
    s2 = "utf8 ";
    union { char s[7]; unsigned char us[7]; } tmp;
    tmp.us[0] = 0xe4; tmp.us[1] = 0xb8; tmp.us[2] = 0x96;
    tmp.us[3] = 0xe7; tmp.us[4] = 0x95; tmp.us[5] = 0x8c;
    tmp.us[6] = 0;
    s2 = tmp.s;


    ///
    HiME_DebugSetMaxAllocSize(1024);
    try
    {
        String s1 = "qwerty";
        String s2 = s1;
        String s3 = s1;
        s3.assign(2048, 'a');
    }
    catch (const std::bad_alloc&)
    {
    }
    catch(...)
    {
        throw TestError("string alloc: wrong exception");
    }
    HiME_DebugSetMaxAllocSize(0);
}

} // namespace HiME

/*

namespace HiME {
	namespace Test {
		extern "C" void
		hime_test_cpp_string(hime_test_context_t* ctx)
		{
			ctx->package = "c++ string";
			ctx->test_group = "c++ string";
			ctx->test_case = "c++ string";

			String s;
			String s1("hello");
			String s2(s1);

			s = "123";
			s.replace(3, 0, "a");
			s.clear();

			s2 = "utf8 ";
			union { char s[7]; unsigned char us[7]; } tmp;
			tmp.us[0] = 0xe4; tmp.us[1] = 0xb8; tmp.us[2] = 0x96;
			tmp.us[3] = 0xe7; tmp.us[4] = 0x95; tmp.us[5] = 0x8c;
			tmp.us[6] = 0;
			s2 += tmp.s;
			s2 += " test";
			s2 += s1;
			s2 += 'a';

			s = s1;
			s.clear();
			s = s1;
			s.truncate();
			String::swap(s1, s2);
			s.reserve(50);
			s.trim();

			s = "1234567890";
			s.replace(1, 2, "abcd");

			s = "1234567890";
			s.replace(1, 2, s2);

			s.fill(4, 2, 8, 'Q');
			s.clear();
			s = "Qwerty";

			s.push('a');
			s.push('b');
			s.push('c');
			s.pop();
			s.front();
			s.back();
			s[0] = 'A';

			const String& cc = s;
			cc.front();
			cc.back();
			char a = cc[0];
			s.push(a);
		}
	}
} // namespace HiME
*/
