/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-thread-pool.h"
#include "hime-string.h"
#include "hime-mutex.h"

namespace HiME
{

class TestJob : public Job
{
public:
    int n;
    String* s;
    Mutex* m;

    TestJob(String* s, Mutex* m)
        : s(s),
        m(m)
    {
        static int nn = 0;
        n = nn++;
    }

    virtual void run()
    {
#ifdef HiME_THREAD_POOL_VERBOSE
        HiME_TimeTrace("job #%0d started\n", n);
#endif
        for (int i = 0; i < 100; ++i)
        {
            m->lock();
            s->append("q");
            m->unlock();
            //HiME_Sleep(1);
        }
#ifdef HiME_THREAD_POOL_VERBOSE
        HiME_TimeTrace("job #%0d finished\n", n);
#endif
    }
};


void testThreadPool1()
{
    ThreadPool* t = new ThreadPool;
    const int NN = 20;
    TestJob* j[NN];
    String ss;
    Mutex mm;

    for (int i = 0; i < NN; ++i)
        j[i] = new TestJob(&ss, &mm);

    for (int i = 0; i < NN; ++i)
    {
        t->start(j[i]);
        //HiME_Sleep(1);
    }

    for (int i = 0; i < NN; ++i)
    {
        t->start(j[i]);
        //HiME_Sleep(1);
    }

    for (int i = 0; i < NN; ++i)
            j[i]->wait();

    for (int i = 0; i < NN; ++i)
        delete j[i];

    delete t;
}

void testThreadPool()
{
    for (int i = 0; i < 1; ++i)
    {
#ifdef HiME_THREAD_POOL_VERBOSE
        HiME_Trace("%d\n", i);
#endif
        testThreadPool1();
    }
}

} // namespace HiME
