/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-test.h"
#include "hime-utf8.h"
#include "hime-vfs-node.h"
#include "hime-vfs-iterator.h"
#include "hime-vfs-driver.h"

#ifdef HiME_ENABLE_VFS_POSIX
#include "hime-vfs-posix.h"
typedef HiME::VFS::PosixDriver TestVFSDriver;
#elif HiME_ENABLE_VFS_WIN32
#include "win32/hime-vfs-win32.h"
typedef HiME::VFS::Win32Driver TestVFSDriver;
#else
#error VFS not defined
#endif

namespace HiME
{

static void simpleRead()
{
    TestVFSDriver drv;
    VFS::Node* node = drv.mount("/", "src/test/res");
    VFS::Node* file = node->open("hello.txt");
    Stream* s = file->stream(0, IO::AccessRead);
    hime_lsize_t sz = s->size();
    char* buff = (char*)HiME_Alloc((size_t)sz + 1);
    s->read(buff, sz);
    buff[sz] = 0;
    if (String::compare(buff, "Hello, HiME!") != 0)
        throw TestError("vfs posix: simple read");

    if (s->tell() != sz)
        throw TestError("vfs tell");

    s->seek(-1 * (int)sz, IO::SeekEnd);
    if (s->tell() != 0)
        throw TestError("vfs seek");

    IO::Stat stat;
    file->stat(stat);
    if (stat.size != sz)
        throw TestError("vfs stats");
    if (stat.flags & IO::NodeDirectory)
        throw TestError("vfs stats");
    if (stat.flags & IO::NodeLink)
        throw TestError("vfs stats");
    if (!(stat.flags & IO::NodeDataFile))
        throw TestError("vfs stats");

    node->stat(stat);
    if (!(stat.flags & IO::NodeDirectory))
        throw TestError("vfs stats");
    if (stat.flags & IO::NodeLink)
        throw TestError("vfs stats");
    if (stat.flags & IO::NodeDataFile)
        throw TestError("vfs stats");

    VFS::Iterator* itr = node->children();
    //HiME_Trace("\n");
    while (itr->next())
    {
        FileName fn;
        itr->current(fn);
        //HiME_Trace("%s\n", fn.c_str());
    }
    itr->reset();
    //HiME_Trace("\n");
    while (itr->next())
    {
        FileName fn;
        itr->current(fn);
        //HiME_Trace("%s\n", fn.c_str());
    }

    HiME_Free(buff, (size_t)sz + 1);
    delete itr;
    delete file;
    delete node;
    delete s;
}


void testVFS()
{
    simpleRead();
}

} // namespace HiME
