/****************************************************************************
Copyright (c) 2011 kona4kona (kona4kona@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

#include "hime-test.h"
#include "hime-arch-debug.h"

namespace HiME
{

void testMemory();
void testArray();
void testVector();
void testString();
void testUTF8();
void testThreadPool();
void testVFS();

void TestContext::run()
{
    typedef void (*tests_t)();

    struct TestGroup
    {
        const char* name;
        tests_t t;
    };

    TestGroup tests[] =
    {
        { "memory",             testMemory },
        { "array",              testArray },
        { "vector",             testVector },
        { "string",             testString },
        { "utf8",               testUTF8 },
        { "thread_pool",        testThreadPool },
        { "vfs",                testVFS },
        //		hime_test_vector,
        //		hime_test_string,
        //		hime_test_map,
        //		hime_test_variant,
        //		hime_test_algorithm,
        //		hime_test_cpp_vector,
        //		hime_test_cpp_string,
        //		hime_test_cpp_map,
        //		hime_test_cpp_variant
    };

    for (size_t i = 0; i < HiME_ARRAY_LENGTH(tests); ++i)
    {
        HiME_Trace("% -20s: ", tests[i].name);
        try
        {
            tests[i].t();
            HiME_Trace("passed\n");
        }
        catch(const std::exception& err)
        {
            HiME_Trace("FAILED at %s\n", err.what());
        }
    }
    HiME_Trace("\n");
    HiME_Trace("Tests finished\n");
}

} // namespace HiME
