/*! Timer in arch layer; use 'arch_timer_t' device defined in configuration */
#pragma once

#include <types/time.h>

/*! interface for kernel  */

/*! Initialize timer 'arch' subsystem: timer device, subsystem data */
void arch_timer_init ();

/*!
 * Set next timer activation
 * \param time Time of next activation
 * \param alarm_func Function to call upon timer expiration
 */
void arch_timer_set ( timespec_t *time, void *alarm_func );

/*!
 * Get 'current' system time
 * \param time Store address for current time
 */
void arch_get_time ( timespec_t *time );

/*!
 * Set 'current' system time
 * \param time Time to set as current
 * NOTE: changing clock may have unpredicted behavior on timers!
 */
void arch_set_time ( timespec_t *time );

/*! Get minimal timer interval supported by hardware timer */
void arch_get_min_interval ( timespec_t *time );

/*! Enable/disable interrupt generated by hardware timer */
void arch_enable_timer_interrupt ();
void arch_disable_timer_interrupt ();
