#define _J_VRIJEME_C_
#include "vrijeme.h"

#include "ispis.h"
#include "razno.h"
#include "prekidi.h"
#include "liste.h"
#include <arh/vrijeme.h>

//struktura podataka

//virtualno vrijeme sustava, pocetno 0:0, azurira se (barem) na svaki prekid
static vrijeme_t j_sat;

//zadnja ucitana vrijednost u brojilo (salje se arh sloju)
static arh_vrijeme_t j_sat_brojilo;

//staticki alocirani elementi za sve alarme
static j_alarm_t j_alarm[NAJ_ALARMA];

//lista aktivnih alarma - slozena prema vremenima aktivacije
static j_alarmi_t j_alarmi;
//u buducnosti neka druga struktura radi brzeg pretrazivanje (neko stablo)

//! pocetna inicijalizacija satnog podsustava
void j_sat_inicijaliziraj ()
{
	int i, prekidi_prije;
	static bool inicijalizirano = FALSE;
	
	if (inicijalizirano) {
		j_ispisi("Podsustav 'jezgra/vrijeme' je vec inicijaliziran!\n");
		return ;
	}
	inicijalizirano = TRUE;

	//zabrani prekide (ako vec nisu), i pamti prethodno stanje
	prekidi_prije = j_dozvoli_prekide(FALSE);


	//resetiraj sat sustava
	j_sat.sekunde = j_sat.nsekunde = 0;
	
	//lista alarma je u pocetku prazna
	j_alarmi.prvi = j_alarmi.zadnji = NULL;

	memset(j_alarm, 0, sizeof(j_alarm));
	for (i = 0; i < NAJ_ALARMA; i++)
		j_alarm[i].slobodno = SLOBODNO; //svi alarmi su slobodni u pocetku

	//postavi prekidnu funkciju za prekid sata
	(void) j_prekidi_postavi(PREKIDI_SAT, j_sat_prekid);

	j_sat_brojilo = ARH_SAT_NAJVECI_BROJ;
	arh_sat_ucitaj_brojilo(j_sat_brojilo);
	arh_dozvoli_prekid_sata();


	//dozvoli prekide, ako su i prije bili dozvoljeni
	j_dozvoli_prekide(prekidi_prije);
}

//! na svaki prekid sustavskog sata poziva se ova f-ja (ako se prekidi prihv.)
static void j_sat_prekid (int x)
{
	j_alarm_t *alarm;

	//j_ispisi("Prekid sata (%d.%d)\n", j_sat.sekunde, j_sat.nsekunde);
	//isteklo "j_sat_brojilo" - dodaj proteklo vrijeme u jezgrin sat
	j_sat = j_vrijeme_zbroji(j_sat, arh_vrijeme_u_j_vrijeme(j_sat_brojilo));

	while (j_alarmi.prvi) { //ima definiranih alarma?
		if (j_vrijeme_usporedi(j_sat, j_alarmi.prvi->budjenje) >= 0)
		{
			alarm = UZMI_PRVI_ELEMENT(&j_alarmi);
			
			if (alarm->zast & PERIODICKI) {
				alarm->budjenje = j_vrijeme_zbroji(
							alarm->period,
							alarm->budjenje);
				
				//stavi nazad u listu na odgovarajuce mjesto
				DODAJ_U_SORT_LISTU(&j_alarmi, alarm,
							j_alarm_usporedi)
			}

			//pozovi zadanu funkciju
			//funkcionira dok se "sve" radi u jezgrenom nacinu rada!
			if (alarm->obrada != NULL)
				alarm->obrada(alarm->param);
			//kada to ne bi bilo tako (sve u jezgrenom), onda ovaj 
			//mehanizam treba promijeniti
			
			j_alarm_azuriraj_sat();
			//pozvana funkcija moze potrajat, pa treba opet ocitat
			//i azurirat vrijeme
		}
		else {
			break;	//nije jos vrijeme alarma, a posto se gleda prvi
		}		//u listi, ostali su sigurno kasnije
	}
}

//azuriraj vrijeme jezgrina sata
static void j_alarm_azuriraj_sat ()
{
	arh_vrijeme_t preostalo, proteklo;
	vrijeme_t iduce;
	
	//dohvati sat iz brojila (sklopovskog)
	preostalo =  arh_sat_procitaj_brojilo();
	proteklo = arh_vrijeme_oduzmi(j_sat_brojilo, preostalo);

	if (proteklo < 0) { //ovo se ne smije dogoditi - ako se ipak dogodi ...
		j_ispisi("preostalo:%d\n", preostalo);
		j_ispisi("proteklo:%d\n", proteklo);
		j_zaustavi_potpuno("Greska u brojilima!");
	}

	//dodaj proteklo vrijeme u jezgrin sat
	j_sat = j_vrijeme_zbroji(j_sat, arh_vrijeme_u_j_vrijeme(proteklo));
	j_sat_brojilo = ARH_SAT_NAJVECI_BROJ;

	if (j_alarmi.prvi != NULL) { //ima li definiranih alarma?
		//ako se treba buditi prije prekida - azuriraj brojilo
		iduce = j_vrijeme_zbroji(j_sat,
				arh_vrijeme_u_j_vrijeme(j_sat_brojilo));

		if (j_vrijeme_usporedi(j_alarmi.prvi->budjenje, iduce) < 0) {
			iduce = j_vrijeme_oduzmi(j_alarmi.prvi->budjenje,j_sat);

			j_sat_brojilo = arh_j_vrijeme_u_arh_vrijeme(iduce);

			//najmanji broj za sat (njegovo brojilo) je definirano!
			if (j_sat_brojilo < ARH_SAT_PRAG)
				j_sat_brojilo = ARH_SAT_PRAG;
		}
	}

	arh_sat_ucitaj_brojilo(j_sat_brojilo);
}


//rezerviranje novog alarma
static int j_alarm_stvori ()
{
	int i, id = -1; //ako nema slobodnog

	for (i = 0; i < NAJ_ALARMA; i++) {
		if (j_alarm[i].slobodno == SLOBODNO) { //nasao slobodan id
			j_alarm[i].periodicki = FALSE;
			j_alarm[i].obrada = NULL;
			j_alarm[i].param = NULL;
			j_alarm[i].zast = 0;
			j_alarm[i].iduci = NULL;

			id = i;
			break;
		}
	}

	return id;
}

//brisi postojeci alarm (sucelje prema 'van')
int j_alarm_obrisi (void *param)
{
	int id, *idp = param;

	PARAMNULL(idp);
	id = *idp;

	if (id < 1 || id >= NAJ_ALARMA) {
		j_ispisi("Krivi id alarma\n");
		return -1;
	}

	id--; //vrati na "pravu vrijednost"

	if (j_alarm[id].slobodno == SLOBODNO) {//vec je slobodan!?
		j_ispisi("alarm nije bio zauzet (id=%d)\n", id + 1);
		return -1;
	}

	j_alarm[id].slobodno = SLOBODNO;

	//makni ga iz liste aktivnih ako je tu - ne mora biti
	MAKNI_IZ_LISTE(&j_alarmi, &j_alarm[id]);
	
	return 0;
}

//! postavljanje parametara alarma (ili modifikacija postojeceg!)
int j_alarm_postavi (void *param)
{
	bool novi = FALSE;
	alarm_t *p = param;
	j_alarm_t *alarm;
	int id;
	
	PARAMNULL(p);
	
	id = p->id;

	if (id == 0 && (p->zast & STVORINOVI)) {
		id = j_alarm_stvori();
		
		if (id == -1) {
			j_ispisi("nema slobodnog alarma\n");
			return -1;
		}

		novi = TRUE;
	}
	else {
		id--; //postojeci alarm
	}

	if (id < 0 || id >= NAJ_ALARMA || j_alarm[id].slobodno == SLOBODNO) {
		j_ispisi("Krivi id alarma\n");
		return -1;
	}

	alarm = &j_alarm[id];

	if (novi || p->zast)
		alarm->zast = p->zast;

	if (novi || p->param)
		alarm->param = p->param;

	if (novi || p->obrada)
		alarm->obrada = p->obrada;

	if (novi)
		alarm->iduci = NULL;

	if (novi || p->kada.sekunde + p->kada.nsekunde > 0) {
		alarm->period = p->kada;

		j_alarm_azuriraj_sat();

		if (p->zast & ABSVRIJEME)
			alarm->budjenje = p->kada;
		else //RELVRIJEME
			alarm->budjenje = j_vrijeme_zbroji(j_sat, p->kada);
		
		//izbaci ga iz liste, ako se radi o postojecem alarmu kojeg se
		//modificira - mijenja vremena ili nesto drugo
		MAKNI_IZ_LISTE(&j_alarmi, alarm);

		//dodaj alarm u listu
		DODAJ_U_SORT_LISTU(&j_alarmi, alarm, j_alarm_usporedi)

		if (j_alarmi.prvi == alarm)	
			j_alarm_azuriraj_sat();
			//ovo je sad prvi alarm, mozda treba reinicijalizirat
			//brojilo u satnom mehanizmu arh sloja
	}

	return id + 1;
}	//vrati za 1 veci - koristi se i 0-ti, ali prema "van" brojke idu od 1

/* usporedba alarma: vraca 0 ako je x == y, -1 za x < y, 1 ako je x > y */
static int j_alarm_usporedi (j_alarm_t *x, j_alarm_t *y)
{
	if (!x && !y)
		return 0;
	if (!x)
		return 1;
	if (!y)
		return -1;

	return j_vrijeme_usporedi(x->budjenje, y->budjenje);
}
