//! Upravljanje prekidima - razina jezgre

/* Obrada prekida ide na slijedeci nacin:
   * u "arh" sloju su za svaki prekid definirane funkcije, ali te funkcije su
     iste i pozivaju "j_prekidna_funkcija" koje dalje delegira posao

   * skraceno:

     prekid_x: //u arh sloju
	...
	j_prekidna_funkcija(x);
	...
	iret
*/

#define _J_PREKIDI_C_
#include "prekidi.h"

#include "razno.h"
#include "ispis.h"
#include "pozivi_jezgre.h"
#include "postavke.h"
#include <arh/prekidi.h>

//opisnici prekida
prekid_t j_prekidna_f[PREKIDI_BROJ];
//uz ovu implementaciju za neki sklopovski prekid moze biti samo 1. funkcija!
//ako treba vise, onda treba ovo nadograditi


void j_prekid_nedefiniran (int n)
{
	j_ispisi("Prekid za koje nije definirano ponasanje (%d) ", n);
	j_ispisi("%s\n", arh_opis_prekida(n));
	
	j_zaustavi_potpuno("Obustavljam daljnji rad!");
}

//poziva se iz arh dijela kao funkcija koja obradjuje prekide
//kontekst je spremljen na stogu aktivne dretve
void j_prekidna_funkcija (int id)
{
	if (j_prekidna_f[id] == NULL)
		j_prekid_nedefiniran(id);
	else
		(j_prekidna_f[id])(id);
}

//definira prihvat prekida za pojedini prekid
prekid_t j_prekidi_postavi (int id, prekid_t obrada)
{
	prekid_t stara_funkcija = NULL;

	if (id <0 || id > PREKIDI_BROJ) {
		j_ispisi("ID prekida izvan granica\n");
	}
	else {
		stara_funkcija = j_prekidna_f[id];
		j_prekidna_f[id] = obrada;
	}

	return stara_funkcija;
}

//stog za jezgrine funkcije (kada se zovu iz prekida)
uint32 j_stogjf[VELICINA_STOGA_JEZGRE/4]; // djelim s 4 jer je jedinica 4 bajta
uint32 *j_prekidni_stog = j_stogjf + (VELICINA_STOGA_JEZGRE - 1); //vrh stoga

uint32 *j_kazaljka_stoga = NULL; 
//u buducnosti ce ovo biti dio konteksta dretve
//sada imam samo jednu dretvu (pocetnu/glavnu) koja vec ima definirani stog
//ova ce varijabla posluziti za pohranu adrese kazaljke stoga pri prekidu kada
//ce se aktivirati drugi stog (prekidni)
//po povratku iz prekida, ponovo se aktivira ovaj stog

uint32 **j_stog_aktivne = &j_kazaljka_stoga;
//arh sloju saljem adresu gdje ce se nalazit adresa za kazaljku!
//kada budem imao vise dretvi i u prekidu se promjeni aktivna, promjeniti ce
//se i ova kazaljka tako da pokazuje na pohranjenu kazaljku stoga za novu dretvu


//inicijalizira prekidni podsustav (brise sve)
void j_prekidi_inicijaliziraj ()
{
	arh_prekidi_inicijaliziraj(	j_prekidna_funkcija, 
					j_prekidni_stog,
					&j_stog_aktivne);
	
	memset(j_prekidna_f, 0, sizeof(j_prekidna_f));

	j_prekidi_postavi(PREKIDI_PROGRAMSKI, j_pozivi_jezgre);
}

