#ifndef _J_LISTE_H_
#define _J_LISTE_H_

/* par macroa za rad s listom

  'lista' i 'elem' su kazaljke na strukture koje moraju imati:
	'lista': *prvi i *zadnji
	'elem': *iduci
  npr.
	struct nesto1 {
		...
		struct nesto2 *prvi;
		struct nesto2 *zadnji;
		...
	} *lista;
	
	struct nesto2 {
		...
		struct nesto2 *iduci;
		...
	} *elem;
*/

#define UZMI_PRVI_ELEMENT(lista)					\
({									\
	typeof(lista) l = (lista);					\
	typeof((lista)->prvi) e = l->prvi;				\
									\
	if (e != NULL) /* nije prazna */				\
		l->prvi = e->iduci;					\
									\
	e;								\
})
//koristenje: a = UZMI_PRVI_ELEMENT(&neka_lista);

//ne brise element iz liste
#define POGLEDAJ_PRVI_ELEMENT(lista)	((lista)->prvi)
//koristenje: a = POGLEDAJ_PRVI_ELEMENT(&neka_lista);

#define POGLEDAJ_IDUCI_ELEMENT(elem)	((elem)->iduci)
//koristenje: a = POGLEDAJ_IDUCI_ELEMENT(&neki_elem);

#define MAKNI_IZ_LISTE(lista, elem)					\
({									\
	typeof(lista) l = (lista);					\
	typeof(elem) e = (elem), pom;					\
	bool nasao = FALSE;						\
									\
	if (l->prvi == e) {						\
		l->prvi = e->iduci;					\
		nasao = TRUE;						\
	}								\
	else {								\
		pom = l->prvi;						\
		while (pom->iduci != NULL) {				\
			if (pom->iduci == e) {				\
				pom->iduci = e->iduci;			\
				if (l->zadnji == e)			\
					l->zadnji = pom;		\
				nasao = TRUE;				\
				break;					\
			}						\
			pom = pom->iduci;				\
		}							\
	}								\
									\
	nasao;								\
})
//koristenje: maknuo = MAKNI_IZ_LISTE(&neka_lista, &element);

#define DODAJ_NA_KRAJ(lista, elem)					\
{									\
	typeof(lista) l = (lista);					\
	typeof(elem) e = (elem);					\
									\
	if (l->prvi == NULL) {						\
		l->prvi = l->zadnji = e;				\
	}								\
	else {								\
		l->zadnji->iduci = e;					\
		l->zadnji = e;						\
	}								\
	e->iduci = NULL;						\
}
//koristenje: DODAJ_NA_KRAJ(&neka_lista, &element);

#define DODAJ_NA_POCETAK(lista, elem)					\
{									\
	typeof(lista) l = (lista);					\
	typeof(elem) e = (elem);					\
									\
	e->iduci = l->prvi;						\
	l->prvi = e;							\
									\
	if (e->iduci == NULL) {						\
		l->zadnji = e;						\
	}								\
}
//koristenje: DODAJ_NA_POCETAK(&neka_lista, &element);

#define DODAJ_U_SORT_LISTU(lista, elem, usporedi)			\
{									\
	typeof(lista) l = (lista);					\
	typeof(elem) e = (elem), p;					\
									\
	if (l->prvi == NULL) {						\
		l->prvi = l->zadnji = e;				\
		e->iduci = NULL;					\
	}								\
	else {								\
		if (usporedi(e, l->prvi) < 0) {				\
			e->iduci = l->prvi;				\
			l->prvi = e;					\
		}							\
		else {							\
			p = l->prvi;					\
			while (p->iduci && usporedi(e, p->iduci) >= 0)	\
				p = p->iduci;				\
									\
			e->iduci = p->iduci;				\
			p->iduci = e;					\
									\
			if (e->iduci == NULL)				\
				l->zadnji = e;				\
		}							\
	}								\
}
//koristenje: DODAJ_U_SORT_LISTU(&neka_lista, &element, funkcija_usporedbe);
//'funkcija_usporedbe' mora znati usporedjivati objekte tipa 'element'-a
//i vracati <0 kad je a < b, =0 za a == b, te >0 za a > b
//sortiranje ide od manjeg prema vecem, ako je ovako - za suprotno treba dati
//suprotne uvjete



#ifndef KORISTI_MASKE

//trazi prvi slobodan element (.slobodno == SLOBODNO)
//struktura mora imati element .slobodno koji je pocetno inicijaliziran s -1

#define DOHVATI(POLJE)							\
({									\
	int i, id = -1;							\
	int n = sizeof(POLJE) / sizeof(typeof(POLJE[0]));		\
									\
	for (i = 0; i < n; i++) {					\
		if (POLJE[i].slobodno == SLOBODNO) {			\
			id = i;						\
			POLJE[i].slobodno = 0;				\
			break;						\
		}							\
	}								\
									\
	id;								\
})

#define OBRISI(POLJE, ID)						\
({									\
	int povr = 0, n = sizeof(POLJE) / sizeof(typeof(POLJE[0]));	\
									\
	if (ID < 0 || ID >= n || POLJE[ID].slobodno == SLOBODNO)	\
		povr = -1;						\
	else								\
		POLJE[ID].slobodno = SLOBODNO;				\
									\
	povr;								\
})

#define MASKA(J, N) //prazno, ne koristi se

#else	/* KORISTI_MASKE */

#include "razno.h"

#define DOHVATI(POLJE)							\
({									\
	int id = -1;							\
	int n = sizeof(POLJE) / sizeof(typeof(POLJE[0]));		\
									\
	id = pronadji_bit_32(m ## POLJE, n);				\
	/*j_ispisi("id=%d, n=%d (%s)\n", id, n, #POLJE);*/		\
	if (id != -1) {							\
		POLJE[id].slobodno = 0;					\
		postavi_bit_32(m ## POLJE, n, id, FALSE);		\
	}					/*^^^^^- obrisi bit*/	\
									\
	id;								\
})

#define OBRISI(POLJE, ID)						\
({									\
	int povr = 0, n = sizeof(POLJE) / sizeof(typeof(POLJE[0]));	\
									\
	if (ID < 0 || ID >= n || POLJE[ID].slobodno == SLOBODNO) {	\
		povr = -1;						\
	} else {							\
		POLJE[ID].slobodno = SLOBODNO;				\
		postavi_bit_32(m ## POLJE, n, ID, TRUE);		\
	}					/*^^^^- postavi bit*/	\
									\
	povr;								\
})

#endif	/* KORISTI_MASKE */

#endif	/* _J_LISTE_H_ */
