#include <dretve.h>
#include <jezgra/pozivi_jezgre.h>
#include <jezgra/tipovi.h>

//stvori novu dretvu koja ce radit funkciju "poc_funkc(param)"
int stvori_dretvu_pr (void (*poc_funkc)(void *), void *param, void *stog,
			int prio, int rasp)
{
	struct {
		void (*poc_funkc)(void *);	//pocetna funckija dreve
		void *param;			//parametar za stog
		void *stog;			//pocetni stog
		int prio;			//prioritet
		int rasp;			//koji rasporedjivac? FIFO, RR
	} p;

	p.poc_funkc = poc_funkc;
	p.param = param;
	p.stog = stog;
	p.prio = prio;
	p.rasp = rasp;
	
	return j_funkcija(STVORI_DRETVU, &p);
}

//stvori novu dretvu koja ce radit funkciju "poc_funkc(param)"
int stvori_dretvu (void (*poc_funkc)(void *), void *param)
{
	return stvori_dretvu_pr(poc_funkc, param, NULL,	PRIO_POCETNI, RASP_PRP);
}


//propusta procesor iducoj dretvi, ako takva postoji
//vraca 0, ako je odstupila, 1 ako nije (npr. nema drugih dretvi)
int dretva_odstupi ()
{
	return j_funkcija(DRETVA_ODSTUPI, NULL);
}

//poziva se kada dretva zeli prije zavrsiti
int zavrsi_dretvu ()
{
	return j_funkcija(DRETVA_ZAVRSETAK, NULL);
}

//vrati 'id' trenutne dretve
int id_dretve ()
{
	return j_funkcija(ID_DRETVE, NULL);
}

//prioriteti
//dohvati prioritet dretve s "id", ili pozivajuce, ako je id = 0
int dohvati_prioritet (int id)
{
	return j_funkcija(DOHVATI_PRIORITET, &id);
}

int postavi_prioritet (int prio)
{
	return j_funkcija(POSTAVI_PRIORITET, &prio);
}

int cekaj_kraj_dretve (int id)
{
	return j_funkcija(CEKAJ_KRAJ_DRETVE, &id);
}


//sinkronizacija: semafor/monitor/red uvjeta
int dohvati_sinkro (int tip, int sort, int zast, int pocvr)
{
	struct {
		uint8	tip;	//tip reda
		uint8	sort;	//nacin sortiranja reda
		uint16	zast;	//zastavice
		uint	poc_vr;	//pocetna vrijednost
	} p;

	p.tip = tip;
	p.sort = sort;
	p.zast = zast;
	p.poc_vr = pocvr;

	return j_funkcija(DOHVATI_SINKRO, &p);
}

int obrisi_sinkro (int id)
{
	return j_funkcija(OBRISI_SINKRO, &id);
}

int postavi_sinkro (int id, bool sve)
{
	struct {
		int id;		//id reda
		bool sve;	//propusta sve ili samo jednu dretvu?
	} p;
	
	p.id = id;
	p.sve = sve;
	
	return j_funkcija(PROPUSTI_SINKRO, &p);
}

int cekaj_sinkro (int id_p, uint zast, int id_s, vrijeme_t *koliko)
{
	struct {
		int id_p;
		//primarni id (id reda uvjeta ili semafora ili monitora)

		uint zast;	//zastavice: PROBAJ, DRUGAOP

		int id_s;
		//sekundarni id (ako je primarni red uvjeta ovo je monitor)

		vrijeme_t *koliko;
		//ako je != 0 kaze koliko ce se cekati u blokiranom redu
		//po isteku vremena dretva se odblokira, a povratna je
		//vrijednost -1 (oznaka greske)
	} p;
	int povvr1 = -1, povvr2 = -1;
	
	p.id_p = id_p;
	p.id_s = id_s;
	p.zast = zast;
	p.koliko = koliko;

	//CekajSem ili udji_u_monitor ili uvrsti_u_red_uvjeta
	povvr1 = j_funkcija(CEKAJ_SINKRO, &p);
	
	//kad dretva dodje do tu, neko ju je oslobodio
	//ako je prije bio 'uvrsti_u_red_uvjeta', sada treba zauzeti monitor
	//(moze i nesto drugo, ali to bas i nema smisla)
	if (povvr1 != -1 && (zast & DRUGAOP)) {
		p.id_p = id_s;
		p.id_s = -1;
		p.koliko = NULL;

		povvr2 = j_funkcija(CEKAJ_SINKRO, &p);
	}

	if (povvr1 < 0)
		return povvr1;
	else
		return povvr2;
}