/* microval (version 3)
 * Hudson Lacerda (2004-2010)
 *
 * Calcula valores para nomes de acidentes microtonais para abcm2ps e tclabc
 *
 * Calculate values for abcm2ps' / tclabc' microtone accidental function names
 */

/* Defina a macro abaixo para obter o programa em portugus.*/
/* #define PORTUGUES */

#include <stdio.h>
#include <stdlib.h>
int main (int argc, char *argv[]) {
	float x=0.0, y=0.0, a=0.0;


	if(argc>4 || argc<2) {
		printf("\n\tmicroval (Hudson Lacerda - 2004-2010)\n\n");
#ifdef PORTUGUES
		printf("\t um programa para calcular o valor a ser utilizado"
		       "\n"
		       "\tna definio de acidentes microtonais para abcm2ps."
		       "\nD\n"
		       "FORMAS DE USO:\n\nmicroval <numerator> <denominator>\n"
		       "\tImprime <micro_value>, o nmero a utilizar no nome"
		       " da funo.\n\n"
		       "microval <micro_value>\n"
		       "\tImprime os valores de <numerator> e <denominator>.\n\n"
		       "microval <numerator> <denominator> <accidental>\n"
		       "\timprime <tclabc_micro_value>, accidental for tclabc."
		       "\n\n");
#else
		printf("\tis a program to calculate the value to be used in\n"
		       "\tabcm2ps' microtone accidental definitions.\n\n"
		       "USAGE:\n\nmicroval <numerator> <denominator>\n"
		       "\tPrint <micro_value>, the number to be used in "
		       "the function name.\n\n"
		       "microval <micro_value>\n"
		       "\tPrint the values of <numerator> and <denominator>\n\n"
		       "microval <numerator> <denominator> <accidental>\n"
		       "\tprints <tclabc_micro_value>, accidental for tclabc."
		       "\n\n");
#endif
		return 0;
	}

	x=atof(argv[1]);
	if (argc>=3) {
		y=atof(argv[2]);
		if(x>256.0 || y>256.0 || x<1 || y<1) {
#ifdef PORTUGUES
			printf(
			       "ERRO: <numerator> e <denominator> devem ser"
			       " nmeros inteiros de 1 a 256\n");
#else
			printf(
			       "ERROR: <numerator> and <denominator> must be"
			       " integers from 1 to 256\n");
#endif
			return 0;
		}

		if(argc==3)
		    printf("<micro_value> = %ld\n",
			   (long int)((256.0 * (x-1.0)) + (y-1.0)));
		else { /* > 3 */
		    /* tclabc accidentals */
#define DFLAT   5
#define FLAT    3
#define NATURAL 2
#define SHARP   1
#define DSHARP  4
#define NONE    0
		    switch(argv[3][0]) {
		    case '_':
			a=FLAT;
			if(argv[3][1]=='_')
			    a=DFLAT;
			break;
		    case '=':
			a=NATURAL;
			break;
		    case '^':
			a=SHARP;
			if(argv[3][1]=='^')
			    a=DSHARP;
			break;
		    default:
#ifdef PORTUGUES
			printf("ERRO: <acidente> deve ser "
			       "\"^\", \"_\", \"=\", \"^^\" ou \"__\"\n");
#else
			printf("ERROR: <accidental> should be "
			       "\"^\", \"_\", \"=\", \"^^\" or \"__\"\n");
#endif
			return 0;
		    }
		    printf("<tclabc_micro_value> = %ld\n",
			   (long int)(
				      ((256.0 * (x-1.0)) + (y-1.0)) * 8.0 + a )
				      );
		}
	}
	else { /* argc==2 */
		if(x>65535.0 || x <0) {
#ifdef PORTUGUES
			printf("ERRO: <micro_value> no pode ser"
			       " negativo ou maior que 65535\n");
#else
			printf("ERROR: <micro_value> cannot be"
			       " negative or greater than 65535\n");
#endif
			return 0;
		}
		printf("<numerator>   = %ld\n", 1L+(((long int)x)/256L));
		printf("<denominator> = %ld\n", 1L+(((long int)x)%256L));
	}

	return 0;
}
