#include "./env.h"
#include <fcntl.h>
#include <unistd.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/signal.h>
#ifdef _LINUX_
# include <sys/ptrace.h>
#endif
#include "./judge.h"
#include "./struct.h"
#include "./log.h"
#include "./get_mmr_u.h"

static int emptysid(char ch) {
	return ch==' '||ch=='\n'||ch=='\t'||ch=='\r'||ch=='\v'||ch=='\f';
}

static int probe_compare(int flen,int fd,const char *std) {
	/* can't judge WA or PE before reading the whole file */
	int length = strlen(std);
	if((flen > length*_CAREFL_) || (flen*_CAREFL_ < length)) {
		return _WRONGA_; /* diff too much */
	}
	char *data = malloc(sizeof(char)*flen+5);
	int rz = 0;
	while(rz < flen) {
		rz += read(fd,data+rz,flen+1);
	}
	data[flen] = '\0'; /* reading from file do not creat a '\0' */
	int peflag = 0;
	int idx = 0;
	int ifx = 0;
	while(std[idx] && data[ifx]) {
		if(std[idx] == data[ifx]) {
			idx++;
			ifx++;
		} else if(emptysid(std[idx])) {
			peflag = 1;
			idx++;
		} else if(emptysid(data[ifx])) {
			peflag = 1;
			ifx++;
		} else {
			free(data);
			return _WRONGA_;
		}
	}
	if(std[idx] == '\0' && data[ifx] == '\0') {
		free(data);
		return peflag?_PRSERR_:_ACCEPT_;
	} else {
		if(std[idx] != '\0') {
			while(emptysid(std[idx])) {
				idx++;
			}
		}
		if(data[ifx] != '\0') {
			while(emptysid(data[ifx])) {
				ifx++;
			}
		}
		if(std[idx] == '\0' && data[ifx] == '\0') {
			free(data);
			return _PRSERR_;
		} else {
			free(data);
			return _WRONGA_;
		}
	}
}

static int run_pro(int lang) {
	if(chroot(".") < 0 || chdir("/") < 0) {
		log_error("[E_JU13]:Change Dir Error");
		/* this is not a fatal error */
	}
	log_write("start judging");
#ifdef _LINUX_
	if(0 > setuid(_NOMUSR_)) {
		log_error("[E_JU14]:SetUid Error");
	}
	/* setresuid(_NOMUSR_,_NOMUSR_,_NOMUSR_); */
	if(0 > ptrace(PTRACE_TRACEME,0,0,0)) {
		log_error("[E_JU15]:Ptrace Error");
		return _FAILURE_;
	}
#endif
	switch(lang) {
		case _ANSI_C_:
		case _GNUCPP_: {
			if(0 > execl("./a.out","./a.out",NULL)) {
				return _FAILURE_;
			}
		}
		default: {
			log_error("[E_JU06]:Unknown Language");
			return _FAILURE_;
		}
	}
	return _FAILURE_;
}

int action_run(struct submit *sub_p,struct problem *prob_p) {
	int status;
	struct rusage res_use;
	struct rlimit res_lmt;
	pid_t pid;
	int limit[] = {
#if 0
		prob_p->mmr_l *1024, /* memory limit */
		/* setrlimit will never caught by wait4 */
#else
# ifdef _LINUX_
		RLIM_INFINITY,
# endif
#endif
		0, /* no core file for debug */
		prob_p->time_l /1000, /* time limit */
		_OUTLMT_*1024,_STKLMT_*1024 /* output limit and stack overflow */
	};
	int res_sig[] = {
#ifdef _LINUX_
		RLIMIT_AS,
#endif
		RLIMIT_CORE,RLIMIT_CPU,RLIMIT_FSIZE,RLIMIT_STACK};
	if(sub_p->r_ptr == NULL) {
		sub_p->r_ptr = malloc(sizeof(struct result));
	}
	struct result *ptr = sub_p->r_ptr;
	ptr->time_u = 0;
	ptr->mmr_u = 0;
	ptr->result = _PENDIN_; /* for all unexcepted exits */
	if(prob_p->std_in) {
		int fd = open("./in",O_WRONLY|O_TRUNC|O_CREAT,_F_MODE_);
		if(fd < 0) {
			log_error("[E_JU07]:Posix Open Error");
			close(fd);
			return _FAILURE_;
		}
		int length = strlen(prob_p->std_in);
		int cnt = _MAXRTY_;
		while(cnt--) {
			if(write(fd,prob_p->std_in,length) != length) {
				ftruncate(fd,0); /* retry rather than try to write the rest */
				sleep(2);
			} else {
				break;
			}
		}
		close(fd);
	}
	if((pid = fork()) < 0) {
		log_error("[E_JU04]:Posix Fork Error");
		return _FAILURE_;
	} else if(pid == 0) {
		/* set resource limits */
		int i;
#ifdef _LINUX_
		int _imax = 5;
#else
		int _imax = 4;
#endif
		for(i=0;i<5;i++) {
			res_lmt.rlim_cur = limit[i];
			res_lmt.rlim_max = limit[i];
			setrlimit(res_sig[i],&res_lmt);
		}
		/* freopen etc. */
		int f_in,f_ot;
		if(prob_p->std_in) {
			f_in = open("./in",O_RDONLY);
		} else {
			f_in = open("/dev/random",O_RDONLY);
		}
		f_ot = open("./out",O_WRONLY|O_TRUNC|O_CREAT,_F_MODE_);
		if(f_in < 0 || f_ot < 0) {
			log_error("[E_JU08]:Posix Open Error");
			close(f_in);
			close(f_ot);
			return _FAILURE_;
		}
		if(0 > dup2(f_in,STDIN_FILENO) || 0 > dup2(f_ot,STDOUT_FILENO)) {
			log_error("[E_JU09]:Duplicate Error");
			close(f_in);
			close(f_ot);
			return _FAILURE_;
		}
		if(run_pro(sub_p->lang) == _FAILURE_) {
			log_error("[E_JU05]:Program Running Error");
			exit(SIGUSR1); /* a never-used signal */
		}
		log_error("[E_JU05]:Program Running Error");
		exit(SIGUSR1);
	} else {
		if(0 > wait4(pid,&status,0,&res_use)) {
			log_error("[E_JU02]:Wait4 Error");
			return _FAILURE_;
		}
		if(WIFSTOPPED(status)) {
		/* calling exec */
#ifdef _LINUX_
			ptrace(PTRACE_SYSCALL,pid,0,0); /* trace it when uses syscalls */
			for(;;) {
				if(0 > wait4(pid,&status,0,&res_use)) {
					log_error("[E_JU16]:Wait4 Error");
					return _FAILURE_;
				}
				if(WIFSTOPPED(status)) {
					/*
					 * SIGTRAP -> Normal Trace Stop
					 * SIGXCPU -> Time Limit Exceeded
					 * SIGXFSZ -> Output Limit Exceeded
					 * SIGSEGV -> RTE: Access Violation
					 *         -> RTE: Stack Overflow
					 * SIGFPE  -> RTE: Integer Divided By Zero
					 */
					int sig = WSTOPSIG(status);
					if(sig == SIGTRAP) {
						/* normally stopped */
						int mmr = get_mmr_u(pid);
						ptr->mmr_u = ptr->mmr_u<mmr?mmr:ptr->mmr_u;
						if(mmr > prob_p->mmr_l) {
							ptr->result = _MEMRLE_;
							ptrace(PTRACE_KILL,pid,0,0);
							break;
						} else {
							ptrace(PTRACE_SYSCALL,pid,0,0);
							continue;
						}
					} else if(sig == SIGXCPU) {
						ptr->result = _TIMELE_;
					} else if(sig == SIGXFSZ) {
						ptr->result = _OUTLME_;
					} else if(sig == SIGSEGV) {
						ptr->result = _RTEACV_;
					} else if(sig == SIGFPE) {
						ptr->result = _RTEDBZ_;
					} else {
						ptr->result = _RTEUKN_;
					}
					ptrace(PTRACE_KILL,pid,0,0);
					break;
				} else if(WIFSIGNALED(status)) { /* will this really happen? */
					int sig = WTERMSIG(status);
					if(sig == SIGXCPU) {
						ptr->result = _TIMELE_;
					} else if(sig == SIGXFSZ) {
						ptr->result = _OUTLME_;
					} else if(sig == SIGSEGV) {
						ptr->result = _RTEACV_;
					} else if(sig == SIGFPE) {
						ptr->result = _RTEDBZ_;
					} else {
						ptr->result = _RTEUKN_;
					}
					ptrace(PTRACE_KILL,pid,0,0); /* actually the process is terminated */
					break;
				} else if(WIFEXITED(status)) {
					/* normally exited */
					if(WEXITSTATUS(status) != SIGUSR1) {
						if(WEXITSTATUS(status)) {
							ptr->result = _RTEUEC_;
						}
					} else {
						ptr->result = _SYSERR_;
						return _FAILURE_;
					}
					break;
				} else {
					log_error("[E_JU17]:Unknown Wait Status");
					return _FAILURE_;
				}
			}
		} else {
			ptr->result = _SYSERR_;
			if(!(WIFEXITED(status))) {
				ptrace(PTRACE_KILL,pid,0,0);
			}
		}
#endif
		/* system time (calls) + user time */
		ptr->time_u = res_use.ru_utime.tv_sec * 1000 + res_use.ru_utime.tv_usec/1000
			+ res_use.ru_stime.tv_sec * 1000 + res_use.ru_stime.tv_usec / 1000;
		/* gnu time-1.7 -> time.c line #345 */
		if(ptr->result == _RTEUKN_ && ptr->time_u >= prob_p->time_l) {
			ptr->result = _TIMELE_;
		} else if(ptr->result == _PENDIN_) {
			/* data comparation */
			struct stat file_stat;
			int fd = open("./out",O_RDONLY);
			if(fd < 0) {
				if(open("./out",O_RDONLY|O_CREAT,_F_MODE_) > 0) {
					if(0 > fstat(fd,&file_stat)) {
						log_error("[E_JU11]:Posix Fstat Error");
					}
					if(file_stat.st_size == 0) {
						/* no output */
						ptr->result = _WRONGA_;
					}
				} else {
					log_error("[E_JU10]:Posix Open Error");
					return _FAILURE_;
				}
			}
			if(0 > fstat(fd,&file_stat)) {
				log_error("[E_JU12]:Posix Fstat Error");
			}
			ptr->result = probe_compare((int)file_stat.st_size,fd,prob_p->std_ot);
			close(fd);
		}
	}
	return _SUCCEED_;
}

#if 0
int main(void) {
	log_init("zlog",0);
	struct submit sub;
	struct problem pro;
	pro.std_in = "1 2\n5 -1\n";
	pro.std_ot = "3\n4\n";
	pro.mmr_l = 1120;
	pro.time_l = 1000;
	sub.sub_id = 0;
	sub.prob_id = 1;
	sub.lang = _ANSI_C_;
	sub.code = malloc(4000);
	FILE *fp = fopen("test.c","r");
	chdir("./p0");
	char ch;
	int idx = 0;
	while((ch = getc(fp))!=EOF) {
		sub.code[idx++] = ch;
	}
	fclose(fp);
	sub.r_ptr = NULL;
	cc_make(&sub);
	if(NULL == sub.r_ptr) {
		action_run(&sub,&pro);
		printf("%d MS\n",sub.r_ptr->time_u);
		printf("%d %d kB\n",sub.r_ptr->result,sub.r_ptr->mmr_u);
	} else {
		printf("%d\n%s",(int)strlen(sub.r_ptr->err_msg),sub.r_ptr->err_msg);
	}
	return 0;
}
#endif