#include "./env.h"
#include <time.h>
#include "./env.h"
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include "./log.h"

static int fd = -1;
static char *buffer = NULL;

static int log_lock(int type) { /* APUE p359 */
	struct flock lock;
	lock.l_type = type;
	lock.l_start = 0;
	lock.l_whence = SEEK_SET;
	lock.l_len = 0;
	if(fcntl(fd,F_SETLK,&lock) != -1) {
		return _SUCCEED_;
	} else {
		return _FAILURE_;
	}
}

int log_init(const char *path,int arg) {
	if((fd = open(path,O_WRONLY|O_APPEND|O_CREAT,_F_MODE_)) < 0) {
		puts("[E_IN01]:Posix Open Error");
		return _FAILURE_;
	}
	if(arg & CLEN_LOG) {
		if(ftruncate(fd,0)) {
			puts("[E_IN02]:Posix Ftruncate Error");
			return _FAILURE_;
		}
	}
	struct stat file_stat;
	buffer = malloc(sizeof(char) * (_BUFFSZ_ + 25));
	if(0 > fstat(fd,&file_stat)) {
		log_error("[E_IN03]:Posix Fstat Error");
	}
#ifdef _VERBOSE_
	if(file_stat.st_size == 0) {
		log_write("log created");
	} else {
		char bufz[80];
		sprintf(bufz,"log found with size: %ldbyte",(long)file_stat.st_size);
		log_write(bufz);
	}
#endif
	return _SUCCEED_;
}

int log_final(void) {
	if(buffer) {
		free(buffer);
	}
	if(fd != -1) {
		close(fd);
	}
	return _SUCCEED_;
}

int log_write(const char *sent) {
#ifdef _VERBOSE_
	time_t lt = time(NULL);
	if(lt != -1) {
		struct tm *tm_p = localtime(&lt);
		if((tm_p==NULL) || 0==strftime(buffer,_TF_LEN_+2,_TF_MSG_,tm_p)) {
			sprintf(buffer,"[--Unknown Time--] ");
		}
	} else {
		sprintf(buffer,"[--Unknown Time--] ");
	}
	int length = strlen(sent);
	if(length > _BUFFSZ_) {
		int sz = _TF_LEN_,wr = 0;
		if(_FAILURE_ == log_lock(F_WRLCK)) {
			return _FAILURE_;
		}
		while(length > wr) {
			snprintf(buffer+sz,_BUFFSZ_-sz,"%s\n",sent+wr);
			int len = strlen(buffer);
			if(len != write(fd,buffer,len)) {
				return _FAILURE_;
			}
			wr += _BUFFSZ_ - sz;
			sz = 0;
		}
		if(_FAILURE_ == log_lock(F_UNLCK)) {
			return _FAILURE_;
		}
	} else {
		sprintf(buffer+_TF_LEN_,"%s\n",sent);
		int len = strlen(buffer);
		if(len != write(fd,buffer,len)) {
			return _FAILURE_;
		}
	}
	return _SUCCEED_;
#endif
}

int log_error(const char *sent) {
	time_t lt = time(NULL);
	if(lt != -1) {
		struct tm *tm_p = localtime(&lt);
		if((tm_p==NULL) || 0==strftime(buffer,_TF_LEN_+2,_TF_ERR_,tm_p)) {
			sprintf(buffer,"<--Unknown Time--> ");
		}
	} else {
		sprintf(buffer,"<--Unknown Time--> ");
	}
	int length = strlen(sent);
	if(length > _BUFFSZ_) {
		int sz = _TF_LEN_,wr = 0;
		if(_FAILURE_ == log_lock(F_WRLCK)) {
			return _FAILURE_;
		}
		while(length > wr) {
			snprintf(buffer+sz,_BUFFSZ_-sz,"%s\n",sent+wr);
			int len = strlen(buffer);
			if(len != write(fd,buffer,len)) {
				return _FAILURE_;
			}
			wr += _BUFFSZ_ - sz;
			sz = 0;
		}
		if(_FAILURE_ == log_lock(F_UNLCK)) {
			return _FAILURE_;
		}
	} else {
		sprintf(buffer+_TF_LEN_,"%s\n",sent);
		int len = strlen(buffer);
		if(len != write(fd,buffer,len)) {
			return _FAILURE_;
		}
	}
	return _SUCCEED_;
}