#include <stdlib.h>
#include <string.h>
#include "_memory.h"
#include "_interp.h"
#include "_parser.h"
#include "_nspace.h"
#ifdef _DEBUG_VS
# include <stdio.h>
# include "_io_inf.h"
#endif

#define IS_NUM(w) ((((w)>='0')&&((w)<='9'))||((w)=='.'))
#define IS_OPR(w) (((w)=='+')||((w)=='-')||((w)=='*')||((w) =='/'))
#define COMP_ABLE(w) (((w)==0)||((w)==3))
#define EPI 0.001
#define SOF 5000
#define SOF_F 50000

int obj_id = 0;
int stack_depth;
static struct _ns_tree *_local = NULL;
struct _i_node* _lambda_create(struct _i_node*);
struct _i_node* _i_node_clone(struct _i_node*);
struct _i_node* _e_node_to_i_node(struct _e_node*);
struct _i_node* _quote_list_clone(struct _i_node*);
struct _i_node* _quote_list_create(struct _e_node*);
void _quote_list_dealloc(struct _i_node*);
int _like_inner_symbol(char*);
int _valid_arg_list(struct _i_node*);

struct _proc {
	struct _ns_tree *local; /* closure */
	struct _i_node *args; /* arg list */
	struct _i_node *proc; /* procedure in _i_node */
};

struct _i_node* _get_i_node(void) {
	struct _i_node *ptr = _alloc(sizeof(struct _i_node));
	ptr->id = obj_id++;
	return ptr;
}

void _destroy_ns(struct _ns_tree *local) {
	if(local) {
		_destroy_ns(local->prev);
		_dealloc_a_ns(local);
	}
}

void _destroy_ns_tree(void) {
	_dealloc(_nihil);
	_destroy_ns(_local);
}

char _is_neg_num(char *str) {
	return (1<strlen(str) && (*str == '-') && (IS_NUM((*(str+1)))));
}

struct _i_node* _arg_list_create(struct _e_node *head) {
	struct _i_node *front = _e_node_to_i_node(head);
	struct _i_node *now = front;
	while(head->next) {
		now->next = _e_node_to_i_node(head->next);
		now = now->next;
		head = head->next;
	}
	now->next = NULL;
	return front;
}

struct _i_node* _arg_list_clone(struct _i_node *head) {
	struct _i_node *front = _i_node_clone(head);
	struct _i_node *now = front;
	while(head->next) {
		now->next = _i_node_clone(head->next);
		now = now->next;
		head = head->next;
	}
	now->next = NULL;
	return front;
}

void _arg_list_dealloc(struct _i_node *front) {
	while(front) {
		_dealloc_i_node(front);
		front = front->next;
	}
}

void _dealloc_i_node(struct _i_node *ptr) {
	switch(ptr->type) {
		case _NUMBER_:
		case _SYMBOL_:
		case _FLOAT__:
		case _BOOLEAN: {
			_dealloc(ptr->data);
			_dealloc(ptr);
			break;
		}
		case _EXPRES_: {
			_dealloc(ptr);
			break;
		}
		case _NIHIL__: {
			if(ptr->data) {
				_dealloc(ptr);
			}
			break;
		}
		case _QUOTE__: {
			if(ptr->data) {
				_quote_list_dealloc((struct _i_node*)(ptr->data));
			}
			_dealloc(ptr);
			break;
		}
		case _LAMBDA_: {
			if(ptr->data) {
				struct _proc *p = (struct _proc*)(ptr->data);
				while(p->proc) {
					_expr_dealloc((struct _e_node*)(p->proc->data));
					struct _i_node *ptr = p->proc->next;
					_dealloc_i_node(p->proc);
					p->proc = ptr;
				}
				_arg_list_dealloc(p->args);
				_reference_ns_sub(p->local);
				_dealloc(p);
				_dealloc(ptr);
			}
			break;
		}
		default: {
		}
	}
}

void* _numberic_parser(char *str,char *type) {
	/* illegal numbers not considered */
	int do_try = 0;
	char neg = 1;
	if((*str) == '-') {
		neg =-1;
		str++;
	}
	while(*str) {
		if(*str=='.') break;
		do_try *=10;
		do_try +=((*str)-'0');
		str++;
	}
	if(*str) {
		str++;
		float fl = do_try;
		float base = 0.1;
		while(*str) {
			fl += base*((*str)-'0');
			base /= 10;
			str++;
		}
		*type = _FLOAT__;
		float *ptr = _alloc(sizeof(float));
		*ptr = neg *fl;
		return (void*)ptr;
	} else {
		*type = _NUMBER_;
		int *ptr = _alloc(sizeof(int));
		*ptr = neg *do_try;
		return (void*)ptr;
	}
}

int _near_enough(float f) {
	int tar = (int)((f>0?0.5:-0.5)+f);
	float var = f-tar;
	if(var<0) var = -var;
	if(var < EPI) {
		return 1;
	} else {
		return 0;
	}
}

void _compress_numeric(struct _i_node *result) { /* _compress_numeric(result); */
	if(result->type == _FLOAT__ && _near_enough(*((float*)(result->data)))) {
		result->type = _NUMBER_;
		float *fl = result->data;
		result->data = _alloc(sizeof(int));
		*(int*)(result->data) = (int)((*fl>0?0.5:-0.5)+*fl);
		_dealloc(fl);
	}
}

struct _i_node* _i_node_proc_clone(struct _i_node *head) {
	struct _i_node *front = NULL;
	struct _i_node *now  = NULL;
	if(head) {
		front = _i_node_clone(head);
		front->data = _expr_duplicate((struct _e_node*)(head->data));
		now = front;
		head = head->next;
	}
	while(head) {
		now->next = _i_node_clone(head);
		now = now->next;
		now->data = _expr_duplicate((struct _e_node*)(head->data));
		head = head->next;
	}
	now->next  = NULL;
	return front;
}

struct _i_node* _i_node_clone(struct _i_node *ptr) {
	struct _i_node *var = _alloc(sizeof(struct _i_node));
	var->type = ptr->type;
	var->next = ptr->next;
	var->id  =  ptr->id;
	switch(ptr->type) {
		case _QUOTE__: {
			var->data = _quote_list_clone((struct _i_node*)(ptr->data));
			break;
		}
		case _LAMBDA_: {
			struct _proc *p = (struct _proc*)(ptr->data);
			struct _proc *n = (struct _proc*)(_alloc(sizeof(struct _proc)));
			n->args = p->args?_arg_list_clone(p->args):NULL;
			n->proc = _i_node_proc_clone(p->proc);
			n->local = p->local; /* add counter */
			_reference_ns_add(n->local);
			var->data = (void*)n;
			break;
		}
		case _EXPRES_: {
			var->data = ptr->data;
			break;
		}
		case _SYMBOL_: {
			var->data = (void*)_str_clone((char*)(ptr->data));
			break;
		}
		case _NUMBER_: {
			var->data = _alloc(sizeof(int));
			*(int*)(var->data) = *(int*)(ptr->data);
			break;
		}
		case _FLOAT__: {
			var->data = _alloc(sizeof(float));
			*(float*)(var->data) = *(float*)(ptr->data);
			break;
		}
		case _BOOLEAN: {
			var->data = _alloc(sizeof(char));
			*(char*)(var->data) = *(char*)(ptr->data);
			break;
		}
		case _NIHIL__: {
			var->data = (void*)_nihil;
			break;
		}
	}
	return var;
}

struct _e_node* _i_node_to_e_node(struct _i_node *i_p) {
	if(i_p->type == _EXPRES_) {
		return _expr_duplicate((struct _e_node*)(i_p->data));
	} else if(i_p->type == _SYMBOL_) {
		struct _e_node *ptr = _alloc(sizeof(struct _e_node));
		ptr->child_e = _str_clone((char*)(i_p->data));
		ptr->child_n = NULL;
		ptr->next = NULL;
		return ptr;
	} else if(i_p->type == _BOOLEAN) {
		struct _e_node *ptr = _alloc(sizeof(struct _e_node));
		ptr->child_e = _str_clone((*(char*)(i_p->data))?"#t":"#f");
		ptr->child_n = NULL;
		ptr->next = NULL;
		return ptr;
	} else if(i_p->type == _NUMBER_ || i_p->type == _FLOAT__) {
		struct _e_node *ptr = _alloc(sizeof(struct _e_node));
		ptr->child_e = _numeric_to_str(i_p);
		ptr->child_n = NULL;
		ptr->next = NULL;
		return ptr;
	} else {
		/* QUOTE, LAMBDA, NAME_SPACE, NIHIL are the types that'll never exist in a
		 * procedure expression, as all of them will be contained in symbols, ex-
		 * pressions, so they're omitted here in the function.
		 */
		return NULL;
	}
}

struct _i_node* _e_node_to_i_node(struct _e_node *e_p) {
	struct _i_node *ptr = _get_i_node();
	if(e_p->child_n) {
		ptr->type = _EXPRES_;
		ptr->data = (void*)( e_p->child_n);
	} else if(e_p->child_e) {
		if((IS_NUM(*(e_p->child_e))) || _is_neg_num(e_p->child_e)) {
			ptr->data = (void*)( _numberic_parser(e_p->child_e,&(ptr->type)));
		} else if(!strcmp(e_p->child_e,"#f") || !strcmp(e_p->child_e,"#t")) {
			ptr->type = _BOOLEAN;
			ptr->data = _alloc(sizeof(char));
			*(char*)(ptr->data) = strcmp(e_p->child_e,"#f")?1:0;
		} else {
			ptr->type = _SYMBOL_;
			ptr->data = (void*) _str_clone(e_p->child_e);
		}
	} else {
		ptr->type = _EXPRES_;
		ptr->data = NULL;
	}
	ptr->next = NULL;
	return ptr;
}

struct _i_node* _expr_rec_interp(struct _e_node*);

int _like_quote_create(char *str) { /* operator of quotes creation */
	return !strcmp(str,"quote") || !strcmp(str,"cons") || !strcmp(str,"list");
}

int _like_quote_calc(char *str) {
	return !strcmp(str,"car") || !strcmp(str,"cdr");
}

int _like_bool_oper(char *str) {
	char ch = *str;
	if((strlen(str) == 1)) {
		if(ch=='>'||ch=='<') {
			return 1;
		}
	} else if((strlen(str) == 2)&&(*(str+1))=='=') {
		if((ch=='>'||ch=='<')) {
			return 1;
		}
	} else if(!strcmp("eq?",str) || !strcmp("equal?",str) || !strcmp("null?",str)) {
		return 1;
	} else if(!strcmp("and",str) || !strcmp("or",str) || !strcmp("not",str)) {
		return 1;
	}
	return 0;
}

int _like_inner_oper(char *str) {
	if((strlen(str) == 1) && IS_OPR(*str)) {
		return 1;
	}
	return 0;
}

int _like_cond_expression(char *str) {
	if( !strcmp(str,"cond") || !strcmp(str,"if") ) {
		return 1;
	}
	return 0;
}

int _like_symbol_def(char *str) {
	if( !strcmp(str,"define") || !strcmp(str,"set!") ) {
		return 1;
	}
	return 0;
}

int _like_procedure(char *str) {
	if(!strcmp(str,"lambda")) {
		return 1;
	} else {
		return 0;
	}
}

int _like_variable(char *str) {
	return !strcmp(str,"let");
}

struct _i_node* _swap_i_node(struct _i_node *element) {
	struct _i_node *result = _expr_rec_interp((struct _e_node*)(element->data));
	if(result->type == _NIHIL__) result = _i_node_clone(_nihil);
	result->next = element->next;
	_dealloc_i_node(element);
	return result;
}

struct _i_node* _swap_find(struct _i_node *front) {
	struct _i_node *trai = NULL;
	while(front->type == _SYMBOL_ && (trai = _select_from_ns(_local,(char*)(front->data)))->type != _NIHIL__) {
		if(trai->type == _EXPRES_) {
			trai = _swap_i_node(trai);
		} else {
			trai->next = front->next;
			_dealloc_i_node(front);
			front = _i_node_clone(trai);
		}
	}
	return front;
}

struct _i_node* _swap_find_nil(struct _i_node *front) {
	int sig = 0;
	struct _i_node *trai = NULL;
	while(front->type == _SYMBOL_ && (trai = _select_from_ns(_local,(char*)(front->data)))->type != _NIHIL__) {
		sig = 1;
		if(trai->type == _EXPRES_) {
			trai = _swap_i_node(trai);
		} else {
			trai->next = front->next;
			_dealloc_i_node(front);
			front = _i_node_clone(trai);
		}
	}
	if(sig) {
		return front;
	} else {
		return _nihil;
	}
}

struct _i_node* _cond_clause_parser(struct _e_node *clause) {
	struct _i_node *cond = _e_node_to_i_node(clause);
	clause = clause->next;
	if(!clause) {
		_dealloc_i_node(cond);
		return NULL;
	}
	struct _i_node *result = _e_node_to_i_node(clause);
	struct _i_node *final = NULL;
	if(clause->next) {
		/* error */
		final = NULL;
	} else {
		if(cond->type == _EXPRES_) {
			cond = _swap_i_node(cond);
		}
		if(cond->type == _SYMBOL_ && !strcmp("else",(char*)cond->data)) {
			cond->type = _FLOAT__;
		}
		if(cond->type == _SYMBOL_) {
			struct _i_node *trai = _swap_find_nil(cond);
			if(trai->type == _NIHIL__) {
				_dealloc_i_node(cond);
			}
			cond = trai;
		}
		if(cond->type == _NIHIL__ || (cond->type == _BOOLEAN && *((char*)(cond->data)) == 0)) {
			final = NULL;
		} else {
			if(result->type == _EXPRES_) {
				final = _expr_rec_interp((struct _e_node*)(result->data));
			} else {
				final = _i_node_clone((struct _i_node*)result);
			}
			if(result->type == _SYMBOL_ && !_like_inner_oper((char*)(result->data))) {
				struct _i_node *trai = _swap_find_nil(result);
				if(trai->type == _NIHIL__) {
					_dealloc_i_node(result);
				}
				result = trai;
			}
		}
	}
	_dealloc_i_node(cond);
	_dealloc_i_node(result);
	return final;
}

struct _i_node* _inner_bool(struct _i_node *front) {
	char *oper = (char*)(front->data);
	struct _i_node *element = front->next;
	struct _i_node *result = _get_i_node();
	result->type = _BOOLEAN;
	result->next = NULL;
	result->data = _alloc(sizeof(char));
	if(!strcmp(oper,">")) {
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type))) {
			_dealloc_i_node(result);
			return _nihil;
		} else { /* switch to next element */
			front->next = element;
			element = element->next;
			front = front->next;
		}
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type)) || element->next) {
			_dealloc_i_node(result);
			return _nihil;
		} /* return front > element; */
		front->next = element;
		if((front->type == _FLOAT__) || (element->type == _FLOAT__)) {
			float a = 0,b = 0;
			if(_FLOAT__ == front->type) {
				a = *(float*)(front->data);
			} else if(_NUMBER_ == front->type) {
				a = (int)(*(int*)(front->data));
			}
			if(_FLOAT__ == element->type) {
				b = *(float*)(element->data);
			} else if(_NUMBER_ == element->type) {
				b = (int)(*(int*)(element->data));
			}
			if(a>b) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		} else {
			if( *(int*)(front->data) > *(int*)(element->data) ) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		}
	} else if(!strcmp(oper,"<")) {
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type))) {
			_dealloc_i_node(result);
			return _nihil;
		} else { /* switch to next element */
			front->next = element;
			element = element->next;
			front = front->next;
		}
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type)) || element->next) {
			_dealloc_i_node(result);
			return _nihil;
		} /* return front > element; */
		front->next = element;
		if((front->type == _FLOAT__) || (element->type == _FLOAT__)) {
			float a = 0,b = 0;
			if(_FLOAT__ == front->type) {
				a = *(float*)(front->data);
			} else if(_NUMBER_ == front->type) {
				a = (int)(*(int*)(front->data));
			}
			if(_FLOAT__ == element->type) {
				b = *(float*)(element->data);
			} else if(_NUMBER_ == element->type) {
				b = (int)(*(int*)(element->data));
			}
			if(a<b) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		} else {
			if( *(int*)(front->data) < *(int*)(element->data) ) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		}
	} else if(!strcmp(oper,"<=")) {
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type))) {
			_dealloc_i_node(result);
			return _nihil;
		} else { /* switch to next element */
			front->next = element;
			element = element->next;
			front = front->next;
		}
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type)) || element->next) {
			_dealloc_i_node(result);
			return _nihil;
		} /* return front > element; */
		front->next = element;
		if((front->type == _FLOAT__) || (element->type == _FLOAT__)) {
			float a = 0,b = 0;
			if(_FLOAT__ == front->type) {
				a = *(float*)(front->data);
			} else if(_NUMBER_ == front->type) {
				a = (int)(*(int*)(front->data));
			}
			if(_FLOAT__ == element->type) {
				b = *(float*)(element->data);
			} else if(_NUMBER_ == element->type) {
				b = (int)(*(int*)(element->data));
			}
			if(a<=b) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		} else {
			if( *(int*)(front->data) <= *(int*)(element->data) ) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		}
	} else if(!strcmp(oper,">=")) {
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type))) {
			_dealloc_i_node(result);
			return _nihil;
		} else { /* switch to next element */
			front->next = element;
			element = element->next;
			front = front->next;
		}
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		if(!(COMP_ABLE(element->type)) || element->next) {
			_dealloc_i_node(result);
			return _nihil;
		} /* return front > element; */
		front->next = element;
		if((front->type == _FLOAT__) || (element->type == _FLOAT__)) {
			float a = 0,b = 0;
			if(_FLOAT__ == front->type) {
				a = *(float*)(front->data);
			} else if(_NUMBER_ == front->type) {
				a = (int)(*(int*)(front->data));
			}
			if(_FLOAT__ == element->type) {
				b = *(float*)(element->data);
			} else if(_NUMBER_ == element->type) {
				b = (int)(*(int*)(element->data));
			}
			if(a>=b) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		} else {
			if( *(int*)(front->data) >= *(int*)(element->data) ) {
				*(char*)(result->data) = 1;
			} else {
				*(char*)(result->data) = 0;
			}
		}
	} else if(!strcmp(oper,"and")) {
		*(char*)(result->data) = 1;
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(element->type != _NIHIL__) front->next = element;
			}
			if(element->type != _BOOLEAN && element->type != _NIHIL__) {
			/* treat as true */
			} else {
				if(element->type == _NIHIL__ || *(char*)(element->data) == 0) {
					*(char*)(result->data) = 0;
					break;
				}
			}
			front = element;
			element = front->next;
		}
	} else if(!strcmp(oper,"or")) {
		*(char*)(result->data) = 0;
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(element->type != _NIHIL__) front->next = element;
			}
			if(element->type == _NIHIL__) {
			} else {
				if(element->type != _BOOLEAN || *(char*)(element->data) == 1) {
					*(char*)(result->data) = 1;
					break;
				}
			}
			front = element;
			element = front->next;
		}
	} else if(!strcmp(oper,"not")) {
		*(char*)(result->data) = 1;
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(element->type != _NIHIL__) front->next = element;
			}
			if(element->type == _NIHIL__) {
			} else {
				if(element->type != _BOOLEAN || *(char*)(element->data) == 1) {
					*(char*)(result->data) = 0;
					break;
				}
			}
			front = element;
			element = front->next;
		}
	} else if(!strcmp(oper,"null?")) {
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
		}
		*(char*)(result->data) = (element->type == _NIHIL__)?1:0;
	} else if(!strcmp(oper,"eq?")) { /* if they're pointer the same thing */
		*(char*)(result->data) = 1;
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
			if(_NIHIL__ == element->type) {
				struct _i_node *temp = _i_node_clone(_nihil);
				temp->next = front->next->next;
				_dealloc_i_node(front->next);
				element = temp;
			}
			front->next = element;
		}
		element = element->next;
		front = front->next;
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__==front->type) {
					struct _i_node *temp = _i_node_clone(_nihil);
					temp->next = front->next->next;
					_dealloc_i_node(front->next);
					element = temp;
				}
				front->next = element;
			}
			if(front->type == element->type) {
				if(front->id != element->id) {
					*(char*)(result->data) = 0;
				}
			} else {
				*(char*)(result->data) = 0;
				break;
			}
			front = front->next;
			element = element->next;
		}
	} else if(!strcmp(oper,"equal?")) { /* if same value */
		*(char*)(result->data) = 1;
		if(element->type == _EXPRES_) {
			element = _swap_i_node(element);
			front->next = element;
		}
		if(element->type == _SYMBOL_) {
			element = _swap_find_nil(element);
			if(_NIHIL__ == element->type) {
				struct _i_node *temp = _i_node_clone(_nihil);
				temp->next = front->next->next;
				_dealloc_i_node(front->next);
				element = temp;
			}
			front->next = element;
		}
		element = element->next;
		front = front->next;
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__==front->type) {
					struct _i_node *temp = _i_node_clone(_nihil);
					temp->next = front->next->next;
					_dealloc_i_node(front->next);
					element = temp;
				}
				front->next = element;
			}
			/* return front == element; */
			if(front->type == element->type) {
				if(front->type == _EXPRES_) {
					/* error */
					*(char*)(result->data) = 0;
					break;
				} else if(front->type == _SYMBOL_) {
					if(strcmp((char*)(front->data),(char*)(element->data))) {
						*(char*)(result->data) = 0;
						break;
					}
				} else if(front->type == _NUMBER_) {
					if( *(int*)(front->data) != *(int*)(element->data) ) {
						*(char*)(result->data) = 0;
						break;
					}
				} else if(front->type == _FLOAT__) {
					if( *(float*)(front->data) != *(float*)(element->data) ) {
						*(char*)(result->data) = 0;
						break;
					}
				} else if(front->type == _BOOLEAN) {
					if( *(char*)(front->data) != *(char*)(element->data) ) {
						*(char*)(result->data) = 0;
						break;
					}
				} /* nihil is always equal to nihil */
			} else {
				*(char*)(result->data) = 0;
				break;
			}
			front = front->next;
			element = element->next;
		}
	}
	return result;
}

struct _i_node* _inner_calc(struct _i_node *front) {
	char *oper = (char*)(front->data);
	struct _i_node *element = front->next;
	if(!element) return _nihil; /* no calculation elements */
	if(element->type == _EXPRES_) {
		element = _swap_i_node(element);
		front->next = element;
	}
	if(element->type == _SYMBOL_) {
		element = _swap_find_nil(element);
		if(_NIHIL__ == element->type) {
			return _nihil;
		}
		front->next = element;
	}
	struct _i_node *result = _get_i_node();
	result->next = NULL;
	front = front->next;
	if(element->type == _NUMBER_) {
		result->type = _NUMBER_;
		result->data = _alloc(sizeof(int));
		*(int*)(result->data) = *(int*)(element->data);
	} else if(element->type == _FLOAT__) {
		result->type = _FLOAT__;
		result->data = _alloc(sizeof(float));
		*(float*)(result->data) = *(float*)(element->data);
	} else {
		_dealloc_i_node(result);
		return _nihil;
	}
	element = element->next;
	if((*oper) == '+') {
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__ == element->type) {
					_dealloc_i_node(result);
					return _nihil;
				}
				front->next = element;
			}
			front = front->next;
			if(element->type == _NUMBER_) {
				if(result->type == _NUMBER_) {
					*(int*)(result->data) += *(int*)(element->data);
				} else if(result->type == _FLOAT__) {
					*(float*)(result->data) += *(int*)(element->data);
				}
			} else if(element->type == _FLOAT__) {
				if(result->type == _NUMBER_) {
					float *fl = _alloc(sizeof(float));
					*fl = (float)(*(int*)(result->data));
					_dealloc(result->data);
					result->data = fl;
					result->type = _FLOAT__;
				}
				if(result->type == _FLOAT__) {
					*(float*)(result->data) += *(float*)(element->data);
				}
			} else {
				_dealloc_i_node(result);
				return _nihil;
			}
			element = element->next;
		}
	} else if((*oper) == '-') {
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__ == element->type) {
					_dealloc_i_node(result);
					return _nihil;
				}
				front->next = element;
			}
			front = front->next;
			if(element->type == _NUMBER_) {
				if(result->type == _NUMBER_) {
					*(int*)(result->data) -= *(int*)(element->data);
				} else if(result->type == _FLOAT__) {
					*(float*)(result->data) -= *(int*)(element->data);
				}
			} else if(element->type == _FLOAT__) {
				if(result->type == _NUMBER_) {
					float *fl = _alloc(sizeof(float));
					*fl = (float)(*(int*)(result->data));
					_dealloc(result->data);
					result->data = fl;
					result->type = _FLOAT__;
				}
				if(result->type == _FLOAT__) {
					*(float*)(result->data) -= *(float*)(element->data);
				}
			} else {
				_dealloc_i_node(result);
				return _nihil;
			}
			element = element->next;
		}
	} else if((*oper) == '*') {
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__ == element->type) {
					_dealloc_i_node(result);
					return _nihil;
				}
				front->next = element;
			}
			front = front->next;
			if(element->type == _NUMBER_) {
				if(result->type == _NUMBER_) {
					*(int*)(result->data) *= *(int*)(element->data);
				} else if(result->type == _FLOAT__) {
					*(float*)(result->data) *= *(int*)(element->data);
				}
			} else if(element->type == _FLOAT__) {
				if(result->type == _NUMBER_) {
					float *fl = _alloc(sizeof(float));
					*fl = (float)(*(int*)(result->data));
					_dealloc(result->data);
					result->data = fl;
					result->type = _FLOAT__;
				}
				if(result->type == _FLOAT__) {
					*(float*)(result->data) *= *(float*)(element->data);
				}
			} else {
				_dealloc_i_node(result);
				return _nihil;
			}
			element = element->next;
		}
		_compress_numeric(result);
	} else if((*oper) == '/') {
		while(element) {
			if(element->type == _EXPRES_) {
				element = _swap_i_node(element);
				front->next = element;
			}
			if(element->type == _SYMBOL_) {
				element = _swap_find_nil(element);
				if(_NIHIL__ == element->type) {
					_dealloc_i_node(result);
					return _nihil;
				}
				front->next = element;
			}
			front = front->next;
			if(element->type == _NUMBER_) {
				if(result->type == _NUMBER_) {
					if(!(*(int*)(result->data) % *(int*)(element->data))) {
						*(int*)(result->data) /= *(int*)(element->data);
					} else {
						float *fl = _alloc(sizeof(float));
						*fl = (float)(*(int*)(result->data));
						_dealloc(result->data);
						result->data = fl;
						result->type = _FLOAT__;
						*(float*)(result->data) /= *(int*)(element->data);
					}
				} else if(result->type == _FLOAT__) {
					*(float*)(result->data) /= *(int*)(element->data);
				}
			} else if(element->type == _FLOAT__) {
				if(result->type == _NUMBER_) {
					float *fl = _alloc(sizeof(float));
					*fl = (float)(*(int*)(result->data));
					_dealloc(result->data);
					result->data = fl;
					result->type = _FLOAT__;
				}
				if(result->type == _FLOAT__) {
					*(float*)(result->data) /= *(float*)(element->data);
				}
			} else {
				_dealloc_i_node(result);
				return _nihil;
			}
			element = element->next;
		}
		_compress_numeric(result);
	}
	return result;
}

struct _i_node* _cond_parser(struct _i_node *front) {
	char *symbol = (char*)(front->data);
	if(!strcmp(symbol,"if")) {
		struct _i_node *cond = front->next;
		if(!cond) {
			return _nihil;
		} else {
			if(cond->type == _EXPRES_) {
				cond = _swap_i_node(cond);
				front->next = cond;
			}
			if(cond->type == _SYMBOL_) {
				struct _i_node *trai = _swap_find_nil(cond);
				if(trai->type == _NIHIL__) {
					trai = _i_node_clone(_nihil);
					trai->next = cond->next;
					_dealloc_i_node(cond);
				}
				cond = trai;
				front->next = cond; /* once there's a fucking bug that '==' is used here */
			}
			char condition = 1;
			if(cond->type == _NIHIL__ || (cond->type == _BOOLEAN && *((char*)(cond->data)) == 0)) condition = 0;
			if(condition) {
				if(cond->next) {
					if(cond->next->type == _EXPRES_) {
						struct _i_node *ptr = _expr_rec_interp((struct _e_node*)(cond->next->data));
						if(ptr->type == _SYMBOL_ && !_like_inner_symbol((char*)(ptr->data))) {
							struct _i_node *rtr = _swap_find_nil(ptr);
							if(rtr->type == _NIHIL__) {
								_dealloc_i_node(ptr);
								return _nihil;
							}
						}
						return ptr;
					} else {
						struct _i_node *ptr = _i_node_clone(cond->next);
						if(ptr->type == _SYMBOL_ && !_like_inner_symbol((char*)(ptr->data))) {
							struct _i_node *rtr = _swap_find_nil(ptr);
							if(rtr->type == _NIHIL__) {
								_dealloc_i_node(ptr);
								return _nihil;
							}
						}
						return ptr;
					}
				} else {
					return _nihil;
				}
			} else {
				cond = cond->next;
				if(cond) {
					if(cond->next) {
						if(cond->next->type == _EXPRES_) {
							struct _i_node *ptr = _expr_rec_interp((struct _e_node*)(cond->next->data));
							if(ptr->type == _SYMBOL_ && !_like_inner_symbol((char*)(ptr->data))) {
								struct _i_node *rtr = _swap_find_nil(ptr);
								if(rtr->type == _NIHIL__) {
									_dealloc_i_node(ptr);
									return _nihil;
								}
							}
							return ptr;
						} else {
							struct _i_node *ptr = _i_node_clone(cond->next);
							if(ptr->type == _SYMBOL_ && !_like_inner_symbol((char*)(ptr->data))) {
								struct _i_node *rtr = _swap_find_nil(ptr);
								if(rtr->type == _NIHIL__) {
									_dealloc_i_node(ptr);
									return _nihil;
								}
							}
							return ptr;
						}
					} else {
						return _nihil;
					}
				} else {
					return _nihil;
				}
			}
		}
	} else {
		struct _i_node *iter = front->next;
		while(iter) {
			if(iter->type == _EXPRES_) {
				struct _i_node *ptr = _cond_clause_parser((struct _e_node*)(iter->data));
				if(ptr) {
					return ptr;
				} else {
					iter = iter->next;
				}
			} else {
				return _nihil;
			}
		}
		return _nihil;
	}
}

int _like_inner_symbol(char *str) {
	return !( strcmp(str,"cond") && strcmp(str,"else") && strcmp(str,"if") && strcmp(str,"#t") && strcmp(str,"#f") && strcmp(str,"define") && strcmp(str,"lambda") && strcmp(str,"+") && strcmp(str,"-") && strcmp(str,"*") && strcmp(str,"/") && strcmp(str,">") && strcmp(str,"<") && strcmp(str,">=") && strcmp(str,"<=") && strcmp(str,"eq?") && strcmp(str,"equal?") && strcmp(str,"and") && strcmp(str,"or") && strcmp(str,"not") && strcmp(str,"lambda") && strcmp(str,"set!") && strcmp(str,"null?") && strcmp(str,"let") && strcmp(str,"quote") && strcmp(str,"car") && strcmp(str,"cdr") && strcmp(str,"cons") && strcmp(str,"eval") );
}

struct _i_node* _variable_proc(struct _i_node *front) {
	struct _i_node *ptr = front->next;
	if(ptr->type == _EXPRES_ && ptr->next) {
		struct _i_node *procedure = ptr->next;
		struct _i_node *variables = _arg_list_create((struct _e_node*)(ptr->data));
		struct _i_node *i_variabs  = variables;
		struct _ns_tree *_child = _create_child_ns(_local);
		while(variables) {
			if(variables->type == _EXPRES_) {
				struct _i_node *pair = _arg_list_create((struct _e_node*)(variables->data));
				if(pair && pair->next && !(pair->next->next)) {
					if(pair->type == _EXPRES_) {
						pair = _swap_i_node(pair);
					}
					if(pair->type == _SYMBOL_) {
						if(pair->next->type == _EXPRES_) {
							pair->next = _swap_i_node(pair->next);
						}
						int insert = 1;
						if(pair->next->type == _SYMBOL_) {
							struct _i_node *trai = _swap_find_nil(pair->next);
							if(trai->type != _NIHIL__) {
								pair->next = trai;
							} else {
								insert = 0;
							}
						}
						if(insert && pair->next->type != _EXPRES_ && pair->next->type != _NIHIL__) {
							_insert_into_ns(_child,(char*)(pair->data),pair->next);
						}
					}
				}
				_arg_list_dealloc(pair);
			}
			variables = variables->next;
		}
		struct _ns_tree *local_save = _local;
		_local = _child; /* chroot */
		_reference_ns_add(_local);
		struct _i_node *result = NULL;
		struct _i_node *proc = _lambda_create(procedure);
		struct _i_node *p = proc;
		while(p) {
			if(result) _dealloc_i_node(result);
			result = _expr_rec_interp((struct _e_node*)(p->data));
			p = p->next;
		}
		p = proc;
		while(p) {
			_expr_dealloc((struct _e_node*)(p->data));
			struct _i_node *ptr = p->next;
			_dealloc_i_node(p);
			p = ptr;
		}
		_reference_ns_sub(_local);
		_local = local_save;
		_arg_list_dealloc(i_variabs);
		return result;
	} else {
		return _nihil;
	}
}

struct _i_node* _lambda_create(struct _i_node *front) {
	struct _e_node *proc = NULL;
	if(!(proc = _i_node_to_e_node(front))) { /* to fit all kinds of elements */
		return NULL;
	}
	struct _i_node *_proc = _get_i_node();
	_proc->type = _EXPRES_;
	_proc->data = (void*)proc;
	_proc->next = NULL;
	front = front->next;
	struct _i_node *n_proc = _proc;
	while(front) {
		if(!(proc = _i_node_to_e_node(front))) {
			front = front->next;
			continue;
		}
		n_proc->next = _get_i_node();
		n_proc = n_proc->next;
		n_proc->type = _EXPRES_;
		n_proc->data = (void*)proc;
		n_proc->next = NULL;
		front = front->next;
	}
	return _proc;
}

struct _i_node* _symbol_insert(struct _i_node *front) {
	int set_ = !strcmp(front->data,"set!");
	if(front->next && front->next->type == _EXPRES_) { /* procedure add */
		struct _i_node *element = front->next;
		if(!(element->next)) { /* no procedure */
			return _nihil;
		} else {
			struct _i_node *args = _arg_list_create((struct _e_node*)(element->data));
			if(!(_valid_arg_list(args))) {
				_arg_list_dealloc(args);
				return _nihil;
			}
			char *name = (char*)(args->data);
			if(_like_inner_symbol(name) || (set_ && _NIHIL__ == (_select_from_ns(_local,name))->type )) { /* not a valid name for atom? */
				_arg_list_dealloc(args);
				return _nihil;
			}
			struct _i_node *procedure = _lambda_create(element->next);
			if(!procedure) {
				_arg_list_dealloc(args);
				return _nihil;
			}
			struct _proc *lambda = _alloc(sizeof(struct _proc));
			lambda->proc = procedure;
			lambda->args = args->next;
			lambda->local = _local;
			_reference_ns_add(_local);
			struct _i_node *result  = _get_i_node();
			result->type = _LAMBDA_;
			result->data = (void*)lambda;
			result->next = NULL;
			_insert_into_ns(_local,name,result);
			_dealloc_i_node(args); /* real name before args */
			_dealloc_i_node(result);
		}
	}
	if(front->next && front->next->type == _SYMBOL_) {
		char *name = (char*)(front->next->data);
		if(_like_inner_symbol(name) || (set_ && _NIHIL__ == (_select_from_ns(_local,name))->type )) {
			return _nihil;
		}
		front = front->next;
		struct _i_node *element = front->next;
		if(!element || element->next) {
			return _nihil;
		}
		if(element->type == _EXPRES_) {
			front->next = _swap_i_node(element);
			element = front->next;
		}
		if(element->type == _SYMBOL_) {
			struct _i_node *temp = _swap_find_nil(element);
			if(temp->type == _NIHIL__) {
				_delete_from_ns(_local,name);
				return _nihil;
			}
			element = temp;
		}
		_insert_into_ns(_local,name,element);
		return _nihil; /* should be a nil~ */
	} else {
		return _nihil;
	}
	return _nihil;
}

int _valid_arg_list(struct _i_node *list) {
	while(list) {
		if(list->type != _SYMBOL_) {
			return 0;
		}
		list = list->next;
	}
	return 1;
}

struct _i_node* _procedure_create(struct _i_node *front) {
	if(front->next && _EXPRES_ == front->next->type) {
		struct _i_node *args = _arg_list_create((struct _e_node*)(front->next->data));
		front = front->next;
		if(_valid_arg_list(args) && front->next) {
			struct _i_node *procedure = _lambda_create(front->next);
			if(!procedure) {
				_arg_list_dealloc(args);
				return _nihil;
			}
			struct _proc *lambda = _alloc(sizeof(struct _proc));
			lambda->proc = procedure;
			lambda->args = args;
			lambda->local = _local;
			_reference_ns_add(_local);
			struct _i_node *result  = _get_i_node();
			result->type = _LAMBDA_;
			result->data = (void*)lambda;
			result->next = NULL;
			return result;
		} else {
			return _nihil;
		}
	} else {
		return _nihil;
	}
}

void _quote_list_dealloc(struct _i_node *ptr) {
	struct _i_node *now = ptr;
	while(now) {
		if(now->type == _QUOTE__) {
			_quote_list_dealloc((struct _i_node*)(now->data));
			now->data = NULL;
		}
		now = now->next;
	}
	_arg_list_dealloc(ptr);
}

struct _i_node* _quote_list_clone(struct _i_node *ptr) {
	struct _i_node *front = _arg_list_clone(ptr);
	struct _i_node *now = front;
	while(now) {
		if(now->type == _QUOTE__) {
			now->data = _quote_list_clone((struct _i_node*)(now->data));
		}
		now = now->next;
	}
	return front;
}

struct _i_node* _quote_list_create(struct _e_node *ptr) {
	struct _i_node *front = _arg_list_create(ptr);
	struct _i_node *now = front;
	while(now) {
		if(now->type == _EXPRES_) { /* do swap */
			now->data = _quote_list_create((struct _e_node*)(now->data));
			now->type = _QUOTE__;
		}
		now = now->next;
	}
	return front;
}

struct _i_node* _quote_fetch_end(struct _i_node *quote) {
	struct _i_node *element = (struct _i_node*)(quote->data);
	if(element->next) {
		return NULL;
	} else {
		return element;
	}
}

struct _i_node* _quote_create(struct _i_node *front) { /* do 'cons' 'quote' */
	char *str = (char*)(front->data);
	if(!strcmp(str,"quote")) {
		struct _i_node *element = front->next;
		if(!element) {
			return _nihil;
		} else {
			if(element->type == _QUOTE__) {
				return _i_node_clone(element);
			} else if(element->type == _EXPRES_) {
				if(!element->data) return _nihil;
				struct _i_node *ptr = _get_i_node();
				ptr->next = NULL;
				ptr->data = (void*)_quote_list_create((struct _e_node*)(element->data));
				ptr->type = _QUOTE__;
				return ptr;
			} else {
				struct _i_node *ptr = _get_i_node();
				struct _i_node *data = _i_node_clone(element);
				data->next = NULL;
				ptr->next = NULL;
				ptr->data = data;
				ptr->type = _QUOTE__;
				return ptr;
			}
		}
	} else if(!strcmp(str,"cons")) { /* bug @    'cons 1 nil'   'cons 1 2' <leaks> */
		if(!front->next || !front->next->next || front->next->next->next) return _nihil;
		struct _i_node *left = front->next;
		int nil_l = 0;
		int nil_r = 0;
		if(left->next->type == _EXPRES_) left->next = _swap_i_node(left->next);
		if(left->next->type == _NIHIL__) nil_r = 1;
		if(left->next->type == _SYMBOL_ && !_like_inner_symbol((char*)(left->next->data))) {
			struct _i_node *temp = _swap_find_nil(left->next);
			if(temp->type == _NIHIL__) {
				nil_r = 1;
			} else {
				left->next = temp;
			}
		}
		struct _i_node *right = left->next;
		if(front->next->type == _EXPRES_) front->next = _swap_i_node(front->next);
		if(front->next->type == _NIHIL__) nil_l = 1;
		if(front->next->type == _SYMBOL_ && !_like_inner_symbol((char*)(front->next->data))) {
			struct _i_node *temp = _swap_find_nil(front->next);
			if(temp->type == _NIHIL__) {
				nil_l = 1;
			} else {
				front->next = temp;
			}
		}
		left = front->next;
		if(!nil_l && left->type == _QUOTE__) {
			struct _i_node *real = _quote_fetch_end(left);
			if(real) left = real;
		}
		if(nil_l) {
			if(!nil_r) {
				return _i_node_clone(right);
			} else {
				return _nihil;
			}
		}
		struct _i_node *result = _get_i_node();
		result->type = _QUOTE__;
		result->next = NULL;
		result->data = _i_node_clone(left); /* never iter and append to its tail */
		struct _i_node *iter = (struct _i_node*)(result->data);
		if(nil_r) {
			iter->next = NULL;
		} else if(right->type == _QUOTE__) {
			iter->next = _quote_list_clone((struct _i_node*)(right->data)); /* open it */
		} else {
			struct _i_node *ptr = _i_node_clone(right);
			ptr->next = NULL;
			iter->next = ptr;
		}
		return result;
	} else if(!strcmp(str,"list")) {
		int _at_least_one = 0;
		struct _i_node *result = _get_i_node();
		struct _i_node *iter = NULL;
		result->type = _QUOTE__;
		result->next = NULL;
		result->data = NULL;
		while(front->next) {
			int is_nil = 0;
			if(front->next->type == _EXPRES_) front->next = _swap_i_node(front->next);
			if(front->next->type == _NIHIL__) is_nil = 1;
			if(front->next->type == _SYMBOL_ && !_like_inner_symbol((char*)(front->next->data))) {
				struct _i_node *temp = _swap_find_nil(front->next);
				if(temp->type == _NIHIL__) {
					is_nil = 1;
				} else {
					front->next = temp;
				}
			}
			if(!is_nil) {
				struct _i_node *element = front->next;
				if(element->type == _QUOTE__) {
					struct _i_node *trai = _quote_fetch_end(element);
					if(trai) {
						element = trai;
					}
				}
				if(_at_least_one) {
					iter->next = _i_node_clone(element);
					iter = iter->next;
					iter->next = NULL;
				} else {
					result->data = _i_node_clone(element);
					iter = (struct _i_node*)(result->data);
					iter->next = NULL;
					_at_least_one = 1;
				}
			}
			front = front->next;
		}
		if(!_at_least_one) {
			_dealloc_i_node(result);
			return _nihil;
		} else {
			return result;
		}
	} else {
		return _nihil;
	}
}

struct _i_node* _quote_calc(struct _i_node *front) {
	char *str = (char*)(front->data);
	if(!strcmp(str,"car")) {
		struct _i_node *element = front->next;
		if(element->type == _EXPRES_) {
			front->next = _swap_i_node(element);
			element = front->next;
		}
		if(element->type == _SYMBOL_) {
			struct _i_node *temp = _swap_find_nil(element);
			if(temp->type == _NIHIL__) {
				return _nihil;
			}
			front->next = temp;
			element = temp;
		}
		if(element->type != _QUOTE__) return _i_node_clone(element);
		front = (struct _i_node*)(element->data);
		if(!front) return _nihil;
		struct _i_node *result = _i_node_clone(front);
		result->next = NULL;
		return result;
	} else if(!strcmp(str,"cdr")) {
		struct _i_node *element = front->next;
		if(element->type == _EXPRES_) {
			front->next = _swap_i_node(element);
			element = front->next;
		}
		if(element->type == _SYMBOL_) {
			struct _i_node *temp = _swap_find_nil(element);
			if(temp->type == _NIHIL__) {
				return _nihil;
			}
			front->next = temp;
			element = temp;
		}
		if(element->type != _QUOTE__) return _nihil;
		front = (struct _i_node*)(element->data);
		if(!front || !(front->next)) return _nihil;
		front = front->next;
		struct _i_node *result = NULL;
		if(front->next) {
			result = _get_i_node();
			result->data = _quote_list_clone(front);
			result->type = _QUOTE__;
			result->next = NULL;
		} else {
			result = _i_node_clone(front);
			result->next = NULL;
		}
		return result;
	} else {
		return NULL;
	}
}

struct _i_node* _base_execute(struct _i_node *front) {
	char *f_element = (char*)(front->data);
	struct _i_node *result = NULL;
	if(_like_inner_oper(f_element)) {
		result = _inner_calc(front);
	} else if(_like_bool_oper(f_element)) {
		result = _inner_bool(front);
	} else if(_like_cond_expression(f_element)) {
		result = _cond_parser(front);
	} else if(_like_symbol_def(f_element)) {
		result = _symbol_insert(front);
	} else if(_like_procedure(f_element)) {
		result = _procedure_create(front);
	} else if(_like_variable(f_element)) {
		result = _variable_proc(front);
	} else if(_like_quote_create(f_element)) {
		result = _quote_create(front);
	} else if(_like_quote_calc(f_element)) {
		result = _quote_calc(front);
	} else {
		result = _nihil; /* error symbol   no such method or variety */
	}
	_arg_list_dealloc(front);
	return result;
}

struct _i_node* _lambda_execute(struct _i_node *front
#ifdef _DEBUG_VS
	,char *name
#endif
) {
	stack_depth++;
	if(stack_depth > SOF_F) {
		exit(1); /* failed and exit */
	} else if(stack_depth > SOF) { /* try to handle stack overflow */
		_arg_list_dealloc(front);
		return _nihil;
	}
	struct _proc *proc = (struct _proc*)(front->data);
	struct _ns_tree *_child = _create_child_ns(proc->local); /* apply closure */
	struct _i_node *bef_args = front;
	struct _i_node *act_args = front->next;
	struct _i_node *for_args = proc->args;
#ifdef _DEBUG_VS
	if(_d_trace_back) printf(">[level %d] call `%s' with : {\n",stack_depth,name);
#endif
	while(act_args && for_args) { /* push args */
		if(act_args->type == _EXPRES_) {
			bef_args->next = _swap_i_node(act_args);
			act_args = bef_args->next;
		}
		int insert_it = 1;
		if(act_args->type == _SYMBOL_ && !_like_inner_symbol((char*)(act_args->data))) {
			struct _i_node *temp = _swap_find_nil(act_args);
			if(temp->type == _NIHIL__) { /* insert nothing */
				insert_it = 0;
			} else {
				bef_args->next = temp;
				act_args = bef_args->next;
			}
		}
		if(insert_it) {
#ifdef _DEBUG_VS
			if(_d_trace_back) {
				printf(" %s ",(char*)(for_args->data));
				_inspect_elements(act_args,1);
			}
#endif
			_insert_into_ns(_child,(char*)(for_args->data),act_args);
		}
		bef_args = bef_args->next;
		act_args = act_args->next;
		for_args = for_args->next;
	}
#ifdef _DEBUG_VS
	if(_d_trace_back) printf("}\n");
#endif
	struct _ns_tree *local_save = _local;
	_local = _child; /* chroot */
	_reference_ns_add(_local);
	struct _i_node *result = NULL;
	struct _i_node *p = proc->proc;
	while(p) {
		if(result) _dealloc_i_node(result);
		result = _expr_rec_interp((struct _e_node*)(p->data));
		p = p->next;
	}
	_arg_list_dealloc(front);
#ifdef _DEBUG_VS
	if(_d_trace_back) {
		printf("<[level %d] rets ",stack_depth);
		_inspect_elements(result,1);
	}
#endif
	_reference_ns_sub(_local);
	_local = local_save;
	stack_depth--;
	return result;
}

void _create_nihil(void) {
	_nihil = _get_i_node();
	_nihil->type = _NIHIL__;
	_nihil->data = NULL; /* only the true nil have the pointer to null */
	_nihil->next = NULL;
}

struct _i_node* _expr_rec_interp(struct _e_node *root) {
	if(!root) {
		return _nihil;
	}
	struct _i_node *front = _arg_list_create(root);
	/* distinguish the front element */
	if(front->type == _EXPRES_) {
		front = _swap_i_node(front);
	}
#ifdef _DEBUG_VS
	char *name = _str_clone((char*)(front->data));
#endif
	if(front->type == _SYMBOL_) {
		front = _swap_find(front);
	}
	struct _i_node *result = _nihil;
	if(front->type == _SYMBOL_) {
		result = _base_execute(front);
	} else if(front->type == _LAMBDA_) {
#ifdef _DEBUG_VS
		result = _lambda_execute(front,name);
#else
		result = _lambda_execute(front);
#endif
	} else if(front->next == NULL) {
		result = front;
	} else {
		/* error! */
		_arg_list_dealloc(front);
	}
#ifdef _DEBUG_VS
	_dealloc(name);
#endif
	return result;
}

struct _i_node* _expr_interp(void *r_ot) { /* error  ^: <symbol> */
	if(!_local) {
		_local = _create_child_ns(_local);
		_local->u_cnt = -1; /* global namespace will not be collected */
	}
	stack_depth = 0;
	struct _e_node *root = (struct _e_node*)(r_ot);
	if(root && ((root->child_e) || (root->child_n))) {
		if(root->child_n) {
			return _expr_rec_interp(root);
		} else {
			struct _i_node *front = _e_node_to_i_node(root);
			if(front->type == _SYMBOL_) {
				front = _swap_find(front);
			}
			return front;
		}
	} else {
		return _nihil;
	}
}