#include <stdlib.h>
#include <string.h>
#include "_memory.h"
#include "_nspace.h"
#include "_parser.h"
#include "_interp.h"

struct _ns_tree* _create_child_ns(struct _ns_tree *parent) {
	struct _ns_tree *ptr = _alloc(sizeof(struct _ns_tree));
	ptr->prev = parent;
	ptr->head = NULL;
	ptr->u_cnt = 0;
	return ptr;
}

void _reference_ns_add(struct _ns_tree *local) {
	if(local->u_cnt >= 0) { /* as there's a global local */
		(local->u_cnt)++;
	}
	if(local->prev) {
		_reference_ns_add(local->prev);
	}
}

void _reference_ns_sub(struct _ns_tree *local) {
	if(local->u_cnt > 0) {
		(local->u_cnt)--;
	}
	if(local->prev) {
		_reference_ns_sub(local->prev);
	}
	if(local->u_cnt == 0) {
		_dealloc_a_ns(local);
	}
}

struct _i_node* _select_from_ns(struct _ns_tree *tree,char *str) {
	while(tree) {
		struct _ns_node *iter = tree->head;
		while(iter) {
			if(!strcmp(iter->name,str)) {
				return iter->data;
			} else {
				iter = iter->next;
			}
		}
		tree = tree->prev;
	}
	return _nihil;
}

void _delete_from_ns(struct _ns_tree *tree,char *str) {
	while(tree) {
		struct _ns_node *iter = tree->head;
		if(iter) {
			if( !strcmp(iter->name,str) ) {
				if(iter->data) _dealloc_i_node(iter->data);
				if(iter->name) _dealloc(iter->name);
				tree->head = iter->next;
				_dealloc(iter);
				return;
			} else {
				while(iter->next) {
					if(!strcmp(iter->next->name,str)) {
						if(iter->next->data) _dealloc_i_node(iter->next->data);
						if(iter->next->name) _dealloc(iter->next->name);
						struct _ns_node *ptr = iter->next;
						iter->next = iter->next->next;
						_dealloc(ptr);
						return;
					} else {
						iter = iter->next;
					}
				}
			}
		}
		tree = tree->prev;
	}
}

void _insert_into_ns(struct _ns_tree *tree,char *str,struct _i_node *data) {
	if(tree->head) {
		struct _ns_node *iter = tree->head;
		while(iter) {
			if(!strcmp(iter->name,str)) {
				_dealloc_i_node(iter->data);
				iter->data = _i_node_clone(data);
				break;
			} else {
				if(!(iter->next)) {
					iter->next = _alloc(sizeof(struct _ns_node));
					iter->next->data = _i_node_clone(data);
					iter->next->next = NULL;
					iter->next->name = _str_clone(str);
					break;
				} else {
					iter = iter->next;
				}
			}
		}
	} else {
		tree->head = _alloc(sizeof(struct _ns_node));
		tree->head->data = _i_node_clone(data);
		tree->head->next = NULL;
		tree->head->name = _str_clone(str);
	}
}

void _dealloc_a_ns(struct _ns_tree *tree) {
	if(tree) {
		struct _ns_node *iter = tree->head;
		while(iter) {
			struct _ns_node *loca = iter;
			iter = iter->next;
			if(loca->data) _dealloc_i_node(loca->data);
			if(loca->name) _dealloc(loca->name);
			_dealloc(loca);
		}
		_dealloc(tree);
	}
}