/*******************************************************************************
 * Copyright (c) 2010, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/

/* $Id $ */

/*!
 * @file        braindead.c
 * @copyright   (c) 2010 - Jean-David Gadina - www.xs-labs.com
 * @abstract    Main executable
 */

#include "braindead.h"

/*!
 * @abstract        main
 * @description     Program's entry point
 * @param           argc    The number of command line arguments
 * @param           argv    An array with the command line arguments
 * @result          The process exit status
 */
int main( int argc, char * argv[] )
{
    FILE           * fh;
    struct sigaction sa1;
    struct sigaction sa2;
    bd_cli_args      args;
    
    bd_get_cli_args( argc, argv, &args );
    
    sa1.sa_handler = bd_sig;
    
    sigemptyset( &sa1.sa_mask );
    
    sa1.sa_flags = 0;
    
    if( sigaction( SIGINT, &sa1, &sa2 ) != 0 )
    {
        fprintf( stderr, "Error: cannot set SIGINT handler" );
        return EXIT_FAILURE;
    }
    
    if( args.help == true )
    {
        bd_print_help( argv[ 0 ] );
    }
    else if( args.version == true )
    {
        bd_print_version();
    }
    else if( args.license == true )
    {
        bd_print_license();
    }
    else if( args.file != NULL )
    {
        if( access( args.file, F_OK ) == -1 )
        {
            fprintf( stderr, "Error: file does not exist (%s)\n", args.file );
            return EXIT_FAILURE;
        }
        
        if( access( args.file, R_OK ) == -1 )
        {
            fprintf( stderr, "Error: file is not readable: (%s)\n", args.file );
            return EXIT_FAILURE;
        }
        
        if( NULL == ( fh = fopen( args.file, "r" ) ) )
        {
            fprintf( stderr, "Error: cannot open file for reading (%s)\n", args.file );
            return EXIT_FAILURE;
        }
        
        bd_interpret_file
        (
            fh,
            args.alloc,
            args.read,
            args.debug,
            args.break_points,
            args.stack_debug,
            args.stack_reset
        );
        
        fclose( fh );
    }
    else
    {
        bd_interpret
        (
            args.alloc,
            args.debug,
            args.break_points,
            args.stack_debug,
            args.stack_reset
        );
    }
    
    return EXIT_SUCCESS;
}
