/*******************************************************************************
 * Copyright (c) 2011, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/
 
/* $Id$ */

/*!
 * @header      file.c
 * @copyright   (c) 2011 - Jean-David Gadina - www.xs-labs.com
 * @abstract    File related functions
 */

/* Local includes */
#include "egpack.h"

/*!
 * @function    egpack_get_destination_filename
 * @abstract    Gets a valid destination filename
 * @param       source      The original filename
 * @param       filename    A pointer to store the final filename
 * @param       archive     Whether we are running an archive or un-archive process
 * @result      True if a file name was created, otherwise false
 */
bool egpack_get_destination_filename( char * source, char * filename, bool archive )
{
    int    i;
    size_t length;
    size_t length_ext;
    char * basename;
    bool   add_ext;
    char   suffix[ 10 ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    
    basename   = source;
    length     = strlen( basename );
    length_ext = strlen( EGPACK_FILE_EXT );
    
    /* Process the string from the end */
    for( i = length - 1; i > -1; i-- )
    {
        /* Checks for a slash */
        if( basename[ i ] == '/' )
        {
            /* Moves the pointer to the file name */
            basename += i + 1;
            break;
        }
    }
    
    /* Gets the string length */
    length = strlen( basename );
    
    /* Resets the buffer */
    memset( filename, 0, EGPK_FILENAME_MAX );
    
    /* Checks if we need to remove or add the file extension */
    if( archive == true )
    {
        /* Copies the file name and add the extension */
        add_ext = true;
        strcpy( filename, basename );
        strcat( filename, EGPACK_FILE_EXT );
    }
    else if( strcmp( basename + length - length_ext, EGPACK_FILE_EXT ) == 0 )
    {
        /* Copies the file name, without the extension */
        add_ext = false;
        strncpy( filename, basename, length - length_ext );
    }
    else
    {
        /* Copies the file name, without the extension */
        add_ext = false;
        strncpy( filename, basename, length );
    }
    
    i = 0;
    
    /* Checks if we need to add a suffix (if a file with the same name already exist) */
    while( access( filename, F_OK ) == 0 && i < 999999999 )
    {
        if( i > 0 )
        {
            memset( filename + ( strlen( filename ) - strlen( suffix ) ), 0, strlen( suffix ) );
        }
        
        /* Suffix to add */
        sprintf( suffix, "-%i", ++i );
        
        /* Checks if we must add the extension */
        if( add_ext == false )
        {
            /* Not enough space for the filename */
            if( strlen( suffix ) + length - length_ext > EGPK_FILENAME_MAX )
            {
                break;
            }
            
            /* Adds the suffix */
            strncpy( filename, basename, length - length_ext );
            strcat( filename, suffix );
        }
        else
        {
            /* Not enough space for the filename */
            if( strlen( suffix ) + length + length_ext > EGPK_FILENAME_MAX )
            {
                break;
            }
            
            /* Adds the suffix */
            strcpy( filename, basename );
            strcat( filename, suffix );
            strcat( filename, EGPACK_FILE_EXT );
        }
    }
    
    /* Returns true if a destination filename was successfully created */
    return ( access( filename, F_OK ) == -1 ) ? true : false;
}
