/*******************************************************************************
 * Copyright (c) 2011, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/
 
/* $Id$ */

/*!
 * @header      archive.h
 * @copyright   (c) 2011 - Jean-David Gadina - www.xs-labs.com
 * @abstract    Archiving functions
 */

#ifndef _EGPACK_COMPRESS_H_
#define _EGPACK_COMPRESS_H_
#pragma once

#ifdef __cplusplus
extern "C" {
#endif
    
    /*!
     * @function    egpack_archive
     * @abstract    Archives source path into destination file
     * @param       source      The path to archive
     * @param       destination The archive file pointer
     * @result      The status code
     */
    egpack_status egpack_archive( char * path, FILE * destination );
    
    /*!
     * @function    egpack_archive_dir
     * @abstract    Archives a complete directory into destination file
     * @param       dirname     The name of the directory
     * @param       depth       The depth from the archive's root directory
     * @param       destination The archive file pointer
     * @result      The status code
     */
    egpack_status egpack_archive_dir( char * dirname, unsigned int depth, FILE * destination );

    /*!
     * @function    egpack_archive_dir
     * @abstract    Archives a single file into destination file
     * @param       dirname     The name of the file
     * @param       depth       The depth from the archive's root directory
     * @param       destination The archive file pointer
     * @result      The status code
     */
    egpack_status egpack_archive_file( char * filename, unsigned int depth, FILE * destination );

    /*!
     * @function    egpack_archive_link
     * @abstract    Archives a symbolic link into destination file
     * @param       dirname     The name of the file
     * @param       depth       The depth from the archive's root directory
     * @param       destination The archive file pointer
     * @result      The status code
     */
    egpack_status egpack_archive_link( char * filename, unsigned int depth, FILE * destination );
    
#ifdef __cplusplus
}
#endif

#endif /* _EGPACK_COMPRESS_H_ */
