/*******************************************************************************
 * Copyright (c) 2011, Jean-David Gadina - www.xs-labs.com
 * Distributed under the Boost Software License, Version 1.0.
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************/
 
/* $Id$ */

/*!
 * @header      unarchive.h
 * @copyright   (c) 2011 - Jean-David Gadina - www.xs-labs.com
 * @abstract    Un-archiving functions
 */

#ifndef _EGPACK_EXPAND_H_
#define _EGPACK_EXPAND_H_
#pragma once

#ifdef __cplusplus
extern "C" {
#endif
    
    /*!
     * @function    egpack_archive
     * @abstract    Un-archives source file into destination path
     * @param       source      The archive file pointer
     * @param       destination The path in wich to write archive file(s)
     * @result      The status code
     */
    egpack_status egpack_unarchive( FILE * source, char * destination );
    
    /*!
     * @function    egpack_archive
     * @abstract    Un-archives source file into destination directory
     * @param       source      The archive file pointer
     * @param       depth       The depth from the archive's root directory
     * @param       destination The path in wich to write archive file(s)
     * @result      The status code
     */
    egpack_status egpack_unarchive_dir( FILE * source, unsigned int depth, char * destination );
    
    /*!
     * @function    egpack_archive
     * @abstract    Un-archives source file into destination file
     * @param       source      The archive file pointer
     * @param       depth       The depth from the archive's root directory
     * @param       destination The path in wich to write the archive file
     * @result      The status code
     */
    egpack_status egpack_unarchive_file( FILE * source, unsigned int depth, char * destination );
    
    /*!
     * @function    egpack_unarchive_link
     * @abstract    Un-archives source file into destination symbolic link
     * @param       source      The archive file pointer
     * @param       depth       The depth from the archive's root directory
     * @param       destination The path in wich to write the archive symolic link
     * @result      The status code
     */
    egpack_status egpack_unarchive_link( FILE * source, unsigned int depth, char * destination );
    
#ifdef __cplusplus
}
#endif

#endif /* _EGPACK_EXPAND_H_ */
