#ifndef TIMER_H
#define TIMER_H

#include <assert.h>
#include <stdint.h>
#include <time.h>

#ifdef HAVE_RDTSC
#	error do not define HAVE_RDTSC; use USE_RDTSC to enable
#endif

#ifdef USE_RDTSC
#	if defined(__i386__)
#		define HAVE_RDTSC
static inline uint64_t clock_rdtsc()
{
	uint64_t d;
	__asm__ __volatile__ ("rdtsc" : "=A" (d) );
	return d;
}
#	elif defined(__amd64__)
#		define HAVE_RDTSC
static inline uint64_t clock_rdtsc()
{
	uint32_t a, d;
	__asm__ __volatile__ ("rdtsc" : "=a" (a), "=d" (d) );
	return ((uint64_t)a << 32) | d;
}
#	endif
#endif

static inline uint64_t _clock_select(clockid_t clock)
{
	/* 64 bits is enough for 34.1 bits of seconds and 29.9 bits of
	 * nanoseconds, for around 584 years */
	struct timespec ts;

	/* the only possible error is that the clock is not available */
	if (clock_gettime(clock, &ts) == -1) assert(0);
	return (uint64_t)ts.tv_sec * 1000000000ULL + ts.tv_nsec;
}

/* this should be available on Linux and the BSDs */
static inline uint64_t clock_monotonic()
{	return _clock_select(CLOCK_MONOTONIC); }

/* the thread's CPU time */
static inline uint64_t clock_cpu_thread()
{
#ifdef CLOCK_THREAD_CPUTIME_ID
	return _clock_select(CLOCK_PROCESS_CPUTIME_ID);
#elif defined(CLOCK_PROF)
	/* just for fun; what the BSDs seem to use, though it is strictly
	 * speaking the same as CLOCK_PROCESS_CPUTIME_ID, and so comparisons
	 * wouldn't be valid */
	return _clock_select(CLOCK_PROF);
#else
#	error no process clock
#endif
}

static inline uint64_t clock_ticks()
{
#ifdef HAVE_RDTSC
	return clock_rdtsc();
#else
	return clock_monotonic();
#endif
}

#endif
