#ifndef DISK_STATS_H
#define DISK_STATS_H

#include <limits.h>
#include <stdint.h>

struct disk_stats {
	/* (notes from linux/Documentation/iostats.txt) */

	/* 1, 5 */
	/* This is the total number of reads/writes completed successfully. */
	uint32_t reads_completed;
	uint32_t writes_completed;

	/* 2, 6 */
	/* Reads and writes which are adjacent to each other may be merged for
	 * efficiency.  Thus two 4K reads may become one 8K read before it is
	 * ultimately handed to the disk, and so it will be counted (and
	 * queued) as only one I/O.  This field lets you know how often this
	 * was done. */
	uint32_t reads_merged;
	uint32_t writes_merged;

	/* 3, 7 */
	/* This is the total number of sectors read/written successfully. */
	uint32_t sectors_read;
	uint32_t sectors_written;

	/* 4, 8 */
	/* This is the total number of milliseconds spent by all
	 * reads/writes (as measured from __make_request() to
	 * end_that_request_last()). */
	uint32_t milli_reading;
	uint32_t milli_writing;

	/* 9 */
	/* # of I/Os currently in progress.  The only field that should go to
	 * zero. Incremented as requests are given to appropriate struct
	 * request_queue and decremented as they finish. */
	uint32_t pending_io;

	/* 10 */
	/* This field increases so long as 'pending_io' is nonzero. */
	uint32_t milli_io;

	/* 11 */
	/* This field is incremented at each I/O start, I/O completion, I/O
	 * merge, or read of these stats by the number of I/Os in progress
	 * ('pending_io') times the number of milliseconds spent doing I/O
	 * since the last update of this field.  This can provide an easy
	 * measure of both I/O completion time and the backlog that may be
	 * accumulating. */
	uint32_t milli_io_weighted;
};

/* 'device' is either the full path to the device or the basename */
int		find_device_stat(const char *device, char[PATH_MAX]);

int		read_disk_stats(const char *caller, const char *statpath,
		    struct disk_stats *);

#endif
