#ifndef PROC_SPEC_H
#define PROC_SPEC_H

#include <sys/time.h>

#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "workload.h"

enum io_priority { PRIO_IDLE, PRIO_NORM, PRIO_RT };

struct proc_spec {
	struct workload		wl;
	struct timespec		finish;
	struct timespec		start;
	uint64_t		ticks_per_sec;
	char			*dataprefix; /* 0 = use CWD */
	char			*path_device; /* set by settestpath */
	char			*path_mountpoint; /* set by settestpath */
	char			*path_stat; /* set by settestpath */
	char			*testpath;
	size_t			sz;
	uint64_t		spins;
	uint16_t		id;
	uint16_t		rapid;
	enum io_priority	prio_cls;
	uint8_t			prio_lvl;
	bool			mmap;
};

static inline void
		proc_spec_init(struct proc_spec *, uint16_t);
static inline void
		proc_spec_destroy(struct proc_spec *);
int		proc_spec_settestpath(struct proc_spec *, const char *);
int		proc_spec_setdataprefix(struct proc_spec *, const char *);

uint64_t	required_spins(const struct timespec *);
void		spin_for(uint64_t);

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static inline void
proc_spec_init(struct proc_spec *s, uint16_t id)
{
	s->wl.dist = 0;
	s->wl.type = WL_RSEQ;

	s->finish.tv_sec = 0;
	s->finish.tv_nsec = 0;
	s->start = s->finish;

	s->ticks_per_sec = 0;
	s->dataprefix = 0;
	s->path_device = 0;
	s->path_mountpoint = 0;
	s->path_stat = 0;
	s->testpath = 0;
	s->rapid = 1;
	s->spins = 0;
	s->sz = 4096;
	s->id = id;
	s->prio_cls = PRIO_NORM;
	s->prio_lvl = 4;
	s->mmap = false;
}

static inline void
proc_spec_destroy(struct proc_spec *s)
{
	if (s->dataprefix) {
		free(s->dataprefix);
		s->dataprefix = 0;
	}
	if (s->path_mountpoint) {
		free(s->path_mountpoint);
		s->path_mountpoint = 0;
	}
	if (s->path_device) {
		free(s->path_device);
		s->path_device = 0;
	}
	if (s->path_stat) {
		free(s->path_stat);
		s->path_stat = 0;
	}
	if (s->testpath) {
		free(s->testpath);
		s->testpath = 0;
	}
}

#endif
