#ifndef PROC_STATS_H
#define PROC_STATS_H

#include <stdint.h>

#include "disk_stats.h"

/* a 10s test with 3 procs and 4kB blocks produces ~65k results each */
#define PS_DATA_PER_NODE 1024

struct ps_node {
	/* no node should ever exist where count==0 */
	uint64_t	data[PS_DATA_PER_NODE];
	struct ps_node	*next;
	uint16_t	count;
};

struct proc_stats {
	struct ps_node		*head;
	struct ps_node		*tail;

	struct disk_stats	disk_end;
	struct disk_stats	disk_start;
	uint64_t		cpu_time;
	uint64_t		time_end;
	uint64_t		time_start;
};

static inline void
proc_stats_init(struct proc_stats *stats)
{
	stats->head = stats->tail = 0;
}

static inline void
proc_stats_destroy(struct proc_stats *stats)
{
	while (stats->head) {
		struct ps_node *next = stats->head->next;
		free(stats->head);
		stats->head = next;
	}
	stats->head = stats->tail = 0;
}

static inline int
proc_stats_push(struct proc_stats *stats, uint64_t data)
{
	struct ps_node *node = 0;

	if (!stats->tail || stats->tail->count == PS_DATA_PER_NODE) {
		if (!(node = malloc(sizeof(struct ps_node))))
			return -1;
		node->next = 0;
		node->count = 0;
		if (stats->tail) {
			stats->tail->next = node;
			stats->tail = node;
		} else
			stats->head = stats->tail = node;
	} else
		node = stats->tail;

	node->data[node->count++] = data;
	return 0;
}

#endif
