#include <sys/types.h>
#include <linux/fs.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

int main(int argc, char **argv)
{
	struct stat	st;
	/* ioctl_list(2) says BLKRAGET takes int*, but really takes long* */
	long		old;
	long		readahead;
	long		tmp_l;
	char		*endptr;
	int		fd;
	bool		error_old = false;
	bool		error_new = false;

	if (argc < 2 || argc > 3) {
		fprintf(stderr, "usage: %s DEV [READAHEAD]\n", *argv);
		return 1;
	}

	if ((fd = open(argv[1], O_RDONLY)) == -1) {
		perror(*argv);
		return 1;
	}

	/* make sure it is a block device that we've opened */
	if (fstat(fd, &st) == -1) {
		fprintf(stderr, "%s: fstat failed: %s\n",
		    *argv, strerror(errno));
		close(fd);
		return 1;
	}
	if (!S_ISBLK(st.st_mode)) {
		fprintf(stderr, "%s: not a block device\n", *argv);
		close(fd);
		return 1;
	}

	if (ioctl(fd, BLKRAGET, &old) == -1) {
		fprintf(stderr, "%s: error getting dev readahead: %s\n",
		    *argv, strerror(errno));
		if (argc == 2) {
			close(fd);
			return 1;
		}
		error_old = true;
	}

	if (argc == 2) {
		/* print out current value and exit */
		printf("%ld\n", old);
		close(fd);
		return 0;
	}

	/* parse second argument */
	errno = 0;
	tmp_l = strtol(argv[2], &endptr, 10);
	if (errno) {
		fprintf(stderr, "%s: bad number: %s\n",
		    *argv, strerror(errno));
		close(fd);
		return 1;
	} else if (*endptr) {
		fprintf(stderr, "%s: bad number\n", *argv);
		close(fd);
		return 1;
	}
	readahead = tmp_l;

	if (ioctl(fd, BLKRASET, readahead) == -1) {
		fprintf(stderr, "%s: error setting dev readahead: %s\n",
		    *argv, strerror(errno));
		close(fd);
		return 1;
	}
	if (ioctl(fd, BLKRAGET, &readahead) == -1) {
		fprintf(stderr, "%s: error getting dev readahead: %s\n",
		    *argv, strerror(errno));
		error_new = true;
	}
	if (error_new && error_old)
		printf("%s: failed to get readahead, "
		    "though apparantly succeeded in setting it\n", *argv);
	else if (error_new)
		printf("%s readahead: %ld => ?", argv[1], old);
	else if (error_old)
		printf("%s readahead: ? => %ld", argv[1], readahead);
	else
		printf("%s readahead: %ld => %ld\n", argv[1], old, readahead);

	close(fd);
	return 0;
}
