/*  NEWPLUGIN.c
 *  (C) 2012-2014 Francesco Romani (fromani a gmail d com)
 *  this file is a frei0r(x) plugin based on
 *
 *  Copyright (C) Tilmann Bitterberg - June 2002
 *    Based on Code from mplayers cropdetect by A'rpi
 *    Updated by Antonio Beamud Montero (Microgenesis S.A.) - Jan 2009
 *   
 *  This file is part of transcode, a video stream processing tool
 *
 *  transcode is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  transcode is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "frei0rx.h"

typedef struct NEWPLUGINinstance_ {
    // TODO
} NEWPLUGIN_instance_t;

#define RETURN_IF_NULL(inst) do { \
    if (!(inst)) { \
        return; \
    } \
} while (0)

enum {
    RGB_BYTES             = 3, /* how much storage a pixel take? */
    RGBA_BYTES            = 4, /* ditto */
    PIXEL_BLACK_THRESHOLD = 24
};

// TODO

int f0r_init()
{
    /* no initialization required */
    return 1;
}

void f0r_deinit()
{
    /* no initialization required */
    return;
}

void f0r_get_plugin_info(f0r_plugin_info_t* info)
{
    info->name = "NEWPLUGIN";
    info->author = "XXX TODO";
    info->plugin_type = F0R_PLUGIN_TYPE_FILTER;
    info->color_model = F0R_COLOR_MODEL_RGBA8888;
    info->frei0r_version = FREI0R_MAJOR_VERSION;
    info->major_version = 0;
    info->minor_version = 1;
    info->num_params =  1;
    info->explanation = ""
"";
}

void f0r_get_param_info(f0r_param_info_t* info, int param_index)
{
    switch (param_index) {
      case 0:
        info->name = "Start";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The first frame to consider.";
        break;
      case 1:
        info->name = "End";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The last frame to consider.";
        break;
      case 2:
        info->name = "Step";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "Apply the check every `Step' frames.";
        break;
      case 3:
        info->name = "Limit";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The sum of a line must be below this"
                            " limit to be considered black.";
        break;
      case 4:
        info->name = "Log path";
        info->type = F0R_PARAM_STRING;
        info->explanation = "File to save a detailed values";
        break;
      default:
        break; // XXX
    }
    return;
}

f0r_instance_t f0r_construct(unsigned int width, unsigned int height)
{
    NEWPLUGIN_instance_t* inst = calloc(1, sizeof(*inst));
    if (inst) {
        // TODO
    }
    return (f0r_instance_t)inst;
}

void f0r_destruct(f0r_instance_t instance)
{
    NEWPLUGIN_instance_t* inst = (NEWPLUGIN_instance_t*)instance;
    RETURN_IF_NULL(inst);
    // TODO
    free(instance);
    return;
}

void f0r_set_param_value(f0r_instance_t instance,
                         f0r_param_t param, int param_index)
{
    NEWPLUGIN_instance_t* inst = (NEWPLUGIN_instance_t*)instance;
    f0r_param_string path;
    RETURN_IF_NULL(inst);
    switch (param_index) {
      case 0: /* Start */
        inst->start = (unsigned int) *((f0r_param_double*)param);
        update_step_data(inst);
        break;
      case 1: /* End */
        inst->end = (unsigned int) *((f0r_param_double*)param);
        break;
      case 2: /* Step */
        inst->step = (unsigned int) *((f0r_param_double*)param);
        update_step_data(inst);
        break;
      case 3: /* Limit */
        inst->limit = (int) *((f0r_param_double*)param);
        break;
      case 4: /* Log path */
        path = *((f0r_param_string*)param);
        if (strcmp(path, inst->log_path) != 0) {
            free(inst->log_path);
            inst->log_path = strdup(path);
        }
        break;
      default:
        break; // XXX
    }
    return;
}

void f0r_get_param_value(f0r_instance_t instance,
                         f0r_param_t param, int param_index)
{
    NEWPLUGIN_instance_t* inst = (NEWPLUGIN_instance_t*)instance;
    RETURN_IF_NULL(inst);
    switch (param_index) {
      case 0: /* Start */
        *((f0r_param_double*)param) = (f0r_param_double)inst->start;
        break;
      case 1: /* End */
        *((f0r_param_double*)param) = (f0r_param_double)inst->end;
        break;
      case 2: /* Step */
        *((f0r_param_double*)param) = (f0r_param_double)inst->step;
        break;
      case 3: /* Limit */
        *((f0r_param_double*)param) = (f0r_param_double)inst->limit;
        break;
      case 4: /* Log path */
        *((f0r_param_string*)param) = inst->log_path;
        break;
      default:
        break; // XXX
    }
    return;
}


void f0r_update2(f0r_instance_t instance, double time,
                const uint32_t* inframe1,
                const uint32_t* inframe2,
                const uint32_t* inframe3,
                uint32_t* outframe)
{
    NEWPLUGIN_instance_t* inst = (NEWPLUGIN_instance_t*)instance;
    RETURN_IF_NULL(inst);
    // TODO
    return;
}


void f0r_update(f0r_instance_t instance, double time,
                const uint32_t* inframe, uint32_t* outframe)
{
    f0r_update2(instance, time, inframe, NULL, NULL, outframe);
    return;
}

/*
 * vim: expandtab shiftwidth=4:
 */

