
#ifndef INCLUDED_FREI0RX_PARAM_H
#define INCLUDED_FREI0RX_PARAM_H


typedef struct f0rxparamattr_ {
    void *field;
    void (*get)(f0r_instance_t instance, int param_index,
                f0r_param_t param);
    void (*set)(f0r_instance_t instance, int param_index,
                const f0r_param_t param);
    void (*post_set)(f0r_instance_t instance, int param_index);
} f0rx_param_attr_t;

static f0rx_param_attr_t *f0rx_get_param_attr(f0r_instance_t instance,
                                              int param_index);

/*************************************************************************/

static void f0rx_hook_post_set_pass(f0r_instance_t instance,
                                    int param_index)
{
    return; /* do nothing (and allow us to avoid an if) */
}

static void f0rx_param_getter(f0r_instance_t instance, int param_index,
                              f0r_param_t param)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    f0r_param_info_t info;

    f0r_get_param_info(&info, param_index);
    switch (info.type) {
      case F0R_PARAM_BOOL:
        *((f0r_param_bool*)param) = *((f0r_param_bool*)attr->field);
        break;
      case F0R_PARAM_DOUBLE:
        *((f0r_param_double*)param) = *((f0r_param_double*)attr->field);
        break;
      case F0R_PARAM_STRING:
        *((f0r_param_string*)param) = *((f0r_param_string*)attr->field);
        break;
      case F0R_PARAM_COLOR:
        *((f0r_param_color_t*)param) = *((f0r_param_color_t*)attr->field);
        break;
      /* TODO: implement the missing types */
      /* TODO: implement the missing types */
      default:
        break; /* suggestions welcome. */
    }
    return;
}

static void f0rx_param_setter(f0r_instance_t instance, int param_index,
                              const f0r_param_t param)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    f0r_param_info_t info;
    f0r_param_string pstr;

    f0r_get_param_info(&info, param_index);
    switch (info.type) {
      case F0R_PARAM_BOOL:
        *((f0r_param_bool*)attr->field) = *((f0r_param_bool*)param);
        break;
      case F0R_PARAM_DOUBLE:
        *((f0r_param_double*)attr->field) = *((f0r_param_double*)param);
        break;
      case F0R_PARAM_STRING:
        pstr = *((f0r_param_string*)param);
        if (strcmp(pstr, *((f0r_param_string*)attr->field)) != 0) {
            free(*((f0r_param_string*)attr->field));
            *((f0r_param_string*)attr->field) = strdup(pstr);
        }
        break;
      case F0R_PARAM_COLOR:
        *((f0r_param_color_t*)attr->field) = *((f0r_param_color_t*)param);
        break;
      /* TODO: implement the missing types */
      default:
        break; /* suggestions welcome. */
    }
    return;
}

/*************************************************************************/

#define F0RX_PARAM_ALL  (-1)

static void f0rx_hook_param_attr_post_set(f0r_instance_t instance,
                                          int param_index,
                                          void (*hook)(f0r_instance_t instance,
                                                       int param_index))
{
    if (param_index == F0RX_PARAM_ALL) {
        int i = 0;
        f0r_plugin_info_t info;
        f0r_get_plugin_info(&info);
        for (i = 0; i < info.num_params; i++) {
            f0rx_hook_param_attr_post_set(instance, i, hook);
        }
    } else {
        f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
        attr->post_set = hook;
    }
    return;
}

static void f0rx_bind_param_attr(f0r_instance_t instance,
                                 int param_index,
                                 void *field)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    attr->field = field;
    attr->get = f0rx_param_getter;
    attr->set = f0rx_param_setter;
    f0rx_hook_param_attr_post_set(instance, param_index,
                                  f0rx_hook_post_set_pass);
    /* to make gcc quiet(er) */
}


/*************************************************************************/

void f0r_set_param_value_raw(f0r_instance_t instance,
                             f0r_param_t param, int param_index)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    attr->set(instance, param_index, param);
}


void f0r_set_param_value(f0r_instance_t instance,
                         f0r_param_t param, int param_index)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    attr->set(instance, param_index, param);
    attr->post_set(instance, param_index); 
}

void f0r_get_param_value(f0r_instance_t instance,
                         f0r_param_t param, int param_index)
{
    f0rx_param_attr_t *attr = f0rx_get_param_attr(instance, param_index);
    attr->get(instance, param_index, param);
}

#endif /* INCLUDED_FREI0RX_PARAM_H */

