/*  logoaway.c
 *  (C) 2011 Francesco Romani (fromani a gmail d com)
 *  this file is a frei0r(x) plugin based on
 *
 *  Copyright (C) Thomas Wehrspann - 2002/2003
 *    This plugin is based on ideas of Krzysztof Wojdon's
 *    logoaway filter for VirtualDub
 * 
 *  This file is part of transcode, a video stream processing tool
 *
 *  transcode is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  transcode is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "memutils.h"
#include "strutils.h"
#include "gmlib.h"

#include "frei0rx.h"
#include "frei0rx_param.h"
#include "frei0rx_paramext.h"
#include "frei0rx_compat.h"
#include "frei0rx_frame.h"


enum {
    MODE_NONE,
    MODE_SOLID,
    MODE_XY,
    MODE_SHAPE
};
/*
static char *mode_name[] = {
    "NONE",
    "SOLID",
    "XY",
    "SHAPE"
};
*/

enum {
    PLUGIN_PARAM_NUM    = 13
};

typedef struct logoawayinstance_ logoaway_instance_t;

typedef int (*logoaway_process_frame_t)(logoaway_instance_t *inst,
                                        f0rx_frame_t *out,
                                        int width, int height);

struct logoawayinstance_ {
    f0rx_param_attr_t attrs[PLUGIN_PARAM_NUM];
    unsigned int    frames;
    unsigned int    start, end;
    int             imgwidth, imgheight;
    int             xpos, ypos;
    int             width, height;
    int             mode;
    int             border;
    int             xweight, yweight;
    f0r_param_color_t usercolor;
    int             rcolor, gcolor, bcolor;
    int             ycolor, ucolor, vcolor;
    char            *file;
    int             instance;

    int             alpha;

    TCMagickContext logo_ctx;
    TCMagickContext dump_ctx;
    PixelPacket     *pixels;

    int             dump;
    uint8_t         *dump_buf;

    int             colorspace;
    int (*draw_border)(logoaway_instance_t *inst,
                       f0rx_frame_t *out, int width, int height);
    int (*copy_frame)(logoaway_instance_t *inst,
                      f0rx_frame_t *out, const f0rx_frame_t *in);
    int (*pack_frame)(logoaway_instance_t *inst,
                      f0rx_frame_t *frame, uint8_t *data, double time);
    /* the last one deserves a special treatment */
    logoaway_process_frame_t process_frame;
    /* just a temporary */
    f0r_param_color_t color;
};


// TODO: implement copy_frame
// TODO: implement pack_frame

#define RETURN_IF_NULL(inst) do { \
    if (!(inst)) { \
        return; \
    } \
} while (0)

#define RETURN_ERROR_IF_NULL(INST) do { \
    if (!(INST)) { \
        return -1; \
    } \
} while (0)

/* ugly hack until I fix out some aclib related issues. */
#define ac_memcpy memmove


enum {
    RGB_BYTES           = 3, /* how much storage a pixel take? */
    RGBA_BYTES          = 4, /* ditto */
};

/*************************************************************************/

static uint8_t alpha_blending(uint8_t srcPixel, uint8_t dstPixel, int alpha)
{
  return (((alpha * (srcPixel - dstPixel) ) >> 8 ) + dstPixel);
}

static void free_dump_buf(logoaway_instance_t *inst)
{
    if (inst->dump) {
        tc_free(inst->dump_buf);
        inst->dump_buf = NULL;
    }   
}

/*************************************************************************/

static void dump_image_rgb(logoaway_instance_t *inst,
                           uint8_t *buffer, int width, int height)
{
    int row = 0, col = 0, buf_off = 0, pkt_off = 0;
    int ret = 0;

    for (row = inst->ypos; row < inst->height; row++) {
        for (col = inst->xpos; col < inst->width; col++) {
            pkt_off = ((row-inst->ypos)*(inst->width-inst->xpos)+(col-inst->xpos)) * RGB_BYTES;
            buf_off = ((height-row)*width+col) * RGBA_BYTES;
            /* R */
            inst->dump_buf[pkt_off +0] = buffer[buf_off +0];
            /* G */
            inst->dump_buf[pkt_off +1] = buffer[buf_off +1];
            /* B */
            inst->dump_buf[pkt_off +2] = buffer[buf_off +2];
        }
    }

    ret = tc_magick_RGBAin(&inst->dump_ctx,
                           inst->width  - inst->xpos,
                           inst->height - inst->ypos,
                           inst->dump_buf);
    if (ret != 0) {
        /* FIXME log */
    } else {
        snprintf(inst->dump_ctx.image_info->filename,
                 MaxTextExtent, "dump[%i].png", inst->instance); /* FIXME */

        WriteImage(inst->dump_ctx.image_info, inst->dump_ctx.image);
    }
    return;
}

/* FIXME: both the inner if(N&1)s can be factored out */
static int draw_border_rgb(logoaway_instance_t *inst,
                           f0rx_frame_t *out, int width, int height)
{
    int row = 0, col = 0, buf_off = 0;
    uint8_t *buffer = out->planes[0];

    for (row = inst->ypos; row < inst->height; row++) {
        if ((row == inst->ypos) || (row==inst->height-1)) {
            for (col = inst->xpos*3; col < inst->width*3; col++)
                if (col & 1)
                    buffer[((height-row)*width*3+col)] = 255 & 0xff;
        }
        if (row & 1) {
            buf_off = ((height-row)*width+inst->xpos)*3;
            buffer[buf_off +0] = 255;
            buffer[buf_off +1] = 255;
            buffer[buf_off +2] = 255;

            buf_off = ((height-row)*width+inst->width)*3;
            buffer[buf_off +0] = 255;
            buffer[buf_off +1] = 255;
            buffer[buf_off +2] = 255;
        }
    }
    return 0;
}


/* FIXME: both the inner if(N&1)s can be factored out */
static int draw_border_yuv(logoaway_instance_t *inst,
                           f0rx_frame_t *out, int width, int height)
{
    int row = 0, col = 0;
    uint8_t *buffer = out->planes[0];

    for (row = inst->ypos; row < inst->height; row++) {
        if ((row == inst->ypos) || (row == inst->height - 1)) {
            for (col = inst->xpos; col < inst->width; col++)
                if (col & 1)
                    buffer[row*width+col] = 255 & 0xff;
        }
        if (row & 1) {
            buffer[row*width+inst->xpos]  = 255 & 0xff;
            buffer[row*width+inst->width] = 255 & 0xff;
        }
    }
    return 0;
}

/*************************************************************************/

static int copy_frame_rgb(logoaway_instance_t *inst,
                          f0rx_frame_t *out, const f0rx_frame_t *in)
{
    out->time = in->time;
    out->refs = in->refs;
    out->pitches[0] = in->pitches[0];
    ac_memcpy(out->planes[0], in->planes[0],
              inst->imgwidth * inst->imgheight * 4); /* FIXME */
    return 0;
}

static int copy_frame_yuv(logoaway_instance_t *inst,
                          f0rx_frame_t *out, const f0rx_frame_t *in)
{
    out->time = in->time;
    out->refs = in->refs;
    out->pitches[0] = in->pitches[0];
    out->pitches[1] = in->pitches[1];
    out->pitches[2] = in->pitches[2];
    ac_memcpy(out->planes[0], in->planes[0],
              inst->imgwidth * inst->imgheight); /* FIXME */
    ac_memcpy(out->planes[1], in->planes[1],
              inst->imgwidth * inst->imgheight / 2); /* FIXME */
    ac_memcpy(out->planes[2], in->planes[2],
              inst->imgwidth * inst->imgheight / 2); /* FIXME */
    return 0;
}

/*************************************************************************/

static int pack_frame_rgb(logoaway_instance_t *inst,
                          f0rx_frame_t *frame, uint8_t *data, double time)
{
    memset(frame, 0, sizeof(*frame));
    frame->time.d = time;
    frame->refs = 0;
    frame->pitches[0] = inst->imgwidth * inst->imgheight * 4; /* FIXME */
    frame->planes[0] = data;
    return 0;
}

static int pack_frame_yuv(logoaway_instance_t *inst,
                          f0rx_frame_t *frame, uint8_t *data, double time)
{
    f0rx_init_frame_yuv420p(frame, data, time,
                            inst->imgwidth, inst->imgheight);
    return 0;
}

/*************************************************************************/

static int draw_border_null(logoaway_instance_t *inst,
                            f0rx_frame_t *out, int width, int height)
{
    return 0;
}

static int process_frame_null(logoaway_instance_t *inst,
                              f0rx_frame_t *out, int width, int height)
{
    return 0;
}

/*************************************************************************/

static int process_frame_rgb_solid(logoaway_instance_t *inst,
                                   f0rx_frame_t *out, int width, int height)
{
    int row, col, buf_off, pkt_off;
    uint8_t px;
    uint8_t *buffer = out->planes[0];

    if (inst->dump) {
        dump_image_rgb(inst, buffer, width, height);
    }

    for (row = inst->ypos; row < inst->height; row++) {
        for (col = inst->xpos; col < inst->width; col++) {
            buf_off = ((height-row)*width+col) * 3;
            pkt_off = (row-inst->ypos) * (inst->width-inst->xpos) + (col-inst->xpos);
            /* R */
            if (!inst->alpha) {
                buffer[buf_off +0] = inst->rcolor;
                buffer[buf_off +1] = inst->gcolor;
                buffer[buf_off +2] = inst->bcolor;
            } else {
                px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
                buffer[buf_off +0] = alpha_blending(buffer[buf_off +0], inst->rcolor, px);
                /* G */
                px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].green);
                buffer[buf_off +1] = alpha_blending(buffer[buf_off +1], inst->gcolor, px);
                /* B */
                px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].blue);
                buffer[buf_off +2] = alpha_blending(buffer[buf_off +2], inst->bcolor, px);
            }
        }
    }
 
    return inst->draw_border(inst, out, width, height);
}


static int process_frame_rgb_xy(logoaway_instance_t *inst,
                                f0rx_frame_t *out, int width, int height)
{
    int row, col, xdistance, ydistance, distance_west, distance_north;
    unsigned char hcalc, vcalc;
    int buf_off, pkt_off, buf_off_xpos, buf_off_width, buf_off_ypos, buf_off_height;
    int alpha_hori, alpha_vert;
    uint8_t npx[3], px[3];
    uint8_t *buffer = out->planes[0];

    if (inst->dump) {
        dump_image_rgb(inst, buffer, width, height);
    }

    xdistance = 256 / (inst->width - inst->xpos);
    ydistance = 256 / (inst->height - inst->ypos);
    for (row = inst->ypos; row < inst->height; row++) {
        distance_north = inst->height - row;
        alpha_vert = ydistance * distance_north;

        buf_off_xpos = ((height-row)*width+inst->xpos) * 3;
        buf_off_width = ((height-row)*width+inst->width) * 3;

        for (col = inst->xpos; col < inst->width; col++) {
            distance_west  = inst->width - col;

            alpha_hori = xdistance * distance_west;

            buf_off_ypos = ((height-inst->ypos)*width+col) * 3;
            buf_off_height = ((height-inst->height)*width+col) * 3;
            buf_off = ((height-row)*width+col) * 3;

            pkt_off = (row-inst->ypos) * (inst->width-inst->xpos) + (col-inst->xpos);

            /* R */
            hcalc  = alpha_blending(buffer[buf_off_xpos +0], buffer[buf_off_width  +0], alpha_hori);
            vcalc  = alpha_blending(buffer[buf_off_ypos +0], buffer[buf_off_height +0], alpha_vert);
            npx[0] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
            /* G */
            hcalc  = alpha_blending(buffer[buf_off_xpos +1], buffer[buf_off_width  +1], alpha_hori);
            vcalc  = alpha_blending(buffer[buf_off_ypos +1], buffer[buf_off_height +1], alpha_vert);
            npx[1] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
            /* B */
            hcalc  = alpha_blending(buffer[buf_off_xpos +2], buffer[buf_off_width  +2], alpha_hori);
            vcalc  = alpha_blending(buffer[buf_off_ypos +2], buffer[buf_off_height +2], alpha_vert);
            npx[2] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
            if (!inst->alpha) {
                buffer[buf_off +0] = npx[0];
                buffer[buf_off +1] = npx[1];
                buffer[buf_off +2] = npx[2];
            } else {
                px[0] = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
                px[1] = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].green);
                px[2] = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].blue);
                buffer[buf_off +0] = alpha_blending(buffer[buf_off +0], npx[0], px[0]);
                buffer[buf_off +1] = alpha_blending(buffer[buf_off +1], npx[1], px[1]);
                buffer[buf_off +2] = alpha_blending(buffer[buf_off +2], npx[2], px[2]);
            }
        }
    }

    return inst->draw_border(inst, out, width, height);
}


static int process_frame_rgb_shape(logoaway_instance_t *inst,
                                   f0rx_frame_t *out, int width, int height)
{
    int row, col, i = 0;
    int xdistance, ydistance, distance_west, distance_north;
    unsigned char hcalc, vcalc;
    int buf_off, pkt_off, buf_off_xpos, buf_off_width, buf_off_ypos, buf_off_height;
    int alpha_hori, alpha_vert;
    uint8_t tmpx, px[3], npx[3];
    uint8_t *buffer = out->planes[0];

    if (inst->dump) {
        dump_image_rgb(inst, buffer, width, height);
    }

    xdistance = 256 / (inst->width - inst->xpos);
    ydistance = 256 / (inst->height - inst->ypos);
    for (row = inst->ypos; row<inst->height; row++) {
        distance_north = inst->height - row;

        alpha_vert = ydistance * distance_north;

        for (col = inst->xpos; col<inst->width; col++) {
            distance_west  = inst->width - col;

            alpha_hori = xdistance * distance_west;

            buf_off = ((height-row)*width+col) * 3;
            pkt_off = (row-inst->ypos) * (inst->width-inst->xpos) + (col-inst->xpos);

            buf_off_xpos = ((height-row)*width+inst->xpos) * 3;
            buf_off_width = ((height-row)*width+inst->width) * 3;
            buf_off_ypos = ((height-inst->ypos)*width+col) * 3;
            buf_off_height = ((height-inst->height)*width+col) * 3;

            tmpx = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i].red);
            i = 0;
            while ((tmpx != 255) && (col-i > inst->xpos))
                i++;
            buf_off_xpos   = ((height-row)*width + col-i) * 3;
            tmpx = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i].red);
            i = 0;
            while ((tmpx != 255) && (col + i < inst->width))
                i++;
            buf_off_width  = ((height-row)*width + col+i) * 3;

            tmpx = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i*(inst->width-inst->xpos)].red);
            i = 0;
            while ((tmpx != 255) && (row - i > inst->ypos))
                i++;
            buf_off_ypos   = (height*width*3)-((row-i)*width - col) * 3;
            tmpx = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i*(inst->width-inst->xpos)].red);
            i = 0;
            while ((tmpx != 255) && (row + i < inst->height))
                i++;
            buf_off_height = (height*width*3)-((row+i)*width - col) * 3;

            /* R */
            hcalc  = alpha_blending(buffer[buf_off_xpos +0], buffer[buf_off_width  +0], alpha_hori);
            vcalc  = alpha_blending(buffer[buf_off_ypos +0], buffer[buf_off_height +0], alpha_vert);
            npx[0] = (hcalc*inst->xweight + vcalc*inst->yweight)/100;
            px[0]  = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
            /* G */
            hcalc = alpha_blending(buffer[buf_off_xpos +1], buffer[buf_off_width  +1], alpha_hori);
            vcalc = alpha_blending(buffer[buf_off_ypos +1], buffer[buf_off_height +1], alpha_vert);
            npx[1] = (hcalc*inst->xweight + vcalc*inst->yweight)/100;
            px[1] = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].green);
            /* B */
            hcalc = alpha_blending(buffer[buf_off_xpos +2], buffer[buf_off_width  +2], alpha_hori);
            vcalc = alpha_blending(buffer[buf_off_ypos +2], buffer[buf_off_height +2], alpha_vert);
            npx[2] = (hcalc*inst->xweight + vcalc*inst->yweight)/100;
            px[2] = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].blue);

            buffer[buf_off +0] = alpha_blending(buffer[buf_off +0], npx[0], px[0]);
            buffer[buf_off +0] = alpha_blending(buffer[buf_off +0], npx[1], px[1]);
            buffer[buf_off +0] = alpha_blending(buffer[buf_off +0], npx[2], px[2]);
        }
    }

    return inst->draw_border(inst, out, width, height);
}

static int process_frame_yuv_solid(logoaway_instance_t *inst,
                                   f0rx_frame_t *out, int width, int height)
{
    uint8_t px;
    int row, col, buf_off, pkt_off = 0;
    uint8_t *Y = NULL, *Cb = NULL/* sic */, *Cr = NULL /* sic */;

    /* Y */
    Y = out->planes[0];
    for (row = inst->ypos; row < inst->height; row++) {
        for (col = inst->xpos; col < inst->width; col++) {

            buf_off = row * width + col;
            pkt_off = (row - inst->ypos) * (inst->width - inst->xpos) + (col - inst->xpos);
            if (!inst->alpha) {
                Y[buf_off] = inst->ycolor;
            } else {
                px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
                Y[buf_off] = alpha_blending(Y[buf_off], inst->ycolor, px);
            }
        }
    }

    /* Cb, Cr */
    Cr = out->planes[1];
    Cb = out->planes[2];
    for(row = inst->ypos/2+1; row < inst->height/2; row++) {
        for(col = inst->xpos/2+1; col < inst->width/2; col++) {
            buf_off = row * width/2 + col;
            pkt_off = (row * 2 - inst->ypos) * (inst->width - inst->xpos) + (col * 2 - inst->xpos);

            if (!inst->alpha) {
                Cr[buf_off] = inst->ucolor;
                Cb[buf_off] = inst->vcolor;
            } else {
                /* sic */
                px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
                Cr[buf_off] = alpha_blending(Cr[buf_off], inst->ucolor, px);
                Cb[buf_off] = alpha_blending(Cb[buf_off], inst->vcolor, px);
            }
        }
    }

    return inst->draw_border(inst, out, width, height);
}

static int process_frame_yuv_xy(logoaway_instance_t *inst,
                                f0rx_frame_t *out, int width, int height)
{
    int row, col;
    int xdistance, ydistance, distance_west, distance_north;
    unsigned char hcalc, vcalc;
    int buf_off, pkt_off=0, buf_off_xpos, buf_off_width, buf_off_ypos, buf_off_height;
    int alpha_hori, alpha_vert;
    uint8_t px;
    uint8_t *Y = NULL, *Cb = NULL/* sic */, *Cr = NULL /* sic */;


    /* Y' */
    Y = out->planes[0];
      xdistance = 256 / (inst->width - inst->xpos);
      ydistance = 256 / (inst->height - inst->ypos);
      for(row=inst->ypos; row<inst->height; row++) {
        distance_north = inst->height - row;

        alpha_vert = ydistance * distance_north;

        buf_off_xpos = row*width+inst->xpos;
        buf_off_width = row*width+inst->width;

        for(col=inst->xpos; col<inst->width; col++) {
          uint8_t npx;
          distance_west  = inst->width - col;

          alpha_hori = xdistance * distance_west;

          buf_off = row*width+col;
          buf_off_ypos = inst->ypos*width+col;
          buf_off_height = inst->height*width+col;

          pkt_off = (row-inst->ypos) * (inst->width-inst->xpos) + (col-inst->xpos);

          hcalc = alpha_blending(Y[buf_off_xpos], Y[buf_off_width],  alpha_hori);
          vcalc = alpha_blending(Y[buf_off_ypos], Y[buf_off_height], alpha_vert);
          npx   = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
          if (!inst->alpha) {
            Y[buf_off] = npx;
          } else {
            px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
            Y[buf_off] = alpha_blending(Y[buf_off], npx, px);
          }
        }
      }

      /* Cb, Cr */
    Cr = out->planes[1];
    Cb = out->planes[2];
      xdistance = 512 / (inst->width - inst->xpos);
      ydistance = 512 / (inst->height - inst->ypos);
      for (row=inst->ypos/2+1; row<inst->height/2; row++) {
        distance_north = inst->height/2 - row;

        alpha_vert = ydistance * distance_north;

        buf_off_xpos = row*width/2+inst->xpos/2;
        buf_off_width = row*width/2+inst->width/2;

        for (col=inst->xpos/2+1; col<inst->width/2; col++) {
          uint8_t npx[2];
          distance_west  = inst->width/2 - col;

          alpha_hori = xdistance * distance_west;

          buf_off = row*width/2+col;
          buf_off_ypos = inst->ypos/2*width/2+col;
          buf_off_height = inst->height/2*width/2+col;

          pkt_off = (row*2-inst->ypos) * (inst->width-inst->xpos) + (col*2-inst->xpos);

          hcalc  = alpha_blending(Cr[buf_off_xpos], Cr[buf_off_width],  alpha_hori);
          vcalc  = alpha_blending(Cr[buf_off_ypos], Cr[buf_off_height], alpha_vert);
          npx[0] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
          hcalc = alpha_blending( Cb[buf_off_xpos], Cb[buf_off_width],  alpha_hori );
          vcalc = alpha_blending( Cb[buf_off_ypos], Cb[buf_off_height], alpha_vert );
          npx[1] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);

          if (!inst->alpha) {
            Cr[buf_off] = npx[0];
            Cb[buf_off] = npx[1];
          } else {
            px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red); /* sic */
            Cr[buf_off] = alpha_blending(Cr[buf_off], npx[0], px);
            Cb[buf_off] = alpha_blending(Cb[buf_off], npx[1], px);
          }
        }
      }

    return inst->draw_border(inst, out, width, height);
}

static int process_frame_yuv_shape(logoaway_instance_t *inst,
                                   f0rx_frame_t *out, int width, int height)
{
    int row, col, i;
    int xdistance, ydistance, distance_west, distance_north;
    unsigned char hcalc, vcalc;
    int buf_off, pkt_off=0, buf_off_xpos, buf_off_width, buf_off_ypos, buf_off_height;
    int alpha_hori, alpha_vert;
    uint8_t px, npx[3];
    uint8_t *Y = NULL, *Cb = NULL/* sic */, *Cr = NULL /* sic */;


    /* Y */
    Y = out->planes[0];
      xdistance = 256 / (inst->width - inst->xpos);
      ydistance = 256 / (inst->height - inst->ypos);
      for(row=inst->ypos; row<inst->height; row++) {
        distance_north = inst->height - row;

        alpha_vert = ydistance * distance_north;

        for(col=inst->xpos; col<inst->width; col++) {
          distance_west  = inst->width - col;

          alpha_hori = xdistance * distance_west;

          buf_off = (row*width+col);
          pkt_off = (row-inst->ypos) * (inst->width-inst->xpos) + (col-inst->xpos);

          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i].red);
          while( (px != 255) && (col-i>inst->xpos) ) i++;
          buf_off_xpos   = (row*width + col-i);
          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i].red);
          while( (px != 255) && (col+i<inst->width) ) i++;
          buf_off_width  = (row*width + col+i);

          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i*(inst->width-inst->xpos)].red);
          while( (px != 255) && (row-i>inst->ypos) ) i++;
          buf_off_ypos   = ((row-i)*width + col);
          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i*(inst->width-inst->xpos)].red);
          while( (px != 255) && (row+i<inst->height) ) i++;
          buf_off_height = ((row+i)*width + col);

          hcalc  = alpha_blending(Y[buf_off_xpos], Y[buf_off_width],  alpha_hori );
          vcalc  = alpha_blending(Y[buf_off_ypos], Y[buf_off_height], alpha_vert );
          px     = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
          npx[0] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100); /* FIXME */
          Y[buf_off] = alpha_blending(Y[buf_off], npx[0], px);
        }
      }

      /* Cb, Cr */
    Cr = out->planes[1];
    Cb = out->planes[2];
      xdistance = 512 / (inst->width - inst->xpos);
      ydistance = 512 / (inst->height - inst->ypos);
      for (row=inst->ypos/2+1; row<inst->height/2; row++) {
        distance_north = inst->height/2 - row;

        alpha_vert = ydistance * distance_north;

        for (col=inst->xpos/2+1; col<inst->width/2; col++) {
          distance_west  = inst->width/2 - col;

          alpha_hori = xdistance * distance_west;

          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i].red);
          while( (px != 255) && (col-i>inst->xpos) ) i++;
          buf_off_xpos   = (row*width/2 + col-i);
          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i].red);
          while( (px != 255) && (col+i<inst->width) ) i++;
          buf_off_width  = (row*width/2 + col+i);

          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off-i*(inst->width-inst->xpos)].red);
          while( (px != 255) && (row-i>inst->ypos) ) i++;
          buf_off_ypos   = ((row-i)*width/2 + col);
          i=0;
          px = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off+i*(inst->width-inst->xpos)].red);
          while( (px != 255) && (row+i<inst->height) ) i++;
          buf_off_height = ((row+i)*width/2 + col);

          buf_off = row*width/2+col;
          buf_off_ypos = inst->ypos/2*width/2+col;
          buf_off_height = inst->height/2*width/2+col;

          pkt_off = (row*2-inst->ypos) * (inst->width-inst->xpos) + (col*2-inst->xpos);

          px     = (uint8_t)ScaleQuantumToChar(inst->pixels[pkt_off].red);
          /* sic */
          hcalc  = alpha_blending(Cr[buf_off_xpos], Cr[buf_off_width],  alpha_hori);
          vcalc  = alpha_blending(Cr[buf_off_ypos], Cr[buf_off_height], alpha_vert);
          npx[1] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);
          hcalc  = alpha_blending(Cb[buf_off_xpos], Cb[buf_off_width],  alpha_hori);
          vcalc  = alpha_blending(Cb[buf_off_ypos], Cb[buf_off_height], alpha_vert);
          npx[2] = ((hcalc*inst->xweight + vcalc*inst->yweight)/100);

          Cr[buf_off] = alpha_blending(Cr[buf_off], npx[1], px);
          Cb[buf_off] = alpha_blending(Cb[buf_off], npx[2], px); 
        }
      }


    return inst->draw_border(inst, out, width, height);
}


/*************************************************************************/

int f0rx_is_ready(f0r_instance_t instance, int param_index)
{
    int ready = 0;
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    if (inst) {
        ready = (inst->process_frame != process_frame_null);
        /* FIXME: this is not properly enforced on errors. */
    }
    return ready;
}

int f0r_init()
{
    /* intialization deferred into f0r_construct */
    return 1;
}

void f0r_deinit()
{
    /* deintialization deferred into f0r_destruct */
    return;
}

void f0r_get_plugin_info(f0r_plugin_info_t* info)
{
    info->name = "logoaway";
    info->author = "Thomas Wehrspann";
    info->plugin_type = F0R_PLUGIN_TYPE_FILTER;
    info->color_model = F0R_COLOR_MODEL_RGBA8888;
    info->frei0r_version = FREI0R_MAJOR_VERSION;
    info->major_version = 0;
    info->minor_version = 1;
    info->num_params =  1;
    info->explanation = ""
"This filter removes an image in a user specified area from the video."
"";
}

void f0rx_get_plugin_info(f0rx_plugin_info_t* info)
{
    f0r_get_plugin_info(&(info->base));
    info->content_type = F0RX_CONTENT_TYPE_VIDEO;
    info->sample_model = 0;
}


void f0r_get_param_info(f0r_param_info_t* info, int param_index)
{
    switch (param_index) {
      case 0:
        info->name = "Start";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The first frame to consider.";
        break;
      case 1:
        info->name = "End";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The last frame to consider.";
        break;
      case 2:
        info->name = "Position X";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The X position of the logo.";
        break;
      case 3:
        info->name = "Position Y";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The Y position of the logo.";
        break;
      case 4:
        info->name = "Size X";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The X size of the logo.";
        break;
      case 5:
        info->name = "Size Y";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The Y size of the logo.";
        break;
      case 6:
        info->name = "Mode";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "The operational mode of the filter.";
        break;
      case 7:
        info->name = "Border";
        info->type = F0R_PARAM_BOOL;
        info->explanation = "Toggle the border visibility.";
        break;
      case 8:
        info->name = "Dump";
        info->type = F0R_PARAM_BOOL;
        info->explanation = "Dump filter area to file.";
        break;
      case 9:
        info->name = "Weight";
        info->type = F0R_PARAM_DOUBLE;
        info->explanation = "X-Y percent weight.";
        break;
      case 10:
        info->name = "Fill";
        info->type = F0R_PARAM_COLOR;
        info->explanation = "Solid fill color (RGB).";
        break;
      case 11:
        info->name = "File";
        info->type = F0R_PARAM_STRING;
        info->explanation = "Image with alpha/shape information";
        break;
      case 12:
        info->name = "Dump";
        info->type = F0R_PARAM_BOOL;
        info->explanation = "TODO";
      default:
        break; /* XXX */
    }
    return;
}

static f0rx_param_attr_t *f0rx_get_param_attr(f0r_instance_t instance,
                                              int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    return &inst->attrs[param_index];
}


static void logoaway_defaults(logoaway_instance_t *inst)
{
    inst->start         = 0;
    inst->end           = (unsigned int)-1;
    inst->xpos          = -1;
    inst->ypos          = -1;
    inst->width         = -1;
    inst->height        = -1;
    inst->mode          = 0;
    inst->border        = 0;
    inst->xweight       = 50;
    inst->yweight       = 50;
    inst->rcolor        = 0;
    inst->gcolor        = 0;
    inst->bcolor        = 0;
    inst->ycolor        = 16;
    inst->ucolor        = 128;
    inst->vcolor        = 128;
    inst->alpha         = 0;
    inst->dump          = 0;
}


/* TODO: where this has to be put?
static int logoaway_options_are_valid(logoaway_instance_t *inst)
{
    if ((inst->xpos > vob->im_v_width) || (inst->ypos > vob->im_v_height)
     || (inst->xpos < 0) || (inst->ypos < 0))  {
        tc_log_error(MOD_NAME, "invalid position");
        return 0;
    }
    if ((inst->width > vob->im_v_width) || (inst->height > vob->im_v_height)
     || (inst->width-inst->xpos < 0) || (inst->height-inst->ypos < 0)) {
        tc_log_error(MOD_NAME, "invalid size");
        return 0;
    }
    if ((inst->xweight > 100) || (inst->xweight < 0)) {
        tc_log_error(MOD_NAME, "invalid x weight");
        return 0;
    }
    if ((inst->mode < 0) || (inst->mode > 3)) {
        tc_log_error(MOD_NAME, "invalid mode");
        return 0;
    }
    if ((inst->mode == 3) && (inst->alpha == 0)) {
        tc_log_error(MOD_NAME, "alpha/shape file needed for SHAPE-mode");
        return 0;
    }
    return 1;
}
*/

static int logoaway_setup(logoaway_instance_t *inst)
{
    /* FIXME magic numbers */
    static const logoaway_process_frame_t procs[4][2] = {
        { process_frame_null,      process_frame_null      },
        { process_frame_yuv_solid, process_frame_rgb_solid },
        { process_frame_yuv_xy,    process_frame_rgb_xy    },
        { process_frame_yuv_shape, process_frame_rgb_shape },
    };

    int isRGB = (inst->colorspace == F0R_COLOR_MODEL_RGBA8888);
    if (inst->dump) {
        inst->dump_buf = tc_malloc((inst->width - inst->xpos)*(inst->height - inst->ypos)*3);
        /* FIXME */
        if (inst->dump_buf == NULL){
            return -1; /* FIXME */
        }

        tc_magick_init(&inst->dump_ctx, GMLIB_QUALITY_DEFAULT);
    }

    if (inst->alpha) {
        int ret = 0;

        tc_magick_init(&inst->logo_ctx, GMLIB_QUALITY_DEFAULT);

        ret =  tc_magick_filein(&inst->logo_ctx, inst->file);
        if (ret != 0) {
            free_dump_buf(inst);
            return ret;
        }

        if ((inst->logo_ctx.image->columns != (inst->width - inst->xpos))
          || (inst->logo_ctx.image->rows != (inst->height - inst->ypos))) {
            /* FIXME 
            tc_log_error(MOD_NAME, "\"%s\" has incorrect size", inst->file);*/
            free_dump_buf(inst);
            return -1;
        }

        inst->pixels = GetImagePixels(inst->logo_ctx.image, 0, 0,
                                    inst->logo_ctx.image->columns,
                                    inst->logo_ctx.image->rows);
    }

    if (isRGB) {
        inst->draw_border = draw_border_rgb;
        inst->pack_frame  = pack_frame_rgb;
        inst->copy_frame  = copy_frame_rgb;
    } else {
        inst->draw_border = draw_border_yuv;
        inst->pack_frame  = pack_frame_yuv;
        inst->copy_frame  = copy_frame_yuv;
    }
    inst->process_frame = procs[inst->mode][isRGB];
    return 0;
}



static int logoaway_init(logoaway_instance_t *inst, int format,
                          unsigned int param1, unsigned int param2)
{
    int err = 0;
    logoaway_defaults(inst);
    inst->imgwidth      = param1;
    inst->imgheight     = param2;
    inst->draw_border   = draw_border_null;
    inst->process_frame = process_frame_null;
    return err;
}

static void post_set_weight(f0r_instance_t instance, int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    inst->yweight = 100 - inst->xweight;
}

static void post_set_color(f0r_instance_t instance, int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    inst->rcolor = inst->color.r * 255;
    inst->gcolor = inst->color.g * 255;
    inst->bcolor = inst->color.b * 255;
    inst->ycolor =  (0.257 * inst->rcolor) + (0.504 * inst->gcolor) + (0.098 * inst->bcolor) + 16;
    inst->ucolor =  (0.439 * inst->rcolor) - (0.368 * inst->gcolor) - (0.071 * inst->bcolor) + 128;
    inst->vcolor = -(0.148 * inst->rcolor) - (0.291 * inst->gcolor) + (0.439 * inst->bcolor) + 128;
}

static void post_set_position(f0r_instance_t instance,
                                       int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    inst->width += inst->xpos;
    inst->height += inst->ypos;
}

static void post_set_file(f0r_instance_t instance,
                                   int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    inst->alpha = 1;
}


static void post_set_setup(f0r_instance_t instance,
                                    int param_index)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    post_set_file(instance, param_index);
    logoaway_setup(inst);
}



f0r_instance_t f0rx_construct(int format,
                              unsigned int param1, unsigned int param2)
{
    logoaway_instance_t* inst = NULL;
    if (format == F0R_COLOR_MODEL_RGBA8888
     || format == F0RX_COLOR_MODEL_PLANAR_YUV420P) {
        inst = tc_zalloc(sizeof(*inst));
    } else {
        /* log it */
    }
    if (inst) {
        int err = logoaway_init(inst, format, param1, param2);
        if (!err) {
            f0rx_bind_param_attr_remap_int(inst,  0, &inst->start);
            f0rx_bind_param_attr_remap_int(inst,  1, &inst->end);
            f0rx_bind_param_attr_remap_int(inst,  2, &inst->xpos);
            f0rx_bind_param_attr_remap_int(inst,  3, &inst->ypos);
            f0rx_bind_param_attr_remap_int(inst,  4, &inst->width);
            f0rx_bind_param_attr_remap_int(inst,  5, &inst->height);
            f0rx_bind_param_attr_remap_int(inst,  6, &inst->mode);
            f0rx_bind_param_attr_remap_int(inst,  7, &inst->border);
            f0rx_bind_param_attr_remap_int(inst,  8, &inst->dump);
            f0rx_bind_param_attr_remap_int(inst,  9, &inst->xweight);
            f0rx_bind_param_attr(inst,           10, &inst->color);
            f0rx_bind_param_attr(inst,           11, &inst->file);
            f0rx_bind_param_attr_remap_int(inst, 12, &inst->dump);

            f0rx_hook_param_attr_post_set(inst,  2, post_set_position);
            f0rx_hook_param_attr_post_set(inst,  3, post_set_position);
            f0rx_hook_param_attr_post_set(inst,  4, post_set_position);
            f0rx_hook_param_attr_post_set(inst,  5, post_set_position);
            f0rx_hook_param_attr_post_set(inst,  9, post_set_weight);
            f0rx_hook_param_attr_post_set(inst, 10, post_set_color);

            f0rx_hook_param_attr_post_set(inst,  6, post_set_setup);
            f0rx_hook_param_attr_post_set(inst, 11, post_set_setup);
            f0rx_hook_param_attr_post_set(inst, 12, post_set_setup);
        } else {
            f0r_destruct(inst);
            inst = NULL;
        }
    }
    return (f0r_instance_t)inst;
}

f0r_instance_t f0r_construct(unsigned int width, unsigned int height)
{
    return f0rx_construct(F0R_COLOR_MODEL_RGBA8888, width, height);
}


void f0r_destruct(f0r_instance_t instance)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    RETURN_IF_NULL(inst);
    tc_magick_fini(&inst->logo_ctx);
    tc_magick_fini(&inst->dump_ctx);
    free_dump_buf(inst);
    free(instance);
    return;
}

int f0rx_update2(f0r_instance_t instance,
                 const f0rx_frame_t* in1,
                 const f0rx_frame_t* in2,
                 const f0rx_frame_t* in3,
                 f0rx_frame_t* out)
{
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    RETURN_ERROR_IF_NULL(inst);
    inst->copy_frame(inst, out, in1);
    if (inst->frames >= inst->start && inst->frames <= inst->end) {
        inst->process_frame(inst, out, inst->width, inst->height);
    }
    inst->frames++;
    return 0;
}

void f0r_update2(f0r_instance_t instance, double time,
                 const uint32_t* inframe1,
                 const uint32_t* inframe2,
                 const uint32_t* inframe3,
                 uint32_t* outframe)
{
    f0rx_frame_t in;
    f0rx_frame_t out;
    logoaway_instance_t* inst = (logoaway_instance_t*)instance;
    RETURN_IF_NULL(inst);
    inst->pack_frame(inst, &in,  (void *)inframe1, time);
    inst->pack_frame(inst, &out, (void *)outframe, time);
    f0rx_update2(instance, &in, NULL, NULL, &out);
    return;
}


/*
 * vim: expandtab shiftwidth=4:
 */

