/*
 * l0ader.h - simpl{e,istic} frei0r(x) plugin loader interface.
 * (C) 2011 Francesco Romani <fromani at gmail dot com>
 * license: ZLIB. Or GPLv2 if you wish.
 */

#ifndef TYCHO_PLUGINS_L0ADER_H
#define TYCHO_PLUGINS_L0ADER_H

#include <stdarg.h>
#include "logkit.h"
#include "frei0rx.h"

/* internal usage for the l0ader. */
typedef struct f0rxplugin_ *f0rx_plugin_t;

typedef struct f0rxpluginmetadata_ f0rx_plugin_meta_data_t;
struct f0rxpluginmetadata_ {
    f0rx_plugin_info_t info;
    f0r_param_info_t *params;
    int extended;
};

typedef struct f0rxelement_ f0rx_element_t;
struct f0rxelement_ {
    f0r_instance_t instance;
    f0rx_plugin_t plugin;
    int id; /* instance id */
    void (*set_param_value)(f0r_instance_t instance,
                             f0r_param_t param, int param_index);
    void (*get_param_value)(f0r_instance_t instance,
                             f0r_param_t param, int param_index);
    void (*update)(f0r_instance_t instance,
                    double time, const uint32_t* inframe, uint32_t* outframe);
    void (*update2)(f0r_instance_t instance,
                     double time,
                     const uint32_t* inframe1,
                     const uint32_t* inframe2,
                     const uint32_t* inframe3,
                     uint32_t* outframe);
    /* extensions */
    int (*xset_params_values)(f0r_instance_t instance,
                              f0r_param_t *params);
    int (*xis_ready)(f0r_instance_t instance, int param_index);
    int (*xupdate)(f0r_instance_t instance,
                   double intime,
                   const uint32_t *inframe1,
                   const uint32_t *inframe2,
                   const uint32_t *inframe3,
                   double *outtime,
                   uint32_t *outframe);
};


enum {
    F0RX_LOADER_FLAG_NONE       = 0,
    F0RX_LOADER_FLAG_EXTENDED   = 1<<0, /**< load an extended plugin */
    F0RX_LOADER_FLAG_FALLBACK   = 1<<1, /**< don't bail out if
                                            extended symbol are missing */
    F0RX_LOADER_FLAG_STUBS      = 1<<2
};
/* meh */
#define F0RX_LOADER_FLAG_DEFAULTS    (F0RX_LOADER_FLAG_EXTENDED|F0RX_LOADER_FLAG_FALLBACK)

// TODO: add "missing plugin" error code
typedef enum {
    F0RX_LOADER_ERROR_NONE           =   0, /**< all clean */
    F0RX_LOADER_ERROR_REF_INVALID    =  -1, /**< invalid/NULL pointer */
    F0RX_LOADER_ERROR_MEM_EXAUSTED   =  -2, /**< no more memory ({c,m,re}alloc failed) */
    F0RX_LOADER_ERROR_PLUGIN_LOADING =  -3, /**< error loading a plugin */
    F0RX_LOADER_ERROR_PLUGIN_MISSING =  -4, /**< error loading a plugin */
    F0RX_LOADER_ERROR_PLUGIN_SYMBOL  =  -5, /**< a plugin lacks a mandatory symbol */
    F0RX_LOADER_ERROR_PLUGIN_ERROR   =  -6  /**< plugin internal error */
} f0rx_loader_error_t;

/* factory pattern */
typedef struct f0rxloader_ *f0rx_loader_t;

int f0rx_loader_build(f0rx_loader_t *loader, CX_LogContext *log_context);
int f0rx_loader_clean(f0rx_loader_t *loader);

int f0rx_loader_get_plugin_meta_data(f0rx_loader_t loader,
                                     const char *vendor, const char *name,
                                     f0rx_plugin_meta_data_t *meta_data);

int f0rx_loader_load_element(f0rx_loader_t loader,
                             const char *vendor, const char *name,
                             uint32_t flags, int format, f0r_param_t *params,
                             unsigned int value1, unsigned int value2,
                             f0rx_element_t **element);

int f0rx_loader_unload_element(f0rx_loader_t loader, f0rx_element_t **element);

/* error handling (helper) */
const char *f0rx_loader_strerror(int error);

#endif /* TYCHO_PLUGINS_L0ADER_H */

/*
 * vim: expandtab shiftwidth=4:
 */

