//
//  Copyright (C) 13-02-2012 Jasper den Ouden.
//
//  This is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later verssion.
//

#pragma once

#include "StackCom.h"
#include "inbuild_names.h"

#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

typedef struct
{
  char* name;
  int len; byte* code;
} NameVal;

//Names all the things.
typedef struct ClientNamer
{
  int cnt; //Total number of name-values.
  int defs_counted; // `StackCom::def_cnt` number.
  NameVal* names;
} ClientNamer;

//Initialize client namer, client name ignored here, but gdbm version uses it
// for the DB name.
ClientNamer* ClientNamer_init(ClientNamer* cn, const char* client_name)
{
  cn->cnt=0; cn->defs_counted=0;
  cn->names= NULL;
  return cn;
}
void ClientNamer_exit(ClientNamer* cn)
{ if( cn->names!= NULL ){ free(cn->names); } }

//Raw function, not available for gdbm version.
// Adds just a name, not the code. Leaves a bad state! 
// `ClientNamer_name_code`, `ClientNamer_name_fun`, `ClientNamer_name_funptr`
// use it.(And are user-facing)
NameVal* ClientNamer_add_name(ClientNamer* cn, char* name)
{
  cn->cnt++;
  if( cn->names==NULL )
    { assert( cn->cnt==1 );
      cn->names= (NameVal*)malloc(sizeof(NameVal));
    }
  else
    { cn->names= (NameVal*)realloc((void*)cn->names, sizeof(NameVal)*cn->cnt);
    }
  NameVal* nv= cn->names + cn->cnt-1;
  nv->name= (char*)malloc(strlen(name));
  memcpy((void*)nv->name,(void*)name, strlen(name)+1);
  return nv;
}
//Give a piece of code a name. That is basically what _all_ the namers, do,
// some of them just provide some more specific functionality.
void ClientNamer_name_code(ClientNamer* cn, 
			  char* name, byte* code, int len)
{
  NameVal* nv= ClientNamer_add_name(cn,name);
  nv->len=len;
  
  nv->code= (byte*)malloc(len);
  memcpy((void*)nv->code, (void*)code, len);
}
//Name a new definition.(They are always numbered in sequence.)
// Returns the index of the definition _on the eval side_
int ClientNamer_name_def(ClientNamer* cn, char* name)
{
  assert( cn->defs_counted< 256 );
  NameVal* nv= ClientNamer_add_name(cn,name);
  nv->len=2;
  nv->code= malloc(2); nv->code[0]= ib_usedef; nv->code[1]= cn->defs_counted;
  cn->defs_counted++;
  return cn->defs_counted-1;
}
//Name a number to be a function, function pointer _must already_ be in
// StackCom::funs (added by for instance `StackCom_add_fun`)
void ClientNamer_name_fun(ClientNamer* cn, char* name, int fun_n)
{
  NameVal* nv= ClientNamer_add_name(cn,name);
  nv->len = (inbuild_cnt + fun_n < 256) ? 1 : 2;
  nv->code= (byte*)malloc(nv->len);
  
  if( nv->len==1 )
    { nv->code[0]= inbuild_cnt + fun_n; }
  else
    { assert( fun_n<256 );
      nv->code[0]= ib_fun; nv->code[1]= fun_n;
    }
}
//Name a function pointer, doesn't have to be in StackCom::funs, but be warned
// that in different runnings of the program the location of the pointer 
// changes. Also, this takes size(StackFun*)+1 bytes, the other one takes 
// one or two bytes.
void ClientNamer_name_funptr(ClientNamer* cn, char* name,StackFun* ptr)
{ 
  NameVal* nv= ClientNamer_add_name(cn,name);
  nv->len = 1+sizeof(StackFun*);
  nv->code[0]= ib_fun_ptr;
  ((StackFun**)(nv->code+1))[0] = ptr;
}

int try_inbuilds(char* name,byte* code)
{
  for( int i=0 ; i<inbuild_cnt ; i++ ) //Look at inbuild ones.
    {
      for( int j=0 ; ; j++ )
	{ if( (name[j]=='\0'||name[j]=='\t'||name[j]=='\n'||name[j]==' ') && 
	      inbuild_names[i][j]=='\0' )
	    { code[0]=i; return 1; }  //At end, found it.
	  if( name[j] != inbuild_names[i][j] ) { break; } //Not this round.
	}
    }  
  return -1;
}

//Writes the code of inbuilds or what was assigned to it in the clientnamer.
// If `cn==NULL` it will just not look at that.
int ClientNamer_write_from_name(ClientNamer* cn, char* name,byte* code)
{
  int ret= try_inbuilds(name,code);
  if( ret>=0 ){ return ret; }

  if( cn==NULL ){ return 0; } 
  assert( !( cn->cnt>0 && cn->names==NULL ) );
  for( int i=0 ; i< cn->cnt ; i++ ) //Look at rest.
    {
      NameVal* cur= cn->names + i;
      for( int j=0 ; ; j++ )
	{ if( (name[j]=='\0'||name[j]=='\t'||name[j]=='\n'||name[j]==' ') && 
	      cur->name[j]=='\0' )
	    { //Found it.
	      memcpy((void*)code,(void*)cur->code, cur->len);
	      return cur->len;
	    }
	  if( name[j] != cur->name[j] ) { break; } //Not this round.
	}
    }
//TODO 'client side' 'macros'.
  return 0; 
}
//Find a defintion as left by `ClientNamer_name_def`
char* ClientNamer_def_name(ClientNamer* cn, int def_n)
{ if( def_n >= cn->defs_counted ){ return NULL; }
  for( int i=0 ; i< cn->cnt ; i++ )
    { NameVal* nv= cn->names + i;
      if( nv->len==2 && nv->code[0]== ib_usedef &&
	  nv->code[1]==def_n )
	{ return nv->name; }
    }
  return NULL;
}
//Find the function in there StackCom::funs array by index.
char* ClientNamer_fun_name(ClientNamer* cn, int fun_n)
{
  for( int i=0 ; i< cn->cnt ; i++ )
    { NameVal* nv= cn->names + i;
      if( nv->len==1 && nv->code[0]>= inbuild_cnt &&
	  nv->code[0]-inbuild_cnt == fun_n )
	{ return nv->name; }
      if( nv->len==2 && nv->code[0]==ib_fun &&
	  nv->code[1] == fun_n )
	{ return nv->name; }
    }
  return NULL;
}
//Find the function pointer and say it's name. The function must be made
// by or in the same way as `ClientNamer_name_funptr`
char* ClientNamer_funptr_name(ClientNamer* cn, StackFun* ptr)
{
  for( int i=0 ; i< cn->cnt ; i++ )
    { NameVal* nv= cn->names + i;
      if( 1+sizeof(StackFun*) == nv->len && nv->code[0]== ib_fun_ptr &&
	  (*(StackFun**)(nv->code+1))== ptr )
	{ return nv->name; }
    }
  return NULL;
}

void code_print_array(ClientNamer* cn, FILE* stream, byte* code,int len);
//Prints all the macros.
void ClientNamer_print_macros(ClientNamer* cn, FILE* stream)
{
  for( int i=0 ; i< cn->cnt ; i++ )
    { NameVal* nv= cn->names + i;
      fprintf(stream, "$%s(%d)  ", nv->name, nv->len);
      code_print_array(NULL,stream, nv->code,nv->len);
      printf("\n");
    }  
}

//TODO instead of printing the macros, identify simple use-macros and locating the code of the
// function involved.
