//
//  Copyright (C) 01-03-2012 Jasper den Ouden.
//
//  This is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//

//Reading in code is not entirely(but nearly) straightforward, can't cut
// `ib_int`, ib_int2 etc in half or things might fail. This library keeps a 
// bit of code and track of where we are in the code.

//This is a streamy solution
// https://www.gnu.org/software/libc/manual/html_node/String-Streams.html
//Good for computer-side, but probably not for the atmel. 

#pragma once

#include "code-util.c"
#include "stack-eval.c"

typedef struct
{ int len, max_len; //Length to use;
  byte* received; //Array of code obtained.
} CodeReceiver;

//Some (semi-)initializers and destructors.
CodeReceiver* CodeReceiver_init_ptr(CodeReceiver* cr, int max_len, byte* buf)
{ cr->len=0; cr->max_len= max_len; cr->received= buf; }
CodeReceiver *CodeReceiver_init(CodeReceiver* cr, int max_len )
{ cr->len=0; cr->max_len= max_len; cr->received= (byte*)malloc(max_len); }
void CodeReceiver_free_stuff(CodeReceiver* cr)
{ assert(cr->received!=NULL); free(cr->received); }
//Receive a single bit.
int CodeReceiver_receive_1(CodeReceiver* cr, byte byte)
{ if( cr->len >= cr->max_len ){ return 0; } //oh-oh, overflow.
  cr->received[cr->len]= byte;
  cr->len++;
  return 1; 
}
//Receive an array.
int CodeReceiver_receive(CodeReceiver* cr, byte* bytes, int cnt)
{ if( cr->len + cnt >= cr->max_len )
    { cnt= cr->max_len - cr->len -1; }
  memcpy((void*)(cr->received+ cr->len), (void*)bytes, cnt);
  return cnt;
}
//Feed what was received to the evaller.
int CodeReceiver_feed_eval(CodeReceiver* cr,StackCom* sc,void* data)
{
  int eval_len = code_feedable_len(cr->received, cr->len);
  if( eval_len==0 ){ return 1; }
  //  code_print_array(NULL,stdout, cr->received, eval_len); 
  //  printf("=(%d)\n",eval_len);
  int ret= StackCom_eval(sc, cr->received, eval_len, data);
  cr->len-= eval_len;
  if( cr->len > 0 )
    { memcpy((void*)cr->received, (void*)(cr->received + eval_len), cr->len);
    }
  assert( cr->len >= 0 );
  return ret;
}
