//
//  Copyright (C) 13-02-2012 Jasper den Ouden.
//
//  This is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//

//Uses gdbm to keep track of variables. TODO doesnt work yet.

#pragma once

#include "StackCom.h"
#include "inbuild_names.h"

#include <string.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <gdbm.h>

//Names all the things.
typedef struct ClientNamer
{
  GDBM_FILE db;
  int cnt;
} ClientNamer;

#define ClientNamer_bucket_size 512

datum string_key(char* name) //Makes a key of that name.
{ datum key; key.dptr= name; key.dsize= strlen(name); return key; }

ClientNamer* ClientNamer_init(ClientNamer* cn, const char* client_name)
{ cn->db= gdbm_open(client_name, ClientNamer_bucket_size, 
		    GDBM_WRITER, 0x0666, 0);
  byte set[2];
  char tmp[256];
  datum data,key;
  data.dptr= &set; data.dsize=2;
  key.dptr= tmp;
  for( int i=0 ; i<inbuild_cnt ; i++ ) //Look at inbuild ones.
    { set[0]=0; set[1]=i;
      key.dsize= strlen(inbuild_names[i])+1;
      memcpy(key.dptr, inbuild_names[i], key.dsize);
      printf("%s == %d , %d\n", tmp, i,key.dsize);
      assert( !gdbm_store(cn->db, key,data, GDBM_REPLACE) );
    }
  return cn;
}

void ClientNamer_exit(ClientNamer* cn)
{ gdbm_sync(cn->db);
  gdbm_close(cn->db);
}

//Not an option here!
NameVal* ClientNamer_add_name(ClientNamer* cn, char* name)
{ assert(0); }

int ClientNamer_name_code(ClientNamer* cn, 
			   char* name, byte* code, int len)
{
  datum key= string_key(name),data;
  byte* newcode= (byte*)malloc(len+1);
  newcode[0]= 0; memcpy((void*)(newcode+1), (void*)code,len);
  data.dptr= newcode; data.dsize= len+1; //And data.
    
  assert( !gdbm_store(cn->db, key,data, GDBM_REPLACE) );
  free(newcode);
  return cn->cnt++;
}
//Name a number to be a function, function pointer _must already_ be in
// StackCom::funs
int ClientNamer_name_fun(ClientNamer* cn, char* name, int fun_n)
{
  datum key= string_key(name),data;
  data.dsize= (inbuild_cnt + fun_n < 256) ? 2 : 3;
  byte code[3]; code[0]= 0; 
  data.dptr= code;
  
  if( data.dsize==2 )
    { code[1]= inbuild_cnt + fun_n; }
  else
    { assert( fun_n<256 );
      code[1]= ib_fun; code[2]= fun_n;
    }
  assert( !gdbm_store(cn->db, key,data, GDBM_REPLACE) );
  return cn->cnt++;
}
//Name a function pointer, doesn't have to be in StackCom::funs, but be warned
// that in different runnings of the program the location of the pointer 
// changes. Also, this takes size(StackFun*)+1 bytes, the other one takes 
// one or two bytes.
int ClientNamer_name_funptr(ClientNamer* cn, char* name,StackFun* ptr)
{ 
  datum key= string_key(name),data;
  data.dsize= 1+sizeof(StackFun*) +1;
  byte* code= (byte*)malloc(data.dsize);
  
  code[0]=0; code[1]= ib_fun_ptr; ((StackFun**)(code+2))[0] = ptr;
  assert( !gdbm_store(cn->db, key,data, GDBM_REPLACE) );
  free(code);
  return cn->cnt++;
}

//Writes the code of inbuilds or what was assigned to it in the clientnamer.
// If `cn==NULL` it will just not look at that.
int ClientNamer_write_from_name(ClientNamer* cn, char* name,byte* code)
{  
  assert(cn!=NULL); //TODO fallback to array of the inbuilds?
  
  while( name[0]==' ' || name[0]=='\t' || name[0]=='\n' )
    { if(name[0]=='\0'){ return 0; }else{ name++; } }
  char* name_end= name;
  while( name_end[0]!=' ' && name_end[0]!='\t' && name_end[0]!='\n' &&
         name_end[0]!='\0' )
    { name_end++; }
  char tmp= name_end[0]; name_end[0]='\0'; //Temporarily end of string.
  
  datum got= gdbm_fetch(cn->db, string_key(name));
  name_end[0]=tmp;
  
  if( got.dptr==NULL ){ return 0; }
  memcpy((void*)code,(void*)(((byte*)got.dptr)+1), got.dsize-1);

  return got.dsize;
}
//Print all the non-inbuild code.
void ClientNamer_print_non_inbuild(ClientNamer* cn, FILE* stream)
{
  datum key= gdbm_firstkey(cn->db);
  while(key.dptr!=0)
    {
      datum value= gdbm_fetch(cn->db,key);
      assert( value.dptr!=NULL );
      
      code_print(cn,stream, ((byte*)value.dptr)+1, value.dsize-1);
      free(value.dptr);
      
      datum oldkey= key;
      key= gdbm_nextkey(cn->db,key);
      free(oldkey.dptr);
    }
}

//TODO printing a bit of code.
