//
//  Copyright (C) 4-03-2012 Jasper den Ouden.
//
//  This is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//

//'Client', basically this end does the 'user side' of the story, contains 
// strings and such to talk to the user. It talks back and forth with the
// 'program side' that lacks all that.

//It depends on a `ClientNamer` which is a little lib to match names with code

#pragma once

#include "StackCom.h"
#include "code-util.c"

#include <ctype.h>
#include <string.h>
//#include <stdlib.h>
#include <stdio.h>
//Why do i need to say it? stdio doesn't?
FILE * fmemopen (void *buf, size_t size, const char *opentype);

#include <limits.h>
#include <assert.h>

#define code_read_tmp_len 64

void skip_line(FILE* f)
{ while( !feof(f) && !ferror(f) && fgetc(f)!='\n' ){} } //Sounds silly.

//Parses the code. TODO bit messy.
int symbol_parse(ClientNamer* cn, byte* code, char* str, byte indication)
{ assert( strchr(str, ' ')==NULL ); //No whitespace allowed.
  assert( strchr(str,'\t')==NULL );
  assert( strchr(str,'\n')==NULL );

  float f; int x;
  if( strchr(str,'.')!=NULL && sscanf(str, "%f", &f)>0 ) //Got a float.
    { code[0]= ib_float; 
      (*(float*)(code+1))= f;
      return 1+sizeof(float);
    }
  if( sscanf(str, "%d",&x) ) //Have an integer.
    { if( indication== ib_def || indication== ib_usedef ||
	  indication== ib_fun ) //See if we're defining/using a function.
	{ assert( x>=0 ); assert( x<256 );
	  code[0]= x;
	  return 1;
	}
      code[0]= ib_int;  //We're not; put integer.
      (*(int*)(code+1))= x;
      return 1+sizeof(int);
    }
  //Have some symbol, try find it..
  int gotlen= ClientNamer_write_from_name(cn, str,code);
  if( indication ==  ib_def ) //Should be defining something.
    { assert(gotlen==0);
      if( cn==NULL )
	{ printf("Seeing an non-inbuild symbol, but namer object for those is NULL\n"); }
      //Add macro that calls "`{ib_usedef, def_n}`" to namer, and put it here.
      code[0]= ClientNamer_name_def(cn,str); //ib_def is already here, of course.
      return 1;
    }
  if( gotlen==0 )//if( indication == client_namer_macro ) TODO writing-sequences.
    { printf("Did not recognize symbol '%s', and not defining anything either.\n", str);
      printf(" ..ignoring the symbol.\n");
      return 0;
    }
  if( gotlen==1 && code[0]== ib_endstop ) //Was an end stop.
    { return -1; }
  return gotlen;
}

int get_upto(FILE* f, char* buffer, char* stop_chars)
{
  int i=0;
  while( !feof(f) && !ferror(f) )
    { buffer[i]=fgetc(f); if( strchr(stop_chars,buffer[i])!=NULL ){ break; } i++; }
  buffer[i+1]= buffer[i]; buffer[i]= '\0';
  return i;
}

int code_parse(ClientNamer* cn, FILE* stream, byte* code, int* lenv, char* stop_chars)
{
  byte indication = ib_endstop;
  char tmp[code_read_tmp_len+1];
  char* tok_chars= malloc(4+strlen(stop_chars));
  tok_chars[0]=' ';tok_chars[1]='\n';tok_chars[2]='\n';tok_chars[2]='#';
  for( int i=0 ; i<strlen(stop_chars) ; i++ ){ tok_chars[3+i]= stop_chars[i]; }
  const int max_len= *lenv; int len=0;
  while( len < max_len && !feof(stream) && !ferror(stream) )
    {//Tokenize.
      int tok_cnt= get_upto(stream, tmp, tok_chars);
      if( tok_cnt==0 )
	{ if( strchr(stop_chars, tmp[tok_cnt+1]) )
	    { return *lenv= len; } 
	  if( tmp[tok_cnt+1]=='#' ){ skip_line(stream); }
	  continue; 
	}
      int step = symbol_parse(cn,code+len, tmp, indication);
      if( step< 0 ){ len-= step; return *lenv =len; }
      len+=step;
      if( strchr(stop_chars, tmp[tok_cnt+1]) )
	{ return *lenv= len; }
      if( tmp[tok_cnt+1]=='#' ){ skip_line(stream); }
      indication = (step==1 ? code[len-1] : ib_endstop);
    }
  return *lenv = len;
}

//Prints code from `in` to `stream` TODO is to actually use the definitions.`
void code_print_stream(ClientNamer* cn, FILE* stream, FILE* in)
{
  while( !feof(in) && !ferror(in) )
    {
      byte name=0;
      //TODO fix, might not have completed value..
      int read_len= fread((void*)&name, 1,1, in);
      //      if( read_len!=1 ){ printf("%d\n",read_len); return; }//
      if( read_len==0 ){ continue; }
      assert( read_len==1 );
      
      switch( name )
	{
	case ib_float: //Floats.
	  { float f;
	    const int read_cnt= fread((void*)&f, sizeof(float),1,in);
	    if( read_cnt==0 ){ continue; } //End of stream or something.
	    assert( read_cnt==1 );
	    fprintf(stream," f %f",&f); break;
	  }
	case ib_int: case ib_int2: case ib_int3: case ib_int4: //Integers.
	  { int iv[4], n= 1 + name - ib_int;
	    const int read_cnt= fread((void*)iv, sizeof(int)*n,1,in);
	    if( read_cnt==0 ){ continue; }
	    assert( read_cnt== 1 );
	    for(int k=0 ; k<n ; k++ ){ fprintf(stream," %d",iv[k]); }
	  } break;
	case ib_def: case ib_usedef:
	  { byte defnr; 
	    int read_len= fread((void*)&defnr, 1,1, in);
	    if( read_len==0 ) //If end of stream put it back.
	      { ungetc(name, in); continue; }
	    assert( read_len==1 );
	    char* name_str=NULL;
	    if( cn!=NULL ){ name_str= ClientNamer_def_name(cn,defnr); }
	    if( name_str==NULL ) //Don't know about name.
	      { fprintf(stream," %s%u", (name==ib_def ? "def" : "use"), 
			defnr); }
	    else //Know about names.
	      { fprintf(stream," %s", name_str); }
	  } break;
	case ib_fun:
	  { byte funnr;
	    int read_len= fread((void*)&funnr, 1,1, in);
	    if( read_len==0 ) //If end of stream put it back.
	      { ungetc(name, in); continue; }
	    char* name_str=NULL;
	    if( cn!=NULL ){ name_str= ClientNamer_fun_name(cn,funnr); }
	    if( name_str==NULL )
	      { fprintf(stream, "fun %u", funnr); }
	    else
	      { fprintf(stream," %s", name_str); } 
	  }break;
	case ib_enddef:
	  fprintf(stream, " enddef"); break;
	case ib_endstop:
	  fprintf(stream, " stop");
	  fflush(stream); break;
	default:
	  if( name< inbuild_cnt ) //Inbuilds.
	    { fprintf(stream, " %s",inbuild_names[name]); }
	  else //Function index/undefined.
	    { const byte fun_n= name - inbuild_cnt;
	      char* name_str=NULL;
	      if( cn!=NULL ){ name_str= ClientNamer_fun_name(cn,fun_n); }
	      if( name_str==NULL )
		{ fprintf(stream, " u%u", fun_n); }
	      else
		{ fprintf(stream, " %s", name_str); }
	    }
	  break;
	}
    }
}

//Prints code, given an array of code, see `code_print_stream`
void code_print_array(ClientNamer* cn, FILE* stream, byte* code,int len)
{
  if( len<0 ){ len = byte_code_len(code); }
  FILE* array_stream= fmemopen((void*)code,len, "r");
  code_print_stream(cn, stream, array_stream); 
  fclose(array_stream);
}
const char* string_eval_results[5]=
  { "success", "stack underflow", "stack overflow", "incorrect access", 
    "definition overflow" };

void print_int(FILE* stream, int* arr,int len)
{ if( len== 0 ){ return; }
  assert( len>0 );
  fprintf(stream, "%d", arr[0]);
  for( int i=1 ; i<len ; i++ )
    { fprintf(stream, "\t%d",arr[i]); }
}

void StackCom_stack_print(StackCom* sc,FILE* stream)
{ print_int(stream, sc->stack,sc->stack_len); }
