//
//  Copyright (C) 01-03-2012 Jasper den Ouden.
//
//  This is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//

#pragma once

#include "StackCom.h"
#include <stdlib.h>
#include <string.h>
#include <limits.h>

//Add a function. Returns success/where the function ended.
int StackCom_add_fun(StackCom* sc, StackFun fun, int pos)
{ 
  if( pos >= sc->funs_max_cnt ) //Too high number.
    { return -1; }
  if( pos<0 )
    { pos= sc->funs_cnt; 
      if( sc->funs_cnt > sc->funs_max_cnt ){ return -2; }//Too many functions.
      sc->funs_cnt++;
    }
  sc->funs[pos]= fun;
  return pos;
}

//Evaluates given code.

//TODO byte-wise stack? Much harder to print though! And will bug up horribly
// if user uses incorrect functions.
//Maybe a byte-wise value-stack + type-stack, distinguish between speedy and 
// slow execution, the firsts checks all the types, and the latter just runs
// it and gives the programmer an headache later.
int StackCom_eval(StackCom* sc, byte* code,int code_len, void* data)
{
  byte *ci= code; int *si= sc->stack + sc->stack_len;
  int *pos,*pos_b;
  while( ci - code < code_len )
    { 
      switch(*ci)
	{ 
	case ib_float: //Values go on the stack.
	  ci++;
	  *(float*)si= *(float*)ci;
	  si++;ci+=sizeof(float); break;
	case ib_int:  //TODO better, and one-byte, etcetera.
	  ci++;
	  *si= *(int*)ci;
	  si++; ci+=sizeof(int); break;
      //Multiple-int references for speed/space efficiency.
	case ib_int2: case ib_int3: case ib_int4: 
	  { char n= (*ci - ib_int);
	    ci++;
	    for(char i=0 ; i<n ; i++ )
	      { *si= *(int*)ci;
		si++; ci+=sizeof(int); 
	      }
	  }
	  break;
	  //Bunch of functions of two arguments. Integers:
	case ib_add: case ib_sub: case ib_mult: case ib_div: case ib_mod:
	case ib_l: case ib_le: case ib_g: case ib_ge: 
	case ib_min: case ib_max:
	  
	case ib_left_shift: case ib_right_shift:
	case ib_bit_and:  case ib_bit_or: case ib_and:
	  si--; pos= si -1;
	  if( pos - sc->stack < 0 ){ return eval_stack_underflow; }
	  switch( *ci )
	    {
	    case ib_add:  *pos += *si; break;
	    case ib_sub:  *pos -= *si; break;
	    case ib_mult: *pos *= *si; break;
	    case ib_div:  *pos /= *si; break;
	    case ib_mod:  *pos %= *si; break;
	      
	    case ib_l:    *pos = *pos < *si; break;
	    case ib_le:   *pos = *pos <= *si; break;
	    case ib_g:    *pos = *pos > *si; break;
	    case ib_ge:   *pos = *pos >= *si; break;
	      
	    case ib_min:  if( *pos > *si ){ *pos=*si; } break;
	    case ib_max:  if( *pos < *si ){ *pos=*si; } break;
	      
	    case ib_left_shift:  *pos = *pos << *si; break;
	    case ib_right_shift: *pos = *pos >> *si; break;
	    case ib_bit_and:     *pos = *pos & *si; break;
	    case ib_bit_or:      *pos = *pos | *si; break;
	    case ib_and:         *pos = *pos && *si; break;
	    }
	  ci++; break;
	  
	case ib_bit_not:
	  pos= si-1; *pos = ~*pos;
	  ci++; break;
	  
	  //TODO float version of same.  
	case ib_dup:
	  *si= *(si-1); si++; ci++; break;
	case ib_drop:
	  si--; ci++; break;
	case ib_swap:
	  pos= si-1; pos_b= pos-1;
	  {int tmp= *pos; *pos= *pos_b; *pos_b=tmp; } ci++; break;
	case ib_over:
	  *si = *(si-2); si++; ci++; break;
	case ib_rot:
	  pos= si-2; pos_b= pos-1;
	  { int tmp= *pos; *pos= *pos_b; *pos_b=*(pos_b+1); *(pos_b+1)=tmp; }
	  ci++; break;
	case ib_when:
	  if( *(si-1) ){ ci++; }
	  else{ return eval_success; }
	  break;
	  
	  //TODO index ranges?
	case ib_get_index: //TODO array for that..
	  ci++; pos= si-1;
	  if( *pos >= sc->arr_len ){ return eval_incorrect_access; }
	  *pos= sc->arr[*pos];
	  break;
	case ib_set_index:
	  ci++; pos= si-1;
	  if( *pos >= sc->arr_len ){ return eval_incorrect_access; }
	  sc->arr[*pos]= *pos; //Note that it doesn't drop it.
	  break;

	case ib_usedef: //Use a definition. 
	  ci++;
	  sc->stack_len = si - sc->stack;
	  if( sc->defs[*ci]!=NULL )
	    { int ret= StackCom_eval(sc, sc->defs[*ci], INT_MAX, data);
	      if( ret ){ return ret; }
	    }
	  si= sc->stack + sc->stack_len;
	  ci++; break;
	case ib_iuse: //Hard-coded definitions.
	  ci++;
	  sc->stack_len = si - sc->stack;
	  { int ret= StackCom_eval(sc, sc->idefs + (*(int*)ci), INT_MAX,
				   data);
	    if( ret ){ return ret; }
	  }
	  si= sc->stack + sc->stack_len;
	  ci+= sizeof(int); break;
	  
	case ib_def: //Make a definition. 
	  //(Number of definition,length of definition, definition itself.
	  { 
	    ci++; byte* pos= ci; ci++;
	    if( *pos >= sc->defs_max_cnt ){ return eval_definition_overflow; }
	    int len=1; //Record length. //TODO max length?
	    //TODO this guy can't wait..
	    while( ci[len]!= ib_enddef )// && len< code_len -(code-ci) )
	      { len++; } len++;
	    //assert( len>2 );
	    if( sc->defs[*pos]==NULL ) //Make it correct length.
	      { sc->defs[*pos]= (void*)malloc(len); }
	    else
	      { sc->defs[*pos]= (void*)realloc((void*)sc->defs[*pos],len); }
	    memcpy((void*)sc->defs[*pos],(void*)ci, len); //Copy over.
	    ci+= len;
	  }
	  break; 
	case ib_enddef: 
	  sc->stack_len = si - sc->stack;
	  return eval_success;
	case ib_fun:
	  ci++; //Next byte holds the function.	  
	  si+= sc->funs[*ci](si,data); 
	  ci++; break;
	case ib_fun_ptr:
	  ci++; //Next sizeof(StackFun*) bytes holds function.
	  si+= ((StackFun)ci)(si,data);
	  ci+=sizeof(StackFun*); break;
	default: 
	  si+= sc->funs[(*ci) - inbuild_cnt](si,data); 
	  ci++; break;
	}
      if( si - sc->stack >= sc->max_stack_len )
	{ return eval_stack_overflow; }
    }
  sc->stack_len = si - sc->stack;
  return eval_success; //==0
}
