/**
 *  @file config-common.h
 *  @brief Tries to fix various differences in compilers and libraries.
 *  @note This file should be included by CMake or autoheader.
 */

/* Fake gettext() support. */
/* TODO: Real internationalization. */
#define _(x) (x)

/* When there's no C99 stdint.h, try to use Microsoft's extension. */
#ifdef HAVE_STDINT_H
	#include <stdint.h>
#elif defined(_MSC_VER)
	#define int8_t   signed   __int8
	#define int16_t  signed   __int16
	#define int32_t  signed   __int32
	#define int64_t  signed   __int64
	#define uint8_t  unsigned __int8
	#define uint16_t unsigned __int16
	#define uint32_t unsigned __int32
	#define uint64_t unsigned __int64
#endif /* _MSC_VER */

#ifdef HAVE_ATTRIBUTE_PACKED
	#define ATTRIBUTE_PACKED __attribute__((packed))
#else /* ! HAVE_ATTRIBUTE_PACKED */
	#define ATTRIBUTE_PACKED
#endif /* ! HAVE_ATTRIBUTE_PACKED */

#ifdef HAVE_ATTRIBUTE_NORETURN
	#define ATTRIBUTE_NORETURN __attribute__((noreturn))
#else /* ! HAVE_ATTRIBUTE_NORETURN */
	#define ATTRIBUTE_NORETURN
#endif /* ! HAVE_ATTRIBUTE_NORETURN */

#ifdef HAVE_ATTRIBUTE_MALLOC
	#define ATTRIBUTE_MALLOC __attribute__((malloc))
#else /* ! HAVE_ATTRIBUTE_MALLOC */
	#define ATTRIBUTE_MALLOC
#endif /* ! HAVE_ATTRIBUTE_MALLOC */

#ifdef HAVE_ATTRIBUTE_FORMAT
	#define ATTRIBUTE_FORMAT(at, si, fi) __attribute__((format (at, si, fi)))
#else /* ! HAVE_ATTRIBUTE_FORMAT */
	#define ATTRIBUTE_FORMAT(at, si, fi)
#endif /* ! HAVE_ATTRIBUTE_FORMAT */


/* Where _vsnwprintf is available, it should be used in place of vswprintf.
 * Microsoft C runtime library has a wrong version of vswprintf.
 */
#ifdef HAVE__VSNWPRINTF
	#define vswprintf _vsnwprintf
#endif /* HAVE__VSNWPRINTF */

