/**
 *  @file getopt.c
 *  @brief Implementation of POSIX getopt (not only) for Windows.
 *
 *  @note Copyright Přemysl Janouch 2007 - 2008. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "config.h"

#include "getopt.h"


char *optarg;
int optind = 1, optopt = 0, optpos = 0, opterr = 1;


int getopt (int argc, char **argv, char *commands)
{
	char *c;
	int result = -1;

	if (!commands || !argv || !argv[optind])
		return -1;

	/* Next parameter -- check for end of options. */
	if (!optpos)
	{
		if (*argv[optind] != '-' || !argv[optind][1])
			return -1;

		if (argv[optind][2] == '-' && !argv[optind][3])
		{
			optind++;
			return -1;
		}
	}

	if ((c = strchr(commands, argv[optind][1 + optpos]))
		&& isalnum(*c))
	{
		optarg = NULL;
		result = *c;

		/* Argument required. */
		if (*(c + 1) == ':')
		{
			/* It is within the parameter. */
			if (*(argv[optind] + 2 + optpos))
				optarg = argv[optind] + 2 + optpos;
			/* It is the next parameter. */
			else if (argv[optind + 1])
				optarg = argv[++optind];
			/* It is missing. */
			else
			{
				optopt = *c;
				if (opterr)
					printf(_("Missing option argument: %c\n"), optopt);
				result = (*commands == ':' ? ':' : '?');
			}

			optind++;
			optpos = 0;
			return result;
		}
	}
	/* Unknown option. */
	else
	{
		optopt = argv[optind][1 + optpos];
		if (opterr)
			printf(_("Unknown option: %c\n"), optopt);
		result = '?';
	}

	/* Move on to the next option character or parameter. */
	if (argv[optind][2 + optpos])
		optpos++;
	else
	{
		optind++;
		optpos = 0;
	}

	return result;
}

