/**
 *  @file testwaa.c
 *  @brief Test the WAA graphics library.
 *
 *  This file is in the public domain.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config.h"

#include "xalloc.h"
#include "waa.h"

/** Draw something onto the right half of the surface. */
int render (WaaSurface *sfc)
{
	int x, y;

	for (x = 0; x < (sfc->width >> 1); x++)
		for (y = 0; y < sfc->height; y++)
			waaPutPixel(sfc, x, y, waaMapRGBA(sfc->format,
				(double) x / (sfc->width >> 1) * 255,
				(double) y /  sfc->height      * 255,
				0, WAA_ALPHA_OPAQUE));
	return 0;
}

/** Test the WAA graphics library. */
/*  TODO: Make the test much more extensive; try to find a way to test
 *        the library without actually using the (probably redirected) stdout.
 *        Also create a separate test for waautil.c
 */
int src_testwaa (int argc, char *argv[])
{
	WaaContext *ctx;
	WaaSurface *sfc;
	WaaRect src, param;

	ctx = waaInit();
	if (!ctx)
		return 1;

	/* Create a surface. */
	sfc = waaCreateSurface(WAA_FORMAT_RGB_ALPHA, ctx->width, ctx->height);
	if (!sfc)
		return 1;

	/* Draw something onto the surface. */
	waaFillRect(sfc, NULL, waaMapRGBA(sfc->format,
		0xFF, 0xFF, 0xFF, WAA_ALPHA_OPAQUE));
	if (render(sfc))
		return 1;

	/* Render the surface onto the buffer. */
	src.x = 0;
	src.y = 0;
	src.w = ctx->width;
	src.h = ctx->height;

	param.x = 0;
	param.y = 0;
	param.w = 1;
	param.h = 1;

	waaSetPalette(ctx, waaPaletteAscii);
	waaRender(ctx, sfc, &src, &param);
	waaFreeSurface(sfc);

	/* Flush the buffer onto the console. */
	waaFlush(ctx);
	waaClose(ctx);

	return 0;
}

