/**
 *  @file testconf.c
 *  @brief Test configuration files.
 *
 *  This file is in the public domain.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


struct
{
	const char *section;
	const char *key;
	const char *value;
}
*value, values[] =
{
	{NULL,   "x",  "a"},
	{NULL,   "y",  "b=c"},

	{"s",    "x",  "d"},
	{"s",    "y",  "d=f"},

	{"tuv",  "x",  "g"},
	{"tuv",  "y",  "h=i"},

	{NULL, NULL, NULL}
};


static void fail (const char *msg)
{
	puts (msg);
	exit (1);
}

static void confTestValues (Conf cfg)
{
	const char *storedValue;

	for (value = values; value->key; value++)
	{
		confSet (cfg, value->section, value->key, value->value);
		storedValue = confGet (cfg, value->section, value->key);
		if (!storedValue)
			fail ("Value not found");
		if (strcmp (storedValue, value->value))
			fail ("Value not stored properly");
	}
}

static void confTestFile (Conf cfg)
{
	char *file;

	file = tmpnam (NULL);

	puts ("-- Writing to a file");
	confWrite (cfg, file);

	puts ("-- Clearing the object");
	confClear (cfg);

	puts ("-- Reading back");
	confRead (cfg, file);
	confTestValues (cfg);
}

/** Test the Conf module. */
int testconf (void)
{
	Conf cfg;

	cfg = confCreate();

	puts ("-- Inserting values");
	for (value = values; value->key; value++)
		confSet (cfg, value->section, value->key, value->value);

	puts ("-- Reading the values back");
	confTestValues (cfg);

	puts ("-- Testing file operations");
	confTestFile (cfg);
	confDestroy (cfg);

	puts ("-- Conf test successful");
	return 0;
}

