/**
 *  @file testcsv.c
 *  @brief Test the CSV file implementation.
 *
 *  This file is in the public domain.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


static void fail (const char *msg)
{
	puts (msg);
	exit (1);
}

/** Write some CSV data to a temporary file and try to read it back again. */
int testcsv (void)
{
	const char *fields[] =
	{
		"1970", "", "ścięśliwy", NULL,
		"", NULL,
		",.-", "\"czeł\n\"owiek\"", NULL
	};
	const int n_fields = sizeof (fields) / sizeof (fields[0]);

	CsvWriter wrt;
	CsvReader rdr;
	FILE *fp;
	int i;
	char *field;

	/* NOTE: On Windows, this tries to create a file in the root. */
	fp = tmpfile ();
	if (!fp)
		fail ("tmpfile() failed");

	puts ("-- Writing CSV fields");
	wrt = csvWriterNew (fp);
	for (i = 0; i < n_fields; i++)
	{
		if (csvWrite (wrt, fields[i]))
			fail ("Write failed");
	}
	csvWriterDestroy (wrt);

	rewind (fp);

	puts ("-- Reading CSV fields back");
	rdr = csvReaderNew (fp);
	for (i = 0; i < n_fields; i++)
	{
		switch (csvRead (rdr, &field))
		{
		case CSV_FIELD:
			if (!fields[i] || strcmp (fields[i], field))
				fail ("Unexpected field or wrong value");
			free (field);
			break;
		case CSV_EOR:
			if (fields[i])
				fail ("Unexpected EOR");
			break;
		case CSV_EOF:
			fail ("Unexpected EOF");
			break;
		case CSV_ERROR:
			fail ("Read failed");
		}
	}
	csvReaderDestroy (rdr);
	fclose (fp);

	puts ("-- CSV test successful");
	return 0;
}

