/**
 *  @file testmemstream.c
 *  @brief Test MemStream.
 *
 *  This file is in the public domain.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


static void fail (const char *msg)
{
	puts (msg);
	exit (1);
}

int testmemstream (void)
{
	const char testStr[] = "Hello world";

	MemStream ms;
	int i;
	int8_t c;
	uint16_t u;

	ms = msCreate (0);

	puts ("-- Writing a string, reading it back");
	msWriteStr (ms, testStr);

	if (msGetSize (ms) != sizeof testStr)
		fail ("Wrong stream size");

	for (i = sizeof testStr; i--; )
	{
		msSetPos (ms, i, 0);
		msReadInt8 (ms, &c);

		if (msGetPos (ms) != i + 1)
			fail ("Wrong position");
		if (c != testStr[i])
			fail ("Wrong stream content [byte by byte]");
	}

	if (strcmp (msGetPtr (ms), testStr))
		fail ("Wrong stream content [strcmp]");

	puts ("-- Testing byte order");
	msSetPos (ms, 0, 0);
	msSetByteOrder (ms, MS_BYTE_ORDER_BE);
	msWriteUint16 (ms, 0x1234);

	msSetPos (ms, 0, 0);
	msSetByteOrder (ms, MS_BYTE_ORDER_LE);
	msReadUint16 (ms, &u);

	if (u != 0x3412)
		fail ("Wrong value");

	puts ("-- MemStream test successful");
	return 0;
}

