/**
 *  @file testsocks.c
 *  @brief Test SOCKS5.
 *
 *  This file is in the public domain.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


#define TEST_HOST "www.google.com"
#define TEST_URL "/pda"


static void fail (const char *msg)
{
	puts (msg);
	exit (1);
}

int testsocks (void)
{
	int sck, r;
	char buff[80];

	const char httpData[] =
		"GET " TEST_URL " HTTP/1.0\n"
		"Host: " TEST_HOST "\n"
		"\n";

	puts ("-- Initializing the library");
	socketsInit ();

	puts ("-- Trying to connect");
	if ((sck = socksTcpConnect ("127.0.0.1", SOCKS_PORT, TEST_HOST, 80)) == -1)
	{
		puts ("Please run 'ssh -N -D 127.0.0.1:1080 localhost' in the bg");
		puts ("and connect to the Internet before running this test.");
		fail ("Error: Cannot connect");
	}

	puts ("-- Sending data");
	if ((r = socketsSend (sck, httpData, sizeof httpData - 1)) == -1)
		fail ("Error: Sending failed.");

	puts ("-- Receiving data");
	do
		r = socketsRecv (sck, buff, sizeof buff);
	while (r > 0);

	if (r == -1)
		fail ("Error: Transmission failed.");

	puts ("-- Finished");
	return 0;
}

