/**
 *  @file ptrlist.c
 *  @brief A pointer list
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2007 - 2009. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#include <stdlib.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


struct PtrList
{
	/** An internal dynamic array. */
	DynArray da;
};


static void plFreeWrapper (const void *p, void *extra)
{
	((PL_FREE) extra) (*((void **) p));
}

PtrList plCreate (PL_FREE free)
{
	if (free)
		return (PtrList) daCreate (sizeof (void *), plFreeWrapper, free);
	else
		return (PtrList) daCreate (sizeof (void *), NULL, NULL);
}

void plFree (PtrList p)
{
	daFree ((DynArray) p);
}

void plClear (PtrList p)
{
	daClear ((DynArray) p);
}

int plGetSize (PtrList p)
{
	return daGetSize ((DynArray) p);
}

void *plGet (PtrList p, unsigned int index)
{
	return *((void **) daGet ((DynArray) p, index));
}

void plSet (PtrList p, unsigned int index, void *item)
{
	daSet ((DynArray) p, index, &item);
}

void plDelete (PtrList p, unsigned int index)
{
	daDelete ((DynArray) p, index);
}

void plInsert (PtrList p, unsigned int index, void *item)
{
	daInsert ((DynArray) p, index, &item);
}

int plAppend (PtrList p, void *item)
{
	return daAppend ((DynArray) p, &item);
}
