/**
 *  @file queue.h
 *  @brief Queue
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2007 - 2009. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#ifndef __XTND_QUEUE_H__
#define __XTND_QUEUE_H__

/**
 *  @defgroup queue Queue
 *  @ingroup ds
 *
 *  This module implements a FIFO, ie.\ a queue.
 *
 *  @{
 */

/** The queue class. */
typedef struct Queue *Queue;


/** Create a queue object.
 *  @param[in] itemsize  The size of items in the queue.
 *  @param[in] maxlen  Max. length of the queue in count of
 *                     items. Zero means unlimited.
 *  @return A new object.
 */
Queue quCreate (size_t itemsize, unsigned int maxlen);

/** Destroy a queue object.
 *  @param[in] qu  A queue object.
 */
void quFree (Queue qu);

/** Returns the length of a queue.
 *  @param[in] qu  A queue object.
 *  @return The length.
 */
unsigned int quGetLength (Queue qu);

/** Remove an item from the queue.
 *  @param[in] qu  A queue object.
 *  @param[out] buffer  A buffer for the item data.
 *  @return Zero on success, -1 on failure (underflow).
 */
int quRead (Queue qu, void *buffer);

/** Append an item to the queue.
 *  @param[in] qu  A queue object.
 *  @param[in] buffer  The item data.
 *  @return Zero on success, -1 on failure.
 */
int quWrite (Queue qu, const void *buffer);

/** @} */

#endif /* ! __XTND_QUEUE_H__ */

