/**
 *  @file getopt.c
 *  @brief Implementation of POSIX getopt (not only) for Windows
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2007 - 2011. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


char *xtnd_optarg;
int xtnd_optind = 1,
    xtnd_optopt = 0,
    xtnd_optpos = 0,
    xtnd_opterr = 1;

#define optarg xtnd_optarg
#define optind xtnd_optind
#define optopt xtnd_optopt
#define optpos xtnd_optpos
#define opterr xtnd_opterr

int xtnd_getopt (int argc, char *const *argv, const char *commands)
{
	char *c;
	int result = -1;

	if (!commands || !argv || !argv[optind])
		return -1;

	/* Next parameter -- check for end of options. */
	if (!optpos)
	{
		if (argv[optind][0] != '-' || !argv[optind][1])
			return -1;

		if (argv[optind][2] == '-' && !argv[optind][3])
		{
			optind++;
			return -1;
		}
	}

	if ((c = strchr (commands, argv[optind][1 + optpos]))
		&& isalnum (*c))
	{
		optarg = NULL;
		result = *c;

		/* Argument required. */
		if (c[1] == ':')
		{
			/* It is within the parameter. */
			if (*(argv[optind] + 2 + optpos))
				optarg = argv[optind] + 2 + optpos;
			/* It is the next parameter. */
			else if (argv[optind + 1])
				optarg = argv[++optind];
			/* It is missing. */
			else
			{
				optopt = *c;
				if (opterr)
					printf (_("Missing option argument: %c\n"), optopt);
				result = (*commands == ':' ? ':' : '?');
			}

			optind++;
			optpos = 0;
			return result;
		}
	}
	/* Unknown option. */
	else
	{
		optopt = argv[optind][1 + optpos];
		if (opterr)
			printf (_("Unknown option: %c\n"), optopt);
		result = '?';
	}

	/* Move to the next option character or parameter. */
	if (argv[optind][2 + optpos])
		optpos++;
	else
	{
		optind++;
		optpos = 0;
	}

	return result;
}

