/**
 *  @file getopt.h
 *  @brief Implementation of getopt()
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2007 - 2011. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#ifndef __XTND_GETOPT_H__
#define __XTND_GETOPT_H__

/**
 *  @defgroup getopt Implementation of getopt()
 *  @ingroup util
 *
 *  This module implements the standard POSIX getopt().
 *
 *  @{
 */

/** Points to the argument. */
extern char *xtnd_optarg;

/** Option character. */
extern int xtnd_optopt;

/** Index of the next argument to be processed. */
extern int xtnd_optind;

/** Set to zero if you don't want getopt to print messages. */
extern int xtnd_opterr;

/** Implementation of getopt().
 *  Gets command-line options from the @a argv array.
 *  @param[in] argc  The argument count value from @em main
 *  @param[in] argv  The argument string array from @em main
 *  @param[in] commands  A list of possible commands (see POSIX documentation).
 *  @return See POSIX documentation.
 */
int xtnd_getopt (int argc, char *const *argv, const char *commands);

/** @} */

#endif /* ! __XTND_GETOPT_H__ */

