/* Generated by re2c 0.13.5 on Fri Jun 15 04:39:11 2012 */
#line 1 "macroparser.c.re"
/**
 *  @file macroparser.c
 *  @brief Simple stack-based macro processor -- parser
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2007 - 2009. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <xtnd/xtnd.h>
#include <xtndprivate.h>


typedef struct ParseContext *ParseContext;

struct ParseContext
{
	int top, depthsize;
	struct Item **begin, **end;

	long num;

	size_t buffsize;
	char *buff, *buffptr;
};


/* Local functions. */
static void parseAppend (ParseContext ctx, enum ItemType type);
static void parseAppendItem (ParseContext ctx, struct Item *item);
static int parseFinishList (ParseContext ctx);

static void macroFreeItem (struct Item *item);
static void macroFreeList (struct Item *list);


/** Remove an item from memory.
 *  @param[in] item  The item
 *  @see macroFreeList()
 */
static void macroFreeItem (struct Item *item)
{
	if (!item)
		return;

	switch (item->type)
	{
		case MACRO_STRING:
		case MACRO_SUB:
			free (item->data.str);
			break;
		case MACRO_LIST:
			macroFreeList (item->data.list);
		default:
			break;
	}
	free (item);
}

/** Remove a whole list from memory.
 *  @param[in] list  The list
 *  @see macroFreeItem()
 */
static void macroFreeList (struct Item *list)
{
	struct Item *p;

	while (list)
	{
		p = list->next;
		macroFreeItem (list);
		list = p;
	}
}

void macroFree (Macro *macro)
{
	macroFreeList (*macro);
	*macro = NULL;
}

int macroParse (const char *str, Macro *macro)
{
	struct ParseContext ctx;
	struct Item *result = NULL;
	int error = MACRO_ERR_SUCCESS;
	enum {COND_BEGIN, COND_END, COND_STRING} condition;

	if (!macro)
		return MACRO_ERR_NULL_POINTER;

	ctx.num = 0;

	/* Allocate the buffer. */
	ctx.buffsize = 64;
	ctx.buffptr = ctx.buff = xmalloc (ctx.buffsize);

	/* Allocate the list stack. */
	ctx.depthsize = 8;
	ctx.begin = xmalloc (ctx.depthsize * sizeof (struct Item *));
	ctx.end = xmalloc (ctx.depthsize * sizeof (struct Item *));

	/* Initialize. */
	ctx.top = 0;
	ctx.begin[0] = NULL;
	ctx.end[0] = NULL;

	condition = COND_BEGIN;

	while (1)
	{
		const char *s;
		const char *marker;

		s = str;

#line 125 "macroparser.c"
	{
		unsigned char yych;
		switch (condition) {
		case COND_BEGIN: goto yyc_BEGIN;
		case COND_END: goto yyc_END;
		case COND_STRING: goto yyc_STRING;
		}
/* *********************************** */
yyc_BEGIN:

		yych = *str;
		switch (yych) {
		case 0x00:	goto yy15;
		case '\t':
		case '\n':
		case '\r':
		case ' ':	goto yy2;
		case '"':	goto yy13;
		case '*':
		case '+':
		case '.':
		case '/':
		case '=':
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
		case 'H':
		case 'I':
		case 'J':
		case 'K':
		case 'L':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'Q':
		case 'R':
		case 'S':
		case 'T':
		case 'U':
		case 'V':
		case 'W':
		case 'X':
		case 'Y':
		case 'Z':
		case 'a':
		case 'b':
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
		case 'h':
		case 'i':
		case 'j':
		case 'k':
		case 'l':
		case 'm':
		case 'n':
		case 'o':
		case 'p':
		case 'q':
		case 'r':
		case 's':
		case 't':
		case 'u':
		case 'v':
		case 'w':
		case 'x':
		case 'y':
		case 'z':	goto yy12;
		case '-':	goto yy8;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':	goto yy10;
		case '[':	goto yy4;
		case ']':	goto yy6;
		default:	goto yy17;
		}
yy2:
		++str;
		yych = *str;
		goto yy26;
yy3:
#line 133 "macroparser.c.re"
		{
			continue;
		}
#line 225 "macroparser.c"
yy4:
		++str;
#line 138 "macroparser.c.re"
		{
			if (ctx.top + 1 >= ctx.depthsize)
			{
				error = MACRO_ERR_TOO_NESTED;
				break;
			}

			ctx.top++;
			ctx.begin[ctx.top] = NULL;
			ctx.end[ctx.top] = NULL;
			continue;
		}
#line 241 "macroparser.c"
yy6:
		++str;
		condition = (COND_END);
#line 152 "macroparser.c.re"
		{
			if (parseFinishList (&ctx))
			{
				error = MACRO_ERR_NOT_IN_LIST;
				break;
			}
			continue;
		}
#line 254 "macroparser.c"
yy8:
		++str;
		switch ((yych = *str)) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':	goto yy23;
		default:	goto yy20;
		}
yy9:
		condition = (COND_END);
#line 169 "macroparser.c.re"
		{
			strncpy (ctx.buff, s, str - s);
			ctx.buff[str - s] = '\0';
			parseAppend (&ctx, MACRO_SUB);
			continue;
		}
#line 279 "macroparser.c"
yy10:
		++str;
		yych = *str;
		goto yy22;
yy11:
		condition = (COND_END);
#line 162 "macroparser.c.re"
		{
			ctx.num = strtol (s, NULL, 10);
			parseAppend (&ctx, MACRO_INT);
			continue;
		}
#line 292 "macroparser.c"
yy12:
		yych = *++str;
		goto yy20;
yy13:
		++str;
		condition = (COND_STRING);
#line 177 "macroparser.c.re"
		{
			ctx.buffptr = ctx.buff;
			continue;
		}
#line 304 "macroparser.c"
yy15:
		++str;
#line 223 "macroparser.c.re"
		{
			if (ctx.top)
				error = MACRO_ERR_LIST_NOT_CLOSED;
			break;
		}
#line 313 "macroparser.c"
yy17:
		++str;
#line 230 "macroparser.c.re"
		{
			error = MACRO_ERR_BAD_CHAR;
			break;
		}
#line 321 "macroparser.c"
yy19:
		++str;
		yych = *str;
yy20:
		switch (yych) {
		case '*':
		case '+':
		case '-':
		case '.':
		case '/':
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		case '=':
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
		case 'H':
		case 'I':
		case 'J':
		case 'K':
		case 'L':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'Q':
		case 'R':
		case 'S':
		case 'T':
		case 'U':
		case 'V':
		case 'W':
		case 'X':
		case 'Y':
		case 'Z':
		case 'a':
		case 'b':
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
		case 'h':
		case 'i':
		case 'j':
		case 'k':
		case 'l':
		case 'm':
		case 'n':
		case 'o':
		case 'p':
		case 'q':
		case 'r':
		case 's':
		case 't':
		case 'u':
		case 'v':
		case 'w':
		case 'x':
		case 'y':
		case 'z':	goto yy19;
		default:	goto yy9;
		}
yy21:
		++str;
		yych = *str;
yy22:
		switch (yych) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':	goto yy21;
		default:	goto yy11;
		}
yy23:
		++str;
		yych = *str;
		switch (yych) {
		case '*':
		case '+':
		case '-':
		case '.':
		case '/':
		case '=':
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
		case 'H':
		case 'I':
		case 'J':
		case 'K':
		case 'L':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'Q':
		case 'R':
		case 'S':
		case 'T':
		case 'U':
		case 'V':
		case 'W':
		case 'X':
		case 'Y':
		case 'Z':
		case 'a':
		case 'b':
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
		case 'h':
		case 'i':
		case 'j':
		case 'k':
		case 'l':
		case 'm':
		case 'n':
		case 'o':
		case 'p':
		case 'q':
		case 'r':
		case 's':
		case 't':
		case 'u':
		case 'v':
		case 'w':
		case 'x':
		case 'y':
		case 'z':	goto yy19;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':	goto yy23;
		default:	goto yy11;
		}
yy25:
		++str;
		yych = *str;
yy26:
		switch (yych) {
		case '\t':
		case '\n':
		case '\r':
		case ' ':	goto yy25;
		default:	goto yy3;
		}
/* *********************************** */
yyc_END:
		yych = *str;
		switch (yych) {
		case 0x00:	goto yy33;
		case '\t':
		case '\n':
		case '\r':
		case ' ':	goto yy29;
		case ']':	goto yy31;
		default:	goto yy35;
		}
yy29:
		++str;
		yych = *str;
		goto yy38;
yy30:
		condition = (COND_BEGIN);
#line 133 "macroparser.c.re"
		{
			continue;
		}
#line 521 "macroparser.c"
yy31:
		++str;
#line 152 "macroparser.c.re"
		{
			if (parseFinishList (&ctx))
			{
				error = MACRO_ERR_NOT_IN_LIST;
				break;
			}
			continue;
		}
#line 533 "macroparser.c"
yy33:
		++str;
#line 223 "macroparser.c.re"
		{
			if (ctx.top)
				error = MACRO_ERR_LIST_NOT_CLOSED;
			break;
		}
#line 542 "macroparser.c"
yy35:
		++str;
#line 230 "macroparser.c.re"
		{
			error = MACRO_ERR_BAD_CHAR;
			break;
		}
#line 550 "macroparser.c"
yy37:
		++str;
		yych = *str;
yy38:
		switch (yych) {
		case '\t':
		case '\n':
		case '\r':
		case ' ':	goto yy37;
		default:	goto yy30;
		}
/* *********************************** */
yyc_STRING:
		yych = *str;
		switch (yych) {
		case 0x00:	goto yy45;
		case '"':	goto yy43;
		case '\\':	goto yy41;
		default:	goto yy47;
		}
yy41:
		++str;
		if ((yych = *str) >= 0x01) goto yy48;
yy42:
#line 214 "macroparser.c.re"
		{
			if (ctx.buffsize <= ctx.buffptr - ctx.buff + 1)
				ctx.buff = xrealloc (ctx.buff, ctx.buffsize += 64);

			*ctx.buffptr++ = *s;
			continue;
		}
#line 583 "macroparser.c"
yy43:
		++str;
		condition = (COND_END);
#line 201 "macroparser.c.re"
		{
			*ctx.buffptr = '\0';
			parseAppend (&ctx, MACRO_STRING);
			continue;
		}
#line 593 "macroparser.c"
yy45:
		++str;
#line 208 "macroparser.c.re"
		{
			error = MACRO_ERR_STRING_NO_END;
			break;
		}
#line 601 "macroparser.c"
yy47:
		yych = *++str;
		goto yy42;
yy48:
		++str;
#line 183 "macroparser.c.re"
		{
			if (ctx.buffsize <= ctx.buffptr - ctx.buff + 1)
				ctx.buff = xrealloc (ctx.buff, ctx.buffsize += 64);

			switch (*(s + 1))
			{
				case 'r': *ctx.buffptr++ = '\r'; break;
				case 'n': *ctx.buffptr++ = '\n'; break;
				case 't': *ctx.buffptr++ = '\t'; break;
				case 'a': *ctx.buffptr++ = '\a'; break;
				case 'b': *ctx.buffptr++ = '\b'; break;
				case 'f': *ctx.buffptr++ = '\f'; break;
				default:  *ctx.buffptr++ = *(s + 1);
			}
			continue;
		}
#line 624 "macroparser.c"
	}
#line 234 "macroparser.c.re"

	}

	if (error)
	{
		if (ctx.top)
			macroFreeList (ctx.begin[ctx.top]);

		/* It is in a consistent state, so we can free it normally. */
		macroFreeList (ctx.begin[0]);
	}
	else
		result = ctx.begin[0];

	free (ctx.buff);
	free (ctx.begin);
	free (ctx.end);

	*macro = result;

	/* MACRO_ERR_SUCCESS is the default error */
	return error;
}


/* Append the data. */
static void parseAppend (ParseContext ctx, enum ItemType type)
{
	struct Item *item;

	/* Make a new item. */
	item = xmalloc (sizeof (struct Item));
	item->next = NULL;
	item->type = type;

	switch (type)
	{
		case MACRO_INT:
			item->data.num = ctx->num;
			break;
		case MACRO_STRING:
		case MACRO_SUB:
			item->data.str = xstrdup (ctx->buff);
			break;
		default:
			/* Should not happen. */
			free (item);
			return;
	}

	parseAppendItem(ctx, item);
}

/* Append an item. */
static void parseAppendItem (ParseContext ctx, struct Item *item)
{
	if (ctx->end[ctx->top])
		ctx->end[ctx->top]->next = item;
	ctx->end[ctx->top] = item;
	if (!ctx->begin[ctx->top])
		ctx->begin[ctx->top] = item;
}


/* Finish processing a list. */
static int parseFinishList (ParseContext ctx)
{
	struct Item *item;

	/* No list to finish. */
	if (!ctx->top)
		return 1;

	/* Append a new item. */
	item = xmalloc (sizeof (struct Item));
	item->next = NULL;
	item->type = MACRO_LIST;
	item->data.list = ctx->begin[ctx->top--];
	parseAppendItem (ctx, item);

	return 0;
}


