/**
 *  @file socks.h
 *  @brief SOCKS5 support
 *
 *  @note This file is a part of libxtnd.
 *        Copyright Premysl Janouch 2012. All rights reserved.
 *  @note See the file LICENSE for licensing information.
 *
 */

#ifndef __XTND_SOCKS_H__
#define __XTND_SOCKS_H__

/**
 *  @defgroup socks SOCKS5 support
 *  @ingroup net
 *
 *  This module simply allows to connect to a server via a SOCKS 5 proxy.
 *
 *  @{
 */

/** The default TCP port for the SOCKS protocol. */
#define SOCKS_PORT 1080

/** Initiate a TCP connection via a SOCKS 5 proxy.
 *  @param[in] sockshost  Proxy hostname or address.
 *  @param[in] socksport  Proxy port (use SOCKS_PORT for the default).
 *  @param[in] targethost  Target hostname or address.
 *  @param[in] targetport  Target port.
 *  @return -1 on error, a socket otherwise.
 */
int socksTcpConnect (const char *sockshost, int socksport,
	const char *targethost, int targetport);

/** @} */

#endif /* ! __XTND_SOCKS_H__ */

