/*  Copyright 2010 Curtis McEnroe <programble@gmail.com>
 *
 *  This file is part of AmusOS.
 *
 *  AmusOS is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  AmusOS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with AmusOS.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MM_H__
#define __MM_H__

#include <syscall.h>

/**
 * Allocate a block of size bytes in heap memory.
 * @param size Number of bytes
 * @return The allocated memory block
 */
void *malloc(int size);

/**
 * Free a block of memory allocated with malloc.
 * @param block The block of allocated memory
 */
void free(void *block);

/**
 * Reallocate a block of memory allocated with malloc.
 * Reallocate a block of memory allocated with malloc to a new size
 * and copy the old data to the new block.
 * @param old The old block of memory
 * @param size The new size
 * @return The new block
 */
void *realloc(void *old, int size);

#endif
