#include "integ_routines.h"
#include "xv2el.h"
#define N_BINS 3000
double period = TWOPI;
void initialize_conditions(double y[], double alpha, double vel_mult){
    int ntp = NVAR/4;
    int i;
    double span = .1;
    for ( i =0; i < ntp; i++)
	{
		double alphai =  alpha + i* (span/(ntp));
		double vkep = 1.0 * pow(alphai, -0.5);
		int j = i*4;
		y[j] = alphai;
		y[j+1] = 0.0;
		y[j+2] = 0.0;
		y[j+3] = vel_mult * vkep;
		
    }
	
	
}
void add_bin_r_theta(double r, double theta, int counts[], double bin_avgs[],double bin_er2[], double bin_var[],double deltheta)
{
	int bin_num = (int) floor((theta+M_PI)/deltheta);
	bin_avgs[bin_num] = bin_avgs[bin_num]*counts[bin_num];
	bin_er2[bin_num] =  bin_er2[bin_num]*counts[bin_num];
	counts[bin_num] += 1;
	bin_avgs[bin_num] += r;
	bin_er2[bin_num] += r*r;
	bin_avgs[bin_num] /= (double) counts[bin_num];
	bin_er2[bin_num] /= (double) counts[bin_num];
	bin_var[bin_num] = bin_er2[bin_num] - bin_avgs[bin_num]*bin_avgs[bin_num];
}
double rms_sum(double vals[])
{
	int fin = sizeof(vals)/sizeof(double);
	double sum=0.0;
	int i;
	for(i=0; i<fin; i++)
	{
		sum += vals[i];
	}
	return sqrt(sum);
}
void write_starxvdat(double t, FILE * starA, FILE * starB, double const star1[], double const star2[])
{
	fprintf(starA, "%-.8E  %-.8E  %-.8E  %-.8E  %-.8E\n",t/period, star1[0], star1[1], star1[2],star1[3]);
	fprintf(starB, "%-.8E  %-.8E  %-.8E  %-.8E  %-.8E\n",t/period, star2[0], star2[1], star2[2],star2[3]);
}
void set_target_a(double * sma,double const y[], int ntp){
	int i, j;
	for (i=0;i<ntp;i++){
		j = 4*i;
		sma[i]=y[j]-.002;
		printf("id: %d taget: %g\n",i,sma[i]);
	}
}
double * run_integration(double y[],double target_a[],double q,double ecc, int ntp,double eps,double * t0, double  tfin, int close_encounters_flag, int * active)
{
	if(close_encounters_flag <=0 ){
		tpxv=fopen("tp.xv","w");  star1=fopen("star1.xv","w");star2=fopen("star2.xv","w");
	}else{
		tpxv=fopen("tp.xv","a");  star1=fopen("star1.xv","a");star2=fopen("star2.xv","a");
		relv=fopen("relv.dat","w");
	}
	if ((tpxv==NULL) || (star1==NULL) ||(star2==NULL)) {
		printf("File handling error in run_integration!\n");
		exit(-1);
	}
	double del_theta = TWOPI/N_BINS; // Binning variables
#if 0
	double rms_bin_variance = 0;
	int theta_bin_count[N_BINS] = { 0 };
	double theta_bin_var[N_BINS] = { 0 } , theta_bin_avg[N_BINS] = { 0 }, theta_bin_er2[N_BINS] = { 0 };
#endif
	double star1_xv[4], star2_xv[4];
	if(close_encounters_flag<=0)
		set_target_a(target_a,y,ntp); //Only set for damping run, otherwise leave the same
	parameters params = {q,ecc,star1_xv, star2_xv,target_a,active}; // parameter to pass to dervis routine
	int tint = 0, lfirst=1; // Time/loop control variales
	double t,tnext,e0;
	double tfinal = tfin * period;
	double dt = eps * period;
	int tintfinal =  1 + (int) ceil((tfinal-*t0) / dt);
	gsl_odeiv2_system sys = {derivs, 0, NVAR, &params}; // GSL integrator variables 
	double hh = 1.0e-8;
	gsl_odeiv2_driver * d = gsl_odeiv2_driver_alloc_y_new(&sys,gsl_odeiv2_step_rk8pd, hh,1.0e-9,0.0);
	double start_t=*t0;	
	for (tint = 0; tint < tintfinal; tint++)
	{
		t = tint * dt + start_t;
		tnext = (tint + 1) * dt + start_t;
		printf("Time: %g\n", t/period);
		int id;
		//if(tint%10==0 && t > 1.99e4 * period){
		for (id=0; id<ntp; id++) {
			if(active[id] > 0)
				check_eject(id,y,active);
				write_tpxv(tpxv,t, id,y,q,star1_xv,star2_xv);
		}//}
		if(close_encounters_flag > 0)
			rel_vel(y,ntp,active,relv);
		write_starxvdat(t,star1,star2,star1_xv,star2_xv);
		int status = gsl_odeiv2_driver_apply (d, &t,tnext , y);
		if (status != GSL_SUCCESS) exit(-1);
	}
	int tpid;
	for (tpid=0; tpid<ntp; tpid++){
		if(active[tpid] <= 0)
			printf("Particle %d was ejected\n",tpid );
	}
	fclose(star1);
	fclose(star2);
	fclose(tpxv);
	if(close_encounters_flag>0)
		fclose(relv);
	gsl_odeiv2_driver_free(d);
//	rms_bin_variance = rms_sum(theta_bin_var);
	*t0 = tnext;
	return y;
}
void check_eject(int id, double y[], int * active){
	int j = id*4;
	double xtp = y[j]; double ytp = y[j+1];
	double r = sqrt(xtp*xtp + ytp*ytp);
	if(r> EJECT_RADIUS){ // EJECT_RADIUS set in integ_routines.h
		printf("Particle %d ejected!\n",id);
		active[id] = -1;
		y[j]=y[j+1]=y[j+2]=y[j+3]=0;
	}
}

void print_status(double t, double const y[], double q,double * e0, int * lfirst)
{
	double mu1 = 1.0/(1.0+q);
	double mu2 = 1 - mu1;
	double r12 = sqrt ( (y[0]-y[4])*(y[0]-y[4]) + (y[1]-y[5])*(y[1]-y[5]) );
	double energy = .5 * mu1 *( y[2]*y[2] + y[3]*y[3]) + .5 * mu2 *( y[6]*y[6] + y[7]*y[7]) - q * mu1 * mu1 / r12;
	if (*lfirst == 1)
	{
		*e0 = energy;
		*lfirst = 0;
	}
	double drift = sqrt( pow(mu1 * y[0] + mu2*y[4],2.0) + pow(mu1 * y[1] + mu2*y[5],2.0) );
	printf("T(periods)= %.8g   COM drift=  %.6g    dE/E = %.5g", t/period, drift,(energy-(*e0))/(*e0)) ; 
	printf("\n") ; 
}
void write_tpxv(FILE * xv, double t, int id, double const y[],double q, double const star1_xv[],double const star2_xv[])
{
	int j = id * 4;
	double mu1 = 1.0/(1.0 + q); 
	double mu2 = 1.0-mu1;
	double jconst, rho1, rho2,smatp,ecctp;
	double xx,yy,vx,vy,r,h,evecx, evecy;
	xx=y[j]; yy=y[j+1]; vx=y[j+2]; vy=y[j+3];
	smatp = get_sma(y[j], y[j+1], y[j+2],y[j+3]);
	ecctp = get_ecc(y[j], y[j+1], y[j+2],y[j+3]);
	if(isnan(smatp))
		smatp = 0.0;
	if(isnan(ecctp))
		ecctp = 0.0;
	// See Szebehely pg. 1j+1 for the following
#if 1
	rho1 = sqrt((y[j] - star1_xv[0])*(y[j] - star1_xv[0]) + (y[j+1] - star1_xv[1])*(y[j+1] - star1_xv[1]));
	rho2 = sqrt((y[j] - star2_xv[0])*(y[j] - star2_xv[0]) + (y[j+1] - star2_xv[1])*(y[j+1] - star2_xv[1]));
	jconst = 2.0 * (y[j]*y[j+3] - y[j+1]*y[j+2]) + 2.0 * (mu1/rho1 + mu2/rho2) - (y[j+2]*y[j+2] + y[j+3]*y[j+3]);
	
#endif
	h= xx*vy-yy*vx;
       	r = sqrt(xx*xx+yy*yy);
	evecx = vy*h - xx/r;
	evecy = -vx*h - yy/r;	
	if(isnan(evecx) || isnan(evecy))
		{evecx=0; evecy=0;} 
	fprintf(xv, "%-.8E %3d  %-.8E  %-.8E  %-.8E  %-.8E  %-.8E  %-.8E %-.8E %-.8E %-.8E\n",t/period,id, y[j], y[j+1], y[j+2],y[j+3],smatp,ecctp,evecx,evecy,jconst);
}
void get_rtheta(int id,double const y[], double * r,double * theta)
{
	int j= id*4;
	*r = sqrt(y[j]*y[j]+y[j+1]*y[j+1]);
	*theta = atan(y[j+1]/y[j]);
	if ((y[j+1] < 0) &&(y[j] < 0)) *theta = *theta - M_PI; //QIII -- x&y<0
	if ((y[j+1]>0) &&(y[j]<0)) *theta= *theta + M_PI;   //QII -- x<0 & y>0
}	
void write_tprtheta(FILE * tprt, double t, int id, double r, double theta)
{
	fprintf(tprt, "%-.8E %3d  %-.8E  %-.8E  \n",t/period,id, r, theta);
}
void rel_vel(double const y[], int ntp,int * active, FILE * writefile){
	int i, j;
	for (i=0; i<ntp-1; i++){
		if(active[i]<=0)
			continue;
		int idex = 4*i;
		for(j=i+1; j<ntp; j++){
			if ( active[j] <=0 )
				continue;
			int jdex = 4*j;
			double dx = y[idex]-y[jdex];
			double dy = y[idex+1]-y[jdex+1];
			double dvx = y[idex+2]-y[jdex+2];
			double dvy = y[idex+3]-y[jdex+3];
			double dist = sqrt(dx*dx+dy*dy);
			double delv = sqrt(dvx*dvx + dvy*dvy);	
			if (dist < D_WRITE_MAX)
				fprintf(writefile, "%g \t %g \t %d \t %d\n",dist, delv,i,j);
		}
	}
}
