#include <avr/interrupt.h> 
#include "as5046.h"

#define CYCLES_PER_TICK 20000

/* Set motor speed of motor B from -255 to 255 */
/* (Using PORTC pin 2-3 for direction and OCR3B for PWM/speed) */
void setMotor(uint8_t servo, uint8_t forward, uint8_t reverse) {  
  if (servo == 1) {
    OCR2B = forward;
    OCR2A = reverse;
  } else {
    OCR0B = forward;
    OCR0A = reverse;
  }
}

void brakeMotor(uint8_t servo) {    
  setMotor(servo, 255, 255);
}

SIGNAL(SIG_OUTPUT_COMPARE1A) {   
  // Perpetually updated shit goes here
}

void motor_init() {
  TCCR1A = 0;	            
  TCCR1B = 0;

	// waveform generation = 0100 = CTC
	TCCR1B |=  (1<<WGM12);

  TCCR1B |= 1<<CS10; // No prescler
  
  TIMSK1 = _BV(OCIE1A); // enable oveflow interrupt
  OCR1A = CYCLES_PER_TICK; // Set maximum value
  TCNT1 = 0;      

  /* MOTOR DRIVER SETUP */
  // These settings in accordance with Baby Orangutan documentation
  // here: http://www.pololu.com/docs/0J14/9
  DDRD |= (1<<3)|(1<<5)|(1<<6); // PD3, PD5, PD6 outputs for motor pwm
  DDRB |= (1<<3); // PB3 output for motor pwm
  TCCR0A = (7<<WGM00)|(2<<COM0A0)|(2<<COM0B0); // Fast PWM, inverted
  TCCR0B = (2 << CS00);
  TCCR2A = (7<<WGM20)|(2<<COM2A0)|(2<<COM2B0); // Fast PWM, inverted
  TCCR2B = (2 << CS20); // Prescaler = 8
  setMotor(0,0,0);
  setMotor(1,0,0);
}
