// A module for managing the actual real boom

#include <avr/io.h> 
#include <avr/interrupt.h> 
#include "boom.h"
#include "config.h"
#include "as5046.h"
#include "pid.h"
                    
// The raw, actual rotation for each joint (0-4095)
int16_t rotation[3];
// The target rotation for each joint (0-4095)
int16_t target[3];                           
// Pid state records for each joint
struct PID_DATA pid[3];

// This interrupt called perpetually according to the config variable CYCLES_PER_TICK
SIGNAL(TIMER1_COMPA_vect) {         
  as_prepare_encoder(0);
  // NOTE: _delay_us(400);
  // There must be at least 310us between as_prepare_encoder and the corresponding as_poll_encoder.
  // 310us == 4960 machine cycles which makes it a good time to perform some horror show
  // floating point maths.
                            
  rotation[0] = as_poll_encoder(0);
  as_prepare_encoder(1);
  rotation[1] = as_poll_encoder(1);
  as_prepare_encoder(2);
  rotation[1] = as_poll_encoder(2);
}

void boom_init() {
  TCCR1A = 0;	            
  TCCR1B = 0;

	// waveform generation = 0100 = CTC
	TCCR1B |=  (1<<WGM12);

  TCCR1B |= 1<<CS10; // No prescler
  
  TIMSK1 = _BV(OCIE1A); // enable oveflow interrupt
  OCR1A = CYCLES_PER_TICK; // Set maximum value
  TCNT1 = 0;
  
  // Initialize the pid controllers
  pid_Init(0.1, 0.0, 0.0, &pid[0]);
  pid_Init(0.1, 0.0, 0.0, &pid[1]);
  pid_Init(0.1, 0.0, 0.0, &pid[2]);
}