// Methods for controlling the power to the motors

#include <avr/io.h>
#include <stdlib.h>
#include "motor.h"
#include "config.h"
/* Set motor speed of motor B from -255 to 255 */
/* (Using PORTC pin 2-3 for direction and OCR3B for PWM/speed) */
void setMotor(uint8_t index, int16_t velocity) {  
  uint8_t reverse = (velocity >> 8) & 1;
  uint8_t speed = abs(velocity);
  switch (index) {
    case 0:
    OCR0A = speed;
    MOTOR_0_DIR_PORT = (MOTOR_0_DIR_PORT & ~(1<<MOTOR_0_DIR_PIN)) | (reverse<<MOTOR_0_DIR_PIN);
    break;
    case 1:
    OCR0B = speed;
    MOTOR_1_DIR_PORT = (MOTOR_1_DIR_PORT & ~(1<<MOTOR_1_DIR_PIN)) | (reverse<<MOTOR_1_DIR_PIN);
    break;
    case 2:
    OCR2A = speed;
    MOTOR_2_DIR_PORT = (MOTOR_2_DIR_PORT & ~(2<<MOTOR_2_DIR_PIN)) | (reverse<<MOTOR_2_DIR_PIN);
    break;
    case 3:
    OCR2B = speed;
    MOTOR_3_DIR_PORT = (MOTOR_3_DIR_PORT & ~(3<<MOTOR_3_DIR_PIN)) | (reverse<<MOTOR_3_DIR_PIN);
    break;
  }
}

void motor_init() {
  /* MOTOR DRIVER SETUP */
  // These settings in accordance with Baby Orangutan documentation
  // here: http://www.pololu.com/docs/0J14/9
  DDRD |= (1<<3)|(1<<5)|(1<<6); // PD3, PD5, PD6 outputs for motor pwm
  DDRB |= (1<<3); // PB3 output for motor pwm
  
  MOTOR_0_DIR_DDR |= (1<<MOTOR_0_DIR_PIN);
  MOTOR_1_DIR_DDR |= (1<<MOTOR_1_DIR_PIN);
  MOTOR_2_DIR_DDR |= (1<<MOTOR_2_DIR_PIN);
  MOTOR_3_DIR_DDR |= (1<<MOTOR_3_DIR_PIN);
  
  TCCR0A = (7<<WGM00)|(2<<COM0A0)|(2<<COM0B0); // Fast PWM, inverted
  TCCR0B = (2 << CS00);
  TCCR2A = (7<<WGM20)|(2<<COM2A0)|(2<<COM2B0); // Fast PWM, inverted
  TCCR2B = (2 << CS20); // Prescaler = 8
  setMotor(0,0);
  setMotor(1,0);
}
