/*
 * See header file for details
 *
 *  This program is free software: you can redistribute it and/or modify\n
 *  it under the terms of the GNU General Public License as published by\n
 *  the Free Software Foundation, either version 3 of the License, or\n
 *  (at your option) any later version.\n
 * 
 *  This program is distributed in the hope that it will be useful,\n
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n
 *  GNU General Public License for more details.\n
 * 
 *  You should have received a copy of the GNU General Public License\n
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.\n
 */
 
/* Includes */
#include <stdint.h>       // For hardcoded type
#include "common.h"       // For common macro
#include "port.h"         // For platform dependent macro
#include "tracker.h"      // For tracker commands
#include "tracker_data.h" // For english note notation
#include "subtimer.h"     // For SubTimer structure
#include "envelope.h"     // For ADSR envelope name
#include "oscillator.h"   // For Waveform name
#include "mixer.h"        // For channel name

/** 
 * Tracker music file data
 *
 * @note This example tracker data file is the openning theme of "Angel Beats".
 */
const uint8_t tracker_music_opcodes[] PROGMEM = {
	/* DO = C, RE = D, MI = E, FA = F, SOL = G, LA = A, SI = B */

		// Angel beats (intro)
		DIRECT_EXEC, 21,
		SET_TEMPO, value(750),
		SET_GLOBAL_VOLUME, 255,
		SET_WAVE | 0, WF_SINUS, SET_VOLUME | 0, 255,
		SET_WAVE | 1, WF_SINUS, SET_VOLUME | 1, 255,
		SET_WAVE | 2, WF_SINUS, SET_VOLUME | 2, 255,
		SET_WAVE | 3, WF_SINUS, SET_VOLUME | 3, 255,
		SET_WAVE | 4, WF_SINUS, SET_VOLUME | 4, 255,
		SET_WAVE | 5, WF_SINUS, SET_VOLUME | 5, 255,
		SET_ADSR_VALUES | ADSR_USER_1, value(50), value(20), 127, value(30),
		SET_ADSR | 0, ADSR_USER_1, //ADSR_NONE,
		SET_ADSR | 1, ADSR_USER_1, //ADSR_NONE,
		SET_ADSR | 2, ADSR_USER_1, //ADSR_NONE,
		SET_ADSR | 3, ADSR_USER_1, //ADSR_NONE,
		SET_ADSR | 4, ADSR_USER_1, //ADSR_NONE,
		SET_ADSR | 5, ADSR_USER_1, //ADSR_NONE,


		NOTE_ON | 0, NOTE_FS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NOTE_ON | 1, NOTE_FS3,	NOTE_ON | 2, NOTE_A5,	NOTE_ON | 3, NOTE_D4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NOTE_ON | 1, NOTE_FS3,	NOTE_ON | 2, NOTE_A5,	NOTE_ON | 3, NOTE_D4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A6,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B7,	NOTE_ON | 1, NOTE_B6,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A5,	NOTE_ON | 1, NOTE_FS4,	NOTE_ON | 2, NOTE_FS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D5,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_B4,	NOTE_ON | 3, NOTE_A4,	NOTE_ON | 4, NOTE_D4,	NOTE_ON | 5, NOTE_B3,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS6,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NOTE_ON | 5, NOTE_B5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS6,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NOTE_ON | 5, NOTE_B5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS6,	NOTE_ON | 2, NOTE_G4,	NOTE_ON | 3, NOTE_D5,	NOTE_ON | 4, NOTE_G5,	NOTE_ON | 5, NOTE_B5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS6,	NOTE_ON | 2, NOTE_B5,	NOTE_ON | 3, NOTE_G4,	NOTE_ON | 4, NOTE_D5,	NOTE_ON | 5, NOTE_G5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS6,	NOTE_ON | 2, NOTE_B5,	NOTE_ON | 3, NOTE_G4,	NOTE_ON | 4, NOTE_D5,	NOTE_ON | 5, NOTE_G5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A5,	NOTE_ON | 3, NOTE_A4,	NOTE_ON | 4, NOTE_D5,	NOTE_ON | 5, NOTE_G5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NOTE_ON | 0, NOTE_A6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_FS4,	NOTE_ON | 3, NOTE_B4,	NOTE_ON | 4, NOTE_E5,	NOTE_ON | 5, NOTE_FS5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E7,	NOTE_ON | 1, NOTE_A6,	NOTE_ON | 2, NOTE_D5,	NOTE_ON | 3, NOTE_A5,	NOTE_ON | 4, NOTE_B5,	NOTE_ON | 5, NOTE_D6,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A7,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G3,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D4,	NOTE_ON | 1, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G4,	NOTE_ON | 1, NOTE_D6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_G5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS5,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A5,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_G5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NOTE_ON | 1, NOTE_E3,	NOTE_ON | 2, NOTE_G5,	NOTE_ON | 3, NOTE_B5,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_E5,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_FS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A5,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS5,	NOTE_ON | 1, NOTE_FS4,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS5,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G3,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D4,	NOTE_ON | 1, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G4,	NOTE_ON | 1, NOTE_D6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NOTE_ON | 1, NOTE_D6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D5,	NOTE_ON | 1, NOTE_CS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G5,	NOTE_ON | 1, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D5,	NOTE_ON | 1, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A3,	NOTE_ON | 1, NOTE_D6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS5,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A5,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_G5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NOTE_ON | 1, NOTE_E3,	NOTE_ON | 2, NOTE_G5,	NOTE_ON | 3, NOTE_B5,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_E5,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS6,	NOTE_ON | 1, NOTE_FS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_AS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_AS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_D3,	NOTE_ON | 2, NOTE_E6,	NOTE_ON | 3, NOTE_D6,	NOTE_ON | 4, NOTE_A5,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A6,	NOTE_ON | 1, NOTE_B5,	NOTE_ON | 2, NOTE_G3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_D4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_G6,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_G4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_GS3,	NOTE_ON | 2, NOTE_GS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_GS4,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_AS3,	NOTE_ON | 2, NOTE_AS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_D6,	NOTE_ON | 1, NOTE_FS4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_AS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B5,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_AS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B3,	NOTE_ON | 2, NOTE_D6,	NOTE_ON | 3, NOTE_B5,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_D5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_DS7,	NOTE_ON | 1, NOTE_DS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E7,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS7,	NOTE_ON | 1, NOTE_DS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS7,	NOTE_ON | 1, NOTE_FS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_A2,	NOTE_ON | 2, NOTE_CS6,	NOTE_ON | 3, NOTE_A5,	NOTE_ON | 4, NOTE_A3,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_E3,	NOTE_ON | 2, NOTE_A5,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A3,	NOTE_ON | 2, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_CS6,	NOTE_ON | 2, NOTE_A2,	NOTE_ON | 3, NOTE_A5,	NOTE_ON | 4, NOTE_A3,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_E3,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS7,	NOTE_ON | 1, NOTE_E4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_CS4,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS3,	NO_ACTION,	NOTE_ON | 3, NOTE_CS4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_GS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS3,	NO_ACTION,	NOTE_ON | 3, NOTE_CS4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_GS3,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS4,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NOTE_ON | 1, NOTE_GS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS4,	NOTE_ON | 1, NOTE_FS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NOTE_ON | 1, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_B5,	NOTE_ON | 3, NOTE_B2,	NOTE_ON | 4, NOTE_B3,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_FS3,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B2,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_B4,	NOTE_ON | 2, NOTE_B5,	NOTE_ON | 3, NOTE_B3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B3,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_A2,	NOTE_ON | 2, NOTE_CS6,	NOTE_ON | 3, NOTE_A5,	NOTE_ON | 4, NOTE_A3,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E3,	NO_ACTION,	NO_ACTION,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A3,	NOTE_ON | 2, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_CS4,	NOTE_ON | 2, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E4,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_A3,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A2,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E4,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_A4,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_A2,	NOTE_ON | 2, NOTE_CS6,	NOTE_ON | 3, NOTE_A5,	NOTE_ON | 4, NOTE_A3,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_E3,	NOTE_ON | 2, NOTE_A5,	NOTE_OFF | 3,	NOTE_OFF | 4,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_A3,	NOTE_ON | 2, NOTE_A5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E6,	NOTE_ON | 1, NOTE_A5,	NOTE_ON | 2, NOTE_CS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_ON | 2, NOTE_A4,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS7,	NOTE_ON | 1, NOTE_E4,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_CS4,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B2,	NO_ACTION,	NOTE_ON | 3, NOTE_B3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS3,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B3,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_DS7,	NOTE_ON | 1, NOTE_B2,	NOTE_ON | 2, NOTE_FS6,	NOTE_ON | 3, NOTE_B3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS3,	NOTE_OFF | 2,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS6,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B6,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B2,	NOTE_ON | 2, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_ON | 1, NOTE_FS4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_FS4,	NOTE_ON | 1, NOTE_GS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NOTE_ON | 1, NOTE_FS7,	NOTE_ON | 2, NOTE_FS6,	NOTE_ON | 3, NOTE_CS4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS4,	NOTE_ON | 1, NOTE_E7,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NOTE_ON | 1, NOTE_CS7,	NOTE_ON | 2, NOTE_CS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS3,	NOTE_ON | 1, NOTE_B6,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NOTE_ON | 1, NOTE_GS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NOTE_ON | 1, NOTE_FS7,	NOTE_ON | 2, NOTE_FS6,	NOTE_ON | 3, NOTE_CS4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS4,	NOTE_ON | 1, NOTE_E7,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A4,	NOTE_ON | 1, NOTE_CS7,	NOTE_ON | 2, NOTE_CS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS3,	NOTE_ON | 1, NOTE_B6,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E5,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NO_ACTION,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NOTE_ON | 1, NOTE_GS6,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS5,	NOTE_ON | 1, NOTE_FS7,	NOTE_ON | 2, NOTE_FS6,	NOTE_ON | 3, NOTE_CS4,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_GS4,	NO_ACTION,	NO_ACTION,	NOTE_OFF | 3,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS4,	NOTE_ON | 1, NOTE_E7,	NOTE_ON | 2, NOTE_E6,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_A3,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_E4,	NOTE_ON | 1, NOTE_B6,	NOTE_ON | 2, NOTE_B5,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_B4,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_ON | 0, NOTE_CS7,	NOTE_ON | 1, NOTE_E6,	NOTE_ON | 2, NOTE_CS6,	NOTE_ON | 3, NOTE_CS5,	NOTE_ON | 4, NOTE_GS4,	NOTE_ON | 5, NOTE_CS4,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NOTE_OFF | 0,	NOTE_OFF | 1,	NOTE_OFF | 2,	NOTE_OFF | 3,	NOTE_OFF | 4,	NOTE_OFF | 5,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,
		NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,	NO_ACTION,

		END_OF_STREAM
};

/* Music tracker data length */
const uint16_t tracker_music_length = sizeof(tracker_music_opcodes);
