/* See COPYRIGHT for copyright information. */

#include <inc/stdio.h>
#include <inc/string.h>
#include <inc/assert.h>

#include <kern/monitor.h>
#include <kern/console.h>
#include <kern/pmap.h>
#include <kern/kclock.h>
#include <kern/env.h>
#include <kern/trap.h>
#include <kern/sched.h>
#include <kern/picirq.h>

//SUNUS MACROS 
#define RUN_MULIT_YIELD 0
/* SUNUS DEBUG MARCO */
#define DEBUG(on,value) do {                                            \
                        if(!on)                                         \
                                break;                                  \
                        cprintf("DEBUG INFO : the %s is %08x\n",#value,(uint32_t)(value)); \
                        if(getchar() == 'c')                            \
                                break;                                  \
                        }while(1)

void
i386_init(void)
{
	extern char edata[], end[];

	// Before doing anything else, complete the ELF loading process.
	// Clear the uninitialized global data (BSS) section of our program.
	// This ensures that all static/global variables start out zero.
	memset(edata, 0, end - edata);

	// Initialize the console.
	// Can't call cprintf until after we do this!
	cons_init();

	cprintf("6828 decimal is %o octal!\n", 6828);

	// Lab 2 memory management initialization functions
	i386_detect_memory();
	i386_vm_init();

	// Lab 3 user environment initialization functions
	env_init();
	idt_init();

	// Lab 4 multitasking initialization functions
	pic_init();
	kclock_init();
	// Should always have an idle process as first one.
	ENV_CREATE(user_idle);	

	// Start fs.
	ENV_CREATE(fs_fs);

#if defined(TEST)
	// Don't touch -- used by grading script!
	ENV_CREATE2(TEST, TESTSIZE);
#else
	// Touch all you want.
	// ENV_CREATE(user_writemotd);
	// ENV_CREATE(user_testfile);
//*    ENV_CREATE(user_init);
//*    ENV_CREATE(user_icode);
#endif // TEST*

	// Schedule and run the first user environment!
	sched_yield();
}


/*
 * Variable panicstr contains argument to first call to panic; used as flag
 * to indicate that the kernel has already called panic.
 */
static const char *panicstr;

/*
 * Panic is called on unresolvable fatal errors.
 * It prints "panic: mesg", and then enters the kernel monitor.
 */
void
_panic(const char *file, int line, const char *fmt,...)
{
	va_list ap;

	if (panicstr)
		goto dead;
	panicstr = fmt;

	// Be extra sure that the machine is in as reasonable state
	__asm __volatile("cli; cld");

	va_start(ap, fmt);
	cprintf("kernel panic at %s:%d: ", file, line);
	vcprintf(fmt, ap);
	cprintf("\n");
	va_end(ap);

dead:
	/* break into the kernel monitor */
	while (1)
		monitor(NULL);
}
static const char * const dbg_infos[10] = 
{
        NULL,
        "ERROR",
        "FINISHED",
        "IN",
        "OUT",
        "WATCH",
        "PANIC"
};
void
_sunusdbg(int on,int act,const char *func, const char *file, int line, const char *fmt,...)
{

        va_list ap;
        if(on)
        {
                cprintf("SUNUS : %s %d %s %s\n",file,line,func,dbg_infos[act]);
                va_start(ap, fmt);
                vcprintf(fmt, ap);
                cprintf("\n");
                if(act == 6)
                        panic("stop!\n");
        }
}

/* like panic, but don't */
void
_warn(const char *file, int line, const char *fmt,...)
{
	va_list ap;

	va_start(ap, fmt);
	cprintf("kernel warning at %s:%d: ", file, line);
	vcprintf(fmt, ap);
	cprintf("\n");
	va_end(ap);
}
