/*
 * Sylvain BERTRAND <digital.ragnarok@gmail.com>
 * code protected by GNU GPL v3
 */
#include <X11/Xlib.h>
#include <X11/extensions/sync.h>
#include <glib.h>
#include <libxml/xinclude.h>
#include "../../toolkit/keyboard.h"
#include "../../toolkit/xml.h"
#include "../misc.h"
#include "../geom.h"
#include "../frame.h"
#include "../actions.h"
#include "../window.h"
#include "../stacking.h"
#include "../client.h"
#include "../config.h"
#include "../place.h"
#include "../moveresize.h"
#include "../screen.h"
struct options {
	gint x;
	gint x_denom;
	gint y;
	gint y_denom;
};

static gpointer setup_func(xmlNodePtr node)
{
	xmlNodePtr n;
	struct options *o;
	gchar *s;
	o = g_slice_new0(struct options);
	if((n = t_xml_find_node(node, "x"))) {
		s = t_xml_node_string(n);
		config_parse_relative_number(s, &o->x, &o->x_denom);
		g_free(s);
	}
	if((n = t_xml_find_node(node, "y"))) {
		s = t_xml_node_string(n);
		config_parse_relative_number(s, &o->y, &o->y_denom);
		g_free(s);
	}
	return o;
}

static void free_func(gpointer o)
{
	g_slice_free(struct options, o);
}

static gboolean run_func(struct wm_actions_data *data, gpointer options)
{
	struct options *o = options;
	if(data->client) {
		struct wm_client *c;
		gint x, y, lw, lh, w, h;
		c = data->client;
		x = o->x;
		y = o->y;
		if(o->x_denom || o->y_denom) {
			const struct wm_rect *carea;
			carea = screen_area(c->desktop, client_monitor(c), NULL);
			if(o->x_denom)
				x = (x * carea->width) / o->x_denom;
			if(o->y_denom)
				y = (y * carea->height) / o->y_denom;
		}
		x = c->area.x + x;
		y = c->area.y + y;
		w = c->area.width;
		h = c->area.height;
		client_try_configure(c, &x, &y, &w, &h, &lw, &lh, TRUE);
		client_find_onscreen(c, &x, &y, w, h, FALSE);
		actions_client_move(data, TRUE);
		client_configure(c, x, y, w, h, TRUE, TRUE, FALSE);
		actions_client_move(data, FALSE);
	}
	return FALSE;
}

void action_moverelative_startup(void)
{
	actions_register("MoveRelative", setup_func, free_func, run_func);
}
