/*
 * Sylvain BERTRAND <digital.ragnarok@gmail.com>
 * code protected by GNU GPL v3
 */
#include <X11/Xlib.h>
#include <glib.h>
#include <libxml/xinclude.h>
#include "../toolkit/xml.h"
#include "../toolkit/keyboard.h"
#include "keytree.h"
#include "misc.h"
#include "geom.h"
#include "frame.h"
#include "actions.h"
#include "translate.h"
#include "keyboard.h"
void tree_destroy(struct wm_key_binding_tree *tree)
{
	struct wm_key_binding_tree *c;
	while(tree) {
		tree_destroy(tree->next_sibling);
		c = tree->first_child;
		if(c == NULL) {
			GList *it;
			GSList *sit;
			for(it = tree->keylist; it != NULL; it = it->next)
				g_free(it->data);
			g_list_free(tree->keylist);
			for(sit = tree->actions; sit != NULL; sit = sit->next)
				actions_act_unref(sit->data);
			g_slist_free(tree->actions);
		}
		g_slice_free(struct wm_key_binding_tree, tree);
		tree = c;
	}
}

struct wm_key_binding_tree *tree_build(GList *keylist)
{
	GList *it;
	struct wm_key_binding_tree *ret = NULL, *p;
	if(g_list_length(keylist) <= 0)
		return NULL;
	for(it = g_list_last(keylist); it; it = g_list_previous(it)) {
		GList *kit;
		p = ret;
		ret = g_slice_new0(struct wm_key_binding_tree);
		for(kit = it; kit != NULL; kit = g_list_previous(kit))
			ret->keylist = g_list_prepend(ret->keylist, g_strdup(kit->data));
		ret->first_child = p;
		if(p != NULL)
			p->parent = ret;
		translate_key(it->data, &ret->state, &ret->key);
	}
	return ret;
}

void tree_assimilate(struct wm_key_binding_tree *node)
{
	struct wm_key_binding_tree *a, *b, *tmp, *last;
	if(keyboard_firstnode == NULL) {
		keyboard_firstnode = node;
	} else {
		a = keyboard_firstnode;
		last = a;
		b = node;
		while(a) {
			last = a;
			if(!(a->state == b->state && a->key == b->key && b->key != 0)) {
				a = a->next_sibling;
			} else {
				tmp = b;
				b = b->first_child;
				g_slice_free(struct wm_key_binding_tree, tmp);
				a = a->first_child;
		}} if(!(last->state == b->state && last->key == b->key && b->key != 0)) {
			last->next_sibling = b;
			b->parent = last->parent;
		} else {
			last->first_child = b->first_child;
			last->first_child->parent = last;
			g_slice_free(struct wm_key_binding_tree, b);
		}
	}
}

struct wm_key_binding_tree *tree_find(struct wm_key_binding_tree *search, gboolean *conflict)
{
	struct wm_key_binding_tree *a, *b;
	*conflict = FALSE;
	a = keyboard_firstnode;
	b = search;
	while(a && b) {
		if(!(a->state == b->state && a->key == b->key && b->key != 0)) {
			a = a->next_sibling;
		} else {
			if((a->first_child == NULL) == (b->first_child == NULL)) {
				if(a->first_child == NULL) {
					return a;
				}
			} else {
				*conflict = TRUE;
				return NULL;
			}
			b = b->first_child;
			a = a->first_child;
		}
	}
	return NULL;
}

gboolean tree_chroot(struct wm_key_binding_tree *tree, GList *keylist)
{
	guint key, state;
	translate_key(keylist->data, &state, &key);
	while(tree != NULL && !(tree->state == state && tree->key == key))
		tree = tree->next_sibling;
	if(tree != NULL) {
		if(keylist->next == NULL) {
			tree->chroot = TRUE;
			return TRUE;
		} else
			return tree_chroot(tree->first_child, keylist->next);
	}
	return FALSE;
}
