// Tomer Elmalem

#include "invaders.h"
#include "vehicles.h"
#include "mylib.h"
#include <stdlib.h>

/* Initialize the invaders. Set them all to
   alive, and increment their column position. */
void init_invaders() {
  for (int i=0; i<INVADERS_AMT; i++) {
    invaders[i].r = 10;
    invaders[i].c = 10+(15*i);
    invaders[i].alive = 1;
    invaders[i].width = ALIEN1_WIDTH;
    invaders[i].height = ALIEN1_HEIGHT;
  }
}

/* Initialize the tank. Starts out with 3 lives */
void init_tank() {
  tank.r = 140;
  tank.position = 110;
  tank.lives = 3;
}

/* Initialize all the invader bullets. 
   fired = 0 -> not fired
   fired = 1 -> fired */
void init_invader_bullets() {
  for (int i=0; i<INVADERS_BLT; i++) {
    invader_bullets[i].r = 0;
    invader_bullets[i].c = 0;
    invader_bullets[i].fired = 0;
  }
}

/* Initialize all the invader bullets. 
   fired = 0 -> not fired
   fired = 1 -> fired */
void init_tank_bullets() {
  for (int i=0; i<TANK_BLT; i++) {
    tank_bullets[i].r = 0;
    tank_bullets[i].c = 0;
    tank_bullets[i].fired = 0;
  }
}

/* Fire invader bullets. Implented by generating a random
   number in every loop of the game and firing if the number
   is 0 or 1. This means that about every 2 of 100 loops of
   the game will result in a bullet being fired from the
   invader. */
void fire_invader_bullet() {
  int fire = rand() % 100;
  for (int i=0; i<INVADERS_BLT; i++) {
    if (!invader_bullets[i].fired) {
      if (fire <= 1) {
        // Find a random invader that is alive.
        int invader;
        do { 
          invader = rand() % 10;
        } while (!invaders[invader].alive);
        
        // Fire ze bullet!
        invader_bullets[i].r = 20;
        invader_bullets[i].c = invaders[invader].c+5;
        invader_bullets[i].fired = 1;
        return;
      }
    }
  }
}

/* Fire the tank bullet if the player has pressed
   the A button and all 5 bullets aren't currently
   in the air. In addition to this, it waits until
   the closest bullet to the tank is atleast 5px
   away so that all of the bullets don't group up. */
void fire_tank_bullet() {
  int bullet_position = 0;
  int to_be_fired = -1;

  for (int i=0; i<5; i++) {
    // Finds the next bullet to fire
    if (!tank_bullets[i].fired && to_be_fired == -1) {
      to_be_fired = i;
    }

    // gets the bullet closest to the tank
    if (tank_bullets[i].fired && tank_bullets[i].r > bullet_position) {
      bullet_position = tank_bullets[i].r;
    }
  }

  // if there is the bullet available to be fired
  // and the closest bullet is farther than 130 rows
  // fire ze bullet!
  if (to_be_fired != -1 && bullet_position < 130) {
    tank_bullets[to_be_fired].r = 140;
    tank_bullets[to_be_fired].c = tank.position + 7;
    tank_bullets[to_be_fired].fired = 1;
  }
}

/* Move the invaders by the specified amount */
void move_invaders(int amount) {
  for (int i=0; i<INVADERS_AMT; i++) {
    invaders[i].c += amount;
  }
}

/* Collision detection for the tank bullets and invaders.
   Gets a pointer to the a bullet and an invader and checks
   to see if there's a collision. If there is kill the 
   alien (woo!), decrease the amount of invaders left, and 
   mark the bullet as not fired. */
void check_invader_collision(Bullet *bullet, Invader *invader) {
  if (invader->alive) {
    if (invader->r+10 >= bullet->r && 
          (invader->c <= bullet->c &&
           invader->c+10 >= bullet->c)) {
      invader->alive = 0;
      invaders_left--;
      bullet->fired = 0;
    }
  }
}

/* Collision detection for the invader bullets and the tank.
   Gets a pointer to an invader bullet and the tank, then
   checks to see if there's a collision. If there is subtract
   a life and mark the bullet as fired. */
void check_tank_collision(Bullet *bullet, Tank *tank) {
  if (bullet->fired) {
    if (bullet->r+3 >= tank->r && 
          (bullet->c >= tank->position &&
           bullet->c <= tank->position+15)) {
      bullet->fired = 0;
      tank->lives--;
    }
  }
}
