// Tomer Elmalem

#include "mylib.h"

u16 *videoBuffer = (u16 *) 0x6000000;

void setPixel(int r, int c, u16 color) {
  videoBuffer[240*r+c] = color;
}

void drawRect(int r, int c, int width, int height, u16 color) {
  for (int i = 0; i < height; i++) { // loop through each row
    for (int j = 0; j < width; j++) { // each column
      setPixel(r+i, j+c, color);
    }
  }
}

void delay(int how_long) {
  for (volatile int i = 0; i < how_long * 1000; i++) {}
}

void wait_for_vblank() {
  u16 *X = SCANLINECOUNTER;

  while (*X > 160) {;} // in vblank, wait to exit
  while (*X <= 160) {;} // wait for beginning of next vblank
}

/* Use DMA to blank out the sreen */
void blank_screen() {
  REG_DMA3SAD = (vu32) RGB(0,0,0);
  REG_DMA3DAD = (vu32) videoBuffer;
  REG_DMA3CNT = 38400|DMA_ON; 
}

/* Use DMA to draw an image. The image is drawn
   line by line instead of entirely at once */
void drawImage3(int r, int c, int width, int height, const u16* image) {
  for (u16 i=0; i < height; i++) {
    REG_DMA3SAD = (vu32) (image + (width*i)); // Source
    REG_DMA3DAD = (vu32) (videoBuffer + (240*(r+i)+c)); // Destination
    REG_DMA3CNT = width|DMA_ON; // Count and start
  }
}
