// Tomer M&M Elmalem

#include "mylib.h"
#include "text.h"
#include "end.h"
#include "start.h"
#include "assets.h"
#include "mc_lib.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

int main() {
  REG_DISPCNT = MODE4 | BG2_ENABLE; // set the display mode
  int frame = 0;
  int start = 0;

  // Declare all of the variables
  int level = 1;
  City cities[CITY_AMT];
  Missile *enemy;
  enemy = malloc((ENEMY_AMT*level)*sizeof(*enemy));
  Missile player[PLAYER_AMT];
  Explosion expl[PLAYER_AMT];
  Cursor cursor;
  char score_buffer[41];
  char level_buffer[41];
  int score=0;
  int city_count = CITY_AMT;
  int level_fired = 0;
  int missile_hit = 0;

  // Initialize everything
  init_cities(cities);
  init_enemy_missiles(enemy, level);
  init_player_missiles(player);
  init_cursor(&cursor);

  // Game loop
  while(1) {
    wait_for_vblank();
    flip_page();

    // Start Screen
    if (!start) {
      // draw start screen
      fill_palette(start_palette);
      drawImage4(0,0,240,160,start_screen);

      // Reinitialize everything
      score=0;
      city_count = CITY_AMT;
      level_fired = 0;
      missile_hit = 0;
      init_cities(cities);
      init_enemy_missiles(enemy, level);
      init_player_missiles(player);
      init_cursor(&cursor);
    }

    // If the city count is 0, game over.
    else if (!city_count) {
      fill_palette(end_palette);
      drawImage4(0,0,240,160,end);
    }
    
    // Or the game
    else {
      blank_screen();
      frame++; 
      
      // Draw the bottom bar
      drawImage4(150,0,BOTTOM_WIDTH,BOTTOM_HEIGHT,bottom);

      // Draw the turret
      drawImage4(139,110,TURRET_WIDTH,TURRET_HEIGHT,turret);

      // Draw the cursor at the current position
      drawImage4(cursor.r-3,cursor.c-3,CURSOR_WIDTH,CURSOR_HEIGHT,cursor_img);

      // Draw all of the cities that are currently 
      for (int i=0; i<CITY_AMT; i++) {
        if (cities[i].alive) {
          drawImage4(cities[i].r, cities[i].c, CITY_WIDTH, CITY_HEIGHT, city_img);
        }
      }

      // Loop through the enemy missiles and take care of everything
      for (int i=0; i<ENEMY_AMT*level; i++) {
        if (enemy[i].fired && enemy[i].draw) {
          // Draw the missile
          draw_line(&enemy[i]);

          // Increment the missile every 10 frames
          if (frame % 10 == 0) enemy[i].itr++;

          // Clear the missile if it reaches its destination
          if (clear_missile(&enemy[i])) {
            missile_hit++; // if hit, increment the counter
          }

          // Loop through the explosions
          for (int j=0; j<PLAYER_AMT; j++) {
            if (expl[j].time) { // if the explosion is active
              if (collision_expl(&enemy[i],&expl[j])) { // if there is a collision with the missile
                score++;
                missile_hit++;
              }
            }
          }

          // Loop through the cities
          for (int j=0; j<CITY_AMT; j++) {
            // if it collides with the city
            if (collision_city(&enemy[i], &cities[j])) {
              city_count--;
              missile_hit++;
            }
          }
        }
      }

      // Loop through the players missiles
      for (int i=0; i<PLAYER_AMT; i++) {
        if (player[i].fired && player[i].draw) {
          draw_line(&player[i]);

          // Increment the player missile every 10 frames
          if (frame % 5 == 0) player[i].itr++;

          // clear the missile if it reaches its destination
          if (clear_missile(&player[i])) {
            explode(&player[i],expl); // then explode
            player[i].fired=0; // bullet is available to be fired
          }
        }
      }

      sprintf(score_buffer, "Score: %d", score);
      drawString(152, 5, score_buffer, ASSETS_PALETTE_SIZE-1);
      sprintf(level_buffer, "Level: %d", level);
      drawString(152, 185, level_buffer, ASSETS_PALETTE_SIZE-1);

      // Draw the explosions if they are active
      for (int i=0; i<PLAYER_AMT; i++) {
        if (expl[i].time)
          drawRect4(expl[i].r,expl[i].c,expl[i].width,expl[i].height,ASSETS_PALETTE_SIZE-1);
      }

      // Fire zeh (enemy) missile! every 6 seconds
      if (frame % 360 == 0) {
        if (fire_enemy_missile(enemy, level)) level_fired++;
      }

      // Decrease the life of the explosion every second.
      if (frame % 60 == 0) {
        for (int i=0; i<PLAYER_AMT; i++) {
          if (expl[i].time > 0) expl[i].time--;
        }
      }

      // If the end of the level is reached
      if (level_fired == ENEMY_AMT*level && missile_hit == ENEMY_AMT*level) {
        for (int i=0; i<CITY_AMT; i++) {
          if (cities[i].alive) score +=10;
        }
        level++;
        level_fired=0;
        missile_hit=0;
        init_enemy_missiles(enemy,level);
        realloc(enemy, (ENEMY_AMT*level)*sizeof(*enemy));
      }

      // If the A button is pressed, fire zeh (player) missile!
      if (KEY_DOWN_NOW(BUTTON_A)) {
        if (cursor.ready) fire_player_missile(player, &cursor);
      }

      // Cursor is ready to fire every 1 second.
      if (frame % 60 == 0) {
        cursor.ready = 1;
      }

      // Movement for the cursor
      if (KEY_DOWN_NOW(BUTTON_UP)) {
        if (cursor.r >= 6) cursor.r--;
      }
      if (KEY_DOWN_NOW(BUTTON_RIGHT)) {
        if (cursor.c <= 234) cursor.c++;
      }
      if (KEY_DOWN_NOW(BUTTON_DOWN)) {
        if (cursor.r <= 130) cursor.r++;
      }
      if (KEY_DOWN_NOW(BUTTON_LEFT)) {
        if (cursor.c >= 6) cursor.c--;
      }
    }

    /* If the start button is pressed then start the game.
       Go go go! Otherwise if the select button is pressed
       reset the game. */
    if (KEY_DOWN_NOW(BUTTON_START)) {
      fill_palette(assets_palette);
      start = 1;
    } else if (KEY_DOWN_NOW(BUTTON_SELECT)) {
      fill_palette(start_palette);
      start = 0;
    }
  }
}
