// Tomer Elmalem

#include "mc_lib.h"
#include "assets.h"
#include "stdlib.h"
#include "mylib.h"
#include <stdlib.h>

/* Initialize the cities. Pass in an array of cities
   and set their default values. */
void init_cities(City *city) {
  for (int i=0; i<CITY_AMT; i++) {
    city[i].alive = 1;
    city[i].w = CITY_WIDTH;
    city[i].h = CITY_HEIGHT;
    city[i].r = 141;
    if (i > 2) city[i].c = (CITY_WIDTH+10)*i+30+30;
    else city[i].c = (CITY_WIDTH+10)*i+30;
  }
}

/* Draw a line using using the Bresenham line algorithm. Takes
   a missile struct and draws the line based on the start and
   end position. Animates by drawing the line up to a certain
   iteration number. */
void draw_line(Missile *m)  {
  int y0 = m->start_c;
  int x0 = m->start_r;
  int y1 = m->end_c;
  int x1 = m->end_r;
  int itr= m->itr;

  int dx = abs(x1-x0);
  int dy = abs(y1-y0);
  int sx = (x0 < x1) ? 1 : -1;
  int sy = (y0 < y1) ? 1 : -1;
  int err = dx-dy;
  int e2;

  for(int i=0; i<itr; i++) {
    setPixel4(x0, y0, ASSETS_PALETTE_SIZE-1);
    if (x0 == x1 && y0 == y1) break;
    e2 = 2*err;
    if (e2 > -(dy)) {
      err -= dy;
      x0 += sx;
    }
    if (e2 < dx) {
      err += dx;
      y0 += sy;
    }

    m->cur_r = x0;
    m->cur_c = y0;
  }
}

/* Initialize the enemy missiles. Takes in a pointer to an
   an array of missile structs and sets their default values. */
void init_enemy_missiles(Missile *missile, int level) {
  for (int i=0; i<(ENEMY_AMT*level); i++) {
    missile[i].fired=0;
    missile[i].start_r=0;
    missile[i].end_r=150;
    missile[i].itr=1;
    missile[i].draw=0;
  }
}

/* Initialize the player missiles. Takes in a pointer to an
   array of missile structs and sets their default values. */
void init_player_missiles(Missile *missile) {
  for (int i=0; i<PLAYER_AMT; i++) {
    missile[i].fired=0;
    missile[i].draw=0;
    missile[i].start_r=138;
    missile[i].start_c=118;
  }
}

/* Initialize the cursor. Reset the position and set the
   cursors status to ready. */
void init_cursor(Cursor *cursor) {
  cursor->r = 80;
  cursor->c = 120;
  cursor->ready = 1;
}

/* Fire an enemy missile. Takes in a pointer to a missile and
   the current level. Randomly generates a start and end
   position and then finds a missile that hasn't been fired
   or drawn and fires that. Returns 1 if fired or 0 if not. */
int fire_enemy_missile(Missile *m, int level) {
  int start_c = rand() % 230 + 5;
  int end_c = rand() % 230 + 5;
  for (int i=0; i<ENEMY_AMT*level; i++) {
    if (!m[i].fired && !m[i].draw) {
      m[i].start_c = start_c;
      m[i].end_c = end_c;
      m[i].fired= 1;
      m[i].draw=1;
      return 1;
    }
  }
  return 0;
}

/* Fire the player missile. Takes in a pointer to the missile
   and a pointer to the cursor. Finds a not fired missile and
   fire the missile to the cursor's position. */
void fire_player_missile(Missile *m, Cursor *cursor) {
  int end_c = cursor->c+3;
  int end_r = cursor->r+3;

  for (int i=0; i<PLAYER_AMT; i++) {
    if (!m[i].fired) {
      m[i].end_c = end_c;
      m[i].end_r = end_r;
      m[i].fired= 1;
      m[i].draw=1;

      cursor->ready=0;
      break;
    }
  }
}

/* Marks a missile as cleared. Takes in a pointer to a missile,
   if the current position equals the end destination reset
   its values. Returns 1 if destination reached otherwise
   return 0 */
int clear_missile(Missile *m) {
  if (m->cur_r == m->end_r && m->cur_c == m->end_c) {
    m->draw=0;
    m->itr=1;
    m->cur_c=0;
    m->cur_r=0;
    return 1;
  }
  return 0;
}

/* Generate an explosion from the player's missile. Takes in
   a pointer to a missile and a pointer to an array of
   explosion structs. Finds an available explosion and triggers
   it. */
void explode(Missile *m, Explosion *e) {
  for (int i=0; i<PLAYER_AMT; i++) {
    if (!e[i].time) {
      e[i].width=10;
      e[i].height=10;
      e[i].r=m->end_r-5;
      e[i].c=m->end_c-5;
      e[i].time=3;
      break;
    }
  }
}

/* Checks to see if an enemy missile collided with an
   explosion. Takes in a pointer to the missile and explosion
   and checks to see if the missiles current position is in
   the bounds of the explosion. Returns 1 if collision and
   0 otherwise. */
int collision_expl(Missile *m, Explosion *e) {
  if (m->cur_r <= (e->r+e->height) &&
      m->cur_r >= e->r &&
      m->cur_c <= (e->c+e->width) &&
      m->cur_c >= e->c) {
    m->fired=0;
    m->itr=1;
    m->cur_c=0;
    m->cur_r=0;
    return 1;
  } else {
    return 0;
  }
}

/* Checks to see if a missile collides with a city. 
   works similarly to the previous function. Returns 1 if
   collision, 0 otherwise. */
int collision_city(Missile *m, City *c) {
  if (m->cur_r <= (c->r+c->h) &&
      m->cur_r >= c->r &&
      m->cur_c <= (c->c+c->w) &&
      m->cur_c >= c->c
      && c->alive) {
    m->fired=0; 
    m->itr=1;
    m->cur_c=0;
    m->cur_r=0;
    c->alive=0;
    return 1;
  } else {
    return 0;
  }
}
