// Tomer Elmalem

#include "mylib.h"

u16 *videoBuffer = (u16 *) 0x6000000;

void setPixel(int r, int c, u16 color) {
  videoBuffer[240*r+c] = color;
}

void setPixel4(int row, int col, u8 index) {
  int pixel = OFFSET(row, col, 240);
  int shortNumber = pixel/2;

  if(col & 1) {
    videoBuffer[shortNumber] = (videoBuffer[shortNumber] & 0x00ff) |  (index<<8) ;
  } else {
    videoBuffer[shortNumber] = (videoBuffer[shortNumber] & 0xff00) |  (index) ;
  }
}

void drawRect(int r, int c, int width, int height, u16 color) {
  for (int i = 0; i < height; i++) { // loop through each row
    for (int j = 0; j < width; j++) { // each column
      setPixel(r+i, j+c, color);
    }
  }
}

/* Draw Rectangle for mode4 using DMA. From Bill's code. */
void drawRect4(int r, int c, int height, int width, u8 color) {
  volatile u16 clr = color<<8 | color;
  for(int i=0; i<height; i++) {
    DMA[3].src = &clr;
    DMA[3].dst = videoBuffer + OFFSET(r+i, c, 240)/2;
    DMA[3].cnt = (width/2) | DMA_ON | DMA_SOURCE_FIXED;
  }
}

void delay(int how_long) {
  for (volatile int i = 0; i < how_long * 1000; i++) {}
}

void wait_for_vblank() {
  u16 *X = SCANLINECOUNTER;

  while (*X > 160) {;} // in vblank, wait to exit
  while (*X <= 160) {;} // wait for beginning of next vblank
}

/* Use DMA to blank out the sreen */
void blank_screen() {
  volatile u16 black = RGB(0,0,0);
  REG_DMA3SAD = (vu32) &black;
  REG_DMA3DAD = (vu32) videoBuffer;
  REG_DMA3CNT = 19200|DMA_ON|DMA_SOURCE_FIXED; 
}

/* Use DMA to draw an image. The image is drawn
   line by line instead of entirely at once */
void drawImage4(int r, int c, int width, int height, const u16* image) {
  width /= 2;
  for (u16 i=0; i < height; i++) {
    DMA[3].src = (vu32 *) (image + (width*i)); // Source
    DMA[3].dst = (vu32 *) (videoBuffer + (240*(r+i)+c)/2); // Destination
    DMA[3].cnt = width|DMA_ON; // Count and start
  }
}

/* Flip the page that is currently being drawn on. Check to see
   which buffer is currently active and then switch it. */
void flip_page() {
  if(REG_DISPCNT & BUFFER1FLAG) {
    REG_DISPCNT &= ~BUFFER1FLAG;
    videoBuffer = BUFFER1;
  } else {
    REG_DISPCNT |= BUFFER1FLAG;
    videoBuffer = BUFFER0;
  }
}

void fill_palette(const short unsigned int *palette) {
  for (int i=0; i<255; i++) {
    PALETTE[i] = palette[i];
  }
}
