// Tomer Elmalem

#ifndef MYLIB_H
#define MYLIB_H

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned int   u32;
typedef volatile unsigned char  vu8;
typedef volatile unsigned short vu16;
typedef volatile unsigned int   vu32;

#define REG_DISPCNT *(u16 *) 0x4000000
#define MODE3 3
#define MODE4 4
#define BG2_ENABLE 1<<10
#define RGB(r,g,b) ((r)|((g)<<5)|((b)<<10))
#define SCANLINECOUNTER ((unsigned short *) 0x4000006)
#define MAXROW 160
#define MAXCOL 240
#define OFFSET(r, c, rowlen)  ((r)*(rowlen)+(c))

#define BUFFER0 ((u16 *) 0x06000000)
#define BUFFER1 ((u16 *) 0x0600A000)
#define PALETTE ((u16 *) 0x05000000)
#define BUFFER1FLAG 0x10 

/* Button macros */
#define BUTTONS (*(unsigned int *) 0x4000130)
#define BUTTON_A      (1<<0)
#define BUTTON_B      (1<<1)
#define BUTTON_SELECT (1<<2)
#define BUTTON_START  (1<<3)
#define BUTTON_RIGHT  (1<<4)
#define BUTTON_LEFT   (1<<5)
#define BUTTON_UP     (1<<6)
#define BUTTON_DOWN   (1<<7)
#define BUTTON_R      (1<<8)
#define BUTTON_L      (1<<9)
#define KEY_DOWN_NOW(key) (~(BUTTONS)&key)

typedef struct {
  const volatile void *src;
  volatile void *dst;
  volatile u32 cnt;
} DMAREC;
#define DMA ((volatile DMAREC *)0x040000B0)

/* DMA macros */
#define REG_DMA3SAD *(vu32*)0x40000D4  /* source address*/
#define REG_DMA3DAD *(vu32*)0x40000D8  /* destination address*/
#define REG_DMA3CNT *(vu32*)0x40000DC  /* control register*/
#define DMA_ON (1 << 31)
#define DMA_SOURCE_FIXED (2<<23)

/* Function prototypes */
void setPixel(int r, int c, u16 color);
void setPixel4(int row, int col, u8 index); 
void drawRect(int r, int c, int width, int height, u16 color);
void delay(int how_long);
void wait_for_vblank();
void drawImage4(int r, int c, int width, int height, const u16* image);
void blank_screen();
void flip_page();
void fill_palette(const short unsigned int *palette);
void drawRect4(int r, int c, int height, int width, u8 color);
extern u16 *videoBuffer;
#endif
