#ifndef _terror__hashmap_h_
#define _terror__hashmap_h_

#include <stdint.h>
#include <terror/darray.h>

#define DEFAULT_NUMBER_OF_BUCKETS 100

typedef int (*Hashmap_compare)(void *a, void *b);
typedef uint32_t (*Hashmap_hash)(void *key);

typedef struct Hashmap {
  DArray *buckets;
  Hashmap_compare compare;
  Hashmap_hash hash;
} Hashmap;

typedef struct HashmapNode {
  void *key;
  void *data;
  int type;
  uint32_t hash;
} HashmapNode;

typedef int (*Hashmap_traverse_cb)(HashmapNode *node);

Hashmap *Hashmap_create(Hashmap_compare compare, Hashmap_hash);
void Hashmap_destroy(Hashmap *map);

int Hashmap_set(Hashmap *map, void *key, void *data, int type);
void *Hashmap_get(Hashmap *map, void *key);

int Hashmap_traverse(Hashmap *map, Hashmap_traverse_cb traverse_cb);

void *Hashmap_delete(Hashmap *map, void *key);

// Functions to compare Terror strings rather than normal strings
int String_compare(void *a, void *b);
uint32_t String_hash(void *a);

#endif
