/* avltree - library for manipulation of binary trees.
 * Ported from libavl.
 *
 * Copyright (C) 2012 Varphone Wong <varphone@qq.com>
 * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2004 Free Software
 * Foundation, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 */

#ifndef LIBGCONTAINERS_AVLTREE_H
#define LIBGCONTAINERS_AVLTREE_H

#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif

#ifndef AVLTREE_T_DEFINED
#define AVLTREE_T_DEFINED

#define AVTLREE_FLAG_COB     0x00000001         ///< Indicate the avltree was create on buffer
#define NULL_AVLTREE        ((avltree_t)0)      ///< Null avltree.

// Forwards.
struct gallocator;

/// AVL tree types.
typedef void* avltree_t;

// Function types.
/// AVL tree compare routine.
typedef int (*avltree_comparison_t)(const void* a, const void* b, void* param);

#ifndef AVLTREE_ITEM_MANAGER
#define AVLTREE_ITEM_MANAGER
typedef void* (*avltree_item_copy_t)(void* item, void* param);
typedef void* (*avltree_item_ctor_t)(void* item, void* param);
typedef void (*avltree_item_dtor_t)(void* item, void* param);
struct avltree_item_manager {
    avltree_item_copy_t copy;
    avltree_item_ctor_t ctor;
    avltree_item_dtor_t dtor;
};
#endif

// Maximum AVL tree height.
#ifndef AVLTREE_MAX_HEIGHT
#define AVLTREE_MAX_HEIGHT 92
#endif

// Table functions.
avltree_t avltree_create(avltree_comparison_t, void*, struct gallocator*);
avltree_t avltree_copy(const avltree_t, avltree_item_copy_t,
                       avltree_item_ctor_t, avltree_item_dtor_t,
                       struct gallocator*);
void avltree_destroy(avltree_t, avltree_item_dtor_t);
void** avltree_probe(avltree_t, void*);
void* avltree_insert(avltree_t, void*);
void* avltree_replace(avltree_t, void*);
void* avltree_delete(avltree_t, const void*);
void* avltree_find(const avltree_t, const void*);

#define avltree_count(table) ((size_t) (table)->avl_count)
#endif

#ifndef AVLTRAVERSER_T_DEFINED
#define AVLTRAVERSER_T_DEFINED

#define AVLTRAVERSER_FLAG_COB      0x00000001              ///< Indicate the avltree traverser was create on buffer
#define AVLTRAVERSER_SIZE          512
#define NULL_AVLTRAVERSER          ((avltraverser_t)0)     ///< Null avltree traverser.

/// AVL tree traverser.
typedef void* avltraverser_t;

// AVL tree traverser functions.
avltraverser_t avltraverser_create(avltree_t);
avltraverser_t avltraverser_create_on_buffer(avltree_t, void*, size_t);
void avltraverser_destroy(avltraverser_t);

void* avltraverser_first(avltraverser_t);
void* avltraverser_last(avltraverser_t);
void* avltraverser_find(avltraverser_t, void*);
void* avltraverser_insert(avltraverser_t, void*);
void* avltraverser_copy(avltraverser_t, const avltraverser_t);
void* avltraverser_next(avltraverser_t);
void* avltraverser_prev(avltraverser_t);
void* avltraverser_cur(avltraverser_t);
void* avltraverser_replace(avltraverser_t, void*);
#endif

#endif

