#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "avltree.h"

static int scmp(const void* a, const void* b, void* param)
{
	(void)param;
	return strcmp(a, b);
}

void test1(void)
{
	avltree_t avl = avltree_create(scmp, NULL, NULL);
	if (!avl) {
		fprintf(stderr, "avltree_create() failure\n");
		exit(1);
	}
    for (int i = 0; i < 30; i++) {
        char buffer[256];
        snprintf(buffer, sizeof(buffer), "Hello, avltree, traverser on heap, item %d", i);
        avltree_insert(avl, strdup(buffer));
    }
    avltraverser_t trav = avltraverser_create(avl);
    void* item = 0;
    while (item = avltraverser_next(trav))
        printf("next: %s\n", (char*)item);
    avltraverser_destroy(trav);
}

void test2(void)
{
	avltree_t avl = avltree_create(scmp, NULL, NULL);
	if (!avl) {
		fprintf(stderr, "avltree_create() failure\n");
		exit(1);
	}
    for (int i = 0; i < 30; i++) {
        char buffer[256];
        snprintf(buffer, sizeof(buffer), "Hello, avltree, traverser on heap, item %d", i);
        avltree_insert(avl, buffer);
    }
    avltraverser_t trav = avltraverser_create(avl);
    void* item = 0;
    while (item = avltraverser_next(trav))
        printf("next: %s\n", (char*)item);
    avltraverser_destroy(trav);
}

void test3(void)
{
	avltree_t avl = avltree_create(scmp, NULL, NULL);
	if (!avl) {
		fprintf(stderr, "avltree_create() failure\n");
		exit(1);
	}
    for (int i = 0; i < 30; i++)
        avltree_insert(avl, "Hello, avltree, traverser on buffer");
    char buffer[AVLTRAVERSER_SIZE];
    avltraverser_t trav = avltraverser_create_on_buffer(avl, buffer, sizeof(buffer));
    void* item = 0;
    while (item = avltraverser_next(trav)) {
        printf("next: %s\n", (char*)item);
    };
    avltraverser_destroy(trav);
}

void run_test(void (*test)(void), const char* header)
{
	int n = printf("%s ", header);
	while (n++ < 80)
		putchar('=');
	putchar('\n');
	test();
	putchar('\n');
}

int main(int argc, char** argv)
{
	run_test(test1, "Test1");
	run_test(test2, "Test2");
	run_test(test3, "Test3");
	return 0;
}
