/*
 * Copyright (C) 2012 Varphone Wong <varphone@qq.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public Licens
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-
 *
 */

#ifdef HAVE_ASSERT_H
#include <assert.h>
#endif
#include "gallocator.h"
#include "circle_buffer.h"

#if 0
#define CIRCLE_BUFFER_STUFF_SIZE    8

struct circle_buffer {
    char*               buffer;
    size_t              size;
    size_t              length;
    size_t              space;
    size_t              rpos;
    size_t              wpos;
    struct gallocator*  allocator;
    struct glocker*     locker;
    struct gsemaphore*  sema;
    unsigned int        flags;
    size_t              length_needed;
    size_t              space_needed;
    
};

circle_buffer_t circle_buffer_create(size_t size, struct gallocator* allocator)
{
    allocator = allocator ? allocator : &gallocator_default;
    struct circle_buffer* cb = allocator->zalloc(allocator, sizeof(*cb) + CIRCLE_BUFFER_STUFF_SIZE + size);
    if (cb) {
        cb->buffer      = ((char*)cb) + sizeof(*cb) + CIRCLE_BUFFER_STUFF_SIZE;
        cb->size        = size;
        cb->allocator   = allocator;
        cb->flags       = 0;
    }
    return (circle_buffer_t)cb;
}

circle_buffer_t circle_buffer_create_on_buffer(void* buffer, size_t size)
{
    assert(buffer);
    struct circle_buffer* cb = buffer;
    cb->buffer      = (char*)buffer + sizeof(*cb) + CIRCLE_BUFFER_STUFF_SIZE;
    cb->size        = size - sizeof(*cb) - CIRCLE_BUFFER_STUFF_SIZE;
    cb->allocator   = 0;
    cb->flags       = CIRCLE_BUFFER_FLAG_COB;
    return (circle_buffer_t)cb;
}

void circle_buffer_destroy(circle_buffer_t cbuf)
{
    assert(cbuf);
    struct circle_buffer* cb = (struct circle_buffer*)cbuf;
    if (!(cb->flags & CIRCLE_BUFFER_FLAG_COB)) {
        assert(cb->allocator);
        cb->allocator->free(cb->allocator, cb);
    }
}

unsigned int circle_buffer_flags(circle_buffer_t cbuf)
{
    assert(cbuf);
    return ((struct circle_buffer*)cbuf)->flags;
}

unsigned int circle_buffer_set_flags(circle_buffer_t cbuf, unsigned int flags)
{
    assert(cbuf);
    return ((struct circle_buffer*)cbuf)->flags = flags;
}

void circle_buffer_clear(circle_buffer_t cbuf);

static int __wait_for_read(struct circle_buffer* cbuf, size_t count)
{
    cbuf->length_needed = count;
    cbuf->
}

static int __read(struct circle_buffer* cbuf, void* buffer, size_t count)
{
    if (cbuf->length < count && cbuf->flags & CIRCLE_BUFFER_FLAG_BLOCKING)
        __wait_for_read(cbuf, count);
    size_t n = min(cbuf->length, count);
    size_t p1 = cbuf->size - cbuf->rpos;
    if (p1 < n) {
        memcpy(buffer, cbuf->buffer + cbuf->rpos, p1);
        memcpy(buffer + p1, cbuf->buffer, n - p1);
        cbuf->rpos = n - p1;
    } else {
        memcpy(buffer, cbuf->buffer + cbuf->rpos, n);
        cbuf->rpos += n;
    }
    cbuf->space += n;
    cbuf->length += n;
    return n;
}

static int __write(struct circle_buffer* cbuf, void* buffer, size_t count)
{
    return 0;
}

static int __read(struct circle_buffer* cbuf)
{
    __lock(cbuf);
    if (cbuf->length < count)
    __unlock(cbuf);
}

static int check_and_wait_for_write(struct circle_buffer* cbuf)
{
}


int circle_buffer_read(circle_buffer_t cbuf, void* buffer, size_t count)
{
    assert(cbuf);
    assert(buffer);
    struct circle_buffer* cb = (struct circle_buffer*)cbuf;
    if (cb->length < count & cb->flags & CIRCLE_BUFFER_FLAG_BLOCKING)
        __wait(cb);
    return 0;
}

int circle_buffer_write(circle_buffer_t cbuf, const void* buffer, size_t count)
{
    assert(cbuf);
    assert(buffer);
    int result = 0;
    struct circle_buffer* cb = (struct circle_buffer*)cbuf;
    if (cb->space < count & cb->flags & CIRCLE_BUFFER_FLAG_BLOCKING)
        __wait(cb);
    result = __write(cb, buffer, count);
    return result;
}

int circle_buffer_peek(circle_buffer_t cbuf, void* buffer, size_t count)
{
    assert(cbuf);
    assert(buffer);
    return 0;
}


size_t circle_buffer_size(circle_buffer_t cbuf)
{
    assert(cbuf);
    return ((struct circle_buffer*)cbuf)->size;
}

size_t circle_buffer_space(circle_buffer_t cbuf)
{
    assert(cbuf);
    return ((struct circle_buffer*)cbuf)->space;
}

size_t circle_buffer_length(circle_buffer_t cbuf)
{
    assert(cbuf);
    return ((struct circle_buffer*)cbuf)->length;
}

#endif

