/*
 * Copyright (C) 2012 Varphone Wong <varphone@qq.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public Licens
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-
 *
 */

#ifndef LIBGCONTAINERS_CIRCLE_BUFFER_H
#define LIBGCONTAINERS_CIRCLE_BUFFER_H

#ifndef CIRCLE_BUFFER_T_DEFINED
#define CIRCLE_BUFFER_T_DEFINED

#define CIRCLE_BUFFER_FLAG_COB          0x00000001      ///< Indicate the circle_buffer was create on preallocated buffer.
#define CIRCLE_BUFFER_FLAG_BLOCKING     0x00000002      ///< Indicate the circle_buffer will blocking when space not enough or no data in buffer.
#define CIRCLE_BUFFER_FLAG_THREADSAFE   0x00000004      ///< Threads safe.
#define NULL_CIRCLE_BUFFER		((circle_buffer_t)0)    ///< Null circle buffer.

enum circle_buffer_error_type {
    CIRCLE_BUFFER_EGENERIC = -1,
    CIRCLE_BUFFER_SUCCESS = 0,
};

typedef void* circle_buffer_t;

circle_buffer_t circle_buffer_create(size_t size, struct gallocator* allocator);
circle_buffer_t circle_buffer_create_on_buffer(void* buffer, size_t size);
void circle_buffer_destroy(circle_buffer_t cbuf);
#define circle_buffer_is_null(cbuf)		((cbuf) != NULL_CIRCLE_BUFFER)

unsigned int circle_buffer_flags(circle_buffer_t cbuf);
unsigned int circle_buffer_set_flags(circle_buffer_t cbuf, unsigned int flags);
#define circle_buffer_add_flags(cbuf, flags)    circle_buffer_set_flags(cbuf, circle_buffer_flags(cbuf) | (flags))
#define circle_buffer_clr_flags(cbuf, flags)    circle_buffer_set_flags(cbuf, circle_buffer_flags(cbuf) & (~flags))

void circle_buffer_clear(circle_buffer_t cbuf);

int circle_buffer_read(circle_buffer_t cbuf, void* buffer, size_t count);
int circle_buffer_write(circle_buffer_t cbuf, const void* buffer, size_t count);
int circle_buffer_peek(circle_buffer_t cbuf, void* buffer, size_t count);

size_t circle_buffer_size(circle_buffer_t cbuf);
size_t circle_buffer_space(circle_buffer_t cbuf);
size_t circle_buffer_length(circle_buffer_t cbuf);

#endif

#endif

