/*
 * Copyright (C) 2012 Varphone Wong <varphone@qq.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public Licens
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-
 *
 */

#ifndef LIBGCONTAINERS_GALLOCATOR_H
#define LIBGCONTAINERS_GALLOCATOR_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif

#ifndef GALLOCATOR_DEFINED
#define GALLOCATOR_DEFINED

// Forwards.
struct gallocator;

// Function types.
typedef void* (*gallocator_calloc_t)(struct gallocator* allocator, size_t nmemb, size_t size);
typedef void (*gallocator_free_t)(struct gallocator* allocator, void* ptr);
typedef void* (*gallocator_malloc_t)(struct gallocator* allocator, size_t size);
typedef void* (*gallocator_realloc_t)(struct gallocator* allocator, void* ptr, size_t size);
typedef void* (*gallocator_zalloc_t)(struct gallocator* allocator, size_t size);

/// Generic memory allocator.
struct gallocator {
    gallocator_calloc_t     calloc;     ///< Array style memory allocator.
    gallocator_free_t       free;       ///< Free allocated memory.
    gallocator_malloc_t     malloc;     ///< Generic memory allocator.
    gallocator_realloc_t    realloc;    ///< Change allocated memory size.
    gallocator_zalloc_t     zalloc;     ///< Allocate memory and fill zero.
    void*                   user;       ///< User data.
};

/// Initialize gallocator with default functions.
/// @param [in] allocator       Pointer pointed to gallocator.
/// @param [in] user            User data.
/// @return no value returns.
extern void gallocator_init(struct gallocator* allocator, void* user);

extern struct gallocator gallocator_default;    ///< Global default gallocator.

#endif

#endif
