/*
 * Copyright (C) 2012 Varphone Wong <varphone@qq.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public Licens
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "vector.h"

int test1_walker(void* elptr, unsigned int index, void* ctx)
{
    int* pv = elptr;
    printf("ptr: %p, value: %d\n", elptr, *pv);
    return VECTOR_SUCCESS;
}

void test1(void)
{
    vector_t vec = vector_create2(0, 4);
    int value = 1;
    for (int i = 0; i < 1000; i++)
        vector_push_back(vec, &value);
    vector_foreach(vec, test1_walker, NULL);
    vector_destroy(vec);
}

int test2_walker(void* elptr, unsigned int index, void* ctx)
{
    char* pv = elptr;
    printf("ptr: %p, value: %s\n", elptr, pv);
    return VECTOR_SUCCESS;
}

void test2(void)
{
    vector_t vec = vector_create2(0, 256);
    vector_set_option(vec, VECTOR_ELEMENT_COPY, (void*)strncpy);
    for (int i = 0; i < 10000; i++)
        vector_push_back(vec, "Hello, vector");
    vector_foreach(vec, test2_walker, NULL);
    printf("size: %d, capacity: %d\n", vector_size(vec), vector_capacity(vec));
    vector_add_flags(vec, VECTOR_FLAG_FASTCLEAR);
    vector_clear(vec);
    vector_destroy(vec);
}

void test3(void)
{
    char buffer[8192];
    vector_t vec = vector_create_on_buffer(20, buffer, sizeof(buffer));
    vector_set_option(vec, VECTOR_ELEMENT_COPY, (void*)strncpy);
    for (int i = 0; i < 10000; i++)
        vector_push_back(vec, "Hello, vector on buffer");
    vector_foreach(vec, test2_walker, NULL);
    printf("size: %d, capacity: %d\n", vector_size(vec), vector_capacity(vec));
    vector_add_flags(vec, VECTOR_FLAG_FASTCLEAR);
    vector_clear(vec);
    vector_destroy(vec);
}

int test4_walker(void* elptr, unsigned int index, void* ctx)
{
    char* pv = (char*)elptr;
    printf("ptr: %p, value: %s\n", elptr, pv);
    return VECTOR_SUCCESS;
}

void test4(void)
{
    vector_t vec = vector_from_string_list(NULL_VECTOR, "1920x1080,1280x720,720x576,720x480,704x576", NULL);
    vector_foreach(vec, test4_walker, NULL);
    printf("size: %d, capacity: %d\n", vector_size(vec), vector_capacity(vec));
    vector_clear(vec);
    vector_destroy(vec);
}

int main(int argc, char** argv)
{
    test1();
    test2();
    test3();
    test4();
    return 0;
}
