#include "StdAfx.h"
#include "ApiHooker.h"
#include "codecvt.hpp"

namespace vx {

#define return_if_null(p, v) if (!(p)) return (v)

static void FooForGetModuleHandle(void)
{
}

//=============================================================================
// ApiHooker
//=============================================================================
ApiHooker::ApiHooker(void)
    : _loadedModules()
    , _replacedFunctions()
    , _mappings()
{
}

ApiHooker::~ApiHooker(void)
{
    UnloadModules();
}

void ApiHooker::LoadModule(const std::wstring& mod)
{
    if (_loadedModules.count(mod) > 0)
        return;
    ModuleInfo mi = {mod, 0, false};
    if (mod == L"") {
        mi.handle = GetModuleHandle(NULL);
        mi.unload = false;
    } else {
        mi.handle = GetModuleHandle(mod.c_str());
        if (!mi.handle) {
            mi.handle = LoadLibraryW(mod.c_str());
            if (mi.handle) mi.unload = true;
        } else {
            mi.unload = true;
        }
    }
    if (!mi.handle) {
        std::wcerr << "[   Warning] " << ((mod == L"") ? L"base" : mod) << " unloaded." << std::endl;
    } else {
        _loadedModules[mod] = mi;
        std::wcerr << "[    Notice] " << ((mod == L"") ? L"base" : mod) << " @ " << mi.handle << " loaded." << std::endl;
    }
}

void ApiHooker::LoadModules(const vx::vector<std::wstring>& mods)
{
    std::for_each(mods.begin(), mods.end(),
                  std::bind(&ApiHooker::LoadModule, this, std::placeholders::_1));
}

void ApiHooker::UnloadModule(HMODULE mod)
{
    FreeLibrary(mod);
}

void ApiHooker::UnloadModules(void)
{
	std::map<std::wstring, ModuleInfo>::iterator itr = _loadedModules.begin();
	std::map<std::wstring, ModuleInfo>::iterator itr_end = _loadedModules.end();
    for (; itr != itr_end; ++itr) {
        if (itr->second.unload) UnloadModule(itr->second.handle);
    }
	_loadedModules.clear();
}

void* ApiHooker::OriginalAddressOf(void* target)
{
    return GetFunctionOriginalAddress(target);
}

IMAGE_IMPORT_DESCRIPTOR* ApiHooker::GetImageImportDescriptor(HMODULE mod, bool delayed)
{
	ULONG size = 0;
	IMAGE_IMPORT_DESCRIPTOR* iid = (IMAGE_IMPORT_DESCRIPTOR*)ImageDirectoryEntryToData(mod,  
			TRUE, delayed ? IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT : IMAGE_DIRECTORY_ENTRY_IMPORT, &size);
	return iid;
}

IMAGE_THUNK_DATA32* ApiHooker::FindImageThunkData32(HMODULE hmod, IMAGE_IMPORT_DESCRIPTOR* iid, const std::string& smod, bool delayed)
{
    if (delayed) {
        ImgDelayDescr* idd = (ImgDelayDescr*)iid;
        while (idd && idd->rvaIAT) {
            if (idd->grAttrs & 1) {
                IMAGE_THUNK_DATA32* result = (IMAGE_THUNK_DATA32*)((PBYTE)hmod + idd->rvaINT);
                std::string stmp = (char*)((PBYTE)hmod + idd->rvaDLLName);
                vx::codecvt::to_lower(stmp);
                std::cerr << "[    Notice] " << smod << " " << LookupModuleHandle(vx::codecvt::to_wide(smod)) << " " << " [ITD] @ "
                          << result << "." << std::endl;
                return result;
            } else {
                printf("fucnk\n");
            }
            idd++;
        }
    } else {
        while (iid && iid->Name) {
            IMAGE_THUNK_DATA32* result = (IMAGE_THUNK_DATA32*)((PBYTE)hmod + iid->FirstThunk);
            std::string stmp = (char*)((PBYTE)hmod + iid->Name);
            codecvt::to_lower(stmp);
            if (stmp == smod) {
                std::cerr << "[    Notice] " << smod << " [ITD] @ "
                          << result << "." << std::endl;
                return result;
            }/* else {
            std::wstring wtmp = vx::codecvt::to_wide(stmp);
            IMAGE_THUNK_DATA32* itd = GetImageThunkData32(wtmp);
            if (itd) return itd;
            }*/
            std::cerr << "Found: " << stmp << " for " << smod << std::endl;
            iid++;
        }
    }
    return 0;
}

IMAGE_THUNK_DATA32* ApiHooker::GetImageThunkData32(const std::wstring& mod)
{
    std::string smod = codecvt::to_narrow(mod);
    codecvt::to_lower(smod);
    HMODULE hmod = LookupModuleHandle(L"");//ModuleFromAddress((void*)&FooForGetModuleHandle);
    printf("hmod: %p, currnet: %p\n", hmod, GetModuleHandle(NULL));
    IMAGE_THUNK_DATA32* result = 0;
	IMAGE_IMPORT_DESCRIPTOR* iid = GetImageImportDescriptor(hmod, false);
    if (iid) result = FindImageThunkData32(hmod, iid, smod, false);
    if (!result) {
        iid = GetImageImportDescriptor(hmod, true);
        if (iid) result = FindImageThunkData32(hmod, iid, smod, true);
    }
    return result;
#if 0
    while (iid && iid->Name) {
        std::string stmp = (char*)((PBYTE)hmod + iid->Name);
        codecvt::to_lower(stmp);
        if (stmp == name) {
            std::wcerr << L"[    Notice] " << mod << L" [ITD] @ "
                       << (IMAGE_THUNK_DATA32*)((PBYTE)hmod + iid->FirstThunk)
                       << L"." << std::endl;
            return (IMAGE_THUNK_DATA32*)((PBYTE)hmod + iid->FirstThunk);
        }/* else {
            std::wstring wtmp = vx::codecvt::to_wide(stmp);
            IMAGE_THUNK_DATA32* itd = GetImageThunkData32(wtmp);
            if (itd) return itd;
        }*/
        std::cout << "Found: " << stmp << " for " << name << std::endl;
        iid++;
    }
    std::wcerr << L"[     Error] " << mod << L" [ITD] not found.\n";
	return 0;
#endif
}

void* ApiHooker::GetFunctionAddress(const std::wstring& mod, const std::wstring& name)
{
    HMODULE handle = LookupModuleHandle(mod);
    if (!handle) {
        std::wcerr << L"[     Error] " << mod << L" does not loaded." << std::endl;
        return 0;
    }
    std::string s = codecvt::to_narrow(name);
    return GetProcAddress(handle, s.c_str());
}

void* ApiHooker::GetFunctionAddressSlot(IMAGE_THUNK_DATA32* thunk, void* addr)
{
    if (!thunk || !addr)
        return 0;
    DWORD ordinal = 0;
    for( ; thunk->u1.Function; thunk++) {  
        if (thunk->u1.Function == (DWORD)addr)
            return &thunk->u1.Function;
        if (thunk->u1.Ordinal & IMAGE_ORDINAL_FLAG)
            ordinal = thunk->u1.Ordinal & (~IMAGE_ORDINAL_FLAG);
        else
            ordinal = thunk->u1.Function;
        IMAGE_IMPORT_BY_NAME* iibn = (IMAGE_IMPORT_BY_NAME*)((char*)LookupModuleHandle(L"") + ordinal);
        printf("Function: %s, target: %p\n", &iibn->Name, addr);
    }
    return 0;
}

void* ApiHooker::GetFunctionOriginalAddress(void* target)
{
    std::map<void*, void*>::iterator itr = _mappings.find(target);
    if (itr != _mappings.end())
        return itr->second;
    return 0;
}

HMODULE ApiHooker::LookupModuleHandle(const std::wstring& mod)
{
    std::map<std::wstring, ModuleInfo>::iterator itr = _loadedModules.find(mod);
    if (itr != _loadedModules.end())
        return itr->second.handle;
    return 0;
}

std::wstring ApiHooker::MakeKey(const std::wstring& mod, const std::wstring& name)
{
    return std::wstring(mod + L"@" + name);
}

HMODULE ApiHooker::ModuleFromAddress(LPVOID pv)
{
    MEMORY_BASIC_INFORMATION mbi = {0};
    return ((VirtualQuery(pv, &mbi, sizeof(mbi)) != 0) ? (HMODULE) mbi.AllocationBase : NULL);
}

bool ApiHooker::ReplaceFuntion(const std::wstring& mod, const std::wstring name, void* target)
{
    IMAGE_THUNK_DATA32* thunk = GetImageThunkData32(mod);
    return_if_null(thunk, false);
    void* original = GetFunctionAddress(mod, name);
    return_if_null(original, false);
    void* slot = GetFunctionAddressSlot(thunk, original);
    return_if_null(slot, false);
    DWORD prev = 0;
    // Switch memory protect mode to PAGE_EXECUTE_READWRITE.
    if (!VirtualProtect(slot, sizeof(void*), PAGE_EXECUTE_READWRITE, &prev)) {
        std::wcerr << L"[     Error] VirtualProtect() failure: " << GetLastError() << std::endl;
        return false;
    }
    // Save the function information.
    _replacedFunctions[MakeKey(mod, name)] = FunctionInfo(mod, name, slot, original, target);
    // Make target <-> original mappings.
    _mappings[target] = original;
    // Replace the API with specified target address.
    *(void**)slot = target;
    // Restore the memory protect mode.
    VirtualProtect(slot, sizeof(void*), prev, &prev);
    //
    std::wcerr << L"[    Notice] " << name << L" @ " << mod << L" hooked." << std::endl;
    return true;
}

bool ApiHooker::RestoreFunction(const std::wstring& mod, const std::wstring name)
{
    // Lookup the FunctionInfo.
    std::map<std::wstring, FunctionInfo>::iterator itr = _replacedFunctions.find(MakeKey(mod, name));
    if (itr == _replacedFunctions.end()) {
        std::wcerr << L"[    Warning] function [" << name << L"] does not hooked." << std::endl;
        return false;
    }
    // Check each fields in FunctionInfo.
    return_if_null(itr->second.slot, false);
    return_if_null(itr->second.original, false);
    return_if_null(itr->second.target, false);
    DWORD prev = 0;
    // Switch memory protect mode to PAGE_EXECUTE_READWRITE.
    if (!VirtualProtect(itr->second.slot, sizeof(void*), PAGE_EXECUTE_READWRITE, &prev)) {
        std::wcerr << L"[    Error] VirtualProtect() failure: " << GetLastError() << std::endl;
        return false;
    }
    // Restore the original address.
    *(void**)(itr->second.slot) = itr->second.original;
    // Restore the memory protect mode.
    VirtualProtect(itr->second.slot, sizeof(void*), prev, &prev);
    // Delete the target <-> original mappings.
    _mappings.erase(itr->second.target);
    // Delete the unsed function information.
    _replacedFunctions.erase(itr);
    //
    std::wcerr << L"[    Notice] " << name << L" @ " << mod << L" restored." << std::endl;
    return true;
}

void ApiHooker::RestoreAllFunctions(void)
{
    // Lookup the FunctionInfo.
    std::map<std::wstring, FunctionInfo>::iterator itr = _replacedFunctions.begin();
    std::map<std::wstring, FunctionInfo>::iterator itr_end = _replacedFunctions.end();
    for ( ; itr != itr_end; ) {
        // Check each fields in FunctionInfo.
        if (itr->second.slot &&
            itr->second.original &&
            itr->second.target) {
            DWORD prev = 0;
            // Switch memory protect mode to PAGE_EXECUTE_READWRITE.
            if (!VirtualProtect(itr->second.slot, sizeof(void*), PAGE_EXECUTE_READWRITE, &prev)) {
                std::wcerr << L"[    Warning] VirtualProtect() failure: " << GetLastError() << std::endl;
            } else {
                // Restore the original address.
                *(void**)(itr->second.slot) = itr->second.original;
                // Restore the memory protect mode.
                VirtualProtect(itr->second.slot, sizeof(void*), prev, &prev);
                std::wcerr << L"[    Notice] " << itr->second.name << L" @ " << itr->second.mod << L" restored." << std::endl;
            }
        }
        // Delete the target <-> original mappings.
        _mappings.erase(itr->second.target);
        // Delete the unsed function information.
        _replacedFunctions.erase(itr++);
    }
}

}