#pragma once

namespace vx {

class ApiHooker : public singleton<ApiHooker>
{
    struct FunctionInfo
    {
        std::wstring mod;
        std::wstring name;
        void* slot;
        void* original;
        void* target;
    
        FunctionInfo(void)
            : mod(L"")
            , name(L"")
            , slot(0)
            , original(0)
            , target(0)
        {
        }

        FunctionInfo(const std::wstring _mod, const std::wstring _name, void* _slot, void* _original, void* _target)
            : mod(_mod)
            , name(_name)
            , slot(_slot)
            , original(_original)
            , target(_target)
        {
        }
    };

    struct ModuleInfo
    {
        std::wstring name;
        HMODULE handle;
        bool unload;
    };

public:
    ApiHooker(void);
    ~ApiHooker(void);

public:
    void LoadModule(const std::wstring& mod);
    void LoadModules(const vx::vector<std::wstring>& mods);
    void UnloadModule(HMODULE mod);
    void UnloadModules(void);

    bool ReplaceFuntion(const std::wstring& mod, const std::wstring name, void* target);
    bool RestoreFunction(const std::wstring& mod, const std::wstring name);
    void RestoreAllFunctions(void);

    void* OriginalAddressOf(void* target);

private:
    IMAGE_THUNK_DATA32* FindImageThunkData32(HMODULE hmod,
            IMAGE_IMPORT_DESCRIPTOR* iid, const std::string& smod, bool delayed);
	IMAGE_IMPORT_DESCRIPTOR* GetImageImportDescriptor(HMODULE mod, bool delayed);
	IMAGE_THUNK_DATA32* GetImageThunkData32(const std::wstring& mod);
    void* GetFunctionAddress(const std::wstring& mod, const std::wstring& name);
    void* GetFunctionAddressSlot(IMAGE_THUNK_DATA32* thunk, void* addr);
    void* GetFunctionOriginalAddress(void* target);
	HMODULE LookupModuleHandle(const std::wstring& mod);
    std::wstring MakeKey(const std::wstring& mod, const std::wstring& name);
    HMODULE ModuleFromAddress(LPVOID pv);

private:
    std::map<std::wstring, ModuleInfo>		_loadedModules;
	std::map<std::wstring, FunctionInfo>    _replacedFunctions;
    std::map<void*, void*>                  _mappings;
};

}