#pragma once

//=============================================================================
// C Memory manager routine proto
//=============================================================================
typedef void* (*generic_calloc_t)(size_t, size_t);
typedef void* (*generic_malloc_t)(size_t);
typedef void* (*generic_realloc_t)(void*, size_t);
typedef void (*generic_free_t)(size_t);

// MSVCRTD/MSVCR*D
typedef void* (*generic__calloc_dbg_t)(size_t, size_t, int, const char*, int);
typedef void* (*generic__malloc_dbg_t)(size_t, int, const char*, int);
typedef void* (*generic__realloc_dbg_t)(void*, size_t, int, const char*, int);
typedef void (*generic__free_dbg_t)(void*, int);
 
//=============================================================================
// C++ Memory manager routine proto
//=============================================================================
typedef void* (*generic_operator_new_t)(size_t);
typedef void (*generic_operator_delete_t)(size_t);

//=============================================================================
// Windows Memory manager routine proto
//=============================================================================
typedef LPVOID (WINAPI *generic_HeapAlloc_t)(HANDLE, DWORD, SIZE_T);
typedef BOOL (WINAPI *generic_HeapFree_t)(HANDLE, DWORD, LPVOID);
typedef LPVOID (WINAPI *generic_VirtualAlloc_t)(LPVOID, SIZE_T, DWORD, DWORD);
typedef LPVOID (WINAPI *generic_VirtualAllocEx_t)(HANDLE, LPVOID, SIZE_T, DWORD, DWORD);
typedef LPVOID (WINAPI *generic_VirtualFree_t)(LPVOID, SIZE_T, DWORD);
typedef LPVOID (WINAPI *generic_VirtualFreeEx_t)(HANDLE, LPVOID, SIZE_T, DWORD);

namespace vx
{

class MemoryManager : public singleton<MemoryManager>
{
public:
    MemoryManager(void);
    ~MemoryManager(void);

public:
    enum {
        kAligned = 64 * 1024,
        kMinSize = 256 * 1024,
        kMaxSize = 16 * 1024 * 1024
    };

    void* calloc(size_t, size_t, generic_calloc_t);
    void* malloc(size_t, generic_malloc_t);
    void* realloc(void*, size_t, generic_realloc_t);
    void free(void*, generic_free_t);

};

};