#pragma once

#define BEA_ENGINE_STATIC  /* specify the usage of a static version of BeaEngine */
#define BEA_USE_STDCALL    /* specify the usage of a stdcall version of BeaEngine */
extern "C" {
#include <BeaEngine.h>
}

namespace vx
{

#define __asm_detour_reserved __asm \
{ \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
    __asm nop \
}

#pragma pack(1)
union instruction
{
    struct {
        unsigned char code;
        unsigned int offset;
    } jmp;
};
#pragma pack()

class virutal_protect_remover
{
public:
    virutal_protect_remover(void* address, size_t size)
        : _address(address)
        , _size(size)
        , _protect_flags(0)
    {
        VirtualProtect(_address, _size, PAGE_EXECUTE_READWRITE, &_protect_flags);
    }

    ~virutal_protect_remover(void)
    {
        VirtualProtect(_address, _size, _protect_flags, &_protect_flags);
    }

private:
    void*       _address;
    size_t      _size;
    DWORD       _protect_flags;
};

class detour
{
public:

    detour(void)
        : _module(L"")
        , _module_handle(0)
        , _module_loaded(false)
        , _symbol("")
        , _original(0)
        , _replacement(0)
    {
        //replace();
    }

    detour(const std::wstring& module, const std::string& symbol, void* replacement)
        : _module(module)
        , _module_handle(0)
        , _module_loaded(false)
        , _symbol(symbol)
        , _original(0)
        , _replacement(replacement)
    {
        replace();
    }

    virtual ~detour(void)
    {
        recover();
    }

private:
    bool backup_opcodes(int size)
    {
        _backup_length = detect_required_length(_original, size);
        if (_backup_length < 0)
            return false;
        memcpy((char*)_replacement+16, _original, _backup_length);
        unsigned int from = (unsigned int)_replacement + 16 + _backup_length;
        unsigned int to = (unsigned int)_original + _backup_length;
        instruction* instr = (instruction*)((char*)_replacement+16+_backup_length);
        instr->jmp.code = 0xE9;
        instr->jmp.offset = from - to - 5;
        return true;
    }

    int detect_required_length(void* address, int size)
    {
        DISASM  dia = {0};
        dia.EIP = (UIntPtr)address;
        int result = 0;
        while (result < size) {
            int n = Disasm(&dia);
            if (n == UNKNOWN_OPCODE)
                return -1;
            dia.EIP = dia.EIP + (UIntPtr)n;
            result += n;
        }
        return result;
    }

    void replace(void)
    {
        _module_handle = GetModuleHandle(_module.c_str());
        if (!_module_handle) {
            _module_handle = LoadLibrary(_module.c_str());
            if (_module_handle) _module_loaded = true;
        }
        if (_module_handle)
            _original = (void*)GetProcAddress(_module_handle, _symbol.c_str());
        std::cout << __FILE__ << __LINE__ << std::endl;
        if (_original) {
            virutal_protect_remover vpr(_original, 16);
            if (backup_opcodes(5)) {
                instruction* instr = (instruction*)_original;
                instr->jmp.code = 0xE9;
                instr->jmp.offset = (unsigned int)_replacement - (unsigned int)_original - 5;
                printf("original: %p, %p\n", _original, instr->jmp.offset);
            }
        }
        std::cout << __FILE__ << __LINE__ << std::endl;
    }

    void recover(void)
    {
        if (_module_loaded) {
            FreeLibrary(_module_handle);
            _module_handle = 0;
            _module_loaded = false;
        }
    }

private:
    std::wstring    _module;
    HMODULE         _module_handle;
    bool            _module_loaded;
    std::string     _symbol;
    void*           _original;
    void*           _replacement;
    unsigned char   _backup_opcodes[32];
    int             _backup_length;
    instruction     _original_instr;
    instruction     _replacement_instr;
};

} //namespace vx