/*
 * cpu_op_table.h
 *	This table is used in cpu.c to map opcodes and paramters to functions
 *	that performing the instruction
 * 
 * (c)2001 Mattias Wadman
 */

#ifndef __CPU_OP_TABLE_H__
#define __CPU_OP_TABLE_H__

#include "cpu.h"
#include "cpu_op.h"

struct cpu_op cpu_op_table[256] =
{
	{cpu_op_nop,               0,	0,	0,	0,	1},	/*0x00*/
	{cpu_op_ld_reg16_imm16,    0,	REG_BC,	2,	0,	3},	/*0x01*/
	{cpu_op_ld_indreg16_reg,   0,	REG_BC,	0,	REG_A,	2},	/*0x02*/
	{cpu_op_inc_reg16,         0,	REG_BC,	0,	0,	2},	/*0x03*/
	{cpu_op_inc_reg,           0,	REG_B,	0,	0,	1},	/*0x04*/
	{cpu_op_dec_reg,           0,	REG_B,	0,	0,	1},	/*0x05*/
	{cpu_op_ld_reg_imm,        0,	REG_B,	1,	0,	2},	/*0x06*/
	{cpu_op_rlca,              0,	0,	0,	0,	1},	/*0x07*/
	{cpu_op_ld_indimm16_reg16, 2,	0,	0,	REG_SP,	5},	/*0x08*/
	{cpu_op_add_reg16_reg16,   0,	REG_HL,	0,	REG_BC,	2},	/*0x09*/
	{cpu_op_ld_reg_indreg16,   0,	REG_A,	0,	REG_BC,	2},	/*0x0a*/
	{cpu_op_dec_reg16,         0,	REG_BC,	0,	0,	2},	/*0x0b*/
	{cpu_op_inc_reg,           0,	REG_C,	0,	0,	1},	/*0x0c*/
	{cpu_op_dec_reg,           0,	REG_C,	0,	0,	1},	/*0x0d*/
	{cpu_op_ld_reg_imm,        0,	REG_C,	1,	0,	2},	/*0x0e*/
	{cpu_op_rrca,              0,	0,	0,	0,	1},	/*0x0f*/
	{cpu_op_stop,              1,	0,	0,	0,	1},	/*0x10*/
	{cpu_op_ld_reg16_imm16,    0,	REG_DE,	2,	0,	3},	/*0x11*/
	{cpu_op_ld_indreg16_reg,   0,	REG_DE,	0,	REG_A,	2},	/*0x12*/
	{cpu_op_inc_reg16,         0,	REG_DE,	0,	0,	2},	/*0x13*/
	{cpu_op_inc_reg,           0,	REG_D,	0,	0,	1},	/*0x14*/
	{cpu_op_dec_reg,           0,	REG_D,	0,	0,	1},	/*0x15*/
	{cpu_op_ld_reg_imm,        0,	REG_D,	1,	0,	2},	/*0x16*/
	{cpu_op_rla,               0,	0,	0,	0,	1},	/*0x17*/
	{cpu_op_jr_imm,            1,	0,	0,	0,	2},	/*0x18*/
	{cpu_op_add_reg16_reg16,   0,	REG_HL,	0,	REG_DE,	2},	/*0x19*/
	{cpu_op_ld_reg_indreg16,   0,	REG_A,	0,	REG_DE,	2},	/*0x1a*/
	{cpu_op_dec_reg16,         0,	REG_DE,	0,	0,	2},	/*0x1b*/
	{cpu_op_inc_reg,           0,	REG_E,	0,	0,	1},	/*0x1c*/
	{cpu_op_dec_reg,           0,	REG_E,	0,	0,	1},	/*0x1d*/
	{cpu_op_ld_reg_imm,        0,	REG_E,	1,	0,	2},	/*0x1e*/
	{cpu_op_rra,               0,	0,	0,	0,	1},	/*0x1f*/
	{cpu_op_jr_cond_imm,       0,	COND_NZ,1,	0,	2},	/*0x20*/
	{cpu_op_ld_reg16_imm16,    0,	REG_HL,	2,	0,	3},	/*0x21*/
	{cpu_op_ldi_indreg16_reg,  0,	REG_HL,	0,	REG_A,	2},	/*0x22*/
	{cpu_op_inc_reg16,         0,	REG_HL,	0,	0,	2},	/*0x23*/
	{cpu_op_inc_reg,           0,	REG_H,	0,	0,	1},	/*0x24*/
	{cpu_op_dec_reg,           0,	REG_H,	0,	0,	1},	/*0x25*/
	{cpu_op_ld_reg_imm,        0,	REG_H,	1,	0,	2},	/*0x26*/
	{cpu_op_daa,               0,	0,	0,	0,	1},	/*0x27*/
	{cpu_op_jr_cond_imm,       0,	COND_Z,	1,	0,	2},	/*0x28*/
	{cpu_op_add_reg16_reg16,   0,	REG_HL,	0,	REG_HL,	2},	/*0x29*/
	{cpu_op_ldi_reg_indreg16,  0,	REG_A,	0,	REG_HL,	2},	/*0x2a*/
	{cpu_op_dec_reg16,         0,	REG_HL,	0,	0,	2},	/*0x2b*/
	{cpu_op_inc_reg,           0,	REG_L,	0,	0,	1},	/*0x2c*/
	{cpu_op_dec_reg,           0,	REG_L,	0,	0,	1},	/*0x2d*/
	{cpu_op_ld_reg_imm,        0,	REG_L,	1,	0,	2},	/*0x2e*/
	{cpu_op_cpl,               0,	0,	0,	0,	1},	/*0x2f*/
	{cpu_op_jr_cond_imm,       0,	COND_NC,1,	0,	2},	/*0x30*/
	{cpu_op_ld_reg16_imm16,    0,	REG_SP,	2,	0,	3},	/*0x31*/
	{cpu_op_ldd_indreg16_reg,  0,	REG_HL,	0,	REG_A,	2},	/*0x32*/
	{cpu_op_inc_reg16,         0,	REG_SP,	0,	0,	2},	/*0x33*/
	{cpu_op_inc_indreg16,      0,	REG_HL,	0,	0,	3},	/*0x34*/
	{cpu_op_dec_indreg16,      0,	REG_HL,	0,	0,	3},	/*0x35*/
	{cpu_op_ld_indreg16_imm,   0,	REG_HL,	1,	0,	3},	/*0x36*/
	{cpu_op_scf,               0,	0,	0,	0,	1},	/*0x37*/
	{cpu_op_jr_cond_imm,       0,	COND_C,	1,	0,	2},	/*0x38*/
	{cpu_op_add_reg16_reg16,   0,	REG_HL,	0,	REG_SP,	2},	/*0x39*/
	{cpu_op_ldd_reg_indreg16,  0,	REG_A,	0,	REG_HL,	2},	/*0x3a*/
	{cpu_op_dec_reg16,         0,	REG_SP,	0,	0,	2},	/*0x3b*/
	{cpu_op_inc_reg,           0,	REG_A,	0,	0,	1},	/*0x3c*/
	{cpu_op_dec_reg,           0,	REG_A,	0,	0,	1},	/*0x3d*/
	{cpu_op_ld_reg_imm,        0,	REG_A,	1,	0,	2},	/*0x3e*/
	{cpu_op_ccf,               0,	0,	0,	0,	1},	/*0x3f*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_B,	1},	/*0x40*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_C,	1},	/*0x41*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_D,	1},	/*0x42*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_E,	1},	/*0x43*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_H,	1},	/*0x44*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_L,	1},	/*0x45*/
	{cpu_op_ld_reg_indreg16,   0,	REG_B,	0,	REG_HL,	2},	/*0x46*/
	{cpu_op_ld_reg_reg,        0,	REG_B,	0,	REG_A,	1},	/*0x47*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_B,	1},	/*0x48*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_C,	1},	/*0x49*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_D,	1},	/*0x4a*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_E,	1},	/*0x4b*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_H,	1},	/*0x4c*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_L,	1},	/*0x4d*/
	{cpu_op_ld_reg_indreg16,   0,	REG_C,	0,	REG_HL,	2},	/*0x4e*/
	{cpu_op_ld_reg_reg,        0,	REG_C,	0,	REG_A,	1},	/*0x4f*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_B,	1},	/*0x50*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_C,	1},	/*0x51*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_D,	1},	/*0x52*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_E,	1},	/*0x53*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_H,	1},	/*0x54*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_L,	1},	/*0x55*/
	{cpu_op_ld_reg_indreg16,   0,	REG_D,	0,	REG_HL,	2},	/*0x56*/
	{cpu_op_ld_reg_reg,        0,	REG_D,	0,	REG_A,	1},	/*0x57*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_B,	1},	/*0x58*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_C,	1},	/*0x59*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_D,	1},	/*0x5a*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_E,	1},	/*0x5b*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_H,	1},	/*0x5c*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_L,	1},	/*0x5d*/
	{cpu_op_ld_reg_indreg16,   0,	REG_E,	0,	REG_HL,	2},	/*0x5e*/
	{cpu_op_ld_reg_reg,        0,	REG_E,	0,	REG_A,	1},	/*0x5f*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_B,	1},	/*0x60*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_C,	1},	/*0x61*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_D,	1},	/*0x62*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_E,	1},	/*0x63*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_H,	1},	/*0x64*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_L,	1},	/*0x65*/
	{cpu_op_ld_reg_indreg16,   0,	REG_H,	0,	REG_HL,	2},	/*0x66*/
	{cpu_op_ld_reg_reg,        0,	REG_H,	0,	REG_A,	1},	/*0x67*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_B,	1},	/*0x68*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_C,	1},	/*0x69*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_D,	1},	/*0x6a*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_E,	1},	/*0x6b*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_H,	1},	/*0x6c*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_L,	1},	/*0x6d*/
	{cpu_op_ld_reg_indreg16,   0,	REG_L,	0,	REG_HL,	2},	/*0x6e*/
	{cpu_op_ld_reg_reg,        0,	REG_L,	0,	REG_A,	1},	/*0x6f*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_B,	2},	/*0x70*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_C,	2},	/*0x71*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_D,	2},	/*0x72*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_E,	2},	/*0x73*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_H,	2},	/*0x74*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_L,	2},	/*0x75*/
	{cpu_op_halt,              0,	0,	0,	0,	1},	/*0x76*/
	{cpu_op_ld_indreg16_reg,   0,	REG_HL,	0,	REG_A,	2},	/*0x77*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_B,	1},	/*0x78*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_C,	1},	/*0x79*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_D,	1},	/*0x7a*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_E,	1},	/*0x7b*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_H,	1},	/*0x7c*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_L,	1},	/*0x7d*/
	{cpu_op_ld_reg_indreg16,   0,	REG_A,	0,	REG_HL,	2},	/*0x7e*/
	{cpu_op_ld_reg_reg,        0,	REG_A,	0,	REG_A,	1},	/*0x7f*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_B,	1},	/*0x80*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_C,	1},	/*0x81*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_D,	1},	/*0x82*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_E,	1},	/*0x83*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_H,	1},	/*0x84*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_L,	1},	/*0x85*/
	{cpu_op_add_reg_indreg16,  0,	REG_A,	0,	REG_HL,	2},	/*0x86*/
	{cpu_op_add_reg_reg,       0,	REG_A,	0,	REG_A,	1},	/*0x87*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_B,	1},	/*0x88*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_C,	1},	/*0x89*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_D,	1},	/*0x8a*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_E,	1},	/*0x8b*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_H,	1},	/*0x8c*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_L,	1},	/*0x8d*/
	{cpu_op_adc_reg_indreg16,  0,	REG_A,	0,	REG_HL,	2},	/*0x8e*/
	{cpu_op_adc_reg_reg,       0,	REG_A,	0,	REG_A,	1},	/*0x8f*/
	{cpu_op_sub_reg,           0,	REG_B,	0,	0,	1},	/*0x90*/
	{cpu_op_sub_reg,           0,	REG_C,	0,	0,	1},	/*0x91*/
	{cpu_op_sub_reg,           0,	REG_D,	0,	0,	1},	/*0x92*/
	{cpu_op_sub_reg,           0,	REG_E,	0,	0,	1},	/*0x93*/
	{cpu_op_sub_reg,           0,	REG_H,	0,	0,	1},	/*0x94*/
	{cpu_op_sub_reg,           0,	REG_L,	0,	0,	1},	/*0x95*/
	{cpu_op_sub_indreg16,      0,	REG_HL,	0,	0,	2},	/*0x96*/
	{cpu_op_sub_reg,           0,	REG_A,	0,	0,	1},	/*0x97*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_B,	1},	/*0x98*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_C,	1},	/*0x99*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_D,	1},	/*0x9a*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_E,	1},	/*0x9b*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_H,	1},	/*0x9c*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_L,	1},	/*0x9d*/
	{cpu_op_sbc_reg_indreg16,  0,	REG_A,	0,	REG_HL,	2},	/*0x9e*/
	{cpu_op_sbc_reg_reg,       0,	REG_A,	0,	REG_A,	1},	/*0x9f*/
	{cpu_op_and_reg,           0,	REG_B,	0,	0,	1},	/*0xa0*/
	{cpu_op_and_reg,           0,	REG_C,	0,	0,	1},	/*0xa1*/
	{cpu_op_and_reg,           0,	REG_D,	0,	0,	1},	/*0xa2*/
	{cpu_op_and_reg,           0,	REG_E,	0,	0,	1},	/*0xa3*/
	{cpu_op_and_reg,           0,	REG_H,	0,	0,	1},	/*0xa4*/
	{cpu_op_and_reg,           0,	REG_L,	0,	0,	1},	/*0xa5*/
	{cpu_op_and_indreg16,      0,	REG_HL,	0,	0,	2},	/*0xa6*/
	{cpu_op_and_reg,           0,	REG_A,	0,	0,	1},	/*0xa7*/
	{cpu_op_xor_reg,           0,	REG_B,	0,	0,	1},	/*0xa8*/
	{cpu_op_xor_reg,           0,	REG_C,	0,	0,	1},	/*0xa9*/
	{cpu_op_xor_reg,           0,	REG_D,	0,	0,	1},	/*0xaa*/
	{cpu_op_xor_reg,           0,	REG_E,	0,	0,	1},	/*0xab*/
	{cpu_op_xor_reg,           0,	REG_H,	0,	0,	1},	/*0xac*/
	{cpu_op_xor_reg,           0,	REG_L,	0,	0,	1},	/*0xad*/
	{cpu_op_xor_indreg16,      0,	REG_HL,	0,	0,	2},	/*0xae*/
	{cpu_op_xor_reg,           0,	REG_A,	0,	0,	1},	/*0xaf*/
	{cpu_op_or_reg,            0,	REG_B,	0,	0,	1},	/*0xb0*/
	{cpu_op_or_reg,            0,	REG_C,	0,	0,	1},	/*0xb1*/
	{cpu_op_or_reg,            0,	REG_D,	0,	0,	1},	/*0xb2*/
	{cpu_op_or_reg,            0,	REG_E,	0,	0,	1},	/*0xb3*/
	{cpu_op_or_reg,            0,	REG_H,	0,	0,	1},	/*0xb4*/
	{cpu_op_or_reg,            0,	REG_L,	0,	0,	1},	/*0xb5*/
	{cpu_op_or_indreg16,       0,	REG_HL,	0,	0,	2},	/*0xb6*/
	{cpu_op_or_reg,            0,	REG_A,	0,	0,	1},	/*0xb7*/
	{cpu_op_cp_reg,            0,	REG_B,	0,	0,	1},	/*0xb8*/
	{cpu_op_cp_reg,            0,	REG_C,	0,	0,	1},	/*0xb9*/
	{cpu_op_cp_reg,            0,	REG_D,	0,	0,	1},	/*0xba*/
	{cpu_op_cp_reg,            0,	REG_E,	0,	0,	1},	/*0xbb*/
	{cpu_op_cp_reg,            0,	REG_H,	0,	0,	1},	/*0xbc*/
	{cpu_op_cp_reg,            0,	REG_L,	0,	0,	1},	/*0xbd*/
	{cpu_op_cp_indreg16,       0,	REG_HL,	0,	0,	2},	/*0xbe*/
	{cpu_op_cp_reg,            0,	REG_A,	0,	0,	1},	/*0xbf*/
	{cpu_op_ret_cond,          0,	COND_NZ,0,	0,	2},	/*0xc0*/
	{cpu_op_pop_reg16,         0,	REG_BC,	0,	0,	3},	/*0xc1*/
	{cpu_op_jp_cond_imm16,     0,	COND_NZ,2,	0,	3},	/*0xc2*/
	{cpu_op_jp_imm16,          2,	0,	0,	0,	4},	/*0xc3*/
	{cpu_op_call_cond_imm16,   0,	COND_NZ,2,	0,	3},	/*0xc4*/
	{cpu_op_push_reg16,        0,	REG_BC,	0,	0,	4},	/*0xc5*/
	{cpu_op_add_reg_imm,       0,	REG_A,	1,	0,	2},	/*0xc6*/
	{cpu_op_rst,               0,	0x00,	0,	0,	4},	/*0xc7*/
	{cpu_op_ret_cond,          0,	COND_Z,	0,	0,	2},	/*0xc8*/
	{cpu_op_ret,               0,	0,	0,	0,	4},	/*0xc9*/
	{cpu_op_jp_cond_imm16,     0,	COND_Z,	2,	0,	3},	/*0xca*/
	{cpu_op_cb,                0,	0,	0,	0,	0},	/*0xcb*/ /* INTERNAL */
	{cpu_op_call_cond_imm16,   0,	COND_Z,	2,	0,	3},	/*0xcc*/
	{cpu_op_call_imm16,        2,	0,	0,	0,	6},	/*0xcd*/
	{cpu_op_adc_reg_imm,       0,	REG_A,	1,	0,	2},	/*0xce*/
	{cpu_op_rst,               0,	0x08,	0,	0,	4},	/*0xcf*/
	{cpu_op_ret_cond,          0,	COND_NC,0,	0,	2},	/*0xd0*/
	{cpu_op_pop_reg16,         0,	REG_DE,	0,	0,	3},	/*0xd1*/
	{cpu_op_jp_cond_imm16,     0,	COND_NC,2,	0,	3},	/*0xd2*/
	{cpu_op_unknown,           0,	0xd3,	0,	0,	0},	/*0xd3*/ /* UNKNOWN */
	{cpu_op_call_cond_imm16,   0,	COND_NC,2,	0,	3},	/*0xd4*/
	{cpu_op_push_reg16,        0,	REG_DE,	0,	0,	4},	/*0xd5*/
	{cpu_op_sub_imm,           1,	0,	0,	0,	2},	/*0xd6*/
	{cpu_op_rst,               0,	0x10,	0,	0,	4},	/*0xd7*/
	{cpu_op_ret_cond,          0,	COND_C,	0,	0,	2},	/*0xd8*/
	{cpu_op_reti,              0,	0,	0,	0,	4},	/*0xd9*/
	{cpu_op_jp_cond_imm16,     0,	COND_C,	2,	0,	3},	/*0xda*/
	{cpu_op_unknown,           0,	0xdb,	0,	0,	0},	/*0xdb*/ /* UNKNOWN */
	{cpu_op_call_cond_imm16,   0,	COND_C,	2,	0,	3},	/*0xdc*/
	{cpu_op_unknown,           0,	0xdd,	0,	0,	0},	/*0xdd*/ /* UNKNOWN */
	{cpu_op_sbc_reg_imm,       0,	REG_A,	1,	0,	2},	/*0xde*/
	{cpu_op_rst,               0,	0x18,	0,	0,	4},	/*0xdf*/
	{cpu_op_ldh_indimm_reg,    1,	0,	0,	REG_A,	3},	/*0xe0*/
	{cpu_op_pop_reg16,         0,	REG_HL,	0,	0,	3},	/*0xe1*/
	{cpu_op_ldh_indreg_reg,    0,	REG_C,	0,	REG_A,	2},	/*0xe2*/
	{cpu_op_unknown,           0,	0xe3,	0,	0,	0},	/*0xe3*/ /* UNKNOWN */
	{cpu_op_unknown,           0,	0xe4,	0,	0,	0},	/*0xe4*/ /* UNKNOWN */
	{cpu_op_push_reg16,        0,	REG_HL,	0,	0,	4},	/*0xe5*/
	{cpu_op_and_imm,           1,	0,	0,	0,	2},	/*0xe6*/
	{cpu_op_rst,               0,	0x20,	0,	0,	4},	/*0xe7*/
	{cpu_op_add_reg16_imm,     0,	REG_SP,	1,	0,	4},	/*0xe8*/
	{cpu_op_jp_reg16,          0,	REG_HL,	0,	0,	1},	/*0xe9*/
	{cpu_op_ld_indimm16_reg,   2,	0,	0,	REG_A,	4},	/*0xea*/
	{cpu_op_unknown,           0,	0xeb,	0,	0,	0},	/*0xeb*/ /* UNKNOWN */
	{cpu_op_unknown,           0,	0xec,	0,	0,	0},	/*0xec*/ /* UNKNOWN */
	{cpu_op_unknown,           0,	0xed,	0,	0,	0},	/*0xed*/ /* UNKNOWN */
	{cpu_op_xor_imm,           1,	0,	0,	0,	2},	/*0xee*/
	{cpu_op_rst,               0,	0x28,	0,	0,	4},	/*0xef*/
	{cpu_op_ldh_reg_indimm,    0,	REG_A,	1,	0,	3},	/*0xf0*/
	{cpu_op_pop_reg16,         0,	REG_AF,	0,	0,	3},	/*0xf1*/
	{cpu_op_ldh_reg_indreg,    0,	REG_A,	0,	REG_C,	2},	/*0xf2*/
	{cpu_op_di,                0,	0,	0,	0,	1},	/*0xf3*/
	{cpu_op_unknown,           0,	0xf4,	0,	0,	0},	/*0xf4*/ /* UNKNOWN */
	{cpu_op_push_reg16,        0,	REG_AF,	0,	0,	4},	/*0xf5*/
	{cpu_op_or_imm,            1,	0,	0,	0,	2},	/*0xf6*/
	{cpu_op_rst,               0,	0x30,	0,	0,	4},	/*0xf7*/
	{cpu_op_ldhl_reg16_imm,    0,	REG_SP,	1,	0,	3},	/*0xf8*/
	{cpu_op_ld_reg16_reg16,    0,	REG_SP,	0,	REG_HL,	2},	/*0xf9*/
	{cpu_op_ld_reg_indimm16,   0,	REG_A,	2,	0,	4},	/*0xfa*/
	{cpu_op_ei,                0,	0,	0,	0,	1},	/*0xfb*/
	{cpu_op_unknown,           0,	0xfc,	0,	0,	0},	/*0xfc*/ /* UNKNOWN */
	{cpu_op_unknown,           0,	0xfd,	0,	0,	0},	/*0xfd*/ /* UNKNOWN */
	{cpu_op_cp_imm,            1,	0,	0,	0,	2},	/*0xfe*/
	{cpu_op_rst,               0,	0x38,	0,	0,	4} 	/*0xff*/
	/* Number of lines: 256 */
};

// 0xcb prefixed opcodes

struct cpu_op cpu_op_table_cb[256] =
{
	{cpu_op_rlc_reg,           0,	REG_B,	0,	0,	2},	/*0x00*/
	{cpu_op_rlc_reg,           0,	REG_C,	0,	0,	2},	/*0x01*/
	{cpu_op_rlc_reg,           0,	REG_D,	0,	0,	2},	/*0x02*/
	{cpu_op_rlc_reg,           0,	REG_E,	0,	0,	2},	/*0x03*/
	{cpu_op_rlc_reg,           0,	REG_H,	0,	0,	2},	/*0x04*/
	{cpu_op_rlc_reg,           0,	REG_L,	0,	0,	2},	/*0x05*/
	{cpu_op_rlc_indreg16,      0,	REG_HL,	0,	0,	4},	/*0x06*/
	{cpu_op_rlc_reg,           0,	REG_A,	0,	0,	2},	/*0x07*/
	{cpu_op_rrc_reg,           0,	REG_B,	0,	0,	2},	/*0x08*/
	{cpu_op_rrc_reg,           0,	REG_C,	0,	0,	2},	/*0x09*/
	{cpu_op_rrc_reg,           0,	REG_D,	0,	0,	2},	/*0x0a*/
	{cpu_op_rrc_reg,           0,	REG_E,	0,	0,	2},	/*0x0b*/
	{cpu_op_rrc_reg,           0,	REG_H,	0,	0,	2},	/*0x0c*/
	{cpu_op_rrc_reg,           0,	REG_L,	0,	0,	2},	/*0x0d*/
	{cpu_op_rrc_indreg16,      0,	REG_HL,	0,	0,	4},	/*0x0e*/
	{cpu_op_rrc_reg,           0,	REG_A,	0,	0,	2},	/*0x0f*/
	{cpu_op_rl_reg,            0,	REG_B,	0,	0,	2},	/*0x10*/
	{cpu_op_rl_reg,            0,	REG_C,	0,	0,	2},	/*0x11*/
	{cpu_op_rl_reg,            0,	REG_D,	0,	0,	2},	/*0x12*/
	{cpu_op_rl_reg,            0,	REG_E,	0,	0,	2},	/*0x13*/
	{cpu_op_rl_reg,            0,	REG_H,	0,	0,	2},	/*0x14*/
	{cpu_op_rl_reg,            0,	REG_L,	0,	0,	2},	/*0x15*/
	{cpu_op_rl_indreg16,       0,	REG_HL,	0,	0,	4},	/*0x16*/
	{cpu_op_rl_reg,            0,	REG_A,	0,	0,	2},	/*0x17*/
	{cpu_op_rr_reg,            0,	REG_B,	0,	0,	2},	/*0x18*/
	{cpu_op_rr_reg,            0,	REG_C,	0,	0,	2},	/*0x19*/
	{cpu_op_rr_reg,            0,	REG_D,	0,	0,	2},	/*0x1a*/
	{cpu_op_rr_reg,            0,	REG_E,	0,	0,	2},	/*0x1b*/
	{cpu_op_rr_reg,            0,	REG_H,	0,	0,	2},	/*0x1c*/
	{cpu_op_rr_reg,            0,	REG_L,	0,	0,	2},	/*0x1d*/
	{cpu_op_rr_indreg16,       0,	REG_HL,	0,	0,	4},	/*0x1e*/
	{cpu_op_rr_reg,            0,	REG_A,	0,	0,	2},	/*0x1f*/
	{cpu_op_sla_reg,           0,	REG_B,	0,	0,	2},	/*0x20*/
	{cpu_op_sla_reg,           0,	REG_C,	0,	0,	2},	/*0x21*/
	{cpu_op_sla_reg,           0,	REG_D,	0,	0,	2},	/*0x22*/
	{cpu_op_sla_reg,           0,	REG_E,	0,	0,	2},	/*0x23*/
	{cpu_op_sla_reg,           0,	REG_H,	0,	0,	2},	/*0x24*/
	{cpu_op_sla_reg,           0,	REG_L,	0,	0,	2},	/*0x25*/
	{cpu_op_sla_indreg16,      0,	REG_HL,	0,	0,	4},	/*0x26*/
	{cpu_op_sla_reg,           0,	REG_A,	0,	0,	2},	/*0x27*/
	{cpu_op_sra_reg,           0,	REG_B,	0,	0,	2},	/*0x28*/
	{cpu_op_sra_reg,           0,	REG_C,	0,	0,	2},	/*0x29*/
	{cpu_op_sra_reg,           0,	REG_D,	0,	0,	2},	/*0x2a*/
	{cpu_op_sra_reg,           0,	REG_E,	0,	0,	2},	/*0x2b*/
	{cpu_op_sra_reg,           0,	REG_H,	0,	0,	2},	/*0x2c*/
	{cpu_op_sra_reg,           0,	REG_L,	0,	0,	2},	/*0x2d*/
	{cpu_op_sra_indreg16,      0,	REG_HL,	0,	0,	4},	/*0x2e*/
	{cpu_op_sra_reg,           0,	REG_A,	0,	0,	2},	/*0x2f*/
	{cpu_op_swap_reg,          0,	REG_B,	0,	0,	2},	/*0x30*/
	{cpu_op_swap_reg,          0,	REG_C,	0,	0,	2},	/*0x31*/
	{cpu_op_swap_reg,          0,	REG_D,	0,	0,	2},	/*0x32*/
	{cpu_op_swap_reg,          0,	REG_E,	0,	0,	2},	/*0x33*/
	{cpu_op_swap_reg,          0,	REG_H,	0,	0,	2},	/*0x34*/
	{cpu_op_swap_reg,          0,	REG_L,	0,	0,	2},	/*0x35*/
	{cpu_op_swap_indreg16,     0,	REG_HL,	0,	0,	4},	/*0x36*/
	{cpu_op_swap_reg,          0,	REG_A,	0,	0,	2},	/*0x37*/
	{cpu_op_srl_reg,           0,	REG_B,	0,	0,	2},	/*0x38*/
	{cpu_op_srl_reg,           0,	REG_C,	0,	0,	2},	/*0x39*/
	{cpu_op_srl_reg,           0,	REG_D,	0,	0,	2},	/*0x3a*/
	{cpu_op_srl_reg,           0,	REG_E,	0,	0,	2},	/*0x3b*/
	{cpu_op_srl_reg,           0,	REG_H,	0,	0,	2},	/*0x3c*/
	{cpu_op_srl_reg,           0,	REG_L,	0,	0,	2},	/*0x3d*/
	{cpu_op_srl_indreg16,      0,	REG_HL,	0,	0,	4},	/*0x3e*/
	{cpu_op_srl_reg,           0,	REG_A,	0,	0,	2},	/*0x3f*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_B,	2},	/*0x40*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_C,	2},	/*0x41*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_D,	2},	/*0x42*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_E,	2},	/*0x43*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_H,	2},	/*0x44*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_L,	2},	/*0x45*/
	{cpu_op_bit_cons_indreg16, 0,	0x00,	0,	REG_HL,	3},	/*0x46*/
	{cpu_op_bit_cons_reg,      0,	0x00,	0,	REG_A,	2},	/*0x47*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_B,	2},	/*0x48*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_C,	2},	/*0x49*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_D,	2},	/*0x4a*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_E,	2},	/*0x4b*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_H,	2},	/*0x4c*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_L,	2},	/*0x4d*/
	{cpu_op_bit_cons_indreg16, 0,	0x01,	0,	REG_HL,	3},	/*0x4e*/
	{cpu_op_bit_cons_reg,      0,	0x01,	0,	REG_A,	2},	/*0x4f*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_B,	2},	/*0x50*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_C,	2},	/*0x51*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_D,	2},	/*0x52*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_E,	2},	/*0x53*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_H,	2},	/*0x54*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_L,	2},	/*0x55*/
	{cpu_op_bit_cons_indreg16, 0,	0x02,	0,	REG_HL,	3},	/*0x56*/
	{cpu_op_bit_cons_reg,      0,	0x02,	0,	REG_A,	2},	/*0x57*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_B,	2},	/*0x58*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_C,	2},	/*0x59*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_D,	2},	/*0x5a*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_E,	2},	/*0x5b*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_H,	2},	/*0x5c*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_L,	2},	/*0x5d*/
	{cpu_op_bit_cons_indreg16, 0,	0x03,	0,	REG_HL,	3},	/*0x5e*/
	{cpu_op_bit_cons_reg,      0,	0x03,	0,	REG_A,	2},	/*0x5f*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_B,	2},	/*0x60*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_C,	2},	/*0x61*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_D,	2},	/*0x62*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_E,	2},	/*0x63*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_H,	2},	/*0x64*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_L,	2},	/*0x65*/
	{cpu_op_bit_cons_indreg16, 0,	0x04,	0,	REG_HL,	3},	/*0x66*/
	{cpu_op_bit_cons_reg,      0,	0x04,	0,	REG_A,	2},	/*0x67*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_B,	2},	/*0x68*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_C,	2},	/*0x69*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_D,	2},	/*0x6a*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_E,	2},	/*0x6b*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_H,	2},	/*0x6c*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_L,	2},	/*0x6d*/
	{cpu_op_bit_cons_indreg16, 0,	0x05,	0,	REG_HL,	3},	/*0x6e*/
	{cpu_op_bit_cons_reg,      0,	0x05,	0,	REG_A,	2},	/*0x6f*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_B,	2},	/*0x70*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_C,	2},	/*0x71*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_D,	2},	/*0x72*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_E,	2},	/*0x73*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_H,	2},	/*0x74*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_L,	2},	/*0x75*/
	{cpu_op_bit_cons_indreg16, 0,	0x06,	0,	REG_HL,	3},	/*0x76*/
	{cpu_op_bit_cons_reg,      0,	0x06,	0,	REG_A,	2},	/*0x77*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_B,	2},	/*0x78*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_C,	2},	/*0x79*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_D,	2},	/*0x7a*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_E,	2},	/*0x7b*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_H,	2},	/*0x7c*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_L,	2},	/*0x7d*/
	{cpu_op_bit_cons_indreg16, 0,	0x07,	0,	REG_HL,	3},	/*0x7e*/
	{cpu_op_bit_cons_reg,      0,	0x07,	0,	REG_A,	2},	/*0x7f*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_B,	2},	/*0x80*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_C,	2},	/*0x81*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_D,	2},	/*0x82*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_E,	2},	/*0x83*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_H,	2},	/*0x84*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_L,	2},	/*0x85*/
	{cpu_op_res_cons_indreg16, 0,	0x00,	0,	REG_HL,	4},	/*0x86*/
	{cpu_op_res_cons_reg,      0,	0x00,	0,	REG_A,	2},	/*0x87*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_B,	2},	/*0x88*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_C,	2},	/*0x89*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_D,	2},	/*0x8a*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_E,	2},	/*0x8b*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_H,	2},	/*0x8c*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_L,	2},	/*0x8d*/
	{cpu_op_res_cons_indreg16, 0,	0x01,	0,	REG_HL,	4},	/*0x8e*/
	{cpu_op_res_cons_reg,      0,	0x01,	0,	REG_A,	2},	/*0x8f*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_B,	2},	/*0x90*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_C,	2},	/*0x91*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_D,	2},	/*0x92*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_E,	2},	/*0x93*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_H,	2},	/*0x94*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_L,	2},	/*0x95*/
	{cpu_op_res_cons_indreg16, 0,	0x02,	0,	REG_HL,	4},	/*0x96*/
	{cpu_op_res_cons_reg,      0,	0x02,	0,	REG_A,	2},	/*0x97*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_B,	2},	/*0x98*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_C,	2},	/*0x99*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_D,	2},	/*0x9a*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_E,	2},	/*0x9b*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_H,	2},	/*0x9c*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_L,	2},	/*0x9d*/
	{cpu_op_res_cons_indreg16, 0,	0x03,	0,	REG_HL,	4},	/*0x9e*/
	{cpu_op_res_cons_reg,      0,	0x03,	0,	REG_A,	2},	/*0x9f*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_B,	2},	/*0xa0*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_C,	2},	/*0xa1*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_D,	2},	/*0xa2*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_E,	2},	/*0xa3*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_H,	2},	/*0xa4*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_L,	2},	/*0xa5*/
	{cpu_op_res_cons_indreg16, 0,	0x04,	0,	REG_HL,	4},	/*0xa6*/
	{cpu_op_res_cons_reg,      0,	0x04,	0,	REG_A,	2},	/*0xa7*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_B,	2},	/*0xa8*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_C,	2},	/*0xa9*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_D,	2},	/*0xaa*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_E,	2},	/*0xab*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_H,	2},	/*0xac*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_L,	2},	/*0xad*/
	{cpu_op_res_cons_indreg16, 0,	0x05,	0,	REG_HL,	4},	/*0xae*/
	{cpu_op_res_cons_reg,      0,	0x05,	0,	REG_A,	2},	/*0xaf*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_B,	2},	/*0xb0*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_C,	2},	/*0xb1*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_D,	2},	/*0xb2*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_E,	2},	/*0xb3*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_H,	2},	/*0xb4*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_L,	2},	/*0xb5*/
	{cpu_op_res_cons_indreg16, 0,	0x06,	0,	REG_HL,	4},	/*0xb6*/
	{cpu_op_res_cons_reg,      0,	0x06,	0,	REG_A,	2},	/*0xb7*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_B,	2},	/*0xb8*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_C,	2},	/*0xb9*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_D,	2},	/*0xba*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_E,	2},	/*0xbb*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_H,	2},	/*0xbc*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_L,	2},	/*0xbd*/
	{cpu_op_res_cons_indreg16, 0,	0x07,	0,	REG_HL,	4},	/*0xbe*/
	{cpu_op_res_cons_reg,      0,	0x07,	0,	REG_A,	2},	/*0xbf*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_B,	2},	/*0xc0*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_C,	2},	/*0xc1*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_D,	2},	/*0xc2*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_E,	2},	/*0xc3*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_H,	2},	/*0xc4*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_L,	2},	/*0xc5*/
	{cpu_op_set_cons_indreg16, 0,	0x00,	0,	REG_HL,	4},	/*0xc6*/
	{cpu_op_set_cons_reg,      0,	0x00,	0,	REG_A,	2},	/*0xc7*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_B,	2},	/*0xc8*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_C,	2},	/*0xc9*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_D,	2},	/*0xca*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_E,	2},	/*0xcb*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_H,	2},	/*0xcc*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_L,	2},	/*0xcd*/
	{cpu_op_set_cons_indreg16, 0,	0x01,	0,	REG_HL,	4},	/*0xce*/
	{cpu_op_set_cons_reg,      0,	0x01,	0,	REG_A,	2},	/*0xcf*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_B,	2},	/*0xd0*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_C,	2},	/*0xd1*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_D,	2},	/*0xd2*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_E,	2},	/*0xd3*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_H,	2},	/*0xd4*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_L,	2},	/*0xd5*/
	{cpu_op_set_cons_indreg16, 0,	0x02,	0,	REG_HL,	4},	/*0xd6*/
	{cpu_op_set_cons_reg,      0,	0x02,	0,	REG_A,	2},	/*0xd7*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_B,	2},	/*0xd8*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_C,	2},	/*0xd9*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_D,	2},	/*0xda*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_E,	2},	/*0xdb*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_H,	2},	/*0xdc*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_L,	2},	/*0xdd*/
	{cpu_op_set_cons_indreg16, 0,	0x03,	0,	REG_HL,	4},	/*0xde*/
	{cpu_op_set_cons_reg,      0,	0x03,	0,	REG_A,	2},	/*0xdf*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_B,	2},	/*0xe0*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_C,	2},	/*0xe1*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_D,	2},	/*0xe2*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_E,	2},	/*0xe3*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_H,	2},	/*0xe4*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_L,	2},	/*0xe5*/
	{cpu_op_set_cons_indreg16, 0,	0x04,	0,	REG_HL,	4},	/*0xe6*/
	{cpu_op_set_cons_reg,      0,	0x04,	0,	REG_A,	2},	/*0xe7*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_B,	2},	/*0xe8*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_C,	2},	/*0xe9*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_D,	2},	/*0xea*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_E,	2},	/*0xeb*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_H,	2},	/*0xec*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_L,	2},	/*0xed*/
	{cpu_op_set_cons_indreg16, 0,	0x05,	0,	REG_HL,	4},	/*0xee*/
	{cpu_op_set_cons_reg,      0,	0x05,	0,	REG_A,	2},	/*0xef*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_B,	2},	/*0xf0*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_C,	2},	/*0xf1*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_D,	2},	/*0xf2*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_E,	2},	/*0xf3*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_H,	2},	/*0xf4*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_L,	2},	/*0xf5*/
	{cpu_op_set_cons_indreg16, 0,	0x06,	0,	REG_HL,	4},	/*0xf6*/
	{cpu_op_set_cons_reg,      0,	0x06,	0,	REG_A,	2},	/*0xf7*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_B,	2},	/*0xf8*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_C,	2},	/*0xf9*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_D,	2},	/*0xfa*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_E,	2},	/*0xfb*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_H,	2},	/*0xfc*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_L,	2},	/*0xfd*/
	{cpu_op_set_cons_indreg16, 0,	0x07,	0,	REG_HL,	4},	/*0xfe*/
	{cpu_op_set_cons_reg,      0,	0x07,	0,	REG_A,	2} 	/*0xff*/
	/* Number of lines: 256 */
};

#endif
