/*
 * debug_op_table.h
 *	Opcode table used to dissassmble instructions in debugger
 * 
 * (c)2001 Mattias Wadman
 */

#ifndef __DEBUG_OP_TABLE_H__
#define __DEBUG_OP_TABLE_H__

struct debug_op
{
	char *string;
	int size;
};

struct debug_op debug_op_table[256] =
{
	{"NOP",                0}, /*0x00*/
	{"LD   BC,$%.2x%.2x",  2}, /*0x01*/
	{"LD   (BC),A",        0}, /*0x02*/
	{"INC  BC",            0}, /*0x03*/
	{"INC  B",             0}, /*0x04*/
	{"DEC  B",             0}, /*0x05*/
	{"LD   B,$%.2x",       1}, /*0x06*/
	{"RLCA",               0}, /*0x07*/
	{"LD   ($%.2x%.2x),SP",2}, /*0x08*/
	{"ADD  HL,BC",         0}, /*0x09*/
	{"LD   A,(BC)",        0}, /*0x0a*/
	{"DEC  BC",            0}, /*0x0b*/
	{"INC  C",             0}, /*0x0c*/
	{"DEC  C",             0}, /*0x0d*/
	{"LD   C,$%.2x",       1}, /*0x0e*/
	{"RRCA",               0}, /*0x0f*/
	{"STOP",               1}, /*0x10*/
	{"LD   DE,$%.2x%.2x",  2}, /*0x11*/
	{"LD   (DE),A",        0}, /*0x12*/
	{"INC  DE",            0}, /*0x13*/
	{"INC  D",             0}, /*0x14*/
	{"DEC  D",             0}, /*0x15*/
	{"LD   D,$%.2x",       1}, /*0x16*/
	{"RLA",                0}, /*0x17*/
	{"JR   $%.2x",         1}, /*0x18*/
	{"ADD  HL,DE",         0}, /*0x19*/
	{"LD   A,(DE)",        0}, /*0x1a*/
	{"DEC  DE",            0}, /*0x1b*/
	{"INC  E",             0}, /*0x1c*/
	{"DEC  E",             0}, /*0x1d*/
	{"LD   E,$%.2x",       1}, /*0x1e*/
	{"RRA",                0}, /*0x1f*/
	{"JR   NZ,$%.2x",      1}, /*0x20*/
	{"LD   HL,$%.2x%.2x",  2}, /*0x21*/
	{"LDI  (HL),A",        0}, /*0x22*/
	{"INC  HL",            0}, /*0x23*/
	{"INC  H",             0}, /*0x24*/
	{"DEC  H",             0}, /*0x25*/
	{"LD   H,$%.2x",       1}, /*0x26*/
	{"DAA",                0}, /*0x27*/
	{"JR   Z,$%.2x",       1}, /*0x28*/
	{"ADD  HL,HL",         0}, /*0x29*/
	{"LDI  A,(HL)",        0}, /*0x2a*/
	{"DEC  HL",            0}, /*0x2b*/
	{"INC  L",             0}, /*0x2c*/
	{"DEC  L",             0}, /*0x2d*/
	{"LD   L,$%.2x",       1}, /*0x2e*/
	{"CPL",                0}, /*0x2f*/
	{"JR   NC,$%.2x",      1}, /*0x30*/
	{"LD   SP,$%.2x%.2x",  2}, /*0x31*/
	{"LDD  (HL),A",        0}, /*0x32*/
	{"INC  SP",            0}, /*0x33*/
	{"INC  (HL)",          0}, /*0x34*/
	{"DEC  (HL)",          0}, /*0x35*/
	{"LD   (HL),$%.2x",    1}, /*0x36*/
	{"SCF",                0}, /*0x37*/
	{"JR   C,$%.2x",       1}, /*0x38*/
	{"ADD  HL,SP",         0}, /*0x39*/
	{"LDD  A,(HL)",        0}, /*0x3a*/
	{"DEC  SP",            0}, /*0x3b*/
	{"INC  A",             0}, /*0x3c*/
	{"DEC  A",             0}, /*0x3d*/
	{"LD   A,$%.2x",       1}, /*0x3e*/
	{"CCF",                0}, /*0x3f*/
	{"LD   B,B",           0}, /*0x40*/
	{"LD   B,C",           0}, /*0x41*/
	{"LD   B,D",           0}, /*0x42*/
	{"LD   B,E",           0}, /*0x43*/
	{"LD   B,H",           0}, /*0x44*/
	{"LD   B,L",           0}, /*0x45*/
	{"LD   B,(HL)",        0}, /*0x46*/
	{"LD   B,A",           0}, /*0x47*/
	{"LD   C,B",           0}, /*0x48*/
	{"LD   C,C",           0}, /*0x49*/
	{"LD   C,D",           0}, /*0x4a*/
	{"LD   C,E",           0}, /*0x4b*/
	{"LD   C,H",           0}, /*0x4c*/
	{"LD   C,L",           0}, /*0x4d*/
	{"LD   C,(HL)",        0}, /*0x4e*/
	{"LD   C,A",           0}, /*0x4f*/
	{"LD   D,B",           0}, /*0x50*/
	{"LD   D,C",           0}, /*0x51*/
	{"LD   D,D",           0}, /*0x52*/
	{"LD   D,E",           0}, /*0x53*/
	{"LD   D,H",           0}, /*0x54*/
	{"LD   D,L",           0}, /*0x55*/
	{"LD   D,(HL)",        0}, /*0x56*/
	{"LD   D,A",           0}, /*0x57*/
	{"LD   E,B",           0}, /*0x58*/
	{"LD   E,C",           0}, /*0x59*/
	{"LD   E,D",           0}, /*0x5a*/
	{"LD   E,E",           0}, /*0x5b*/
	{"LD   E,H",           0}, /*0x5c*/
	{"LD   E,L",           0}, /*0x5d*/
	{"LD   E,(HL)",        0}, /*0x5e*/
	{"LD   E,A",           0}, /*0x5f*/
	{"LD   H,B",           0}, /*0x60*/
	{"LD   H,C",           0}, /*0x61*/
	{"LD   H,D",           0}, /*0x62*/
	{"LD   H,E",           0}, /*0x63*/
	{"LD   H,H",           0}, /*0x64*/
	{"LD   H,L",           0}, /*0x65*/
	{"LD   H,(HL)",        0}, /*0x66*/
	{"LD   H,A",           0}, /*0x67*/
	{"LD   L,B",           0}, /*0x68*/
	{"LD   L,C",           0}, /*0x69*/
	{"LD   L,D",           0}, /*0x6a*/
	{"LD   L,E",           0}, /*0x6b*/
	{"LD   L,H",           0}, /*0x6c*/
	{"LD   L,L",           0}, /*0x6d*/
	{"LD   L,(HL)",        0}, /*0x6e*/
	{"LD   L,A",           0}, /*0x6f*/
	{"LD   (HL),B",        0}, /*0x70*/
	{"LD   (HL),C",        0}, /*0x71*/
	{"LD   (HL),D",        0}, /*0x72*/
	{"LD   (HL),E",        0}, /*0x73*/
	{"LD   (HL),H",        0}, /*0x74*/
	{"LD   (HL),L",        0}, /*0x75*/
	{"HALT",               0}, /*0x76*/
	{"LD   (HL),A",        0}, /*0x77*/
	{"LD   A,B",           0}, /*0x78*/
	{"LD   A,C",           0}, /*0x79*/
	{"LD   A,D",           0}, /*0x7a*/
	{"LD   A,E",           0}, /*0x7b*/
	{"LD   A,H",           0}, /*0x7c*/
	{"LD   A,L",           0}, /*0x7d*/
	{"LD   A,(HL)",        0}, /*0x7e*/
	{"LD   A,A",           0}, /*0x7f*/
	{"ADD  A,B",           0}, /*0x80*/
	{"ADD  A,C",           0}, /*0x81*/
	{"ADD  A,D",           0}, /*0x82*/
	{"ADD  A,E",           0}, /*0x83*/
	{"ADD  A,H",           0}, /*0x84*/
	{"ADD  A,L",           0}, /*0x85*/
	{"ADD  A,(HL)",        0}, /*0x86*/
	{"ADD  A,A",           0}, /*0x87*/
	{"ADC  A,B",           0}, /*0x88*/
	{"ADC  A,C",           0}, /*0x89*/
	{"ADC  A,D",           0}, /*0x8a*/
	{"ADC  A,E",           0}, /*0x8b*/
	{"ADC  A,H",           0}, /*0x8c*/
	{"ADC  A,L",           0}, /*0x8d*/
	{"ADC  A,(HL)",        0}, /*0x8e*/
	{"ADC  A,A",           0}, /*0x8f*/
	{"SUB  B",             0}, /*0x90*/
	{"SUB  C",             0}, /*0x91*/
	{"SUB  D",             0}, /*0x92*/
	{"SUB  E",             0}, /*0x93*/
	{"SUB  H",             0}, /*0x94*/
	{"SUB  L",             0}, /*0x95*/
	{"SUB  (HL)",          0}, /*0x96*/
	{"SUB  A",             0}, /*0x97*/
	{"SBC  A,B",           0}, /*0x98*/
	{"SBC  A,C",           0}, /*0x99*/
	{"SBC  A,D",           0}, /*0x9a*/
	{"SBC  A,E",           0}, /*0x9b*/
	{"SBC  A,H",           0}, /*0x9c*/
	{"SBC  A,L",           0}, /*0x9d*/
	{"SBC  A,(HL)",        0}, /*0x9e*/
	{"SBC  A,A",           0}, /*0x9f*/
	{"AND  B",             0}, /*0xa0*/
	{"AND  C",             0}, /*0xa1*/
	{"AND  D",             0}, /*0xa2*/
	{"AND  E",             0}, /*0xa3*/
	{"AND  H",             0}, /*0xa4*/
	{"AND  L",             0}, /*0xa5*/
	{"AND  (HL)",          0}, /*0xa6*/
	{"AND  A",             0}, /*0xa7*/
	{"XOR  B",             0}, /*0xa8*/
	{"XOR  C",             0}, /*0xa9*/
	{"XOR  D",             0}, /*0xaa*/
	{"XOR  E",             0}, /*0xab*/
	{"XOR  H",             0}, /*0xac*/
	{"XOR  L",             0}, /*0xad*/
	{"XOR  (HL)",          0}, /*0xae*/
	{"XOR  A",             0}, /*0xaf*/
	{"OR   B",             0}, /*0xb0*/
	{"OR   C",             0}, /*0xb1*/
	{"OR   D",             0}, /*0xb2*/
	{"OR   E",             0}, /*0xb3*/
	{"OR   H",             0}, /*0xb4*/
	{"OR   L",             0}, /*0xb5*/
	{"OR   (HL)",          0}, /*0xb6*/
	{"OR   A",             0}, /*0xb7*/
	{"CP   B",             0}, /*0xb8*/
	{"CP   C",             0}, /*0xb9*/
	{"CP   D",             0}, /*0xba*/
	{"CP   E",             0}, /*0xbb*/
	{"CP   H",             0}, /*0xbc*/
	{"CP   L",             0}, /*0xbd*/
	{"CP   (HL)",          0}, /*0xbe*/
	{"CP   A",             0}, /*0xbf*/
	{"RET  NZ",            0}, /*0xc0*/
	{"POP  BC",            0}, /*0xc1*/
	{"JP   NZ,$%.2x%.2x",  2}, /*0xc2*/
	{"JP   $%.2x%.2x",     2}, /*0xc3*/
	{"CALL NZ,$%.2x%.2x",  2}, /*0xc4*/
	{"PUSH BC",            0}, /*0xc5*/
	{"ADD  A,$%.2x",       1}, /*0xc6*/
	{"RST  0",             0}, /*0xc7*/
	{"RET  Z",             0}, /*0xc8*/
	{"RET",                0}, /*0xc9*/
	{"JP   Z,$%.2x%.2x",   2}, /*0xca*/
	{"INTERNAL",           0}, /*0xcb*/ /* INTERNAL */
	{"CALL Z,$%.2x%.2x",   2}, /*0xcc*/
	{"CALL $%.2x%.2x",     2}, /*0xcd*/
	{"ADC  A,$%.2x",       1}, /*0xce*/
	{"RST  8",             0}, /*0xcf*/
	{"RET  NC",            0}, /*0xd0*/
	{"POP  DE",            0}, /*0xd1*/
	{"JP   NC,$%.2x%.2x",  2}, /*0xd2*/
	{"UNKNOWN",            0}, /*0xd3*/ /* UNKNOWN */
	{"CALL NC,$%.2x%.2x",  2}, /*0xd4*/
	{"PUSH DE",            0}, /*0xd5*/
	{"SUB  $%.2x",         1}, /*0xd6*/
	{"RST  10",            0}, /*0xd7*/
	{"RET  C",             0}, /*0xd8*/
	{"RETI",               0}, /*0xd9*/
	{"JP   C,$%.2x%.2x",   2}, /*0xda*/
	{"UNKNOWN",            0}, /*0xdb*/ /* UNKNOWN */
	{"CALL C,$%.2x%.2x",   2}, /*0xdc*/
	{"UNKNOWN",            0}, /*0xdd*/ /* UNKNOWN */
	{"SBC  A,$%.2x",       1}, /*0xde*/
	{"RST  18",            0}, /*0xdf*/
	{"LDH  ($%.2x),A",     1}, /*0xe0*/
	{"POP  HL",            0}, /*0xe1*/
	{"LDH  (C),A",         0}, /*0xe2*/
	{"UNKNOWN",            0}, /*0xe3*/ /* UNKNOWN */
	{"UNKNOWN",            0}, /*0xe4*/ /* UNKNOWN */
	{"PUSH HL",            0}, /*0xe5*/
	{"AND  $%.2x",         1}, /*0xe6*/
	{"RST  20",            0}, /*0xe7*/
	{"ADD  SP,$%.2x",      1}, /*0xe8*/
	{"JP   HL",            0}, /*0xe9*/
	{"LD   ($%.2x%.2x),A", 2}, /*0xea*/
	{"UNKNOWN",            0}, /*0xeb*/ /* UNKNOWN */
	{"UNKNOWN",            0}, /*0xec*/ /* UNKNOWN */
	{"UNKNOWN",            0}, /*0xed*/ /* UNKNOWN */
	{"XOR  $%.2x",         1}, /*0xee*/
	{"RST  28",            0}, /*0xef*/
	{"LDH  A,($%.2x)",     1}, /*0xf0*/
	{"POP  AF",            0}, /*0xf1*/
	{"LDH  A,(C)",         0}, /*0xf2*/
	{"DI",                 0}, /*0xf3*/
	{"UNKNOWN",            0}, /*0xf4*/ /* UNKNOWN */
	{"PUSH AF",            0}, /*0xf5*/
	{"OR   $%.2x",         1}, /*0xf6*/
	{"RST  30",            0}, /*0xf7*/
	{"LDHL SP,$%.2x",      1}, /*0xf8*/
	{"LD   SP,HL",         0}, /*0xf9*/
	{"LD   A,($%.2x%.2x)", 2}, /*0xfa*/
	{"EI",                 0}, /*0xfb*/
	{"UNKNOWN",            0}, /*0xfc*/ /* UNKNOWN */
	{"UNKNOWN",            0}, /*0xfd*/ /* UNKNOWN */
	{"CP   $%.2x",         1}, /*0xfe*/
	{"RST  38",            0}  /*0xff*/
	/* Number of lines: 256 */
};

// 0xcb prefixed opcodes

struct debug_op debug_op_table_cb[256] =
{
	{"RLC  B",             0}, /*0x00*/
	{"RLC  C",             0}, /*0x01*/
	{"RLC  D",             0}, /*0x02*/
	{"RLC  E",             0}, /*0x03*/
	{"RLC  H",             0}, /*0x04*/
	{"RLC  L",             0}, /*0x05*/
	{"RLC  (HL)",          0}, /*0x06*/
	{"RLC  A",             0}, /*0x07*/
	{"RRC  B",             0}, /*0x08*/
	{"RRC  C",             0}, /*0x09*/
	{"RRC  D",             0}, /*0x0a*/
	{"RRC  E",             0}, /*0x0b*/
	{"RRC  H",             0}, /*0x0c*/
	{"RRC  L",             0}, /*0x0d*/
	{"RRC  (HL)",          0}, /*0x0e*/
	{"RRC  A",             0}, /*0x0f*/
	{"RL   B",             0}, /*0x10*/
	{"RL   C",             0}, /*0x11*/
	{"RL   D",             0}, /*0x12*/
	{"RL   E",             0}, /*0x13*/
	{"RL   H",             0}, /*0x14*/
	{"RL   L",             0}, /*0x15*/
	{"RL   (HL)",          0}, /*0x16*/
	{"RL   A",             0}, /*0x17*/
	{"RR   B",             0}, /*0x18*/
	{"RR   C",             0}, /*0x19*/
	{"RR   D",             0}, /*0x1a*/
	{"RR   E",             0}, /*0x1b*/
	{"RR   H",             0}, /*0x1c*/
	{"RR   L",             0}, /*0x1d*/
	{"RR   (HL)",          0}, /*0x1e*/
	{"RR   A",             0}, /*0x1f*/
	{"SLA  B",             0}, /*0x20*/
	{"SLA  C",             0}, /*0x21*/
	{"SLA  D",             0}, /*0x22*/
	{"SLA  E",             0}, /*0x23*/
	{"SLA  H",             0}, /*0x24*/
	{"SLA  L",             0}, /*0x25*/
	{"SLA  (HL)",          0}, /*0x26*/
	{"SLA  A",             0}, /*0x27*/
	{"SRA  B",             0}, /*0x28*/
	{"SRA  C",             0}, /*0x29*/
	{"SRA  D",             0}, /*0x2a*/
	{"SRA  E",             0}, /*0x2b*/
	{"SRA  H",             0}, /*0x2c*/
	{"SRA  L",             0}, /*0x2d*/
	{"SRA  (HL)",          0}, /*0x2e*/
	{"SRA  A",             0}, /*0x2f*/
	{"SWAP B",             0}, /*0x30*/
	{"SWAP C",             0}, /*0x31*/
	{"SWAP D",             0}, /*0x32*/
	{"SWAP E",             0}, /*0x33*/
	{"SWAP H",             0}, /*0x34*/
	{"SWAP L",             0}, /*0x35*/
	{"SWAP (HL)",          0}, /*0x36*/
	{"SWAP A",             0}, /*0x37*/
	{"SRL  B",             0}, /*0x38*/
	{"SRL  C",             0}, /*0x39*/
	{"SRL  D",             0}, /*0x3a*/
	{"SRL  E",             0}, /*0x3b*/
	{"SRL  H",             0}, /*0x3c*/
	{"SRL  L",             0}, /*0x3d*/
	{"SRL  (HL)",          0}, /*0x3e*/
	{"SRL  A",             0}, /*0x3f*/
	{"BIT  0,B",           0}, /*0x40*/
	{"BIT  0,C",           0}, /*0x41*/
	{"BIT  0,D",           0}, /*0x42*/
	{"BIT  0,E",           0}, /*0x43*/
	{"BIT  0,H",           0}, /*0x44*/
	{"BIT  0,L",           0}, /*0x45*/
	{"BIT  0,(HL)",        0}, /*0x46*/
	{"BIT  0,A",           0}, /*0x47*/
	{"BIT  1,B",           0}, /*0x48*/
	{"BIT  1,C",           0}, /*0x49*/
	{"BIT  1,D",           0}, /*0x4a*/
	{"BIT  1,E",           0}, /*0x4b*/
	{"BIT  1,H",           0}, /*0x4c*/
	{"BIT  1,L",           0}, /*0x4d*/
	{"BIT  1,(HL)",        0}, /*0x4e*/
	{"BIT  1,A",           0}, /*0x4f*/
	{"BIT  2,B",           0}, /*0x50*/
	{"BIT  2,C",           0}, /*0x51*/
	{"BIT  2,D",           0}, /*0x52*/
	{"BIT  2,E",           0}, /*0x53*/
	{"BIT  2,H",           0}, /*0x54*/
	{"BIT  2,L",           0}, /*0x55*/
	{"BIT  2,(HL)",        0}, /*0x56*/
	{"BIT  2,A",           0}, /*0x57*/
	{"BIT  3,B",           0}, /*0x58*/
	{"BIT  3,C",           0}, /*0x59*/
	{"BIT  3,D",           0}, /*0x5a*/
	{"BIT  3,E",           0}, /*0x5b*/
	{"BIT  3,H",           0}, /*0x5c*/
	{"BIT  3,L",           0}, /*0x5d*/
	{"BIT  3,(HL)",        0}, /*0x5e*/
	{"BIT  3,A",           0}, /*0x5f*/
	{"BIT  4,B",           0}, /*0x60*/
	{"BIT  4,C",           0}, /*0x61*/
	{"BIT  4,D",           0}, /*0x62*/
	{"BIT  4,E",           0}, /*0x63*/
	{"BIT  4,H",           0}, /*0x64*/
	{"BIT  4,L",           0}, /*0x65*/
	{"BIT  4,(HL)",        0}, /*0x66*/
	{"BIT  4,A",           0}, /*0x67*/
	{"BIT  5,B",           0}, /*0x68*/
	{"BIT  5,C",           0}, /*0x69*/
	{"BIT  5,D",           0}, /*0x6a*/
	{"BIT  5,E",           0}, /*0x6b*/
	{"BIT  5,H",           0}, /*0x6c*/
	{"BIT  5,L",           0}, /*0x6d*/
	{"BIT  5,(HL)",        0}, /*0x6e*/
	{"BIT  5,A",           0}, /*0x6f*/
	{"BIT  6,B",           0}, /*0x70*/
	{"BIT  6,C",           0}, /*0x71*/
	{"BIT  6,D",           0}, /*0x72*/
	{"BIT  6,E",           0}, /*0x73*/
	{"BIT  6,H",           0}, /*0x74*/
	{"BIT  6,L",           0}, /*0x75*/
	{"BIT  6,(HL)",        0}, /*0x76*/
	{"BIT  6,A",           0}, /*0x77*/
	{"BIT  7,B",           0}, /*0x78*/
	{"BIT  7,C",           0}, /*0x79*/
	{"BIT  7,D",           0}, /*0x7a*/
	{"BIT  7,E",           0}, /*0x7b*/
	{"BIT  7,H",           0}, /*0x7c*/
	{"BIT  7,L",           0}, /*0x7d*/
	{"BIT  7,(HL)",        0}, /*0x7e*/
	{"BIT  7,A",           0}, /*0x7f*/
	{"RES  0,B",           0}, /*0x80*/
	{"RES  0,C",           0}, /*0x81*/
	{"RES  0,D",           0}, /*0x82*/
	{"RES  0,E",           0}, /*0x83*/
	{"RES  0,H",           0}, /*0x84*/
	{"RES  0,L",           0}, /*0x85*/
	{"RES  0,(HL)",        0}, /*0x86*/
	{"RES  0,A",           0}, /*0x87*/
	{"RES  1,B",           0}, /*0x88*/
	{"RES  1,C",           0}, /*0x89*/
	{"RES  1,D",           0}, /*0x8a*/
	{"RES  1,E",           0}, /*0x8b*/
	{"RES  1,H",           0}, /*0x8c*/
	{"RES  1,L",           0}, /*0x8d*/
	{"RES  1,(HL)",        0}, /*0x8e*/
	{"RES  1,A",           0}, /*0x8f*/
	{"RES  2,B",           0}, /*0x90*/
	{"RES  2,C",           0}, /*0x91*/
	{"RES  2,D",           0}, /*0x92*/
	{"RES  2,E",           0}, /*0x93*/
	{"RES  2,H",           0}, /*0x94*/
	{"RES  2,L",           0}, /*0x95*/
	{"RES  2,(HL)",        0}, /*0x96*/
	{"RES  2,A",           0}, /*0x97*/
	{"RES  3,B",           0}, /*0x98*/
	{"RES  3,C",           0}, /*0x99*/
	{"RES  3,D",           0}, /*0x9a*/
	{"RES  3,E",           0}, /*0x9b*/
	{"RES  3,H",           0}, /*0x9c*/
	{"RES  3,L",           0}, /*0x9d*/
	{"RES  3,(HL)",        0}, /*0x9e*/
	{"RES  3,A",           0}, /*0x9f*/
	{"RES  4,B",           0}, /*0xa0*/
	{"RES  4,C",           0}, /*0xa1*/
	{"RES  4,D",           0}, /*0xa2*/
	{"RES  4,E",           0}, /*0xa3*/
	{"RES  4,H",           0}, /*0xa4*/
	{"RES  4,L",           0}, /*0xa5*/
	{"RES  4,(HL)",        0}, /*0xa6*/
	{"RES  4,A",           0}, /*0xa7*/
	{"RES  5,B",           0}, /*0xa8*/
	{"RES  5,C",           0}, /*0xa9*/
	{"RES  5,D",           0}, /*0xaa*/
	{"RES  5,E",           0}, /*0xab*/
	{"RES  5,H",           0}, /*0xac*/
	{"RES  5,L",           0}, /*0xad*/
	{"RES  5,(HL)",        0}, /*0xae*/
	{"RES  5,A",           0}, /*0xaf*/
	{"RES  6,B",           0}, /*0xb0*/
	{"RES  6,C",           0}, /*0xb1*/
	{"RES  6,D",           0}, /*0xb2*/
	{"RES  6,E",           0}, /*0xb3*/
	{"RES  6,H",           0}, /*0xb4*/
	{"RES  6,L",           0}, /*0xb5*/
	{"RES  6,(HL)",        0}, /*0xb6*/
	{"RES  6,A",           0}, /*0xb7*/
	{"RES  7,B",           0}, /*0xb8*/
	{"RES  7,C",           0}, /*0xb9*/
	{"RES  7,D",           0}, /*0xba*/
	{"RES  7,E",           0}, /*0xbb*/
	{"RES  7,H",           0}, /*0xbc*/
	{"RES  7,L",           0}, /*0xbd*/
	{"RES  7,(HL)",        0}, /*0xbe*/
	{"RES  7,A",           0}, /*0xbf*/
	{"SET  0,B",           0}, /*0xc0*/
	{"SET  0,C",           0}, /*0xc1*/
	{"SET  0,D",           0}, /*0xc2*/
	{"SET  0,E",           0}, /*0xc3*/
	{"SET  0,H",           0}, /*0xc4*/
	{"SET  0,L",           0}, /*0xc5*/
	{"SET  0,(HL)",        0}, /*0xc6*/
	{"SET  0,A",           0}, /*0xc7*/
	{"SET  1,B",           0}, /*0xc8*/
	{"SET  1,C",           0}, /*0xc9*/
	{"SET  1,D",           0}, /*0xca*/
	{"SET  1,E",           0}, /*0xcb*/
	{"SET  1,H",           0}, /*0xcc*/
	{"SET  1,L",           0}, /*0xcd*/
	{"SET  1,(HL)",        0}, /*0xce*/
	{"SET  1,A",           0}, /*0xcf*/
	{"SET  2,B",           0}, /*0xd0*/
	{"SET  2,C",           0}, /*0xd1*/
	{"SET  2,D",           0}, /*0xd2*/
	{"SET  2,E",           0}, /*0xd3*/
	{"SET  2,H",           0}, /*0xd4*/
	{"SET  2,L",           0}, /*0xd5*/
	{"SET  2,(HL)",        0}, /*0xd6*/
	{"SET  2,A",           0}, /*0xd7*/
	{"SET  3,B",           0}, /*0xd8*/
	{"SET  3,C",           0}, /*0xd9*/
	{"SET  3,D",           0}, /*0xda*/
	{"SET  3,E",           0}, /*0xdb*/
	{"SET  3,H",           0}, /*0xdc*/
	{"SET  3,L",           0}, /*0xdd*/
	{"SET  3,(HL)",        0}, /*0xde*/
	{"SET  3,A",           0}, /*0xdf*/
	{"SET  4,B",           0}, /*0xe0*/
	{"SET  4,C",           0}, /*0xe1*/
	{"SET  4,D",           0}, /*0xe2*/
	{"SET  4,E",           0}, /*0xe3*/
	{"SET  4,H",           0}, /*0xe4*/
	{"SET  4,L",           0}, /*0xe5*/
	{"SET  4,(HL)",        0}, /*0xe6*/
	{"SET  4,A",           0}, /*0xe7*/
	{"SET  5,B",           0}, /*0xe8*/
	{"SET  5,C",           0}, /*0xe9*/
	{"SET  5,D",           0}, /*0xea*/
	{"SET  5,E",           0}, /*0xeb*/
	{"SET  5,H",           0}, /*0xec*/
	{"SET  5,L",           0}, /*0xed*/
	{"SET  5,(HL)",        0}, /*0xee*/
	{"SET  5,A",           0}, /*0xef*/
	{"SET  6,B",           0}, /*0xf0*/
	{"SET  6,C",           0}, /*0xf1*/
	{"SET  6,D",           0}, /*0xf2*/
	{"SET  6,E",           0}, /*0xf3*/
	{"SET  6,H",           0}, /*0xf4*/
	{"SET  6,L",           0}, /*0xf5*/
	{"SET  6,(HL)",        0}, /*0xf6*/
	{"SET  6,A",           0}, /*0xf7*/
	{"SET  7,B",           0}, /*0xf8*/
	{"SET  7,C",           0}, /*0xf9*/
	{"SET  7,D",           0}, /*0xfa*/
	{"SET  7,E",           0}, /*0xfb*/
	{"SET  7,H",           0}, /*0xfc*/
	{"SET  7,L",           0}, /*0xfd*/
	{"SET  7,(HL)",        0}, /*0xfe*/
	{"SET  7,A",           0}  /*0xff*/
	/* Number of lines: 256 */
};

#endif
