#include "type.h"
#include "stdio.h"
#include "const.h"
#include "protect.h"
#include "string.h"
#include "fs.h"
#include "proc.h"
#include "tty.h"
#include "console.h"
#include "global.h"
#include "proto.h"


typedef char s8;

u8 mouse_cycle = 0;		//unsigned char
s8 mouse_byte[3];		//signed char
s8 mouse_x = 0;			//signed char
s8 mouse_y = 0;			//signed char
s8 mlbtn,mrbtn;
u8 *old_mem = 0;
int mx, my;

#define G_FACTOR 40
#define G_HEIGHT 24 * G_FACTOR
#define G_WIDTH 79 * G_FACTOR


PRIVATE void draw_mouse(unsigned int start, u8 color);
PRIVATE void mouse_handler (int irq);
PRIVATE void mouse_wait (u8 a_type);
PRIVATE void mouse_write (u8 a_write);
PRIVATE u8   mouse_read ();

PUBLIC void
init_mouse ()
{
  u8 _status;

  mx = G_WIDTH / 2;
  my = G_HEIGHT / 2;

  mouse_wait (1);
  out_byte (0x64, 0xA8);

  //Enable the interrupts
  mouse_wait (1);
  out_byte (0x64, 0x20);
  mouse_wait (0);
  _status = (in_byte (0x60) | 2);
  mouse_wait (1);
  out_byte (0x64, 0x60);
  mouse_wait (1);
  out_byte (0x60, _status);

  //Tell the mouse to use default settings
  mouse_write (0xF6);
  mouse_read ();		//Acknowledge

  //Enable the mouse
  mouse_write (0xF4);
  mouse_read ();		//Acknowledge

  put_irq_handler (MOUSE_IRQ, mouse_handler);
  enable_irq (CASCADE_IRQ);
  enable_irq (MOUSE_IRQ);
}

//Mouse functions
PRIVATE void
mouse_handler (int irq)
{
  switch (mouse_cycle)
    {
    case 0:
      mouse_byte[0] = in_byte (0x60);
      mouse_cycle++;
      mlbtn = mouse_byte[0] & 0x1;
      mrbtn = mouse_byte[0] & 0x2;
      draw_mouse(console_table[current_console].crtc_start,0x42);
      if (mlbtn) {
          beep(1000);
      }
      if (mrbtn) {
          nosound();
      }
      break;
    case 1:
      mouse_byte[1] = in_byte (0x60);
      mouse_cycle++;
      mouse_x = mouse_byte[1];
      mx += mouse_x;
      if (mx < 0) mx = 0;
      if (mx > G_WIDTH) mx = G_WIDTH;
      break;
    case 2:
      mouse_byte[2] = in_byte (0x60);
      mouse_y = mouse_byte[2];
      mouse_cycle = 0;
      my -= mouse_y;
      if (my < 0) my = 0;
      if (my > G_HEIGHT) my = G_HEIGHT;
      break;
    }
}

PRIVATE inline void
mouse_wait (u8 a_type)		//unsigned char
{
  u32 _time_out = 100000;	//unsigned int
  if (a_type == 0)
    {
      while (_time_out--)	//Data
	{
	  if ((in_byte (0x64) & 1) == 1)
	    {
	      return;
	    }
	}
      return;
    }
  else
    {
      while (_time_out--)	//Signal
	{
	  if ((in_byte (0x64) & 2) == 0)
	    {
	      return;
	    }
	}
      return;
    }
}

PRIVATE inline void
mouse_write (u8 a_write)	//unsigned char
{
  //Wait to be able to send a command
  mouse_wait (1);
  //Tell the mouse we are sending a command
  out_byte (0x64, 0xD4);
  //Wait for the final part
  mouse_wait (1);
  //Finally write
  out_byte (0x60, a_write);
}

PRIVATE u8
mouse_read ()
{
  //Get's response from mouse
  mouse_wait (0);
  return in_byte (0x60);
}

PRIVATE void
draw_mouse(unsigned int start, u8 color) {
  u8 * pch = (u8*)(V_MEM_BASE + (start + 80 * (my/G_FACTOR) + (mx/G_FACTOR)) * 2);
  if (old_mem == pch)
	return;
  if (old_mem)
	(*old_mem) ^= color;

  pch[1] = pch[1] ^ color;
  old_mem = pch + 1;
}

