/*****************************************************************************
 *    Copyright (C) 2011 Younghyung Cho. <yhcting77@gmail.com>
 *
 *    This file is part of Writer.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as
 *    published by the Free Software Foundation either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License
 *    (<http://www.gnu.org/licenses/lgpl.html>) for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.	If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef _TEST_VALUEs_h_
#define _TEST_VALUEs_h_

#include <stdint.h>

/*
 * +----->
 * |  +----------+----------+----------+
 * |  |*         |          |          |
 * v  |        * |          |          |
 *    |          |          |          |
 *    |          |    *     |          |
 *    +----------+----------+----------+
 *    |          |          |          |
 *    |          |          |          |
 *    |          |       *  |          |
 *    |          |          |    *     |
 *    +----------+----------+----------+
 */
static const int32_t _pts0[] __attribute__((unused)) = {
	1,  1,
	9,  3,
	14, 9,
	17, 15,
	24, 19
};

/*
 * +----->
 * |  +----------+----------+----------+
 * |  |          |          |          |
 * v  |        * |          |          |
 *    |          |          |          |
 *    |          *          |          |
 *    +----------+----------+----------+
 *    |          |          |          |
 *    |          *          |          |
 *    |          |          |          |
 *    |          | *        |          |
 *    +----------+---*----*-+----------+
 *    |          |          |   *      |
 *    |          |          |          |
 *    |          |          |          |
 *    |          |          |          |
 *    +----------+----------+----------+
 */
static const int32_t _pts1[] __attribute__((unused)) = {
	8,  3,
	10, 8,
	10, 13,
	12, 17,
	14, 20,
	18, 20,
	24, 23
};

/*
 * +----->
 * |  +----------+----------+----------+----------+
 * |  |          |          |          |          |
 * v  |        * |          |          |          |
 *    |          |          | *        |          |
 *    |          |          |        * |          |
 *    +----------+----------+----------+--------*-+
 */
static const int32_t _pts2[] __attribute__((unused)) = {
	8,  3,
	23, 6,
	28, 7,
	37, 10,
};

/*
 * +----->
 * |  +----------+----------+----------+----------+
 * |  |          |          |          |          |
 * v  |          |          |          |          |
 *    |          |          |          |          |
 *    | *        |          |          |          |
 *    +----------+----------+----------+----------+
 *    |          |          |          |          |
 *    |          | *        |          |          |
 *    |          |          |          |          |
 *    |          |       *  |          |          |
 *    +----------+----------+----------+----------+
 *    |          |          |          |          |
 *    |          |          |          |          |
 *    |          |          |          |          |
 *    |          |          |*         |          |
 *    +----------+----------+----------+----------+
 *    |          |          |          |          |
 *    |          |          |          |          |
 *    |          |          |          |          |
 *    |          |          |          | *        |
 *    +----------+----------+----------+----------+
 */
static const int32_t _pts3[] __attribute__((unused)) = {
	3,  7,
	12, 14,
	17, 17,
	22, 27,
	33, 37,
};

static const struct {
	int32_t  l, t, r, b; /* rect */
	int32_t  lns[][4];   /* lines */
} _set1 __attribute__((unused)) = { /* Rect / LiNe SET */
	10, 10, 20, 20,
	{
		/* < 0 >
		 *         C--+---O----+
		 *            |        |
		 *            |        |
		 *            |        |
		 *            +--------+
		 */
		{5, 10, 15, 10},

		/* < 1 >
		 *        C-------O
		 *
		 *            +--------+
		 *            |        |
		 *            |        |
		 *            |        |
		 *            +--------+
		 */
		{5, 5, 15, 5},

		/* < 2 >
		 *
		 *            +--------+
		 *            |        |
		 *            |        |
		 *            |        |
		 *         C--+---O----+
		 */
		{5, 20, 15, 20},

		/* < 3 >
		 *            C
		 *            |
		 *            +--------+
		 *            |        |
		 *            O        |
		 *            |        |
		 *            +--------+
		 */
		{10, 5, 10, 15},

		/* < 4 >
		 *         C
		 *         |
		 *         |  +--------+
		 *         |  |        |
		 *         O  |        |
		 *            |        |
		 *            +--------+
		 */
		{5, 5, 5, 15},

		/* < 5 >
		 *            +--------+
		 *            |        |
		 *        C---+---O    |
		 *            |        |
		 *            +--------+
		 */
		{5, 15, 15, 15},

		/* < 6 >
		 *            +--------+
		 *            |        |
		 *        O---+---C    |
		 *            |        |
		 *            +--------+
		 */
		{15, 15, 5, 15},

		/* < 7 >
		 *            +--------+
		 *            |        |
		 *        C---+--------+---O
		 *            |        |
		 *            +--------+
		 */
		{5, 15, 25, 15},

		/* < 8 >
		 *            +--------+
		 *            |        |
		 *        O---+--------+---C
		 *            |        |
		 *            +--------+
		 */
		{25, 15, 5, 15},

		/* < 9 >
		 *                C
		 *                |
		 *            +---+----+
		 *            |   |    |
		 *            |   O    |
		 *            |        |
		 *            +--------+
		 */
		{15, 5, 15, 15},

		/* < 10 >
		 *                O
		 *                |
		 *            +---+----+
		 *            |   |    |
		 *            |   C    |
		 *            |        |
		 *            +--------+
		 */
		{15, 15, 15, 5},

		/* < 11 >
		 *                C
		 *                |
		 *            +---+----+
		 *            |   |    |
		 *            |   |    |
		 *            |   |    |
		 *            +---+----+
		 *                |
		 *                O
		 */
		{15, 5, 15, 25},

		/* < 12 >
		 *                O
		 *                |
		 *            +---+----+
		 *            |   |    |
		 *            |   |    |
		 *            |   |    |
		 *            +---+----+
		 *                |
		 *                C
		 */
		{15, 25, 15, 5},

		/* < 13 >
		 *            +----O---+
		 *            C        |
		 *            |        |
		 *            |        |
		 *            +--------+
		 */
		{10, 12, 16, 10},

		/* < 14 >
		 *            +----C---+
		 *            O        |
		 *            |        |
		 *            |        |
		 *            +--------+
		 */
		{16, 10, 10, 12},

		/* < 15 >
		 *
		 *            +----O---+
		 *            |        |
		 *         C  |        |
		 *            |        |
		 *            +--------+
		 */
		{5, 15, 16, 10},

		/* < 16 >
		 *            +----C---+
		 *            |        |
		 *         O  |        |
		 *            |        |
		 *            +--------+
		 */
		{16, 10, 5, 15},

		/* < 17 >
		 *                 O
		 *            +--------+
		 *            |        |
		 *          C |        |
		 *            |        |
		 *            +--------+
		 */
		{8, 15, 16, 8},

		/* < 18 >
		 *                 C
		 *            +--------+
		 *            |        |
		 *          O |        |
		 *            |        |
		 *            +--------+
		 */
		{16, 8, 8, 15},

		/* < 19 >
		 *            +--------+
		 *            |        O
		 *          C |        |
		 *            |        |
		 *            +--------+
		 */
		{8, 15, 20, 12},

		/* < 20 >
		 *            +--------+
		 *            |        C
		 *          O |        |
		 *            |        |
		 *            +--------+
		 */
		{20, 12, 8, 15},

		/* < 21 >
		 *            +--------+
		 *            |        |
		 *            C        |
		 *            |        |
		 *            +-----O--+
		 */
		{10, 15, 18, 20},

		/* < 22 >
		 *            +--------+
		 *            |        |
		 *            O        |
		 *            |        |
		 *            +---C----+
		 */
		{15, 20, 10, 15},

		/* < 23 >
		 *            +--------+
		 *            |        |
		 *            |        O
		 *            |        |
		 *            +---C----+
		 */
		{15, 20, 20, 15},

		/* < 24 >
		 *            +--------+
		 *            |        | O
		 *            |        |
		 *            |        |
		 *            +---C----+
		 */
		{15, 20, 25, 13},

		/* < 25 >
		 *            +--------+
		 *            |        |
		 *            |        O
		 *            |        |
		 *            +--------+
		 *               C
		 */
		{13, 23, 20, 15},

		/* < 26 >
		 *            +--------+
		 *            |        | O
		 *            |        |
		 *            |        |
		 *            +--------+
		 *               C
		 */
		{13, 23, 25, 13},


		/* < 27 >
		 *            +--------+
		 *            |  C     |
		 *            |        |
		 *            |        |
		 *            +--------O
		 */
		{13, 13, 20, 20},
	},
};

#endif /* _TEST_VALUEs_h_ */
