#include <stddef.h>
#include <signal.h>
#include <cunit/cunit.h>
#include <libdank/utils/string.h>
#include <libdank/utils/memlimit.h>
#include <libdank/objects/lrupat.h>
#include <libdank/objects/objustring.h>

static int
test_lrupatnullkill(void){
	printf(" Verifying safe destruction of NULL lrupat...\n");
	destroy_lrupat(NULL);
	return 0;
}

static int
test_lrupatbasic(void){
	ustring u = USTRING_INITIALIZER;
	struct lrupat *lp;
	int ret = -1;

	printf(" Verifying basic lrupat operations...\n");
	if((lp = create_lrupat(NULL)) == NULL){
		goto done;
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	reset_ustring(&u);
	ret = 0;

done:
	destroy_lrupat(lp);
	return ret;
}

static const char *LRUPAT_INPUT[] = {
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=951",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: static.cache.l.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/2008052906 Firefox/3.0",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: NID=13=ko2MOSH8SE8__tMUa_c51f9fmCHrzbUS0wWK5T8yK8gzNjZp5DXF3bC92UVPk7EQ6ZVitb7vgyeRXJPajUCBRs8N8nX9wtfjshqwYPFqBTs43cN1Wm0Ud-hFOyzLhhty; PREF=ID=62bba4c265eff849:TM=1185332533:LM=1218662800:L=0WskDHT0A:GM=1:S=KQCRg8n7wayV3aY8; S=quotestreamer=yr4BsfdmBTgQ4nGeFXKGew; TZ=300; GMAIL_RTT=54; SID=DQAAAHEAAAA-yg0NF2HYX0Kt3uQywJ57fRz5S2ljwoT5__w1XT2Gca5LD9S0HSp7Htmb-Ny64zJG7AMXuRk2QoqY6fmVY_VwEXubyqKpr9vJ8sdlrPTQDT-_-5PshvRUBU312plsWaT3Nbi3tHSSs5QnQMkhLLZlhT7pNipHgFgBSL6H_RV3TA",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60923; eid: 2; uid: 2; gid: 1; rep: 0; cat: 177; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=452",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60924; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=460",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60925; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=454",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60926; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=448",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60927; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=448",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60928; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=446",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60929; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=449",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60930; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=458",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60931; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=450",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60932; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=460",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60933; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=461",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60934; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=442",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: airtran.com",
 "Connection: Keep-Alive",
 "Cookie: CP=null*; dest=MDW; orig=MSP; stop_mobi=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60935; eid: 2; uid: 2; gid: 1; rep: -127; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1279",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://airtran.com/Home.aspx",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: ehg-airtran.hitbox.com",
 "Connection: Keep-Alive",
 "Cookie: CTG=1218282279; WSS_GW=V1z%%B%%iBi%%ieQ; DM56051620DAV6=V1rX(#Xrz%%B%%iBi%%ieQXer%%^%%z%%zrzC%%C%%B%%iBi%%ieQz%%B%%iBi%%ieQ%%B%%iBiBB@^%%B%%iBi%%ieQXer%%^%%%%rz(xB$':maxBrYIWaz7}z)O:ma6rOuKr6rz]%%6%%B%%iBiBr%%e6%%zA6':maHYIWaA682cdafF6}a~acfA682cdafF6}aIhcOA682cdafF6}a~acfA682cdafF6}aIhcOA682cdafF6}a~acfA68hIqa~H)T7:hmIf2:TA6M:uF6(mG~:3maTfH<GG:hfDT2f2aFA6M:uF6)T7:H}aFF2:TFzu::HB??hc5E1o@@==3@5PTLdmIhd",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60936; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=312",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: feeds.feedburner.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60937; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1910",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*",
 "Referer: http://messenger.yahoo.com/external/plugins/ct/stock_chart_viewer.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: messenger.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFTHhh6E?@@w==3Z@dW)emPh;d3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60938; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122 157; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2006",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://insider.msg.yahoo.com/client_ad.php?p=842673",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: us.bc.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUHD+hc5Eo@@2==3@5^PVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFTd3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60939; eid: 2; uid: 2; gid: 1; rep: -127; cat: 141; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=214",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://list.smartfilter.com/cgi-bin/updatelist HTTP/1.0",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: list.smartfilter.com",
 "Content-Type: application/x-www-form-urlencoded",
 "Content-Length: 473",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60940; eid: 2; uid: 2; gid: 1; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; MSOffice 12)",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Thu, 10 Jul 2008 21:46:16 GMT",
 "If-None-Match: 0841961d6e2c81:1203",
 "Host: blogs.msdn.com",
 "Connection: Keep-Alive",
 "Cookie: CommunityServer-UserCookie1001=lv=5/28/2008 10:51:26 AM&mra=5/30/2008 11:25:35 AM; CommunityServer-UserCookie139032=lv=1/1/1999 12:00:00 AM&mra=5/30/2008 11:25:37 AM",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60941; eid: 2; uid: 2; gid: 1; rep: 0; cat: 165; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=139",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 208.73.116.2",
 "Accept: */*",
 "Host: swps-proxy.securecomputing.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: res-hdr=0, res-body=102",
 "Server: SCUR ICAP server (snare)",
 "Content-Type: application/x-ns-proxy-autoconfig",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.23",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.21",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=976",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://www.google.com/",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60942; eid: 2; uid: 2; gid: 1; rep: -127; cat: 145; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1042",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://www.google.com/",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60943; eid: 2; uid: 2; gid: 1; rep: -127; cat: 145; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=479",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://24.wcmcs.net/m/fc.aspx?sid=1059&poid=WM",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: 24.wcmcs.net",
 "Connection: Keep-Alive",
 "Cookie: wcm_userkey=cf6be9e1-4205-48e3-81b1-cac791c9cc1a; ASP.NET_SessionId=bvo0wj55zchu2i55nfjetq45; wcm_sessionid=22560969",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60944; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1215",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://us.mg2.mail.yahoo.com/dc/launch?.rand=2v1puk56t0k0d",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: us.mg2.mail.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: Y=v=1&n=4tlcrfc65b0vo&l=j78b54h/o&p=f221lqmb530g0300&jb=21|45|5&r=4l&lg=en-US&intl=us&np=1; T=z=BTEoIBBnrsIBKkrTccTNdb2MjcxBjE0TjQyNDY3Mw--&a=QAE&sk=DAANUJvHUBrr.1&ks=EAAutXzFyoRTrtIDVmA__.RbA--~A&d=c2wBTlRBMkFUWXpPVE0xTXpFd05BLS0BYQFRQUUBZwFTSk43TkJBSUNCNlo1U0VYWjRUQTNBVU4yTQF0aXABQWZqMUJBAXp6AUJURW9JQkE3RQ--; B=7i1v2qt3sbmg8&b=4&d=1v1HScRpYEK5DFt7YvnfCHBTJqGX5xuArlK9BQ--&s=pq; YLS=v=1&p=0&n=0; F=a=spRHBpkMvT_6Dmtndi0t8WjwP5PnDqA83G0eHBeLag0kMCV0s7iKLxpfEubJIZcd1J3xIeE-&b=o5dT; C=mg=1; PL=V=1.1&d=X80tZCGIS.xP7HLE4Iib7ldS_wn9XWfclaJpirB7czTpXMNC.xZRPQz4Ac48G6XORpX8wjeywnCwnNA-; PH=fn=r1LIjby2LgdlVn45RTI-&l=en-US; U=mt=yQ45MZ2MhYmRU2VJj83PBOH2wQOBhKw5RVFN&ux=KjKkIB&un=4tlcrfc65b0vo",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=992",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://www.google.com/",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60945; eid: 2; uid: 2; gid: 1; rep: -127; cat: 145; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=964",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.google.com",
 "Connection: Keep-Alive",
 "Cookie: SS=Q0=aSBsb3ZlIG15IGRhdWdodGVyIG15c3BhY2UgY29tbWVudHM; PREF=ID=5bd3d0df13d6e132:TB=2:TM=1218252118:LM=1218255696:S=gREcv6gCq8O0p2iA; NID=13=rwecPtGQDcqUkTHgfZYp2gREOIREg-P7R_-X84mLyQTbo75EhspSVe_cJzrr5fPuirHEgTP8o0YOwDylH7fVro4vR3yK-CkgCirEUh5wL3YtR0vSgGM-p4j87s3kbKAy",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60946; eid: 2; uid: 2; gid: 1; rep: -127; cat: 145; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=403",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60947; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=334",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60948; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=688",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: data.moneycentral.msn.com",
 "Connection: Keep-Alive",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60949; eid: 2; uid: 2; gid: 1; rep: 0; cat: 141; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=473",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60950; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=689",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: data.moneycentral.msn.com",
 "Connection: Keep-Alive",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60951; eid: 2; uid: 2; gid: 1; rep: 0; cat: 141; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=472",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60952; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60953; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60954; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60955; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60956; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60957; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60958; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=489",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60959; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=489",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60960; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=489",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60961; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=489",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60962; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60963; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60964; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60965; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60966; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60967; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=497",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60968; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=478",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60969; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=652",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60970; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=654",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60971; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=794",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: fls.doubleclick.net",
 "Connection: Keep-Alive",
 "Cookie: id=8000017149748d1|685973/181145/14101|",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60972; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=637",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://fls.doubleclick.net/activityi;src=1662572;type=homep408;cat=homep100;ord=1;num=1685606910836.5002?",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: ad.doubleclick.net",
 "Connection: Keep-Alive",
 "Cookie: id=8000017149748d1|685973/181145/14101|",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60973; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152 154; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=3673",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://fls.doubleclick.net/activityi;src=1662572;type=homep408;cat=homep100;ord=1;num=1685606910836.5002?",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: ad.yieldmanager.com",
 "Connection: Keep-Alive",
 "Cookie: uid=lastCounted=1218685930&uid=b8650ec8-653f-11dd-9d1f-001e6837e133&_hmacv=1&_salt=688556182&_keyid=k1&_hmac=8b9f0f5f692651fdcc40af4c9eaf49a17dd5a271; bh=b!!!!^!!!?I!!!!$:@qYq!!+Vm!!!!#:AJp]!!,WN!!!!#:AJp]!!,^s!!!!#:AJp]!!-yu!!!!(:AJfr!!.+B!!!!':AJIq!!./v!!!!#:AJp]!!.3F!!4U/:AJiP!!.3K!!4U/:AJiP!!.xS!!!!#:AJp]!!/!*!!!!#:@rH:!!/%%x~~!!/q-!!!!#:AJp]!!0+B!!!!(:AJeu!!04b!!!!%%:AJeu!!0=_!!!!':@d6q!!0=q!!!!#:AJet!!0KN!!!!#:AJcU!!0Lf!!!!#:AJp]!!0ca~~!!0cb~~!!0cc~~!!0cd~~!!0ce~~!!0cf~~!!0cg~~!!0ch~~!!0ci~~!!0cj!!!!#:AJp]!!0ck~~!!0cl~~!!0cm~~!!0cn~~!!0co~~!!0cp~~!!0cq~~!!0nx!!!!#:AHNq!!0u@!!!!#:AH8Q!!187!!!!*:AJK-!!2*C~~!!2*D~~!!2*E~~!!2*F~~!!2*G~~!!2*H~~!!2*I~~!!2*J~~!!2*K~~!!2*L~~!!2]'!!4U/:AJiP!!2u@!!!!#:@o3B!!3I%%!!4U/:AJiP!!3I'!!4U/:AJiP!!3fI!!!!$:@qYq!!3fJ!!!!$:@qYq!!3fN!!!!$:HhbEK@@==3@KYPD@qYq!!4)/!!!#m:AJiu; vuday1=!!!!#8sxE.[ml*1q>nei!9>r.8FENL; ih=b!!!!a!!AgE!!!!':@d6q!!HBJ!!!!#:AIqG!!`_y!!!!#:@rsj!!kiR!!!!#:AJgI!!sEq!!!!#:@cK2!##X*!!!!#:AJc^!#(?=!!!!#:AH8Q!#.FD!!!!#:AJcU!#07B!!!!(:AJgI!#07F!!!!,:AJga!#0]v!!!!#:AHNq!#5:u!!!!*:AJfL!#:k3!!!!#:AJet!#>DM!!!!#:@nnw!#C/,!!!!#:AJb6!#F`6!!!!#:AJ`D!#H=H!!!!#:@doA!#KXZ!!!!#:AJb5!#LQs!!!!#:AJaT!#V%%=!!!!#:AJa[!#V+!!!!!%%:@nWf!#V+$!!!!#:@coU!#V+(!!!!#:@c0%%!#X^C!!!!$:AI3i!#[DC!!!!#:@cpB!#]$l!!!!#:@crx!#^vZ!!!!#:AH?^!#^vx!!!!#:AJga!#_<b!!!!#:@d6g!#_>v!!!!#:AJQa!#_Ac!!!!#:AJQP!#_B'!!!!#:AJK-!#_BQ!!!!#:AJTU!#aLf!!!!$:@cK,!#aV-!!!!#:AIpF!#aV<!!!!#:@rrP!#cE<!!!!#:@cK-!#f!g!!!!#:@rrj!#f6'!!!!#:@cK4!#gLm!!!!$:AJcR!#gV0!!!!#:@ngY!#h)L!!!!#:@f4d!#hXj!!!!%%:@eXW!#iQX!!!!(:AHHG!#iR:!!!!#:@o0Z!#j'i!!!!#:AJb7!#kDQ!!!!#:@f6_!#k[f!!!!#:@ndf!#k[k!!!!#:AI_'!#kf9!!!!#:@c.Y!#kgZ!!!!$:AJa5!#kmr!!!!#:AJfq!#krI!!!!#:@cK/!#lOf!!!!#:@nd+!#mY<!!!!#:@d#A!#mhc!!!!#:@nTL!#nUN!!!!#:@o0U!#nUt!!!!#:@cK/!#noO!!!!#:@ndd!#oa(!!!!$:AJIp; lifb=c>DX[!8=q-mcWe[9C)jDm2#<H3TD:(QGt.c?!(nh$k!>RV.a8Vr8/^W!8=q--g%%T0; pv1=b!!!!3!!13t!!!e#!!$@P!#_BQ!!L>1?_'PI!?5%%!#X-p-!wDW,!!%%i=!!%%T%%!!a'b~~~~~:AJTU~M.jTN!!2s/!#7[)!!ATv!#X^C!!Sas?^3v?!#W(2![1T+!ZmB)!!eE7!!kpB~~~~~~:AI3i:E[us~!!16x!'Oc0!!WYR!#gLm!!Rw6!.us?!#1g.#X-p.!ZmB)!!jj4!!jj=~~~~~~:AJcR:AL6W~!!3le!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:@o8+~!!3lf!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:@qcn~!!3lg!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2H66bhE(@@1h=3=@KYlPzHG66bhE(@@1g=3=@KYlPRHhbEK@@==3@lKYPD>^$![:Z-!!gol!!gop~~~~~~:@ndd:@tbx~!!3lh!'Oc0!!WYR!#gLm!!Rw6!.us?!#1g.#X-p.!ZmB)!!jj4!!jj=~~~~~~:AJcR:AV`#~!!3li!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:A/]_~!!3lj!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:A<Y0~!!3lk!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:AmK%%~!!3vw!'Oc0!!WYR!#gLm!!Rw6!.us?!#1g.#X-p.!ZmB)!!jj4!!jj=~~~~~~:AJcR:AL6W~!!49Z!#7[)!!USl!#noO!!Q#K!!mT-!?5%%!!?H2$![:Z-!!gol!!gop~~~~~~:@ndd:A!a5~!!1@:!#7[)!!ATu!#^vZ!!Pcm?[(R,!?5%%!!?mK)!x$$2!!/nj!!EU,!!gMc~~~~~:AH?^:Aa7XM.jTN!!/$P!'Oc0!!L6(!#j'i!!SPn!+,Ft!?5%%!#X-p-#=A.3!!W1]!!j@:!!Xi4!!kXR~~~~:AJb7:CH.kM.jTN!!0$R!'Oc0!!L6(!#.FD!!FT]!06iN!?5%%!#X-p-#=A.3!!W1]!!)gU!!`2O!!X%%n~~~~:AJcU:AoWy!!!NB!!/o'!'Oc0!!L6(!#:k3!!CF@!2hNg!?5%%!#X-p-#<MT+!!W1]!!atV!!`i`!!SVB~~~~:AJet:E^P'M.jTN; liday1=N2NPq[$8fC33bYD6Em!ErDJPf`FOn=cHV:[d(6M^+>2Xo!9>r.B#0]J; crfb=/j]_=F(ev!@LWq(kCuC78-ktA; crday1=W4K9VXj%%h[YJ_z^!9>r.[manG; fl_inst=1",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60974; eid: 2; uid: 2; gid: 1; rep: 0; cat: 154; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=656",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60975; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=734",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://fls.doubleclick.net/activityi;src=1662572;type=homep408;cat=homep100;ord=1;num=1685606910836.5002?",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: a.tribalfusion.com",
 "Connection: Keep-Alive",
 "Cookie: ANON_ID=aymRwQySnwMrMZdNqroLbHtpEiFSpbMMGvOTrr7U0kZa8hMBMdbbQcYd0sjr1pMl2tvg3FQG3EZcUWTcxZdDMIPeMkT9DbX9mHR9QSAcbmmimZcVa3hMTAJSMB6OK9tXrUc1U7Y4ZcvJrQLxYc7LQVADpeJCwil32t3i5bZbvoyPCNmdd0rvURsyA9dAFqZaLZcQEvb3rftS9UoNTEH30Uke4",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60976; eid: 2; uid: 2; gid: 1; rep: 0; cat: 154; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=653",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60977; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=652",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60978; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=655",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.suncountry.com/SCA/index.jsf",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.suncountry.com",
 "Connection: Keep-Alive",
 "Cookie: JSESSIONID=00000abvhxIVxcgHv_arDYVFG0m:12ks3tvoq; __utma=169857955.1725946353.1218705102.1218705102.1218705102.1; __utmb=169857955; __utmc=169857955; __utmz=169857955.1218705102.1.1.utmccn=(direct)|utmcsr=(direct)|utmcmd=(none)",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60979; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1692",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://mail.google.com/mail/?ui=2&ik=dd194b5987&view=tl&start=0&num=70&auto=1&am=R_E4pdz3cTG-410&ari=600&rt=j&search=inbox HTTP/1.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Content-Type: application/x-www-form-urlencoded;charset=utf-8",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Content-Length: 0",
 "Cookie: __utmx=173272373.; __utma=173272373.1916445834.1180453148.1200494341.1200575782.141; GX=DQAAAHUAAACBAXSmRS0cN0bv6fTDLgdtrIRtnYo4EzCxrTne3unkucKlWz0zbUHI94RoH-yP7lJLCiySGwDdVhfTMNlfuAbTF9axtT1_kl8xeJn6NaA7NU4v6LHxJiO3QGtSG3FCGRtpdnAhAsU1E6i819YG-ZmaxLntIbO5lqO4l3-WjTSoWw; S=gmail=zLBRf7qXrbHpgoCWZ7mFHw:gmail_yj=4sI8h5ocOk0L7ewbWlW08w:gmproxy=CNrihF13nR4:gmproxy_yj=wrDdFO6EXG0:gmproxy_yj_sub=6qRbJkPOy94; GMAIL_AT=xn3j36057mmea44j5srgmuruoja1ep; gmailchat=jdtschida@gmail.com/489762; rememberme=false; PREF=ID=77bae0bafa4c66ec:FF=1:TM=1180453130:LM=1210680165:GM=1:S=RSZyBmX54dkjie7n; NID=13=m",
 "Pragma: no-cache",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=622",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.nbcolympics.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: ak-mobile-detected=no; s_cc=true; s_nr=1218657732998; s_lv=1218668526821; s_sq=%%5B%%5BB%%5D%%5D; __qca=1207077133-42804709-87523247; __qcb=1225013213",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60980; eid: 2; uid: 2; gid: 1; rep: 0; cat: 146; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2006",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://insider.msg.yahoo.com/client_ad.php?p=842673",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: us.bc.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oO",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60981; eid: 2; uid: 2; gid: 1; rep: -127; cat: 141; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1518",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: close",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Cookie: GX=DQAAAHwAAABYNrT0gvD3MdvObqBin13wuP2Y5oWM5xUC8wQfTav2nbtmlA9dieBDtrC8mPMZFAqdxXFRDbmLC1orYeBUBUIqsexbYrkp1iu63mC7uet888jRHigkcWsxC829b--i6fw7AOygNVYvQcHXSBmWSYvv3T9bjKvrg2oOodIVYLhD0g; S=gmail=xtXhxEuCvTamqobh5ynGMg:gmail_yj=CIHgVnpdNM3_oe69hfVM9A:gmproxy=pj758NCLqwQ:gmproxy_yj=U2vx_QQc1TI:gmproxy_yj_sub=ZEx3um5sCw0; GMAIL_AT=xn3j37n8o82z9zuqs1k2g6gdcmergb; gmailchat=john.nemitz@gmail.com/276172; PREF=ID=7a0c72b072d813b6:FF=4:LD=en:NR=10:TM=1206024211:LM=1209402478:GM=1:S=r7lR9NGbWCv1_aHr; NID=13=M3GeTpkhEQmV2l7zVueZgREWpsIMlC52ojAbz0mOLDDA7aY5NzO1VrdmeP2mw_4V1XvNN6BOMSYBIKtC-hxDBum9MVdWU",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=815",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: sb.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: PREF=ID=0669467ae9e9bec9:TM=1210296425:LM=1210371970:GM=1:S=SPXW6kd312wsariw; NID=13=CrB3iEWf16O4ZKWefpSLO1HELpB-ieHIVJm037CBqi19CXjESRp5Y_IiV-JVVy130GbU0Umk6QTZsI-PU9A1B6Oe-592aQYLWhRctAMc2IDo_jHW3dGL3E2ron31-PHO",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60982; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=113",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://list.smartfilter.com/cgi-bin/updatelist HTTP/1.0",
 "Host: list.smartfilter.com",
 "Content-Length: 354",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60983; eid: 2; uid: 2; gid: 1; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60984; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1532",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: close",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Cookie: GX=DQAAAHYAAAAbRrZCLsXh5ZcGGwEILrrjTVdhJqfJyGbe_lJexgdJdk7NRavoCGCt434pDemR7kZp2t81DKajo1P_1CE8_KvtmIOgK0TT4rEJvhviaOVZ53_OkN0WfVgymfxGW1B3f8uejfKXvgnTDmNiUymv-tZFpa5QFDfNEu2OXQDupfDD6A; S=gmail=7U1b1OZo3Lf33qU6HKFFjw:gmail_yj=wTaX_WKW1kMGxdZ7jLbt-w:gmproxy=YSSHn1lmwE8:gmproxy_yj=nerwXYjZ-x8:gmproxy_yj_sub=a__riccSVj4; GMAIL_AT=xn3j3614qdid9j1mu3drhslywvd8b9; gmailchat=steve.goers@gmail.com/720839; PREF=ID=67c802120d3b2caf:TM=1217710846:LM=1217714251:GM=1:S=5SPkSiho_Do6198S; NID=13=jWExaO4XhQ6HJgCJkQAO5prppQAukiQxHR-MJO4WsvLB6KUy5Fs4u4ifnnMpVp-Aphi65eUTKGQkjs0chBU63ESsuDkViB9apjLihTVgy_gdzUjER48yXqxH",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2484",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 216.235.158.34",
 "Host: www.okcupid.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/javascript, text/html, application/xml, text/xml, */*",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "X-Requested-With: XMLHttpRequest",
 "X-Prototype-Version: 1.6.0.2",
 "Referer: http://www.okcupid.com/forum?tid=1616731738136776031",
 "Cookie: __qca=1208617279-56396514-56980535; __utma=132982672.1882520404.1216291135.1218688172.1218698304.122; __utmz=132982672.1217307396.75.3.utmccn=(referral)|utmcsr=okcupid.com|utmcct=/forum|utmcmd=referral; username=eplossl; ru15836121488829250307=6888236873690812350%%7CSluzin%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/52x0/202x150/2/16539671370562616888.jpeg___82_82_82_82_2dfc4813_.jpg%%7CKennesaw%%7C29%%7Cf%%7Cs*3093072485526931756%%7Clovensunshine%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/0x63/358x421/2/13298148684615234817.jpeg___82_82_82_82_2dfc4813_.jpg%%7CGray%%7C24%%7Cf%%7Cs*17993932046903613536%%7Csweetgagrl%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/64x0/352x288/2/1414773HO",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60985; eid: 2; uid: 6; gid: 1; rep: 0; cat: 192; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=320",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 160.81.118.42",
 "Host: swps-proxy.securecomputing.com",
 "Connection: Keep-Alive",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; GomezAgent 2.0; Windows NT)",
 "Accept: image/gif, image/x-bitmap, image/jpeg, image/pjpeg, */*",
 "Accept-Language: en-us",
 "Accept-Encoding: gzip, deflate",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: req-hdr=0, null-body=297",
 "Host: swps-proxy.securecomputing.com",
 "Connection: Keep-Alive",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; GomezAgent 2.0; Windows NT)",
 "Accept: image/gif, image/x-bitmap, image/jpeg, image/pjpeg, */*",
 "Accept-Language: en-us",
 "Accept-Encoding: gzip, deflate",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=479",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://24.wcmcs.net/m/fc.aspx?sid=1059&poid=WM",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: 24.wcmcs.net",
 "Connection: Keep-Alive",
 "Cookie: wcm_userkey=cf6be9e1-4205-48e3-81b1-cac791c9cc1a; ASP.NET_SessionId=bvo0wj55zchu2i55nfjetq45; wcm_sessionid=22560969",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60986; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=599",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60987; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=653",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Connection: Keep-Alive",
 "Host: www.amtrak.com",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60988; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=527",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60989; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=482",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Cache-Control: no-cache",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60990; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=432",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60991; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=440",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60992; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=440",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60993; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=443",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60994; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=441",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60995; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=437",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60996; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=437",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60997; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=437",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60998; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=437",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 60999; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=441",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61000; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=428",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61001; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=439",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61002; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=431",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61003; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=446",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61004; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=435",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61005; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=447",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61006; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=442",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61007; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=435",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61008; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=326",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61009; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=596",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app?serviceId=Echo.Synchronize HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: text/xml; charset=UTF-8",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 1182",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61010; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=509",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61011; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=505",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61012; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=594",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app?serviceId=Echo.Synchronize HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: text/xml; charset=UTF-8",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 32",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61013; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61014; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=527",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61015; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=530",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61016; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=522",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61017; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 46",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61018; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=599",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 136",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61019; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2482",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 216.235.158.34",
 "Host: www.okcupid.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/javascript, text/html, application/xml, text/xml, */*",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "X-Requested-With: XMLHttpRequest",
 "X-Prototype-Version: 1.6.0.2",
 "Referer: http://www.okcupid.com/forum?tid=1616731738136776031",
 "Cookie: __qca=1208617279-56396514-56980535; __utma=132982672.1882520404.1216291135.1218688172.1218698304.122; __utmz=132982672.1217307396.75.3.utmccn=(referral)|utmcsr=okcupid.com|utmcct=/forum|utmcmd=referral; username=eplossl; ru15836121488829250307=6888236873690812350%%7CSluzin%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/52x0/202x150/2/16539671370562616888.jpeg___82_82_82_82_2dfc4813_.jpg%%7CKennesaw%%7C29%%7Cf%%7Cs*3093072485526931756%%7Clovensunshine%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/0x63/358x421/2/13298148684615234817.jpeg___82_82_82_82_2dfc4813_.jpg%%7CGray%%7C24%%7Cf%%7Cs*17993932046903613536%%7Csweetgagrl%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/64x0/352x288/2/1414773H",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61020; eid: 2; uid: 6; gid: 1; rep: 0; cat: 192; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=312",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: feeds.feedburner.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61021; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=312",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: feeds.feedburner.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61022; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=315",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: feeds.feedburner.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61023; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=313",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: feeds.feedburner.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61024; eid: 2; uid: 2; gid: 1; rep: -127; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=433",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61025; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=599",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 492",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61026; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 12",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61027; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=480",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61028; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=500",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61029; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=488",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61030; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=609",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:26 GMT",
 "If-None-Match: 1b22-488c1e36",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61031; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=612",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:16 GMT",
 "If-None-Match: 12e6-488c1e2c",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61032; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=603",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:26 GMT",
 "If-None-Match: 9cd-488c1e36",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61033; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=509",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61034; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1910",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*",
 "Referer: http://messenger.yahoo.com/external/plugins/ct/stock_chart_viewer.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: messenger.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFTHhh6EY@@]==3Z@W)n@Pd3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61035; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122 157; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61036; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2008",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://insider.msg.yahoo.com/client_ad.php?p=842673",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: us.bc.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0Hhc5E",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61037; eid: 2; uid: 2; gid: 1; rep: -127; cat: 141; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61038; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61039; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61040; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61041; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61042; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61043; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61044; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.168.60.3",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: req-hdr=0, null-body=128",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.168.60.3",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: req-hdr=0, null-body=128",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.21",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.21",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 64.238.216.105",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: req-hdr=0, null-body=128",
 "User-Agent: check_http/1.96 (nagios-plugins 1.4.5)",
 "Host: swps-proxy.securecomputing.com",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=192",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "HEAD http://downloads.sophos.com/downloads/ide/430_ides.zip HTTP/1.1",
 "TE: deflate,gzip;q=0.3",
 "Connection: TE, close",
 "Host: downloads.sophos.com",
 "User-Agent: SCC Downloader/v1.0 Fetchmaster",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61045; eid: 2; uid: 2; gid: 1; rep: 0; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=191",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "TE: deflate,gzip;q=0.3",
 "Connection: TE, close",
 "Host: downloads.sophos.com",
 "User-Agent: SCC Downloader/v1.0 Fetchmaster",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61046; eid: 2; uid: 2; gid: 1; rep: 0; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img3.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61047; eid: 2; uid: 2; gid: 1; rep: 0; cat: 177; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img1.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61048; eid: 2; uid: 2; gid: 1; rep: 0; cat: 177; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=486",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_AL.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61049; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 75.45.245.220",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "CONNECT usrpc.securecomputing.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: usrpc.securecomputing.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61050; eid: 56; uid: 46; gid: 38; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img2.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61051; eid: 2; uid: 2; gid: 1; rep: 0; cat: 112 147; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=1566",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://mail.google.com/mail/channel/bind?at=xn3j37l23r1zl5l6zkwu4avt58xm34&VER=6&it=47099997&SID=FFFC2FDA088318E5&RID=49613&zx=3181sm-hp5brb&t=1 HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; InfoPath.1; .NET CLR 2.0.50727)",
 "Host: mail.google.com",
 "Content-Length: 60",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img2.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61052; eid: 2; uid: 2; gid: 1; rep: 0; cat: 112 147; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=449",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61053; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img4.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61054; eid: 2; uid: 2; gid: 1; rep: 0; cat: 177; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=222",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://ws.ourpictures.com/ws HTTP/1.1",
 "Content-Length: 563",
 "Content-Type: text/xml; charset=utf-8",
 "SOAPAction: urn:OP#GetVersion",
 "Accept: text/xml",
 "Host: ws.ourpictures.com",
 "User-Agent: Microsoft-ATL-Native/7.00",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61055; eid: 2; uid: 2; gid: 1; rep: 0; cat: 136; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=133",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 208.73.116.2",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.23",
 "CONNECT exg3.exghost.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: exg3.exghost.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 81; uid: 58; gid: 63; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61056; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img5.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61057; eid: 2; uid: 2; gid: 1; rep: 0; cat: 112 147; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=691",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: img5.catalog.video.msn.com",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: MC1=V=3&GUID=8f655b8c96cc4257a17e33fd78199a24; CULTURE=EN-US; MUID=4DBED1C70F734A5CB825FFF2AAC5B28E; ANON=A=2C077DDF53B76AC5168C14A8FFFFFFFF&E=6e6&W=1; NAP=V=1.7&E=68c&C=wPdikcp0BE6eISD2e-RORwHigVNwTwFZMENYUobjxBisVcaghiXalA&W=2; mh=MSFT; s_vsn_msnbcom_1=5523264945857; MSNTVID=8f655b8c96cc4257a17e33fd78199a24",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61058; eid: 2; uid: 2; gid: 1; rep: 0; cat: 112 147; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 75.45.245.220",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "CONNECT usrpc.securecomputing.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: usrpc.securecomputing.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61059; eid: 56; uid: 46; gid: 38; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=756",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Referer: http://www.amtrak.com/html/stations_AL.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61060; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=192",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "HEAD http://downloads.sophos.com/downloads/ide/431_ides.zip HTTP/1.1",
 "TE: deflate,gzip;q=0.3",
 "Connection: TE, close",
 "Host: downloads.sophos.com",
 "User-Agent: SCC Downloader/v1.0 Fetchmaster",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61061; eid: 2; uid: 2; gid: 1; rep: 0; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=191",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "TE: deflate,gzip;q=0.3",
 "Connection: TE, close",
 "Host: downloads.sophos.com",
 "User-Agent: SCC Downloader/v1.0 Fetchmaster",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61062; eid: 2; uid: 2; gid: 1; rep: 0; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=479",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://24.wcmcs.net/m/fc.aspx?sid=1059&poid=WM",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: 24.wcmcs.net",
 "Connection: Keep-Alive",
 "Cookie: wcm_userkey=cf6be9e1-4205-48e3-81b1-cac791c9cc1a; ASP.NET_SessionId=bvo0wj55zchu2i55nfjetq45; wcm_sessionid=22560969",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61063; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 75.45.245.220",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.23",
 "CONNECT usrpc.securecomputing.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: usrpc.securecomputing.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61064; eid: 56; uid: 46; gid: 38; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2484",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 216.235.158.34",
 "Host: www.okcupid.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/javascript, text/html, application/xml, text/xml, */*",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "X-Requested-With: XMLHttpRequest",
 "X-Prototype-Version: 1.6.0.2",
 "Referer: http://www.okcupid.com/forum?tid=1616731738136776031",
 "Cookie: __qca=1208617279-56396514-56980535; __utma=132982672.1882520404.1216291135.1218688172.1218698304.122; __utmz=132982672.1217307396.75.3.utmccn=(referral)|utmcsr=okcupid.com|utmcct=/forum|utmcmd=referral; username=eplossl; ru15836121488829250307=6888236873690812350%%7CSluzin%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/52x0/202x150/2/16539671370562616888.jpeg___82_82_82_82_2dfc4813_.jpg%%7CKennesaw%%7C29%%7Cf%%7Cs*3093072485526931756%%7Clovensunshine%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/0x63/358x421/2/13298148684615234817.jpeg___82_82_82_82_2dfc4813_.jpg%%7CGray%%7C24%%7Cf%%7Cs*17993932046903613536%%7Csweetgagrl%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/64x0/352x288/2/1414773HU7",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61065; eid: 2; uid: 6; gid: 1; rep: 0; cat: 192; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=622",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.nbcolympics.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: ak-mobile-detected=no; s_cc=true; s_nr=1218657732998; s_lv=1218668526821; s_sq=%%5B%%5BB%%5D%%5D; __qca=1207077133-42804709-87523247; __qcb=1225013213",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61066; eid: 2; uid: 2; gid: 1; rep: 0; cat: 146; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1627",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61067; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=192",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "HEAD http://downloads.sophos.com/downloads/ide/433_ides.zip HTTP/1.1",
 "TE: deflate,gzip;q=0.3",
 "Connection: TE, close",
 "Host: downloads.sophos.com",
 "User-Agent: SCC Downloader/v1.0 Fetchmaster",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61068; eid: 2; uid: 2; gid: 1; rep: 0; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 46",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61069; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1800",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://insider.msg.yahoo.com/client_ad.php?p=842673",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: insider.msg.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: ebNewBandWidth_.insider.msg.yahoo.com=3325%%3A1214600642588; F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFTd3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=H,HHhc5E:o@@6==3@6HP !915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61070; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122 157; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=479",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://24.wcmcs.net/m/fc.aspx?sid=1059&poid=WM",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: 24.wcmcs.net",
 "Connection: Keep-Alive",
 "Cookie: wcm_userkey=cf6be9e1-4205-48e3-81b1-cac791c9cc1a; ASP.NET_SessionId=bvo0wj55zchu2i55nfjetq45; wcm_sessionid=22560969",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61071; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=1626",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://mail.google.com/mail/channel/bind?at=xn3j3614qdid9j1mu3drhslywvd8b9&VER=6&it=187220758&SID=9CC57A8A87B3291A&RID=20149&zx=5hh415-vt8nty&t=1 HTTP/1.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Content-Type: application/x-www-form-urlencoded",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Content-Length: 61",
 "Cookie: GX=DQAAAHYAAAAbRrZCLsXh5ZcGGwEILrrjTVdhJqfJyGbe_lJexgdJdk7NRavoCGCt434pDemR7kZp2t81DKajo1P_1CE8_KvtmIOgK0TT4rEJvhviaOVZ53_OkN0WfVgymfxGW1B3f8uejfKXvgnTDmNiUymv-tZFpa5QFDfNEu2OXQDupfDD6A; S=gmail=7U1b1OZo3Lf33qU6HKFFjw:gmail_yj=wTaX_WKW1kMGxdZ7jLbt-w:gmproxy=YSSHn1lmwE8:gmproxy_yj=nerwXYjZ-x8:gmproxy_yj_sub=a__riccSVj4; GMAIL_AT=xn3j3614qdid9j1mu3drhslywvd8b9; gmailchat=steve.goers@gmail.com/720839; PREF=ID=67c802120d3b2caf:TM=1217710846:LM=1217714251:GM=1:S=5SPkSiho_Do6198S; NID=13=jWExaO4XhQ6HJgCJkQAO5prppQAukiQxHR-MJO4WsvLB6KUy5Fs4u4ifnnMpVp-Aphi65HLhbEL@@==3@PKePPeUTKGQkjs0chBU63ESsuDkViB9apjLihTVgy_gdzUjER48yXqxxnkqvajJN; __utmxx=173272373.; __utmx=173272373.; TZ=300; GMAIL_RTT=31; SID=DQAAAHUAAABnxDxI0SMR7WQc9UcPSduPf4Z9it84MoCGstefTLXcdJJU5Jan5wLxGqzjpl8QAneb8551wkvjQyobg25EwCjEflqGVHek8bwefKnpMX_LPrjxumD5wMyNIzdiwOiidd0u_06Bq6WatW_lT8DS9vyUQUlbFwJ3hhV43jqA556z5w",
 "Pragma: no-cache",
 "Cache-Control: no-cache",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 75.45.245.220",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "CONNECT usrpc.securecomputing.com:443 HTTP/1.0",
 "User-Agent: MSRPC",
 "Host: usrpc.securecomputing.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61072; eid: 56; uid: 46; gid: 38; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=657",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.nbcolympics.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: ak-mobile-detected=no; s_cc=true; s_nr=1218657732998; s_lv=1218668526821; s_sq=%%5B%%5BB%%5D%%5D; __qca=1207077133-42804709-87523247; __qcb=1225013213",
 "If-None-Match: 86e84077edfdc81:0",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61073; eid: 2; uid: 2; gid: 1; rep: 0; cat: 146; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=730",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: www.nbcolympics.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Cookie: ak-mobile-detected=no; s_cc=true; s_nr=1218657732998; s_lv=1218668526821; s_sq=%%5B%%5BB%%5D%%5D; __qca=1207077133-42804709-87523247; __qcb=1225013213",
 "If-Modified-Since: Thu, 14 Aug 2008 03:10:12 GMT",
 "If-None-Match: 6e6e6044bbfdc81:790",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61074; eid: 2; uid: 2; gid: 1; rep: 0; cat: 146; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=562",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:03:22 GMT",
 "If-None-Match: 1eb1-488c1dba",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61075; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61076; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=581",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:08 GMT",
 "If-None-Match: 1aa-488c1e24",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61077; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61078; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 12",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61079; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, req-body=598",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "POST http://www.amtrak.com/convert/app HTTP/1.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Content-Type: application/x-www-form-urlencoded",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Content-Length: 12",
 "Connection: Keep-Alive",
 "Cache-Control: no-cache",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61080; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=590",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:52 GMT",
 "If-None-Match: 8ae-488c1e50",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61081; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=620",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:16 GMT",
 "If-None-Match: 10cc-488c1e2c",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61082; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=617",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "If-Modified-Since: Sun, 27 Jul 2008 07:05:15 GMT",
 "If-None-Match: 1618-488c1e2b",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61083; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61084; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61085; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61086; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61087; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=615",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "Referer: http://mail.aliastec.net/webmail/index.php",
 "Cookie: IMAIL_TEST_COOKIE=test; IMAIL_SESS_KEY_1218705179-45360=pLblo2eiNGfPG629",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61088; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=614",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "Referer: http://mail.aliastec.net/webmail/index.php",
 "Cookie: IMAIL_TEST_COOKIE=test; IMAIL_SESS_KEY_1218705179-45360=pLblo2eiNGfPG629",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61089; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=627",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "Referer: http://mail.aliastec.net/webmail/index.php",
 "Cookie: IMAIL_TEST_COOKIE=test; IMAIL_SESS_KEY_1218705179-45360=pLblo2eiNGfPG629",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61090; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1613",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Cookie: __utmx=173272373.; __utma=173272373.1916445834.1180453148.1200494341.1200575782.141; GX=DQAAAHUAAACBAXSmRS0cN0bv6fTDLgdtrIRtnYo4EzCxrTne3unkucKlWz0zbUHI94RoH-yP7lJLCiySGwDdVhfTMNlfuAbTF9axtT1_kl8xeJn6NaA7NU4v6LHxJiO3QGtSG3FCGRtpdnAhAsU1E6i819YG-ZmaxLntIbO5lqO4l3-WjTSoWw; S=gmail=zLBRf7qXrbHpgoCWZ7mFHw:gmail_yj=4sI8h5ocOk0L7ewbWlW08w:gmproxy=CNrihF13nR4:gmproxy_yj=wrDdFO6EXG0:gmproxy_yj_sub=6qRbJkPOy94; GMAIL_AT=xn3j36057mmea44j5srgmuruoja1ep; gmailchat=jdtschida@gmail.com/489762; rememberme=false; PREF=ID=77bae0bafa4c66ec:FF=1:TM=1180453130:LM=1210680165:GM=1:S=RSZyBmX54dkjie7n; NID=13=mmQGXgjg4JETlgO4YHur1mwdmasqzz2Ey7MK8HLhc5Ep@@==3@Cl6)kP 4huVs7vt311e2d9v9V5jvK38OyvzabvJrXnO-XgHzFynFjkcV2mBAeAn2WxvgFMv-8StXk8edbf7xgpZUN798iifOo0P; TZ=300; GMAIL_RTT=34; SID=DQAAAHMAAACqJ137NorN2HIoYsDLSI9sYF1xpON8wgy5WhOI5EMCUKBjLcO2T7Mi1ASVhyloksXTOUKPYI9uXobgqoLtGgnQP1M0h-BSGb3fSE28c0T6ZEGrXWNG5bl2suUpvHoWUqyePWKHNdzzWJ9vRsI8G40jqikSQY-2tzLLqWuuKoMNSw; S=photos_html=usUdjzYKHFJ94Fhdwll66w",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61091; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61092; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61093; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=151",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTPS-Proxy6.7.2",
 "X-Client-IP: 75.45.245.220",
 "X-Encapsulated-Protocol: HTTPS",
 "X-Proxy-Addr: 192.168.60.22",
 "User-Agent: MSRPC",
 "Host: usrpc.securecomputing.com",
 "Content-Length: 0",
 "Proxy-Connection: Keep-Alive",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61094; eid: 56; uid: 46; gid: 38; rep: -127; cat: 105 175; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61095; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=196",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 208.73.116.2",
 "Accept: */*",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Win32)",
 "Host: swps-proxy.securecomputing.com",
 "Cache-Control: max-age=0",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: res-hdr=0, res-body=102",
 "Server: SCUR ICAP server (snare)",
 "Content-Type: application/x-ns-proxy-autoconfig",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=754",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, */*",
 "Referer: http://www.amtrak.com/html/stations_A.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61096; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Host: 192.168.61.51",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: null-body=0",
 "Methods: REQMOD",
 "Max-Connections: 1024",
 "Transfer-Complete: *",
 "Service-ID: snare",
 "Service: SCUR ICAP server (snare)",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61097; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Host: 192.168.61.51",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: null-body=0",
 "Methods: REQMOD",
 "Max-Connections: 1024",
 "Transfer-Complete: *",
 "Service-ID: snare",
 "Service: SCUR ICAP server (snare)",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=658",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/css,*/*;q=0.1",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "Referer: http://mail.aliastec.net/webmail/read_message.php?user=1218705179-45360&folder=INBOX&id=888&uid=20060&start=0&num_msgs=889&sort_field=DATE&sort_order=DESC",
 "Cookie: IMAIL_TEST_COOKIE=test; IMAIL_SESS_KEY_1218705179-45360=pLblo2eiNGfPG629",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61098; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Host: 192.168.61.51",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "Encapsulated: null-body=0",
 "Methods: REQMOD",
 "Max-Connections: 1024",
 "Transfer-Complete: *",
 "Service-ID: snare",
 "Service: SCUR ICAP server (snare)",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2484",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 216.235.158.34",
 "Host: www.okcupid.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/javascript, text/html, application/xml, text/xml, */*",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "X-Requested-With: XMLHttpRequest",
 "X-Prototype-Version: 1.6.0.2",
 "Referer: http://www.okcupid.com/forum?tid=1616731738136776031",
 "Cookie: __qca=1208617279-56396514-56980535; __utma=132982672.1882520404.1216291135.1218688172.1218698304.122; __utmz=132982672.1217307396.75.3.utmccn=(referral)|utmcsr=okcupid.com|utmcct=/forum|utmcmd=referral; username=eplossl; ru15836121488829250307=6888236873690812350%%7CSluzin%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/52x0/202x150/2/16539671370562616888.jpeg___82_82_82_82_2dfc4813_.jpg%%7CKennesaw%%7C29%%7Cf%%7Cs*3093072485526931756%%7Clovensunshine%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/0x63/358x421/2/13298148684615234817.jpeg___82_82_82_82_2dfc4813_.jpg%%7CGray%%7C24%%7Cf%%7Cs*17993932046903613536%%7Csweetgagrl%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/64x0/352x288/2/1414773*H)T",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61099; eid: 2; uid: 6; gid: 1; rep: 0; cat: 192; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1551",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 208.73.116.2",
 "Accept: */*",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Windows Live Messenger 8.1.0178)",
 "Host: rad.msn.com",
 "Proxy-Connection: Keep-Alive",
 "Cookie: MSPAuth=9THid5YxaCaF262l1yKrkW6DQv02cgtXIATrPRcksty8GJ0DlkF0ipnapmqaLtAytxhDkja0fxA3pchA*aEKxtm1T3G886YWKotTcTpyHtXj5dyKd0L7z7XeNtu6rO!mL1uPYsU18O7BI$; MSPProf=9yibySHXsDibcvBflqf4gr*8G8romBDTwBaPo6vLkLEfV3xS5DE1oN83EjcEyghI7QuZoHecok0OHevqHRcENcDAkCZymwDeKeYoLvZ02QAX4464jWKY1teiYymRjD2FWeES3Tw2ld74cWgAv0tVN!3NDfJzGhDjkcVP!y1FA!xFfJN6ogr0Awwg$$; MSNRPSShare=1; MyMsn=335c21d0476bd05391f60da51be1405a1200327530; s_cc=true; s_sq=msnportalenustoday%%3D%%2526pid%%253DTitle1%%2526pidt%%253D1%%2526oid%%253Dhttp%%25253A//today.msnbc.msn.com/id/26124639/%%25253FGT1%%25253D43001%%2526ot%%253DA%%2526oi%%253D665; ANON=A=10F545EED9D1E5DE95DD5D76FFFFFFFF&E=6cc&W=1; CULTURE=EN-US; MC1=V=3&GUID=7d787d12132243bebf6cd33bcd3c5c0b; MUID=B1DC65C2F2114863A77F3C3F2F7B4D38; NAP=V=1.7&E=6e3&C=yilSVvHdu88yGDABjO4FA_7stZd-Da38kTvedeMxueCJy10HMSSXsQ&W=2; mh=MSFT; s_vsn_msnbcom_1=17221371940; slideshow=A:0; MSNTVID=cb8ee97c3dbe419495e51df215819b56; zip=z:32561|la:30.36|lo:-87.17*HOOhc5EAN@@==3<@!<sP8|c:us|hr:1; __qca=1188410671-76322209-56148011; AxData=; FC01=FB=AgEA3AoJIOAB; FC05=FB=AgEAZwrbFTgB; FC06=FB=AgEAegrsFNAC; FC07=FB=AgEAdAqRFqAC; FC08=FB=AgEApgp+GzAB; FC09=FB=AgEAsQqTFogB; ebNewBandWidth_.rad.msn.com=273%%3A1205526073203; FC00=FB=; FC02=FB=; FC03=FB=; FC04=FB=",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61100; eid: 81; uid: 58; gid: 63; rep: 0; cat: 154; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=531",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://www.amtrak.com/servlet/ContentServer?pagename=Amtrak/HomePage",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: www.amtrak.com",
 "Connection: Keep-Alive",
 "Cookie: CVTSESSION=9DA704EDC18C4BAE0C3C21D147B81FEE; Engage1UID=14125298",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61101; eid: 2; uid: 2; gid: 1; rep: 0; cat: 152; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=814",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 69.141.82.28",
 "Host: mail.aliastec.net",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.16) Gecko/20080702 Firefox/2.0.0.16",
 "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Proxy-Connection: keep-alive",
 "Referer: http://mail.aliastec.net/webmail/read_message.php?user=1218705179-45360&folder=INBOX&id=888&uid=20060&start=0&num_msgs=889&sort_field=DATE&sort_order=DESC",
 "Cookie: IMAIL_TEST_COOKIE=test; IMAIL_SESS_KEY_1218705179-45360=pLblo2eiNGfPG629",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61102; eid: 61; uid: 52; gid: 43; rep: 0; cat:; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1910",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*",
 "Referer: http://messenger.yahoo.com/external/plugins/ct/stock_chart_viewer.html",
 "Accept-Language: en-us",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: messenger.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFT0Hhh6EXf@@P==3@mLb%%P|d3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61103; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122 157; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1800",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://insider.msg.yahoo.com/client_ad.php?p=842673",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2)",
 "Host: insider.msg.yahoo.com",
 "Connection: Keep-Alive",
 "Cookie: ebNewBandWidth_.insider.msg.yahoo.com=3325%%3A1214600642588; F=a=sncCKXIMvT9yYlXp46OwzY4m_hSEiby6DyNL5vbZfDdTCZwaP8PPBXXxhJLaoII1PENwZwWGJMPTjifIg34tiI9_8rLy6NVKlYuDPoID7Lh3&b=Q1QW; YLS=v=1&p=0&n=9; B=78jvjch44vo28&b=4&d=h3jl3yxpYEJjwNMDsXV_avfq5n.feRn22n3K.A--&s=5c; U=mt=vYhLo52MhYkv7aA6J5NUVHcpyNGf.B_l2f21&ux=TB.TIB&un=915nt39k23ucv; PL=V=1.1&d=1V0GJTmIS.yoaoIPe.rJNSxuPGO0lWlfUqz0P7LtfQjIMAb.i7BKkrLKRWF3dqIOZfTBWKHfJ_c7Y6T8lGDjYnl.AMDBlgNcXc1Ho4N.gAoizuWl52dHcgkFlg--; C=mg=1; Y=v=1&n=915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; T=z=SwvmIBS2EnIBIpphLU4ej2ETzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAQuCnPJbJMXv&ks=EAAT9g24hTF4NK5oOgawJ3F5A--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFTd3ZtSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTgxMTY2MjYmcHM9a0hBQ2xobzZ1Nm9zLlFUOXBPVmJTZy0t; msgrY=v=1&n=0HtEHHhc5E:p@@==3@_6/P c915nt39k23ucv&l=h44i4uuu/o&p=m271ko6c13000200&r=44&lg=en-US&intl=us&np=1; msgrT=z=KUuoIBKaDpIBtiHuU/8u7mvTzc3BjIwN080TjQ0Tg--&a=YAE&sk=DAAKxpUEbMeIKp&ks=EAASlE5DX5ftcfoPoZkQGjeGA--~A&d=c2wBT0RBd0FUVTNNRGd6T1RNek9RLS0BYQFZQUUBaWcBUWlmZ0JBAWcBUEwyNzRZMlRXU1RYVERYTVRaVEdOTkhZSkUBb2sBWlcwLQF6egFLVXVvSUJnV0EBdGlwAUFmajFCQQ--&af=QWZBQlJsJnRzPTEyMTg2MzUwMTgmcHM9MFAwMVIua3hydWFpeW00bE90YzZDUS0t; msgrLYC=l_v=2&l_lv=10&l_l=10d0d05ewz&l_s=qz54t4qwrsqquyv51w0z4xxwtx31x1t0&l_lid=1jkci0c&l_r=ea&l_um=0_0_0_0_0; msgrLYS=l_fh=1&l_vo=myla",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61104; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122 157; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=1556",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Host: mail.google.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/2008052906 Firefox/3.0",
 "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "Referer: http://mail.google.com/mail/?ui=2&view=js&name=js&ver=MKdHACveLj0&am=R_E4pdz3cTG-410",
 "Cookie: GX=DQAAAHQAAAD9i81WKcOnEvQXVIQpYc-4tniDvyVLxleicplZWjD0xPQ5oDaRZ1aMimd_XLCdzsgdIqsZiN8NqiHPp5B2fGczCKfFRuEN_FCQMXdp9X1dgfdnwl1oVN5Sw6rcHfsb7iztnyQtPReFk-yfnCMguHNx5ivw5Dqu8QA0mh58VaM_UA; S=quotestreamer=yr4BsfdmBTgQ4nGeFXKGew:gmail=lsiseoaOZUDGd3kVmL4NEQ:gmail_yj=1kXiDdKhrhQPPa12dO05tA:gmproxy=gNo2KX87aZE:gmproxy_yj=8UcvbuD0_Wg:gmproxy_yj_sub=FSDM1c6RZKc; GMAIL_AT=xn3j36sv24d5757moxxnr5ee2smucs; gmailchat=joey.bobblehead@gmail.com/179140; NID=13=ko2MOSH8SE8__tMUa_c51f9fmCHrzbUS0wWK5T8yK8gzNjZp5DXF3bC92UVPk7EQ6ZVitb7vgyeRXJPajUCBRs8N8nX9wtfjshqwYPFqBTs43cN1Wm0Ud-hFOyzLhhty; PREF=ID=62bba4c265eff849:TM=1185332533:LM=1218662800:L=01HnTThc5EFS@@==3<@!G+2PWskDHT0A:GM=1:S=KQCRg8n7wayV3aY8; S=quotestreamer=yr4BsfdmBTgQ4nGeFXKGew; TZ=300; GMAIL_RTT=54; SID=DQAAAHEAAAA-yg0NF2HYX0Kt3uQywJ57fRz5S2ljwoT5__w1XT2Gca5LD9S0HSp7Htmb-Ny64zJG7AMXuRk2QoqY6fmVY_VwEXubyqKpr9vJ8sdlrPTQDT-_-5PshvRUBU312plsWaT3Nbi3tHSSs5QnQMkhLLZlhT7pNipHgFgBSL6H_RV3TA",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-Attribute: eid: 2; uid: 2; gid: 1; action: allow; detail: whitelisted",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=479",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-us",
 "Referer: http://24.wcmcs.net/m/fc.aspx?sid=1059&poid=WM",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)",
 "Host: 24.wcmcs.net",
 "Connection: Keep-Alive",
 "Cookie: wcm_userkey=cf6be9e1-4205-48e3-81b1-cac791c9cc1a; ASP.NET_SessionId=bvo0wj55zchu2i55nfjetq45; wcm_sessionid=22560969",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61105; eid: 2; uid: 2; gid: 1; rep: 0; cat: 178; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=586",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 192.55.214.1",
 "Accept: */*",
 "Accept-Language: en-US",
 "Referer: http://cache.static.userplane.com/presence/presence.swf",
 "UA-CPU: x86",
 "Accept-Encoding: gzip, deflate",
 "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; Zune 2.5)",
 "Host: 03.presence.userplane.com",
 "Connection: Keep-Alive",
 "Cookie: NSC_wjq_qsftfodf.e_03_80=9e153c2629a2",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61106; eid: 2; uid: 2; gid: 1; rep: 0; cat: 122; action: allow",
 "Allow: 204",
 "Encapsulated: req-hdr=0, null-body=2488",
 "Host: 192.168.61.51",
 "User-Agent: WW-HTTP-Proxy 6.7.2",
 "X-Client-IP: 216.235.158.34",
 "Host: www.okcupid.com",
 "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1",
 "Accept: text/javascript, text/html, application/xml, text/xml, */*",
 "Accept-Language: en-us,en;q=0.5",
 "Accept-Encoding: gzip,deflate",
 "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7",
 "Keep-Alive: 300",
 "Connection: keep-alive",
 "X-Requested-With: XMLHttpRequest",
 "X-Prototype-Version: 1.6.0.2",
 "Referer: http://www.okcupid.com/forum?tid=1616731738136776031",
 "Cookie: __qca=1208617279-56396514-56980535; __utma=132982672.1882520404.1216291135.1218688172.1218698304.122; __utmz=132982672.1217307396.75.3.utmccn=(referral)|utmcsr=okcupid.com|utmcct=/forum|utmcmd=referral; username=eplossl; ru15836121488829250307=6888236873690812350%%7CSluzin%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/52x0/202x150/2/16539671370562616888.jpeg___82_82_82_82_2dfc4813_.jpg%%7CKennesaw%%7C29%%7Cf%%7Cs*3093072485526931756%%7Clovensunshine%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/0x63/358x421/2/13298148684615234817.jpeg___82_82_82_82_2dfc4813_.jpg%%7CGray%%7C24%%7Cf%%7Cs*17993932046903613536%%7Csweetgagrl%%7Chttp%%3A//cdn.okcimg.com/php/load_okc_image.php/images/64x0/352x288/2/14147739HSj",
 "ISTag: 2.0.0-rc6-1763",
 "Server: snare/2.0.0-rc6",
 "X-ICAP-RESPMOD-Profile: NoRESPMOD",
 "X-Attribute: qid: 61107; eid: 2; uid: 6; gid: 1; rep: 0; cat: 192; action: allow",
 NULL };

static void
free_string(void *s){
	Free(s);
}

static int
test_lrupatadd(void){
	ustring u = USTRING_INITIALIZER;
	const char **input;
	struct lrupat *lp;
	int ret = -1;

	if((lp = create_lrupat(free_string)) == NULL){
		goto done;
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	reset_ustring(&u);
	printf(" Adding keys to lrupat...\n");
	for(input = LRUPAT_INPUT ; *input ; ++input){
		char *erp,*erpy;
		char *colon;

		if((erp = Strdup(*input)) == NULL){
			goto done;
		}
		if((colon = strchr(erp,':')) == NULL){
			fprintf(stderr," Bad input line: %s\n",erp);
			Free(erp);
			goto done;
		}
		*colon = '\0';
		if((erpy = Strdup(colon + 1)) == NULL){
			Free(erp);
			goto done;
		}
		if(add_lrupat(lp,erp,erpy)){
			fprintf(stderr," Failed adding %s:%s\n",erp,erpy);
			Free(erpy);
			Free(erp);
			goto done;
		}
		Free(erp);
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	reset_ustring(&u);
	ret = 0;

done:
	destroy_lrupat(lp);
	return ret;
}

static int
test_lrupatcheck(void){
	ustring u = USTRING_INITIALIZER;
	const char *LRUPAT_CHECKS[] = {
		"LRU", "LRUPAT 0", "LRUPAT 1",
		"TAPURL 2", "TAPURL 3", "TAP", NULL
	},**input;
	struct lrupat *lp;
	int ret = -1;

	if((lp = create_lrupat(free_string)) == NULL){
		goto done;
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	reset_ustring(&u);
	printf(" Adding keys to lrupat...\n");
	for(input = LRUPAT_CHECKS ; *input ; ++input){
		char *erp;

		if((erp = Strdup(*input)) == NULL){
			goto done;
		}
		if(add_lrupat(lp,*input,erp)){
			fprintf(stderr," Failed adding %s.\n",*input);
			Free(erp);
			goto done;
		}
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	for(input = LRUPAT_CHECKS ; *input ; ++input){
		void *r;

		if(lookup_lrupat(lp,*input,&r) == 0){
			fprintf(stderr," Couldn't find %s.\n",*input);
			goto done;
		}
		if(strcmp(r,*input)){
			fprintf(stderr," Invalid result for %s (%s).\n",*input,(const char *)r);
			goto done;
		}
	}
	reset_ustring(&u);
	ret = 0;

done:
	destroy_lrupat(lp);
	return ret;
}

static int
test_lrupatchecknocase(void){
	ustring u = USTRING_INITIALIZER;
	const char *LRUPAT_CHECKS[] = {
		"LRU", "LRUPAT 0", "LRUPAT 1",
		"TAPURL 2", "TAPURL 3", "TAP", NULL
	},**input;
	struct lrupat *lp;
	int ret = -1;

	if((lp = create_lrupat(free_string)) == NULL){
		goto done;
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	reset_ustring(&u);
	printf(" Adding keys to lrupat...\n");
	for(input = LRUPAT_CHECKS ; *input ; ++input){
		char *erp;

		if((erp = Strdup(*input)) == NULL){
			goto done;
		}
		if(add_lrupat_nocase(lp,*input,erp)){
			fprintf(stderr," Failed adding %s.\n",*input);
			Free(erp);
			goto done;
		}
	}
	if(stringize_lrupat(&u,lp)){
		goto done;
	}
	printf(" Stringized: %s\n",u.string);
	for(input = LRUPAT_CHECKS ; *input ; ++input){
		void *r;

		if(lookup_lrupat_nocase(lp,*input,&r) == 0){
			fprintf(stderr," Couldn't find %s.\n",*input);
			goto done;
		}
		if(strcmp(r,*input)){
			fprintf(stderr," Invalid result for %s (%s).\n",*input,(const char *)r);
			goto done;
		}
	}
	reset_ustring(&u);
	ret = 0;

done:
	destroy_lrupat(lp);
	return ret;
}

const declared_test LRUPAT_TESTS[] = {
	{	.name = "lrupatnullkill",
		.testfxn = test_lrupatnullkill,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	},
	{	.name = "lrupatbasic",
		.testfxn = test_lrupatbasic,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	},
	{	.name = "lrupatadd",
		.testfxn = test_lrupatadd,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	},
	{	.name = "lrupatcheck",
		.testfxn = test_lrupatcheck,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	},
	{	.name = "lrupatchecknocase",
		.testfxn = test_lrupatchecknocase,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	},
	{	.name = NULL,
		.testfxn = NULL,
		.expected_result = EXIT_TESTSUCCESS,
		.sec_required = 0, .mb_required = 0, .disabled = 0,
	}
};
