/*
 * Copyright 2011 Florian Albrechtskirchinger. All rights reserved.
 *
 * This file is part of libat32uc3.
 *
 * libat32uc3 is licensed under the Simplified BSD License
 * (the "License").
 *
 * A copy of the License should be included in the distribution.
 * Alternatively, you may obtain a copy of the License at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Author(s): Florian Albrechtskirchinger <falbrechtskirchinger@gmail.com>
 */

#ifndef USB_H_
#define USB_H_

#include <at32uc3/inttypes.h>

/// USB Slave Interface Address
#define USB_SLAVE_INTERFACE_ADDRESS			0xe0000000
/// USB Configuration Interface Address
#define USB_CONFIGURATION_INTERFACE_ADDRESS		0xfffe0000

/// USB Device General Control Register (Read/Write)
#define USB_REG_UDCON					0x00000000
/// USB Device General Control Register USB Address offset
#define USB_REG_UDCON_UADD_OFFSET			0
/// USB Device General Control Register USB Address size
#define USB_REG_UDCON_UADD_SIZE				7
/// USB Device General Control Register USB Address mask
#define USB_REG_UDCON_UADD_MASK				(((1 << USB_REG_UDCON_UADD_SIZE) - 1) << USB_REG_UDCON_UADD_OFFSET)
/// USB Device General Control Register Address Enable offset
#define USB_REG_UDCON_ADDEN_OFFSET			7
/// USB Device General Control Register Address Enable size
#define USB_REG_UDCON_ADDEN_SIZE			1
/// USB Device General Control Register Address Enable mask
#define USB_REG_UDCON_ADDEN_MASK			(((1 << USB_REG_UDCON_ADDEN_SIZE) - 1) << USB_REG_UDCON_ADDEN_OFFSET)
/// USB Device General Control Register Detach offset
#define USB_REG_UDCON_DETACH_OFFSET			8
/// USB Device General Control Register Detach size
#define USB_REG_UDCON_DETACH_SIZE			1
/// USB Device General Control Register Detach mask
#define USB_REG_UDCON_DETACH_MASK			(((1 << USB_REG_UDCON_DETACH_SIZE) - 1) << USB_REG_UDCON_DETACH_OFFSET)
/// USB Device General Control Register Remote Wake-Up offset
#define USB_REG_UDCON_RMWKUP_OFFSET			9
/// USB Device General Control Register Remote Wake-Up size
#define USB_REG_UDCON_RMWKUP_SIZE			1
/// USB Device General Control Register Remote Wake-Up mask
#define USB_REG_UDCON_RMWKUP_MASK			(((1 << USB_REG_UDCON_RMWKUP_SIZE) - 1) << USB_REG_UDCON_RMWKUP_OFFSET)
/// USB Device General Control Register Low-Speed Mode Force offset
#define USB_REG_UDCON_LS_OFFSET				12
/// USB Device General Control Register Low-Speed Mode Force size
#define USB_REG_UDCON_LS_SIZE				1
/// USB Device General Control Register Low-Speed Mode Force mask
#define USB_REG_UDCON_LS_MASK				(((1 << USB_REG_UDCON_LS_SIZE) - 1) << USB_REG_UDCON_LS_OFFSET)
/// USB Device Global Interrupt Register (Read)
#define USB_REG_UDINT					0x00000004
/// USB Device Global Interrupt Register Suspend Interrupt Flag offset
#define USB_REG_UDINT_SUSP_OFFSET			0
/// USB Device Global Interrupt Register Suspend Interrupt Flag size
#define USB_REG_UDINT_SUSP_SIZE				1
/// USB Device Global Interrupt Register Suspend Interrupt Flag mask
#define USB_REG_UDINT_SUSP_MASK				(((1 << USB_REG_UDINT_SUSP_SIZE) - 1) << USB_REG_UDINT_SUSP_OFFSET)
/// USB Device Global Interrupt Register Start of Frame Interrupt Flag offset
#define USB_REG_UDINT_SOF_OFFSET			2
/// USB Device Global Interrupt Register Start of Frame Interrupt Flag size
#define USB_REG_UDINT_SOF_SIZE				1
/// USB Device Global Interrupt Register Start of Frame Interrupt Flag mask
#define USB_REG_UDINT_SOF_MASK				(((1 << USB_REG_UDINT_SOF_SIZE) - 1) << USB_REG_UDINT_SOF_OFFSET)
/// USB Device Global Interrupt Register End of Reset Interrupt Flag offset
#define USB_REG_UDINT_EORST_OFFSET			3
/// USB Device Global Interrupt Register End of Reset Interrupt Flag size
#define USB_REG_UDINT_EORST_SIZE			1
/// USB Device Global Interrupt Register End of Reset Interrupt Flag mask
#define USB_REG_UDINT_EORST_MASK			(((1 << USB_REG_UDINT_EORST_SIZE) - 1) << USB_REG_UDINT_EORST_OFFSET)
/// USB Device Global Interrupt Register Wake-Up Interrupt Flag offset
#define USB_REG_UDINT_WAKEUP_OFFSET			4
/// USB Device Global Interrupt Register Wake-Up Interrupt Flag size
#define USB_REG_UDINT_WAKEUP_SIZE			1
/// USB Device Global Interrupt Register Wake-Up Interrupt Flag mask
#define USB_REG_UDINT_WAKEUP_MASK			(((1 << USB_REG_UDINT_WAKEUP_SIZE) - 1) << USB_REG_UDINT_WAKEUP_OFFSET)
/// USB Device Global Interrupt Register End of Resume Interrupt Flag offset
#define USB_REG_UDINT_EORSM_OFFSET			5
/// USB Device Global Interrupt Register End of Resume Interrupt Flag size
#define USB_REG_UDINT_EORSM_SIZE			1
/// USB Device Global Interrupt Register End of Resume Interrupt Flag mask
#define USB_REG_UDINT_EORSM_MASK			(((1 << USB_REG_UDINT_EORSM_SIZE) - 1) << USB_REG_UDINT_EORSM_OFFSET)
/// USB Device Global Interrupt Register Upstream Resume Interrupt Flag offset
#define USB_REG_UDINT_UPRSM_OFFSET			6
/// USB Device Global Interrupt Register Upstream Resume Interrupt Flag size
#define USB_REG_UDINT_UPRSM_SIZE			1
/// USB Device Global Interrupt Register Upstream Resume Interrupt Flag mask
#define USB_REG_UDINT_UPRSM_MASK			(((1 << USB_REG_UDINT_UPRSM_SIZE) - 1) << USB_REG_UDINT_UPRSM_OFFSET)
/// USB Device Global Interrupt Register Endpoint 0 Interrupt Flag offset
#define USB_REG_UDINT_EP0INT_OFFSET			12
/// USB Device Global Interrupt Register Endpoint 0 Interrupt Flag size
#define USB_REG_UDINT_EP0INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 0 Interrupt Flag mask
#define USB_REG_UDINT_EP0INT_MASK			(((1 << USB_REG_UDINT_EP0INT_SIZE) - 1) << USB_REG_UDINT_EP0INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 1 Interrupt Flag offset
#define USB_REG_UDINT_EP1INT_OFFSET			13
/// USB Device Global Interrupt Register Endpoint 1 Interrupt Flag size
#define USB_REG_UDINT_EP1INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 1 Interrupt Flag mask
#define USB_REG_UDINT_EP1INT_MASK			(((1 << USB_REG_UDINT_EP1INT_SIZE) - 1) << USB_REG_UDINT_EP1INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 2 Interrupt Flag offset
#define USB_REG_UDINT_EP2INT_OFFSET			14
/// USB Device Global Interrupt Register Endpoint 2 Interrupt Flag size
#define USB_REG_UDINT_EP2INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 2 Interrupt Flag mask
#define USB_REG_UDINT_EP2INT_MASK			(((1 << USB_REG_UDINT_EP2INT_SIZE) - 1) << USB_REG_UDINT_EP2INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 3 Interrupt Flag offset
#define USB_REG_UDINT_EP3INT_OFFSET			15
/// USB Device Global Interrupt Register Endpoint 3 Interrupt Flag size
#define USB_REG_UDINT_EP3INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 3 Interrupt Flag mask
#define USB_REG_UDINT_EP3INT_MASK			(((1 << USB_REG_UDINT_EP3INT_SIZE) - 1) << USB_REG_UDINT_EP3INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 4 Interrupt Flag offset
#define USB_REG_UDINT_EP4INT_OFFSET			16
/// USB Device Global Interrupt Register Endpoint 4 Interrupt Flag size
#define USB_REG_UDINT_EP4INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 4 Interrupt Flag mask
#define USB_REG_UDINT_EP4INT_MASK			(((1 << USB_REG_UDINT_EP4INT_SIZE) - 1) << USB_REG_UDINT_EP4INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 5 Interrupt Flag offset
#define USB_REG_UDINT_EP5INT_OFFSET			17
/// USB Device Global Interrupt Register Endpoint 5 Interrupt Flag size
#define USB_REG_UDINT_EP5INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 5 Interrupt Flag mask
#define USB_REG_UDINT_EP5INT_MASK			(((1 << USB_REG_UDINT_EP5INT_SIZE) - 1) << USB_REG_UDINT_EP5INT_OFFSET)
/// USB Device Global Interrupt Register Endpoint 6 Interrupt Flag offset
#define USB_REG_UDINT_EP6INT_OFFSET			18
/// USB Device Global Interrupt Register Endpoint 6 Interrupt Flag size
#define USB_REG_UDINT_EP6INT_SIZE			1
/// USB Device Global Interrupt Register Endpoint 6 Interrupt Flag mask
#define USB_REG_UDINT_EP6INT_MASK			(((1 << USB_REG_UDINT_EP6INT_SIZE) - 1) << USB_REG_UDINT_EP6INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 1 Interrupt Flag offset
#define USB_REG_UDINT_DMA1INT_OFFSET			25
/// USB Device Global Interrupt Register DMA Channel 1 Interrupt Flag size
#define USB_REG_UDINT_DMA1INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 1 Interrupt Flag mask
#define USB_REG_UDINT_DMA1INT_MASK			(((1 << USB_REG_UDINT_DMA1INT_SIZE) - 1) << USB_REG_UDINT_DMA1INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 2 Interrupt Flag offset
#define USB_REG_UDINT_DMA2INT_OFFSET			26
/// USB Device Global Interrupt Register DMA Channel 2 Interrupt Flag size
#define USB_REG_UDINT_DMA2INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 2 Interrupt Flag mask
#define USB_REG_UDINT_DMA2INT_MASK			(((1 << USB_REG_UDINT_DMA2INT_SIZE) - 1) << USB_REG_UDINT_DMA2INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 3 Interrupt Flag offset
#define USB_REG_UDINT_DMA3INT_OFFSET			27
/// USB Device Global Interrupt Register DMA Channel 3 Interrupt Flag size
#define USB_REG_UDINT_DMA3INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 3 Interrupt Flag mask
#define USB_REG_UDINT_DMA3INT_MASK			(((1 << USB_REG_UDINT_DMA3INT_SIZE) - 1) << USB_REG_UDINT_DMA3INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 4 Interrupt Flag offset
#define USB_REG_UDINT_DMA4INT_OFFSET			28
/// USB Device Global Interrupt Register DMA Channel 4 Interrupt Flag size
#define USB_REG_UDINT_DMA4INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 4 Interrupt Flag mask
#define USB_REG_UDINT_DMA4INT_MASK			(((1 << USB_REG_UDINT_DMA4INT_SIZE) - 1) << USB_REG_UDINT_DMA4INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 5 Interrupt Flag offset
#define USB_REG_UDINT_DMA5INT_OFFSET			29
/// USB Device Global Interrupt Register DMA Channel 5 Interrupt Flag size
#define USB_REG_UDINT_DMA5INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 5 Interrupt Flag mask
#define USB_REG_UDINT_DMA5INT_MASK			(((1 << USB_REG_UDINT_DMA5INT_SIZE) - 1) << USB_REG_UDINT_DMA5INT_OFFSET)
/// USB Device Global Interrupt Register DMA Channel 6 Interrupt Flag offset
#define USB_REG_UDINT_DMA6INT_OFFSET			30
/// USB Device Global Interrupt Register DMA Channel 6 Interrupt Flag size
#define USB_REG_UDINT_DMA6INT_SIZE			1
/// USB Device Global Interrupt Register DMA Channel 6 Interrupt Flag mask
#define USB_REG_UDINT_DMA6INT_MASK			(((1 << USB_REG_UDINT_DMA6INT_SIZE) - 1) << USB_REG_UDINT_DMA6INT_OFFSET)
/// USB Device Global Interrupt Clear Register (Write)
#define USB_REG_UDINTCLR				0x00000008
/// USB Device Global Interrupt Clear Register Suspend Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_SUSPC_OFFSET			0
/// USB Device Global Interrupt Clear Register Suspend Interrupt Flag Clear size
#define USB_REG_UDINTCLR_SUSPC_SIZE			1
/// USB Device Global Interrupt Clear Register Suspend Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_SUSPC_MASK			(((1 << USB_REG_UDINTCLR_SUSPC_SIZE) - 1) << USB_REG_UDINTCLR_SUSPC_OFFSET)
/// USB Device Global Interrupt Clear Register Start of Frame Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_SOFC_OFFSET			2
/// USB Device Global Interrupt Clear Register Start of Frame Interrupt Flag Clear size
#define USB_REG_UDINTCLR_SOFC_SIZE			1
/// USB Device Global Interrupt Clear Register Start of Frame Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_SOFC_MASK			(((1 << USB_REG_UDINTCLR_SOFC_SIZE) - 1) << USB_REG_UDINTCLR_SOFC_OFFSET)
/// USB Device Global Interrupt Clear Register End of Reset Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_EORSTC_OFFSET			3
/// USB Device Global Interrupt Clear Register End of Reset Interrupt Flag Clear size
#define USB_REG_UDINTCLR_EORSTC_SIZE			1
/// USB Device Global Interrupt Clear Register End of Reset Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_EORSTC_MASK			(((1 << USB_REG_UDINTCLR_EORSTC_SIZE) - 1) << USB_REG_UDINTCLR_EORSTC_OFFSET)
/// USB Device Global Interrupt Clear Register Wake-Up Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_WAKEUPC_OFFSET			4
/// USB Device Global Interrupt Clear Register Wake-Up Interrupt Flag Clear size
#define USB_REG_UDINTCLR_WAKEUPC_SIZE			1
/// USB Device Global Interrupt Clear Register Wake-Up Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_WAKEUPC_MASK			(((1 << USB_REG_UDINTCLR_WAKEUPC_SIZE) - 1) << USB_REG_UDINTCLR_WAKEUPC_OFFSET)
/// USB Device Global Interrupt Clear Register End of Resume Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_EORSMC_OFFSET			5
/// USB Device Global Interrupt Clear Register End of Resume Interrupt Flag Clear size
#define USB_REG_UDINTCLR_EORSMC_SIZE			1
/// USB Device Global Interrupt Clear Register End of Resume Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_EORSMC_MASK			(((1 << USB_REG_UDINTCLR_EORSMC_SIZE) - 1) << USB_REG_UDINTCLR_EORSMC_OFFSET)
/// USB Device Global Interrupt Clear Register Upstream Resume Interrupt Flag Clear offset
#define USB_REG_UDINTCLR_UPRSMC_OFFSET			6
/// USB Device Global Interrupt Clear Register Upstream Resume Interrupt Flag Clear size
#define USB_REG_UDINTCLR_UPRSMC_SIZE			1
/// USB Device Global Interrupt Clear Register Upstream Resume Interrupt Flag Clear mask
#define USB_REG_UDINTCLR_UPRSMC_MASK			(((1 << USB_REG_UDINTCLR_UPRSMC_SIZE) - 1) << USB_REG_UDINTCLR_UPRSMC_OFFSET)
/// USB Device Global Interrupt Set Register (Write)
#define USB_REG_UDINTSET				0x0000000c
/// USB Device Global Interrupt Set Register Suspend Interrupt Flag Set offset
#define USB_REG_UDINTSET_SUSPS_OFFSET			0
/// USB Device Global Interrupt Set Register Suspend Interrupt Flag Set size
#define USB_REG_UDINTSET_SUSPS_SIZE			1
/// USB Device Global Interrupt Set Register Suspend Interrupt Flag Set mask
#define USB_REG_UDINTSET_SUSPS_MASK			(((1 << USB_REG_UDINTSET_SUSPS_SIZE) - 1) << USB_REG_UDINTSET_SUSPS_OFFSET)
/// USB Device Global Interrupt Set Register Start of Frame Interrupt Flag Set offset
#define USB_REG_UDINTSET_SOFS_OFFSET			2
/// USB Device Global Interrupt Set Register Start of Frame Interrupt Flag Set size
#define USB_REG_UDINTSET_SOFS_SIZE			1
/// USB Device Global Interrupt Set Register Start of Frame Interrupt Flag Set mask
#define USB_REG_UDINTSET_SOFS_MASK			(((1 << USB_REG_UDINTSET_SOFS_SIZE) - 1) << USB_REG_UDINTSET_SOFS_OFFSET)
/// USB Device Global Interrupt Set Register End of Reset Interrupt Flag Set offset
#define USB_REG_UDINTSET_EORSTS_OFFSET			3
/// USB Device Global Interrupt Set Register End of Reset Interrupt Flag Set size
#define USB_REG_UDINTSET_EORSTS_SIZE			1
/// USB Device Global Interrupt Set Register End of Reset Interrupt Flag Set mask
#define USB_REG_UDINTSET_EORSTS_MASK			(((1 << USB_REG_UDINTSET_EORSTS_SIZE) - 1) << USB_REG_UDINTSET_EORSTS_OFFSET)
/// USB Device Global Interrupt Set Register Wake-Up Interrupt Flag Set offset
#define USB_REG_UDINTSET_WAKEUPS_OFFSET			4
/// USB Device Global Interrupt Set Register Wake-Up Interrupt Flag Set size
#define USB_REG_UDINTSET_WAKEUPS_SIZE			1
/// USB Device Global Interrupt Set Register Wake-Up Interrupt Flag Set mask
#define USB_REG_UDINTSET_WAKEUPS_MASK			(((1 << USB_REG_UDINTSET_WAKEUPS_SIZE) - 1) << USB_REG_UDINTSET_WAKEUPS_OFFSET)
/// USB Device Global Interrupt Set Register End of Resume Interrupt Flag Set offset
#define USB_REG_UDINTSET_EORSMS_OFFSET			5
/// USB Device Global Interrupt Set Register End of Resume Interrupt Flag Set size
#define USB_REG_UDINTSET_EORSMS_SIZE			1
/// USB Device Global Interrupt Set Register End of Resume Interrupt Flag Set mask
#define USB_REG_UDINTSET_EORSMS_MASK			(((1 << USB_REG_UDINTSET_EORSMS_SIZE) - 1) << USB_REG_UDINTSET_EORSMS_OFFSET)
/// USB Device Global Interrupt Set Register Upstream Resume Interrupt Flag Set offset
#define USB_REG_UDINTSET_UPRSMS_OFFSET			6
/// USB Device Global Interrupt Set Register Upstream Resume Interrupt Flag Set size
#define USB_REG_UDINTSET_UPRSMS_SIZE			1
/// USB Device Global Interrupt Set Register Upstream Resume Interrupt Flag Set mask
#define USB_REG_UDINTSET_UPRSMS_MASK			(((1 << USB_REG_UDINTSET_UPRSMS_SIZE) - 1) << USB_REG_UDINTSET_UPRSMS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 1 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA1INTS_OFFSET		25
/// USB Device Global Interrupt Set Register DMA Channel 1 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA1INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 1 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA1INTS_MASK			(((1 << USB_REG_UDINTSET_DMA1INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA1INTS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 2 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA2INTS_OFFSET		26
/// USB Device Global Interrupt Set Register DMA Channel 2 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA2INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 2 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA2INTS_MASK			(((1 << USB_REG_UDINTSET_DMA2INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA2INTS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 3 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA3INTS_OFFSET		27
/// USB Device Global Interrupt Set Register DMA Channel 3 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA3INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 3 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA3INTS_MASK			(((1 << USB_REG_UDINTSET_DMA3INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA3INTS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 4 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA4INTS_OFFSET		28
/// USB Device Global Interrupt Set Register DMA Channel 4 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA4INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 4 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA4INTS_MASK			(((1 << USB_REG_UDINTSET_DMA4INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA4INTS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 5 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA5INTS_OFFSET		29
/// USB Device Global Interrupt Set Register DMA Channel 5 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA5INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 5 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA5INTS_MASK			(((1 << USB_REG_UDINTSET_DMA5INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA5INTS_OFFSET)
/// USB Device Global Interrupt Set Register DMA Channel 6 Interrupt Flag Set offset
#define USB_REG_UDINTSET_DMA6INTS_OFFSET		30
/// USB Device Global Interrupt Set Register DMA Channel 6 Interrupt Flag Set size
#define USB_REG_UDINTSET_DMA6INTS_SIZE			1
/// USB Device Global Interrupt Set Register DMA Channel 6 Interrupt Flag Set mask
#define USB_REG_UDINTSET_DMA6INTS_MASK			(((1 << USB_REG_UDINTSET_DMA6INTS_SIZE) - 1) << USB_REG_UDINTSET_DMA6INTS_OFFSET)
/// USB Device Global Interrupt Enable Register (Read)
#define USB_REG_UDINTE					0x00000010
/// USB Device Global Interrupt Enable Register Suspend Interrupt Flag Enable offset
#define USB_REG_UDINTE_SUSPE_OFFSET			0
/// USB Device Global Interrupt Enable Register Suspend Interrupt Flag Enable size
#define USB_REG_UDINTE_SUSPE_SIZE			1
/// USB Device Global Interrupt Enable Register Suspend Interrupt Flag Enable mask
#define USB_REG_UDINTE_SUSPE_MASK			(((1 << USB_REG_UDINTE_SUSPE_SIZE) - 1) << USB_REG_UDINTE_SUSPE_OFFSET)
/// USB Device Global Interrupt Enable Register Start of Frame Interrupt Flag Enable offset
#define USB_REG_UDINTE_SOFE_OFFSET			2
/// USB Device Global Interrupt Enable Register Start of Frame Interrupt Flag Enable size
#define USB_REG_UDINTE_SOFE_SIZE			1
/// USB Device Global Interrupt Enable Register Start of Frame Interrupt Flag Enable mask
#define USB_REG_UDINTE_SOFE_MASK			(((1 << USB_REG_UDINTE_SOFE_SIZE) - 1) << USB_REG_UDINTE_SOFE_OFFSET)
/// USB Device Global Interrupt Enable Register End of Reset Interrupt Flag Enable offset
#define USB_REG_UDINTE_EORSTE_OFFSET			3
/// USB Device Global Interrupt Enable Register End of Reset Interrupt Flag Enable size
#define USB_REG_UDINTE_EORSTE_SIZE			1
/// USB Device Global Interrupt Enable Register End of Reset Interrupt Flag Enable mask
#define USB_REG_UDINTE_EORSTE_MASK			(((1 << USB_REG_UDINTE_EORSTE_SIZE) - 1) << USB_REG_UDINTE_EORSTE_OFFSET)
/// USB Device Global Interrupt Enable Register Wake-Up Interrupt Flag Enable offset
#define USB_REG_UDINTE_WAKEUPE_OFFSET			4
/// USB Device Global Interrupt Enable Register Wake-Up Interrupt Flag Enable size
#define USB_REG_UDINTE_WAKEUPE_SIZE			1
/// USB Device Global Interrupt Enable Register Wake-Up Interrupt Flag Enable mask
#define USB_REG_UDINTE_WAKEUPE_MASK			(((1 << USB_REG_UDINTE_WAKEUPE_SIZE) - 1) << USB_REG_UDINTE_WAKEUPE_OFFSET)
/// USB Device Global Interrupt Enable Register End of Resume Interrupt Flag Enable offset
#define USB_REG_UDINTE_EORSME_OFFSET			5
/// USB Device Global Interrupt Enable Register End of Resume Interrupt Flag Enable size
#define USB_REG_UDINTE_EORSME_SIZE			1
/// USB Device Global Interrupt Enable Register End of Resume Interrupt Flag Enable mask
#define USB_REG_UDINTE_EORSME_MASK			(((1 << USB_REG_UDINTE_EORSME_SIZE) - 1) << USB_REG_UDINTE_EORSME_OFFSET)
/// USB Device Global Interrupt Enable Register Upstream Resume Interrupt Flag Enable offset
#define USB_REG_UDINTE_UPRSME_OFFSET			6
/// USB Device Global Interrupt Enable Register Upstream Resume Interrupt Flag Enable size
#define USB_REG_UDINTE_UPRSME_SIZE			1
/// USB Device Global Interrupt Enable Register Upstream Resume Interrupt Flag Enable mask
#define USB_REG_UDINTE_UPRSME_MASK			(((1 << USB_REG_UDINTE_UPRSME_SIZE) - 1) << USB_REG_UDINTE_UPRSME_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 0 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP0INTE_OFFSET			12
/// USB Device Global Interrupt Enable Register Endpoint 0 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP0INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 0 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP0INTE_MASK			(((1 << USB_REG_UDINTE_EP0INTE_SIZE) - 1) << USB_REG_UDINTE_EP0INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 1 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP1INTE_OFFSET			13
/// USB Device Global Interrupt Enable Register Endpoint 1 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP1INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 1 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP1INTE_MASK			(((1 << USB_REG_UDINTE_EP1INTE_SIZE) - 1) << USB_REG_UDINTE_EP1INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 2 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP2INTE_OFFSET			14
/// USB Device Global Interrupt Enable Register Endpoint 2 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP2INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 2 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP2INTE_MASK			(((1 << USB_REG_UDINTE_EP2INTE_SIZE) - 1) << USB_REG_UDINTE_EP2INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 3 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP3INTE_OFFSET			15
/// USB Device Global Interrupt Enable Register Endpoint 3 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP3INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 3 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP3INTE_MASK			(((1 << USB_REG_UDINTE_EP3INTE_SIZE) - 1) << USB_REG_UDINTE_EP3INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 4 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP4INTE_OFFSET			16
/// USB Device Global Interrupt Enable Register Endpoint 4 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP4INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 4 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP4INTE_MASK			(((1 << USB_REG_UDINTE_EP4INTE_SIZE) - 1) << USB_REG_UDINTE_EP4INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 5 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP5INTE_OFFSET			17
/// USB Device Global Interrupt Enable Register Endpoint 5 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP5INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 5 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP5INTE_MASK			(((1 << USB_REG_UDINTE_EP5INTE_SIZE) - 1) << USB_REG_UDINTE_EP5INTE_OFFSET)
/// USB Device Global Interrupt Enable Register Endpoint 6 Interrupt Flag Enable offset
#define USB_REG_UDINTE_EP6INTE_OFFSET			18
/// USB Device Global Interrupt Enable Register Endpoint 6 Interrupt Flag Enable size
#define USB_REG_UDINTE_EP6INTE_SIZE			1
/// USB Device Global Interrupt Enable Register Endpoint 6 Interrupt Flag Enable mask
#define USB_REG_UDINTE_EP6INTE_MASK			(((1 << USB_REG_UDINTE_EP6INTE_SIZE) - 1) << USB_REG_UDINTE_EP6INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 1 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA1INTE_OFFSET			25
/// USB Device Global Interrupt Enable Register DMA Channel 1 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA1INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 1 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA1INTE_MASK			(((1 << USB_REG_UDINTE_DMA1INTE_SIZE) - 1) << USB_REG_UDINTE_DMA1INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 2 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA2INTE_OFFSET			26
/// USB Device Global Interrupt Enable Register DMA Channel 2 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA2INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 2 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA2INTE_MASK			(((1 << USB_REG_UDINTE_DMA2INTE_SIZE) - 1) << USB_REG_UDINTE_DMA2INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 3 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA3INTE_OFFSET			27
/// USB Device Global Interrupt Enable Register DMA Channel 3 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA3INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 3 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA3INTE_MASK			(((1 << USB_REG_UDINTE_DMA3INTE_SIZE) - 1) << USB_REG_UDINTE_DMA3INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 4 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA4INTE_OFFSET			28
/// USB Device Global Interrupt Enable Register DMA Channel 4 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA4INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 4 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA4INTE_MASK			(((1 << USB_REG_UDINTE_DMA4INTE_SIZE) - 1) << USB_REG_UDINTE_DMA4INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 5 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA5INTE_OFFSET			29
/// USB Device Global Interrupt Enable Register DMA Channel 5 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA5INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 5 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA5INTE_MASK			(((1 << USB_REG_UDINTE_DMA5INTE_SIZE) - 1) << USB_REG_UDINTE_DMA5INTE_OFFSET)
/// USB Device Global Interrupt Enable Register DMA Channel 6 Interrupt Flag Enable offset
#define USB_REG_UDINTE_DMA6INTE_OFFSET			30
/// USB Device Global Interrupt Enable Register DMA Channel 6 Interrupt Flag Enable size
#define USB_REG_UDINTE_DMA6INTE_SIZE			1
/// USB Device Global Interrupt Enable Register DMA Channel 6 Interrupt Flag Enable mask
#define USB_REG_UDINTE_DMA6INTE_MASK			(((1 << USB_REG_UDINTE_DMA6INTE_SIZE) - 1) << USB_REG_UDINTE_DMA6INTE_OFFSET)
/// USB Device Global Interrupt Enable Clear Register (Write)
#define USB_REG_UDINTECLR				0x00000014
/// USB Device Global Interrupt Enable Clear Register Suspend Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_SUSPEC_OFFSET			0
/// USB Device Global Interrupt Enable Clear Register Suspend Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_SUSPEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Suspend Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_SUSPEC_MASK			(((1 << USB_REG_UDINTECLR_SUSPEC_SIZE) - 1) << USB_REG_UDINTECLR_SUSPEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Start of Frame Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_SOFEC_OFFSET			2
/// USB Device Global Interrupt Enable Clear Register Start of Frame Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_SOFEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Start of Frame Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_SOFEC_MASK			(((1 << USB_REG_UDINTECLR_SOFEC_SIZE) - 1) << USB_REG_UDINTECLR_SOFEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register End of Reset Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EORSTEC_OFFSET		3
/// USB Device Global Interrupt Enable Clear Register End of Reset Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EORSTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register End of Reset Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EORSTEC_MASK			(((1 << USB_REG_UDINTECLR_EORSTEC_SIZE) - 1) << USB_REG_UDINTECLR_EORSTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Wake-Up Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_WAKEUPEC_OFFSET		4
/// USB Device Global Interrupt Enable Clear Register Wake-Up Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_WAKEUPEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Wake-Up Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_WAKEUPEC_MASK			(((1 << USB_REG_UDINTECLR_WAKEUPEC_SIZE) - 1) << USB_REG_UDINTECLR_WAKEUPEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register End of Resume Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EORSMEC_OFFSET		5
/// USB Device Global Interrupt Enable Clear Register End of Resume Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EORSMEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register End of Resume Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EORSMEC_MASK			(((1 << USB_REG_UDINTECLR_EORSMEC_SIZE) - 1) << USB_REG_UDINTECLR_EORSMEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Upstream Resume Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_UPRSMEC_OFFSET		6
/// USB Device Global Interrupt Enable Clear Register Upstream Resume Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_UPRSMEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Upstream Resume Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_UPRSMEC_MASK			(((1 << USB_REG_UDINTECLR_UPRSMEC_SIZE) - 1) << USB_REG_UDINTECLR_UPRSMEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 0 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP0INTEC_OFFSET		12
/// USB Device Global Interrupt Enable Clear Register Endpoint 0 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP0INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 0 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP0INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP0INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP0INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 1 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP1INTEC_OFFSET		13
/// USB Device Global Interrupt Enable Clear Register Endpoint 1 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP1INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 1 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP1INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP1INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP1INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 2 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP2INTEC_OFFSET		14
/// USB Device Global Interrupt Enable Clear Register Endpoint 2 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP2INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 2 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP2INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP2INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP2INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 3 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP3INTEC_OFFSET		15
/// USB Device Global Interrupt Enable Clear Register Endpoint 3 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP3INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 3 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP3INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP3INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP3INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 4 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP4INTEC_OFFSET		16
/// USB Device Global Interrupt Enable Clear Register Endpoint 4 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP4INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 4 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP4INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP4INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP4INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 5 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP5INTEC_OFFSET		17
/// USB Device Global Interrupt Enable Clear Register Endpoint 5 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP5INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 5 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP5INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP5INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP5INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register Endpoint 6 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_EP6INTEC_OFFSET		18
/// USB Device Global Interrupt Enable Clear Register Endpoint 6 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_EP6INTEC_SIZE			1
/// USB Device Global Interrupt Enable Clear Register Endpoint 6 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_EP6INTEC_MASK			(((1 << USB_REG_UDINTECLR_EP6INTEC_SIZE) - 1) << USB_REG_UDINTECLR_EP6INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 1 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA1INTEC_OFFSET		25
/// USB Device Global Interrupt Enable Clear Register DMA Channel 1 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA1INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 1 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA1INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA1INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA1INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 2 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA2INTEC_OFFSET		26
/// USB Device Global Interrupt Enable Clear Register DMA Channel 2 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA2INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 2 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA2INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA2INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA2INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 3 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA3INTEC_OFFSET		27
/// USB Device Global Interrupt Enable Clear Register DMA Channel 3 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA3INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 3 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA3INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA3INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA3INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 4 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA4INTEC_OFFSET		28
/// USB Device Global Interrupt Enable Clear Register DMA Channel 4 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA4INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 4 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA4INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA4INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA4INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 5 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA5INTEC_OFFSET		29
/// USB Device Global Interrupt Enable Clear Register DMA Channel 5 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA5INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 5 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA5INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA5INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA5INTEC_OFFSET)
/// USB Device Global Interrupt Enable Clear Register DMA Channel 6 Interrupt Flag Enable Clear offset
#define USB_REG_UDINTECLR_DMA6INTEC_OFFSET		30
/// USB Device Global Interrupt Enable Clear Register DMA Channel 6 Interrupt Flag Enable Clear size
#define USB_REG_UDINTECLR_DMA6INTEC_SIZE		1
/// USB Device Global Interrupt Enable Clear Register DMA Channel 6 Interrupt Flag Enable Clear mask
#define USB_REG_UDINTECLR_DMA6INTEC_MASK		(((1 << USB_REG_UDINTECLR_DMA6INTEC_SIZE) - 1) << USB_REG_UDINTECLR_DMA6INTEC_OFFSET)
/// USB Device Global Interrupt Enable Set Register (Write)
#define USB_REG_UDINTESET				0x00000018
/// USB Device Global Interrupt Enable Set Register Suspend Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_SUSPES_OFFSET			0
/// USB Device Global Interrupt Enable Set Register Suspend Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_SUSPES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Suspend Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_SUSPES_MASK			(((1 << USB_REG_UDINTESET_SUSPES_SIZE) - 1) << USB_REG_UDINTESET_SUSPES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Start of Frame Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_SOFES_OFFSET			2
/// USB Device Global Interrupt Enable Set Register Start of Frame Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_SOFES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Start of Frame Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_SOFES_MASK			(((1 << USB_REG_UDINTESET_SOFES_SIZE) - 1) << USB_REG_UDINTESET_SOFES_OFFSET)
/// USB Device Global Interrupt Enable Set Register End of Reset Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EORSTES_OFFSET		3
/// USB Device Global Interrupt Enable Set Register End of Reset Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EORSTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register End of Reset Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EORSTES_MASK			(((1 << USB_REG_UDINTESET_EORSTES_SIZE) - 1) << USB_REG_UDINTESET_EORSTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Wake-Up Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_WAKEUPES_OFFSET		4
/// USB Device Global Interrupt Enable Set Register Wake-Up Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_WAKEUPES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Wake-Up Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_WAKEUPES_MASK			(((1 << USB_REG_UDINTESET_WAKEUPES_SIZE) - 1) << USB_REG_UDINTESET_WAKEUPES_OFFSET)
/// USB Device Global Interrupt Enable Set Register End of Resume Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EORSMES_OFFSET		5
/// USB Device Global Interrupt Enable Set Register End of Resume Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EORSMES_SIZE			1
/// USB Device Global Interrupt Enable Set Register End of Resume Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EORSMES_MASK			(((1 << USB_REG_UDINTESET_EORSMES_SIZE) - 1) << USB_REG_UDINTESET_EORSMES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Upstream Resume Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_UPRSMES_OFFSET		6
/// USB Device Global Interrupt Enable Set Register Upstream Resume Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_UPRSMES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Upstream Resume Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_UPRSMES_MASK			(((1 << USB_REG_UDINTESET_UPRSMES_SIZE) - 1) << USB_REG_UDINTESET_UPRSMES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 0 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP0INTES_OFFSET		12
/// USB Device Global Interrupt Enable Set Register Endpoint 0 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP0INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 0 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP0INTES_MASK			(((1 << USB_REG_UDINTESET_EP0INTES_SIZE) - 1) << USB_REG_UDINTESET_EP0INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 1 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP1INTES_OFFSET		13
/// USB Device Global Interrupt Enable Set Register Endpoint 1 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP1INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 1 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP1INTES_MASK			(((1 << USB_REG_UDINTESET_EP1INTES_SIZE) - 1) << USB_REG_UDINTESET_EP1INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 2 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP2INTES_OFFSET		14
/// USB Device Global Interrupt Enable Set Register Endpoint 2 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP2INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 2 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP2INTES_MASK			(((1 << USB_REG_UDINTESET_EP2INTES_SIZE) - 1) << USB_REG_UDINTESET_EP2INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 3 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP3INTES_OFFSET		15
/// USB Device Global Interrupt Enable Set Register Endpoint 3 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP3INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 3 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP3INTES_MASK			(((1 << USB_REG_UDINTESET_EP3INTES_SIZE) - 1) << USB_REG_UDINTESET_EP3INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 4 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP4INTES_OFFSET		16
/// USB Device Global Interrupt Enable Set Register Endpoint 4 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP4INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 4 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP4INTES_MASK			(((1 << USB_REG_UDINTESET_EP4INTES_SIZE) - 1) << USB_REG_UDINTESET_EP4INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 5 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP5INTES_OFFSET		17
/// USB Device Global Interrupt Enable Set Register Endpoint 5 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP5INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 5 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP5INTES_MASK			(((1 << USB_REG_UDINTESET_EP5INTES_SIZE) - 1) << USB_REG_UDINTESET_EP5INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register Endpoint 6 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_EP6INTES_OFFSET		18
/// USB Device Global Interrupt Enable Set Register Endpoint 6 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_EP6INTES_SIZE			1
/// USB Device Global Interrupt Enable Set Register Endpoint 6 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_EP6INTES_MASK			(((1 << USB_REG_UDINTESET_EP6INTES_SIZE) - 1) << USB_REG_UDINTESET_EP6INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 1 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA1INTES_OFFSET		25
/// USB Device Global Interrupt Enable Set Register DMA Channel 1 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA1INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 1 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA1INTES_MASK		(((1 << USB_REG_UDINTESET_DMA1INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA1INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 2 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA2INTES_OFFSET		26
/// USB Device Global Interrupt Enable Set Register DMA Channel 2 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA2INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 2 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA2INTES_MASK		(((1 << USB_REG_UDINTESET_DMA2INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA2INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 3 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA3INTES_OFFSET		27
/// USB Device Global Interrupt Enable Set Register DMA Channel 3 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA3INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 3 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA3INTES_MASK		(((1 << USB_REG_UDINTESET_DMA3INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA3INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 4 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA4INTES_OFFSET		28
/// USB Device Global Interrupt Enable Set Register DMA Channel 4 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA4INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 4 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA4INTES_MASK		(((1 << USB_REG_UDINTESET_DMA4INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA4INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 5 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA5INTES_OFFSET		29
/// USB Device Global Interrupt Enable Set Register DMA Channel 5 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA5INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 5 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA5INTES_MASK		(((1 << USB_REG_UDINTESET_DMA5INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA5INTES_OFFSET)
/// USB Device Global Interrupt Enable Set Register DMA Channel 6 Interrupt Flag Enable Set offset
#define USB_REG_UDINTESET_DMA6INTES_OFFSET		30
/// USB Device Global Interrupt Enable Set Register DMA Channel 6 Interrupt Flag Enable Set size
#define USB_REG_UDINTESET_DMA6INTES_SIZE		1
/// USB Device Global Interrupt Enable Set Register DMA Channel 6 Interrupt Flag Enable Set mask
#define USB_REG_UDINTESET_DMA6INTES_MASK		(((1 << USB_REG_UDINTESET_DMA6INTES_SIZE) - 1) << USB_REG_UDINTESET_DMA6INTES_OFFSET)
/// USB Endpoint Enable/Reset Register (Read/Write)
#define USB_REG_UERST					0x0000001c
/// USB Endpoint Enable/Reset Register Endpoint 0 Enable offset
#define USB_REG_UERST_EPEN0_OFFSET			0
/// USB Endpoint Enable/Reset Register Endpoint 0 Enable size
#define USB_REG_UERST_EPEN0_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 0 Enable mask
#define USB_REG_UERST_EPEN0_MASK			(((1 << USB_REG_UERST_EPEN0_SIZE) - 1) << USB_REG_UERST_EPEN0_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 1 Enable offset
#define USB_REG_UERST_EPEN1_OFFSET			1
/// USB Endpoint Enable/Reset Register Endpoint 1 Enable size
#define USB_REG_UERST_EPEN1_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 1 Enable mask
#define USB_REG_UERST_EPEN1_MASK			(((1 << USB_REG_UERST_EPEN1_SIZE) - 1) << USB_REG_UERST_EPEN1_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 2 Enable offset
#define USB_REG_UERST_EPEN2_OFFSET			2
/// USB Endpoint Enable/Reset Register Endpoint 2 Enable size
#define USB_REG_UERST_EPEN2_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 2 Enable mask
#define USB_REG_UERST_EPEN2_MASK			(((1 << USB_REG_UERST_EPEN2_SIZE) - 1) << USB_REG_UERST_EPEN2_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 3 Enable offset
#define USB_REG_UERST_EPEN3_OFFSET			3
/// USB Endpoint Enable/Reset Register Endpoint 3 Enable size
#define USB_REG_UERST_EPEN3_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 3 Enable mask
#define USB_REG_UERST_EPEN3_MASK			(((1 << USB_REG_UERST_EPEN3_SIZE) - 1) << USB_REG_UERST_EPEN3_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 4 Enable offset
#define USB_REG_UERST_EPEN4_OFFSET			4
/// USB Endpoint Enable/Reset Register Endpoint 4 Enable size
#define USB_REG_UERST_EPEN4_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 4 Enable mask
#define USB_REG_UERST_EPEN4_MASK			(((1 << USB_REG_UERST_EPEN4_SIZE) - 1) << USB_REG_UERST_EPEN4_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 5 Enable offset
#define USB_REG_UERST_EPEN5_OFFSET			5
/// USB Endpoint Enable/Reset Register Endpoint 5 Enable size
#define USB_REG_UERST_EPEN5_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 5 Enable mask
#define USB_REG_UERST_EPEN5_MASK			(((1 << USB_REG_UERST_EPEN5_SIZE) - 1) << USB_REG_UERST_EPEN5_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 6 Enable offset
#define USB_REG_UERST_EPEN6_OFFSET			6
/// USB Endpoint Enable/Reset Register Endpoint 6 Enable size
#define USB_REG_UERST_EPEN6_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 6 Enable mask
#define USB_REG_UERST_EPEN6_MASK			(((1 << USB_REG_UERST_EPEN6_SIZE) - 1) << USB_REG_UERST_EPEN6_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 0 Reset offset
#define USB_REG_UERST_EPRST0_OFFSET			16
/// USB Endpoint Enable/Reset Register Endpoint 0 Reset size
#define USB_REG_UERST_EPRST0_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 0 Reset mask
#define USB_REG_UERST_EPRST0_MASK			(((1 << USB_REG_UERST_EPRST0_SIZE) - 1) << USB_REG_UERST_EPRST0_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 1 Reset offset
#define USB_REG_UERST_EPRST1_OFFSET			17
/// USB Endpoint Enable/Reset Register Endpoint 1 Reset size
#define USB_REG_UERST_EPRST1_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 1 Reset mask
#define USB_REG_UERST_EPRST1_MASK			(((1 << USB_REG_UERST_EPRST1_SIZE) - 1) << USB_REG_UERST_EPRST1_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 2 Reset offset
#define USB_REG_UERST_EPRST2_OFFSET			18
/// USB Endpoint Enable/Reset Register Endpoint 2 Reset size
#define USB_REG_UERST_EPRST2_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 2 Reset mask
#define USB_REG_UERST_EPRST2_MASK			(((1 << USB_REG_UERST_EPRST2_SIZE) - 1) << USB_REG_UERST_EPRST2_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 3 Reset offset
#define USB_REG_UERST_EPRST3_OFFSET			19
/// USB Endpoint Enable/Reset Register Endpoint 3 Reset size
#define USB_REG_UERST_EPRST3_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 3 Reset mask
#define USB_REG_UERST_EPRST3_MASK			(((1 << USB_REG_UERST_EPRST3_SIZE) - 1) << USB_REG_UERST_EPRST3_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 4 Reset offset
#define USB_REG_UERST_EPRST4_OFFSET			20
/// USB Endpoint Enable/Reset Register Endpoint 4 Reset size
#define USB_REG_UERST_EPRST4_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 4 Reset mask
#define USB_REG_UERST_EPRST4_MASK			(((1 << USB_REG_UERST_EPRST4_SIZE) - 1) << USB_REG_UERST_EPRST4_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 5 Reset offset
#define USB_REG_UERST_EPRST5_OFFSET			21
/// USB Endpoint Enable/Reset Register Endpoint 5 Reset size
#define USB_REG_UERST_EPRST5_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 5 Reset mask
#define USB_REG_UERST_EPRST5_MASK			(((1 << USB_REG_UERST_EPRST5_SIZE) - 1) << USB_REG_UERST_EPRST5_OFFSET)
/// USB Endpoint Enable/Reset Register Endpoint 6 Reset offset
#define USB_REG_UERST_EPRST6_OFFSET			22
/// USB Endpoint Enable/Reset Register Endpoint 6 Reset size
#define USB_REG_UERST_EPRST6_SIZE			1
/// USB Endpoint Enable/Reset Register Endpoint 6 Reset mask
#define USB_REG_UERST_EPRST6_MASK			(((1 << USB_REG_UERST_EPRST6_SIZE) - 1) << USB_REG_UERST_EPRST6_OFFSET)
/// USB Device Frame Number Register (Read)
#define USB_REG_UDFNUM					0x00000020
/// USB Device Frame Number Register Frame Number offset
#define USB_REG_UDFNUM_FNUM_OFFSET			3
/// USB Device Frame Number Register Frame Number size
#define USB_REG_UDFNUM_FNUM_SIZE			11
/// USB Device Frame Number Register Frame Number mask
#define USB_REG_UDFNUM_FNUM_MASK			(((1 << USB_REG_UDFNUM_FNUM_SIZE) - 1) << USB_REG_UDFNUM_FNUM_OFFSET)
/// USB Device Frame Number Register Frame Number CRC Error offset
#define USB_REG_UDFNUM_FNCERR_OFFSET			15
/// USB Device Frame Number Register Frame Number CRC Error size
#define USB_REG_UDFNUM_FNCERR_SIZE			1
/// USB Device Frame Number Register Frame Number CRC Error mask
#define USB_REG_UDFNUM_FNCERR_MASK			(((1 << USB_REG_UDFNUM_FNCERR_SIZE) - 1) << USB_REG_UDFNUM_FNCERR_OFFSET)
/// USB Endpoint 0 Configuration Register (Read/Write)
#define USB_REG_UECFG0					0x00000100
/// USB Endpoint 1 Configuration Register (Read/Write)
#define USB_REG_UECFG1					0x00000104
/// USB Endpoint 2 Configuration Register (Read/Write)
#define USB_REG_UECFG2					0x00000108
/// USB Endpoint 3 Configuration Register (Read/Write)
#define USB_REG_UECFG3					0x0000010c
/// USB Endpoint 4 Configuration Register (Read/Write)
#define USB_REG_UECFG4					0x00000110
/// USB Endpoint 5 Configuration Register (Read/Write)
#define USB_REG_UECFG5					0x00000114
/// USB Endpoint 6 Configuration Register (Read/Write)
#define USB_REG_UECFG6					0x00000118
/// USB Endpoint X Configuration Register Endpoint Memory Allocate offset
#define USB_REG_UECFGX_ALLOC_OFFSET			1
/// USB Endpoint X Configuration Register Endpoint Memory Allocate size
#define USB_REG_UECFGX_ALLOC_SIZE			1
/// USB Endpoint X Configuration Register Endpoint Memory Allocate mask
#define USB_REG_UECFGX_ALLOC_MASK			(((1 << USB_REG_UECFGX_ALLOC_SIZE) - 1) << USB_REG_UECFGX_ALLOC_OFFSET)
/// USB Endpoint X Configuration Register Endpoint Banks offset
#define USB_REG_UECFGX_EPBK_OFFSET			2
/// USB Endpoint X Configuration Register Endpoint Banks size
#define USB_REG_UECFGX_EPBK_SIZE			2
/// USB Endpoint X Configuration Register Endpoint Banks mask
#define USB_REG_UECFGX_EPBK_MASK			(((1 << USB_REG_UECFGX_EPBK_SIZE) - 1) << USB_REG_UECFGX_EPBK_OFFSET)
/// USB Endpoint X Configuration Register Endpoint Size offset
#define USB_REG_UECFGX_EPSIZE_OFFSET			4
/// USB Endpoint X Configuration Register Endpoint Size size
#define USB_REG_UECFGX_EPSIZE_SIZE			3
/// USB Endpoint X Configuration Register Endpoint Size mask
#define USB_REG_UECFGX_EPSIZE_MASK			(((1 << USB_REG_UECFGX_EPSIZE_SIZE) - 1) << USB_REG_UECFGX_EPSIZE_OFFSET)
/// USB Endpoint X Configuration Register Endpoint Direction offset
#define USB_REG_UECFGX_EPDIR_OFFSET			8
/// USB Endpoint X Configuration Register Endpoint Direction size
#define USB_REG_UECFGX_EPDIR_SIZE			1
/// USB Endpoint X Configuration Register Endpoint Direction mask
#define USB_REG_UECFGX_EPDIR_MASK			(((1 << USB_REG_UECFGX_EPDIR_SIZE) - 1) << USB_REG_UECFGX_EPDIR_OFFSET)
/// USB Endpoint X Configuration Register Automatic Switch offset
#define USB_REG_UECFGX_AUTOSW_OFFSET			9
/// USB Endpoint X Configuration Register Automatic Switch size
#define USB_REG_UECFGX_AUTOSW_SIZE			1
/// USB Endpoint X Configuration Register Automatic Switch mask
#define USB_REG_UECFGX_AUTOSW_MASK			(((1 << USB_REG_UECFGX_AUTOSW_SIZE) - 1) << USB_REG_UECFGX_AUTOSW_OFFSET)
/// USB Endpoint X Configuration Register Endpoint Type offset
#define USB_REG_UECFGX_EPTYPE_OFFSET			11
/// USB Endpoint X Configuration Register Endpoint Type size
#define USB_REG_UECFGX_EPTYPE_SIZE			2
/// USB Endpoint X Configuration Register Endpoint Type mask
#define USB_REG_UECFGX_EPTYPE_MASK			(((1 << USB_REG_UECFGX_EPTYPE_SIZE) - 1) << USB_REG_UECFGX_EPTYPE_OFFSET)
/// USB Endpoint 0 Status Register (Read)
#define USB_REG_UESTA0					0x00000130
/// USB Endpoint 1 Status Register (Read)
#define USB_REG_UESTA1					0x00000134
/// USB Endpoint 2 Status Register (Read)
#define USB_REG_UESTA2					0x00000138
/// USB Endpoint 3 Status Register (Read)
#define USB_REG_UESTA3					0x0000013c
/// USB Endpoint 4 Status Register (Read)
#define USB_REG_UESTA4					0x00000140
/// USB Endpoint 5 Status Register (Read)
#define USB_REG_UESTA5					0x00000144
/// USB Endpoint 6 Status Register (Read)
#define USB_REG_UESTA6					0x00000148
/// USB Endpoint X Status Register Transmitted IN Data Interrupt Flag offset
#define USB_REG_UESTAX_TXINI_OFFSET			0
/// USB Endpoint X Status Register Transmitted IN Data Interrupt Flag size
#define USB_REG_UESTAX_TXINI_SIZE			1
/// USB Endpoint X Status Register Transmitted IN Data Interrupt Flag mask
#define USB_REG_UESTAX_TXINI_MASK			(((1 << USB_REG_UESTAX_TXINI_SIZE) - 1) << USB_REG_UESTAX_TXINI_OFFSET)
/// USB Endpoint X Status Register Received OUT Data Interrupt Flag offset
#define USB_REG_UESTAX_RXOUTI_OFFSET			1
/// USB Endpoint X Status Register Received OUT Data Interrupt Flag size
#define USB_REG_UESTAX_RXOUTI_SIZE			1
/// USB Endpoint X Status Register Received OUT Data Interrupt Flag mask
#define USB_REG_UESTAX_RXOUTI_MASK			(((1 << USB_REG_UESTAX_RXOUTI_SIZE) - 1) << USB_REG_UESTAX_RXOUTI_OFFSET)
/// USB Endpoint X Status Register Received SETUP Interrupt Flag offset
#define USB_REG_UESTAX_RXSTPI_OFFSET			2
/// USB Endpoint X Status Register Received SETUP Interrupt Flag size
#define USB_REG_UESTAX_RXSTPI_SIZE			1
/// USB Endpoint X Status Register Received SETUP Interrupt Flag mask
#define USB_REG_UESTAX_RXSTPI_MASK			(((1 << USB_REG_UESTAX_RXSTPI_SIZE) - 1) << USB_REG_UESTAX_RXSTPI_OFFSET)
/// USB Endpoint X Status Register Underflow Interrupt Flag offset
#define USB_REG_UESTAX_UNDERFI_OFFSET			2
/// USB Endpoint X Status Register Underflow Interrupt Flag size
#define USB_REG_UESTAX_UNDERFI_SIZE			1
/// USB Endpoint X Status Register Underflow Interrupt Flag mask
#define USB_REG_UESTAX_UNDERFI_MASK			(((1 << USB_REG_UESTAX_UNDERFI_SIZE) - 1) << USB_REG_UESTAX_UNDERFI_OFFSET)
/// USB Endpoint X Status Register NAKed OUT Interrupt Flag offset
#define USB_REG_UESTAX_NAKOUTI_OFFSET			3
/// USB Endpoint X Status Register NAKed OUT Interrupt Flag size
#define USB_REG_UESTAX_NAKOUTI_SIZE			1
/// USB Endpoint X Status Register NAKed OUT Interrupt Flag mask
#define USB_REG_UESTAX_NAKOUTI_MASK			(((1 << USB_REG_UESTAX_NAKOUTI_SIZE) - 1) << USB_REG_UESTAX_NAKOUTI_OFFSET)
/// USB Endpoint X Status Register NAKed IN Interrupt Flag offset
#define USB_REG_UESTAX_NAKINI_OFFSET			4
/// USB Endpoint X Status Register NAKed IN Interrupt Flag size
#define USB_REG_UESTAX_NAKINI_SIZE			1
/// USB Endpoint X Status Register NAKed IN Interrupt Flag mask
#define USB_REG_UESTAX_NAKINI_MASK			(((1 << USB_REG_UESTAX_NAKINI_SIZE) - 1) << USB_REG_UESTAX_NAKINI_OFFSET)
/// USB Endpoint X Status Register Overflow Interrupt Flag offset
#define USB_REG_UESTAX_OVERFI_OFFSET			5
/// USB Endpoint X Status Register Overflow Interrupt Flag size
#define USB_REG_UESTAX_OVERFI_SIZE			1
/// USB Endpoint X Status Register Overflow Interrupt Flag mask
#define USB_REG_UESTAX_OVERFI_MASK			(((1 << USB_REG_UESTAX_OVERFI_SIZE) - 1) << USB_REG_UESTAX_OVERFI_OFFSET)
/// USB Endpoint X Status Register STALLed Interrupt Flag offset
#define USB_REG_UESTAX_STALLEDI_OFFSET			6
/// USB Endpoint X Status Register STALLed Interrupt Flag size
#define USB_REG_UESTAX_STALLEDI_SIZE			1
/// USB Endpoint X Status Register STALLed Interrupt Flag mask
#define USB_REG_UESTAX_STALLEDI_MASK			(((1 << USB_REG_UESTAX_STALLEDI_SIZE) - 1) << USB_REG_UESTAX_STALLEDI_OFFSET)
/// USB Endpoint X Status Register CRC Error Interrupt Flag offset
#define USB_REG_UESTAX_CRCERRI_OFFSET			6
/// USB Endpoint X Status Register CRC Error Interrupt Flag size
#define USB_REG_UESTAX_CRCERRI_SIZE			1
/// USB Endpoint X Status Register CRC Error Interrupt Flag mask
#define USB_REG_UESTAX_CRCERRI_MASK			(((1 << USB_REG_UESTAX_CRCERRI_SIZE) - 1) << USB_REG_UESTAX_CRCERRI_OFFSET)
/// USB Endpoint X Status Register Short Packet Interrupt Flag offset
#define USB_REG_UESTAX_SHORTPACKET_OFFSET		7
/// USB Endpoint X Status Register Short Packet Interrupt Flag size
#define USB_REG_UESTAX_SHORTPACKET_SIZE			1
/// USB Endpoint X Status Register Short Packet Interrupt Flag mask
#define USB_REG_UESTAX_SHORTPACKET_MASK			(((1 << USB_REG_UESTAX_SHORTPACKET_SIZE) - 1) << USB_REG_UESTAX_SHORTPACKET_OFFSET)
/// USB Endpoint X Status Register Data Toggle Sequence offset
#define USB_REG_UESTAX_DTSEQ_OFFSET			8
/// USB Endpoint X Status Register Data Toggle Sequence size
#define USB_REG_UESTAX_DTSEQ_SIZE			2
/// USB Endpoint X Status Register Data Toggle Sequence mask
#define USB_REG_UESTAX_DTSEQ_MASK			(((1 << USB_REG_UESTAX_DTSEQ_SIZE) - 1) << USB_REG_UESTAX_DTSEQ_OFFSET)
/// USB Endpoint X Status Register Number of Busy Banks offset
#define USB_REG_UESTAX_NBUSYBK_OFFSET			12
/// USB Endpoint X Status Register Number of Busy Banks size
#define USB_REG_UESTAX_NBUSYBK_SIZE			2
/// USB Endpoint X Status Register Number of Busy Banks mask
#define USB_REG_UESTAX_NBUSYBK_MASK			(((1 << USB_REG_UESTAX_NBUSYBK_SIZE) - 1) << USB_REG_UESTAX_NBUSYBK_OFFSET)
/// USB Endpoint X Status Register Current Bank offset
#define USB_REG_UESTAX_CURRBK_OFFSET			14
/// USB Endpoint X Status Register Current Bank size
#define USB_REG_UESTAX_CURRBK_SIZE			2
/// USB Endpoint X Status Register Current Bank mask
#define USB_REG_UESTAX_CURRBK_MASK			(((1 << USB_REG_UESTAX_CURRBK_SIZE) - 1) << USB_REG_UESTAX_CURRBK_OFFSET)
/// USB Endpoint X Status Register Read/Write Allowed offset
#define USB_REG_UESTAX_RWALL_OFFSET			16
/// USB Endpoint X Status Register Read/Write Allowed size
#define USB_REG_UESTAX_RWALL_SIZE			1
/// USB Endpoint X Status Register Read/Write Allowed mask
#define USB_REG_UESTAX_RWALL_MASK			(((1 << USB_REG_UESTAX_RWALL_SIZE) - 1) << USB_REG_UESTAX_RWALL_OFFSET)
/// USB Endpoint X Status Register Control Direction offset
#define USB_REG_UESTAX_CTRLDIR_OFFSET			17
/// USB Endpoint X Status Register Control Direction size
#define USB_REG_UESTAX_CTRLDIR_SIZE			1
/// USB Endpoint X Status Register Control Direction mask
#define USB_REG_UESTAX_CTRLDIR_MASK			(((1 << USB_REG_UESTAX_CTRLDIR_SIZE) - 1) << USB_REG_UESTAX_CTRLDIR_OFFSET)
/// USB Endpoint X Status Register Configuration OK Status offset
#define USB_REG_UESTAX_CFGOK_OFFSET			18
/// USB Endpoint X Status Register Configuration OK Status size
#define USB_REG_UESTAX_CFGOK_SIZE			1
/// USB Endpoint X Status Register Configuration OK Status mask
#define USB_REG_UESTAX_CFGOK_MASK			(((1 << USB_REG_UESTAX_CFGOK_SIZE) - 1) << USB_REG_UESTAX_CFGOK_OFFSET)
/// USB Endpoint X Status Register Byte Count offset
#define USB_REG_UESTAX_BYCT_OFFSET			20
/// USB Endpoint X Status Register Byte Count size
#define USB_REG_UESTAX_BYCT_SIZE			11
/// USB Endpoint X Status Register Byte Count mask
#define USB_REG_UESTAX_BYCT_MASK			(((1 << USB_REG_UESTAX_BYCT_SIZE) - 1) << USB_REG_UESTAX_BYCT_OFFSET)
/// USB Endpoint 0 Status Clear Register (Write)
#define USB_REG_UESTA0CLR				0x00000160
/// USB Endpoint 1 Status Clear Register (Write)
#define USB_REG_UESTA1CLR				0x00000164
/// USB Endpoint 2 Status Clear Register (Write)
#define USB_REG_UESTA2CLR				0x00000168
/// USB Endpoint 3 Status Clear Register (Write)
#define USB_REG_UESTA3CLR				0x0000016c
/// USB Endpoint 4 Status Clear Register (Write)
#define USB_REG_UESTA4CLR				0x00000170
/// USB Endpoint 5 Status Clear Register (Write)
#define USB_REG_UESTA5CLR				0x00000174
/// USB Endpoint 6 Status Clear Register (Write)
#define USB_REG_UESTA6CLR				0x00000178
/// USB Endpoint X Status Clear Register Transmitted IN Data Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_TXINIC_OFFSET			0
/// USB Endpoint X Status Clear Register Transmitted IN Data Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_TXINIC_SIZE			1
/// USB Endpoint X Status Clear Register Transmitted IN Data Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_TXINIC_MASK			(((1 << USB_REG_UESTAXCLR_TXINIC_SIZE) - 1) << USB_REG_UESTAXCLR_TXINIC_OFFSET)
/// USB Endpoint X Status Clear Register Received OUT Data Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_RXOUTIC_OFFSET		1
/// USB Endpoint X Status Clear Register Received OUT Data Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_RXOUTIC_SIZE			1
/// USB Endpoint X Status Clear Register Received OUT Data Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_RXOUTIC_MASK			(((1 << USB_REG_UESTAXCLR_RXOUTIC_SIZE) - 1) << USB_REG_UESTAXCLR_RXOUTIC_OFFSET)
/// USB Endpoint X Status Clear Register Received SETUP Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_RXSTPIC_OFFSET		2
/// USB Endpoint X Status Clear Register Received SETUP Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_RXSTPIC_SIZE			1
/// USB Endpoint X Status Clear Register Received SETUP Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_RXSTPIC_MASK			(((1 << USB_REG_UESTAXCLR_RXSTPIC_SIZE) - 1) << USB_REG_UESTAXCLR_RXSTPIC_OFFSET)
/// USB Endpoint X Status Clear Register Underflow Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_UNDERFIC_OFFSET		2
/// USB Endpoint X Status Clear Register Underflow Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_UNDERFIC_SIZE			1
/// USB Endpoint X Status Clear Register Underflow Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_UNDERFIC_MASK			(((1 << USB_REG_UESTAXCLR_UNDERFIC_SIZE) - 1) << USB_REG_UESTAXCLR_UNDERFIC_OFFSET)
/// USB Endpoint X Status Clear Register NAKed OUT Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_NAKOUTIC_OFFSET		3
/// USB Endpoint X Status Clear Register NAKed OUT Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_NAKOUTIC_SIZE			1
/// USB Endpoint X Status Clear Register NAKed OUT Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_NAKOUTIC_MASK			(((1 << USB_REG_UESTAXCLR_NAKOUTIC_SIZE) - 1) << USB_REG_UESTAXCLR_NAKOUTIC_OFFSET)
/// USB Endpoint X Status Clear Register NAKed IN Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_NAKINIC_OFFSET		4
/// USB Endpoint X Status Clear Register NAKed IN Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_NAKINIC_SIZE			1
/// USB Endpoint X Status Clear Register NAKed IN Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_NAKINIC_MASK			(((1 << USB_REG_UESTAXCLR_NAKINIC_SIZE) - 1) << USB_REG_UESTAXCLR_NAKINIC_OFFSET)
/// USB Endpoint X Status Clear Register Overflow Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_OVERFIC_OFFSET		5
/// USB Endpoint X Status Clear Register Overflow Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_OVERFIC_SIZE			1
/// USB Endpoint X Status Clear Register Overflow Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_OVERFIC_MASK			(((1 << USB_REG_UESTAXCLR_OVERFIC_SIZE) - 1) << USB_REG_UESTAXCLR_OVERFIC_OFFSET)
/// USB Endpoint X Status Clear Register STALLed Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_STALLEDIC_OFFSET		6
/// USB Endpoint X Status Clear Register STALLed Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_STALLEDIC_SIZE		1
/// USB Endpoint X Status Clear Register STALLed Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_STALLEDIC_MASK		(((1 << USB_REG_UESTAXCLR_STALLEDIC_SIZE) - 1) << USB_REG_UESTAXCLR_STALLEDIC_OFFSET)
/// USB Endpoint X Status Clear Register CRC Error Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_CRCERRIC_OFFSET		6
/// USB Endpoint X Status Clear Register CRC Error Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_CRCERRIC_SIZE			1
/// USB Endpoint X Status Clear Register CRC Error Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_CRCERRIC_MASK			(((1 << USB_REG_UESTAXCLR_CRCERRIC_SIZE) - 1) << USB_REG_UESTAXCLR_CRCERRIC_OFFSET)
/// USB Endpoint X Status Clear Register Short Packet Interrupt Flag Clear offset
#define USB_REG_UESTAXCLR_SHORTPACKETC_OFFSET		7
/// USB Endpoint X Status Clear Register Short Packet Interrupt Flag Clear size
#define USB_REG_UESTAXCLR_SHORTPACKETC_SIZE		1
/// USB Endpoint X Status Clear Register Short Packet Interrupt Flag Clear mask
#define USB_REG_UESTAXCLR_SHORTPACKETC_MASK		(((1 << USB_REG_UESTAXCLR_SHORTPACKETC_SIZE) - 1) << USB_REG_UESTAXCLR_SHORTPACKETC_OFFSET)
/// USB Endpoint 0 Status Set Register (Write)
#define USB_REG_UESTA0SET				0x00000190
/// USB Endpoint 1 Status Set Register (Write)
#define USB_REG_UESTA01ET				0x00000194
/// USB Endpoint 2 Status Set Register (Write)
#define USB_REG_UESTA02ET				0x00000198
/// USB Endpoint 3 Status Set Register (Write)
#define USB_REG_UESTA03ET				0x0000019c
/// USB Endpoint 4 Status Set Register (Write)
#define USB_REG_UESTA04ET				0x000001a0
/// USB Endpoint 5 Status Set Register (Write)
#define USB_REG_UESTA05ET				0x000001a4
/// USB Endpoint 6 Status Set Register (Write)
#define USB_REG_UESTA06ET				0x000001a8
/// USB Endpoint X Status Set Register Transmitted IN Data Interrupt Flag Set offset
#define USB_REG_UESTAXSET_TXINIS_OFFSET			0
/// USB Endpoint X Status Set Register Transmitted IN Data Interrupt Flag Set size
#define USB_REG_UESTAXSET_TXINIS_SIZE			1
/// USB Endpoint X Status Set Register Transmitted IN Data Interrupt Flag Set mask
#define USB_REG_UESTAXSET_TXINIS_MASK			(((1 << USB_REG_UESTAXSET_TXINIS_SIZE) - 1) << USB_REG_UESTAXSET_TXINIS_OFFSET)
/// USB Endpoint X Status Set Register Received OUT Data Interrupt Flag Set offset
#define USB_REG_UESTAXSET_RXOUTIS_OFFSET		1
/// USB Endpoint X Status Set Register Received OUT Data Interrupt Flag Set size
#define USB_REG_UESTAXSET_RXOUTIS_SIZE			1
/// USB Endpoint X Status Set Register Received OUT Data Interrupt Flag Set mask
#define USB_REG_UESTAXSET_RXOUTIS_MASK			(((1 << USB_REG_UESTAXSET_RXOUTIS_SIZE) - 1) << USB_REG_UESTAXSET_RXOUTIS_OFFSET)
/// USB Endpoint X Status Set Register Received SETUP Interrupt Flag Set offset
#define USB_REG_UESTAXSET_RXSTPIS_OFFSET		2
/// USB Endpoint X Status Set Register Received SETUP Interrupt Flag Set size
#define USB_REG_UESTAXSET_RXSTPIS_SIZE			1
/// USB Endpoint X Status Set Register Received SETUP Interrupt Flag Set mask
#define USB_REG_UESTAXSET_RXSTPIS_MASK			(((1 << USB_REG_UESTAXSET_RXSTPIS_SIZE) - 1) << USB_REG_UESTAXSET_RXSTPIS_OFFSET)
/// USB Endpoint X Status Set Register Underflow Interrupt Flag Set offset
#define USB_REG_UESTAXSET_UNDERFIS_OFFSET		2
/// USB Endpoint X Status Set Register Underflow Interrupt Flag Set size
#define USB_REG_UESTAXSET_UNDERFIS_SIZE			1
/// USB Endpoint X Status Set Register Underflow Interrupt Flag Set mask
#define USB_REG_UESTAXSET_UNDERFIS_MASK			(((1 << USB_REG_UESTAXSET_UNDERFIS_SIZE) - 1) << USB_REG_UESTAXSET_UNDERFIS_OFFSET)
/// USB Endpoint X Status Set Register NAKed OUT Interrupt Flag Set offset
#define USB_REG_UESTAXSET_NAKOUTIS_OFFSET		3
/// USB Endpoint X Status Set Register NAKed OUT Interrupt Flag Set size
#define USB_REG_UESTAXSET_NAKOUTIS_SIZE			1
/// USB Endpoint X Status Set Register NAKed OUT Interrupt Flag Set mask
#define USB_REG_UESTAXSET_NAKOUTIS_MASK			(((1 << USB_REG_UESTAXSET_NAKOUTIS_SIZE) - 1) << USB_REG_UESTAXSET_NAKOUTIS_OFFSET)
/// USB Endpoint X Status Set Register NAKed IN Interrupt Flag Set offset
#define USB_REG_UESTAXSET_NAKINIS_OFFSET		4
/// USB Endpoint X Status Set Register NAKed IN Interrupt Flag Set size
#define USB_REG_UESTAXSET_NAKINIS_SIZE			1
/// USB Endpoint X Status Set Register NAKed IN Interrupt Flag Set mask
#define USB_REG_UESTAXSET_NAKINIS_MASK			(((1 << USB_REG_UESTAXSET_NAKINIS_SIZE) - 1) << USB_REG_UESTAXSET_NAKINIS_OFFSET)
/// USB Endpoint X Status Set Register Overflow Interrupt Flag Set offset
#define USB_REG_UESTAXSET_OVERFIS_OFFSET		5
/// USB Endpoint X Status Set Register Overflow Interrupt Flag Set size
#define USB_REG_UESTAXSET_OVERFIS_SIZE			1
/// USB Endpoint X Status Set Register Overflow Interrupt Flag Set mask
#define USB_REG_UESTAXSET_OVERFIS_MASK			(((1 << USB_REG_UESTAXSET_OVERFIS_SIZE) - 1) << USB_REG_UESTAXSET_OVERFIS_OFFSET)
/// USB Endpoint X Status Set Register STALLed Interrupt Flag Set offset
#define USB_REG_UESTAXSET_STALLEDIS_OFFSET		6
/// USB Endpoint X Status Set Register STALLed Interrupt Flag Set size
#define USB_REG_UESTAXSET_STALLEDIS_SIZE		1
/// USB Endpoint X Status Set Register STALLed Interrupt Flag Set mask
#define USB_REG_UESTAXSET_STALLEDIS_MASK		(((1 << USB_REG_UESTAXSET_STALLEDIS_SIZE) - 1) << USB_REG_UESTAXSET_STALLEDIS_OFFSET)
/// USB Endpoint X Status Set Register CRC Error Interrupt Flag Set offset
#define USB_REG_UESTAXSET_CRCERRIS_OFFSET		6
/// USB Endpoint X Status Set Register CRC Error Interrupt Flag Set size
#define USB_REG_UESTAXSET_CRCERRIS_SIZE			1
/// USB Endpoint X Status Set Register CRC Error Interrupt Flag Set mask
#define USB_REG_UESTAXSET_CRCERRIS_MASK			(((1 << USB_REG_UESTAXSET_CRCERRIS_SIZE) - 1) << USB_REG_UESTAXSET_CRCERRIS_OFFSET)
/// USB Endpoint X Status Set Register Short Packet Interrupt Flag Set offset
#define USB_REG_UESTAXSET_SHORTPACKETS_OFFSET		7
/// USB Endpoint X Status Set Register Short Packet Interrupt Flag Set size
#define USB_REG_UESTAXSET_SHORTPACKETS_SIZE		1
/// USB Endpoint X Status Set Register Short Packet Interrupt Flag Set mask
#define USB_REG_UESTAXSET_SHORTPACKETS_MASK		(((1 << USB_REG_UESTAXSET_SHORTPACKETS_SIZE) - 1) << USB_REG_UESTAXSET_SHORTPACKETS_OFFSET)
/// USB Endpoint X Status Set Register Number of Busy Banks Interrupt Flag Set offset
#define USB_REG_UESTAXSET_NBUSYBKS_OFFSET		12
/// USB Endpoint X Status Set Register Number of Busy Banks Interrupt Flag Set size
#define USB_REG_UESTAXSET_NBUSYBKS_SIZE			1
/// USB Endpoint X Status Set Register Number of Busy Banks Interrupt Flag Set mask
#define USB_REG_UESTAXSET_NBUSYBKS_MASK			(((1 << USB_REG_UESTAXSET_NBUSYBKS_SIZE) - 1) << USB_REG_UESTAXSET_NBUSYBKS_OFFSET)
/// USB Endpoint 0 Control Register (Read)
#define USB_REG_UECON0					0x000001c0
/// USB Endpoint 1 Control Register (Read)
#define USB_REG_UECON1					0x000001c4
/// USB Endpoint 2 Control Register (Read)
#define USB_REG_UECON2					0x000001c8
/// USB Endpoint 3 Control Register (Read)
#define USB_REG_UECON3					0x000001cc
/// USB Endpoint 4 Control Register (Read)
#define USB_REG_UECON4					0x000001d0
/// USB Endpoint 5 Control Register (Read)
#define USB_REG_UECON5					0x000001d4
/// USB Endpoint 6 Control Register (Read)
#define USB_REG_UECON6					0x000001d8
/// USB Endpoint X Control Register Transmitted IN Data Interrupt Enable offset
#define USB_REG_UECONX_TXINE_OFFSET			0
/// USB Endpoint X Control Register Transmitted IN Data Interrupt Enable size
#define USB_REG_UECONX_TXINE_SIZE			1
/// USB Endpoint X Control Register Transmitted IN Data Interrupt Enable mask
#define USB_REG_UECONX_TXINE_MASK			(((1 << USB_REG_UECONX_TXINE_SIZE) - 1) << USB_REG_UECONX_TXINE_OFFSET)
/// USB Endpoint X Control Register Received OUT Data Interrupt Enable offset
#define USB_REG_UECONX_RXOUTE_OFFSET			1
/// USB Endpoint X Control Register Received OUT Data Interrupt Enable size
#define USB_REG_UECONX_RXOUTE_SIZE			1
/// USB Endpoint X Control Register Received OUT Data Interrupt Enable mask
#define USB_REG_UECONX_RXOUTE_MASK			(((1 << USB_REG_UECONX_RXOUTE_SIZE) - 1) << USB_REG_UECONX_RXOUTE_OFFSET)
/// USB Endpoint X Control Register Received SETUP Interrupt Enable offset
#define USB_REG_UECONX_RXSTPE_OFFSET			2
/// USB Endpoint X Control Register Received SETUP Interrupt Enable size
#define USB_REG_UECONX_RXSTPE_SIZE			1
/// USB Endpoint X Control Register Received SETUP Interrupt Enable mask
#define USB_REG_UECONX_RXSTPE_MASK			(((1 << USB_REG_UECONX_RXSTPE_SIZE) - 1) << USB_REG_UECONX_RXSTPE_OFFSET)
/// USB Endpoint X Control Register Underflow Interrupt Enable offset
#define USB_REG_UECONX_UNDERFE_OFFSET			2
/// USB Endpoint X Control Register Underflow Interrupt Enable size
#define USB_REG_UECONX_UNDERFE_SIZE			1
/// USB Endpoint X Control Register Underflow Interrupt Enable mask
#define USB_REG_UECONX_UNDERFE_MASK			(((1 << USB_REG_UECONX_UNDERFE_SIZE) - 1) << USB_REG_UECONX_UNDERFE_OFFSET)
/// USB Endpoint X Control Register NAKed OUT Interrupt Enable offset
#define USB_REG_UECONX_NAKOUTE_OFFSET			3
/// USB Endpoint X Control Register NAKed OUT Interrupt Enable size
#define USB_REG_UECONX_NAKOUTE_SIZE			1
/// USB Endpoint X Control Register NAKed OUT Interrupt Enable mask
#define USB_REG_UECONX_NAKOUTE_MASK			(((1 << USB_REG_UECONX_NAKOUTE_SIZE) - 1) << USB_REG_UECONX_NAKOUTE_OFFSET)
/// USB Endpoint X Control Register NAKed IN Interrupt Enable offset
#define USB_REG_UECONX_NAKINE_OFFSET			4
/// USB Endpoint X Control Register NAKed IN Interrupt Enable size
#define USB_REG_UECONX_NAKINE_SIZE			1
/// USB Endpoint X Control Register NAKed IN Interrupt Enable mask
#define USB_REG_UECONX_NAKINE_MASK			(((1 << USB_REG_UECONX_NAKINE_SIZE) - 1) << USB_REG_UECONX_NAKINE_OFFSET)
/// USB Endpoint X Control Register Overflow Interrupt Enable offset
#define USB_REG_UECONX_OVERFE_OFFSET			5
/// USB Endpoint X Control Register Overflow Interrupt Enable size
#define USB_REG_UECONX_OVERFE_SIZE			1
/// USB Endpoint X Control Register Overflow Interrupt Enable mask
#define USB_REG_UECONX_OVERFE_MASK			(((1 << USB_REG_UECONX_OVERFE_SIZE) - 1) << USB_REG_UECONX_OVERFE_OFFSET)
/// USB Endpoint X Control Register STALLed Interrupt Enable offset
#define USB_REG_UECONX_STALLEDE_OFFSET			6
/// USB Endpoint X Control Register STALLed Interrupt Enable size
#define USB_REG_UECONX_STALLEDE_SIZE			1
/// USB Endpoint X Control Register STALLed Interrupt Enable mask
#define USB_REG_UECONX_STALLEDE_MASK			(((1 << USB_REG_UECONX_STALLEDE_SIZE) - 1) << USB_REG_UECONX_STALLEDE_OFFSET)
/// USB Endpoint X Control Register CRC Error Interrupt Enable offset
#define USB_REG_UECONX_CRCERRE_OFFSET			6
/// USB Endpoint X Control Register CRC Error Interrupt Enable size
#define USB_REG_UECONX_CRCERRE_SIZE			1
/// USB Endpoint X Control Register CRC Error Interrupt Enable mask
#define USB_REG_UECONX_CRCERRE_MASK			(((1 << USB_REG_UECONX_CRCERRE_SIZE) - 1) << USB_REG_UECONX_CRCERRE_OFFSET)
/// USB Endpoint X Control Register Short Packet Interrupt Enable offset
#define USB_REG_UECONX_SHORTPACKETE_OFFSET		7
/// USB Endpoint X Control Register Short Packet Interrupt Enable size
#define USB_REG_UECONX_SHORTPACKETE_SIZE		1
/// USB Endpoint X Control Register Short Packet Interrupt Enable mask
#define USB_REG_UECONX_SHORTPACKETE_MASK		(((1 << USB_REG_UECONX_SHORTPACKETE_SIZE) - 1) << USB_REG_UECONX_SHORTPACKETE_OFFSET)
/// USB Endpoint X Control Register Number of Busy Banks Interrupt Enable offset
#define USB_REG_UECONX_NBUSYBKE_OFFSET			12
/// USB Endpoint X Control Register Number of Busy Banks Interrupt Enable size
#define USB_REG_UECONX_NBUSYBKE_SIZE			1
/// USB Endpoint X Control Register Number of Busy Banks Interrupt Enable mask
#define USB_REG_UECONX_NBUSYBKE_MASK			(((1 << USB_REG_UECONX_NBUSYBKE_SIZE) - 1) << USB_REG_UECONX_NBUSYBKE_OFFSET)
/// USB Endpoint X Control Register Kill IN Bank offset
#define USB_REG_UECONX_KILLBK_OFFSET			13
/// USB Endpoint X Control Register Kill IN Bank size
#define USB_REG_UECONX_KILLBK_SIZE			1
/// USB Endpoint X Control Register Kill IN Bank mask
#define USB_REG_UECONX_KILLBK_MASK			(((1 << USB_REG_UECONX_KILLBK_SIZE) - 1) << USB_REG_UECONX_KILLBK_OFFSET)
/// USB Endpoint X Control Register FIFO Control offset
#define USB_REG_UECONX_FIFOCON_OFFSET			14
/// USB Endpoint X Control Register FIFO Control size
#define USB_REG_UECONX_FIFOCON_SIZE			1
/// USB Endpoint X Control Register FIFO Control mask
#define USB_REG_UECONX_FIFOCON_MASK			(((1 << USB_REG_UECONX_FIFOCON_SIZE) - 1) << USB_REG_UECONX_FIFOCON_OFFSET)
/// USB Endpoint X Control Register Endpoint Interrupts Disable HDMA Request Enable offset
#define USB_REG_UECONX_EPDISHDMA_OFFSET			16
/// USB Endpoint X Control Register Endpoint Interrupts Disable HDMA Request Enable size
#define USB_REG_UECONX_EPDISHDMA_SIZE			1
/// USB Endpoint X Control Register Endpoint Interrupts Disable HDMA Request Enable mask
#define USB_REG_UECONX_EPDISHDMA_MASK			(((1 << USB_REG_UECONX_EPDISHDMA_SIZE) - 1) << USB_REG_UECONX_EPDISHDMA_OFFSET)
/// USB Endpoint X Control Register Reset Data Toggle offset
#define USB_REG_UECONX_RSTDT_OFFSET			18
/// USB Endpoint X Control Register Reset Data Toggle size
#define USB_REG_UECONX_RSTDT_SIZE			1
/// USB Endpoint X Control Register Reset Data Toggle mask
#define USB_REG_UECONX_RSTDT_MASK			(((1 << USB_REG_UECONX_RSTDT_SIZE) - 1) << USB_REG_UECONX_RSTDT_OFFSET)
/// USB Endpoint X Control Register STALL Request offset
#define USB_REG_UECONX_STALLRQ_OFFSET			19
/// USB Endpoint X Control Register STALL Request size
#define USB_REG_UECONX_STALLRQ_SIZE			1
/// USB Endpoint X Control Register STALL Request mask
#define USB_REG_UECONX_STALLRQ_MASK			(((1 << USB_REG_UECONX_STALLRQ_SIZE) - 1) << USB_REG_UECONX_STALLRQ_OFFSET)
/// USB Endpoint 0 Control Set Register (Write)
#define USB_REG_UECON0SET				0x000001f0
/// USB Endpoint 1 Control Set Register (Write)
#define USB_REG_UECON1SET				0x000001f4
/// USB Endpoint 2 Control Set Register (Write)
#define USB_REG_UECON2SET				0x000001f8
/// USB Endpoint 3 Control Set Register (Write)
#define USB_REG_UECON3SET				0x000001fc
/// USB Endpoint 4 Control Set Register (Write)
#define USB_REG_UECON4SET				0x00000200
/// USB Endpoint 5 Control Set Register (Write)
#define USB_REG_UECON5SET				0x00000204
/// USB Endpoint 6 Control Set Register (Write)
#define USB_REG_UECON6SET				0x00000208
/// USB Endpoint X Control Set Register Transmitted IN Data Interrupt Enable Set offset
#define USB_REG_UECONXSET_TXINES_OFFSET			0
/// USB Endpoint X Control Set Register Transmitted IN Data Interrupt Enable Set size
#define USB_REG_UECONXSET_TXINES_SIZE			1
/// USB Endpoint X Control Set Register Transmitted IN Data Interrupt Enable Set mask
#define USB_REG_UECONXSET_TXINES_MASK			(((1 << USB_REG_UECONXSET_TXINES_SIZE) - 1) << USB_REG_UECONXSET_TXINES_OFFSET)
/// USB Endpoint X Control Set Register Received OUT Data Interrupt Enable Set offset
#define USB_REG_UECONXSET_RXOUTES_OFFSET		1
/// USB Endpoint X Control Set Register Received OUT Data Interrupt Enable Set size
#define USB_REG_UECONXSET_RXOUTES_SIZE			1
/// USB Endpoint X Control Set Register Received OUT Data Interrupt Enable Set mask
#define USB_REG_UECONXSET_RXOUTES_MASK			(((1 << USB_REG_UECONXSET_RXOUTES_SIZE) - 1) << USB_REG_UECONXSET_RXOUTES_OFFSET)
/// USB Endpoint X Control Set Register Received SETUP Interrupt Enable Set offset
#define USB_REG_UECONXSET_RXSTPES_OFFSET		2
/// USB Endpoint X Control Set Register Received SETUP Interrupt Enable Set size
#define USB_REG_UECONXSET_RXSTPES_SIZE			1
/// USB Endpoint X Control Set Register Received SETUP Interrupt Enable Set mask
#define USB_REG_UECONXSET_RXSTPES_MASK			(((1 << USB_REG_UECONXSET_RXSTPES_SIZE) - 1) << USB_REG_UECONXSET_RXSTPES_OFFSET)
/// USB Endpoint X Control Set Register Underflow Interrupt Enable Set offset
#define USB_REG_UECONXSET_UNDERFES_OFFSET		2
/// USB Endpoint X Control Set Register Underflow Interrupt Enable Set size
#define USB_REG_UECONXSET_UNDERFES_SIZE			1
/// USB Endpoint X Control Set Register Underflow Interrupt Enable Set mask
#define USB_REG_UECONXSET_UNDERFES_MASK			(((1 << USB_REG_UECONXSET_UNDERFES_SIZE) - 1) << USB_REG_UECONXSET_UNDERFES_OFFSET)
/// USB Endpoint X Control Set Register NAKed OUT Interrupt Enable Set offset
#define USB_REG_UECONXSET_NAKOUTES_OFFSET		3
/// USB Endpoint X Control Set Register NAKed OUT Interrupt Enable Set size
#define USB_REG_UECONXSET_NAKOUTES_SIZE			1
/// USB Endpoint X Control Set Register NAKed OUT Interrupt Enable Set mask
#define USB_REG_UECONXSET_NAKOUTES_MASK			(((1 << USB_REG_UECONXSET_NAKOUTES_SIZE) - 1) << USB_REG_UECONXSET_NAKOUTES_OFFSET)
/// USB Endpoint X Control Set Register NAKed IN Interrupt Enable Set offset
#define USB_REG_UECONXSET_NAKINES_OFFSET		4
/// USB Endpoint X Control Set Register NAKed IN Interrupt Enable Set size
#define USB_REG_UECONXSET_NAKINES_SIZE			1
/// USB Endpoint X Control Set Register NAKed IN Interrupt Enable Set mask
#define USB_REG_UECONXSET_NAKINES_MASK			(((1 << USB_REG_UECONXSET_NAKINES_SIZE) - 1) << USB_REG_UECONXSET_NAKINES_OFFSET)
/// USB Endpoint X Control Set Register Overflow Interrupt Enable Set offset
#define USB_REG_UECONXSET_OVERFES_OFFSET		5
/// USB Endpoint X Control Set Register Overflow Interrupt Enable Set size
#define USB_REG_UECONXSET_OVERFES_SIZE			1
/// USB Endpoint X Control Set Register Overflow Interrupt Enable Set mask
#define USB_REG_UECONXSET_OVERFES_MASK			(((1 << USB_REG_UECONXSET_OVERFES_SIZE) - 1) << USB_REG_UECONXSET_OVERFES_OFFSET)
/// USB Endpoint X Control Set Register STALLed Interrupt Enable Set offset
#define USB_REG_UECONXSET_STALLEDES_OFFSET		6
/// USB Endpoint X Control Set Register STALLed Interrupt Enable Set size
#define USB_REG_UECONXSET_STALLEDES_SIZE		1
/// USB Endpoint X Control Set Register STALLed Interrupt Enable Set mask
#define USB_REG_UECONXSET_STALLEDES_MASK		(((1 << USB_REG_UECONXSET_STALLEDES_SIZE) - 1) << USB_REG_UECONXSET_STALLEDES_OFFSET)
/// USB Endpoint X Control Set Register CRC Error Interrupt Enable Set offset
#define USB_REG_UECONXSET_CRCERRES_OFFSET		6
/// USB Endpoint X Control Set Register CRC Error Interrupt Enable Set size
#define USB_REG_UECONXSET_CRCERRES_SIZE			1
/// USB Endpoint X Control Set Register CRC Error Interrupt Enable Set mask
#define USB_REG_UECONXSET_CRCERRES_MASK			(((1 << USB_REG_UECONXSET_CRCERRES_SIZE) - 1) << USB_REG_UECONXSET_CRCERRES_OFFSET)
/// USB Endpoint X Control Set Register Short Packet Interrupt Enable Set offset
#define USB_REG_UECONXSET_SHORTPACKETES_OFFSET		7
/// USB Endpoint X Control Set Register Short Packet Interrupt Enable Set size
#define USB_REG_UECONXSET_SHORTPACKETES_SIZE		1
/// USB Endpoint X Control Set Register Short Packet Interrupt Enable Set mask
#define USB_REG_UECONXSET_SHORTPACKETES_MASK		(((1 << USB_REG_UECONXSET_SHORTPACKETES_SIZE) - 1) << USB_REG_UECONXSET_SHORTPACKETES_OFFSET)
/// USB Endpoint X Control Set Register Number of Busy Banks Interrupt Enable Set offset
#define USB_REG_UECONXSET_NBUSYBKES_OFFSET		12
/// USB Endpoint X Control Set Register Number of Busy Banks Interrupt Enable Set size
#define USB_REG_UECONXSET_NBUSYBKES_SIZE		1
/// USB Endpoint X Control Set Register Number of Busy Banks Interrupt Enable Set mask
#define USB_REG_UECONXSET_NBUSYBKES_MASK		(((1 << USB_REG_UECONXSET_NBUSYBKES_SIZE) - 1) << USB_REG_UECONXSET_NBUSYBKES_OFFSET)
/// USB Endpoint X Control Set Register Kill IN Bank Set offset
#define USB_REG_UECONXSET_KILLBKS_OFFSET		13
/// USB Endpoint X Control Set Register Kill IN Bank Set size
#define USB_REG_UECONXSET_KILLBKS_SIZE			1
/// USB Endpoint X Control Set Register Kill IN Bank Set mask
#define USB_REG_UECONXSET_KILLBKS_MASK			(((1 << USB_REG_UECONXSET_KILLBKS_SIZE) - 1) << USB_REG_UECONXSET_KILLBKS_OFFSET)
/// USB Endpoint X Control Set Register Endpoint Interrupts Disable HDMA Request Enable Set offset
#define USB_REG_UECONXSET_EPDISHDMAC_OFFSET		16
/// USB Endpoint X Control Set Register Endpoint Interrupts Disable HDMA Request Enable Set size
#define USB_REG_UECONXSET_EPDISHDMAC_SIZE		1
/// USB Endpoint X Control Set Register Endpoint Interrupts Disable HDMA Request Enable Set mask
#define USB_REG_UECONXSET_EPDISHDMAC_MASK		(((1 << USB_REG_UECONXSET_EPDISHDMAC_SIZE) - 1) << USB_REG_UECONXSET_EPDISHDMAC_OFFSET)
/// USB Endpoint X Control Set Register Reset Data Toggle Set offset
#define USB_REG_UECONXSET_RSTDTS_OFFSET			18
/// USB Endpoint X Control Set Register Reset Data Toggle Set size
#define USB_REG_UECONXSET_RSTDTS_SIZE			1
/// USB Endpoint X Control Set Register Reset Data Toggle Set mask
#define USB_REG_UECONXSET_RSTDTS_MASK			(((1 << USB_REG_UECONXSET_RSTDTS_SIZE) - 1) << USB_REG_UECONXSET_RSTDTS_OFFSET)
/// USB Endpoint X Control Set Register STALL Request Set offset
#define USB_REG_UECONXSET_STALLRQC_OFFSET		19
/// USB Endpoint X Control Set Register STALL Request Set size
#define USB_REG_UECONXSET_STALLRQC_SIZE			1
/// USB Endpoint X Control Set Register STALL Request Set mask
#define USB_REG_UECONXSET_STALLRQC_MASK			(((1 << USB_REG_UECONXSET_STALLRQC_SIZE) - 1) << USB_REG_UECONXSET_STALLRQC_OFFSET)
/// USB Endpoint 0 Control Clear Register (Write)
#define USB_REG_UECON0CLR				0x00000220
/// USB Endpoint 1 Control Clear Register (Write)
#define USB_REG_UECON1CLR				0x00000224
/// USB Endpoint 2 Control Clear Register (Write)
#define USB_REG_UECON2CLR				0x00000228
/// USB Endpoint 3 Control Clear Register (Write)
#define USB_REG_UECON3CLR				0x0000022c
/// USB Endpoint 4 Control Clear Register (Write)
#define USB_REG_UECON4CLR				0x00000230
/// USB Endpoint 5 Control Clear Register (Write)
#define USB_REG_UECON5CLR				0x00000234
/// USB Endpoint 6 Control Clear Register (Write)
#define USB_REG_UECON6CLR				0x00000238
/// USB Endpoint X Control Clear Register Transmitted IN Data Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_TXINEC_OFFSET			0
/// USB Endpoint X Control Clear Register Transmitted IN Data Interrupt Enable Clear size
#define USB_REG_UECONXCLR_TXINEC_SIZE			1
/// USB Endpoint X Control Clear Register Transmitted IN Data Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_TXINEC_MASK			(((1 << USB_REG_UECONXCLR_TXINEC_SIZE) - 1) << USB_REG_UECONXCLR_TXINEC_OFFSET)
/// USB Endpoint X Control Clear Register Received OUT Data Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_RXOUTEC_OFFSET		1
/// USB Endpoint X Control Clear Register Received OUT Data Interrupt Enable Clear size
#define USB_REG_UECONXCLR_RXOUTEC_SIZE			1
/// USB Endpoint X Control Clear Register Received OUT Data Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_RXOUTEC_MASK			(((1 << USB_REG_UECONXCLR_RXOUTEC_SIZE) - 1) << USB_REG_UECONXCLR_RXOUTEC_OFFSET)
/// USB Endpoint X Control Clear Register Received SETUP Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_RXSTPEC_OFFSET		2
/// USB Endpoint X Control Clear Register Received SETUP Interrupt Enable Clear size
#define USB_REG_UECONXCLR_RXSTPEC_SIZE			1
/// USB Endpoint X Control Clear Register Received SETUP Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_RXSTPEC_MASK			(((1 << USB_REG_UECONXCLR_RXSTPEC_SIZE) - 1) << USB_REG_UECONXCLR_RXSTPEC_OFFSET)
/// USB Endpoint X Control Clear Register Underflow Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_UNDERFEC_OFFSET		2
/// USB Endpoint X Control Clear Register Underflow Interrupt Enable Clear size
#define USB_REG_UECONXCLR_UNDERFEC_SIZE			1
/// USB Endpoint X Control Clear Register Underflow Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_UNDERFEC_MASK			(((1 << USB_REG_UECONXCLR_UNDERFEC_SIZE) - 1) << USB_REG_UECONXCLR_UNDERFEC_OFFSET)
/// USB Endpoint X Control Clear Register NAKed OUT Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_NAKOUTEC_OFFSET		3
/// USB Endpoint X Control Clear Register NAKed OUT Interrupt Enable Clear size
#define USB_REG_UECONXCLR_NAKOUTEC_SIZE			1
/// USB Endpoint X Control Clear Register NAKed OUT Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_NAKOUTEC_MASK			(((1 << USB_REG_UECONXCLR_NAKOUTEC_SIZE) - 1) << USB_REG_UECONXCLR_NAKOUTEC_OFFSET)
/// USB Endpoint X Control Clear Register NAKed IN Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_NAKINEC_OFFSET		4
/// USB Endpoint X Control Clear Register NAKed IN Interrupt Enable Clear size
#define USB_REG_UECONXCLR_NAKINEC_SIZE			1
/// USB Endpoint X Control Clear Register NAKed IN Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_NAKINEC_MASK			(((1 << USB_REG_UECONXCLR_NAKINEC_SIZE) - 1) << USB_REG_UECONXCLR_NAKINEC_OFFSET)
/// USB Endpoint X Control Clear Register Overflow Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_OVERFEC_OFFSET		5
/// USB Endpoint X Control Clear Register Overflow Interrupt Enable Clear size
#define USB_REG_UECONXCLR_OVERFEC_SIZE			1
/// USB Endpoint X Control Clear Register Overflow Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_OVERFEC_MASK			(((1 << USB_REG_UECONXCLR_OVERFEC_SIZE) - 1) << USB_REG_UECONXCLR_OVERFEC_OFFSET)
/// USB Endpoint X Control Clear Register STALLed Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_STALLEDEC_OFFSET		6
/// USB Endpoint X Control Clear Register STALLed Interrupt Enable Clear size
#define USB_REG_UECONXCLR_STALLEDEC_SIZE		1
/// USB Endpoint X Control Clear Register STALLed Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_STALLEDEC_MASK		(((1 << USB_REG_UECONXCLR_STALLEDEC_SIZE) - 1) << USB_REG_UECONXCLR_STALLEDEC_OFFSET)
/// USB Endpoint X Control Clear Register CRC Error Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_CRCERREC_OFFSET		6
/// USB Endpoint X Control Clear Register CRC Error Interrupt Enable Clear size
#define USB_REG_UECONXCLR_CRCERREC_SIZE			1
/// USB Endpoint X Control Clear Register CRC Error Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_CRCERREC_MASK			(((1 << USB_REG_UECONXCLR_CRCERREC_SIZE) - 1) << USB_REG_UECONXCLR_CRCERREC_OFFSET)
/// USB Endpoint X Control Clear Register Short Packet Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_SHORTPACKETEC_OFFSET		7
/// USB Endpoint X Control Clear Register Short Packet Interrupt Enable Clear size
#define USB_REG_UECONXCLR_SHORTPACKETEC_SIZE		1
/// USB Endpoint X Control Clear Register Short Packet Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_SHORTPACKETEC_MASK		(((1 << USB_REG_UECONXCLR_SHORTPACKETEC_SIZE) - 1) << USB_REG_UECONXCLR_SHORTPACKETEC_OFFSET)
/// USB Endpoint X Control Clear Register Number of Busy Banks Interrupt Enable Clear offset
#define USB_REG_UECONXCLR_NBUSYBKEC_OFFSET		12
/// USB Endpoint X Control Clear Register Number of Busy Banks Interrupt Enable Clear size
#define USB_REG_UECONXCLR_NBUSYBKEC_SIZE		1
/// USB Endpoint X Control Clear Register Number of Busy Banks Interrupt Enable Clear mask
#define USB_REG_UECONXCLR_NBUSYBKEC_MASK		(((1 << USB_REG_UECONXCLR_NBUSYBKEC_SIZE) - 1) << USB_REG_UECONXCLR_NBUSYBKEC_OFFSET)
/// USB Endpoint X Control Clear Register FIFO Control Clear offset
#define USB_REG_UECONXCLR_FIFOCONC_OFFSET		14
/// USB Endpoint X Control Clear Register FIFO Control Clear size
#define USB_REG_UECONXCLR_FIFOCONC_SIZE			1
/// USB Endpoint X Control Clear Register FIFO Control Clear mask
#define USB_REG_UECONXCLR_FIFOCONC_MASK			(((1 << USB_REG_UECONXCLR_FIFOCONC_SIZE) - 1) << USB_REG_UECONXCLR_FIFOCONC_OFFSET)
/// USB Endpoint X Control Clear Register Endpoint Interrupts Disable HDMA Request Enable Clear offset
#define USB_REG_UECONXCLR_EPDISHDMAC_OFFSET		16
/// USB Endpoint X Control Clear Register Endpoint Interrupts Disable HDMA Request Enable Clear size
#define USB_REG_UECONXCLR_EPDISHDMAC_SIZE		1
/// USB Endpoint X Control Clear Register Endpoint Interrupts Disable HDMA Request Enable Clear mask
#define USB_REG_UECONXCLR_EPDISHDMAC_MASK		(((1 << USB_REG_UECONXCLR_EPDISHDMAC_SIZE) - 1) << USB_REG_UECONXCLR_EPDISHDMAC_OFFSET)
/// USB Endpoint X Control Clear Register STALL Request Clear offset
#define USB_REG_UECONXCLR_STALLRQC_OFFSET		19
/// USB Endpoint X Control Clear Register STALL Request Clear size
#define USB_REG_UECONXCLR_STALLRQC_SIZE			1
/// USB Endpoint X Control Clear Register STALL Request Clear mask
#define USB_REG_UECONXCLR_STALLRQC_MASK			(((1 << USB_REG_UECONXCLR_STALLRQC_SIZE) - 1) << USB_REG_UECONXCLR_STALLRQC_OFFSET)
/// USB Device DMA Channel 1 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA1_NEXTDESC				0x00000310
/// USB Device DMA Channel 1 HSB Address Register (Read/Write)
#define USB_REG_UDDMA1_ADDR				0x00000314
/// USB Device DMA Channel 1 Control Register (Read/Write)
#define USB_REG_UDDMA1_CONTROL				0x00000318
/// USB Device DMA Channel 1 Status Register (Read/Write)
#define USB_REG_UDDMA1_STATUS				0x0000031c
/// USB Device DMA Channel 2 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA2_NEXTDESC				0x00000320
/// USB Device DMA Channel 2 HSB Address Register (Read/Write)
#define USB_REG_UDDMA2_ADDR				0x00000324
/// USB Device DMA Channel 2 Control Register (Read/Write)
#define USB_REG_UDDMA2_CONTROL				0x00000328
/// USB Device DMA Channel 2 Status Register (Read/Write)
#define USB_REG_UDDMA2_STATUS				0x0000032c
/// USB Device DMA Channel 3 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA3_NEXTDESC				0x00000330
/// USB Device DMA Channel 3 HSB Address Register (Read/Write)
#define USB_REG_UDDMA3_ADDR				0x00000334
/// USB Device DMA Channel 3 Control Register (Read/Write)
#define USB_REG_UDDMA3_CONTROL				0x00000338
/// USB Device DMA Channel 3 Status Register (Read/Write)
#define USB_REG_UDDMA3_STATUS				0x0000033c
/// USB Device DMA Channel 4 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA4_NEXTDESC				0x00000340
/// USB Device DMA Channel 4 HSB Address Register (Read/Write)
#define USB_REG_UDDMA4_ADDR				0x00000344
/// USB Device DMA Channel 4 Control Register (Read/Write)
#define USB_REG_UDDMA4_CONTROL				0x00000348
/// USB Device DMA Channel 4 Status Register (Read/Write)
#define USB_REG_UDDMA4_STATUS				0x0000034c
/// USB Device DMA Channel 5 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA5_NEXTDESC				0x00000350
/// USB Device DMA Channel 5 HSB Address Register (Read/Write)
#define USB_REG_UDDMA5_ADDR				0x00000354
/// USB Device DMA Channel 5 Control Register (Read/Write)
#define USB_REG_UDDMA5_CONTROL				0x00000358
/// USB Device DMA Channel 5 Status Register (Read/Write)
#define USB_REG_UDDMA5_STATUS				0x0000035c
/// USB Device DMA Channel 6 Next Descriptor Address Register (Read/Write)
#define USB_REG_UDDMA6_NEXTDESC				0x00000360
/// USB Device DMA Channel 6 HSB Address Register (Read/Write)
#define USB_REG_UDDMA6_ADDR				0x00000364
/// USB Device DMA Channel 6 Control Register (Read/Write)
#define USB_REG_UDDMA6_CONTROL				0x00000368
/// USB Device DMA Channel 6 Status Register (Read/Write)
#define USB_REG_UDDMA6_STATUS				0x0000036c
/// USB Device DMA Channel X Next Descriptor Address Register Next Descriptor Address offset
#define USB_REG_UDDMAX_NEXTDESC_NXT_DESC_ADDR_OFFSET	4
/// USB Device DMA Channel X Next Descriptor Address Register Next Descriptor Address size
#define USB_REG_UDDMAX_NEXTDESC_NXT_DESC_ADDR_SIZE	28
/// USB Device DMA Channel X Next Descriptor Address Register Next Descriptor Address mask
#define USB_REG_UDDMAX_NEXTDESC_NXT_DESC_ADDR_MASK	(((1 << USB_REG_UDDMAX_NEXTDESC_NXT_DESC_ADDR_SIZE) - 1) << USB_REG_UDDMAX_NEXTDESC_NXT_DESC_ADDR_OFFSET)
/// USB Device DMA Channel X HSB Address Register HSB Address offset
#define USB_REG_UDDMAX_ADDR_HSB_ADDR_OFFSET		0
/// USB Device DMA Channel X HSB Address Register HSB Address size
#define USB_REG_UDDMAX_ADDR_HSB_ADDR_SIZE		32
/// USB Device DMA Channel X HSB Address Register HSB Address mask
#define USB_REG_UDDMAX_ADDR_HSB_ADDR_MASK		(((1 << USB_REG_UDDMAX_ADDR_HSB_ADDR_SIZE) - 1) << USB_REG_UDDMAX_ADDR_HSB_ADDR_OFFSET)
/// USB Device DMA Channel X Control Register Channel Enable offset
#define USB_REG_UDDMAX_CONTROL_CH_EN_OFFSET		0
/// USB Device DMA Channel X Control Register Channel Enable size
#define USB_REG_UDDMAX_CONTROL_CH_EN_SIZE		1
/// USB Device DMA Channel X Control Register Channel Enable mask
#define USB_REG_UDDMAX_CONTROL_CH_EN_MASK		(((1 << USB_REG_UDDMAX_CONTROL_CH_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_CH_EN_OFFSET)
/// USB Device DMA Channel X Control Register Load Next Channel Descriptor Enable offset
#define USB_REG_UDDMAX_CONTROL_LD_NXT_CH_DESC_EN_OFFSET	1
/// USB Device DMA Channel X Control Register Load Next Channel Descriptor Enable size
#define USB_REG_UDDMAX_CONTROL_LD_NXT_CH_DESC_EN_SIZE	1
/// USB Device DMA Channel X Control Register Load Next Channel Descriptor Enable mask
#define USB_REG_UDDMAX_CONTROL_LD_NXT_CH_DESC_EN_MASK	(((1 << USB_REG_UDDMAX_CONTROL_LD_NXT_CH_DESC_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_LD_NXT_CH_DESC_EN_OFFSET)
/// USB Device DMA Channel X Control Register Buffer Close Input Enable offset
#define USB_REG_UDDMAX_CONTROL_BUFF_CLOSE_IN_EN_OFFSET	2
/// USB Device DMA Channel X Control Register Buffer Close Input Enable size
#define USB_REG_UDDMAX_CONTROL_BUFF_CLOSE_IN_EN_SIZE	1
/// USB Device DMA Channel X Control Register Buffer Close Input Enable mask
#define USB_REG_UDDMAX_CONTROL_BUFF_CLOSE_IN_EN_MASK	(((1 << USB_REG_UDDMAX_CONTROL_BUFF_CLOSE_IN_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_BUFF_CLOSE_IN_EN_OFFSET)
/// USB Device DMA Channel X Control Register End of DMA Buffer Output Enable offset
#define USB_REG_UDDMAX_CONTROL_DMAEND_EN_OFFSET		3
/// USB Device DMA Channel X Control Register End of DMA Buffer Output Enable size
#define USB_REG_UDDMAX_CONTROL_DMAEND_EN_SIZE		1
/// USB Device DMA Channel X Control Register End of DMA Buffer Output Enable mask
#define USB_REG_UDDMAX_CONTROL_DMAEND_EN_MASK		(((1 << USB_REG_UDDMAX_CONTROL_DMAEND_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_DMAEND_EN_OFFSET)
/// USB Device DMA Channel X Control Register End of USB Transfer Interrupt Enable offset
#define USB_REG_UDDMAX_CONTROL_EOT_IRQ_EN_OFFSET	4
/// USB Device DMA Channel X Control Register End of USB Transfer Interrupt Enable size
#define USB_REG_UDDMAX_CONTROL_EOT_IRQ_EN_SIZE		1
/// USB Device DMA Channel X Control Register End of USB Transfer Interrupt Enable mask
#define USB_REG_UDDMAX_CONTROL_EOT_IRQ_EN_MASK		(((1 << USB_REG_UDDMAX_CONTROL_EOT_IRQ_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_EOT_IRQ_EN_OFFSET)
/// USB Device DMA Channel X Control Register End of Buffer Interrupt Enable offset
#define USB_REG_UDDMAX_CONTROL_EOBUFF_IRQ_EN_OFFSET	5
/// USB Device DMA Channel X Control Register End of Buffer Interrupt Enable size
#define USB_REG_UDDMAX_CONTROL_EOBUFF_IRQ_EN_SIZE	1
/// USB Device DMA Channel X Control Register End of Buffer Interrupt Enable mask
#define USB_REG_UDDMAX_CONTROL_EOBUFF_IRQ_EN_MASK	(((1 << USB_REG_UDDMAX_CONTROL_EOBUFF_IRQ_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_EOBUFF_IRQ_EN_OFFSET)
/// USB Device DMA Channel X Control Register Descriptor Loaded Interrupt Enable offset
#define USB_REG_UDDMAX_CONTROL_DESC_LD_IRQ_EN_OFFSET	6
/// USB Device DMA Channel X Control Register Descriptor Loaded Interrupt Enable size
#define USB_REG_UDDMAX_CONTROL_DESC_LD_IRQ_EN_SIZE	1
/// USB Device DMA Channel X Control Register Descriptor Loaded Interrupt Enable mask
#define USB_REG_UDDMAX_CONTROL_DESC_LD_IRQ_EN_MASK	(((1 << USB_REG_UDDMAX_CONTROL_DESC_LD_IRQ_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_DESC_LD_IRQ_EN_OFFSET)
/// USB Device DMA Channel X Control Register Burst Lock Enable offset
#define USB_REG_UDDMAX_CONTROL_BURST_LOCK_EN_OFFSET	7
/// USB Device DMA Channel X Control Register Burst Lock Enable size
#define USB_REG_UDDMAX_CONTROL_BURST_LOCK_EN_SIZE	1
/// USB Device DMA Channel X Control Register Burst Lock Enable mask
#define USB_REG_UDDMAX_CONTROL_BURST_LOCK_EN_MASK	(((1 << USB_REG_UDDMAX_CONTROL_BURST_LOCK_EN_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_BURST_LOCK_EN_OFFSET)
/// USB Device DMA Channel X Control Register Channel Byte Length offset
#define USB_REG_UDDMAX_CONTROL_CH_BYTE_LENGTH_OFFSET	16
/// USB Device DMA Channel X Control Register Channel Byte Length size
#define USB_REG_UDDMAX_CONTROL_CH_BYTE_LENGTH_SIZE	16
/// USB Device DMA Channel X Control Register Channel Byte Length mask
#define USB_REG_UDDMAX_CONTROL_CH_BYTE_LENGTH_MASK	(((1 << USB_REG_UDDMAX_CONTROL_CH_BYTE_LENGTH_SIZE) - 1) << USB_REG_UDDMAX_CONTROL_CH_BYTE_LENGTH_OFFSET)
/// USB Device DMA Channel X Status Register Channel Enable offset
#define USB_REG_UDDMAX_STATUS_CH_EN_OFFSET		0
/// USB Device DMA Channel X Status Register Channel Enable size
#define USB_REG_UDDMAX_STATUS_CH_EN_SIZE		1
/// USB Device DMA Channel X Status Register Channel Enable mask
#define USB_REG_UDDMAX_STATUS_CH_EN_MASK		(((1 << USB_REG_UDDMAX_STATUS_CH_EN_SIZE) - 1) << USB_REG_UDDMAX_STATUS_CH_EN_OFFSET)
/// USB Device DMA Channel X Status Register Channel Active offset
#define USB_REG_UDDMAX_STATUS_CH_ACTIVE_OFFSET		1
/// USB Device DMA Channel X Status Register Channel Active size
#define USB_REG_UDDMAX_STATUS_CH_ACTIVE_SIZE		1
/// USB Device DMA Channel X Status Register Channel Active mask
#define USB_REG_UDDMAX_STATUS_CH_ACTIVE_MASK		(((1 << USB_REG_UDDMAX_STATUS_CH_ACTIVE_SIZE) - 1) << USB_REG_UDDMAX_STATUS_CH_ACTIVE_OFFSET)
/// USB Device DMA Channel X Status Register End of USB Transfer Status offset
#define USB_REG_UDDMAX_STATUS_EOT_STA_OFFSET		4
/// USB Device DMA Channel X Status Register End of USB Transfer Status size
#define USB_REG_UDDMAX_STATUS_EOT_STA_SIZE		1
/// USB Device DMA Channel X Status Register End of USB Transfer Status mask
#define USB_REG_UDDMAX_STATUS_EOT_STA_MASK		(((1 << USB_REG_UDDMAX_STATUS_EOT_STA_SIZE) - 1) << USB_REG_UDDMAX_STATUS_EOT_STA_OFFSET)
/// USB Device DMA Channel X Status Register End of Channel Buffer Status offset
#define USB_REG_UDDMAX_STATUS_EOCH_BUFF_STA_OFFSET	5
/// USB Device DMA Channel X Status Register End of Channel Buffer Status size
#define USB_REG_UDDMAX_STATUS_EOCH_BUFF_STA_SIZE	1
/// USB Device DMA Channel X Status Register End of Channel Buffer Status mask
#define USB_REG_UDDMAX_STATUS_EOCH_BUFF_STA_MASK	(((1 << USB_REG_UDDMAX_STATUS_EOCH_BUFF_STA_SIZE) - 1) << USB_REG_UDDMAX_STATUS_EOCH_BUFF_STA_OFFSET)
/// USB Device DMA Channel X Status Register Descriptor Loaded Status offset
#define USB_REG_UDDMAX_STATUS_DESC_LD_STA_OFFSET	6
/// USB Device DMA Channel X Status Register Descriptor Loaded Status size
#define USB_REG_UDDMAX_STATUS_DESC_LD_STA_SIZE		1
/// USB Device DMA Channel X Status Register Descriptor Loaded Status mask
#define USB_REG_UDDMAX_STATUS_DESC_LD_STA_MASK		(((1 << USB_REG_UDDMAX_STATUS_DESC_LD_STA_SIZE) - 1) << USB_REG_UDDMAX_STATUS_DESC_LD_STA_OFFSET)
/// USB Device DMA Channel X Status Register Channel Byte Count offset
#define USB_REG_UDDMAX_STATUS_CH_BYTE_CNT_OFFSET	16
/// USB Device DMA Channel X Status Register Channel Byte Count size
#define USB_REG_UDDMAX_STATUS_CH_BYTE_CNT_SIZE		16
/// USB Device DMA Channel X Status Register Channel Byte Count mask
#define USB_REG_UDDMAX_STATUS_CH_BYTE_CNT_MASK		(((1 << USB_REG_UDDMAX_STATUS_CH_BYTE_CNT_SIZE) - 1) << USB_REG_UDDMAX_STATUS_CH_BYTE_CNT_OFFSET)
/// USB Host General Control Register (Read/Write)
#define USB_REG_UHCON					0x00000400
/// USB Host General Control Register Start of Frame Generation Enable offset
#define USB_REG_UHCON_SOFE_OFFSET			8
/// USB Host General Control Register Start of Frame Generation Enable size
#define USB_REG_UHCON_SOFE_SIZE				1
/// USB Host General Control Register Start of Frame Generation Enable mask
#define USB_REG_UHCON_SOFE_MASK				(((1 << USB_REG_UHCON_SOFE_SIZE) - 1) << USB_REG_UHCON_SOFE_OFFSET)
/// USB Host General Control Register Send USB Reset offset
#define USB_REG_UHCON_RESET_OFFSET			9
/// USB Host General Control Register Send USB Reset size
#define USB_REG_UHCON_RESET_SIZE			1
/// USB Host General Control Register Send USB Reset mask
#define USB_REG_UHCON_RESET_MASK			(((1 << USB_REG_UHCON_RESET_SIZE) - 1) << USB_REG_UHCON_RESET_OFFSET)
/// USB Host General Control Register Send USB Resume offset
#define USB_REG_UHCON_RESUME_OFFSET			10
/// USB Host General Control Register Send USB Resume size
#define USB_REG_UHCON_RESUME_SIZE			1
/// USB Host General Control Register Send USB Resume mask
#define USB_REG_UHCON_RESUME_MASK			(((1 << USB_REG_UHCON_RESUME_SIZE) - 1) << USB_REG_UHCON_RESUME_OFFSET)
/// USB Host General Interrupt Register (Read)
#define USB_REG_UHINT					0x00000404
/// USB Host General Interrupt Register Device Connection Interrupt Flag offset
#define USB_REG_UHINT_DCONNI_OFFSET			0
/// USB Host General Interrupt Register Device Connection Interrupt Flag size
#define USB_REG_UHINT_DCONNI_SIZE			1
/// USB Host General Interrupt Register Device Connection Interrupt Flag mask
#define USB_REG_UHINT_DCONNI_MASK			(((1 << USB_REG_UHINT_DCONNI_SIZE) - 1) << USB_REG_UHINT_DCONNI_OFFSET)
/// USB Host General Interrupt Register Device Disconnection Interrupt Flag offset
#define USB_REG_UHINT_DDISCI_OFFSET			1
/// USB Host General Interrupt Register Device Disconnection Interrupt Flag size
#define USB_REG_UHINT_DDISCI_SIZE			1
/// USB Host General Interrupt Register Device Disconnection Interrupt Flag mask
#define USB_REG_UHINT_DDISCI_MASK			(((1 << USB_REG_UHINT_DDISCI_SIZE) - 1) << USB_REG_UHINT_DDISCI_OFFSET)
/// USB Host General Interrupt Register USB Reset Sent Interrupt Flag offset
#define USB_REG_UHINT_RSTI_OFFSET			2
/// USB Host General Interrupt Register USB Reset Sent Interrupt Flag size
#define USB_REG_UHINT_RSTI_SIZE				1
/// USB Host General Interrupt Register USB Reset Sent Interrupt Flag mask
#define USB_REG_UHINT_RSTI_MASK				(((1 << USB_REG_UHINT_RSTI_SIZE) - 1) << USB_REG_UHINT_RSTI_OFFSET)
/// USB Host General Interrupt Register Downstream Resume Sent Interrupt Flag offset
#define USB_REG_UHINT_RSMEDI_OFFSET			3
/// USB Host General Interrupt Register Downstream Resume Sent Interrupt Flag size
#define USB_REG_UHINT_RSMEDI_SIZE			1
/// USB Host General Interrupt Register Downstream Resume Sent Interrupt Flag mask
#define USB_REG_UHINT_RSMEDI_MASK			(((1 << USB_REG_UHINT_RSMEDI_SIZE) - 1) << USB_REG_UHINT_RSMEDI_OFFSET)
/// USB Host General Interrupt Register Upstream Resume Received Interrupt Flag offset
#define USB_REG_UHINT_RXRSMI_OFFSET			4
/// USB Host General Interrupt Register Upstream Resume Received Interrupt Flag size
#define USB_REG_UHINT_RXRSMI_SIZE			1
/// USB Host General Interrupt Register Upstream Resume Received Interrupt Flag mask
#define USB_REG_UHINT_RXRSMI_MASK			(((1 << USB_REG_UHINT_RXRSMI_SIZE) - 1) << USB_REG_UHINT_RXRSMI_OFFSET)
/// USB Host General Interrupt Register Host Start of Frame Interrupt Flag offset
#define USB_REG_UHINT_HSOFI_OFFSET			5
/// USB Host General Interrupt Register Host Start of Frame Interrupt Flag size
#define USB_REG_UHINT_HSOFI_SIZE			1
/// USB Host General Interrupt Register Host Start of Frame Interrupt Flag mask
#define USB_REG_UHINT_HSOFI_MASK			(((1 << USB_REG_UHINT_HSOFI_SIZE) - 1) << USB_REG_UHINT_HSOFI_OFFSET)
/// USB Host General Interrupt Register Host Wake-Up Interrupt Flag offset
#define USB_REG_UHINT_HWUPI_OFFSET			6
/// USB Host General Interrupt Register Host Wake-Up Interrupt Flag size
#define USB_REG_UHINT_HWUPI_SIZE			1
/// USB Host General Interrupt Register Host Wake-Up Interrupt Flag mask
#define USB_REG_UHINT_HWUPI_MASK			(((1 << USB_REG_UHINT_HWUPI_SIZE) - 1) << USB_REG_UHINT_HWUPI_OFFSET)
/// USB Host General Interrupt Register Pipe 0 Interrupt Flag offset
#define USB_REG_UHINT_P0INT_OFFSET			8
/// USB Host General Interrupt Register Pipe 0 Interrupt Flag size
#define USB_REG_UHINT_P0INT_SIZE			1
/// USB Host General Interrupt Register Pipe 0 Interrupt Flag mask
#define USB_REG_UHINT_P0INT_MASK			(((1 << USB_REG_UHINT_P0INT_SIZE) - 1) << USB_REG_UHINT_P0INT_OFFSET)
/// USB Host General Interrupt Register Pipe 1 Interrupt Flag offset
#define USB_REG_UHINT_P1INT_OFFSET			9
/// USB Host General Interrupt Register Pipe 1 Interrupt Flag size
#define USB_REG_UHINT_P1INT_SIZE			1
/// USB Host General Interrupt Register Pipe 1 Interrupt Flag mask
#define USB_REG_UHINT_P1INT_MASK			(((1 << USB_REG_UHINT_P1INT_SIZE) - 1) << USB_REG_UHINT_P1INT_OFFSET)
/// USB Host General Interrupt Register Pipe 2 Interrupt Flag offset
#define USB_REG_UHINT_P2INT_OFFSET			10
/// USB Host General Interrupt Register Pipe 2 Interrupt Flag size
#define USB_REG_UHINT_P2INT_SIZE			1
/// USB Host General Interrupt Register Pipe 2 Interrupt Flag mask
#define USB_REG_UHINT_P2INT_MASK			(((1 << USB_REG_UHINT_P2INT_SIZE) - 1) << USB_REG_UHINT_P2INT_OFFSET)
/// USB Host General Interrupt Register Pipe 3 Interrupt Flag offset
#define USB_REG_UHINT_P3INT_OFFSET			11
/// USB Host General Interrupt Register Pipe 3 Interrupt Flag size
#define USB_REG_UHINT_P3INT_SIZE			1
/// USB Host General Interrupt Register Pipe 3 Interrupt Flag mask
#define USB_REG_UHINT_P3INT_MASK			(((1 << USB_REG_UHINT_P3INT_SIZE) - 1) << USB_REG_UHINT_P3INT_OFFSET)
/// USB Host General Interrupt Register Pipe 4 Interrupt Flag offset
#define USB_REG_UHINT_P4INT_OFFSET			12
/// USB Host General Interrupt Register Pipe 4 Interrupt Flag size
#define USB_REG_UHINT_P4INT_SIZE			1
/// USB Host General Interrupt Register Pipe 4 Interrupt Flag mask
#define USB_REG_UHINT_P4INT_MASK			(((1 << USB_REG_UHINT_P4INT_SIZE) - 1) << USB_REG_UHINT_P4INT_OFFSET)
/// USB Host General Interrupt Register Pipe 5 Interrupt Flag offset
#define USB_REG_UHINT_P5INT_OFFSET			13
/// USB Host General Interrupt Register Pipe 5 Interrupt Flag size
#define USB_REG_UHINT_P5INT_SIZE			1
/// USB Host General Interrupt Register Pipe 5 Interrupt Flag mask
#define USB_REG_UHINT_P5INT_MASK			(((1 << USB_REG_UHINT_P5INT_SIZE) - 1) << USB_REG_UHINT_P5INT_OFFSET)
/// USB Host General Interrupt Register Pipe 6 Interrupt Flag offset
#define USB_REG_UHINT_P6INT_OFFSET			14
/// USB Host General Interrupt Register Pipe 6 Interrupt Flag size
#define USB_REG_UHINT_P6INT_SIZE			1
/// USB Host General Interrupt Register Pipe 6 Interrupt Flag mask
#define USB_REG_UHINT_P6INT_MASK			(((1 << USB_REG_UHINT_P6INT_SIZE) - 1) << USB_REG_UHINT_P6INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 1 Interrupt Flag offset
#define USB_REG_UHINT_DMA1INT_OFFSET			25
/// USB Host General Interrupt Register DMA Channel 1 Interrupt Flag size
#define USB_REG_UHINT_DMA1INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 1 Interrupt Flag mask
#define USB_REG_UHINT_DMA1INT_MASK			(((1 << USB_REG_UHINT_DMA1INT_SIZE) - 1) << USB_REG_UHINT_DMA1INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 2 Interrupt Flag offset
#define USB_REG_UHINT_DMA2INT_OFFSET			26
/// USB Host General Interrupt Register DMA Channel 2 Interrupt Flag size
#define USB_REG_UHINT_DMA2INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 2 Interrupt Flag mask
#define USB_REG_UHINT_DMA2INT_MASK			(((1 << USB_REG_UHINT_DMA2INT_SIZE) - 1) << USB_REG_UHINT_DMA2INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 3 Interrupt Flag offset
#define USB_REG_UHINT_DMA3INT_OFFSET			27
/// USB Host General Interrupt Register DMA Channel 3 Interrupt Flag size
#define USB_REG_UHINT_DMA3INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 3 Interrupt Flag mask
#define USB_REG_UHINT_DMA3INT_MASK			(((1 << USB_REG_UHINT_DMA3INT_SIZE) - 1) << USB_REG_UHINT_DMA3INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 4 Interrupt Flag offset
#define USB_REG_UHINT_DMA4INT_OFFSET			28
/// USB Host General Interrupt Register DMA Channel 4 Interrupt Flag size
#define USB_REG_UHINT_DMA4INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 4 Interrupt Flag mask
#define USB_REG_UHINT_DMA4INT_MASK			(((1 << USB_REG_UHINT_DMA4INT_SIZE) - 1) << USB_REG_UHINT_DMA4INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 5 Interrupt Flag offset
#define USB_REG_UHINT_DMA5INT_OFFSET			29
/// USB Host General Interrupt Register DMA Channel 5 Interrupt Flag size
#define USB_REG_UHINT_DMA5INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 5 Interrupt Flag mask
#define USB_REG_UHINT_DMA5INT_MASK			(((1 << USB_REG_UHINT_DMA5INT_SIZE) - 1) << USB_REG_UHINT_DMA5INT_OFFSET)
/// USB Host General Interrupt Register DMA Channel 6 Interrupt Flag offset
#define USB_REG_UHINT_DMA6INT_OFFSET			30
/// USB Host General Interrupt Register DMA Channel 6 Interrupt Flag size
#define USB_REG_UHINT_DMA6INT_SIZE			1
/// USB Host General Interrupt Register DMA Channel 6 Interrupt Flag mask
#define USB_REG_UHINT_DMA6INT_MASK			(((1 << USB_REG_UHINT_DMA6INT_SIZE) - 1) << USB_REG_UHINT_DMA6INT_OFFSET)
/// USB Host General Interrupt Clear Register (Write)
#define USB_REG_UHINTCLR				0x00000408
/// USB Host General Interrupt Clear Register Device Connection Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_DCONNIC_OFFSET			0
/// USB Host General Interrupt Clear Register Device Connection Interrupt Flag Clear size
#define USB_REG_UHINTCLR_DCONNIC_SIZE			1
/// USB Host General Interrupt Clear Register Device Connection Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_DCONNIC_MASK			(((1 << USB_REG_UHINTCLR_DCONNIC_SIZE) - 1) << USB_REG_UHINTCLR_DCONNIC_OFFSET)
/// USB Host General Interrupt Clear Register Device Disconnection Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_DDISCIC_OFFSET			1
/// USB Host General Interrupt Clear Register Device Disconnection Interrupt Flag Clear size
#define USB_REG_UHINTCLR_DDISCIC_SIZE			1
/// USB Host General Interrupt Clear Register Device Disconnection Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_DDISCIC_MASK			(((1 << USB_REG_UHINTCLR_DDISCIC_SIZE) - 1) << USB_REG_UHINTCLR_DDISCIC_OFFSET)
/// USB Host General Interrupt Clear Register USB Reset Sent Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_RSTIC_OFFSET			2
/// USB Host General Interrupt Clear Register USB Reset Sent Interrupt Flag Clear size
#define USB_REG_UHINTCLR_RSTIC_SIZE			1
/// USB Host General Interrupt Clear Register USB Reset Sent Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_RSTIC_MASK			(((1 << USB_REG_UHINTCLR_RSTIC_SIZE) - 1) << USB_REG_UHINTCLR_RSTIC_OFFSET)
/// USB Host General Interrupt Clear Register Downstream Resume Sent Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_RSMEDIC_OFFSET			3
/// USB Host General Interrupt Clear Register Downstream Resume Sent Interrupt Flag Clear size
#define USB_REG_UHINTCLR_RSMEDIC_SIZE			1
/// USB Host General Interrupt Clear Register Downstream Resume Sent Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_RSMEDIC_MASK			(((1 << USB_REG_UHINTCLR_RSMEDIC_SIZE) - 1) << USB_REG_UHINTCLR_RSMEDIC_OFFSET)
/// USB Host General Interrupt Clear Register Upstream Resume Received Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_RXRSMIC_OFFSET			4
/// USB Host General Interrupt Clear Register Upstream Resume Received Interrupt Flag Clear size
#define USB_REG_UHINTCLR_RXRSMIC_SIZE			1
/// USB Host General Interrupt Clear Register Upstream Resume Received Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_RXRSMIC_MASK			(((1 << USB_REG_UHINTCLR_RXRSMIC_SIZE) - 1) << USB_REG_UHINTCLR_RXRSMIC_OFFSET)
/// USB Host General Interrupt Clear Register Host Start of Frame Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_HSOFIC_OFFSET			5
/// USB Host General Interrupt Clear Register Host Start of Frame Interrupt Flag Clear size
#define USB_REG_UHINTCLR_HSOFIC_SIZE			1
/// USB Host General Interrupt Clear Register Host Start of Frame Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_HSOFIC_MASK			(((1 << USB_REG_UHINTCLR_HSOFIC_SIZE) - 1) << USB_REG_UHINTCLR_HSOFIC_OFFSET)
/// USB Host General Interrupt Clear Register Host Wake-Up Interrupt Flag Clear offset
#define USB_REG_UHINTCLR_HWUPIC_OFFSET			6
/// USB Host General Interrupt Clear Register Host Wake-Up Interrupt Flag Clear size
#define USB_REG_UHINTCLR_HWUPIC_SIZE			1
/// USB Host General Interrupt Clear Register Host Wake-Up Interrupt Flag Clear mask
#define USB_REG_UHINTCLR_HWUPIC_MASK			(((1 << USB_REG_UHINTCLR_HWUPIC_SIZE) - 1) << USB_REG_UHINTCLR_HWUPIC_OFFSET)
/// USB Host General Interrupt Set Register (Write)
#define USB_REG_UHINTSET				0x0000040c
/// USB Host General Interrupt Set Register Device Connection Interrupt Flag Set offset
#define USB_REG_UHINTSET_DCONNIS_OFFSET			0
/// USB Host General Interrupt Set Register Device Connection Interrupt Flag Set size
#define USB_REG_UHINTSET_DCONNIS_SIZE			1
/// USB Host General Interrupt Set Register Device Connection Interrupt Flag Set mask
#define USB_REG_UHINTSET_DCONNIS_MASK			(((1 << USB_REG_UHINTSET_DCONNIS_SIZE) - 1) << USB_REG_UHINTSET_DCONNIS_OFFSET)
/// USB Host General Interrupt Set Register Device Disconnection Interrupt Flag Set offset
#define USB_REG_UHINTSET_DDISCIS_OFFSET			1
/// USB Host General Interrupt Set Register Device Disconnection Interrupt Flag Set size
#define USB_REG_UHINTSET_DDISCIS_SIZE			1
/// USB Host General Interrupt Set Register Device Disconnection Interrupt Flag Set mask
#define USB_REG_UHINTSET_DDISCIS_MASK			(((1 << USB_REG_UHINTSET_DDISCIS_SIZE) - 1) << USB_REG_UHINTSET_DDISCIS_OFFSET)
/// USB Host General Interrupt Set Register USB Reset Sent Interrupt Flag Set offset
#define USB_REG_UHINTSET_RSTIS_OFFSET			2
/// USB Host General Interrupt Set Register USB Reset Sent Interrupt Flag Set size
#define USB_REG_UHINTSET_RSTIS_SIZE			1
/// USB Host General Interrupt Set Register USB Reset Sent Interrupt Flag Set mask
#define USB_REG_UHINTSET_RSTIS_MASK			(((1 << USB_REG_UHINTSET_RSTIS_SIZE) - 1) << USB_REG_UHINTSET_RSTIS_OFFSET)
/// USB Host General Interrupt Set Register Downstream Resume Sent Interrupt Flag Set offset
#define USB_REG_UHINTSET_RSMEDIS_OFFSET			3
/// USB Host General Interrupt Set Register Downstream Resume Sent Interrupt Flag Set size
#define USB_REG_UHINTSET_RSMEDIS_SIZE			1
/// USB Host General Interrupt Set Register Downstream Resume Sent Interrupt Flag Set mask
#define USB_REG_UHINTSET_RSMEDIS_MASK			(((1 << USB_REG_UHINTSET_RSMEDIS_SIZE) - 1) << USB_REG_UHINTSET_RSMEDIS_OFFSET)
/// USB Host General Interrupt Set Register Upstream Resume Received Interrupt Flag Set offset
#define USB_REG_UHINTSET_RXRSMIS_OFFSET			4
/// USB Host General Interrupt Set Register Upstream Resume Received Interrupt Flag Set size
#define USB_REG_UHINTSET_RXRSMIS_SIZE			1
/// USB Host General Interrupt Set Register Upstream Resume Received Interrupt Flag Set mask
#define USB_REG_UHINTSET_RXRSMIS_MASK			(((1 << USB_REG_UHINTSET_RXRSMIS_SIZE) - 1) << USB_REG_UHINTSET_RXRSMIS_OFFSET)
/// USB Host General Interrupt Set Register Host Start of Frame Interrupt Flag Set offset
#define USB_REG_UHINTSET_HSOFIS_OFFSET			5
/// USB Host General Interrupt Set Register Host Start of Frame Interrupt Flag Set size
#define USB_REG_UHINTSET_HSOFIS_SIZE			1
/// USB Host General Interrupt Set Register Host Start of Frame Interrupt Flag Set mask
#define USB_REG_UHINTSET_HSOFIS_MASK			(((1 << USB_REG_UHINTSET_HSOFIS_SIZE) - 1) << USB_REG_UHINTSET_HSOFIS_OFFSET)
/// USB Host General Interrupt Set Register Host Wake-Up Interrupt Flag Set offset
#define USB_REG_UHINTSET_HWUPIS_OFFSET			6
/// USB Host General Interrupt Set Register Host Wake-Up Interrupt Flag Set size
#define USB_REG_UHINTSET_HWUPIS_SIZE			1
/// USB Host General Interrupt Set Register Host Wake-Up Interrupt Flag Set mask
#define USB_REG_UHINTSET_HWUPIS_MASK			(((1 << USB_REG_UHINTSET_HWUPIS_SIZE) - 1) << USB_REG_UHINTSET_HWUPIS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 1 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA1INTS_OFFSET		25
/// USB Host General Interrupt Set Register DMA Channel 1 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA1INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 1 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA1INTS_MASK			(((1 << USB_REG_UHINTSET_DMA1INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA1INTS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 2 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA2INTS_OFFSET		26
/// USB Host General Interrupt Set Register DMA Channel 2 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA2INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 2 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA2INTS_MASK			(((1 << USB_REG_UHINTSET_DMA2INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA2INTS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 3 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA3INTS_OFFSET		27
/// USB Host General Interrupt Set Register DMA Channel 3 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA3INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 3 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA3INTS_MASK			(((1 << USB_REG_UHINTSET_DMA3INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA3INTS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 4 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA4INTS_OFFSET		28
/// USB Host General Interrupt Set Register DMA Channel 4 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA4INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 4 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA4INTS_MASK			(((1 << USB_REG_UHINTSET_DMA4INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA4INTS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 5 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA5INTS_OFFSET		29
/// USB Host General Interrupt Set Register DMA Channel 5 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA5INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 5 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA5INTS_MASK			(((1 << USB_REG_UHINTSET_DMA5INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA5INTS_OFFSET)
/// USB Host General Interrupt Set Register DMA Channel 6 Interrupt Flag Set offset
#define USB_REG_UHINTSET_DMA6INTS_OFFSET		30
/// USB Host General Interrupt Set Register DMA Channel 6 Interrupt Flag Set size
#define USB_REG_UHINTSET_DMA6INTS_SIZE			1
/// USB Host General Interrupt Set Register DMA Channel 6 Interrupt Flag Set mask
#define USB_REG_UHINTSET_DMA6INTS_MASK			(((1 << USB_REG_UHINTSET_DMA6INTS_SIZE) - 1) << USB_REG_UHINTSET_DMA6INTS_OFFSET)
/// USB Host General Interrupt Enable Register (Read)
#define USB_REG_UHINTE					0x00000410
/// USB Host General Interrupt Enable Register Device Connection Interrupt Enable offset
#define USB_REG_UHINTE_DCONNIE_OFFSET			0
/// USB Host General Interrupt Enable Register Device Connection Interrupt Enable size
#define USB_REG_UHINTE_DCONNIE_SIZE			1
/// USB Host General Interrupt Enable Register Device Connection Interrupt Enable mask
#define USB_REG_UHINTE_DCONNIE_MASK			(((1 << USB_REG_UHINTE_DCONNIE_SIZE) - 1) << USB_REG_UHINTE_DCONNIE_OFFSET)
/// USB Host General Interrupt Enable Register Device Disconnection Interrupt Enable offset
#define USB_REG_UHINTE_DDISCIE_OFFSET			1
/// USB Host General Interrupt Enable Register Device Disconnection Interrupt Enable size
#define USB_REG_UHINTE_DDISCIE_SIZE			1
/// USB Host General Interrupt Enable Register Device Disconnection Interrupt Enable mask
#define USB_REG_UHINTE_DDISCIE_MASK			(((1 << USB_REG_UHINTE_DDISCIE_SIZE) - 1) << USB_REG_UHINTE_DDISCIE_OFFSET)
/// USB Host General Interrupt Enable Register USB Reset Sent Interrupt Enable offset
#define USB_REG_UHINTE_RSTIE_OFFSET			2
/// USB Host General Interrupt Enable Register USB Reset Sent Interrupt Enable size
#define USB_REG_UHINTE_RSTIE_SIZE			1
/// USB Host General Interrupt Enable Register USB Reset Sent Interrupt Enable mask
#define USB_REG_UHINTE_RSTIE_MASK			(((1 << USB_REG_UHINTE_RSTIE_SIZE) - 1) << USB_REG_UHINTE_RSTIE_OFFSET)
/// USB Host General Interrupt Enable Register Downstream Resume Sent Interrupt Enable offset
#define USB_REG_UHINTE_RSMEDIE_OFFSET			3
/// USB Host General Interrupt Enable Register Downstream Resume Sent Interrupt Enable size
#define USB_REG_UHINTE_RSMEDIE_SIZE			1
/// USB Host General Interrupt Enable Register Downstream Resume Sent Interrupt Enable mask
#define USB_REG_UHINTE_RSMEDIE_MASK			(((1 << USB_REG_UHINTE_RSMEDIE_SIZE) - 1) << USB_REG_UHINTE_RSMEDIE_OFFSET)
/// USB Host General Interrupt Enable Register Upstream Resume Received Interrupt Enable offset
#define USB_REG_UHINTE_RXRSMIE_OFFSET			4
/// USB Host General Interrupt Enable Register Upstream Resume Received Interrupt Enable size
#define USB_REG_UHINTE_RXRSMIE_SIZE			1
/// USB Host General Interrupt Enable Register Upstream Resume Received Interrupt Enable mask
#define USB_REG_UHINTE_RXRSMIE_MASK			(((1 << USB_REG_UHINTE_RXRSMIE_SIZE) - 1) << USB_REG_UHINTE_RXRSMIE_OFFSET)
/// USB Host General Interrupt Enable Register Host Start of Frame Interrupt Enable offset
#define USB_REG_UHINTE_HSOFIE_OFFSET			5
/// USB Host General Interrupt Enable Register Host Start of Frame Interrupt Enable size
#define USB_REG_UHINTE_HSOFIE_SIZE			1
/// USB Host General Interrupt Enable Register Host Start of Frame Interrupt Enable mask
#define USB_REG_UHINTE_HSOFIE_MASK			(((1 << USB_REG_UHINTE_HSOFIE_SIZE) - 1) << USB_REG_UHINTE_HSOFIE_OFFSET)
/// USB Host General Interrupt Enable Register Host Wake-Up Interrupt Enable offset
#define USB_REG_UHINTE_HWUPIE_OFFSET			6
/// USB Host General Interrupt Enable Register Host Wake-Up Interrupt Enable size
#define USB_REG_UHINTE_HWUPIE_SIZE			1
/// USB Host General Interrupt Enable Register Host Wake-Up Interrupt Enable mask
#define USB_REG_UHINTE_HWUPIE_MASK			(((1 << USB_REG_UHINTE_HWUPIE_SIZE) - 1) << USB_REG_UHINTE_HWUPIE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 0 Interrupt Enable offset
#define USB_REG_UHINTE_P0INTE_OFFSET			8
/// USB Host General Interrupt Enable Register Pipe 0 Interrupt Enable size
#define USB_REG_UHINTE_P0INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 0 Interrupt Enable mask
#define USB_REG_UHINTE_P0INTE_MASK			(((1 << USB_REG_UHINTE_P0INTE_SIZE) - 1) << USB_REG_UHINTE_P0INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 1 Interrupt Enable offset
#define USB_REG_UHINTE_P1INTE_OFFSET			9
/// USB Host General Interrupt Enable Register Pipe 1 Interrupt Enable size
#define USB_REG_UHINTE_P1INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 1 Interrupt Enable mask
#define USB_REG_UHINTE_P1INTE_MASK			(((1 << USB_REG_UHINTE_P1INTE_SIZE) - 1) << USB_REG_UHINTE_P1INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 2 Interrupt Enable offset
#define USB_REG_UHINTE_P2INTE_OFFSET			10
/// USB Host General Interrupt Enable Register Pipe 2 Interrupt Enable size
#define USB_REG_UHINTE_P2INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 2 Interrupt Enable mask
#define USB_REG_UHINTE_P2INTE_MASK			(((1 << USB_REG_UHINTE_P2INTE_SIZE) - 1) << USB_REG_UHINTE_P2INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 3 Interrupt Enable offset
#define USB_REG_UHINTE_P3INTE_OFFSET			11
/// USB Host General Interrupt Enable Register Pipe 3 Interrupt Enable size
#define USB_REG_UHINTE_P3INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 3 Interrupt Enable mask
#define USB_REG_UHINTE_P3INTE_MASK			(((1 << USB_REG_UHINTE_P3INTE_SIZE) - 1) << USB_REG_UHINTE_P3INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 4 Interrupt Enable offset
#define USB_REG_UHINTE_P4INTE_OFFSET			12
/// USB Host General Interrupt Enable Register Pipe 4 Interrupt Enable size
#define USB_REG_UHINTE_P4INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 4 Interrupt Enable mask
#define USB_REG_UHINTE_P4INTE_MASK			(((1 << USB_REG_UHINTE_P4INTE_SIZE) - 1) << USB_REG_UHINTE_P4INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 5 Interrupt Enable offset
#define USB_REG_UHINTE_P5INTE_OFFSET			13
/// USB Host General Interrupt Enable Register Pipe 5 Interrupt Enable size
#define USB_REG_UHINTE_P5INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 5 Interrupt Enable mask
#define USB_REG_UHINTE_P5INTE_MASK			(((1 << USB_REG_UHINTE_P5INTE_SIZE) - 1) << USB_REG_UHINTE_P5INTE_OFFSET)
/// USB Host General Interrupt Enable Register Pipe 6 Interrupt Enable offset
#define USB_REG_UHINTE_P6INTE_OFFSET			14
/// USB Host General Interrupt Enable Register Pipe 6 Interrupt Enable size
#define USB_REG_UHINTE_P6INTE_SIZE			1
/// USB Host General Interrupt Enable Register Pipe 6 Interrupt Enable mask
#define USB_REG_UHINTE_P6INTE_MASK			(((1 << USB_REG_UHINTE_P6INTE_SIZE) - 1) << USB_REG_UHINTE_P6INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 1 Interrupt Enable offset
#define USB_REG_UHINTE_DMA1INTE_OFFSET			25
/// USB Host General Interrupt Enable Register DMA Channel 1 Interrupt Enable size
#define USB_REG_UHINTE_DMA1INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 1 Interrupt Enable mask
#define USB_REG_UHINTE_DMA1INTE_MASK			(((1 << USB_REG_UHINTE_DMA1INTE_SIZE) - 1) << USB_REG_UHINTE_DMA1INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 2 Interrupt Enable offset
#define USB_REG_UHINTE_DMA2INTE_OFFSET			26
/// USB Host General Interrupt Enable Register DMA Channel 2 Interrupt Enable size
#define USB_REG_UHINTE_DMA2INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 2 Interrupt Enable mask
#define USB_REG_UHINTE_DMA2INTE_MASK			(((1 << USB_REG_UHINTE_DMA2INTE_SIZE) - 1) << USB_REG_UHINTE_DMA2INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 3 Interrupt Enable offset
#define USB_REG_UHINTE_DMA3INTE_OFFSET			27
/// USB Host General Interrupt Enable Register DMA Channel 3 Interrupt Enable size
#define USB_REG_UHINTE_DMA3INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 3 Interrupt Enable mask
#define USB_REG_UHINTE_DMA3INTE_MASK			(((1 << USB_REG_UHINTE_DMA3INTE_SIZE) - 1) << USB_REG_UHINTE_DMA3INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 4 Interrupt Enable offset
#define USB_REG_UHINTE_DMA4INTE_OFFSET			28
/// USB Host General Interrupt Enable Register DMA Channel 4 Interrupt Enable size
#define USB_REG_UHINTE_DMA4INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 4 Interrupt Enable mask
#define USB_REG_UHINTE_DMA4INTE_MASK			(((1 << USB_REG_UHINTE_DMA4INTE_SIZE) - 1) << USB_REG_UHINTE_DMA4INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 5 Interrupt Enable offset
#define USB_REG_UHINTE_DMA5INTE_OFFSET			29
/// USB Host General Interrupt Enable Register DMA Channel 5 Interrupt Enable size
#define USB_REG_UHINTE_DMA5INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 5 Interrupt Enable mask
#define USB_REG_UHINTE_DMA5INTE_MASK			(((1 << USB_REG_UHINTE_DMA5INTE_SIZE) - 1) << USB_REG_UHINTE_DMA5INTE_OFFSET)
/// USB Host General Interrupt Enable Register DMA Channel 6 Interrupt Enable offset
#define USB_REG_UHINTE_DMA6INTE_OFFSET			30
/// USB Host General Interrupt Enable Register DMA Channel 6 Interrupt Enable size
#define USB_REG_UHINTE_DMA6INTE_SIZE			1
/// USB Host General Interrupt Enable Register DMA Channel 6 Interrupt Enable mask
#define USB_REG_UHINTE_DMA6INTE_MASK			(((1 << USB_REG_UHINTE_DMA6INTE_SIZE) - 1) << USB_REG_UHINTE_DMA6INTE_OFFSET)
/// USB Host General Interrupt Enable Clear Register (Write)
#define USB_REG_UHINTECLR				0x00000414
/// USB Host General Interrupt Enable Clear Register Device Connection Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DCONNIEC_OFFSET		0
/// USB Host General Interrupt Enable Clear Register Device Connection Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DCONNIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Device Connection Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DCONNIEC_MASK			(((1 << USB_REG_UHINTECLR_DCONNIEC_SIZE) - 1) << USB_REG_UHINTECLR_DCONNIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Device Disconnection Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DDISCIEC_OFFSET		1
/// USB Host General Interrupt Enable Clear Register Device Disconnection Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DDISCIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Device Disconnection Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DDISCIEC_MASK			(((1 << USB_REG_UHINTECLR_DDISCIEC_SIZE) - 1) << USB_REG_UHINTECLR_DDISCIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register USB Reset Sent Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_RSTIEC_OFFSET			2
/// USB Host General Interrupt Enable Clear Register USB Reset Sent Interrupt Enable Clear size
#define USB_REG_UHINTECLR_RSTIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register USB Reset Sent Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_RSTIEC_MASK			(((1 << USB_REG_UHINTECLR_RSTIEC_SIZE) - 1) << USB_REG_UHINTECLR_RSTIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Downstream Resume Sent Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_RSMEDIEC_OFFSET		3
/// USB Host General Interrupt Enable Clear Register Downstream Resume Sent Interrupt Enable Clear size
#define USB_REG_UHINTECLR_RSMEDIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Downstream Resume Sent Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_RSMEDIEC_MASK			(((1 << USB_REG_UHINTECLR_RSMEDIEC_SIZE) - 1) << USB_REG_UHINTECLR_RSMEDIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Upstream Resume Received Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_RXRSMIEC_OFFSET		4
/// USB Host General Interrupt Enable Clear Register Upstream Resume Received Interrupt Enable Clear size
#define USB_REG_UHINTECLR_RXRSMIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Upstream Resume Received Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_RXRSMIEC_MASK			(((1 << USB_REG_UHINTECLR_RXRSMIEC_SIZE) - 1) << USB_REG_UHINTECLR_RXRSMIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Host Start of Frame Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_HSOFIEC_OFFSET		5
/// USB Host General Interrupt Enable Clear Register Host Start of Frame Interrupt Enable Clear size
#define USB_REG_UHINTECLR_HSOFIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Host Start of Frame Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_HSOFIEC_MASK			(((1 << USB_REG_UHINTECLR_HSOFIEC_SIZE) - 1) << USB_REG_UHINTECLR_HSOFIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Host Wake-Up Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_HWUPIEC_OFFSET		6
/// USB Host General Interrupt Enable Clear Register Host Wake-Up Interrupt Enable Clear size
#define USB_REG_UHINTECLR_HWUPIEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Host Wake-Up Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_HWUPIEC_MASK			(((1 << USB_REG_UHINTECLR_HWUPIEC_SIZE) - 1) << USB_REG_UHINTECLR_HWUPIEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 0 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P0INTEC_OFFSET		8
/// USB Host General Interrupt Enable Clear Register Pipe 0 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P0INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 0 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P0INTEC_MASK			(((1 << USB_REG_UHINTECLR_P0INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P0INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 1 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P1INTEC_OFFSET		9
/// USB Host General Interrupt Enable Clear Register Pipe 1 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P1INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 1 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P1INTEC_MASK			(((1 << USB_REG_UHINTECLR_P1INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P1INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 2 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P2INTEC_OFFSET		10
/// USB Host General Interrupt Enable Clear Register Pipe 2 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P2INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 2 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P2INTEC_MASK			(((1 << USB_REG_UHINTECLR_P2INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P2INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 3 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P3INTEC_OFFSET		11
/// USB Host General Interrupt Enable Clear Register Pipe 3 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P3INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 3 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P3INTEC_MASK			(((1 << USB_REG_UHINTECLR_P3INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P3INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 4 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P4INTEC_OFFSET		12
/// USB Host General Interrupt Enable Clear Register Pipe 4 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P4INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 4 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P4INTEC_MASK			(((1 << USB_REG_UHINTECLR_P4INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P4INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 5 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P5INTEC_OFFSET		13
/// USB Host General Interrupt Enable Clear Register Pipe 5 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P5INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 5 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P5INTEC_MASK			(((1 << USB_REG_UHINTECLR_P5INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P5INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register Pipe 6 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_P6INTEC_OFFSET		14
/// USB Host General Interrupt Enable Clear Register Pipe 6 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_P6INTEC_SIZE			1
/// USB Host General Interrupt Enable Clear Register Pipe 6 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_P6INTEC_MASK			(((1 << USB_REG_UHINTECLR_P6INTEC_SIZE) - 1) << USB_REG_UHINTECLR_P6INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 1 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA1INTEC_OFFSET		25
/// USB Host General Interrupt Enable Clear Register DMA Channel 1 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA1INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 1 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA1INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA1INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA1INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 2 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA2INTEC_OFFSET		26
/// USB Host General Interrupt Enable Clear Register DMA Channel 2 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA2INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 2 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA2INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA2INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA2INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 3 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA3INTEC_OFFSET		27
/// USB Host General Interrupt Enable Clear Register DMA Channel 3 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA3INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 3 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA3INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA3INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA3INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 4 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA4INTEC_OFFSET		28
/// USB Host General Interrupt Enable Clear Register DMA Channel 4 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA4INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 4 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA4INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA4INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA4INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 5 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA5INTEC_OFFSET		29
/// USB Host General Interrupt Enable Clear Register DMA Channel 5 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA5INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 5 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA5INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA5INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA5INTEC_OFFSET)
/// USB Host General Interrupt Enable Clear Register DMA Channel 6 Interrupt Enable Clear offset
#define USB_REG_UHINTECLR_DMA6INTEC_OFFSET		30
/// USB Host General Interrupt Enable Clear Register DMA Channel 6 Interrupt Enable Clear size
#define USB_REG_UHINTECLR_DMA6INTEC_SIZE		1
/// USB Host General Interrupt Enable Clear Register DMA Channel 6 Interrupt Enable Clear mask
#define USB_REG_UHINTECLR_DMA6INTEC_MASK		(((1 << USB_REG_UHINTECLR_DMA6INTEC_SIZE) - 1) << USB_REG_UHINTECLR_DMA6INTEC_OFFSET)
/// USB Host General Interrupt Enable Set Register (Write)
#define USB_REG_UHINTESET				0x00000418
/// USB Host General Interrupt Enable Set Register Device Connection Interrupt Enable Set offset
#define USB_REG_UHINTESET_DCONNIES_OFFSET		0
/// USB Host General Interrupt Enable Set Register Device Connection Interrupt Enable Set size
#define USB_REG_UHINTESET_DCONNIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Device Connection Interrupt Enable Set mask
#define USB_REG_UHINTESET_DCONNIES_MASK			(((1 << USB_REG_UHINTESET_DCONNIES_SIZE) - 1) << USB_REG_UHINTESET_DCONNIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Device Disconnection Interrupt Enable Set offset
#define USB_REG_UHINTESET_DDISCIES_OFFSET		1
/// USB Host General Interrupt Enable Set Register Device Disconnection Interrupt Enable Set size
#define USB_REG_UHINTESET_DDISCIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Device Disconnection Interrupt Enable Set mask
#define USB_REG_UHINTESET_DDISCIES_MASK			(((1 << USB_REG_UHINTESET_DDISCIES_SIZE) - 1) << USB_REG_UHINTESET_DDISCIES_OFFSET)
/// USB Host General Interrupt Enable Set Register USB Reset Sent Interrupt Enable Set offset
#define USB_REG_UHINTESET_RSTIES_OFFSET			2
/// USB Host General Interrupt Enable Set Register USB Reset Sent Interrupt Enable Set size
#define USB_REG_UHINTESET_RSTIES_SIZE			1
/// USB Host General Interrupt Enable Set Register USB Reset Sent Interrupt Enable Set mask
#define USB_REG_UHINTESET_RSTIES_MASK			(((1 << USB_REG_UHINTESET_RSTIES_SIZE) - 1) << USB_REG_UHINTESET_RSTIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Downstream Resume Sent Interrupt Enable Set offset
#define USB_REG_UHINTESET_RSMEDIES_OFFSET		3
/// USB Host General Interrupt Enable Set Register Downstream Resume Sent Interrupt Enable Set size
#define USB_REG_UHINTESET_RSMEDIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Downstream Resume Sent Interrupt Enable Set mask
#define USB_REG_UHINTESET_RSMEDIES_MASK			(((1 << USB_REG_UHINTESET_RSMEDIES_SIZE) - 1) << USB_REG_UHINTESET_RSMEDIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Upstream Resume Received Interrupt Enable Set offset
#define USB_REG_UHINTESET_RXRSMIES_OFFSET		4
/// USB Host General Interrupt Enable Set Register Upstream Resume Received Interrupt Enable Set size
#define USB_REG_UHINTESET_RXRSMIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Upstream Resume Received Interrupt Enable Set mask
#define USB_REG_UHINTESET_RXRSMIES_MASK			(((1 << USB_REG_UHINTESET_RXRSMIES_SIZE) - 1) << USB_REG_UHINTESET_RXRSMIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Host Start of Frame Interrupt Enable Set offset
#define USB_REG_UHINTESET_HSOFIES_OFFSET		5
/// USB Host General Interrupt Enable Set Register Host Start of Frame Interrupt Enable Set size
#define USB_REG_UHINTESET_HSOFIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Host Start of Frame Interrupt Enable Set mask
#define USB_REG_UHINTESET_HSOFIES_MASK			(((1 << USB_REG_UHINTESET_HSOFIES_SIZE) - 1) << USB_REG_UHINTESET_HSOFIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Host Wake-Up Interrupt Enable Set offset
#define USB_REG_UHINTESET_HWUPIES_OFFSET		6
/// USB Host General Interrupt Enable Set Register Host Wake-Up Interrupt Enable Set size
#define USB_REG_UHINTESET_HWUPIES_SIZE			1
/// USB Host General Interrupt Enable Set Register Host Wake-Up Interrupt Enable Set mask
#define USB_REG_UHINTESET_HWUPIES_MASK			(((1 << USB_REG_UHINTESET_HWUPIES_SIZE) - 1) << USB_REG_UHINTESET_HWUPIES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 0 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P0INTES_OFFSET		8
/// USB Host General Interrupt Enable Set Register Pipe 0 Interrupt Enable Set size
#define USB_REG_UHINTESET_P0INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 0 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P0INTES_MASK			(((1 << USB_REG_UHINTESET_P0INTES_SIZE) - 1) << USB_REG_UHINTESET_P0INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 1 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P1INTES_OFFSET		9
/// USB Host General Interrupt Enable Set Register Pipe 1 Interrupt Enable Set size
#define USB_REG_UHINTESET_P1INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 1 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P1INTES_MASK			(((1 << USB_REG_UHINTESET_P1INTES_SIZE) - 1) << USB_REG_UHINTESET_P1INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 2 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P2INTES_OFFSET		10
/// USB Host General Interrupt Enable Set Register Pipe 2 Interrupt Enable Set size
#define USB_REG_UHINTESET_P2INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 2 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P2INTES_MASK			(((1 << USB_REG_UHINTESET_P2INTES_SIZE) - 1) << USB_REG_UHINTESET_P2INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 3 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P3INTES_OFFSET		11
/// USB Host General Interrupt Enable Set Register Pipe 3 Interrupt Enable Set size
#define USB_REG_UHINTESET_P3INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 3 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P3INTES_MASK			(((1 << USB_REG_UHINTESET_P3INTES_SIZE) - 1) << USB_REG_UHINTESET_P3INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 4 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P4INTES_OFFSET		12
/// USB Host General Interrupt Enable Set Register Pipe 4 Interrupt Enable Set size
#define USB_REG_UHINTESET_P4INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 4 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P4INTES_MASK			(((1 << USB_REG_UHINTESET_P4INTES_SIZE) - 1) << USB_REG_UHINTESET_P4INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 5 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P5INTES_OFFSET		13
/// USB Host General Interrupt Enable Set Register Pipe 5 Interrupt Enable Set size
#define USB_REG_UHINTESET_P5INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 5 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P5INTES_MASK			(((1 << USB_REG_UHINTESET_P5INTES_SIZE) - 1) << USB_REG_UHINTESET_P5INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register Pipe 6 Interrupt Enable Set offset
#define USB_REG_UHINTESET_P6INTES_OFFSET		14
/// USB Host General Interrupt Enable Set Register Pipe 6 Interrupt Enable Set size
#define USB_REG_UHINTESET_P6INTES_SIZE			1
/// USB Host General Interrupt Enable Set Register Pipe 6 Interrupt Enable Set mask
#define USB_REG_UHINTESET_P6INTES_MASK			(((1 << USB_REG_UHINTESET_P6INTES_SIZE) - 1) << USB_REG_UHINTESET_P6INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 1 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA1INTES_OFFSET		25
/// USB Host General Interrupt Enable Set Register DMA Channel 1 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA1INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 1 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA1INTES_MASK		(((1 << USB_REG_UHINTESET_DMA1INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA1INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 2 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA2INTES_OFFSET		26
/// USB Host General Interrupt Enable Set Register DMA Channel 2 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA2INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 2 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA2INTES_MASK		(((1 << USB_REG_UHINTESET_DMA2INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA2INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 3 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA3INTES_OFFSET		27
/// USB Host General Interrupt Enable Set Register DMA Channel 3 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA3INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 3 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA3INTES_MASK		(((1 << USB_REG_UHINTESET_DMA3INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA3INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 4 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA4INTES_OFFSET		28
/// USB Host General Interrupt Enable Set Register DMA Channel 4 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA4INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 4 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA4INTES_MASK		(((1 << USB_REG_UHINTESET_DMA4INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA4INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 5 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA5INTES_OFFSET		29
/// USB Host General Interrupt Enable Set Register DMA Channel 5 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA5INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 5 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA5INTES_MASK		(((1 << USB_REG_UHINTESET_DMA5INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA5INTES_OFFSET)
/// USB Host General Interrupt Enable Set Register DMA Channel 6 Interrupt Enable Set offset
#define USB_REG_UHINTESET_DMA6INTES_OFFSET		30
/// USB Host General Interrupt Enable Set Register DMA Channel 6 Interrupt Enable Set size
#define USB_REG_UHINTESET_DMA6INTES_SIZE		1
/// USB Host General Interrupt Enable Set Register DMA Channel 6 Interrupt Enable Set mask
#define USB_REG_UHINTESET_DMA6INTES_MASK		(((1 << USB_REG_UHINTESET_DMA6INTES_SIZE) - 1) << USB_REG_UHINTESET_DMA6INTES_OFFSET)
/// USB Pipe Enable/Reset Register (Read/Write)
#define USB_REG_UPRST					0x0000041c
/// USB Pipe Enable/Reset Register Pipe 0 Enable offset
#define USB_REG_UPRST_PEN0_OFFSET			0
/// USB Pipe Enable/Reset Register Pipe 0 Enable size
#define USB_REG_UPRST_PEN0_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 0 Enable mask
#define USB_REG_UPRST_PEN0_MASK				(((1 << USB_REG_UPRST_PEN0_SIZE) - 1) << USB_REG_UPRST_PEN0_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 1 Enable offset
#define USB_REG_UPRST_PEN1_OFFSET			1
/// USB Pipe Enable/Reset Register Pipe 1 Enable size
#define USB_REG_UPRST_PEN1_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 1 Enable mask
#define USB_REG_UPRST_PEN1_MASK				(((1 << USB_REG_UPRST_PEN1_SIZE) - 1) << USB_REG_UPRST_PEN1_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 2 Enable offset
#define USB_REG_UPRST_PEN2_OFFSET			2
/// USB Pipe Enable/Reset Register Pipe 2 Enable size
#define USB_REG_UPRST_PEN2_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 2 Enable mask
#define USB_REG_UPRST_PEN2_MASK				(((1 << USB_REG_UPRST_PEN2_SIZE) - 1) << USB_REG_UPRST_PEN2_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 3 Enable offset
#define USB_REG_UPRST_PEN3_OFFSET			3
/// USB Pipe Enable/Reset Register Pipe 3 Enable size
#define USB_REG_UPRST_PEN3_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 3 Enable mask
#define USB_REG_UPRST_PEN3_MASK				(((1 << USB_REG_UPRST_PEN3_SIZE) - 1) << USB_REG_UPRST_PEN3_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 4 Enable offset
#define USB_REG_UPRST_PEN4_OFFSET			4
/// USB Pipe Enable/Reset Register Pipe 4 Enable size
#define USB_REG_UPRST_PEN4_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 4 Enable mask
#define USB_REG_UPRST_PEN4_MASK				(((1 << USB_REG_UPRST_PEN4_SIZE) - 1) << USB_REG_UPRST_PEN4_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 5 Enable offset
#define USB_REG_UPRST_PEN5_OFFSET			5
/// USB Pipe Enable/Reset Register Pipe 5 Enable size
#define USB_REG_UPRST_PEN5_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 5 Enable mask
#define USB_REG_UPRST_PEN5_MASK				(((1 << USB_REG_UPRST_PEN5_SIZE) - 1) << USB_REG_UPRST_PEN5_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 6 Enable offset
#define USB_REG_UPRST_PEN6_OFFSET			6
/// USB Pipe Enable/Reset Register Pipe 6 Enable size
#define USB_REG_UPRST_PEN6_SIZE				1
/// USB Pipe Enable/Reset Register Pipe 6 Enable mask
#define USB_REG_UPRST_PEN6_MASK				(((1 << USB_REG_UPRST_PEN6_SIZE) - 1) << USB_REG_UPRST_PEN6_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 0 Reset offset
#define USB_REG_UPRST_PRST0_OFFSET			16
/// USB Pipe Enable/Reset Register Pipe 0 Reset size
#define USB_REG_UPRST_PRST0_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 0 Reset mask
#define USB_REG_UPRST_PRST0_MASK			(((1 << USB_REG_UPRST_PRST0_SIZE) - 1) << USB_REG_UPRST_PRST0_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 1 Reset offset
#define USB_REG_UPRST_PRST1_OFFSET			17
/// USB Pipe Enable/Reset Register Pipe 1 Reset size
#define USB_REG_UPRST_PRST1_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 1 Reset mask
#define USB_REG_UPRST_PRST1_MASK			(((1 << USB_REG_UPRST_PRST1_SIZE) - 1) << USB_REG_UPRST_PRST1_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 2 Reset offset
#define USB_REG_UPRST_PRST2_OFFSET			18
/// USB Pipe Enable/Reset Register Pipe 2 Reset size
#define USB_REG_UPRST_PRST2_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 2 Reset mask
#define USB_REG_UPRST_PRST2_MASK			(((1 << USB_REG_UPRST_PRST2_SIZE) - 1) << USB_REG_UPRST_PRST2_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 3 Reset offset
#define USB_REG_UPRST_PRST3_OFFSET			19
/// USB Pipe Enable/Reset Register Pipe 3 Reset size
#define USB_REG_UPRST_PRST3_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 3 Reset mask
#define USB_REG_UPRST_PRST3_MASK			(((1 << USB_REG_UPRST_PRST3_SIZE) - 1) << USB_REG_UPRST_PRST3_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 4 Reset offset
#define USB_REG_UPRST_PRST4_OFFSET			20
/// USB Pipe Enable/Reset Register Pipe 4 Reset size
#define USB_REG_UPRST_PRST4_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 4 Reset mask
#define USB_REG_UPRST_PRST4_MASK			(((1 << USB_REG_UPRST_PRST4_SIZE) - 1) << USB_REG_UPRST_PRST4_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 5 Reset offset
#define USB_REG_UPRST_PRST5_OFFSET			21
/// USB Pipe Enable/Reset Register Pipe 5 Reset size
#define USB_REG_UPRST_PRST5_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 5 Reset mask
#define USB_REG_UPRST_PRST5_MASK			(((1 << USB_REG_UPRST_PRST5_SIZE) - 1) << USB_REG_UPRST_PRST5_OFFSET)
/// USB Pipe Enable/Reset Register Pipe 6 Reset offset
#define USB_REG_UPRST_PRST6_OFFSET			22
/// USB Pipe Enable/Reset Register Pipe 6 Reset size
#define USB_REG_UPRST_PRST6_SIZE			1
/// USB Pipe Enable/Reset Register Pipe 6 Reset mask
#define USB_REG_UPRST_PRST6_MASK			(((1 << USB_REG_UPRST_PRST6_SIZE) - 1) << USB_REG_UPRST_PRST6_OFFSET)
/// USB Host Frame Number Register (Read/Write)
#define USB_REG_UHFNUM					0x00000420
/// USB Host Frame Number Register Frame Number offset
#define USB_REG_UHFNUM_FNUM_OFFSET			3
/// USB Host Frame Number Register Frame Number size
#define USB_REG_UHFNUM_FNUM_SIZE			11
/// USB Host Frame Number Register Frame Number mask
#define USB_REG_UHFNUM_FNUM_MASK			(((1 << USB_REG_UHFNUM_FNUM_SIZE) - 1) << USB_REG_UHFNUM_FNUM_OFFSET)
/// USB Host Frame Number Register Frame Length offset
#define USB_REG_UHFNUM_FLENHIGH_OFFSET			16
/// USB Host Frame Number Register Frame Length size
#define USB_REG_UHFNUM_FLENHIGH_SIZE			8
/// USB Host Frame Number Register Frame Length mask
#define USB_REG_UHFNUM_FLENHIGH_MASK			(((1 << USB_REG_UHFNUM_FLENHIGH_SIZE) - 1) << USB_REG_UHFNUM_FLENHIGH_OFFSET)
/// USB Host Address 1 Register (Read/Write)
#define USB_REG_UHADDR1					0x00000424
/// USB Host Address 1 Register USB Host Address Pipe 0 offset
#define USB_REG_UHADDR1_UHADDR_P0_OFFSET		0
/// USB Host Address 1 Register USB Host Address Pipe 0 size
#define USB_REG_UHADDR1_UHADDR_P0_SIZE			7
/// USB Host Address 1 Register USB Host Address Pipe 0 mask
#define USB_REG_UHADDR1_UHADDR_P0_MASK			(((1 << USB_REG_UHADDR1_UHADDR_P0_SIZE) - 1) << USB_REG_UHADDR1_UHADDR_P0_OFFSET)
/// USB Host Address 1 Register USB Host Address Pipe 1 offset
#define USB_REG_UHADDR1_UHADDR_P1_OFFSET		8
/// USB Host Address 1 Register USB Host Address Pipe 1 size
#define USB_REG_UHADDR1_UHADDR_P1_SIZE			7
/// USB Host Address 1 Register USB Host Address Pipe 1 mask
#define USB_REG_UHADDR1_UHADDR_P1_MASK			(((1 << USB_REG_UHADDR1_UHADDR_P1_SIZE) - 1) << USB_REG_UHADDR1_UHADDR_P1_OFFSET)
/// USB Host Address 1 Register USB Host Address Pipe 2 offset
#define USB_REG_UHADDR1_UHADDR_P2_OFFSET		16
/// USB Host Address 1 Register USB Host Address Pipe 2 size
#define USB_REG_UHADDR1_UHADDR_P2_SIZE			7
/// USB Host Address 1 Register USB Host Address Pipe 2 mask
#define USB_REG_UHADDR1_UHADDR_P2_MASK			(((1 << USB_REG_UHADDR1_UHADDR_P2_SIZE) - 1) << USB_REG_UHADDR1_UHADDR_P2_OFFSET)
/// USB Host Address 1 Register USB Host Address Pipe 3 offset
#define USB_REG_UHADDR1_UHADDR_P3_OFFSET		24
/// USB Host Address 1 Register USB Host Address Pipe 3 size
#define USB_REG_UHADDR1_UHADDR_P3_SIZE			7
/// USB Host Address 1 Register USB Host Address Pipe 3 mask
#define USB_REG_UHADDR1_UHADDR_P3_MASK			(((1 << USB_REG_UHADDR1_UHADDR_P3_SIZE) - 1) << USB_REG_UHADDR1_UHADDR_P3_OFFSET)
/// USB Host Address 2 Register (Read/Write)
#define USB_REG_UHADDR2					0x00000428
/// USB Host Address 2 Register USB Host Address Pipe 4 offset
#define USB_REG_UHADDR2_UHADDR_P4_OFFSET		0
/// USB Host Address 2 Register USB Host Address Pipe 4 size
#define USB_REG_UHADDR2_UHADDR_P4_SIZE			7
/// USB Host Address 2 Register USB Host Address Pipe 4 mask
#define USB_REG_UHADDR2_UHADDR_P4_MASK			(((1 << USB_REG_UHADDR2_UHADDR_P4_SIZE) - 1) << USB_REG_UHADDR2_UHADDR_P4_OFFSET)
/// USB Host Address 2 Register USB Host Address Pipe 5 offset
#define USB_REG_UHADDR2_UHADDR_P5_OFFSET		8
/// USB Host Address 2 Register USB Host Address Pipe 5 size
#define USB_REG_UHADDR2_UHADDR_P5_SIZE			7
/// USB Host Address 2 Register USB Host Address Pipe 5 mask
#define USB_REG_UHADDR2_UHADDR_P5_MASK			(((1 << USB_REG_UHADDR2_UHADDR_P5_SIZE) - 1) << USB_REG_UHADDR2_UHADDR_P5_OFFSET)
/// USB Host Address 2 Register USB Host Address Pipe 6 offset
#define USB_REG_UHADDR2_UHADDR_P6_OFFSET		16
/// USB Host Address 2 Register USB Host Address Pipe 6 size
#define USB_REG_UHADDR2_UHADDR_P6_SIZE			7
/// USB Host Address 2 Register USB Host Address Pipe 6 mask
#define USB_REG_UHADDR2_UHADDR_P6_MASK			(((1 << USB_REG_UHADDR2_UHADDR_P6_SIZE) - 1) << USB_REG_UHADDR2_UHADDR_P6_OFFSET)
/// USB Pipe 0 Configuration Register (Read/Write)
#define USB_REG_UPCFG0					0x00000500
/// USB Pipe 1 Configuration Register (Read/Write)
#define USB_REG_UPCFG1					0x00000504
/// USB Pipe 2 Configuration Register (Read/Write)
#define USB_REG_UPCFG2					0x00000508
/// USB Pipe 3 Configuration Register (Read/Write)
#define USB_REG_UPCFG3					0x0000050c
/// USB Pipe 4 Configuration Register (Read/Write)
#define USB_REG_UPCFG4					0x00000510
/// USB Pipe 5 Configuration Register (Read/Write)
#define USB_REG_UPCFG5					0x00000514
/// USB Pipe 6 Configuration Register (Read/Write)
#define USB_REG_UPCFG6					0x00000518
/// USB Pipe 6 Configuration Register Pipe Memory Allocate offset
#define USB_REG_UPCFG6_ALLOC_OFFSET			1
/// USB Pipe 6 Configuration Register Pipe Memory Allocate size
#define USB_REG_UPCFG6_ALLOC_SIZE			1
/// USB Pipe 6 Configuration Register Pipe Memory Allocate mask
#define USB_REG_UPCFG6_ALLOC_MASK			(((1 << USB_REG_UPCFG6_ALLOC_SIZE) - 1) << USB_REG_UPCFG6_ALLOC_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Banks offset
#define USB_REG_UPCFG6_PBK_OFFSET			2
/// USB Pipe 6 Configuration Register Pipe Banks size
#define USB_REG_UPCFG6_PBK_SIZE				2
/// USB Pipe 6 Configuration Register Pipe Banks mask
#define USB_REG_UPCFG6_PBK_MASK				(((1 << USB_REG_UPCFG6_PBK_SIZE) - 1) << USB_REG_UPCFG6_PBK_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Size offset
#define USB_REG_UPCFG6_PSIZE_OFFSET			4
/// USB Pipe 6 Configuration Register Pipe Size size
#define USB_REG_UPCFG6_PSIZE_SIZE			3
/// USB Pipe 6 Configuration Register Pipe Size mask
#define USB_REG_UPCFG6_PSIZE_MASK			(((1 << USB_REG_UPCFG6_PSIZE_SIZE) - 1) << USB_REG_UPCFG6_PSIZE_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Token offset
#define USB_REG_UPCFG6_PTOKEN_OFFSET			8
/// USB Pipe 6 Configuration Register Pipe Token size
#define USB_REG_UPCFG6_PTOKEN_SIZE			2
/// USB Pipe 6 Configuration Register Pipe Token mask
#define USB_REG_UPCFG6_PTOKEN_MASK			(((1 << USB_REG_UPCFG6_PTOKEN_SIZE) - 1) << USB_REG_UPCFG6_PTOKEN_OFFSET)
/// USB Pipe 6 Configuration Register Automatic Switch offset
#define USB_REG_UPCFG6_AUTOSW_OFFSET			10
/// USB Pipe 6 Configuration Register Automatic Switch size
#define USB_REG_UPCFG6_AUTOSW_SIZE			1
/// USB Pipe 6 Configuration Register Automatic Switch mask
#define USB_REG_UPCFG6_AUTOSW_MASK			(((1 << USB_REG_UPCFG6_AUTOSW_SIZE) - 1) << USB_REG_UPCFG6_AUTOSW_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Type offset
#define USB_REG_UPCFG6_PTYPE_OFFSET			12
/// USB Pipe 6 Configuration Register Pipe Type size
#define USB_REG_UPCFG6_PTYPE_SIZE			2
/// USB Pipe 6 Configuration Register Pipe Type mask
#define USB_REG_UPCFG6_PTYPE_MASK			(((1 << USB_REG_UPCFG6_PTYPE_SIZE) - 1) << USB_REG_UPCFG6_PTYPE_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Endpoint Number offset
#define USB_REG_UPCFG6_PEPNUM_OFFSET			16
/// USB Pipe 6 Configuration Register Pipe Endpoint Number size
#define USB_REG_UPCFG6_PEPNUM_SIZE			4
/// USB Pipe 6 Configuration Register Pipe Endpoint Number mask
#define USB_REG_UPCFG6_PEPNUM_MASK			(((1 << USB_REG_UPCFG6_PEPNUM_SIZE) - 1) << USB_REG_UPCFG6_PEPNUM_OFFSET)
/// USB Pipe 6 Configuration Register Pipe Interrupt Request Frequency offset
#define USB_REG_UPCFG6_INTFRQ_OFFSET			24
/// USB Pipe 6 Configuration Register Pipe Interrupt Request Frequency size
#define USB_REG_UPCFG6_INTFRQ_SIZE			8
/// USB Pipe 6 Configuration Register Pipe Interrupt Request Frequency mask
#define USB_REG_UPCFG6_INTFRQ_MASK			(((1 << USB_REG_UPCFG6_INTFRQ_SIZE) - 1) << USB_REG_UPCFG6_INTFRQ_OFFSET)
/// USB Pipe 0 Status Register (Read)
#define USB_REG_UPSTA0					0x00000530
/// USB Pipe 1 Status Register (Read)
#define USB_REG_UPSTA1					0x00000534
/// USB Pipe 2 Status Register (Read)
#define USB_REG_UPSTA2					0x00000538
/// USB Pipe 3 Status Register (Read)
#define USB_REG_UPSTA3					0x0000053c
/// USB Pipe 4 Status Register (Read)
#define USB_REG_UPSTA4					0x00000540
/// USB Pipe 5 Status Register (Read)
#define USB_REG_UPSTA5					0x00000544
/// USB Pipe 6 Status Register (Read)
#define USB_REG_UPSTA6					0x00000548
/// USB Pipe 0 Status Register Received IN Data Interrupt Flag offset
#define USB_REG_UPSTAX_RXINI_OFFSET			0
/// USB Pipe 0 Status Register Received IN Data Interrupt Flag size
#define USB_REG_UPSTAX_RXINI_SIZE			1
/// USB Pipe 0 Status Register Received IN Data Interrupt Flag mask
#define USB_REG_UPSTAX_RXINI_MASK			(((1 << USB_REG_UPSTAX_RXINI_SIZE) - 1) << USB_REG_UPSTAX_RXINI_OFFSET)
/// USB Pipe 0 Status Register Transmitted OUT Data Interrupt Flag offset
#define USB_REG_UPSTAX_TXOUTI_OFFSET			1
/// USB Pipe 0 Status Register Transmitted OUT Data Interrupt Flag size
#define USB_REG_UPSTAX_TXOUTI_SIZE			1
/// USB Pipe 0 Status Register Transmitted OUT Data Interrupt Flag mask
#define USB_REG_UPSTAX_TXOUTI_MASK			(((1 << USB_REG_UPSTAX_TXOUTI_SIZE) - 1) << USB_REG_UPSTAX_TXOUTI_OFFSET)
/// USB Pipe 0 Status Register Transmitted SETUP Interrupt Flag offset
#define USB_REG_UPSTAX_TXSTPI_OFFSET			2
/// USB Pipe 0 Status Register Transmitted SETUP Interrupt Flag size
#define USB_REG_UPSTAX_TXSTPI_SIZE			1
/// USB Pipe 0 Status Register Transmitted SETUP Interrupt Flag mask
#define USB_REG_UPSTAX_TXSTPI_MASK			(((1 << USB_REG_UPSTAX_TXSTPI_SIZE) - 1) << USB_REG_UPSTAX_TXSTPI_OFFSET)
/// USB Pipe 0 Status Register Underflow Interrupt Flag offset
#define USB_REG_UPSTAX_UNDERFI_OFFSET			2
/// USB Pipe 0 Status Register Underflow Interrupt Flag size
#define USB_REG_UPSTAX_UNDERFI_SIZE			1
/// USB Pipe 0 Status Register Underflow Interrupt Flag mask
#define USB_REG_UPSTAX_UNDERFI_MASK			(((1 << USB_REG_UPSTAX_UNDERFI_SIZE) - 1) << USB_REG_UPSTAX_UNDERFI_OFFSET)
/// USB Pipe 0 Status Register Pipe Error Interrupt Flag offset
#define USB_REG_UPSTAX_PERRI_OFFSET			3
/// USB Pipe 0 Status Register Pipe Error Interrupt Flag size
#define USB_REG_UPSTAX_PERRI_SIZE			1
/// USB Pipe 0 Status Register Pipe Error Interrupt Flag mask
#define USB_REG_UPSTAX_PERRI_MASK			(((1 << USB_REG_UPSTAX_PERRI_SIZE) - 1) << USB_REG_UPSTAX_PERRI_OFFSET)
/// USB Pipe 0 Status Register NAKed Interrupt Flag offset
#define USB_REG_UPSTAX_NAKEDI_OFFSET			4
/// USB Pipe 0 Status Register NAKed Interrupt Flag size
#define USB_REG_UPSTAX_NAKEDI_SIZE			1
/// USB Pipe 0 Status Register NAKed Interrupt Flag mask
#define USB_REG_UPSTAX_NAKEDI_MASK			(((1 << USB_REG_UPSTAX_NAKEDI_SIZE) - 1) << USB_REG_UPSTAX_NAKEDI_OFFSET)
/// USB Pipe 0 Status Register Overflow Interrupt Flag offset
#define USB_REG_UPSTAX_OVERFI_OFFSET			5
/// USB Pipe 0 Status Register Overflow Interrupt Flag size
#define USB_REG_UPSTAX_OVERFI_SIZE			1
/// USB Pipe 0 Status Register Overflow Interrupt Flag mask
#define USB_REG_UPSTAX_OVERFI_MASK			(((1 << USB_REG_UPSTAX_OVERFI_SIZE) - 1) << USB_REG_UPSTAX_OVERFI_OFFSET)
/// USB Pipe 0 Status Register Received STALLed Interrupt Flag offset
#define USB_REG_UPSTAX_RXSTALLDI_OFFSET			6
/// USB Pipe 0 Status Register Received STALLed Interrupt Flag size
#define USB_REG_UPSTAX_RXSTALLDI_SIZE			1
/// USB Pipe 0 Status Register Received STALLed Interrupt Flag mask
#define USB_REG_UPSTAX_RXSTALLDI_MASK			(((1 << USB_REG_UPSTAX_RXSTALLDI_SIZE) - 1) << USB_REG_UPSTAX_RXSTALLDI_OFFSET)
/// USB Pipe 0 Status Register CRC Error Interrupt Flag offset
#define USB_REG_UPSTAX_CRCERRI_OFFSET			6
/// USB Pipe 0 Status Register CRC Error Interrupt Flag size
#define USB_REG_UPSTAX_CRCERRI_SIZE			1
/// USB Pipe 0 Status Register CRC Error Interrupt Flag mask
#define USB_REG_UPSTAX_CRCERRI_MASK			(((1 << USB_REG_UPSTAX_CRCERRI_SIZE) - 1) << USB_REG_UPSTAX_CRCERRI_OFFSET)
/// USB Pipe 0 Status Register Short Packet Interrupt Flag offset
#define USB_REG_UPSTAX_SHORTPACKETI_OFFSET		7
/// USB Pipe 0 Status Register Short Packet Interrupt Flag size
#define USB_REG_UPSTAX_SHORTPACKETI_SIZE		1
/// USB Pipe 0 Status Register Short Packet Interrupt Flag mask
#define USB_REG_UPSTAX_SHORTPACKETI_MASK		(((1 << USB_REG_UPSTAX_SHORTPACKETI_SIZE) - 1) << USB_REG_UPSTAX_SHORTPACKETI_OFFSET)
/// USB Pipe 0 Status Register Data Toggle Sequence offset
#define USB_REG_UPSTAX_DTSEQ_OFFSET			8
/// USB Pipe 0 Status Register Data Toggle Sequence size
#define USB_REG_UPSTAX_DTSEQ_SIZE			2
/// USB Pipe 0 Status Register Data Toggle Sequence mask
#define USB_REG_UPSTAX_DTSEQ_MASK			(((1 << USB_REG_UPSTAX_DTSEQ_SIZE) - 1) << USB_REG_UPSTAX_DTSEQ_OFFSET)
/// USB Pipe 0 Status Register Number of Busy Banks offset
#define USB_REG_UPSTAX_NBUSYBK_OFFSET			12
/// USB Pipe 0 Status Register Number of Busy Banks size
#define USB_REG_UPSTAX_NBUSYBK_SIZE			2
/// USB Pipe 0 Status Register Number of Busy Banks mask
#define USB_REG_UPSTAX_NBUSYBK_MASK			(((1 << USB_REG_UPSTAX_NBUSYBK_SIZE) - 1) << USB_REG_UPSTAX_NBUSYBK_OFFSET)
/// USB Pipe 0 Status Register Current Bank offset
#define USB_REG_UPSTAX_CURRBK_OFFSET			14
/// USB Pipe 0 Status Register Current Bank size
#define USB_REG_UPSTAX_CURRBK_SIZE			2
/// USB Pipe 0 Status Register Current Bank mask
#define USB_REG_UPSTAX_CURRBK_MASK			(((1 << USB_REG_UPSTAX_CURRBK_SIZE) - 1) << USB_REG_UPSTAX_CURRBK_OFFSET)
/// USB Pipe 0 Status Register Read/Write Allowed offset
#define USB_REG_UPSTAX_RWALL_OFFSET			16
/// USB Pipe 0 Status Register Read/Write Allowed size
#define USB_REG_UPSTAX_RWALL_SIZE			1
/// USB Pipe 0 Status Register Read/Write Allowed mask
#define USB_REG_UPSTAX_RWALL_MASK			(((1 << USB_REG_UPSTAX_RWALL_SIZE) - 1) << USB_REG_UPSTAX_RWALL_OFFSET)
/// USB Pipe 0 Status Register Configuration OK Status offset
#define USB_REG_UPSTAX_CFGOK_OFFSET			18
/// USB Pipe 0 Status Register Configuration OK Status size
#define USB_REG_UPSTAX_CFGOK_SIZE			1
/// USB Pipe 0 Status Register Configuration OK Status mask
#define USB_REG_UPSTAX_CFGOK_MASK			(((1 << USB_REG_UPSTAX_CFGOK_SIZE) - 1) << USB_REG_UPSTAX_CFGOK_OFFSET)
/// USB Pipe 0 Status Register Pipe Byte Count offset
#define USB_REG_UPSTAX_PBYCT_OFFSET			20
/// USB Pipe 0 Status Register Pipe Byte Count size
#define USB_REG_UPSTAX_PBYCT_SIZE			11
/// USB Pipe 0 Status Register Pipe Byte Count mask
#define USB_REG_UPSTAX_PBYCT_MASK			(((1 << USB_REG_UPSTAX_PBYCT_SIZE) - 1) << USB_REG_UPSTAX_PBYCT_OFFSET)
/// USB Pipe 0 Status Clear Register (Write)
#define USB_REG_UPSTA0CLR				0x00000560
/// USB Pipe 1 Status Clear Register (Write)
#define USB_REG_UPSTA1CLR				0x00000564
/// USB Pipe 2 Status Clear Register (Write)
#define USB_REG_UPSTA2CLR				0x00000568
/// USB Pipe 3 Status Clear Register (Write)
#define USB_REG_UPSTA3CLR				0x0000056c
/// USB Pipe 4 Status Clear Register (Write)
#define USB_REG_UPSTA4CLR				0x00000570
/// USB Pipe 5 Status Clear Register (Write)
#define USB_REG_UPSTA5CLR				0x00000574
/// USB Pipe 6 Status Clear Register (Write)
#define USB_REG_UPSTA6CLR				0x00000578
/// USB Pipe 0 Status Clear Register Received IN Data Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_RXINIC_OFFSET			0
/// USB Pipe 0 Status Clear Register Received IN Data Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_RXINIC_SIZE			1
/// USB Pipe 0 Status Clear Register Received IN Data Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_RXINIC_MASK			(((1 << USB_REG_UPSTAXCLR_RXINIC_SIZE) - 1) << USB_REG_UPSTAXCLR_RXINIC_OFFSET)
/// USB Pipe 0 Status Clear Register Transmitted OUT Data Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_TXOUTIC_OFFSET		1
/// USB Pipe 0 Status Clear Register Transmitted OUT Data Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_TXOUTIC_SIZE			1
/// USB Pipe 0 Status Clear Register Transmitted OUT Data Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_TXOUTIC_MASK			(((1 << USB_REG_UPSTAXCLR_TXOUTIC_SIZE) - 1) << USB_REG_UPSTAXCLR_TXOUTIC_OFFSET)
/// USB Pipe 0 Status Clear Register Transmitted SETUP Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_TXSTPIC_OFFSET		2
/// USB Pipe 0 Status Clear Register Transmitted SETUP Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_TXSTPIC_SIZE			1
/// USB Pipe 0 Status Clear Register Transmitted SETUP Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_TXSTPIC_MASK			(((1 << USB_REG_UPSTAXCLR_TXSTPIC_SIZE) - 1) << USB_REG_UPSTAXCLR_TXSTPIC_OFFSET)
/// USB Pipe 0 Status Clear Register Underflow Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_UNDERFIC_OFFSET		2
/// USB Pipe 0 Status Clear Register Underflow Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_UNDERFIC_SIZE			1
/// USB Pipe 0 Status Clear Register Underflow Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_UNDERFIC_MASK			(((1 << USB_REG_UPSTAXCLR_UNDERFIC_SIZE) - 1) << USB_REG_UPSTAXCLR_UNDERFIC_OFFSET)
/// USB Pipe 0 Status Clear Register NAKed Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_NAKEDIC_OFFSET		4
/// USB Pipe 0 Status Clear Register NAKed Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_NAKEDIC_SIZE			1
/// USB Pipe 0 Status Clear Register NAKed Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_NAKEDIC_MASK			(((1 << USB_REG_UPSTAXCLR_NAKEDIC_SIZE) - 1) << USB_REG_UPSTAXCLR_NAKEDIC_OFFSET)
/// USB Pipe 0 Status Clear Register Overflow Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_OVERFIC_OFFSET		5
/// USB Pipe 0 Status Clear Register Overflow Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_OVERFIC_SIZE			1
/// USB Pipe 0 Status Clear Register Overflow Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_OVERFIC_MASK			(((1 << USB_REG_UPSTAXCLR_OVERFIC_SIZE) - 1) << USB_REG_UPSTAXCLR_OVERFIC_OFFSET)
/// USB Pipe 0 Status Clear Register Received STALLed Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_RXSTALLDIC_OFFSET		6
/// USB Pipe 0 Status Clear Register Received STALLed Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_RXSTALLDIC_SIZE		1
/// USB Pipe 0 Status Clear Register Received STALLed Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_RXSTALLDIC_MASK		(((1 << USB_REG_UPSTAXCLR_RXSTALLDIC_SIZE) - 1) << USB_REG_UPSTAXCLR_RXSTALLDIC_OFFSET)
/// USB Pipe 0 Status Clear Register CRC Error Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_CRCERRIC_OFFSET		6
/// USB Pipe 0 Status Clear Register CRC Error Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_CRCERRIC_SIZE			1
/// USB Pipe 0 Status Clear Register CRC Error Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_CRCERRIC_MASK			(((1 << USB_REG_UPSTAXCLR_CRCERRIC_SIZE) - 1) << USB_REG_UPSTAXCLR_CRCERRIC_OFFSET)
/// USB Pipe 0 Status Clear Register Short Packet Interrupt Flag Clear offset
#define USB_REG_UPSTAXCLR_SHORTPACKETIC_OFFSET		7
/// USB Pipe 0 Status Clear Register Short Packet Interrupt Flag Clear size
#define USB_REG_UPSTAXCLR_SHORTPACKETIC_SIZE		1
/// USB Pipe 0 Status Clear Register Short Packet Interrupt Flag Clear mask
#define USB_REG_UPSTAXCLR_SHORTPACKETIC_MASK		(((1 << USB_REG_UPSTAXCLR_SHORTPACKETIC_SIZE) - 1) << USB_REG_UPSTAXCLR_SHORTPACKETIC_OFFSET)
/// USB Pipe 0 Status Set Register (Write)
#define USB_REG_UPSTA0SET				0x00000590
/// USB Pipe 1 Status Set Register (Write)
#define USB_REG_UPSTA1SET				0x00000594
/// USB Pipe 2 Status Set Register (Write)
#define USB_REG_UPSTA2SET				0x00000598
/// USB Pipe 3 Status Set Register (Write)
#define USB_REG_UPSTA3SET				0x0000059c
/// USB Pipe 4 Status Set Register (Write)
#define USB_REG_UPSTA4SET				0x000005a0
/// USB Pipe 5 Status Set Register (Write)
#define USB_REG_UPSTA5SET				0x000005a4
/// USB Pipe 6 Status Set Register (Write)
#define USB_REG_UPSTA6SET				0x000005a8
/// USB Pipe 0 Status Set Register Received IN Data Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_RXINIS_OFFSET			0
/// USB Pipe 0 Status Set Register Received IN Data Interrupt Flag Set size
#define USB_REG_UPSTAXSET_RXINIS_SIZE			1
/// USB Pipe 0 Status Set Register Received IN Data Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_RXINIS_MASK			(((1 << USB_REG_UPSTAXSET_RXINIS_SIZE) - 1) << USB_REG_UPSTAXSET_RXINIS_OFFSET)
/// USB Pipe 0 Status Set Register Transmitted OUT Data Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_TXOUTIS_OFFSET		1
/// USB Pipe 0 Status Set Register Transmitted OUT Data Interrupt Flag Set size
#define USB_REG_UPSTAXSET_TXOUTIS_SIZE			1
/// USB Pipe 0 Status Set Register Transmitted OUT Data Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_TXOUTIS_MASK			(((1 << USB_REG_UPSTAXSET_TXOUTIS_SIZE) - 1) << USB_REG_UPSTAXSET_TXOUTIS_OFFSET)
/// USB Pipe 0 Status Set Register Transmitted SETUP Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_TXSTPIS_OFFSET		2
/// USB Pipe 0 Status Set Register Transmitted SETUP Interrupt Flag Set size
#define USB_REG_UPSTAXSET_TXSTPIS_SIZE			1
/// USB Pipe 0 Status Set Register Transmitted SETUP Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_TXSTPIS_MASK			(((1 << USB_REG_UPSTAXSET_TXSTPIS_SIZE) - 1) << USB_REG_UPSTAXSET_TXSTPIS_OFFSET)
/// USB Pipe 0 Status Set Register Underflow Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_UNDERFIS_OFFSET		2
/// USB Pipe 0 Status Set Register Underflow Interrupt Flag Set size
#define USB_REG_UPSTAXSET_UNDERFIS_SIZE			1
/// USB Pipe 0 Status Set Register Underflow Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_UNDERFIS_MASK			(((1 << USB_REG_UPSTAXSET_UNDERFIS_SIZE) - 1) << USB_REG_UPSTAXSET_UNDERFIS_OFFSET)
/// USB Pipe 0 Status Set Register Pipe Error Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_PERRIS_OFFSET			3
/// USB Pipe 0 Status Set Register Pipe Error Interrupt Flag Set size
#define USB_REG_UPSTAXSET_PERRIS_SIZE			1
/// USB Pipe 0 Status Set Register Pipe Error Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_PERRIS_MASK			(((1 << USB_REG_UPSTAXSET_PERRIS_SIZE) - 1) << USB_REG_UPSTAXSET_PERRIS_OFFSET)
/// USB Pipe 0 Status Set Register NAKed Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_NAKEDIS_OFFSET		4
/// USB Pipe 0 Status Set Register NAKed Interrupt Flag Set size
#define USB_REG_UPSTAXSET_NAKEDIS_SIZE			1
/// USB Pipe 0 Status Set Register NAKed Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_NAKEDIS_MASK			(((1 << USB_REG_UPSTAXSET_NAKEDIS_SIZE) - 1) << USB_REG_UPSTAXSET_NAKEDIS_OFFSET)
/// USB Pipe 0 Status Set Register Overflow Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_OVERFIS_OFFSET		5
/// USB Pipe 0 Status Set Register Overflow Interrupt Flag Set size
#define USB_REG_UPSTAXSET_OVERFIS_SIZE			1
/// USB Pipe 0 Status Set Register Overflow Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_OVERFIS_MASK			(((1 << USB_REG_UPSTAXSET_OVERFIS_SIZE) - 1) << USB_REG_UPSTAXSET_OVERFIS_OFFSET)
/// USB Pipe 0 Status Set Register Received STALLed Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_RXSTALLDIS_OFFSET		6
/// USB Pipe 0 Status Set Register Received STALLed Interrupt Flag Set size
#define USB_REG_UPSTAXSET_RXSTALLDIS_SIZE		1
/// USB Pipe 0 Status Set Register Received STALLed Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_RXSTALLDIS_MASK		(((1 << USB_REG_UPSTAXSET_RXSTALLDIS_SIZE) - 1) << USB_REG_UPSTAXSET_RXSTALLDIS_OFFSET)
/// USB Pipe 0 Status Set Register CRC Error Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_CRCERRIS_OFFSET		6
/// USB Pipe 0 Status Set Register CRC Error Interrupt Flag Set size
#define USB_REG_UPSTAXSET_CRCERRIS_SIZE			1
/// USB Pipe 0 Status Set Register CRC Error Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_CRCERRIS_MASK			(((1 << USB_REG_UPSTAXSET_CRCERRIS_SIZE) - 1) << USB_REG_UPSTAXSET_CRCERRIS_OFFSET)
/// USB Pipe 0 Status Set Register Short Packet Interrupt Flag Set offset
#define USB_REG_UPSTAXSET_SHORTPACKETIS_OFFSET		7
/// USB Pipe 0 Status Set Register Short Packet Interrupt Flag Set size
#define USB_REG_UPSTAXSET_SHORTPACKETIS_SIZE		1
/// USB Pipe 0 Status Set Register Short Packet Interrupt Flag Set mask
#define USB_REG_UPSTAXSET_SHORTPACKETIS_MASK		(((1 << USB_REG_UPSTAXSET_SHORTPACKETIS_SIZE) - 1) << USB_REG_UPSTAXSET_SHORTPACKETIS_OFFSET)
/// USB Pipe 0 Status Set Register Number of Busy Banks Set offset
#define USB_REG_UPSTAXSET_NBUSYBKS_OFFSET		12
/// USB Pipe 0 Status Set Register Number of Busy Banks Set size
#define USB_REG_UPSTAXSET_NBUSYBKS_SIZE			1
/// USB Pipe 0 Status Set Register Number of Busy Banks Set mask
#define USB_REG_UPSTAXSET_NBUSYBKS_MASK			(((1 << USB_REG_UPSTAXSET_NBUSYBKS_SIZE) - 1) << USB_REG_UPSTAXSET_NBUSYBKS_OFFSET)
/// USB Pipe 0 Control Register (Read)
#define USB_REG_UPCON0					0x000005c0
/// USB Pipe 1 Control Register (Read)
#define USB_REG_UPCON1					0x000005c4
/// USB Pipe 2 Control Register (Read)
#define USB_REG_UPCON2					0x000005c8
/// USB Pipe 3 Control Register (Read)
#define USB_REG_UPCON3					0x000005cc
/// USB Pipe 4 Control Register (Read)
#define USB_REG_UPCON4					0x000005d0
/// USB Pipe 5 Control Register (Read)
#define USB_REG_UPCON5					0x000005d4
/// USB Pipe 6 Control Register (Read)
#define USB_REG_UPCON6					0x000005d8
/// USB Pipe X Control Register Received IN Data Interrupt Enable offset
#define USB_REG_UPCONX_RXINE_OFFSET			0
/// USB Pipe X Control Register Received IN Data Interrupt Enable size
#define USB_REG_UPCONX_RXINE_SIZE			1
/// USB Pipe X Control Register Received IN Data Interrupt Enable mask
#define USB_REG_UPCONX_RXINE_MASK			(((1 << USB_REG_UPCONX_RXINE_SIZE) - 1) << USB_REG_UPCONX_RXINE_OFFSET)
/// USB Pipe X Control Register Transmitted OUT Data Interrupt Enable offset
#define USB_REG_UPCONX_TXOUTE_OFFSET			1
/// USB Pipe X Control Register Transmitted OUT Data Interrupt Enable size
#define USB_REG_UPCONX_TXOUTE_SIZE			1
/// USB Pipe X Control Register Transmitted OUT Data Interrupt Enable mask
#define USB_REG_UPCONX_TXOUTE_MASK			(((1 << USB_REG_UPCONX_TXOUTE_SIZE) - 1) << USB_REG_UPCONX_TXOUTE_OFFSET)
/// USB Pipe X Control Register Transmitted SETUP Interrupt Enable offset
#define USB_REG_UPCONX_TXSTPE_OFFSET			2
/// USB Pipe X Control Register Transmitted SETUP Interrupt Enable size
#define USB_REG_UPCONX_TXSTPE_SIZE			1
/// USB Pipe X Control Register Transmitted SETUP Interrupt Enable mask
#define USB_REG_UPCONX_TXSTPE_MASK			(((1 << USB_REG_UPCONX_TXSTPE_SIZE) - 1) << USB_REG_UPCONX_TXSTPE_OFFSET)
/// USB Pipe X Control Register Underflow Interrupt Enable offset
#define USB_REG_UPCONX_UNDERFIE_OFFSET			2
/// USB Pipe X Control Register Underflow Interrupt Enable size
#define USB_REG_UPCONX_UNDERFIE_SIZE			1
/// USB Pipe X Control Register Underflow Interrupt Enable mask
#define USB_REG_UPCONX_UNDERFIE_MASK			(((1 << USB_REG_UPCONX_UNDERFIE_SIZE) - 1) << USB_REG_UPCONX_UNDERFIE_OFFSET)
/// USB Pipe X Control Register Pipe Error Interrupt Enable offset
#define USB_REG_UPCONX_PERRE_OFFSET			3
/// USB Pipe X Control Register Pipe Error Interrupt Enable size
#define USB_REG_UPCONX_PERRE_SIZE			1
/// USB Pipe X Control Register Pipe Error Interrupt Enable mask
#define USB_REG_UPCONX_PERRE_MASK			(((1 << USB_REG_UPCONX_PERRE_SIZE) - 1) << USB_REG_UPCONX_PERRE_OFFSET)
/// USB Pipe X Control Register NAKed Interrupt Enable offset
#define USB_REG_UPCONX_NAKEDE_OFFSET			4
/// USB Pipe X Control Register NAKed Interrupt Enable size
#define USB_REG_UPCONX_NAKEDE_SIZE			1
/// USB Pipe X Control Register NAKed Interrupt Enable mask
#define USB_REG_UPCONX_NAKEDE_MASK			(((1 << USB_REG_UPCONX_NAKEDE_SIZE) - 1) << USB_REG_UPCONX_NAKEDE_OFFSET)
/// USB Pipe X Control Register Overflow Interrupt Enable offset
#define USB_REG_UPCONX_OVERFIE_OFFSET			5
/// USB Pipe X Control Register Overflow Interrupt Enable size
#define USB_REG_UPCONX_OVERFIE_SIZE			1
/// USB Pipe X Control Register Overflow Interrupt Enable mask
#define USB_REG_UPCONX_OVERFIE_MASK			(((1 << USB_REG_UPCONX_OVERFIE_SIZE) - 1) << USB_REG_UPCONX_OVERFIE_OFFSET)
/// USB Pipe X Control Register Received STALLed Interrupt Enable offset
#define USB_REG_UPCONX_RXSTALLDE_OFFSET			6
/// USB Pipe X Control Register Received STALLed Interrupt Enable size
#define USB_REG_UPCONX_RXSTALLDE_SIZE			1
/// USB Pipe X Control Register Received STALLed Interrupt Enable mask
#define USB_REG_UPCONX_RXSTALLDE_MASK			(((1 << USB_REG_UPCONX_RXSTALLDE_SIZE) - 1) << USB_REG_UPCONX_RXSTALLDE_OFFSET)
/// USB Pipe X Control Register CRC Error Interrupt Enable offset
#define USB_REG_UPCONX_CRCERRE_OFFSET			6
/// USB Pipe X Control Register CRC Error Interrupt Enable size
#define USB_REG_UPCONX_CRCERRE_SIZE			1
/// USB Pipe X Control Register CRC Error Interrupt Enable mask
#define USB_REG_UPCONX_CRCERRE_MASK			(((1 << USB_REG_UPCONX_CRCERRE_SIZE) - 1) << USB_REG_UPCONX_CRCERRE_OFFSET)
/// USB Pipe X Control Register Short Packet Interrupt Enable offset
#define USB_REG_UPCONX_SHORTPACKETIE_OFFSET		7
/// USB Pipe X Control Register Short Packet Interrupt Enable size
#define USB_REG_UPCONX_SHORTPACKETIE_SIZE		1
/// USB Pipe X Control Register Short Packet Interrupt Enable mask
#define USB_REG_UPCONX_SHORTPACKETIE_MASK		(((1 << USB_REG_UPCONX_SHORTPACKETIE_SIZE) - 1) << USB_REG_UPCONX_SHORTPACKETIE_OFFSET)
/// USB Pipe X Control Register Number of Busy Banks Interrupt Enable offset
#define USB_REG_UPCONX_NBUSYBKE_OFFSET			12
/// USB Pipe X Control Register Number of Busy Banks Interrupt Enable size
#define USB_REG_UPCONX_NBUSYBKE_SIZE			1
/// USB Pipe X Control Register Number of Busy Banks Interrupt Enable mask
#define USB_REG_UPCONX_NBUSYBKE_MASK			(((1 << USB_REG_UPCONX_NBUSYBKE_SIZE) - 1) << USB_REG_UPCONX_NBUSYBKE_OFFSET)
/// USB Pipe X Control Register FIFO Control offset
#define USB_REG_UPCONX_FIFOCON_OFFSET			14
/// USB Pipe X Control Register FIFO Control size
#define USB_REG_UPCONX_FIFOCON_SIZE			1
/// USB Pipe X Control Register FIFO Control mask
#define USB_REG_UPCONX_FIFOCON_MASK			(((1 << USB_REG_UPCONX_FIFOCON_SIZE) - 1) << USB_REG_UPCONX_FIFOCON_OFFSET)
/// USB Pipe X Control Register Pipe Interrupts Disable HDMA Request Enable offset
#define USB_REG_UPCONX_PDISHDMA_OFFSET			16
/// USB Pipe X Control Register Pipe Interrupts Disable HDMA Request Enable size
#define USB_REG_UPCONX_PDISHDMA_SIZE			1
/// USB Pipe X Control Register Pipe Interrupts Disable HDMA Request Enable mask
#define USB_REG_UPCONX_PDISHDMA_MASK			(((1 << USB_REG_UPCONX_PDISHDMA_SIZE) - 1) << USB_REG_UPCONX_PDISHDMA_OFFSET)
/// USB Pipe X Control Register Pipe Freeze offset
#define USB_REG_UPCONX_PFREEZE_OFFSET			17
/// USB Pipe X Control Register Pipe Freeze size
#define USB_REG_UPCONX_PFREEZE_SIZE			1
/// USB Pipe X Control Register Pipe Freeze mask
#define USB_REG_UPCONX_PFREEZE_MASK			(((1 << USB_REG_UPCONX_PFREEZE_SIZE) - 1) << USB_REG_UPCONX_PFREEZE_OFFSET)
/// USB Pipe X Control Register Reset Data Toggle offset
#define USB_REG_UPCONX_RSTDT_OFFSET			18
/// USB Pipe X Control Register Reset Data Toggle size
#define USB_REG_UPCONX_RSTDT_SIZE			1
/// USB Pipe X Control Register Reset Data Toggle mask
#define USB_REG_UPCONX_RSTDT_MASK			(((1 << USB_REG_UPCONX_RSTDT_SIZE) - 1) << USB_REG_UPCONX_RSTDT_OFFSET)
/// USB Pipe 0 Control Set Register (Write)
#define USB_REG_UPCON0SET				0x000005f0
/// USB Pipe 1 Control Set Register (Write)
#define USB_REG_UPCON1SET				0x000005f4
/// USB Pipe 2 Control Set Register (Write)
#define USB_REG_UPCON2SET				0x000005f8
/// USB Pipe 3 Control Set Register (Write)
#define USB_REG_UPCON3SET				0x000005fc
/// USB Pipe 4 Control Set Register (Write)
#define USB_REG_UPCON4SET				0x00000600
/// USB Pipe 5 Control Set Register (Write)
#define USB_REG_UPCON5SET				0x00000604
/// USB Pipe 6 Control Set Register (Write)
#define USB_REG_UPCON6SET				0x00000608
/// USB Pipe X Control Set Register Received IN Data Interrupt Enable Set offset
#define USB_REG_UPCONXSET_RXINES_OFFSET			0
/// USB Pipe X Control Set Register Received IN Data Interrupt Enable Set size
#define USB_REG_UPCONXSET_RXINES_SIZE			1
/// USB Pipe X Control Set Register Received IN Data Interrupt Enable Set mask
#define USB_REG_UPCONXSET_RXINES_MASK			(((1 << USB_REG_UPCONXSET_RXINES_SIZE) - 1) << USB_REG_UPCONXSET_RXINES_OFFSET)
/// USB Pipe X Control Set Register Transmitted OUT Data Interrupt Enable Set offset
#define USB_REG_UPCONXSET_TXOUTES_OFFSET		1
/// USB Pipe X Control Set Register Transmitted OUT Data Interrupt Enable Set size
#define USB_REG_UPCONXSET_TXOUTES_SIZE			1
/// USB Pipe X Control Set Register Transmitted OUT Data Interrupt Enable Set mask
#define USB_REG_UPCONXSET_TXOUTES_MASK			(((1 << USB_REG_UPCONXSET_TXOUTES_SIZE) - 1) << USB_REG_UPCONXSET_TXOUTES_OFFSET)
/// USB Pipe X Control Set Register Transmitted SETUP Interrupt Enable Set offset
#define USB_REG_UPCONXSET_TXSTPES_OFFSET		2
/// USB Pipe X Control Set Register Transmitted SETUP Interrupt Enable Set size
#define USB_REG_UPCONXSET_TXSTPES_SIZE			1
/// USB Pipe X Control Set Register Transmitted SETUP Interrupt Enable Set mask
#define USB_REG_UPCONXSET_TXSTPES_MASK			(((1 << USB_REG_UPCONXSET_TXSTPES_SIZE) - 1) << USB_REG_UPCONXSET_TXSTPES_OFFSET)
/// USB Pipe X Control Set Register Underflow Interrupt Enable Set offset
#define USB_REG_UPCONXSET_UNDERFIES_OFFSET		2
/// USB Pipe X Control Set Register Underflow Interrupt Enable Set size
#define USB_REG_UPCONXSET_UNDERFIES_SIZE		1
/// USB Pipe X Control Set Register Underflow Interrupt Enable Set mask
#define USB_REG_UPCONXSET_UNDERFIES_MASK		(((1 << USB_REG_UPCONXSET_UNDERFIES_SIZE) - 1) << USB_REG_UPCONXSET_UNDERFIES_OFFSET)
/// USB Pipe X Control Set Register Pipe Error Interrupt Enable Set offset
#define USB_REG_UPCONXSET_PERRES_OFFSET			3
/// USB Pipe X Control Set Register Pipe Error Interrupt Enable Set size
#define USB_REG_UPCONXSET_PERRES_SIZE			1
/// USB Pipe X Control Set Register Pipe Error Interrupt Enable Set mask
#define USB_REG_UPCONXSET_PERRES_MASK			(((1 << USB_REG_UPCONXSET_PERRES_SIZE) - 1) << USB_REG_UPCONXSET_PERRES_OFFSET)
/// USB Pipe X Control Set Register NAKed Interrupt Enable Set offset
#define USB_REG_UPCONXSET_NAKEDES_OFFSET		4
/// USB Pipe X Control Set Register NAKed Interrupt Enable Set size
#define USB_REG_UPCONXSET_NAKEDES_SIZE			1
/// USB Pipe X Control Set Register NAKed Interrupt Enable Set mask
#define USB_REG_UPCONXSET_NAKEDES_MASK			(((1 << USB_REG_UPCONXSET_NAKEDES_SIZE) - 1) << USB_REG_UPCONXSET_NAKEDES_OFFSET)
/// USB Pipe X Control Set Register Overflow Interrupt Enable Set offset
#define USB_REG_UPCONXSET_OVERFIES_OFFSET		5
/// USB Pipe X Control Set Register Overflow Interrupt Enable Set size
#define USB_REG_UPCONXSET_OVERFIES_SIZE			1
/// USB Pipe X Control Set Register Overflow Interrupt Enable Set mask
#define USB_REG_UPCONXSET_OVERFIES_MASK			(((1 << USB_REG_UPCONXSET_OVERFIES_SIZE) - 1) << USB_REG_UPCONXSET_OVERFIES_OFFSET)
/// USB Pipe X Control Set Register Received STALLed Interrupt Enable Set offset
#define USB_REG_UPCONXSET_RXSTALLDES_OFFSET		6
/// USB Pipe X Control Set Register Received STALLed Interrupt Enable Set size
#define USB_REG_UPCONXSET_RXSTALLDES_SIZE		1
/// USB Pipe X Control Set Register Received STALLed Interrupt Enable Set mask
#define USB_REG_UPCONXSET_RXSTALLDES_MASK		(((1 << USB_REG_UPCONXSET_RXSTALLDES_SIZE) - 1) << USB_REG_UPCONXSET_RXSTALLDES_OFFSET)
/// USB Pipe X Control Set Register CRC Error Interrupt Enable Set offset
#define USB_REG_UPCONXSET_CRCERRES_OFFSET		6
/// USB Pipe X Control Set Register CRC Error Interrupt Enable Set size
#define USB_REG_UPCONXSET_CRCERRES_SIZE			1
/// USB Pipe X Control Set Register CRC Error Interrupt Enable Set mask
#define USB_REG_UPCONXSET_CRCERRES_MASK			(((1 << USB_REG_UPCONXSET_CRCERRES_SIZE) - 1) << USB_REG_UPCONXSET_CRCERRES_OFFSET)
/// USB Pipe X Control Set Register Short Packet Interrupt Enable Set offset
#define USB_REG_UPCONXSET_SHORTPACKETIES_OFFSET		7
/// USB Pipe X Control Set Register Short Packet Interrupt Enable Set size
#define USB_REG_UPCONXSET_SHORTPACKETIES_SIZE		1
/// USB Pipe X Control Set Register Short Packet Interrupt Enable Set mask
#define USB_REG_UPCONXSET_SHORTPACKETIES_MASK		(((1 << USB_REG_UPCONXSET_SHORTPACKETIES_SIZE) - 1) << USB_REG_UPCONXSET_SHORTPACKETIES_OFFSET)
/// USB Pipe X Control Set Register Number of Busy Banks Interrupt Enable Set offset
#define USB_REG_UPCONXSET_NBUSYBKES_OFFSET		12
/// USB Pipe X Control Set Register Number of Busy Banks Interrupt Enable Set size
#define USB_REG_UPCONXSET_NBUSYBKES_SIZE		1
/// USB Pipe X Control Set Register Number of Busy Banks Interrupt Enable Set mask
#define USB_REG_UPCONXSET_NBUSYBKES_MASK		(((1 << USB_REG_UPCONXSET_NBUSYBKES_SIZE) - 1) << USB_REG_UPCONXSET_NBUSYBKES_OFFSET)
/// USB Pipe X Control Set Register Pipe Interrupts Disable HDMA Request Enable Set offset
#define USB_REG_UPCONXSET_PDISHDMAS_OFFSET		16
/// USB Pipe X Control Set Register Pipe Interrupts Disable HDMA Request Enable Set size
#define USB_REG_UPCONXSET_PDISHDMAS_SIZE		1
/// USB Pipe X Control Set Register Pipe Interrupts Disable HDMA Request Enable Set mask
#define USB_REG_UPCONXSET_PDISHDMAS_MASK		(((1 << USB_REG_UPCONXSET_PDISHDMAS_SIZE) - 1) << USB_REG_UPCONXSET_PDISHDMAS_OFFSET)
/// USB Pipe X Control Set Register Pipe Freeze Set offset
#define USB_REG_UPCONXSET_PFREEZES_OFFSET		17
/// USB Pipe X Control Set Register Pipe Freeze Set size
#define USB_REG_UPCONXSET_PFREEZES_SIZE			1
/// USB Pipe X Control Set Register Pipe Freeze Set mask
#define USB_REG_UPCONXSET_PFREEZES_MASK			(((1 << USB_REG_UPCONXSET_PFREEZES_SIZE) - 1) << USB_REG_UPCONXSET_PFREEZES_OFFSET)
/// USB Pipe X Control Set Register Reset Data Toggle Set offset
#define USB_REG_UPCONXSET_RSTDTS_OFFSET			18
/// USB Pipe X Control Set Register Reset Data Toggle Set size
#define USB_REG_UPCONXSET_RSTDTS_SIZE			1
/// USB Pipe X Control Set Register Reset Data Toggle Set mask
#define USB_REG_UPCONXSET_RSTDTS_MASK			(((1 << USB_REG_UPCONXSET_RSTDTS_SIZE) - 1) << USB_REG_UPCONXSET_RSTDTS_OFFSET)
/// USB Pipe 0 Control Clear Register (Write)
#define USB_REG_UPCON0CLR				0x00000620
/// USB Pipe 1 Control Clear Register (Write)
#define USB_REG_UPCON1CLR				0x00000624
/// USB Pipe 2 Control Clear Register (Write)
#define USB_REG_UPCON2CLR				0x00000628
/// USB Pipe 3 Control Clear Register (Write)
#define USB_REG_UPCON3CLR				0x0000062c
/// USB Pipe 4 Control Clear Register (Write)
#define USB_REG_UPCON4CLR				0x00000630
/// USB Pipe 5 Control Clear Register (Write)
#define USB_REG_UPCON5CLR				0x00000634
/// USB Pipe 6 Control Clear Register (Write)
#define USB_REG_UPCON6CLR				0x00000638
/// USB Pipe X Control Clear Register Received IN Data Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_RXINEC_OFFSET			0
/// USB Pipe X Control Clear Register Received IN Data Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_RXINEC_SIZE			1
/// USB Pipe X Control Clear Register Received IN Data Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_RXINEC_MASK			(((1 << USB_REG_UPCONXCLR_RXINEC_SIZE) - 1) << USB_REG_UPCONXCLR_RXINEC_OFFSET)
/// USB Pipe X Control Clear Register Transmitted OUT Data Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_TXOUTEC_OFFSET		1
/// USB Pipe X Control Clear Register Transmitted OUT Data Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_TXOUTEC_SIZE			1
/// USB Pipe X Control Clear Register Transmitted OUT Data Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_TXOUTEC_MASK			(((1 << USB_REG_UPCONXCLR_TXOUTEC_SIZE) - 1) << USB_REG_UPCONXCLR_TXOUTEC_OFFSET)
/// USB Pipe X Control Clear Register Transmitted SETUP Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_TXSTPEC_OFFSET		2
/// USB Pipe X Control Clear Register Transmitted SETUP Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_TXSTPEC_SIZE			1
/// USB Pipe X Control Clear Register Transmitted SETUP Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_TXSTPEC_MASK			(((1 << USB_REG_UPCONXCLR_TXSTPEC_SIZE) - 1) << USB_REG_UPCONXCLR_TXSTPEC_OFFSET)
/// USB Pipe X Control Clear Register Underflow Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_UNDERFIEC_OFFSET		2
/// USB Pipe X Control Clear Register Underflow Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_UNDERFIEC_SIZE		1
/// USB Pipe X Control Clear Register Underflow Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_UNDERFIEC_MASK		(((1 << USB_REG_UPCONXCLR_UNDERFIEC_SIZE) - 1) << USB_REG_UPCONXCLR_UNDERFIEC_OFFSET)
/// USB Pipe X Control Clear Register Pipe Error Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_PERREC_OFFSET			3
/// USB Pipe X Control Clear Register Pipe Error Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_PERREC_SIZE			1
/// USB Pipe X Control Clear Register Pipe Error Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_PERREC_MASK			(((1 << USB_REG_UPCONXCLR_PERREC_SIZE) - 1) << USB_REG_UPCONXCLR_PERREC_OFFSET)
/// USB Pipe X Control Clear Register NAKed Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_NAKEDEC_OFFSET		4
/// USB Pipe X Control Clear Register NAKed Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_NAKEDEC_SIZE			1
/// USB Pipe X Control Clear Register NAKed Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_NAKEDEC_MASK			(((1 << USB_REG_UPCONXCLR_NAKEDEC_SIZE) - 1) << USB_REG_UPCONXCLR_NAKEDEC_OFFSET)
/// USB Pipe X Control Clear Register Overflow Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_OVERFIEC_OFFSET		5
/// USB Pipe X Control Clear Register Overflow Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_OVERFIEC_SIZE			1
/// USB Pipe X Control Clear Register Overflow Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_OVERFIEC_MASK			(((1 << USB_REG_UPCONXCLR_OVERFIEC_SIZE) - 1) << USB_REG_UPCONXCLR_OVERFIEC_OFFSET)
/// USB Pipe X Control Clear Register Received STALLed Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_RXSTALLDEC_OFFSET		6
/// USB Pipe X Control Clear Register Received STALLed Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_RXSTALLDEC_SIZE		1
/// USB Pipe X Control Clear Register Received STALLed Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_RXSTALLDEC_MASK		(((1 << USB_REG_UPCONXCLR_RXSTALLDEC_SIZE) - 1) << USB_REG_UPCONXCLR_RXSTALLDEC_OFFSET)
/// USB Pipe X Control Clear Register CRC Error Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_CRCERREC_OFFSET		6
/// USB Pipe X Control Clear Register CRC Error Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_CRCERREC_SIZE			1
/// USB Pipe X Control Clear Register CRC Error Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_CRCERREC_MASK			(((1 << USB_REG_UPCONXCLR_CRCERREC_SIZE) - 1) << USB_REG_UPCONXCLR_CRCERREC_OFFSET)
/// USB Pipe X Control Clear Register Short Packet Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_SHORTPACKETIEC_OFFSET		7
/// USB Pipe X Control Clear Register Short Packet Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_SHORTPACKETIEC_SIZE		1
/// USB Pipe X Control Clear Register Short Packet Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_SHORTPACKETIEC_MASK		(((1 << USB_REG_UPCONXCLR_SHORTPACKETIEC_SIZE) - 1) << USB_REG_UPCONXCLR_SHORTPACKETIEC_OFFSET)
/// USB Pipe X Control Clear Register Number of Busy Banks Interrupt Enable Clear offset
#define USB_REG_UPCONXCLR_NBUSYBKEC_OFFSET		12
/// USB Pipe X Control Clear Register Number of Busy Banks Interrupt Enable Clear size
#define USB_REG_UPCONXCLR_NBUSYBKEC_SIZE		1
/// USB Pipe X Control Clear Register Number of Busy Banks Interrupt Enable Clear mask
#define USB_REG_UPCONXCLR_NBUSYBKEC_MASK		(((1 << USB_REG_UPCONXCLR_NBUSYBKEC_SIZE) - 1) << USB_REG_UPCONXCLR_NBUSYBKEC_OFFSET)
/// USB Pipe X Control Clear Register FIFO Control Clear offset
#define USB_REG_UPCONXCLR_FIFOCONC_OFFSET		14
/// USB Pipe X Control Clear Register FIFO Control Clear size
#define USB_REG_UPCONXCLR_FIFOCONC_SIZE			1
/// USB Pipe X Control Clear Register FIFO Control Clear mask
#define USB_REG_UPCONXCLR_FIFOCONC_MASK			(((1 << USB_REG_UPCONXCLR_FIFOCONC_SIZE) - 1) << USB_REG_UPCONXCLR_FIFOCONC_OFFSET)
/// USB Pipe X Control Clear Register Pipe Interrupts Disable HDMA Request Enable Clear offset
#define USB_REG_UPCONXCLR_PDISHDMAC_OFFSET		16
/// USB Pipe X Control Clear Register Pipe Interrupts Disable HDMA Request Enable Clear size
#define USB_REG_UPCONXCLR_PDISHDMAC_SIZE		1
/// USB Pipe X Control Clear Register Pipe Interrupts Disable HDMA Request Enable Clear mask
#define USB_REG_UPCONXCLR_PDISHDMAC_MASK		(((1 << USB_REG_UPCONXCLR_PDISHDMAC_SIZE) - 1) << USB_REG_UPCONXCLR_PDISHDMAC_OFFSET)
/// USB Pipe X Control Clear Register Pipe Freeze Clear Clear offset
#define USB_REG_UPCONXCLR_PFREEZEC_OFFSET		17
/// USB Pipe X Control Clear Register Pipe Freeze Clear Clear size
#define USB_REG_UPCONXCLR_PFREEZEC_SIZE			1
/// USB Pipe X Control Clear Register Pipe Freeze Clear Clear mask
#define USB_REG_UPCONXCLR_PFREEZEC_MASK			(((1 << USB_REG_UPCONXCLR_PFREEZEC_SIZE) - 1) << USB_REG_UPCONXCLR_PFREEZEC_OFFSET)
/// USB Pipe 0 IN Request Register (Read/Write)
#define USB_REG_UPINRQ0					0x00000650
/// USB Pipe 1 IN Request Register (Read/Write)
#define USB_REG_UPINRQ1					0x00000654
/// USB Pipe 2 IN Request Register (Read/Write)
#define USB_REG_UPINRQ2					0x00000658
/// USB Pipe 3 IN Request Register (Read/Write)
#define USB_REG_UPINRQ3					0x0000065c
/// USB Pipe 4 IN Request Register (Read/Write)
#define USB_REG_UPINRQ4					0x00000660
/// USB Pipe 5 IN Request Register (Read/Write)
#define USB_REG_UPINRQ5					0x00000664
/// USB Pipe 6 IN Request Register (Read/Write)
#define USB_REG_UPINRQ6					0x00000668
/// USB Pipe X IN Request Register IN Request Number before Freeze offset
#define USB_REG_UPINRQX_INRQ_OFFSET			0
/// USB Pipe X IN Request Register IN Request Number before Freeze size
#define USB_REG_UPINRQX_INRQ_SIZE			8
/// USB Pipe X IN Request Register IN Request Number before Freeze mask
#define USB_REG_UPINRQX_INRQ_MASK			(((1 << USB_REG_UPINRQX_INRQ_SIZE) - 1) << USB_REG_UPINRQX_INRQ_OFFSET)
/// USB Pipe X IN Request Register IN Request Mode offset
#define USB_REG_UPINRQX_INMODE_OFFSET			8
/// USB Pipe X IN Request Register IN Request Mode size
#define USB_REG_UPINRQX_INMODE_SIZE			1
/// USB Pipe X IN Request Register IN Request Mode mask
#define USB_REG_UPINRQX_INMODE_MASK			(((1 << USB_REG_UPINRQX_INMODE_SIZE) - 1) << USB_REG_UPINRQX_INMODE_OFFSET)
/// USB Pipe 0 Error Register (Read/Write)
#define USB_REG_UPERR0					0x00000680
/// USB Pipe 1 Error Register (Read/Write)
#define USB_REG_UPERR1					0x00000684
/// USB Pipe 2 Error Register (Read/Write)
#define USB_REG_UPERR2					0x00000688
/// USB Pipe 3 Error Register (Read/Write)
#define USB_REG_UPERR3					0x0000068c
/// USB Pipe 4 Error Register (Read/Write)
#define USB_REG_UPERR4					0x00000690
/// USB Pipe 5 Error Register (Read/Write)
#define USB_REG_UPERR5					0x00000694
/// USB Pipe 6 Error Register (Read/Write)
#define USB_REG_UPERR6					0x00000698
/// USB Pipe X Error Register Data Toggle Error offset
#define USB_REG_UPERRX_DATATGL_OFFSET			0
/// USB Pipe X Error Register Data Toggle Error size
#define USB_REG_UPERRX_DATATGL_SIZE			1
/// USB Pipe X Error Register Data Toggle Error mask
#define USB_REG_UPERRX_DATATGL_MASK			(((1 << USB_REG_UPERRX_DATATGL_SIZE) - 1) << USB_REG_UPERRX_DATATGL_OFFSET)
/// USB Pipe X Error Register Data PID Error offset
#define USB_REG_UPERRX_DATAPID_OFFSET			1
/// USB Pipe X Error Register Data PID Error size
#define USB_REG_UPERRX_DATAPID_SIZE			1
/// USB Pipe X Error Register Data PID Error mask
#define USB_REG_UPERRX_DATAPID_MASK			(((1 << USB_REG_UPERRX_DATAPID_SIZE) - 1) << USB_REG_UPERRX_DATAPID_OFFSET)
/// USB Pipe X Error Register PID Error offset
#define USB_REG_UPERRX_PID_OFFSET			2
/// USB Pipe X Error Register PID Error size
#define USB_REG_UPERRX_PID_SIZE				1
/// USB Pipe X Error Register PID Error mask
#define USB_REG_UPERRX_PID_MASK				(((1 << USB_REG_UPERRX_PID_SIZE) - 1) << USB_REG_UPERRX_PID_OFFSET)
/// USB Pipe X Error Register Time-Out Error offset
#define USB_REG_UPERRX_TIMEOUT_OFFSET			3
/// USB Pipe X Error Register Time-Out Error size
#define USB_REG_UPERRX_TIMEOUT_SIZE			1
/// USB Pipe X Error Register Time-Out Error mask
#define USB_REG_UPERRX_TIMEOUT_MASK			(((1 << USB_REG_UPERRX_TIMEOUT_SIZE) - 1) << USB_REG_UPERRX_TIMEOUT_OFFSET)
/// USB Pipe X Error Register CRC16 Error offset
#define USB_REG_UPERRX_CRC16_OFFSET			4
/// USB Pipe X Error Register CRC16 Error size
#define USB_REG_UPERRX_CRC16_SIZE			1
/// USB Pipe X Error Register CRC16 Error mask
#define USB_REG_UPERRX_CRC16_MASK			(((1 << USB_REG_UPERRX_CRC16_SIZE) - 1) << USB_REG_UPERRX_CRC16_OFFSET)
/// USB Pipe X Error Register Error Counter offset
#define USB_REG_UPERRX_COUNTER_OFFSET			5
/// USB Pipe X Error Register Error Counter size
#define USB_REG_UPERRX_COUNTER_SIZE			2
/// USB Pipe X Error Register Error Counter mask
#define USB_REG_UPERRX_COUNTER_MASK			(((1 << USB_REG_UPERRX_COUNTER_SIZE) - 1) << USB_REG_UPERRX_COUNTER_OFFSET)
/// USB Host DMA Channel 1 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA1_NEXTDESC				0x00000710
/// USB Host DMA Channel 1 HSB Address Register (Read/Write)
#define USB_REG_UHDMA1_ADDR				0x00000714
/// USB Host DMA Channel 1 Control Register (Read/Write)
#define USB_REG_UHDMA1_CONTROL				0x00000718
/// USB Host DMA Channel 1 Status Register (Read/Write)
#define USB_REG_UHDMA1_STATUS				0x0000071c
/// USB Host DMA Channel 2 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA2_NEXTDESC				0x00000720
/// USB Host DMA Channel 2 HSB Address Register (Read/Write)
#define USB_REG_UHDMA2_ADDR				0x00000724
/// USB Host DMA Channel 2 Control Register (Read/Write)
#define USB_REG_UHDMA2_CONTROL				0x00000728
/// USB Host DMA Channel 2 Status Register (Read/Write)
#define USB_REG_UHDMA2_STATUS				0x0000072c
/// USB Host DMA Channel 3 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA3_NEXTDESC				0x00000730
/// USB Host DMA Channel 3 HSB Address Register (Read/Write)
#define USB_REG_UHDMA3_ADDR				0x00000734
/// USB Host DMA Channel 3 Control Register (Read/Write)
#define USB_REG_UHDMA3_CONTROL				0x00000738
/// USB Host DMA Channel 3 Status Register (Read/Write)
#define USB_REG_UHDMA3_STATUS				0x0000073c
/// USB Host DMA Channel 4 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA4_NEXTDESC				0x00000740
/// USB Host DMA Channel 4 HSB Address Register (Read/Write)
#define USB_REG_UHDMA4_ADDR				0x00000744
/// USB Host DMA Channel 4 Control Register (Read/Write)
#define USB_REG_UHDMA4_CONTROL				0x00000748
/// USB Host DMA Channel 4 Status Register (Read/Write)
#define USB_REG_UHDMA4_STATUS				0x0000074c
/// USB Host DMA Channel 5 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA5_NEXTDESC				0x00000750
/// USB Host DMA Channel 5 HSB Address Register (Read/Write)
#define USB_REG_UHDMA5_ADDR				0x00000754
/// USB Host DMA Channel 5 Control Register (Read/Write)
#define USB_REG_UHDMA5_CONTROL				0x00000758
/// USB Host DMA Channel 5 Status Register (Read/Write)
#define USB_REG_UHDMA5_STATUS				0x0000075c
/// USB Host DMA Channel 6 Next Descriptor Address Register (Read/Write)
#define USB_REG_UHDMA6_NEXTDESC				0x00000760
/// USB Host DMA Channel 6 HSB Address Register (Read/Write)
#define USB_REG_UHDMA6_ADDR				0x00000764
/// USB Host DMA Channel 6 Control Register (Read/Write)
#define USB_REG_UHDMA6_CONTROL				0x00000768
/// USB Host DMA Channel 6 Status Register (Read/Write)
#define USB_REG_UHDMA6_STATUS				0x0000076c
/// USB Host DMA Channel X Next Descriptor Address Register Next Descriptor Address offset
#define USB_REG_UHDMAX_NEXTDESC_NXT_DESC_ADDR_OFFSET	4
/// USB Host DMA Channel X Next Descriptor Address Register Next Descriptor Address size
#define USB_REG_UHDMAX_NEXTDESC_NXT_DESC_ADDR_SIZE	28
/// USB Host DMA Channel X Next Descriptor Address Register Next Descriptor Address mask
#define USB_REG_UHDMAX_NEXTDESC_NXT_DESC_ADDR_MASK	(((1 << USB_REG_UHDMAX_NEXTDESC_NXT_DESC_ADDR_SIZE) - 1) << USB_REG_UHDMAX_NEXTDESC_NXT_DESC_ADDR_OFFSET)
/// USB Host DMA Channel X HSB Address Register HSB Address offset
#define USB_REG_UHDMAX_ADDR_HSB_ADDR_OFFSET		0
/// USB Host DMA Channel X HSB Address Register HSB Address size
#define USB_REG_UHDMAX_ADDR_HSB_ADDR_SIZE		32
/// USB Host DMA Channel X HSB Address Register HSB Address mask
#define USB_REG_UHDMAX_ADDR_HSB_ADDR_MASK		(((1 << USB_REG_UHDMAX_ADDR_HSB_ADDR_SIZE) - 1) << USB_REG_UHDMAX_ADDR_HSB_ADDR_OFFSET)
/// USB Host DMA Channel X Control Register Channel Enable offset
#define USB_REG_UHDMAX_CONTROL_CH_EN_OFFSET		0
/// USB Host DMA Channel X Control Register Channel Enable size
#define USB_REG_UHDMAX_CONTROL_CH_EN_SIZE		1
/// USB Host DMA Channel X Control Register Channel Enable mask
#define USB_REG_UHDMAX_CONTROL_CH_EN_MASK		(((1 << USB_REG_UHDMAX_CONTROL_CH_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_CH_EN_OFFSET)
/// USB Host DMA Channel X Control Register Load Next Channel Descriptor Enable offset
#define USB_REG_UHDMAX_CONTROL_LD_NXT_CH_DESC_EN_OFFSET	1
/// USB Host DMA Channel X Control Register Load Next Channel Descriptor Enable size
#define USB_REG_UHDMAX_CONTROL_LD_NXT_CH_DESC_EN_SIZE	1
/// USB Host DMA Channel X Control Register Load Next Channel Descriptor Enable mask
#define USB_REG_UHDMAX_CONTROL_LD_NXT_CH_DESC_EN_MASK	(((1 << USB_REG_UHDMAX_CONTROL_LD_NXT_CH_DESC_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_LD_NXT_CH_DESC_EN_OFFSET)
/// USB Host DMA Channel X Control Register Buffer Close Input Enable offset
#define USB_REG_UHDMAX_CONTROL_BUFF_CLOSE_IN_EN_OFFSET	2
/// USB Host DMA Channel X Control Register Buffer Close Input Enable size
#define USB_REG_UHDMAX_CONTROL_BUFF_CLOSE_IN_EN_SIZE	1
/// USB Host DMA Channel X Control Register Buffer Close Input Enable mask
#define USB_REG_UHDMAX_CONTROL_BUFF_CLOSE_IN_EN_MASK	(((1 << USB_REG_UHDMAX_CONTROL_BUFF_CLOSE_IN_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_BUFF_CLOSE_IN_EN_OFFSET)
/// USB Host DMA Channel X Control Register End of DMA Buffer Output Enable offset
#define USB_REG_UHDMAX_CONTROL_DMAEND_EN_OFFSET		3
/// USB Host DMA Channel X Control Register End of DMA Buffer Output Enable size
#define USB_REG_UHDMAX_CONTROL_DMAEND_EN_SIZE		1
/// USB Host DMA Channel X Control Register End of DMA Buffer Output Enable mask
#define USB_REG_UHDMAX_CONTROL_DMAEND_EN_MASK		(((1 << USB_REG_UHDMAX_CONTROL_DMAEND_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_DMAEND_EN_OFFSET)
/// USB Host DMA Channel X Control Register End of USB Transfer Interrupt Enable offset
#define USB_REG_UHDMAX_CONTROL_EOT_IRQ_EN_OFFSET	4
/// USB Host DMA Channel X Control Register End of USB Transfer Interrupt Enable size
#define USB_REG_UHDMAX_CONTROL_EOT_IRQ_EN_SIZE		1
/// USB Host DMA Channel X Control Register End of USB Transfer Interrupt Enable mask
#define USB_REG_UHDMAX_CONTROL_EOT_IRQ_EN_MASK		(((1 << USB_REG_UHDMAX_CONTROL_EOT_IRQ_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_EOT_IRQ_EN_OFFSET)
/// USB Host DMA Channel X Control Register End of Buffer Interrupt Enable offset
#define USB_REG_UHDMAX_CONTROL_EOBUFF_IRQ_EN_OFFSET	5
/// USB Host DMA Channel X Control Register End of Buffer Interrupt Enable size
#define USB_REG_UHDMAX_CONTROL_EOBUFF_IRQ_EN_SIZE	1
/// USB Host DMA Channel X Control Register End of Buffer Interrupt Enable mask
#define USB_REG_UHDMAX_CONTROL_EOBUFF_IRQ_EN_MASK	(((1 << USB_REG_UHDMAX_CONTROL_EOBUFF_IRQ_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_EOBUFF_IRQ_EN_OFFSET)
/// USB Host DMA Channel X Control Register Descriptor Loaded Interrupt Enable offset
#define USB_REG_UHDMAX_CONTROL_DESC_LD_IRQ_EN_OFFSET	6
/// USB Host DMA Channel X Control Register Descriptor Loaded Interrupt Enable size
#define USB_REG_UHDMAX_CONTROL_DESC_LD_IRQ_EN_SIZE	1
/// USB Host DMA Channel X Control Register Descriptor Loaded Interrupt Enable mask
#define USB_REG_UHDMAX_CONTROL_DESC_LD_IRQ_EN_MASK	(((1 << USB_REG_UHDMAX_CONTROL_DESC_LD_IRQ_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_DESC_LD_IRQ_EN_OFFSET)
/// USB Host DMA Channel X Control Register Burst Lock Enable offset
#define USB_REG_UHDMAX_CONTROL_BURST_LOCK_EN_OFFSET	7
/// USB Host DMA Channel X Control Register Burst Lock Enable size
#define USB_REG_UHDMAX_CONTROL_BURST_LOCK_EN_SIZE	1
/// USB Host DMA Channel X Control Register Burst Lock Enable mask
#define USB_REG_UHDMAX_CONTROL_BURST_LOCK_EN_MASK	(((1 << USB_REG_UHDMAX_CONTROL_BURST_LOCK_EN_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_BURST_LOCK_EN_OFFSET)
/// USB Host DMA Channel X Control Register Channel Byte Length offset
#define USB_REG_UHDMAX_CONTROL_CH_BYTE_LENGTH_OFFSET	16
/// USB Host DMA Channel X Control Register Channel Byte Length size
#define USB_REG_UHDMAX_CONTROL_CH_BYTE_LENGTH_SIZE	16
/// USB Host DMA Channel X Control Register Channel Byte Length mask
#define USB_REG_UHDMAX_CONTROL_CH_BYTE_LENGTH_MASK	(((1 << USB_REG_UHDMAX_CONTROL_CH_BYTE_LENGTH_SIZE) - 1) << USB_REG_UHDMAX_CONTROL_CH_BYTE_LENGTH_OFFSET)
/// USB Host DMA Channel X Status Register Channel Enable offset
#define USB_REG_UHDMAX_STATUS_CH_EN_OFFSET		0
/// USB Host DMA Channel X Status Register Channel Enable size
#define USB_REG_UHDMAX_STATUS_CH_EN_SIZE		1
/// USB Host DMA Channel X Status Register Channel Enable mask
#define USB_REG_UHDMAX_STATUS_CH_EN_MASK		(((1 << USB_REG_UHDMAX_STATUS_CH_EN_SIZE) - 1) << USB_REG_UHDMAX_STATUS_CH_EN_OFFSET)
/// USB Host DMA Channel X Status Register Channel Active offset
#define USB_REG_UHDMAX_STATUS_CH_ACTIVE_OFFSET		1
/// USB Host DMA Channel X Status Register Channel Active size
#define USB_REG_UHDMAX_STATUS_CH_ACTIVE_SIZE		1
/// USB Host DMA Channel X Status Register Channel Active mask
#define USB_REG_UHDMAX_STATUS_CH_ACTIVE_MASK		(((1 << USB_REG_UHDMAX_STATUS_CH_ACTIVE_SIZE) - 1) << USB_REG_UHDMAX_STATUS_CH_ACTIVE_OFFSET)
/// USB Host DMA Channel X Status Register End of USB Transfer Status offset
#define USB_REG_UHDMAX_STATUS_EOT_STA_OFFSET		4
/// USB Host DMA Channel X Status Register End of USB Transfer Status size
#define USB_REG_UHDMAX_STATUS_EOT_STA_SIZE		1
/// USB Host DMA Channel X Status Register End of USB Transfer Status mask
#define USB_REG_UHDMAX_STATUS_EOT_STA_MASK		(((1 << USB_REG_UHDMAX_STATUS_EOT_STA_SIZE) - 1) << USB_REG_UHDMAX_STATUS_EOT_STA_OFFSET)
/// USB Host DMA Channel X Status Register End of Channel Buffer Status offset
#define USB_REG_UHDMAX_STATUS_EOCH_BUFF_STA_OFFSET	5
/// USB Host DMA Channel X Status Register End of Channel Buffer Status size
#define USB_REG_UHDMAX_STATUS_EOCH_BUFF_STA_SIZE	1
/// USB Host DMA Channel X Status Register End of Channel Buffer Status mask
#define USB_REG_UHDMAX_STATUS_EOCH_BUFF_STA_MASK	(((1 << USB_REG_UHDMAX_STATUS_EOCH_BUFF_STA_SIZE) - 1) << USB_REG_UHDMAX_STATUS_EOCH_BUFF_STA_OFFSET)
/// USB Host DMA Channel X Status Register Descriptor Loaded Status offset
#define USB_REG_UHDMAX_STATUS_DESC_LD_STA_OFFSET	6
/// USB Host DMA Channel X Status Register Descriptor Loaded Status size
#define USB_REG_UHDMAX_STATUS_DESC_LD_STA_SIZE		1
/// USB Host DMA Channel X Status Register Descriptor Loaded Status mask
#define USB_REG_UHDMAX_STATUS_DESC_LD_STA_MASK		(((1 << USB_REG_UHDMAX_STATUS_DESC_LD_STA_SIZE) - 1) << USB_REG_UHDMAX_STATUS_DESC_LD_STA_OFFSET)
/// USB Host DMA Channel X Status Register Channel Byte Count offset
#define USB_REG_UHDMAX_STATUS_CH_BYTE_CNT_OFFSET	16
/// USB Host DMA Channel X Status Register Channel Byte Count size
#define USB_REG_UHDMAX_STATUS_CH_BYTE_CNT_SIZE		16
/// USB Host DMA Channel X Status Register Channel Byte Count mask
#define USB_REG_UHDMAX_STATUS_CH_BYTE_CNT_MASK		(((1 << USB_REG_UHDMAX_STATUS_CH_BYTE_CNT_SIZE) - 1) << USB_REG_UHDMAX_STATUS_CH_BYTE_CNT_OFFSET)
/// USB General Control Register (Read/Write)
#define USB_REG_USBCON					0x00000800
/// USB General Control Register ID Transition Interrupt Enable offset
#define USB_REG_USBCON_IDTE_OFFSET			0
/// USB General Control Register ID Transition Interrupt Enable size
#define USB_REG_USBCON_IDTE_SIZE			1
/// USB General Control Register ID Transition Interrupt Enable mask
#define USB_REG_USBCON_IDTE_MASK			(((1 << USB_REG_USBCON_IDTE_SIZE) - 1) << USB_REG_USBCON_IDTE_OFFSET)
/// USB General Control Register VBus Transition Interrupt Enable offset
#define USB_REG_USBCON_VBUSTE_OFFSET			1
/// USB General Control Register VBus Transition Interrupt Enable size
#define USB_REG_USBCON_VBUSTE_SIZE			1
/// USB General Control Register VBus Transition Interrupt Enable mask
#define USB_REG_USBCON_VBUSTE_MASK			(((1 << USB_REG_USBCON_VBUSTE_SIZE) - 1) << USB_REG_USBCON_VBUSTE_OFFSET)
/// USB General Control Register SRP Interrupt Enable offset
#define USB_REG_USBCON_SRPE_OFFSET			2
/// USB General Control Register SRP Interrupt Enable size
#define USB_REG_USBCON_SRPE_SIZE			1
/// USB General Control Register SRP Interrupt Enable mask
#define USB_REG_USBCON_SRPE_MASK			(((1 << USB_REG_USBCON_SRPE_SIZE) - 1) << USB_REG_USBCON_SRPE_OFFSET)
/// USB General Control Register VBus Error Interrupt Enable offset
#define USB_REG_USBCON_VBERRE_OFFSET			3
/// USB General Control Register VBus Error Interrupt Enable size
#define USB_REG_USBCON_VBERRE_SIZE			1
/// USB General Control Register VBus Error Interrupt Enable mask
#define USB_REG_USBCON_VBERRE_MASK			(((1 << USB_REG_USBCON_VBERRE_SIZE) - 1) << USB_REG_USBCON_VBERRE_OFFSET)
/// USB General Control Register B-Connection Error Interrupt Enable offset
#define USB_REG_USBCON_BCERRE_OFFSET			4
/// USB General Control Register B-Connection Error Interrupt Enable size
#define USB_REG_USBCON_BCERRE_SIZE			1
/// USB General Control Register B-Connection Error Interrupt Enable mask
#define USB_REG_USBCON_BCERRE_MASK			(((1 << USB_REG_USBCON_BCERRE_SIZE) - 1) << USB_REG_USBCON_BCERRE_OFFSET)
/// USB General Control Register Role Exchange Interrupt Enable offset
#define USB_REG_USBCON_ROLEEXE_OFFSET			5
/// USB General Control Register Role Exchange Interrupt Enable size
#define USB_REG_USBCON_ROLEEXE_SIZE			1
/// USB General Control Register Role Exchange Interrupt Enable mask
#define USB_REG_USBCON_ROLEEXE_MASK			(((1 << USB_REG_USBCON_ROLEEXE_SIZE) - 1) << USB_REG_USBCON_ROLEEXE_OFFSET)
/// USB General Control Register HNP Error Interrupt Enable offset
#define USB_REG_USBCON_HNPERRE_OFFSET			6
/// USB General Control Register HNP Error Interrupt Enable size
#define USB_REG_USBCON_HNPERRE_SIZE			1
/// USB General Control Register HNP Error Interrupt Enable mask
#define USB_REG_USBCON_HNPERRE_MASK			(((1 << USB_REG_USBCON_HNPERRE_SIZE) - 1) << USB_REG_USBCON_HNPERRE_OFFSET)
/// USB General Control Register Suspend Time-Out Interrupt Enable offset
#define USB_REG_USBCON_STOE_OFFSET			7
/// USB General Control Register Suspend Time-Out Interrupt Enable size
#define USB_REG_USBCON_STOE_SIZE			1
/// USB General Control Register Suspend Time-Out Interrupt Enable mask
#define USB_REG_USBCON_STOE_MASK			(((1 << USB_REG_USBCON_STOE_SIZE) - 1) << USB_REG_USBCON_STOE_OFFSET)
/// USB General Control Register VBus Hardware Control offset
#define USB_REG_USBCON_VBUSHWC_OFFSET			8
/// USB General Control Register VBus Hardware Control size
#define USB_REG_USBCON_VBUSHWC_SIZE			1
/// USB General Control Register VBus Hardware Control mask
#define USB_REG_USBCON_VBUSHWC_MASK			(((1 << USB_REG_USBCON_VBUSHWC_SIZE) - 1) << USB_REG_USBCON_VBUSHWC_OFFSET)
/// USB General Control Register SRP Selection offset
#define USB_REG_USBCON_SRPSEL_OFFSET			9
/// USB General Control Register SRP Selection size
#define USB_REG_USBCON_SRPSEL_SIZE			1
/// USB General Control Register SRP Selection mask
#define USB_REG_USBCON_SRPSEL_MASK			(((1 << USB_REG_USBCON_SRPSEL_SIZE) - 1) << USB_REG_USBCON_SRPSEL_OFFSET)
/// USB General Control Register SRP Request offset
#define USB_REG_USBCON_SRPREQ_OFFSET			10
/// USB General Control Register SRP Request size
#define USB_REG_USBCON_SRPREQ_SIZE			1
/// USB General Control Register SRP Request mask
#define USB_REG_USBCON_SRPREQ_MASK			(((1 << USB_REG_USBCON_SRPREQ_SIZE) - 1) << USB_REG_USBCON_SRPREQ_OFFSET)
/// USB General Control Register HNP Request offset
#define USB_REG_USBCON_HNPREQ_OFFSET			11
/// USB General Control Register HNP Request size
#define USB_REG_USBCON_HNPREQ_SIZE			1
/// USB General Control Register HNP Request mask
#define USB_REG_USBCON_HNPREQ_MASK			(((1 << USB_REG_USBCON_HNPREQ_SIZE) - 1) << USB_REG_USBCON_HNPREQ_OFFSET)
/// USB General Control Register OTG Pad Enable offset
#define USB_REG_USBCON_OTGPADE_OFFSET			12
/// USB General Control Register OTG Pad Enable size
#define USB_REG_USBCON_OTGPADE_SIZE			1
/// USB General Control Register OTG Pad Enable mask
#define USB_REG_USBCON_OTGPADE_MASK			(((1 << USB_REG_USBCON_OTGPADE_SIZE) - 1) << USB_REG_USBCON_OTGPADE_OFFSET)
/// USB General Control Register VBus Polarity offset
#define USB_REG_USBCON_VBUSPO_OFFSET			13
/// USB General Control Register VBus Polarity size
#define USB_REG_USBCON_VBUSPO_SIZE			1
/// USB General Control Register VBus Polarity mask
#define USB_REG_USBCON_VBUSPO_MASK			(((1 << USB_REG_USBCON_VBUSPO_SIZE) - 1) << USB_REG_USBCON_VBUSPO_OFFSET)
/// USB General Control Register Freeze USB Clock offset
#define USB_REG_USBCON_FRZCLK_OFFSET			14
/// USB General Control Register Freeze USB Clock size
#define USB_REG_USBCON_FRZCLK_SIZE			1
/// USB General Control Register Freeze USB Clock mask
#define USB_REG_USBCON_FRZCLK_MASK			(((1 << USB_REG_USBCON_FRZCLK_SIZE) - 1) << USB_REG_USBCON_FRZCLK_OFFSET)
/// USB General Control Register USB Macro Enable offset
#define USB_REG_USBCON_USBE_OFFSET			15
/// USB General Control Register USB Macro Enable size
#define USB_REG_USBCON_USBE_SIZE			1
/// USB General Control Register USB Macro Enable mask
#define USB_REG_USBCON_USBE_MASK			(((1 << USB_REG_USBCON_USBE_SIZE) - 1) << USB_REG_USBCON_USBE_OFFSET)
/// USB General Control Register Timer Value offset
#define USB_REG_USBCON_TIMVALUE_OFFSET			16
/// USB General Control Register Timer Value size
#define USB_REG_USBCON_TIMVALUE_SIZE			2
/// USB General Control Register Timer Value mask
#define USB_REG_USBCON_TIMVALUE_MASK			(((1 << USB_REG_USBCON_TIMVALUE_SIZE) - 1) << USB_REG_USBCON_TIMVALUE_OFFSET)
/// USB General Control Register Timer Page offset
#define USB_REG_USBCON_TIMPAGE_OFFSET			20
/// USB General Control Register Timer Page size
#define USB_REG_USBCON_TIMPAGE_SIZE			2
/// USB General Control Register Timer Page mask
#define USB_REG_USBCON_TIMPAGE_MASK			(((1 << USB_REG_USBCON_TIMPAGE_SIZE) - 1) << USB_REG_USBCON_TIMPAGE_OFFSET)
/// USB General Control Register Timer Access Unlock offset
#define USB_REG_USBCON_UNLOCK_OFFSET			22
/// USB General Control Register Timer Access Unlock size
#define USB_REG_USBCON_UNLOCK_SIZE			1
/// USB General Control Register Timer Access Unlock mask
#define USB_REG_USBCON_UNLOCK_MASK			(((1 << USB_REG_USBCON_UNLOCK_SIZE) - 1) << USB_REG_USBCON_UNLOCK_OFFSET)
/// USB General Control Register USB_ID Pin Enable offset
#define USB_REG_USBCON_UIDE_OFFSET			24
/// USB General Control Register USB_ID Pin Enable size
#define USB_REG_USBCON_UIDE_SIZE			1
/// USB General Control Register USB_ID Pin Enable mask
#define USB_REG_USBCON_UIDE_MASK			(((1 << USB_REG_USBCON_UIDE_SIZE) - 1) << USB_REG_USBCON_UIDE_OFFSET)
/// USB General Control Register USB Macro Mode offset
#define USB_REG_USBCON_UIMOD_OFFSET			25
/// USB General Control Register USB Macro Mode size
#define USB_REG_USBCON_UIMOD_SIZE			1
/// USB General Control Register USB Macro Mode mask
#define USB_REG_USBCON_UIMOD_MASK			(((1 << USB_REG_USBCON_UIMOD_SIZE) - 1) << USB_REG_USBCON_UIMOD_OFFSET)
/// USB General Status Register (Read)
#define USB_REG_USBSTA					0x00000804
/// USB General Status Register ID Transition Interrupt Flag offset
#define USB_REG_USBSTA_IDTI_OFFSET			0
/// USB General Status Register ID Transition Interrupt Flag size
#define USB_REG_USBSTA_IDTI_SIZE			1
/// USB General Status Register ID Transition Interrupt Flag mask
#define USB_REG_USBSTA_IDTI_MASK			(((1 << USB_REG_USBSTA_IDTI_SIZE) - 1) << USB_REG_USBSTA_IDTI_OFFSET)
/// USB General Status Register VBus Transition Interrupt Flag offset
#define USB_REG_USBSTA_VBUSTI_OFFSET			1
/// USB General Status Register VBus Transition Interrupt Flag size
#define USB_REG_USBSTA_VBUSTI_SIZE			1
/// USB General Status Register VBus Transition Interrupt Flag mask
#define USB_REG_USBSTA_VBUSTI_MASK			(((1 << USB_REG_USBSTA_VBUSTI_SIZE) - 1) << USB_REG_USBSTA_VBUSTI_OFFSET)
/// USB General Status Register SRP Interrupt Flag offset
#define USB_REG_USBSTA_SRPI_OFFSET			2
/// USB General Status Register SRP Interrupt Flag size
#define USB_REG_USBSTA_SRPI_SIZE			1
/// USB General Status Register SRP Interrupt Flag mask
#define USB_REG_USBSTA_SRPI_MASK			(((1 << USB_REG_USBSTA_SRPI_SIZE) - 1) << USB_REG_USBSTA_SRPI_OFFSET)
/// USB General Status Register VBus Error Interrupt Flag offset
#define USB_REG_USBSTA_VBERRI_OFFSET			3
/// USB General Status Register VBus Error Interrupt Flag size
#define USB_REG_USBSTA_VBERRI_SIZE			1
/// USB General Status Register VBus Error Interrupt Flag mask
#define USB_REG_USBSTA_VBERRI_MASK			(((1 << USB_REG_USBSTA_VBERRI_SIZE) - 1) << USB_REG_USBSTA_VBERRI_OFFSET)
/// USB General Status Register B-Connection Error Interrupt Flag offset
#define USB_REG_USBSTA_BCERRI_OFFSET			4
/// USB General Status Register B-Connection Error Interrupt Flag size
#define USB_REG_USBSTA_BCERRI_SIZE			1
/// USB General Status Register B-Connection Error Interrupt Flag mask
#define USB_REG_USBSTA_BCERRI_MASK			(((1 << USB_REG_USBSTA_BCERRI_SIZE) - 1) << USB_REG_USBSTA_BCERRI_OFFSET)
/// USB General Status Register Role Exchange Interrupt Flag offset
#define USB_REG_USBSTA_ROLEEXI_OFFSET			5
/// USB General Status Register Role Exchange Interrupt Flag size
#define USB_REG_USBSTA_ROLEEXI_SIZE			1
/// USB General Status Register Role Exchange Interrupt Flag mask
#define USB_REG_USBSTA_ROLEEXI_MASK			(((1 << USB_REG_USBSTA_ROLEEXI_SIZE) - 1) << USB_REG_USBSTA_ROLEEXI_OFFSET)
/// USB General Status Register HNP Error Interrupt Flag offset
#define USB_REG_USBSTA_HNPERRI_OFFSET			6
/// USB General Status Register HNP Error Interrupt Flag size
#define USB_REG_USBSTA_HNPERRI_SIZE			1
/// USB General Status Register HNP Error Interrupt Flag mask
#define USB_REG_USBSTA_HNPERRI_MASK			(((1 << USB_REG_USBSTA_HNPERRI_SIZE) - 1) << USB_REG_USBSTA_HNPERRI_OFFSET)
/// USB General Status Register Suspend Time-Out Interrupt Flag offset
#define USB_REG_USBSTA_STOI_OFFSET			7
/// USB General Status Register Suspend Time-Out Interrupt Flag size
#define USB_REG_USBSTA_STOI_SIZE			1
/// USB General Status Register Suspend Time-Out Interrupt Flag mask
#define USB_REG_USBSTA_STOI_MASK			(((1 << USB_REG_USBSTA_STOI_SIZE) - 1) << USB_REG_USBSTA_STOI_OFFSET)
/// USB General Status Register VBus Request offset
#define USB_REG_USBSTA_VBUSRQ_OFFSET			9
/// USB General Status Register VBus Request size
#define USB_REG_USBSTA_VBUSRQ_SIZE			1
/// USB General Status Register VBus Request mask
#define USB_REG_USBSTA_VBUSRQ_MASK			(((1 << USB_REG_USBSTA_VBUSRQ_SIZE) - 1) << USB_REG_USBSTA_VBUSRQ_OFFSET)
/// USB General Status Register USB_ID Pin State offset
#define USB_REG_USBSTA_ID_OFFSET			10
/// USB General Status Register USB_ID Pin State size
#define USB_REG_USBSTA_ID_SIZE				1
/// USB General Status Register USB_ID Pin State mask
#define USB_REG_USBSTA_ID_MASK				(((1 << USB_REG_USBSTA_ID_SIZE) - 1) << USB_REG_USBSTA_ID_OFFSET)
/// USB General Status Register VBus Level offset
#define USB_REG_USBSTA_VBUS_OFFSET			11
/// USB General Status Register VBus Level size
#define USB_REG_USBSTA_VBUS_SIZE			1
/// USB General Status Register VBus Level mask
#define USB_REG_USBSTA_VBUS_MASK			(((1 << USB_REG_USBSTA_VBUS_SIZE) - 1) << USB_REG_USBSTA_VBUS_OFFSET)
/// USB General Status Register Speed Status offset
#define USB_REG_USBSTA_SPEED_OFFSET			12
/// USB General Status Register Speed Status size
#define USB_REG_USBSTA_SPEED_SIZE			2
/// USB General Status Register Speed Status mask
#define USB_REG_USBSTA_SPEED_MASK			(((1 << USB_REG_USBSTA_SPEED_SIZE) - 1) << USB_REG_USBSTA_SPEED_OFFSET)
/// USB General Status Clear Register (Write)
#define USB_REG_USBSTACLR				0x00000808
/// USB General Status Clear Register ID Transition Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_IDTIC_OFFSET			0
/// USB General Status Clear Register ID Transition Interrupt Flag Clear size
#define USB_REG_USBSTACLR_IDTIC_SIZE			1
/// USB General Status Clear Register ID Transition Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_IDTIC_MASK			(((1 << USB_REG_USBSTACLR_IDTIC_SIZE) - 1) << USB_REG_USBSTACLR_IDTIC_OFFSET)
/// USB General Status Clear Register VBus Transition Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_VBUSTIC_OFFSET		1
/// USB General Status Clear Register VBus Transition Interrupt Flag Clear size
#define USB_REG_USBSTACLR_VBUSTIC_SIZE			1
/// USB General Status Clear Register VBus Transition Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_VBUSTIC_MASK			(((1 << USB_REG_USBSTACLR_VBUSTIC_SIZE) - 1) << USB_REG_USBSTACLR_VBUSTIC_OFFSET)
/// USB General Status Clear Register SRP Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_SRPIC_OFFSET			2
/// USB General Status Clear Register SRP Interrupt Flag Clear size
#define USB_REG_USBSTACLR_SRPIC_SIZE			1
/// USB General Status Clear Register SRP Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_SRPIC_MASK			(((1 << USB_REG_USBSTACLR_SRPIC_SIZE) - 1) << USB_REG_USBSTACLR_SRPIC_OFFSET)
/// USB General Status Clear Register VBus Error Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_VBERRIC_OFFSET		3
/// USB General Status Clear Register VBus Error Interrupt Flag Clear size
#define USB_REG_USBSTACLR_VBERRIC_SIZE			1
/// USB General Status Clear Register VBus Error Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_VBERRIC_MASK			(((1 << USB_REG_USBSTACLR_VBERRIC_SIZE) - 1) << USB_REG_USBSTACLR_VBERRIC_OFFSET)
/// USB General Status Clear Register B-Connection Error Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_BCERRIC_OFFSET		4
/// USB General Status Clear Register B-Connection Error Interrupt Flag Clear size
#define USB_REG_USBSTACLR_BCERRIC_SIZE			1
/// USB General Status Clear Register B-Connection Error Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_BCERRIC_MASK			(((1 << USB_REG_USBSTACLR_BCERRIC_SIZE) - 1) << USB_REG_USBSTACLR_BCERRIC_OFFSET)
/// USB General Status Clear Register Role Exchange Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_ROLEEXIC_OFFSET		5
/// USB General Status Clear Register Role Exchange Interrupt Flag Clear size
#define USB_REG_USBSTACLR_ROLEEXIC_SIZE			1
/// USB General Status Clear Register Role Exchange Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_ROLEEXIC_MASK			(((1 << USB_REG_USBSTACLR_ROLEEXIC_SIZE) - 1) << USB_REG_USBSTACLR_ROLEEXIC_OFFSET)
/// USB General Status Clear Register HNP Error Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_HNPERRIC_OFFSET		6
/// USB General Status Clear Register HNP Error Interrupt Flag Clear size
#define USB_REG_USBSTACLR_HNPERRIC_SIZE			1
/// USB General Status Clear Register HNP Error Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_HNPERRIC_MASK			(((1 << USB_REG_USBSTACLR_HNPERRIC_SIZE) - 1) << USB_REG_USBSTACLR_HNPERRIC_OFFSET)
/// USB General Status Clear Register Suspend Time-Out Interrupt Flag Clear offset
#define USB_REG_USBSTACLR_STOIC_OFFSET			7
/// USB General Status Clear Register Suspend Time-Out Interrupt Flag Clear size
#define USB_REG_USBSTACLR_STOIC_SIZE			1
/// USB General Status Clear Register Suspend Time-Out Interrupt Flag Clear mask
#define USB_REG_USBSTACLR_STOIC_MASK			(((1 << USB_REG_USBSTACLR_STOIC_SIZE) - 1) << USB_REG_USBSTACLR_STOIC_OFFSET)
/// USB General Status Clear Register VBus Request Clear offset
#define USB_REG_USBSTACLR_VBUSRQC_OFFSET		9
/// USB General Status Clear Register VBus Request Clear size
#define USB_REG_USBSTACLR_VBUSRQC_SIZE			1
/// USB General Status Clear Register VBus Request Clear mask
#define USB_REG_USBSTACLR_VBUSRQC_MASK			(((1 << USB_REG_USBSTACLR_VBUSRQC_SIZE) - 1) << USB_REG_USBSTACLR_VBUSRQC_OFFSET)
/// USB General Status Set Register (Write)
#define USB_REG_USBSTASET				0x0000080c
/// USB General Status Set Register ID Transition Interrupt Flag Set offset
#define USB_REG_USBSTASET_IDTIS_OFFSET			0
/// USB General Status Set Register ID Transition Interrupt Flag Set size
#define USB_REG_USBSTASET_IDTIS_SIZE			1
/// USB General Status Set Register ID Transition Interrupt Flag Set mask
#define USB_REG_USBSTASET_IDTIS_MASK			(((1 << USB_REG_USBSTASET_IDTIS_SIZE) - 1) << USB_REG_USBSTASET_IDTIS_OFFSET)
/// USB General Status Set Register VBus Transition Interrupt Flag Set offset
#define USB_REG_USBSTASET_VBUSTIS_OFFSET		1
/// USB General Status Set Register VBus Transition Interrupt Flag Set size
#define USB_REG_USBSTASET_VBUSTIS_SIZE			1
/// USB General Status Set Register VBus Transition Interrupt Flag Set mask
#define USB_REG_USBSTASET_VBUSTIS_MASK			(((1 << USB_REG_USBSTASET_VBUSTIS_SIZE) - 1) << USB_REG_USBSTASET_VBUSTIS_OFFSET)
/// USB General Status Set Register SRP Interrupt Flag Set offset
#define USB_REG_USBSTASET_SRPIS_OFFSET			2
/// USB General Status Set Register SRP Interrupt Flag Set size
#define USB_REG_USBSTASET_SRPIS_SIZE			1
/// USB General Status Set Register SRP Interrupt Flag Set mask
#define USB_REG_USBSTASET_SRPIS_MASK			(((1 << USB_REG_USBSTASET_SRPIS_SIZE) - 1) << USB_REG_USBSTASET_SRPIS_OFFSET)
/// USB General Status Set Register VBus Error Interrupt Flag Set offset
#define USB_REG_USBSTASET_VBERRIS_OFFSET		3
/// USB General Status Set Register VBus Error Interrupt Flag Set size
#define USB_REG_USBSTASET_VBERRIS_SIZE			1
/// USB General Status Set Register VBus Error Interrupt Flag Set mask
#define USB_REG_USBSTASET_VBERRIS_MASK			(((1 << USB_REG_USBSTASET_VBERRIS_SIZE) - 1) << USB_REG_USBSTASET_VBERRIS_OFFSET)
/// USB General Status Set Register B-Connection Error Interrupt Flag Set offset
#define USB_REG_USBSTASET_BCERRIS_OFFSET		4
/// USB General Status Set Register B-Connection Error Interrupt Flag Set size
#define USB_REG_USBSTASET_BCERRIS_SIZE			1
/// USB General Status Set Register B-Connection Error Interrupt Flag Set mask
#define USB_REG_USBSTASET_BCERRIS_MASK			(((1 << USB_REG_USBSTASET_BCERRIS_SIZE) - 1) << USB_REG_USBSTASET_BCERRIS_OFFSET)
/// USB General Status Set Register Role Exchange Interrupt Flag Set offset
#define USB_REG_USBSTASET_ROLEEXIS_OFFSET		5
/// USB General Status Set Register Role Exchange Interrupt Flag Set size
#define USB_REG_USBSTASET_ROLEEXIS_SIZE			1
/// USB General Status Set Register Role Exchange Interrupt Flag Set mask
#define USB_REG_USBSTASET_ROLEEXIS_MASK			(((1 << USB_REG_USBSTASET_ROLEEXIS_SIZE) - 1) << USB_REG_USBSTASET_ROLEEXIS_OFFSET)
/// USB General Status Set Register HNP Error Interrupt Flag Set offset
#define USB_REG_USBSTASET_HNPERRIS_OFFSET		6
/// USB General Status Set Register HNP Error Interrupt Flag Set size
#define USB_REG_USBSTASET_HNPERRIS_SIZE			1
/// USB General Status Set Register HNP Error Interrupt Flag Set mask
#define USB_REG_USBSTASET_HNPERRIS_MASK			(((1 << USB_REG_USBSTASET_HNPERRIS_SIZE) - 1) << USB_REG_USBSTASET_HNPERRIS_OFFSET)
/// USB General Status Set Register Suspend Time-Out Interrupt Flag Set offset
#define USB_REG_USBSTASET_STOIS_OFFSET			7
/// USB General Status Set Register Suspend Time-Out Interrupt Flag Set size
#define USB_REG_USBSTASET_STOIS_SIZE			1
/// USB General Status Set Register Suspend Time-Out Interrupt Flag Set mask
#define USB_REG_USBSTASET_STOIS_MASK			(((1 << USB_REG_USBSTASET_STOIS_SIZE) - 1) << USB_REG_USBSTASET_STOIS_OFFSET)
/// USB General Status Set Register VBus Request Set offset
#define USB_REG_USBSTASET_VBUSRQS_OFFSET		9
/// USB General Status Set Register VBus Request Set size
#define USB_REG_USBSTASET_VBUSRQS_SIZE			1
/// USB General Status Set Register VBus Request Set mask
#define USB_REG_USBSTASET_VBUSRQS_MASK			(((1 << USB_REG_USBSTASET_VBUSRQS_SIZE) - 1) << USB_REG_USBSTASET_VBUSRQS_OFFSET)
/// USB IP Version Register (Read)
#define USB_REG_UVERS					0x00000818
/// USB IP Version Register IP Version Number offset
#define USB_REG_UVERS_VERSION_NUM_OFFSET		0
/// USB IP Version Register IP Version Number size
#define USB_REG_UVERS_VERSION_NUM_SIZE			16
/// USB IP Version Register IP Version Number mask
#define USB_REG_UVERS_VERSION_NUM_MASK			(((1 << USB_REG_UVERS_VERSION_NUM_SIZE) - 1) << USB_REG_UVERS_VERSION_NUM_OFFSET)
/// USB IP Version Register Number of Metal Fixes offset
#define USB_REG_UVERS_METAL_FIX_NUM_OFFSET		16
/// USB IP Version Register Number of Metal Fixes size
#define USB_REG_UVERS_METAL_FIX_NUM_SIZE		3
/// USB IP Version Register Number of Metal Fixes mask
#define USB_REG_UVERS_METAL_FIX_NUM_MASK		(((1 << USB_REG_UVERS_METAL_FIX_NUM_SIZE) - 1) << USB_REG_UVERS_METAL_FIX_NUM_OFFSET)
/// USB IP Features Register (Read)
#define USB_REG_UFEATURES				0x0000081c
/// USB IP Features Register Maximal Number of Pipes/Endpoints offset
#define USB_REG_UFEATURES_EPT_NBR_MAX_OFFSET		0
/// USB IP Features Register Maximal Number of Pipes/Endpoints size
#define USB_REG_UFEATURES_EPT_NBR_MAX_SIZE		4
/// USB IP Features Register Maximal Number of Pipes/Endpoints mask
#define USB_REG_UFEATURES_EPT_NBR_MAX_MASK		(((1 << USB_REG_UFEATURES_EPT_NBR_MAX_SIZE) - 1) << USB_REG_UFEATURES_EPT_NBR_MAX_OFFSET)
/// USB IP Features Register Number of DMA Channels offset
#define USB_REG_UFEATURES_DMA_CHANNEL_NBR_OFFSET	4
/// USB IP Features Register Number of DMA Channels size
#define USB_REG_UFEATURES_DMA_CHANNEL_NBR_SIZE		3
/// USB IP Features Register Number of DMA Channels mask
#define USB_REG_UFEATURES_DMA_CHANNEL_NBR_MASK		(((1 << USB_REG_UFEATURES_DMA_CHANNEL_NBR_SIZE) - 1) << USB_REG_UFEATURES_DMA_CHANNEL_NBR_OFFSET)
/// USB IP Features Register DMA Buffer Size offset
#define USB_REG_UFEATURES_DMA_BUFFER_SIZE_OFFSET	7
/// USB IP Features Register DMA Buffer Size size
#define USB_REG_UFEATURES_DMA_BUFFER_SIZE_SIZE		1
/// USB IP Features Register DMA Buffer Size mask
#define USB_REG_UFEATURES_DMA_BUFFER_SIZE_MASK		(((1 << USB_REG_UFEATURES_DMA_BUFFER_SIZE_SIZE) - 1) << USB_REG_UFEATURES_DMA_BUFFER_SIZE_OFFSET)
/// USB IP Features Register DMA FIFO Depth in Words offset
#define USB_REG_UFEATURES_DMA_FIFO_WORD_DEPTH_OFFSET	8
/// USB IP Features Register DMA FIFO Depth in Words size
#define USB_REG_UFEATURES_DMA_FIFO_WORD_DEPTH_SIZE	4
/// USB IP Features Register DMA FIFO Depth in Words mask
#define USB_REG_UFEATURES_DMA_FIFO_WORD_DEPTH_MASK	(((1 << USB_REG_UFEATURES_DMA_FIFO_WORD_DEPTH_SIZE) - 1) << USB_REG_UFEATURES_DMA_FIFO_WORD_DEPTH_OFFSET)
/// USB IP Features Register Maximal FIFO Size offset
#define USB_REG_UFEATURES_FIFO_MAX_SIZE_OFFSET		12
/// USB IP Features Register Maximal FIFO Size size
#define USB_REG_UFEATURES_FIFO_MAX_SIZE_SIZE		3
/// USB IP Features Register Maximal FIFO Size mask
#define USB_REG_UFEATURES_FIFO_MAX_SIZE_MASK		(((1 << USB_REG_UFEATURES_FIFO_MAX_SIZE_SIZE) - 1) << USB_REG_UFEATURES_FIFO_MAX_SIZE_OFFSET)
/// USB IP Features Register DPRAM Byte-Write Capability offset
#define USB_REG_UFEATURES_BYTE_WRITE_DPRAM_OFFSET	15
/// USB IP Features Register DPRAM Byte-Write Capability size
#define USB_REG_UFEATURES_BYTE_WRITE_DPRAM_SIZE		1
/// USB IP Features Register DPRAM Byte-Write Capability mask
#define USB_REG_UFEATURES_BYTE_WRITE_DPRAM_MASK		(((1 << USB_REG_UFEATURES_BYTE_WRITE_DPRAM_SIZE) - 1) << USB_REG_UFEATURES_BYTE_WRITE_DPRAM_OFFSET)
/// USB IP PB Address Size Register (Read)
#define USB_REG_UADDRSIZE				0x00000820
/// USB IP PB Address Size Register IP PB Address Size offset
#define USB_REG_UADDRSIZE_UADDRSIZE_OFFSET		0
/// USB IP PB Address Size Register IP PB Address Size size
#define USB_REG_UADDRSIZE_UADDRSIZE_SIZE		32
/// USB IP PB Address Size Register IP PB Address Size mask
#define USB_REG_UADDRSIZE_UADDRSIZE_MASK		(((1 << USB_REG_UADDRSIZE_UADDRSIZE_SIZE) - 1) << USB_REG_UADDRSIZE_UADDRSIZE_OFFSET)
/// USB IP Name Register 1 (Read)
#define USB_REG_UNAME1					0x00000824
/// USB IP Name Register 1 IP Name Part 1 offset
#define USB_REG_UNAME1_UNAME1_OFFSET			0
/// USB IP Name Register 1 IP Name Part 1 size
#define USB_REG_UNAME1_UNAME1_SIZE			32
/// USB IP Name Register 1 IP Name Part 1 mask
#define USB_REG_UNAME1_UNAME1_MASK			(((1 << USB_REG_UNAME1_UNAME1_SIZE) - 1) << USB_REG_UNAME1_UNAME1_OFFSET)
/// USB IP Name Register 2 (Read)
#define USB_REG_UNAME2					0x00000828
/// USB IP Name Register 2 IP Name Part 2 offset
#define USB_REG_UNAME2_UNAME2_OFFSET			0
/// USB IP Name Register 2 IP Name Part 2 size
#define USB_REG_UNAME2_UNAME2_SIZE			32
/// USB IP Name Register 2 IP Name Part 2 mask
#define USB_REG_UNAME2_UNAME2_MASK			(((1 << USB_REG_UNAME2_UNAME2_SIZE) - 1) << USB_REG_UNAME2_UNAME2_OFFSET)
/// USB USB Finite State Machine Status Register (Read)
#define USB_REG_USBFSM					0x0000082c
/// USB USB Finite State Machine Status Register USB Controller State offset
#define USB_REG_USBFSM_DRDSTATE_OFFSET			0
/// USB USB Finite State Machine Status Register USB Controller State size
#define USB_REG_USBFSM_DRDSTATE_SIZE			4
/// USB USB Finite State Machine Status Register USB Controller State mask
#define USB_REG_USBFSM_DRDSTATE_MASK			(((1 << USB_REG_USBFSM_DRDSTATE_SIZE) - 1) << USB_REG_USBFSM_DRDSTATE_OFFSET)

extern uint32_t usb_read_configuration_register(uint32_t const offset);
extern void usb_write_configuration_register(uint32_t const offset, uint32_t const value);

#endif /* USB_H_ */

